#ifndef PROTOS_H
#define PROTOS_H

void set_const
(
    long bsize           /* input:   size of dct blocks */
);

void fdct
(
    float **imaged,     /* input:  perform dct on this image data */
    float **dctdata    /* output: dct */
);

void app_weights
(
    float **fweights,   /* input:  filter weights */
    float **dctdata    /* input as dct, output as filtered dct */
);

void rdct
(
    float **dctdata,    /* input:  filtered dct */
    float **imaged     /* output: filtered image data */
);

void fix_truncate
(
    float **imaged   /* input and output:  image data to correct */
);

void free_struct
(void);

void read_wfile
(
    char *wfile,               /* input:   file to read */
    long nbands,               /* input:   number of bands in input image */
    long fwband[MAXBND],       /* output:  a lut to the band index for fweights,
				 accommodates separate/identical filter
				 weights for each image band */
    float **fweights[MAXBND+1]/* output:  weights in file, +1 aids check for
				 wrong number of data values in wfile */
);

void bimage
(
    float **fweights[MAXBND+1], /* input:  dct filter weights */
    long fwband[MAXBND],        /* input:  lut index for band for fweights */
    struct lasinimage *ini,    /* input:  specs for input image */
    struct lasoutimage *outi   /* input:  specs for output image */
);

void begin_las
(
    char *proc             /* input:  name of las proc */
);

void integer_parm
(
    char *parmname,         /* input:  name of parameter */
    struct iparm *parmval  /* output: parm values */
);

void real_parm
(
    char *parmname,         /* input:  name of parameter */
    struct rparm *parmval  /* output: parm values */
);

void string_parm
(
    char *parmname,         /* input:  name of parameter */
    struct sparm *parmval  /* output: parm values */
);

void clean2
(
    char *message  /* input:  message to write */
);

void getin2
(
    char *parmname,             /* input:  tae parameter name for image */
    struct lasinimage *ini     /* output: data structure of input las image */
);

void getout2
(
    char *parmname,            /* input:  tae parameter name for image */
    struct lasoutimage *outi  /* output: data structure of las image */
);

void opensin
(
    struct lasinimage *ini       /* input and output: input image to open */
);

void opensout
(
    struct lasinimage *ini,     /* input:  input image, needed for nl,ns,nbands*/
    long minmax,                /* input:  flag, update min/max */
    long msgtim,                /* input:  $msgtim */
    char *odtype,               /* input:  data type for output image */
    struct lasoutimage *outi   /* input and output:  output image to open */
);

void openrin
(
    long iotype,             /* input:  type of IO = RIO or BIO */
    long blocksize,          /* input:  size of image blocks, largely ignored
                               for RIO */
    struct lasinimage *ini  /* input and output: input image to open */
);

void openrout
(
    long iotype,                /* input:  type of IO = RIO or BIO */
    struct lasinimage *ini,     /* input:  data structure with blocksize, nl,
                                  ns, nbands to be used for outi */
    long minmax,                /* input:  flag, update min/max */
    long msgtim,                /* input:  $mstgim */
    char *odtype,               /* input:  data type for output image */
    struct lasoutimage *outi   /* input and output:  output image to open */
);

void rread
(
    long curline,           /* input: starting line of block of data to read, 
			      0 relative */
    long curband,           /* input: band to read, 0 relative */
    struct lasinimage *ini /* input and output: data structure of input image 
			      to read */
);

void rwrite
(
    long curline,             /* input: starting line of block of data to write, 
			        0 relative */
    long curband,             /* input: band to write, 0 relative */
    struct lasoutimage *outi /* input: data structure of image to write */
);

void bread
(
    long curblock,          /* input: read data for this block of lines */
    long curband,           /* input: band to read, 0 relative */
    struct lasinimage *ini /* input and output: data structure of input image 
			      to read */
);

void bwrite
(
    long curblock,            /* input: write data for this block of lines */
    long curband,             /* input: band to write, input 0 relative, 
				converted to 1 relative */
    struct lasoutimage *outi /* input: data structure of input image to write */
);

void minmax
(
    long band,                /* input:  image buffer holds data for this band */
    struct lasoutimage *outi /* input and output:  structure of image data to 
				update min/max for */
);

void end_las
(
    struct lasinimage *ini,
    struct lasoutimage *outi
);

#endif


