/* include files, global parameters, and definitions for MINDIST   */

#include <stdio.h>
#include <math.h>
#include "las.h"

struct PARBLK parblk;                  /* TAE parameter block     */

struct FDESC *infd;                    /* file descriptors        */
struct FDESC *outfd;                   /* file descriptors        */
long	min_max;                       /* minmax option flag      */
long 	msgtim;                        /* message interval        */
 
extern	long	curcat;                /* parameters for processing */
extern	long	curent;                /* message utility         */

#define	HIST_SIZE 65536
#define	KERNEL_SIZE 501
#define	KERNEL_SIZE_MED 51
/*
*************************************************************************
NAME: DESPIKE

PURPOSE: Performs a low-pass, high-pass, or median filter on an
         input image.  Low-pass filters and median filters are
  	 used most often for noise suppression or smoothing, while 
         high-pass filters are typically used for image enhancement. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    4	         8/89   J. Marmon     Changes to work on PN9050
    5            9/99   D. Hames      Remove references to MAXNS and replace
				      with ns to support images >2gig.
				      Add prototypes.
				      
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
       Call GETPAR to get the user-specified parameters and tae 
       global variables
       Open output image 
       For each input band
          Call appropriate processing subroutines to perform
   	  the desired filter
       Update the output image DDR, min/max values, and history
*****************************************************************
*/
#include "util.h"
#include "protos.h"

void main
(void) 
{
double		line_inc = 1;        /* line interval for upddr        */
static double		maxval[MAXBND+1];    /* output image maximum value    */
static double		minval[MAXBND+1];    /* output image minimum value    */
double		samp_inc = 1;        /* sample interval for upddr      */

float		low;                 /* lower limit for data           */
float		kersum;              /* sum of data in the kernel      */
float           varsum;              /* sum of squared data in kernel  */
float		totpix;              
float		high;                /* upper limit for data           */
float           nstd;                /* number of standard deviations  */
float           tol;                 /* tolerance value                */

long	access;
static long	bands[MAXIMG][MAXBND+1];     /* image band numbers            */
long	bndout;                      /* output band number             */
long	comb_flag = NOCOMB;          /* for upddr                      */
long	curimg;                    
long	dtype;                       /* input image data type          */
long	i;
long	intype;                      
long	invld;                       /* invalid pixel option           */
long	klines;                      /* no. lines in the kernel        */
long	ksamps;                      /* no. samples in the kernel      */
static long	nbands[MAXIMG];              /* no. bands in each image       */
long	nl;                          /* no. lines in image             */
long	nmrimg;                      /* no. images                     */
long	ns;                          /* no. samples/line in image      */
long	onecat = ONECAT;             /* for inittm                     */
long	opt[2];                      /* for eopenr                     */
long	otype;                       /* output image data type         */
long	status;                      /* status                         */
long	totbnd;                      /* total no. bands                */
long	valsum;                      /* no. elements in the kernel     */
static long	window[MAXIMG][4];           /* image windows                 */
long	xhalf;                       /* half kernel x dimension        */
long	yhalf;                       /* half kernel y dimension        */

long    fatal_err = -1;
long    inf_err = 1;
long    one = 1;

static char	hosin[MAXIMG][CMLEN];
char	hosout[CMLEN];
char	subcmd[8];
char	taeout[CMLEN];

/*
	get user-specified parameters and tae global variables
	------------------------------------------------------
*/
getpar (hosin, taeout, hosout, &totbnd, bands, nbands,
        &nl, &ns, window, &nmrimg, subcmd, &ksamps, &klines,  
        &low, &high, &otype, &invld, &nstd,&tol);


/*
	set filter processing parameters
	--------------------------------
*/
xhalf = (ksamps - 1) / 2;
yhalf = (klines - 1) / 2;
totpix = (float) klines * (float) ksamps;

if (subcmd[0] !=  'M') 
   intype = EREAL;
else
   intype = EWORD;

/*
	initialize display message
	--------------------------
*/
c_inittm ("DESPIKE", "Processed", "line", "band", " ", &nl, 
          &totbnd, &msgtim, &onecat);

/*
	initialize min and max arrays
	-----------------------------
*/
c_inmnmx (&intype, &totbnd, minval, maxval);

/*
	open output image
	-----------------
*/
opt[0] = otype;
opt[1] = 0;
access = IWRITE;
status = c_eopenr(&outfd, hosout, &totbnd, &one, &one, &nl, &ns, &access, 
                  &intype, opt, &one);
if (status  !=  E_SUCC) 
   c_errmsg ("Fatal error encountered", "despike-fatal", &fatal_err);

/*
	initialize output band counter
	------------------------------
*/
bndout = 0;
curcat = 1;
for (curimg = 0; curimg < nmrimg; curimg++)
   {
/*
	   open input image
	   ----------------
*/
   initio (hosin[curimg], window[curimg], &intype);

/*
	   process each band
	   -----------------
*/
   for (i = 0; i < nbands[curimg]; i++, curcat++)
      {
      bndout++;
      if (subcmd[0] !=  'M') 
         {
         prehl (hosout, xhalf, yhalf, &ns, &bands[curimg][i], 
                high, low, &varsum, &kersum, &valsum);

         hilo (hosout, nl, &ns, otype, &bands[curimg][i], &bndout,  
               xhalf, yhalf, ksamps, high, low, &varsum, &kersum, 
	       &valsum, invld, minval, maxval, nstd, tol);
         }
      else
         {
         premed (hosout, xhalf, yhalf, klines, &ksamps, &ns, &bands[curimg][i]);

         median (hosout, nl, &ns, &bands[curimg][i], &bndout, xhalf, 
                 yhalf, totpix, &ksamps, dtype, minval, maxval);
         } 

      }

/*
	   close input and output images
	   -----------------------------
*/
   status = c_eclose (&infd);
   if (status  !=  E_SUCC) 
      c_errmsg ("Nonfatal error encountered", "despike-warn", &inf_err);
   status = c_eclose (&outfd);
   if (status  !=  E_SUCC) 
      c_errmsg ("Nonfatal error encountered", "despike-warn", &inf_err);
   }

/*
	update ddr and minmax and for output image
	------------------------------------------
*/
c_upddr (&parblk, hosin, hosout, &nmrimg, window, bands,
         nbands, &line_inc, &samp_inc, &totbnd, &comb_flag);

if (min_max)
   c_svmnmx (hosout, &totbnd, minval, maxval, "YES");

/*
	create associated history file for output image
	-----------------------------------------------
*/
status = c_hstout(&parblk, hosout, hosin, &nmrimg, " ");

/*
	output completion message
	-------------------------
*/
c_complt ("DESPIKE", (char (*)[256])taeout, &one);

}

