/*
*************************************************************************
NAME: PREMED

PURPOSE: reads the initial kernel and determines its histogram
         for median filters.


PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    4            9/99   D. Hames      Remove references to MAXNS and replace
                                      with ns to support images >2gig.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************
*/
#include "despike.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

long	*hist;                  /* pointer to the middle of the histogram */
long	*hist_buf;              /* histogram array                     */

short	*image_buf;             /* holds image lines being processed   */
short	*imagep[KERNEL_SIZE_MED];/* pointers to image lines             */
short	**image;                /* pointer to the middle of the kernel */

void premed
(
    char hosout[],       /* output image host name              */
    long xhalf,          /* half kernel x dim                   */
    long yhalf,          /* half kernel y dim                   */
    long klines,         /* no. lines in kernel                 */
    long *ksamps,        /* no. samples/line in kernel          */
    long *ns,            /* no. samples/line in image           */
    long *curbnd         /* current input band number           */
)
{
long    zero = 0;
long    one = 1;

char    *prog_name = "DESPIKE"; /* program name            */

long	count;
long	elong = ELONG;
long	eword = EWORD;

long	hist_size = HIST_SIZE;
long	i;
long	j;
long	k;
long	length;
long	status;

short	*medin;

medin = (short *) malloc(*ns * sizeof(short));
if (medin == NULL)
   {
   c_eclose (&outfd);
   c_abort (&parblk, "Error allocating dynamic memory", "despike-alloc", 
            prog_name, (char (*)[256])hosout, &one);
   }

if ((image_buf = (short *) malloc (klines * (*ns + *ksamps - 1) *
                  sizeof (short))) == NULL ||
   (hist_buf = (long *) malloc (HIST_SIZE * sizeof (long))) == NULL)
   {
   c_eclose (&outfd);
   c_abort (&parblk, "Error allocating dynamic memory", "despike-alloc", 
            prog_name, (char (*)[256])hosout, &one);
   }
for (i = 0; i < klines; i++)
   imagep[i] = &image_buf[i * (*ns + *ksamps - 1) + ((*ksamps - 1) / 2) - 1];

image = &imagep[(klines - 1) / 2 - 1];
hist = hist_buf + HIST_SIZE/2;
/*
	initialize histogram array
	--------------------------
*/
c_pxfill ((unsigned char *)hist_buf, &elong, &hist_size, 
		(unsigned char *)&zero);

for (j = 1; j <= yhalf + 1; j++)
   {
   
/*
	   get a line from the input image
	   -------------------------------
*/
   status = c_eread (&infd, curbnd, &j, (unsigned char*)medin, &one);
   if (status  !=  *ns) 
      {
      c_eclose (&outfd);
      c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
               prog_name, (char (*)[256])hosout, &one);
      }
/*
	   copy image line into array
	   --------------------------
*/
   c_pxcopy ((unsigned char *)medin, (unsigned char *)&image[j][1], 
		&eword, ns);

/*
	   do the left edge mirror
	   -----------------------
*/
   for (k = 1 - xhalf; k <= 0; k++)
      image[j][k] = image[j][abs (k - 2)];

/*
	   do the right edge mirror
	   ------------------------
*/
   for (k = *ns + 1; k <= *ns + xhalf; k++)
      image[j][k] = image[j][(2 * *ns) - k];

/*
	   copy the mirrored lines
	   -----------------------
*/
   if (j  !=  1) 
      {
     length = *ns + (*ksamps - 1);
     c_pxcopy ((unsigned char *)&image[j][1 - xhalf], 
	        (unsigned char *)&image[2-j][1 - xhalf], &eword, &length);
      } 
   }

/*
	do the histogram for the first kernel
	-------------------------------------
*/
for (k = 1 - yhalf; k <= 1 + yhalf; k++)
   {
   status = c_pxhist(ksamps, (long *)&image[k][1 - xhalf], &eword,
                     hist, &count);
   if (status  !=  E_SUCC) 
      {
      c_eclose (&outfd);
      c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
               prog_name, (char (*)[256])hosout, &one);
      }
   }
}
