/*
*************************************************************************
NAME: SET 

PURPOSE: sets the position for the next sample to be processed.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************
	arguments:
	name		description
	----		-----------
	 k		input buffer position	
	 l		output buffer position 
	 ns		number of samples per line to be processed

************************************************************************
*/
#include "despike.h"
#include "protos.h"

void set
(
   long k,                      /* I: input buffer position                  */
   long *l,                     /* O: output buffer position                 */
   long ns                      /* I: num of samples per line to be processed*/
)
{

/*
	--------------------------------------------------------------
	|if (input position extends beyond leftmost sample) then     |
	|   mirror to set output position 			     |
	|elseif (input position extends beyond rightmost sample) then|
	|   mirror to set output position			     |
	|else							     |
	|   set output position to input position since 	     |
	|   input is within the bounds of the image		     |
	|endif							     |
	--------------------------------------------------------------
*/
if (k  <  1) 
   *l = abs(k - 2);
else if (k  >  ns) 
   *l = 2 * ns - k;
else
   *l = k;

(*l)--;
}

