/*****************************************************************************
NAME:		DLTF2PAL

PURPOSE:	

PROGRAM HISTORY:
VERSION	     DATE	PROGRAMMER   CODE/CONT	REQUEST
-------	     ----  	----------   ---------	-------

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	DLTF2PAL must be run under TAE.

PROJECT:	LAS

ALGORITHM:

*****************************************************************************/
#define UTIL 1
#define DMSLEN 12
#define LUTSIZ 256
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "dltf.h"
#include "protos.h"

int main
(void)
{
long	j,k,m,	/* Loop indices; working variables.		*/
	bnd,		/* Current band number to be printed.		*/
	status,		/* Return status indicator. 			*/
	fatal = -1,	/* Specifies a fatal error.			*/
	one = 1,	/* Specifies a non-fatal error.			*/
	start,		/* Starting point of the LUT.			*/
	end,		/* Ending point of the LUT.			*/
	fd,		/* File descriptor for the DLTF file. 		*/
	dlen;		/* Length of the data portion of the DLTF.	*/

double	slope,		/* Slope of the LUT; this is used if DMS saves
			   a non-byte LUT (see SAVDLTF).		*/
	offset;		/* Offset of the LUT; this is used if DMS saves
			   a non-byte LUT (see SAVDLTF).		*/

FILE	*ofp;		/* Pointer to output file. 			*/

char	hostnm[CMLEN],  /* Host name of the DLTF assoiciated file.	*/
	outfile[1][CMLEN],
	dltfnam[DMSLEN+1],/* Name of the DLTF entry to be read. 	*/
	dtype[DTYPL],	/* Data type of the DLTF.			*/
	line[12][80],	/* Working line format buffer.			*/
	desc[DESLEN];	/* Description associated with the entry. 	*/

unsigned char   *rlut,	/* Red lookup table data. 			*/
	   	*glut,	/* Green lookup table data. 			*/
		*blut,	/* Blue lookup table data. 			*/
	*rbyteptr,	/* Working BYTE pointer into the red LUT buffer.*/
	*gbyteptr,	/* Working BYTE pointer into the green LUT buffer.*/
	*bbyteptr;	/* Working BYTE pointer into the blue LUT buffer.*/

struct ASDLTF asdltf;	/* Structure to contain DLTF information for a
			   detailed listing of a DLTF entry.		*/

struct PARBLK vblock;	/* TAE variable block.				*/

/* Get the user specified parameters.
  -----------------------------------*/
getpar(&vblock,dltfnam,outfile[0],hostnm);

/* Read the first record of the DLTF.
   ----------------------------------*/
if (c_r1dltf(&fd,hostnm,dltfnam,desc,&asdltf,&dlen,dtype) != E_SUCC)
    c_errmsg("Fatal error encountered","dltf2pal-fatal",&fatal);

if ((strcmp(dtype,"B") != 0))
    {
    c_errmsg("LUT must be BYTE to convert to IMDSP pallette","dltf2pal",&one);
    c_errmsg("Fatal error encountered","dltf2pal-fatal",&fatal);
    }

/* If successful then allocate memory to contain the actual LUT(s)
   from the second type of DLTF record.
   -----------------------------------------------*/
if ((rlut = (unsigned char *)malloc(dlen)) == 0)
    c_errmsg("Error allocating buffer space for DLTF record","dltf2pal-nomem",
	&one);

if ((glut = (unsigned char *)malloc(dlen)) == 0)
    c_errmsg("Error allocating buffer space for DLTF record","dltf2pal-nomem",
	&one);

if ((blut = (unsigned char *)malloc(dlen)) == 0)
    c_errmsg("Error allocating buffer space for DLTF record","dltf2pal-nomem",
	&one);

ofp = fopen(outfile[0],"w");
if (ofp == NULL)
    {
    c_errmsg("Error opening output palette file","dltf2pal-outopen",&one);
    c_errmsg("Fatal error encountered","dltf2pal-fatal",&fatal);
    }

/* Output the header info.
   -----------------------*/
fprintf(ofp,"FILE_TYPE                      = TABLE\r\n");
fprintf(ofp,"RECORD_TYPE                    = STREAM\r\n");
fprintf(ofp,"TABLE_ROWS                     = 256\r\n");
fprintf(ofp,"ROW_COLUMNS                    =   4\r\n");
fprintf(ofp,"COLUMN_NAME                    = (COLOR_NUMBER,\r\n");
fprintf(ofp,"                                  RED_VALUE,\r\n");
fprintf(ofp,"                                  GREEN_VALUE,\r\n");
fprintf(ofp,"                                  BLUE_VALUE)\r\n");
fprintf(ofp,"COLUMN_TYPE                    = (INTEGER,INTEGER,INTEGER,INTEGER)\r\n");
fprintf(ofp,"NOTE                           = \"DLTF2PAL generated\"\r\n");
fprintf(ofp,"END\r\n");

/* Read each of the second type of records (one for each LUT)
   and output the information accordingly.  
   ---------------------------------------------------------*/

bnd = 1;
status = c_r2dltf(&fd,dltfnam,&bnd,&start,&end,&dlen,rlut,&slope,&offset);
if (status != E_SUCC)
    {
    fclose(ofp);
    c_clean(&vblock,"dltf2pal",outfile,&one);
    }

bnd = 2;
status = c_r2dltf(&fd,dltfnam,&bnd,&start,&end,&dlen,glut,&slope,&offset);
if (status != E_SUCC)
    {
    fclose(ofp);
    c_clean(&vblock,"dltf2pal",outfile,&one);
    }

bnd = 3;
status = c_r2dltf(&fd,dltfnam,&bnd,&start,&end,&dlen,blut,&slope,&offset);
if (status != E_SUCC)
    {
    fclose(ofp);
    c_clean(&vblock,"dltf2pal",outfile,&one);
    }

rbyteptr = rlut;
gbyteptr = glut;
bbyteptr = blut;

for (j = 0; j < LUTSIZ; j+=8)
    {
/* Format the BYTE LUT for output.  For efficiency output 
   eight lines at a time.
   ------------------------------------------*/
    for (k = 0, m = j; k < 8; k++, m++, rbyteptr++,gbyteptr++,bbyteptr++)
	sprintf(line[k],"%3d %3d %3d %3d",
	    m,*rbyteptr,*gbyteptr,*bbyteptr);

/* Output the LUT information.
   ---------------------------*/
    for (k = 0; k < 8; k++)
	fprintf(ofp,"%s\r\n",line[k]);
    }

/* Free the dynamically allocated memory.
   --------------------------------------*/
free(rlut);
free(glut);
free(blut);

/* Issue a standard completion message.
   ------------------------------------*/
fclose(ofp);
c_complt("DLTF2PAL",outfile,&one);

return EXIT_SUCCESS;

}
