#include <string.h>
#include "drophist.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void delrec
(
    char key[16],			/* key of the record to be read      */
    char *buf[2], 			/* buffer for history label record   */
    long *buflen,				/* length of history record buffer   */
    long *hiscnt,			/* history record count              */
    long *delcnt,			/* delete record count               */
    long confirm			/* confirmation flag                 */
)

/*
*****************************************************************************

NAME:	 DELREC	

PURPOSE:  Deletes history label of the specified file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          03-Nov-83    J. Hwang           SAR 
   2          26-Nov-85    K. Narayanan       STX      
   3          01-Mar-90    K. Kunska          Conversion to 5.0   
   4	      29-Nov-90	   D. Akkerman	      Fixed return status for repos.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize varibles.
   While not at end of the history file,
      Read one history record at a time.
      If confirmation option was selected, 
         Display the history record to the user.
         Ask if user wants to delete this record or exit.
         If the answer = YES, 
            Call REPOS to reposition the file pointer to the beginning of 
            the header and mark the record for deletion.
      End-f
   End-while 

*****************************************************************************
*/
{

long    fatal_err = -1;        /* fatal error return status */

char	ans[5], 			/* answer to a prompt                */
 	cmd[15],			/* tae command in the history record */
        date[7],			/* date in the history record        */
	fill_val,			/* fill value                        */
	msg[79],			/* message text buffer               */
	ops[10],			/* operating system in history record*/
	quit[5],			/* EXIT string                       */
	time[9],			/* time of day in the history record */
	usr[15],			/* user name in the history recod    */
        wday[4];			/* day of week in the history record */

long 	alen,				/* length of the answer string       */
	doit,				/* looping flag                      */
	fill_num,			/* number of values to fill in       */
	fill_type,			/* data type of value to be filled   */
	i, 				/* loop counter and array index      */
	ncount,				/* number of newline characters      */
	start,				/* start of the display buffer       */
	status,				/* return status                     */
	year;				/* year in the history record        */

/*
Initialize varibles.
---------------------
*/
strcpy (quit, "EXIT");
strcpy (key, " ");;
strcpy (wday, "   ");
strcpy (cmd, "              ");
strcpy (date, "      ");
strcpy (time, "        ");
strcpy (ops, "         ");
strcpy (usr, "              ");
fill_type = EBYTE;
fill_num = 79;
fill_val = ' ';
*hiscnt = 0; 
*delcnt = 0;
doit = TRUE;

/*
While not at end of the history file,
Read one history record at a time.
----------------------------------------
*/
while (doit)
   {
   status = c_hsread (&fdhist, key, buf, buflen);
   
   if (status == E_EOF)
      break;
   else if (status == E_SUCC)    /* got a history record */
      {
      *hiscnt = *hiscnt + 1;

/*
If confirmation option was selected, display the history record to the user.
-----------------------------------------------------------------------------
*/
      if (confirm) 
         {
         start = 0;
	 ncount = 0;
         sscanf (buf[0], "\n%s %6c %8c %4d\n%s\n%s\n%s", wday, date, time,
                 &year, ops, usr, cmd);
         while (ncount <= 5)
            {
            if (*(buf[0]+start) == '\n') 
               ncount++;
            start++;
            }
         c_pxfill (msg, &fill_type, &fill_num, &fill_val);
         sprintf (msg, "\n%s", usr);
         sprintf (&msg[15], "%s", cmd);
         sprintf (&msg[44], "%s %d", date, year);
         sprintf (&msg[66], "%s", time);
         for (i = 0; i < 66; i++)
            {
            if (msg[i] == '\0')
               msg[i] = ' ';
            }
         printf ("%s\n", msg);
         printf ("%s", &buf[0][start]);
         printf ("\n\n");
   
/*
Ask if user wants to delete this record or exit.
--------------------------------------------------
*/
         printf ("$_DELETE (Y/N/EXIT) : ");
         scanf ("%s", ans);
         alen = (long) (strlen (ans));
         c_low2up (ans, &alen);
         if (strncmp (ans, quit, alen) == 0) 
            break;
         }
      else
         strcpy (ans, "YES");
   
/*
If the answer = YES, call REPOS to reposition the file pointer to the
beginning of the header and mark the record for deletion. 
-----------------------------------------------------------------------
*/ 
     if (ans[0] == 'Y')        
         {
         status = repos (&fdhist, buflen);
         if (status != E_SUCC) 
            c_errmsg ("Unable to reposition file pointer for history record",
			"drophist_unabldel",&fatal_err); 
         status = c_hsdel (&fdhist, key);
         if (status != E_SUCC) 
            c_errmsg ("Unable to delete history record", "drophist_unabldel",
                      &fatal_err); 
         else
            {
            *delcnt = *delcnt + 1;
            printf ("\nDELETED HISTORY RECORD %4d", *hiscnt);
            }  
         } 
      }
   else 
      c_errmsg ("Failed to read history label", "drophist_read", &fatal_err);
   } /* end-while */

}
