/******************************************************************************

NAME:	 DROPHIST	

PURPOSE:  Deletes history label of the specified file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          03-Nov-83    J. Hwang           SAR 
   2          26-Nov-85    K. Narayanan       STX      
   3          01-Mar-90    K. Kunska          Conversion to 5.0   
   4          02-Oct-90    K. Kunska          Changed wording of an error msg
   5 	      24-Oct-92	   T. Mittan	      Modified include file to 
					      eliminate warnings
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   Call GETPAR to retrieve the user supplied parameters.
   Open the history file.
   Call DELREC to display and delete history records.
   Report the number of records that were deleted.
   Close the history file removing records marked for deletion.
   Print the completion message.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct  PARBLK  block;                 /* TAE parameter block     */

char    *prog_name = "DROPHIST";  /* program name            */
long    fdhist;                 /* input file descriptors   */

extern long     curent;                /* current entity           */
extern long     curcat;                /* current category         */

static  long    fatal_err = -1;        /* fatal error return status */
static  long    inf_err = 1;           /* info error return status  */

static  long    zero = 0;


int main
(void) /* drophist */
{

char	*buf[2], 			/* buffer for history label record   */
	hosin[CMLEN],			/* host file name for which histry   */
					/* records should be retrieved       */ 
	key[16],			/* key of the record to be read      */
	taein[CMLEN];			/* TAE file name for which history   */
					/* records should be retrieved       */ 

long 	access,				/* access type for history file      */
	action,				/* action flag for c_hsclos          */
	buflen,				/* length of history record buffer   */
	confirm,			/* confirmation flag                 */
	delcnt,				/* delete record count               */
	hiscnt,				/* history record count              */
	status;				/* return status                     */

char tmpstr[1][CMLEN];

/*
Initialize varibles.
---------------------
*/
access = 3;
action = 1;
strcpy (key, " ");
hiscnt = 0; 
delcnt = 0;

/*
Call GETPAR to retrieve the user supplied parameters.
------------------------------------------------------
*/
getpar (taein, hosin, &confirm);

/*
Open the history file.
-----------------------
*/
status = c_hsopen (&fdhist, hosin, &access);
if (status != E_SUCC)
   c_errmsg ("Cannot open history file", "drophist-open", &fatal_err);

/*
Call DELREC to display and delete history records.
---------------------------------------------------
*/
delrec (key, buf, &buflen, &hiscnt, &delcnt, confirm);

/*
Report the number of records that were deleted.
-----------------------------------------------
*/
if (hiscnt == 0) 
   {
   action = 0;
   c_errmsg ("No history record found", "drophist_nohist", &inf_err); 
   }
else
   printf ("\nDELETED %4d OUT OF %4d HISTORY(S) FOUND\n", delcnt, hiscnt);

/*
Close the history file removing records marked for deletion.
------------------------------------------------------------
*/
histclose (&fdhist, hosin, action);

/*
Print the completion message.
-----------------------------
*/
strcpy(tmpstr[0], "\0");
c_complt (prog_name, tmpstr, &zero);

return EXIT_SUCCESS;

}
