#include "dsphistry.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void disprec
(
    char key[16],			/* key of the record to be read      */
    char *buf[2],			/* buffer for history label record   */
    long *buflen,				/* length of history record buffer   */
    long *hiscnt,			/* history record count              */
    long nrecs,				/* number of records to be displayed */
    char dspflg[4]			/* display flag                      */
)

/*
*****************************************************************************

NAME:	 DISPREC

PURPOSE:  Displays history record of the specified file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          03-Nov-83    J. Hwang   
   2          29-Nov-88    J. Lavergne     Reduced output line length to 79 
                                           bytes for file or lp output so that
                                           fileutil handles the length o.k.
   3          22-Mar-90    K. Kunska       Conversion to 5.0   
   4          18-Apr-90    K. Kunska       Fixed display when options TERM and
                                           LP are selected
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   While not at end of the history file,
      Read one history record at a time.
      If dspflg = YES, 
         Format the information in the history record.
         Display the history record.
      End-if
   End-while 

*****************************************************************************
*/

{

long    fatal_err = -1;            /* fatal error return status */

char	cmd[15],			/* tae command in the history record */
        date[7],			/* date in the history record        */
	fill_val,			/* fill value                        */
	msg[79],			/* message text buffer               */
	ops[10],			/* operating system in history record*/
	quit[5],			/* EXIT string                       */
	time[9],			/* time of day in the history record */
	usr[15],			/* user name in the history recod    */
        wday[4];			/* day of week in the history record */

long 	
	doit,				/* looping flag                      */
	fill_num,			/* number of values to fill          */
	fill_type,			/* type of the fill value            */
	i,  				/* loop counter and array index      */
	ncount,				/* number of newline characters      */
	reset_flg,			/* set to TRUE if the print[TERM]    */
					/* spec. was reset - needed to get   */
					/* around c_txtout display problem   */
	start,				/* start of the display buffer       */
	status,				/* return status                     */
	year;				/* year in the history record        */

/*
Initialize variables.
---------------------
*/
strcpy (quit, "EXIT");
strcpy (key, " ");;
strcpy (wday, "   ");
strcpy (cmd, "              ");
strcpy (date, "      ");
strcpy (time, "        ");
strcpy (ops, "         ");
strcpy (usr, "              ");
*hiscnt = 0; 
doit = TRUE;
fill_type = EBYTE;
fill_num = 78;
fill_val = ' ';
reset_flg = FALSE;

/*
While not at end of the history file,
Read one history record at a time.
----------------------------------------
*/
while (doit)
   {
   status = c_hsread (&fdhist, key, buf, buflen);
   
   if (status == E_EOF)
      break;
   else if (status == E_SUCC) 
      {
      *hiscnt = *hiscnt + 1;
      if (*hiscnt > nrecs) 
         {
         *hiscnt = *hiscnt - 1;
         break;
         }
/*
If dspflg = YES, format the information in the history record.
---------------------------------------------------------------
*/
      if (strcmp (dspflg, "YES") == 0)
         {
         start = 0;
	 ncount = 0;
         sscanf (buf[0], "\n%s %6c %8c %4d\n%s\n%s\n%s", wday, date, time,
                 &year, ops, usr, cmd);
         while (ncount <= 5)
            {
            if (*(buf[0]+start) == '\n') 
               ncount++;
            start++;
            }
         c_pxfill (msg, &fill_type, &fill_num, &fill_val);
         sprintf (msg, "--- HISTORY RECORD %3d ---", *hiscnt);
         c_txtout (msg, print, fd1);
/*
Display the history record.
----------------------------
*/
         c_pxfill (msg, &fill_type, &fill_num, &fill_val);
         msg[0] = '\0';
         sprintf (msg, "%s", usr);
         sprintf (&msg[15], "%s", cmd);
         sprintf (&msg[44], "%s %d", date, year);
         sprintf (&msg[66], "%s", time);
         for (i = 0; i < 66; i++)
            {
            if (msg[i] == '\0')
               msg[i] = ' ';
            }
         if (print[TERM])
            {
            printf ("%s\n", msg);
            printf ("%s\n\n", &buf[0][start]);
            } 
         if (print[LP] || print[FIL])
            {
            if (print[TERM])
               {
               print[TERM] = 0;
               reset_flg = TRUE;
               }
            c_txtout (msg, print, fd1);
            c_txtout ((buf[0]+start), print, fd1);
            c_txtout ("\n", print, fd1);
            if (reset_flg)
              {
              print[TERM] = 1;
              reset_flg = FALSE;
              }
            }
         }
      }
   else 
      c_errmsg ("Unable to read history record", "dsphistry-read", &fatal_err); 
   } /* while-doit */


}
