
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define TERM    0
#define FIL     1
#define LP      2

struct  PARBLK  block;                 /* TAE parameter block     */
FILE    *fd1;                          /* temp file descriptor    */

static  char    *prog_name = "DSPHISTRY";  /* program name            */
char    out_files[2][CMLEN];           /* output file names        */
long    fdhist;                        /* input file descriptor    */
long    print[3];                      /* print options            */
long     num_outfiles;                  /* number of output files   */

extern long     curent;                        /* current entity           */
extern long     curcat;                        /* current category         */

static  long    fatal_err = -1;                /* fatal error return status */
static  long    inf_err = 1;                   /* info error return status  */


/******************************************************************************

NAME:	 DSPHISTRY

PURPOSE:  Displays history label of the specified file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          03-Nov-83    J. Hwang   
   2          29-Nov-88    J. Lavergne     Reduced output line length to 79 
                                           bytes for file or lp output so that
                                           fileutil handles the length o.k.
   3          22-Mar-90    K. Kunska       Conversion to 5.0   
   4          18-Apr-90    K. Kunska       Fixed DISPREC display when options
                                           TERM and LP are selected 
   5          02-Oct-90    K. Kunska       Changed wording of an error message
   6 	      24-Sep-92	   T. Mittan	   Modified include file to eliminate
					   warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   Call GETPAR to retrieve the user supplied parameters.
   Open the history file.
   Call DISPREC to format and display a history record.
   Report number of history records found.
   Close history file.
   Print the completion message.

*****************************************************************************
*/

int main
(void) /* dsphistry */
{

char	
 	*buf[2],			/* buffer for history label record   */
	dspflg[4],			/* display flag                      */
	hosin[1][CMLEN],			/* host file name for which histry   */
					/* records should be retrieved       */ 
	key[16],			/* key of the record to be read      */
	print_file[CMLEN],		/* print file name                   */
	taein[1][CMLEN];			/* TAE file name for which history   */
					/* record is to be displayed         */

long 	access,				/* access type for history file      */
	buflen,				/* length of history record buffer   */
	hiscnt,				/* history record count              */
	nrecs,				/* number of records to be displayed */
	status;				/* return status                     */

/*
Initialize variables.
---------------------
*/
access = 0;
strcpy (key, " ");;
hiscnt = 0; 

/*
Call GETPAR to retrieve the user supplied parameters.
-----------------------------------------------------
*/
getpar (taein, hosin, dspflg, &nrecs, print_file);

/*
Open the history file.
-----------------------
*/
status = c_hsopen (&fdhist, hosin[0], &access);
if (status != E_SUCC)
   c_errmsg ("Cannot open history file", "dsphistry-open", &fatal_err);

printf ("\nProcessing label of : %s\n", taein[0]);

/*
Call DISPREC to format and display a history record.
----------------------------------------------------
*/
disprec (key, buf, &buflen, &hiscnt, nrecs, dspflg);

/*
Report number of history records found.
-----------------------------------------
*/
if (hiscnt  ==  0) 
   c_errmsg ("History file contains no history records", "dsphistry-norec", 
             &inf_err); 
else
   printf ("\nFound %4d history record(s)\n", hiscnt);

/*
Close history file.
-------------------
*/
c_hsclos (&fdhist, hosin[0]);

/*
Print the completion message.
------------------------------
*/
c_complt (prog_name, out_files, &num_outfiles);

return EXIT_SUCCESS;

}
