#include <string.h>
#include "dsphistry.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char taein[][CMLEN],		/* tae name of the input image              */
    char hosin[][CMLEN],		/* host name of the input image             */
    char dspflg[4],		/* display flag                             */
    long *nrecs,			/* number of history records                */
    char print_file[]		/* print file name                          */
)

/*
*****************************************************************************

NAME:	 GETPAR	

PURPOSE: Retrieves the user supplied parameters. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          03-Nov-83    J. Hwang   
   2          29-Nov-88    J. Lavergne     Reduced output line length to 79 
                                           bytes for file or lp output so that
                                           fileutil handles the length o.k.
   3          22-Mar-90    K. Kunska       Conversion to 5.0   
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Retrieve tae parameter block.
   Parse the input image specification.
   If the return status is not success then abort with a fatal error message.
   Get the display flag and number of records.
   Get the print option and open the print file.

*****************************************************************************
*/

{

char    *prog_name = "DSPHISTRY";  /* program name            */
long    fatal_err = -1;            /* fatal error return status */
long    zero = 0;
long    one = 1;

char	fileutil_file[CMLEN];	/* file for fileutil facility                */

long	access,			/* access type for getnam                    */
	partype,		/* data type of a parameter                  */
	status,			/* return status                             */
	std_out;		/* logical unit for standard output          */

struct VARIABLE  *var;	/* pointer to a variable in the tae block     */

/*
Retrieve tae parameter block.
------------------------------
*/
c_inlas (&block, &std_out); 

/*
Parse the input image specification.
If the return status is not success then abort with a fatal error message.
---------------------------------------------------------------------------
*/
access = IREAD;
status = c_getnam (&block, "IN", &zero, hosin[0], "img", &access); 
if (status != E_SUCC)
   c_errmsg ("Cannot access input file name", "dsphistry-getnam", &fatal_err);
strcpy(taein[0], hosin[0]);

/*
Get the display flag and number of records.
-------------------------------------------
*/
var = p_find (&block, "DSPFLG");
strcpy (dspflg, SVAL(*var, 0)); 
partype = ELONG;
c_getnum (&block, "NRECS", &one, &partype, (void *)nrecs);

/*
Get the print option and open the print file.
-----------------------------------------------
*/
strcpy (fileutil_file, prog_name);
if (p_find (&block, "PRINT")->v_count != 0)
   c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
             print_file);
if (print[LP] || print[FIL])
   {
   fd1 = fopen (fileutil_file, "w");
   strcpy (out_files[num_outfiles++], print_file);
   }
else
   fd1 = NULL;

}      
