/*******************************************************************************
NAME:				FASTMAP

PURPOSE:    Maps a single value to another value.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   R. McKinney	   9/90 	  EDC		Original Development
  2.0	   T. Mittan	   9/92		  EDC		Added extern to curent
							and curcat to eliminate
							warnings on link
  3.0	   S. Nelson	   6/92		  EDC 		In getpar, set count
							variable "i" to zero to
							call c_gettyp.
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   FASTMAP must be run under TAE.

PROJECT:			LAS

ALGORITHM DESCRIPTION:
 	The input parameters and globals are retrieved.
	Error checking is done on the input image(s) to ensure that they all 
	have equal window sizes.  
	Each line of the input image is mapped to the corresponding line and
        written back into the original image.
	The associated history file is created to represent the output 
        image.
	A completion message is written.

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
struct FDESC *fdesc[MAXIMG+1];	/* image file descriptors used for image I/O */
struct GDESC *gdesc;		/* group descriptor used for image I/O       */
long offset1[MAXIMG*MAXBND];/* offsets into thebuf for input bands */
extern long curent;	/* current processing line		     */
extern long curcat;	/* current processing band		     */

int main
(void)
{
struct PARBLK vblock;		/* TAE parameter block               */

static long bands[MAXIMG][MAXBND+1];/* bands to be processed - all images*/
long    from;                           /* from value to be mapped           */
long    to;                             /* new value to be inserted          */
long	msgtim;				/* elapsed time between display msg  */
long	nbands[MAXIMG];			/* number of bands per input image   */
long	nmrimg;				/* number of input images            */
long    odtype;                         /* data type of the image            */
long	one = 1;			/* value of one			     */
long	std_out;			/* LUN to standard output    	     */
long	totbnd;				/* total number of bands             */
long	window[MAXIMG][4];		/* window for each input image       */

char	hosin[MAXIMG][CMLEN];		/* host names of input images        */
char	hosout[1][CMLEN];			/* host name of output image 	     */
char	minmax[4];			/* min max flag                      */
char	taein[MAXIMG][CMLEN];		/* TAE names of input images         */

/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);

/* Get parameters and globals
-----------------------------*/
getpar(&vblock,taein,hosin,&nmrimg,window,bands,nbands,
       &totbnd,&odtype,&msgtim,minmax,&from,&to);

/* Process image
----------------------------------------------*/
  proces(&vblock,hosin,hosout,nmrimg,window,bands,nbands,totbnd,
	 odtype,msgtim,from,to);

/*  Update history file 
-----------------------*/
/*
if (c_hstout(&vblock,hosout[0],hosin[0],&nmrimg," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","fastmap-warn",&one);
*/

/* Output completion message 
----------------------------*/
c_complt("FASTMAP",taein,&one);

return EXIT_SUCCESS;

}


/******************************************************************************
NAME:   			GETPAR 

ALGORITHM:
   Retrieve the required parameters and globals from the TAE parameter block
   Parse the input image specification by calling GETIN which will validate 
    the window and band specifications  
   Checking to ensure that all the input images have equal window sizes  
   Retreive the from value
   Retreive the to value
   Retreive the global parameters
   Return
******************************************************************************/

void getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block            */
    char taein[][CMLEN],         /* TAE file names of input images  	     */
    char hosin[][CMLEN],         /* host file names of input images 	     */
    long *nmrimg,		/* number of input images                    */
    long (*window)[4],		/* window specifications for each input image*/
    long (*bands)[MAXBND+1],	/* band specifications for each input image  */
    long *nbands,		/* number of bands per input image           */
    long *totbnd,		/* total number of bands to be processed     */
    long *odtype,                /* data type of the image                    */
    long *msgtim,		/* elapsed time between display messages     */
    char *minmax,		/* min max flag buffer                       */
    long *from,                  /* from value to be mapped                   */
    long *to                    /* to value to be inserted                   */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long	element;		/* parameter element number		     */
long    fatal = -1;		/* fatal error flag			     */
long	in_dtype;		/* input image data type		     */
long	imglim;			/* maximum number of images to be processed  */
long    nonfatal = 1;		/* nonfatal error flag			     */
long    one = 1;		/* value of one				     */
long	qflag[MAXIMG][2];	/* flag indicating if subwindows specified   */
long	same;			/* flag indicating if images have same window*/
long	status;			/* function return status code               */


/* Retrieve and parse input image specification 
-----------------------------------------------*/
element = 0;
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,nbands,
		totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","fastmap-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","fastmap-window",
	    &nonfatal);
   c_errmsg("Fatal error encountered","fastmap-fatal",&fatal);
   }

/* Retrieve output data type
----------------------------*/

status = c_gettyp(hosin,&one,&in_dtype);
if ( status != E_SUCC)
    c_errmsg("Fatal error encountered","fastmap-fatal",&fatal);
*odtype = in_dtype;

/* Retrieve FROM value
----------------------*/
val = p_find(vblock,"FROM");
*from = IVAL(*val,0);

/* Retrieve TO value
--------------------*/
val = p_find(vblock,"TO");
*to = IVAL(*val,0);

/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

return;
}
