/*****************************************************************************
NAME:				EZIO

PURPOSE:  Contains TAE & LAS specific code for simplifing IO sequences

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT
-------		----	------		---------
  1.0		2/91	D. Steinwand	 AB/ISS
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "ezio.h"

#define MAX_IMG	     1
#define MAX_BND       1

char function_name[25]; /* Function name (as passed) */
char func_name[25];   	/* Function name (lower case) */
char name[1][CMLEN];	/* Output image TAE name */
char minmax[10];	/* Minmax flag */
long msgtim;		/* Message timer */
struct FDESC *outptr;
struct FDESC *inptr;

long inimage;
long outimage;

char hosin[MAX_IMG][CMLEN];
char hosout[1][CMLEN];

long nmrimg;
long window[MAX_IMG][4];
long bands[MAXIMG][MAXBND+1];
long nbands[MAX_IMG];
long totbnd;

long min_max;

double minval[MAX_BND];
double maxval[MAX_BND];

long in_image_band;
long in_image_line;
long in_image_ns;

long in_type;

long out_image_band;
long out_image_line;
long out_image_ns;

struct PARBLK vblk;		/* TAE parameter block */

IMPORT long curent, curcat;

unsigned char *init_buf
(
    long length,
    long dtype
);



/***************** Initialize LAS & TAE processing **************************/

void start
(
    char name[25]			/* Name of LAS function */
)
{
struct VARIABLE *var;		/* Pointer to a TAE variable in vblk */
long std_out;			/* Standard output (is this ever used?) */
long twentyfive=25;		/* Constant of 25 */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Copy function name to global for later use
  ------------------------------------------*/
strcpy(function_name, name);
strcpy(func_name, name);
c_up2low(func_name, &twentyfive);

/* Get TAE/LAS parameters and store in globals for later use
  ---------------------------------------------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);

var = p_find(&vblk, "$minmax");
strcpy(minmax, SVAL(*var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

outimage = FALSE;
inimage = FALSE;

curent = curcat = 1;

return;
}

/***************** Get the subcommand used **********************************/

void get_subcmd
(
    char *method
)
{
struct VARIABLE *var;           /* Pointer to a TAE variable in vblk */

var = p_find(&vblk, "_SUBCMD");
strcpy(method, SVAL(*var, 0));
return;
}

/***************** Get a string variable ************************************/

void get_string
(
    char parm[25],
    char *str
)
{
struct VARIABLE *var;           /* Pointer to a TAE variable in vblk */

var = p_find(&vblk, parm);
strcpy(str, SVAL(*var, 0));
return;
}

/***************** Get an integer variable **********************************/

int get_long
(
    char parm[25]
)
{
struct VARIABLE *var;           /* Pointer to a TAE variable in vblk */

var = p_find(&vblk, parm);
return(IVAL(*var, 0));
}

/***************** Get a floating point variable ****************************/

float get_float
(
    char parm[25]
)
{
struct VARIABLE *var;           /* Pointer to a TAE variable in vblk */

var = p_find(&vblk, parm);
return(RVAL(*var, 0));
}

/***************** Get a floating point vector ******************************/

void get_float_vector
(
    char parm[25],
    struct FVECTOR *vector
)
{
struct VARIABLE *var;           /* Pointer to a TAE variable in vblk */
long i;

var = p_find(&vblk, parm);
vector->dim = (*var).v_count;

vector->element = (float *)init_buf(vector->dim, FLOAT);

for (i = 0; i < vector->dim; i++)
   vector->element[i] = RVAL(*var,i);

return;
}

/***************** Close down processing & exit *****************************/

void finish
(void)
{
long num_out = 0;		/* Number of output images */
long comb_flag = 0;
double one = 1.0;

c_stoptm();

if (inimage) c_eclose(&inptr);
if (outimage) 
   {
   c_eclose(&outptr);
   num_out++;
   }

/* Update ddr and minmax and for output image
---------------------------------------------*/
c_upddr (&vblk, hosin, hosout[0], &nmrimg, window, bands,
         nbands, &one, &one, &totbnd, &comb_flag);

if (min_max) c_svmnmx (hosout[0], &totbnd, minval, maxval, "YES");

/* Create associated history file for output image
--------------------------------------------------*/
/* status = c_hstout(&vblk, hosout[0], hosin, &nmrimg, " "); */

/* Write completion message
  ------------------------*/
c_complt(function_name, name, &num_out);
}

/***************** Allocate buffer space ***********************************/

unsigned char *init_buf
(
    long length,
    long dtype
)
{
long size;
long pass = 1;
long fail = -1;
unsigned char *buffer;
char temp_str[80];

if (dtype == SHORT) size = (long) (length * sizeof(short));
else if (dtype == LONG) size = (long) (length * sizeof(long));
else if (dtype == FLOAT) size = (long) (length * sizeof(float));
else if (dtype == DOUBLE) size = (long) (length * sizeof(double));
else if (dtype == COMPLEX) size = (long) (length * sizeof(float) * 2);
else if (dtype == DCOMPLEX) size = (long) (length * sizeof(double) * 2);
else size = length; /* dtype == BYTE */

buffer = (unsigned char *)malloc(size);
if (buffer == NULL)
   {
   strcat(temp_str, func_name);
   strcat(temp_str, "-alloc");
   c_errmsg("Error allocating memory", temp_str, &pass); 
   strcat(temp_str, func_name);
   strcat(temp_str, "-fatal");
   c_errmsg("Fatal error encountered", temp_str, &fail); 
   }
return(buffer);
}
/***************** Initialize processing--current band *********************/

void init_band
(void)
{
static long first = 1;
static long imgcnt = -1;
static long bndcnt;
long acc = IREAD;
long zero = 0;
long one = 1;
long fail = -1;
long fbands;
long status;
char temp_str[80];

in_image_line = 1;
in_image_band = bands[imgcnt][bndcnt];

if (first) curent = curcat =  0;
if (first || (in_image_band == 0))
   {
   curcat++;
   imgcnt++;
   bndcnt = 0;
   in_image_band = bands[imgcnt][bndcnt];
   if (!first) c_eclose(&inptr);
   first = 0;
   fbands = 0;
   status = c_eopenr(&inptr, &(hosin[imgcnt][0]), &fbands, &window[imgcnt][0],
        &window[imgcnt][1], &window[imgcnt][2], &window[imgcnt][3], &acc, 
	&in_type, &zero, &one);
   if (status != E_SUCC)
      {
      strcat(temp_str, func_name);
      strcat(temp_str, "-fatal");
      c_errmsg("Fatal error encountered", temp_str, &fail);
      }
   inimage= TRUE;
   }

out_image_line = 1;
out_image_band++;

return;
}

/***************** Get an image line ***************************************/

void get_line
(
    unsigned char *buf
)
{
long status;
long one = 1;
char temp_str[80];

curent++;
status = c_eread (&inptr, &in_image_band, &in_image_line, buf, &one);
if (status  !=  in_image_ns)
   {
/*   c_eclose(&outptr); */
   strcat(temp_str, func_name);
   strcat(temp_str, "-fatal");
   c_abort(&vblk, "Fatal error encountered",temp_str,func_name,hosout,&one);
   }   
in_image_line++;
return;
}
/***************** Put an image line ***************************************/

void put_line
(
    void *buf
)
{
long status;
long one = 1;
char temp_str[80];

status = c_ewrite (&outptr, &out_image_band, &out_image_line, buf, &one);
if (status  !=  out_image_ns) 
   {
   c_eclose (&outptr);
   strcat(temp_str, func_name);
   strcat(temp_str, "-fatal");
   c_abort(&vblk, "Fatal error encountered",temp_str,func_name,hosout,&one);
   }
out_image_line++;
return;
}
/***************** Open the output image ************************************/

void init_outimg
(
    long nl,
    long ns,
    long nb,
    long odtype
)
{
char temp_str[80];
long fail= -1;
long one = 1;
long acc;
long otype;
long status;
long opt[2];

/* Set otype
  ---------*/
if (odtype == SHORT) otype = EWORD;
else if (odtype == LONG) otype = ELONG;
else if (odtype == FLOAT) otype = EREAL;
else if (odtype == DOUBLE)
   {
   otype = EREAL;
   ns *= 2;
   }
else if (odtype == COMPLEX)
   {
   otype = EREAL;
   ns *= 2;
   }
else if (odtype == DCOMPLEX)
   {
   otype = EREAL;
   ns *= 4;
   }
else otype = EBYTE; /* odtype == BYTE */


/* Get output file name
-----------------------*/
status = c_getout (&vblk, "OUT", name, hosout);
if (status  !=  E_SUCC)
   {
   strcat(temp_str, func_name);
   strcat(temp_str, "-fatal");
   c_errmsg ("Fatal error encountered", temp_str, &fail);
   }

/* Open the output image
  ---------------------*/
opt[0] = 0;
opt[1] = 0;
acc = IWRITE;
status=c_eopenr(&outptr,hosout[0],&nb,&one,&one,&nl,&ns,&acc,&otype,opt,&one);
if(status != E_SUCC)
   {
   strcat(temp_str, func_name);
   strcat(temp_str, "-fatal");
   c_errmsg ("Fatal error encountered", temp_str, &fail);
   }

outimage = TRUE;
out_image_band = 0;
out_image_ns = ns;

/* Initialize & start display message
------------------------------------*/
c_inittm(function_name,"Processed","line","band"," ", &nl, &nb, &msgtim, &one);
c_startm();

/* Initialize min and max arrays
--------------------------------*/
c_inmnmx (&otype, &nb, minval, maxval);

return;
}

/***************** Open the input image ************************************/

void init_inimg
(
    long *nl,
    long *ns,
    long *nb,
    long var_num,
    long dtype
)
{
char taein[MAX_IMG][CMLEN];
long same;
long qflag[MAX_IMG][2];
long pass = 1;
long fail = -1;
long maximg = MAX_IMG;
long status;
char temp_str[80];

var_num--;

/* Get input file information
-----------------------------*/
status = c_getin(&vblk, "IN", &var_num, taein, hosin, &nmrimg, window, bands,
                 nbands, &totbnd, &maximg, &same, qflag);
if(status != E_SUCC)
   {
   strcat(temp_str, func_name);
   strcat(temp_str, "-fatal");
   c_errmsg ("Fatal error encountered", temp_str, &fail);
   }

if (same  !=  1)
   {
   strcat(temp_str, func_name);
   strcat(temp_str, "-samewind");
   c_errmsg ("Input images must have same window sizes", temp_str, &pass);
   strcat(temp_str, func_name);
   strcat(temp_str, "-fatal");
   c_errmsg ("Fatal error encountered", temp_str, &fail);
   }


status = c_gettyp(hosin, &nmrimg, &dtype);
if (status  !=  E_SUCC)
   {
   strcat(temp_str, func_name);
   strcat(temp_str, "-sametype");
   c_errmsg ("Input images must be the same data type", temp_str, &pass);
   strcat(temp_str, func_name);
   strcat(temp_str, "-fatal");
   c_errmsg ("Fatal error encountered", temp_str, &fail);
   }

*nl = window[0][2];
*ns = window[0][3];
*nb = totbnd;

if (dtype == SHORT) in_type = EWORD;
else if (dtype == LONG) in_type = ELONG;
else if (dtype == FLOAT) in_type = EREAL;
else if (dtype == DOUBLE)
   {
   in_type = EREAL;
   *ns /= 2;
   }
else if (dtype == COMPLEX)
   {
   in_type = EREAL;
   *ns /= 2;
   }
else if (dtype == DCOMPLEX)
   {
   in_type = EREAL;
   *ns /= 4;
   }
else in_type = EBYTE; /* dtype == BYTE */

in_image_ns = *ns;

return;
}
