/*****************************************************************************
NAME:				FFT1

PURPOSE:  Performs a one-dimensional Fast Fourier Transformation on an image

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT 
-------		----	------		---------
  1.0		2/91	D. Steinwand	 AB/ISS	    

ALGORITHM DESCRIPTION:
	Initialize processing
	Initialize images and buffer space
	For each band
	  For each line
	    Get an image line
	    Perform 1-D FFT
	    Put a line of results to the output file
	Close down processing and exit
*****************************************************************************/
#include <stdlib.h>
#include "ezio.h"
#include "fft.h"
#include "protos_fft1.h"

int main
(void)
{
long nl,ns,nb;			/* Number of lines, samples, bands */
long out_ns;			/* Number of output image samples */
long i,j;			/* Loop counters */
unsigned char *inbuf;		/* Input buffer */
float *outbuf;			/* Output buffer */

/* Initialize processing
  ---------------------*/
start("FFT1");

/* Initialize images and buffer space
  ----------------------------------*/
init_inimg(&nl, &ns, &nb, 1, BYTE);
out_ns = nearest_power2(ns);
init_outimg( nl, out_ns, nb, COMPLEX);
inbuf = (unsigned char *)init_buf(ns, BYTE);
outbuf = (float *)init_buf(out_ns+1, COMPLEX);

/* Loop thru the input image(s), band-by-band, line-by-line
  --------------------------------------------------------*/
for (i = nb; i--;)
  {
  init_band();
  for (j = nl; j--;)
    {
    get_line(inbuf);
    pad_to_pow2(inbuf, ns, outbuf, out_ns);
    fft1d(outbuf-1, out_ns, FORWARD);
    put_line(outbuf);
    }
  }

/* Close down processing and exit
  ------------------------------*/
finish();

return EXIT_SUCCESS;

}
