/*****************************************************************************
NAME:                           FFT1D

PURPOSE:  Replaces the "buf" array by its discrete Fourier transform (or its
	  inverse discrete Fourier transform).

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT 
-------         ----    ------          --------- 
  1.0           2/91    D. Steinwand     AB/ISS

ALGORITHM REFERENCE:
   This routine was obtained from:  "Numerical Recipes, the Art of Scientific
      Computing", by Press, Flannery, Teukolsky, and Vetterling, Cambridge
      University Press, 1986.

*****************************************************************************/
#include <math.h>
#include "ezio.h"
#include "fft.h"

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void fft1d
(
    float *buf,		/* Input/output--Complex array of length 2 * dim */
    long dim,		/* Input--dimension of buffer.  Must be a power of 2 */
    long flag		/* Forward/inverse flag.  flag = 1 is forward, -1 inv */
)
{
long n,mmax,m,j,istep,i;
double wtemp,wr,wpr,wpi,wi,theta;
float tempr,tempi;

n=dim << 1;
j=1;

/* Perform bit reversal
  --------------------*/
for (i=1;i<n;i+=2) 
	{
	if (j > i) 
	   {
	   SWAP(buf[j],buf[i]);
	   SWAP(buf[j+1],buf[i+1]);
	   }
	m=n >> 1;
	while (m >= 2 && j > m) 
	   {
	   j -= m;
	   m >>= 1;
	   }
	j += m;
        }

/* The Danielson-Lanczos section of the routine begins here.  The outter
   loop is executed log2NN times
  -----------------------------*/
mmax=2;
while (n > mmax) 
	{
	istep=2*mmax;

/* Initialize for the trigonometric recurrence
  -------------------------------------------*/
	theta=6.28318530717959/(flag*mmax);
	wtemp=sin(0.5*theta);
	wpr = -2.0*wtemp*wtemp;
	wpi=sin(theta);
	wr=1.0;
	wi=0.0;
	for (m=1;m<mmax;m+=2) 
	   {
	   for (i=m;i<=n;i+=istep) 
		{

/* The Danielson-Lanczos formula...
  --------------------------------*/
		j=i+mmax;
		tempr=wr*buf[j]-wi*buf[j+1];
		tempi=wr*buf[j+1]+wi*buf[j];
		buf[j]=buf[i]-tempr;
		buf[j+1]=buf[i+1]-tempi;
		buf[i] += tempr;
		buf[i+1] += tempi;
		}
/* The trigonometric recurrence
  ----------------------------*/
	   wr=(wtemp=wr)*wpr-wi*wpi+wr;
	   wi=wi*wpr+wtemp*wpi+wi;
	   }
	mmax=istep;
        }
return;
}
