/*****************************************************************************
NAME:				FFT1FIL

PURPOSE:  Performs a one-dimensional Fast Fourier Transformation (FFT) on an 
	image, filters at the given frequencies, and performs an inverse
	FFT resulting in a filtered image.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT 
-------		----	------		---------
  1.0		2/91	D. Steinwand	 AB/ISS	    

*****************************************************************************/
#include <stdlib.h>
#include "ezio.h"
#include "fft.h"
#include "protos_fft1fil.h"

int main
(void)
{
struct FVECTOR freq;		/* Frequencies at which to filter */
long nl,ns,nb;			/* Number of lines, samples, bands */
long out_ns;			/* Number of output image samples */
long i,j;			/* Loop counters */
unsigned char *inbuf;		/* Input buffer */
float *outbuf;			/* Output buffer */
float *filter;			/* Filter buffer */
float size;			/* Notch size (in frequency) */

/* Initialize processing
  ---------------------*/
start("FFT1FIL");

/* Initialize images and buffer space
  ----------------------------------*/
init_inimg( &nl, &ns, &nb, 1, BYTE);
out_ns = nearest_power2(ns);
init_outimg(nl, ns, nb,BYTE);
inbuf = (unsigned char *)init_buf(ns, BYTE);
outbuf = (float *)init_buf(out_ns+1, COMPLEX);
filter = (float *)init_buf(out_ns+1, COMPLEX);
size = get_float("SIZE");
get_float_vector("FREQ", &freq);
create_filter(size, &freq, out_ns, filter);

/* Loop thru the input image(s), band-by-band, line-by-line
  --------------------------------------------------------*/
for (i = nb; i--;)
  {
  init_band();
  for (j = nl; j--;)
    {
    get_line(inbuf);
    pad_to_pow2(inbuf, ns, outbuf, out_ns);
    fft1d(outbuf-1, out_ns, FORWARD);
    apply_filter(outbuf, filter, out_ns);
    fft1d(outbuf-1, out_ns, INVERSE);
    pad_back(outbuf, out_ns, inbuf, ns);
    put_line(inbuf);
    }
  }

/* Close down processing and exit
  ------------------------------*/
finish();

return EXIT_SUCCESS;

}
