#include <math.h>
#include "ezio.h"
#include "fft.h"
#include "protos_fft1fil.h"

void create_filter
(
    double size,		/* Size of notch in frequencies */
    struct FVECTOR *freq,	/* Filter parameters */
    long dim,		/* Dimension of complex fft buffer */
    float *buf		/* Filter buffer */
)
{
long n;
long i;
float half;
long num_freqs;		/* Number of frequency components--inc. DC term */
long pixel;		/* Pixel number at which to create filter around */

n = dim << 1;
num_freqs = dim >> 1;	
half = size * num_freqs;

for (i = n; i--; buf[i] = 1.0);

for (i = 0; i < freq->dim; i++)
   {
   pixel = (long)(freq->element[i] * (float) num_freqs / 0.5 + 1.5);
   pack(pixel, half, buf);
   pixel = (long)((1.0 - freq->element[i]) * num_freqs / 0.5 + 1.5);
   pack(pixel, half, buf);
   }

return;
}

void pack
(
    long n,
    float half_dim,
    float *buf
)
{
long index;
double temp;
double minimum = 0.000001;
long x,i;

index = (n - 1) * 2;
buf[index] = 0.0;
buf[index+1] = 0.0;
for (x = 1, i = 2;; x++, i++)
   {
   temp = (double)x / half_dim;
   temp *= temp;
   temp /= -2.0;
   temp = 1.0 - exp(temp);
   if ((1.0 - temp) < minimum) break;
   buf[index+i] = buf[index-i] = temp;
   i++;
   buf[index+i] = buf[(index-i)+2] = temp;
   }

return;
}

void apply_filter
(
    float *buf,
    float *filter,
    long dim
)
{
long n;
long i;

n = dim << 1;
for (i = n; i--; buf[i] *= filter[i]); 
return;
}
