/***************************************************************************
NAME: DOBLOB

PURPOSE: Gathers regions for the pixels having the specified class number 
         and assigns a unique number to each region.  

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1            Jan-85   Y. Sheu
   2         18-Sep-89   K. Narayanan       Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Compare the current image line(work line) with the previous 
      line(reference line).
   Assign to the pixels in the work line the numbers in the reference 
      line for the contiguous regions.
   If merged regions are detected create a look-up table for 
      the higher class numbers.
   If no contigious regions are detected, assign new class numbers.

   Note: after this pass, the output image may have gaps in the assigned 
     numbers and the merged regions may possess more than
     one class numbers.

***************************************************************************/
#include "getblob.h"
#include "pixman.h"

void doblob
(
  long wind[],              /* I: Input image window */
  long *classnum,           /* O: Class to be searched */
  long lcntrs[],            /* O: No. classes in each line */
  short table[],            /* O: Table for merge */
  long *iter,               /* O: Flag for second pass */
  long *next                /* O: Last assigned class number */
)
{

long    zero = 0;
char    *prog_name = "GETBLOB"; /* program name            */

long	class, 		/* class number to be assigned to current region   */
	class_r, 	/* current class being processed in refrow         */
	class_w, 	/* current class being processed in workrow        */
	cntrs[2][2],	/* buffer holding edge & class counters for current*/
                  	/* strata line being processed in work array  	   */
	end_r, 		/* right edge (sample no.) of class currently      */
	                /* being processed in refrow of work array         */
	end_w, 		/* right edge (sample no.) of class currently      */
	                /* being processed in workrow of work array        */
	hicls,
	i, icount, ipt, 
	line, 
	line_end_r, 	/* end of line flag for refrow			   */
	lowcls, 
	numclass_w, 	/* number of non-background classes in workrow     */
	len_r, 		/* actual record length of refrow                  */
	len_w, 		/* actual record length of workrow                 */
	loc_r, 		/* pointer to class type currently being processed */
	                /* in refrow of work array                         */
	loc_w, 		/* pointer to class type currently being processed */
	                /* in workrow of work array                        */
	msg_flag = GEN2,
	row_r, 		/* pointer to refrow				   */
	row_w, 		/* pointer to workrow				   */
	start_r, 	/* left edge (sample no.) of class currently       */
	                /* being processed in refrow of work array         */
	start_w, 	/* left edge (sample no.) of class currently       */
	                /* being processed in workrow of work array        */
	temp, 
	otype = EWORD,
	work[2][400], 	/* circular buffer to process pairs of lines from  */
	                /* strata mask file                                */
	*wp;
	
short	newp;

void get_ref_class
(
  long work[],
  long *loc,
  long len,
  long *line_end,
  long *classnum,
  long *start,
  long *end
);

/*
  *                                                      *             
  *  search strata-mask 2 lines at a time for contiguous *
  *  regions and assign a new class number for each new  *
  *  region detected.                                    *

*/

/*
initialize buffer reference and work rows                      
-----------------------------------------
*/
row_r  = 0;                                                   
row_w = 1;                                                    
for (i = 0, wp = work[row_r]; i < 400; i++) *(wp++) = 0;
cntrs[row_r][0] =  cntrs[row_r][1] = 0; 
*iter = 0;
table[0] = 2;
class = 2;
*next = class;                                                 
len_r = cntrs[row_r][0] * 2 + 2;

/*
initialize and enable processing message utility
------------------------------------------------
*/
c_inittm (prog_name, "Processed", "line", "", " ", &wind[NL], 
          &zero, &msgtim, &msg_flag);
curent = 1;
c_startm();

for (line = 0; line < wind[NL]; line++, curent++)
   {

   c_estep (&gdesc_in);
   c_estep (&gdesc_out);

/*
generate strata mask data buffer for current image line
-------------------------------------------------------
*/
   strata (&wind[NS], *classnum, work[row_w], cntrs[row_w]);
/*
                                                                 
   get data counters                           
                                                              
*/
   len_w  = cntrs[row_w][0] * 2 + 2;                            
   numclass_w = cntrs[row_w][1];                              
   if (numclass_w != 0)
      {
/*
      compare refrow and workrow to gather regions                      
      --------------------------------------------
*/
/*
     initialize counters and logical flags
      ---------------------------------------
*/
      end_r  = 0;                                                      
      end_w  = 0;                                                
      line_end_r = FALSE;                                            
      loc_r = 1;                                                   
      loc_w = 1;                                                   

      get_ref_class (work[row_r], &loc_r, len_r, &line_end_r, &class_r, 
                     &start_r, &end_r);

      while (loc_w+1 < len_w) 
         {
         class_w = work[row_w][loc_w-1];  
         
         start_w = end_w + 1;                                     
         end_w  += work[row_w][loc_w];  

         if (class_w == -1)  
            {
            loc_w += 2;                                         
            continue;
            }

         if (!line_end_r)
            while (start_w > end_r+1)  
               {
/*
               current region boundary starts beyond current region in          
               reference line -- get next reference class                       
               -------------------------------------------------------
*/
               loc_r += 2;                                         
               get_ref_class (work[row_r], &loc_r, len_r, &line_end_r, &class_r, 
                              &start_r, &end_r);
               if (line_end_r)  break;
               }
         if (line_end_r || (end_w+1 < start_r))
            {
/*
            found end of reference line before end of work line or
            start of new region, so:  class_w = next                     
            load class_w into work buffer                                      
            ------------------------------------------------------
*/
            if (class_w  ==  1) 
               {
               if (*next  >  32767) *next = 32767;
               work[row_w][loc_w-1] = *next;                              
               table[*next-1] = (short) (*next);
               (*next)++;
               }
            loc_w += 2;
            continue;         
            }
/*
         overlaps current class in reference row                    
         ---------------------------------------
*/
         if (class_w != 1  &&  class_w != class_r) loc_w += 2;
         else
            {
            class_w = class_r;                                          
            work[row_w][loc_w-1] = class_w;                             
            loc_w += 2;                                      
/*
            blob growth upward -- treat like a merge
            see if trailing edge of region extends past reference region
            ------------------------------------------------------------
*/
            if (end_w  <=  end_r)  continue;
/*
            region extends beyond current boundary, look for merging    
            with another region                                         
            --------------------------------------------------------
*/
            loc_r += 2;                                    
            get_ref_class (work[row_r], &loc_r, len_r, &line_end_r, &class_r, 
                           &start_r, &end_r);
            if (line_end_r) continue;
            }
/*
         new reference class found, check for overlap with current   
         class in work buffer.                                       
         ---------------------------------------------------------
*/
         while (end_w+1  >=  start_r)  
            {
/*
            regions are merging -- find the smaller class number        
            ----------------------------------------------------
*/
            if (class_w >= class_r) 
               {
               lowcls = class_r;                                      
               work[row_w][loc_w-3] = lowcls;                       
               hicls = class_w;                                       
               }
            else
               {
               hicls = class_r;
               lowcls = class_w;
               }
/*
            save reference buffer and search previous lines of mask     
            for hicls and change to lowcls.                             
            -------------------------------------------------------
*/
            if (lowcls == -1)
               {
               lowcls = hicls;
               }
            table[hicls-1]=(short) (lowcls);      
            (*iter) ++;

            if (end_w <= end_r) break;
               {
               class_w = lowcls;                                      
               loc_r += 2;                                    
               get_ref_class (work[row_r], &loc_r, len_r, &line_end_r, &class_r, 
                       &start_r, &end_r);
               if (line_end_r) break;
               }
            }
         }
      }
/*
all classes identified in current line -- post reference buffer to mask 
file, switch buffers and process next line; composite the output image line
---------------------------------------------------------------------------
*/
   ipt = 0;
   icount = 0;
   if (cntrs[row_w][0]  ==  0) 
      {
      c_pxfill (&outbuf[opoint], &wind[NS], &otype, &zero); 
      table[0] = 0;
      }  

   else
      while ((newp = (short) (work[row_w][2*icount])) != 0)
         {
/*
         if not end of line either fill with zero or assigned numbers
         ------------------------------------------------------------
*/
         if (newp  ==  -1) 
            {
            c_pxfill (&outbuf[opoint+ipt], &otype, &work[row_w][2*icount+1], 
                      &zero);
            table[0] = 0;
            }
         else 
            c_pxfill (&outbuf[opoint+ipt], &otype, &work[row_w][2*icount+1], 
                      &newp); 

         ipt += work[row_w][2*icount+1];
         icount++;
         }

/*
   ref. row posted -- now flip row pointers and read next line     
   -----------------------------------------------------------
*/
   len_r = len_w;                                              
   lcntrs[line] = cntrs[row_w][0];
   temp = row_r;                                         
   row_r  = row_w;                                      
   row_w = temp;                                         
   }
(*next)--;

/*
close images and free buffers
-----------------------------
*/
c_estep (&gdesc_out);
c_stoptm ();
c_egclse (&gdesc_out);
free (inbuf);
free (outbuf);
}                            

/* Read new class information from refrow
-----------------------------------------*/
void get_ref_class
(
  long work[],
  long *loc,
  long len,
  long *line_end,
  long *classnum,
  long *start,
  long *end
)
{
while (TRUE)
   {
   if (*loc+1 >= len) 
      {
      *line_end = TRUE;
      return;
      }
   else
      {  
      *classnum = work[*loc-1];
      *start = *end + 1;                                        
      *end  += work[*loc];
      if (*classnum != -1)
        return;
      else *loc += 2;
      }
   }
}
