/***************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Sep-89  K. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   If iteration is zero 
      Open input and output images
      Update output image DDR and history
   Else 
      Reopen output image

***************************************************************************/
#include <string.h>
#include "getblob.h"

void filspec
(
  long iter,    
  char hos_in[][CMLEN],             /* I: Input image host name */
  char hos_out[],                   /* I: Output image host name */
  long image_band_num[][MAXBND+1],  /* I: Band numbers */
  long wind[][4]                    /* I: Input image window */
)
{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;


double	line_inc = 1.0,
	samp_inc = 1.0;

long	access,                      /* file access type                   */
	bands[2],
	bufsiz,                      /* image i/o buffer size              */
	comb_flag = COMB,            /* flag for upddr                     */
	stat,                        /* return status                      */
	type;                        /* file type                          */


if (iter == 0)
   {
/*
   open input image
   ----------------
*/
   access = IREAD;
   type = ELONG;

   stat = c_eopens (fdesc, hos_in[0], image_band_num[0], &wind[0][SL], 
                    &wind[0][SS], &wind[0][NL], &wind[0][NS], &ipoint, 
                    &access, &type, &zero, &zero);
   if (stat != E_SUCC )
      c_errmsg ("Error opening input image", "getblob-openin", &fatal_err);

   fdesc[1] = 0;

/*
   set up group for input image
   ----------------------------
*/
   if ((inbuf = (long *) c_ealloc(fdesc, &bufsiz)) == NULL)
      c_errmsg ("Error allocating dynamic memory", "getblob-alloc", &fatal_err);

   c_egroup (fdesc, &gdesc_in, inbuf, &bufsiz);
   }

/*
open output image - with update access if second iter
-----------------------------------------------------
*/
if (iter == 0) access = IWRITE;
else access = IUPDATE;

type = EWORD;
bands[0] = 1;
bands[1] = 0;
stat = c_eopens (fdesc, hos_out, bands, &one, &one,
                &wind[0][NL], &wind[0][NS], &opoint, &access, &type, 
                &zero, &zero);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "getblob-openout", &fatal_err);

fdesc[1] = 0;

/*
set up group for output image
-----------------------------
*/
if ((outbuf = (short *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_errmsg ("Error allocating dynamic memory", "getblob-alloc", &fatal_err);

c_egroup (fdesc, &gdesc_out, outbuf, &bufsiz);

/*
update output image DDR and history
-----------------------------------
*/
if (iter == 0)
   {
   c_upddr (&block, hos_in, hos_out, &one, wind, image_band_num, 
            &one, &line_inc, &samp_inc, &one, &comb_flag);
   c_hstout (&block, hos_out, hos_in, &one, "");
   strcpy (out_files[0], hos_out);
   num_outfiles = 1;
   }
}
