/***************************************************************************
NAME: GETBLOB

PURPOSE: Searches the input image for the specified class number and 
         assigns a unique number to each individual region.  


PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1          Jan-85   Y. Sheu
     2       18-Sep-89   K. Narayanan      conversion to 5.0
     3       24-Sep-92	 T. Mittan	   Modified include file to eliminate
					   warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get and validate parameters 
   Call FILSPEC to set up image file specifications
   Call DOBLOB to assign numbers for the regions
   Call MERGE to generate a sequential look-up table for the merged 
      regions and get the output image max and min values
   Update output image DDR max and min fields
   If a second pass is needed
      Call FILSPEC to reopen the output image
      Call BLOB2 to apply the sequential look-up table to re-assign
         the numbers on the output image.
   Issue completion message

****************************************************************************/
#include <stdlib.h>
#include "getblob.h"

  struct PARBLK block;           /* TAE parameter block */

  struct FDESC *fdesc[2];                 /* File descriptors */
  struct GDESC *gdesc_in;                 /* Group descriptor */
  struct GDESC *gdesc_out;                /* Group descriptor */
  long *inbuf;                   /* Image i/o buffer */
  long ipoint;                   /* Offsets into image i/o */
  long opoint;                   /* For input and output bands */
  long min_max;                  /* Minmax option flag */
  long msgtim;                   /* Message interval */
  long num_outfiles;             /* Number of output files */

  short *outbuf;                 /* Image i/o buffer */

  char out_files[1][CMLEN];         /* Output file names */

  extern long curcat;            /* Parameters for processing */
  extern long curent;            /* Message utility */

int main
(
  void
) 
{

long    one = 1;
char    *prog_name = "GETBLOB"; /* program name            */

  double maxval;                     /* Output image maximum pixel value */
  double minval;                     /* Output image minimum pixel value */

  long class_num;
  long image_band_num[1][MAXBND + 1];   /* Band numbers */
  long iter = 0;                     /* Flag for pass 2 */
  long last;                         /* Last index for table */
  long *lcntrs;
  long wind[1][4];                   /* Input image window */

  short lut[32768];                  /* Merge lookup table */
  short table[32768];                /* Table for merge information */

  char hos_in[1][CMLEN];                /* Input images host name */
  char hos_out[1][CMLEN];               /* Output image host name */
  char las_out[1][CMLEN];               /* Output image las name */

  /* Get parameters 
  -----------------*/
  getpar(hos_in,las_out,hos_out,image_band_num,wind,&class_num);

  lcntrs = (long *)malloc(sizeof(long) * wind[0][NL]);
  if (lcntrs == NULL)
    c_lasmsg(TRUE,"getblob-alloc","Error allocating memory");

  /* Open image files
  -------------------*/
  filspec(iter,hos_in,hos_out[0],image_band_num,wind);

  /* Pass 1 - assign different class numbers to pixels in
              different regions in each line, and build
              up a table for merging regions
  -------------------------------------------------------*/
  doblob(wind[0],&class_num,lcntrs,table,&iter,&last);

  /* Make a look-up table of sequential numbers
     for merged regions and find max and min pixel
     values; update output image ddr
  ------------------------------------------------*/
  if (iter > 0) 
    merge(table,lut,last,&maxval,&minval);
  else
  {
    minval = table[0];
    maxval = table[last-1];
  }  

  if (min_max)
    c_svmnmx(hos_out[0],&one,&minval,&maxval,"YES");

  /* Pass 2 - apply look-up table to assign lower
              class numbers to merged regions   
  -----------------------------------------------*/
  if (iter > 0) 
  {
    filspec(iter,hos_in,hos_out[0],image_band_num,wind);
    blob2(lcntrs,wind[0],lut);
  }  

  free(lcntrs);

  c_complt(prog_name,las_out,&one); 

return EXIT_SUCCESS;
  
}                                                
