#include "grid.h"

FUNCTION comcalc ( num_bands, wind, dx, dy, xo, yo, gtext,
                   dtype, hos_out, nmring)

long    num_bands,		/* total no. of bands */
        wind[][4],		/* processing window */
	dx,			/* delta x grid value */
	dy,			/* delta y grid value */
	xo,			/* x-origin */
	yo,			/* y-origin */
	nmring,			/* no. of images */
        dtype;			/* processing type */
float   gtext;			/* grid value */
char    hos_out[];		/* output host name */
/*
NAME: COMCALC 

FUNCTION:
  Performs gridding of the image

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    4.0       14-JUN-90   B. ALLAM, STX     Fixes
    3.0       30-MAR-90   J. MCGUTHRY	    Conversion
    2.0       20-APR-87   B. KOVALICK
    1.0       01-OCT-84   B. DEALY

ALGORITHM:
For each line
  For each band
    for each sample
      if sample falls on grid 
        replace sample with grid value in output buffer
      else 
        copy sample to output buffer

*/
{
long    msg_flag,		/* message flag */
        img, i, j, n,		/* counter variables */
	size,			/* number of bytes per input pixel */
        nyp, nxp, ix, jx;	/* temp variables */

rinbuf = (float *) buf;
routbuf = (float *) outbuf;

msg_flag = ONECAT;
switch(dtype)
{
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:	size = 4; break;
   case EREAL:  size = 4; break;
}
c_inittm ("GRID","Processed","line","bands"," ",&wind[0][NL],&num_bands,
           &msgtim, &msg_flag);
curent = 1;
curcat = 1;
c_startm();
curent = 1;
for (i=0; i < wind[0][NL];i++,curent++)
  {
  if (c_estep (&gdescout) == E_FAIL)
	c_abort(&vblock, "Error in image read/write", "grid_estep");
  if (c_estep (&gdesc) == E_FAIL)
	c_abort(&vblock, "Error in image read/write", "grid_estep");
  curcat=1;
  for(n=0; n < num_bands; n++, curcat++)
    {
/*
process each sample
------------------- */
    for (j=0;j < wind[0][NS]; j++)
      {
      nyp= (yo % dy);
      nxp= (xo % dx);
      ix=i-nyp+1;
      jx=j-nxp+1;
      if ((ix % dy) == 0)
        switch(dtype)
          {
          case EBYTE:  outbuf[(out[n]+j) * size] = (unsigned char)gtext; break;
          case EWORD:  outbuf[(out[n]+j) * size] = (short )gtext; break;
          case ELONG:  outbuf[(out[n]+j) * size] = (long )gtext; break;
          case EREAL:  routbuf[(out[n]+j)] = (float) gtext; break;
          }
      else if ((jx % dx) == 0)
        switch(dtype)
          {
          case EBYTE:  outbuf[(out[n]+j) * size] = (unsigned char)gtext; break;
          case EWORD:  outbuf[(out[n]+j) * size] = (short )gtext; break;
          case ELONG:  outbuf[(out[n]+j) * size] = (long )gtext; break;
          case EREAL:  routbuf[(out[n]+j)] = (float) gtext; break;
          }
      else
        switch(dtype)
          {
          case EBYTE:  outbuf[(out[n]+j) * size] = (unsigned char)buf[(in[n]+j) * size]; break;
          case EWORD:  outbuf[(out[n]+j) * size] = (short )buf[(in[n]+j) * size]; break;
          case ELONG:  outbuf[(out[n]+j) * size] = (long )buf[(in[n]+j) * size]; break;
          case EREAL:  routbuf[(out[n]+j)] = (rinbuf[in[n]+j]); break;
          }
      }
    } 
   }

if (c_estep (&gdescout) == E_FAIL)
	c_abort(&vblock, "Error in image read/write", "grid_estep");

c_stoptm();
c_egclse (&gdesc);
c_egclse (&gdescout);

free(buf);
free(outbuf);


}   
