#include "grid.h"

FUNCTION getparms (hos_in, hos_out, las_in, las_out, nmring,
                   wind, totbnd, dx, dy, xo, yo, gtext, dtype, bands)

long    *nmring,             /* number of images                 */
        *totbnd,             /* total nuber of bands             */
 	*dtype,		     /* input data type 		 */
	bands[][MAXBND +1],  /* band numbers 		 	 */
	wind[][4];           /* input image windows              */

char	hos_in[][CMLEN],     /* input images host names          */
	hos_out[],           /* output image host name           */
	las_out[],           /* output image las name            */
	las_in[][CMLEN];     /* output image las name            */
        
long    *dx,                 /* delta x grid value               */
	*dy,                 /* delta y grid value               */
        *xo,                 /* x-origin                         */
        *yo;                 /* y-origin                         */
         
float   *gtext;              /* grid value                       */
/*
NAME: GETPARMS

FUNCTION: Gets input and output parameters

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    4.0       14-JUN-90   B. ALLAM, STX     Fixes
    3.0       30-MAR-90   J. MCGUTHRY	    Conversion
    2.0       20-APR-87   B. KOVALICK
    1.0       01-OCT-84   B. DEALY

ALGORITHM:
	Get input and output parameters
        Get horizontal and vertical skip factors
        Get grid line intersection
        Get intensity value for grid lines
	Get global variables
*/
{
long    stat,                     /* return status                    */
        type,                     /* datatype                         */
        same_window,
        flag[2],
        imglimit = MAXIMG;	/* image limit */

struct VARIABLE *p_find(),
                *var;

static char minmax[4];		/* min/max option */

/*
get and validate input image parameters 
--------------------------------------- */

stat = c_getin (&vblock,"IN",&zero, las_in, hos_in, nmring, wind, bands, 
                nbands, totbnd, &imglimit, &same_window, flag);         
if (stat != E_SUCC) 
  c_errmsg ("Error accessing input image1 ", "grid-in", 
             &fatal_err);
c_gettyp( hos_in, nmring, dtype);

/*
get and validate output image parameters 
---------------------------------------- */
stat = c_getout (&vblock, "OUT", las_out, hos_out);
if (stat != E_SUCC)
  c_abort (&vblock, "Error getting output image name", "grid-out", 
           prog_name, " ", &zero);
/*
get horizontal and vertical skip factors
----------------------------------------*/
type = ELONG;
c_getnum (&vblock,"XSTEP",&one,&type,dx);
c_getnum (&vblock,"YSTEP",&one,&type,dy);

/*
get grid line intersection
--------------------------*/
stat = c_getnum (&vblock,"XORIGIN",&one,&type,xo);
if (stat <= 0 )
  *xo = *dx;

stat = c_getnum (&vblock,"YORIGIN",&one,&type,yo);
if (stat <= 0 )
  *yo = *dy;

/*
get intensity value for grid lines
----------------------------------*/
type = EREAL;
c_getnum (&vblock,"GRIDVAL",&one,&type,gtext);

/*
get global variables
--------------------*/
var = p_find( &vblock, "$msgtim");
msgtim = IVAL ( *var, 0);
var = p_find( &vblock, "$minmax");
strcpy(minmax, SVAL(*var,0));
if (minmax[0] == 'Y') min_max = TRUE;
else                  min_max = FALSE;

}
