
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "typlim.h"

struct PARBLK vblock;                  /* TAE parameter block     */

long    nbands[MAXIMG];
int     fdesc[MAXIMG];                 /* file descriptors        */
int     fdescout[2];                   /* output file descriptor  */
int     gdesc;                         /* group descriptor        */
int     gdescout;
long    msgtim;                  /* message interval        */
long    in[MAXIMG * MAXBND + 1],   /* offsets into image i/o  */
                out[MAXBND+1];              /* for input and output bands */
long    min_max;

unsigned char *buf;                  /* image i/o buffer        */
unsigned char *outbuf;               /* image i/o buffer        */
float           *rinbuf;             /* float input pointer     */
float           *routbuf;            /* float output pointer    */
static  char *prog_name = "grid";      /* program name            */

extern long     curcat;                        /* message utility         */
extern long     curent;                        /* parameters for processing */

static  long    fatal_err = -1;
static  long    zero = 0;
static  long    one = 1;

/**********************************************************************

NAME:  GRID

PURPOSE:
  Superimposes a grid on an input image

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    5.0	      24-SEP-92	  T. Mittan	    Modified include file to eliminate
					    warnings on link
    4.0       14-JUN-90   B. ALLAM, STX     Fixes
    3.0       30-MAR-90   J. MCGUTHRY	    Conversion
    2.0       20-APR-87   B. KOVALICK
    1.0       01-OCT-84   B. DEALY


ALGORITHM DESCRIPTION
	Initialize LAS
	Get input and output parameters
	Check data type for grid
	Set up input and output images
	Draw grid on image
        Update history file 
***********************************************************************
*/
main () 
{

long	nmring,                  /*   number of images               */
	totbnd,                  /*   total no. of bands 	     */
        dtype,                   /*   input data type 		     */
	wind[MAXIMG][4];         /*   input image windows            */

char    hos_in[MAXIMG][CMLEN];   /* input host names */
char    las_in[MAXIMG][CMLEN];   /* input las names */
char	hos_out[CMLEN]; 	 /* output host name */
char	las_out[CMLEN];		 /* output las name */


long    std_out;
long    dx,			/* delta x grid value */
	dy,			/* delta y grid value */
	xo,			/* x-origin */
	yo,			/* y-origin */
	type,			/* processing type */
	stat;
float   gtext;			/* grid value */
long  bands[MAXIMG][MAXBND+1];		/* band numbers */
long  out_img_bands[MAXIMG][MAXBND+1];		/* band numbers */

double                                                                     
	amax[MAXBND],		  /*    maximum pixel value        */
        amin[MAXBND];              /*    minimum pixel value        */   
/* 
Initialize LAS
--------------*/        
c_inlas (&vblock, &std_out);

/*
Get input and output parameters
------------------------------- */
getparms (hos_in, hos_out, las_in, las_out, &nmring, 
          wind, &totbnd, &dx, &dy, &xo, &yo, &gtext, &dtype, bands);
/*
check data type for grid
------------------------ */
dcheck (dtype, gtext);
/*
set up the files for imageio 
-----------------------------*/
filspec (hos_in, wind, nmring, totbnd, dtype, hos_out, bands,out_img_bands);

/*
draw grid on image
------------------ */
comcalc (totbnd, wind, dx, dy, xo, yo, gtext, dtype, hos_out,
         nmring);
/*
update output image ddr min max fields
-----------------------------------*/
   if ( min_max )
      {
      wind[0][SL] = wind[0][SS] = 1;
      c_minmax (hos_out, &one, wind, out_img_bands, 
      		  &totbnd, amin, amax); 
      c_svmnmx (hos_out, &totbnd, amin, amax, "YES");
      }
/*
Completion message
------------------ */
c_hstout(&vblock, hos_out, hos_in, &one," ");
c_complt ("GRID", las_out, &one);
}
