c      program IDIMSIN
c
c      purpose:  read idtrans tapes and create las images from
c                the image file and ascii files from any related
c                files
c
c  program history:
c
c  version   date     author           code/contr     change request
c    1.0     3/88     m. drefs           csb             
c    2.0     3/92     T. Mittan		 csb	      allow multiple band
c						      images, fixed minmax
c						      option
c
c  project:  landsat-d assessment system
c
c  language:   fortran-77
c
c  pdl:
c  begin
c    call inlas to initialize the program
c    call getpar to get parameters
c    call tpopen to open the tape
c    check the status of tpopen
c    if subcommand is list
c       if not end of tape or end of volume
c         call rdheadr to read the tape header
c         call prheadr to print the header to a file
c         call tpfsr to get to next header record
c       endif
c    else
c       for each file to be read
c         call tpfsf to skip to specified image file
c         check status of tpfsf
c         call rdheadr to read tape header
c         call getname to get tae and host names for the output image
c         if file is image 
c            if data type is byte 
c               call rbyte to read image file into output image
c            else if data type is i*2
c               call rword to read image file into output image
c            else if data type is i*4
c               call rlong to read image file into output image
c            else call rreal to read image file into output image
c            end if
c            call fgcre_group to create a cataloged file group
c            call hist_out to update image history
c            call minmax to update the min and max values in the output image
c                   files ddr
c         else 
c            if related files are specified
c              call getfile to read specified related files
c            else
c              call tpfsr to get to next header file
c            endif
c         endif
c    endif
c    call tpclos to close the tape
c    call complt to print a standard completion message
c    end


       program idimsin

	include 'las.fin'

	include 'tapeio.fin'

        integer*4 block(XPRDIM),stdout,maxsiz,nrcd,i,j,k,totalbnd,ou
        integer*4 tpopen, tpclos,tpfsf,status,volume,loop
        integer*4 count, unit, icount,msgtim
	integer*4 bands(MAXIMG,MAXBND + 1)
        integer*4 nfile,filid,bndno,idtype,ns,inum(50)
        integer*4 one, filcnt, imgtot, skip
	integer*4 window(4)
        real*8    minv(MAXBND), maxv(MAXBND)
    
        character*(CMLEN) hosname(MAXBND)
        character*(CMLEN) taename(MAXBND), coment, savecom
	character*(CMLEN) tname2(MAXBND), prtfil, filena
        character*(CMLEN) tapeid
        character*255 prtfil2
        character*36 name, lstname
	character*1 density
        character    subcmd*5, mnmx*4
        character    ftype*4
        logical*4    eov,term,file,lp

        integer*4 offset(MAXBND)
        integer*4 gdesc,fdesc(2)

        common/imgio/fdesc,gdesc,offset
        common/tape/ volume,density

        volume = 1
        eov = .false.
        lstname='      '
        stdout=6
        ou=23
        one= 1
	totalbnd = 0
        call inlas(block,stdout)
c
c  call getpar to receive subcommand and parameters
c
        call getpar(block,coment,term,file,lp,inum,ftype,subcmd,
     +              mnmx,icount,msgtim,prtfil,filena)
	savecom = coment
C	if (lp .or. file) then
C	  ou=23
	  prtfil2 = prtfil
C	endif
c
c  open the tape and check status of tpopen
c
        density = '0'
        status=tpopen(unit,0,HANDLE,TREWND,density,TREAD,coment,tapeid)
        if (status.lt.TSUCC) call errmsg ('error opening tape',
     +     'idimsin-open',-1)
c
c  if subcommand is list, call rdheadr and prheadr for each file 
c  to read and print the header from the tape
c
        if (subcmd .eq. 'list') then
100         continue
            call rdheadr(nfile,bndno,filid,idtype,ns,maxsiz,nrcd,
     +                   name,eov,unit)
c
c   keep count of total number of files and a count of number
c   of file groups to print out.
c
            count=count +1            
            if (name .ne. lstname) imgtot = imgtot +1
C            if (file .or. lp) 
              call prheadr(nfile,bndno,filid,idtype,ns,maxsiz,nrcd,
     +              name,lstname,count,ou,imgtot,savecom,prtfil2)
C            if (term)
C     +         call prheadr(nfile,bndno,filid,idtype,ns,maxsiz,nrcd,
C     +              name,lstname,count,stdout,imgtot,savecom,prtfil2)
            lstname=name
            status = tpfsf(unit,one,HANDLE)
            if (status .lt. TSUCC) then
               if (status .ne. 	TEOV) then
                   status = tpclos(unit,HANDLE,TRWOFL,one)
               call errmsg('error skipping files','idimsin-skip',-1)
               else 
                   eov=.true.
               endif
            endif
199         if (.not. eov)  go to 100
            write(ou,*) ' '
            write(ou,'(x,a,i4)') 'TOTAL NUMBER OF FILES = ',count
            close(ou)
        else
c 
c   subcommand is image.  skip forward to the 
c   image group containing the specified image in inum
c           
	    if (inum(1).gt.1) then
		do 600 k = 1, inum(1)-1
		call rdheadr(nfile,bndno,filid,idtype,
     +				ns,maxsiz,nrcd,name,eov,unit)
	 	skip = nfile
		status = tpfsf(unit,skip,HANDLE)
		if (status .ne. TSUCC) then
			status = tpclos(unit,HANDLE,TRWOFL,one)
		call errmsg('error skipping files','idimsin-skip',-1)
		endif
 600		continue
	    endif
            do 200 i = 1,icount
              
                call rdheadr(nfile,bndno,filid,idtype,
     +                                   ns,maxsiz,nrcd,name,eov,unit)
		filcnt = nfile
		do 250 j = 1, filcnt
		if (j .gt. 1) call rdheadr(nfile,bndno,filid,idtype,
     +					 ns,maxsiz,nrcd,name,eov,unit)
                  if (filid .eq. 15) then
c
c  call getnam to get tae and host names for the output file.  
c  then call rdimg to open these images and write image file into it.
c
                call getnam(taename,hosname,bndno,name,inum(i))      
                     if (idtype .eq. 1 .or. idtype .gt. 5) then
                        call rbyte(unit,ns,maxsiz,bndno,eov,taename,
     +                             hosname,msgtim)
 		        else if (idtype .eq. 2) then
			call rword(unit,ns,maxsiz,bndno,eov,taename,
     +				   hosname,msgtim)
			else if (idtype .eq. 3) then
			call rlong(unit,ns,maxsiz,bndno,eov,taename,
     +				   hosname,msgtim)
			else if (idtype .eq. 4) then
			call rreal(unit,ns,maxsiz,bndno,eov,taename,
     +				   hosname,msgtim)
			else if (idtype .eq. 5) then
		call errmsg('handling 64-bit real data is not implemented'
     +		,'idimsin-data',1)
	     	     endif
c
c Update history 
c files and min/max values.
c
                      call hstout(block,hosname,' ',0,' ')
                      if (mnmx .eq. 'YES') then
                        do 10 k = 1,bndno
                           bands(k,1) = k
 10		        continue
		        bands(1,bndno+1) = 0
			window(1) = 1
			window(2) = 1
			window(3) = maxsiz
			window(4) = ns
                 call minmax(hosname,1,window,bands,bndno,minv,maxv)
		      endif
		      do 300 k = 1,bndno
			totalbnd = totalbnd + 1
			tname2(totalbnd) = taename(k)
300		      continue
		      status=tpfsf(unit,one,HANDLE)
		      if (status .lt. TSUCC) then
		        if (status .ne. TEOV) then
			status = tpclos(unit,HANDLE,TRWOFL,one)
		call errmsg('error skipping files','idimsin-skip',-1)
		        endif
		      endif
                  else
c
c  file is an associated file.  if related files are wanted, call 
c  getfile to get related file.
c
                     if ((ftype(1:4) .eq. 'hist') .or.
     +		 	 (ftype(1:3) .eq. 'all')) then
                         call getfile(unit,filid,name,ns,ftype,nrcd)
			 status=tpfsf(unit,one,HANDLE)
			 if (status .lt. TSUCC) then
			   if (status .ne. TEOV) then
			   status = tpclos(unit,HANDLE,TRWOFL,one)
	    call errmsg('error skipping files','idimsin-skip',-1)
			   endif
			 endif
                     else
			 skip = 1
                         status= tpfsf(unit,skip,HANDLE)
                         if (status .lt. TSUCC) then
			   if (status .ne. TEOV) then
                           status = tpclos(unit,HANDLE,TRWOFL,one)
	    call errmsg('error skipping files','idimsin-skip',-1)
                           endif
			 endif
                     endif
                  endif
 250           continue
c
c	skip the middle files if not requested
c
	       if (i .lt. icount) then
		  if ((inum(i+1)) .gt. (inum(i)+1)) then
		  loop = (inum(i+1)) - (inum(i)+1)
		  do 500 k = 1, loop
		  call rdheadr(nfile,bndno,filid,idtype,
     +                         ns,maxsiz,nrcd,name,eov,unit)
		  skip = nfile
		  status = tpfsf(unit,skip,HANDLE)
			if (status .lt. TSUCC) then
			  status = tpclos(unit,HANDLE,TRWOFL,one)
	    call errmsg('error skipping files','idimsin-skip',-1)
			endif
 500		  continue 
		  endif
		endif
 200        continue
         endif
999      continue
         status = tpclos(unit,HANDLE,TRWOFL,one)
         if (subcmd .eq. 'list') then
            call complt('IDIMSIN',' ',0)
         else
            call complt('IDIMSIN',tname2,totalbnd)
         endif
         end

       
c
c  subroutine chkerr(status,unit,buff,bufsiz,eov)
c
c  purpose:  check the status of a call to tpread
c
c  arguments:
c    name         type   i/o     description
c    ----         ----   ---     -----------
c    status       i*4     i      return status code from tape i/o calls
c    unit         i*4     i      tape drive logical unit number
c    buff 	  byte    o      buffer to read record into
c    bufsiz       i*4     i      size of buff
c    eov          l*4     o      end of volume flag
c
c  file references: none
c
c  common block references:  none
c
c  subroutines called:
c    name           description
c    ----           -----------
c    nxtvol         dismounts and signals to mount next tape volume
c    
c    (tape i/o routines)
c    tpclos         close the tape device
c    tpread         read from a tape device
c
c    (las programmer's routines)
c    errmsg         write a standard format error message
c
c   non-fatal error conditions: none
c
c   fatal error conditions:
c      -'error reading tape'
c      -'eof encountered before end of image was reached
c
c*
c**pdl:
c  begin
c    if status is end of volume
c        set flag
c        call tpclos to dismount the tape
c    else if status is end of tape
c        call nxtvol to mount next tape volume
c    else if status is end of file
c        call errmsg with error message
c    else if status is not successful
c        call tpclos to dismount the tape
c        call errmsg with fatal error
c    endif
c    return
c**end


       subroutine chkerr (status,unit,buff,bufsiz,eov)

	include 'las.fin'

	include 'tapeio.fin'

	character*1 buff(MAXNS)

        integer*4  status, unit, bufsiz, tpclos, stat, one
        logical*4  eov

        one = 1
        if (status .eq. TEOV) then
               stat= tpclos(unit,HANDLE,TRWOFL,one)
               if (stat .lt. TSUCC) 
     +          call errmsg('error closing tape','idimsin-close',-1)
               eov= .true.
        else if(status .eq. TEOT) then
               call nxtvol(unit)
        else if (status .eq. TEEOF) then
               stat= tpclos(unit,HANDLE,TRWOFL,one)
               if (stat .lt. TSUCC) 
     +          call errmsg('error closing tape','idimsin-close',-1)
       call errmsg('eof encountered before end of image was reached'
     +          ,'idimsin-eof',-1)
        else if (status .lt. TSUCC) then
               stat= tpclos(unit,HANDLE,TRWOFL,one)
               if (stat .lt. TSUCC) 
     +          call errmsg('error closing tape','idimsin-close',1)
               call errmsg('error reading tape','idimsin-read',-1)
        end if
        
        return
        end


c
c  subroutine getfile(unit,filid,name,ns,ftype)
c
c  purpose:  create ascii files from the image-related files on tape 
c          
c  arguments:
c     name	type   i/o	description
c     ----	----   ---	-----------
c     unit	 i*4    i	tape device logical unit number
c     filid	 i*4    i	id number signalling type of file
c                               2=cpar,4=hequ,6=gcpt,7=hist,10=site
c                               11=lsat,12=mens,18=stat,19=tset,23=mask
c     ns	 i*4    i       1/2 size of records in file
c     filname	 c*36   i       name of the image
c     ftype      c*4    i       type of related file specified by user
c      
c  file references:
c
c  common block references: none
c
c  subroutines called:
c    name	description
c    ----	-----------
c    chkerr	check status following tpread
c
c    (tape i/o routines)
c    tpread	read from a tape device
c
c*
c**pdl:
c  begin
c    open ascii file with name according to file type
c    call tpread to read the tape
c    write buffer into file
c    return
c**end



        subroutine getfile(unit,filid,filname,ns,ftype,nrcd)

	include 'las.fin'

	include 'tapeio.fin'

	character*1 buff(1024)

        integer*4 unit,filid, ou, ns, bufsiz,
     +            tpread, nmlen, stat, i, j,nrcd
        character filtyp*5,filname*41, ftype*4
        character cbuff*(1024)
        logical*4 eov

        equivalence(cbuff,buff)

c
c  determine file name according to file id and open file with that name
c
        if (filid .eq. 2) then
           filtyp = '.cpar'
        else if (filid .eq. 4) then
           filtyp= '.hequ'
        else if (filid .eq. 6) then
           filtyp= '.gcpt'
        else if (filid .eq. 7) then
           filtyp= '.hist'
        else if (filid .eq. 10) then
           filtyp= '.site'
        else if (filid .eq. 11) then
          filtyp= '.lsat'
        else if (filid .eq. 12) then
          filtyp= '.mens'
        else if (filid .eq. 18) then
           filtyp= '.stat'
        else if (filid .eq. 19) then
           filtyp= '.tset'
        else if (filid .eq. 23) then
           filtyp= '.mask'
        else  
           filtyp= '.unkn'
        endif
        if ((ftype(1:4) .eq. 'hist' .and. filtyp(2:5).eq.'hist')
     +             .or. (ftype(1:3) .eq. 'all')) then
           ou = 23
	do 300 i = 1,41
	  if (filname(i:i).eq.' ') then
	     nmlen = i
	     go to 301
	  endif
 300    continue
 301    continue
	   do 400 i = 1,nmlen-1
		if (filname(i:i) .eq. '.')
     +    	    filname(i:i) = '_'
400	   continue
           filname(nmlen:nmlen+4) = filtyp(1:5)
           nmlen = nmlen+5
           open(unit=ou, file=(filname(1:nmlen)),status='unknown')
c
c  read the tape and copy file off the tape into the ascii file
c
	   bufsiz=1024
	   do 200 j = 1,nrcd
             stat = tpread(unit,buff, bufsiz, HANDLE)
             if ((stat.lt.TSUCC).and.(stat.ne.TEREAD)) 
     +              call chkerr(stat,unit,buff,bufsiz,eov)
	     do 100 i = 0, stat, 80
               write(ou,'(a)') cbuff(i+1:i+80)
100	     continue
200        continue
        endif
        close(ou)
        return
        end


c
c  subroutine getpar(block,coment,term,file,lp,inum,ftype,subcmd,mnmx,
c                    icount,msgtim,prtfil,filena)
c
c  purpose:  determine the subcommand being run and receive the
c            parameters
c
c  arguments:
c     name         type  i/o   description
c     ----         ----  ---   -----------
c     block        i*4    i    variable block for the parameters
c     coment       c*100  o    description of tape
c     term         i*4    o    flag-terminal is output destination
c     file         i*4    o    flag -file is output destination
c     lp           i*4    o    flag-line printer is output destination
c     inum(50)     i*4    o    images to read in
c     ftype        c*4    o    type of related files to read
c     subcmd       c*5    o    subcommand the program was run under
c     mnmx         c*3    o    compute min/max flag
c     icount       i*4    o    number of images specified
c     msgtim       i*4    o    value of message timing
c
c  file references:  none
c
c  common block references:  none
c
c  subroutines called:
c     name           description
c     ----           -----------
c     (tae routines)
c     xrstr          get a string variable from the parameter
c     xrintg         get an integer variable from the parameter
c
c  pdl:
c  begin
c     call xrstr to receive the subcmd program was run under
c     call xrstr to receive parameter comment
c     if subcmd is list
c         call xrstr to receive parameter print
c         call setprint to set integer values from print
c     else
c         call xrstr to receive parameter oftype
c         call xrintg to receive parameter imagenum
c         call xrstr to receive min max flag ($minmax)
c         call xrstr to receive update history flag ($histry)
c     endif
c     return
c     end
c

       subroutine getpar(block,coment,term,file,lp,inum,ftype,subcmd,
     *                   mnmx,icount,msgtim,prtfil,filena)

	include 'las.fin'

	include 'tapeio.fin'

       integer*4   block(XPRDIM),inum(50),lng,count,stat, icount,
     +             msgtim
       logical*4   term,file,lp
       character*4 ftype
       character   mnmx*4, subcmd*5
       character*(CMLEN) coment
       character*(CMLEN) filena
       character*(CMLEN) prtfil

       call xrstr(block,'_SUBCMD',1,subcmd,lng,count,stat)
       call up2low(subcmd,5)
       call xrstr(block,'COMMENT',1,coment,lng,count,stat)
       if (subcmd .eq. 'list') then
	    prtfil = 'IDIMSIN'
            call setprt(block,term,file,lp,prtfil,filena)
       else
           call xrstr(block,'OFTYPE',1,ftype,lng,count,stat)
           call up2low(ftype,4)
           call xrintg(block,'IMAGENUM',50,inum,icount,stat)
           call xrstr(block,'$MINMAX',1,mnmx,lng,count,stat)
           call xrintg(block,'$MSGTIM',1,msgtim,count,stat)
       endif
       return
       end
       

c
c  subroutine getnam(taename,hosname,bndno,name)
c
c  arguments:  
c    name        type     i/o     description
c    ----        ----     ---     -----------
c    taename     c*(*)     o      the tae name of the output image(s)
c    hosname     c*(*)     o      the host names of the output images
c    bndno       i*4       i      number of bands in output image
c    name        c*36      i      the name of the image from tape
c
c*
c**pdl
c  begin
c    if one band in image
c       call xbfile to create host name
c    else
c       create group name
c    endif
c    return
c  end
c
 

       subroutine getnam(taename,hosname,bndno,name,icnt)
	include 'las.fin'

	include 'tapeio.fin'

      
       character name*36, msg*55
       character*(*) taename, hosname
       character bicnt*4
       integer*4 bndno,istat, lng, i
       integer*4 aflag, icnt, lng2
       integer*4 cmhost,mkimg

       aflag = 0
       lng=0
	do 300 i = 1,36
	  if (name(i:i).eq.' ') then
	     lng = i
	     go to 301
	  endif
 300    continue
 301    continue
 	taename = name(1:lng)
        istat = cmhost(taename(1:lng),hosname)
	   if (istat .eq. -1) then
	      msg='bad file name; '
	      lng2 = 16 + lng
	      msg(16:lng2) = name(1:lng)
	      call errmsg(msg,'idimsin-name',1)
	      name(1:36) = ' '
	      name(1:8) = 'idims.in'
              bicnt = char(icnt)
	      name(9:12) = bicnt
		do 100 i = 9,12
		if (name(i:i).eq.' ') name(i:i)='0'
 100		continue
	      msg='the new name is '
	      msg(17:17+12)=name(1:12)
	      call errmsg(msg,'idimsin-name',1)
	      lng = 12
 	   taename = name(1:lng)
           istat = cmhost(taename(1:lng),hosname)
	   endif 
c       open (status='new',file=hosname,unit=1)
c       close (unit=1,status='delete')
       istat = mkimg(name(1:lng),' ',taename(1:lng),hosname,1)
       return
       end 


c
c  subroutine nxtvol(unit)
c
c  purpose:  dismount the current tape, signal the user and operator
c            to load the next tape and then mount the new tape.
c
c  arguments:  
c    name	type  i/o	description
c    ----	----  ---	-----------
c    unit        i*4   i        tape drive logical unit number
c
c  file references:  none
c
c  common block references:
c    name	field	type	i/o	description
c    ----	-----	----	---	-----------
c    tape       volume,
c               density  i*4     i/o    tape volume number
c
c  subroutines called:
c    name 	description
c    ----	-----------
c    (tape i/o routines)
c    tpclos	close the tape device and dismount
c    tpopen	open the tape device and mount
c
c  non-fatal error conditions: none
c
c  fatal error conditions:
c    error opening tape
c    error closing tape
c
c*
c**pdl:
c  begin
c    call tpclos to rewind,close the tape drive unit and dismount
c    increment the tape volume counter by one
c    write message signaling to mount the next tape volume
c    call tpopen to mount and open the tape drive unit
c    return
c  end
c

        subroutine nxtvol(unit)
	include 'las.fin'

	include 'tapeio.fin'

 
        integer*4 tpclos,tpopen,volume,unit,status
        character mesg*35, cvol*4
 	character*1 density
	character*(CMLEN) tapeid

        common/tape/ volume,density

        data mesg/'please mount next tape volume      '/

        status = tpclos(unit,HANDLE,TRWOFL,0)
        if (status .lt. TSUCC)
     +   call errmsg('error closing tape','idimsin-close',-1)
        volume=volume +1
        write(cvol,'(i4)') volume
        mesg(31:35) = cvol
        status=tpopen(unit,0, HANDLE, TNOREW, density,TREAD,mesg,tapeid)
        if (status .lt. TSUCC)
     +   call errmsg('error opening tape','idimsin-open',-1)
        return
        end


c
c  subroutine prheadr(nfile,bndno,filid,idtype,ns,maxsiz,nrcd
c                    name,filnum,ou,imgtot,savecom,prtfil2)
c  purpose:  print the header to a file.
c
c  arguments:
c    name	type  i/o	description
c    ----	----  ---	-----------
c    nfile      i*4    i        number of files for this image
c    bndno	i*4    i	number of bands in image
c    filid	i*4    i	id to tell type of file
c    idtype	i*4    i        data type of file (if image)
c    ns		i*4    i        number of samples in image
c    maxsiz	i*4    i        number of lines(if image file) or
c                               max no. of record(non-image file)
c    nrcd	i*4    i        actual number of records in non-image file
c    name       c*36   i        name of file on tape
c    lstname    c*36   i        name of previous file on tape
c    filnum     i*4    i        number of file being read (tape relative)
c    ou         i*4    i        logical unit number for output destination
c    imgtot     i*4    i        the number of the file group being read
c    savecom    c*100  i        comment block to retain tape number
c
c*
c**pdl:
c   begin
c     open file idimsin.pr
c     format header information
c     write header information to the file
c     return
c**end
c

        subroutine prheadr(nfile,bndno,filid,idtype,ns,nl,nrcd,
     +             name,lstname,filnum,ou,imgtot,savecom,prtfil2)

	include 'las.fin'

	include 'tapeio.fin'

        integer*4  nfile,bndno,filid,idtype,ns,nl,nrcd, ou,filnum,
     +             imgtot
        character  name*36, lstname*36
        character*255 prtfil2
        character*(CMLEN) savecom
        character*4 cpar,hequ,gcpt,hist,site,lsat,mens,stat,tset,mask,
     +              unkn, img,filtyp

        data cpar/'cpar'/,hequ/'hequ'/,gcpt/'gcpt'/,hist/'hist'/
        data site/'site'/,lsat/'lsat'/,mens/'mens'/,stat/'stat'/
        data tset/'tset'/,mask/'mask'/,unkn/'unkn'/, img/'imag'/

        if (filnum .eq. 1) then
C           if (ou .eq. 23) 
C     +       open(unit=ou, file=prtfil2, status='unknown')
           open(unit=ou, file=prtfil2, status='unknown')
           write(ou,'(a)') ' IDTRANS TAPE DESCRIPTION'
	   write(ou,'(2x,a)') savecom(1:51)
	   write(ou,'(2x,a)') savecom(51:100)
           write(ou,*) ' '
           write(ou,'(a)') 
     +            ' ALL FILES CONTAIN 1 HEADER RECORD 100 BYTES '//
     +             'LONG (NOT INCLUDED IN #RECS)'
           write(ou,*) ' '
           write(ou,'(2x,a,3x,a,5x,a,3x,a,3x,a,4x,a,4x,a,3x,a)') 'IMG',
     +             'NAME','FILE','DATA','#BANDS','NL','NS OR','#RECS'
           write(ou,'(2x,a,12x,a,3x,a,19x,a)') 'NUM','TYPE','TYPE',
     +               'RECSZ'
           write(ou,*) ' '
        endif           
c        
c  find type of file based on file prefix number in header information
c        
        if (filid .eq. 2) then 
            filtyp=cpar
        else if (filid .eq.4) then
            filtyp=hequ
        else if (filid .eq. 6) then
            filtyp=gcpt
        else if (filid .eq. 7) then
            filtyp=hist
        else if (filid .eq. 10) then 
            filtyp=site
        else if (filid .eq. 11) then
            filtyp=lsat
        else if (filid .eq. 12) then
            filtyp=mens
        else if (filid .eq. 15) then
            filtyp=img
        else if (filid .eq. 18) then
            filtyp=stat
        else if (filid .eq. 19) then
            filtyp=tset
        else if (filid .eq. 23) then
            filtyp=mask
        else
            filtyp=unkn
        endif
     
c
c  write selected header information to file
c
        if (name.ne. lstname) then
           write(ou,'(2x,i3,3x,a)') imgtot, name(1:36)
           write(ou,*) ' '
        endif
        if (filid .eq.15) then
c          nrcd=bndno * nl 
          write(ou,'(17x,a,3x,i4,5x,i3,3x,i5,3x,i5,3x,i5)')
     +              filtyp(1:4),idtype,bndno,nl,ns,nrcd
        else 
c          ns= ns*2
          write(ou,'(17x,a,26x,i5,3x,i5)') filtyp(1:4),ns,nrcd
        endif
        write(ou,*) ' '
        return
        end


c 
c  subroutine rdheadr(nfile,bndno,filid,idtype,ns,maxsiz,nrcd
c                     name,eov,unit)
c
c  purpose:  read the header for a file on tape
c
c  arguments: 
c    name	type  i/o	description
c    ----	----  ---	-----------
c    unit       i*4    i        tape device logical unit number
c    nfile      i*4    o        number of files for this image
c    bndno	i*4    o	number of bands in image
c    filid	i*4    o	id to tell type of file
c    idtype	i*4    o        data type of file (if image)
c    ns		i*4    o        number of samples in image
c    maxsiz	i*4    o        number of lines(if image file) or
c                                max no. of record(non-image file)
c    nrcd	i*4    o        actual number of records in non-image file
c    title      c*8    o        string showing it is an idtrans tape
c    name       c*36   o        name of image on tape
c    eov        l*4    o        end of volume flag
c
c
c*
c**pdl:
c   begin
c     call tpread to read header of file
c     change i*2 variables to i*4 variables to use later
c   return
c**end
c


       subroutine rdheadr(nfile,bndno,filid,idtype,ns,maxsiz,nrcd,
     +                    name,eov,unit)

	include 'las.fin'

	include 'tapeio.fin'

	character*1 buff1(CMLEN)

       integer*2  totfile, fnum, nobnd, dtype, size, nofile
       integer*4  unit, ns,bndno, nfile,filid,idtype, flag, bufsiz
       integer*4  status, tpread, msiz, rtot, maxsiz, nrcd
       character  title*8, label*36, name*36
       logical*4  eov

        equivalence(title,buff1(1))
        equivalence(totfile,buff1(9))
        equivalence(fnum,buff1(11))
        equivalence(nobnd,buff1(13))
        equivalence(dtype,buff1(15))
        equivalence(msiz,buff1(17))
        equivalence(rtot,buff1(21))
        equivalence(size,buff1(25))
        equivalence(nofile,buff1(35))
        equivalence(label,buff1(65))

        bufsiz=100
        status= tpread(unit,buff1, bufsiz, HANDLE)
        if (status.lt.TSUCC) call chkerr(status,unit,buff1,bufsiz,eov)
        if (eov) go to 999

        nrcd=rtot
        maxsiz=msiz
        name=label
        call up2low(name,36)
        call up2low(title,8)
        nfile= totfile
        filid= fnum
        bndno=nobnd
        idtype=dtype
        if (size .ge. 0) then
          ns = size
        else
          ns = -size
        endif
        flag=nofile
        if (filid.eq.15) nrcd = bndno*maxsiz
999     continue
        return
        end


c
c  subroutine rbyte(unit,ns,nl,nobnd,eov,taename,hosname,
c                   msgtim)
c
c  purpose:  performs the image i/o routines for a byte image. check 
c            for enough space on disk, initializes the file action 
c            group and copies data into las image.
c
c  arguments:
c    name       type   i/o    description
c    ----       ----   ---    -----------
c    unit       i*4     i     tape device logical unit number
c    nobnd      i*4     i     number of bands in the output iamge
c    nl         i*4     i     number of lines in image
c    ns         i*4     i     number of samples in image
c    taename    c*100   i     tae names of the output image
c    hosname    c*80    i     host names of the output image
c    eov        l*4     o     end of volume flag
c    msgtim     i*4     i     value of message timing $msgtim
c
c
c  file references:  none
c 
c  common block references: 
c       common/iocom/buffer(1)
c       common/curent/curlin
c       common/curcat/curbnd
c   
c*
c**pdl:
c  begin
c    for each band of image
c       for each line in image
c           call tpread to read image line from tape
c           call fstep to set pointer
c       call endfag to clear fagid
c
c    return
c**end
c

       subroutine rbyte(unit,ns,nl,nobnd,eov,taename,
     +                  hosname,msgtim)
	include 'las.fin'

	include 'tapeio.fin'

       character*4    buffer(MAXNS)

       integer*4 offset(MAXBND)
       integer*4 unit, ns,nl, nobnd, bufsiz, status
       integer*4 dtype, one, curlin, curbnd, j, i, 
     +           tpread, msgtim
       integer*4 eopenr,
     +		 ewrite,eclose,gdesc,fdesc(2)
       integer   itemp
       character*(CMLEN) taename(MAXBND)
       character*(CMLEN) hosname(MAXBND)
       character temp(4*MAXNS)
       logical eov

       common/iocom/buffer
       common/curent/curlin
       common/curcat/curbnd
       common/imgio/fdesc,gdesc,offset
       data one/1/

       bufsiz = MAXNS
       dtype= 1

       curbnd=1
       curlin = 1
       call inittm('IDIMSIN','Processed','line','band',' ',nl,nobnd,
     +             msgtim,ONECAT)
       call startm
c
c   check to make sure there is enough room on the disk for output image
c   initialize processing 
c
c  open and create an image file on disk and set a new file action group id
c
	fdesc(2) = 0
	   status=eopenr(fdesc,hosname,nobnd,1,1,nl,ns,IWRITE,
     +		EBYTE,0,1)
c
       do 300 j=1,nobnd
          curlin=1
          curbnd=j
          do 200 i = 1, nl
            status = tpread(unit,buffer,bufsiz,HANDLE)
            if (status.lt.TSUCC) then
              write(temp) (buffer(itemp), itemp=1, bufsiz)
              call chkerr(status,unit,temp,bufsiz,eov)
            endif
            if (eov .eqv. .true.) go to 999
c
c  write buffers to las output image
c

           status = ewrite(fdesc,j,i,buffer,1)
           curlin= curlin+1
200      continue
300   continue
      status = eclose(fdesc)
999   continue
      call stoptm      
      return
      end

c
c  subroutine rword(unit,ns,nl,nobnd,eov,taename,hosname,
c                   msgtim)
c
c  purpose:  performs the image i/o routines for an i*2 image. check 
c            for enough space on disk, initializes the file action 
c            group and copies data into las image.
c
c  arguments:
c    name       type   i/o    description
c    ----       ----   ---    -----------
c    unit       i*4     i     tape device logical unit number
c    nobnd      i*4     i     number of bands in the output iamge
c    nl         i*4     i     number of lines in image
c    ns         i*4     i     number of samples in image
c    taename    c*100   i     tae names of the output image
c    hosname    c*80    i     host names of the output image
c    eov        l*4     o     end of volume flag
c    msgtim     i*4     i     value of message timing $msgtim
c
c
c  file references:  none
c 
c  common block references: 
c       common/iocom/buffer(1)
c       common/curent/curlin
c       common/curcat/curbnd
c   
c*
c**pdl:
c  begin
c    for each band of image
c       for each line in image
c           call tpread to read image line from tape
c           call lcopy_w to copy line into buffer
c           call fstep to set pointer
c       call endfag to clear fagid
c
c    return
c**end
c

       subroutine rword(unit,ns,nl,nobnd,eov,taename,
     +                  hosname,msgtim)
	include 'las.fin'

	include 'tapeio.fin'

       integer*4 offset(MAXBND)
       integer*4 buffer(MAXNS)
       integer*4 unit, ns,nl, nobnd, bufsiz, status
       integer*4 dtype, one, curlin, curbnd, j, i, 
     +           tpread, msgtim
       integer*4 ewrite,
     +	  	 eclose,eopenr,gdesc,fdesc(2)
       integer   itemp
       character*(CMLEN) taename(MAXBND)
       character*(CMLEN) hosname(MAXBND)
       character temp(4*MAXNS)
       logical eov

       common/iocom/buffer
       common/curent/curlin
       common/curcat/curbnd
       common/imgio/fdesc,gdesc,offset

       data one/1/

       dtype= 2

       curbnd=1
       curlin = 1
       call inittm('IDIMSIN','Processed','line','band',' ',nl,nobnd,
     +             msgtim,ONECAT)
       call startm
c
c  open and create an image file on disk and set a new file action group id
c
	   fdesc(2) = 0
           status=eopenr(fdesc,hosname,nobnd,1,1,nl,ns,IWRITE,
     +          EWORD,0,1)
c	   status=egroup(fdesc,gdesc,buffer,MAXNS)
c
       do 300 j=1,nobnd
          curlin=1
          curbnd=j
c
c
c  set the file size to the window size, set the data set reference
c  number, initialize the file action group, and set the output pointers to
c  the image file
	  bufsiz= MAXNS
          do 200 i = 1, nl
            status = tpread(unit,buffer,bufsiz,HANDLE)
            if (status.lt.TSUCC) then
              write(temp) (buffer(itemp), itemp=1, bufsiz)
              call chkerr(status,unit,temp,bufsiz,eov)
            endif
            if (eov .eqv. .true.) go to 999

c
c  write buffers to las output image
c
	   status = ewrite(fdesc,j,i,buffer,1)
           curlin= curlin+1
200      continue
300   continue
      status = eclose(fdesc)
999   continue
      call stoptm      
      return
      end

c
c  subroutine rlong(unit,ns,nl,nobnd,eov,taename,hosname,
c                   msgtim)
c
c  purpose:  performs the image i/o routines for an i*4 image. check 
c            for enough space on disk, initializes the file action 
c            group and copies data into las image.
c
c  arguments:
c    name       type   i/o    description
c    ----       ----   ---    -----------
c    unit       i*4     i     tape device logical unit number
c    nobnd      i*4     i     number of bands in the output iamge
c    nl         i*4     i     number of lines in image
c    ns         i*4     i     number of samples in image
c    taename    c*100   i     tae names of the output image
c    hosname    c*80    i     host names of the output image
c    eov        l*4     o     end of volume flag
c    msgtim     i*4     i     value of message timing $msgtim
c
c
c  file references:  none
c 
c  common block references: 
c       common/iocom/buffer(1)
c       common/curent/curlin
c       common/curcat/curbnd
c   
c*
c**pdl:
c  begin
c    for each band of image
c       for each line in image
c           call tpread to read image line from tape
c           call lcopy_fu to copy line into buffer
c           call fstep to set pointer
c       call endfag to clear fagid
c
c    return
c**end
c

       subroutine rlong(unit,ns,nl,nobnd,eov,taename,
     +                  hosname,msgtim)
	include 'las.fin'

	include 'tapeio.fin'

       integer*4 offset(MAXBND)
       integer*4 buffer(MAXNS)
       integer*4 unit, ns,nl, nobnd, bufsiz, status
       integer*4 dtype, one, curlin, curbnd, j, i, 
     +           tpread, msgtim
       integer*4 ewrite,
     +	  	 eclose,eopenr,gdesc,fdesc(2)
       integer   itemp
       character*(CMLEN) taename(MAXBND)
       character*(CMLEN) hosname(MAXBND)
       character temp(4*MAXNS)
       logical eov

       common/iocom/buffer
       common/curent/curlin
       common/curcat/curbnd
       common/imgio/fdesc,gdesc,offset

       data one/1/

       dtype= 3

       curbnd=1
       curlin = 1
       call inittm('IDIMSIN','Processed','line','band',' ',nl,nobnd,
     +             msgtim,ONECAT)
       call startm
c
c  open and create an image file on disk and set a new file action group id
c
	   fdesc(2) = 0
           status=eopenr(fdesc,hosname,nobnd,1,1,nl,ns,IWRITE,
     +          ELONG,0,1)
c
       do 300 j=1,nobnd
          curlin=1
          curbnd=j
c
c  set the file size to the window size, set the data set reference
c  number, initialize the file action group, and set the output pointers to
c  the image file
	  bufsiz=MAXNS
          do 200 i = 1, nl
            status = tpread(unit,buffer,bufsiz,HANDLE)
            if (status.lt.TSUCC) then
              write(temp) (buffer(itemp), itemp=1, bufsiz)
              call chkerr(status,unit,temp,bufsiz,eov)
            endif
            if (eov .eqv. .true.) go to 999

c
c  write buffers to las output image
c
           status = ewrite(fdesc,j,i,buffer,1)
           curlin= curlin+1
200      continue
300   continue
      status = eclose(fdesc)
999   continue
      call stoptm      
      return
      end

C
C  subroutine rreal(unit,ns,nl,nobnd,eov,taename,hosname,
C                   msgtim)
C
C  purpose:  performs the image i/o routines for a real*4 image. check 
C            for enough space on disk, initializes the file action 
C            group and copies data into las image.
C
C  arguments:
C    name       type   i/o    description
C    ----       ----   ---    -----------
C    unit       i*4     i     tape device logical unit number
C    nobnd      i*4     i     number of bands in the output iamge
C    nl         i*4     i     number of lines in image
C    ns         i*4     i     number of samples in image
C    taename    c*100   i     tae names of the output image
C    hosname    c*80    i     host names of the output image
C    eov        l*4     o     end of volume flag
C    msgtim     i*4     i     value of message timing $msgtim
C
C
C  file references:  none
C 
C  common block references: 
C       common/iocom/buffer(1)
C       common/curent/curlin
C       common/curcat/curbnd
C   
C*
C**pdl:
C  begin
C    for each band of image
C       for each line in image
C           call tpread to read image line from tape
C           call lcopy_re to copy line into buffer
C           call fstep to set pointer
C       call endfag to clear fagid
C
C    return
C**end
C

       subroutine rreal(unit,ns,nl,nobnd,eov,taename,
     +                  hosname,msgtim)
	include 'las.fin'

	include 'tapeio.fin'

C#include "sysdef.mfi"

       real*4    buffer(MAXNS)
       integer*4 offset(MAXBND)
       integer*4 unit, ns,nl, nobnd, bufsiz, status
       integer*4 dtype, one, curlin, curbnd, j, i, 
     +           tpread, msgtim
       integer*4 ewrite,
     +	  	 eclose,eopenr,gdesc,fdesc(2)
       integer   itemp
       character*(CMLEN) taename(MAXBND)
       character*(CMLEN) hosname(MAXBND)
       character temp(4*MAXNS)
       logical eov

       common/iocom/buffer
       common/curent/curlin
       common/curcat/curbnd
       common/imgio/fdesc,gdesc,offset

       data one/1/

       dtype= 4

       curbnd=1
       curlin = 1
       call inittm('IDIMSIN','Processed','line','band',' ',nl,nobnd,
     +             msgtim,ONECAT)
       call startm
c
c  open and create an image file on disk and set a new file action group id
c
	   fdesc(2) = 0
           status=eopenr(fdesc,hosname,nobnd,1,1,nl,ns,IWRITE,
     +          EBYTE,0,1)
c
       do 300 j=1,nobnd
          curlin=1
          curbnd=j
c
c  set the file size to the window size, set the data set reference
c  number, initialize the file action group, and set the output pointers to
c  the image file
	  bufsiz=MAXNS
          do 200 i = 1, nl
            status = tpread(unit,buffer,bufsiz,HANDLE)
            if (status.lt.TSUCC) then
              write(temp) (buffer(itemp), itemp=1, bufsiz)
              call chkerr(status,unit,temp,bufsiz,eov)
            endif
            if (eov .eqv. .true.) go to 999

c
c  write buffers to las output image
c
            status = ewrite(fdesc,j,i,buffer,1)
            curlin= curlin+1
200      continue
300   continue
      status = eclose(fdesc)
999   continue
      call stoptm      
      return
      end
