C*****************************************************************************
C 	NAME: ASCAVHRR
C
C	FUNCTION:
C	ASCOUT reads an IBIS graphics or tabular file and outputs the 
C	data to a file.
C
C	PROGRAM HISTORY:
C	Version		Date	Author		Request
C	-------		----	------		-------
C	   5.1          9/91    B.Ailts		Original development -- Taken
C						from ASCOUT
C
C	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C			Run under TAE 
C
C	PROJECT:	LAMS
C
C	ALGORITHM DESCRIPTION:
C
C	ALGORITHM REFERENCES:
C	Originally developed under the name ASCOUT
C
C******************************************************************************
      program ascavhrr

	include 'las.fin'

	include 'lams.fin'

	integer*4 col(MMXCOL)
	integer*4 count
	integer*4 ncol
	integer*4 nrows
      	integer*4 parb(XPRDIM)
	integer*4 srow
	integer*4 stdout
	integer*4 status

	character*86 format
	character*(CMLEN) taeout

C  Initialize variables
C----------------------
	nrows = MMXROW

C  retreive the TAE parameters
C-----------------------------
       	call inlas(parb,stdout)

        format = '(i5,2(1X,i3),x,f6.2,x,f7.2,x,i1,4(1x,i3)
     .,x,f5.1,x,5(1x,i3),x,i4)'

C  Call the subroutine required by the code in the input file
C------------------------------------------------------------
	col(1) = 7
	col(2) = 3
	col(3) = 4
	col(4) = 5
	col(5) = 6
	col(6) = 8
	col(7) = 9
	col(8) = 10
	col(9) = 11
	col(10) = 12
	col(11) = 13
	col(12) = 14
	col(13) = 15
	col(14) = 16
	col(15) = 17
	col(16) = 18
	col(17) = 2
        ncol = 17

	call xrintg(parb,'SROW',1,srow,count,status)
	call xrintg(parb,'NROWS',1,nrows,count,status)
	call wrtab(ncol,parb,format,col,nrows,srow,taeout)
		
	call complt('ASCOUT',taeout,1)

	end
C******************************************************************************
	subroutine wrtab(ncol,parb,format,col,nrows,srow,taeout)

	include 'las.fin'

	include 'lams.fin'

	real*4 data(MMXCOL)

	integer*4 unit,ncol,clen,srow,nrows,tcol,nc,nr,col(1)
	integer*4 rec,line
	integer*4 getnam
	integer   fdesc
      	integer*4 parb(1)
	integer*4 status
	integer*4 tbopen,tbclos,tbrrow
	integer*4 c1
	integer*4 c2
	integer*4 c3
	real*4 c4
	real*4 c5
	integer*4 c6
	integer*4 c7
	integer*4 c8
	integer*4 c9
	integer*4 c10
	real*4 c11
	integer*4 c12
	integer*4 c13
	integer*4 c14
	integer*4 c15
	integer*4 c16
	integer*4 c17

	character*(CMLEN) hosin,hosout,taeout
	character*86 format

	unit = 4

       	status = getnam(parb,'OUTFILE',1,hosout,'txt',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','ascavhrr-fatal',-1)
	end if

C  Fortran corriage control characters do not work well through VAX NCP
C  so the output file needs to be opened on the VAX with no carriage 
C  control characters
C-------------------------------------------------------------------

	open (unit,file=hosout,status='new',err=25)

	go to 26
25	continue
	   call errmsg('Error opening output file','ascavhrr-open',1)
	   close(unit)
	   call clean(parb,'ascavhrr',hosout,1)
26	continue

       	status = getnam(parb,'INFILE',1,hosin,'ibis_tab',IREAD)
	if (status .ne. ESUCC) then
	   close(unit)
	   call clean(parb,'ascavhrr',hosout,1)
	end if

C  Open input file
C-----------------
	status = tbopen(fdesc,hosin,clen,tcol,0)
	if (status .lt. ESUCC) then
	   call errmsg('Error opening input file','ascavhrr-open',1)
	   close(unit)
	   call clean(parb,'ascavhrr',hosout,1)
	end if
	nc = min0(ncol,tcol)
	nr = min0(clen,nrows)
	rec = srow

C  Read and write on a line by line basis
C----------------------------------------
	do 30 line = 1,nr
	   status = tbrrow(fdesc,col,nc,rec,data)
	   if (status .lt. ESUCC) then
	      call errmsg('Error reading input file','ascavhrr-read',1)
	      close(unit)
	      call clean(parb,'ascavhrr',hosout,1)
	   end if
	   rec = rec + 1
	   c1 = data(1)
	   c2 = data(2)
	   c3 = data(3)
	   c4 = data(4)
	   c5 = data(5)
	   c6 = data(6)
	   c7 = data(7)
	   c8 = data(8)
	   c9 = data(9)
	   c10 = data(10)
	   c11 = data(11)
	   c12 = data(12)
	   c13 = data(13)
	   c14 = data(14)
	   c15 = data(15)
	   c16 = data(16)
	   c17 = data(17)
	   write (unit,format,err=150,iostat=status) c1,c2,c3,c4,c5,c6,
     .            c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17
30	continue
	go to 160
150	continue
           print *,'Error number = ',status
	   call errmsg('Error writing to output file',
     .     	       'ascavhrr-write',1)
	   close(unit)
	   call clean(parb,'ascavhrr',hosout,1)
160	continue
	status = tbclos(fdesc,hosin)
	close (unit)

C  Build the tae name for the output file
C----------------------------------------
        taeout = hosout

	return
	end
