C*****************************************************************************
C 	NAME: ASCIN
C
C	FUNCTION:
C	ASCIN reads a text file and outputs the 
C	data to an IBIS graphics or tabular file.
C
C	PROGRAM HISTORY:
C	Version		Date	Author		Code/conr	Request
C	-------		----	------		---------	-------
C	   1.0					   CSB		original
C	   2.0		1/87	B.AILTS		   CSB		LAMS conversion
C	   3.0		6/87	B.AILTS		   CSB		Remove GR-2 
C								option
C	   4.0		2/88	B.Ailts		   CSB		LAS 5.0 con.
C
C	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C			Run under TAE 
C
C	PROJECT:	LAMS
C
C	ALGORITHM DESCRIPTION:
C	The tae parmaters are read in.  These parameters consist of:
C		1. INFILE--input file -- created by ASCOUT
C		2. OUTFILE--output file
C
C	The INFILE file is opened  and the data	is read and written 
C	to the text file that is specified by OUTFILE. 
C
C	After all the data is written, the completion message is 
C	displayed.
C
C	ALGORITHM REFERENCES:	none
C
C******************************************************************************
      program ascin

	include 'las.fin'

	include 'lams.fin'

      	integer*4 getnam

      	integer*4 parb(XPRDIM)

	integer*4 stdout
	integer*4 status
	integer*4 unit

	character*(CMLEN) hosin
	character*(CMLEN) taeout

	character*4 iform

C  Initialize variables
C----------------------
	unit = 4

C  retreive the TAE parameters
C-----------------------------
       	call inlas(parb,stdout)

       	status = getnam(parb,'INFILE',1,hosin,'txt',IREAD)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','ascin-fatal',-1)
	end if

	open (unit,file=hosin,status='old',err=20)

	go to 21
20	continue
	   call errmsg('Error opening input file','ascin-open',1)
	   call errmsg('Fatal error encountered','ascin-fatal',-1)
21	continue
	rewind (unit)

	read (unit,'(A4)',err=40) iform
C  call subroutine that is specified by the code of the first line of infile
C---------------------------------------------------------------------------
	if (iform .eq. ' GR') then
		call wrgraph1(parb,unit,taeout)
	else if (iform .eq. ' TAB') then
		call wrtab(parb,unit,taeout)
	else
	  call errmsg ('The input file does not have the correct header'
     .			,'ascin-header',1)
	   call errmsg('Fatal error encountered','ascin-fatal',-1)
	end if

	go to 41
40	continue
	   call errmsg('Error reading input file','ascin-read',1)
	   call errmsg('Fatal error encountered','ascin-fatal',-1)
41	continue

	call complt('ASCIN',taeout,1)

	end
C******************************************************************************
	subroutine wrgraph1(parb,unit,taeout)

	include 'las.fin'

	include 'lams.fin'

	real*4 rbuf(MEMXPT)

	integer*4 getnam
	integer*4 i
	integer   fdesc
	integer*4 gropen
	integer*4 grwrit
	integer*4 ip
	integer*4 iu
	integer*4 nb
	integer*4 nw

      	integer*4 parb(1)

	integer*4 rec
	integer*4 status
	integer*4 unit

	character*(CMLEN) hosout
	character*(CMLEN) taeout
	
C  Get name of output file
C-------------------------
       	status = getnam(parb,'OUTFILE',1,hosout,'ibis_graph',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','ascin-fatal',-1)
	end if

	read (unit,*,err=150) iu,nw
	Nb = Nw

C  Open output file
C------------------
	status = gropen(fdesc,hosout,nb,iu,1)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','ascin-open',1)
	   close (unit)
	   call grclos(fdesc,hosout)
	   call clean(parb,'ascin',hosout,1)
	end if
	REC = 1

C  Read input file and write data to output file on a line by line basis
C-----------------------------------------------------------------------
	DO 201 I=1,IU
	read(unit,*,err=150)(RBUF(IP),IP=1,NW)
	status = grwrit(fdesc,rec,RBUF)
	if (status .ne. ESUCC) then
	   call errmsg('Error writing to the output file',
     .    	       'ascin-write',1)
	   close (unit)
	   call grclos(fdesc,hosout)
	   call clean(parb,'ascin',hosout,1)
	end if
	REC = REC+1
 201	CONTINUE
	go to 160
150	continue
	   call errmsg('Error reading input file','ascin-read',1)
	   close (unit)
	   call grclos(fdesc,hosout)
	   call clean(parb,'ascin',hosout,1)
160	continue

	close (unit)
	CALL grclos(fdesc,hosout)

C  Build tae name for output file
C--------------------------------
        taeout = hosout

	return
	end
C******************************************************************************
	subroutine wrtab(parb,unit,taeout)

	include 'las.fin'

	include 'lams.fin'

	real*4 data(MMXCOL)

	integer*4 clen
	integer*4 col(MMXCOL)
	integer*4 getnam
	integer*4 i
	integer   fdesc
	integer*4 line
	integer*4 maxcol

      	integer*4 parb(1)

	integer*4 rec
	integer*4 status
	integer*4 tbopen
	integer*4 tbwrow
	integer*4 tcol
	integer*4 unit

	character*(CMLEN) hosout
	character*(CMLEN) taeout

       	status = getnam(parb,'OUTFILE',1,hosout,'ibis_tab',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','ascin-fatal',-1)
	end if
	read (unit,*,err=150) clen,tcol

C  Read in the columns that will be written to
C---------------------------------------------
	read (unit,*,err=150) (col(i),i=1,tcol)
	rec = 1

C  Find the largest column number
C--------------------------------
	maxcol = 0
	do 10 i = 1,tcol 
	   maxcol = max(col(i),maxcol)
10	continue

C  Open output file
C------------------
	status = tbopen(fdesc,hosout,clen,maxcol,1)
	if (status .lt. ESUCC) then
	   call errmsg('Error opening output file','ascin-open',1)
	   call tbclos(fdesc,hosout)
	   close (unit)
	   call clean(parb,'ascin',hosout,1)
	end if

C  Read input file and write data to output file on a line by line basis
C-----------------------------------------------------------------------
	do 30 line = 1,clen
	   read (unit,*,err=150) (data(i),i = 1,tcol)
	   status = tbwrow(fdesc,col,tcol,rec,data)
	   if (status .lt. ESUCC) then
	      call errmsg('Error writing to output file',
     .     		  'ascin-write',1)
	      call tbclos(fdesc,hosout)
	      close (unit)
	      call clean(parb,'ascin',hosout,1)
	  end if
	  rec = rec + 1
30	continue
	go to 160
150	continue
	   call errmsg('Error reading input file','ascin-read',1)
	   call tbclos(fdesc,hosout)
	   close (unit)
	   call clean(parb,'ascin',hosout,1)
160	continue

	call tbclos(fdesc,hosout)
	close (unit)

C  Build tae name for output file
C--------------------------------
        taeout = hosout

	return
	end
