C*****************************************************************************
C 	NAME: ASCNEW
C
C	FUNCTION:
C	ASCNEW reads an IBIS graphics or tabular file and outputs the 
C	data to a file.
C
C	PROGRAM HISTORY:
C	Version		Date	Author		Code/conr	Request
C	-------		----	------		---------	-------
C	   1.0					   CSB		original
C	   2.0		1/87	B.AILTS		   CSB		LAMS conversion
C	   3.0		6/87	B.AILTS		   CSB		Remove GR2 
C								option
C	   5.0		2/89	B.Ailts		   CSB		LAS 5.0 conver.
C	   5.1		3/97	D.Etrheim		  Combine DDP versions
C							  into one version
C
C	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C			Run under TAE 
C
C	PROJECT:	LAMS
C
C	ALGORITHM DESCRIPTION:
C	The tae parmaters are read in.
C
C	The INFILE file is opened  and the data	is read and written 
C	to the text file that is specified by OUTFILE. 
C
C	After all the data is written, the completion message is 
C	displayed.
C
C	ALGORITHM REFERENCES:	none
C
C******************************************************************************
      program ascnew

	include 'las.fin'

	include 'lams.fin'

	integer*4 col(MMXCOL)
	integer*4 count
	integer*4 i
	integer*4 len
	integer*4 ncol
	integer*4 nrows

      	integer*4 parb(XPRDIM)

	integer*4 srow
	integer*4 stdout
	integer*4 status

	character*86 format,form
	character*10 subcmd
	character*(CMLEN) taeout

C	Place the large variables in a common block to work around a
C	recent change in the sgi compiler that causes core dumps when
C	large variables are declared on the stack
	common parb


C  Initialize variables
C----------------------
	nrows = MMXROW

C  retreive the TAE parameters
C-----------------------------
       	call inlas(parb,stdout)

       	call xrstr(parb,'_SUBCMD',1,subcmd,len,count,status)

	if (subcmd .eq. 'INT') then
       	   call xrstr(parb,'OUTFORM',1,form,len,count,status)
	   if (form(1:1) .ne. '(') then
	      format = '(1X,'//form(1:len)//')'
	   else
	      format = form(1:1)//'1X,'//form(2:len)
	   end if
        else if (subcmd .eq. 'STAT') then
	      format = '(I11,2(F13.7),2(I11),F13.7,3(I11))'
        else if (subcmd .eq. 'CSTAT') then
	      format = '(I6,2(F12.6),2(I4),F12.6,2(I4),I3)'
        else if (subcmd .eq. 'CNTY') then
	      format = '(I4,F8.2,I4,F8.3,2(I4),F8.2,I4)'
        else if (subcmd .eq. 'AGGR') then
	      format = '(I4,2(F12.6),2(I4),I11)'
        else if (subcmd .eq. 'MID') then
	      format = '(i5,2(1X,i3),x,i1,x,f6.2,x,f7.2,x,i1,
     .        4(1x,i3),x,f5.1,5(1x,i3),x,i5,x,i1)'
	else if (subcmd .eq. 'NIS') then
	      format = '(i5,2(1X,i3),x,f6.2,x,f7.2,x,i1,4(1x,i3)
     .        ,x,f5.1,5(1x,i3),x,i4,x,i1)'
	endif

C  Call the subroutine required by the code in the input file
C------------------------------------------------------------
	if (subcmd .eq. 'INT') then
	   call xrintg(parb,'COLNUM',35,col,ncol,status)
        else if (subcmd .eq. 'STAT') then
	    ncol = 9
            col(1) = 1
            col(2) = 4
            col(3) = 5
            col(4) = 7
            col(5) = 6
            col(6) = 9
            col(7) = 8
            col(8) = 3
            col(9) = 10
        else if (subcmd .eq. 'CSTAT') then
	    ncol = 9
            col(1) = 1
            col(2) = 4
            col(3) = 5
            col(4) = 7
            col(5) = 6
            col(6) = 9
            col(7) = 8
            col(8) = 10
            col(9) = 11
        else if (subcmd .eq. 'CNTY') then
	    ncol = 8
            col(1) = 1
            col(2) = 4
            col(3) = 10
            col(4) = 5
            col(5) = 7
            col(6) = 6
            col(7) = 9
            col(8) = 8
        else if (subcmd .eq. 'AGGR') then
	    ncol = 6
            col(1) = 1
            col(2) = 4
            col(3) = 5
            col(4) = 7
            col(5) = 6
            col(6) = 3
        else if (subcmd .eq. 'MID') then
            ncol = 19
            col(1) = 7
            col(2) = 3
            col(3) = 4
            col(4) = 17
            col(5) = 5 
            col(6) = 6
            col(7) = 8
            col(8) = 9
            col(9) = 10
            col(10) = 11
            col(11) = 12
            col(12) = 13
            col(13) = 14
            col(14) = 15
            col(15) = 16
            col(16) = 18
            col(17) = 19
            col(18) = 2
            col(19) = 20
        else if (subcmd .eq. 'NIS') then
            ncol = 18
            col(1) = 7
            col(2) = 3
            col(3) = 4
            col(4) = 5
            col(5) = 6
            col(6) = 8
            col(7) = 9
            col(8) = 10
            col(9) = 11
            col(10) = 12
            col(11) = 13
            col(12) = 14
            col(13) = 15
            col(14) = 16
            col(15) = 17
            col(16) = 18
            col(17) = 2
            col(18) = 20
        endif

C  if COLNUM equals null then all columns should be printed.
C------------------------------------------------------------
	   if (ncol .le. 0) then
	      do 10 i = 1,35
	         col(i) = i
10	      continue
	      ncol = 35
	   end if
	   call xrintg(parb,'SROW',1,srow,count,status)
	   call xrintg(parb,'NROWS',1,nrows,count,status)
           
	   call wrtab(ncol,parb,format,col,nrows,srow,taeout,subcmd)
		
	call complt('ASCNEW',taeout,1)

	end
C******************************************************************************
	subroutine wrgraph1(parb,taeout,form)

	include 'las.fin'

	include 'lams.fin'

	real*4 rbuf(MEMXPT)

	integer*4 getnam
	integer*4 i
	integer   fdesc
	integer*4 ip
	integer*4 iu
	integer*4 nw

      	integer*4 parb(1)

	integer*4 rec
	integer*4 status
	integer*4 gropen,grclos,grread
	integer*4 unit

	character*(CMLEN) hosin
	character*(CMLEN) hosout
	character*(CMLEN) taeout

	character*86 form
	
	unit = 4
C  Get the output file name
C--------------------------
       	status = getnam(parb,'OUTFILE',1,hosout,'txt',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered'
     .                ,'ascnew-fatal',-1)
	end if

C  Fortran corriage control characters do not work well through VAX NCP
C  so the output file needs to be opened on the VAX with no carriage 
C  control characters
C-------------------------------------------------------------------

	open (unit,file=hosout,status='new',err=20)

	go to 21
20	continue
	   call errmsg('Error opening output file','ascnew-open',1)
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
21	continue

       	status = getnam(parb,'INFILE',1,hosin,'ibis_graph',IREAD)
	if (status .ne. ESUCC) then
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
	end if

	status = gropen(fdesc,hosin,nw,iu,0)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','ascnew-open',1)
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
	end if
	REC = 1

C  write header information to output file
C-----------------------------------------
	write (unit,'(A3)',err=150) ' GR'
	write (unit,'(A,2I8)',err=150) ' ',iu,nw

C  Read and write data
C---------------------
	DO 201 I=1,IU
	status = grread(fdesc,rec,RBUF)
	if (status .ne. ESUCC) then
	   call errmsg('Error reading input file','ascnew-read',1)
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
	end if
	REC = REC+1
	WRITE(unit,form,err=150)(RBUF(IP),IP=1,NW)
 201	CONTINUE
	go to 160
150	continue
	   call errmsg('Error writing to output file',
     .    	       'ascnew-write',1)
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
160	continue
	close (unit)
	status = grclos(fdesc,hosin)

C  Build the tae name for the output file
C----------------------------------------
        taeout = hosout

	return
	end
C******************************************************************************
	subroutine wrtab(ncol,parb,format,col,nrows,srow,taeout,subcmd)

	include 'las.fin'

	include 'lams.fin'

	real*4 data(MMXCOL)

	character*10 subcmd
	integer*4 unit,ncol,clen,i,srow,nrows,tcol,nc,nr,col(1)
	integer*4 rec,line
	integer*4 getnam
	integer   fdesc

      	integer*4 parb(1)

	integer*4 status
	integer*4 tbopen,tbclos,tbrrow

	character*(CMLEN) hosin,hosout,taeout
	character*86 format

	unit = 4

       	status = getnam(parb,'OUTFILE',1,hosout,'txt',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','ascnew-fatal',-1)
	end if

C  Fortran corriage control characters do not work well through VAX NCP
C  so the output file needs to be opened on the VAX with no carriage 
C  control characters
C-------------------------------------------------------------------

	open (unit,file=hosout,status='new',err=25)

	go to 26
25	continue
	   call errmsg('Error opening output file','ascnew-open',1)
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
26	continue

       	status = getnam(parb,'INFILE',1,hosin,'ibis_tab',IREAD)
	if (status .ne. ESUCC) then
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
	end if

C  Open input file
C-----------------
	status = tbopen(fdesc,hosin,clen,tcol,0)
	if (status .lt. ESUCC) then
	   call errmsg('Error opening input file','ascnew-open',1)
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
	end if
	nc = min0(ncol,tcol)
	nr = min0(clen,nrows)

C  Write header information to output file
C-----------------------------------------
C	write (unit,'(A4)',err=150) ' TAB'
C	write (unit,'(A,2I8)',err=150) ' ',nr,nc
C	write (unit,'(A,35I3)',err=150) ' ',(col(i),i=1,nc)
	rec = srow

C  Read and write on a line by line basis
C----------------------------------------
	do 30 line = 1,nr
	   status = tbrrow(fdesc,col,nc,rec,data)
	   if (status .lt. ESUCC) then
	      call errmsg('Error reading input file','ascnew-read',1)
	      close(unit)
	      call clean(parb,'ascnew',hosout,1)
	   end if
	   rec = rec + 1

	if (subcmd .eq. 'INT') then
	   write (unit,format,err=150) (INT(data(i)),i = 1,nc)
	else if (subcmd .eq. 'STAT') then
	   write (unit,format,err=150) INT(data(1)),data(2),data(3),
     .          INT(data(4)),INT(data(5)),data(6),INT(data(7)),
     .          INT(data(8)),INT(data(9))
	else if (subcmd .eq. 'CSTAT') then
	   write (unit,format,err=150) INT(data(1)),data(2),data(3),
     .          INT(data(4)),INT(data(5)),data(6),INT(data(7)),
     .          INT(data(8)),INT(data(9))
	else if (subcmd .eq. 'CNTY') then
	   write (unit,format,err=150) INT(data(1)),data(2),
     .          INT(data(3)),data(4),INT(data(5)),INT(data(6)),
     .          data(7),INT(data(8))
	else if (subcmd .eq. 'AGGR') then
	   write (unit,format,err=150) INT(data(1)),data(2),data(3),
     .          INT(data(4)),INT(data(5)),INT(data(6))
	else if (subcmd .eq. 'MID') then
	   write (unit,format,err=150) INT(data(1)),INT(data(2)),
     .          INT(data(3)),INT(data(4)),data(5),data(6),INT(data(7)),
     .          INT(data(8)),INT(data(9)),INT(data(10)),INT(data(11)),
     .          data(12),INT(data(13)),INT(data(14)),INT(data(15)),
     .          INT(data(16)),INT(data(17)),INT(data(18)),INT(data(19))
	else if (subcmd .eq. 'NIS') then
	   write (unit,format,err=150) INT(data(1)),INT(data(2)),
     .          INT(data(3)),data(4),data(5),INT(data(6)),INT(data(7)),
     .          INT(data(8)),INT(data(9)),INT(data(10)),data(11),
     .          INT(data(12)),INT(data(13)),INT(data(14)),
     .          INT(data(15)),INT(data(16)),INT(data(17)),INT(data(18))
        end if

30	continue
	go to 160
150	continue
	   call errmsg('Error writing to output file',
     .     	       'ascnew-write',1)
	   close(unit)
	   call clean(parb,'ascnew',hosout,1)
160	continue
	status = tbclos(fdesc,hosin)
	close (unit)

C  Build the tae name for the output file
C----------------------------------------
        taeout = hosout

	return
	end
