C*****************************************************************************
C 	NAME: ASCOUT
C
C	FUNCTION:
C	ASCOUT reads an IBIS graphics or tabular file and outputs the 
C	data to a file.
C
C	PROGRAM HISTORY:
C	Version		Date	Author		Code/conr	Request
C	-------		----	------		---------	-------
C	   1.0					   CSB		original
C	   2.0		1/87	B.AILTS		   CSB		LAMS conversion
C	   3.0		6/87	B.AILTS		   CSB		Remove GR2 
C								option
C	   5.0		2/89	B.Ailts		   CSB		LAS 5.0 conver.
C
C	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C			Run under TAE 
C
C	PROJECT:	LAMS
C
C	ALGORITHM DESCRIPTION:
C	The tae parmaters are read in.
C
C	The INFILE file is opened  and the data	is read and written 
C	to the text file that is specified by OUTFILE. 
C
C	After all the data is written, the completion message is 
C	displayed.
C
C	ALGORITHM REFERENCES:	none
C
C******************************************************************************
      program ascout

	include 'las.fin'

	include 'lams.fin'

	integer*4 col(MMXCOL)
	integer*4 count
	integer*4 i
	integer*4 len
	integer*4 ncol
	integer*4 nrows

      	integer*4 parb(XPRDIM)

	integer*4 srow
	integer*4 stdout
	integer*4 status

	character*86 format,form
	character*10 subcmd
	character*(CMLEN) taeout

C  Initialize variables
C----------------------
	nrows = MMXROW

C  retreive the TAE parameters
C-----------------------------
       	call inlas(parb,stdout)

       	call xrstr(parb,'_SUBCMD',1,subcmd,len,count,status)

       	call xrstr(parb,'OUTFORM',1,form,len,count,status)
	if (form(1:1) .ne. '(') then
	   format = '(1X,'//form(1:len)//')'
	else
	   format = form(1:1)//'1X,'//form(2:len)
	end if

C  Call the subroutine required by the code in the input file
C------------------------------------------------------------
	if (subcmd .eq. 'GRAPHICS') then
	   call wrgraph1(parb,taeout,format)
	else
	   call xrintg(parb,'COLNUM',35,col,ncol,status)

C  if COLNUM equals null then all columns should be printed.
C------------------------------------------------------------
	   if (ncol .le. 0) then
	      do 10 i = 1,35
	         col(i) = i
10	      continue
	      ncol = 35
	   end if
	   call xrintg(parb,'SROW',1,srow,count,status)
	   call xrintg(parb,'NROWS',1,nrows,count,status)
	   call wrtab(ncol,parb,format,col,nrows,srow,taeout)
	end if
		
	call complt('ASCOUT',taeout,1)

	end
C******************************************************************************
	subroutine wrgraph1(parb,taeout,form)

	include 'las.fin'

	include 'lams.fin'

	real*4 rbuf(MEMXPT)

	integer*4 getnam
	integer*4 i
	integer   fdesc
	integer*4 ip
	integer*4 iu
	integer*4 nw

      	integer*4 parb(1)

	integer*4 rec
	integer*4 status
	integer*4 gropen,grclos,grread
	integer*4 unit

	character*(CMLEN) hosin
	character*(CMLEN) hosout
	character*(CMLEN) taeout

	character*86 form
	
	unit = 4
C  Get the output file name
C--------------------------
       	status = getnam(parb,'OUTFILE',1,hosout,'txt',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered'
     .                ,'ascout-fatal',-1)
	end if

C  Fortran corriage control characters do not work well through VAX NCP
C  so the output file needs to be opened on the VAX with no carriage 
C  control characters
C-------------------------------------------------------------------

	open (unit,file=hosout,status='new',err=20)

	go to 21
20	continue
	   call errmsg('Error opening output file','ascout-open',1)
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
21	continue

       	status = getnam(parb,'INFILE',1,hosin,'ibis_graph',IREAD)
	if (status .ne. ESUCC) then
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
	end if

	status = gropen(fdesc,hosin,nw,iu,0)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','ascout-open',1)
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
	end if
	REC = 1

C  write header information to output file
C-----------------------------------------
	write (unit,'(A3)',err=150) ' GR'
	write (unit,'(A,2I8)',err=150) ' ',iu,nw

C  Read and write data
C---------------------
	DO 201 I=1,IU
	status = grread(fdesc,rec,RBUF)
	if (status .ne. ESUCC) then
	   call errmsg('Error reading input file','ascout-read',1)
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
	end if
	REC = REC+1
	WRITE(unit,form,err=150)(RBUF(IP),IP=1,NW)
 201	CONTINUE
	go to 160
150	continue
	   call errmsg('Error writing to output file',
     .    	       'ascout-write',1)
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
160	continue
	close (unit)
	status = grclos(fdesc,hosin)

C  Build the tae name for the output file
C----------------------------------------
        taeout = hosout

	return
	end
C******************************************************************************
	subroutine wrtab(ncol,parb,format,col,nrows,srow,taeout)

	include 'las.fin'

	include 'lams.fin'

	real*4 data(MMXCOL)

	integer*4 unit,ncol,clen,i,srow,nrows,tcol,nc,nr,col(1)
	integer*4 rec,line
	integer*4 getnam
	integer   fdesc

      	integer*4 parb(1)

	integer*4 status
	integer*4 tbopen,tbclos,tbrrow

	character*(CMLEN) hosin,hosout,taeout
	character*86 format

	unit = 4

       	status = getnam(parb,'OUTFILE',1,hosout,'txt',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','ascout-fatal',-1)
	end if

C  Fortran corriage control characters do not work well through VAX NCP
C  so the output file needs to be opened on the VAX with no carriage 
C  control characters
C-------------------------------------------------------------------

	open (unit,file=hosout,status='new',err=25)

	go to 26
25	continue
	   call errmsg('Error opening output file','ascout-open',1)
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
26	continue

       	status = getnam(parb,'INFILE',1,hosin,'ibis_tab',IREAD)
	if (status .ne. ESUCC) then
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
	end if

C  Open input file
C-----------------
	status = tbopen(fdesc,hosin,clen,tcol,0)
	if (status .lt. ESUCC) then
	   call errmsg('Error opening input file','ascout-open',1)
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
	end if
	nc = min0(ncol,tcol)
	nr = min0(clen,nrows)

C  Write header information to output file
C-----------------------------------------
	write (unit,'(A4)',err=150) ' TAB'
	write (unit,'(A,2I8)',err=150) ' ',nr,nc
	write (unit,'(A,35I3)',err=150) ' ',(col(i),i=1,nc)
	rec = srow

C  Read and write on a line by line basis
C----------------------------------------
	do 30 line = 1,nr
	   status = tbrrow(fdesc,col,nc,rec,data)
	   if (status .lt. ESUCC) then
	      call errmsg('Error reading input file','ascout-read',1)
	      close(unit)
	      call clean(parb,'ascout',hosout,1)
	   end if
	   rec = rec + 1
	   write (unit,format,err=150) (data(i),i = 1,nc)
30	continue
	go to 160
150	continue
	   call errmsg('Error writing to output file',
     .     	       'ascout-write',1)
	   close(unit)
	   call clean(parb,'ascout',hosout,1)
160	continue
	status = tbclos(fdesc,hosin)
	close (unit)

C  Build the tae name for the output file
C----------------------------------------
        taeout = hosout

	return
	end
