C     *************************************************************************
C     NAME 			MTB2TAB       	    		
C
C     PURPOSE 	   Generates a label table file from an IBIS tabular file 
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	B. Ailts  	     4/87	        Original
C	B. Ailts	     2/89		LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	MTB2TAB must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  
C     The input and output files are opened, and the output label table
C     file is generated from the input IBIS tabular file according to
C     the given specifications.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
      program mtb2tab

	include 'las.fin'
	include 'ltable.fin'
	include 'lams.fin'

      	real*4 rowbuf(3)

	integer*4 dtype(3)
	integer*4 dtypesub(3)
	integer*4 fd
	integer*4 fdsub
      	integer*4 getnam
      	integer*4 i
	integer*4 icols(3)
      	integer*4 icount
      	integer   fdesc1
      	integer*4 il
	integer*4 mode
	integer*4 ncol
	integer*4 nicol
      	integer*4 nrows
	integer*4 nsub
	integer*4 nsubsub
	integer*4 outsize(2)

      	integer*4 parblk(XPRDIM)

	integer*4 rec(3)
	integer*4 recsub(3)
	integer*4 size(2,3)
	integer*4 sizesub(2,3)
      	integer*4 status
      	integer*4 stdout
	integer*4 tbcol
	integer*4 tbcolsub
	integer*4 tbopen
	integer*4 tbrrow
	integer*4 temp1
	integer*4 temp2
	integer*4 temp3
	integer*4 otype
	integer*4 length

        character*(CMLEN) infile
        character*(CMLEN) ofile(2)
        character*(CMLEN) tofile(2)
	character*(CMLEN) subfilsub
	character*(FLDLEN) desc(3)
	character*(FLDLEN) descsub(3)
	character*(FLDLEN) fdesc
	character*(FLDLEN) ftype
	character*(FLDLEN) ftypesub
	character*(FLDLEN) label(3)
	character*(FLDLEN) labelsub(3)
	character*4 type

C  Initialization of variables
C  ---------------------------
      	call inlas(parblk,stdout)

	mode = IWRITE

C     	Get output label table file name
C     	--------------------------------
      	status = getnam(parblk,'OUTLT',1,ofile(1),'lt',IWRITE)
      	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','mtb2tab-fatal',-1)
      	endif

C     	Get the input IBIS tabular file name
C     	-------------------------------------
      	status = getnam(parblk,'INFILE',1,infile,'ibis_tab',IREAD)
      	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','mtb2tab-fatal',-1)
      	endif

C  Get the INCOL, and ELMNTNUM parameters
C  --------------------------------------
      	call xrintg(parblk,'INCOL',3,icols,icount,status)

	nicol = icount

C     Get output data type
C     -------------------- 
      call xrstr(parblk,'ODTYPE',1,type,length,icount,status)
      if (type .eq. 'SAME') then
         otype = 4
      else if (type .eq. 'BYTE') then
	 otype = 1
      else if (type .eq. 'I*2') then
	 otype = 2
      else if (type .eq. 'I*4') then
	 otype = 3
      else
	 otype = 4
      end if

C  Open input IBIS tabular file
C  -----------------------------
      	status = tbopen(fdesc1,infile,nrows,ncol,0)
      	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','mtb2tab-open',1)
           call errmsg('Fatal error encountered','mtb2tab-fatal',-1)
      	endif

C  Initialize the parameters for the label table
C  ---------------------------------------------
	outsize(1) = 1
	outsize(2) = 1
	nsub=1
	nsubsub = 0
	tbcol = 2
	tbcolsub = 3
	ftype = 'PNTATT'
	ftypesub = 'PNTXY'
	fdesc = 'File generated from file '//infile
	label(1) = 'PNTNUM'
	label(2) = 'LABEL'

	labelsub(1) = 'PNTNUM'
	labelsub(2) = 'X'
	labelsub(3) = 'Y'

	if (otype .eq. 4) then
	   dtype(1) = R4
	   dtype(2) = CHSTR
	else
	   dtype(1) = I4
	   dtype(2) = CHSTR
	endif

	dtypesub(1) = otype
	dtypesub(2) = otype
	dtypesub(3) = otype

	desc(1) = 'unique region number'
	desc(2) = 'user defined label description'

	descsub(1) = 'unique region number'
	descsub(2) = 'column number'
	descsub(3) = 'row number'

	size(1,1) = 1
	size(2,1) = 1
	size(1,2) = 1
	size(2,2) = 16

	do 75 i = 1,3
	   sizesub(1,i) = 1
	   sizesub(2,i) = 1
75	continue

C  Open the label table and its associated file
C  --------------------------------------------
	call fmklt(ofile(1),ofile(2),1)

	call foptab(fdsub,ofile(2),IWRITE,nsubsub,subfilsub,ftypesub,
     .		    fdesc)
	call foptab(fd,ofile(1),IWRITE,nsub,ofile(2),ftype,fdesc)

	call fpvec(fdsub,tbcolsub,labelsub,dtypesub,descsub,
     .		   sizesub,recsub)
	call fpvec(fd,tbcol,label,dtype,desc,size,rec)

C  Process input file row by row
C  -----------------------------
      	do 80 il = 1,nrows

	   if (icols(1) .eq. 0) then
              status = tbrrow(fdesc1,icols(2),nicol - 1,il,rowbuf(2))
	      if (status .ne. ESUCC) then
	         call errmsg('Error reading input file',
     .			     'mtb2tab-read',1)
	         call fcltab(fd)
	         call fcltab(fdsub)
	         call clean(parblk,'mtb2tab',ofile,2)
	      endif
	    
	      temp1 = il
	   else
              status = tbrrow(fdesc1,icols,nicol,il,rowbuf)
	      if (status .ne. ESUCC) then
	         call errmsg('Error reading input file',
     .			     'mtb2tab-read',1)
	         call fcltab(fd)
	         call fcltab(fdsub)
	         call clean(parblk,'mtb2tab',ofile,2)
	      endif

	      if (otype .ne. 4) then
	         temp1 = int(rowbuf(1))
	   	 temp2 = int(rowbuf(2))
	   	 temp3 = int(rowbuf(3))
	      endif

	   endif

	   if (otype .ne. 4) then
	      call fpmat(rec(1),outsize,temp1,FSEP1)
	      call fpmat(recsub(1),outsize,temp1,FSEP1)
	      call fpmat(recsub(2),outsize,temp2,FSEP1)
	      call fpmat(recsub(3),outsize,temp3,FSEP1)
	   else
	      call fpmat(rec(1),outsize,rowbuf(1),FSEP1)
	      call fpmat(recsub(1),outsize,rowbuf(1),FSEP1)
	      call fpmat(recsub(2),outsize,rowbuf(2),FSEP1)
	      call fpmat(recsub(3),outsize,rowbuf(3),FSEP1)
	   endif
	   call fwrtab(fd)
	   call fwrtab(fdsub)

   80 	continue

C  Close the input and output files
C  --------------------------------
      	call grclos(fdesc1,infile)
	call fcltab(fd)
	call fcltab(fdsub)

C  Output completion message
C  -------------------------
        tofile(1) = ofile(1)
        tofile(2) = ofile(2)

      	call complt('MTB2TAB',ofile,2)

      	end
