C     *************************************************************************
C     NAME 			OMTB2MTB       	    		
C
C     PURPOSE 	   Generates a LAS 5.0 IBIS tabular file from a newLAS
C		   IBIS tabular file.
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	B. Ailts	  Dec. 1988		Original
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	OMTB2MTB must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  If necessary, the
C     defaulted parameters are set with the appropriate values.  
C     The input and output files are opened, and the output tabular
C     file is generated from the input tabular file according to
C     the given specifications.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
      	program omtb2mtb

	include 'las.fin'

	include 'lams.fin'

	real*4		ain(128,MMXCOL)
      	real*4		buffin(MMXCOL)

	integer*4	clen
      	integer*4	getnam
      	integer*4	i
      	integer*4	icols(MMXCOL)
      	integer*4	icount
      	integer    	fdesc1
	integer*4	ifcb2(XIFDIM)
      	integer*4	il
      	integer*4	ncols
      	integer*4	ncount
      	integer*4	nrow
      	integer*4	nrows
      	integer*4	ocols(MMXCOL)
	integer*4	ocount
      	integer*4	oncols

      	integer*4	parblk(XPRDIM)

      	integer*4	rcount
      	integer*4	status
      	integer*4	stdout
	integer*4	tbclos
	integer*4	tbopen
	integer*4	tbwrow
	integer*4	temp

      	character*(CMLEN)	infile
      	character*132		tinfile
      	character*(CMLEN)	ofile
      	character*(CMLEN)	tname

C       initialization
C       --------------
        call inlas(parblk,stdout)

	do 3 i = 1, MMXCOL
	   ocols(i) = 0
	   icols(i) = 0
3	continue
	ncols = 0
	oncols = 0

C       get input IBIS graphics file name
C       ---------------------------------
        status = getnam(parblk,'INFILE',1,infile,'ibis_tab',0)
        if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','omtb2mtb-fatal',-1)
        endif

C       get the NCOLS and INCOL parameters
C       -----------------------------------
        call xrintg(parblk,'INCOL',MMXCOL,icols,icount,status)

        call xrintg(parblk,'NCOLS',1,oncols,ncount,status)
	if (ncount .eq. 0) oncols = 0

        call xrintg(parblk,'OUTCOL',MMXCOL,ocols,ocount,status)
	if ((ocount .ne. 0) .and. (ocount .ne. icount) .and.
     .	    (icount .ne. 0)) then
	   call errmsg('Number of input colums and OUTCOL must be equal',
     +	  	       'omtb2mtb-equal',1)
           call errmsg('Fatal error encountered','omtb2mtb-fatal',-1)
        endif

	tinfile = infile(1:132)
C       check that both were not defaulted
C       ----------------------------------
        if ((ncount .eq. 0) .and. (icount .eq. 0) .and. (ocount .eq. 0))
     .   then
C	   open the input IBIS file
C	   ------------------------
	   call rdfil(ifcb2,2,tinfile,clen,oncols,status)
	   if (status .ne. ESUCC) then
	      call errmsg('Error opening input file','omtb2mtb-open',1)
	      call errmsg('Fatal error encountered','omtb2mtb-fatal',-1)
	   endif
	   ncols = oncols
      	else
C	   open the input IBIS file and do not set NCOLS from input file
C	   -------------------------------------------------------------
	   call rdfil(ifcb2,2,tinfile,clen,temp,status)
	   if (status .ne. ESUCC) then
	      call errmsg('Error opening input file','omtb2mtb-open',1)
	      call errmsg('Fatal error encountered','omtb2mtb-fatal',-1)
	   endif
	endif
	nrow = clen

C	------------------------------------------------------
C	| if NCOLS was specified and OUTCOL + INCOL were not |
C	| specified -- set ncols to oncols		     |
C	------------------------------------------------------
	if ((ncount .ne. 0) .and. (ocount .eq. 0) .and. 
     .	    (icount .eq. 0)) ncols = oncols

	ncols = MAX(ncols,ocount)
	ncols = MAX(ncols,icount)

	do 4 i = 1, ncols
	   oncols = MAX(oncols,ocols(i))
	   oncols = MAX(oncols,icols(i))
4	continue

	if (icount .eq. 0) then
	   if (ocount .eq. 0) then
	      do 5 i = 1, ncols
	         icols(i) = i
5	      continue
	   else
	      do 6 i = 1, ncols
	         icols(i) = ocols(i)
6	      continue
	   endif
	endif

	if (ocount .eq. 0) then
	   if (icount .eq. 0) then
	      do 7 i = 1, ncols
	         ocols(i) = i
7	      continue
	   else
	      do 8 i = 1, ncols
	         ocols(i) = icols(i)
8	      continue
	   endif
	endif

        call xrintg(parblk,'NROWS',1,nrows,rcount,status)
        if (rcount .eq. 0)  nrows = clen
	if (nrows .lt. nrow) nrow = nrows

C     	get the output IBIS tabular file name
C     	-------------------------------------
      	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_tab',1)
      	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','omtb2mtb-fatal',-1)
      	endif

C       open output IBIS tabular file
C       -----------------------------
        status = tbopen(fdesc1,ofile,nrows,oncols,1)
        if (status .ne. ESUCC) then
	   status = tbclos(fdesc1,ofile)
	   call errmsg('Error opening output file','omtb2mtb-open',1)
	   call clean(parblk,'omtb2mtb',ofile,1)
        endif

C       process input file
C       -------------------
        do 80 il = 1,nrow

	  call getfil(ifcb2,ncols,icols,buffin,il,clen,ain,status)
	  if (status .ne. ESUCC) then
	     call errmsg('Error reading input file','omtb2mtb-read',1)
	     call errmsg('Fatal error encountered','omtb2mtb-fatal',-1)
	  endif
	
          status = tbwrow(fdesc1,ocols,ncols,il,buffin)
	  if (status .ne. ESUCC) then
	     status = tbclos(fdesc1,ofile)
	     call errmsg('Error writing output file',
     .	   	         'omtb2mtb-write',1)
	     call clean(parblk,'omtb2mtb',ofile,1)
	  endif
	    
   80   continue

C       column end fill
C       ---------------

        do 84 i = 1,ncols
           buffin(i) = 0
   84   continue

        do 88 i = il,nrows
           status = tbwrow(fdesc1,ocols,ncols,i,buffin)
	   if (status .ne. ESUCC) then
	      status = tbclos(fdesc1,ofile)
 	      call errmsg('Error writing output file','omtb2mtb-write',1)
	      call clean(parblk,'omtb2mtb',ofile,1)
	   endif
   88   continue

C       close the input and output IBIS files
C       -------------------------------------
        status = tbclos(fdesc1,ofile)
        if (status .ne. ESUCC) then
           call errmsg('Error closing output file','omtb2mtb-close',1)
        endif

	call xiclse(ifcb2,XSAVE)

C       output completion message
C       -------------------------
        tname = ofile

        call complt('MGR2MTB',tname,1)

        end
