/*****************************************************************************
NAME: ALASKAKEYSET

PURPOSE:  Set the geoTIFF keys for Alaska Conformal projection. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/
#include "las.h"
#include "tiffio.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "protos.h"

void alaskakeyset 
(
    const struct DDR *ddr,     /* I: LAS image file descriptor record    */
    GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
) 

{
    GTIFKeySet(gtif, ProjCoordTransGeoKey, TYPE_SHORT, 1,
            CT_TransvMercator_Modified_Alaska);
    GTIFKeySet(gtif, PCSCitationGeoKey, TYPE_ASCII, 0,
            "Transverse Mercator Modified Alaska converted from LAS");

    /* Set the major and minor axis values
       ----------------------------------- */
    set_major_minor(ddr,gtif);

    if (ddr->valid[3])
    {
        /* Set the false easting and false northing
           ---------------------------------------- */
        GTIFKeySet(gtif,ProjFalseEastingGeoKey,TYPE_DOUBLE,1,ddr->proj_coef[6]);
        GTIFKeySet(gtif,ProjFalseNorthingGeoKey,TYPE_DOUBLE,1,
                   ddr->proj_coef[7]);
    }
    else
    {
        /* Set the GeographicTypeGeoKey only when projection coefficients are 
           invalid (for consistency with algorithm in original development)
           ---------------------------------------------------------------- */
        GTIFKeySet(gtif, GeographicTypeGeoKey, TYPE_SHORT, 1, 
                   get_geocode(ddr));
    }
    GTIFKeySet(gtif, GeogLinearUnitsGeoKey, TYPE_SHORT, 1,
            get_linear_units(ddr->proj_units));
}
