/*****************************************************************************
NAME: GEOKEYSET

PURPOSE:  Sets the geoTIFF keys for Geographic projection.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/
#include "las.h"
#include "geompak.h"
#include "tiffio.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "protos.h"


void geokeyset
(
    const struct DDR *ddr,     /* I/O: LAS image file descriptor record  */
    GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
) 

{
    unsigned short units;      /* geoTIFF unit of measure code        */

    /* Set projection key to Geographic 
       --------------------------------*/
    GTIFKeySet(gtif, GTModelTypeGeoKey, TYPE_SHORT, 1, ModelTypeGeographic);
    GTIFKeySet(gtif, GTRasterTypeGeoKey, TYPE_SHORT, 1, RasterPixelIsArea);

    if (ddr->valid[2])
    {
        /* the ellipsoid code is valid so try to get the geoTIFF 
           geocode.  Returns zero if it can't find one to match
           ----------------------------------------------------- */
        units = get_geocode(ddr);
        if (units)
        {
            GTIFKeySet(gtif, GeogEllipsoidGeoKey, TYPE_SHORT, 1, units+3000);
            GTIFKeySet(gtif, GeographicTypeGeoKey, TYPE_SHORT, 1, units);
        }
        else
            GTIFKeySet(gtif, GeogEllipsoidGeoKey, TYPE_SHORT, 1, 32767);
    }
    else
        GTIFKeySet(gtif, GeogEllipsoidGeoKey, TYPE_SHORT, 1, 32767);

    if (ddr->valid[4])
    {
        /* the ground units code is valid so try to get the geoTIFF
           equivalent unit code, otherwise returns zero.
           -------------------------------------------------------- */
        GTIFKeySet(gtif, GeogAngularUnitsGeoKey, TYPE_SHORT, 1, 
                get_angular_units(ddr->proj_units));

    }

    GTIFKeySet(gtif, GeogCitationGeoKey, TYPE_ASCII, 0,
            "Geographic converted from LAS");
}
