/*****************************************************************************
NAME: GET_ANGULAR_UNITS

PURPOSE: Returns the geoTIFF unit code for the angular unit in the ddr. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/
#include "las.h"
#include "geompak.h"
#include "cproj.h"
#include "protos.h"

unsigned short get_angular_units
(
    const char *unitstr         /* I: Units used (in English)              */
)
{
    unsigned short unit;          /* geoTIFF unit code                       */
    int out_unit;                 /* Output projection unit code             */

    out_unit = c_string2unit(unitstr);

    if (out_unit == RADIAN)
        unit = 9101;
    else if (out_unit == DEGREE)
        unit = 9102;
    else if (out_unit == SECOND)
        unit = 9104;
    else if (out_unit == DMS)
        unit = 9107;
    else
    {
        c_lasmsg(FALSE, "las2geotiff-warn",
                "GeoTIFF does not recognize the DDR projection units %s as "
                "valid", unitstr);
        c_lasmsg(FALSE,"las2geotiff-warn","Setting angular units to undefined");
        unit = 0;   /* 0 = undefined */
    }

    return (unit); 
}
