/*****************************************************************************
NAME: WAGIVKEYSET

PURPOSE:  Set the geoTIFF keys for Wagner IV projection.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/
#include "las.h"
#include "tiffio.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "geompak.h"
#include "protos.h"

void wagivkeyset
(
    const struct DDR *ddr,     /* I: LAS image file descriptor record    */
    GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
) 

{
    double lon = 0;          /* Longitude                           */

    /* Wagner IV is not a known projection in the current version of geoTIFF.
       ---------------------------------------------------------------------- */
    GTIFKeySet(gtif, ProjectedCSTypeGeoKey, TYPE_SHORT, 1, 32767);
    GTIFKeySet(gtif, PCSCitationGeoKey, TYPE_ASCII, 0,
            "Wagner IV--incomplete implementation in geoTIFF");

    /* Set the major and minor axis values
       ----------------------------------- */
    set_major_minor(ddr, gtif);

    if (ddr->valid[3])
    {
        /* Set the longitude of the central meridian
           ----------------------------------------- */  
        lon = ddr->proj_coef[4];
        c_decdeg (&lon, "DMS", "LON");
        GTIFKeySet(gtif, GeogPrimeMeridianGeoKey, TYPE_DOUBLE, 1, lon);

        /* Set the false easting and false northing
           ---------------------------------------- */
        GTIFKeySet(gtif, ProjFalseEastingGeoKey, TYPE_DOUBLE, 1, 
                ddr->proj_coef[6]);
        GTIFKeySet(gtif, ProjFalseNorthingGeoKey, TYPE_DOUBLE, 1, 
                ddr->proj_coef[7]);
    }
    else
    {
        /* Set the GeographicTypeGeoKey only when projection coefficients are 
           invalid (for consistency with algorithm in original development)
           ---------------------------------------------------------------- */
        GTIFKeySet(gtif, GeographicTypeGeoKey, TYPE_SHORT, 1, 
                   get_geocode(ddr));
    }
    GTIFKeySet(gtif, GeogLinearUnitsGeoKey, TYPE_SHORT, 1,
            get_linear_units(ddr->proj_units));
}
