
/* filename.c:  function to get file names from a user parm */ 

#include <ctype.h>    /* for call to isspace() */ 
#include <stdio.h>    /* for call to popen() */ 
#include <string.h>   /* for calling various string functions */ 
#include <unistd.h>   /* for calling access() */ 
#include "lasshell.h" /* proc header file */ 
#include "protos.h"




/* FUNCTION:  get tae and host file names from user parm */
void file_names
(
    char *parmname,            /* input:  name of the parm being processed, used
					 only for error messages */
    long parmcount,           /* input:  number of values in inlist */
    char *inlist[PCOUNT],      /* input:  user values of file names */
    long execute,             /* input:  if (execute) this file must have 
					 execute access, else it must have read
					 access. */
    char ohost[PCOUNT][CMLEN] /* output:  host name of input files */
) 
{
/* declare variables */ 
   long curfile;  /* current file, for loop index */ 
/* loop for all files */ 
   for (curfile=0;  curfile<parmcount;  curfile++)
   {
   /* hopefully, this is a tae file.  convert to host and check existence */
      if ( c_cmhost(inlist[curfile],ohost[curfile]) 
         ||  access(ohost[curfile],0)  )
      /* not a tae file, translate ~ character, see if host file exists */ 
      {
	 parse_tilde (inlist[curfile], ohost[curfile]);   
	 if (access (ohost[curfile], 0))  /* no host file, abort */ 
	    c_lasmsg (TRUE, "lasshell-nofile", "File %s for parameter %s %s", 
	    inlist[curfile], parmname, 
	    "does not exist as either a TAE or a host file"); 
      }
   /* check file for execute or read access */ 
      if (execute && access (ohost[curfile], 1) )
	 c_lasmsg (TRUE, "lasshell-access", "File %s for parameter %s does %s", 
	 inlist[curfile], parmname, "not allow execute access"); 
      if (!execute && access (ohost[curfile], 4) )
	 c_lasmsg (TRUE, "lasshell-access", "File %s for parameter %s does %s", 
	 inlist[curfile], parmname, "not allow read access"); 
   }
}





/* FUNCTION:  parse an input path to expand the '~' character.  The character 
   is parsed via "popen (csh -c echo ...)".  This requires two sub-shells.
   Reading /etc/passwd could be faster but it fails on systems that run 
   yppasswd and would also require getting an environment variable ($user) */ 
void parse_tilde
(
    char *inname,   /* input:   filename with a '~' optionally imbedded */
    char *outname  /* output:  expanded filename, with no '~' */
) 
{
/* declare variables */
   char *copied,          /* a character that was copied */ 
	*newline,         /* points to a trailing newline char */ 
	 syscmd[CMLEN],   /* an operating system command */ 
        *tocopy,          /* points to a character to copy */ 
	 rootdir[CMLEN];  /* user's root directory with a '~' character */ 
   FILE *comout;          /* used to open a pipe to a unix command */ 
/* if ~ is not the first character, no parsing is necessary */ 
   if (*inname != '~')
   {
      strcpy (outname, inname);
      return;
   }
/* get rootdir (user's root dir as in ~diane).  copy it from inname to rootdir */
   for (copied=rootdir, tocopy=inname;  
   *tocopy !='/' && *tocopy !='\0';  copied++, tocopy++)
      *copied = *tocopy; 
   *copied = '\0';  /* terminate string */ 
/* open pipe to translate rootdir via a cshell */ 
   sprintf (syscmd, "csh -c \" echo %s \"", rootdir);
   comout = popen (syscmd, "r");
   if (comout == NULL)
      c_lasmsg (TRUE, "lasshell-fatal", 
      "Fatal error encountered:  error opening pipe via popen()");
/* read pipe */ 
   if (fgets (outname, CMLEN, comout) == NULL)
   {
      pclose (comout);
      c_lasmsg (TRUE, "lasshell-fatal", 
      "Fatal error encountered:  error reading pipe output");
   }
   pclose (comout); 
   /* eliminate trailing newline */ 
   newline = outname + strlen(outname) -1;
   if (isspace(*newline))
      *newline = '\0';  
/* append rest of path to user's root directory (outname) */ 
   for (copied=outname + strlen(outname);  
   *tocopy !='\0';  copied++, tocopy++)
      *copied = *tocopy; 
   *copied = '\0';  /* terminate string */ 
}
