
/* getpar.c:  functions for getting user parameters */ 
 
#include <ctype.h>    /* for calling isdigit(), tolower() */
#include <stdio.h>    /* for calling sscanf() */ 
#include <string.h>   /* for calling various string functions */ 
#include "lasshell.h" /* proc header file */ 
#include "protos.h"
 
/* macro to skip white space in a string */ 
#define SKIP_WHITE_SPACE(string)  while (isspace(*string))  string++





/* FUNCTION:  initialize user parameters, function was written mostly for
   debugging */ 
void init_parms
(
    struct UPARMS *uparms   /* output:  data structure of user parms that has
                                        been set to zero */
) 
{
   long parmval;   /* for loop index, parameter value */ 
/* initialize parms command, script, infile */ 
   for (parmval=0;  parmval<COMCOUNT;  parmval++)
      uparms->command[parmval]    = NULL;
   for (parmval=0;  parmval<PCOUNT;  parmval++)
   {
      uparms->shost  [parmval][0] = '\0'; 
      uparms->ihost  [parmval][0] = '\0'; 
   }
   uparms->comcount = 0;  
   uparms->scount   = 0; 
   uparms->icount   = 0; 
/* outtype */ 
   uparms->typestr[0] = '\0'; 
   uparms->outtype    = INTEGER; 
   uparms->outlen     = 0; 
/* outcount */ 
   uparms->countstr[0] = '\0'; 
   uparms->actcount    = 0;  /* value is incremented in get_out() */ 
   uparms->mincount    = 0;   
   uparms->maxcount    = 0;   
/* out */ 
   for (parmval=0;  parmval<MAXVAL;  parmval++)
   {
      uparms->outint [parmval] = 0; 
      uparms->outreal[parmval] = 0.0; 
      uparms->outstr [parmval] = NULL;
   }
/* fieldsep */ 
   uparms->fscount  = 0; 
   uparms->fieldsep = NULL; 
/* messages */ 
   uparms->messages.outtype  = FALSE;
   uparms->messages.outval   = FALSE;
   uparms->messages.shellcmd = FALSE;
   uparms->messages.stanout  = FALSE;
}





/* FUNCTION:  get and return user tae parameters */ 
void getpar
(
    struct PARBLK *pblock,  /* input:   tae parameter block */
    struct UPARMS *uparms  /* output:  user parameter values */
) 
{
/* declare variables */ 
   char *fnames[PCOUNT],   /* user values for file names */ 
	*messages;         /* string value of parameter messages */ 
   long pcount;            /* for loop index, parameter count */ 
   struct VARIABLE *tparm; /* points to a tae parm in pblock */ 
/* get parm command */ 
   tparm = p_find (pblock, "command");
   uparms->comcount = tparm->v_count; 
   for (pcount=0;  pcount<uparms->comcount;  pcount++)
      uparms->command[pcount] = SVAL (*tparm, pcount); 
/* script */ 
   tparm = p_find (pblock, "script");
   uparms->scount = tparm->v_count; 
   for (pcount=0;  pcount<uparms->scount;  pcount++)
      fnames[pcount] = SVAL (*tparm, pcount); 
   file_names ("SCRIPT", uparms->scount, fnames, TRUE, uparms->shost); 
/* infile */ 
   tparm = p_find (pblock, "infile");
   uparms->icount = tparm->v_count; 
   for (pcount=0;  pcount<uparms->icount;  pcount++)
      fnames[pcount] = SVAL (*tparm, pcount); 
   file_names ("INFILE", uparms->icount, fnames, FALSE, uparms->ihost); 
/* outtype */ 
   tparm = p_find (pblock, "outtype");
   if (tparm->v_count == 1)
      get_type (SVAL(*tparm,0), NULL          , uparms); 
   else /* count == 2 */ 
      get_type (SVAL(*tparm,0), SVAL(*tparm,1), uparms); 
/* outcount */ 
   tparm = p_find (pblock, "outcount");
   get_count (SVAL(*tparm,0), uparms); 
/* fieldsep */ 
   tparm = p_find (pblock, "fieldsep");
   uparms->fscount = tparm->v_count; 
   if (uparms->fscount) 
      uparms->fieldsep = SVAL (*tparm,0);
/* messages */ 
   tparm = p_find (pblock, "messages");
   messages = SVAL(*tparm,0);
   if      ( !strcmp (messages, "OUTTYPE" ))  uparms->messages.outtype  = TRUE;
   else if ( !strcmp (messages, "OUTVAL"  ))  uparms->messages.outval   = TRUE;
   else if ( !strcmp (messages, "SHELLCMD"))  uparms->messages.shellcmd = TRUE;
   else if ( !strcmp (messages, "STANOUT" ))  uparms->messages.stanout  = TRUE;
   else if ( !strcmp (messages, "ALL"     ))  
   {
      uparms->messages.outtype  = TRUE;
      uparms->messages.outval   = TRUE;
      uparms->messages.shellcmd = TRUE;
      uparms->messages.stanout  = TRUE;
   }
} 





/* FUNCTION:  parse user-specified value for outtype */ 
void get_type
(
    char *outtype1,         /* input:   1st user value of outtype */
    char *outtype2,         /* input:   2nd user value of outtype, may be NULL */
    struct UPARMS *uparms  /* output:  parsed outtype */
) 
{
/* declare vars */ 
   char typeerr[30];       /* outtype1 and outtype2 for errors */ 
/* save input values for error messages */ 
   if (outtype2 == NULL)
      strcpy (typeerr, outtype1);
   else
      sprintf (typeerr, "(%s %s)", outtype1, outtype2); 
/* lowercase user value */ 
   tolow (outtype1); 
/* get type of parm */ 
   if      (!strcmp (outtype1, "integer"))
      uparms->outtype = INTEGER; 
   else if (!strcmp (outtype1, "real"   ))
      uparms->outtype = REAL; 
   else if (!strcmp (outtype1, "string" ))
      uparms->outtype = STRING; 
   else 
      /* use same error text as below */ 
      c_lasmsg (TRUE, "lasshell-type",  
      "Invalid parameter type '%s' was given for OUTTYPE", typeerr); 
/* get length of string */ 
   if (uparms->outtype == STRING) 
   {
      if (outtype2 != NULL)
      {
      /* get length */ 
	 SKIP_WHITE_SPACE (outtype2); 
	 if (sscanf (outtype2, "%d", &uparms->outlen) != 1)
	    /* use same error text as above */
	    c_lasmsg (TRUE, "lasshell-type",   
	    "Invalid parameter type '%s' was given for OUTTYPE", typeerr); 
      /* check for bad trailing chars */ 
	 if (*outtype2 == '-') /* '-' for -1 */
	    outtype2++; 
	 while ( isdigit (*outtype2))
	    outtype2++;
	 SKIP_WHITE_SPACE (outtype2); 
	 if (*outtype2 != '\0') 
	    /* use same error text as above */
	    c_lasmsg (TRUE, "lasshell-type",   
	    "Invalid parameter type '%s' was given for OUTTYPE", typeerr); 
      }
      else 
	 uparms->outlen = 132;     /* set default value */ 
      /* error check:  bad length */ 
	 if (uparms->outlen < 0 || uparms->outlen > 132)
	    c_lasmsg (TRUE, "lasshell-slength", 
	    "An invalid STRING length of '%d' %s %s", uparms->outlen, 
	    "was given for OUTTYPE.  The length of output strings", 
	    "must be between 0 and 132");
   }
/* set typestr, string for output type */ 
   switch (uparms->outtype)
   {
      case INTEGER: strcpy  (uparms->typestr, "INTEGER");
		    break; 
      case REAL:    strcpy  (uparms->typestr, "REAL");
		    break; 
      case STRING:  sprintf (uparms->typestr, "(STRING %d)", uparms->outlen);
		    break; 
      default:      c_lasmsg (TRUE, "lasshell-fatal", 
		    "Fatal error encountered:  logic error in get_type()"); 
		    break; 
   }
}





/* FUNCTION:  parse user-specified value for outcount */ 
void get_count
(
    char *outcount0,        /* input:   user value of outcount */
    struct UPARMS *uparms  /* output:  parsed outtype */
) 
{
/* declare vars */ 
   char *charp;            /* general character pointer */ 
   charp = outcount0; 
/* get min */ 
   SKIP_WHITE_SPACE (charp);
   if (sscanf (charp, "%d", &uparms->mincount) != 1) 
      /* use same error text as below */ 
      c_lasmsg (TRUE, "lasshell-count1",  
      "Invalid value '%s' was given for OUTCOUNT", outcount0); 
/* skip '-' and digits and space characters, this section is same as below */ 
   if (*charp == '-')
      charp++; 
   while ( isdigit(*charp) )
      charp++; 
   SKIP_WHITE_SPACE (charp); 
/* get max */ 
   if (*charp == ':') 
   {
      charp++;  /* skip over the colon character */
      if (sscanf (charp, "%d", &uparms->maxcount) != 1) 
	 /* use same error text as above */ 
	 c_lasmsg (TRUE, "lasshell-count1",  
	 "Invalid value '%s' was given for OUTCOUNT", outcount0); 
   }
   else
      uparms->maxcount = uparms->mincount;
/* skip '-' and digits and space characters, this section is same as above */ 
   if (*charp == '-')
      charp++; 
   while ( isdigit(*charp) )
      charp++;
   SKIP_WHITE_SPACE (charp); 
/* error check:  did user specify bad trailing charcters */ 
   if (*charp != '\0') /* use same error text as above */ 
      c_lasmsg (TRUE, "lasshell-count1",   
      "Invalid value '%s' was given for OUTCOUNT", outcount0); 
/* error checks */ 
   if (uparms->mincount < 0 || uparms->mincount > 300 || 
       uparms->maxcount < 0 || uparms->maxcount > 300  )
      c_lasmsg (TRUE, "lasshell-count2", 
      "OUTCOUNT was given a value of %d:%d not between 0 and 300", 
      uparms->mincount, uparms->maxcount); 
   if (uparms->mincount > uparms->maxcount)
      c_lasmsg (TRUE, "lasshell-count3", 
      "Minimum count '%d' for OUTCOUNT is greater than maximum count '%d'",
      uparms->mincount, uparms->maxcount); 
/* write to countstr for user messages */ 
   if (uparms->mincount == uparms->maxcount)
      sprintf (uparms->countstr, "%d", uparms->mincount);
   else 
      sprintf (uparms->countstr, "%d:%d", uparms->mincount, uparms->maxcount);
}





/* FUNCTION:  convert input string to lower case */
void tolow
(
    char *string  /*  input as mixed case string, 
                     output as lower case string */
)
{
   for (; *string!='\0'; string++)
      *string = tolower (*string);
}



