
/* shell.c:  functions for dealing with shell commands */ 

#include <ctype.h>    /* for calling isspace() */ 
#include <string.h>   /* for calling various string functions */ 
#include <stdlib.h>   /* for calling popen */ 
#include "lasshell.h" /* proc header file */ 
#include "protos.h"
 
#define SBUFFER 1500  /* a huge buffer for standard output */ 





/* FUNCTION:  build and execute a composite shell command */ 
void run_shell
(
    struct UPARMS *uparms  /* input:  user parameters for shell command,
			      output:  data to return to user via OUT */
)
{
/* declare variables */
   char *lastchar,          /* last character of stanout[] */ 
        stanout[SBUFFER],   /* standard output of syscmd[] */ 
        syscmd[COMCOUNT*CMLEN + PCOUNT*2*CMLEN];  /* system command to execute.
			       parm command has COMCOUNT values, each CMLEN. 
			       SCRIPT and INFILE each have PCOUNT values */ 
   long nonfatal = 1,       /* parm to c_errmsg(), do not abort */ 
        parmval,            /* for loop index, parameter value */
	printed = FALSE;    /* flag, was a header printed */ 
   FILE *comout;            /* points to system command output */ 
/* build system command */ 
   /* initialize syscmd */ 
      syscmd[0] = '\0';  
   /* print parm command to syscmd */ 
      for (parmval=0;  parmval<uparms->comcount;  parmval++)
      {
	 strcat (syscmd, uparms->command[parmval]);
	 strcat (syscmd, " "); 
      }
   /* print parm script */ 
      if (uparms->comcount && uparms->scount)
	 strcat (syscmd, "; ");
      for (parmval=0;  parmval<uparms->scount;  parmval++)
      {
	 strcat (syscmd, uparms->shost[parmval]);
	 strcat (syscmd, " ; ");
      }
      if (uparms->scount)
         *( syscmd+strlen(syscmd) -2) = '\0';  /* delete last "; " */ 
   /* print parm infile */ 
      for (parmval=0;  parmval<uparms->icount;  parmval++)
      {
	 strcat (syscmd, uparms->ihost[parmval]);
	 strcat (syscmd, " ");
      }
   /* optionally echo system command */ 
      if (uparms->messages.shellcmd)
      {
	 c_lasmsg (FALSE, "informational", "Composite shell command is:"); 
	 if (strlen(syscmd) <= CMLEN)
	    /* %S in syscmd causes c_lasmsg() memory error so use c_errmsg() */
	    c_errmsg (syscmd, "informational", &nonfatal);
	 else  /* syscmd is too big for c_lasmsg() */ 
	    fprintf (stdout, "[informational] %s\n", syscmd); 
      }
/* open pipe to system command */ 
   comout = popen (syscmd, "r");
   if (comout == NULL)
      c_lasmsg (TRUE, "lasshell-fatal", "Fatal error encountered:  unable %s", 
      "to open pipe to system command via popen()"); 
/* read and parse shell command */ 
   while ( fgets (stanout, SBUFFER, comout) != NULL )   /* get command output */
   {
   /* delete newline char */ 
      lastchar = stanout+strlen(stanout) -1;
      if ( *lastchar == '\n') 
	 *lastchar = '\0'; 
   /* write command output */ 
      if (uparms->messages.stanout) 
      {
	 /* print header if it was not printed yet */ 
	    if (!printed)
	    {
	       c_lasmsg (FALSE, "informational", 
	       "Shell command standard output is:"); 
	       printed = TRUE;
	    }
	 /* call c_lasmsg() if practical, else use fprintf() */ 
	    if (strlen(stanout) <= CMLEN)
	       c_lasmsg (FALSE, "informational", stanout);
	    else  /* stdout is too big for c_lasmsg() */ 
	       fprintf (stdout, "[informational] %s\n", stanout); 
      }
   /* translate standard output to parm out */ 
      get_out (comout, stanout, uparms); 
   }
/* user message:  no standard output */ 
   if (uparms->messages.stanout && !printed)
      c_lasmsg (FALSE, "informational", 
      "Shell command produced no standard output"); 
/* close pipe */ 
   if (pclose (comout) )
      c_lasmsg (TRUE, "lasshell-command", 
      "Shell command exited with a failure status"); 
/* error check:  too few values were returned */ 
   if (uparms->actcount < uparms->mincount)
      c_lasmsg (TRUE, "lasshell-toofew", "Less than %d parameter %s", 
      uparms->mincount, "values were returned from shell command"); 
/* optionally print OUT for the user */ 
   print_out (uparms);
}





/* FUNCTION:  print values of OUT for the user */ 
void print_out
(
    struct UPARMS *uparms  /* input:  user parameters */
) 
{
/* declare variable */ 
   long parmval;           /* for loop index, parameter value */
/* optionally print type of OUT */ 
   if (uparms->messages.outtype)
      c_lasmsg (FALSE, "informational", "Parm OUT will be TYPE=%s COUNT=%s",
      uparms->typestr, uparms->countstr);
/* return if user does not want to print OUT values */ 
   if (!uparms->messages.outval)
      return; 
/* The logic in this function was too deeply-nested for a switch statement on
   (uparms->actcount) so I used if statements instead. */ 
/* print 0 values */ 
   if (uparms->actcount == 0)
   {
      c_lasmsg (FALSE, "informational", "OUT will have zero values");
      return; 
   }
/* print 1 value */ 
   if (uparms->actcount == 1)
      switch (uparms->outtype)
      {
	 case INTEGER:  c_lasmsg(FALSE, "informational", 
			"OUT will have the following value:  %d", 
			uparms->outint[0] );
			return; 
	 case REAL:     c_lasmsg(FALSE, "informational", 
			"OUT will have the following value:  %f",
			uparms->outreal[0] ); 
			return; 
	 case STRING:   c_lasmsg(FALSE, "informational", 
			"OUT will have the following value:  %s",
			uparms->outstr[0] ); 
			return; 
      }
/* print multiple values */ 
   c_lasmsg (FALSE, "informational", 
	    "OUT will have the following %d values:", uparms->actcount); 

   for (parmval=0;  parmval<uparms->actcount;  parmval++)
      switch (uparms->outtype)
      {
	 case INTEGER:  
	      c_lasmsg (FALSE, "informational", "%d) %d", parmval+1,
	      uparms->outint[parmval] ); 
	      break; 
	 case REAL:  
	      c_lasmsg (FALSE, "informational", "%d) %f", parmval+1,
	      uparms->outreal[parmval] ); 
	      break; 
	 case STRING:  
	      c_lasmsg (FALSE, "informational", "%d) %s", parmval+1,
	      uparms->outstr[parmval] ); 
	      break; 
	 default:
	      c_lasmsg (TRUE, "lasshell-fatal", 
	      "logic error in run_shell()");
      }
}





/* FUNCTION:  translate standard output to tae parm out */ 
void get_out
(
    FILE *comout,          /* input:   pipe to close on fatal error */
    char *stanout,         /* input:   command's standard output */
    struct UPARMS *uparms /* output:  data to return to user via parm OUT */
) 
{
/* declare variables */ 
   char *nextval,         /* next value for uparms->outstr */ 
        *token,           /* token being processed */ 
        *splitter;        /* used to split stanout into tokens */ 
/* set splitter */ 
   if (uparms->fscount) 
      splitter = uparms->fieldsep;  /* use user-specified string */ 
   else 
      splitter = " \t\n\v\r\f";     /* use general white space */ 
/* split stanout into tokens */ 
   for (;  token = strtok (stanout, splitter);  )
   {
      /* revise stanout for next call to strtok() */ 
	 stanout = (char *) NULL; 
      /* return if no more values */ 
	 if (token == NULL)
	    return;
      /* error check:  too many values */
	 if (uparms->actcount >= uparms->maxcount)
	 {
	    pclose (comout); 
	    c_lasmsg (TRUE, "lasshell-toomany", "More than %d parameter %s", 
	    uparms->maxcount, "values were returned from shell command"); 
	 }
      /* convert token to a value */ 
	 switch (uparms->outtype)
	 {
	    case INTEGER:  
	       if (sscanf (token, "%d",  uparms->outint  +uparms->actcount) !=1)
	       {
		  pclose (comout); 
	          c_lasmsg (TRUE, "lasshell-out", 
		  "Error converting '%s' to INTEGER", token);
	       }
	       break;
	    case REAL:  
	       if (sscanf (token, "%lf", uparms->outreal +uparms->actcount) !=1)
	       {
		  pclose (comout); 
	          c_lasmsg (TRUE, "lasshell-out", 
		  "Error converting '%s' to REAL", token);
	       }
	       break;
	    case STRING:
	       /* make sure string length is not exceeded */ 
		  if (strlen (token) > uparms->outlen)
		  {
		     pclose (comout); 
		     c_lasmsg (TRUE, "lasshell-outlen", 
		      "Value '%s' for OUT has more than %d characters", 
		      token, uparms->outlen); 
		  }
	       /* get memory for string.  memory freed in main() */ 
		  nextval = (char *) malloc (sizeof(char) * (strlen(token) +1));
		  if (nextval == NULL)
		  {
		     pclose (comout); 
		     c_lasmsg (TRUE, "lasshell-fatal", 
		     "Fatal error encountered:  out of memory"); 
		  }
	       /* copy string */ 
		  strcpy (nextval, token); 
		  uparms->outstr[ uparms->actcount ] = nextval; 
	       break; 
	 }
      /* increment n values for out.  actcount was initialized in init_parms()*/
	 uparms->actcount++;
   }
}


