
/******************************************************************************
NAME:     getsize.c 
PURPOSE:  get most TAE parameters, return size of disk file to allocate
******************************************************************************/




// include files
#include <string.h>   // for calling strcmp()
extern "C" 
{
#include "las.h"      // LAS header file
#include "typlim.h"   // for use of MAXLONG 
}




// declare class for input TAE parameters
   class taeparm 
   {
   public:  
      long count,   // count of TAE parameter, = 0 or 1 
           value;   // value of TAE parameter 
      void get_val   (struct PARBLK *pblock, char *parmname);
      void get_dsize (struct PARBLK *pblock, char *parmname);
   };




/******************************************************************************
NAME:     getsize()
PURPOSE:  get TAE parameters, return size of disk file to allocate
******************************************************************************/
FUNCTION long getsize (
   struct PARBLK *pblock)  // input:  TAE parameter block
{
// declare class for input parameters
   class taeparm 
   nl,     // number of lines   in image 
   ns,     // number of samples in image 
   nbands, // number of bands   in image 
   dsize,  // bytes per pixel 
   bytes;  // number of bytes in disk file 

// get values of parameters 
   nl    .get_val   (pblock, "nl"    );
   ns    .get_val   (pblock, "ns"    );
   nbands.get_val   (pblock, "nbands");
   dsize .get_dsize (pblock, "dtype" );
   bytes .get_val   (pblock, "bytes" ); 

// check nl and ns to ensure they are greater than or equal to 1
   if (nl.value < 1)
      c_lasmsg (TRUE, "makelink-nl", "NL must be greater than or equal to 1");
	
   if (ns.value < 1)
      c_lasmsg (TRUE, "makelink-ns", "NS must be greater than or equal to 1");
	
// check for parameters not being specified or too many parms given 
   if (!bytes.count && !
   (nl.count && ns.count && nbands.count && dsize.count) )
      c_lasmsg (TRUE, "makelink-parms1", "If BYTES is nulled, then each "
      "of NL, NS, NBANDS, and DTYPE must be specified"); 

   if (bytes.count && 
   (nl.count || ns.count || nbands.count || dsize.count) )
      c_lasmsg (TRUE, "makelink-parms2", "If BYTES is specified, then each "
      "of NL, NS, NBANDS, and DTYPE must be null (--)"); 


// return file size if BYTES was given
   if (bytes.count)
      return (bytes.value);

// calculate and check file size
   double size = (double) nl.value * ns.value * nbands.value * dsize.value;  

   if (size > MAXLONG)
      c_lasmsg (TRUE, "makelink-toobig", "The size of the data file was "
      "calculated to be greater than the LAS limit of %ld bytes", MAXLONG); 

   return (nl.value * ns.value * nbands.value * dsize.value);
}






/******************************************************************************
NAME:     taeparm::get_val()
PURPOSE:  return the value and count (0 or 1) of a TAE integer parm 
******************************************************************************/
FUNCTION void taeparm::get_val (
   struct PARBLK *pblock,  // input:  TAE parm block 
   char *parmname)         // input:  name of TAE parm 
{
   struct VARIABLE *tparm;  // points to a TAE parameter 
   tparm = p_find (pblock, parmname);
   this->count = tparm->v_count;
   if (!this->count)
      return;

   this->value = IVAL (*tparm, 0); 
}




/******************************************************************************
NAME:     taeparm::get_dsize()
PURPOSE:  return the value and count of TAE parameter DTYPE,
          translate data type to data size in bytes per pixel
******************************************************************************/
FUNCTION void taeparm::get_dsize (
   struct PARBLK *pblock,  // input:  TAE parm block 
   char *parmname)         // input:  name of TAE parm 
{
// get DTYPE
   struct VARIABLE *tparm;  // points to a TAE parameter 
   tparm = p_find (pblock, parmname);
   this->count = tparm->v_count;
   if (!this->count)
      return;

// translate DTYPE to bytes per pixel 
   char *dtype = SVAL (*tparm, 0);
   if      (!strcmp (dtype, "BYTE"))  this->value = 1;
   else if (!strcmp (dtype, "I*2" ))  this->value = 2;
   else if (!strcmp (dtype, "I*4" ))  this->value = 4;
   else if (!strcmp (dtype, "R*4" ))  this->value = 4;
   else c_lasmsg (TRUE, "makelink-program", "Program error in get_val()");
}


