
/******************************************************************************
NAME:    MAKELINK, a C++ program 
PURPOSE: make a symbolic link, allocate space for a data file  

  Date      Programmer                        Changes
--------  --------------  -----------------------------------------------------
Jan 1997  David Zokaites  Original development 
Dec 1999  D Hames	  Remove valid=(1:50000) from NL and NS in makelink.pdf 
			  Add an error check to getsize to ensure the value
			  for NL and NS is greater than or equal to 1. SRF #893
******************************************************************************/




// include files
extern "C" 
{
#include "las.h"      // LAS header file
#include "util.h"
#include "diskio.h"   // for c_dkcre() return codes 
}

// function prototypes
   long getsize (struct PARBLK *pblock);
   char * getname (const char *filename);






int main ()
{
// initialize LAS and TAE 
   struct PARBLK pblock;  // TAE parameter block 
   long std_out;          // standard output
   c_inlas (&pblock, &std_out);

// determine size of disk file to allocate
   long filesize = getsize (&pblock); 

// get value of TAE parm LINKFILE 
   struct VARIABLE *tparm = p_find (&pblock, "linkfile");
   char *linkfile = SVAL (*tparm, 0);

// create LINKFILE, allocate space for DATAFILE  
   switch (c_dkcre (NULL, linkfile, filesize, 0))
   {
      case E_SUCC:       break;  // success
      case E_FAIL:       c_lasmsg (TRUE, "makelink-fatal",   // failure 
		         "Fatal error encountered"); 
      case DISK_EXISTS:  c_lasmsg (TRUE, "makelink-exist", 
			 "LINKFILE %s already exists", linkfile);
      case DISK_NOSPACE: c_lasmsg (TRUE, "makelink-full", 
			 "The %ld bytes requested for DATAFILE is not "
			 "available", filesize);
   }

// get value for DATAFILE 
   const char *datafile[1];  // name of data file 
   datafile[0] = getname (linkfile); 

// user message
   c_lasmsg (FALSE, "informational", 
   "%ld bytes have been allocated for DATAFILE %s", filesize, datafile[0]);
   c_lasmsg (FALSE, "informational", "Created LINKFILE %s", linkfile); 

// return DATAFILE to TAE
   if (q_string (&pblock, "datafile", 1, datafile, P_UPDATE) != SUCCESS ||
       q_out    (&pblock                                   ) != SUCCESS  )
     c_lasmsg (TRUE, "makelink-totae", "Error returning DATAFILE to TAE");

// complete.  User already received notice of creating LINKFILE.    
   char outfiles[1][CMLEN];  // list of all 0 output files
   long outcount = 0;        // number of output files 
   c_complt ("MAKELINK", outfiles, &outcount); 
}




