/* Prepare a tape label for msredist.pdf, return sceneid.  7-1992 by David
Zokaites, original development */ 

#include <stdio.h>   /* for file access */ 
#include <string.h>  /* for call to strcpy(), strcat() */ 
#include <time.h>    /* for call to time(), localtime() */ 
#include "las.h"

#define ANNOTFILE   "msredist_1.annot" /* annotation file containing scene id */
#define LABEL_FILE  "msredist.label"   /* file to prepare with tape label */ 


main ()
{
/* declare vars */ 
   char date_str[50];     /* date the tape is created */
   char message[CMLEN];   /* a message to the user */ 
   char *sceneid[1];      /* id of mss scene, must be a pointer for q_string()*/
   char time_str[50];     /* time the tape is created */
   FILE *filep;           /* file pointer, used for two files */ 
   long fatal= -1;        /* fatal error */ 
   long nonfatal= 1;      /* non fatal error, info message */ 
   long now;              /* current date and time as a long */
   long std_out;	  /* standard out put */
   struct tm *today;      /* structure with the current date and time */
   struct PARBLK vblock;  /* tae parameter block */

/* initialize tae and las */ 
   c_inlas(&vblock,&std_out);

/* determine current date and time */ 
   now   = time(NULL);
   today = localtime(&now);
   today->tm_mon  += 1;       /* months are zero indexed */
   today->tm_year += 1900;    /* years are based on 1900 */
   sprintf(date_str, "%02d/%02d/%4d", 
   today->tm_mon, today->tm_mday, today->tm_year); 
   sprintf(time_str, "%02d%02d",    today->tm_hour, today->tm_min);

/* get id of mss scene */ 
   /* allocate memory */ 
      sceneid[0] = (char *) malloc (50);
      if (sceneid[0] == NULL)
	 c_errmsg ("Unable to allocate memory.", "msredist-fail", &fatal); 
   /* open annotation file */ 
      filep = fopen (ANNOTFILE, "r");
      if (filep == NULL)
	 c_errmsg ("Unable to open annotation file for reading.", 
	 "msredist-fail", &fatal); 
   /* get scene id */ 
      /* read first part, skip a junk char, read 2nd part */ 
	 if (fscanf (filep, "%*107c %6s %*c %6s", sceneid[0], sceneid[0]+6) 
    	 != 2) 
	    c_errmsg("Error reading annotation file.", "msredist-fail", &fatal);
      /* set constant parts of sceneid */ 
	 sceneid[0][0] = '8';  
	 sprintf (sceneid[0]+11, "x0"); 
   fclose (filep); 

/* write tape label info to disk file */ 
   filep = fopen (LABEL_FILE, "w");
   if (filep == NULL)
      c_errmsg ("Unable to open file for tape label information.",
      "msredist-fail", &fatal); 
   fprintf (filep, "****************************\n");
   fprintf (filep, "                            \n");
   fprintf (filep, "  MSS Colorfire Input Tape  \n");
   fprintf (filep, "                            \n");
   fprintf (filep, "   Sceneid: %s \n", *sceneid);
   fprintf (filep, "                            \n");
   fprintf (filep, "   Date: %s \n", date_str);
   fprintf (filep, "   Time: %s \n", time_str);
   fprintf (filep, "                            \n");
   fprintf (filep, "   Drive: _______           \n");
   fprintf (filep, "                            \n");
   fprintf (filep, "****************************\n");
   fclose (filep); 

/* return sceneid to calling proc */ 
   if (q_string (&vblock, "SCENEID", 1, sceneid, P_UPDATE) != SUCCESS ||
   q_out        (&vblock)                                  != SUCCESS  )
      c_errmsg ("Unable to return SCENEID to calling proc.", "msredist-fail",
      &fatal); 

/* complete the process */ 
   /* don't call c_complet because the procedure msredist is not complete */ 
   sprintf (message, "Created tape label file %s", LABEL_FILE); 
   c_errmsg (message, "MSREDIST-INFO", &nonfatal); 
}

