#include "neighbor.h"
#include "protos.h"
/*
--------------------------------------------------------------------------

NAME: FILES
 
FUNCTION: sets up the image i/o interface

PDL:
Begin files
        Open input image (single band only in neighbor).
        Allocate input buffer.
        Group input image.
        Open output image (single band only in neighbor).
        Allocate output buffer.
        Group output image.
        Update DDR.
End files
--------------------------------------------------------------------------*/
void files
(
    char hosin[][CMLEN],                /* I: host input image name     */
    char hosout[],                      /* I: host output image names   */
    long window[][4],                   /* I: processing window         */
    long nimages                        /* I: no. of images             */
)
{
    long    stat,                   /* flag for EOPENS procedure    */
    access,                         /* file specifications          */
    type,                           /* data type                    */
    n, nib,                         /* counter variables            */
    bufsiz;                         /* image i/o buffer size        */
    long zero = 0;
    long one = 1;
    double  l_inc, s_inc;


    /*
       Open input file.
       ----------------*/
    access = IREAD;
    type = EBYTE;
    for (n = 0, nib = 0; n < nimages; n++)
    {
        stat = c_eopens(&fdescin[n], hosin[n], bands[n],
                &window[n][SL], &window[n][SS], &window[n][NL],
                &window[n][NS], &ipoint[nib], &access, &type,&zero, &zero);
        if (stat != E_SUCC)
            msg_err("!Error opening input image", "neighbor-input");
        nib += nbands[n]; 
    }
    fdescin[++n] = 0;

    if ((inbuf = (unsigned char *)c_ealloc(fdescin,&bufsiz))==NULL)
        msg_err("!Error allocating dynamic memory", "neighbor-alloc");

    /*
       Group the input image file.
       ----------------------------*/
    c_egroup (fdescin, &gdescin, inbuf, &bufsiz);

    /*
       Open output file.
       -----------------*/
    n = 0;
    access = IWRITE;
    stat = c_eopens(&fdescout[n], hosout, &one, &one, 
            &one, &window[0][NL], &window[0][NS], opoint,
            &access, &type, &zero, &zero);
    if (stat != E_SUCC)
        msg_err("!Error opening output image", "neighbor-output");

    fdescout[++n] = 0;

    /*
       Allocate output buffer.
       -----------------------*/
    if ((outbuf=(unsigned char *)c_ealloc(fdescout,&bufsiz))==NULL)
        msg_err("!Error allocating dynamic memory", "neighbor-alloc");

    /* 
       Group the output image file.
       ----------------------------*/
    c_egroup (fdescout,&gdescout,outbuf,&bufsiz);

    /*
       Update DDR.
       -----------*/
    l_inc = 1;
    s_inc = 1;
    c_upddr (&block, hosin, hosout, &nimages, window, bands, nbands, &l_inc,
            &s_inc, &one, &one);

}
