#include "neighbor.h"
#include "util.h"
#include "protos.h"

/*
--------------------------------------------------------------------------

NAME: PARMS
 
PDL:
Begin parms
        Initialize TAE functions.
        Get input image names.
        Check number of input bands is not greater than one
        Get input data type from ddr.
        Get output image names.
        Get radius for searching area.
        Check for minimum input size
        Set search option as specified.
        Get pixel value for options 1 and 2.
        Get TAE global values.
End parms

----------------------------------------------------------------------- */

void parms
(
    char lasin[][CMLEN],                /* O: las i/o image names       */
    char hosin[][CMLEN],                /* O: host i/o image names      */
    char lasout[],                      /* O: las i/o image names       */
    char hosout[],                      /* O: host i/o image names      */
    long wind[][4],                     /* O: processing window         */
    long *msgtim,                       /* O: message interval          */
    char minmax[4],                     /* O: min/max option            */
    long *radius,                       /* O: search radius             */
    long *option,                       /* O: type of search            */
    long *value,                        /* O: pixel value               */
    long *nimages                       /* O: no. of images             */
)
{

    struct  VARIABLE *var,          /* TAE p_find scenario vars     */
    *coption,                       /*        "                     */
    *pix;                           /*        "                     */

    char    copt[9];                /* temporary option             */

    long    flag[MAXIMG][2],        /* window or band option flag   */
    idia,                           /* Minimum input size variable  */
    itype,                          /* input image data type        */
    imglimit = MAXIMG,              /* max no. of images            */
    same_window,                    /* flag for same window size    */
    status,                         /* Function return status       */
    std_out,                        /* C_inlas Parameter            */
    totbnd;                         /* total no. of bands           */
    long zero = 0;

    /*
       Initialize TAE functions.
       -------------------------*/
    c_inlas (&block, &std_out);

    /*
       Get input image names.
       ----------------------*/
    status = c_getin (&block, "IN", &zero, lasin, hosin, nimages, wind, 
            bands, nbands, &totbnd, &imglimit, &same_window, flag);
    if (status != E_SUCC)
        msg_err("!Error accessing the input file name", "neighbor-infile");

    if (totbnd > 1)
        msg_err("!INPUT IMAGE MUST BE A SINGLE BAND IMAGE", "neighbor-nbnd");

    /*
       Get input data type.
       --------------------*/
    c_getddr (hosin[0], &ddr);
    itype = ddr.dtype;
    if (itype  >  1) 
        msg_err("!INPUT IMAGE MUST BE BYTE DATA", "neighbor-type");

    /*
       Get output image names.
       -----------------------*/
    c_getout (&block, "OUT", (char (*)[256])lasout,(char (*)[256])hosout);

    /*
       Get radius for searching area.
       ------------------------------*/
    var = p_find (&block, "SRCHRAD");
    *radius = IVAL(*var, 0);
    idia=2 * *radius + 1;
    if (idia  >  wind[0][2]  ||  idia  >  wind[0][3]) 
    {
        msg_err("!IMAGE SIZE IS SMALLER THAN CELL SIZE", "neighbor-size");
    }  

    /*
       Set search option as specified.
       -------------------------------*/
    coption = p_find(&block, "SRCHOPT");
    strcpy(copt, SVAL(*coption, 0));

    switch (copt[1]) {
        case 'N'    : *option = 1;
                      break;
        case 'E'    : *option = 2;
                      break;
        case 'D'    : *option = 3;
                      break;
        case 'O'    : *option = 4;
                      break;
        case 'A'    : *option = 5;
                      break;
        case 'I'    : *option = 6;
    }

    /*
       Get pixel value for options 1 and 2.
       ------------------------------------*/
    if (*option  ==  1  ||  *option  ==  2) 
    {
        pix = p_find(&block, "PIXVAL");
        *value = IVAL(*pix, 0);
        if ((*pix).v_count  ==  0) 
            msg_err("!For SRCHOPT=UNWEIGHT or WEIGHT, PIXVAL must be specified",
                    "neighbor-pix");
    }  

    /*
       Get TAE global values.
       ----------------------*/
    var = p_find(&block, "$MSGTIM"); 
    *msgtim = IVAL(*var,0);

    var = p_find(&block, "$MINMAX"); 
    strcpy(minmax, SVAL(*var, 0));

}
