#include <stdio.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

static  long    one = 1;         /* Constant                     */

struct   PARBLK block;           /* TAE parameter block          */
struct   DDR  ddr;               /* data descriptior record      */

char    out[MAXIMG][CMLEN];      /* names of files opened        */
unsigned char    *inbuf;         /* image input buffer           */
unsigned char    *outbuf;        /* image output buffer          */
long    bands[MAXIMG][MAXBND+1]; /* Band numbers per image       */
long    count;                   /* no. of files opened          */
extern long    curent;           /* parameters for PLM           */

long     fdescin[MAXIMG+1];      /* input file descriptors       */
long     fdescout[2];            /* output file descriptor       */
long     gdescin;                /* input group descriptor       */
long     gdescout;               /* output group descriptor      */
long     ipoint[MAXIMG * MAXBND + 1],/* offsets for image i/o    */
                opoint[MAXBND+1];/* for input and output bands   */
long    nbands[MAXIMG];          /* Number of bands per image    */

/*

NAME: NEIGHBOR
 
PROGRAM HISTORY:
      Version          date                Author
      -------          ----                ------
         7            Nov-99              D. Hames Remove MAXNS from neisub and
                                                allocate obuf using wind[0][NS].
                                                Was modified to support 
                                                images > 2gig.  Add prototypes. 
         6            Mar-93              D. Etrheim  Corrected so that 
                                                allocation of workbuf is based
                                                on window[NS]
         5            Sep-92              T. Mittan modified include file to 
                                                    eliminate warnings on link
         4            Nov-90              F. Irani  inline doc cleanup
         3            may-89              B. Allam      C Conversion
         2            jan-85              K. Narayanan
         1            oct-84              Y. Sheu       Original
 
FUNCTION: FOR EACH PIXEL IN THE INPUT IMAGE, SEARCHES THE AREA
          WITHIN A SPECIFIED RADIUS FOR CERTAIN CHARACTERISTICS 
PDL:
Begin neighbor
        Get input parameters.
        Set up spiral search offset and distance tables
        Determine size of spread area and work area       
        Allocate dynamic memory for work buffer.
        Address first dimension of work area.
        Set file groups for input and output images
        Perform minimum distance spread                                   
        Free local work area.
        Update label and history.
End neighbor


----------------------------------------------------------------------- */
void main(void)
{

    char    lasin[MAXIMG][CMLEN], lasout[CMLEN],    /* las I/O image names  */
    hosin[MAXIMG][CMLEN], hosout[CMLEN],    /* host I/O image names */
    minmax[4];                              /* min/max option       */

    double  minp, maxp;     /* min and max pixel values             */

    long    adim,           /* size of spread                       */
    dist[861],              /* spiral search distances              */
    i, j,                   /* counter variables                    */
    msgtim,                 /* interval between messages            */
    nimages,                /* no. of images                        */
    option,                 /* type of search                       */
    rad,                    /* search radius                        */
    value,                  /* pixel value                          */
    window[MAXIMG][4],      /* processing window                    */
    *work[17],              /* Two dimensional work area            */
    *workbuf,               /* Work space                           */
    work_size,              /* size of work buffer                  */
    wrksiz,                 /* size of work area                    */
    xoff[861], yoff[861];   /* spiral search offsets                */

    /*
       Get input parameters.
       ---------------------*/
    parms (lasin, hosin, lasout, hosout, window, &msgtim, minmax, &rad,
            &option, &value, &nimages);

    /*
       Set up spiral search offset and distance tables.
       ------------------------------------------------*/
    setdis(xoff, yoff, dist, rad);

    /*
       Determine size of spread area and work area.
       --------------------------------------------*/
    adim = 2 * rad + 1;                                            
    /*wrksiz = window[0][2]+ 2 * rad;*/                                        
    wrksiz = window[0][NS]+ 2 * rad;                                        

    /*
       Allocate dynamic memory for work buffer.
       ----------------------------------------*/
    work_size = (long)(adim * wrksiz * sizeof(long));
    if ((workbuf = (long *) malloc (work_size)) == NULL)
        msg_err ("!Cannot allocate dynamic memory for work buffer", 
                "neighbor-alloc");
    /*
       Address first dimension of work area.
       -------------------------------------*/
    for (i = 0, j = 0; i < adim; i++, j += wrksiz)
        work[i] = &workbuf[j];

    /*
       Set file groups for input and output images.
       --------------------------------------------*/
    files (hosin, hosout, window, nimages);

    /*
       Perform minimum distance spread.
       --------------------------------*/
    neisub (adim, wrksiz, work, window, xoff, yoff, rad, option, value, 
            msgtim, &minp, &maxp); 

    /*
       Free local work area.
       ---------------------*/
    free (workbuf);

    /*
       Update label and history.
       -------------------------*/
    neilbl (hosin, hosout, minp, maxp, minmax);

    /*
       Write completion message.
       -------------------------*/
    c_complt("NEIGHBOR",(char (*)[256])lasout, &one);

}    
