#include "neighbor.h"
#include "protos.h"
/*
----------------------------------------------------------------------- 

NAME: SETDIS

Purpose:
Set up spiral search offset and distance tables                         
 
Begin setdis
        Initialize variables.
        For dimension of search radius.
                For theta = 1 - 360.
                        Calculate table value.
                        Reinitialize or increment variables.
End setdis

----------------------------------------------------------------------- */

void setdis
(
    long xoff[861],                     /* O: spiral search offsets     */
    long yoff[861],                     /* O: search offsets            */
    long dist[861],                     /* O: search distances          */
    long rad                            /* I: search radius             */
)
{

    double  x, y, d;

    float   thrad, theta;           /* Temporary variables          */

    long    ifirst,                 /* first flag                   */
    xlast,                          /* last x flag                  */
    ylast,                          /* last y flag                  */
    ktheta,                         /* Loop index 1-360             */
    nx, ny, nd, n,                  /* Temporary values             */
    ir,                             /* Radius dimension loop index  */
    r;                              /* Temp radius value            */

    /*
       Initialize variables.
       ---------------------*/
    xoff[0] = 0;                                                         
    yoff[0] = 0;                                                         
    dist[0] = 0;                                                         
    xlast = 0;                                                           
    ylast = 0;                                                           
    n = 1;                                                               

    /*
       Do for dimension of search radius.
       -------------------------------*/
    for (ir = 1; ir <= rad; ir++)
    {
        theta = 90.;                                                         
        ifirst = 1;                                                          

        /*
           Do for theta = 1 - 360.
           -----------------------*/
        for (ktheta = 1; ktheta <= 360; ktheta++)
        {
            /*
               Calculate table values.
               -----------------------*/
            thrad = theta / 57.29578;
            r = ir;               
            x = r * cos(thrad);
            y = r * sin(thrad);
            if (x < 0)
                nx = (long)(x - 0.5);
            else
                nx = (long)(x + 0.5);  
            if (y < 0)
                ny = (long)(y - 0.5);
            else
                ny = (long)(y + 0.5);
            d = sqrt((float)(nx * nx) + (float)(ny * ny)); 
            if (d < 0)
                nd = (long)(d - 0.5);
            else
                nd = (long)(d + 0.5);
            if (((nx == xlast)  &&  (ny == ylast))  || ( nd != ir))
            {
                ++theta;
                continue;
            }
            if (nx == 0  &&  ny == ir  &&  ifirst == 0)
            {
                ++theta;
                continue;
            }

            /*
               Reinitialize or increment variables.
               ------------------------------------*/
            ifirst = 0;     
            xoff[n] = nx;   
            yoff[n] = ny;   
            dist[n] = nd;   
            xlast = nx;
            ylast = ny;     
            ++n;
            ++theta;

        }       /* For 1 - 360 */

    }    /* For dimension of search radius */

}
