#include "power.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char inhos[][CMLEN],     /* input image host names                        */
    char outhos[],           /* output image host name                        */
    long *num_inimages,	    /* number of input images			*/
    long num_image_bands[],       /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each input image   */
    long *num_outbands,  	/* number of output bands */
    long out_img_bands[][MAXBND+1] /* band numbers for the output image     */
)
	
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         30-Mar-90   W. Xia/STX         conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open input image files.
   Open the output image file.                                               
   Update the ddr file.
   Group all the images files into a group file
**************************************************************************
*/
{

long    fatal_err = -1;            /* fatal error return status */
long    zero = 0;
long    one = 1;
char    *prog_name = "POWER"; /* program name            */

/* local variables */

double	line_inc = 1.0,		/* line and sample increments for          */
	samp_inc = 1.0;		/* output image ddr                        */

long	access,			/* file access type                        */
	bufsiz,			/* image i/o buffer size                   */
	comb_flag=0,		/* image combination flag 		   */
        n, nib,			/* loop index */
      	stat,			/* return status                           */
	type;           	/* output image file type                  */

/*
Open input image files.                       
-------------------------                      
*/
access = IREAD;
type = EREAL;                                
for (n = 0, nib = 0; n < *num_inimages; n++)
{
   stat = c_eopens( &fdesc[n], inhos[n], image_band_num[n], &wind[n][SL], 
                   &wind[n][SS], &wind[n][NL], &wind[n][NS],       
    		   &ipoint[nib], &access, &type, &zero, &zero);

   if (stat != E_SUCC )
      {
      c_errmsg ("Error opening input image", "power-eopens",&fatal_err);
      }

   nib += num_image_bands[n];
}  

/*
Open the output image file.
--------------------------
*/
access = IWRITE;
type = EREAL;

stat = c_eopens (&fdesc[n], outhos, num_outbands, &one, &one,
&wind[0][NL], &wind[0][NS], opoint, &access, &type, &zero, &zero);
                             
if (stat != E_SUCC )       
   {
   c_errmsg ("Error opening output image", "power-eopens", &fatal_err);
   }
                     
fdesc[n+1]=0;

for (n = 0; n < *num_outbands; n++)
   out_img_bands[0][n] = n + 1;
                
/*                                        
set up group containing input, and output bands
-----------------------------------------------------------
*/
if ((iobuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
   {                                                 
   c_errmsg ("Error allocating dynamic memory", "power-ealloc",&fatal_err);
   }

if (c_egroup (fdesc, &gdesc, iobuf, &bufsiz) != E_SUCC)
    c_abort (&block, "Error grouping image bands", "Power-group",
              prog_name, out_files, &num_outfiles);                
  
/*
Update the ddr file.
--------------------
*/                                      
c_upddr (&block, inhos, outhos, num_inimages, wind, image_band_num, 
         num_image_bands, &line_inc, &samp_inc, num_outbands, &comb_flag);

strcpy (out_files[num_outfiles++], outhos);
}
