
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct  PARBLK  block;                 /* TAE parameter block     */

float   *iobuf;                      /* image i/o buffer        */
static  char    *prog_name = "POWER"; /* program name            */
char    out_files[2][CMLEN];             /* output file names       */

struct FDESC *fdesc[MAXIMG+2];           /* input file descriptors  */
struct GDESC *gdesc;                       /*  group descriptor  */

long    ipoint[MAXIMG * MAXBND + 1];   /* offsets into image i/o  */
long    opoint[MAXBND];                /* for input and output bands */
long    wind[MAXIMG][4];         /* input image window          */


long    min_max;                       /* minmax option flag      */
long    msgtim;                        /* message interval        */
long    num_outfiles;                  /* number of output files  */

extern long     curent;                        /* current entity           */
extern long     curcat;                        /* current category         */

static  long    one = 1;

int main
(void)
{

/*
********************************************************************

NAME: POWER

PURPOSE:    POWER is used to raise a single or multi-band input image
	    to a user-specified power.

PROGRAM HISTORY:
  Version   Date       Author                       Request
  -------   ----       ------                       -------
                          
  1.0     10-Apr-84    S. Ross
  2.0     26-Mar-86    J. Reed
  3.0     30-Mar-90    W. Xia                 conversion to 5.0
  4.0	  24-Sep-92    T. Mittan	      Modified include file to 
					      eliminate warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
 
call getpar to receive the parameters from the user
call filspec to open input and output images
call runexp  execute Image I/O for function option.
call c_minmax to calculate the minimum and maximum values for output image
call c_egclse to close all the images
call c_hstout to generate the history file
call c_complt to print a completion message

*************************************************************************
*/                     
long
	image_band_num[MAXIMG][MAXBND+1],  /* band numbers for each image */
	num_image_bands[MAXIMG], /* number of bands in each input image  */ 
	num_inbands,             /* total number of input image bands*/
	num_inimages,            /* number of input images           */
	num_outbands,            /* number of bands in output image  */
    out_img_bands[MAXIMG][MAXBND+1];  /* bands numbers for the output 
                                             image */
char
        inhos[MAXIMG][CMLEN],         /*    input image host name      */  
        intae[MAXIMG][CMLEN],         /*    input image tae name       */
        outhos[1][CMLEN],         /*    output image host name     */
        outtae[1][CMLEN];         /*    output image tae name      */  

double  
  	amax[MAXBND],		  /*    maximum pixel value        */
  	amin[MAXBND];		  /*    minimum pixel value        */
	
float	expo;                     /* exponent value */

/*
get the user specified parameters
---------------------------------
*/                 
                            
getpar(intae, inhos, outtae, outhos, &num_inimages, num_image_bands, 
image_band_num, &num_inbands, &num_outbands, &expo);

/*     
Set up file specifications.
---------------------------
*/

if (num_outbands != 0)                                          
  filspec (inhos, outhos[0],  &num_inimages, num_image_bands, 
  image_band_num, &num_outbands, out_img_bands);

/*
 raise input image to the specified power.
 -----------------------------------------
*/                      
   runexp (num_inbands, &num_outbands, &expo);         
              
/*
calculate the minimum and maximum pixel values for the output image
-------------------------------------------------------------------
*/
if(min_max)
{  
    wind[0][SS] = wind[0][SL] = 1;
    c_minmax (outhos, &one, wind, image_band_num, out_img_bands[0], amin, amax);
    c_svmnmx (outhos[0], &num_outbands, amin, amax, "YES");
}
/*
Generate the history file and print the completion message.
------------------------------------------------------------
*/
c_hstout (&block, outhos[0], inhos, &num_inimages, " ");
c_complt (prog_name, out_files, &num_outfiles);

return EXIT_SUCCESS;

}
