#include "power.h"
#include "util.h"
#include "protos.h"

void runexp
(
    long num_inbands,             /* total number of input image bands*/
    long *num_outbands,           /* number of bands in output image  */
    float *expo                     /* exponent value for input images */
)

/*
********************************************************************

NAME: P3DSUB

PURPOSE: RUNEXP is called by POWER to perform the exponential
         operation on a single input image.

PROGRAM HISTORY:
  Version   Date       Author                       Request
  -------   ----       ------                       -------
  1.0     10-Apr-84    S. Ross
  2.0     26-Mar-86    J. Reed
  3.0     30-Mar-90    W. Xia                 conversion to 5.0
                 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

- Use c_inittm, startm to initialize and start the processing message utility
- To perform the exponential operation for each line of the input image
- To free dynamic arrays
- To stop processing message utility
- To close all the images

****************************************************************************
*/
{       

long    fatal_err = -1;            /* fatal error return status */
long    inf_err = 1;                   /* info error return status  */
long    zero = 0;
char    *prog_name = "POWER"; /* program name            */

/* local variables */
long    
	firmsg,                  /* massage flag   */ 
        ibnd,                    /* process band loop index      */
        line,                    /* process line loop index      */
        px,                      /* process pixel loop index */           
        stat;                  

/*                       
initialize and enable the processing message utility        
-----------------------------------------------------
*/                                             
c_inittm (prog_name, "Processed", "line", "bands", " ", 
&wind[0][NL], num_outbands, &msgtim, &zero);            
curent=1;    
c_startm();
firmsg=TRUE;    

/*
 loop to process exponentiation for each line 
 ---------------------------------------------              
*/
for (line = 0; line< wind[0][NL]; line++, curent++)
{                                       /* start of line loop */
   stat=c_estep (&gdesc);
  
   if (stat != E_SUCC )
      {
      c_errmsg ("Error in estep", "stereo-estep",&fatal_err);
      }
 
   for (ibnd=0; ibnd < num_inbands; ibnd++)
   {                                       /* start of band loop */
     for(px = 0; px < wind[0][NS]; px++)
     {
      if (iobuf[ipoint[ibnd]+px] <= 0)
         {
         if (firmsg)
            {
            c_errmsg ("Input data contains negative values.",
                      "power-runexp",&inf_err);
            c_errmsg ("These will be set to zero for output.",
                      "power-runexp",&inf_err);
            firmsg = FALSE;
            }  /* if */
         iobuf[opoint[ibnd]+px] = 0.0;
         }
      else
         {
         iobuf[opoint[ibnd]+px] = pow(iobuf[ipoint[ibnd]+px],*expo);
         }  /* if */
     }   			/* end of px loop */ 
   }  				/* end of ibnd loop */      
}				/* end of line loop */              

stat=c_estep (&gdesc);

if (stat != E_SUCC )
   {
   c_errmsg ("Error in estep", "stereo-estep",&fatal_err);
   }

/*                                                 
disable the display message utility
-----------------------------------
*/    
c_stoptm(); 

/*
close image files 
-----------------
*/
c_egclse( &gdesc);

}
