#include "redist.h"
#include "util.h"
#include "protos.h"

/*
**********************************************************************

   name- breakp (onbins, scale, offset, prob, grey, diff)
                                    
   version - 1.0       date- sept-84   programmer - y. sheu 
   version - 2.0       date- Sept-90   programmer - F. Irani

   function- computes the grey sclaes of break points and
             the ranges between break points
             the grey scales and the ranges will be used
             for interpolation later

PDL:
Begin breakp
	Allocate local work space.
	For each band
		Compute grey levels at each break points.
		Compute the ratios of probability difference 
		and grey differences for each histogram band.
	Free local work space.
End breakp
**********************************************************************

*/

void breakp
(
    long *onbins,	/* Number of bins pdf band array	*/
    float *scale,		/* Scalars pdf band array		*/
    float *offset,	/* Offsets per band array		*/
    float **prob,		/* matrix of probabilities		*/
    float **grey,		/* matrix of GLV's 			*/
    float **diff		/* matrix of differences		*/
)

{

char    *prog_name = "REDIST";  /* program name            */
long    zero = 0;

float	*oscale,	/* Local scalars per band		*/
	*outset;	/* Local proportional offsets per band	*/

long	band,		/* Band loop counter			*/
	bin;		/* Bin loop caounter			*/

/*
Allocate local work space.
--------------------------					*/
if ((oscale = (float *) malloc(totbands * sizeof(float))) == NULL)
{   
	c_abort (&block, "Error allocating dynamic memory",
		 "redist-alloc", prog_name, hosout, &zero);
}

if ((outset= (float *) malloc(totbands * sizeof(float))) == NULL)
{
	c_abort (&block, "Error allocating dynamic memory",
		 "redist-alloc", prog_name, hosout, &zero);
}


for (band = 0; band < totbands; ++band)
{
	/*
	Compute grey levels at each break points.
	-----------------------------------------		*/
	oscale[band] = 1 / scale[band];
	
	outset[band] = -1. * (offset[band] / scale[band]);
	
	for (bin = 0; bin < onbins[band]; ++bin)
		grey[band][bin] = (oscale[band] * (float) bin) + outset[band];
	
	/*
	Compute the ratios of probability difference 
	and grey differences for each histogram band.
	---------------------------------------------		*/
	diff[band][0] = prob[band][0] * scale[band];

	for (bin = 1; bin < onbins[band]; ++bin)
		diff[band][bin] = (prob[band][bin] - prob[band][bin-1]) * 
				   scale[band];
}

/*
Free local work space.
----------------------						*/
free(oscale);
free(outset);

return;

}
