#include "redist.h"
#include "util.h"
#include "protos.h"

/*
**********************************************************************

   Program files()
 
   version- 1.0    date- sept-84  programmer- y. sheu
   version- 2.0    date- Sept-90  programmer- F. Irani, STX: V5.0 conversion
 
   function- establishes file specifications for second pass image I/O
PDL:
	Begin files
		Open input image files.
		Open output image files to 
		convert from input to output datatype on write.
		Allocate input/output buffer space.
		Set up group containing input and output bands.
		Update ddr.
	End files
**********************************************************************

*/

void files
(void)

{

char    *prog_name = "REDIST";  /* program name            */
long    fatal_err = -1;
long    zero = 0;
long    one = 1;

double	d_one;			/* c_upddr parameter constant   	*/

long	access,                 /* image i/o file access type		*/
	comb_flag = NOCOMB,	/* UPDDR flag 				*/
	image,			/* Image loop counter			*/
	nib,			/* band number loop index		*/
	opt[2];			/* For eopens, output conversion 	*/

d_one = (double) one;

/*
Open input image files.
-----------------------							*/
access = IREAD;
for (image = 0, nib = 0; image < images; ++image)
{
	if ( c_eopens(  &fdin[image], hosin[image],
			bandnums[image], &windows[0][SL], 
			&windows[0][SS], &windows[0][NL], &windows[0][NS], 
			&ipoint[nib], &access, &itype, &zero, &zero) 
 	!= E_SUCC)
	   		c_errmsg("Error opening input image", 
				 "redist-open", &fatal_err);

	nib+= bands[image];
}  

/*
Open output image files to convert from input to output datatype on write.
-------------------------------------------------------------------------- */
access = IWRITE;
opt[0] = otype;
opt[1] = 0;
if (c_eopens (&fdin[images], hosout[0], &totbands, &one, &one,
	      &windows[0][NL], &windows[0][NS], 
	      &ipoint[totbands], &access, &itype, &zero, &opt[0])    != E_SUCC)
		      c_abort (&block, "Error opening output image",
		               "redist-outopen", prog_name, hosout, &one);
fdin[images + 1] = 0;

/*
Allocate input/output buffer space.
-----------------------------------					*/
if ((inbuf = (unsigned char *) c_ealloc(fdin, &insiz)) == NULL)
	c_abort (&block, "Error allocating dynamic memory",
		 "redist-alloc", prog_name, hosout, &zero);

/*
Set up group containing input bands.
------------------------------------					*/
c_egroup (fdin, &gdin, inbuf, &insiz);

/*
Update ddr.
-----------								*/
if (c_upddr (&block, hosin, hosout[0], &images, windows, bandnums, 
         bands, &d_one, &d_one, &totbands, &comb_flag) != E_SUCC)
	c_abort (&block, "Error updating ddr",
		 "redist-nupddr", prog_name, hosout, &zero);

return;

}
