#include <string.h>
#include "redist.h"
#include "util.h"
#include "protos.h"

/*
     getpar()

**********************************************************************

   name- getpar

   language- C

   version- 1.0      date- oct-84       programmer- y.sheu
   version- 2.0      date- Sep-84       Programmer- F. Irani V5.0 Conversion

   function- receives parameters from tae process definition file 

	pdl:
	begin getpar
		Call c_inlas to initialize TAE.
		Retrieve two TAE global values.
		Get input image names.
		Get output image names.
		Get output data type.
		Get input data type.
		Get output gray scale indicator if output data type is not byte.
	end getpar
**********************************************************************

*/

void getpar
(void)

{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;

struct	DDR	ddr;			/* Input image data descriptor */

char	gryout[13],			/* Processing method		*/
	mnmx[7],			/* ddr minmax update flag	*/
	typout[7];			/* Output data type 		*/

long	count,				/* Tae parameter count variable */
	imglmt = MAXIMG,		/* maximum input images allowed */
	same_window,			/* Getin flag: Same image size  */
	sdim,				/* Length of TAE parameter 	*/
	std_out,			/* Parameter for inlas	   	*/
	type = ELONG;			/* $MSGTIME value data type	*/

/*
Initialize TAE.
---------------- 		*/
c_inlas (&block, &std_out); 

/* 
Retrieve two TAE global values.
-------------------------------					*/
sdim = 5;
count = 1;
if (c_getstr( &block, "$minmax", &count, &sdim, mnmx) <= 0)
    c_errmsg("Error retrieving global value $minmax",
	     "redist-nogbl", &fatal_err);
if (strncmp(mnmx, "Y", 1) == 0)
	min_max = TRUE;
else
	min_max = FALSE;

if (c_getnum (&block, "$msgtim", &one, &type, &msgtim) == E_FAIL)
    c_errmsg("Error retrieving global value $msgtim",
	     "redist-nogbl", &fatal_err);

/*
Get input image names.
----------------------						*/
if (c_getin (&block, "IN", &zero, taein, hosin, &images, 
	     windows, bandnums, bands, &totbands,
	     &imglmt, &same_window, img_flag) == E_FAIL)
   c_errmsg ("Cannot access input file names",
	     "redist-getin", &fatal_err);
else if (!same_window)
	c_errmsg ("All bands must have the same input window size.",
		  "redist-getin", &fatal_err);

/*
Get output image names.
-----------------------				*/
else if (c_getout (&block, "OUT", taeout, hosout) == E_FAIL)
   	 c_errmsg ("Cannot access input parameter OUT", 
	     	   "redist-getout", &fatal_err);
/*
Get output data type.
---------------------				*/
sdim = 5;
count = 1;
if (c_getstr( &block, "ODTYPE", &count, &sdim, typout) <= 0)
    c_errmsg("Error retrieving parameter ODTYPE",
	     "redist-nogbl", &fatal_err);

/*
Get input data type.
--------------------				*/
c_getddr(hosin[0], &ddr);
itype = ddr.dtype;

if (strcmp(typout,"BYTE") == 0)
	otype = EBYTE;
else
	otype = itype;

/*
Get output gray scale indicator if output data type is not byte.
----------------------------------------------------------------	*/
if (otype  !=  EBYTE) 
{
	if (c_getstr( &block, "PROCMETH", &count, &sdim, gryout) <= 0)
	    c_errmsg("Error retrieving value for PROCMETH",
	    "redist-nogbl", &fatal_err);

	gout = FALSE;          
	if (strncmp(gryout, "I", 1) == 0) 
		gout = TRUE;
}

return;
}
