#include "redist.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
**********************************************************************

   name- histgm(taein,hosin,offset,scale,hist,onbins,ibands)

   language- fortran   

   version - 1.0       date- sept-84   programmer - Y. Sheu:Original development
   version - 2.0       date- sept-90   programmer - F. Irani:V5.0 Conversion

   function- compute cumulative histogram for the input image

PDL:
Begin histgm

	Allocate local work space.
	Check allocation status.
	Set multiplier for inbuf buffer indexing.
	Check the validity of the min-max values, and compute them if not valid.
	Initialize the scale and offset arrays for all bands.
	for each band
		Check if input images have constant pixel value or not.
		assign onbins value
		If pixel type is other than byte, compute scale and offset
		factor to normalize pixel values from 0 to onbins-1.
	Allocate histogram workspace.
	Zero histogram and "count" arrays.
	Call inimg to Set up file groups for input image.
	Initialize timed processing message.
	Begin processing line message.
	Read one input line of all bands (of all images).
	For each image line
		For each band
			Zero out temporay work space.
			Convert to work-space data type.
			Compute histogram:
		Write out image line and read in new line.
	Compute accumulative histogram.
	Free input image buffer.
	Stop Processing Line Message
	Free local work space.
End histgm
**********************************************************************

*/


void histgm
(
    long **hist,			/* Hisogram per band array		*/
    float *offset,		/* Offset per band array		*/
    long *onbins,		/* Number of bins per band array	*/
    float *scale			/* Scale  per band array		*/
)

{

long    msg_flag = ALLCAT; /* Message types:        */
long    fatal_err = -1;
long    zero = 0;
long    wtype = EWORD;
long    ltype = ELONG;
long    rtype = EREAL;

float	range,			/* GLV Range 				*/
	rzero,			/* zero value for pxfill		*/
	*temp1,			/* temporary buffer allocate		*/
	*tempf;		 	/* temporary buffer allocate		*/

long	band,			/* Band counter				*/
	bin,			/* Histogram bin loop counter		*/
	*count,			/* c_pxhist # bins filled		*/
	line, 			/* Loop counters			*/
	*temp4;			/* temp buffer -allocate?		*/

short	*temp2,			/* temp buffer -allocate?		*/
	wzero;			/* zero value for pxfill		*/

wzero = (short) zero;
rzero = (float) zero;

/*
Allocate local work space.
--------------------------						*/
tempf = (float *) malloc(windows[0][NS] * sizeof(float));		
temp1 = (float *) malloc(windows[0][NS] * sizeof(float)); 
temp4 =  (long *) malloc(windows[0][NS] * sizeof(long));
count =  (long *) malloc(totbands * sizeof(long));
temp2 = (short *) malloc(windows[0][NS] * sizeof(short));

/*
Check allocation status.
------------------------						*/
if ( tempf == NULL || temp1 == NULL || temp4 == NULL || 
     count == NULL || temp2 == NULL)
{
	c_errmsg ("Error allocating dynamic memory","redist-alloc", 
	          &fatal_err);
}

/*
Set multiplier for inbuf buffer indexing.
-----------------------------------------	*/
switch (itype)
{
	case EBYTE: isiz = 1; break;
	case EWORD: isiz = 2; break;
	case ELONG: isiz = 4; break;
	case EREAL: isiz = 4; break;
}

/*
Check the validity of the min-max values, and compute them if not valid.
------------------------------------------------------------------------ */
c_minmax (hosin, &images, windows, bandnums, bands, minval, maxval);

/*
Initialize the scale and offset arrays for all bands.
-----------------------------------------------------			*/
for (band = 0; band < totbands; ++band)
{
	/*
	Check if input images have constant pixel value or not.
	-------------------------------------------------------		*/
	range = maxval[band] - minval[band];
	if (range  ==  0.0)
		c_errmsg ("All values in an image band are equal.",
			  "REDIST-NORANGE", &fatal_err);

	if (itype  ==  EBYTE) 
		onbins[band] = (long) (maxval[band] + 1);
	else if (range  <  MAXBIN) 
		onbins[band] = (long) (range + 1);
	else 
		onbins[band] = MAXBIN;
	
	/*
	If pixel type is other than byte, compute scale 
	factor to normalize pixel values from 0 to onbins-1.
	----------------------------------------------------		*/
	if (itype  !=  EBYTE)  
	{
		if (range  <  MAXBIN) 
		{
			scale[band]  =  1.0;
			offset[band] = -1.0 * minval[band];
		}
		else if (itype == EREAL) 
		{
			/*
			Scale for REAL.
			---------------				*/
			scale[band]  = (float) onbins[band] / 
				       (maxval[band]  - minval[band]) +
					0.0000001;

			offset[band] =  -1.0 * minval[band] *
				  	 scale[band];
		}
		else 
		{
			/*
			Scale for INTEGERS.
			-------------------			*/
			scale[band]  = (float) onbins[band]   / 
				       (maxval[band]  - 
				        minval[band] + 1.0);
			offset[band] = -1.0 * minval[band] * 
					scale[band];
		}
	}
}	/* For each band of an input image */

/*
Allocate histogram workspace. (Free this is HCUM).
--------------------------------------------------			*/
for (band = 0; band < totbands; ++band)
{
	if ((hist[band] = 
	   (long *) malloc(onbins[band] * sizeof (long))) == NULL)
	{
		c_errmsg ("Error allocating dynamic memory","redist-alloc", 
			  &fatal_err);
	}
}

/*
Zero histogram and count arrays.
--------------------------------					*/
for (band = 0; band < totbands; ++band)
{
	c_pxfill (hist[band], &ltype, &onbins[band], &zero);
	count[band] =  0;
}

/*
Set up file groups for input image.
-----------------------------------				*/
inimg();

/*
Initialize timed processing message.
------------------------------------				*/
c_inittm ("REDIST", "Building histogram: Processing", "line", "bands", "",
	  &windows[0][NL], &totbands, &msgtim, &msg_flag);

curent = 1;

/*
Begin processing line message.
------------------------------					*/
c_startm();

/*
Read one input line of all bands (of all images).
-------------------------------------------------	*/
c_estep (&gdh);

for (line = 1; line <= windows[0][NL]; ++line, ++curent)
{
	for (band = 0; band < totbands; ++band)
	{
		/*
		Zero out temporay work space.
		-----------------------------			*/
		c_pxfill (temp1, &rtype, &windows[0][NS], &rzero);
		c_pxfill (tempf, &rtype, &windows[0][NS], &rzero);
		c_pxfill (temp4, &ltype, &windows[0][NS], &zero);
		c_pxfill (temp2, &wtype, &windows[0][NS], &wzero);

		/*
		Convert to work-space data type. (Go for V4.1 precision.)
		--------------------------------------------------------- */
		if (itype == EBYTE) 
		{
			c_pxconv(&itype, &wtype, &hbuf[hpoint[band]], 
				 temp2, &windows[0][NS]);

			c_pxconv(&wtype, &ltype, temp2, temp4,
				 &windows[0][NS]);
		}
		else 
		{

			/*
			Conversions are to reduce precision.
			------------------------------------		*/
			c_pxconv(&itype, &rtype, &hbuf[hpoint[band]*isiz], 
				 tempf, &windows[0][NS]);

			c_pxline(tempf, temp1, &windows[0][NS], &rtype,
				 &scale[band], &offset[band]);

			c_pxconv(&rtype, &wtype, temp1, temp2, 
				 &windows[0][NS]);

			c_pxconv(&wtype, &ltype, temp2, temp4,
				 &windows[0][NS]);
		}

		/*
		Compute histogram:
		------------------					*/
		c_pxhist(&windows[0][NS], temp4, &ltype, hist[band], 
			 &count[band]);

	}	/* For each band  */

	/*
	Read/Write.
	------------			*/
	c_estep (&gdh);

}	/* For each input line  */

/*
Compute accumulative histogram.
-------------------------------		*/
for (band = 0; band < totbands; ++band)
	for (bin = 1; bin < onbins[band]; ++bin)
		hist[band][bin] = hist[band][bin - 1] + hist[band][bin];

/*
Free input image buffer.
------------------------						*/
free(hbuf);

c_egclse(&gdh);

c_stoptm();

/*
Free local work space.
----------------------							*/
free(temp1);
free(tempf);
free(temp4);
free(count);
free(temp2);

return;

} 
