
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAXBIN  1024

static char    *prog_name = "REDIST";  /* program name            */

static  long    fatal_err = -1;
static  long    one = 1;

struct  PARBLK  block;                 /* TAE parameter block     */

char    hosin[MAXIMG][CMLEN];   /* input hostfile name          */
char    hosout[1][CMLEN];          /* output hostfile name         */
char    taein[MAXIMG][CMLEN];   /* input LAS name               */
char    taeout[1][CMLEN];          /* output LAS name              */


double  minval[MAXBND];         /* Min-Max values for svmnmx    */
double  maxval[MAXBND];         /* Min-Max values for svmnmx    */

unsigned char   *inbuf;         /* image i/o buffer             */
unsigned char   *hbuf;          /* image i/o buffer             */

long    bandnums[MAXIMG][MAXBND+1];/* band numbers per image    */
long    bands[MAXIMG];          /* Number of bands per image    */
long    insiz;                  /* input buffer size            */
long    isiz;                   /* input data type size         */
extern long     curent;                 /* parameters for processing    */
struct FDESC *fdh[MAXIMG];            /* histogram file descriptors   */
struct FDESC *fdin[MAXIMG];           /* input file descriptors       */
struct GDESC *gdh;                    /* group descriptor             */
struct GDESC *gdin;                   /* group descriptor             */
long    gout;                   /* output result option flag    */
long    images;                 /* Number of input images       */
long    img_flag[MAXIMG][2];    /* c_getin parm                 */
long    *hpoint;  /* input buffer offsets  */
long    *ipoint;  /* input buffer offsets  */
long    itype;                  /* inputdata type               */
long    min_max;                /* minmax option flag           */
long    msgtim;                 /* message interval             */
long    otype;                  /* output data type             */
long    totbands;               /* Total number on input bands  */
long    windows[MAXIMG][4];     /* input image dimensions       */

/**********************************************************************

   program redist

   main driver
 
   name- redist                         

   language- C

   version- 1.0   date- sept - 84      programmer- Y. Sheu, SAR: Original 
   version- 2.0   date- Sept - 90      programmer- F. Irani/Stx:V5.0 Conversion
		  date- Nov 2, 1988    programmer- EDC: Renamed routine PARMS
				       to GETPARM to fix conflict if
				       CM$ROOT:INSLOGIN was invoked.
	          date- Oct 18, 1990   programmer- F. Irani/STX@GSFC: Renamed
				       routine GETPARM to GETPAR to conform with
				       other GSFC las applications conventions.
		 date Sept -1992       modified include file to eliminate 
				       warnings on link
 
   function- performs an intensity mapping on the input image
             a uniform histogram equalization is applied

PDL:
Begin redist
	Call getpar to get TAE  parameters.
	Allocate function parameter Work areas.
	Allocate histogram workspace.
	call histgm to obtain accumulative histogram for input image(s).
	Allocate probability pointer.
	Allocate transformation pointer.
	Call hcum to compute transformation function for all bands
	based on computed accumulative histogram.
	Free histogram pointer.
	Call files to Set up input & output images to do mapping.
	Call tranfm to apply transformation to input image 
	and create output image.
	Free work areas.
	Free Matrix Pointers.
	if $MINMAX is turned on
		Determine the minimum and maximum values of the output image.
		Write the min, max and the validity flag values to the ddr file.
	Generate the history file and print the completion message.
end redist
**********************************************************************


*/

int main
(void)

{

float	*offset,		/* Specified offsets per band		*/
	**prob,			/* probability matrix per band    	*/
	*scale;			/* Specified scales per band		*/
	
long	**hist,			/* Cumulative histogram array		*/
	band,			/* Band loop counter			*/
	*onbins;		/* Number of histogram bins per band 	*/

short	**transf;		/* Transformations per band matrix	*/


/*
Get TAE  parameters.
--------------------							*/
getpar ();

/*
Allocate function parameter Work areas.
---------------------------------------					*/
offset =  (float *) malloc(totbands * sizeof(float));
scale  =  (float *) malloc(totbands * sizeof(float));
onbins =   (long *) malloc(totbands * sizeof(long));
hpoint =   (long *) malloc((MAXIMG*MAXBND+1) * sizeof(long));
ipoint =   (long *) malloc((MAXIMG*MAXBND+1) * sizeof(long));


/*
Allocate histogram workspace. (Free this after hcum).
-----------------------------------------------------			*/
if ((hist = (long **) malloc (totbands * sizeof(long *) ) ) == NULL)
{
	c_errmsg ("Error allocating dynamic memory","redist-alloc", 
	          &fatal_err);
}

/*
Obtain accumulative histogram for input image(s).
-------------------------------------------------			*/
histgm(hist, offset, onbins, scale);

/*
Allocate probability pointer. (Free this after tranfm).
-------------------------------------------------------			*/
if ((prob = (float **) malloc (totbands * sizeof(float *) ) ) == NULL)
{
	c_errmsg ("Error allocating dynamic memory","redist-alloc", 
	          &fatal_err);
}

/*
Allocate transformation pointer. (Free this after tranfm).
-------------------------------------------------------			*/
if ((transf = (short **) malloc (totbands * sizeof(short *) ) ) == NULL)
{
	c_errmsg ("Error allocating dynamic memory","redist-alloc", 
	          &fatal_err);
}

/*
Compute transformation function for all bands
based on computed accumulative histogram.
-----------------------------------------				*/
hcum (hist, onbins, prob, transf);	
	
/*
Free histogram pointer.
-----------------------							*/
free(hist);

/*
Set up input & output images to do mapping.
-------------------------------------------				*/
files ();
	
/*
Following is for DEBUG purposes:
--------------------------------					
printf("\n onbins %10.5d \n", onbins[0]);

printf("\n offset %10.5f \n", offset[0]);

printf("\n scale %10.5f \n", scale[0]);

printf("\n hist \n");
for (i = 0; i < 255; ++i)
{
	if ( (i % 5) == 0)
		printf("\n%4d",i);
	printf("%10d ", hist[0][i]);
}

printf("\n prob \n");
for (i = 0; i < 255; ++i)
{
	if ( (i % 5) == 0)
		printf("\n%4d",i);
	printf("%10.5f     ", prob[0][i]);
}

printf("\n transf\n");
for (i = 0; i < 255; ++i)
{
	if ( (i % 11) == 0)
		printf("\n%4d",i);
	printf("%6d ", transf[0][i]);
}

End DEBUG.
----------								*/

/*
Apply transformation to input image and create output image.
------------------------------------------------------------  		*/
tranfm (onbins, offset, prob, scale, transf); 
	

/*
Free work areas.
----------------							*/
free(offset);
free(scale);
free(onbins);

/*
Free Matrix Pointers.
---------------------							*/
free(prob);
free(transf);

/* 
Determine the minimum and maximum values of the output image.
Write the min, max and the validity flag values to the ddr file.
---------------------------------------------------------------- 	*/
if (min_max)
{
	for (band = 0; band < totbands; ++band)
	{
		bandnums[0][band] = band + 1;
	}
	windows[0][0] = windows[0][1] = 1;
	bands[0] = totbands;

	c_minmax (hosout, &one, windows, bandnums,
		  bands, minval, maxval);

   	c_svmnmx (hosout[0], &totbands, minval, maxval, "YES"); 
}

/*
Generate the history file and print the completion message.
------------------------------------------------------------ 		*/
c_hstout (&block, hosout[0], hosin, &images, " ");

c_complt (prog_name, taeout, &one);

return EXIT_SUCCESS;

}
