#include "redist.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
**********************************************************************

   name- tranfm(onbins,scale,offse,transf,prob,bands)

   version - 1.0       date- oct-84  programmer - y. sheu:  Original
   version - 1.0       date- Sep-90  programmer - F. Irani: V5.0 conversion

   function- apply the transformation function to the input image
             to create an output image

PDL:
Begin tranfm
	Allocate temporary space for diff matrix.
	Allocate temporary space for grey matrix.
	Allocate other local work space.
	Check allocation status.
	Initialize timed processing message.
	if output data type is notyte and interpolation is required.
	Read input line and initialize/write output line.
	Start Processing line message.
	For each image line
		for each band
			if the input image is not byte data
				Scale the grey levels 
			else if input image is floating point data
				Rescale input image data.
			else if input data is I*2 or I*4 data 
				Rescale input image data 
				and apply linear scaling.
			Convert scaled grey levels to I*2.
			Apply the transformation 
			function to the input grey levels.
		Write or flush output line and read input line.
	Free matricies work space.
	Free local work space.
	Free local pointers.
	Free remaining local work space.
	Close files.
	Stop Processing line message.
End tranfm
**********************************************************************

*/

void tranfm
(
    long *onbins,	/* Number of bins pdf band array		*/
    float *offset,	/* Offsets per band array			*/
    float **prob,		/* Matrix of probailities			*/
    float *scale,		/* Scalers per band array			*/
    short **transf	/* Matrix of transformations			*/
)

{

char    *prog_name = "REDIST";  /* program name            */
long    msg_flag = ALLCAT; /* Message types:        */
long    zero = 0;
long    wtype = EWORD;
long    rtype = EREAL;

float	**diff,		/* matrix of differences 			*/
	**grey,		/* matrix of GLV's 				*/
	*outset,	/* Proportional offset array			*/
	*oscale,	/* Local scaler array				*/
	*tran,		/* local work space				*/
	*temp2,		/* local work space				*/
	*temp3;		/* local work space				*/

long 	band,		/* Band loop counter				*/
	line,		/* Image line loop counter			*/
	oband,		/* Output band pointer index			*/
	samp;		/* Line sample loop counter			*/

short	*temp,		/* local work space				*/
	*temp4;		/* local work space				*/

/*
Allocate temporary space for diff matrix.
-----------------------------------------				*/
if ((diff = (float **) malloc (totbands * sizeof(float *) ) ) == NULL)
{
	c_abort (&block, "Error allocating dynamic memory",
		 "redist-alloc", prog_name, hosout, &zero);
}

for (band = 0; band < totbands; ++band)
{
	if ((diff[band] = 
	   (float *) malloc(onbins[band] * sizeof (float))) == NULL)
	{
		c_abort (&block, "Error allocating dynamic memory",
			 "redist-alloc", prog_name, hosout, &zero);
	}
}

/*
Allocate temporary space for grey matrix.
-----------------------------------------				*/
if ( ( grey = (float **) malloc (totbands * sizeof(float *)  ) ) == NULL)
{
	c_abort (&block, "Error allocating dynamic memory",
		 "redist-alloc", prog_name, hosout, &zero);
}

for (band = 0; band < totbands; ++band)
{
	if ((grey[band] = 
           (float *) malloc(onbins[band] * sizeof (float) ) ) == NULL)
	{
		c_abort (&block, "Error allocating dynamic memory",
			 "redist-alloc", prog_name, hosout, &zero);
	}
}

/*
Allocate other local work space.
--------------------------------					*/
outset = (float *) malloc(totbands 	 * sizeof(float));
oscale = (float *) malloc(totbands 	 * sizeof(float));
tran   = (float *) malloc(windows[0][NS] * sizeof(float));
temp2  = (float *) malloc(windows[0][NS] * sizeof(float));
temp3  = (float *) malloc(windows[0][NS] * sizeof(float));
temp   = (short *) malloc(windows[0][NS] * sizeof(short));
temp4  = (short *) malloc(windows[0][NS] * sizeof(short));

/*
Check allocation status.
------------------------						*/
if ( outset == NULL || oscale == NULL || tran   == NULL || 
     temp2  == NULL || temp3  == NULL || temp   == NULL || temp4  == NULL) 
{
	c_abort (&block, "Error allocating dynamic memory",
		 "redist-alloc", prog_name, hosout, &zero);
}


/*
Initialize timed processing message.
------------------------------------					*/
c_inittm ("REDIST", "Applying mapping for", "line", "bands", "",
	  &windows[0][NL], &totbands, &msgtim, &msg_flag);

curent = 1;

/*
Compute grey levels at each break point if otype is not byte.
gout=FALSE for discrete output, gout=TRUE for interpolated output.
------------------------------------------------------------------	*/
if (otype  !=  EBYTE  &&  gout) 
	breakp (onbins, scale, offset, prob, grey, diff);

/*
Read input line and initialize/write output line.
-------------------------------------------------		*/
if (c_estep (&gdin) != E_SUCC)
{
	c_abort (&block, "Error in image read/write.",
		 "redist-estep", prog_name, hosout, &zero);
}

/*
Start Processing line message.
------------------------------						*/
c_startm();

for (line = 1, curent = 1; line <= windows[0][NL]; ++line, ++curent)
{

	for (band = 0; band < totbands; ++band)
	{
		oband = totbands + band;

		/*
		Scale the grey levels if the input image is not byte data.
		----------------------------------------------------------  */
		if (itype  ==  EBYTE) 
		{
			/*
			No need to rescale BYTE input image data.
			-----------------------------------------	*/
			c_pxconv(&itype, &rtype, &inbuf[ipoint[band]*isiz],
				 temp3, &windows[0][NS]);
		}
		else if (itype  ==  EREAL) 
 		{
			/*
			Rescale input image data.
			-------------------------			*/
			c_pxline(&inbuf[ipoint[band]*isiz], temp3, 
				 &windows[0][NS], &rtype, &scale[band], 
				 &offset[band]);
		}
		else    /* I*2 or Long Data Types */
 		{
			/*
			Rescale input image data.
			-------------------------			*/
			c_pxconv(&itype, &rtype, 
				 &inbuf[ipoint[band]*isiz], temp2, 
				 &windows[0][NS]);
			c_pxline(temp2, temp3, &windows[0][NS],
				 &rtype, &scale[band], &offset[band]);
		}
		/*
		Convert (scaled) grey levels to I*2.
		------------------------------------			*/
		c_pxconv(&rtype, &wtype, temp3, temp, &windows[0][NS]);
	
		/*
		Apply the transformation function to the input grey
		levels.  Use look up table for byte OUTPUT data.
		(Buffer is still handling data as type "itype" data.)
		-----------------------------------------------------	*/
		if (otype  ==  EBYTE) 
		{
			c_pxlkup(temp, temp4, &windows[0][NS], 
				 &wtype, transf[band], &wtype);

			c_pxconv(&wtype, &itype, temp4, 
				 &inbuf[ipoint[oband]*isiz], &windows[0][NS]);
		}
		else 
		{
			/*
			Scale back to original range.
			-----------------------------			*/
			if (!gout) 
			{
				/*
				Case 1: Quantize output grey 
				levels to onbins levels.    
				----------------------------		*/
				c_pxlkup(temp, temp4, &windows[0][NS], 
					 &wtype, transf[band], &wtype);

				c_pxconv(&wtype, &rtype, temp4, tran, 
					 &windows[0][NS]);
			}
			else                 
			{
				/*
				Transform the functions linearly.
				---------------------------------	*/
				for (samp = 0; samp < windows[0][NS]; ++samp)
				{
					tran[samp] = (float)
						prob[band][temp[samp]] + 
						diff[band][temp[samp]] * 
						(temp3[samp]       -
						 grey[band][temp[samp]]);
		            	}
			}
			oscale[band] =  1.0 / scale[band];
			outset[band] = -1.0 * (offset[band] / scale[band]);
			c_pxline(tran, temp3, 
				 &windows[0][NS], &rtype,
				 &oscale[band], &outset[band]);

			if (itype == EREAL)
				c_pxconv(&rtype, &itype, 
					 temp3, &inbuf[ipoint[oband]*isiz],
					 &windows[0][NS]);
			else
				c_pxconv(&rtype, &itype, 
					 temp3, &inbuf[ipoint[oband]*isiz],
					 &windows[0][NS]);
		}

	}	/* For each band  					*/


	/*
	Write or flush output line and read input line.
	-----------------------------------------------			*/
	if (c_estep (&gdin) != E_SUCC)
	{
		c_abort (&block, "Error in image read/write.",
			 "redist-estep", prog_name, hosout, &zero);
	}

}	/* For each line  						*/

/*
Free matricies work space.
--------------------------						*/
for (band = 0; band < totbands; ++band)
{
	/*
	Pointers are freed in main.
	---------------------------					*/
	free(transf[band]);
	free(prob[band]);

	/*
	Free local work space.
	----------------------						*/
	free(diff[band]);
	free(grey[band]);
}

/*
Free local pointers here.
-------------------------						*/
free(diff);
free(grey);

/*
Free remaining local work space.
--------------------------------					*/
free( temp4);
free( temp);
free( temp3);
free( temp2); 
free( tran);
free( oscale);
free( outset);

/*
Close files.
------------								*/
c_egclse(&gdin);

/*
Stop Processing line message.
-----------------------------						*/
c_stoptm();

return;

} 
