
/* program history 
May 1992   David Zokaites of EDC revised the contributed proc redist and 
           created redist2.  I added parms MINBV and MAXBV to specify the 
           output data range of each band.  I added parms FROMMASK and 
           TOMASK to allow for masking pixels from the histogram calculation.  
           I fixed some problems including deleting parm PROCMETH, created a 
           problem, and identified still more problems. 
Apr 1993   David Zokaites of EDC scrapped the existing code and started over. 
           I changed the parameters and only allowed byte data. */ 

/* preprocessor directives */ 
#define GLOBALVARS   /* defined only in this file, flag for redist2.h */
#define MAXBV   255  /* max bv for byte data */ 
#define HBINS   256  /* # of histogram bins, program logic assumes 
                        min bv = 0, MAXBV = HBINS -1 */
#include "redist2.h" /* for global vars and local image i/o functions */ 
#include "util.h"
#ifdef DEBUG         /* only defined while debugging */ 
#include <stdio.h>   /* needed for file i/o */ 
FILE *filep;         /* pointer to debugging file */ 
#endif 
/* current entity, for processing message, must be global for las i/o */ 
extern long curent;  


/* FUNCTION:  histogram input image */ 
void make_hist
(
    long msgtim,                 /* input: $msgtim */
    double bweight,              /* band-specific weight for histogram */
    struct iparm *frommask,      /* BV to mask out of histogram */
    struct lasinimage *histimg,  /* input: image to histogram */
    long *chist[MAXBND]          /* output: cumulative histogram for image */
) 
{
   /* declare variables */ 
   double scalar;              /* used to scale histogram */ 
   unsigned char *hbuffer;     /* location of image line in histimg->ibuffer */
   long band,                  /* for loop index:  band */ 
        line,                  /* for loop index:  line */ 
        messform=ALLCAT,       /* output message format */ 
        index,                 /* for loop index:  frommask values */ 
        pixval,                /* for loop index:  pixel value */
        sample,                /* for loop index:  sample */ 
        total[HBINS];          /* total cumulative histogram for all bands */ 
        
   /* initialize and start processing line message */ 
   c_inittm ("REDIST2", "Histogrammed", "line", "bands", "", 
   &histimg->window[0][NL], &histimg->totalbands, &msgtim, &messform); 
   curent = 1; 
   c_startm (); 
   
   /* allocate memory for chist, memory will be freed by make_xform(), zero out
      chist */ 
   for (band=0; band<histimg->totalbands; band++)
   {
      if ((chist[band] = (long *) malloc (sizeof(long) * HBINS) ) == NULL)
         clean2 ("Unable to allocate memory for histogram."); 
      for (pixval=0; pixval<HBINS; pixval++)
         chist[band][pixval] = 0; 
   }
   /* make histogram */ 
   for (line=0; line<histimg->window[0][NL]; line++, curent++)
   {
      for (band=0; band<histimg->totalbands; band++)
      {
         /* set hbuffer */ 
         hbuffer = histimg->ibuffer + histimg->bufoff[band]; 
         /* increment chist */ 
         for (sample=0; sample<histimg->window[0][NS]; sample++)
            chist[band][*(hbuffer+sample)]++; 
      }
      /* step through image, reading a line of data */ 
      if (c_estep (&histimg->groupd) )
         clean2 ("Error stepping through images.");
   }
   c_stoptm(); 

   /* in the following logic, one cannot factor out the loop for all image 
      bands */
   /* apply histogram mask */ 
   for (band=0; band<histimg->totalbands; band++)
      for (index=0; index<frommask->count; index++)
         chist[band][ frommask->value[index] ] = 0; 
   
   /* accumulate histogram */ 
   for (band=0; band<histimg->totalbands; band++)
      for (pixval=1; pixval<HBINS; pixval++)
         chist[band][ pixval ] += chist[band][ pixval-1 ]  ; 
   
   /* average histograms */
   /* determine cumulative histogram for all bands */ 
      for (pixval=0; pixval<HBINS; pixval++)   /* initialize total */ 
         total[pixval]=0;
      for (band=0; band<histimg->totalbands; band++)
         for (pixval=0; pixval<HBINS; pixval++)
            total[pixval] += chist[band][pixval]; 
   
   /* average histogram with total */ 
      scalar = 1. - bweight;          /* scalar and bweight add to 1 */ 
      bweight *= histimg->totalbands; /* scale to # of bands in total[] */
      for (band=0; band<histimg->totalbands; band++)
         for (pixval=0; pixval<HBINS; pixval++)
            chist[band][pixval] = (long)
            ( chist[band][pixval] * bweight  +  total[pixval] * scalar  +.5 );
}






/* FUNCTION:  generate histogram transforming lut from histogram */ 
void make_xform
(
    long *chist[MAXBND],   /* input: cumulative histogram for image */
    long hbands,          /* input: for loop limit, bands in image */
    long maxshift,        /* input: max shift of a BV during transform */
    struct iparm *frommask,/* BV to mask out of histogram */
    struct iparm *tomask,  /* output BV for masked pixels */
    long *tlut[MAXBND]    /* output: histogram transforming lut */
)
{
   /* declare vars */ 
   float cfactor[MAXBND]; /* correction factor for histogram transform */ 
   long band,             /* for loop index, band number */ 
        curshift,         /* current shift of tlut, used to implement parm
                             maxshift */ 
        diff,             /* difference between two consecutive values of tlut*/
        firstbin[MAXBND], /* first histogram bin with > 0 pixels */ 
        maski,            /* for loop index, mask value */ 
        pixval;           /* for loop index, pixel value */ 

   /* open loop for each band to be processed */ 
   for (band=0; band<hbands; band++)
   {
      /* allocate memory for tlut, memory will be freed in apply_xform() */ 
      if ((tlut[band] = (long *) malloc (sizeof(long) * HBINS) ) == NULL)
      {
         clean2 ("Unable to allocate memory for lut."); 
      }
      
      /* determine correction factor to histogram transform */ 
      /* prevent divide by zero error.
         may occur when all image pixels have been masked.  in this rare
         case, many of the calculations here are irrelevant. */ 
      if (!chist[band][HBINS-1]) 
          chist[band][HBINS-1] = 1; 

      /* find and save the location of the first non-zero histogram bin */ 
      for (pixval=0; pixval<HBINS; pixval++)
      {
         if (chist[band][pixval])  
            break; 
      }
      firstbin[band] = pixval; 
      
      /* set correction factor. 
         cfactor = -.5 * first scaled cumulative histogram bin with > 0 
         brightness values + .5 for round off */
      cfactor[band] = 
            (-.5 * MAXBV * chist[band][pixval]) / chist[band][HBINS-1] + .5; 
            
      /* convert cumulative histogram to tlut.  
         tlut = scaled cumulative histogram + correction factor */ 
      for (pixval=0; pixval<HBINS; pixval++)
      {
         tlut[band][pixval] = (long) ( MAXBV * (float)chist[band][pixval] / 
                                chist[band][HBINS-1]  + cfactor[band]);
      }
      
      /* modify tlut to implement parm maxshift.
         start modification at the start of histogram data */ 
      /* basic calculation */ 
      for (curshift=0, pixval=firstbin[band]+1; pixval<HBINS; pixval++)
      {
         tlut[band][pixval] -= curshift; 
         diff = tlut[band][pixval] - tlut[band][pixval-1] ; 
         if (diff > maxshift) 
         {
            curshift += diff - maxshift; 
            tlut[band][pixval] = tlut[band][pixval-1] + maxshift; 
         }
      }

      /* center histogram in middle of its range
         using curshift shifted entire histogram toward zero. adjust histogram
         .5 * curshift to balance histogram toward middle of its range */ 
      for (curshift/=2, pixval=0; pixval<HBINS; pixval++)
         tlut[band][pixval] += curshift; 
         
      /* correct negative values.  previous calculation of tlut included use 
         of a correction factor (cfactor) which may give tlut negative values.
         This is irrelevant if histimg=-- because where tlut is negative, there
         are no pixels to adjust.  If histimg has a value, negative values of 
         tlut would cause errors in mapping inimg*/
      for (pixval=0; tlut[band][pixval] < 0;  pixval++)
         tlut[band][pixval] = 0; 

      /* implement parms frommask and tomask, must be last modification to 
         tlut */ 
      for (maski=0; maski<tomask->count; maski++)
         tlut[band][ frommask->value[maski] ] = tomask->value[maski ]; 
   
      /* free memory allocated in make_hist() */ 
      free (chist[band]); 
   }
}


/* FUNCTION:  apply transform, creating output image */ 
void apply_xform
(
    long msgtim,                /* input: $msgtim */
    long *tlut[MAXBND],         /* input: histogram transforming lut */
    struct lasinimage *inimg,   /* input: image to modify */
    struct lasoutimage *outimg /* input: info on output image */
) 
{
   /* declare vars */ 
   unsigned char *inbuf,       /* location of image line in inimg->ibuffer */
                 *outbuf;      /* location of image line in outimg->ibuffer */
   long band,                  /* for loop index: image band */ 
        line,                  /* for loop index: image line */ 
        messform=ALLCAT,       /* output message format */ 
        sample;                /* for loop index: image sample */ 
        
   /* initialize and start processing line message */ 
   c_inittm ("REDIST2", "Output", "line", "bands", "", 
             &outimg->nl, &outimg->totalbands, &msgtim, &messform); 
   curent = 1; 
   c_startm (); 
   
   /* create image */ 
   for (line=0; line<outimg->nl; line++, curent++)
   {
      for (band=0; band<outimg->totalbands; band++)
      {
         inbuf  = inimg->ibuffer  + inimg->bufoff[band]; 
         outbuf = outimg->ibuffer + outimg->bufoff[band]; 
         for (sample=0; sample<outimg->ns; sample++)
            *(outbuf + sample) = tlut[band][ *(inbuf + sample) ]; 
   
         /* update minmax data */ 
         minmax (band, outimg); 
      }
      /* step through images, reading and writing a line of data */ 
      if (c_estep (&inimg->groupd) || c_estep (&outimg->groupd) )
         clean2 ("Error stepping through images.");
   }
   c_stoptm(); 
  
   /* free memory allocated in make_form() */ 
   for (band=0; band<inimg->totalbands; band++)
      free (tlut[band]); 
}




#ifdef DEBUG 
/* DEBUGGING FUNCTION: dump one band of a lut or a histogram to disk */ 
void debug_dump
(
    char *idstring,      /* input:  informational string to dump to disk */
    long *dlut          /* input:  data to dump */
)
{
   /* declare variables */
   long index;          /* for loop index */ 
   
   /* dump idstring */ 
   fprintf (filep, "%s\n", idstring); 
   
   /* dump dlut */ 
   fprintf (filep, "%3d %3d\n", 0, dlut[0]); 
   for (index=1; index<HBINS; index++)
      if (dlut[index] != dlut[index-1] ) 
         fprintf (filep, "%3d %3d\n", index, dlut[index]); 
}
#endif 


int main(void)
{          
   /* declare variables */ 
   double bweight;            /* band-specific weight for histogram transform */
   long *chist[MAXBND],       /* cumulative histogram for entire image */ 
        fatal = -1,           /* fatal error, parm to c_errmsg() */ 
        go_on = 1,            /* warning, parm to c_errmsg() */ 
        maski,                /* for loop index, mask value */ 
        maxshift,             /* max shift of a BV during histogram
                                 transformation */ 
        minmax,               /* $minmax as a flag, = TRUE or FALSE */
        msgtim,               /* tae global $msgtim */ 
        *tlut[MAXBND];        /* histogram transforming lut (look-up-table) */
   struct iparm frommask,     /* BV to mask out of histogram */ 
                tomask;       /* output BV for masked pixels */ 
   struct VARIABLE *in;       /* pointer to var within parameter block */
   
   /* open debugging file */ 
#ifdef DEBUG 
   if ((filep = fopen ("debug", "w")) == NULL)
       c_errmsg ("unable to open dump file", "fail", &fatal); 
#endif 
   /* get parameters and image names from tae. begin_las() must be called 
      first */
   begin_las ("redist2");     /* initialize las and tae */ 
   
   /* get frommask and tomask parms */ 
   integer_parm ("FROMMASK", &frommask); 
   integer_parm ("TOMASK", &tomask); 

   /* check # of values */ 
   if (tomask.count && tomask.count != frommask.count)
   {
       c_errmsg ("Parm TOMASK was not defaulted but has a", "redist2-fail",
                 &go_on); 
       c_errmsg ("different number of values than parm FROMMASK.", 
                 "redist2-fail", &fatal); 
   }
   if (tomask.count != frommask.count)
   {
      tomask.count = frommask.count; 
      for (maski=0; maski<tomask.count; maski++)
          tomask.value[maski] = frommask.value[maski]; 
   }

   /* bweight parm */ 
   in = p_find (&pblock, "BWEIGHT");    
   bweight = (double) RVAL(*in,0);  
   
   /* maxshift parm */ 
   in = p_find (&pblock, "MAXSHIFT");    
   maxshift = IVAL(*in, 0); 

   /* $msgtim tae global */ 
   in = p_find (&pblock, "$MSGTIM");    
   msgtim = IVAL(*in, 0); 
   
   /* $minmax tae global */ 
   in = p_find (&pblock, "$MINMAX");    
   if (*SVAL(*in,0)=='Y')
      minmax = TRUE;
   else 
      minmax = FALSE;

   /* see if histimg parm has a value */ 
   in = p_find (&pblock, "HISTIMG");    
   histimg.imagevalid = (*in).v_count; 
   inimg.imagevalid = 1; 

   /* parse all image names */
   getin2 ("IN", &inimg);  
   if (histimg.imagevalid)
      getin2 ("HISTIMG", &histimg); 
   getout2 ("OUT", &outimg); 

   /* open input images */ 
   if (histimg.imagevalid) 
      opensin (&histimg); 
   opensin (&inimg); 

   /* open output image after checking input images (window size and data 
      type) */
   if (histimg.imagevalid && inimg.totalbands != histimg.totalbands)
   {
      c_errmsg ("Parm HISTIMG was not defaulted and input INIMG and ", 
                "redist2-fail", &go_on); 
      c_errmsg ("HISTIMG images do not have the same number of bands.",  
                "redist2-fail", &fatal); 
   }
   
   if ((histimg.imagevalid && histimg.datatype != EBYTE ) || 
       inimg.datatype!= EBYTE)
   {
      c_errmsg ("Input images are not byte.", "redist2-fail", &fatal); 
   }
   opensout (&inimg, minmax, "BYTE", &outimg); 
   
   /* create image */ 
   /* histogram image */ 
   make_hist (msgtim, bweight, &frommask, 
              (histimg.imagevalid ? &histimg : &inimg), chist);
   /* generate transform */
   make_xform(chist, inimg.totalbands, maxshift, &frommask, &tomask, tlut);
   
   /* optionally close and re-open inimg, before re-processing inimg */ 
   if (!histimg.imagevalid)
   {
      if (c_egclse(&inimg.groupd) )   /* close image file */ 
      {
         c_errmsg ("Error closing input image files.", "redist2-warn", &go_on); 
      }
      free (inimg.ibuffer);           /* free buffer */ 
      opensin (&inimg);               /* re-open image */ 
   }

   /* apply transform, creating output image */ 
   apply_xform (msgtim, tlut, &inimg, &outimg); 

   /* close image files and end program */ 
   end_las (); 
}
