#include "scale.h"
#include "util.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	FILSPEC

PURPOSE: Open the input and output image files.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        26-Sep-84   S. Ross
     2        31-Mar-91   K. Donaldson	     Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open the input image file for random access.
   Open the output image files for random access.
   Update the ddr records.

*****************************************************************************
*/
void filspec
(
    char hosin[][CMLEN],			/* Input image host name */
    char hosout[],			/* Output image host name */
    long bands_per_img[],		/* Number of bands for each image */
    long num_img_bands[][MAXBND+1],	/* Band numbers for each input image  */
    long num_inimages,			/* Total number of input images */
    long out_bands,			/* Number of output image bands */
    long out_img_bands[][MAXBND+1],	/* Band numbers for each output image */
    long outtype			/* Data type of the output image */
)
{

long    zero = 0;
long    one = 1;
char    *prog_name = "SCALE";   /* program name           */


long	access,				/* Access type */
	comb_flag,			/* Combine flag for 1-band output img */
	i, n,				/* Loop counter and array index */
	opt[2],				/* Data type conversion option */
	status,				/* Return status */
	type;				/* Data type */


double	line_inc;			/* Line increment */
double	samp_inc;			/* Sample increment */
/*
Open the input image files for random access.
---------------------------------------------
*/
access = IREAD;
type = 0;
for (n = 0; n < num_inimages; n++)
   {
   status = c_eopenr (&fdescin[n], hosin[n], &bands_per_img[n], &window[n][SL],
                      &window[n][SS], &window[n][NL], &window[n][NS], 
                      &access, &type, &zero, &one);
   if (status != E_SUCC)
      c_abort (&block, "Error opening the input image", "scale-inopen",
               prog_name, out_files, &num_outfiles);
   }
fdescin[++n] = 0;

/*
Open the output image files for random access.
-----------------------------------------------
*/
access = IWRITE;
type = ELONG;
opt[0] = outtype;
opt[1] = 0;
n = 0;
for (i = 0; i < out_bands; i++)
   out_img_bands[0][i] = i + 1;

status = c_eopenr (&fdescout[n], hosout, &out_bands, &one, &one, 
                   &window[0][NL], &window[0][NS], &access, &type, opt, &one);
if (status != E_SUCC)
   c_abort (&block, "Error opening the output image", "scale-outopen",
            prog_name, out_files, &num_outfiles);

fdescout[++n] = 0;

/*
Update the ddr records.
------------------------
*/
line_inc = 1.0;
samp_inc = 1.0;
comb_flag = NOCOMB;

c_upddr (&block, hosin, hosout, &num_inimages, window, num_img_bands,
         bands_per_img, &line_inc, &samp_inc, &out_bands, &comb_flag);
}
