#include <string.h>
#include "scale.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	PERCEN2

PURPOSE:  Calculates the clip value from a given percentage for a byte image

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1         03-May-84  J. LaVergne
     2         18-Jul-84  S. Ross            SCN#20-320
     3         31-Mar-91  K. Donaldson       Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Compute the total number pixels in one band.
   If clipping type for low end data is "PERCENT", 
      Compute the lowest pixel value at which to clip in order to achieve 
      the requested clip percentage.
   If clipping type is "VALUE", 
      Use the user specified low clip value.
   If clipping type for high end data is "PERCENT", 
      Compute the highest pixel value at which to clip in order to achieve 
      the requested clip percentage.
   If clipping type is "VALUE", 
      Use the user specified high clip value.

*****************************************************************************
*/
void percen2
(
    long histo[],		/* Histogram array */
    double zmin,			/* Minimum pixel value for the current band */
    double zmax,			/* Maximum pixel value for the current band */
    char cltyphi[],		/* Clipping type for high end data */
    char cltyplo[],		/* Clipping type for low end data */
    float *clipval_hi, 		/* Final (computed) high end clip value */
    float *clipval_lo,		/* Final (computed) low end clip value */
    float cliphi,			/* User-requested high end clip value */
    float cliplo			/* User-requested low end clip value */
)

{

long    seven = 7;

long	index, 			/* Index into histogram array */
	totpix;			/* Total number of pixels in the current band */

float	highpoint,		/* Number of pixels to be clipped for the */
				/* high end percentage */
	lowpoint;		/* Number of pixels to be clipped for the */
				/* low end percentage */

/*
Compute the total number pixels in one band.
-------------------------------------------
*/
totpix = window[0][NL] * window[0][NS]; 

/*
If clipping type for low end data is "PERCENT", compute the lowest pixel 
value at which to clip in order to achieve the requested clip percentage.
-------------------------------------------------------------------------
*/
if (strncmp (cltyplo, "PERCENT", seven) == 0)
   {
   lowpoint = (float) totpix * (cliplo / 100.0);
   *clipval_lo = 0.0;
   index = (long) zmin;

   while (*clipval_lo < lowpoint) 
      {
      *clipval_lo = *clipval_lo + (float) histo[index];
      index++;
      }
   *clipval_lo = (float) (index - 1);
   }

/* 
If clipping type is "VALUE", use the user specified low clip value.
-------------------------------------------------------------------
*/
else
   *clipval_lo = cliplo;

/*
If clipping type for high end data is "PERCENT", compute the highest pixel 
value at which to clip in order to achieve the requested clip percentage.
---------------------------------------------------------------------------
*/
if (strncmp (cltyphi, "PERCENT", seven) == 0)
   {
   highpoint = (float) (totpix) * (cliphi / 100.0);
   *clipval_hi = 0.0;
   index = (long) zmax;
   while (highpoint > *clipval_hi)
      {
      *clipval_hi = *clipval_hi + (float) histo[index];
      index--;
      }  
   *clipval_hi = (float) (index + 1);
   }

/* 
If clipping type is "VALUE", use the user specified high clip value.
--------------------------------------------------------------------
*/
else
   *clipval_hi = cliphi;

}
