#include <string.h>
#include "scale.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	PROLAS

PURPOSE:  Process the input images by clipping and scaling.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1         05-Mar-84  J. LaVergne
     2         18-Jul-84  S. Ross            SCN#20-320
     3         31-Mar-91  K. Donaldson       Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   For all lines in the current band, 
      Read a line of data into inbuf.
      For all pixels in the current line, 
         If clipping flag is set, 
            If the current input data is below the low clip limit, 
               Set the correspondig output image value to the user requested 
               minimum value.
            If the current input data is above the high clip limit, 
               Set the correspondig output image value to the user requested 
               maximum value.
            If the current input data is within the low and high clip limits, 
               Set the correspondig output image value to the scaled input 
               image value. 
         End-If
         If the clip flag is not set, 
            Set the output image values to the scaled input image values.
      End-For
   Write out the output image line.
   End-For

*****************************************************************************
*/
void prolas
(
    char clip_flg[4],	/* Clip flag */
    float clipval_hi, 	/* Final (computed) high end clip value */
    float clipval_lo,	/* Final (computed) low end clip value */
    float fmaxval,	/* Maximum pixel value for the output image */
    float fminval,	/* Minimum pixel value for the output image */
    float slope,		/* Scaling factor for calculting output image values */
    float offset,		/* Offset for calculating output image values*/
    long img,		/* Current input image */
    long inband,		/* Input image band being read */
    long outband,	/* Output image band being written */
    long dtype		/* Input image data type */
)
{

long    one = 1;
long    three = 3;


float	data,		/* Input image data */
	*rinbuf;	/* Pointer to float data type */

long	*linbuf,	/* Pointer to long data type */
	line,		/* Current line being read */
	outline,	/* Output image line */
	pix;		/* Current pixel being processed */

short	*sinbuf;	/* Pointer to short data type */

outline = 1;
rinbuf = (float *) inbuf;
linbuf = (long *) inbuf;
sinbuf = (short *) inbuf;

/*
For all lines in the current band, read a line of data into inbuf.
-----------------------------------------------------------------
*/
for (line = 1; line <= window[0][NL]; line++)
   {
   curent = line;
   c_eread (&fdescin[img], &inband, &line, inbuf, &one);

/*
For all pixels in the current line, if clipping flag is set, store appropriate
data in the output line buffer.
If the current input data is below the low clip limit, set the correspondig
output image value to the user requested minimum value.
If the current input data is above the high clip limit, set the correspondig
output image value to the user requested maximum value.
If the current input data is within the low and high clip limits, set the 
correspondig output image value to the scaled input image value.
------------------------------------------------------------------------------
*/
   for (pix = 0; pix < window[0][NS]; pix++)
      {
      if (dtype == EREAL)
         data = rinbuf[pix];
      else if (dtype == ELONG)
         data = (float) linbuf[pix];
      else if (dtype == EWORD)
         data = (float) sinbuf[pix];
      else
         data = (float) inbuf[pix];
      if (strncmp (clip_flg, "YES", three) == 0)
         {
         if (data <= clipval_lo) 
            outbuf[pix] = (long) fminval;
         else if (data >= clipval_hi) 
            outbuf[pix] = (long) fmaxval;
         else
            outbuf[pix] = (long) ((slope * data) + offset);
         }
/*
If the clip flag is not set, set the output image values to the scaled 
input image values.
----------------------------------------------------------------------
*/
      else
         outbuf[pix] = (long) ((slope * data) + offset);
      }  

/*
Write out the output image line.
--------------------------------
*/
   c_ewrite (&fdescout[0], &outband, &outline, outbuf, &one);
   outline++;
   }  

}
