#include <string.h>
#include "scale.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	SETT2

PURPOSE:  Genereates a histogram from a byte input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        05-Mar-84  J. Lavergne
     2        18-Jul-85  F. Irani            SCN#20-506
     3        31-Mar-91  K. Donaldson        Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Set the local minimum and maximum pixel values.
   For every pixel of the current line in the current band, 
      Update the histogram array.
   Update the image minimum and maximum pixel values.

*****************************************************************************
*/
void sett2
(
    long hist[],			/* Histogram array */
    double *zmin,			/* Minimum pixel value for the current band */
    double *zmax,			/* Maximum pixel value for the current band */
    char scaltyp[],		/* Type of scaling to be performed */
    long img,			/* Current image being processed */
    long inband			/* Current band being processed */
)
{

long    one = 1;
long    five = 5;

long	data,			/* Input data */
	line,			/* Current line being read */
	maxval, 		/* Maximum pixel value for the output image */
	minval,			/* Minimum pixel value for the output image */
	pix;			/* Current pixel being processed */

/* 
Set the local minimum and maximum pixel values.
-------------------------------------------------
*/
if (strncmp (scaltyp, "LOCAL", five) == 0)
   {
   minval = 255;
   maxval = 0;
   }
else
   {
   minval = (long) *zmin;
   maxval = (long) *zmax;
   }  

/*
For every pixel of the current line in the current band, update the 
histogram array.
--------------------------------------------------------------------
*/
for (line = 1; line <= window[0][NL]; line++)
   {
   c_eread (&fdescin[img], &inband, &line, inbuf, &one);
   for (pix = 0; pix < window[0][NS]; pix++)
      {
      data = inbuf[pix];
      minval = (long) min(minval, data);
      maxval = (long) max(maxval, data);
      hist[data] = hist[data] + 1;
      }  
   } 

/*
Update the image minimum and maximum pixel values.
--------------------------------------------------
*/
*zmin = (float) minval;
*zmax = (float) maxval;

}
