#include "scale.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	SETT4

PURPOSE:  Generates a histogram from an I*2, I*4, or R*4 input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         05-Mar-84   J. Lavergne
    2         19-Jul-84   S. Ross	     SCN#20-320
    3         31-Mar-91   K. Donaldson       Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   For every pixel of the current line in the current band, 
      Update the histogram array.

*****************************************************************************
*/
void sett4
(
    long hist[],			/* Histogram array */
    float hscale,			/* Histogram scaling factor */
    float hoffset,		/* Histogram offset */
    long img,			/* Current image being processed */
    long inband,			/* Current band being processed */
    long dtype			/* Input image data type */
)
{

long    one = 1;

float	data,			/* Input image data */
	*rinbuf;		/* Pointer to float data type */

long	index,			/* Histogram index */
	*linbuf,		/* Pointer to long data type */
	line,			/* Current line being read */
	pix;			/* Current pixel being processed */

short	*sinbuf;		/* Pointer to short data type */

/*
For every pixel of the current line in the current band, update the 
histogram array.
--------------------------------------------------------------------
*/
rinbuf = (float *) inbuf;
linbuf = (long *) inbuf;
sinbuf = (short *) inbuf;

for (line = 1; line <= window[0][NL]; line++)
   {
   c_eread (&fdescin[img], &inband, &line, inbuf, &one);
   for (pix = 0; pix < window[0][NS]; pix++)
      {
      if (dtype == EREAL)
         data = rinbuf[pix];
      else if (dtype == ELONG)
         data = (float) linbuf[pix];
      else 
         data = (float) sinbuf[pix];
      index = (long) (hscale * data + hoffset);
      hist[index] = hist[index] + 1;
      }  
   }  

}
