#include "stretch.h"
#include "protos.h"

void stadapt
(
    long lut_type		/* data type of look up table */
)
{
/*
----------------------------------------------------------------------------
NAME: STADAPT

FUNCTION:   Produces an adaptive power law look up table.

PROGRAM HISTORY:
      Version          Date           Author		Request
      -------          ----           ------		-------
        1      	     28-Jan-82      J. Hagedorn       
	2     	     24-Dec-86      R. Mckinney  	prf 3034         
        3	      7-Jul-89	    B. Allam            conversion
        4            22-Mar-91      K. Donaldson        added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   If the look-up table type is BYTE, 
      Set the value limits to 0 and 255.
   If the look-up table type is I*2, 
      Set the value limits to -32768 and 32767.
   Get the minimum and maximum requested lut entry values.
   Verify that the requested look-up table entry values are in the valid range.
   Allocate dynamic memory for lut.
   Create an adaptive power law look-up table.
---------------------------------------------------------------------------
*/

double  temp;			/* Intermediate result */

float	expon;			/* Exponent */

long	j,			/* Loop counter and array index */
	l_size, 		/* LUT size */
	lmax, 			/* Maximum requested LUT entry value */
	lmin,			/* Minimum requested LUT entry value */
	lutmax, 		/* Maximum LUT range value */
	lutmin;			/* Minimum LUT range value */

struct VARIABLE *var; /* Function/variable for parameter retrieval */

/*
If the look-up table type is BYTE, set the value limits to 0 and 255.
If the look-up table type is I*2, set the value limits to -32768 and 32767.
-----------------------------------------------------------------------------
*/
if (lut_type == 1) 
   {
   lutmax = 255;
   lutmin = 0;
   }
else
   {
   lutmax = 32767;
   lutmin = -32768;
   } 

/*
Get the minimum and maximum requested lut entry values.
--------------------------------------------------------
*/
var = p_find (&block, "MAXVAL");
lmax = IVAL (*var, 0);
var = p_find (&block, "MINVAL");
lmin = IVAL (*var, 0);

/*
Verify that the requested look-up table entry values are in the valid range.
----------------------------------------------------------------------------
*/
if (lmax > lutmax  ||  lmin < lutmin  ||  lmin >= lmax) 
   msg_err ("!Specified range is not valid", "stretch-range");

/*
Allocate dynamic memory for lut.
---------------------------------
*/
l_size = (long) ((lutmax-lutmin+1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table", 
            "stretch-lutalloc");
if (lut_type == 1)
  lut = &templut[0];
else
  lut = &templut[(lutmax-lutmin+1)/2];

/*
Create an adaptive power law look-up table.
--------------------------------------------
*/
if (lmin == lutmin) 
   lmin = lutmin+1;

expon = log ((float)(lutmax-lutmin)) / log((float)(lmax-lutmin)/
                                           (float)(lmin-lutmin)); 
for (j = lutmin; j <= lutmax; j++)
   {
   if (j < lmin) 
      lut[j] = (short) lutmin;
   else if (j > lmax) 
      lut[j] = (short) lutmax;
   else
      {
      temp = pow (((float)(j-lutmin) / (float) (lmin-lutmin)), expon);
      lut[j] = (short) (lutmin + (int)(temp + 0.5));
      }  
   }

}
