#include "stretch.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void stapplylut
(
    char hosnam[][CMLEN],	/* Input image host name */
    char hosout[][CMLEN],		/* Output image host name */
    long totbnd,			/* Total number of bands */
    long window[][4],		/* Processing window */
    long nimages,		/* Number of images */
    long otype 			/* Output image data type */
)

/*
---------------------------------------------------------------------------
NAME: STAPPLYLUT

FUNCTION:   Applies a look up table to an image to create a new image.

PROGRAM HISTORY:
      Version          Date            Author		Request
      -------          ----            ------		-------
        1      	     28-Jan-82       J. Hagedorn       
	2     	      5-Dec-88       K. Narayanan	handle dpf
        3	      7-Jul-89	     B. Allam		conversion
        4            22-Mar-91       K. Donaldson       added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Get the processing message interval and the min-max flag.
   Initialize the processing message utility.
   Call STSETIO to open the input and output images.
   Pass each line of the input image through the look-up table.
   Compute the minimum and maximum pixel values for each band.
   Update history file.

-----------------------------------------------------------------------------
*/
{

long    one = 1;

char	mnmx[4];			/* Min-max option */

double	amax[MAXBND],			/* Maximum input image pixel value */
	amin[MAXBND];			/* Minimum input image pixel value */

long	band, 				/* Current band index */
	line, 				/* Current line */
	ltyp, 				/* LUT data type */
	msgtim,				/* Processing message interval */
	msg_flag,			/* Processing message flag */
	nl,				/* Number of lines */
	np, 				/* Number of pixels */
	out_bands[2],			/* Zero terminated array with the */
  					/* number of bands for the output img*/
	out_img_band[1][MAXBND + 1],	/* Band numbers of the output image  */
	outwin[1][4];			/* Output window specifications              */

struct VARIABLE *var; /* Function/variable for parameter retrieval */

np = window[0][NS];
nl = window[0][NL];

/*
Get the processing message interval and the min-max flag.
---------------------------------------------------------
*/
var = p_find(&block, "$MSGTIM"); 
msgtim = IVAL(*var,0);
var = p_find(&block, "$MINMAX"); 
strcpy(mnmx, SVAL(*var, 0));

/*
Initialize the processing message utility.
------------------------------------------
*/
msg_flag = ONECAT;
c_inittm ("STRETCH", "Processed", "line", "band", "", &nl, &totbnd, &msgtim, 
          &msg_flag);

curent = 1;
curcat = 1;
c_startm(); 

/*
Call STSETIO to open the input and output images.
--------------------------------------------------
*/
stsetio (hosnam, hosout[0], window, &otype, nimages, totbnd, out_img_band);
out_bands[0] = totbnd;
out_bands[1] = 0;
ltyp = EWORD;

/*
Pass each line of the input image through the look-up table.
-------------------------------------------------------------
*/
for (line = 1; line <= nl; line++)
   {
   c_estep (&gdescin);
   c_estep (&gdescout);
   curcat = 1;
   for (band = 0; band < totbnd; band++)
      {
      c_pxlkup (&ibuf[ipoint[band]], &obuf[opoint[band]], &np, &ltyp, 
                lut, &ltyp);
      curcat++;
      }
   curent++;
   }
c_estep (&gdescout);
c_stoptm(); 
c_egclse (&gdescin);
c_egclse (&gdescout);


/*
Compute the minimum and maximum pixel values for each band.
------------------------------------------------------------
*/
if ((mnmx[0] == 'Y') || (mnmx[0] == 'y'))
   {
   outwin[0][SL] = outwin[0][SS] = 1;
   outwin[0][NL] = window[0][NL];
   outwin[0][NS] = window[0][NS];
   c_minmax (hosout, &nimages, outwin, out_img_band, out_bands, amin, amax);
   c_svmnmx (hosout[0], out_bands, amin, amax, mnmx);
   }

free (ibuf); 
free (obuf); 
/*
Update history file.
---------------------
*/
c_hstout (&block, hosout[0], hosnam, &one, " ");

}
