#include "stretch.h"
#include "protos.h"

void stclip
(
    long lut_type		/* data type of look up table */
)
{
/*
--------------------------------------------------------------------------
NAME: STCLIP

FUNCTION:   Produces a bit clipping look up table; i.e. it produces a 
            look-up table each entry of which is the position number 
            with a specified number of bits clipped off by shifting the bits.

PROGRAM HISTORY:
      Version          Date            Author		Request
      -------          ----            ------		-------
        1      	     28-Jan-82        J. Hagedorn       
	2     	     24-Dec-86        R. Mckinney	prf 3034         
        3	      8-Jul-89	      B. Allam		conversion
        4            22-Mar-91        K. Donaldson      added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Get the number of bits to clip.
   If the look-up table type is BYTE, 
      Set the value limits to 0 and 255 and 
      Validate the number of bits to clip.
   End-If
   If the look-up table type is I*2, 
      Set the value limits to -32768 and 32767.
   Allocate dynamic memory for lut.
   Shift each lut entry by the specified number of bits.  
   If the clip number is positive then bits are shifted to the left.
   If the number is negative then bits are shifted right.
   Truncate high-order bits for unsigned byte shift.
----------------------------------------------------------------------------
*/

short	i2nbit, 		/* Temporary variable for the number of bits */
	lutindex;		/* Loop counter and array index */

char	msg[ERRLEN];		/* Error message buffer */

long	l_size, 		/* LUT size */
	lutmax,			/* Maximum requested LUT entry value */
	lutmin, 		/* Minimum requested LUT entry value */
	nbit, 			/* Number of bits to clip */
	nbit_abs;		/* Absolute number of bits to clip */

struct VARIABLE  *var; /* Function/variable for parameter retrieval*/

/*
Get the number of bits to clip.
--------------------------------
*/
var = p_find (&block, "NBITS");
nbit = IVAL (*var, 0);

/*
If the look-up table type is BYTE, set the value limits to 0 and 255
and validate the number of bits to clip.
------------------------------------------------------------------------
*/
if (lut_type == 1) 
   {
   nbit_abs = abs(nbit);
   if (nbit_abs >= 7) 
      {
      sprintf (msg, "!Invalid number of bits for clip: %2d", nbit_abs);
      msg_err (msg, "stretch-nbit");
      } 
   lutmin = 0;
   lutmax = 255;
   }

/*
If the look-up table type is I*2, set the value limits to -32768 and 32767.
---------------------------------------------------------------------------
*/
else
   {
   lutmax = 32767;
   lutmin = -32768;
   }  

/*
Allocate dynamic memory for lut
-------------------------------
*/
l_size = (long) ((lutmax-lutmin+1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table", 
            "stretch-lutalloc");
if (lut_type == 1)
   lut = &templut[0];
else
   lut = &templut[(lutmax-lutmin+1)/2];

/*
Shift each lut entry by the specified number of bits.  If the clip number
is positive then bits are shifted to the left.  If the number is negative
then bits are shifted right.
---------------------------------------------------------------------------
*/
i2nbit = (short) nbit;
for (lutindex = (short) lutmin; lutindex <= lutmax; lutindex++)
   {
   lut[lutindex] = (lutindex << i2nbit);

/*
Truncate high-order bits for unsigned byte shift.
--------------------------------------------------
*/
   if (lut_type == 1) 
      lut[lutindex] = (lut[lutindex] % 256);
   }

}
