#include "stretch.h"
#include "protos.h"

void stcontour
(
    long lut_type
)
{
/*
-----------------------------------------------------------------------------
NAME: STCONTOUR

FUNCTION:   Produces a contour look up table.

PROGRAM HISTORY:
      Version          date                Author		Request
      -------          ----                ------		-------
        1      	     28-Jan-82   	  J. Hagedorn       
	2     	     24-Dec-86   	  R. Mckinney	        prf 3034
        3	      7-Jul-89	 	  B. Allam              conversion
        4            22-Mar-91		  K. Donaldson          added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   If the look-up table type is BYTE, 
      Set the value limits to 0 and 255.
   If the look-up table type is I*2, 
      Set the value limits to -32768 and 32767.
   Get the contour interval.
   If the user-specified interval exceeds the maximum look-up table value,
      Print an error message.
   Allocate dynamic memory for the look-up table.
   Create contour stretch look-up table.
-----------------------------------------------------------------------------
*/

char	msg[ERRLEN];		/* Error message buffer */

long	i,			/* Loop counter and array index */
        interval, 		/* Contour interval */
	l_size, 		/* LUT size */
	lutmax, 		/* Maximum LUT value */
	lutmin;			/* Minimum LUT value*/

struct VARIABLE *var; /* Function/variable for parameter retrieval */

/*
If the look-up table type is BYTE, set the value limits to 0 and 255.
---------------------------------------------------------------------
*/
if (lut_type == 1) 
   {
   lutmax = 255;
   lutmin = 0;
   }
/*
If the look-up table type is I*2, set the value limits to -32768 and 32767.
----------------------------------------------------------------------------
*/
else
   {
   lutmax = 32767;
   lutmin = -32768;
   } 

/*
Get the contour interval.
--------------------------
*/
var = p_find (&block, "CONTINT");
interval = IVAL(*var, 0);

/*
If the user-specified interval exceeds the maximum look-up table value,
print an error message.
-----------------------------------------------------------------------
*/
if (interval > lutmax) 
   {
   sprintf (msg, "!Invalid interval value: %5d", interval);
   msg_err (msg, "stretch-inter");
   } 

/*
Allocate dynamic memory for the look-up table.
-----------------------------------------------
*/
l_size = (long) ((lutmax - lutmin + 1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table", 
            "stretch-lutalloc");
if (lut_type == 1)
  lut = &templut[0];
else
  lut = &templut[(lutmax-lutmin+1)/2];


/*
Create contour stretch look-up table.
--------------------------------------
*/
for (i = lutmin; i <= lutmax; i++)
   {
   if ((i/interval)*interval == i) 
      {
      lut[i] = 1;
      }
   else
      {
      lut[i] = 0;
      }  
   }

}
