#include "stretch.h"
#include "protos.h"

void stcuberoot
(
    long lut_type		/* data type of LUT */
)
{
/*
-----------------------------------------------------------------------------
NAME: STCUBEROOT

FUNCTION:   Produces a cuberoot stretch look up table.

PROGRAM HISTORY:
      Version          Date            Author		Request
      -------          ----            ------		-------
        1      	     28-Jan-82        J. Hagedorn       
	2     	     24-Dec-86        R. Mckinney	prf 3034         
        3	      7-Jul-89        B. Allam          conversion
        4            22-Mar-91        K. Donaldson      added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   If the look-up table type is BYTE, 
      Set the value limits to 0 and 255.
   If the look-up table type is I*2, 
      Set the value limits to -32768 and 32767.
   Allocate dynamic memory for lut.
   Create the cube root stretch look-up table.
---------------------------------------------------------------------------
*/
double  base;			/* Base for root function */

long	j,			/* Loop counter and array index */
	l_size, 		/* LUT size */
	lutmax, 		/* Maximum LUT range value */
	lutmin;			/* Minimum LUT range value */

short   temp;			/* Temporary value */

/*
If the look-up table type is BYTE, set the value limits to 0 and 255.
If the look-up table type is I*2, set the value limits to -32768 and 32767.
---------------------------------------------------------------------------
*/
if (lut_type == 1) 
   {
   lutmax = 255;
   lutmin = 0;
   }
else
   {
   lutmax = 32767;
   lutmin = -32768;
   }

/*
Allocate dynamic memory for lut.
--------------------------------
*/
l_size = (long) ((lutmax-lutmin+1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table", 
            "stretch-lutalloc");
if (lut_type == 1)
  lut = &templut[0];
else
  lut = &templut[(lutmax-lutmin+1)/2];

/*
Create the cube root stretch look-up table.
--------------------------------------------
*/
for (j = lutmin; j <= lutmax; j++)
   {
   base = (float)(j-lutmin) / (float)(lutmax - lutmin);
   temp = (short) (lutmin + (int)((lutmax-lutmin) * pow (base, 0.333333333) + 
                   0.5));
   lut[j] = temp;
   }

}
