#include "stretch.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void stdisklut
(
    long lut_type		/* Data type of look up table */
)
{
/*
---------------------------------------------------------------------------
NAME: STDISKLUT

FUNCTION:   Gets a look up table from a disk display parameter file.

PROGRAM HISTORY:
      Version          Date           Author		Request
      -------          ----           ------		-------
        1      	     28-Jan-82      J. Hagedorn       
	2     	      2-Jun-88	    K. Narayanan	handle dltf
        3	     29-May-89	    B. Allam		conversion
        4            31-Dec-90      W. Xia		fix call to r2dltf
        5            22-Mar-91      K. Donaldson        added comments; passed
         						correct parameters to
                                                        read DLTF routines;
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   If the look-up table type is BYTE, 
      Set the value limits to 0 and 255.
   If the look-up table type is I*2, 
      Set the value limits to -32768 and 32767.
   Allocate dynamic memory for the look-up table.
   Get the DLTF file name and entry name.
   Read in the look-up table.
   Convert lut to integer*2 for further processing
-----------------------------------------------------------------------------
*/ 

long    zero = 0;
long    one = 1;


unsigned char *buffer;		/* Pointer to the look-up table read-in array*/

char	desc[81],		/* Description of DLTF entry */
	dltf_type[3],		/* DLTF data type */
	dltfentry[13], 		/* DLTF entry */
	host_dltf[CMLEN], 	/* DLTF host file name */
	msg[ERRLEN];		/* Error message buffer */

double  offset, 		/* LUT offset */
        slope;			/* LUT slope */

long	access,			/* File access type */
	endpt,			/* LUT range ending point */
	fd_dltf, 		/* File descriptor for DLTF file */
	ftyp, 			/* "FROM" type for conversion */
	i, j,			/* Loop counter and array index */
	l_size, 		/* LUT size */
	lutmax, 		/* Maximum LUT range value */
	lutmin,			/* Minimum LUT range value */
	size,			/* Look-up table size */
        startpt, 		/* LUT range starting point */
	status, 		/* Return status */
	dltf_len,		/* Length of the data portion of the DLTF */
	totyp;			/* "TO" type for conversion */

short	*shortlut;		/* Pointer to a buffer element */

struct VARIABLE *var; /* Function/variable for parameter retrival */

/*
If the look-up table type is BYTE, set the value limits to 0 and 255.
If the look-up table type is I*2, set the value limits to -32768 and 32767.
---------------------------------------------------------------------------
*/
if (lut_type == 1) 
   {
   lutmax = 255;
   lutmin = 0;
   }
else
   {
   lutmax = 32767;
   lutmin = -32768;
   }  

/*
Allocate dynamic memory for the look-up table.
-----------------------------------------------
*/
l_size = (long) ((lutmax-lutmin+1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table",
            "stretch-lutalloc");
if (lut_type == 1)
  {
  lut = &templut[0];
  l_size = lutmax - lutmin + 1 + 2;
  }
else
  {
  lut = &templut[(lutmax-lutmin+1)/2];
  l_size = (lutmax - lutmin + 1 + 2) * 2;
  }

l_size = (long) (l_size * sizeof(unsigned char));
if ((buffer = (unsigned char *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table",
            "stretch-lutalloc");

/*       
Get the DLTF file name and entry name.
---------------------------------------
*/
access = IREAD;
c_getnam (&block, "DLTFIN", &zero, host_dltf, "dltf", &access);
if (strlen (host_dltf) == 0)
   msg_err ("!Input DLTF must be specified for this subcommand", 
            "stretch-nolut");
else if (strlen (host_dltf) != 0)
   {
   var = p_find (&block, "INDLTEN");
   if ((*var).v_count == 0)
      msg_err ("!No DLTF entry specified", "stretch-dltfent");
   else
     strcpy(dltfentry, SVAL(*var, 0));
   }

/*                                         
Read in the look-up table.
---------------------------
*/
status = c_r1dltf (&fd_dltf, host_dltf, dltfentry, desc, &asdltf, &dltf_len, 
                   dltf_type);
if (status == E_SUCC) 
   {
   if (asdltf.numbnd != 1)  
      {  
      sprintf( msg, "!Color input look up table may not be specified");
      msg_err (msg, "stretch-color");
      }  
   if (lut_type == 2) 
      msg_err ("!Data type of input image does not match data type of input LUT", 
               "stretch-intypes");
   }
else
   msg_err ("!Error reading DLTF", "stretch-lread");

startpt = 0;
endpt = 0;
status = c_r2dltf (&fd_dltf, dltfentry, &one, &startpt, &endpt, &dltf_len, 
                   buffer, &slope,  &offset);
if (status != E_SUCC)
   msg_err ("!Error reading DLTF", "stretch-lread");

/*
Convert lut to integer*2 for further processing.
------------------------------------------------
*/
if (!(strcmp(dltf_type, "B"))) 
   {
   ftyp = EBYTE;
   totyp = EWORD;
   size = 256;
   c_pxconv( &ftyp, &totyp, buffer, &lut[0], &size);
   }
else
   {
   j = 0;
   for (i = 0; i < 65536; i++)
      {
      shortlut = (short *)&buffer[i*2];
      templut[j] = (short)*shortlut;
      j++;
      }
   }
}
