#include "stretch.h"
#include "pixman.h"
#include "protos.h"

void stputlut
(
    char dltfentry[],		/* DLTF entry name */
    long window[][4]		/* Processing window */
)
{
/*
----------------------------------------------------------------------------
NAME: STPUTLUT

FUNCTION:   Writes an unsigned byte look up table to disk.

PROGRAM HISTORY:
      Version          Date 	     Author	     Request
      -------          ----          ------          -------
        1      	     2-Jun-88  	   K. Narayanan     handle dpf
        2	     7-Jul-89	   B. Allam         conversion
        3           31-Dec-90      W. Xia           fix call to w2dltf
        4           22-Mar-90      K. Donaldson

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Convert lut to unsigned byte.
   Set the DLTF description field.
   Set the image and graphics planes shift and zoom factors.
   Set the DLTF window dimensions.
   Set the number of bands and the individual band numbers.
   Set the DLTF data type.
   Set the look-up table starting and ending points.
   Write the 1st DLTF record.
   Write the 2nd DLTF record.
----------------------------------------------------------------------------
*/           

long    one = 1;

unsigned char *outlut;		/* Byte LUT */

char    desc[81],		/* Description of DLTF entry */
	dltf_type[3];		/* DLTF data type */

double  offset = 0.0, 		/* Look-up table offset */
        slope = 1.0;    	/* Look-up table slope */

long endpt,			/* LUT range ending point */
	ftyp, 			/* "FROM" type for conversion */
	totyp, 			/* "TO" type for conversion */
	size,			/* Look-up table size */
        startpt, 		/* LUT range starting point */
	status,			/* Return status */
	tsize;

tsize = (256 + 2) * sizeof (unsigned char);
if ((outlut = (unsigned char *) malloc(tsize)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table", 
            "stretch-lutalloc");
                                            
/*               
Convert lut to unsigned byte.
-----------------------------
*/
ftyp = EWORD;
totyp = EBYTE;
size = 256;
c_pxconv (&ftyp, &totyp, lut, outlut, &size);

/*
Set the DLTF description field.
-------------------------------
*/
sprintf (desc, "LUT created by LAS STRETCH");

/*
Set the image and graphics planes shift and zoom factors.
---------------------------------------------------------
*/
asdltf.izoom[0] = 1;  
asdltf.izoom[1] = 1;
asdltf.ishift[0] = 0; 
asdltf.ishift[1] = 0;

/*
Set the DLTF window dimensions.
--------------------------------
*/
asdltf.window[0] = window[0][SL];
asdltf.window[1] = window[0][SS];
asdltf.window[2] = window[0][NL];
asdltf.window[3] = window[0][NS];

/*
Set the number of bands and the individual band numbers.
---------------------------------------------------------
*/
asdltf.numbnd = 1;
asdltf.band[0] = 1;

/*
Set the DLTF data type.
-----------------------
*/
strcpy(dltf_type, "B");

/*
Set the look-up table starting and ending points.
-------------------------------------------------
*/
startpt = 0;
endpt = 255;

/*
Write the 1st DLTF record.
--------------------------
*/
status = c_w1dltf(&fd, dltfentry, desc, dltf_type, &asdltf);
if (status != E_SUCC)
  msg_err ("!Error writing to DLTF", "stretch-wdltf");

/*
Write the 2nd DLTF record.
--------------------------
*/
status = c_w2dltf(&fd, dltf_type, dltfentry, &slope, &offset, &startpt, &endpt,
                 &one, outlut);
if (status != E_SUCC)
  msg_err ("!Error writing to DLTF", "stretch-wdltf");

/* free (outlut); */
}
