
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "statio.h"
#include "dltf.h"
#include "protos.h"

struct PARBLK block;                   /* TAE parameter block     */
struct DDR  ddr;
struct ASDLTF asdltf;
long    fd;
struct FDESC *fdescin[MAXIMG+1];             /* input file descriptors   */
struct FDESC *fdescout[2];                   /* output file descriptor   */
struct GDESC *gdescin;                       /* input group descriptor   */
struct GDESC *gdescout;                      /* output group descriptor  */
long    ipoint[MAXIMG * MAXBND + 1],   /* offsets into image i/o  */
                opoint[MAXBND+1];              /* for input and output bands */
short   *templut, *lut;


long    count;                         /* no. of files opened */
long    bands[MAXIMG][MAXBND+1];
long    nbands[MAXIMG];
static  char    *prog_name = "STRETCH"; /* program name           */
char    out[MAXIMG][CMLEN];             /* names of files opened */
short   *ibuf;                        /* image input buffer      */
short   *obuf;                         /* image output buffer     */

extern long     curcat;                        /* message utility         */
extern long     curent;                        /* parameters for processing */


/*
----------------------------------------------------------------------------
NAME: STRETCH

FUNCTION:  The program stretch produces a look up table according to user 
           specification, optionally writes it to disk and applies the table
           to an image to produce another image.
 
PROGRAM HISTORY:
      Version          Date                Author	Request
      -------          ----                ------	-------
        1      	     28-Jan-82   	  J. Hagedorn       
	2     	      5-Dec-88   	  K. Narayanan	handle dpf
        3	     24-May-90	 	  B. Allam	conversion
        4            23-Mar-91		  K. Donaldson  
        5	     24-Sep-92		  T. Mittan	Modified include file
							to eliminate warnings
							on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Initialize TAE.
   Call STMAKELUT to create lut and get its data type.
   If output file name was specified, 
      Write the look-up table to disk.
      Open DLTF for output.
      If LUT data type is BYTE, 
         Call STPUTLUT to write out the look-up table.
      If LUT data type is not BYTE,
         Print an error message.
   End-If
   Call STAPPLYUT to apply the lut to the input image.
   Free the dynamic memory.
   Write the history record.
   Print the completion message.
----------------------------------------------------------------------------
*/
int main
(void) /* stretch */
{
char    dltfent[13], 			/* Entry in input DPF */
	hosnam[MAXIMG][CMLEN],		/* Input image host name */
	hosout[1][CMLEN],			/* Output image host name */
	hsource[CMLEN],			/* Input Display Parameter File */
        msg[ERRLEN],			/* Error message buffer */
	source[CMLEN];			/* Input DLTF file */

long	access, 			/* File access type */
	status, 
	lut_type, 			/* Data type of look-up table */
        nimages,			/* Number of images */
	otype,				/* output image type */
	std_out, 			/* Logical unit for input and output*/
	totbnd,				/* Total number of bands */
	window[MAXIMG][4];		/* Processing window */


count = 0;

/*
Initialize TAE.
----------------
*/
c_inlas (&block, &std_out);

/*
Call STMAKELUT to create lut and get its data type.
---------------------------------------------------
*/
stmakelut (&totbnd, &nimages, hosnam, hosout, &lut_type, &otype, source, 
           dltfent, window);

/*
If output file name was specified, write the look-up table to disk.
--------------------------------------------------------------------
*/
if (strlen (source) != 0) 
   {

/*
Open DLTF for output.
----------------------
*/
   access= IWRITE;
   c_mknam (source, hsource,"dltf", &access);
   status = c_ckdltf (hsource, dltfent, &fd);
   if (status != E_SUCC)
     c_abort (&block, "Error opening DLTF", "stretch-dltf", prog_name, 
              out, &count);

/* 
If LUT data type is BYTE, call STPUTLUT to write out the look-up table.
-----------------------------------------------------------------------
*/
   if (lut_type == 1) 
      {
      stputlut (dltfent, window);
      sprintf(out[count++], "%s", hsource);
      c_cldltf (&fd, hsource);
      }

/*
If LUT data type is not BYTE print an error message.
----------------------------------------------------
*/
   else
      {
      sprintf(msg, "?Unable to write LUT to disk - LUT data type must be BYTE");
      msg_err (msg, "stretch-luttype"); 
      }  
   } 

/*
Call STAPPLYUT to apply the lut to the input image.
---------------------------------------------------
*/
if (nimages != 0)
  stapplylut (hosnam, hosout, totbnd, window, nimages, otype);

/*
Free the dynamic memory.
-------------------------
*/
free (templut);

/*
Write the history record.
-------------------------
*/

/*
Print the completion message.
-----------------------------
*/
c_complt ("STRETCH", out, &count);

return EXIT_SUCCESS;

}
