#include "stretch.h"
#include "util.h"
#include "protos.h"
 
void stsetio
(
    char hosnam[][CMLEN], 	/* Input image host file name */
    char hosout[],		/* Output image host file name */
    long wind[][4],		/* Processing window */
    long *otype,			/* Output image data type */
    long nimages,		/* Number of images */
    long totbnd, 		/* Total number of bands */
    long (*out_img_bands)[MAXBND+1]  /* Band numbers for the output image    */
)
{
/*
---------------------------------------------------------------------------
NAME: STSETIO

FUNCTION:   Open input and output images.
        
PROGRAM HISTORY:
      Version          Date            Author		Request
      -------          ----            ------		-------
        1      	     28-Jan-82        J. Hagedorn       
	2     	     14-Jul-87        B. Rieger
        3	      7-Jul-89	      B. Allam		conversion
        4            22-Mar-91        K. Donaldson      added comments; changed
 							changed iobuf to ibuf
							and obuf

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Open the input image.
   Get the data type of the input image.
   If the data type of the input image is not BYTE or I*2, 
      Print an error message.
   Allocate buffer space for the input image.
   Group the input image bands.
   Open the output image.
   Allocate buffer for the output image.
   Group the output image bands.
   Set up group containing input and output bands
   Update the DDR of the output image file.
----------------------------------------------------------------------------
*/

long    zero = 0;
long    one = 1;
char    *prog_name = "STRETCH"; /* program name           */

double  l_inc,			/* Line increment */
	s_inc;			/* Sample increment */

long    access,			/* File access type */
        bufsiz,			/* Image i/o buffer size */
        n, nib,			/* Counter variables */
	opt[2],			/* Conversion option parameter */
	status,			/* Retur status */
        type;			/* Data type */
                                     
/*
Open the input image.
----------------------
*/
access = IREAD;
type = EWORD;
for (n = 0, nib = 0; n < nimages; n++)
   {
   status = c_eopens(&fdescin[n], hosnam[n], bands[n], &wind[n][SL],
                     &wind[n][SS], &wind[n][NL], &wind[n][NS], &ipoint[nib],
                     &access, &type, &zero, &zero);
   if (status != E_SUCC)
      msg_err("!Error opening input image", "stretch-input");
   nib += nbands[n]; 
   }

for (n = 0; n < totbnd; n++)
   out_img_bands[0][n] = n+1;

/*
Allocate buffer space for the input image.
------------------------------------------
*/
if ((ibuf = (short *) c_ealloc(fdescin, &bufsiz)) == NULL)
   msg_err("!Error allocating dynamic memory", "scatter-alloc");

/*
Group the input image bands.
----------------------------
*/
if (c_egroup(fdescin, &gdescin, ibuf, &bufsiz) != E_SUCC)
   msg_err("!Error grouping image bands", "scatter-group");

/*
Open the output image.
----------------------
*/
access = IWRITE;
opt[0] = *otype;        
opt[1] = 0;
n = 0;
status = c_eopens(&fdescout[n], hosout, &totbnd, &one, &one, &wind[0][NL], 
                  &wind[0][NS], opoint, &access, &type, &zero, opt);
if (status != E_SUCC)
   c_abort (&block, "Error opening output image", "stretch-output",
            prog_name, out, &zero);
fdescout[++n] = 0;

/*
Allocate buffer for the output image.
-------------------------------------
*/
if ((obuf = (short *) c_ealloc(fdescout, &bufsiz)) == NULL)
   msg_err("!Error allocating dynamic memory", "scatter-alloc");

/*
Group the output image bands.
------------------------------
*/
if (c_egroup (fdescout, &gdescout, obuf, &bufsiz) != E_SUCC)
   msg_err("!Error grouping image bands", "scatter-group");

/*
Update the DDR of the output image file.
----------------------------------------
*/
l_inc = 1;
s_inc = 1;
c_upddr (&block, hosnam, hosout, &nimages, wind, bands, nbands, &l_inc,
         &s_inc, &totbnd, &zero);

}
