#include "stretch.h"
#include "protos.h"

void stthresh
(
    long maxt,		/* Maximum threshold value */
    long mint,		/* Minimum threshold value */
    long maxlut, 	/* Maximum LUT range value */
    long minlut 	/* Minimum LUT range value */
)
{
/*
---------------------------------------------------------------------------
NAME: STTHRESH

FUNCTION:   Constructs a threshold look up table, given the upper and lower 
            limits as input parameters.  The entries of the table are
            one between the threshold limits and zero elsewhere.

PROGRAM HISTORY:

      Version          Date           Author		Request
      -------          ----           ------		-------
        1      	     28-Jan-82      J. Hagedorn   
	2      	     24-Dec-86      R. Mckinney         prf 3034
	3	     10-Jul-89	    B. Allam		conversion
        4            22-Mar-91      K. Donaldson        added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Check validity of the threshold values.
   Create a threshold by value stretch look-up table.
----------------------------------------------------------------------------
*/
char	msg[ERRLEN];		/* Error message buffer */

long	i;			/* Loop counter and array index */

/*
Check validity of the threshold values.
----------------------------------------
*/
if (maxt > maxlut  ||  mint < minlut) 
   {
   sprintf(msg,  "?Specified threshold values are invalid for");
   sprintf(&msg[strlen(msg)], "data type of input image. ");
   msg_err (msg, "stretch_error"); 
   sprintf(msg, "!Both must be in range [%5d, %5d]",minlut, maxlut);
   msg_err (msg, "stretch_error"); 
   }  

/*
Create a threshold by value stretch look-up table.
---------------------------------------------------
*/
for (i = minlut; i <= maxlut; i++)
   {
   if (i < mint  ||  i > maxt) 
      lut[i] = 0;
   else
      lut[i] = 1;
   }

}
