#include "stretch.h"
#include "protos.h"

void stvthresh
(
    long lut_type		/* Data type of LUT */
)
{
/*
--------------------------------------------------------------------------
NAME: STVTHRESH

FUNCTION:   Produces a threshold look up table by values specified by the
            user.  The result is a look up table with entries of one at
            a specified range of positions, and zero elsewhere.

PROGRAM HISTORY:

      Version          Date           Author		Request
      -------          ----           ------		-------
        1      	     28-Jan-82      J. Hagedorn   
	2      	     24-Dec-86      R. Mckinney         prf 3034
	3	     10-Jul-89	    B. Allam		conversion
        4            22-Mar-91      K. Donaldson        added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   If the look-up table type is BYTE, 
      Set the value limits to 0 and 255.
   If the look-up table type is I*2, 
      Set the value limits to -32768 and 32767.
   Get the requested minimum and maximum look-up table entry values.
   Verify that the requested look-up table entry values are in the valid range.
   Allocate dynamic memory for lut.
   Call STTHRESH to make threshold look-up table.
----------------------------------------------------------------------------
*/

long	l_size, 		/* LUT size */
	lmax, 			/* Maximum threshold value */
	lmin,			/* Minimum threshold value */
	lutmax, 		/* Maximum LUT range value */
	lutmin;			/* Minimum LUT range value */

struct VARIABLE *var; /* Function/variable for parameter retrieval */

/*
If the look-up table type is BYTE, set the value limits to 0 and 255.
If the look-up table type is I*2, set the value limits to -32768 and 32767.
-----------------------------------------------------------------------------
*/
if (lut_type == 1) 
   {
   lutmin = 0;
   lutmax = 255;
   }
else
   {
   lutmin = -32768;
   lutmax =  32767;
   }  

/*
Get the requested minimum and maximum threshold values.
---------------------------------------------------------
*/
var = p_find (&block, "MAXVAL");
lmax = IVAL (*var, 0);
var = p_find (&block, "MINVAL");
lmin = IVAL (*var, 0);

/*
Verify that the requested look-up threshold values are in the valid range.
----------------------------------------------------------------------------
*/
if (lmax > lutmax  ||  lmin < lutmin  ||  lmin >= lmax) 
   msg_err ("!Specified range is not valid", "stretch-range");

/*
Allocate dynamic memory for lut.
-------------------------------
*/
l_size = (long) ((lutmax-lutmin+1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for lut", "stretch-lutalloc");
if (lut_type == 1)
   lut = &templut[0];
else
   lut = &templut[(lutmax-lutmin+1)/2];

/*
Call STTHRESH to make threshold look-up table.
-----------------------------------------------
*/
stthresh (lmax, lmin, lutmax, lutmin);

}
