/**********************************************************************
NAME: GETCOLOR

PURPOSE:
     To convert a line from palette color to 3 bands (r,g,b) using the
     color map associated with the file.  getcolor goes through the
     buffer and writes the r,g,b colormap value for each pixel.
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "tiffio.h"
#include "las.h"
#include "protos.h"

void getcolor
(
    unsigned char *inbuf,       /* I: Buffer that holds the current line  */ 
    int image_width,            /* I: Number of pixels in the row         */
    uint16 *rmap,               /* I: Red palette values                  */
    uint16 *gmap,               /* I: Green palette values                */
    uint16 *bmap,               /* I: Blue palette values                 */
    unsigned char *buf_ptr      /* O: Buffer to store the color values in.
                                      Output is stored in BIL format.     */
)

{
   int j;
   unsigned char temp;
   unsigned char *red_ptr = buf_ptr;
   unsigned char *green_ptr = &buf_ptr[image_width];
   unsigned char *blue_ptr = &buf_ptr[2 * image_width];

   for(j = 0; j < image_width; j++)
   {
      temp = inbuf[j];
      *red_ptr++ = (unsigned char) rmap[temp];
      *green_ptr++ = (unsigned char) gmap[temp];
      *blue_ptr++ = (unsigned char) bmap[temp];
   }
}
