/******************************************************************************
NAME: lng2tae

PURPOSE: Retrieve a long variable from a C program and place it in the TAE 
         variable block.

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ----------------------------------------
    1    04/90  T.Baltzer        EDC   Origination
    2    12/91  J.Fenno          EDC   Modifications for reading Fast Format
				       version B tapes used with LandSAT 6.
				       Changed so that only the required
				       variables are passed back through the
				       TAE variable block.  All variables are
				       written out to a project parameter file.
    3    12/92  K.Sheffield      EDC   Updated for use with the fast format
                                       header library.  The need for a project
				       parameter file has also been eliminated.
    4    01/94  K.Lowell         EDC   Updated for use with NDFIN

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"

extern struct PARBLK vblk;  /* TAE parameter block                           */

int lng2tae
(
    long data[],         /* I: array of longs to transfer              */
    long skipf,          /* I: skip factor                             */
    long num_longs,      /* I: number of elements                      */
    char varname[],      /* I: tae name of variable                    */
    long *result         /* O: value of parameter                      */
)

{
   char tmp_str[CMLEN]; /* temporary string for printing messages            */
   
   long *numbers;       /* the numbers retrieved from the buffer             */
   long *ptr;           /* pointer to walk down the list of numbers retrieved*/
   long ii;             /* loop control                                      */
   long index;          /* data index                                        */
   long nonfatal = 1;   /* nonfatal error status                             */
   
   numbers = (long *)malloc(num_longs * sizeof(long));
   if (numbers == NULL)
   {
      c_errmsg("Error allocating dynamic memory.","arc2las-alloc",&nonfatal);
      return(E_FAIL);
   }
   
   ptr = numbers;
   index = 0;
   for (ii = 0; ii < num_longs; ii++)
   {
      *ptr = data[index];
      index += skipf;
      ptr++;
   }
   
   /* Place the required variables in the TAE variable block
      ---------------------------------------------------------*/
   if (q_intg(&vblk, varname, num_longs, numbers, P_UPDATE) != SUCCESS)
   {
      sprintf(tmp_str,"Error passing %s to TAE",varname);
      c_errmsg(tmp_str,"arc2las-pblk",&nonfatal);
      return(E_FAIL);
   }
   
   *result = *numbers;
   free(numbers);
   return(E_SUCC);
}
