/*****************************************************************************
NAME				 TPL2PCS 

PURPOSE   writes a PCS control point file 

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand    1/90        EDC      Original development
  5.1      D. Etrheim     10/90        EDC	PRF #5526 Write image and haat 
						file names to the gcp only if
						entered

PROJECT			LAS

ALGORITHM DESCRIPTION
	Initialize LAS processing
	Print completion message & exit
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "geompak.h"

main()
{
struct PARBLK  vblk;		/* TAE variable block */
struct VARIABLE *var;		/* TAE variable value... */
struct TPLDATA data;		/* Tie point location data records */
struct TPLHEAD header;		/* Tie point location header records */
FILE *filep;			/* File pointer to output file */
int *tie;
long count;			/* paramter count */
long  std_out;			/* Standard output LUN for inlas */
long  mode = IREAD;		/* IREAD flag for getnam routine */
long  fatal = -1;		/* Fatal error flag for c_errmsg */
long  zero = 0;			/* Constant of zero */
long one = 1;
long  acc = IREAD;		/* Access code for output host file */
long ptyp = TPL;		/* Tie point type is TPL */
long status;			/* Return status flag */
char  fname[CMLEN];		/* Host name of tie point file */
char  out[CMLEN];		/* Name of output file for complt */
char  image[CMLEN];		/* Name of output file for complt */
char  haat[CMLEN];		/* Name of output file for complt */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

if(c_getnam(&vblk, "INTL", &zero, fname, "tpl", &mode) != E_SUCC)
       c_errmsg("Fatal error encountered","tpl2pcs-fatal",&fatal);

mode = IWRITE;
if(c_getnam(&vblk, "OUTFILE", &zero, out, "gcp", &mode) != E_SUCC)
       c_errmsg("Fatal error encountered","tpl2pcs-fatal",&fatal);

filep = fopen(out, "w");

if (c_getcnt(&vblk,"IMAGE",&count) != E_SUCC)
       c_errmsg("Nonfatal error encountered","tpl2pcs-warning",&one);
if (count != 0)
   {
   var = p_find(&vblk, "IMAGE");
   strcpy(image, SVAL(*var, 0));
   fprintf(filep,"%s\n",image);
   }

if (c_getcnt(&vblk,"HAAT",&count) != E_SUCC)
       c_errmsg("Nonfatal error encountered","tpl2pcs-warning",&one);
if (count != 0)
   {
   var = p_find(&vblk, "HAAT");
   strcpy(haat, SVAL(*var, 0));
   fprintf(filep,"%s\n",haat);
   }

/* Open TPL file
  -------------*/
if (c_ptopen(&tie, fname, &ptyp, &acc, &header) != E_SUCC) 
       c_errmsg("Fatal error encountered","tpl2pcs-fatal",&fatal);

/* Make PCS gcp file
  -----------------*/
for(;;)
   {
   status = c_ptio(&tie, &data);
   if (status == E_EOF) break;
   if (status == E_FAIL) 
       c_errmsg("Fatal error encountered","tpl2pcs-fatal",&fatal);

   if ((data.active == 0) || (data.active == 1) || (data.active > 9))
      fprintf(filep,"%9.3f %9.3f %14.9f %14.9f %9.3f\n", data.sea_coord[1], 
        data.sea_coord[0], data.geo_coord[1], data.geo_coord[0],
        data.elevation);
   }

/* Write completion message and exit program
  -----------------------------------------*/
c_ptclse(&tie);
fclose(filep);
c_complt("TPL2PCS", out, &one);
}
