/****************************************************************************
NAME:				TPLPOLY

PURPOSE:  Scales image coordinates in a Tiepoint Location file 

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 7/92	D. Steinwand

*****************************************************************************/
#include "las.h"
#include "geompak.h"

main()		
{
struct PARBLK vblk;		/* TAE parameter block */
struct VARIABLE *p_find();	/* TAE parameter function */
struct VARIABLE *var;		/* TAE variable pointer */
struct TPLHEAD header;		/* TPL header data */
struct TPLDATA tpldata;		/* TPL data record */
struct GEOGRID grid;		/* Geometric Mapping Grid structure */
double c_eval();		/* Routine to evaluate a polynomial */
long degree;			/* Degree of polynomial (1-4) */
long msgtim;			/* Message timer */
long std_out;			/* Standard output LUN */
long status;			/* Function return status */
long fatal = -1;		/* Fatal flag for errmsg */
long zero = 0;			/* Constant of 0 */
long one = 1;			/* Constant of 1 */
long two = 2;			/* Constant of 2 */
long ptyp = TPL;		/* Type of TPL file used (for I/O) */
long acc;			/* Access mode of TPL file */
int *intpl,*outtpl;		/* File descriptors */
char tpl_name[CMLEN];		/* The input Tie Point Location file name */
char out_tpl[CMLEN];		/* The output Tie Point Location file name */
char grid_name[CMLEN];		/* The Geometric Mapping Grid file name */
extern long curent;
extern long curcat;

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk, "INTL", &zero, tpl_name, "tpl", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplpoly-fatal",&fatal);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(&vblk, "OUTTL", &zero, out_tpl, "tpl", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplpoly-fatal",&fatal);


/* Get the grid name, open the grid and extract the polynomial coefficients
  ------------------------------------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk, "INGRID", &zero, grid_name, "grid", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplpoly-fatal",&fatal);
if(c_getgrd(grid_name, &grid) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplpoly-fatal",&fatal);
if (grid.ncoeff == 3) degree = 1;
if (grid.ncoeff == 6) degree = 2;
if (grid.ncoeff == 10) degree = 3;
if (grid.ncoeff == 15) degree = 4;

/* Get the message timer
  ---------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);

/* Open the input Tie Point Location file
  ---------------------------------------*/
c_inittm("TPLPOLY","Processed","coordinate number","","",&two,&two, &msgtim, &two);
acc = IREAD;
if(c_ptopen(&intpl, tpl_name, &ptyp, &acc, &header) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplpoly-fatal",&fatal);

/* Open output Tie Point Location file
  ------------------------------------*/
acc = IWRITE;
if(c_ptopen(&outtpl, out_tpl, &ptyp, &acc, &header) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplpoly-fatal",&fatal);

/* Process all points in the file (Loop until EOF)
  -----------------------------------------------*/
curent = 1; c_startm(); curent = 0;
for (;;)
   {
   status = c_ptio(&intpl, &tpldata);
   if (status == E_EOF) break;
   if (status == E_FAIL) 
	c_errmsg("Fatal error encountered","tplpoly-fatal",&fatal);
   curent++;
   tpldata.sea_coord[0] = c_eval(&degree, grid.coeffs, &tpldata.ref_coord[1], 
		&tpldata.ref_coord[0]);
   tpldata.sea_coord[1] = c_eval(&degree, &(grid.coeffs[grid.ncoeff]), 
		&tpldata.ref_coord[1], &tpldata.ref_coord[0]);
   if (c_ptio(&outtpl, &tpldata) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplpoly-fatal",&fatal);
   }
c_stoptm();
c_complt("TPLPOLY", out_tpl, &one);
}
