#include "tsclass.h"

double	FUNCTION angle( x0, y0, x1, y1, x2, y2 )

long	x1, y1,           /* x and y coordinates of first vertex */ 
        x2, y2,           /* x and y coordinates of second vertex */
	y0;               /* y coordinate of pivot point */
float   x0;               /* x coordinate of pivot point */
/*
------------------------------------------------------------------
NAME: ANGLE

PURPOSE
          finds rotation angle between two given vertices relative
          to given pivot point.
 
PROGRAM HISTORY:
      Version           date               Author
      -------           ----               ------   
         1            15-mar-82           J. Marvin
         2            27-feb-84           Y. Jun (scn20-180)
         3            10-May-90           W. Xia (stx)
 
 COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
                                                             
  finds the rotation angle between the points (x1,y1) and     
  (x2,y2) relative to the central pivot point (x0,y0)         
                                                              
		calculated by using the ratio of the cross     
		product (proportonal to sine(angle)) to the    
		dot product (proportional to cosine(angle))    
                                                              
		the sense of the rotation (+ or -) is          
		important.                                     
-------------------------------------------------------------------
*/
{
double  angle,       /* rotation angle in radians */
	cross,       /* cross product */
        dot;         /* dot product */
float   xx1, xx2,    /* difference of x coordinates */
        yy1, yy2;    /* difference of y coordinates */

xx1 = x1-x0;
yy1 = y1-y0;
xx2 = x2-x0;
yy2 = y2-y0;

cross = xx1*yy2 - xx2*yy1;
dot   = xx1*xx2 + yy1*yy2;

/*
---------------------------------------------------------------
		check for 90 degree angle:                     
---------------------------------------------------------------
*/
if ( abs(dot)  ==  0.0 )
   angle = 1.5707963;
else
  {
   angle = atan( abs(cross/dot) );
   if ( dot  <  0.0 ) angle = 3.1415926 - angle;
  }

if ( cross  <  0.0) angle = -angle;

return(angle);
}
