#include "tsclass.h"

FUNCTION byclasfy (x, totbnd, ns, icount, nclass_used,
                   clsind, class)

float	*x[MAXBND];		/* i   image data                             */
long	totbnd,			/* i   number of bands                        */
	ns,			/* i   number of pixels to be classified      */
	nclass_used,			/* i   number of classes                      */
	clsind[],		/* i   class indices                          */
        icount;                 /* polygon number    */         
float   class[];  	  	/* o   class assignments                      */
                           
/***********************************************************************
                
NAME:	byclasfy  

FUNCTION:
	Do bayesian maximum likelihood classification

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0         Aug-84     Y. Sheu           version 1.0
    2.0      10-Jan-85     Y. Sheu            revision 1.0
    3.0      10-May-90     W. Xia(stx)       coversion to c

     **	Note: This calls IMSL subroutines vmulfs & viprff

***********************************************************************/
{                    
/* local variables */         
static float	g[MAXCLS][MAX_SEGP];	/*  g values                              */
	
float	gmax,			/* Maximum g value                            */
	offset,			/* Offset for scaling                         */
	scale1;			/* Scale for scaling                          */

float	*temp[MAXBND],	        /* Temporary work buffer                      */
	*temp1[MAXBND],	        /* Temporary work buffer                      */
        *temp_matrix,
        *temp1_matrix,
     	temp2[MAX_SEGP];	/* Temporary work buffer                      */
     
long	dtype = EREAL,		/* Data type to do work with                  */
        itype=ELONG,
	imax,			/* Index of class with the maximum g value    */
	i, j,			/* Loop counter on bands                      */
	k, m,			/* Loop counter on classes                    */
	l,			/* Loop counter on pixels                     */
        c_size, 
 
      	num_rows = MAX_SEGP;	/* Number to pass by reference                */

long    zero = 0;
  
/*
  Allocate storage for temporary buffers
  --------------------------------------
*/
c_size = totbnd * MAX_SEGP * sizeof (float);
temp_matrix = (float *) malloc (c_size);
temp1_matrix = (float *) malloc (c_size);  
for (i=0, m=0; i < totbnd;i++, m+= MAX_SEGP) {
    temp[i] = &temp_matrix[m];
    temp1[i] = &temp1_matrix[m];
}

/*
initialize sclass
-----------------
  c_pxfill(sclass[icount],&itype, &nclass_used,&zero);
*/
  
for (k = 0; k < nclass_used; k++)
   {
/*
   Get difference vectors
   Subtract class mean values, mean values have been multiplied
   by -1.0 in subroutine do_stats
   ------------------------------------------------------------
*/
   for (j = 0; j < totbnd; j++)
      {
      scale1 = 1.0;
      c_pxline(x[j],temp[j],&ns,&dtype,&scale1,&sstat.mean[k][j]);
      }
/*
   Do bayes classification

   Multiply the transpose of difference vectors and the inverse covariance
   Data are stored as MAXBND by MAX_SEGP matrix.
   It is in transpose form already
   -----------------------------------------------------------------------
*/
/*
   *** IMSL routine
   ----------------
*/
   vmulfs (temp[0], sstat.covinv[k], &ns, &totbnd, &num_rows, temp1[0], 
           &num_rows);
/*
   Multiplied with the difference vectors
   --------------------------------------
*/
   for (j = 0; j < ns; j++)
/*
   *** IMSL routine
   ----------------
*/
      viprff (&temp1[0][j], &temp[0][j], &totbnd, 
              &num_rows, &num_rows, &temp2[j]);
/*
   Compute g values
   ----------------
*/
   scale1 = -0.5;
   c_pxline(temp2,g[k],&ns,&dtype,&scale1,&cconst[k]);
   }

/* Free temporary buffers
-------------------------*/
free (temp_matrix);
free (temp1_matrix);

/*
   Find the class index with maximum g value
   -----------------------------------------
*/

/*  
 initialize the class to zero 
------------------------------
*/
   c_pxfill(class, &dtype, &num_rows, &zero);   
    

for (l = 0; l < ns; l++)
   {

   gmax = -999999.0;
   imax = 1;
   for (k = 0; k < nclass_used; k++)
      {
      if (g[k][l] > gmax)
         {
         gmax = g[k][l];
         imax = k+1;
         }
      }                                            
   class[l] = (float) imax;  
   ++sclass[icount][imax-1];
   }       

return;
}
