#include "tsclass.h"

FUNCTION bypriori (prior, nclass, aprior, clsind)

float	prior[MAXBND];		/* o   prior probabilities                    */
long	nclass,			/* i   # classes                              */
	clsind[];		/* Class indeces                              */

char	aprior[6];		/* i   user option for prior probabilities    */

/************************************************************************

NAME:	priori_prob

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0         Aug-84     Y. Sheu          version 1.0
    2.0      10-Jan-85     Y. Sheu          revision 1.0
    3.0         May-90     W. Xia           conversion to c

FUNCTION:
	Finds a priori probabilities according to user option

	Comments-     The prior probabilities entered from the terminal are
	              summed.  if the sum != 1 the user is given a 2nd
	              chance to enter valid values.  if the values still
	              don't sum to 1 then the program aborts with priors

***********************************************************************/
{
float	prob,			/* Probability                                */
	sum;			/* Sum of prior probabilities                 */

short	less_than_0;		/* Indicates prior probabilty is less than 0  */

long	count,			/* Number of tries on prob sum                */
	ierr,			/* Return status                              */
	ind,			/* Loop counter on classes                    */
	k,			/* A class index                              */
	lj,			/* Loop counter on class name string          */
	shape[9],		/* stats data descriptor                      */
	term;			/* How the string was terminated              */

char	cname[80],		/* class name in character format             */
	lname[200],		/* class name as returned from stats file     */
	pval1[80];		/* A prior probability for the class          */

static	char	messg[2][80] = {"Enter prior prob for class  ",
			        "Prior probs must be > 0. please reenter "};
			/* array to hold terminal i/o messages                */
long    fatal_err = -1;                /* fatal error return status */
long    inf_err = 1;                   /* info error return status  */
long    zero = 0;

/*
   Equal prior probabilities
   -------------------------
*/
if ((strcmp (aprior, "EQUAL")) == 0)
   {
   prob = 1.0 / ((float) nclass);
   for (ind = 0; ind < nclass; ind++)
      prior[ind] = prob;
   }
else if ((strcmp (aprior, "SFILE")) == 0)
   {   
/*
   Retrieve prior probabilities from stats file
   --------------------------------------------
*/
   sum = 0.0;
   for (ind = 0; ind < nclass; ind++)
      {
      k = clsind[ind];
      ierr = c_stget (&sdesc[0], "PRIOR_PROBABILITY",&k, &zero, shape,
             &prior[ind]);
      if (ierr != E_SUCC)
         { 
         c_errmsg ("No prior probabilities in stats file",
                   "bayes_noprior", &fatal_err);             
         }
      sum += prior[ind];
      }
   if (sum != 1.0)
      c_errmsg ("Sum of prior probs for selected classes is not 1",
               "bayes_prior", &fatal_err);
   }
else
   {
/*
   Enter prior probabilities at terminal
   -------------------------------------
*/
   count = 0;
   do {
      sum = 0.0;
      for (ind = 0; ind < nclass; ind++)
	 {
	 k = clsind[ind];
         strcpy (lname, "");
	 ierr = c_stget (&sdesc[0], "NAME", &k, &zero, shape, lname);
         if (ierr != E_SUCC)
            {
            c_errmsg("No class name in stats file", "bayes_noclass",
		     &fatal_err);
            } 
         lname[shape[2]] = '\0';
	 strcpy (cname, lname);
         for (lj = strlen (cname); lj < 80; cname[lj++] = ' ')
            ;
/*
   Change to write the mess 1 and wait for cname
   ---------------------------------------------
*/
	 t_write (messg[0], T_STDCC);
	 t_write (cname, T_STDCC);
         less_than_0 = TRUE;
         do {
	    t_read (pval1, &term);
            sscanf (pval1, "%f6.4", &prior[ind]);
/*
   Change to mess 2 and get back for the value
   -------------------------------------------
*/
	    if (prior[ind] <= 0)
	       t_write (messg[1], T_STDCC);
            else
               less_than_0 = FALSE;
            } while (less_than_0);
	 sum += prior[ind];
	 }
/*
   Check values sum to 1
   (user has a 2nd chance if they don't)
   -------------------------------------
*/
      if (((sum - 1.0) > .00001) || ((sum - 1.0) < -.00001))
	 {
	 count++;
	 if (count == 2)
	    c_errmsg ("Prior probs did not sum to 1 on 2nd attempt. Restart",
                     "bayes_prior", &fatal_err);
	 else
	    c_errmsg ("Prior probs do not sum to 1. You have one more chance",
                     "bayes_prior", &inf_err);
	 }
      } while (fabs(sum-1.0) > 0.0001);
/*            
   Write values into stats file
   ----------------------------
*/
   shape[0] = EREAL;
   shape[1] = 1;
   shape[2] = 1;
   t_write ("", T_STDCC);
   t_write ("Priori probabilities are being written into stats file\n", T_NULL);
   for (ind = 0; ind < nclass; ind++)
      {
      k = clsind[ind];
      ierr = c_stput (&sdesc[0], "PRIOR_PROBABILITY", &k, &zero, shape, 
             &prior[ind]);
      if (ierr == E_GNLIO)
        c_errmsg ("Error writing probs to statistics file",
		 "bayes_write", &fatal_err);
      }
   }
return;
}
