#include "tsclass.h"

FUNCTION getstat (stahos, nclass_used, aprior,
		  num_inbands, clsind, maxmin)
long	
	maxmin,                 /* classifire indicator 
		                   (=1 : Bayesian; =2 : Minimum Distance   */
	*nclass_used, 	/* Number of clases in stats file             */
	num_inbands,		/* i   # input images                         */
      	clsind[MAXCLS];	        /* o   class indices       */
      
char
    	aprior[6],      	/* i   option for prior probabilities  */
        stahos[];       
                                
/************************************************************************

NAME: getstat.c

FUNCTION:
	Retrieves and controls processing of statistics from
	stats file and controls receipt and processing of
	prior probabilities

     **	Note: This calls IMSL subroutine linv2p

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0         AUG-84     Y. SHEU           version 1.0
    2.0      31-May-90     W. Xia/STX        conversion 5.0
  
ALGORITHM DESCRIPTION:

1. Open statistic file
2. Select the number of classes to be processed
3. Get prior probabilities      
      
***********************************************************************/
{
static double	*cfile,         /* covariance matrix returned from stats file */
	cmean[MAXBND],		/* mean vector returned from stats file       */
	dtemp,			/* Temporary storage for determinant          */
	pi2;			/* 2 * PI                                     */

static float	cdet,		/* Log of the determinant                     */
	*covar,         	/* covariance matrix for a class              */
	det[2],			/* Temporary float value                      */
	determ,			/* Determinant of the class matrix            */
	logpi2,			/* Temporary variable for multiplication      */
	offset,			/* Offset for scaling (not used)              */
	prior[MAXCLS], 	        /* prior probabilities                        */
	scale;			/* Scale value                                */


static long	
	cshape[9],		/* stats data descriptor for covariance       */
	mshape[9],		/* stats data descriptor for mean             */
	dtype = EREAL,		/* The data type we are working with          */
	i,			/* Loop counter                               */
	ierr,			/* Return status                              */
	ind,			/* Loop counter on classes                    */
	k,			/* A class number                             */
        c_size, t_size,
	nval;			/* Number of values in covariance matrix      */

static char	msg[80];	/* Output message buffer                      */

long	access;                   /* access type for getnam             */
long    fatal_err = -1;                /* fatal error return status */
long    inf_err = 1;                   /* info error return status  */
long    zero = 0;

/*
  Allocate storage for temporary buffers
  --------------------------------------
*/
c_size = (num_inbands) * (num_inbands+1)/2 * sizeof (float);
t_size = (num_inbands) * (num_inbands+1)/2 * sizeof (double);
cfile = (double *) malloc (t_size);
covar = (float *) malloc (c_size);
if ((cfile == NULL) || (covar == NULL))
   {
   c_errmsg("Error allocating dynamic memory","getstat-alloc",&inf_err);
   c_errmsg("Fatal error encountered","getstat-fatal",&fatal_err);
   }
        
if (maxmin == 1) 
{                                                            
  pi2 = 3.1415926 * 2.0;  
  logpi2 = 0.5 * (num_inbands) * log (pi2);

  /* Get prior probabilities 
  ----------------------------*/                 
  bypriori (prior, *nclass_used, aprior, clsind);
  
  /*
  Get mean and covariance from stats file
  Check # values corresponds to #bands 
  -----------------------------------------
  */
  nval = (num_inbands * (num_inbands + 1)) / 2;
}
                        
/*  Allocate dynamic storage */
dyn_alloc (num_inbands, *nclass_used);
                               
for (ind = 0; ind < *nclass_used; ind++)
   {
   k = clsind[ind];
   ierr = c_stget (&sdesc[0], "MEAN_VECTOR", &k, &zero, mshape, cmean);
   if (ierr != 0)
      {   
      sprintf (msg, "!No mean vector present for class # %d", k);
      c_errmsg (msg, "tsclass-mean", &fatal_err);
      }
   if (mshape[2] != num_inbands)
      {
      sprintf (msg, "!Mean vector does not have %d values for class # %d", 
               num_inbands, k);                        
      c_errmsg (msg, "tsclass-mean", &fatal_err);
      }                        

/*                              
Multiply class means by -1.0 for later use
Later the addition of vectors will be used instead of substraction
------------------------------------------------------------------
*/
   for (i = 0; i < num_inbands; i++)                   
      {
      sstat.mean[ind][i] = cmean[i];                                   
      }                                     
   scale = -1.0;
   offset = 0.0;
   c_pxline (sstat.mean[ind], sstat.mean[ind], &num_inbands, &dtype, &scale,
             &offset);

   if (maxmin ==1 )
   {
   ierr = c_stget (&sdesc[0], "COVARIANCE_MATRIX", &k, &zero, cshape, cfile);
   if (ierr != 0)
      {
      sprintf (msg, "!No Covariance matrix present for class # %d", k);
      c_errmsg (msg, "getstat_covar", &fatal_err);
      }                    
   if (cshape[2] != nval)
      {
      sprintf (msg, "!Covariance matrix has too few/many values for class # %d",
                k);
      c_errmsg (msg, "getstat_fewcov", &fatal_err);
      }            
   
/*                   
Compute inverse of covariance matrix
The covariance and inverse of covariance are stored in symmetric mode
---------------------------------------------------------------------
*/
   for (i = 0; i < nval; i++)
      covar[i] = cfile[i];
   
/*                 
*** IMSL routine
----------------
*/
   linv2p (covar, &num_inbands, sstat.covinv[ind], det);
/*
Compute determinant of class matrix
-----------------------------------
*/
   determ = det[0] * pow (10.0, ((double) det[1]));
   if (determ <= 0.0)
      {
      sprintf (msg, "!Covar matrix contains all 0's for class # %d", k);
      c_errmsg (msg, "getstat_covar", &fatal_err);
      }
/*
free dynamic array
------------------
*/               
free (covar);
free (cfile);

/*
Compute class constant
----------------------
*/
   dtemp = determ;
   cdet = log (dtemp);
   dtemp = prior[ind];
   cconst[ind] = log (dtemp) - logpi2 - 0.5 * cdet;
   }   /* for Bayes  */ 
  }   /* end of ind loop */
return;
}
