
#include "tsclass.h"

FUNCTION indices (nclass_asked, nclass_used, clsind, nsites)

long
  	nclass_used,        /* # of classes specified by user */
	nclass_asked,       /* # of classes to be analyzed       */
	clsind[],       /* set of class indices              */
	nsites[];       /* # of sites to be analyzed for a class */

/*
****************************************************************************
NAME: INDICES

PURPOSE:    Receives user specified class and site names and          
             retrieves their associated index numbers from statistics   
             file.

PROGRAM HISTORY:
      Version            Date             Author            Request
      -------            ----             ------
	1.0            10-Jan-85          Y. Sheu          version 1.0        
        2.0               Mar-90          W. Xia/stx      conversion to c
  
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

ALGORITHEM:

************************************************************************
*/

{
/* local variables */
long 
        count=0,
        i, 		/* loop index */
	ic,             /* class index number */
	ierr,
        k,
	found,   
        ns,
	shape[9],	/* statistics data descriptor */
	stat;	 		/* return status                           */
static char	cname[MAXCLS][CMLEN];    /* class names                       */
long    fatal_err = -1;                /* fatal error return status */
long    inf_err = 1;                   /* info error return status  */
long    zero = 0;

/*
 get user class and site specification and associated
 stats file index numbers
---------------------------------------------------------
*/

for (i=0; i < nclass_used; i++)
   {
   count=0;
   if (nclass_asked != 0) 
      {
      found=0;
/*
 not all classes being used => prompt user for name and
 check against stats file
--------------------------------------------------------
*/
      while (!found)
        {
         printf("\nEnter Class name : ");    
         scanf("%s", cname[i]);
         ierr = c_stfdcl (&sdesc[0], cname[i], &ic);         
         if (ierr == E_GNLIO)
           c_errmsg("general I/O error", "tsclass-i/o", &fatal_err);
/*                   
---------------------------------------------
 class name not found.
 user is given 3 chances to enter valid name.
---------------------------------------------
*/
         if (ierr == E_NOCLN) 
           {
            count=count+1;
            if (count == 3) 
              c_errmsg("3 Errors for class name.  Restart.", "tsclass-cname",
                        &fatal_err);
            else
             {
              c_errmsg("Class not found.  3 chances to reenter a class name.",
                       "tsclass-cname", &inf_err);
             } 
           } 
         else
           found = 1;
        }                        /* end of while loop */ 
/*         
 get # of sites in file
-----------------------
*/
       stat = c_stget (&sdesc[0], "NSITES", &ic, &zero, shape, &ns);
       if (stat  !=  E_SUCC )                                     
         {                                                       
         c_errmsg ("Error geting no. of sites","tsclass-indices",
                    &fatal_err);                
         }
       nsites[i]=ns;
/*
 write class index # into list of class indices
-----------------------------------------------
*/
      clsind[i]=ic; 

/*
 all classes requested
 class indices incremented sequentially
-----------------------------------------
*/
      }
   else
      {
      k = i+1;
      clsind[i]= k;
      stat = c_stget (&sdesc[0], "NSITES", &k, &zero, shape, &nsites[i]);
      if (stat  !=  E_SUCC )      
         {        
         c_errmsg ("STAT I/O error for getting sites no.","tsclass-indices",
                    &fatal_err);
         }
      }			 /* if */
   }                     /* end of i loop */
}
