#include "tsclass.h"

FUNCTION insde( xverts, yverts, nverts, xpos1, xpos2, ypos )

long
	nverts,             /* number of vertices */
        xpos1,              /* x coordinate of first intersection point */
        xpos2,              /* x coordinate of second intersection point */
        ypos,               /* y coordinate of test point */
    	xverts[],          /* x coordinate of vertices */
    	yverts[];          /* y coordinate of vertices */

/*      
********************************************************************

NAME:  insde.c 

PURPOSE: determines whether a point is interior to a polygon or not.

PROGRAM HISTORY:
  Version      Date       Author                       Request
  -------      ----       ------                       -------
   1.0     15-Mar-82      J. Marvin                    version 1.0
   2.0     27-Feb-84      Y. Jun                       revision 1.0
   3.0       Junb-84      Y. Sheu                      revision 2.0
   4.0       May-90       W. Xia                       conversion to C

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
 

*************************************************************************
*/

{
/* local variables */

long	insde,                /* true if point is inside; else false */
	i;                    /* counter */

float	xpos;                 /* x-coord of two intersection points */
double  angle();              /* finds angle between two vertices */
double  angsum,               /* sum of rotation angles between vertices */
        angnew;               /* angle between two vertices */
 
insde = FALSE;

/*
 check if two intersections are th same point or not
 if they are the same point, this point is 
 polygon vertex or on the polygon side
*/             
if (xpos1  !=  xpos2)
  {
  xpos = (xpos1+xpos2)/2.;        
  xverts[nverts] = xverts[0];
  yverts[nverts] = yverts[0];
  if (nverts < 3 )  
    return(insde);
  else
   {
   angsum = 0.;

   for (i = 1; i <= nverts; i++)
     {
     angnew = angle(xpos, ypos, xverts[i-1], yverts[i-1], xverts[i], yverts[i]);
/*
 check for the polygon line
 if the angle is pi, the mid-point of two intersections is
 lying on the line connecting two vertices.  this means that it is
 on a polygon side line and this line is horizontal
------------------------------------------------------------------             
*/
     if (angnew  !=  3.1415926)
       angsum = angsum + angnew; 
     else
       {
       insde = TRUE;
       return(insde);
       }
     } /* for */
   } /* if */
 } /* if */
else insde = TRUE;

if ( abs(angsum) >=  3.1415926 ) 
   insde = TRUE;  

return(insde); 
}
