#include "uhist.h"
#include "protos.h"

void defaults
(
    char hosin[][CMLEN],			/* input image host name             */
    long nbands,				/* total number of input image bands */
    long num_inimg,			/* number of input images            */
    long num_img_bands[][MAXBND+1],	/* band numbers for each input image */
    long bands_per_img[],		/* number of bands in each image     */
    double amin[],				/* minimim pixel values              */
    double amax[],	 			/* maximum pixel values              */
    float *scale				/* scaling factor for gradient       */
)
/*
*****************************************************************************

NAME:	 DEFAULT	

PURPOSE: Computes a default scaling factor if user has not provided one. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         11-Apr-83    R.J. Tighe
   2          5-Apr-85    S. Ross            SCN# 20-466
   3         12-Dec-89    K. Kunska          conversion to 5.0   
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   For each band, retrieve or calculate max & min pixel values from the ddr. 
   Compute the multiplicative scaling factor.
   Tell the user what scaling factor will be used.

*****************************************************************************
*/
{

double	diff,				/* difference between the maximum    */
					/* and the minimum pixel values      */
	sum;				/* running sum of squares of diff    */ 

long	band;				/* current band number               */


/*
For each band, retrieve or calculate max & min pixel values from the ddr. 
-------------------------------------------------------------------------
*/
c_minmax (hosin, &num_inimg, window, num_img_bands, bands_per_img, amin, amax);

/*
Compute the multiplicative scaling factor.
---------------------------------------------
*/
sum = 0.0;
for (band = 0; band < nbands; band++)
   {
   diff = amax[band] - amin[band];
   sum = sum + diff * diff;
   }
*scale = sqrt(sum);

/*
Tell user what scaling factor will be used.
-------------------------------------------
*/
printf ("\nComputed default scale factor used: %12.5e\n", *scale);

*scale = 100.0 / *scale;

}
