#include "uhist.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char hostin[][CMLEN], 		/* host name for input image          */
    char hostout[], 			/* host name for output image         */
    long num_inimages,			/* total number of input images       */
    long num_img_bands[][MAXBND+1],	/* band numbers for each input image  */
    long out_img_bands[MAXBND+1],	/* band numbers for each output image */
    long out_bands,			/* number of output image bands       */
    long bands_per_img[],		/* number of bands for each image     */
    long *dtype				/* input image data type	      */
)

/*
*****************************************************************************

NAME:	 FILSPEC	

PURPOSE: Opens the input and output images. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          11-Apr-83    R.J. TIGHE
   2          14-Feb-85    S. ROSS,           SCN#20-418
   3          12-Dec-89    K. Kunska          conversion to 5.0   
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open the input image file.
   Allocate buffer space for the input image files.
   Group the input image files.
   Open the output image file.
   Allocate buffer space for the output image file.
   Group the output image files.
   Update the ddr records.

*****************************************************************************
*/

{

long	access,				/* access type                        */
	bufsize,			/* buffer size                        */
	comb_flag,			/* combine flag for 1-band output img */
	i,				/* loop counter and array index       */
	n,				/* loop counter and array index       */
	nib,				/* band index                         */
	opt[2],			 	/* type conversion option             */
	stat;				/* return status                      */

double 	line_inc,			/* line increment                     */
	samp_inc;			/* sample increment                   */

long zero = 0;
long one = 1;
/*
Open the input image file.
---------------------------
*/
access = IREAD;
/* type = EREAL;	*/
for (n = 0, nib = 0; n < num_inimages; n++)
   {
   stat = c_eopens (&fdescin[n], hostin[n], num_img_bands[n], &window[n][SL],
                    &window[n][SS], &window[n][NL], &window[n][NS], 
                    &ipoint[nib], &access, dtype, &zero, &zero);
   if (stat != E_SUCC)
      c_abort (&block, "Error opening the input image", "uhist-inopen",
               "uhist", out_files, &num_outfiles);
   nib += bands_per_img[n];
   }
fdescin[++n] = 0;

/*
Allocate buffer space for the input image files.
-------------------------------------------------
*/
if ((inbuf = (unsigned char *) c_ealloc (fdescin, &bufsize)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "uhist-alloc",
            "uhist", out_files, &num_outfiles);

/* 
Group the input image files.
-----------------------------
*/
c_egroup (fdescin, &gdescin, inbuf, &bufsize);


/*
Open the output image file.
-----------------------------
*/
n = 0;
access = IWRITE;
opt[0] = EBYTE;
opt[1] = 0;
for (i = 0; i < out_bands; i++)
   out_img_bands[i] = i + 1;

stat = c_eopens (&fdescout[n], hostout, &out_bands, &one, &one, &window[n][NL], 
                 &window[n][NS], opoint, &access, &opt[0], &zero, opt);
if (stat != E_SUCC)
   c_abort (&block, "Error opening the output image", "uhist-outopen",
            "uhist", out_files, &num_outfiles);
fdescout[++n] = 0;

/*
Allocate buffer space for the output image file.
-------------------------------------------------
*/
if ((outbuf = (unsigned char *) c_ealloc (fdescout, &bufsize)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "uhist-alloc",
            "uhist", out_files, &num_outfiles);

/* 
Group the output image files.
--------------------------------
*/
c_egroup (fdescout, &gdescout, outbuf, &bufsize);

/*
Update the ddr records.
------------------------
*/
line_inc = 1;
samp_inc = 1;
comb_flag = COMB;

c_upddr (&block, hostin, hostout, &num_inimages, window, num_img_bands,
         bands_per_img, &line_inc, &samp_inc, &out_bands, &comb_flag);
}
