#include "uhist.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char taein[][CMLEN],		/* tae name of the input image              */
    char hosin[][CMLEN],		/* host name of the input image             */
    char taeout[][CMLEN], 		/* tae name of the output image             */
    char hosout[][CMLEN],		/* host name of the output image            */
    float *scale,			/* scaling factor                           */
    long *bands_per_img, 	/* number of bands per input imgae           */
    long *dtype,			/* data type of the input image              */
    long *num_inimg,		/* total number of input images              */
    long (*num_img_band)[MAXBND+1],	/* band numbers for each input image */
    long img_limit,		/* maximum number of images                  */
    long (*img_flag)[2],		/* specified image/band options              */
    long *same_wnd, 		/* same window size? 1=Y 2=N                 */
    long *tot_inbands		/* total number of input bands               */
)

/*
*****************************************************************************

NAME:	GETPAR

PURPOSE: Retrieves the user supplied parameters from TAE.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          11-Apr-83    R. J. Tighe
   2          14-Feb-85    S. ROSS            SCN# 20-418
   3          12-Dec-89    K. Kunska          conversion to 5.0    
   4          03-Oct-90    K. Kunska          changes declaration of mnmx

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	

	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   
   Initialize tae.
   Parse the input image specification. 
   If the return status is not success then 
      Abort with a fatal error message.
   Determine the input image type.
   Parse the output image specification.
   If the return status is not success then 
      Abort with a fatal error message.
   Get the scaling factor.
   Get the LAS global paramters.

*****************************************************************************
*/

{
char	mnmx[4];		/* minmax flag                               */

long    i,			/* loop counter and array index              */
	partype,		/* data type of a parameter                  */
	status,			/* return status                             */
	std_out;		/* logical unit for standard output          */

struct VARIABLE   *var;	/* pointer to a variable in the tae block     */

long zero = 0;
long one = 1;
long fatal_err = -1;

/*
Initialize tae. 
---------------
*/
c_inlas (&block, &std_out); 

/*
Parse the input image specification.
If the return status is not success then abort with a fatal error message.
--------------------------------------------------------------------------
*/
status = c_getin (&block, "IN", &zero, taein, hosin, num_inimg, window, 
                  num_img_band, bands_per_img, tot_inbands, &img_limit, 
                  same_wnd, img_flag); 

if (status != E_SUCC)
   c_errmsg ("Cannot access input file names", "uhist-getin", &fatal_err);
if (*same_wnd == 0)
   c_errmsg ("Image windows differ in size", "uhist_window", &fatal_err);

/*
Determine the input image type.
-------------------------------
*/
for (i = 0; i < *num_inimg; i++)
   { 
   status = c_getddr (hosin[i], &ddr);
   if (status != E_SUCC)
      c_errmsg ("DDR cannot be found", "uhist-getddr", &fatal_err);
   }
*dtype = ddr.dtype;

/*
Parse the output image specification.
If the return status is not success then abort with a fatal error message.
--------------------------------------------------------------------------
*/
status = c_getout (&block, "OUT", taeout, hosout); 
if (status != E_SUCC)
   c_errmsg ("Cannot access output file names", "uhist-getout", 
              &fatal_err);
num_outfiles = 0;
strcpy (out_files[num_outfiles++], taeout[0]);

/*
Get the scaling factor.
------------------------
*/
partype = EREAL; 
c_getnum (&block, "SCALFACT", &one, &partype, scale); 
if (*scale != 0.0) 
   *scale = 100.0 / *scale;

/* 
Get the LAS global paramters.
------------------------------ 
*/
partype = ELONG;
c_getnum (&block, "$MSGTIM", &one, &partype, &msgtim);
var = p_find(&block, "$MINMAX");
strcpy (mnmx, SVAL(*var, 0));
if ((mnmx[0] == 'Y') || (mnmx[0] == 'y'))
   min_max = TRUE;
else
   min_max = FALSE;

}
