/****************************************************************************
NAME:				UPTMDDR

PURPOSE:  

VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 12/93	D. Steinwand

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#define X 100
#define Y 200

main()		
{
struct PARBLK vblk;     /* TAE parameter block */
struct GEOGRID forgrid; /* Geometric mapping grid structure (forward) */
struct GEOGRID invgrid; /* Geometric mapping grid structure (inverse) */
struct DDR ddr;         /* DDR structure */
struct BDDR bddr[MAXBND]; /* Band DDR structure */
long fatal = -1;	/* Fail flag for fatal error message */
long zero = 0;		/* Constant of zero */
long one = 1;		/* Constant of one */
long std_out;		/* LUN of standard output */
long acc;		/* File access mode */
long i;			/* Loop counter */
long bndno;		/* One-relative band number */
char inimg[CMLEN];	/* Input image name */
char forgridname[CMLEN];/* Geometric Mapping Grid file name (forward) */
char invgridname[CMLEN];/* Geometric Mapping Grid file name (inverse) */
double a[3],b[3];	/* Polynomial mapping input l/s to output l/s */
double pa[3],pb[3];	/* Polynomial mapping output l/s to proj coords */
double calc_corn();	/* Routine to calculate new corner projection coords */

/* Initalize processing
  --------------------*/
c_inlas(&vblk, &std_out);

/* Read the Geometric Mapping Grid file
  ------------------------------------*/
acc = IREAD;
if(c_getnam(&vblk, "FORGRID", &zero, forgridname, "grid", &acc) != E_SUCC)
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
if(c_getgrd(forgridname, &forgrid) != E_SUCC) 
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
if(c_getnam(&vblk, "INVGRID", &zero, invgridname, "grid", &acc) != E_SUCC)
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
if(c_getgrd(invgridname, &invgrid) != E_SUCC) 
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);

/* Extract the forward mapping polynomial from the Grid structure
  --------------------------------------------------------------*/
if(forgrid.ncoeff != 3) {
   c_errmsg("UPTMDDR requires a first order polynomial adjustment",
	"uptmddr-notaffine",&one);
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
   }
b[0] = forgrid.coeffs[0];
b[1] = forgrid.coeffs[1];
b[2] = forgrid.coeffs[2];
a[0] = forgrid.coeffs[3];
a[1] = forgrid.coeffs[4];
a[2] = forgrid.coeffs[5];

/* Read the image DDR structure
  ----------------------------*/
if(c_getnam(&vblk, "IN", &zero, inimg, "img", &acc) != E_SUCC)
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
if (c_getddr(inimg, &ddr) != E_SUCC) {
   c_errmsg("Projection information not available from DDR", 
	"uptmddr-noproj",&one);
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
   }

/* Get the band DDR info so it doesn't get clobbered later
  -------------------------------------------------------*/
for (i=0; i < ddr.nbands; i++) {
   bndno = i + 1;
   if(c_getbdr(inimg,&bddr[i],&bndno) != E_SUCC) {
      c_errmsg("Band info not available from DDR","uptmddr-bandinfo",&one);
      c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
      }
   }

/* Setup a transformation which maps output line/sample to output projection
   coordinates.
  ------------*/
calc_coefs(&invgrid,pa,pb);

/* Calculate new projection coordinates for the Upper Left image corner.
  ---------------------------------------------------------------------*/
ddr.upleft[0] = calc_corn(1,1,a,b,pa,pb,Y);
ddr.upleft[1] = calc_corn(1,1,a,b,pa,pb,X);

/* Calculate new projection coordinates for the Upper Right image corner.
  ----------------------------------------------------------------------*/
ddr.upright[0] = calc_corn(1,ddr.ns,a,b,pa,pb,Y);
ddr.upright[1] = calc_corn(1,ddr.ns,a,b,pa,pb,X);

/* Calculate new projection coordinates for the Lower Left image corner.
  ---------------------------------------------------------------------*/
ddr.loleft[0] = calc_corn(ddr.nl,1,a,b,pa,pb,Y);
ddr.loleft[1] = calc_corn(ddr.nl,1,a,b,pa,pb,X);

/* Calculate new projection coordinates for the Upper Right image corner.
  ----------------------------------------------------------------------*/
ddr.loright[0] = calc_corn(ddr.nl,ddr.ns,a,b,pa,pb,Y);
ddr.loright[1] = calc_corn(ddr.nl,ddr.ns,a,b,pa,pb,X);

/* Write DDR
  ---------*/
if (c_putddr(inimg, &ddr) != E_SUCC)
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);

/* Put back the band DDR 'cause it just got clobbered 
  --------------------------------------------------*/
for (i=0; i < ddr.nbands; i++) {
   if(c_putbdr(inimg,&bddr[i]) != E_SUCC)
      c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
   }

/* Complete & Exit
  ---------------*/
c_complt("UPTMDDR", " ", &zero);
}




/* Routine to calcluate new corner projection coordinates 
  ------------------------------------------------------*/
double calc_corn(line,sample,a,b,pa,pb,flag)

long line;	/* Line location */
long sample;	/* Sample location */
double *a,*b;	/* Coefs mapping input line/sample to output line/sample */
double *pa,*pb;	/* Coefs mapping output line/sample to projection X/Y */
long flag;	/* Flag for calculating either X or Y projection coordinate */
{
double out_line, out_samp;

out_line = line * b[2] + sample * b[1] + b[0];
out_samp = line * a[2] + sample * a[1] + a[0];
if(flag == Y)return(out_line * pa[0] + out_samp * pa[1] + pa[2]);
return(out_line * pb[0] + out_samp * pb[1] + pb[2]);
}





/* Routine to calculate coefficients mapping form output image line/sample
   to output space projection coordinates
  --------------------------------------*/
calc_coefs(grid,pa,pb)

struct GEOGRID *grid;	/* GRID structure */
double *pa,*pb;		/* Coefs mapping output line/sample to projection X/Y */
{
double corners[8];	/* Projection coordinates of corners */
long imc[8];		/* Image corners */
double cc[6];		/* Temporary coefficient buffer */
long fatal = -1;	/* Fail flag for fatal error message */
long prt_flag = TRUE;

imc[0] = imc[1] = imc[3] = imc[4] = 1;
imc[5] = imc[7] = grid->samples;
imc[6] = imc[2] = grid->lines;
corners[0] = grid->corners[0];
corners[1] = grid->corners[1];
corners[2] = grid->corners[2];
corners[3] = grid->corners[3];
corners[4] = grid->corners[4];
corners[5] = grid->corners[5];
corners[6] = grid->corners[6];
corners[7] = grid->corners[7];
if(c_rtcoef(corners,imc,cc,&prt_flag) != E_SUCC)
   c_errmsg("Fatal error encountered","uptmddr-fatal",&fatal);
pa[0] = cc[0];
pa[1] = cc[1];
pa[2] = cc[2];
pb[0] = cc[3];
pb[1] = cc[4];
pb[2] = cc[5];
return;
}
