/*****************************************************************************
NAME:			    SIATREAD7

PURPOSE:  Reads CCT-X SIAT file, record 7

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CHANGE REQUEST
-------		----	------		--------------
  1.0		3/92	D. Steinwand
  1.1		6/94	S. Nelson	Added a version 2 option for checking
					valid siat file and set fseek to
					read version 2.
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "siat.h"

/* Routine to read CCT-X SIAT file, record 7
  -----------------------------------------*/
siatread7(finame, siat)
char *finame;
SIAT *siat;
{
FILE *fopen(), *fptr;
unsigned char buf[324];
int i;
short get_short_var();
long get_long_var();
short get_bcd();
float get_frac();
struct stat status_buf;

/* Open SIAT file, position to record 7, and read it
  -------------------------------------------------*/
fptr=fopen(finame,"r");
if(!fptr){printf("error opening siat file\n");exit(1);}
stat(finame,&status_buf);
if(status_buf.st_size==3658) fseek(fptr, 2853, 0);
else if(status_buf.st_size==4138) fseek(fptr, 3178, 0);
else if(status_buf.st_size==4168) fseek(fptr, 3208, 0);
else {printf("error reading siat file (unknown size of %d bytes)\n",
   status_buf.st_size);exit(1);}
fread(buf,1,324,fptr);

/* Extract information
  -------------------*/
siat->timesc_ddd = get_bcd(&buf[2])*100+get_bcd(&buf[3]);
siat->timesc_hh = get_bcd(&buf[4]);
siat->timesc_mm = get_bcd(&buf[5]);
siat->timesc_ss = get_bcd(&buf[6]);
siat->timesc_cc = get_bcd(&buf[7]);

siat->gmtsc_ddd = get_bcd(&buf[9])*100+get_bcd(&buf[10]);
siat->gmtsc_hh = get_bcd(&buf[11]);
siat->gmtsc_mm = get_bcd(&buf[12]);
siat->gmtsc_ss = get_bcd(&buf[13]);
siat->gmtsc_mmm = get_bcd(&buf[14])*100+get_bcd(&buf[15]);

for(i=0;i<9;i++)
   {
   siat->norm_alt_change_ic[i] = get_frac(&buf[16+i*2]);
   siat->alt[i] = get_short_var(&buf[34+i*2]) / 32.0;
   }

siat->vehicle_roll_ic = get_frac(&buf[52]);
siat->vehicle_pitch_ic = get_frac(&buf[54]);
siat->vehicle_yaw_ic = get_frac(&buf[56]);

for(i=0;i<11;i++)
   {
   siat->roll[i] = get_frac(&buf[58+i*2]);
   siat->pitch[i] = get_frac(&buf[76+i*2]);
   siat->yaw[i] = get_frac(&buf[94+i*2]);
   }

siat->skew = get_frac(&buf[112]);
siat->norm_vel_change = get_frac(&buf[114]);
siat->mean_pitch = get_long_var(&buf[116])*1e-6;
siat->mean_roll = get_long_var(&buf[120])*1e-6;
siat->mean_yaw = get_long_var(&buf[124])*1e-6;
siat->mean_pitch_rate = get_long_var(&buf[128])*1e-6;
siat->mean_roll_rate = get_long_var(&buf[132])*1e-6;
siat->mean_yaw_rate = get_long_var(&buf[136])*1e-6;
siat->mean_alt = get_long_var(&buf[140]);
siat->mean_alt_rate = get_long_var(&buf[144]);

for(i=0;i<11;i++)
   {
   siat->gmt_ms[i]=get_long_var(&buf[192+i*4]);
   siat->latitude[i]=get_long_var(&buf[192+i*4])*1e-6;
   siat->longitude[i]=get_long_var(&buf[236+i*4])*1e-6;
   siat->latlon_alt[i]=get_long_var(&buf[280+i*4]);
   }
}


/* Prints the contents of the SIAT structure to the standard output device
  -----------------------------------------------------------------------*/
siatrpt7(siat)
SIAT *siat;
{
printf("Spacecraft time at scene center:  %3d:%2d:%2d:%2d:%2d\n", 
        siat->timesc_ddd, siat->timesc_hh, siat->timesc_mm, siat->timesc_ss, 
        siat->timesc_cc);
printf("GMT of scene center:              %3d:%2d:%2d:%2d:%3d\n\n",
        siat->gmtsc_ddd, siat->gmtsc_hh, siat->gmtsc_mm, siat->gmtsc_ss,
        siat->gmtsc_mmm);

printf("Normalized altitude change at times:\n");
printf("  image center - 13.80300:  %f\n", siat->norm_alt_change_ic[0]);
printf("  image center - 10.35225:  %f\n", siat->norm_alt_change_ic[1]);
printf("  image center -  6.90150:  %f\n", siat->norm_alt_change_ic[2]);
printf("  image center -  3.45075:  %f\n", siat->norm_alt_change_ic[3]);
printf("  image center time:        %f\n", siat->norm_alt_change_ic[4]);
printf("  image center +  3.45075:  %f\n", siat->norm_alt_change_ic[5]);
printf("  image center +  6.90150:  %f\n", siat->norm_alt_change_ic[6]);
printf("  image center + 10.35225:  %f\n", siat->norm_alt_change_ic[7]);
printf("  image center + 13.80300:  %f\n\n", siat->norm_alt_change_ic[8]);

printf("Altitude (N.M.) at times:\n");
printf("  image center - 13.80300:  %f\n", siat->alt[0]);
printf("  image center - 10.35225:  %f\n", siat->alt[1]);
printf("  image center -  6.90150:  %f\n", siat->alt[2]);
printf("  image center -  3.45075:  %f\n", siat->alt[3]);
printf("  image center time:        %f\n", siat->alt[4]);
printf("  image center +  3.45075:  %f\n", siat->alt[5]);
printf("  image center +  6.90150:  %f\n", siat->alt[6]);
printf("  image center + 10.35225:  %f\n", siat->alt[7]);
printf("  image center + 13.80300:  %f\n\n", siat->alt[8]);

printf("Vehicle Roll at image center (rad):  %f\n", siat->vehicle_roll_ic);
printf("Vehicle Pitch at image center (rad): %f\n", siat->vehicle_pitch_ic);
printf("Vehicle Yaw at image center (rad):   %f\n\n", siat->vehicle_yaw_ic);
printf("Roll at times:\n");
printf("  image center - 13.80300:  %f\n", siat->roll[0]);
printf("  image center - 10.35225:  %f\n", siat->roll[1]);
printf("  image center -  6.90150:  %f\n", siat->roll[2]);
printf("  image center -  3.45075:  %f\n", siat->roll[3]);
printf("  image center time:        %f\n", siat->roll[4]);
printf("  image center +  3.45075:  %f\n", siat->roll[5]);
printf("  image center +  6.90150:  %f\n", siat->roll[6]);
printf("  image center + 10.35225:  %f\n", siat->roll[7]);
printf("  image center + 13.80300:  %f\n\n", siat->roll[8]);
printf("Pitch at times:\n");
printf("  image center - 13.80300:  %f\n", siat->pitch[0]);
printf("  image center - 10.35225:  %f\n", siat->pitch[1]);
printf("  image center -  6.90150:  %f\n", siat->pitch[2]);
printf("  image center -  3.45075:  %f\n", siat->pitch[3]);
printf("  image center time:        %f\n", siat->pitch[4]);
printf("  image center +  3.45075:  %f\n", siat->pitch[5]);
printf("  image center +  6.90150:  %f\n", siat->pitch[6]);
printf("  image center + 10.35225:  %f\n", siat->pitch[7]);
printf("  image center + 13.80300:  %f\n\n", siat->pitch[8]);
printf("Yaw at times:\n");
printf("  image center - 13.80300:  %f\n", siat->yaw[0]);
printf("  image center - 10.35225:  %f\n", siat->yaw[1]);
printf("  image center -  6.90150:  %f\n", siat->yaw[2]);
printf("  image center -  3.45075:  %f\n", siat->yaw[3]);
printf("  image center time:        %f\n", siat->yaw[4]);
printf("  image center +  3.45075:  %f\n", siat->yaw[5]);
printf("  image center +  6.90150:  %f\n", siat->yaw[6]);
printf("  image center + 10.35225:  %f\n", siat->yaw[7]);
printf("  image center + 13.80300:  %f\n\n", siat->yaw[8]);
printf("Image skew (rad):  %f\n\n", siat->skew);
printf("Normalized velocity change:  %f\n\n", siat->norm_vel_change);
printf("Mean pitch (rad):  %f\n", siat->mean_pitch);
printf("Mean roll (rad):   %f\n", siat->mean_roll);
printf("Mean yaw(rad):     %f\n\n", siat->mean_yaw);
printf("Mean pitch rate (rad/sec):  %f\n", siat->mean_pitch_rate);
printf("Mean roll rate (rad/sec):   %f\n", siat->mean_roll_rate);
printf("Mean yaw rate (rad/sec):    %f\n\n", siat->mean_yaw_rate);
printf("Mean Mean altitude (meters):      %f\n", siat->mean_alt);
printf("Mean altitude rate (meters/sec):  %f\n\n", siat->mean_alt_rate);
printf("GMT milliseconds of Day at:\n");
printf("  image center time - 25 sec:  %d\n", siat->gmt_ms[0]);
printf("  image center time - 20 sec:  %d\n", siat->gmt_ms[1]);
printf("  image center time - 15 sec:  %d\n", siat->gmt_ms[2]);
printf("  image center time - 10 sec:  %d\n", siat->gmt_ms[3]);
printf("  image center time -  5 sec:  %d\n", siat->gmt_ms[4]);
printf("  image center time:           %d\n", siat->gmt_ms[5]);
printf("  image center time +  5 sec:  %d\n", siat->gmt_ms[6]);
printf("  image center time + 10 sec:  %d\n", siat->gmt_ms[7]);
printf("  image center time + 15 sec:  %d\n", siat->gmt_ms[8]);
printf("  image center time + 20 sec:  %d\n", siat->gmt_ms[9]);
printf("  image center time + 25 sec:  %d\n\n", siat->gmt_ms[10]);
printf("Latitude (rad) at times of GMT:\n");
printf("  image center time - 25 sec:  %f\n", siat->latitude[0]);
printf("  image center time - 20 sec:  %f\n", siat->latitude[1]);
printf("  image center time - 15 sec:  %f\n", siat->latitude[2]);
printf("  image center time - 10 sec:  %f\n", siat->latitude[3]);
printf("  image center time -  5 sec:  %f\n", siat->latitude[4]);
printf("  image center time:           %f\n", siat->latitude[5]);
printf("  image center time +  5 sec:  %f\n", siat->latitude[6]);
printf("  image center time + 10 sec:  %f\n", siat->latitude[7]);
printf("  image center time + 15 sec:  %f\n", siat->latitude[8]);
printf("  image center time + 20 sec:  %f\n", siat->latitude[9]);
printf("  image center time + 25 sec:  %f\n\n", siat->latitude[10]);
printf("Longitude (rad) at times of GMT:\n");
printf("  image center time - 25 sec:  %f\n", siat->longitude[0]);
printf("  image center time - 20 sec:  %f\n", siat->longitude[1]);
printf("  image center time - 15 sec:  %f\n", siat->longitude[2]);
printf("  image center time - 10 sec:  %f\n", siat->longitude[3]);
printf("  image center time -  5 sec:  %f\n", siat->longitude[4]);
printf("  image center time:           %f\n", siat->longitude[5]);
printf("  image center time +  5 sec:  %f\n", siat->longitude[6]);
printf("  image center time + 10 sec:  %f\n", siat->longitude[7]);
printf("  image center time + 15 sec:  %f\n", siat->longitude[8]);
printf("  image center time + 20 sec:  %f\n", siat->longitude[9]);
printf("  image center time + 25 sec:  %f\n\n", siat->longitude[10]);
printf("Altitude (meters) at times of GMT:\n");
printf("  image center time - 25 sec:  %f\n", siat->latlon_alt[0]);
printf("  image center time - 20 sec:  %f\n", siat->latlon_alt[1]);
printf("  image center time - 15 sec:  %f\n", siat->latlon_alt[2]);
printf("  image center time - 10 sec:  %f\n", siat->latlon_alt[3]);
printf("  image center time -  5 sec:  %f\n", siat->latlon_alt[4]);
printf("  image center time:           %f\n", siat->latlon_alt[5]);
printf("  image center time +  5 sec:  %f\n", siat->latlon_alt[6]);
printf("  image center time + 10 sec:  %f\n", siat->latlon_alt[7]);
printf("  image center time + 15 sec:  %f\n", siat->latlon_alt[8]);
printf("  image center time + 20 sec:  %f\n", siat->latlon_alt[9]);
printf("  image center time + 25 sec:  %f\n\n", siat->latlon_alt[10]);
return;
}

short get_short_var(val) short *val; { return(*val); }
long get_long_var(val) long *val; { return(*val); }
short get_bcd(val) unsigned char *val;{return((*val&0x0f)+(*val>>4&0x0f)*10);}

float get_frac(val) short *val; { return(*val*7.62939453e-6); }
