/*****************************************************************************
NAME:			    SIAT.H

PURPOSE:  Include file defining structure for SIAT file, record 7

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		
-------		----	------	
  1.0		3/92	D. Steinwand
*****************************************************************************/
typedef struct
   {
   short	timesc_ddd;		/* Spacecraft time at scene center */
   short 	timesc_hh;		/* Spacecraft time at scene center */
   short 	timesc_mm;		/* Spacecraft time at scene center */
   short 	timesc_ss;		/* Spacecraft time at scene center */
   short 	timesc_cc;		/* Spacecraft time at scene center */
   short    	gmtsc_ddd;		/* GMT of scene center */
   short    	gmtsc_hh;		/* GMT of scene center */
   short    	gmtsc_mm;		/* GMT of scene center */
   short    	gmtsc_ss;		/* GMT of scene center */
   short    	gmtsc_mmm;		/* GMT of scene center */
   float	norm_alt_change_ic[10];	/* Normalized altitude change at times:
					    [0]:  image center - 13.80300
					    [1]:  image center - 10.35225 
					    [2]:  image center -  6.90150
					    [3]:  image center -  3.45075
					    [4]:  image center time
					    [5]:  image center +  3.45075
					    [6]:  image center +  6.90150
					    [7]:  image center + 10.35225
					    [8]:  image center + 13.80300 */
   float 	alt[10];		/* Altitude (N.M.) at times above */
   float 	vehicle_roll_ic;	/* Vehicle Roll at image center (rad)*/
   float 	vehicle_pitch_ic;	/* Vehicle Pitch at image center (rad)*/
   float 	vehicle_yaw_ic;		/* Vehicle Yaw at image center (rad)*/
   float	roll[10];		/* Roll at times above (rad)*/
   float	pitch[10];		/* Pitch at times above (rad)*/
   float	yaw[10];		/* Yaw at times above (rad)*/
   float	skew;			/* Image skew (rad) */
   float 	norm_vel_change;	/* Normalized velocity change */
   float 	mean_pitch;		/* Mean pitch (rad) */
   float	mean_roll;		/* Mean roll (rad) */
   float 	mean_yaw;		/* Mean yaw (rad) */
   float	mean_pitch_rate;	/* Mean pitch rate (rad/sec) */
   float	mean_roll_rate;		/* Mean roll rate (rad/sec) */
   float 	mean_yaw_rate;		/* Mean yaw rate (rad/sec) */
   float 	mean_alt;		/* Mean altitude (meters) */
   float 	mean_alt_rate;		/* Mean altitude rate (meters/sec) */
   long		gmt_ms[12];		/* GMT milliseconds of Day at:
					    [0]:  image center time - 25 sec
					    [1]:  image center time - 20 sec
					    [2]:  image center time - 15 sec
					    [3]:  image center time - 10 sec
					    [4]:  image center time -  5 sec
					    [5]:  image center time 
					    [6]:  image center time +  5 sec
					    [7]:  image center time + 10 sec
					    [8]:  image center time + 15 sec
					    [9]:  image center time + 20 sec
					    [10]: image center time + 25 sec */
   float	latitude[12];		/* Latitude (rad) at times of gmt_ms */
   float	longitude[12];		/* Longitude(rad) at times of gmt_ms */
   float	latlon_alt[12];		/* Alt (meters) at times of gmt_ms */
   } SIAT;
