#!/bin/csh -f
#
# this file should be sourced with the $EDCSOFT environment variable set to
# the install directory.  

if ($#argv == 1) then
    setenv EDCSOFT $1
else if (! $?EDCSOFT) then
    if (($#argv < 1) || ($#argv > 2)) then
        echo "   Usage: lasenv.csh <install_path>"
        echo ""
        echo "          where <install_path> is the full directory path to"
        echo "          use for the installation (overrides default path)."
        echo ""
        echo "          Example:  source lasenv.csh /home/las"
        exit 1
    endif
endif   

if (! -d $EDCSOFT) then
    echo "  Error: Install path $EDCSOFT does not exist"
    exit 1
endif

set assign="$EDCSOFT/env/las/assign"
if (! -d $assign) then
    echo "  Error: Assign path $assign does not exist"
    exit 1
endif

#
# Reduce path and environment variables to a more or less pristine state,
# similar to what a system might see when first installing LAS.
#
# Determine the platform type.
#
set tmp = `uname -a | awk '{print $1}' | tr "[A-Z]" "[a-z]"`
switch ($tmp)
    case irix*:
        set path = (. /usr/local/bin /usr/etc /bin /lib /usr/bin /usr/bin/X11 \
                    /usr/bsd /usr/local /usr/sbin /etc /usr/local/gnu/bin)
        breaksw

    case sunos:
        set path = (. /lib /usr/local/bin /usr/local/gnu/bin /opt/SUNWspro/bin \
                    /usr/openwin/bin /usr/local /usr/ccs/bin /usr/ccs/lib /bin \
                    /usr/bin /usr/bin/X11)

        breaksw

    case linux:
        set path = (. /usr/bin /bin /usr/X11R6/bin /usr/local/bin /opt/bin \
                    /usr/X11R6/bin $HOME/bin)
        breaksw

    default:
        set path = (/usr/local/bin /bin /usr/bin /usr/X11/bin /usr/sbin)
endsw
#

unsetenv CCMD

unset response

#----
# TAE
#----
unsetenv TAE
unsetenv TAEADA
unsetenv TAEADALIB
unsetenv TAEBIN
unsetenv TAEBITMAPS
unsetenv TAECXXLIB
unsetenv TAEDEMO
unsetenv TAEDEMOADA
unsetenv TAEDEMOBIN
unsetenv TAEDEMOCLASSIC
unsetenv TAEDEMOHELP
unsetenv TAEDEMOPS
unsetenv TAEDEMORES
unsetenv TAEDEMORFG
unsetenv TAEDEMOSRC
unsetenv TAEHELP
unsetenv TAEHELPINX
unsetenv TAEHELPMSG
unsetenv TAEHELPTM
unsetenv TAEHELPWB
unsetenv TAEINC
unsetenv TAEINCXM
unsetenv TAELIB
unsetenv TAELOG
unsetenv TAEMAN
unsetenv TAEMENU
unsetenv TAEPDF
unsetenv TAEPERLLIB
unsetenv TAEPLAT
unsetenv TAERES
unsetenv TAEXTAELIB
unsetenv ULIB

#-------------------------------------------------------------------
# root directory for CVS controlled source code
# (Set to garbage because if unset, any process spawning a new shell
# will rerun the .cshrc and reset it.
#-------------------------------------------------------------------

unsetenv LASSYS
#------------------------
# ADAPS Environment Paths
#------------------------
unsetenv ADAPSTABLES

#-----------------------
# ADAPS Executable Paths
#-----------------------
unsetenv ADAPSBASE
unsetenv ADAPSCONTRIB
unsetenv ADAPSDOC
unsetenv ADAPSLOCAL

unsetenv ADAPSACQ
unsetenv ADAPSARC
unsetenv ADAPSBRW
unsetenv ADAPSDB
unsetenv ADAPSDBTMP
unsetenv ADAPSERR
unsetenv ADAPSFAX
unsetenv ADAPSIMG
unsetenv ADAPSING
unsetenv ADAPSLOG
unsetenv ADAPSMIC
unsetenv ADAPSORB
unsetenv ADAPSPROD
unsetenv WDBII
unsetenv AIRTABLES
unsetenv AIRLOG

#--------------------------
# AIRCRAFT Executable Paths
#--------------------------
unsetenv AIRBASE

unsetenv AIRDOC

#----------------------
# LAS Environment Paths
#----------------------
unsetenv APPLOG
unsetenv LASASSIGN
unsetenv LASTABLES
unsetenv WORCONFIG

#---------------------
# LAS Executable Paths
#---------------------
unsetenv LASBASE
unsetenv LASCONTRIB
unsetenv LASDITTY
unsetenv LASDITTO
unsetenv LASDOC
unsetenv LASLOCAL
unsetenv LASBIN
unsetenv LASDDP
unsetenv LASTOOLS

#------------------------------------------
# Environmet variable for drainage software
#------------------------------------------
unsetenv LASDRAIN

#----------------------------
# stuff from pgmassign
#----------------------------
unsetenv ADAPSINCLUDE
unsetenv ADAPSLIB
unsetenv LASINCLUDE
unsetenv LASLIB
unsetenv LASPUBLIC
unsetenv AIRINCLUDE
unsetenv AIRLIB
unsetenv CONTRIBINCLUDE
unsetenv CONTRIBLIB

unsetenv SYSDEF

unsetenv LASNETWORK
unsetenv LAS_RESP
#
unsetenv TPLIB
unsetenv TRLIB
unsetenv PDSLIB
unsetenv PDSINCLUDE
unsetenv CAGIS
unsetenv CAGISBASE
#
unsetenv CC
unsetenv FC
unsetenv FLINKS
unsetenv CFLAGS
unsetenv FFLAGS
unsetenv AR
unsetenv ARFLAGS
unsetenv CXX
unsetenv CXXFLAGS

#
# Run the masterlog and set the path appropriately for this system.
#
eval `$assign/masterlog response comfile las74d $EDCSOFT`
set path=($path $EDCSOFT/run/bin)
