/**************************************************************************

NAME:   ARCH8MM

PURPOSE: Archive AVHRR scenes to 8mm, and write scene information to 
	 the passrec/8mm directory's tapeid file 

PROGRAM HISTORY:
  Version       Date        Author          Reason
  --------      ----        -----------     ----------
     0.0        04/96       K. Lowell       Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Runs under TAE/UNIX

PROJECT:  LAS

ALGORITHM DESCRIPTION: See inline comments

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "protos.h"


int main
(void)
{
struct PARBLK parblk;		/* TAE parameter block   		*/
 				   
static char sceneid[CMLEN];        /* input sceneid                 */
char drive[100];		    /* 8mm tape drive name	*/

FILE *fp;			/* file pointer				*/
char scenei[CMLEN];		/* ANSI scene data		*/
char sceneh[CMLEN];		/* ANSI scene header		*/
char scene[CMLEN];		/* partial sceneid		*/
char temp[80];			/* temp file buffer	*/
char file[CMLEN];		/* output tape listing file		*/
char year[4];			/* year string		*/
char label[20];			/* tape label				*/
char proot[CMLEN];		/* system passrec root			*/
long ii=0;                        /* Number of lines 			*/
long jj=0;                        /* Number of samples 			*/  
long mode = 0;			/* flag value for lsstat		*/

char callstr[CMLEN];   		/* Data type				*/

long fatal = -1;		/* fatal error flag for errmsg		*/
long std_out=0; 		/* logical unit for standard ouput   	*/
long status=0;		/* status flag				*/
long count = 18;	/* Nominal number of AVHRR scenes per tape	*/

/* Initialize TAE parameter block
----------------------------------*/
c_inlas(&parblk,&std_out);

/* Get LAS parameters (if needed)
---------------------------------*/
if ((getpar(&parblk,sceneid,drive,&count))
   != E_SUCC)
   {
   c_errmsg("Fatal error encountered","ndf2las-fatal",&fatal);
   }

/* get the passrec root directory
------------------------------------*/
strcpy(proot,getenv("PROOT"));
 

/* make the file name, and capture file name if it exists 
-----------------------------------------------------------*/
strncpy(year,&sceneid[8],2);
for (ii=1; ; ii++)
  {
  sprintf(file,"%s%s%s%s%d%s",proot,"/8mm/a",year,"_",ii,".cur");
  status = c_lsstat(file,&mode);
  if (status == E_SUCC)
    {
    if ((fp=fopen(file,"r+")) == NULL)
      c_errmsg("Error opening tape file","arch8mm-open",&fatal);
    break;
    }
  }

sprintf(label,"%s%s%s%d","a",year,"_",ii); 

strncpy(scene,&sceneid[1],15);
scene[15]='\0';
sprintf(scenei,"%s%s","i",scene);
sprintf(sceneh,"%s%s","h",scene);

for (jj=0;jj<19;jj++)
  {
  if ((status = fscanf(fp,"%s",temp)) != 1) break;
  }

sprintf(callstr,"%s %s%s%s%s %s","mv",proot,"/archive/",sceneid,".ahdr",sceneh);
system(callstr);
sprintf(callstr,"%s %s%s%s%s %s","mv",proot,"/archive/",sceneid,".arch",scenei);
system(callstr);

if ( jj == 0)
  {
  sprintf(callstr,"%s %s %s %s %d %s %s","ansiout -TvFbdrftR",drive,
          label,"80 0 80", &jj,"9999", sceneh);
  system (callstr);
  sprintf(callstr,"%s %s %s %s %d %s %s","ansiout -ATvFbdrftR",drive,
          label,"27592 0 13796", &jj,"9999", scenei);
  system (callstr); 
  }
else if (jj <= count - 2)
  {
  sprintf(callstr,"%s %s %s %s %d %s %s","ansiout -aTvFbdrftR",drive,
          label,"80 0 80",&jj,"9999", sceneh);
  system (callstr); 
  sprintf(callstr,"%s %s %s %s %d %s %s","ansiout -ATvFbdrft",drive,
          label,"27592 0 13796",&jj,"9999", scenei);
  system (callstr);
  } 
else
  {
  sprintf(callstr,"%s %s %s %s %d %s %s","ansiout -aTvFbdrftR",drive,
          label,"80 0 80",&jj,"9999", sceneh);
  system (callstr); 
  sprintf(callstr,"%s %s %s %s %d %s %s","ansiout -ATvFbdrft",drive,
          label,"27592 0 13796",&jj,"9999", scenei);
  system (callstr);
  }



/* put the files back with original names
-------------------------------------------*/
sprintf(callstr,"%s %s %s%s%s%s","mv",sceneh, proot,"/archive/",sceneid,".ahdr");
system(callstr);
sprintf(callstr,"%s %s %s%s%s%s","mv",scenei, proot,"/archive/",sceneid,".arch");
system(callstr);


/* put the sceneid into the file
----------------------------------*/
sprintf(callstr,"%s %s %s %s","echo",sceneid, ">>",file);
system(callstr);
ii = ii + 1;
if (ii == count)
  {
  sprintf(callstr,"%s %s %s %s%s%s%d%s","mv",file, proot,"/8mm/a",
          year,"_",ii,".full");
  system(callstr);
  }

return EXIT_SUCCESS;

}
