/***************************************************************************
NAME      		  CONCAT2

PURPOSE:  To provide a means of inserting rectangular groups of
	  pixels into a region of an image.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
E.HARRIS	JAN  1982	Original development
N.DEODHAR	SEPT 1987	Fixed windowing problem
R.MCKINNEY      SEPT 1988       Conversion to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   CONCAT2 must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)

   The LAS program is initialized.  Next the parameters are
   retrieved thru GETPAR.  The main processing routine is called
   for the correction of rectangular regions of an image.  Finally,
   history file is updated and the routine completed.
ALGORITHM REFERENCES     

   NONE
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC * fdesc[MAXIMG+1]; /* image file descriptors used for image I/O */
struct FDESC * fdesc1;          /* image file descriptors used for image I/O */
struct GDESC * gdesc;		    /* group descriptor used for image I/O       */
extern long curent;		        /* current processing line		     */
extern long curcat;		        /* current processing band		     */

int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		     */

static long bands[MAXIMG][MAXBND + 1];
                                /* band specification for first image        */
static long bands2[MAXIMG][MAXBND + 1];
                                /* band specification for first image        */
long msgtim;			/* elpased time between display messages     */
long nmrimg;			/* number of input images for image          */
long nbands[MAXIMG];		/* number of bands per input image           */
long one = 1;			/* number one				     */
long pixval;		        /* pixel values for constant option	     */
long sline;
long ssample;
long std_out;			/* logical unit # for standard output        */
long totbnd;			/* total number of bands in first image      */
long window[MAXIMG][4];		/* window specification for first image      */
long window1[MAXIMG][4];	/* window specification for first image      */

char	hosin[MAXIMG][CMLEN];	/* buffer storage        	   	     */
char    hosout[1][CMLEN];          /* buffer storage        		     */
char    minmax[4];	        /* minmax flag buffer 			     */
char    option[9];		/* option of processing			     */
char	taein[MAXIMG][CMLEN];	/* buffer storage       		     */
char    taeout[1][CMLEN];          /* buffer storage       		     */


c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/*
       get user-specified parameters and TAE globals
*/
getpar(&vblock, taein, hosin, taeout, hosout,
       &nmrimg, bands, nbands, window, window1, &msgtim, 
       minmax, &totbnd,option,&pixval,&sline,&ssample,bands2);
/*
   perform the rectangular correction
*/
  if (strcmp(option,"LEAVE") == 0)
  leave(hosin,hosout[0],nmrimg,bands,
	window,msgtim,totbnd,pixval,sline,ssample,bands2); 
  else if (strcmp(option,"REPLACE") == 0)
  replace(hosin,hosout[0],nmrimg,bands,
	window,msgtim,totbnd,pixval,sline,ssample,bands2); 
  else if (strcmp(option,"MAX") == 0)
  maxpix(hosin,hosout[0],nmrimg,bands,
	window,msgtim,totbnd,sline,ssample,bands2); 
  else if (strcmp(option,"MIN") == 0)
  minpix(hosin,hosout[0],nmrimg,bands,
	window,msgtim,totbnd,sline,ssample,bands2); 
/*
   create associated history file
*/
/*
if (c_hstout(&vblock, hosout[0], hosin[0] , &nmrimg, " ") != E_SUCC)
   c_errmsg("Nonfatal error encountered -- Image data saved",
	    "concat2-warn",&nonfatal);
*/
/*
   inform user of successful completion
*/
c_complt("CONCAT2", taeout, &one);

return EXIT_SUCCESS;

}
