/*******************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the concat2 function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      May  1988       converted to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	       LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   The global parameters are first received.
   Ne[i]t the input and output images are gotten.
   Various validity checks are then made before returning.
*******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,      /* pointer to TAE block	 	      */
    char taein[][CMLEN],	/* TAE file names of input images	      */
    char hosin[][CMLEN],  	/* host file names of input images	      */
    char taeout[][CMLEN],		/* TAE file name of output image	      */
    char hosout[][CMLEN],		/* host file name of output image	      */
    long *nmrimg, 		 /* number of input images		      */
    long (*bands)[MAXBND+1],     /* band specification 		              */
    long *nbands,		 /* number of bands per input image           */
    long (*window)[4],	         /* window specification		      */
    long (*window1)[4],	         /* window specification		      */
    long *msgtim,		 /* elapsed time between display messages     */
    char *minmax,		 /* min max flag buffer		              */
    long *totbnd,		 /* total number of bands		      */
    char *option,		/* replacement option		              */
    long *pixval,		 /* pixel values for insertion		      */
    long *sline,
    long *ssample,
    long (*bands2)[MAXBND+1]    /* band specification 		              */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block    */

long     element = 0;		 /* parameter element number		      */
long     fatal = -1;		 /* fatal error flag			      */
long     imglim;		 /* max number of images to be processed      */
long     nonfatal = 1;		 /* value of one			      */
long	 odtype = 0;		 /* data type of input image		      */
long     qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified   */
long     same;		    	 /* flag indicating if images have same window*/
long     status;	   	 /* function return status code		      */
 
/*
GETPAR retreives the required TAE parameters from the TAE parameter block.

The TAE local parameters used by the program are:
    1.  In--the input TAE image name
    2.  Out--the output TAE image name
    3.  Overopt--replacement option
    4.  Pixval--pixel values used for replacement
    5.  Inwind--window specifications

The TAE global parameters used by the program are:
    1.  Histry--a flag specifying if the history file should be updated or not
    2.  Minmax--a flag specifying if the minmax file should be updated or not
    3.  Msgtim--the amount of time between display messages

This subroutine then figures the center of the image and checks to verify that
the window lies within the image.  Finally, it calculates the size of the output
image.
*/

/* Retrieve the TAE global variables from the parameter block and store them
   in their respective program variables. 
-----------------------------------------------------------------------------*/

/* Build the string descriptor for TAE variable "IN".
   Call getin to parse the input image specification.
   Getin returns the TAE image name(s), host image name(s),
   bands to be processed, window specification, and number
   of images parsed.
----------------------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"UPDATE",&element,taeout,hosout,nmrimg,window1,bands2,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","concat2-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
   }

status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","concat2-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
   }

/*
  check the data type
*/
 status = c_gettyp(hosin,nmrimg,&odtype);
    if (status != E_SUCC)
        {
        c_errmsg("Input images do not have same data type","concat2-type",
		&nonfatal);
	c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
 	}
 if (odtype != EBYTE && odtype != EWORD)
   {
	c_errmsg(
   "Invalid input pixel type, only BYTE and INTEGER*2 are acceptable",
   "concat2-type",&nonfatal);
   c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
   }
/*
  get the option
*/
  val = p_find(vblock,"overopt");
  strcpy(option, SVAL(*val,0));
/*
  get the pixel values for insertion
*/
  val = p_find(vblock,"BACKGRND");
    *pixval = IVAL(*val,0);
/*
  get the starting line
*/
  val = p_find(vblock,"SL");
    *sline = IVAL(*val,0);
/*
  get the starting sample
*/
  val = p_find(vblock,"SS");
    *ssample = IVAL(*val,0);

/* Retrieve the TAE global variables from the parameter block and store them
   in their respective program variables. 
-----------------------------------------------------------------------------*/

val = p_find(vblock, "$msgtim");
*msgtim = IVAL(*val, 0);

val = p_find(vblock, "$minmax");
strcpy(minmax, SVAL(*val,0));
return;
}
