/******************************************************************************
NAME:   		      MAXPIX

PURPOSE    To fix bad rectangular areas within an image. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   E. Harris	   1/82		 SASC		Original development
  4.0	   N. Deodhar	   9/87		  SAR	        Fixed windowing
  5.0	   R. McKinney     9/88		  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.0	   T. Baltzer      11/89	  EDC		Add checks for the size
							of the repair window
							relative to the image
							size.
  5.0	   J. LaVergne     03/91	  EDC		Added MIN/MAX options
                                                        to OVEROPT

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MAXPIX must be run under TAE.

PROJECT			LAS

ALGORITHM:

	The input data type is retrieved thru GETTYP
	Both the input and output images are opened
	The I/O buffer areas are allocated dynamically
	The input image is copied to the output image
	Rectangular windows specified for correcting is next done
	   Random I/O is used so only the areas needing fixing
	     is read.
	   All the windows are corrected 
 	The DDR file is updated
	The minimum and maximum values are stored if desired 
	The files are closed
	The buffer areas are freed   

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[];	/* image file descriptors used for image I/O */
extern struct FDESC * fdesc1;	/* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	/* group descriptor used for image I/O       */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

void maxpix
(
    char hosin[][CMLEN],		/* host names of input images		     */
    char *hosout,		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long (*window)[4],		/* window specification of input images      */
    long msgtim,			/* elapsed time between display messages     */
    long totbnd,			/* total number of bands to be processed     */
    long sline,
    long ssample,
    long (*bands2)[MAXBND+1]	/* bands to be processed for all images      */
)

{

long	access;			/* image I/O access type                     */
long    fatal = -1;		/* fatal error flag			     */
long    j;			/* just a counter			     */
long	image;			/* loop index counting input images          */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long    nline;
long    nsample;
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    odtype = 0;		/* data type				     */
long    one = 1;		/* one					     */
long    online;
long    sample;			/* sample indicator			     */
long	size = 0;			/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long	zero = 0;		/* value of zero                             */


unsigned char    *thebuf;       /* image input buffer                        */
unsigned char    *thebuf1;       /* image update buffer                      */

/*
   get the data type of the input image
---------------------------------------*/

      status = c_gettyp(hosin,&nmrimg,&odtype);
      if (status != E_SUCC)
          c_errmsg("Fatal error encountered","concat2-fatal",&fatal);

	switch(odtype)
	{
	case EBYTE:  	size = 1; break;
	case EWORD:	size = 2; break;
	case ELONG:	size = 4; break;
	case EREAL:	size = 4; break;
	}

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];
nline = nl;
nsample = ns;
for (image=0;image<nmrimg;image++)
{
sl = window[image][0];
ss = window[image][1];

   status = c_eopenr(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &access,&odtype,&zero,&one);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
}
fdesc[1] = 0;

/* Allocate I/O buffer
----------------------*/

if ((thebuf = (unsigned char *) malloc(ns*size)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","concat2-alloc",&nonfatal);
   if (c_eclose(&fdesc[0]) != E_SUCC)
     c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
   }


/* Open update image
--------------------*/
access = IUPDATE;	
/*
nl = window1[0][2];
ns = window1[0][3];
sl = window1[0][0];
ss = window1[0][1];
*/
nl = nline;
ns = nsample;
sl = sline;
ss = ssample;
for (image=0;image<nmrimg;image++)
{
status = c_eopenr(&fdesc1,hosout,bands2[image],&sl,&ss,&nl,&ns,
         &access,&odtype,&zero,&one);
if (status != E_SUCC)
     c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
}

/* Allocate I/O buffer
----------------------*/

if ((thebuf1 = (unsigned char *) malloc(ns*size)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","concat2-alloc",&nonfatal);
   if (c_eclose(&fdesc1) != E_SUCC)
     c_errmsg("Fatal error encountered","concat2-fatal",&fatal);
   }


zero = 0;
online = nline;
c_inittm("concat2","Processed","line","bands"," ",
         &online,&totbnd,&msgtim,&msg_flag);
curent = 1;
c_startm();

/*
     process the image windows to be fixed
------------------------------------------*/
image = 0;
  for (curcat = 0; curcat < totbnd; curcat++)
   {
/*	process byte data     */
     j = 0;
/*
     for (curent = sline; curent < nline+sline; curent++)
*/
     for (curent = 1; curent <= nline; curent++)
      {
      j += 1;
	        status = c_eread(&fdesc[image], &bands[image][curcat],
			 &j, thebuf, &one);
	        if (status == E_FAIL)
	           {
	           c_errmsg("Error reading input image","concat2-read",
	        	&nonfatal);
                      if (c_eclose(&fdesc1) != E_SUCC)
                          c_errmsg("Fatal error encountered",
                                   "concat2-fatal",&fatal);
	           }
	        status = c_eread(&fdesc1, &bands2[image][curcat],
			 &curent, thebuf1, &one);
	        if (status == E_FAIL)
	           {
	           c_errmsg("Error reading update image","concat2-read",
	        	&nonfatal);
                      if (c_eclose(&fdesc1) != E_SUCC)
                          c_errmsg("Fatal error encountered",
                                   "concat2-fatal",&fatal);
	           }
          for (sample = 0; sample < nsample; sample++)
              {
              if (*(thebuf+sample) > *(thebuf1+sample) )
              *(thebuf1+sample) = *(thebuf+sample);
              }
                
/*
    write out the lines with fixed windows
------------------------------------------*/
/*
                if (strcmp(minmax,"YES") == 0)
                status=c_pxmnmx(thebuf1,&ns,
                  &minval[curcat],&maxval[curcat],&odtype);
*/
                status = c_ewrite(&fdesc1, &bands2[image][curcat],
                       &curent, thebuf1, &one);
                if (status == E_FAIL)
		{
	        c_errmsg("Error writing output image","concat2-write",
			&nonfatal);
                      if (c_eclose(&fdesc1) != E_SUCC)
                          c_errmsg("Fatal error encountered",
                                   "concat2-fatal",&fatal);
	        }
     }   /* end of curlin loop    */
   }     /* end of curcat loop    */
/* Turn off display messages
----------------------------*/
c_stoptm();
/* Update the DDR
-----------------*/
/*
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = COMB;
status = c_upddr(vblock,hosin[0],hosout,&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
      {
      if (c_eclose(&fdesc1) != E_SUCC)
          c_errmsg("Fatal error encountered",
                   "concat2-fatal",&fatal);
      }
*/
/* Create associated min/max file
---------------------------------*/
/*
if (c_svmnmx(hosout,&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","concat2-warn",&one);
*/
/* Close all image files in the group
-------------------------------------*/
      if (c_eclose(&fdesc1) != E_SUCC)
          c_errmsg("Nonfatal error encountered",
                   "concat2-warn",&nonfatal);

      if (c_eclose(&fdesc[0]) != E_SUCC)
          c_errmsg("Nonfatal error encountered",
                   "concat2-warn",&nonfatal);

/* Free buffer
--------------*/
free(thebuf); 
free(thebuf1);

return;
}
