#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"

/* Report destinations
  -------------------*/
#define ALL	100
#define TERM	200
#define PFILE	300

/* Processing modes
  ----------------*/
#define PROJ_ONLY	101
#define PROJ_GRID	102
#define POLYNOMIAL	103
#define SUBSET          104

/* Frame modes
  -----------*/
#define INPUT		501
#define OUTPUT_LS	502
#define OUTPUT_XY	503

/* Misc constants
  --------------*/
#define NULL_PROJ	999

#define COORS		301
#define IMAGE_COORS	302

#define NEG_PI	       -3.14159265359
#define HALF_PI		1.5707963268
#define NEG_HALF_PI    -1.5707963268

/* Misc macros
  -----------*/
#define DMOD(A,B)	(A) - ((long)((A) / (B)) * (B))

/* Structure containing input and output projection information
  ------------------------------------------------------------*/
struct PRO
   {
   long in_proj;	/* Input projection code */
   long in_zone;	/* Input projection zone */
   long in_units;	/* Input projection units code */
   long in_datum;	/* Input projection datum code */
   long out_proj;	/* Output projection code */
   long out_zone;	/* Output projection zone */
   long out_units;	/* Output projection units code */
   long out_datum;	/* Output projection datum code */
   double in_parms[15];	/* Input projection GCTP parameters */
   double out_parms[15];/* Output projection GCTP parameters */
   };

/* Structure containing report destination options & related parameters 
  --------------------------------------------------------------------*/
struct PRINT 
   {
   FILE *fptr;		/* Print file pointer (NULL if print file not used) */
   char name[CMLEN];	/* Print file name (NULL if print file not used) */
   char host_name[CMLEN];/* Host name of print file */
   };

/* Structure containing framing parameters 
  ---------------------------------------*/
struct FRAME 
   {
   double p_dist[2];	/* Pixel size */
   double ul_corner[2];	/* Upper left corner coordinates */
   double lr_corner[2];	/* Lower right corner coordinates */
   long lscoors[2];	/* Line, sample coordinates of ULCOORS in proj coords */
   long nl;		/* Number of lines in output when ULCOORS is proj coor*/
   long ns;		/* Number of samps in output when ULCOORS is proj coor*/
   long mode;		/* UNKNOWN, INPUT, or OUTPUT */
   long unit_code;	/* Geographic unit code */
   char units[12];	/* Geographic units */
   };

/* Structure containing polynomial information for the POLY subcommand
  -------------------------------------------------------------------*/
struct FIT 
   {
   double a[15];	/* Transformation coefficients in x */
   double b[15];	/* Transformation coefficients in y */
   double offset[4];	/* Transformation offsets (y,x,y',x') */
   double window[4];    /* Windowing */
   long ncoef;		/* Number of coefficients */
   long field_mode;	/* Which coordinate field to process--(X,Y) or image */
   };
