C PROGRAM ANUDEM
C ------- ------
C
C CALCULATES A REGULAR GRID OF VALUES OF A SMOOTH SURFACE FITTED TO
C LARGE NUMBERS OF IRREGULARLY SPACED ELEVATION DATA POINTS, CONTOUR
C LINES, STREAM LINES AND SINK POINTS.  THE PROGRAM IMPOSES A GLOBAL
C DRAINAGE CONDITION WHICH REMOVES SINKS IN THE FITTED SURFACE WHERE
C POSSIBLE BY ENFORCING A DESCENDING CHAIN CONDITION FROM EACH SINK
C TO A LOWER SINK, DATA POINT OR EDGE VIA THE LOWEST INTERVENING SADDLE.
C THE PROGRAM ALSO AUTOMATICALLY CALCULATES RIDGE LINES AND STREAM
C LINES FROM POINTS ON DATA CONTOUR LINES WITH MAXIMUM CURVATURE.
C
C VALUES ARE CALCULATED ITERATIVELY USING A CONSIDERABLY MODIFIED
C VERSION OF A TECHNIQUE DESCRIBED IN CSIRO DIVISION OF MATHEMATICS
C AND STATISTICS CONSULTING REPORT ACT 84/6 BY M.F. HUTCHINSON.
C ITERATIONS ARE PERFORMED USING A SIMPLE NESTED GRID METHOD WHICH
C EMPLOYS GAUSS SEIDEL ITERATION WITH OVERRELAXATION (SOR METHOD) FOR
C EACH SUCCEEDING FINER GRID.
C
C LOCAL MAXIMA MAY OPTIONALLY BE CONSTRAINED TO LIE ONLY AT DATA POINTS
C ON THE ASSUMPTION THAT ALL HIGH POINTS HAVE BEEN SAMPLED IN THE DATA.
C
C THE OUTPUT GRID MAY BE LIMITED TO A SUBSET OF THE SPECIFIED RECTANGULAR
C GRID BY READING POLYGONS WHICH ENCLOSE THE REQUIRED SUBSET.
C
C POINT AND LINE DATA ARE READ FROM UP TO MAXFIL DATA FILES AND STORED ON
C THE UNFORMATTED SCRATCH FILES SAVFIL, STRFIL, SNKFIL, POLFIL AND CONFIL
C FOR QUICK RETRIEVAL.
C
C FINAL CALCULATED GRID VALUES ARE WRITTEN TO GRDFIL.  GRID VALUES
C OUTSIDE THE SUBSET SPECIFIED BY ANY POLYGONS ARE SET TO A SPECIAL VALUE
C LESS THAN THE MINIMUM Z VALUE AS SPECIFIED BY THE USER DIRECTIVES.
C
C ORDERED CHAIN INFORMATION AS INCORPORATED INTO THE GRID FROM DATA
C STREAMLINES AND PROGRAM CALCULATED STREAM LINES AND RIDGE LINES MAY
C OPTIONALLY BE WRITTEN TO DIGFIL IN A FORM SUITABLE FOR PLOTTING.
C
C LARGE RESIDUALS OF THE FITTED SURFACE MAY OPTIONALLY BE CALCULATED
C AND WRITTEN TO RESFIL.
C
C N.B. THE INTEGER*2 VARIABLES MAY BE SYSTEMATICALLY REPLACED BY
C ---  SOME VERSION OF "SHORT" INTEGER.  THE ARRAY IW TAKES VALUES
C      IN THE RANGE -2 TO 50.  THE ARRAY IV TAKES VALUES IN THE RANGE
C      0 TO 40.
C
C WRITTEN APRIL 1985 BY
C     M.F. HUTCHINSON
C     CENTRE FOR RESOURCE AND ENVIRONMENTAL STUDIES
C     AUSTRALIAN NATIONAL UNIVERSITY
C     G.P.O. BOX 4
C     CANBERRA  ACT 2601
C     AUSTRALIA
C
C LAST REVISION: 3 MARCH 1991.
C
      INTEGER   INFIL,OUTFIL,DATFIL,GRDFIL,RESFIL,SAVFIL,STRFIL,SNKFIL,
     +          POLFIL,CONFIL,IZ,MAXD,ND,MAXIT,M,N,M0,N0,M1,N1,NZ,
     +          MAXNS,NS,NUMREF,MAXREF,MAXFIL,NUMFIL,ISTEP,
     +          IDOPT,MOPT,IOPT,IGOPT,
     +          ODTYPE,
     +		NST,NSNK,ICOP,NCON,MAXB,
     +          MAXC,DIGFIL,NW,NM,IWFIL,IVFIL

      INTEGER*2 MAXW,MAXW1
C

      include 'anudem_size.fin'

      PARAMETER (MAXD=IZ*IZ/4, MAXREF=6, MAXM=2*IZ)

C Original settings
C      PARAMETER (IZ=901, MAXD=IZ*IZ/4, MAXREF=6, MAXM=2*IZ)
C      PARAMETER (IZ=2001, MAXD=IZ*IZ/4, MAXREF=6, MAXM=2*IZ)
C      PARAMETER (IZ=3601, MAXD=IZ*IZ/4, MAXREF=6, MAXM=2*IZ)
C      PARAMETER (MAXFIL=80, MAXNS=10000, MAXB=10000, MAXC=8000)
C      PARAMETER (MAXFIL=80, MAXNS=110000, MAXB=110000, MAXC=90000)
C      PARAMETER (MAXW=4, MAXW1=50)
C      PARAMETER (MAXW=8, MAXW1=100)
C
      INTEGER          SINK(8,MAXNS),IP(8),JP(8),FILTYP(MAXFIL),
     +                 KMAX2(MAXM,2),IC(6,MAXC),NC,II,JJ,IS
      REAL             U(IZ,IZ),Z(MAXD),RHO,RHO1,RMS,TOL(4),RES,
     +                 OMEGA,Z0,Z1,ZERO,ONE,ONE5,SPVAL,
     +                 T,DELTA,V(MAXM),DP(8),FAC,CTOL,
     +                 UMAX(MAXM),UMAX2(MAXM,2),XX,YY
      DOUBLE PRECISION X0,Y0,X1,Y1,H,H1,HH(MAXREF),X(MAXD),Y(MAXD),
     +                 XB(MAXB),YB(MAXB),TB(MAXB),ZMN,SSQ,RMAX
      INTEGER*2        IW(0:IZ+1,0:IZ+1),IV(IZ,IZ)
C     Output data type used for LAS image option 4 in setup.
      CHARACTER*40     DATNAM(MAXFIL),DATFMT(MAXFIL),HEDFMT(MAXFIL),
     +                 RESNAM,GRDNAM,GRDFMT,SNKNAM,DIGNAM,BLANK,
     +                 IWNAM,IVNAM
C
      COMMON/SPLIN2H/X,Y
      COMMON/LIMIT2H/X0,Y0,X1,Y1,H1
      COMMON/SFLOW/II,JJ,IS,XX,YY
C
C---EQUIVALENCE PUTS X AND Y ARRAYS WITHIN STORAGE OF U---
      EQUIVALENCE (X(1), U(1,1))
C
      DATA INFIL,OUTFIL,DATFIL,GRDFIL,RESFIL,DIGFIL/5, 6, 1, 2, 3, 3/
      DATA SAVFIL,STRFIL,SNKFIL,POLFIL,CONFIL/7, 8, 9, 10, 11/
      DATA IWFIL,IVFIL/12, 13/
      DATA BLANK/' '/
      DATA ZERO,ONE,ONE5,DELTA/0.0, 1.0, 1.5, 0.01/
      DATA IP/-1, 0, 1, 1, 1, 0,-1,-1/
      DATA JP/-1,-1,-1, 0, 1, 1, 1, 0/
C
C---FIRST EXECUTABLE STATEMENT---
      WRITE(OUTFIL,10)
  10  FORMAT(1H1/' ANUDEM VERSION 3 03/03/91')
C
      CALL SETUP(INFIL,OUTFIL,DATFIL,DATNAM,HEDFMT,DATFMT,FILTYP,
     +           MAXFIL,NUMFIL,RESNAM,GRDNAM,GRDFMT,SNKNAM,DIGNAM,
     +           SPVAL,X0,X1,Y0,Y1,Z0,Z1,H,IZ,MAXM,M,N,RMS,TOL,RES,
     +           T,MAXIT,IDOPT,MOPT,ICOP,RMAX,IGOPT,
     +           ODTYPE,
     +           OMEGA,FAC,DELTA,
     +           IWNAM,IVNAM)
	ispace=idopt
	idopt=iabs(idopt)
C
C---CALCULATE RECIPROCAL LENGTH OF POINTERS---
      DO 15 I=1,8
      DP(I)=ONE/SQRT(FLOAT(IP(I)*IP(I) + JP(I)*JP(I)))
  15  CONTINUE
C
C---CALCULATE FORMAT DESCRIPTORS FOR OUTPUT POSITION COORDINATES---
      NW=DLOG10(DMAX1(ABS(X0),ABS(X1),ABS(Y0),ABS(Y1))) + 3.001
      NW=MAX0(NW,5)
      NM=MAX0(10-NW,0)
      NW=NW + NM
C
C---SET CONTOUR PARAMETERS---
      J3=2
      IF (ICOP.EQ.1)THEN
         CTOL=4.0*H
      ELSE
         CTOL=(X1 - X0) + (Y1 - Y0)
      ENDIF
      WRITE(OUTFIL,20)
  20  FORMAT(1H1)
C
C---OPTIONALLY READ POLYGONS---
      NP=0
      OPEN(POLFIL,FORM='UNFORMATTED',STATUS='SCRATCH')
      DO 25 K=1,NUMFIL
      IF (FILTYP(K).NE.4) GOTO 25
      CALL RDPOLY(K,OUTFIL,DATFIL,DATNAM(K),HEDFMT(K),DATFMT(K),
     +            POLFIL,M,N,X0,X1,Y0,Y1,H,X,Y,MAXD,NP)
  25  CONTINUE
C
C---INITIALIZE COUNTER GRID---
      CALL INITIW(POLFIL,IW,M,N,H,NP,XB,YB,MAXB)
C
C---READ DATA---
      NZ=0
      ZMN=ZERO
      ND=0
      OPEN(SAVFIL,FORM='UNFORMATTED',STATUS='SCRATCH')
      DO 30 K=1,NUMFIL
      IF (FILTYP(K).NE.1) GOTO 30
      CALL RDATA(K,OUTFIL,DATFIL,DATNAM(K),DATFMT(K),SAVFIL,IW,MAXW,
     +           M,N,X0,X1,Y0,Y1,H,Z0,Z1,ND,NZ,ZMN)
  30  CONTINUE
C
C---READ CONTOUR LINE DATA---
      NCON=0
      OPEN(CONFIL,FORM='UNFORMATTED',STATUS='SCRATCH')
      DO 35 K=1,NUMFIL
      IF (FILTYP(K).NE.5) GOTO 35
      CALL RDCONT(K,OUTFIL,DATFIL,DATNAM(K),HEDFMT(K),DATFMT(K),CONFIL,
     +            IW,M,N,X0,X1,Y0,Y1,H,Z0,Z1,X,Y,MAXD,MAXB,NCON,NZ,ZMN)
  35  CONTINUE
C
C---READ STREAMLINES---
      NST=0
      OPEN(STRFIL,FORM='UNFORMATTED',STATUS='SCRATCH')
      DO 40 K=1,NUMFIL
      IF (FILTYP(K).NE.3) GOTO 40
      CALL RDSTRM(K,OUTFIL,DATFIL,DATNAM(K),HEDFMT(K),DATFMT(K),STRFIL,
     +            M,N,X0,X1,Y0,Y1,H,Z0,Z1,SPVAL,X,Y,MAXD,MAXB,NST)
  40  CONTINUE
C
C---READ SINKS---
      NSNK=0
      OPEN(SNKFIL,FORM='UNFORMATTED',STATUS='SCRATCH')
      DO 50 K=1,NUMFIL
      IF (FILTYP(K).NE.2) GOTO 50
      CALL RDSINK(K,OUTFIL,DATFIL,DATNAM(K),DATFMT(K),SNKFIL,
     +            IW,M,N,X0,X1,Y0,Y1,H,NSNK)
  50  CONTINUE
C
C---INITIALIZE STARTING GRID---
      CALL SETNUM(X0,X1,Y0,Y1,H,NUMREF,HH,MAXREF,DELTA)
      WRITE(OUTFIL,60) NUMREF
  60  FORMAT(/' NUMBER OF GRID RESOLUTIONS      =',I7)
      H1=HH(1)
      M1=(X1 - X0)/H1 + ONE5 + DELTA
      N1=(Y1 - Y0)/H1 + ONE5 + DELTA
      CALL PLNGRD(U,M1,N1,Z0,Z1,NZ,ZMN)
      ISTEP=5
      MAXIT=ISTEP*((MAXIT + ISTEP - 1)/ISTEP)
C
C---LOOP THROUGH GRID RESOLUTIONS---
      DO 100 K=1,NUMREF
      M0=M1
      N0=N1
      H0=H1
      H1=HH(K)
      WRITE(OUTFIL,70) K,H1
  70  FORMAT(/' RESOLUTION',I3/' SPACING =',E12.4)
      M1=(X1 - X0)/H1 + ONE5 + DELTA
      N1=(Y1 - Y0)/H1 + ONE5 + DELTA
      NS=0
C
C---INITIALIZE GRIDS FOR ITERATION---
      IF (K.GT.1) CALL INGRID(U,M0,N0,U,M1,N1)
      CALL INITIW(POLFIL,IW,M1,N1,H1,NP,XB,YB,MAXB)
      CALL INITIV(IV,M1,N1)
C
C---SELECT DATA POINTS---
      CALL SELDAT(OUTFIL,SAVFIL,ND,U,IW,MAXW1,M1,N1,H1)
C
C---SELECT CONTOUR LINE POINTS---
      IF (ICOP.GT.0 .AND. (K.GT.1 .OR. K.EQ.NUMREF)) THEN
         NC=0
      ELSE
         NC=-1
      ENDIF
      CALL SELCON(OUTFIL,CONFIL,NCON,U,IW,MAXW1,IV,IC,MAXC,NC,
     +            M1,N1,H1,Z0,Z1,IP,JP,DP,XB,YB,TB,MAXB,J3,CTOL)
C
C---RESET IW GRID---
      CALL RSETIW(IW,M1,N1)
C
C---SELECT STREAMLINES---
      IF (NST.GT.0 .AND. K.GE.NUMREF-1) THEN
         CALL SELSTR(OUTFIL,STRFIL,NST,U,IW,IV,M1,N1,H1,
     +               Z0,Z1,IP,JP,XB,YB,MAXB,ICOP)
         CALL INCSTR(OUTFIL,U,IW,IV,M1,N1,X0,Y0,H1,Z0,Z1,IP,JP,
     +               TOL(3),ICOP)
      ENDIF
C
C---SELECT SINK POINTS---
      IF (NSNK.GT.0 .AND. K.GE.NUMREF-1) THEN
         CALL SELSNK(OUTFIL,SNKFIL,NSNK,U,IW,IV,M1,N1,H1,Z0,Z1,IP,JP)
      ENDIF
C
C---COUNT DISTINCT DATA POINTS---
      CALL SRTDAT(U,IW,M1,N1,Z,MAXD,ND1)
      WRITE(OUTFIL,80) ND1
  80  FORMAT(' NUMBER OF DISTINCT DATA POINTS  =',I7)
      IF (ND1.LE.3) STOP 'ANUDEM - TOO FEW DATA POINTS SELECTED'
C
C---ITERATE---
      IF (K.EQ.1) THEN
         RHO=1.0/ND1
         ALAM=RHO/ND1
      ELSE
         ALAM=4.0*ALAM
         RHO=ALAM*ND1
      ENDIF
      ZGAP=2.0*RMS
      TOL(4)=2.0*ZGAP
      SSQ0=ND1*RMS*RMS
      IF (RMS.EQ.ZERO) RHO=ZERO
      DO 90 I=1,MAXIT
      IF (K.LT.NUMREF .AND. I.GT.30) GOTO 90
      IF (K.EQ.NUMREF .AND. I.GT.MAXIT-ISTEP) OMEGA=AMIN1(OMEGA,1.5)
      CALL ITERAT(U,IW,IV,M1,N1,V,Z0,Z1,Z,ND1,UMAX,KMAX2,UMAX2,
     +            IP,JP,DP,T,OMEGA,RHO,SSQ,FAC,ZGAP,MOPT)
      IF (SSQ.GT.ZERO) THEN
         RHO1=RHO - (SSQ - SSQ0)*(ONE + 20.0*RHO)*RHO/SSQ
         RHO1=AMAX1(AMIN1(RHO1,2.0*RHO),RHO/2.0)
         RHO=AMAX1(AMIN1(RHO1,1.0E10),1.0E-10)
      ENDIF
C
C---PROCESS SINKS, STREAMS AND RIDGES---
      IF (K.LE.2 .AND. K.LT.NUMREF-1) GOTO 90
      IF (IDOPT.LE.0) GOTO 90
      IF (MOD(I,ISTEP).NE.0) GOTO 90
      IF (I.LE.ISTEP) GOTO 90
      IF (NC.GT.0 .AND. I.LT.MAXIT) THEN
         CALL PROLIN(OUTFIL,U,IW,IV,IC,MAXC,NC,M1,N1,H1,IP,JP,DP,
     +               TOL,RMAX)
      ENDIF
      CALL PROSNK(OUTFIL,SINK,MAXNS,NS,U,IW,IV,M1,N1,IP,JP,DP,Z,ND1,
     +               IDOPT,TOL,ICOP)
      IF (NC.GT.0 .AND. I.LT.MAXIT) THEN
         CALL FIXLIN(OUTFIL,U,IW,IV,IC,MAXC,NC,M1,N1,IP,JP,DP,TOL)
      ENDIF
  90  CONTINUE
C
C---OUTPUT STATISTICS FOR RESOLUTION---
      WRITE(6,95) RHO/ND1,DSQRT(SSQ/ND1)
  95  FORMAT(' LAMBDA =',E12.4,'   RMS =',E12.4)
      IF (K.GT.1 .OR. K.EQ.NUMREF) THEN
         IOPT=1
         IF (K.EQ.NUMREF) IOPT=2
         CALL FNDSNK(OUTFIL,SINK,MAXNS,NS,U,IW,IV,M1,N1,IP,JP,Z,ND1,
     +               IOPT,IDOPT,TOL(1))
      ENDIF
      ALAM=RHO/ND1
 100  CONTINUE
C
C---OPTIONALLY WRITE REMAINING SINK POSITIONS TO SNKNAM---
      IF (SNKNAM.NE.BLANK) THEN
         OPEN(RESFIL,FILE=SNKNAM,STATUS='NEW')
         CALL WRSINK(OUTFIL,RESFIL,U,IW,M1,N1,X0,Y0,H1,NW,NM)
         CLOSE(RESFIL)
      ENDIF
C
C---OPTIONALLY WRITE SINK DIAGNOSTICS TO OUTFIL---
      IF (IDOPT.GE.2) THEN
         CALL SNKDIG(OUTFIL,U,IW,IV,M1,N1)
      ENDIF
C
C---OPTIONALLY WRITE STREAM INFORMATION TO DIGFIL---
      IF (DIGNAM.NE.BLANK) THEN
         OPEN(DIGFIL,FILE=DIGNAM,STATUS='NEW')
         CALL WRSTRM(OUTFIL,DIGFIL,U,IW,IV,M1,N1,X0,Y0,H,IP,JP,NW,NM)
         CLOSE(DIGFIL)
      ENDIF
C
C---OPTIONALLY WRITE INDICATOR GRIDS---
      if (ispace.lt.0) then
         open(iwfil,file=iwnam,status='new',form='unformatted')
         open(ivfil,file=ivnam,status='new',form='unformatted')
         call wriwiv(iwfil,ivfil,iw,iv,m1,n1)
         close(iwfil)
         close(ivfil)
      endif
C
C---OPTIONALLY WRITE LARGE RESIDUALS TO RESFIL---
      IF (RESNAM.NE.BLANK) THEN
         OPEN(RESFIL,FILE=RESNAM,STATUS='NEW')
         DO 110 K=1,NUMFIL
         IF (FILTYP(K).NE.1 .OR. DATNAM(K).EQ.BLANK) GOTO 110
         CALL RESID1(OUTFIL,K,DATFIL,DATNAM(K),DATFMT(K),RESFIL,
     +               U,IW,M1,N1,X0,X1,Y0,Y1,H1,Z0,Z1,RES,NW,NM)
 110     CONTINUE
         CLOSE(RESFIL)
      ENDIF
C
C---WRITE GRID TO GRDFIL---
      IF (NP.GT.0) WRITE(OUTFIL,130) SPVAL
 130  FORMAT(/' SPECIAL VALUE OF OUTPUT GRID   =',F8.1)
C
C---USGS (ARC/INFO) DEM FORMAT---
      IF (IGOPT.EQ.2) THEN
         OPEN(GRDFIL,FILE=GRDNAM,STATUS='NEW')
         CALL USDEM(GRDNAM,GRDFIL,U,IW,M1,N1,X0,X1,Y0,Y1,H,
     +              Z0,Z1,SPVAL)
      CLOSE(GRDFIL)
      else
C
C---  Arc/LAS BIL format with header for Arc
C
         if (igopt .eq. 3) then
            call bilhdr(GRDNAM,M1,N1,X0,X1,Y0,Y1,H,U,IW,SPVAL)
C
C---  LAS Image format 
C
C	The las image was added to the original code by EDC,CSB.

         else 
          if (igopt .eq. 4) then
            call lasimage(GRDNAM,M1,N1,X0,X1,Y0,Y1,H,U,IW,SPVAL,ODTYPE)
C
C---USER DEFINED DEM FORMAT---
         ELSE
            IF (GRDFMT.EQ.BLANK) THEN
               OPEN(GRDFIL,FILE=GRDNAM,STATUS='NEW',FORM='UNFORMATTED')
            ELSE
               OPEN(GRDFIL,FILE=GRDNAM,STATUS='NEW')
            ENDIF
            CALL WRGRID(GRDFIL,GRDFMT,U,IW,M1,N1,X0,X1,Y0,Y1,H1,
     +                  SPVAL,IGOPT)
            CLOSE(GRDFIL)
         ENDIF
       endif
      endif
C
C---END OF PROGRAM---
      WRITE(OUTFIL,1)
  1   FORMAT(/'ANUDEM - SUCCESSFUL COMPLETION')
      END
      SUBROUTINE SETUP(INFIL,OUTFIL,DATFIL,DATNAM,HEDFMT,DATFMT,
     +                 FILTYP,MAXFIL,NUMFIL,RESNAM,GRDNAM,GRDFMT,
     +                 SNKNAM,DIGNAM,SPVAL,X0,X1,Y0,Y1,Z0,Z1,H,IZ,
     +                 MAXM,M,N,RMS,TOL,RES,T,MAXIT,IDOPT,MOPT,ICOP,
     +                 RMAX,IGOPT,
     +                 ODTYPE,
     +                 OMEGA,FAC,DELTA,IWNAM,IVNAM)
C
      INTEGER          INFIL,OUTFIL,DATFIL,MAXFIL,FILTYP(MAXFIL),M,N,
     +                 IZ,MAXM,MAXIT,NUMFIL,IDOPT,MOPT,ICOP,IGOPT,
     +    	       ODTYPE
      REAL             Z0,Z1,RMS,TOL(4),RES,T,OMEGA,FAC,DELTA
      DOUBLE PRECISION X0,Y0,X1,Y1,H,RMAX
      CHARACTER*40     DATNAM(MAXFIL),HEDFMT(MAXFIL),DATFMT(MAXFIL),
     +                 RESNAM,GRDNAM,SNKNAM,DIGNAM,GRDFMT,IWNAM,IVNAM
C
C---LOCAL VARIABLES---
      INTEGER          NFIL
      REAL             ZERO,ONE,ONE5
      DOUBLE PRECISION TEM1,TEM2
      CHARACTER*40     BLANK
C
      DATA             ZERO,ONE,ONE5/0.0, 1.0, 1.5/
      DATA             BLANK/' '/
C
C---DRAINAGE OPTION---
      WRITE(OUTFIL,20)
  20  FORMAT(/' DRAINAGE OPTION (0,1,2  OR 3):'/
     +        '   0 NO DRAINAGE ENFORCEMENT'/
     +        '   1 DRAINAGE ENFORCEMENT'/
     +        '   2 DRAINAGE ENFORCEMENT WITH SINK DIAGNOSTICS'/
     +        '   3 DRAINAGE ENFORCEMENT WITH FULL DIAGNOSTICS')
      READ(INFIL,*) IDOPT
      WRITE(OUTFIL,21) IDOPT
  21  FORMAT(1X,I4)
      IF (IDOPT.LT.-3 .OR. IDOPT.GT.3) STOP 'SETUP - ILLEGAL OPTION'
C
C---CONTOUR DATA OPTION AND MAXIMUM RIDGE LENGTH---
      WRITE(OUTFIL,22)
  22  FORMAT(/' CONTOUR DATA OPTION (0 OR 1):'/
     +        '  0 - ELEVATION DATA CONSISTS MAINLY OF SPOT HEIGHTS'/
     +        '  1 - ELEVATION DATA CONSISTS MAINLY OF CONTOURS')
      READ(INFIL,*) ICOP
      WRITE(6,21) ICOP
      IF (ICOP.LT.0 .OR. ICOP.GT.1)
     +     STOP 'SETUP - ILEGAL CONTOUR OPTION'
      IF (ICOP.EQ.1) THEN
         WRITE(OUTFIL,12)
  12     FORMAT(/' MAXIMUM RIDGE LENGTH (AT LEAST 0.0):')
         READ(INFIL,*) RMAX
         WRITE(OUTFIL,14) RMAX
  14     FORMAT(1X,F10.2)
         IF (RMAX.LT.0.0D0) STOP 'ILLEGAL RIDGE LENGTH'
      ENDIF
C
C---INPUT ROOT MEAN SQUARE RESIDUAL---
      WRITE(OUTFIL,10)
  10  FORMAT(/' ROOT MEAN SQUARE RESIDUAL (AT LEAST 0.0):'/
     +        '  NORMALLY 0.0 FOR POINT DATA'/
     +        '  NORMALLY 1% CONTOUR INTERVAL FOR CONTOUR DATA')
      READ(INFIL,*) RMS
      WRITE(OUTFIL,15) RMS
  15  FORMAT(1X,F10.5)
      IF (RMS.LT.ZERO) STOP 'SETUP - ILLEGAL RMS'
C
C---MAXIMUM OPTION---
      WRITE(OUTFIL,23)
  23  FORMAT(/' MAXIMUM OPTION (0 OR 1):'/
     +        '  0 - MAXIMA NOT CONSTRAINED TO LIE AT DATA POINTS'/
     +        '  1 - MAXIMA CONSTRAINED TO LIE AT DATA POINTS')
      READ(INFIL,*) MOPT
      WRITE(OUTFIL,21) MOPT
      IF (MOPT.LT.0 .OR. MOPT .GT. 1) STOP 'SETUP - ILLEGAL MOPT'
C
C---ROUGHNESS PENALTY TRADE-OFF---
      WRITE(OUTFIL,24)
  24  FORMAT(/' ROUGHNESS PENALTY TRADE-OFF (AT LEAST 0.0):'/
     +        '  0.0 - MINIMUM CURVATURE - FOR CONTOUR DATA'/
     +        '  0.5 - RECOMMENDED FOR POINT DATA'/
     +        '  1.0 - APPROACHING MINIMUM POTENTIAL (NOT RECOMMENDED)')
      READ(INFIL,*) T
      WRITE(OUTFIL,25) T
  25  FORMAT(1X,F10.5)
      IF (T.LT.0.0) STOP 'SETUP - ILLEGAL TRADE-OFF'
C
C---ELEVATION TOLERANCES---
      WRITE(OUTFIL,26)
  26  FORMAT(/' THREE NON-NEGATIVE ELEVATION TOLERANCES:'/
     +        ' (1ST TOLERANCE MUST BE AT LEAST FOUR TIMES RMS)'/
     +        ' (2ND TOLERANCE MUST BE AT LEAST TWICE 1ST TOLERANCE)'/
     +        ' (3RD TOLERANCE MUST BE AT LEAST TWICE 2ND TOLERANCE)')
      READ(INFIL,*) (TOL(I),I=1,3)
      WRITE(OUTFIL,30) (TOL(I),I=1,3)
  30  FORMAT(1X,3F10.2)
      IF (TOL(1).LT.4.0*RMS .OR. TOL(2).LE.TOL(1) .OR.
     +    TOL(2).LT.2.0*TOL(1) .OR. TOL(3).LE.2.0*TOL(2))
     +    STOP 'SETUP - ILLEGAL TOLERANCES'
C
C---NUMBER OF ITERATIONS--
      WRITE(OUTFIL,35)
  35  FORMAT(/' MAXIMUM NO. OF ITERATIONS (NORMALLY 20-30):')
      READ(INFIL,*) MAXIT
      WRITE(OUTFIL,40) MAXIT
  40  FORMAT(1X,I6)
      IF (MAXIT.LE.0) STOP 'SETUP - TOO FEW ITERATIONS'
C
C---ELEVATION LIMITS---
      WRITE(OUTFIL,45)
  45  FORMAT(/' HEIGHT LIMITS:')
      READ(INFIL,*) Z0,Z1
      WRITE(OUTFIL,50) Z0,Z1
  50  FORMAT(1X,2F12.4)
      IF (Z0.GE.Z1) STOP 'SETUP - MINIMUM EXCEEDS MAXIMUM'
C
C---SET SPECIAL VALUE---
      SPVAL=ZERO
      DO 60 I=1,10
      IF (SPVAL.LT.Z0) GOTO 70
      SPVAL=10.0*SPVAL - 9.0
  60  CONTINUE
      STOP 'SETUP - SPVAL ERROR'
C
C---RESIDUAL OPTION---
  70  WRITE(OUTFIL,90)
  90  FORMAT(/' RESIDUAL FILE NAME (BLANK IF NOT REQUIRED):')
      READ(INFIL,100) RESNAM
 100  FORMAT(A40)
      WRITE(OUTFIL,110) RESNAM
 110  FORMAT(1X,A40)
      IF (RESNAM.EQ.BLANK) GOTO 120
      WRITE(OUTFIL,80)
  80  FORMAT(/' MINIMUM RESIDUAL (NON-NEGATIVE REAL):')
      READ(INFIL,*) RES
      WRITE(OUTFIL,50) RES
C
C---READ GRID PARAMETERS---
 120  WRITE(OUTFIL,130)
 130  FORMAT(/' X (LONGITUDE) LIMITS:')
      READ(INFIL,*) X0,X1
      WRITE(OUTFIL,140) X0,X1
 140  FORMAT(1X,2F15.4)
      IF (X0.GE.X1) STOP 'SETUP - ILLEGAL X LIMITS'
C
      WRITE(OUTFIL,150)
 150  FORMAT(/' Y (LATITUDE) LIMITS:')
      READ(INFIL,*) Y0,Y1
      WRITE(OUTFIL,160) Y0,Y1
 160  FORMAT(1X,2F15.4)
      IF (Y0.GE.Y1) STOP 'SETUP - ILLEGAL Y LIMITS'
C
C---GRID SPACING---
      WRITE(OUTFIL,170)
 170  FORMAT(/' GRID SPACING:')
      READ(INFIL,*) H
      WRITE(OUTFIL,175) H
 175  FORMAT(1X,F15.6)
      M=(X1 - X0)/H + ONE5 + DELTA
      N=(Y1 - Y0)/H + ONE5 + DELTA
      WRITE(OUTFIL,180) M,N
 180  FORMAT(/' ROW DIMENSION OF OUTPUT  GRID =',I6/
     +        ' COL DIMENSION OF OUTPUT  GRID =',I6)
      TEM1=(X1 - X0)/H + ONE
      TEM2=(Y1 - Y0)/H + ONE
      IF (M.LT.5 .OR. N.LT.5) STOP 'SETUP - GRID TOO SMALL'
      IF (M.GT.MAXM .OR.(M+2)*(N+2).GT.(IZ+2)*(IZ+2))
     +     STOP 'SETUP - SPACING TOO SMALL'
      IF (DABS(TEM1-M).GT.DELTA .OR. DABS(TEM2-N)
     +     .GT.DELTA) STOP 'SETUP - INCOMPATIBLE GRID SPACING'
C
C---DATA FILES---
      WRITE(OUTFIL,190) MAXFIL
 190  FORMAT(/' NUMBER OF DATA FILES (MAX =',I3,'):'/
     +        '  TYPE 1 = DATA POINT FILE'/
     +        '  TYPE 2 = SINK POINT FILE'/
     +        '  TYPE 3 = STREAMLINE FILE'/
     +        '  TYPE 4 = POLYGON FILE'/
     +        '  TYPE 5 = CONTOUR FILE')
      READ(INFIL,*) NUMFIL
      WRITE(OUTFIL,200) NUMFIL
 200  FORMAT(1X,I5)
      IF (NUMFIL.LT.1 .OR. NUMFIL.GT.MAXFIL)
     +     STOP 'SETUP - ILLEGAL NUMBER OF DATA FILES'
C
C---READ EACH DATA FILE NAME---
      DO 290 NFIL=1,NUMFIL
      WRITE(OUTFIL,210) NFIL
 210  FORMAT(/' DATA FILE',I3,':')
      READ(INFIL,100) DATNAM(NFIL)
      WRITE(OUTFIL,110) DATNAM(NFIL)
C
C---CHECK FOR EXISTENCE OF DATA FILE---
      OPEN(DATFIL,FILE=DATNAM(NFIL),STATUS='OLD',ERR=220)
      CLOSE(DATFIL)
      GOTO 225
 220  STOP 'SETUP - DATA FILE NOT FOUND'
C
C---READ DATA FILE SPECIFICATIONS---
 225  WRITE(OUTFIL,230)
 230  FORMAT(/' DATA FILE TYPE (1,2,3,4 OR 5):')
      READ(INFIL,*) FILTYP(NFIL)
      WRITE(OUTFIL,235) FILTYP(NFIL)
 235  FORMAT(1X,I4)
      IF (FILTYP(NFIL).LT.1 .OR. FILTYP(NFIL).GT.5)
     +     STOP 'SETUP - ILLEGAL DATA FILE TYPE'
      IF (FILTYP(NFIL).LE.2) GOTO 250
      WRITE(OUTFIL,240)
 240  FORMAT(/' DATA FILE HEADER FORMAT:')
      READ(INFIL,100) HEDFMT(NFIL)
      WRITE(OUTFIL,110) HEDFMT(NFIL)
 250  IF (FILTYP(NFIL).EQ.1) WRITE(OUTFIL,260)
 260  FORMAT(/' DATA FILE (X,Y,Z) FORMAT:')
      IF (FILTYP(NFIL).EQ.2) WRITE(OUTFIL,270)
 270  FORMAT(/' SINK DATA FILE (X,Y,Z) FORMAT:')
      IF (FILTYP(NFIL).GE.3) WRITE(OUTFIL,280)
 280  FORMAT(/' DATA FILE LINE STRING FORMAT:')
      READ(INFIL,100) DATFMT(NFIL)
      WRITE(OUTFIL,110) DATFMT(NFIL)
 290  CONTINUE
C
C---SET ACCELERATION PARAMETER OMEGA (BETWEEN 1.0 AND 2.0)---
      OMEGA=1.6
      IF (OMEGA.LT.1.0 .OR. OMEGA.GE.2.0) 
     +     STOP 'SETUP - ILLEGAL ACCELERATION PARAMETER'
C
C---SET LINEARITY FACTOR (ALMOST 1.0)---
      FAC=0.95
      IF (FAC.LE.0.0 .OR. FAC.GT.1.0)
     +     STOP 'SETUP - ILLEGAL LINEARITY FACTOR'
C
C---OUTPUT GRID---
      WRITE(OUTFIL,340)
 340  FORMAT(/' OUTPUT FILE NAME:')
      READ(INFIL,100) GRDNAM
      WRITE(OUTFIL,110) GRDNAM
      WRITE(OUTFIL,350)
C
C   Added option 3 for generating a BIL file.
C   June 22, 1992
C   Added option 4 for generating a LAS image file.
C   Fall 1992
C
 350  FORMAT(/' MODE OF OUTPUT GRID (0 - 4):'/
     +        '  0 - GRID VALUES BY ROWS'/
     +        '  1 - X,Y,Z FORMAT'/
     +        '  2 - STANDARD USGS (ARC/INFO) DEM FORMAT'/
     +        '  3 - ARC/LAS BIL FORMAT WITH ARC HEADER'/
     +        '  4 - LAS IMAGE FORMAT ') 
      READ(INFIL,*) IGOPT 
      WRITE(OUTFIL,360) IGOPT
 360  FORMAT(1X,I5)
      IF (IGOPT.LT.0 .OR. IGOPT.GT.4) STOP 'ILLEGAL GRID OPTION'
      IF (IGOPT.EQ.0) WRITE(OUTFIL,370)
 370  FORMAT(/' OUTPUT GRID ROW FORMAT (BLANK FOR UNFORMATTED):')
      IF (IGOPT.EQ.1) WRITE(OUTFIL,380)
 380  FORMAT(/' OUTPUT GRID X,Y,Z FORMAT (BLANK FOR UNFORMATTED):')
      IF (IGOPT.LT.2) THEN
         READ(INFIL,100) GRDFMT
         WRITE(OUTFIL,110) GRDFMT
      ENDIF
C     Prompt for the output data type for the LAS image.
C     This prompt was added in addtion to the the other anudem setup parmeters
C     The only time this prompt is encountered is when the user specifies
C     a las image.

      IF (IGOPT.EQ.4) THEN
           WRITE(OUTFIL,385)
           READ(INFIL,*) ODTYPE 
           WRITE(OUTFIL,387) ODTYPE
      ENDIF
 385  FORMAT(/' (1 - 4):'/
     +        '  1 - BYTE'/
     +        '  2 - INTEGER*2'/
     +        '  3 - INTEGER*4'/
     +        '  4 - REAL*4') 
 387  FORMAT(1X,I5)
C
C---OUTPUT SINKS---
      WRITE(OUTFIL,390)
 390  FORMAT(/' OUTPUT SINK FILE NAME (BLANK IF NOT REQUIRED):')
      READ(INFIL,100) SNKNAM
      WRITE(OUTFIL,110) SNKNAM
C
C---OUTPUT STREAM FILE---
      WRITE(OUTFIL,400)
 400  FORMAT(/' OUTPUT STREAM FILE NAME (BLANK IF NOT REQUIRED):')
      READ(INFIL,100) DIGNAM
      WRITE(OUTFIL,110) DIGNAM
C
C---INDICATOR FILE NAMES---
	if (idopt.lt.0) then
         write(outfil,410)
 410     format(/' output iw grid file name:')
         read(infil,100) iwnam
         write(outfil,110) iwnam
         write(outfil,420)
 420     format(/' output iv grid file name:')
         read(infil,100) ivnam
         write(outfil,110) ivnam
      endif
      RETURN
      END
      SUBROUTINE WRSINK(OUTFIL,SNKFIL,U,IW,M,N,X0,Y0,H,NW,NM)
C
C WRITES OUT SINKS TO SNKFIL FOR PLOTTING.
C
      INTEGER          OUTFIL,SNKFIL,M,N,NW,NM
      REAL             U(M,N)
      DOUBLE PRECISION X0,Y0,H
      INTEGER*2        IW(0:M+1,0:N+1)
C
C---LOCAL VARIABLES---
      INTEGER          I,J
      CHARACTER*40     SFMT
C
C---SET UP OUTPUT FORMAT---
      WRITE(SFMT,10) NW,NM
  10  FORMAT('(2F',I2,'.',I1,','' S'',I1,F10.3)')
C
C---LOOP THROUGH SINKS---
      DO 20 J=1,N
      DO 20 I=1,M
      IF (IW(I,J).LT.2 .OR. IW(I,J).GT.3) GOTO 20
      WRITE(SNKFIL,SFMT,ERR=30) X0+(I-1)*H, Y0+(J-1)*H, IW(I,J), U(I,J)
  20  CONTINUE
      RETURN
C
C---ERROR CONDITION---
  30  WRITE(6,35) SFMT
  35  FORMAT(/' WRSINK - ERROR WRITING SINK DIAGNOSTICS'/
     +        ' RUN TIME FORMAT = ',A40)
      STOP
      END
      SUBROUTINE INITIW(POLFIL,IW,M,N,H,NP,XB,YB,MAXB)
C
C INITIALIZES IW ACCORDING TO THE POLYGON POINTS (IF ANY) ON
C THE UNFORMATTED SCRATCH FILE POLFIL.  THE EDGE VALUES OF
C IW ARE SET TO -1.
C
      INTEGER          POLFIL,M,N,NP,MAXB
      DOUBLE PRECISION H,XB(MAXB),YB(MAXB)
      INTEGER*2        IW(0:M+1,0:N+1)
C
C---LOCAL VARIABLES---
      INTEGER          I,J,IJ
C
C---NO POLYGON---
      IF (NP.LE.0) THEN
         DO 10 I=0,M+1
         IW(I,0)=-1
         IW(I,N+1)=-1
  10     CONTINUE
         DO 20 J=1,N
         IW(0,J)=-1
         IW(M+1,J)=-1
         DO 20 I=1,M
         IW(I,J)=0
  20     CONTINUE
C
C---POLYGON---
      ELSE
         CALL GETPOL(POLFIL,NP,IW,M,N,H,XB,YB,MAXB)
         DO 30 I=0,M+1
         IW(I,0)=-1
         IW(I,N+1)=-1
  30     CONTINUE
         DO 40 J=1,N
         IW(0,J)=-1
         IW(M+1,J)=-1
         DO 40 I=1,M
         IJ=1 - IW(I,J)
         IW(I,J)=MIN0(IJ,0)
  40     CONTINUE
      ENDIF
      END
      SUBROUTINE INITIV(IV,M,N)
C
C INITIALISES THE POINTER ARRAY USED TO REPRESENT ORDERED CHAINS.
C
      INTEGER   M,N
      INTEGER*2 IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER   I,J
C
      DO 10 J=1,N
      DO 10 I=1,M
      IV(I,J)=0
  10  CONTINUE
      RETURN
      END
      SUBROUTINE RSETIW(IW,M,N)
C
C RESETS DATA POINT COUNTERS LARGER THAN 1 TO 1.
C
      INTEGER   M,N
      INTEGER*2 IW(0:M+1,0:N+1)
C
C---LOCAL VARIABLES---
      INTEGER   I,J
C
C---RESET IW---
      DO 10 J=1,N
      DO 10 I=1,M
      IF (IW(I,J).GT.1) IW(I,J)=1
  10  CONTINUE
      RETURN
      END
      SUBROUTINE WRGRID(GRDFIL,GRDFMT,U,IW,M,N,X0,X1,Y0,Y1,H,SPVAL,
     +                  IGOPT)
C
C WRITES THE GRID U TO GRDFIL IN THE MODE INDICATED BY IGOPT.
C
C IF IGOPT=0 THE GRID IS WRITTEN BY ROWS TO GRDFIL USING THE FORMAT
C GRDFMT.  SPECIAL VALUES ARE INSERTED OUTSIDE THE SUBSET OF THE GRID
C SPECIFIED BY IW.
C
C IF IGOPT=1 THE GRID IS WRITTEN IN X,Y,Z FORMAT USING THE FORMAT
C GRDFMT.  IN THIS CASE, GRID VALUES LYING OUTSIDE THE SUBSET OF THE
C GRID SPECIFIED BY IW ARE NOT WRITTEN.
C
C IF GRDFMT IS BLANK THEN THE GRID IS WRITTEN IN UNFORMATTED MODE.
C
      INTEGER          GRDFIL,M,N,IGOPT
      REAL             U(M,N),SPVAL
      DOUBLE PRECISION X0,X1,Y0,Y1,H
      INTEGER*2        IW(0:M+1,0:N+1)
      CHARACTER*40     GRDFMT
C
C---LOCAL VARIABLES---
      INTEGER          I,J
      DOUBLE PRECISION X,Y
      CHARACTER*40     BLANK
C
      DATA             BLANK/' '/
C
C---WRITE GRID BY ROWS---
      IF (IGOPT.NE.0) GOTO 30
      DO 20 J=1,N
      DO 10 I=1,M
      IF (IW(I,J).LT.0) U(I,J)=SPVAL
  10  CONTINUE
      IF (GRDFMT.EQ.BLANK) THEN
         WRITE(GRDFIL) (U(I,J),I=1,M)
      ELSE
         WRITE(GRDFIL,GRDFMT,ERR=80) (U(I,J),I=1,M)
      ENDIF
  20  CONTINUE
      RETURN
C
C---WRITE GRID IN X,Y,Z FORMAT---
  30  DO 50 J=1,N
      Y=Y0 + (J-1)*H
      DO 50 I=1,M
      X=X0 + (I-1)*H
      IF (IW(I,J).LT.0) GOTO 50
      IF (GRDFMT.EQ.BLANK) THEN
         WRITE(GRDFIL) X,Y,U(I,J)
      ELSE
         WRITE(GRDFIL,GRDFMT,ERR=80) X,Y,U(I,J)
      ENDIF
  50  CONTINUE
      RETURN
C
C---ERROR CONDITION---
  80  STOP 'WRGRID - OUTPUT GRID FORMAT ERROR'
      END
      SUBROUTINE RESID1(OUTFIL,NFIL,DATFIL,DATNAM,DATFMT,RESFIL,U,IW,
     +                  M,N,X0,X1,Y0,Y1,H,Z0,Z1,RES,NW,NM)
C
C CALCULATES ROOT MEAN SQUARE RESIDUAL FROM THE FITTED GRID, USING
C BILINEAR INTERPOLATION ACROSS GRID CELLS, OF ALL DATA POINTS LYING
C WITHIN THE SUBSET OF THE GRID SPECIFIED BY IW AND WRITES ALL
C INDIVIDUAL RESIDUALS LARGER THAN RES TO RESFIL FOR POSSIBLE 
C CORRECTION OF ERRORS.
C
      INTEGER          OUTFIL,NFIL,DATFIL,RESFIL,M,N,NW,NM
      REAL             U(M,N),Z0,Z1,RES
      DOUBLE PRECISION X0,X1,Y0,Y1,H
      INTEGER*2        IW(0:M+1,0:N+1)
      CHARACTER*40     DATNAM,DATFMT
C
C---LOCAL VARIABLES---
      INTEGER          M1,N1,I1,I2,J1,J2,K,NUM1,NUM2
      REAL             ZERO,ONE,A,B,Z,ZA,ZB,ZC,DZ,RMS
      DOUBLE PRECISION X,Y,SUM
      CHARACTER*40     RFMT
C
      DATA             ZERO,ONE/0.0, 1.0/
C
C---INITIALIZE---
      WRITE(OUTFIL,10) NFIL,DATNAM
  10  FORMAT(/' DATA FILE',I3,': ',A40)
      OPEN(DATFIL,FILE=DATNAM,STATUS='OLD')
      REWIND DATFIL
      WRITE(RFMT,20) NW,NM
  20  FORMAT('(1X,I7,2F',I2,'.',I1,',3F8.2)')
      M1=M - 1
      N1=N - 1
      NUM1=0
      NUM2=0
      SUM=ZERO
      K=0
C
C---READ LOOP---
  25  K=K + 1
      READ(DATFIL,DATFMT,END=40) X,Y,Z
      IF (X.LT.X0 .OR. X.GT.X1 .OR. Y.LT.Y0 .OR. Y.GT.Y1
     +     .OR. Z.LT.Z0 .OR. Z.GT.Z1) GOTO 30
      A=(X - X0)/H + ONE
      B=(Y - Y0)/H + ONE
      I1=MIN0(INT(A), M1)
      J1=MIN0(INT(B), N1)
      IF (IW(I1,J1).LT.0) GOTO 30
      NUM1=NUM1 + 1
      A=A - I1
      B=B - J1
      I2=MIN0(I1+1,M1)
      J2=MIN0(J1+1,N1)
      ZA=(ONE - A)*U(I1,J1) + A*U(I2,J1)
      ZB=(ONE - A)*U(I1,J2) + A*U(I2,J2)
      ZC=(ONE - B)*ZA + B*ZB
      DZ=ZC - Z
      SUM=SUM + DZ*DZ
      IF (ABS(DZ).LE.RES) GOTO 30
      NUM2=NUM2 + 1
      WRITE(RESFIL,RFMT,ERR=90) K,X,Y,Z,ZC,DZ
  30  GOTO 25
C
  40  IF (NUM1.LE.0) GOTO 70
      RMS=SUM/NUM1
      WRITE(OUTFIL,50) SQRT(RMS)
  50  FORMAT(' ROOT MEAN SQUARE RESIDUAL    =',E10.3)
      WRITE(OUTFIL,60) NUM2, (100.0*NUM2)/NUM1, RES
  60  FORMAT(1X,I7,' RESIDUALS (',F5.1,'%) EXCEED ',E10.3)
  70  CLOSE(DATFIL)
      RETURN
C
C---ERROR CONDITION---
  90  WRITE(OUTFIL,95) RFMT
  95  FORMAT(/' RESID1 - ERROR WRITING RESIDUAL FILE'/
     +        ' RUN TIME FORMAT = ',A40)
      STOP
      END
