/*  Bilhdr_.c     By:  Kevin S. Larson     Created:  June 22, 1992   */
/*            Update:  Kevin S. Larson        Date:  Nov. 18, 1992
    Added:
      Error checking on whether the output files exist, and if
      an error occurs during output. 
*/

/*            Update:  Kevin S. Larson        Date:  Nov. 24, 1992

    Added:
    Computes the minimum and maximun values in the array containing the
    elevation values.  Also computes the mean and standard deviations.

    Printing out all the file LAS2BIL is, which can be used by ARC/INFO,
    this output can be read directly by BIL2LAS.*/


/*  This subprogram will be used to print elevation values from Anudem in
    a i*2 format in bil form.

    Based on gridname, a .bil and .hdr extention will be appended to 
    gridname, if no .bil extention is present, or if one is, copy the
    name, truncate the .bil extention and append a .hdr extention.
    The .hdr is used by Imagegrid, Image, and Imageview in ARC/Info, that
    is way the first part the there name must be identical.

    No values are changed.
  
    Please note that this subroutine is being called from anudem.f,
    where Fortran stores it's arrays as row major and C is in 
    column major.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>



#define max 84               /* Maximun size of output file names.
				   Based on the length of the name string
				   in ANUDEM.F of 80, if this changes, this
				   max size must also change. */

#define errorflag 1          /* An error has occured during processing.*/

   char bilname[max],           /* Used for holding the .bil file name. */
	hdrname[max],           /* Used for holding the .hdr file name. */
	stxname[max],           /* Used for holding the .stx file name. */
	bilwname[max],          /* Used for holding the .bilw file name. */
	tikname[max],           /* Used for holding the .tik file name. */
	output[512],            /* Used to output .hdr information.  Data
				   should not exceed 512 bytes in length. */
	gridname2[max-4];       /* Used to contain gridname with a end of
				   string marker. */

   FILE *bilout,                /* Pointer to bilname. */
	*stxout,                /* Pointer to stxname. */
	*bilwout,               /* Pointer to bilwname. */
	*tikout,                /* Pointer to tikname. */
	*hdrout, *fopen();      /* Pointer to hdrname. */


   /* The following are error routines to stop execution of the program,
      the routine called depends on the last file opened.  They will
      remove the last opened file. */

   void early_term();    /* Early termination routine. */

   void bilerror();

   void hdrerror();

   void bilwerror();

   void stxerror();

   void tikerror();




/***********************************************************************
*****   Passed Parameters:                                         *****
*****    Gridname - Contains the user entered grid name.           *****
*****    ncols - The number of colomns present in the grid.        *****
*****    nrows - The number of rows present in the grid.           *****
*****    xmin - The minimum x value in the coordinate system       *****
*****           entered by the user.                               *****
*****    xmax - The maximum x value in the coordinate system       *****
*****           entered by the user.                               *****
*****    ymin - The minimum y value in the corrdinate system       *****
*****           entered by the user.                               *****
*****    ymax - The maximum y value in the coordinate system       *****
*****           entered by the user.                               *****
*****    gridsp - The size of each square grid cell based on the   *****
*****             coordinate system used by x and y.               *****
*****    u[] - Array containing the grid elevation values.         *****
*****    iw[] - Array indicating the used of special values in     *****
*****             u[].                                             *****
*****    spval - Special value to be entered into u[], based on    *****
*****            iw[].                                             *****
*****    namelen - The length of the character array declaration   *****
*****              of gridname.                                    *****
***********************************************************************/

/* PLEASE NOTE:  In Anudem, u and iw are 2 dimentional arrays, u
                 is declared as u(iz,iz) and iw is declared as
                 iw(0:iz+1,0:iz+1).  When the arrays where filled
                 in subroutine calls, they where declared as a 
                 subsection the array, u as u(m,n) and iw as
                 iw(0:m+1,0:n+1).  Because of this, u and iw are
                 brought in as a single dimension array and indexed
                 as a 2 dimensional array.  Since as arrays, u is
                 located inside of iw when indexing, and nrows and 
                 ncols are based on u, 2 must be added to these values
                 for the array size of iw. */

void bilhdr_(char *gridname, long *ncols, long *nrows, double *xmin, 
             double *xmax, double *ymin, double *ymax, double *gridsp,
             float (*u), short (*iw), float *spval, int namelen)

{

   long
       i,                       /* Column loop counter. */
       j,                       /* Row loop counter. */
       k,                       /* Counter. */
       minval,                  /* Minimum elvation value. */
       maxval,                  /* Maximun elevation value. */
       index,                   /* Used for indexing u[]. */
       index2,                  /* Used for indexing iw[]. */
       flag,                    /* Flag value used in indicating the end
                                   of the character string in gridname. */
       tmp,                     /* Temporary value. */
       num_ele,                 /* # of elevations present. */
       rowbytes,                /* # of bytes per line of output. Default
				   with one band. */
       length;                  /* Length of gridname. */

   short datasize = 2,          /* # of bytes for output data size,
				   current default is 2 for short integers. */
	 elev[1];               /* Used for outputing u[][] as a i*2 bil. */

   double 
       elev_double[1],          /* Used to contain elev[] as a floating point.*/
       mean = 0,                /* Mean elevation value. */
       sumx = 0,                /* Sum of all the values in u[]. */
       sumx_2 = 0,              /* Sum of all the values in u[] sqared. */
       std;                     /* Standard Deviation. */


   /* Initializing values. */
   flag = 0;
   i = 0;
   rowbytes = *ncols * datasize;

   /* Copying gridname to gridname2, based on the passed name length.*/
   strncpy(gridname2,gridname,namelen);
  
   /* Determining the location the the last character in gridname2.*/
   while ((flag == 0) && (i < namelen))  
       if (gridname2[i] == ' ') 
       {
          gridname2[i] = 0;
          flag = 1;
       }
       else
          i++;
   /* Setting the length of gridname2. */
   length = i;

   /* Determining how the create bilname and hdrname. */
   if (strstr(gridname2, ".bil") == NULL)
   {
      strcpy(bilname,gridname2);
      strcpy(hdrname,gridname2);
      strcpy(bilwname,gridname2);
      strcpy(stxname,gridname2);
      strcpy(tikname,gridname2);
      strcat(bilname,".bil");
      strcat(hdrname,".hdr");
      strcat(bilwname,".bilw");
      strcat(stxname,".stx");
      strcat(tikname,".tik");
   }
   else
   {
      strcpy(bilname,gridname2);
      strncpy(hdrname,gridname2,(length - 4));
      hdrname[(length - 4)] = 0;
      strcpy(bilwname,hdrname);
      strcpy(stxname,hdrname);
      strcpy(tikname,hdrname);
      strcat(hdrname,".hdr");
      strcat(bilwname,".bilw");
      strcat(stxname,".stx");
      strcat(tikname,".tik");
   }

   /* Checking if any of the names already exist, and if they do
      flagging it as an ERROR. */
   if ((bilout = fopen(bilname,"rb")) != NULL)
   {
      printf("%s already exists.  Stopping Processing.\n",bilname);
      fclose(bilout);
      early_term();
   }
   if ((hdrout = fopen(hdrname,"r")) != NULL)
   {
      printf("%s already exists.  Stopping Processing.\n",hdrname);
      fclose(hdrout);
      early_term();
   }
   if ((bilwout = fopen(bilwname,"r")) != NULL)
   {
      printf("%s already exists.  Stopping Processing.\n",bilwname);
      fclose(hdrout);
      early_term();
   }
   if ((stxout = fopen(stxname,"r")) != NULL)
   {
      printf("%s already exists.  Stopping Processing.\n",stxname);
      fclose(stxout);
      early_term();
   }
   if ((tikout = fopen(tikname,"r")) != NULL)
   {
      printf("%s already exists.  Stopping Processing.\n",tikname);
      fclose(tikout);
      early_term();
   }


   /*Opening all the output files, if unable to open, returning to subroutine
      call. */
   if ((bilout = fopen(bilname,"wb")) == NULL)
   {
      printf("Error opening %s\n",bilname);
      early_term();
   }
   if ((hdrout = fopen(hdrname,"w")) == NULL)
   {
      printf("Error opening %s\n",hdrname);
      bilerror();
   }
   if ((bilwout = fopen(bilwname,"w")) == NULL)
   {
      printf("Error opening %s\n",bilwname);
      hdrerror();
   }
   if ((stxout = fopen(stxname,"w")) == NULL)
   {
      printf("Error opening %s\n",stxname);
      bilwerror();
   }
   if ((tikout = fopen(tikname,"w")) == NULL)
   {
      printf("Error opening %s\n",tikname);
      stxerror();
   }


   /* Printing message to the screen. */
   printf("Writing to output files.\n\n");

   /* Initializing the minimum and maximun values. */
   minval = (short)u[0];
   maxval = (short)u[0];

   /* Adjusting u[] with spval, based on iw[][]. */
   for (j = ((*nrows) - 1),k = 1; j >= 0; j--,k++)
   {
      for (i = 0; i < (*ncols); i++)
      {
         /* Indexing u. */
         index = i + (j * (*ncols));
         /* Finding subsection of iw that corresponds to u. */
         index2 = ((*ncols) + 2) * (j + 1) + i + 1;
         if (iw[index2] < 0)
	    u[index] = *spval;

         /* Writing information to data file. */
         elev[0] = (short) u[index];
         elev_double[0] = elev[0];
         if ((tmp = fwrite((void *) elev, sizeof(short),1,bilout)) < 1)
         {
            printf("\nError writing to %s\n",bilname);
            printf("Stopping processing\n");
            tikerror();
         }

	 /* Resetting the minimum and maximun values if necessary. */
	 if (elev[0] < minval)
	    minval = elev[0];
	 else
	    if (elev[0] > maxval)
	       maxval = elev[0];

	 /* Computing the Sum values. */
	 sumx = sumx + elev_double[0];
	 sumx_2 = sumx_2 + pow(elev_double[0],2.0);
      }
      printf("\rPercent compete:  %lf                ",((double)k /
                                                                 (*nrows)));
   }

   /* Computing the standard deviation. */
   num_ele = (*ncols) * (*nrows);

   /* Computational formula for standard deviation; p. 190;
      Introduction to Probabilities and Statistics; J. S. Milton,
      Jesse C. Arnold; Second Edition */
   std = sqrt(fabs(((double)num_ele * sumx_2 - pow(sumx,2.0))/
                                           ((double)num_ele * (num_ele - 1))));

   /* Computing the mean value. */
   mean = (double) sumx / num_ele;


   /* Writing out bilname in bil format. */
   /* NOT USING to speed up processing, writing infomration out above. */
/*   for (j = ((*nrows) - 1); j >= 0; j--)
   {
      for (i = 0; i <= ((*ncols) - 1); i++)
      {
         index = i + (j * (*ncols));
         elev[0] = (short) u[index];
         if ((tmp = fwrite((void *) elev, sizeof(short),1,bilout)) < 1)
         {
	    printf("\nError writing to %s\n",bilname);
            printf("Stopping processing\n");
	    tikerror();
	 }
      }
      printf("\rPercent compete:  %lf                ",((double)(j + 1) / 
                                                                 (*nrows)));
   }
*/

   /* Writing out hdrname, a .hdr for ARC/Info. */
   sprintf(output,"byteorder M\nlayout BIL\n");
   sprintf(output,"%snrows %ld\nncols %ld\n",output, *nrows, *ncols);
   sprintf(output,"%snbands 1\nnbits 16\n",output);
   sprintf(output,"%sbandrowbytes %ld\ntotalrowbytes %ld\n",
	   output,rowbytes,rowbytes);
   sprintf(output,"%sulxmap %35.25lf\n",output, *xmin);
   sprintf(output,"%sulymap %35.25lf\n",output, *ymax);
   sprintf(output,"%sxdim %35.25lf\nydim %35.25lf\n",output,*gridsp, *gridsp);

   if (fprintf(hdrout,"%s",output) < strlen(output))
   {
      printf("\nError writing to %s.  Stopping processing.\n",hdrname);
      tikerror();
   }

   /* Writing out stxname. */
   if (fprintf(stxout,"1 %25ld.0 %25ld.0 %35.25lf %35.25lf\n",minval,
	       maxval,mean,std) < 0)
   {
      printf("\nError writing to %s.\n",stxname);
      tikerror();
   }

   /* Setting values for tikname. */
   sprintf(output,"ulxmap %35.25lf\nulymap %35.25lf\nurxmap %35.25lf\n",
	   *xmin,*ymax,*xmax);
   sprintf(output,"%surymap %35.25lf\nllxmap %35.25lf\nllymap %35.25lf\n",
	   output,*ymax,*xmin,*ymin);
   sprintf(output,"%slrxmap %35.25lf\nlrymap %35.25lf\n",
	   output,*xmax,*ymin);
   sprintf(output,"%sxdim %35.25lf\nydim %35.25lf\n",
	   output,*gridsp,*gridsp);

   /* Writing out tikname. */
   if (fprintf(tikout,"%s",output) < 0)
   {
      printf("\nError writing to %s.\n",tikname);
      tikerror();
   }

   /* Setting values for bilwname. */
   /* Zero's represent the rotation values.  They should be 0. */
   sprintf(output,"%35.25lf\n%35.25lf\n%35.25lf\n%35.25lf\n%35.25lf\n%35.25lf\n",
	   *gridsp,0.0,0.0,(*gridsp * -1),*xmin,*ymax);

   /* Writing out bilwname. */
   if (fprintf(bilwout,"%s",output) < 0)
   {
      printf("\nError writing to %s.\n",bilwname);
      tikerror();
   }


   printf("\rProcess complete.                                      \n");
   /* Closing the output file.*/
   fclose(bilout);
   fclose(hdrout);
   fclose(bilwout);
   fclose(stxout);
   fclose(tikout);

   /* Returning to calling point. */
   return;
}


/* Terminates run for program. */
void early_term()
{
   exit(EXIT_FAILURE);        /*   EARLY TERMINATION. */
}


/* Removing .bil file. */
void bilerror()
{
   fclose(bilout);
   sprintf(output,"rm %s",bilname);
   system(output);
   early_term();
}



/* Removing .hdr file. */
void hdrerror()
{
   fclose(hdrout);
   sprintf(output,"rm %s",hdrname);
   system(output);
   bilerror();
}


/* Removing .bilw file. */
void bilwerror()
{
   fclose(bilwout);
   sprintf(output,"rm %s",bilwname);
   system(output);
   hdrerror();
}


/* Removing .stx file. */
void stxerror()
{
   fclose(stxout);
   sprintf(output,"rm %s",stxname);
   system(output);
   bilwerror();
}



/* Removing .tik file. */
void tikerror()
{
   fclose(tikout);
   sprintf(output,"rm %s",tikname);
   system(output);
   stxerror();
}
