      SUBROUTINE RDCONT(NFIL,OUTFIL,DATFIL,DATNAM,HEDFMT,DATFMT,CONFIL,
     +            IW,M,N,X0,X1,Y0,Y1,H,Z0,Z1,X,Y,MAXD,MAXB,NCON,NZ,ZMN)
C
C SUBROUTINE READS CONTOUR LINES FROM DATFIL IN A USER DEFINED FORMAT.
C PORTIONS OF CONTOUR LINES OUTSIDE THE X,Y LIMITS ARE REMOVED.
C REMAINING CONTOUR COORDINATES ARE GENERALIZED TO THE RESOLUTION
C OF THE GRID AND WRITTEN TO THE SCRATCH FILE CONFIL.
C
      INTEGER          NFIL,OUTFIL,CONFIL,DATFIL,MAXD,MAXB,NCON,NZ
      REAL             Z0,Z1
      DOUBLE PRECISION H,X0,X1,Y0,Y1,X(MAXD),Y(MAXD),ZMN
      INTEGER*2        IW(0:M+1,0:N+1)
      CHARACTER*40     DATNAM,HEDFMT,DATFMT
C
C---LOCAL VARIABLES---
      INTEGER          NC,NUM,NN,I,J,K,K1,KK,N1,N2,N3
      REAL             Z,ONE,ONE5
C
      DATA             ONE,ONE5/1.0, 1.5/
C
C---INITIALIZE---
      WRITE(OUTFIL,10) NFIL,DATNAM
  10  FORMAT(/' CONTOUR LINE DATA FILE',I3,': ',A40)
      OPEN(DATFIL,FILE=DATNAM,STATUS='OLD')
      REWIND DATFIL
      N1=0
      N2=0
      N3=0
C
C---LOOP THROUGH CONTOUR LINES---
      DO 130 NC=1,MAXD
      READ(DATFIL,HEDFMT,END=200,ERR=220) NUM,Z
      IF (NUM.GT.MAXD) STOP 'RDCONT - TOO MANY POINTS FOR BUFFER'
      READ(DATFIL,DATFMT,ERR=230) (X(K),Y(K),K=1,NUM)
      N1=N1 + NUM
C
C---CHECK FOR ERRORS---
      IF (Z.LT.Z0 .OR. Z.GT.Z1) THEN
         WRITE(OUTFIL,20) NC,Z
  20     FORMAT(/' ILLEGAL HEIGHT FOR',I5,'TH CONTOUR LINE:'/
     +           '   Z  = ',F10.3)
         GOTO 130
      ENDIF
C
C---TRIM LINE TO X/Y LIMITS AND POLYGON---
      K1=1
  50  DO 60 K=K1,NUM
      IF (X(K).LT.X0 .OR. X(K).GT.X1 .OR. Y(K).LT.Y0 .OR.
     +    Y(K).GT.Y1) GOTO 60
      I=(X(K) - X0)/H + ONE5
      J=(Y(K) - Y0)/H + ONE5
      IF (IW(I,J).GE.0) GOTO 70
  60  CONTINUE
      GOTO 130
C
C---LINE WITHIN LIMITS---
  70  KK=0
      DO 80 K1=K,NUM
      IF (X(K1).LT.X0 .OR. X(K1).GT.X1 .OR. Y(K1).LT.Y0 .OR.
     +    Y(K1).GT.Y1) GOTO 100
      I=(X(K1) - X0)/H + ONE5
      J=(Y(K1) - Y0)/H + ONE5
      IF (IW(I,J).LT.0) GOTO 100
      KK=KK + 1
      X(KK)=X(K1) - X0
      Y(KK)=Y(K1) - Y0
  80  CONTINUE
 100  NN=KK
      N2=N2 + NN
      IF (NN.LE.0) GOTO 120
C
C---GENERALIZE LINE TO RESOLUTION OF GRID---
      CALL SIMSTR(NN,X,Y,MAXD,H)
      N3=N3 + NN
      IF (NN.LE.0) GOTO 120
C
C---WRITE SELECTED POINTS TO CONFIL---
      IF (NN.GT.MAXB) GOTO 240
      NCON=NCON + 1
      WRITE(CONFIL) NN,Z
      DO 110 K=1,NN
      WRITE(CONFIL) X(K),Y(K)
 110  CONTINUE
C
C---UPDATE MEAN Z---
      NZ=NZ + NN
      ZMN=ZMN + NN*Z
C
C---CHECK FOR REMAINING POINTS---
 120  IF (K1.LT.NUM) GOTO 50
 130  CONTINUE
      STOP 'RDCONT - TOO MANY CONTOUR LINES'
C
C---FINISH UP---
 200  CLOSE(DATFIL)
      WRITE(OUTFIL,210) N1,N2,N3
 210  FORMAT(' NUMBER OF CONTOUR LINE PTS      =',I7/
     +       ' NUMBER OF POINTS WITHIN LIMITS  =',I7/
     +       ' NUMBER OF POINTS ACCEPTED       =',I7)
      RETURN
C
C---ERROR CONDITIONS---
 220  WRITE(OUTFIL,225) NC
 225  FORMAT(/' ERROR READING',I5,' CONTOUR LINE HEADER')
      STOP 'RDCONT - ERROR'
 230  WRITE(OUTFIL,235) NC
 235  FORMAT(/' ERROR READING',I5,' CONTOUR LINE')
      STOP 'RDCONT - ERROR'
 240  WRITE(OUTFIL,245) NUM,MAXB
 245  FORMAT(/' WINDOWED CONTOUR SIZE',I6,' EXCEEDS MAXIMUM',I6)
      STOP 'RDCONT - ERROR'
      END
      SUBROUTINE SELCON(OUTFIL,CONFIL,NCON,U,IW,MAXW,IV,IC,MAXC,NC,
     +                  M,N,H,Z0,Z1,IP,JP,DP,X,Y,T,MAXB,J3,CTOL)
C
C READS CONTOUR LINES FROM THE UNFORMATTED FILE CONFIL AND
C INCORPORATES THE CORRESPONDING CONTOUR HEIGHTS AND DATA POINT
C INDICATORS INTO THE GRIDS U,IW AND IV.
C IF NC IS AT LEAST 0 THEN POINTS OF LOCAL EXTREME CURVATURE ARE
C OBTAINED BY SUBROUTINE GETCRT AND THE NUMBER OF SUCH POINTS IS
C RETURNED IN NC.  OTHERWISE NC IS UNCHANGED.
C
      INTEGER          OUTFIL,CONFIL,NCON,MAXC,NC,M,N,IP(8),JP(8),
     +                 IC(6,MAXC),MAXB,J3
      REAL             U(M,N),Z0,Z1,CTOL
      DOUBLE PRECISION H,X(MAXB),Y(MAXB),T(MAXB)
      INTEGER*2        IW(0:M+1,0:N+1),IV(M,N),MAXW
C
C---LOCAL VARIABLES---
      INTEGER          NN,NUM,I,J,I1,J1,I2,J2,K,KL,KK,K1,K2
      REAL             Z,U1,ONE5
C
      DATA             ONE5/1.5/
C
C---INITIALIZE---
      REWIND CONFIL
C
C---LOOP THROUGH CONTOUR LINES---
      DO 50 NN=1,NCON
      READ(CONFIL,ERR=100) NUM,Z
      IF (NUM.GT.MAXB) STOP 'SELCON - ERROR'
      IF (Z.LT.Z0 .OR. Z.GT.Z1) GOTO 110
      DO 10 K=1,NUM
      READ(CONFIL) X(K),Y(K)
  10  CONTINUE
C
C---GENERALIZE CONTOUR LINE---
      CALL SIMSTR(NUM,X,Y,MAXB,H)
      IF (NUM.LE.0) GOTO 50
C
C---OPTIONALLY GET CRITICAL POINTS FOR CONTOUR---
      IF (NC.GE.0) THEN
         CALL GETCRT(IC,MAXC,NC,H,IP,JP,DP,X,Y,T,NUM,J3,CTOL)
      ENDIF
C
C---SET HEIGHTS FOR CONTOUR---
      CALL SETBES(U,Z,IW,MAXW,M,N,H,IP,JP,X,Y,NUM)
  50  CONTINUE
C
C---ADJUST CRITICAL POINTS---
      DO 80 K=1,NC
      I=IC(1,K)
      J=IC(2,K)
      KL=IC(3,K)
      I1=I + IP(KL)
      J1=J + JP(KL)
C
C---MOVE POINTERS WHICH POINT TO DATA POINTS---
      IF (IW(I1,J1).GT.0) THEN
         U1=U(I1,J1)
         DO 60 KK=0,6,6
         K1=MOD(KL+KK,8) + 1
         I1=I + IP(K1)
         J1=J + JP(K1)
         IF (IW(I1,J1).NE.0) GOTO 60
         K2=MOD(K1+KK,8) + 1
         I1=I + IP(K2)
         J1=J + JP(K2)
         IF (IW(I1,J1).LE.0 .OR. ABS(U(I1,J1)-U1).GT.0.01) GOTO 60
         KL=K1
         IC(3,K)=KL
         GOTO 70
  60     CONTINUE
      ENDIF
C
C---INSERT DATA POINT IN SMOOTHED CORNERS---
  70  IF (IW(I,J).EQ.0) THEN
         K1=MOD(KL,8) + 1
         I1=I + IP(K1)
         J1=J + JP(K1)
         K1=MOD(KL+6,8) + 1
         I2=I + IP(K1)
         J2=J + JP(K1)
         IF (IW(I1,J1).GT.0 .AND. IW(I2,J2).GT.0
     +        .AND. ABS(U(I1,J1)-U(I2,J2)).LT.0.01) THEN
            IW(I,J)=1
            U(I,J)=U(I1,J1)
         ENDIF
      ENDIF
  80  CONTINUE
      RETURN
C
C---ERROR CONDITIONS---
 100  STOP 'SELCON - ERROR READING CONFIL'
 110  STOP 'SELCON - ILLEGAL CONTOUR HEIGHT'
      END
      SUBROUTINE GETCRT(IC,MAXC,NC,H,IP,JP,DP,X,Y,T,NUM,J3,TOL)
C
C OBTAINS POINTS OF LOCAL EXTREME CURVATURE ALONG THE LINE WITH
C COORDINATES GIVEN BY THE ARRAYS X,Y.  THE ARRAY POSITION OF
C EACH CRITICAL POINT AND THE CORRESPONDING NORMAL DIRECTION
C ARE STORED IN THE ARRAY IC.  POINTS CLOSER THAN TOL TO
C PREVIOUSLY DETERMINED CRITICAL POINTS ARE IGNORED.
C
      INTEGER          MAXC,IC(6,MAXC),NC,IP(8),JP(8),NUM,J3
      REAL             DP(8),TOL
      DOUBLE PRECISION H,X(NUM+1),Y(NUM+1),T(NUM+1)
C
C---LOCAL VARIABLES---
      INTEGER          J,K
      REAL             ZERO,ONE,TWO,FOUR,ONE5,DS1,DS2,DX,DX1,DX2,
     +                 DY,DY1,DY2,P,Q,PC,QC,DDX,DDY,Z,Z1,ZC,ZC1,
     +                 DZ,DZ1
      DOUBLE PRECISION XC,YC,TC,TC1
C
      DATA             ZERO,ONE,TWO,FOUR/0.0, 1.0, 2.0, 4.0/
      DATA             ONE5/1.5/
C
C---CHECK FOR CLOSED LINE STRING---
      IF (NUM.LT.2) RETURN
      IF (X(1).EQ.X(NUM) .AND. Y(1).EQ.Y(NUM)) THEN
C
C---STRING CLOSED---
         X(NUM+1)=X(2)
         Y(NUM+1)=Y(2)
      ELSE
C
C---STRING NOT CLOSED---
         X(NUM+1)=TWO*X(NUM) - X(NUM-1)
         Y(NUM+1)=TWO*Y(NUM) - Y(NUM-1)
      ENDIF
C
C---CALCULATE PATH LENGTHS---
      T(1)=ZERO
      DO 10 I=2,NUM+1
      DX=X(I) - X(I-1)
      DY=Y(I) - Y(I-1)
      T(I)=T(I-1) + SQRT(DX*DX + DY*DY)
  10  CONTINUE
C
C---LOOP THROUGH LINE STRING---
      J1=1 + J3
      J2=(NUM/J3)*J3 + 1 - J3
      Z1=ZERO
      ZC=ZERO
      TC=ZERO
      TC1=ZERO
      DO 40 J=J1,J2
      DS1=T(J) - T(J-J3)
      DS2=T(J+J3) - T(J)
      DX1=(X(J) - X(J-J3))/DS1
      DX2=(X(J+J3) - X(J))/DS2
      DY1=(Y(J) - Y(J-J3))/DS1
      DY2=(Y(J+J3) - Y(J))/DS2
      DDX=(DX2 - DX1)/(DS1 + DS2)
      P=(DX1*DS2 + DX2*DS1)/(DS1 + DS2)
      DDY=(DY2 - DY1)/(DS1 + DS2)
      Q=(DY1*DS2 + DY2*DS1)/(DS1 + DS2)
C
C---CALCULATE CROSS-PRODUCT OF TANGENT AND CURVATURE---
      Z=(P*DDY - Q*DDX)/(P*P + Q*Q)**ONE5
      IF (ZC.GE.AMAX1(Z1,Z,ZERO) .OR. ZC.LE.AMIN1(Z1,Z,ZERO)) THEN
C
C---STORE POINT OF LOCAL EXTREME CURVATURE---
         IF (TC-TC1.GT.TOL) THEN
            NC=NC + 1
            IF (NC.GE.MAXC) GOTO 100
            IC(1,NC)=XC/H + ONE5
            IC(2,NC)=YC/H + ONE5
            DZ1=FOUR
            DO 20 K=1,8
            DX=PC - IP(K)*DP(K)
            DY=QC - JP(K)*DP(K)
            DZ=DX*DX + DY*DY
            IF (DZ.LT.DZ1) THEN
               DZ1=DZ
               IC(3,NC)=K
            ENDIF
  20        CONTINUE
            IF (ZC.GT.ZERO) THEN
               IC(3,NC)=MOD(IC(3,NC)+1,8) + 1
            ELSE
               IC(3,NC)=MOD(IC(3,NC)+5,8) + 1
            ENDIF
            IC(4,NC)=0
            IC(5,NC)=0
            IC(6,NC)=0
            ZC1=ZC
            TC1=TC
         ENDIF
      ENDIF
      XC=X(J)
      YC=Y(J)
      TC=T(J)
      PC=P
      QC=Q
      Z1=ZC
      ZC=Z
  40  CONTINUE
      RETURN
C
C---ERROR CONDITION---
 100  WRITE(6,110)
 110  FORMAT(/' GETCRT - TOO MANY CRITICAL POINTS')
      STOP
      END
      SUBROUTINE SIMCON(NUM,X,Y,MAXD,H)
C
C GENERALIZES CONTOUR LINE TO RESOLUTION H OF GRID.
C
      INTEGER          NUM,MAXD
      DOUBLE PRECISION X(MAXD),Y(MAXD),H
C
C---LOCAL VARIABLES---
      INTEGER I,J,I1,J1,K,KK
      REAL ONE5
C
      DATA ONE5/1.5/
C
C---GENERALIZE LINE---
      IF (NUM.LE.1) GOTO 90
      KK=1
      I1=X(1)/H + ONE5
      J1=Y(1)/H + ONE5
      DO 20 K=2,NUM
      I=X(K)/H + ONE5
      J=Y(K)/H + ONE5
      IF (I.EQ.I1 .AND. J.EQ.J1) GOTO 20
      KK=KK + 1
      I1=I
      J1=J
      X(KK)=X(K)
      Y(KK)=Y(K)
  20  CONTINUE
      X(KK)=X(NUM)
      Y(KK)=Y(NUM)
      NUM=KK
  90  RETURN
C
C---ERROR CONDITION---
 100  STOP 'SIMCON ERROR'
      END
      SUBROUTINE SETBES(U,Z,IW,MAXW,M,N,H,IP,JP,X,Y,NUM)
C
C SETS HEIGHTS IN U AND DATA POINT INDICATORS IN IW FOR THE
C CONTOUR LINE HELD IN THE ARRAYS X,Y.  INTERMEDIATE POINTS ALONG
C THE LINE ARE INTERPOLATED USING CUBIC BESSEL INTERPOLATION.
C
      INTEGER          M,N,IP(8),JP(8),NUM
      REAL             U(M,N),Z
      DOUBLE PRECISION H,X(NUM+1),Y(NUM+1)
      INTEGER*2        IW(0:M+1,0:N+1),MAXW
C
C---LOCAL VARIABLES---
      INTEGER          I0,J0,I1,J1,I,J,K,KL,NN
      REAL             ZERO,ONE5,TWO,D11,D12,E11,E12,DT1,DT2,
     +                 P1,P2,Q1,Q2,D20,D21,E20,E21,D30,E30,T
      DOUBLE PRECISION D01,D02,D03,E01,E02,E03,F
C
      DATA             ZERO,ONE5,TWO/0.0, 1.5, 2.0/
C
C---INTERPOLATION POLYNOMIAL---
      F(D01,P1,D20,D30)=D01 + (P1 + (D20 + D30*(T - DT1))*T)*T
C
C---INITIALISE DIVIDED DIFFERENCES---
      IF (NUM.LT.2) RETURN
      D02=X(1)
      D03=X(2)
      E02=Y(1)
      E03=Y(2)
      IF (D02.EQ.X(NUM) .AND. E02.EQ.Y(NUM)) THEN
C
C---STRING CLOSED---
         D01=X(NUM-1)
         X(NUM+1)=D03
         E01=X(NUM-1)
         Y(NUM+1)=E03
      ELSE
C
C---STRING NOT CLOSED---
         D01=TWO*D02 - D03
         X(NUM+1)=TWO*X(NUM) - X(NUM-1)
         E01=TWO*E02 - E03
         Y(NUM+1)=TWO*Y(NUM) - Y(NUM-1)
      ENDIF
C
C---REMAINING DIFFERENCES---
      D11=D02 - D01
      E11=E02 - E01
      DT1=SQRT(D11*D11 + E11*E11)
      D11=D11/DT1
      E11=E11/DT1
      D12=D03 - D02
      E12=E03 - E02
      DT2=SQRT(D12*D12 + E12*E12)
      D12=D12/DT2
      E12=E12/DT2
      P2=(D11*DT2 + D12*DT1)/(DT1 + DT2)
      Q2=(E11*DT2 + E12*DT1)/(DT1 + DT2)
      D21=(D12 - D11)/(DT1 + DT2)
      E21=(E12 - E11)/(DT1 + DT2)
C
C---SET FIRST POSITION---
      I1=D02/H + ONE5
      J1=E02/H + ONE5
      I0=-1
      J0=-1
C
C---LOOP THROUGH LINE STRING---
      DO 70 K=3,NUM+1
      D01=D02
      D02=D03
      D03=X(K)
      E01=E02
      E02=E03
      E03=Y(K)
      D11=D12
      D12=D03 - D02
      E11=E12
      E12=E03 - E02
      DT1=DT2
      DT2=SQRT(D12*D12 + E12*E12)
      D12=D12/DT2
      E12=E12/DT2
      D20=D21
      D21=(D12 - D11)/(DT1 + DT2)
      D30=(D21 - D20)/DT1
      P1=P2
      P2=(D11*DT2 + D12*DT1)/(DT1 + DT2)
      E20=E21
      E21=(E12 - E11)/(DT1 + DT2)
      E30=(E21 - E20)/DT1
      Q1=Q2
      Q2=(E11*DT2 + E12*DT1)/(DT1 + DT2)
      I=D02/H + ONE5
      J=E02/H + ONE5
      NN=1
      DT=DT1
      IF (IABS(I-I1).LE.1 .AND. IABS(J-J1).LE.1) GOTO 30
  20  NN=MAX1(2.0*DT1/SNGL(H), 2.0)
      DT=DT1/NN
C
C---REFINE LINE TO RESOLUTION OF GRID---
  30  T=ZERO
      DO 60 KK=1,NN
      T=T + DT
      I=F(D01,P1,D20,D30)/H + ONE5
      J=F(E01,Q1,E20,E30)/H + ONE5
  40  IF (I.EQ.I1 .AND. J.EQ.J1) GOTO 60
      IF (IABS(I-I0).LE.1 .AND. IABS(J-J0).LE.1) THEN
         I1=I0
         I0=-1
         J1=J0
         J0=-1
         GOTO 40
      ENDIF
      DO 41 KL=1,8
      IF (I-I1.EQ.IP(KL) .AND. J-J1.EQ.JP(KL)) GOTO 50
  41  CONTINUE
      IF (NN.EQ.1) GOTO 20
      STOP 'SETBES - ERROR'
C
C---SET POINTER AND UPDATE INDICES---
  50  IF (IW(I1,J1).GE.0 .AND. IW(I1,J1).LT.MAXW) THEN
         U(I1,J1)=(IW(I1,J1)*U(I1,J1) + Z)/(IW(I1,J1) + 1)
         IW(I1,J1)=IW(I1,J1) + 1
      ENDIF
      I0=I1
      I1=I
      J0=J1
      J1=J
  60  CONTINUE
  70  CONTINUE
C
C---SET FINAL POINT---
      IF (IW(I1,J1).GE.0 .AND. IW(I1,J1).LT.MAXW) THEN
         U(I1,J1)=(IW(I1,J1)*U(I1,J1) + Z)/(IW(I1,J1) + 1)
         IW(I1,J1)=IW(I1,J1) + 1
      ENDIF
      RETURN
      END
