/******************************************************************************
NAME:	GETNAME

FUNCTION:
	GETNAM is a routine which retrieves a file name specification from
	the fname parameter.  It appends the specified extension to the 
	name if the extension was not specified.  If the file is to be opened
	for output, a bad status is returned if the file already exists on disk.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0        12/86       K. Gacke     initial development
    1.1        12/87       B. Ailts     Change include directory specifications
					Replace DESC arguments with char. string
					Use raw 'C' types
					Place bridge routines in seperate file
    5.0        04/88	   D. Hollaren  changed include spec to las
					changed cerrmsg to c_errmsg and
					passed status flag by address
    5.1		09/88	   B. Ailts	Now returns a null value in the host
					name when the parameter has a null value
    5.2		10/88	   B. Ailts	Added argument to access multiple
					dimensioned parameters
     5.3        07/90      D. VanderZee Standardized error handling 
     5.4	01/93	   R. Kruse	Modified support c_getname.  Refrences
					to vblock and other tae vars were taken
					out.  fname is now (i/o).


PROJECT:	 LAS

ALGORITHM:
	Replace any aliases within the TAE parameter
	If it does not have the specified extension,
	    the extension is appended.
	The full host file name is retrieved.
	If the file is to be opened for output and the file already exists,
	    a bad status code is returned.
	If the file is to be opened for input and the file does not exist,
	    a bad status code is returned.

******************************************************************************/

#include <stdio.h>

#include "las.h"

FUNCTION getname(fname, ext, access)
	char *fname;		/* Host file name to be returned (I/O)  */
	char *ext;		/* default extension             (I)  */
        long access;		/* Type of I/O access            (I)  */
{
long  mode = 0;
long  nonfatal = 1;
long  length;
char msg[ERRLEN+1];
char *ptr;
char tfull[CMLEN];
char tname[CMLEN];
char *strchr(), *strrchr();


if (c_getalias(fname,tname) != E_SUCC)
	{
	c_errmsg("Error returned from getalias","anudem-alias",&nonfatal);
	return (E_FAIL);
	}
if (strchr(tname, TAE_DIR_BEG) || strchr(tname, TAE_EXT)) /* VMS */
   {
   if (strrchr(tname, TAE_EXT) == NULL)   /* if no extension add default*/
      sprintf(tfull, "%s%c%s", tname, TAE_EXT, ext);
   else
      strcpy(tfull, tname);
   }
else /* Unix */
   {
   /* Since dots can reside in the path as well as the filename,
      we need to ensure that we are checking for dots (as extensions)
      after any path that may be included.
      ---------------------------------------------------------------*/
   if ((ptr = strrchr(tname, HOST_DIR_SEP)) == NULL)
      ptr = tname;
   if (strrchr(ptr, HOST_EXT) == NULL)  /* if no extension add default*/
      sprintf(tfull, "%s%c%s", tname, HOST_EXT, ext);
   else
      strcpy(tfull, tname);
   }

if (c_cmhost(tfull,fname) != E_SUCC)
   {
   c_errmsg("Error returned from cmhost","anudem-cmhost",&nonfatal);
   sprintf(msg,"Error retrieving host file name for %s",tfull);
   c_errmsg(msg,"anudem-cmhost",&nonfatal);
   return(E_FAIL);
   }

if (access == IREAD)			/* if file is opened for read access */
   {					/*   the file must exist             */
   if (c_lsstat(fname,&mode) != E_SUCC)
      {
      sprintf(msg,"Error: %s does not exist",tfull);
      c_errmsg(msg,"anudem-exist",&nonfatal);
      return(E_FAIL);
      }
   }

if (access == IWRITE)			/* if file is opened for write access */
   {					/*   the file cannot exist            */
   if (c_lsstat(fname,&mode) == E_SUCC)
      {
      sprintf(msg,"Error: %s already exists",tfull);
      c_errmsg(msg,"anudem-exists",&nonfatal);
      return(E_FAIL);
      }
   }

return(E_SUCC);
}
