      SUBROUTINE ITERAT(U,IW,IV,M,N,V,Z0,Z1,Z,ND,UMAX,KMAX2,UMAX2,
     +                  IP,JP,DP,T,OMEGA,RHO,SSQ,FAC,ZGAP,MOPT)
C
C PERFORMS ONE GAUSS SEIDEL ITERATION WITH OVERRELAXATION
C PARAMETER OMEGA.
C
      INTEGER          M,N,ND,KMAX2(M,2),
     +                 IP(8),JP(8),MOPT
      REAL             U(M,N),V(M),Z0,Z1,Z(ND),UMAX(M),UMAX2(M,2),
     +                 DP(8),T,OMEGA,RHO,FAC,ZGAP
      DOUBLE PRECISION SSQ
      INTEGER*2        IW(0:M+1,0:N+1),IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER          I,J,J1,KD
C
C---INITIALISE---
      DO 10 I=1,M
      KMAX2(I,2)=0
      UMAX2(I,2)=Z1
  10  CONTINUE
      J1=1
      CALL GETMAX(J1,U,IV,M,N,KMAX2,UMAX2,IP,JP,DP)
      KD=0
      SSQ=0.0D0
C
C---LOOP THROUGH ROWS---
      DO 40 J=1,N
C
C---GET MAXIMUM VALUES---
      J1=J + 1
      CALL GETMAX(J1,U,IV,M,N,KMAX2,UMAX2,IP,JP,DP)
      CALL GETNBR(J,U,IW,M,N,Z1,UMAX,IP,JP,MOPT)
C
C---CALCULATE FINITE DIFFERENCE EQUATION COEFFICIENTS---
      CALL RHSIDE(J,U,M,N,V,T)
C
C---SOLVE EQUATION---
      CALL SOLVE(J,U,IW,IV,M,N,V,Z0,Z1,Z,ND,KD,UMAX,KMAX2,
     +           UMAX2,IP,JP,DP,T,OMEGA,RHO,SSQ,FAC,ZGAP)
  40  CONTINUE
      IF (KD.NE.ND) then
	write(6,52) kd,nd
  52	format(//' kd =',i8/' nd =',i8)
      STOP 'ITERAT - ERROR'
	endif
      RETURN
      END
      SUBROUTINE GETMAX(J,U,IV,M,N,KMAX2,UMAX2,IP,JP,DP)
C
C GETS MAXIMUM VALUES DETERMINED BY NEIGHBOURING UPSTREAM ORDERED
C CHAIN POINTS WHICH LIE AHEAD OF THE ITERATION.
C
      INTEGER   J,M,N,KMAX2(M,2),IP(8),JP(8)
      REAL      U(M,N),UMAX2(M,2),DP(8)
      INTEGER*2 IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER   I,KL,IK,JK,JJK
      REAL      UK
C
C---SHIFT KMAX2 AND UMAX2---
      DO 10 I=1,M
      KMAX2(I,1)=KMAX2(I,2)
      KMAX2(I,2)=0
      UMAX2(I,1)=UMAX2(I,2)
  10  CONTINUE
      IF (J.GT.N) RETURN
C
C---TRAVERSE ROW OF U---
      DO 20 I=1,M
      KL=IV(I,J)
      IF (KL.LE.0) GOTO 20
      IF (JP(KL).GT.0) GOTO 20
      IF (JP(KL).EQ.0 .AND. IP(KL).GT.0) GOTO 20
      IK=I + IP(KL)
      JK=J + JP(KL)
      JJK=2 + JP(KL)
      UK=(U(I,J) - U(IK,JK))*DP(KL)
      IF (KMAX2(IK,JJK).LE.0 .OR. UK.LT.UMAX2(IK,JJK)) THEN
         KMAX2(IK,JJK)=KL
         UMAX2(IK,JJK)=UK
      ENDIF
  20  CONTINUE
      RETURN
      END
      SUBROUTINE GETNBR(J,U,IW,M,N,Z1,UMAX,IP,JP,MOPT)
C
C OPTIONALLY GETS MAXIMUM HEIGHT OF THE EIGHT IMMEDIATE NEIGHBOURS.
C IF MOPT=0 THEN THE MAXIMUM HEIGHT IS SET TO Z1.
C
      INTEGER   J,M,N,IP(8),JP(8),MOPT
      REAL      U(M,N),Z1,UMAX(M)
      INTEGER*2 IW(0:M+1,0:N+1)
C
      INTEGER   I,J1,K1
C
C---TRAVERSE ROW OF U---
      IF (MOPT.EQ.0 .OR. J.EQ.1 .OR. J.EQ.N) THEN
         DO 10 I=1,M
         UMAX(I)=Z1
  10     CONTINUE
      ELSE
         UMAX(1)=Z1
         UMAX(M)=Z1
         J1=J - 1
         K1=J + 1
         DO 20 I=2,M-1
         UMAX(I)=AMAX1(U(I-1,J1),U(I,J1),U(I+1,J1),
     +                 U(I-1, J),        U(I+1, J),
     +                 U(I-1,K1),U(I,K1),U(I+1,K1))
  20     CONTINUE
      ENDIF
      RETURN
      END
      SUBROUTINE RHSIDE(J,U,M,N,V,T)
C
C INITIALIZES THE RIGHT HAND SIDE OF THE EQUATION FOR THE JTH ROW.
C
      INTEGER   J,M,N
      REAL      U(M,N),V(M),T
C
C---LOCAL VARIABLES---
      INTEGER   I,J1,J2,K,K1,K2
      REAL      ONE,TWO,FOUR,SIX,EIGHT
C
C---SET COEFFICIENTS---
      ONE=T*1.0
      TWO=2.0
      FOUR=4.0 + ONE
      SIX=6.0 + ONE
      EIGHT=8.0 + ONE
C
C---SET POINTERS---
      K=1
      IF (J.GT.N-2) K=-K
      J1=J + K
      J2=J1 + K
      K1=J - K
      K2=K1 - K
C
C---EDGE ROW---
      IF (J.GT.2 .AND. J.LT.N-1) GOTO 40
      IF (J.GT.1 .AND. J.LT.N) GOTO 20
      V(1)=FOUR*U(1,J1) -  TWO*U(2,J1)  - U(1,J2)
      V(M)=FOUR*U(M,J1) - TWO*U(M-1,J1) - U(M,J2)
      DO 10 I=2,M-1
      V(I)=SIX*U(I,J1) - TWO*(U(I-1,J1) + U(I+1,J1))
     +                 - U(I,J2)
  10  CONTINUE
      GOTO 60
C
C---NEXT TO EDGE---
  20  V(1)=FOUR*U(1,K1) + SIX*U(1,J1) - TWO*(U(2,K1)   + U(2,J1))
     +                  - U(1,J2)
      V(M)=FOUR*U(M,K1) + SIX*U(M,J1) - TWO*(U(M-1,K1) + U(M-1,J1))
     +                   - U(M,J2)
      DO 30 I=2,M-1
      V(I)=SIX*U(I,K1) + EIGHT*U(I,J1) - U(I,J2)
     +     - TWO*(U(I-1,K1) + U(I+1,K1) + U(I-1,J1) + U(I+1,J1))
  30  CONTINUE
      GOTO 60
C
C---AWAY FROM EDGE ROWS---
  40  V(1)=SIX*(U(1,K1) + U(1,J1)) - TWO*(U(2,K1)   + U(2,J1))
     +                             - (U(1,K2)   + U(1,J2))
      V(M)=SIX*(U(M,K1) + U(M,J1)) - TWO*(U(M-1,K1) + U(M-1,J1))
     +                             - (U(M,K2)   + U(M,J2))
      DO 50 I=2,M-1
      V(I)=EIGHT*(U(I,K1) +  U(I,J1)) - (U(I,K2) + U(I,J2))
     +     - TWO*(U(I-1,K1) + U(I+1,K1) + U(I-1,J1) + U(I+1,J1))
  50  CONTINUE
  60  RETURN
      END
      SUBROUTINE COEFFT(J,M,N,A11,A12,A22,A23,A33,T)
C
C CALCULATES ELEMENTS OF THE FIRST THREE BANDS OF THE UPPER
C TRIANGLE OF THE SYMMETRIC FINITE DIFFERENCE MATRIX GIVING
C THE ROUGHNESS PENALTY OF THE J TH ROW OF THE GRID.
C
      INTEGER J,M,N
      REAL    A11,A12,A22,A23,A33,T
C
C---LOCAL VARIABLES---
      REAL    FOUR,SIX,EIGHT,TEN,ELEVEN,ATEEN,NINTEN,TWENTY,
     +        ONE,TWO,THREE
C
      DATA    FOUR, SIX, EIGHT,  TEN, ELEVEN, ATEEN, NINTEN, TWENTY/
     +         4.0, 6.0,   8.0, 10.0,   11.0,  18.0,   19.0,   20.0/
      DATA    ONE,TWO,THREE/1.0, 2.0, 3.0/
C
C---EDGE ROW---
      IF (J.GT.2 .AND. J.LT.N-1) GOTO 20
      IF (J.GT.1 .AND. J.LT.N) GOTO 10
      A11=FOUR + T*TWO
      A12=-FOUR - T*ONE
      A22=TEN + T*THREE
      A23=-SIX - T*ONE
      A33=ELEVEN + T*THREE
      GOTO 30
C
C---NEXT TO EDGE---
  10  A11=TEN + T*THREE
      A12=-SIX - T*ONE
      A22=ATEEN + T*FOUR
      A23=-EIGHT - T*ONE
      A33=NINTEN + T*FOUR
      GOTO 30
C
C---AWAY FROM EDGE ROWS---
  20  A11=ELEVEN + T*THREE
      A12=-SIX - T*ONE
      A22=NINTEN + T*FOUR
      A23=-EIGHT - T*ONE
      A33=TWENTY + T*FOUR
C
  30  RETURN
      END
      SUBROUTINE SOLVE(J,U,IW,IV,M,N,V,Z0,Z1,Z,ND,KD,UMAX,KMAX2,
     +                 UMAX2,IP,JP,DP,T,OMEGA,RHO,SSQ,FAC,ZGAP)
C
C SOLVES FOR J TH ROW OF THE GRID USING GAUSS SEIDEL ITERATION WITH
C OVERRELAXATION PARAMETER OMEGA USING COEFFICIENTS CALCULATED BY
C SUBROUTINE COEFFT AND INITIAL RIGHT HAND SIDE HELD IN V.
C THE ITERATED VALUES ARE SUBJECT TO LOWER AND UPPER LIMITING CONDITIONS
C GIVEN VIA THE VALUES IN UMAX,UMAX2 AND THE POINTERS IN IV,KMAX2.
C GRID POINTS ASSOCIATED WITH DATA POINTS ARE SUBJECT IN ADDITION
C TO A MAXIMUM DEVIATION FROM THE DATA VALUES DETERMINED BY ZGAP.
C
      INTEGER          J,M,N,ND,KD,KMAX2(M,2),IP(8),JP(8)
      REAL             U(M,N),V(M),Z0,Z1,Z(ND),UMAX(M),UMAX2(M,2),
     +                 DP(8),T,OMEGA,RHO,FAC,ZGAP
      DOUBLE PRECISION SSQ
      INTEGER*2        IW(0:M+1,0:N+1),IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER          I,IK,JK,KL,IH,JH,KH
      REAL             A11,A12,A22,A23,A33,U1,UK,UMIN,UMX,OMEGA1,
     +                 ZERO,ONE,TWO,DIS,GAP1,GAP2,DZ,GAP,GAP4,GAP8
C
      DATA             ZERO,ONE,TWO/0.0, 1.0, 2.0/
C
C---INITIALIZE---
      I=0
      OMEGA1=ONE - OMEGA
      GAP4=ZGAP/4.0
      GAP8=ZGAP/8.0
      CALL COEFFT(J,M,N,A11,A12,A22,A23,A33,T)
C
C---FIRST TWO COLUMNS---
      DO 50 I=1,2
      IF (I.EQ.1) THEN
         Q=RHO*A11
         U1=(V(I) - A12*U(I+1,J) - U(I+2,J))/A11
      ELSE
         Q=RHO*A22
         U1=(V(I) - A12*U(I-1,J) - A23*U(I+1,J) - U(I+2,J))/A22
      ENDIF
      GAP=ZGAP
      KH=KMAX2(I,1)
      IF (KH.GT.0) THEN
         IH=I - IP(KH)
         JH=J - JP(KH)
         UMX=U(IH,JH)
      ELSE
         UMX=UMAX(I)
      ENDIF
      KL=IV(I,J)
      IF (KL.GT.0) THEN
         IK=I + IP(KL)
         JK=J + JP(KL)
         UMIN=U(IK,JK)
         GAP=GAP4
         IF (KH.EQ.0) THEN
            IH=I - IP(KL)
            JH=J - JP(KL)
            IF (IW(IH,JH).GE.0) THEN
               KH=KL
               UMX=AMAX1(U(IH,JH),U(I,J))
            ENDIF
         ELSE
            GAP=GAP8
         ENDIF
         IF (KH.GT.0) THEN
            UMX=AMAX1(UMX,UMIN)
            DIS=FAC*(UMX - UMIN)/(DP(KL) + DP(KH))
            GAP1=DIS*DP(KH)
            GAP2=DIS*DP(KL)
         ENDIF
      ELSE
         UMIN=Z0
      ENDIF
      IF (IW(I,J).EQ.1 .OR. IW(I,J).EQ.5) THEN
         KD=KD + 1
         U1=(Z(KD) + Q*U1)/(ONE + Q)
         U1=AMAX1(AMIN1(U1,Z(KD)+GAP),Z(KD)-GAP)
         DZ=U1 - Z(KD)
         SSQ=SSQ + DZ*DZ
      ELSE
         U1=OMEGA*U1 + OMEGA1*U(I,J)
         IF (KL*KH.NE.0) THEN
            U1=AMAX1(AMIN1(U1,UMX-GAP2),UMIN+GAP1)
         ELSE
            U1=AMAX1(AMIN1(U1,UMX),UMIN)
         ENDIF
      ENDIF
      IF (KL.LE.0) GOTO 40
      IF (JP(KL).LT.0) GOTO 40
      IF (JP(KL).EQ.0 .AND. IP(KL).LT.0) GOTO 40
      JJK=1 + JP(KL)
      UK=(U1 - UMIN)*DP(KL)
      IF (KMAX2(IK,JJK).LE.0 .OR. UK.LT.UMAX2(IK,JJK)) THEN
         KMAX2(IK,JJK)=KL
         UMAX2(IK,JJK)=UK
      ENDIF
  40  U(I,J)=U1
  50  CONTINUE
C
C---MIDDLE COLUMNS---
      Q=RHO*A33
      DO 80 I=3,M-2
      U1=(V(I) - A23*(U(I-1,J) + U(I+1,J))
     +           - (U(I-2,J) + U(I+2,J)))/A33
      GAP=ZGAP
      KH=KMAX2(I,1)
      IF (KH.GT.0) THEN
         IH=I - IP(KH)
         JH=J - JP(KH)
         UMX=U(IH,JH)
      ELSE
         UMX=UMAX(I)
      ENDIF
      KL=IV(I,J)
      IF (KL.GT.0) THEN
         IK=I + IP(KL)
         JK=J + JP(KL)
         UMIN=U(IK,JK)
         GAP=GAP4
         IF (KH.EQ.0) THEN
            IH=I - IP(KL)
            JH=J - JP(KL)
            IF (IW(IH,JH).GE.0) THEN
               KH=KL
               UMX=AMAX1(U(IH,JH),U(I,J))
            ENDIF
         ELSE
            GAP=GAP8
         ENDIF
         IF (KH.GT.0) THEN
            UMX=AMAX1(UMX,UMIN)
            DIS=FAC*(UMX - UMIN)/(DP(KL) + DP(KH))
            GAP1=DIS*DP(KH)
            GAP2=DIS*DP(KL)
         ENDIF
      ELSE
         UMIN=Z0
      ENDIF
      IF (IW(I,J).EQ.1 .OR. IW(I,J).EQ.5) THEN
         KD=KD + 1
         U1=(Z(KD) + Q*U1)/(ONE + Q)
         U1=AMAX1(AMIN1(U1,Z(KD)+GAP),Z(KD)-GAP)
         DZ=U1 - Z(KD)
         SSQ=SSQ + DZ*DZ
      ELSE
         U1=OMEGA*U1 + OMEGA1*U(I,J)
         IF (KL*KH.NE.0) THEN
            U1=AMAX1(AMIN1(U1,UMX-GAP2),UMIN+GAP1)
         ELSE
            U1=AMAX1(AMIN1(U1,UMX),UMIN)
         ENDIF
      ENDIF
      IF (KL.LE.0) GOTO 70
      IF (JP(KL).LT.0) GOTO 70
      IF (JP(KL).EQ.0 .AND. IP(KL).LT.0) GOTO 70
      JJK=1 + JP(KL)
      UK=(U1 - UMIN)*DP(KL)
      IF (KMAX2(IK,JJK).LE.0 .OR. UK.LT.UMAX2(IK,JJK)) THEN
         KMAX2(IK,JJK)=KL
         UMAX2(IK,JJK)=UK
      ENDIF
  70  U(I,J)=U1
  80  CONTINUE
C
C---LAST TWO COLUMNS---
      DO 140 I=M-1,M
      IF (I.LT.M) THEN
         Q=RHO*A22
         U1=(V(I) - A23*U(I-1,J) - A12*U(I+1,J) - U(I-2,J))/A22
      ELSE
         Q=RHO*A11
         U1=(V(I) - A12*U(I-1,J) - U(I-2,J))/A11
      ENDIF
      GAP=ZGAP
      KH=KMAX2(I,1)
      IF (KH.GT.0) THEN
         IH=I - IP(KH)
         JH=J - JP(KH)
         UMX=U(IH,JH)
      ELSE
         UMX=UMAX(I)
      ENDIF
      KL=IV(I,J)
      IF (KL.GT.0) THEN
         IK=I + IP(KL)
         JK=J + JP(KL)
         UMIN=U(IK,JK)
         GAP=GAP4
         IF (KH.EQ.0) THEN
            IH=I - IP(KL)
            JH=J - JP(KL)
            IF (IW(IH,JH).GE.0) THEN
               KH=KL
               UMX=AMAX1(U(IH,JH),U(I,J))
            ENDIF
         ELSE
            GAP=GAP8
         ENDIF
         IF (KH.GT.0) THEN
            UMX=AMAX1(UMX,UMIN)
            DIS=FAC*(UMX - UMIN)/(DP(KL) + DP(KH))
            GAP1=DIS*DP(KH)
            GAP2=DIS*DP(KL)
         ENDIF
      ELSE
         UMIN=Z0
      ENDIF
      IF (IW(I,J).EQ.1 .OR. IW(I,J).EQ.5) THEN
         KD=KD + 1
         U1=(Z(KD) + Q*U1)/(ONE + Q)
         U1=AMAX1(AMIN1(U1,Z(KD)+GAP),Z(KD)-GAP)
         DZ=U1 - Z(KD)
         SSQ=SSQ + DZ*DZ
      ELSE
         U1=OMEGA*U1 + OMEGA1*U(I,J)
         IF (KL*KH.NE.0) THEN
            U1=AMAX1(AMIN1(U1,UMX-GAP2),UMIN+GAP1)
         ELSE
            U1=AMAX1(AMIN1(U1,UMX),UMIN)
         ENDIF
      ENDIF
      IF (KL.LE.0) GOTO 130
      IF (JP(KL).LT.0) GOTO 130
      IF (JP(KL).EQ.0 .AND. IP(KL).LT.0) GOTO 130
      JJK=1 + JP(KL)
      UK=(U1 - UMIN)*DP(KL)
      IF (KMAX2(IK,JJK).LE.0 .OR. UK.LT.UMAX2(IK,JJK)) THEN
         KMAX2(IK,JJK)=KL
         UMAX2(IK,JJK)=UK
      ENDIF
 130  U(I,J)=U1
 140  CONTINUE
      RETURN
      END
