#include <stdio.h>
#include <string.h>
#include "las.h" 
/*****************************************************************************
 *
 * NAME         anudem (lasimage)
 *
 * PURPOSE	This function was written as an extension to anudem.
 *		It providesa means of outputing a las image directly from
 *		anudem. 
 *             
 *
 * PROGRAM HISTORY
 *
 *      Programmer              Date            Reason
 *      ----------              ----            ------
 *      R. Kruse / EDC,CSB    14-JAN-93       Output a lasimage
 *
 * Credits: The fundamental algorithm was taken from bilhdr_.c (anudem), which
 * 	    was written by Kevin Larson.
 *****************************************************************************/
/***********************************************************************
*****   Passed Parameters:                                         *****
*****    Gridname - Contains the user entered grid name.           *****
*****    ncols - The number of colomns present in the grid.        *****
*****    nrows - The number of rows present in the grid.           *****
*****    xmin - The minimum x value in the coordinate system       *****
*****           entered by the user.                               *****
*****    xmax - The maximum x value in the coordinate system       *****
*****           entered by the user.                               *****
*****    ymin - The minimum y value in the corrdinate system       *****
*****           entered by the user.                               *****
*****    ymax - The maximum y value in the coordinate system       *****
*****           entered by the user.                               *****
*****    gridsp - The size of each square grid cell based on the   *****
*****             coordinate system used by x and y.               *****
*****    u[] - Array containing the grid elevation values.         *****
*****    iw[] - Array indicating the used of special values in     *****
*****             u[].                                             *****
*****    spval - Special value to be entered into u[], based on    *****
*****            iw[].                                             *****
*****    namelen - The length of the character array declaration   *****
*****              of gridname.                                    *****
*****                                                              *****
*****                                                              *****
*****    output_type - Contains the user specified output data     *****
*****                  type.  This value corresponds to the        *****
*****                  following values for input:                 *****
*****                      1 : EBTYE                               *****
*****                      2 : EWORD                               *****
*****                      3 : ELONG                               *****
*****                      4 : EREAL                               *****
*****                                                              *****
*****                   NOTE: worgen is note included in the       *****
*****                         fortran code.  Therefore the values  *****
*****                         are should be converted to the       *****
*****                         equivalent of EBYTE,...              *****
*****                                                              *****
*****                                                              *****
***********************************************************************/

/* PLEASE NOTE:  In Anudem, u and iw are 2 dimentional arrays, u
                 is declared as u(iz,iz) and iw is declared as
                 iw(0:iz+1,0:iz+1).  When the arrays where filled
                 in subroutine calls, they where declared as a 
                 subsection the array, u as u(m,n) and iw as
                 iw(0:m+1,0:n+1).  Because of this, u and iw are
                 brought in as a single dimension array and indexed
                 as a 2 dimensional array.  Since as arrays, u is
                 located inside of iw when indexing, and nrows and 
                 ncols are based on u, 2 must be added to these values
                 for the array size of iw. */

void lasimage_(lasname, ncols, nrows, xmin, xmax, ymin, ymax, gridsp,
             u, iw, spval, output_type, namelen)

	char		*lasname;
	long		*ncols,*nrows;
	double		*xmin, *xmax, *ymin, *ymax, *gridsp;
        float		*u,*spval;
	short		*iw;
	int		*output_type;
	int		namelen;

{
   	int i,                  /* Column loop counter. */
	j,                       /* Row loop counter. */
       index_u,                 /* Used for indexing u[]. */
       index_iw,                /* Used for indexing iw[]. */
       flag,                    /* Flag value used in indicating the end
                                   of the character string in gridname. */
       odtype,			/* Output data type */
       length;                  /* Length of gridname. */
       float *elev;             /* Used for outputing u[][] as a LAS odtype. */

       char	hostname[CMLEN]; /* host image file name */

       int lasout;                  /* fdesc for las out file. */
        
	long			status;
	long			zero,one,two,three,four,access,
				fatal_err; /* constants */
        long 			las_line;  /* las line number of output image */
	long		opt[] = {0,0,0,0}; /* array for eopenr */
 	zero	= 0;		/* variables needed to pass to las */
	one	= 1;		/* support functions */
	two	= 2;
	three	= 3;
	four	= 4;
	fatal_err = -1;
	
   elev = (float *)malloc( sizeof(float) * (*ncols) );
   /* Initializing values. */
   flag = 0;
   i = 0;

  
   /* Determining the location the the last character in lasname.*/
   while ((flag == 0) && (i < namelen))  
       if (lasname[i] == ' ') 
       {
          lasname[i] = 0;
          flag = 1;
       }
       else
          i++;

   strcpy(hostname,lasname);
   access = IWRITE;
   status = getname(hostname, "img" , access);
   if (status != E_SUCC)
	c_errmsg ("Fatal error encountered", "anudem-getname", &fatal_err);

   /* Find the output data type */
   switch (*output_type) {
      case 1: odtype = EBYTE;
		break;
      case 2: odtype = EWORD;
		break;
      case 3: odtype = ELONG;
		break;
      case 4: odtype = EREAL;
		break;
      default: odtype = EBYTE;
		break;
     }
   /* setup opt array for eopenr */
   opt[0] = odtype;
   opt[1] = 0;
   if (status = (c_eopenr(&lasout,hostname, &one, &one, &one,
		nrows,ncols, &one, &four, opt, &one) ) != E_SUCC)
   {
      c_errmsg ("Fatal error opening output image", "anudem-eopenr", &fatal_err);
      return;     /**  EARLY TERMINATION!!  **/
   }
   
 
   /* Writing out bilname in bil format. */
   for (j = ((*nrows) - 1); j >= 0; j--)
     {
      index_iw = ((*ncols)+2) * (j+1) + 1;
      index_u  = (*ncols) * j;
      for (i = 0; i < (*ncols); i++)
      {
	 if (iw[index_iw + i] < 0)
		elev[i] = *spval;
	 else
		elev[i] = u[index_u + i];
      }
      las_line = (*nrows) - j ;
      if ( (status = (c_ewrite(&lasout,&one, &las_line, elev, &one))) == E_FAIL)
	{
	   c_errmsg ("Fatal error writing image", "anudem-write", &fatal_err);
	   return;     /**  EARLY TERMINATION!!  **/
	}
     }

   /* Closing the output file.*/
   if ( status = (c_eclose(&lasout)) )
     {
      c_errmsg ("Fatal error closing image", "anudem-close", &fatal_err);
      return;     /**  EARLY TERMINATION!!  **/
     }

   /* Returning to calling point. */
   free(elev);
   return;
}


