      SUBROUTINE RDPOLY(NFIL,OUTFIL,DATFIL,DATNAM,HEDFMT,DATFMT,
     +                  POLFIL,M,N,X0,X1,Y0,Y1,H,X,Y,MAXD,NP)
C
C SUBROUTINE READS POINTS DEFINING A CLOSED POLYGON (OR
C POLYGONS) FROM A USER DEFINED FILE IN A USER DEFINED FORMAT.
C THE POLYGON COORDINATES ARE GENERALIZED TO THE RESOLUTION H OF
C THE GRID AND STORED ON THE UNFORMATTED SCRATCH FILE POLFIL.
C THE NUMBER OF POLYGONS IS STORED IN NP.
C
C SUCCESSIVE POLYGON STRINGS ARE JOINED IF THEY ARE CONTIGUOUS
C AND CLOSED POLYGON STRINGS ARE RECOGNISED IF POSSIBLE.  THE
C DIFFERENT POSSIBILITIES ARE DENOTED BY THE VALUES OF IP
C AS FOLLOWS:
C    IP = 0    NORMAL CLOSED POLYGONS
C    IP = 1    NON-CLOSED STRING FOLLOWED BY END OF FILE
C    IP = 2    NON-CLOSED STRING FOLLOWED BY A NON-CONNECTING STRING
C
C
      INTEGER          NFIL,OUTFIL,DATFIL,POLFIL,M,N,MAXD,NP
      DOUBLE PRECISION X0,X1,Y0,Y1,H,X(MAXD),Y(MAXD)
      CHARACTER*40     DATNAM,HEDFMT,DATFMT
C
C---LOCAL VARIABLES---
      INTEGER          NUM,NGAP,N1,N2,N3,IP,I0,J0,I,J,K,KP,NN1,NN2
      REAL             ONE5,ARMIN
      DOUBLE PRECISION XX1,YY1,XX2,YY2,TOL
C
      DATA             ONE5,TOL/1.5, 0.005D0/
C
C---INITIALIZE---
      IF (NFIL.GT.0) WRITE(OUTFIL,5) NFIL,DATNAM
   5  FORMAT(/' DATA FILE',I3,': ',A40)
      OPEN(DATFIL,FILE=DATNAM,STATUS='OLD')
      REWIND DATFIL
      ARMIN=H*H/8.0
      IP=0
      NGAP=4*25
      NN1=0
      NN2=0
C
C---LOOP THROUGH POLYGONS---
      DO 90 L=1,MAXD
      N2=NGAP
C
C---MOVE LAST STRING TO TOP OF BUFFER---
      IF (IP.NE.2) GOTO 15
      N1=NGAP + 1
      N2=NGAP + NUM
      DO 10 I=N1,N2
      N3=N3 + 1
      X(I)=X(N3)
      Y(I)=Y(N3)
  10  CONTINUE
      XX1=X(N1)
      YY1=Y(N1)
      XX2=X(N2)
      YY2=Y(N2)
      IF (DABS(XX1-XX2).LE.TOL .AND. DABS(YY1-YY2).LE.TOL) GOTO 50
C
C---READ NEXT POLYGON STRING---
  15  DO 30 K=1,MAXD
      READ(DATFIL,HEDFMT,END=40,ERR=120) NUM
      NN1=NN1 + NUM
      N1=N2 + 1
      N2=N2 + NUM
      IF (N2.GT.MAXD) STOP 'RDPOLY - TOO MANY POINTS FOR BUFFER'
      READ(DATFIL,DATFMT,ERR=130) (X(I),Y(I),I=N1,N2)
C
C---SET PARAMETERS FOR FIRST STRING---
      IF (N1.GT.NGAP+1) GOTO 20
      XX1=X(N1)
      YY1=Y(N1)
      XX2=X(N2)
      YY2=Y(N2)
      GOTO 25
C
C---SET PARAMETERS FOR NEXT STRING OR EXIT IF NOT CONNECTED---
  20  IF (DABS(X(N1)-XX2).GT.TOL .OR. DABS(Y(N1)-YY2).GT.TOL) GOTO 45
      XX2=X(N2)
      YY2=Y(N2)
C
C---EXIT IF POLYGON CLOSED---
  25  IF (DABS(XX1-XX2).LE.TOL .AND. DABS(YY1-YY2).LE.TOL) GOTO 50
  30  CONTINUE
      STOP 'RDPOLY - TOO MANY POINTS FOR BUFFER'
C
C---END OF POLYGON FILE---
  40  IF (N2.LE.NGAP) GOTO 100
      IP=1
      WRITE(OUTFIL,42)
  42  FORMAT(/' RDPOLY - WARNING - POLYGON STRING NOT CLOSED')
      WRITE(OUTFIL,43) NUM,X(N1),Y(N1),X(N2),Y(N2)
  43  FORMAT(' NUM =',I5,' X1,Y1 =',2F10.5,' XN,YN =',2F10.5)
      GOTO 55
C
C---STRING NOT CONNECTED---
  45  IP=2
      N3=N2 - NUM
      WRITE(OUTFIL,42)
      WRITE(OUTFIL,43) NUM,X(N1),Y(N1),X(N2),Y(N2)
      GOTO 55
C
C---TRIM POLYGON TO X/Y LIMITS---
  50  IP=0
  55  CALL WINDOW(N2,X,Y,MAXD,X0,X1,Y0,Y1,NGAP,IP)
      IF (N2.LE.0) GOTO 80
C
C---INSERT GENERALIZED POLYGON IN XP,YP ARRAYS---
      NNP=1
      X(NNP)=X(1) - X0
      Y(NNP)=Y(1) - Y0
      I0=X(1)/H + ONE5
      J0=Y(1)/H + ONE5
C
C---GENERALIZE POLYGON POINTS---
      DO 60 KP=2,N2-1
      I=X(KP)/H + ONE5
      J=Y(KP)/H + ONE5
      IF (I.EQ.I0 .AND. J.EQ.J0) GOTO 60
      I0=I
      J0=J
      NNP=NNP + 1
      X(NNP)=X(KP) - X0
      Y(NNP)=Y(KP) - Y0
  60  CONTINUE
      NNP=NNP + 1
      X(NNP)=X(N2) - X0
      Y(NNP)=Y(N2) - Y0
C
C---SKIP SMALL POLYGONS---
      IF (ARPOLY(NNP,X,Y).LE.ARMIN) GOTO 90
      NP=NP + 1
      WRITE(POLFIL) NNP
      DO 70 NN=1,NNP
      WRITE(POLFIL) X(NN),Y(NN)
  70  CONTINUE
      NN2=NN2 + NNP
  80  IF (IP.EQ.1) GOTO 100
  90  CONTINUE
      STOP 'RDPOLY - TOO MANY POLYGONS'
 100  WRITE(OUTFIL,105) NN1,NN2
 105  FORMAT(' NUMBER OF POLYGON POINTS  =',I6/
     +       ' NUMBER OF POINTS ACCEPTED =',I6)
      RETURN
 110  STOP 'RDPOLY - TOO MANY POLYGON POINTS'
 120  STOP 'RDPOLY - ERROR READING POLYGON HEADER'
 130  STOP 'RDPOLY - ERROR READING POLYGON STRING'
      END
      SUBROUTINE WINDOW(N,X,Y,MAXN,X0,X1,Y0,Y1,NGAP,IP)
C
C TRIMS A STRING OF POINTS TO WITHIN THE GIVEN X,Y LIMITS.
C IF AN INPUT STRING IS CLOSED THEN THE OUTPUT WINDOWED STRING
C IS EITHER CLOSED OR EMPTY.
C
C IF IP=0 THEN THE STRING IS CLOSED, OTHERWISE IT IS NOT CLOSED.
C
C NGAP IS AN INTEGRAL MULTIPLE OF 4 WHICH GIVES THE SIZE OF BUFFER
C AT THE BEGINNING OF X,Y ARRAYS FOR THE CREATION OF EXTRA POINTS
C IN THE WINDOWING PROCESS.  THIS BUFFER IS REDUCED TO 0 ON EXIT.
C
      INTEGER          N,MAXN,NGAP,IP
      DOUBLE PRECISION X0,X1,Y0,Y1,X(MAXN),Y(MAXN)
C
C---LOCAL VARIABLES---
      INTEGER          N1,N2
C
C---MINIMUM X---
      IF (N.LE.NGAP) GOTO 80
      N2=NGAP
      N1=N2 - NGAP/4
      CALL WIND1(N,X,Y,MAXN,X0,N1,N2)
      IF (N.LE.N1) GOTO 80
      IF (IP.NE.0) GOTO 20
      IF (X(N1+1).EQ.X(N) .AND. Y(N1+1).EQ.Y(N)) GOTO 20
      N=N + 1
      X(N)=X(N1 + 1)
      Y(N)=Y(N1 + 1)
C
C---MAXIMUM X---
  20  N2=N1
      N1=N2 - NGAP/4
      CALL WIND2(N,X,Y,MAXN,X1,N1,N2)
      IF (N.LE.N1) GOTO 80
      IF (IP.NE.0) GOTO 30
      IF (X(N1+1).EQ.X(N) .AND. Y(N1+1).EQ.Y(N)) GOTO 30
      N=N + 1
      X(N)=X(N1 + 1)
      Y(N)=Y(N1 + 1)
C
C---MINIMUM Y---
  30  N2=N1
      N1=N2 - NGAP/4
      CALL WIND1(N,Y,X,MAXN,Y0,N1,N2)
      IF (N.LE.N1) GOTO 80
      IF (IP.NE.0) GOTO 40
      IF (X(N1+1).EQ.X(N) .AND. Y(N1+1).EQ.Y(N)) GOTO 40
      N=N + 1
      X(N)=X(N1+1)
      Y(N)=Y(N1+1)
C
C---MAXIMUM Y---
  40  N2=N1
      N1=N2 - NGAP/4
      CALL WIND2(N,Y,X,MAXN,Y1,N1,N2)
      IF (N.LE.N1) GOTO 80
      IF (IP.NE.0) GOTO 70
      IF (X(N1+1).EQ.X(N) .AND. Y(N1+1).EQ.Y(N)) GOTO 70
      N=N + 1
      X(N)=X(N1+1)
      Y(N)=Y(N1+1)
  70  RETURN
  80  N=0
      RETURN
      END
      SUBROUTINE WIND1(N,X,Y,MAXN,X0,N1,N2)
C
C REMOVES POINTS WITH X COORDINATE LESS THAN X0.
C
      INTEGER          N,MAXN,N1,N2
      DOUBLE PRECISION X0,X(MAXN),Y(MAXN)
C
      IF (N.GT.N2) GOTO 5
      N=N1
      RETURN
   5  IN=1
      J=N1 + 1
      X(J)=X(N2+1)
      Y(J)=Y(N2+1)
      IF (X(N2+1).GE.X0) GOTO 10
      IN=0
      J=N1
C
  10  N3=N2 + 2
      DO 80 I=N3,N
      IF (X(I).LT.X0) GOTO 30
C
C---POINT INSIDE---
      IF (IN.EQ.1) GOTO 20
      J=J + 1
      IF (J.GT.I-1) GOTO 100
      X(J)=X0
      Y(J)=Y(I-1) + (X0 - X(I-1))*(Y(I) - Y(I-1))/(X(I) - X(I-1))
  20  J=J + 1
      X(J)=X(I)
      Y(J)=Y(I)
      IN=1
      GOTO 80
C
C---POINT OUTSIDE---
  30  IF (IN.EQ.0) GOTO 40
      J=J + 1
      X(J)=X0
      Y(J)=Y(I-1) + (X0 - X(I-1))*(Y(I) - Y(I-1))/(X(I) - X(I-1))
  40  IN=0
  80  CONTINUE
      N=J
      RETURN
C
C---ERROR CONDITION---
 100  STOP 'WIND1 - BUFFER FULL'
      END
      SUBROUTINE WIND2(N,X,Y,MAXN,X1,N1,N2)
C
C REMOVES POINTS WITH X COORDINATE GREATER THAN X1.
C
      INTEGER          N,MAXN,N1,N2
      DOUBLE PRECISION X1,X(MAXN),Y(MAXN)
C
      IF (N.GT.N2) GOTO 5
      N=N1
      RETURN
   5  IN=1
      J=N1 + 1
      X(J)=X(N2+1)
      Y(J)=Y(N2+1)
      IF (X(N2+1).LE.X1) GOTO 10
      IN=0
      J=N1
C
  10  N3=N2 + 2
      DO 80 I=N3,N
      IF (X(I).GT.X1) GOTO 30
C
C---POINT INSIDE---
      IF (IN.EQ.1) GOTO 20
      J=J + 1
      IF (J.GT.I-1) GOTO 100
      X(J)=X1
      Y(J)=Y(I-1) + (X1 - X(I-1))*(Y(I) - Y(I-1))/(X(I) - X(I-1))
  20  J=J + 1
      X(J)=X(I)
      Y(J)=Y(I)
      IN=1
      GOTO 80
C
C---POINT OUTSIDE---
  30  IF (IN.EQ.0) GOTO 40
      J=J + 1
      X(J)=X1
      Y(J)=Y(I-1) + (X1 - X(I-1))*(Y(I) - Y(I-1))/(X(I) - X(I-1))
  40  IN=0
  80  CONTINUE
      N=J
      RETURN
C
C---ERROR CONDITION---
 100  STOP 'WIND2 - BUFFER FULL'
      END
      SUBROUTINE GETPOL(POLFIL,NP,IW,M,N,H,X,Y,MAXD)
C
C CALCULATES A GRID OF BYTES IW WHICH DENOTE THE POSITION OF
C EACH GRID POINT WITH RESPECT TO THE CLOSED POLYGON(S), STORED
C ON THE UNFORMATTED SCRATCH FILE POLFIL, AS FOLLOWS:
C         0  INSIDE POLYGON
C         1  ON POLYGON BOUNDARY
C         2  OUTSIDE BUT ADJACENT TO POLYGON BOUNDARY
C         3  REMAINING GRID POINTS OUTSIDE POLYGON
C THE NUMBER OF POLYGON STRINGS IS HELD IN NP.
C
C THE SUBROUTINE USES THE GIVEN POLYGON(S) TO
C DEFINE THE POLYGON BOUNDARY POINTS, THEN DETERMINES THE ADJACENT
C POINTS OUTSIDE THE BOUNDARY USING THE SUBROUTINE EDGE, AND FINALLY
C INSERTS THE REMAINING INTERIOR AND EXTERIOR GRID POINTS.
C
C THE VARIABLES IW(*,*) AND MODE,N0,N1,N2,N3,N4 SHOULD BE OF THE
C SAME TYPE.
C
      INTEGER          POLFIL,NP,M,N,MAXD
      DOUBLE PRECISION H,X(MAXD),Y(MAXD)
      INTEGER*2        IW(0:M+1,0:N+1)
C
C---LOCAL VARIABLES---
      PARAMETER        (MAXNED=1000)
      INTEGER          I,J,K,II,JJ,KK,KP,IS0,NP1,ILO,IHI,JLO,JHI,
     +                 IP(MAXNED),JP(MAXNED),IS(MAXNED),NED,I1,I2
      REAL             DX,DY,DIS,ONE5
      DOUBLE PRECISION XP,YP,XX,YY,TOL
      INTEGER*2        MODE,N0,N1,N2,N3,N4
C
      DATA             ONE5/1.5/
      DATA             N0,N1,N2,N3,N4/0, 1, 2, 3, 4/
C
C---INITIALIZE---
      REWIND POLFIL
      DO 10 J=0,N+1
      DO 10 I=0,M+1
      IW(I,J)=N3
  10  CONTINUE
      TOL=H/100.0D0
C
C---LOOP THROUGH POLYGONS---
      DO 140 K=1,NP
      NED=0
      READ(POLFIL) NP1
      IF (NP1.GT.MAXD) STOP 'GETPOL - POLYGON TOO LARGE'
      DO 20 KP=1,NP1
      READ(POLFIL) X(KP),Y(KP)
  20  CONTINUE
C
C---SET POLYGON BOUNDARY POINTS---
      I0=X(1)/H + ONE5
      J0=Y(1)/H + ONE5
      IW(I0,J0)=N1
      ILO=I0
      IHI=I0
      JLO=J0
      JHI=J0
C
      DO 40 KP=2,NP1
      I=X(KP)/H + ONE5
      J=Y(KP)/H + ONE5
      ILO=MIN0(ILO,I)
      IHI=MAX0(IHI,I)
      JLO=MIN0(JLO,J)
      JHI=MAX0(JHI,J)
      IF (IABS(I-I0) + IABS(J-J0) .LE. 1) GOTO 35
      DX=X(KP) - X(KP-1)
      DY=Y(KP) - Y(KP-1)
      DIS=SQRT(DX*DX + DY*DY)
      NNP=MAX1(2.0*DIS/SNGL(H), 2.0)
      DX=DX/NNP
      DY=DY/NNP
C
      DO 30 KK=1,NNP-1
      XX=X(KP-1) + KK*DX
      YY=Y(KP-1) + KK*DY
      II=XX/H + ONE5
      JJ=YY/H + ONE5
      IW(II,JJ)=N1
  30  CONTINUE
  35  IW(I,J)=N1
      I0=I
      J0=J
  40  CONTINUE
C
C---SET POLYGON EDGES FROM LEFT---
      DO 70 J=JLO,JHI
      YP=(J-1)*H
      IS0=4
      DO 50 I=ILO,IHI
      IF (IW(I,J).NE.N1 .OR. IW(I-1,J).NE.N3) GOTO 50
      XP=(I-2)*H
      CALL PTLOC(NP1,X,Y,XP,YP,LOC,II1,II2,ISR,TOL)
      IF (LOC.GE.0) THEN
         MODE=N0
      ELSE
         MODE=N2
         NED=NED + 1
         IF (NED.GT.MAXNED) GOTO 250
         IP(NED)=I-1
         JP(NED)=J
         IS(NED)=IS0
      ENDIF
      CALL EDGE(IW,M,N,I-1,J,N1,MODE,IS0)
  50  CONTINUE
C
C---SET EDGES FROM RIGHT---
      IS0=8
      DO 60 I=IHI,ILO,-1
      IF (IW(I,J).NE.N1 .OR. IW(I+1,J).NE.N3) GOTO 60
      XP=I*H
      CALL PTLOC(NP1,X,Y,XP,YP,LOC,II1,II2,ISR,TOL)
      IF (LOC.GE.0) THEN
         MODE=N0
      ELSE
         MODE=N2
         NED=NED + 1
         IF (NED.GT.MAXNED) GOTO 250
         IP(NED)=I+1
         JP(NED)=J
         IS(NED)=IS0
      ENDIF
      CALL EDGE(IW,M,N,I+1,J,N1,MODE,IS0)
  60  CONTINUE
  70  CONTINUE
C
C---SET INSIDE OF POLYGON---
      DO 120 J=JLO,JHI
      I2=ILO - 1
  80  DO 90 I1=I2+1,IHI
      IF (IW(I1,J).EQ.N0) GOTO 100
  90  CONTINUE
      GOTO 120
 100  DO 110 I2=I1+1,IHI
      IF (IW(I2,J).EQ.N1) GOTO 80
      IW(I2,J)=N0
 110  CONTINUE
 120  CONTINUE
C
C---REMOVE OUTSIDE EDGES---
      IF (NED.LE.0) GOTO 140
      DO 130 I=1,NED
      CALL EDGE(IW,M,N,IP(I),JP(I),N1,N4,IS(I))
 130  CONTINUE
 140  CONTINUE
C
C---RESTORE OUTSIDE EDGES---
      DO 180 J=0,N+1
      DO 180 I=0,M+1
      IF (IW(I,J).EQ.N4) IW(I,J)=N2
 180  CONTINUE
      RETURN
C
C---ERROR CONDITION---
 250  STOP 'GETPOL - TOO MANY EDGES'
      END
      SUBROUTINE EDGE(IW,M,N,I0,J0,N1,N2,IS0)
C
C INSERTS THE VALUE N2 AROUND A POLYGON BOUNDARY FROM
C THE STARTING POSITION DETERMINED FROM I0,J0.
C
      INTEGER   M,N,I0,J0,IS0
      INTEGER*2 IW(0:M+1,0:N+1),N1,N2
C
C---LOCAL VARIABLES---
      INTEGER   IX(8),IY(8),I,J,I1,J1,I2,J2,K,KK,IS,IS2,IFLAG
C
      DATA      IX/0,1,1,1,0,-1,-1,-1/
      DATA      IY/1,1,0,-1,-1,-1,0,1/
C
C---INITIALISE---
      IFLAG=0
      IW(I0,J0)=N2
      I=I0
      J=J0
      IS=IS0
      DO 40 KK=1,100
      DO 40 K=1,10000
      I1=I + IX(IS)
      J1=J + IY(IS)
      IF (IFLAG.EQ.0 .AND. K.GT.10) GOTO 50
      IF (I1.EQ.I0 .AND. J1.EQ.J0) GOTO 50
      IF (IW(I1,J1).NE.N1) GOTO 10
      IS=MOD(IS,8) + 1
      GOTO 40
  10  IF (MOD(IS,2).EQ.1) GOTO 30
      IS2=MOD(IS,8) + 1
      I2=I + IX(IS2)
      J2=J + IY(IS2)
      IF (IW(I2,J2).NE.N1) GOTO 20
      IS=MOD(IS2,8) + 1
      GOTO 40
  20  IW(I2,J2)=N2
  30  IW(I1,J1)=N2
      IFLAG=1
      I=I1
      J=J1
      IS=MOD(IS+5,8) + 1
  40  CONTINUE
  45  CALL WRGRD(IW,M,N,I0,J0,20)
      STOP 'EDGE - ERROR'
  50  RETURN
      END
      SUBROUTINE WRGRD(IW,M,N,I0,J0,K)
C
C WRITES COUNTER GRID BY ROWS TO UNIT 6.
C
      INTEGER   M,N,I0,J0,K
      INTEGER*2 IW(0:M+1,0:N+1)
C
C---LOCAL VARIABLES---
      INTEGER   M1,M2,N1,N2,I,J
C
      WRITE(6,10) I0,J0
  10  FORMAT('1'/' WRGRD I0,J0 = ',2I5)
C
      N1=MAX0(J0-K,0)
      N2=MIN0(J0+K,N+1)
      M1=MAX0(I0-K,0)
      M2=MIN0(I0+K,M+1)
      DO 30 J=N2,N1,-1
      WRITE(6,20) (IW(I,J),I=M1,M2)
  20  FORMAT(1X,121I1)
  30  CONTINUE
      RETURN
      END
