      SUBROUTINE PTLOC(N,X,Y,XP,YP,LOC,I1,I2,ISR,TOL)
C
C DETERMINES LOCATION OF A POINT XP,YP IN RELATION TO THE POLYGON
C HELD IN ARRAYS X,Y SO THAT:
C          LOC = -1  POINT OUTSIDE
C          LOC =  0  POINT ON BOUNDARY BETWEEN I1,I2
C          LOC =  1  POINT INSIDE BOUNDARY
C          LOC =  2  POINT ON BOUNDARY POINT I1
C
C CODE BY J.K.HALL, "PTLOC - A FORTRAN SUBROUTINE FOR DETERMINING
C         THE POSITION OF A POINT RELATIVE TO A CLOSED BOUNDARY",
C         MATH. GEOL., VOL.7, 1975, 75-79.
C
      INTEGER N,LOC,I1,I2,ISR
      DOUBLE PRECISION X(N),Y(N),XP,YP,TOL
C
C---LOCAL VARIABLES---
      INTEGER IEC,IP,I
      DOUBLE PRECISION C180,TSC,SC,TCC,CC,X1,Y1,R1,X2,Y2,R2,VPL,ST,CT,
     +                 ZERO,ONE
      DATA C180,ZERO,ONE/0.9999999998D0, 0.0D0, 1.0D0/
C
C---CHECK FOR CLOSURE---
      IF (N.LE.0) RETURN
      IF (DABS(X(1)-X(N)).GT.TOL .OR. DABS(Y(1)-Y(N)).GT.TOL) THEN
          WRITE(6,10) N,X(1),X(N),Y(1),Y(N)
  10      FORMAT(/' N,X,Y ',I5,4F10.5)
      ENDIF
      I1=0
      I2=0
      ISR=0
      IEC=0
      TSC=ZERO
      SC=ZERO
      TCC=ONE
      CC=ONE
      IP=N
      X1=X(N) - XP
      Y1=Y(N) - YP
      R1=DSQRT(X1*X1 + Y1*Y1)
      DO 100 I=1,N
      X2=X(I) - XP
      Y2=Y(I) - YP
      R2=DSQRT(X2*X2 + Y2*Y2)
      VPL=R1*R2
      IF (R1) 20,140,20
  20  IF (R2) 30,140,30
  30  ST=(X1*Y2 - X2*Y1)/VPL
      CT=(X1*X2 + Y1*Y2)/VPL
      IF (CT+C180) 120,120,40
  40  IF (I-N) 60,50,50
  50  IF ((IEC/2)*2-IEC) 130,110,110
  60  TSC=SC*CT + CC*ST
      TCC=CC*CT - SC*ST
      IF (TSC*SC) 70,70,90
  70  IF (TSC*ST) 80,90,90
  80  IEC=IEC + 1
  90  SC=TSC
      CC=TCC
      X1=X2
      Y1=Y2
      R1=R2
      IP=I
 100  CONTINUE
 110  LOC=-1
      RETURN
 120  LOC=0
      I1=IP
      I2=I
      RETURN
 130  ISR=1
      LOC=+1
      IF (SC.LT.ZERO) ISR=-1
      RETURN
 140  LOC=+2
      I1=I
      I2=I
      IF (R1.NE.ZERO) RETURN
      I1=IP
      I2=IP
      RETURN
      END
      REAL FUNCTION ARPOLY(N,X,Y)
C
C CALCULATES AREA ENCLOSED BY A CLOSED POLYGON
C
      INTEGER N
      DOUBLE PRECISION X(N),Y(N)
C
C---LOCAL VARIABLES---
      INTEGER I
      DOUBLE PRECISION X1,X2,Y1,Y2,SUM,TOL
C
C---CHECK FOR CLOSURE---
      IF (N.LE.0) RETURN
      TOL=0.0D0
      IF (DABS(X(1)-X(N)).GT.TOL .OR. DABS(Y(1)-Y(N)).GT.TOL)
     +    WRITE(6,5) N,X(1),X(N),Y(1),Y(N)
   5      FORMAT(/' ARPOLY ERROR  N =',I6,' X1,XN =',2F10.5,
     +            ' Y1,YN =',2F10.5)
C
      SUM=0.0D0
      X2=X(1)
      Y2=Y(1)
      DO 10 I=2,N
      X1=X2
      Y1=Y2
      X2=X(I)
      Y2=Y(I)
      SUM=SUM + (X2 - X1)*(Y1 + Y2)
  10  CONTINUE
      ARPOLY=DABS(SUM)/2.0D0
      RETURN
      END
