      SUBROUTINE RDSTRM(NFIL,OUTFIL,DATFIL,DATNAM,HEDFMT,DATFMT,STRFIL,
     +             M,N,X0,X1,Y0,Y1,H,Z0,Z1,SPVAL,X,Y,MAXD,MAXB,NST)
C
C SUBROUTINE READS STREAMLINES FROM DATFIL IN A USER DEFINED FORMAT.
C PORTIONS OF STREAMLINES OUTSIDE THE X,Y LIMITS ARE REMOVED.
C REMAINING STREAMLINE COORDINATES ARE GENERALIZED TO THE RESOLUTION
C OF THE GRID AND WRITTEN TO THE SCRATCH FILE STRFIL.
C
      INTEGER          NFIL,OUTFIL,DATFIL,STRFIL,MAXD,MAXB,NST
      REAL             Z0,Z1,SPVAL
      DOUBLE PRECISION X0,X1,Y0,Y1,H,X(MAXD),Y(MAXD)
      CHARACTER*40     DATNAM,HEDFMT,DATFMT
C
C---LOCAL VARIABLES---
      INTEGER          NUM,NN,J,K,K1,KK,N1,N2,N3
      REAL             ZHI,ZLO,ZZ1,ZZ2
      DOUBLE PRECISION XX0,YY0,XX,YY
C
C---INITIALIZE---
      WRITE(OUTFIL,10) NFIL,DATNAM
  10  FORMAT(/' STREAM LINE DATA FILE',I3,': ',A40)
      OPEN(DATFIL,FILE=DATNAM,STATUS='OLD')
      REWIND DATFIL
      N1=0
      N2=0
      N3=0
C
C---LOOP THROUGH STREAMLINES---
      DO 130 J=1,MAXD
      READ(DATFIL,HEDFMT,END=200,ERR=220) NUM,ZHI,ZLO
      IF (NUM.GT.MAXD) STOP 'RDSTRM - TOO MANY POINTS FOR BUFFER'
C
C---CHECK FOR ERRORS---
      IF (ZHI.GT.Z1 .OR. ZLO.GT.Z1) GOTO 20
      IF (ZHI.GE.ZERO .AND. ZHI.LT.Z0) GOTO 20
      IF (ZLO.GE.ZERO .AND. ZLO.LT.Z0) GOTO 20
      IF (ZHI.GE.Z0 .AND. ZHI.LE.Z1 .AND. ZLO.GE.Z0 .AND. ZLO.LE.Z1
     +    .AND. ZLO.GT.ZHI) GOTO 20
      GOTO 40
C
C---ERROR FOUND---
  20  WRITE(OUTFIL,30) J,ZHI,ZLO
  30  FORMAT(/' ILLEGAL HEIGHTS FOR',I5,'TH STREAMLINE:'/
     +        '   ZHI= ',F10.3,'  ZLO= ',F10.3)
      STOP 'RDSTRM - ERROR'
C
C---READ STREAMLINE---
  40  READ(DATFIL,DATFMT,ERR=230) (X(K),Y(K),K=1,NUM)
      N1=N1 + NUM
C
C---TRIM LINE TO X/Y LIMITS---
      K1=1
  50  DO 60 K=K1,NUM
      IF (X(K).GE.X0 .AND. X(K).LE.X1 .AND. Y(K).GE.Y0 .AND.
     +    Y(K).LE.Y1) GOTO 70
  60  CONTINUE
      GOTO 130
C
C---LINE WITHIN LIMITS---
  70  IF (K.EQ.1) THEN
         ZZ1=ZHI
      ELSE
         ZZ1=SPVAL
      ENDIF
      KK=0
      DO 80 K1=K,NUM
      IF (X(K1).LT.X0 .OR. X(K1).GT.X1 .OR. Y(K1).LT.Y0 .OR.
     +    Y(K1).GT.Y1) GOTO 90
      KK=KK + 1
      X(KK)=X(K1) - X0
      Y(KK)=Y(K1) - Y0
  80  CONTINUE
      ZZ2=ZLO
      GOTO 100
  90  ZZ2=SPVAL
C
C---INTERPOLATE EXITING STREAM TO X/Y LIMITS---
      XX0=X(KK) + X0
      YY0=Y(KK) + Y0
      XX=X(K1)
      YY=Y(K1)
      IF (XX.GT.X1) THEN
         YY=YY0 + (YY - YY0)*(X1 - XX0)/(XX - XX0)
         XX=X1
      ENDIF
      IF (XX.LT.X0) THEN
         YY=YY0 + (YY - YY0)*(X0 - XX0)/(XX - XX0)
         XX=X0
      ENDIF
      IF (YY.GT.Y1) THEN
         XX=XX0 + (XX - XX0)*(Y1 - YY0)/(YY - YY0)
         YY=Y1
      ENDIF
      IF (YY.LT.Y0) THEN
         XX=XX0 + (XX - XX0)*(Y0 - YY0)/(YY - YY0)
         YY=Y0
      ENDIF
      KK=KK + 1
      K1=K1 + 1
      X(KK)=XX - X0
      Y(KK)=YY - Y0
 100  NN=KK
      N2=N2 + NN
      IF (NN.LE.1) GOTO 120
C
C---GENERALIZE LINE TO RESOLUTION OF GRID---
      CALL SIMSTR(NN,X,Y,MAXD,H)
      N3=N3 + NN
      IF (NN.LE.1) GOTO 120
C
C---WRITE SELECTED STREAMLINE POINTS TO STRFIL---
      IF (NN.GT.MAXB) GOTO 240
      NST=NST + 1
      WRITE(STRFIL) NN,ZZ1,ZZ2
      DO 110 K=1,NN
      WRITE(STRFIL) X(K),Y(K)
 110  CONTINUE
C
C---CHECK FOR REMAINING POINTS---
 120  IF (K1.LT.NUM) GOTO 50
 130  CONTINUE
      STOP 'RDSTRM - TOO MANY STREAMLINES'
C
C---FINISH UP---
 200  CLOSE(DATFIL)
      WRITE(OUTFIL,210) N1,N2,N3
 210  FORMAT(' NUMBER OF STREAM LINE PTS       =',I7/
     +       ' NUMBER OF POINTS WITHIN LIMITS  =',I7/
     +       ' NUMBER OF POINTS ACCEPTED       =',I7)
      RETURN
C
C---ERROR CONDITIONS---
 220  WRITE(OUTFIL,225) J
 225  FORMAT(/' ERROR READING',I5,' STREAMLINE HEADER')
      STOP 'RDSTRM - ERROR'
 230  WRITE(OUTFIL,235) J
 235  FORMAT(/' ERROR READING',I5,' STREAMLINE')
      STOP 'RDSTRM - ERROR'
 240  WRITE(OUTFIL,245) NUM,MAXB
 245  FORMAT(/' WINDOWED STREAM SIZE',I6,' EXCEEDS MAXIMUM',I6)
      STOP 'RDSTRM - ERROR'
      END
      SUBROUTINE SIMSTR(NUM,X,Y,MAXD,H)
C
C GENERALIZES STREAM LINE TO RESOLUTION H OF GRID AND REMOVES
C UNNECESSARY KINKS.
C
      INTEGER          NUM,MAXD
      DOUBLE PRECISION X(MAXD),Y(MAXD),H
C
C---LOCAL VARIABLES---
      INTEGER I,J,I1,J1,K,KK,IK
      REAL ONE5
C
      DATA ONE5/1.5/
C
C---GENERALIZE LINE---
      IF (NUM.LE.1) GOTO 90
      KK=1
      I1=X(1)/H + ONE5
      J1=Y(1)/H + ONE5
      DO 20 K=2,NUM
      I=X(K)/H + ONE5
      J=Y(K)/H + ONE5
      IF (I.EQ.I1 .AND. J.EQ.J1) GOTO 20
      KK=KK + 1
      I1=I
      J1=J
      X(KK)=X(K)
      Y(KK)=Y(K)
  20  CONTINUE
      X(KK)=X(NUM)
      Y(KK)=Y(NUM)
      NUM=KK
      IF (NUM.LE.1) GOTO 90
C
C---REMOVE KINKS---
      KK=2
      I1=X(1)/H + ONE5
      J1=Y(1)/H + ONE5
      DO 60 K=3,NUM
      I=X(K)/H + ONE5
      J=Y(K)/H + ONE5
  30  IK=1
      IF (IABS(I-I1).LE.1 .AND. IABS(J-J1).LE.1) IK=IK-1
      IF (I.EQ.I1 .AND. J.EQ.J1) IK=IK-1
      KK=KK + IK
      IF (KK.LT.1) GOTO 100
      X(KK)=X(K)
      Y(KK)=Y(K)
      IF (IK.LT.1) KK=KK - 1
      IF (KK.LE.1) THEN
         I1=-1
         J1=-1
      ELSE
         I1=X(KK-1)/H + ONE5
         J1=Y(KK-1)/H + ONE5
      ENDIF
      IF (IK.LT.1) GOTO 30
  60  CONTINUE
      IF (KK.LT.1) GOTO 100
      X(KK)=X(NUM)
      Y(KK)=Y(NUM)
      NUM=KK
  90  RETURN
C
C---ERROR CONDITION---
 100  STOP 'SIMSTR ERROR'
      END
      SUBROUTINE SELSTR(OUTFIL,STRFIL,NST,U,IW,IV,M,N,H,Z0,Z1,IP,JP,
     +                  X,Y,MAXB,ICOP)
C
C READS STREAMLINES FROM THE UNFORMATTED FILE STRFIL AND
C INCORPORATES THE CORRESPONDING STREAM HEIGHTS AND ORDERED CHAIN
C CONDITIONS INTO THE GRIDS U,IW AND IV.  THE STREAM HEIGHTS ARE
C CHECKED FOR COMPATIBILITY WITH DESCENT DOWN EACH STREAMLINE.
C CIRCULAR STREAMS ARE REMOVED AND STREAMS WHICH FAIL TO CONNECT
C WITH ANOTHER STREAM BY JUST ONE GRID POINT ARE EXTENDED BY ONE
C GRID CELL SO THAT THEY DO CONNECT.
C
      INTEGER          OUTFIL,STRFIL,NST,M,N,IP(8),JP(8),MAXB,ICOP
      REAL             U(M,N),Z0,Z1
      DOUBLE PRECISION H,X(MAXB),Y(MAXB)
      INTEGER*2        IW(0:M+1,0:N+1),IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER          NS,NUM,I,J,K,I1,J1,K1,I2,J2,II2,JJ2,KK2,
     +                 KK,KS1,KK3,KS(3),IW2
      REAL             ZHI,ZLO,ONE5
C
      DATA ONE5/1.5/
      DATA KS/7, 6, 0/
C
C---INITIALIZE---
      REWIND STRFIL
C
C---LOOP THROUGH STREAMLINES---
      DO 20 NS=1,NST
      READ(STRFIL,ERR=100) NUM,ZHI,ZLO
      IF (NUM.GT.MAXB) STOP 'SELSTR - ERROR'
      DO 10 K=1,NUM
      READ(STRFIL,ERR=100) X(K),Y(K)
  10  CONTINUE
C
C---GENERALIZE STREAMLINE---
      CALL SIMSTR(NUM,X,Y,MAXB,H)
      IF (NUM.LE.1) GOTO 20
C
C---SET FIRST HEIGHT---
      I=X(1)/H + ONE5
      J=Y(1)/H + ONE5
      IF (ZHI.GT.Z0 .AND. ZHI.LT.Z1) THEN
         IW(I,J)=1
         U(I,J)=ZHI
      ENDIF
C
C---SET LAST HEIGHT---
      I=X(NUM)/H + ONE5
      J=Y(NUM)/H + ONE5
      IF (ZLO.GT.Z0 .AND. ZLO.LT.Z1) THEN
         IW(I,J)=1
         U(I,J)=ZLO
      ENDIF
C
C---SET POINTERS FOR STREAMLINE---
      CALL SETLIN(U,IW,IV,M,N,H,IP,JP,X,Y,NUM,ICOP)
  20  CONTINUE
C
C---REMOVE CIRCULAR STREAMS---
      DO 30 J=1,N
      DO 30 I=1,M
      K=IV(I,J)
      IF (K.LE.0) GOTO 30
      I1=I + IP(K)
      J1=J + JP(K)
      K1=IV(I1,J1)
      IF (K1.LE.0) GOTO 30
      IF (K.EQ.MOD(K1+3,8)+1) THEN
         IV(I,J)=0
         IV(I1,J1)=0
      ENDIF
  30  CONTINUE
C
C---CONNECT STREAMS WHERE POSSIBLE---
      DO 70 J=1,N
      DO 70 I=1,M
      K=IV(I,J)
      IF (K.LE.0) GOTO 70
      I1=I + IP(K)
      J1=J + JP(K)
      K1=IV(I1,J1)
      IF (K1.LE.0) GOTO 70
      I2=I1 + IP(K1)
      J2=J1 + JP(K1)
      IF (IV(I2,J2).GT.0) GOTO 70
C
C---FIRST TRY TO REDIRECT LAST POINTER WITHOUT SHARP KINKS---
      DO 50 KK=1,3
      KK1=MOD(K+KS(KK),8) + 1
      IF (KK1.EQ.K1) GOTO 50
      II2=I1 + IP(KK1)
      JJ2=J1 + JP(KK1)
      IF (IW(II2,JJ2).LT.0) GOTO 50
      KK2=IV(II2,JJ2)
      IF (KK2.LE.0) GOTO 50
      DO 40 KS1=1,3
      IF (KK2.EQ.MOD(KK1+KS(KS1),8)+1) THEN
         IV(I1,J1)=KK1
         GOTO 70
      ENDIF
  40  CONTINUE
  50  CONTINUE
C
C---ELSE TRY TO EXTEND STREAM---
      DO 60 KK=1,3
      KK2=MOD(K1+KS(KK),8) + 1
      II2=I2 + IP(KK2)
      JJ2=J2 + JP(KK2)
      IF (IW(II2,JJ2).LT.0) GOTO 60
      KK3=IV(II2,JJ2)
      IF (KK3.GT.0 .AND. KK3.NE.MOD(KK2+3,8)+1) THEN
         IV(I2,J2)=KK2
         GOTO 70
      ENDIF
  60  CONTINUE
C
C---INSERT SINK IF STREAM EXITS---
      IF (IW(I2,J2).GE.0) THEN
         II2=I2 + IP(K1)
         JJ2=J2 + JP(K1)
         IF (IW(II2,JJ2).LT.0) THEN
            IW2=IW(I2,J2)
            IW(I2,J2)=MOD(IW2,2) + 4
         ENDIF
      ENDIF
  70  CONTINUE
      RETURN
C
C---ERROR CONDITIONS---
 100  STOP 'SELSTR - ERROR READING STRFIL'
      END
      SUBROUTINE SETLIN(U,IW,IV,M,N,H,IP,JP,X,Y,NUM,ICOP)
C
C SETS POINTERS IN THE ARRAY IV, WITHOUT KINKS, FOR THE LINE
C STRING HELD IN THE ARRAYS X,Y.  IF ICOP > 0 THEN LINES ARE
C CHECKED FOR CROSSING OF CONTOUR LINES AND ADDITIONAL CONTOUR
C DATA POINTS ARE INSERTED WHERE NECESSARY.
C
      INTEGER          M,N,IP(8),JP(8),NUM,ICOP
      REAL             U(M,N)
      DOUBLE PRECISION H,X(NUM),Y(NUM)
      INTEGER*2        IW(0:M+1,0:N+1),IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER          I0,J0,I1,J1,I,J,K,KK,KL,NN
      REAL             DX,DY,DIS,ONE5
      DOUBLE PRECISION X1,Y1,XK,YK,XX,YY
C
      DATA             ONE5/1.5/
C
C---SET FIRST POSITION---
      X1=X(1)
      Y1=Y(1)
      I1=X1/H + ONE5
      J1=Y1/H + ONE5
      I0=-1
      J0=-1
C
C---LOOP THROUGH REST OF LINE---
      DO 70 K=2,NUM
      XK=X(K)
      YK=Y(K)
      I=XK/H + ONE5
      J=YK/H + ONE5
      IF (I.EQ.I1 .AND. J.EQ.J1) GOTO 70
      DX=XK - X1
      DY=YK - Y1
      NN=1
      IF (IABS(I-I1).LE.1 .AND. IABS(J-J1).LE.1) GOTO 30
  20  DIS=SQRT(DX*DX + DY*DY)
      NN=MAX1(2.0*DIS/SNGL(H), 2.0)
      DX=DX/NN
      DY=DY/NN
C
C---REFINE LINE TO RESOLUTION OF GRID---
  30  DO 60 KK=1,NN
      XX=X1 + KK*DX
      YY=Y1 + KK*DY
      I=XX/H + ONE5
      J=YY/H + ONE5
      IF (I.EQ.I1 .AND. J.EQ.J1) GOTO 60
      IF (IABS(I-I0).LE.1 .AND. IABS(J-J0).LE.1) THEN
         I1=I0
         I0=-1
         J1=J0
         J0=-1
      ENDIF
      IF (I.EQ.I1 .AND. J.EQ.J1) GOTO 60
      DO 40 KL=1,8
      IF (I-I1.EQ.IP(KL) .AND. J-J1.EQ.JP(KL)) GOTO 50
  40  CONTINUE
      IF (NN.EQ.1) GOTO 20
      STOP 'SETLIN - ERROR'
C
C---SET POINTER AND OPTIONALLY CHECK FOR CONTOUR CROSSING---
  50  if (i1.lt.0 .or. i1.gt.m .or. j1.lt.0 .or. j1.gt.n
     +   .or. i.lt.0 .or. i.gt.m .or. j.lt.0 .or. j.gt.n) then
	write(6,52) 1,x(1),y(1),k,x(k),y(k),num,x(num),y(num)
  52	format(/' setlin error '/(i5,2f10.4))
	write(6,53) kk,nn,xx,yy,i0,j0,i,j,i1,j1,kl
  53	format(/' kk,nn =',2i5/'  xx,yy =',2f10.4/
     +   ' i0,j0 =',2i4,'  i,j =',2i4,'  i1,j1 =',2i4,'  kl =',i3)
	stop
	endif
      IV(I1,J1)=KL
      IF (ICOP.GT.0) THEN
         IF (I.NE.I1 .AND. J.NE.J1 .AND. IW(I1,J1).EQ.0) THEN
            IF (IW(I,J1).GT.0 .AND. IW(I1,J).GT.0) THEN
               IW(I1,J1)=1
               U(I1,J1)=0.5*(U(I,J1) + U(I1,J))
            ENDIF
         ENDIF
      ENDIF
C
C---UPDATE INDICES---
      I0=I1
      I1=I
      J0=J1
      J1=J
  60  CONTINUE
      X1=XK
      Y1=YK
  70  CONTINUE
      RETURN
      END
      SUBROUTINE INCSTR(OUTFIL,U,IW,IV,M,N,X0,Y0,H,Z0,Z1,IP,JP,
     +                  TOL,ICOP)
C
C INCORPORATES STREAMLINES INTO GRIDS U,IW,IV.
C
      INTEGER          OUTFIL,M,N,IP(8),JP(8),ICOP
      REAL             U(M,N),Z0,Z1,TOL
      DOUBLE PRECISION X0,Y0,H
      INTEGER*2        IW(0:M+1,0:N+1),IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER          IOP1,IOP2
C
C---SET FLAGS---
      CALL SETFLG(IV,M,N,IP,JP)
C
C---SET STREAMS---
      IOP1=0
      IOP2=2
      CALL SETSTR(OUTFIL,U,IW,IV,M,N,X0,Y0,H,Z0,Z1,IP,JP,TOL,
     +            IOP1,IOP2)
C
C---SET SIDES---
      IOP1=0
      IOP2=3
      CALL SETSID(OUTFIL,U,IW,IV,M,N,X0,Y0,H,IP,JP,TOL,
     +            IOP1,IOP2,ICOP)
      IOP1=2
      CALL SETSTR(OUTFIL,U,IW,IV,M,N,X0,Y0,H,Z0,Z1,IP,JP,TOL,
     +            IOP1,IOP2)
C
C---DELETE FLAGS---
      CALL DELFLG(IV,M,N)
      RETURN
      END
      SUBROUTINE SETFLG(IV,M,N,IP,JP)
C
C SETS A FLAG ON ALL LOWER STREAM POINTS.
C
      INTEGER   M,N,IP(8),JP(8)
      INTEGER*2 IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER   I,J,I1,J1,K
C
      DO 10 J=1,N
      DO 10 I=1,M
      K=IV(I,J)
      K=MOD(K,10)
      IF (K.LE.0) GOTO 10
      I1=I + IP(K)
      J1=J + JP(K)
      K=IV(I1,J1)
      IV(I1,J1)=MOD(K,10) + 10
  10  CONTINUE
      RETURN
      END
      SUBROUTINE DELFLG(IV,M,N)
C
C DELETES FLAGS ON ALL LOWER STREAM POINTS.
C
      INTEGER   M,N
      INTEGER*2 IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER   I,J,K
C
      DO 10 J=1,N
      DO 10 I=1,M
      K=IV(I,J)
      IV(I,J)=MOD(K,10)
  10  CONTINUE
      RETURN
      END
      SUBROUTINE SETSTR(OUTFIL,U,IW,IV,M,N,X0,Y0,H,Z0,Z1,IP,JP,
     +                  TOL,IOP1,IOP2)
C
C SETS HEIGHTS OF STREAM POINTS.  CONFLICTS BETWEEN DATA POINTS
C AND STREAMLINE CONDITIONS WHICH EXCEED TOL ARE WRITTEN TO OUTFIL.
C IF IOP2 IS GREATER THAN 2 THEN ONLY STREAMS WHOSE FIRST POINTS
C ARE DATA POINTS ARE SET.  A FLAG IS SET BY SUBROUTINE SETSEG FOR 
C ALL LOWER STREAM POINTS AS THEY ARE PROCESSED TO PREVENT MULTIPLE
C PROCESSING OF STREAM POINTS.
C
      INTEGER          OUTFIL,M,N,IP(8),JP(8),IOP1,IOP2
      REAL             U(M,N),Z0,Z1,TOL
      DOUBLE PRECISION X0,Y0,H
      INTEGER*2        IW(0:M+1,0:N+1),IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER          MN,I,J,K,IK,I0,J0,I1,J1,KS,KK,NUM,IFLG
      REAL             ZHI,DU
C
C---INITIALISE---
      MN=M + N
      DU=AMIN1((Z1 - Z0)/100.0, 1.0)
      IF (IOP1.LT.0 .OR. IOP1.GE.IOP2) STOP 'SETSTR - ILLEGAL IOP1'
      IF (IOP2.LT.2 .OR. IOP2.GT.3) STOP 'SETSTR - ILLEGAL IOP2'
      IFLG=IOP2*10
C
C---SET HEIGHTS FROM TOP OF EACH STREAM---
      DO 80 J=1,N
      DO 80 I=1,M
      K=IV(I,J)
      IK=K/10
      K=MOD(K,10)
      IF (K.LE.0 .OR. IK.NE.IOP1) GOTO 80
      IF (IW(I,J).EQ.1) THEN
         ZHI=U(I,J)
      ELSE
         IF (IOP2.GT.2) GOTO 80
         ZHI=Z1
      ENDIF
      I0=I
      J0=J
      I1=I0
      J1=J0
C
C---LOOP THROUGH STREAM SEGMENTS---
      NUM=0
      DO 60 KS=1,MN
      DO 30 KK=1,MN
      NUM=NUM + 1
      I1=I1 + IP(K)
      J1=J1 + JP(K)
      K=IV(I1,J1)
      K=MOD(K,10)
      IF (IW(I1,J1).EQ.1) THEN
         IF (U(I1,J1).LT.ZHI) THEN
            GOTO 40
         ELSE
            IW(I1,J1)=0
            IF (U(I1,J1).GT.ZHI+TOL) THEN
               WRITE(OUTFIL,20) X0+(I0-1)*H,Y0+(J0-1)*H,U(I0,J0),
     +                          X0+(I1-1)*H,Y0+(J1-1)*H,U(I1,J1)
  20           FORMAT(' DOWNSTREAM DATA POINT IGNORED:'/
     +         '   UPPER  (X,Y,Z) = (',2F10.3,F8.1,')',
     +         '   LOWER  (X,Y,Z) = (',2F10.3,F8.1,')')
            ENDIF
         ENDIF
      ENDIF
      IF (K.LE.0) GOTO 40
  30  CONTINUE
C
C---CIRCULAR STREAM DETECTED---
      CALL WRGRDS(OUTFIL,U,IW,IV,M,N,I0,J0,6)
      IF (IABS(I1-I0).GT.5 .OR. IABS(J1-J0).GT.5) THEN
         CALL WRGRDS(OUTFIL,U,IW,IV,M,N,I1,J1,6)
      ENDIF
      WRITE(OUTFIL,35) IV(I1,J1),I1,J1
  35  FORMAT(//' CIRCULAR STREAM - POINTER',I3,' REMOVED AT  I=',I4,
     +         '  J=',I4)
      K=IV(I1,J1)/10
      IV(I1,J1)=K*10
C
C---CHECK HEIGHTS OF START AND END OF SEGMENT---
  40  IF (U(I1,J1).LE.U(I0,J0)) GOTO 50
      IF (IW(I0,J0).EQ.0) THEN
         IF (IW(I1,J1).EQ.1) THEN
            U(I0,J0)=AMIN1(U(I1,J1)+DU, Z1)
         ELSE
            U(I0,J0)=AMIN1((U(I0,J0) + U(I1,J1) + DU)*0.5, Z1)
            U(I1,J1)=AMAX1(U(I0,J0)-DU, Z0)
         ENDIF
      ELSE
         U(I1,J1)=AMAX1(U(I0,J0)-DU, Z0)
      ENDIF
C
C---SET SEGMENT---
  50  CALL SETSEG(I0,J0,I1,J1,NUM,U,IV,M,N,IP,JP,IOP2)
      K=IV(I1,J1)
      IF (K.GE.IFLG) GOTO 80
      K=MOD(K,10)
      IV(I1,J1)=K + IFLG
      IF (K.LE.0) GOTO 80
      NUM=0
      I0=I1
      J0=J1
      ZHI=U(I0,J0)
  60  CONTINUE
      STOP 'SETSTR - TOO MANY SEGMENTS'
  80  CONTINUE
      RETURN
      END
      SUBROUTINE SETSID(OUTFIL,U,IW,IV,M,N,X0,Y0,H,IP,JP,TOL,
     +                  IOP1,IOP2,ICOP)
C
C SETS SIDE ORDERED CHAIN CONDITIONS FOR STEAMLINES.  IF ICOP
C EXCEEDS 0 THEN SIDE CONDITIONS ARE NOT SET FOR DATA POINTS
C BESIDE STREAMS WHICH WOULD POINT TO A NON-HIGHER DATA POINT
C ON THE STREAM.
C
      INTEGER          OUTFIL,M,N,IP(8),JP(8),IOP1,IOP2,ICOP
      REAL             U(M,N),TOL
      DOUBLE PRECISION X0,Y0,H
      INTEGER*2        IW(0:M+1,0:N+1),IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER          MN,I,J,K,IK,JK,IK1,JK1,II,IFLG,KS(3)
      REAL             UK,UK1
C
      DATA             KS/7, 6, 0/
C
C---INITIALISE---
      MN=4*(M + N)
      IF (IOP1.LT.0 .OR. IOP1.GE.IOP2) STOP 'SETSID - ILLEGAL IOP1'
      IF (IOP2.NE.3) STOP 'SETSID - ILLEGAL IOP2'
      IFLG=IOP2*10
C
C---SET SIDES FROM TOP OF EACH STREAM---
      DO 80 J=1,N
      DO 80 I=1,M
      K=IV(I,J)
      IK=K/10
      K=MOD(K,10)
      IF (K.LE.0 .OR. IK.NE.IOP1) GOTO 80
C
C---STEP DOWN STREAM---
      UK=U(I,J)
      IK=I + IP(K)
      JK=J + JP(K)
      DO 60 KK=1,MN
      UK1=UK
      UK=U(IK,JK)
C
C---SET FLAG ON STREAM POINT OR EXIT IF FLAG SET---
      K=IV(IK,JK)
      IF (K.GE.IFLG) GOTO 80
      K=MOD(K,10)
      IV(IK,JK)=K + IFLG
      IF (K.LE.0) GOTO 80
C
C---SET POINTERS---
      DO 30 II=1,3
      K1=MOD(K+KS(II),8) + 1
      IK1=IK - IP(K1)
      JK1=JK - JP(K1)
      IF (IW(IK1,JK1).LT.0) GOTO 30
      IF (IV(IK1,JK1).NE.0) GOTO 30
      IF (IW(IK1,JK1).GT.0) THEN
C
C---CHECK FOR DATA POINT POINTING TO DATA POINT---
         IF (ICOP.GT.0 .AND. IW(IK,JK).GT.0) THEN
            IF (UK.LE.U(IK1,JK1)) GOTO 20
         ENDIF
C
C---CHECK FOR DATA POINT HEIGHT DISCREPANCY EXCEEDING TOL---
         IF (U(IK1,JK1)+TOL.LT.UK) THEN
            WRITE(OUTFIL,10) X0+(IK1-1)*H,Y0+(JK1-1)*H,U(IK1,JK1),
     +                       UK-U(IK1,JK1)
  10        FORMAT(' SIDE DATA POINT IGNORED: ',
     +      '   (X,Y,Z) = (',2F10.3,F8.1,')   DIFF =',F8.1)
            GOTO 20
         ENDIF
C
C---NON-DATA POINT---
      ELSE
         U(IK1,JK1)=AMAX1(U(IK1,JK1),UK1)
      ENDIF
      IV(IK1,JK1)=K1 + 20
  20  IF (II.EQ.1) GOTO 40
  30  CONTINUE
C
C---STEP DOWN STREAM---
  40  IK=IK + IP(K)
      JK=JK + JP(K)
  60  CONTINUE
      STOP 'SETSID - STREAM TOO LONG'
  80  CONTINUE
      RETURN
      END
      SUBROUTINE SETSEG(I0,J0,I1,J1,NUM,U,IV,M,N,IP,JP,IOP)
C
C SETS HEIGHTS OF STREAM SEGMENT FROM I0,J0 TO I1,J1.
C
      INTEGER   I0,J0,I1,J1,NUM,M,N,IP(8),JP(8),IOP
      REAL      U(M,N)
      INTEGER*2 IV(M,N)
C
C---LOCAL VARIABLES---
      INTEGER   I,J,K,IKL,KL,IFLG
      REAL      U0,U1,UH,DU
C
C---INITIALISE---
      IF (NUM.LE.1) RETURN
      IFLG=IOP*10
      U0=U(I0,J0)
      U1=U(I1,J1)
      IF (U1.GT.U0) GOTO 100
      DU=(U0 - U1)/NUM
      UH=U0
      I=I0
      J=J0
      IKL=IV(I,J)
      KL=MOD(IKL,10)
C
C---STEP DOWN SEGMENT---
      DO 10 K=1,NUM-1
      IF (KL.GT.0) THEN
         I=I + IP(KL)
         J=J + JP(KL)
         IKL=IV(I,J)
         U(I,J)=AMAX1(AMIN1(U(I,J), UH, U0-K*DU), U1)
         UH=U(I,J)
      ELSE
         GOTO 20
      ENDIF
      KL=MOD(IKL,10)
      IV(I,J)=KL + IFLG
  10  CONTINUE
  20  RETURN
C
C---ERROR CONDITION---
 100  STOP 'SETSEG - INCOMPATIBLE STREAM HEIGHTS'
      END
