/****************************************************************************  
* Name  ASC2GEN
*
* Purpose  Convert an ASCII file to an Arc Generate-line 
* ingest format
*
* Program History
*       Programmer         Date           Reason
*       ----------         ----           ------
*       K. Lowell          08/93          Original development
*
* Computer Hardware and/or software limitations
*       Must be run under TAE
*
* PROJECT      LAS
*
* LANGUAGE     C      
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"  
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK parblk;           /* TAE parameter block */

FILE  *file_ptr,                /* ptr to input file */
      *out_ptr;                 /* ptr to output file */ 

long  	count = 0,              /* counter variable  */
        fatal = 0,		/* fatal error constant */
        hold = 0,               /* comparison value for loop */
        id,                     /* ID of Arc  */
        msgtim, 		/* time for processing message */
        one = 1,                /* constant of 1 */
        std_out;                /* standard output */

float   x,                      /* X value of arc node */
        y;                      /* Y value of arc node */

static char	infile[CMLEN],  /* input file */
                outfile[1][CMLEN], /* output file */
                line[CMLEN];    /* input file read buffer */

/* Initialize TAE parameter block
---------------------------------*/
c_inlas(&parblk,&std_out);

/*  Initialize LINE
-------------------*/
strcpy(line,"");

/* Get user-specified parameters
--------------------------------*/
if ((getpar(&parblk,infile,outfile[0],&msgtim))
   != E_SUCC)
   {
   c_errmsg("Fatal error encountered","asc2gen-fatal",&fatal);
   } 

/* Open input file
------------------*/
if ((file_ptr = fopen(infile,"r")) == NULL)
  {
  c_errmsg("Error opening input file","asc2gen-open",&fatal);
  }

/* Open output file
-------------------*/
if ((out_ptr = fopen(outfile[0],"w")) == NULL)
  {
  c_errmsg("Error opening output file","asc2gen-open",&fatal);
  }

/* Read the input file until End of File (EOF) is found
   In the output file the value of ID is written only
   once per arc, this value is stored in the parameter
   HOLD, and a comparison is done on each read line of
   input data.
-------------------------------------------------------*/
for (;;)
   {
   if (fgets(line,80,file_ptr) != NULL) 
     {
     sscanf(line,"%d %f %f", &id,&x,&y);
     if(id != hold) 
       {
       if(count == 0) 
         {
         fprintf(out_ptr,"%d\n", id);
         fprintf(out_ptr,"%f %f\n", x, y);
         hold = id;
         count++;
         }
       else 
         {
         fprintf(out_ptr, "END\n");
         fprintf(out_ptr, "%d\n", id);
         fprintf(out_ptr,"%f %f\n", x, y);
         hold = id;
         }
       }
     else fprintf(out_ptr, "%f %f\n", x, y);
     } 
   else 
     {
     fprintf(out_ptr,"END\n");
     fprintf(out_ptr,"END\n");
     fclose(out_ptr);
     fclose(file_ptr);
     break;
     } 
  }

/* Output completion message
-----------------------------*/
c_complt("asc2gen",outfile,&one);

return EXIT_SUCCESS;

} 
