/**************************************************************************
NAME: DOQARCHIVE
 Looks for doq files with extension of .4 in the /sg1/doq/dlt
 directory, and writes them out to DLT (up to 20Gb of data) 
 DOQ files that have problems will be placed in the /sg1/doq/dlt/error
 directory for problem resolution.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0                ???                 initial development
 2.0       10/98    D. Hames            Add code to to comply
                                        with localtime & y2k



***************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include "las.h"


main()
{
FILE  *ptr;

struct tm *curtime,             /* current time                         */
          *localtime();         /* get current time                     */

long    ii = 0;			/* loop counters		*/
long	status=0;               /* Status indicator       	*/
long 	tmpsize = 0;            /* Tmp file size hold     	*/
long 	zero = 0;		/* constant of zero	 	*/
long 	bytesize[20];		/* individual file sizes	*/
long    now = 0;		/* time now			*/

char 	temp[256];		/* temp hold string		*/
char 	sstatus[5];		/* string status check		*/
char	callstr[256];		/* system call string		*/
char	logfile[CMLEN];		/* message logfile		*/
char  	trakfile[CMLEN];	/* tracking file 		*/
char  	*char_buf;		/* character i/o buffer		*/
char    *p;                     /* header string hold           */
char 	tstamp[20];		/* time stamp			*/	
char dd[3];                     /* day                                  */
char mm[3];                     /* month                                */
char yy[3];                     /* year                                 */
char hour[3];                   /* hours                                */
char mins[3];                   /* minute                               */
char secs[3];                   /* second                               */
char *sptr[1];                  /* pointer array for updating vblock    */
long one = 1;                   /* value of one                         */
long std_out;                   /* standard output device               */
long year;                      /* year                                 */

char dir[256];   	/* Directory portion of the file specification  */
char ext[256];    	/* Extension portion of the file specification  */
char filename[256];	/* User supplied file specification             */
char root[256];  	/* Root portion of the file specification       */
char trash[100];	/* trash for scanf				*/

char filespec[CMLEN];           /* host file specification                   */
char hnames[MAXFILES][CMLEN];   /* array for all files in the input list     */
char tapecap[CMLEN];		/* current tape capacity	*/
char tapecont[CMLEN];		/* current tape contents	*/
char tmpfile[CMLEN];		/* temp working file		*/		
long ck_lock = 0;               /* 1 = return list of unlocked files only,   */
                                /* 0 = return list of all files found        */
long first_call = 1;            /* only one call to c_getfiles               */
long maxfiles = 10;             /* maximum number of files to retrieve       */
long nfiles = 0;                /* number of files found                     */
long firstfile = 1;             /* first filename number                     */
long tapestatus = 0;		/* 0=first, 1=continue, 2=last		*/
long paxflag = 1;		/* 0=no pax, 1=make pax tape		*/

double dltsize = 19000000;	/* DLT tape capacity		*/
double volsize = 0.0;
double totsize = 0.0;

/* get the current date and time for timestamps
------------------------------------------------*/
now = time(NULL);
curtime = localtime(&now);

/*    used to retrieve 2 year digits for dates after the year 2000
-------------------------------------------------------------------*/
year = curtime->tm_year;
year = year%100;
 
sprintf(mm,"%02d",curtime->tm_mon + 1);
sprintf(dd,"%02d",curtime->tm_mday);
sprintf(yy,"%02d",year);
sprintf(hour,"%02d",curtime->tm_hour);
sprintf(mins,"%02d",curtime->tm_min);
sprintf(secs,"%02d",curtime->tm_sec);
sprintf(tstamp,"_%s%s%s",hour,mins,secs);
strcpy(trakfile,"/sg1/doq/dlt/trakfile");

/* Get the input file names
------------------------------*/
strcpy(filespec,"/sg1/doq/dlt/tar/*.4");
status = c_getfiles(filespec,hnames,&nfiles,maxfiles,first_call,ck_lock);

for (ii=0;ii<nfiles;ii++) bytesize[ii]=0;

for (ii=0;ii<nfiles;ii++)
  {
  tmpsize = 0;
  sprintf(callstr,"%s %s %s %s","ls -l",hnames[ii],
		" > ","tmpfile");
  system(callstr);
 
  c_splitname(hnames[ii],dir,root,ext);
  sprintf(callstr,"mv %s %s%s",hnames[ii],dir,root);
  system(callstr); 
  sprintf(callstr,"echo %s >> %s",root,trakfile);
  system(callstr);

  if ((ptr = fopen("tmpfile","r")) != NULL)
    {
    fscanf(ptr,"%s %s %s %s %d",trash,trash,trash,trash,&tmpsize);
    fclose(ptr);
    wait(&status);
    totsize = totsize + tmpsize;
    bytesize[ii] = bytesize[ii] + tmpsize;
    }
  unlink("tmpfile");
  }

/* get the current tape info
-----------------------------*/
strcpy(tapecap,"/sg1/doq/dlt/tapecap");
if ((ptr = fopen(tapecap,"r")) != NULL)
   {
   fscanf(ptr,"%lf",&volsize);
   fclose(ptr);
   wait(&status);
   }

totsize = volsize + totsize;
if (volsize == 0) 
  {
  tapestatus = 0;
  }
else if (totsize < (dltsize*1000))
   {
   tapestatus=1;
   }
else
   {
   tapestatus=2;
   }



/* based on tape status, make appropriate call to pax, provided
the PAXFLAG is set to true.
----------------------------------------------------------------*/
if (paxflag !=0)
  {
  strcpy(callstr,"/sg1/doq/dlt/tar");
  chdir(callstr); 
  if (tapestatus == 0)
    {
    strcpy(callstr,"pax -wqvLb 10240 -M \"mount DLT sg1@dlt1\"  -R yes *");
    status = system(callstr);
    if (status == 0)
      {
      sprintf(callstr,"echo %f > /sg1/doq/dlt/tapecap",totsize);
      system(callstr);
      }
    }
  else if (tapestatus == 1)
    {
    strcpy(callstr,"pax -AwqvLb 10240 -T dlt1 -R yes  *");
    status = system(callstr);
    if (status == 0)
      {
      sprintf(callstr,"echo %f > /sg1/doq/dlt/tapecap",totsize);
      system(callstr);
      }
    }
  else /* tapestatus == 2 */
    {
    strcpy(callstr,"pax -AwqvLb 10240 -T dlt1 *");
    status = system(callstr);
    if (status == 0)
      {
      strcpy(callstr,"echo 0.0 > /sg1/doq/dlt/tapecap");
      system(callstr);
      sprintf(callstr,"mv %s %s.%s%s%s",trakfile,trakfile,hour,mins,secs);
      system(callstr);
      }
    }
  }

  if (status == 0)
    {
    strcpy(callstr,"rm /sg1/doq/dlt/tar/*");
    system(callstr);
    }
  else
    {
    strcpy(callstr,"mv /sg1/doq/dlt/tar/* /sg1/doq/dlt/error");
    system(callstr);
    }

}
