#include <stdlib.h>
#include <stdio.h>
#include <string.h>

char **split(char *src, int delimiter, int *nbrfields);

/*
 *	Look up the GLIS browse header information.
 *	Returns a pointer to a static area that is overwritten with each call.
 */
char *
glisbrhdr(char *targetfilename, char *jpegfilename)
	{
	FILE *hdrinfo;
	static char *hdrinfofilename = "browse_info.lst";
	char hdrinbuf[1025];
	static char hdr[136];	/* to be returned */
	char **fields;
	int nbrfields;
	int width, height;	/* width and height of JPEG image */

	if (!(hdrinfo = fopen(hdrinfofilename, "r")))
		{
		perror(hdrinfofilename);
		exit(1);
		}
	while (fgets(hdrinbuf, 1024, hdrinfo))
		{
		int i;
		if (strncmp(hdrinbuf, targetfilename, strlen(targetfilename)) != 0)
			continue;
		hdrinbuf[strlen(hdrinbuf)-1] = '\0';	/* remove the newline */
		if (!((fields = split(hdrinbuf, '|', &nbrfields)) && (nbrfields >= 14)))
			continue;
		/*
		 * The first field is the file name.  Compare it
		 * to the file name passed in.
		 */
		if (strcmp(targetfilename, fields[0]) != 0)
			continue;

		/*
		 * We have a match.  Copy the rest of the fields.
		 */
		hdr[0] = '\006';
		/* The second field is the entity ID */
		strncpy_blankfill(hdr+1, fields[1], 30);

		/* B&W or color flag */
		strncpy_blankfill(hdr+31, fields[2], 1);

		/* number of lines and samples */
		if (!jpegfilename)
			{
			jpegfilename = malloc(strlen(targetfilename) + 4);
			strcpy(jpegfilename, targetfilename);
			strcpy(jpegfilename+strlen(jpegfilename)-2, ".jpg");
			}
		height = width = 0;
		getjpegsize(jpegfilename, &height, &width);
		sprintf(hdr+32, "%4d%4d", height, width);

		/* acquisition date */
		strncpy_blankfill(hdr+40, fields[3], 8);

		/* corner coordinates */
		for (i=0; i<4; i++)
			{
			fmtnum(hdr+48+(i*17) , fields[4+(i*2)], 2, 8);
			fmtnum(hdr+56+(i*17) , fields[5+(i*2)], 3, 9);
			}
		/* center coordinates */
		fmtnum(hdr+116 , fields[12], 2, 9);
		fmtnum(hdr+125 , fields[13], 3, 10);

		hdr[135] = '\0';
		fclose(hdrinfo);
		return(hdr);
		}
	fclose(hdrinfo);
	return((char *)0);
	}
/*
 *	Split a delimted string into individual components.
 *	Returns an array of pointers.
 */
char **split(char *src, int delimiter, int *nbrfields)
	{
	int n;			/* number of fields to be retured */
	register char *p;	/* scratch pointer */
	char **array;		/* to be returned */

	/*
	 *	First, count the number of fields so we can allocate space for the array.
	 */
	for (p=src, n=1; ; n++,p++)
		{
		if (!(p = strchr(p, delimiter)))
			break;
		}
	array = (char **)malloc((n + 1) * sizeof(char *));
	if (!array)
		return (char **)0;

	/*
	 *	Now go back and get the fields.
	 */
	for (p=src, n=0; ; n++,p++)
		{
		char *q;
		int sz;
		if (p = strchr(q=p, delimiter))
			sz = p-q;
		else
			sz = strlen(q);
		if (!(array[n] = (char *)malloc(sz + 1)))
			return (char **)0;
		strncpy(array[n], q, sz);
		array[n][sz] = '\0';
		if (!p)
			break;
		}
	array[n+1] = (char *)0;
	if (nbrfields)
		*nbrfields = n;
	return array;
	}
/*
 *	Just like strncpy, but blank fills to exactly n
 */
strncpy_blankfill(char *dst, char *src, int n)
	{
	int srcsz = strlen(src);
	strncpy(dst, src, n);
	while (n > srcsz)
		{
		dst[srcsz] = ' ';
		srcsz++;
		}
	}
/*
 *	Read a JPEG file and return the width and height.
 *	Returns 1 for success and 0 for failure.
 */
static short get2bytes(FILE *);
getjpegsize(char *jpgfile, int *height, int *width)
	{
	FILE *jpg;
	int c;

	if (!(jpg = fopen(jpgfile, "r")))
		{
		perror(jpgfile);
		return 0;
		}
	while ((c = getc(jpg)) != -1)
		{
		if (c == 0xff)
			{
			c = getc(jpg);
			if (c == -1)
				return 0;
			switch (c)
				{
				case 0xd8:
					break;
				case 0xc0:
				case 0xc1:
					/* SOF marker -- get the width and height */
					c = get2bytes(jpg);	/* skip the size */
					getc(jpg);		/* skip the precision */
					*height = get2bytes(jpg);
					*width = get2bytes(jpg);
					fclose(jpg);
					return 1;
					break;
				default:
					/* just skip any other marker */
					c = get2bytes(jpg);
					for (c -= 2; c > 0; c--)
						{
						if (getc(jpg) == -1)
							break;
						}
				}
			}
		}
	fclose(jpg);
	return 0;
	}
static short
get2bytes(FILE *jpg)
	{
	int c1, c2;

	c1 = getc(jpg);
	c2 = getc(jpg);
	if ((c1 == -1) || (c2 == -1))
		return 0;
	return ((c1 & 0xff) << 8) | (c2 & 0xff);
	}

/*
 *	Take a latitude or longitude value (e.g. 40.90625) and put it
 *	"canonical" form.  Parameters are as follows:
 *		dst	destination buffer.  Must be at least "width"
 *		src	source string
 *		left	number of digits to the left of the decimal point
 *		width	total width of the field, including sign and decimal
 *	For example, fmtnum(x, "41", 2, 8) returns the string "+41.0000" in x.
 *	and fmtnum(x, "-98.4375", 3, 9) returns "-098.4375" in x.
 */
fmtnum(char *dst, char *src, int left, int width)
	{
	char *decimal;		/* pointer to decimal point in input */
	int sz;

	/*
	 *	Skip any leading blanks.
	 */
	while (*src == ' ')
		src++;

	/*
	 *	Look for a sign, if there isn't one, put in a plus.
	 */
	if ((*src == '+') || (*src == '-'))
		{
		*dst = *src;
		src++;
		}
	else
		*dst = '+';
	dst++;
	width--;

	/*
	 *	Look for a decimal point.
	 */
	decimal = strchr(src, '.');
	if (decimal)
		sz = decimal - src;
	else
		sz = strlen(src);
	/*
	 *	Copy the mantissa, inserting leading zeroes if necessary.
	 */
	while (sz < left)
		{
		*dst = '0';
		width--;
		dst++;
		left--;
		}
	strncpy(dst, src, sz);
	width -= sz;
	src += sz;
	dst += sz;
	if (decimal)
		src++;
	*dst = '.';
	dst++;
	width--;
	/*
	 *	Now do the fractional part, appending zeroes if necessary.
	 */
	sz = strlen(src);
	strncpy(dst, src, width);
	width -= sz;
	dst += sz;
	while (width > 0)
		{
		*dst = '0';
		dst++;
		width--;
		}
	}
