/*
 *	Insert a GLIS header in the comment field of a JPEG file.
 *	Note that XGLIS depends on the comment marker immediately following
 *	the SOI marker.
 *	
 *	The header must be passed as argv[1], the input and ouput files
 *	can be optionally passed as second and third parameters, otherwise
 *	they default to stdin and stdout.
 */
#include <stdio.h>
char *Program;
main(int argc, char **argv)
	{
	FILE *input = stdin;
	FILE *output = stdout;
	char *hdr = argv[1];
	int c;			/* character buffer */
	int msb, lsb;		/* most and least significant portion of 16-bit values */
	int sz;			/* size of variable portion of the SOI marker */

	Program = argv[0];
	if (strlen(hdr) != 135)
		usage();
	if (argc > 2)
		{
		input = fopen(argv[2], "r");
		if (!input)
			{
			perror(argv[2]);
			exit(1);
			}
		}
	if (argc > 3)
		{
		output = fopen(argv[3], "w");
		if (!output)
			{
			perror(argv[3]);
			exit(1);
			}
		}
	expect(0xff, input, output);
	expect(0xd8, input, output);
	expect(0xff, input, output);
	expect(0xe0, input, output);
	msb = fgetc(input);
	fputc(msb, output);
	lsb = fgetc(input);
	fputc(lsb, output);
	sz = (msb & 0xff << 8) | (lsb & 0xff);
	copybytes(input, output, sz-2);
	/*
	 *	Output the header.
	 */
	fputc(0xff, output);
	fputc(0xfe, output);
	sz = strlen(hdr) + 2;
	msb = (sz >> 8) & 0xff;
	lsb = sz & 0xff;
	fputc(msb, output);
	fputc(lsb, output);
	fputs(hdr, output);
	/*
	 *	Copy the rest of the file.
	 */
	copybytes(input, output, -1);
	exit(0);
	}
copybytes(FILE *input, FILE *output, int sz)
	{
	int c;

	while ((c = fgetc(input)) != -1)
		{
		fputc(c, output);
		if ((sz > 0) && (--sz == 0))
			break;
		}
	}
expect(int expected, FILE *input, FILE *output)
	{
	int c;
	
	c = fgetc(input);
	if (c == expected)
		fputc(c, output);
	else
		{
		fprintf(stderr, "malformed JPEG file: expected 0x%x, got 0x%x\n", expected, c);
		exit(1);
		}
	}
usage()
	{
	fprintf(stderr, "usage: %s hdr [input] [output]\n", Program);
	fprintf(stderr, "hdr must be exactly 135 bytes\n");
	exit(1);
	}
