/**************************************************************************
  MKDOQBROWSE 
    pulls apart DOQ into seperate PGM files, then generates
    browse files with a .jpg extension.  Completed browse
    images are placed on the NOFS system in the
    /edcsns2/ofs/doq_qq/incoming directory for further processing.
***************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include "las.h"
#include "util.h"

extern char *glisbrhdr(char *targetfilename, char *jpegfilename);


main(argc, argv)
  int   argc;
  char *argv[];
{
  int   infd;
  FILE  *fpin;
  FILE  *pgm1; 
  FILE  *pgm2;
  FILE  *pgm3;

  int   bandtype;
long    nl = 0;			/* number of lines		*/
long 	ns = 0;			/* number of samples		*/
long	status =0;		/* return status check		*/
long    ii, jj;			/* loop counters		*/
long 	rec_size = 4000;	/* initial header record size	*/
char  	ch;			/* bip extract character	*/
char	temp[256];		/* temp hold string		*/
char	callstr[256];		/* system call string		*/
char	errstr[256];		/* error string			*/
long  	lcol;			/* number of samples per line	*/
char  	infile[256], ofname1[256],ofname2[256],ofname3[256];
char  	*char_buf;		/* character i/o buffer		*/
char    *p;                     /* header string hold           */
char	*glishdr;		/* GLIS browse header (goes in JPEG comment) */

char dir[256];   	/* Directory portion of the file specification        */
char ext[256];    	/* Extension portion of the file specification        */
char root[256];  	/* Root portion of the file specification             */

char *browsedir = "/sg1/doq/browse";	/* working directory for browse generation */
char *unitreedir = "/sg1/doq/unitree";	/* dir. to move the DOQ to after we are done */
char *nofsdir = "/edcsns2/ofs/doq_qq/incoming";	/* directory on NOFS where the browse file will reside */

if (0)
  {
  /* this is for testing only */
  getwd(temp);
  browsedir = malloc(strlen(temp) + 8);
  strcpy(browsedir, temp);
  strcat(browsedir, "/browse");
  unitreedir = malloc(strlen(temp) + 9);
  strcpy(unitreedir, temp);
  strcat(unitreedir, "/unitree");
  nofsdir = malloc(strlen(temp) + 8);
  strcpy(nofsdir, temp);
  strcat(nofsdir, "/nofs");
  printf("THIS IS THE TEST VERION\n");
  }


/* Change to the browse directory */
if (chdir(browsedir) < 0)
  {
  perror(browsedir);
  exit(1);
  }

/* Grab the input doq file name
-----------------------------------*/
sscanf(argv[1],"%s",infile);

/* break up the input file name to strip root
----------------------------------------------*/
c_splitname(infile,dir,root,ext);
printf("dir=%s \n root=%s \n ext=%s\n",dir,root,ext);
sprintf(errstr,"mv %s%s* %s/error",dir,root,browsedir);

printf("mkdoqbrowse: Infile=%s Outfile=%s.jpg\n",infile,root);

/* open input file
----------------------*/
status = (infd=open(infile,O_RDONLY)); 
if (status == -1)
  {
  close(infd);
  printf("ERROR: Cannot open %s for reading\n", infile);
  system(errstr);
  exit(0);
  }

/* Alloc header buffer
-----------------------*/
char_buf = (char *) malloc(rec_size);
if (char_buf == NULL)
  {
  close(infd);
  printf("Error allocating memory\n");
  system(errstr);
  exit(0);
  }

/* Read the first record
  ---------------------*/
status = read(infd, char_buf, rec_size);
if (status != rec_size)
   {
   printf("Error reading doq file");
   close(infd);
   system(errstr);
   exit(0);
   }

/* Extract the number of pixels per line
-----------------------------------------*/
if ((p=strstr(char_buf,"SAMPLES_AND_LINES")) != NULL)
  {
  sscanf(&p[17],"%d %d",&ns, &nl);
  }
else
  {
  close(infd);
  printf("Unable to locate SAMPLES_AND_LINES");
  system(errstr);
  exit(0);
  }

/* Extract band orginization
-----------------------------*/
if ((p=strstr(char_buf,"BAND_ORGANIZATION")) != NULL)
  {
  sscanf(&p[17],"%s",temp);
  if ((p=strstr (temp,"SINGLE")) != 0) bandtype = 1;
  else if (strcmp (temp,"BSQ") == 0) bandtype = 5;
  else if (strcmp (temp,"BIL") == 0) bandtype = 5;
  else if (strcmp (temp,"BIP") == 0) bandtype = 5;
  else
    {
    close(infd);
    printf("Unsupported band organization");
    system(errstr);
    exit(0);
    }
  }
else
  {
  close(infd);
  printf("Unable to locate band organization");
  system(errstr);
  exit(0);
  }
 
/* Rewind file and skip past header record
---------------------------------------------*/
if (bandtype == 5)
  {
  lcol = 3 * ns;
  }
else
  {
  lcol = ns;
  }

close(infd);
fpin = fopen(infile,"rb");
 
status = fseek(fpin,lcol,0);

printf("%s has %d lines and %d samples\n",infile,nl, ns);
printf("bandtype= %d   \n",
       bandtype);
  

  /* Handle single band B&W    
  -------------------------*/
  if (bandtype == 1) 
    {
    /* Open temporary working file
    -------------------------------*/
    sprintf(ofname1,"%s%s",dir,root);
    if ((pgm1 = fopen(ofname1, "wb")) == NULL)
      { printf("Can't open %s as pgm file \n", ofname1); exit(-1);}

   /* Add pgm header info to temp file
   ------------------------------------*/
   fprintf(pgm1,"P5\n%d %d\n%d\n", ns, nl, 255);

   /* Extract data from DOQ
   --------------------------*/
   for (ii=0;ii<nl;ii++) {
      for (jj=0;jj<ns;jj++) {
        ch = getc(fpin);
        putc(ch,pgm1);
      }
    }

   fclose(pgm1);

  /* convert file to jpg
  -----------------------*/
  sprintf(callstr,"%s %s%s %s %s/%s%s","sub 2 < ",dir,root,
          "| wave 2 |cjpeg > ",browsedir,root,".jpg");
  system(callstr);

  /* extract the glis header from the browse_info.lst file and add it
  --------------------------------------------------------------------*/
  strcpy(callstr, root);
  strcat(callstr, ".2");
  strcpy(temp, root);
  strcat(temp, ".jpg");
  glishdr = glisbrhdr(callstr, temp);
  sprintf(callstr, "inserthdr \"%s\" %s.jpg %s.jpg.hdr && mv %s.jpg.hdr %s.jpg",
    glishdr, root, root, root, root);
  system(callstr);

  sprintf(callstr,"%s %s/%s","/bin/rm",browsedir,root);
  system(callstr);
  sprintf(callstr,"mv %s/%s.2 %s/%s.3",browsedir,root,unitreedir,root);
  status = system(callstr);
  if (status != 0)
    {
    sprintf(callstr,"mv %s/%s.2 %s/error",browsedir,root,browsedir);
    system(callstr);
    }
  sprintf(callstr,"mv %s/%s.jpg %s",browsedir,root,nofsdir);
  status = system(callstr);
  if (status != 0)
    {
    sprintf(callstr,"mv %s/%s.jpg %s/error",browsedir,root,browsedir);
    system(callstr);
    }
  } 
  /*    Handle Color Band interleaved by pixel
  ---------------------------------------------*/
  if (bandtype == 5 ) 
    {
    /* Open temporary files for each band
    -------------------------------------*/
    sprintf(ofname1,"%s%s.r",dir,root);
    if ((pgm1 = fopen(ofname1, "wb")) == NULL)
      { 
      printf("Can't open %s as First Output pgm file \n", ofname1); 
      system(errstr);
      exit(-1);
      }

    sprintf(ofname2,"%s%s.g",dir,root);
    if ((pgm2 = fopen(ofname2, "wb")) == NULL)
      { 
      printf("Can't open %s as second Output pgm file \n", ofname2); 
      system(errstr);
      exit(-1);
      }

    sprintf(ofname3,"%s%s.b",dir,root);
    if ((pgm3 = fopen(ofname3, "wb")) == NULL)
      { 
      printf("Can't open %s as Third Output pgm file \n", ofname3); 
      system(errstr);
      exit(-1);
      }

    /* Add pgm header info to files
    --------------------------------*/
    fprintf(pgm1,"P5\n%d %d\n%d\n", ns, nl, 255);
    fprintf(pgm2,"P5\n%d %d\n%d\n", ns, nl, 255);
    fprintf(pgm3,"P5\n%d %d\n%d\n", ns, nl, 255);

    /* extract band data from doq file
    ----------------------------------*/
    for (ii=0;ii<nl;ii++) {
      for (jj=0;jj<ns;jj++) {
      /* Do Red record  */ 
        ch = getc(fpin);
        putc(ch,pgm1);
      /*  Do Green Record  */
        ch = getc(fpin);
        putc(ch,pgm2);
      /*  Do Blue record */
        ch = getc(fpin);
        putc(ch,pgm3);
      }
    }

   /* close temporary band files
   -----------------------------*/
   fclose(pgm1);
   fclose(pgm2);
   fclose(pgm3);

  /* subsample and convert files to one jpg
  ------------------------------------------*/
  sprintf(callstr,"%s %s %s %s%s%s","sub 2 < ",ofname1,
          "| wave 2 > ",dir,root,"1w4.r");
  system(callstr);
  sprintf(callstr,"%s %s %s %s%s%s","sub 2 < ",ofname2,
          "| wave 2 > ",dir,root,"1w4.g");
  system(callstr);
  sprintf(callstr,"%s %s %s %s%s%s","sub 2 < ",ofname3,
          "| wave 2 > ",dir,root,"1w4.b");
  system(callstr);
  sprintf(callstr,"%s %s%s%s %s%s%s %s%s%s > %s%s%s","rgb3toppm",dir,root,
          "1w4.r",dir,root,"1w4.g",dir,root,"1w4.b",dir,root,"1w4.ppm");
  /*sprintf(callstr,"%s %s%s%s %s%s%s %s%s%s %s%s%s","pgm2ppm",dir,root,
          "1w4.r",dir,root,"1w4.g",dir,root,"1w4.b",dir,root,"1w4.ppm"); */
  system(callstr);
  sprintf(callstr,"%s %s%s%s %s/%s%s","cjpeg < ",dir,root,
          "1w4.ppm >",browsedir,root,".jpg");
  system(callstr);

  /* extract the glis header from the browse_info.lst file and add it
  --------------------------------------------------------------------*/
  strcpy(callstr, root);
  strcat(callstr, ".2");
  strcpy(temp, root);
  strcat(temp, ".jpg");
  glishdr = glisbrhdr(callstr, temp);
  sprintf(callstr, "inserthdr \"%s\" %s.jpg %s.jpg.hdr && mv %s.jpg.hdr %s.jpg",
    glishdr, root, root, root, root);
  system(callstr);

  sprintf(callstr,"%s %s%s%s","/bin/rm",dir,root,"1w4*");
  system(callstr);
  sprintf(callstr,"%s %s%s%s","/bin/rm",dir,root,".r");
  system(callstr);
  sprintf(callstr,"%s %s%s%s","/bin/rm",dir,root,".g");
  system(callstr);
  sprintf(callstr,"%s %s%s%s","/bin/rm",dir,root,".b");
  system(callstr);
  sprintf(callstr,"mv %s%s.2 %s/%s.3",dir,root,unitreedir,root);
  system(callstr);
  sprintf(callstr,"mv %s/%s.jpg %s", browsedir,root,nofsdir);
  status = system(callstr); 
  if (status != 0)
    {
    system(errstr);
    }
  }
}
