/*******************************************************************************
NAME:                           FF123T

PURPOSE:      Read an fast format header file, create the metadata file, and
              create the tape label description and pass it back to TAE. 

PROGRAM HISTORY:
PROGRAMMER              DATE            REASON
----------              ----            ------
D. Etrheim              10/93           Initial development
K. Sheffield            01/94           Project and Description field changes
D. Etrheim              04/94           Changed project field for tape label 
                                        for CEES.  
D. Etrheim              05/94           Chagned tapedesc as specified. P6358

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "header.h"
#include "fhdr.h"

int main
(void)
{
struct PARBLK parblk;         /* TAE parameter block */
struct VARIABLE *val;         /* pointer to a TAE variable within
v-block */

char date[6];                /* julian date */
char hname[CMLEN];           /* Host name of input file */
char header[HEADER_SIZE];    /* buffer containing the header file */
char momx[5];                /* mode/mux value */
char newpr[10];              /* path/row without hrz offset */
char outstr[1000];           /* output string for metadata file */
char outfile[CMLEN];         /* Host name of output metadata file */
char ppprrr[10];             /* path/row value */
char proj[13];                /* project for which tapes are being copied */
char satno[3];               /* statillte number */
char sceneid[18];            /* Scene identification */
char tapedesc[50];           /* description for tape label */
char tapeloc[3][7];          /* new tape locations */
char tstr[80];               /* temp buffer for return values */
char tstr2[80];              /* temp buffer for return values */
char month[5],day[5],year[5];/* temp work variables */


char *ptr;                   /* string pointer */
char *str_ptr[1];            /* Pointer to a string for call to q_string */
char projopt[4];     /* Should I add projnum to project name? (yes, no) */

long i;              /* loop counter */
long ltemp;          /* temp work value */
long len;            /* string length */
long mode = IREAD;   /* file access */
long mn,dy,yr,jdate; /* temp work values */
long proj_num;       /* projection number */
long seq;            /* sequence number */
long std_out;        /* standard out */
long zero = 0;       /* value of zero */


double projp[15];    /* projection parameters */

FILE *fp;            /* metadata file pointer */

/* Initialization */
c_inlas(&parblk,&std_out);    

/* Get the input file name */
if (c_getnam(&parblk,"INFILE",&zero,hname,"fhdr",&mode) != E_SUCC)
    c_lasmsg(TRUE,"ff123-fatal","Fatal error encountered");

/* Get the output file name */
val = p_find(&parblk,"OUTFILE");
strcpy(outfile,SVAL(*val,0));

/* Get the projopt flag */
val = p_find(&parblk,"PROJOPT");
strcpy(projopt,SVAL(*val,0));

/* Get the sequence number */
val = p_find(&parblk,"seq");
seq = IVAL(*val,0);

/* Get the project */
val = p_find(&parblk,"PROJ");
strcpy(proj,SVAL(*val,0));

/* Open and read header file */
mode = IREAD;
if (c_open_header(header,hname,&mode) != E_SUCC)
    c_lasmsg(TRUE,"ff123-fatal","Fatal error encountered");

/* Only worry about version B for now */
c_fhdrrd(header,117,tstr);
if ((strncmp(tstr,"B",1) == 0) || (strncmp(tstr,"C",1) == 0))
    {
    /* Satellite number */
    c_fhdrrd(header,8,satno);
    sprintf(sceneid,"%s",&satno[1]);
    
    /* path/row */
    c_fhdrrd(header,4,ppprrr);
    strncat(sceneid,ppprrr,3);
    strncat(sceneid,&ppprrr[4],5);
    
    /* acquistion date */
    c_fhdrrd(header,6,tstr);
    strncat(sceneid,&tstr[2],2);
    strncpy(year,tstr,4);
    strncpy(month,&tstr[4],2);
    strncpy(day,&tstr[6],2);
    yr = atol(year);
    mn = atol(month);
    dy = atol(day);
    c_grejul(&yr,&mn,&dy,&jdate);
    if (jdate < 10)
        sprintf(tstr,"00%d",jdate);
    else if (jdate < 100)
        sprintf(tstr,"0%d",jdate);
    else
        sprintf(tstr,"%d",jdate);
    strncat(sceneid,tstr,3);
    strncpy(date,&sceneid[9],5);
    
    /* mode/mux */
    c_fhdrrd(header,10,momx);
    strncat(sceneid,&momx[2],2);
    
    }
else
    c_lasmsg(TRUE,"ff123-format","Unexpected format");

/* build the description for the tape label */
strncpy(newpr,ppprrr,7);
newpr[3] = ' ';
sprintf(tapedesc,"%s %s %02d/%02d/%02d",sceneid,newpr,mn, dy, (yr % 100));

/* Send the stuff back to the calling pdf */
str_ptr[0] = tapedesc;
if (q_string(&parblk,"DESC",1, str_ptr, P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"ff123-pass","Error passing variables to TAE");

if (q_out(&parblk) != SUCCESS)
    c_lasmsg(TRUE,"ff123-pass","Error passing variables to TAE");


/* We need to extract the projection info 
-----------------------------------------------------------*/
/* get the projection number */
c_fhdrrd(header,45,&proj_num);

/* update the project field */
if ((projopt[0]=='y') || (projopt[0]=='Y'))
    {
    sprintf(tstr,"P%02d ",proj_num);
    strcat(proj,tstr);
    }

/* Send the stuff back to the calling pdf */
str_ptr[0] = proj;
if (q_string(&parblk,"PROJ",1, str_ptr, P_UPDATE) != SUCCESS)
	c_lasmsg(TRUE,"ff123-pass","Error passing variables to TAE");

if (q_out(&parblk) != SUCCESS)
	c_lasmsg(TRUE,"ff123-pass","Error passing variables to TAE");

/* If this is the last tape, we need to write the meta data to a file */
if (seq == 3)
	{
	val = p_find(&parblk,"TAPELOC");
	for (i=0; i < 3; i++)
		strcpy(tapeloc[i],SVAL(*val,i));
	
	/* build the string to write to the file*/
	ppprrr[3] = ',';
	strncpy(outstr,ppprrr,7);
	sprintf(tstr,"%s",",");
	strncat(outstr,tstr,1);
	
	/* hrz offset */
	strncat(outstr,&ppprrr[7],2);
	strncat(outstr,tstr,1);
	
	/* date */
	strncat(outstr,date,5);
	strncat(outstr,tstr,1);
	
	/* satellite */
	strncat(outstr,&satno[1],1);
	strncat(outstr,tstr,1);
	
	/* mode/mux */
	strncat(outstr,&momx[2],2);
	strncat(outstr,tstr,1);
	
	/* geodetic process */
	c_fhdrrd(header,17,tstr2);
	strncat(outstr,tstr2,10);
	strncat(outstr,tstr,1);
	
	/* resampling */
	c_fhdrrd(header,19,tstr2);
	strncat(outstr,tstr2,2);
	strncat(outstr,tstr,1);
	
	/* projection name */
	c_fhdrrd(header,43,tstr2);
	
	/* Remove all white spaces */
	len = (long) strlen(tstr2);
	ptr = squeeze(tstr2,len);
	strncat(outstr,ptr,len);
	strncat(outstr,tstr,1);
	
	/* projection number */
	sprintf(tstr2,"%d",proj_num);
	strncat(outstr,tstr2,2);
	strncat(outstr,tstr,1);
	
	/* map zone */
	c_fhdrrd(header,47,&ltemp);
	sprintf(tstr2,"%d",ltemp);
	strncat(outstr,tstr2,4);
	strncat(outstr,tstr,1);
	
	/* projection parameters */
	c_fhdrrd(header,49,projp);
	for (i = 0; i < 15; i++)
		{
		sprintf(tstr2,"%1.14E",projp[i]);
		strncat(outstr,tstr2,24);
		strncat(outstr,tstr,1);
		}
	
	/* ellipsoid */
	c_fhdrrd(header,51,tstr2);
	/* Remove all white spaces */
	len = (long )strlen(tstr2);
	ptr = squeeze(tstr2,len);
	strncat(outstr,ptr,len);
	strncat(outstr,tstr,1);
	
	/* pixel size */
	strncat(outstr,&header[1063],5);
	strncat(outstr,tstr,1);
	
	/* pixel/line */
	c_fhdrrd(header,59,&ltemp);
	sprintf(tstr2,"%d",ltemp);
	strncat(outstr,tstr2,4);
	strncat(outstr,tstr,1);
	
	/* lines/image */
	c_fhdrrd(header,61,&ltemp);
	sprintf(tstr2,"%d",ltemp);
	strncat(outstr,tstr2,5);
	strncat(outstr,tstr,1);
	
	/* UL long */
	strncat(outstr,&header[1116],13);
	strncat(outstr,tstr,1);
	
	/* UL lat */
	strncat(outstr,&header[1130],12);
	strncat(outstr,tstr,1);
	
	/* UR long */
	strncat(outstr,&header[1174],13);
	strncat(outstr,tstr,1);
	
	/* UR lat */
	strncat(outstr,&header[1188],12);
	strncat(outstr,tstr,1);
	
	/* LR long */
	strncat(outstr,&header[1232],13);
	strncat(outstr,tstr,1);
	
	/* LR lat */
	strncat(outstr,&header[1246],12);
	strncat(outstr,tstr,1);
	
	/* LL long */
	strncat(outstr,&header[1290],13);
	strncat(outstr,tstr,1);
	
	/* LL lat */
	strncat(outstr,&header[1304],12);
	strncat(outstr,tstr,1);
	
	/* Sun elevation */
	c_fhdrrd(header,101,&ltemp);
	sprintf(tstr2,"%d",ltemp);
	strncat(outstr,tstr2,2);
	strncat(outstr,tstr,1);
	
	/* Sun azimuth */
	c_fhdrrd(header,103,&ltemp);
	sprintf(tstr2,"%d",ltemp);
	strncat(outstr,tstr2,3);
	strncat(outstr,tstr,1);
	
	/* Center pt long */
	strncat(outstr,&header[1453],13);
	strncat(outstr,tstr,1);
	
	/* Center pt lat */
	strncat(outstr,&header[1467],12);
	strncat(outstr,tstr,1);
	
	/* Tape locations */
	strncat(outstr,tapeloc[0],6);
	strncat(outstr,tstr,1);
	strncat(outstr,tapeloc[1],6);
	strncat(outstr,tstr,1);
	strncat(outstr,tapeloc[2],6);
	
	/* Write the metadata file */
	fp = fopen(outfile,"a");
	fprintf(fp,"%s\n",outstr);
	fclose(fp);
	}

/* Done */
z_exit(1," ");

return EXIT_SUCCESS;

}

