      SUBROUTINE INVRT(MATRIX,N,L,M,N1)
***********************************************************************
*
*     NAME- INVRT                         EXPIRES- 12/31/99
*
*     LANGUAGE- FORTRAN     TYPE- SUBROUTINE
*
*     VERSION- 1.0    DATE- 01-JUL-87 PROGRAMMER- John R. Allen (EDC)
*	       2.0	    15-MAY-92		  Tom Mittan    (EDC)
*
*     FUNCTION- 
*     This subroutine computes the inverse of a non-singular square
*     matrix.
*
************************************************************************
*                                                                      *
*     DECLARATION SECTION                                              *
*                                                                      *
************************************************************************
      DOUBLE PRECISION MATRIX(*)
     .,                L(*)
     .,                M(*)
     .,                BIGA
     .,                VV
     .,                HOLD
     .,                ZERO
      INTEGER          I
     .,                J
     .,                K
     .,                II
     .,                IJ
     .,                IK
     .,                JI
     .,                JK
     .,                KI
     .,                KJ
     .,                KK
     .,                N
     .,                N1
     .,                KSAVE
      PARAMETER       (ZERO = 0.D0)
************************************************************************
*                                                                      *
*     CODE SECTION                                                     *
*                                                                      *
************************************************************************
*
*     initiate the continued product of pivots which will become the
*     determinant of the matrix and start the main elimination loop
*
      DO 1000 K = 1,N1
*
*        search for the largest element
*
         L(K) = K
         M(K) = K
         KK = K + N * (K-1)
         BIGA = MATRIX(KK)
         DO 200 I = K,N1
            DO 100 J = K,N1
               IJ = I + N * (J-1)
               VV = DABS(BIGA) - DABS(MATRIX(IJ))
               IF (VV .LT. ZERO) THEN
                  BIGA = MATRIX(IJ)
                  L(K) = I
                  M(K) = J
               END IF
  100       CONTINUE
  200    CONTINUE
*
*        a zero largest element means the largest matrix in MATRIX is less
*        than N by N
*
         IF (BIGA .EQ. ZERO) GO TO 1100
*
*        interchange rows
*
         J = L(K)
         VV = L(K) - K
         IF (VV .GT. ZERO) THEN
            DO 300 I = 1,N1
               KI = K + N * (I-1)
               HOLD =-MATRIX(KI)
               JI = J + N * (I-1)
               MATRIX(KI) = MATRIX(JI)
               MATRIX(JI) = HOLD
  300       CONTINUE
         END IF
         I = M(K)
         VV = M(K) - K
         IF (VV .LE. ZERO) GO TO 500
         DO 400 J = 1,N1
            JK = J + N * (K-1)
            HOLD =-MATRIX(JK)
            JI = J + N * (I-1)
            MATRIX(JK) = MATRIX(JI)
            MATRIX(JI) = HOLD
  400    CONTINUE
*
*        divide column by minus pivot
*
  500    CONTINUE
         DO 600 I = 1,N1
            II = I - K
            IF (II .NE. 0) THEN
               IK = I + N * (K-1)
               MATRIX(IK) = MATRIX(IK) / (-MATRIX(KK))
            END IF
  600    CONTINUE
*
*        reduce matrix
*
         DO 800 I = 1,N1
            DO 700 J = 1,N1
               II = I - K
               IF (II .NE. 0) THEN
                  II = J - K
                  IF (II .NE. 0) THEN
                     IJ = I + N * (J-1)
                     IK = I + N * (K-1)
                     KJ = K + N * (J-1)
                     MATRIX(IJ) = MATRIX(IK) * MATRIX(KJ) + MATRIX(IJ)
                  END IF
               END IF
  700       CONTINUE
  800    CONTINUE
*
*        divide row by pivot
*
         DO 900 J = 1,N1
            II = J - K
            IF (II .NE. 0) THEN
               KJ = K + N * (J-1)
               MATRIX(KJ) = MATRIX(KJ) / MATRIX(KK)
            END IF
  900    CONTINUE
*
*        continued product of pivots
*
         MATRIX(KK) = 1.0D0 / MATRIX(KK)
 1000 CONTINUE
*
*     final row and column interchange
*
      K = N1
 1100 CONTINUE
      K = K - 1
      KSAVE = K
      IF (K .LE. 0) GO TO 1400
      I = L(K)
      II = I - K
      IF (II .GT. 0) THEN
         DO 1200 J = 1,N1
            JK = J + N * (K-1)
            HOLD = MATRIX(JK)
            JI = J + N * (I-1)
            MATRIX(JK) =-MATRIX(JI)
            MATRIX(JI) = HOLD
 1200    CONTINUE
      END IF
      J = M(K)
      II = J - K
      IF (II .LE. 0) GO TO 1100
      DO 1300 I = 1,N1
         KI = K + N * (I-1)
         HOLD = MATRIX(KI)
         JI = J + N * (I-1)
         MATRIX(KI) =-MATRIX(JI)
         MATRIX(JI) = HOLD
 1300 CONTINUE
      GO TO 1100
 1400 CONTINUE
      RETURN
      END
