      SUBROUTINE LINES(WINDOW,SCALE,TRNPAR,PRJPAR)
***********************************************************************
*
*     NAME- LINES                         EXPIRES- 12/31/99
*
*     LANGUAGE- FORTRAN     TYPE- SUBROUTINE
*
*     VERSION- 1.0    DATE- 01-JUL-87 PROGRAMMER- John R. Allen (EDC)
*	       2	    15-MAY-92		  Tom Mittan
*
*     FUNCTION- 
*     Plots flight lines and nadir circles.
*
************************************************************************
*                                                                      *
*     DECLARATION SECTION                                              *
*                                                                      *
************************************************************************
      DOUBLE PRECISION
     .          DEGCOR(2)
     .,         TRNPAR(8)
     .,         PRJPAR(15)
      REAL      PLTCOR(2)
     .,         RADIUS(2)
     .,         INC
     .,         INC2
     .,         HORLEN
     .,         WINDOW(4)
     .,         XOFF(2)
     .,         YOFF(2)
      INTEGER   X
     .,         Y
     .,         TXTINT
     .,         HORINC
     .,         VRTINC
     .,         STNUM
     .,         NCHAR
     .,         SCALE
     .,         INDEX
     .,         NPTS(2)
      CHARACTER TXTBUF*4
      PARAMETER(HORLEN = .125
     .,         INC    = .0625
     .,         INC2   = .03125)
      EQUIVALENCE(TXTINT,TXTBUF)
      DATA RADIUS/.13,.058/
     .,    NPTS  /36,15/
     .,    XOFF  /.2,.1/
     .,    YOFF  /.3,.2/
************************************************************************
*                                                                      *
*     CODE SECTION                                                     *
*                                                                      *
************************************************************************
*
*     select array index to use based on scale (100000 = 1, 250000 = 2)
*
      INDEX = INT(SCALE/100000)
*
*     determine number of horizontal and vertical steps to take
*
      HORINC = INT(WINDOW(4)/INC)
      VRTINC = INT(WINDOW(3)/INC)
*
*     plot vertical lines and tick marks
*
      DO 200 X = 0,HORINC
*
*        go from West to East stepping INC degrees
*
         DEGCOR(1) = WINDOW(2) + (X * INC)
         DO 100 Y = 0,VRTINC
*
*           go from South to North stepping INC degrees
*
            DEGCOR(2) = WINDOW(1) + (Y * INC)
*
*           determine plotter coordinates of point
*
            CALL DEGPLT(DEGCOR,SCALE,TRNPAR,PRJPAR,PLTCOR)
            IF (Y .NE. 0) THEN
*
*              not at South edge of flight line - draw from last point
*              to current point
*
               CALL PLOT(PLTCOR(1),PLTCOR(2),2)
            END IF
*
*           move pen to start of tick mark
*
            CALL PLOT(PLTCOR(1)+HORLEN,PLTCOR(2),3)
*
*           draw tick mark
*
            CALL PLOT(PLTCOR(1)-HORLEN,PLTCOR(2),2)
*
*           move pen back to line
*
            CALL PLOT(PLTCOR(1),PLTCOR(2),3)
  100    CONTINUE
  200 CONTINUE
*
*     determine number of vertical steps to take for circles
*
      VRTINC = INT(WINDOW(3)/INC2)
*
*     plot flight lines
*
      DO 400 X = 1,HORINC
*
*        go from West - INC2 to East - INC2 stepping INC degrees
*
         DEGCOR(1) = WINDOW(2) + (X * INC) - INC2
*
*        calculate station number of current point
*
         STNUM = INT((32 * WINDOW(1)) - 767)
         DO 300 Y = 0,VRTINC
*
*           go from South to North stepping INC2 degrees
*
            DEGCOR(2) = WINDOW(1) + (Y * INC2)
*
*           convert point to plotter coordinates
*
            CALL DEGPLT(DEGCOR,SCALE,TRNPAR,PRJPAR,PLTCOR)
            IF (Y .NE. 0) THEN
*
*              not at South edge of flight line - draw from last point
*              to current point - circle radius
*
               CALL PLOT(PLTCOR(1),PLTCOR(2) -
     .                   RADIUS(INDEX),2)
            END IF
*
*
*           write station numbers every other circle
*
            IF (MOD(Y,2) .EQ. 0) THEN
               IF (STNUM .GE. 100) THEN
                  WRITE(TXTBUF,'(I3)') STNUM
                  NCHAR = 3
               ELSE IF (STNUM .GE. 10) THEN
                  WRITE(TXTBUF,'(I2)') STNUM
                  NCHAR = 2
               ELSE
                  WRITE(TXTBUF,'(I1)') STNUM
                  NCHAR = 1
               END IF
               CALL SYMBOL(PLTCOR(1)+XOFF(INDEX),PLTCOR(2)-YOFF(INDEX),
     .                     .1,TXTBUF,0,0,NCHAR)
            END IF
*
*           plot the circle
*
            CALL CIRCL1(PLTCOR(1),PLTCOR(2),
     .                  RADIUS(INDEX),NPTS(INDEX))
*
*           increment the station number
*
            STNUM = STNUM + 1
  300    CONTINUE
  400 CONTINUE

      RETURN
      END
