/****************************************************************************  
* Name  GEN2ASC
*
* Purpose  Convert an Arc Generate-line 
* ingest format to an ASCII format compatable with Oracle
*
* Program History
*       Programmer         Date           Reason
*       ----------         ----           ------
*       K. Lowell          09/93          Original development
*
* Computer Hardware and/or software limitations
*       Must be run under TAE
*
* PROJECT      LAS
*
* LANGUAGE     C      
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"  
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK parblk;           /* TAE parameter block */

FILE  *file_ptr,                /* ptr to input file */
      *out_ptr;                 /* ptr to output file */ 

long  	count = 0,              /* counter variable  */
        fatal = 0,		/* fatal error constant */
        msgtim, 		/* time for processing message */
        one = 1,                /* constant of 1 */
        std_out;                /* standard output */

static char	infile[CMLEN],  /* input file */
                id[CMLEN],      /* ID of Arc */
                outfile[1][CMLEN], /* output file */
                line[CMLEN],    /* input file read buffer */
                str1[CMLEN],    /* temp read string */
                str2[CMLEN];    /* temp read string */

/* Initialize TAE parameter block
---------------------------------*/
c_inlas(&parblk,&std_out);

/*  Initialize LINE
-------------------*/
strcpy(line,"");

/* Get user-specified parameters
--------------------------------*/
if ((getpar(&parblk,infile,outfile[0],&msgtim))
   != E_SUCC)
   {
   c_errmsg("Fatal error encountered","gen2asc-fatal",&fatal);
   } 

/* Open input file
------------------*/
if ((file_ptr = fopen(infile,"r")) == NULL)
  {
  c_errmsg("Error opening input file","gen2asc-open",&fatal);
  }

/* Open output file
-------------------*/
if ((out_ptr = fopen(outfile[0],"w")) == NULL)
  {
  c_errmsg("Error opening output file","gen2asc-open",&fatal);
  }

/* Read the input file until End of File (EOF) is found
   In the output file the value of ID is written only
   once per arc, this value is stored in the parameter
   HOLD, and a comparison is done on each read line of
   input data.
-------------------------------------------------------*/
for (;;)
   {
   if (fgets(line,80,file_ptr) != NULL) 
     {
     sscanf(line,"%s %s", str1,str2);
     if(strcmp(str1,"END")) 
       {
       if((strcmp(str2,"")) == 0) 
         {
         strcpy(id,str1);
         count=0;
         }
       else 
         {
         fprintf(out_ptr,"%s %s %s\n", id, str1, str2);
         strcpy(str2,"");
         }
       }
     else 
       {
       count++;
       if (count >= 2) break;
       }
     } 
   else 
     {
     fclose(out_ptr);
     fclose(file_ptr);
     break;
     } 
  }

/* Output completion message
-----------------------------*/
c_complt("GEN2ASC",outfile,&one);

return EXIT_SUCCESS;

} 
