
/******************************************************************************
NAME:		GETINFO_AVH

PURPOSE:	Get the information from the AVHRR header file.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  1.0	 T. Baltzer    01/96	Original development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "glcts.h"
#include "protos_avh.h"


int getinfo_avh
(
    char headfile[TOTLSV][TOTSAM],	/* Total saved lines in header	*/
    long nheadf,				/* Number of entries in headfile*/
    long start,				/* Starting scene number 	*/
    long setnum,				/* AVHRR set number 		*/
    int  *nscenes,			/* Number of scenes in sceneid	*/
    char sceneid[MAXSC][17]		/* Scene ID's 			*/
)

{

long i, j;			/* Counter */
long len;			/* String length */
long startfile;			/* File number of first scene to get */
long found;			/* Found first file flag */
long fnum;			/* File number as found in the header */
long fsize;			/* File size as found in the header */
long totalsize;			/* Cumulative file size */
long num_prelim;		/* Number of preliminary files */

char tmpscid[17];		/* Temporary scene id */


/*
Initialize the number of scenes to zero.
*/
*nscenes = 0;


/*
  Determine the size of the preliminary files.  The size of the preliminary
  files is considered only if START is 1 (inscinuates that this is the first
  output tape for this set).
  For AVHRR periods 1-12 (SETNUM = 1) the preliminary files are:
    The header, ancillary, gif, and dem.
  For AVHRR periods 13-24 (SETNUM = 2) and periods 25-36 (SETNUM = 3):
    The only preliminary file is the header file.
*/
totalsize = 0;
if (start == 1)
    {
    if (setnum == 1)
	num_prelim = 4;
    else
	num_prelim = 1;

    for (i = 0; i < num_prelim; i++)
	{
	if (getsize(headfile[i],&fsize) != E_SUCC)
	    return(E_FAIL);
	totalsize += fsize;
	}
    }


/*
  Determine the file number of the scene requested by START.
  For AVHRR periods 1-12 (SETNUM = 1):
    Add 4 to START since the header, ancillary, gif, dem preceed the scenes.
  For AVHRR periods 13-24 (SETNUM = 2) and periods 25-36 (SETNUM = 3):
    Add 1 to START since the header preceeds the scene files.
*/
if (setnum == 1)
    startfile = start + 4;
else
    startfile = start + 1;


/*
  Find the line of the first file of interest
*/
found = FALSE;
for (i = 0; i < nheadf; i++)
    {
    sscanf(headfile[i],"%*s %d: %s %d %*s",&fnum,tmpscid,&fsize);
    if (fnum == startfile)
	{
	found = TRUE;
	break;
	}
    }

/*
  Did not find requested scene in header
*/
if (!found)
    return(E_SUCC);


/*
  Found the first file, get the info on it and all other subsequent files
  until the maximum amount to be held on one output tape has been exceeded,
  or until we include all of the subsequent available files.
*/
for (j = startfile; i < nheadf; i++, j++)
    {
    sscanf(headfile[i],"%*s %d: %s %d %*s",&fnum,tmpscid,&fsize);
    if (fnum != j)
	{
	c_lasmsg(1,"glcts_avh-seq","File not in expected sequence in header");
	return(E_FAIL);
	}
    
    totalsize += fsize;
    if (totalsize <= MAX_TAPE_SIZE)
	{
	len = (long) strlen(tmpscid);
	c_up2low(tmpscid,&len);
	strcpy(sceneid[*nscenes],tmpscid);
	(*nscenes)++;
	}
    else
	break;
    }

return(E_SUCC);
}

