/*******************************************************************************
NAME:                           GLCTS_AVH

PURPOSE:      Read an AVHRR header file and return appropriate information.
	      (NOT a ".ahdr" AVHRR header type of file).

PROGRAM HISTORY:
PROGRAMMER              DATE            REASON
----------              ----            ------
T. Baltzer		01/96		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "glcts.h"
#include "protos_avh.h"


int main
(void)
{
   struct PARBLK parblk;		/* TAE parameter block */

   char hname[CMLEN];			/* Header file name */
   char headfile[TOTLSV][TOTSAM];	/* Buffer containing the header file */
   char sceneid[MAXSC][17];		/* Scene ID's */

   long std_out;			/* Standard out */
   long status;				/* Return status */
   int  nscenes;			/* Number of scenes in header file */
   long start;				/* Starting scene number */
   long nheadf;				/* Number of entries in headfile */
   long setnum;				/* AVHRR set number 		*/
					/* SETNUM=1, AVHRR period  1-12	*/
					/* SETNUM=2, AVHRR period 13-24	*/
					/* SETNUM=3, AVHRR period 25-36	*/


/* Initialize LAS */
c_inlas(&parblk,&std_out);     


/* Get the user input parameters */
getpar_avh(&parblk,hname,&start,&setnum);


/* Load the header file info into memory */
status = load_avh(hname,headfile,&nheadf);
if (status != E_SUCC)
    c_lasmsg(1,"glcts_avh-fatal","Fatal error encountered");


/* Get the information from the header file */
getinfo_avh(headfile,nheadf,start,setnum,&nscenes,sceneid);
if (status != E_SUCC)
    c_lasmsg(1,"glcts_avh-fatal","Fatal error encountered");


/* Return the information to TAE */
status = taeland_avh(&parblk,&nscenes,sceneid);
if (status != E_SUCC)
    c_lasmsg(1,"glcts_avh-fatal","Fatal error encountered");


z_exit(1," ");

return EXIT_SUCCESS;

}
