/*******************************************************************************
NAME:                           GLCTS_VNV

PURPOSE:      Read a header file and mapper listing to determine if
	      an output tape contains the correct files.

PROGRAM HISTORY:
PROGRAMMER              DATE            REASON
----------              ----            ------
T. Baltzer		02/96		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "glcts.h"
#include "protos_vnv.h"

int main
(void)
{
   struct PARBLK parblk;		/* TAE parameter block */

   char hname[CMLEN];			/* Header file name */
   char mhfile[CMLEN];			/* Mapper host file name */
   char headfile[TOTLSV][TOTSAM];	/* Buffer containing the header file */
   char mapfile[TOTLMP][TOTSAM];	/* Buffer containing the mapper file */
   char tapeid[7];			/* Tape ID */

   long filesiz[MAXSC];			/* File sizes from header */
   long mfilesize[MAXSC];		/* File sizes from mapper */
   long std_out;			/* Standard out */
   long status;				/* Return status */
   long nfiles;				/* Number of files in header file */
   long nmfiles;			/* Number of files in mapper file */
   long start;				/* Starting scene number */
   long maxfiles;			/* Maximum number of files to get */
   long nheadf;				/* Number of entries in headfile */
   long nmapf;				/* Number of entries in mapfile */


/* Initialize LAS */
c_inlas(&parblk,&std_out);     


/* Get the user input parameters */
getpar_vnv(&parblk,hname,&start,&maxfiles,tapeid,mhfile);


/* Load the header file info into memory */
status = load_avh(hname,headfile,&nheadf);
if (status != E_SUCC)
    c_lasmsg(1,"glcts_vnv-fatal","Fatal error encountered");


/* Get the information from the header file */
status = getinfo_vnv(headfile,nheadf,start,maxfiles,&nfiles,filesiz);
if (status != E_SUCC)
    c_lasmsg(1,"glcts_vnv-fatal","Fatal error encountered");


/* Load the mapper file info into memory */
status = load_map(mhfile,mapfile,&nmapf);
if (status != E_SUCC)
    c_lasmsg(1,"glcts_vnv-fatal","Fatal error encountered");


/* Extract the info from the mapper file */
status = getinfo_map(mapfile,nmapf,&nmfiles,mfilesize);
if (status != E_SUCC)
    c_lasmsg(1,"glcts_vnv-fatal","Fatal error encountered");


/* Compare the header file and mapper file */
status = compare_em(nfiles,filesiz,nmfiles,mfilesize,tapeid);
if (status != E_SUCC)
    c_lasmsg(1,"glcts_vnv-fatal","Fatal error encountered");


z_exit(1," ");

return EXIT_SUCCESS;

}
