/*******************************************************************************
NAME:                           INPEHDR

PURPOSE:      Read a Fast Format header file and create a new header with
              a corrected:
		1) "bands available" field
		2) "sequential tape number" field
		3) "number of tapes in set" field

PROGRAM HISTORY:
PROGRAMMER              DATE            REASON
----------              ----            ------
D. Etrheim	        10/93	       Initial development for ff123
T. Baltzer	        02/95	       Modified for INPE project

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS

ALGORITHM:

The INPE tapes on 8 mm tape consist of bands 1,2,3,4,5,7.  The 3480 archive
tapes to be generated will contain bands 1,2,3 (tape 1 of 2), and bands 4,5,7
(tape 2 of 2).

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "header.h"
#include "fhdr.h"

int main
(void)
{
struct PARBLK parblk;          	/* TAE parameter block */
struct VARIABLE *val;           /* pointer to a TAE variable within v-block  */

char hname[CMLEN];      	/* Host name of input file */
char header[HEADER_SIZE];      	/* buffer containing the header file */
char oname[CMLEN];      	/* Host name of output file */
char tstr[80];          	/* temp buffer for return values */

long fatal = -1;	/* fatal error code */
long mode = IREAD;	/* file access */
long seq;		/* output sequence number */
long std_out;		/* standard out */
long zero = 0;		/* value of zero */

/* Initialization */
c_inlas(&parblk,&std_out);     

/* Get the input file name */
if (c_getnam(&parblk,"INFILE",&zero,hname,"fhdr",&mode) != E_SUCC)
   c_errmsg("Fatal error encountered","inpehdr-fatal",&fatal);

/* Get the output file name */
mode = IWRITE;
if (c_getnam(&parblk,"OUTFILE",&zero,oname,"fhdr",&mode) != E_SUCC)
   c_errmsg("Fatal error encountered","inpehdr-fatal",&fatal);

/* Get the sequence number */
val = p_find(&parblk,"seq");
seq = IVAL(*val,0);

/* Open and read header file */
mode = IREAD;
if (c_open_header(header,hname,&mode) != E_SUCC)
   c_errmsg("Fatal error encountered","inpehdr-fatal",&fatal);

/* Comment out the check for version B, since INPE tapes         */
/* will be version A and "A" does not appear in version A headers*/
c_fhdrrd(header,117,tstr);

/*if (strncmp(tstr,"B",1) != 0)			      */
/*c_errmsg("Unexpected format","inpehdr-format",&fatal);*/
   
/* We need to create a new header file for the output tape that
   contains the actual bands on it */
if (seq == 1)
   {
   strcpy(tstr,"123");
   c_fhdrup(header,95,tstr);
   header[1464] = '2';		/* number of tapes */
   header[1473] = '1';		/* seq tape number */
   }
else
   {
   strcpy(tstr,"457");
   c_fhdrup(header,95,tstr);
   header[1464] = '2';		/* number of tapes */
   header[1473] = '2';		/* seq tape number */
   }

/* Open and write header file */
mode = IWRITE;
if (c_open_header(header,oname,&mode) != E_SUCC)
   c_errmsg("Fatal error encountered","inpehdr-fatal",&fatal);

/* Done */
z_exit(1," ");

return EXIT_SUCCESS;

}
