/******************************************************************************
FUNCTION:       GETSCENEID 

PURPOSE:        Read the header file copied by DITTO and extract a scene ID
		to use on the output tape label

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      6/96   C. Engebretson   Original development

ALGORITHM REFERENCES:

	"Format Specifications for Canadian Landsat MSS System
	 Computer Compatible Tape", Jennifer Murphy, Research
	 Report 79-2, August 1979

*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "jsc.h"
#include "las.h"
#include "util.h"
#include "header.h"
#include "tae.h"

int main
(void)
{
struct PARBLK parblk;           /* TAE parameter block                  */

char header[CMLEN];		/* Name of Landsat Header file		*/
char tempstr[255]; 		/* Array to hold extracted data		*/
char sceneid[255];		/* Scene ID built by function		*/
char *ptr;			/* Temporary character pointer		*/
char *str_ptr[1];		/* Temporary pointer for q_string()	*/

long mode = IREAD;		/* File access mode			*/
long std_out;			/* Standard output			*/
long zero = 0;			/* Constant of zero			*/

struct JSCLH lhdr;		/* JSC Landsat Header structure		*/


/* Initialize LAS and retrieve input parameters from TAE
--------------------------------------------------------*/
c_inlas(&parblk, &std_out);
if (c_getnam(&parblk, "HEADER", &zero, header, "lhdr", &mode) != E_SUCC)
        c_lasmsg(TRUE, "jsccopy-fatal", "Fatal error encountered");

/* Call the header library routines and initialize metadata module
------------------------------------------------------------------*/
c_jsc_lhdr(header, &lhdr);

/* Build the scene ID with the satellite number, the path/row, and the
   acquisition date.  Get rid of all whitespace, and pad path/row
   numbers with leading zeroes if necessary.
----------------------------------------------------------------------*/
sceneid[0] = 0;
strcpy(tempstr, lhdr.mission_id);
for (ptr = tempstr; *ptr == 32; ptr++);
strcat(sceneid, ptr);

strcpy(tempstr, lhdr.path_number);
for (ptr = tempstr; *ptr == 32; ptr++);
while (strlen(ptr) < 3) *(--ptr) = '0';
strcat(sceneid, ptr);

strcpy(tempstr, lhdr.row_number);
for (ptr = tempstr; *ptr == 32; ptr++);
while (strlen(ptr) < 3) *(--ptr) = '0';
strcat(sceneid, ptr);

strcpy(tempstr, lhdr.data_acquisition_ddmmyy);
for (ptr = tempstr; *ptr == 32; ptr++);
strcat(sceneid, ptr);

str_ptr[0] = sceneid;
if (q_string(&parblk, "SCENEID", 1, str_ptr, P_UPDATE) != SUCCESS)
        c_lasmsg(TRUE, "jsccopy-pass", "Error passing variables to TAE");
if (q_out(&parblk) != SUCCESS)
        c_lasmsg(TRUE, "jsccopy-pass", "Error passing variables to TAE");

return EXIT_SUCCESS;

}
