C     *************************************************************************
C     NAME 			CATFILE       	    		
C
C     PURPOSE	 Concatenates up to ten IBIS tabular files to produce
C	         a new IBIS tabular file.  The number of columns in 
C	         each input file must be equal.
C	
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	JPL(H. Wilczynski)  		        Original
C	K. Zanter	  Apr. l987		LAMS conversion  
C	B. Ailts	  Oct. 1988		LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	CATFILE must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  The input and output
C     IBIS tabular files are opened.  Corresponding columns from
C     the input tabular files are transferred to the same column
C     of the output tabular file in such a manner that the first
C     entry from each tabular file directly follows the last entry
C     from the previous tabular file.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program catfile

	include 'las.fin'

	include 'lams.fin'

	real*4 		col(MMXROW)

	integer*4	access
	integer*4	getcnt
	integer*4	getnam
	integer*4	i
	integer*4	icol
	integer		infdesc(100)
	integer*4	nfiles
	integer*4	ncol(100)
	integer*4	nrow(100)
	integer		outfdesc
	integer*4	outnc
	integer*4	outnr

      	integer*4	parblk(XPRDIM)

	integer*4	ptr
	integer*4	status
	integer*4	stdout
	integer*4	tbopen
	integer*4	tbrcol
	integer*4	tbwcol
	integer*4	tbclos

	character*(CMLEN)	infile(100)
	character*(CMLEN)	ofile
	
	character*(ERRLEN)	msg

C	Place the large variables in a common block to work around a
C	recent change in the sgi compiler that causes core dumps when
C	large variables are declared on the stack
	common parblk


1	FORMAT('Error opening input file ',I2)
2       FORMAT('Input file ',I2,' has ',I2,' columns')
3       FORMAT('The first input file has ',I2,' columns')
4       FORMAT('Output file would have ',I6,' number of rows')
5       FORMAT('Error reading input file ',I2)

C	initialization
C	--------------
	call inlas(parblk,stdout)

C	get the output IBIS file name
C	-----------------------------
	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_tab',1)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','catfile-fatal',-1)
	endif

C	get the input IBIS file names
C	-----------------------------
	status = getcnt(parblk,'INFILE',nfiles)
	do 10 i = 1,nfiles
	   status = getnam(parblk,'INFILE',i,infile(i),'ibis_tab',IREAD)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','catfile-fatal',-1)
	   endif
10	continue

C	open the first input file
C	-------------------------
	access = 0
	status = tbopen(infdesc(1),infile(1),nrow(1),ncol(1),access)
	if (status .ne. ESUCC) then
	  write (msg,1) 1
	  call errmsg(msg,'catfile-open',1)
	  call errmsg('Fatal error encountered','catfile-fatal',-1)
	endif
	
	outnc = ncol(1)
	outnr = nrow(1)

C	open remaining input files
C	--------------------------
	do 100 i = 2,nfiles
	   status = tbopen(infdesc(i),infile(i),nrow(i),ncol(i),access)
	   if (status .ne. ESUCC) then
	      write (msg,1) i
	      call errmsg(msg,'catfile-open',1)
	      call errmsg('Fatal error encountered','catfile-fatal',-1)
	   endif

	   if (ncol(i) .ne. outnc) then
	      call errmsg
     +        ('The number of columns in each input file are not equal',
     +         'catfile-ncols',1)
	      write (msg,2) i,ncol(i)
	      call errmsg(msg,'catfile-ncols',1)
	      write (msg,3) outnc
	      call errmsg(msg,'catfile-ncols',-1)
	   endif

	   outnr = outnr + nrow(i)
100	continue

C	validatation
C	------------
	if (outnr .gt. MMXROW) then
	   write (msg,4) outnr
	   call errmsg(msg,'catfile-rows',1)
	   call errmsg('This exceeds the limit of MMXROW rows',
     +		       'catfile-rows',1)
	   call errmsg('Fatal error encountered','catfile-fatal',-1)
	endif

C	open the output IBIS file
C	-------------------------
	access = 1
	status = tbopen(outfdesc,ofile,outnr,outnc,access)
	if (status .ne. ESUCC) then
	   status = tbclos(outfdesc,ofile)
	   call errmsg('Error opening output file','catfile-open',1)
	   call clean(parblk,'catfile',ofile,1)
	endif

	do 150 icol = 1,outnc
C	   get data for current column in input files
C	   ------------------------------------------
	   ptr = 1
	   do 120 i = 1,nfiles
	      status = tbrcol(infdesc(i),icol,col(ptr))
	      if (status .ne. ESUCC) then
	          status = tbclos(outfdesc,ofile)
		  write (msg,5) i
		  call errmsg(msg,'catfile-read',1)
	          call clean(parblk,'catfile',ofile,1)
	      endif

	      ptr = ptr + nrow(i)
120	   continue

C	   output new column
C	   -----------------
	   status = tbwcol(outfdesc,icol,col)
	   if (status .ne. ESUCC) then
	      status = tbclos(outfdesc,ofile)
	      call errmsg('Error writing output file','catfile-write',1)
	      call clean(parblk,'catfile',ofile,1)
	   endif

150	continue

C	close the input IBIS files
C	--------------------------
	do 500 i = 1,nfiles
	   status = tbclos(infdesc(i),infile(i))
	   if (status .ne. ESUCC) then
	      call errmsg('Error closing input file','catfile-close',1)
	   endif
500	continue

C	close the output IBIS file
C	--------------------------
	status = tbclos(outfdesc,ofile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','catfile-close',1)
	endif

C	output completion message
C	-------------------------
	call complt('CATFILE',ofile,1)

	end
