C     *************************************************************************
C     NAME			COPYFILE        	    		
C
C     PURPOSE      Copies gouldected columns of the input IBIS tabular file
C	           into gouldected columns of the output IBIS tabular file.
C		   The rest of the columns of the output file are set to 
C		   zero.	 			
C
C     PROGRAM HISTORY
C	PROGRAMMER	  DATE		REASON
C	----------	  ----		------
C	JPL (B.Gokhman)   		Original
C	K. Zanter        Dec. 1986      LAMS conversion		 
C	B. Ailts	 Nov. 1988	LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C       COPYFILE must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins by getting the user-specified parameters.
C     If necessary, the defaulted parameters are set with appropriate
C     values and validation checks are made.  The input and output files
C     are opened, and the columns of the input file are copied into the
C     columns of the output file.  The rest of the columns of the output 
C     file are set to zero.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program copyfile

	include 'las.fin'

	include 'lams.fin'

	real*4		buf(MMXROW)

	integer*4	access
	integer*4	count
	integer*4	clen
	integer		fdesc1
	integer		fdesc2
	integer*4	fncol
	integer*4	getnam
	integer*4	i
	integer*4	incol(MMXCOL)
	integer*4	ncol
	integer*4	nincol
	integer*4	noutcl
	integer*4	outcol(MMXCOL)

      	integer*4	parblk(XPRDIM)

	integer*4	status
	integer*4	stdout
	integer*4	tbopen
	integer*4	tbrcol
	integer*4	tbwcol
	integer*4	tbclos

	character*(CMLEN)	infile
	character*(ERRLEN)	msg
	character*(CMLEN)	ofile

C	initialization
C	--------------
	call inlas(parblk,stdout)

C	get the input IBIS file name
C	----------------------------
	status = getnam(parblk,'INFILE',1,infile,'ibis_tab',IREAD)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','copyfil-fatal',-1)
	endif

C	open the input IBIS file
C	------------------------
	access = 0
	status = tbopen(fdesc1,infile,clen,fncol,access)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','copyfil-open',1)
	   call errmsg('Fatal error encountered','copyfil-fatal',-1)
	endif

C	get the number of columns in the output file
C	--------------------------------------------
	call xrintg(parblk,'NCOLS',1,ncol,count,status)
	if (count .eq. 0) ncol = fncol

C	get the input and output column numbers
C	---------------------------------------
	call xrintg(parblk,'INCOL',MMXCOL,incol,nincol,status)
	call xrintg(parblk,'OUTCOL',MMXCOL,outcol,noutcl,status)

C	if both INCOL and OUTCOL were NOT defaulted, verify 
C       that the number of rows and columns are the same
C       ---------------------------------------------------
10	FORMAT ('The number of values specified for INCOL and OUTCOL',
     +	        ' are not equal')

	if ((nincol .ne. 0) .and. (noutcl .ne. 0)) then	
	   if (nincol .ne. noutcl) then
	      write (msg,10)
	      call errmsg(msg,'copyfil-unequal',1)
	      call errmsg('Fatal error encountered','copyfil-fatal',-1)
	   endif
	endif

C	------------------------------------------------------------------
C	|if (INCOL was defaulted) then					 |
C	|   set the number of input columns equal to the max in the input|
C	|   fill the input column array					 |
C	|endif								 |
C	------------------------------------------------------------------
	if (nincol .eq. 0) then
	   nincol = fncol
	   do 2 i = 1,nincol
	      incol(i) = i
2	   continue
	endif

C	------------------------------------------------------------------
C	|if (OUTCOL was defaulted) then					 |
C	|   set the number of output columns equal to NCOLS              |
C	|   fill the output column array				 |
C	|endif								 |
C	------------------------------------------------------------------
	if (noutcl .eq. 0) then
	   noutcl = ncol
	   do 5 i = 1,noutcl
	      outcol(i) = i
5	   continue
	endif

C	verify that all the specified columns are valid
C	-----------------------------------------------
15	FORMAT('There is no column ', I5, ' in the IBIS file')
16      FORMAT('The total number of columns is ',I5)

	do 20 i = 1,nincol
	   if (incol(i) .gt. fncol) then
	      write (msg,15) incol(i)
	      call errmsg(msg,'copyfil-column',1)
	      write (msg,16) fncol
	      call errmsg(msg,'copyfil-column',1)
	      call errmsg('Fatal error encountered','copyfil-fatal',-1)
	   endif
	   if (outcol(i) .gt. fncol) then
	      ncol = outcol(i)
	   endif
20	continue

C	get the output IBIS file name
C	-----------------------------
	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_tab',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','copyfil-fatal',-1)
	endif

C	open the output IBIS file
C	-------------------------
	access = 1
	status = tbopen(fdesc2,ofile,clen,ncol,access)
	if (status .ne. ESUCC) then
	   status = tbclos(fdesc2,ofile)
	   call errmsg('Error opening output file','copyfil-open',1)
	   call clean(parblk,'copyfile',ofile,1)
	endif

C	initialize the buffer with zeroes
C	---------------------------------
	do 30 i = 1,clen
	   buf(i) = 0.
30	continue

C	copy the input columns to output columns 
C	----------------------------------------
	do 40 i = 1,nincol
	   status = tbrcol(fdesc1,incol(i),buf)
	   if (status .ne. ESUCC) then
	      status = tbclos(fdesc2,ofile)
	      call errmsg('Error reading input file','copyfil-read',1)
	      call clean(parblk,'copyfile',ofile,1)
	   endif

	   status = tbwcol(fdesc2,outcol(i),buf)
	   if (status .ne. ESUCC) then
	     status = tbclos(fdesc2,ofile)
	     call errmsg('Error writing output file','copyfil-write',1)
	     call clean(parblk,'copyfile',ofile,1)
	   endif
40	continue

C	close the input and output IBIS files
C	-------------------------------------
	status = tbclos(fdesc1,infile)
	if (status .ne. ESUCC) then
	  call errmsg('Error closing input file','copyfil-close',1)
	endif
	status = tbclos(fdesc2,ofile)
	if (status .ne. ESUCC) then
	  call errmsg('Error closing output file','copyfil-close',1)
	endif

C	output completion message
C       -------------------------
	call complt('COPYFILE',ofile,1)

	end
