C     ******************************************************************
C     NAME 			CUTADD       	    		
C
C     PURPOSE	   Adds in points along the cutline.  The coordinates
C		   for the cutline are either stored in an IBIS 
C		   graphics file or input by the user in the LSCOORS
C		   parameter.
C	
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	Joy Peterson      April l984		Original
C	K. Zanter	  March l987		LAMS conversion  
C	B.Ailts		  Nov. 1988		LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	CUTADD must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  The points of the
C     input polygon are read in from either the IBIS graphics file
C     or the LSCOORS parameter.  Points are generated according to
C     the specified step size (PNTINC) to be added in to the 
C     cutline.  The resulting points are written to the output IBIS
C     graphics file.
C
C     ALGORITHM REFERENCES 		none
C     ******************************************************************
	program cutadd

	include 'las.fin'

	include 'lams.fin'

	double precision  deltal
	double precision  deltas
	double precision  slope(MMXPPR)

	real*4		buff(MEMXPT)
	real*4		datarr(100)
	real*4		grbuf(MMXPT)
	real*4		rdeltl
	real*4		rdelts
	real*4		xvert(MMXPPR)
	real*4		yvert(MMXPPR)

	integer*4	access
	integer*4	count
	integer*4	dcount
	integer*4	getnam
	integer*4	grclos
	integer*4	gropen
	integer*4	grwrit
	integer*4	grread
	integer*4	i
	integer		fdesc1
	integer		fdesc3
	integer*4	il
	integer*4	innl
	integer*4	inns
	integer*4	isgn
	integer*4	istep
	integer*4	ivert
	integer*4	j
	integer*4	jsgn
	integer*4	length
	integer*4	listou(10)
	integer*4	npairs
	integer*4	npoly
	integer*4	nrec
	integer*4	outnl
	integer*4	outns

      	integer*4	parblk(XPRDIM)

	integer*4	reclen
	integer*4	rstep
	integer*4	status
	integer*4	stdout
	
	logical		data
	logical		newpol
	logical		gfile

	character*(CMLEN)	infile
	character*(CMLEN)	ofile
	character*(CMLEN)	subcmd

      	common/buffer/grbuf

C	initialization
C	--------------
	call inlas(parblk,stdout)
        listou(1) = 1
        npoly = 0
        ivert = 0
        newpol = .true.

C	initialize output buffer
C	-----------------------
	do 10 i = 1,MEMXPT
	   buff(i) = 0.0
10	continue

C	determine which subcommand was specified
C	----------------------------------------
	call xrstr(parblk,'_SUBCMD',1,subcmd,length,count,status)

C	get the source of the input coordinates
C	---------------------------------------
	if (subcmd .eq. 'FILE') then
	   status = getnam(parblk,'INFILE',1,infile,'ibis_graph',0)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','cutadd-fatal',-1)
	   endif
	   gfile = .true.
	   data = .false.
	else 
	   call xrreal(parblk,'LSCOORS',100,datarr,dcount,status)
	   data = .true.
	   gfile = .false.
	endif

C	get the output IBIS file name
C	-----------------------------
	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_graph',1)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','cutadd-fatal',-1)
	endif

C	get the PNTINC parameter
C	--------------------------
        call xrintg(parblk,'PNTINC',1,rstep,count,status)

C	read in and process the polygon points
C	(Note that the points are expeced to be 
C	 listed in y,x or line,sample order )
C       --------------------------------------
	if (gfile) then

C	   open the IBIS graphics file
C	   ---------------------------
	   access = 0
           status = gropen(fdesc3,infile,reclen,nrec,access)
	   if (status .ne. ESUCC) then
	      call errmsg('Error opening input file','cutadd-open',1)
	      call errmsg('Fatal error encountered','cutadd-fatal',-1)
	   endif

C	   read the points from the graphics file
C	   --------------------------------------
           innl = nrec
           inns = reclen
           npairs = inns/2
           do 20 il = 1,innl
              status = grread(fdesc3,il,grbuf)
	      if (status .ne. ESUCC) then
	         call errmsg('Error reading input file','cutadd-read',1)
	         call errmsg('Fatal error encountered',
     +			     'cutadd-fatal',-1)
	      endif

              call polyin(npairs,newpol,npoly,ivert,xvert,yvert,
     +			  listou)
20	   continue

C	   close the graphics file
C	   -----------------------
	   status = grclos(fdesc3,infile)
	   if (status .ne. ESUCC) then
	      call errmsg('Error closing input file','cutadd-close',1)
	   endif

	else

C	   use the points from the input COORD parameter
C	   ---------------------------------------------
           do 40 i = 1,dcount
              grbuf(i) = datarr(i)
40	   continue

           call polyin(dcount/2,newpol,npoly,ivert,xvert,yvert,listou)

	endif

C       generate set of points along contour
C       ------------------------------------
        j = 1
        do 1119 i = 1,ivert-1
	   if (xvert(i+1) .eq. xvert(i)) then
	      deltas = 0.
	      if (yvert(i+1) .gt. yvert(i)) then
		 jsgn = 1
	      else
		 jsgn = -1
	      endif

	      deltal = rstep * jsgn
	   else
              slope(i) = (yvert(i+1)-yvert(i))/(xvert(i+1)-xvert(i))

              if (xvert(i+1) .gt. xvert(i)) then
                 isgn = 1
              else 
                 isgn = -1
	      endif

              if (yvert(i+1) .gt. yvert(i)) then
                 jsgn = 1
              else
                 jsgn = -1
	      endif

              deltas = isgn * SQRT(rstep**2/(1+slope(i)**2))
              deltal = jsgn * SQRT(rstep**2 - deltas**2)
	   endif

	   rdelts = SNGL(deltas)
	   rdeltl = SNGL(deltal)

           buff(j) = yvert(i)
           buff(j+1) = xvert(i)
           j = j + 2

           do 1119 istep = 1, INT(SQRT((yvert(i+1)-yvert(i))**2 +
     +               (xvert(i+1)-xvert(i))**2)/rstep)
              buff(j) = buff(j-2) + rdeltl
              buff(j+1) = buff(j-1) + rdelts
              j = j + 2
 1119	continue

C	process the last segment
C	------------------------
	if (xvert(1) .eq. xvert(i)) then 
	   deltas = 0

	   if (yvert(1) .gt. yvert(i)) then
	      jsgn = 1
	   else
	      jsgn = -1
	   endif

	   deltal = rstep * jsgn
	else

       	   slope(i) = (yvert(1)-yvert(i))/(xvert(1)-xvert(i))

           if (xvert(1) .gt. xvert(i)) then
              isgn = 1
           else 
              isgn = -1
	   endif

           if (yvert(1) .gt. yvert(i)) then
              jsgn = 1
           else
              jsgn = -1
	   endif

           deltas = isgn * SQRT(rstep**2/(1+slope(i)**2))
           deltal = jsgn * SQRT(rstep**2 - deltas**2)
	endif

	rdelts = SNGL(deltas)
	rdeltl = SNGL(deltal)

        buff(j) = yvert(i)
        buff(j+1) = xvert(i)
        j = j + 2

        do 1118 istep = 1, INT(SQRT((yvert(1)-yvert(i))**2 +
     .               (xvert(1)-xvert(i))**2)/rstep)
           buff(j) = buff(j-2) + rdeltl
           buff(j+1) = buff(j-1) + rdelts
           j = j + 2
 1118   continue

C	put in closing point
C	--------------------
	buff(j) = yvert(1)
	buff(j+1) = xvert(1)
	j = j + 1

C	validation
C	----------
	if (j .gt. MEMXPT) then
	   call errmsg('Excess number of points generated',
     +		       'cutadd-limit',1)
	   call errmsg('Fatal error encountered','cutadd-fatal',-1)
	endif

C	set the size for the output graphics file
C	-----------------------------------------
	if (j  .le. MMXPPR) then
	   outnl = 1
	   outns = j
	else
	   outnl = (j + MMXPPR - 1)/MMXPPR
	   outns = MMXPPR
	endif

C	open the output IBIS graphics file
C	----------------------------------
	access = 1
	status = gropen(fdesc1,ofile,outns,outnl,access)
	if (status .ne. ESUCC) then
	   status = grclos(fdesc1,ofile)
	   call errmsg('Error opening output file','cutadd-open',1)
	   call clean(parblk,'cutadd',ofile,1)
	endif

C	write to the output graphics file
C	---------------------------------
	i = 1
	do 2000 il = 1,outnl
	   status = grwrit(fdesc1,il,buff(i))
	   if (status .ne. ESUCC) then
	      status = grclos(fdesc1,ofile)
	      call errmsg('Error writing output file','cutadd-write',1)
	      call clean(parblk,'cutadd',ofile,1)
	   endif

	   i = i + MMXPPR
2000	continue

C	close the output IBIS file
C	--------------------------
	status = grclos(fdesc1,ofile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','cutadd-close',1)
	endif

C	output completion message
C	-------------------------
	call complt('CUTADD',ofile,1)

      	end

C       ****************************************************************
C	POLYIN puts the points of the input polygon into the approrpriate
C       x and y arrays and determines the number of segments.
C       ****************************************************************
        subroutine polyin(npairs,newpol,npoly,ivert,xvert,yvert,
     +			  listou)

	include 'las.fin'

	include 'lams.fin'

	real*4		grbuf(MMXPT)
	real*4		x
	real*4		xvert(1)
	real*4		y
	real*4		yvert(1)

	integer*4	ipair
	integer*4	ivert
	integer*4	listou(10)
	integer*4	npairs
	integer*4	npoly

        logical 	newpol

        common/buffer/grbuf

        do 10 ipair = 1,npairs
           x = grbuf(2*ipair)
           y = grbuf(2*ipair-1)
           if ((x.ne.0. .or. y.ne.0.).and.(x.ne.-99. .or. y.ne.-99.)
     +        .and. (x.ne.xvert(listou(npoly+1)) .or. y.ne.
     +        yvert(listou(npoly+1))))  go to 12
           if (newpol) go to 10
           npoly = npoly + 1
           listou(npoly+1) = ivert+1
           newpol = .true.
           go to 10
   12      continue
           ivert = ivert + 1
           newpol = .false.
           xvert(ivert) = x
           yvert(ivert) = y
   10   continue

        return
        end
