C*****************************************************************************
C 	NAME: CUTCHG
C
C	FUNCTION:
C	CUTCHG reads the cutline file and outputs the 
C	the coordinates to the screen.  These coordinates can then be
C	added to, deleted, or modified.   
C
C	PROGRAM HISTORY:
C	Version		Date	Author		Code/conr	Request
C	-------		----	------		---------	-------
C	   1.0		4/84	J.PETERSON	   CSB		Initial devel.
C	   1.0		7/86	B.AILTS		   CSB		LAMS conversion
C								to 1 and
C								PRF 3435
C
C	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C			Run under TAE 
C
C	PROJECT:	LAMS
C
C	ALGORITHM DESCRIPTION:
C	The tae parmaters are read in.  These parameters consist of:
C	1.  INFILE--the input cutline file
C	2.  OUTFILE--the output cutline file
C
C	The cutline coordinates are read in and printed to the screen.
C	the user is prompted for a command.  The valid commands are:
C		A:  Add a point                              
C		D:  Delete a point                           
C		H:  Display help                             
C		M:  Modify a point                           
C		P:  Print out the points to the terminal     
C		Q:  End session  
C		W:  Write points to OUTFILE
C
C	After the user is finished modifying the cutline, all the values are 
C	written to the output file, the completion message is displayed.
C
C	ALGORITHM REFERENCES:	none
C
C******************************************************************************

	program cutchg

	include 'las.fin'

	include 'lams.fin'

	real*4 grbuf(MEMXPT)
	real*4 xvert(MEMXPT/2)
	real*4 yvert(MEMXPT/2)

	integer*4 access
	integer*4 columns
	integer*4 count
	integer*4 getnam
	integer*4 grclos
	integer*4 gropen
	integer*4 grread
	integer*4 grwrit
	integer*4 i
      	integer   fdesc2
	integer   fdesc3
	integer*4 il
	integer*4 ipair
	integer*4 ivert
	integer*4 length
	integer*4 lines
	integer*4 listin(10)
	integer*4 listou(10)
	integer*4 nlgrap
	integer*4 npairs
	integer*4 npoly
	integer*4 nrec
	integer*4 nsgrap

      	integer*4	parb(XPRDIM)

	integer*4 reclen
      	integer*4 status
	integer*4 start
	integer*4 stdout
	integer*4 term
	integer*4 type
	integer*4 up2low

      	character*(CMLEN) hosin
	character*(CMLEN) hosout
	character*2 iopt

      	LOGICAL NEWPOL

C---- INITIALIZE, GET PARAMETERS, OPEN FILES
C-------------------------------------------
      	call inlas(parb,stdout)

	call xtinit(type,lines,columns)
	if (type .eq. XNOCRT) then
	   call errmsg('Terminal is not a support CRT','cutchg-term',1)
	   call errmsg('Fatal error encountered','cutchg-fatal',-1)
	else if (type .eq. XNOTRM) then
	   call errmsg('This program cannot be run in batch mode',
     .			'cutchg-batch',1)
	   call errmsg('Fatal error encountered','cutchg-fatal',-1)
	endif

	if (lines .lt. 23) then
	   call errmsg('This terminal does not have enough lines',
     .			'cutchg-lines',1)
	   call errmsg('Fatal error encountered','cutchg-fatal',-1)
	endif
	  
       	status = getnam(parb,'INFILE',1,hosin,'ibis_graph',IREAD)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','cutchg-fatal',-1)
	end if

       	status = getnam(parb,'OUTFILE',1,hosout,'ibis_graph',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','cutchg-fatal',-1)
	end if

C---- READ IN AND PROCESS THE POLYGON VERTICES.
C----------------------------------------------
      	LISTOU(1) = 1
      	NPOLY = 0
      	IVERT = 0
      	NEWPOL = .TRUE.

C---- READ THE GRAPHICS FILE.
C----------------------------
      	status = gropen(fdesc3,hosin,reclen,nrec,access)
	if (status .ne. ESUCC) then
	   call errmsg ('Error opening input file','cutchg-open',1)
	   call errmsg('Fatal error encountered','cutchg-fatal',-1)
	end if

      	NLGRAP = nrec
      	NPAIRS = reclen
      	DO 20 IL=1,NLGRAP
           status = grread(fdesc3,il,grbuf)
	   if (status .ne. ESUCC) then
	     call errmsg ('Error reading input file','cutchg-read',1)
	     call errmsg('Fatal error encountered','cutchg-fatal',-1)
	   end if
           CALL POLYIN(NPAIRS,NEWPOL,IVERT,listin,listou,xvert,
     .		       yvert,npoly,grbuf)
   20 	CONTINUE

C--- PROMPT THE OPERATOR FOR THE type OF MODIFICATION
C----------------------------------------------------
	start = 1
	call prtpt(ivert,start,xvert,yvert,lines)
50	continue

C  Clear the previous answer from the screen.
C--------------------------------------------
	call xtout(lines - 1,40,'                                  ')
      	call xtout(lines - 1,2,
     .			'Enter command:                             ')
      	call xtin(lines - 1,18,IOPT,length,term)

C  Branch to correct subroutine according to the command entered
C---------------------------------------------------------------
	status = up2low(iopt,1)
	if (iopt .eq. 'a') then
	   call addpt(ivert,xvert,yvert,lines)
	else if (iopt .eq. 'd') then
           call delpt(ivert,xvert,yvert,lines)
	else if (iopt .eq. 'h') then
	   call command(lines)
      	else if (iopt .eq. 'm') then
           call modpt(ivert,xvert,yvert,lines)
	else if (iopt .eq. 'p') then
	   call prtpt(ivert,start,xvert,yvert,lines)
	else if (iopt .eq. 'w') then
	   go to 100
	else if (iopt .eq. 'q') then
	   go to 200
	else
	   call xtpos (lines - 1,2)
	   call errmsg('Command entered is not a valid command       ',
     .	   	       'cutchg-command',1)
	   call command(lines)
      	end if
	go to 50

C---- open output file.
C----------------------
100 	continue
	if ((ivert * 2) .gt. MMXPPR) then
	   nsgrap = MMXPPR
	   nlgrap = ((ivert * 2 + MMXPPR - 1) / MMXPPR)
	else
	   nlgrap = 1
	   nsgrap = ivert * 2
	end if
	access = 1
      	status = gropen(fdesc2,hosout,nsgrap,nlgrap,access)
	if (status .ne. ESUCC) then
	   status = grclos(fdesc2,hosout)
	   call errmsg ('Error opening input file','cutchg-open',1)
	   call clean(parb,'cutchg',hosout,1)
	end if

C-----rewrite file.
C------------------
	call xtpos (lines,1)
      	do 33 il=1,nlgrap

C  Zero out the output array
C---------------------------
	   do 203 i = 1,MMXPPR
	     grbuf(2*i)=0.0
	     grbuf(2*i-1)=0.0
 203	   continue

	   count = 1
	   do 202 ipair = ((MMXPPR / 2 * (il - 1)) + 1), 
     .			  min((ivert * 2),(il * MMXPPR / 2)) 
	     grbuf(count + 1)=xvert(ipair)
	     grbuf(count)=yvert(ipair)
	     count = count + 2
 202       continue

           status = grwrit(fdesc2,il,grbuf)
	   if (status .ne. ESUCC) then
	     status = grclos(fdesc2,hosout)
	     call errmsg ('Error writing to output file',
     .			  'cutchg-write',1)
	     call clean(parb,'cutchg',hosout,1)
	   end if

  33  	continue
  30  	continue

C  Close input and output file
C-----------------------------
	status = grclos(fdesc2,hosout)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','cutchg-close',1)
	end if

200	continue
	status = grclos(fdesc3,hosin)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','cutchg-close',1)
	end if

	call xtpos (lines - 1,1)
C	If output file was written, send tae name of output file to complt
C	------------------------------------------------------------------
	if (iopt .eq. 'w') then
	   call complt('CUTCHG',hosout,1)
	else
	   call complt('CUTCHG',hosout,0)
	endif

	end

C *************************************************************************
C
C	Subroutine POLYIN reads in the polygon coordinates
C
C**************************************************************************
      SUBROUTINE POLYIN(NPAIRS,NEWPOL,IVERT,listin,listou,xvert
     .			,yvert,npoly,grbuf)

	include 'las.fin'

	include 'lams.fin'

	real*4 grbuf(1)
	real*4 x
	real*4 xvert(1)
	real*4 y
	real*4 yvert(1)

	integer*4 ipair
	integer*4 ivert
	integer*4 listin(10)
	integer*4 listou(10)
	integer*4 npairs
	integer*4 npoly

      	LOGICAL NEWPOL

C  Place the input coordinates within the correct arrays
C-------------------------------------------------------
      	DO 10 IPAIR=1,NPAIRS
           X = GRBUF(2*IPAIR)
           Y = GRBUF(2*IPAIR-1)
           IF((X.NE.0..OR.Y.NE.0.).AND.(X.NE.-99..OR.Y.NE.-99.))GOTO 12
           IF(NEWPOL) GO TO 10
           NPOLY = NPOLY+1
           LISTIN(NPOLY) = IVERT
           LISTOU(NPOLY+1) = IVERT+1
           NEWPOL = .TRUE.
           GO TO 10
   12      CONTINUE
           IVERT = IVERT+1
           NEWPOL = .FALSE.
           XVERT(IVERT) = X
           YVERT(IVERT) = Y
   10 	CONTINUE

      	RETURN
      	END

C ***********************************************************************
C
C	Subroutine DELPT deletes a point
C
C************************************************************************
      SUBROUTINE delpt(ivert,xvert,yvert,lines)

	include 'las.fin'

	real*4 xvert(1)
	real*4 yvert(1)
	
	integer*4 count
	integer*4 ivert
	integer*4 i
	integer*4 j
	integer*4 k
	integer*4 length
	integer*4 lines
	integer*4 num
	integer*4 temp(450)
	integer*4 term

	character*10 point
	character*3 ans

C  Get the point number of the point to be deleted
C-------------------------------------------------
	count = 1
 300  CONTINUE
	call xtout(lines - 1,40,'                                  ')
      	call xtout (lines - 1,2,
     .		    'Enter the number of the point to be deleted.')
	call xtin (lines - 1,47,point,length,term)
	read(point(1:length),'(I6)') num 

C  Keep the point numbers to be deleted in ascending order
C---------------------------------------------------------
	do 50 i = 1,count
	   if (num .lt. temp(i)) then
	      do 75 k = count + 1,i,-1
		 temp(k) = temp(k - 1)
75	      continue
	      temp(i) = num
	      go to 60
	   else if ( i .eq. count) then
	      temp(count) = num
	   end if
50	continue
60	continue
	if (temp(count) .gt. ivert) then
	   call xtpos (lines,1)
	   call errmsg('Point could not be found','cutchg-point',1)
	   go to 300
	end if

C  Clear the previous answer from the screen.
C--------------------------------------------
	call xtout(lines - 1,40,'                                    ')

        call xtout (lines - 1,2,
     .		    'Do you wish to delete another point? (yes,no)')
      	call xtin (lines - 1,47,ans,length,term)
      	if ((ans(1:1) .eq. 'Y') .or. (ans(1:1) .eq. 'y')) then
	   count = count + 1
	   GO TO 300
	end if

C  Delete all points by moving all other points above each point down one place
C  in the array
C------------------------------------------------------------------------------
	temp(count + 1) = ivert 
	do 200 i = 1, count
	   k = i - 1
   	   DO 101 J=(temp(i) - k),(temp(i + 1) - i)
              XVERT(J)=XVERT(J+i)
              YVERT(J)=YVERT(J+i)
  101 	   continue
200	continue

      	IVERT=IVERT-count

      	RETURN
      	END

C*****************************************************************************
C
C	Subroutine MODPT modifies an existing point
C
C*****************************************************************************
      SUBROUTINE MODPT(ivert,xvert,yvert,lines)

	include 'las.fin'

	real*4 line
	real*4 sample
	real*4 xvert(1)
	real*4 yvert(1)

	integer*4 ivert
	integer*4 length
	integer*4 lines
	integer*4 num
	integer*4 term

	character*10 charl
	character*10 chars
	character*10 point
	character*3 ans

   50 CONTINUE

C  Clear the previous answer from the screen.
C--------------------------------------------
	call xtout(lines - 1,40,'                                   ')

C  Get to number of the point to be modified
C-------------------------------------------
      	call xtout (lines - 1,2,
     .		    'Enter the number of the point to modified.')
	call xtin(lines - 1,45,point,length,term)
	read (point(1:length),'(I5)') num
	if (num .gt. ivert) then
	   call xtpos (lines,1)
	   call errmsg('Point could not be found','cutchg-point',1)
	   go to 50
	end if

C  Get the new line,sample location of the point to be modified.
C---------------------------------------------------------------
C  Clear the previous answer from the screen.
C--------------------------------------------
	call xtout(lines - 1,40,'                                    ')
      	call xtout (lines - 1,2,
     .		    'Enter the new line location of the point.   ')
	call xtin (lines - 1,44,charl,length,term)
	read (charl(1:length),'(F9.2)') line

C  Clear the previous answer from the screen.
C--------------------------------------------
	call xtout(lines - 1,40,'                                    ')
      	call xtout (lines - 1,2,
     .		    'Enter the new sample location of the point. ')
	call xtin (lines - 1,46,chars,length,term)
	read (chars(1:length),'(F9.2)') sample

        xVERT(num) = sample
	yVERT(num) = line

 150  continue

C  Clear the previous answer from the screen.
C--------------------------------------------
	call xtout(lines - 1,40,'                                    ')

        call xtout (lines - 1,2,
     .		    'Do you wish to modify another point? (yes,no)')
      	call xtin (lines - 1,48,ans,length,term)
      	IF ((ans(1:1) .EQ. 'Y') .or. (ans(1:1) .eq. 'y')) GO TO 50

      	RETURN
      	end

C*************************************************************************
C
C	Subroutine ADDPT adds a point to the polygon
C
C*************************************************************************
	subroutine addpt(ivert,xvert,yvert,lines)

	include 'las.fin'

	real*4 line
	real*4 samp
	real*4 tempx(450)
	real*4 tempy(450)
	real*4 xvert(1)
	real*4 yvert(1)

	integer*4 cnt
	integer*4 count
	integer*4 ivert
	integer*4 i
	integer*4 j
	integer*4 k
	integer*4 length
	integer*4 lines
	integer*4 num
	integer*4 temp(450)
	integer*4 term

	character*10 newline
	character*10 newsamp
	character*10 point
	character*3 ans

C  Initialize of variables
C-------------------------
	count = 1
	do 25 i= 1,450
	   temp(i) = 0
25	continue

C  Get the number of the point before the point to be added
C----------------------------------------------------------
300	continue
	call xtout(lines - 1,2,'Enter the number of the point which ')
	call xtout(lines - 1,38,
     .		   'precedes the point to be added.             ')
	call xtin(lines - 1,70,point,length,term)
	read(point(1:length),'(I6)') num
	if (num .gt. ivert) then
	   call xtpos (lines,1)
	   call errmsg('Point could not be found','cutchg-point',1)
	   go to 300
	end if

C  Get the line,sample location of the new point.
C------------------------------------------------
	call xtout(lines - 1,2,'Enter the line location of the point')
	call xtout(lines - 1,38,
     .		   ' to be added.                                ')
	call xtin(lines - 1,52,newline,length,term)
	read (newline(1:length),'(F9.2)') line

	call xtout(lines - 1,2,'Enter the sample location of the point')
	call xtout(lines - 1,40,
     .		   ' to be added.                                ')
	call xtin(lines - 1,54,newsamp,length,term)
	read (newsamp(1:length),'(F9.2)') samp

C  Keep the point numbers to be added in ascending order
C-------------------------------------------------------
	do 50 i = 1,count
	   if (num .lt. temp(i)) then
	      do 75 k = count + 1,i,-1
		 temp(k) = temp(k - 1)
		 tempx(k) = tempx(k - 1)
		 tempy(k) = tempy(k - 1)
75	      continue
	      temp(i) = num
	      tempx(i) = samp
	      tempy(i) = line
	      go to 60
	   else if ( i .eq. count) then
	      temp(count) = num
	      tempx(count) = samp
	      tempy(count) = line
	   end if
50	continue
60	continue

C  Clear the previous answer from the screen.
C--------------------------------------------
	call xtout(lines - 1,40,'                                    ')

      	call xtout (lines - 1,2,
     .		    'Do you wish to add another point? (yes,no)  ')
      	call xtin (lines - 1,45,ans,length,term)
      	if ((ans(1:1) .eq. 'Y') .or. (ans(1:1) .eq. 'y')) then
	   count = count + 1
	   GO TO 300
	end if

C  Move the points higher than the added point up one position in the arrays
C---------------------------------------------------------------------------
	do 200 i = 1, count
	   cnt = ivert
	   k = i - 1
   	   DO 101 J=(temp(i) + k),ivert
	   xvert(cnt + 1) = xvert(cnt)
	   yvert(cnt + 1) = yvert(cnt)
	   cnt = cnt - 1
101	continue

C  Add the new line,sample location to the array
C-----------------------------------------------
	yvert(temp(i) + i) = tempy(i)
	xvert(temp(i) + i) = tempx(i)
	ivert = ivert + 1
200	continue

	return
	end

C***********************************************************************
C
C	Subroutine COMMAND prints the valid commands on the screen
C
C***********************************************************************
	subroutine command(lines)

	include 'las.fin'

	integer*4 lines
C  Print the commands to the screen
C----------------------------------
	call xtout(lines - 10,2,
     .		   'The valid commands are:                      ')
	call xtout(lines - 9,5,
     .		   'A:  Add a point                              ')
	call xtout(lines - 8,5,
     .		   'D:  Delete a point                           ')
	call xtout(lines - 7,5,
     .		   'H:  Display help                             ')
	call xtout(lines - 6,5,
     .		   'M:  Modify a point                           ')
	call xtout(lines - 5,5,
     .		   'P:  Print out the points to the terminal     ')
	call xtout(lines - 4,5,
     .		   'Q:  End session                              ')
	call xtout(lines - 3,5,
     .		   'W:  Write points to OUTFILE and end session  ')

	return 
	end

C****************************************************************************
C
C	Subroutine PRTPT prints the point coordinates on the screen
C
C*****************************************************************************
	subroutine prtpt(ivert,start,xvert,yvert,lines)

	include 'las.fin'

	real*4 xvert(1)
	real*4 yvert(1)

	integer*4 finish
	integer*4 i
	integer*4 ivert
	integer*4 lines
	integer*4 outline
	integer*4 start

	character*80 string
	character*10 line
	character*10 point
	character*10 sample

	outline=1

	call xtclr
	if (start .gt. ivert) start = 1

	finish = min(ivert,start + lines - 4)
	string = 'Point number'//'  Line location'//'  Sample location'
	call xtout(outline,4,string)

C Print twenty points to the screen
C----------------------------------
	do 100 i = start,finish
	   outline = outline + 1
	   write (line,'(F8.2)') yvert(i)
	   write (sample,'(F8.2)') xvert(i)
	   write (point,'(I6)') i
	   string = point//'      '//line//'      '//sample
	   call xtout(outline,4,string)
100	continue

	start = start + lines - 3

	return
	end
