/*****************************************************************************
NAME				 DSPRGRD 

PURPOSE    Reads a radiometric mapping grid file and displays it on the
           user's terminal, the line printer, or to a file


PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   S. Reichardt   11/82        CSC      Original Development
  4.0      R. Mckinney    12/86        EDC      Misc fixes
  5.0      D. Steinwand   10/88        EDC      LAS 5.0 development,
						C conversion
  5.1      D. Steinwand   10/88        EDC      Conversion to display a radio-
						metric grid

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   DSPRGRD must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION

	Initialize LAS processing
	Get the radiometric mapping grid file name
	Get the output device parameter
	Read the radiometric mapping grid
	Write the radiometric mapping grid to the approiate output device
	Print completion message

ALGORITHM REFERENCES		none
*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "radpak.h"

main()
{
struct RADGRID grid;		/* radiometric mapping grid */
struct PARBLK  vblk;		/* TAE variable block */
FILE *filep;			/* File pointer to output file */
long  std_out;			/* Standard output LUN for inlas */
long  mode = IREAD;		/* IREAD flag for getnam routine */
long  fatal = -1;		/* Fatal error flag for c_errmsg */
long  zero = 0;			/* Constant of zero */
long  num_out;			/* Number of output files 0 or 1 */
long  term, file, ln_pr;	/* Print option flags */
char  fname[CMLEN];		/* Host name of radiometric mapping grid file */
char  temp_file[CMLEN];		/* Temporary file name for output */
char  filename[CMLEN];		/* Partial file name (wo/ext) for complt */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the name of the radiometric mapping grid file
  -----------------------------------------------*/
if(c_getnam(&vblk, "INGRID", &zero, fname, "grid", &mode) != E_SUCC)
  c_errmsg("Fatal error encountered","dsprgrd-fatal",&fatal);

/* Get the print option.  Make a file name (if neccessary) and open the
   temporary file
  --------------*/
strcpy(temp_file, "dsprgrd");
if (c_setprt(&vblk, &term, &file, &ln_pr, temp_file, filename) != E_SUCC)
   c_errmsg("Fatal error encountered","dsprgrd-fatal",&fatal);
if (file)
   num_out = 1;
else
   num_out= 0;

filep = fopen(temp_file, "w");

/* Get the geometric mapping grid
  ------------------------------*/
if(c_getrgd(fname, &grid) != E_SUCC)
  c_errmsg("Fatal error encountered","dsprgrd-fatal",&fatal);

/* Display geometric mapping grid values 
  -------------------------------------*/
if(show_grid(fname, filep, &grid) != E_SUCC)
   {
   fclose(filep);
   c_clean(&vblk,"dspgrid",temp_file,&num_out);
   }

/* Write completion message and exit program
  -----------------------------------------*/
fclose(filep);
c_complt("DSPGRID", filename, &num_out);
}
