/****************************************************************************
NAME:				cal_corners

PURPOSE:  Update projection corners to correspond to output mapping grid

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 2/89	B. Ailts      CSB      LAS 5.0 development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Calculate the number of lines and samples in the framed space.
	If the window is null, use the framed space as the window
	Else,
	   Place the user entered window in a double buffer
	   Calcluate frame space line,sample coordinates of corners
	   Call c_rotcor to update corners for user entered window
	Return

ALGORITHM REFERENCES:
	Taken from the subroutine find_corners of POLYFIT
*****************************************************************************/
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "finite.h"

int cal_corners
(
    int  *window,			/* window parameters */
    double *corners,		/* corner values		*/
    struct GEOGRID *geogrid	/* geometric grid data */
)
{
double dwind[4];	/* Input window; double precision */
double a,b;		/* Temporary variables for nlines,nsamps calculation */
long ls[8];		/* Line, sample coordinates in frame space */
long count;		/* Loop counter */
long nlines;		/* Number of lines in input frame space */
long nsamps;		/* Number of samples in input frame space */
long pass = 1;		/* Pass flag for nonfatal error message */

if (geogrid->proj_valid == GEO_NOTVALID)
   {
   for (count = 0; count < 8; count++)
      geogrid->corners[count] = corners[count];
   return(E_SUCC);
   }
else
   {
   a = (corners[0] - corners[2]) / geogrid->pdist[0];
   b = (corners[1] - corners[3]) / geogrid->pdist[1];
   nlines = (long)(sqrt((a * a) + (b * b)) + 1.5);
   a = (corners[0] - corners[4]) / geogrid->pdist[0];
   b = (corners[5] - corners[1]) / geogrid->pdist[1];
   nsamps = (long)(sqrt((a * a) + (b * b)) + 1.5);
   }

if ((window[2] == 0) && (window[3] == 0))
   {
   window[0] = window[1] = 1;
   window[2] = nlines;
   window[3] = nsamps;
   }
else
   {
   for (count = 0; count < 4; count++)
      dwind[count] = (double)window[count];
   ls[0] = 1;
   ls[1] = 1;
   ls[2] = nlines;
   ls[3] = 1;
   ls[4] = 1;
   ls[5] = nsamps;
   ls[6] = nlines;
   ls[7] = nsamps;
   if (c_rotcor(corners, ls, dwind, geogrid->corners) != E_SUCC)
      {
      c_errmsg("Nonfatal error encountered","finite-warn",&pass);
      for (count = 0; count < 8; count++)
         geogrid->corners[count] = 0.0;
      }
   }
return(E_SUCC);
}
