#ifndef FINITE_H
#define FINITE_H

#include "defs.h"
#include "geompak.h"

#define SZSUBCMD 16     /* Size of subcommand character string              */


int cal_corners
(
    int  *window,			/* window parameters */
    double *corners,		/* corner values		*/
    struct GEOGRID *geogrid	/* geometric grid data */
);

long dgefa
(
    double *a,
    long m,
    double *pivi,
    long *ipiv,
    double eps
);

long dgeso
(
    double *r,
    double *a,
    long m,
    long n,
    double *pivi,
    long *ipiv
);

void ELinitialize 
(
    struct Site *bottomsite_in,
    double xmin_in,
    double xmax_in,
    int   sqrt_nsites
);

struct Halfedge *HEcreate
(
    struct Edge *e,
    int pm
);

void ELinsert
(
    struct Halfedge *lb,
    struct Halfedge *newedge
);

struct Halfedge *ELgethash
(
    int b
);

struct Halfedge *ELleftbnd
(
    struct Point *p
);

void ELdelete
(
    struct Halfedge *he
);

struct Halfedge *ELright
(
    struct Halfedge *he
);

struct Halfedge *ELleft
(
    struct Halfedge *he
);

struct Site *leftreg
(
    struct Halfedge *he
);

struct Site *rightreg
(
    struct Halfedge *he
);

int fof
(
    double *rpar,           /* Array containing control point locations.    */
    double *csol,           /* Coefficients of polynomial computed by llsq  */
    double *elsqxy,         /* Residuals calculated after llsq is run       */
    long *nklsq,            /* Number of control points used in least       */
    double *avx,            /* Average radiometric difference value applied */
    double *maxz,           /* Maximum radiometric shift of input points    */
    double *maxzx,          /* Line location of maxz                        */
    double *maxzy,          /* Sample location of maxz                      */
    double *minz,           /* Minimum radiometric shift of input points    */
    double *minzx,          /* Line location of minz                        */
    double *minzy,          /* Sample location of minz                      */
    double *tmaxx,          /* Maximum line location in output space        */
    double *tmaxy,          /* maximum sample location in output space      */
    double *tminx,          /* minimum line location in output space        */
    double *tminy,          /* minimum sample location in output space      */
    double *tpts,           /* address of temporary workspace for control   */
    long nn,                /* Number of control points                     */
    long npoint,            /* Defaulted to 4 for pointing into rpar for a  */
    int zgeom               /* If radiometric transformation, then true(1), */
                            /* if geometric transformation, then false(0)   */
);

void geominit 
(
    int list_growsize
);

struct Edge *bisect
(
    struct Site *s1,
    struct Site *s2
);

struct Site *intersect
(
    struct Halfedge *el1,
    struct Halfedge *el2
);

int right_of
(
    struct Halfedge *el,
    struct Point *p
);

void endpoint
(
    struct Edge *e,
    int lr,
    struct Site *s
);

double dist
(
    struct Site *s,
    struct Site *t
);

void makevertex
(
    struct Site *v
);

void deref
(
    struct Site *v
);

void ref
(
    struct Site *v
);

void PQinsert
(
    struct Halfedge *he,
    struct Site *v,
    double offset
);

void PQdelete
(
    struct Halfedge *he
);

int PQbucket
(
    struct Halfedge *he
);

int PQempty (void);

struct Point PQ_min (void);

struct Halfedge *PQextractmin (void);

void PQinitialize 
(
    double ymin_in,
    double ymax_in,
    int    growsize
);

int insided
(
    double y,       /* line */
    double x,       /* sample */
    long nt,
    short *con1,
    short *con2,
    short *con3,
    double *rpar
);

long llsqd
(
    double *a,
    double *b,
    long m,
    long n,
    long l,
    double *x,
    long *ipiv,
    double eps,
    double *aux
);

void freeinit
(
    struct Freelist *fl,
    int size,
    int growsize
);

char *getfree
(
    struct Freelist *fl
);

void makefree
(
    struct Freenode *curr,
    struct Freelist *fl
);

int c_mkggrd
(
    struct GEOGRID *in_grid,
    char *hname,
    int  window[4],
    int  *gridns,           /* I/O: samples per grid cell */
    int  *gridnl,           /* I/O: lines per grid cell */
    double coeff[][6],
    short *con1,
    short *con2,
    short *con3,
    double *rpar,
    double tol,
    long ntri,
    long *grid_rows,        /* O: rows in the created grid */
    long *grid_cols         /* O: columns in the created grid */
);

int c_mkrgrd
(
    char *hname,
    int  window[4],
    long *grid_rows,
    long *grid_cols,
    int  *gridns,
    int  *gridnl,
    double coeff[][6],
    short *con1,
    short *con2,
    short *con3,
    double *rpar,
    double tol,
    long  ntri,
    double *maxg,
    double *maxgx,
    double *maxgy,
    double *ming,
    double *mingx,
    double *mingy
);

void prec6_
(
    double *in,
    long *nnum,
    double *out
);

int tri_
(
    int  *npts,
    long *nedge,
    long *ntri,
    double *xpt,
    double *ypt,
    short *n1,
    short *n2,
    short *n3,
    double *reject,
    long *numrej,
    double *rejpt,
    long *abendl
);

int scomp
(
    const void *s1,
    const void *s2
);

struct Site *nextone (void);
int setsites
(
    int *npts,
    double *xpt,
    double *ypt,
    double *reject,
    long *numrej,
    double *rejpt,
    long *abendl,
    double *xmin,
    double *xmax,
    double *ymin,
    double *ymax
);

void voronoi
(
    struct Site *(*nextsite)(void),
    long *ntri,
    short *n1,
    short *n2,
    short *n3,
    double xmin,
    double xmax,
    double ymin,
    double ymax,
    int   nsites
);

void getpar
(
    struct PARBLK *parblk,  /* TAE variable block                           */
    short *zgeom,           /* If radiometric transformation, then true(1)  */
    char outfile[4][CMLEN], /* Host filenames: 0 = output grid              */
                            /*                 1 = finite.pr                */
                            /*                 2 = input space plot file    */
                            /*                 3 = output space plot file   */
    FILE *outfp[4],         /* Output files:   0 = output grid              */
                            /*                 1 = finite.pr                */
                            /*                 2 = input space plot file    */
                            /*                 3 = output space plot file   */
    long *numfile,          /* Number of files opened                       */
    char ha4[CMLEN],        /* Input control point filename (IBIS)          */
    char ha5[CMLEN],        /* Input control point filename (TPL)           */
    long *ftype,            /* Type of input file: 1 = IBIS graphics        */
                            /*                     2 = tie points location  */
    int  window[4],         /* Image window: SL, SS, NL, NS                 */
    double *reject,         /* Tolerance defining maximum separation of     */
                            /* duplicate control points                     */
    double *tol,            /* Grid tolerance value                         */
    long *cnt1,             /* Input plot flag: 0 = don't plot input points */
                            /*                  1 = plot                    */
    long *cnt2,             /* Output plot flag:0 = don't plot output points*/
                            /*                  1 = plot                    */
    char trend[SZSUBCMD],   /* The method applied to calculate the          */
                            /* radiometric difference value for the four    */
                            /* corner points far outside the output image   */
                            /* area.                                        */
                            /* = FIRST, the first order polynomial fit of   */
                            /* all the radiometric difference values        */
                            /* = AVERAGE, the average of all the radiometric*/
                            /*   difference values                          */
                            /* = USER, the user supplies the four corner    */
                            /*   point values                               */
    double corner[4],       /* If trend="USER", the user supplies the four  */
                            /* corner radiometric difference values in the  */
                            /* order: left, lower, upper, right             */
    char *debug,            /* If debug desired, then true(1)               */
    int  *gridnl,           /* Number of lines within a single grid cell    */
    int  *gridns,           /* Number of samples within a single grid cell  */
    double *miny,           /* Minimum sample location in final output space*/
    double *minx,           /* Minimum line location in final output space  */
    double *maxy,           /* Maximum sample location in final output space*/
    double *maxx            /* Maximum line location in final output space  */
);

void cleanup
(
    struct PARBLK *parblk, 
    char hosout[][CMLEN], 
    FILE *outfp[]
);

int c_getcfp
(
    char *ha1,
    char *ha2,
    long *ftype,
    double *buf,
    long *tcount,
    short *zgflg,
    struct GEOGRID **geogrid,
    int  *window
);

void outheader
(
    FILE *outfp,             /* Output file                                  */
    double avx,              /* Average radiometric difference value applied */
                             /* to four corner points far outside output     */
                             /* image area if option trend="average"         */
    double *csol,            /* Coefficients of polynomial computed by llsq  */
    double maxz,             /* Maximum radiometric shift of input points    */
    double maxzx,            /* Line location of maxz                        */
    double maxzy,            /* Sample location of maxz                      */
    double minz,             /* Minimum radiometric shift of input points    */
    double minzx,            /* Line location of minz                        */
    double minzy,            /* Sample location of minz                      */
    char *tname4,            /* Input control point filename (IBIS)          */
    short zgeom,             /* If radiometric transformation, then true(1)  */
    char debug               /* If debug desired, then true(1)               */
);

void fourcorners
(
    double tmaxx,    /* Maximum line location in output space               */
    double tmaxy,    /* maximum sample location in output space             */
    double tminx,    /* minimum line location in output space               */
    double tminy,    /* minimum sample location in output space             */
    int  *nn,        /* The number of control points                        */
    long npoint,     /* Defaulted to 4 for pointing into rpar for a         */
                     /* geometric transformation.  Reset to 3 for           */
                     /* radiometric transformation.                         */
    short zgeom,     /* If radiometric transformation, then true(1)         */
    double *rpar,    /* Array containing control point locations.           */
                     /* If geometric transformation requested, the          */
                     /* order is input line,input sample, output            */
                     /* line, output sample.  If radiometric                */
                     /* transformation output line, output sample,          */
                     /* radiometric difference.                             */
    FILE *outfp,     /* Output file                                         */
    double *tpts,    /* address of temporary workspace for control          */
    double *csol,    /* Coefficients of polynomial computed by llsq         */
    char *trend,     /* The method applied to calculate the                 */
                     /* radiometric difference value for the four corner    */
                     /* points far outside the output image area.           */
    double avx,      /* Average radiometric difference value applied to     */
                     /* four corner points far outside output image area    */
                     /* if option trend="average"                           */
    double corner[4] /* If trend="USER", the user supplies the four         */
                     /* corner radiometric difference values in the         */
                     /* order: left, lower, upper, right                    */
);

void solve
(
    double *rpar,           /* Array containing control point locations.    */
                            /* If geometric transformation requested, the   */
                            /* order is input line,input sample, output     */
                            /* line, output sample.  If radiometric         */
                            /* transformation output line, output sample,   */
                            /* radiometric difference.                      */
    double coeff[][6],      /* Coefficients of polynomial.                  */
    short  *con1,           /* Pointers to nodes of triangle                */ 
    short  *con2,           /* Pointers to nodes of triangle                */ 
    short  *con3,           /* Pointers to nodes of triangle                */
    long   ntri,            /* Number of triangles in triangulation         */
    FILE   *outfp,          /* Output file finite.pr                        */
    char   debug,           /* If debug desired, then true(1)               */
    short  zgeom,           /* If radiometric transformation, then true(1)  */
    long   npoint           /* Defaulted to 4 for pointing into rpar for a  */
                            /* geometric transformation.  Reset to 3 for    */
                            /* radiometric transformation.                  */
);

void outplot
(
    double *rpar,           /* Array containing control point locations.    */
                            /* If geometric transformation requested, the   */
                            /* order is input line,input sample, output     */
                            /* line, output sample.  If radiometric         */
                            /* transformation output line, output sample,   */
                            /* radiometric difference.                      */
    short *con1,            /* Pointers to nodes of triangle                */
    short *con2,            /* Pointers to nodes of triangle                */ 
    short *con3,            /* Pointers to nodes of triangle                */
    long ntri,              /* Number of triangles in triangulation         */
    FILE *outfp,            /* Output plot file                             */
    long offset             /* Pointer offset: 2 if INPLOT file, else 0     */
);

#endif
