/****************************************************************************
NAME:				heap

PURPOSE:  Deal with heap-related information.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0	 01/99	L. Johnson    CSB      Minor modifications for LAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  None

PROJECT:  LAS


  The author of this software is Steven Fortune.  Copyright (c) 1994 by AT&T
  Bell Laboratories.
  Permission to use, copy, modify, and distribute this software for any
  purpose without fee is hereby granted, provided that this entire notice
  is included in all copies of any software which is or includes a copy
  or modification of this software and in all copies of the supporting
  documentation for such software.
  THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTY.  IN PARTICULAR, NEITHER THE AUTHORS NOR AT&T MAKE ANY
  REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
  OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
 
*****************************************************************************/

#include "defs.h"
#include "las.h"
#include "finite.h"

extern long fatal;
static int PQhashsize;
static struct	Halfedge *PQhash;
static int PQcount;
static int PQmin;
static double ymin, deltay;

void PQinsert
(
    struct Halfedge *he,
    struct Site *v,
    double offset
)
{
  struct Halfedge *last, *next;

  he->vertex = v;
  ref(v);
  he->ystar = v->coord.y + offset;
  last = &PQhash[PQbucket(he)];
  while ((next = last->PQnext) != (struct Halfedge *)NULL &&
      (he->ystar > next->ystar ||
      (he->ystar == next->ystar && v->coord.x > next->vertex->coord.x)))
    last = next;

  he->PQnext = last->PQnext; 
  last->PQnext = he;
  PQcount++;
}


void PQdelete(struct Halfedge *he)
{
  struct Halfedge *last;

  if(he->vertex != (struct Site *)NULL)
    {
    last = &PQhash[PQbucket(he)];
    while(last->PQnext != he)
      last = last->PQnext;
    last->PQnext = he->PQnext;
    PQcount--;
    deref(he->vertex);
    he->vertex = (struct Site *)NULL;
    }
}


int PQbucket(struct Halfedge *he)
{
  int bucket;

  bucket = (he->ystar - ymin)/deltay*PQhashsize;
  if(bucket < 0)
    bucket = 0;
  if(bucket >= PQhashsize)
    bucket = PQhashsize - 1;
  if(bucket < PQmin)
    PQmin = bucket;
  return(bucket);
}


int PQempty
(void)
{
  return(PQcount == 0);
}


struct Point PQ_min
(void)
{
  struct Point answer;

  while(PQhash[PQmin].PQnext == (struct Halfedge *)NULL)
    PQmin++;
  answer.x = PQhash[PQmin].PQnext->vertex->coord.x;
  answer.y = PQhash[PQmin].PQnext->ystar;
  return(answer);
}


struct Halfedge *PQextractmin
(void)
{
  struct Halfedge *curr;

  curr = PQhash[PQmin].PQnext;
  PQhash[PQmin].PQnext = curr->PQnext;
  PQcount--;
  return(curr);
}


void PQinitialize
(
    double ymin_in,
    double ymax_in,
    int    growsize
)
{
  int i; 

  PQcount = 0;
  PQmin = 0;
  ymin = ymin_in;
  deltay = ymax_in - ymin_in;
  PQhashsize = 4*growsize;
  if((PQhash = (struct Halfedge *)malloc(PQhashsize*sizeof(*PQhash))) == NULL)
  {
    c_errmsg("Error allocating dynamic memory","finite-alloc", &fatal);
  }
  for(i=0; i<PQhashsize; i++)
    PQhash[i].PQnext = (struct Halfedge *)NULL;
}

