/****************************************************************************
NAME:				insided

PURPOSE:  Calculate whether point x,y is contained within triangle number nt.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0	 01/99	L. Johnson    CSB      Header added.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  Run under TAE

PROJECT:  LAS

ALGORITHM:

  Given a triangle, define three vectors (V1, V2, V3) by subtracting the
       next vertex from the current vertex, moving around the triangle in
       a counterclockwise direction.
  For all three vertices do
      Create a vector (V) by subtracting the point in question from the
           current vertex.
      Compute the magnitude of the cross product of V and Vn.
      Since |V x Vn| = |V| |Vn| sin A (A = angle between V and Vn), if
           the cross product is negative, the point is outside the triangle.
           (Draw it out on paper and it will hopefully make sense.)
  End do
  Return true if point inside triangle, else false.

*****************************************************************************/

#include "las.h"
#include "finite.h"

int insided
(
    double y,       /* line */
    double x,       /* sample */
    long nt,
    short *con1,
    short *con2,
    short *con3,
    double *rpar
)

{  /*  insided  */

double x1;
double x2;
double y1;
double y2;
double cross;

long i;
long edge[4];


edge[0] = *(con1 + nt);
edge[1] = *(con2 + nt);
edge[2] = *(con3 + nt);
edge[3] = *(con1 + nt);
for (i = 0; i < 3; i++)
   {
   y1 = rpar[edge[i]];
   y2 = rpar[edge[i + 1]];
   x1 = rpar[edge[i] + 1];
   x2 = rpar[edge[i + 1] + 1];
   cross = (y - y1) * (x2 - x1) - (x - x1) * (y2 - y1);
   if (cross > 0) 
      return(FALSE);
   }

return(TRUE);
}  /*  insided  */
