/****************************************************************************
NAME:				memory

PURPOSE:  Deal with memory-related information.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0	 01/99	L. Johnson    CSB      Minor modifications for LAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  None

PROJECT:  LAS


  The author of this software is Steven Fortune.  Copyright (c) 1994 by AT&T
  Bell Laboratories.
  Permission to use, copy, modify, and distribute this software for any
  purpose without fee is hereby granted, provided that this entire notice
  is included in all copies of any software which is or includes a copy
  or modification of this software and in all copies of the supporting
  documentation for such software.
  THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTY.  IN PARTICULAR, NEITHER THE AUTHORS NOR AT&T MAKE ANY
  REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
  OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
 
*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "finite.h"


void freeinit(struct Freelist *fl, int size, int growsize)
{
  fl->head = (struct Freenode *)NULL;
  fl->nodesize = size;
  fl->growsize = growsize;
}


char *getfree(struct Freelist *fl)
{
  int i;
  struct Freenode *t;
  long fatal = -1;

  if(fl->head == (struct Freenode *)NULL)
    {
    if ((t = (struct Freenode *)malloc(fl->growsize * fl->nodesize)) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","finite-alloc", &fatal);
      }
    for(i=0; i<fl->growsize; i++) 	
      makefree((struct Freenode *)((char *)t+i*fl->nodesize), fl);
    }
  t = fl->head;
  fl->head = (fl->head)->nextfree;
  return((char *)t);
}


void makefree(struct Freenode *curr, struct Freelist *fl)
{
  curr->nextfree = fl->head;
  fl->head = curr;
}
