/*****************************************************************************
FUNCTION:	OUTHEADER

PURPOSE:     Write header information to output file.

PROGRAM HISTORY:
  Version   Date      Author     Code/Conr  Request
  -------   ----      ------     ---------  -------
   1.0     01/99    L. Johnson     CSB      Extracted from finite.c.
   1.1     04/99    L. Johnson     CSB      SR2613 Added missing braces.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT:	LAS(LAMS)

ALGORITHM:

*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "finite.h"

void outheader
(
    FILE *outfp,             /* Output file                                  */
    double avx,              /* Average radiometric difference value applied */
                             /* to four corner points far outside output     */
                             /* image area if option trend="average"         */
    double *csol,            /* Coefficients of polynomial computed by llsq  */
    double maxz,             /* Maximum radiometric shift of input points    */
    double maxzx,            /* Line location of maxz                        */
    double maxzy,            /* Sample location of maxz                      */
    double minz,             /* Minimum radiometric shift of input points    */
    double minzx,            /* Line location of minz                        */
    double minzy,            /* Sample location of minz                      */
    char *tname4,            /* Input control point filename (IBIS)          */
    short zgeom,             /* If radiometric transformation, then true(1)  */
    char debug               /* If debug desired, then true(1)               */
)
{
  fprintf(outfp, "    INPUT CONTROL POINT FILE = %s\n", tname4);

  if (zgeom)
    {
    fprintf(outfp, "\n    AVERAGE RADIOMETRIC SHIFT VALUE = %11.7f\n",avx);
    fprintf(outfp, "    MINIMUM RADIOMETRIC SHIFT VALUE OF INPUT POINTS ");
    fprintf(outfp, "AND ITS LOCATION (L,S):\n             ");
    fprintf(outfp, "%11.7f    %8.2f    %8.2f\n", minz, minzx, minzy);
    fprintf(outfp, "    MAXIMUM RADIOMETRIC SHIFT VALUE OF INPUT POINTS ");
    fprintf(outfp, "AND ITS LOCATION (L,S):\n             ");
    fprintf(outfp, "%11.7f    %8.2f    %8.2f\n\n", maxz, maxzx, maxzy);
    fprintf(outfp, "    COEFFICIENTS TO MAP OUTPUT SPACE TO RADIOMETRIC ");
    fprintf(outfp, "DIFF:\n        ");
    fprintf(outfp, "%11.2f %11.2f %11.2f\n", csol[0], csol[1], csol[2]);
    }
  else
    {
    fprintf(outfp,
            "\n    COEFFICIENTS TO MAP OUTPUT SPACE TO INPUT LINE:\n");
    fprintf(outfp, " %11.2f %11.2f %11.2f\n", csol[0], csol[1], csol[2]);
    fprintf(outfp, "    COEFFICIENTS TO MAP OUTPUT SPACE TO INPUT ");
    fprintf(outfp, "SAMPLE:\n");
    fprintf(outfp, " %11.2f %11.2f %11.2f\n", csol[3], csol[4], csol[5]);
    }

  /* If debug is set print out the header for points and residuals
     of the polynomial fit.
  ----------------------------------------------------------------*/
  if (debug)
    {
    if (!zgeom)
      {
      fprintf(outfp, "    OUTPUT LOCATIONS(LINE,SAMP) ");
      fprintf(outfp, "RESIDUAL(LINE,SAMP)\n");
      }
    else
      fprintf(outfp, "   LOCATION(LINE,SAMP)     RESIDUAL Z\n");
    }
}
