/*****************************************************************************
FUNCTION:	OUTPLOT

PURPOSE:     Produce plot file for input or output space.

PROGRAM HISTORY:
  Version	Date	Author		Code/Conr	Request
  -------	----	------		---------	-------
   1.0		1/99	L.Johnson	   CSB	      Extracted from finite.c

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:	LAS(LAMS)

ALGORITHM:

  For all triangles
      Extract 3 triangle vertices from array.
      Compute dx, dy for vertex 1 to vertex 2.
      Print x1, y1, dx, dy.
      Compute dx, dy for vertex 2 to vertex 3.
      Print x2, y2, dx, dy.
      Compute dx, dy for vertex 3 to vertex 1.
      Print x3, y3, dx, dy.
  End for      

*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "finite.h"

void outplot
(
    double *rpar,           /* Array containing control point locations.    */
                            /* If geometric transformation requested, the   */
                            /* order is input line,input sample, output     */
                            /* line, output sample.  If radiometric         */
                            /* transformation output line, output sample,   */
                            /* radiometric difference.                      */
    short *con1,            /* Pointers to nodes of triangle                */
    short *con2,            /* Pointers to nodes of triangle                */ 
    short *con3,            /* Pointers to nodes of triangle                */
    long ntri,              /* Number of triangles in triangulation         */
    FILE *outfp,            /* Output plot file                             */
    long offset             /* Pointer offset: 2 if INPLOT file, else 0     */
)
{
  double ldiff;             /* Line difference for plot of triangles        */
  double rout[6];           /* Array used for transformation grid.          */
  double sdiff;             /* Sample difference for plot of triangles      */
  long ii;                  /* Loop counter                                 */


  for (ii=0; ii<ntri; ii++)
    {
    rout[0] = rpar[con1[ii]-offset];
    rout[1] = rpar[con1[ii]-offset+1];
    rout[2] = rpar[con2[ii]-offset];
    rout[3] = rpar[con2[ii]-offset+1];
    rout[4] = rpar[con3[ii]-offset];
    rout[5] = rpar[con3[ii]-offset+1];

    ldiff = rout[2] - rout[0];
    sdiff = rout[3] - rout[1];
    fprintf(outfp, "%5d %9.2f %9.2f %9.2f %9.2f\n",
            ii+1, rout[0], rout[1], ldiff, sdiff);
    ldiff = rout[4] - rout[2];
    sdiff = rout[5] - rout[3];
    fprintf(outfp, "%5d %9.2f %9.2f %9.2f %9.2f\n",
            ii+1, rout[2], rout[3], ldiff, sdiff);
    ldiff = rout[0] - rout[4];
    sdiff = rout[1] - rout[5];
    fprintf(outfp, "%5d %9.2f %9.2f %9.2f %9.2f\n",
            ii+1, rout[4], rout[5], ldiff, sdiff);
    }
}
