/*  NAME:  SUBROUTINE PREC6
  
  FUNCTION:
	To limit the precision of a real*4 array to 6 or 7 digits -- the number
	of significan digits on the PN9050 and VAX respectively.  The output 
	is placed in a double precision array in order to be able the truely 
	represent the number in binary form.

  PROGRAM HISTORY:
  Version	Date	Author		Code/Cont	Request
  -------	----	------		---------	-------
    1.0	       12/86	B.AILTS		   CSB		
    2.0	       09/88    B.Ailts		   CSB		Updated to LAS 5.0
    3.0        07/98    L. Johnson         CSB          Converted from FORTRAN
                                                        to C.

  PROJECT:	LAMS

  LANGUAGE:	C

-----------------------------------------------------------------------------*/
#include <las.h>
#include <math.h>

void prec6_(double *in,long *nnum,double *out)
                 /* in            array of input floats        */
                 /* nnum          number of values in array    */
                 /* out           array of output floats       */
{
  long i;        /* Loop counter                               */
  double temp;   /* Temporary storage                          */


#ifndef vms
  for (i=0; i<*nnum; i++)
    {
    if (fabs(in[i]) < 1.)
      {
      temp = (double)((int)(in[i]*1000000. + 0.5));
      out[i] = temp/1000000.;
      }
    else if ((fabs(in[i]) >= 1.) && (fabs(in[i]) < 10.))
      {
      temp = (double)((int)(in[i]*100000. + 0.5));
      out[i] = temp/100000.;
      }
    else if ((fabs(in[i]) >= 10.) && (fabs(in[i]) < 100.))
      {
      temp = (double)((int)(in[i]*10000. + 0.5));
      out[i] = temp/10000.;
      }
    else if ((fabs(in[i]) >= 100.) && (fabs(in[i]) < 1000.))
      {
      temp = (double)((int)(in[i]*1000. + 0.5));
      out[i] = temp/1000.;
      }
    else if ((fabs(in[i]) >= 1000.) && (fabs(in[i]) < 10000.))
      {
      temp = (double)((int)(in[i]*100. + 0.5));
      out[i] = temp/100.;
      }
    else if ((fabs(in[i]) >= 10000.) && (fabs(in[i]) < 100000.))
      {
      temp = (double)((int)(in[i]*10. + 0.5));
      out[i] = temp/10.;
      }
    else
      out[i] = (double)(in[i]);
    }

#else

  for (i=0; i<nnum; i++)
    {
    if (fabs(in[i]) < 1.)
      {
      temp = (double)((int)(in[i]*10000000. + 0.5));
      out[i] = temp/10000000.;
      }
   else if ((abs(in[i]) .ge. 1) .and. (abs(in[i]) .lt. 10))
      {
      temp = (double)((int)(in[i]*1000000. + 0.5));
      out[i] = temp/1000000.;
      }
   else if ((abs(in[i]) .ge. 10) .and. (abs(in[i]) .lt. 100))
      {
      temp = (double)((int)(in[i]*100000. + 0.5));
      out[i] = temp/100000.;
      }
   else if ((abs(in[i]) .ge. 100) .and. (abs(in[i]).lt.1000))
      {
      temp = (double)((int)(in[i]*10000. + 0.5));
      out[i] = temp/10000.;
      }
   else if ((abs(in[i]).ge.1000) .and. (abs(in[i]).lt.10000))
      {
      temp = (double)((int)(in[i]*1000. + 0.5));
      out[i] = temp/1000.;
      }
   else if ((abs(in[i]).ge.10000).and.(abs(in[i]).lt.100000))
      {
      temp = (double)((int)(in[i]*100. + 0.5));
      out[i] = temp/100.;
      }
   else if ((abs(in[i]).ge.100000).and.(abs(in[i]).lt.1000000))
      {
      temp = (double)((int)(in[i]*10. + 0.5));
      out[i] = temp/10.;
      }
   else
      out[i] = (double)(in[i]);
    }

#endif

}
