/*****************************************************************************
   NAME:                          CLOSELAS

   FUNCTION:
     Closes all the input files

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2     11/99    G.Schmidt          CSB      FORTRAN to C conversion
*****************************************************************************/
#include "las.h"
#include "imageio.h"
#include "geomz.h"

void closelas (
  struct FDESC *fdesc1,    /* I: input file descriptor                  */
  struct GDESC *gdesc)     /* I: group descriptor for OUT               */
{
  long fatal = -1;    /* fatal error flag     */
  long one = 1;       /* constant one         */
  long status;        /* return status        */

  /* Catalog and close output files
     -------------------------------*/
  status = c_egclse (&gdesc);
  if (status != E_SUCC)
  {
    c_errmsg ("Error closing output file", "geomz-close", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Close the input file
     --------------------*/
  status = c_eclose (&fdesc1);
  if (status != E_SUCC)
  {
    c_errmsg ("Error closing input file", "geomz-close", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }
}
