******************************************************************************
C  NAME:	GETZVAL
C
C  FUNCTION: 
C
C  PROGRAM HISTORY:
C	Version		Date	Author		Code/Conr	Request
C	-------		----	------		---------	-------
C	   1.0			A.L.ZOBRIST	JPL/VICAR
C	   2.0		4/87	B.AILTS		   CSB		Conversion to
C								unix(newLAS)
C	   3.0		11/88   B.Ailts		   CSB		Conversion to
C								LAS 5.0
C
C  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C		Run under TAE
C
C  PROJECT:	las(LAMS)
C
C  ALGORITHM/DESCRIPTION:
C
C	After reading the input parameters which consist of the following:
C
C	Local varibles are:
C  	1.  IN--Input image.    
C  	2.  INFILE--Input file.
C	3.  LINECOL--Line column
C	4.  SAMPCOL--Sample column
C	5.  OCOLUMN--Output column
C	6.  KERNDIM--Kernal dimensions
C	7.  LINEINC--Line increment
C	8.  SAMPINC--Sample increment
C	9.  THRVAL--Threshold value
C	10. DEFVAL--Default value
C
C  IN and INFILE are opened for read.  Row by row the line,sample locations are
C  read from the columns specified.  These locations are then sent to the 
C  subroutine GETGRD which samples the radiometric vlaues around each point 
C  accourding to the kernal size and line and sample increment values.  The 
C  average of these values is then written to the ouput column of the IBIS 
C  tabular file.
C
C	After every row of the IBIS file is processed, all the files are closed,
C  the history and minmax files are updated and the completion message is 
C  displayed.
C
C*******************************************************************************

	program getzval

	include 'las.fin'

	include 'lams.fin'

        real*4 DATA(10000)
	real*4 defval
        REAL*4 PRED(2)
	real*4 RMAG(2)
	real*4 ROWBUF(3)
	real*4 SHIFT(2)
        REAL*4 SUM
	real*4 XG(6)

	integer*4 bands(2)
	integer*4 chop
	integer*4 count
	integer*4 eopenr
	integer   fdesc
	integer   fdesc2
	integer*4 getarr(3)
	integer*4 getin
	integer*4 getnam
	integer*4 ic
	integer*4 ire
	integer*4 row
	integer*4 kernel
	integer*4 msgtim
	integer*4 nbands
	integer*4 ncol
	integer*4 nl
	integer*4 nmimg
	integer*4 nrow
	integer*4 ns
	integer*4 one

      	integer*4	parb(XPRDIM)

	integer*4 qflag
	integer*4 reje
	integer*4 same
	integer*4 sl
	integer*4 ss
	integer*4 status
	integer*4 stdout
	integer*4 tbclos
	integer*4 tbopen
	integer*4 tbrrow
	integer*4 tbwrow
	integer*4 totbnd
	integer*4 window(4)
	integer*4 wsq
	integer*4 zero

	character*(CMLEN) hosout
        character*(CMLEN) hosin
	character*(CMLEN) taein
	character*(ERRLEN) msg

	common/curent/row
C       Place the large variables in a common block to work around a
C       recent change in the sgi compiler that causes core dumps when
C       large variables are declared on the stack
        common DATA

      DATA RMAG/1.,1./,XG/1.,0.,0.,0.,1.,0./

C Initialize variables
C---------------------
      call inlas(parb,stdout)
	row = 1
	one = 1
	sl = 1
	ss = 1
	nl = 0
	ns = 0
	zero = 0

C  Retreive input parameters and open files
C------------------------------------------
	status = getin(parb,'IN',1,taein,hosin,nmimg,window,
     .		       bands,nbands,totbnd,1,same,qflag)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','getzval-fatal',-1)
	end if

	if (totbnd .gt. 1) then
	  call errmsg('Too many bands in the input image',
     .		      'getzval-bands',1)
	  call errmsg('Fatal error encountered','getzval-fatal',-1)
	end if

	status = eopenr(fdesc,hosin,one,sl,ss,nl,ns,IREAD,EREAL,zero,1)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input image','getzval-open',1)
	   call errmsg('Fatal error encountered','getzval-fatal',-1)
	end if

	if ((nl .ne. window(3)) .and. (ns .ne. window(4))) then
	msg ='Windowing is not allowed-full image size will be used'
	   call errmsg(msg,'getzval-window',1)
	end if

        status = getnam(parb,'INFILE',1,hosout,'ibis_tab',IREAD)
	if (status .ne. ESUCC) then
           call errmsg('Fatal error encountered','getzval-fatal',-1)
	end if

        status = tbopen(fdesc2,hosout,nrow,ncol,3)
	if (status .ne. ESUCC) then
           call errmsg('Fatal error encountered','getzval-fatal',-1)
	end if

        call xrintg(parb,'LINECOL',1,getarr(1),count,status)
        call xrintg(parb,'SAMPCOL',1,getarr(2),count,status)
        call xrintg(parb,'OCOLUMN',1,getarr(3),count,status)
        CALL XRINTG(PARB,'KERNDIM',1,kernel,count,status)
        CALL XRREAL(PARB,'LINEINC',1,xg(1),count,status)
        CALL XRREAL(PARB,'SAMPINC',1,xg(5),count,status)
        CALL XRINTG(PARB,'THRVAL',1,REJE,count,status)
        CALL XRREAL(PARB,'DEFVAL',1,defval,count,status)
        call xrintg(parb,'$MSGTIM',1,msgtim,count,status)
	
        WSQ = kernel*kernel

C  MAIN PROCESSING LOOP
C---------------------
	call inittm('GETZVAL','Processed','row',' ','of tabular file',
     .			nrow,1,msgtim,GEN2)
	call startm

        DO 10 row = 1,nrow
           status = tbrrow(fdesc2,getarr,2,row,ROWBUF)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','getzval-fatal',-1)
	   end if

           PRED(1) = ROWBUF(1)
           PRED(2) = ROWBUF(2)

           CALL GETGRD(fdesc,DATA,kernel,kernel,NL,NS,
     .                 PRED,XG,SHIFT,RMAG,CHOP,bands,status)
	   if (status .eq. MOPIX) then
	      go to 100
	   else if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','getzval-fatal',-1)
	   end if
  
C           IF (CHOP.GE.1) GO TO 100
           SUM = 0
           IRE = 0
  
C  Calculate the average of the grid pixel values returned from getgrd
C---------------------------------------------------------------------
	   DO 40 IC=1,WSQ
              IF (data(ic).EQ.0) IRE = IRE+1
              SUM = SUM+data(ic)
40	   CONTINUE
           IF (IRE.GT.REJE) GO TO 100
           ROWBUF(3) = SUM/FLOAT(WSQ - ire)
           GO TO 30
100	   ROWBUF(3) = defval
30	   CONTINUE
           status = tbwrow(fdesc2,GETARR(3),1,row,ROWBUF(3))
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','getzval-fatal',-1)
	   end if
10	CONTINUE
	row = nrow
	call stoptm
	call eclose(fdesc)

	status = tbclos(fdesc2,hosout)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','getzval-close',1)
	 end if

	call complt('GETZVAL',hosout,1)

	END
