C     *************************************************************************
C     NAME 			INSCOUNT       	    		
C
C     PURPOSE	   Inserts a count in an IBIS graphics file.
C
C		   The default value indicates that the count
C		   of values from the input file is used.
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	JPL  		        		Original
C	K. Zanter	  Feb. l987		LAMS conversion  
C	B. Ailts	  Dec. 1988		LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	INSCOUNT must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and retrieval
C     of user-specified parameters.  The input graphics file is read in.
C     The appropriate count is written to the output graphics file, followed
C     by the values from the input file.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program inscount

	include 'las.fin'

	include 'lams.fin'

      	real*4 		rbuf(MMXPPR)
	real*4		rcount
	real*4		temp

	integer*4	cnt
	integer*4	count
	integer*4	getnam
	integer*4	grclos
	integer*4	gropen
	integer*4	grwrit
	integer*4	grread
	integer  	fdesc1
	integer  	fdesc2
	integer*4	i
	integer*4	ix
	integer*4	nline
	integer*4	nwd
	integer*4	outnl

      	integer*4	parblk(XPRDIM)

	integer*4	ptr
	integer*4	status
	integer*4	stout

	character*(CMLEN)	infile
	character*(CMLEN)	ofile

C	initialization
C	--------------
	call inlas(parblk,stout)
	count = 0
	temp = 0

C	get the input IBIS file name
C	----------------------------
	status = getnam(parblk,'INFILE',1,infile,'ibis_graph',0)
        if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','inscount-fatal',-1)
	endif

C	open the input IBIS graphics file
C	---------------------------------
      	status = gropen(fdesc2,infile,nwd,nline,0)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','inscount-open',1)
	   call errmsg('Fatal error encountered','inscount-fatal',-1)
	endif

C	get the INSTVAL parameter
C	------------------------
      	call xrintg(parblk,'INSTVAL',1,count,cnt,status)

C	get the output IBIS file name
C	-----------------------------
	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_graph',1)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','inscount-fatal',-1)
	endif
	

	if (cnt .eq. 0) then
C	   read input file
C	   ---------------
      	   do 110 ix = 1,nline

              status = grread(fdesc2,ix,rbuf)
	      if (status .ne. ESUCC) then
	         call errmsg('Error reading input file','inscount-read',1)
	         call errmsg('Fatal error encountered','inscount-fatal',-1)
	      endif
   
              if ((temp.eq.0).and.(rbuf(1).eq.0))go to 130
	      do 105 i = 1,nwd - 1
                 if ((rbuf(i).eq.0).and.(rbuf(i+1).eq.0))go to 130
	         count = count + 1
105	      continue
C	      Add 1 to count because above loop does not count the last
C	      sample
C	      ----------------------------------------------------------
	      count = count + 1
	      temp = rbuf(nwd)

 110  	   continue
130	   continue
	endif

C	put the count in the output buffer
C	----------------------------------
        rcount = FLOAT(count)
   	rbuf(1) = rcount

	if (nline .gt. 1) then
	   if (count .lt. (nwd * nline)) then
	      outnl = nline
	   else
	      outnl = nline + 1
	   endif 
	else
	   nwd = nwd + 1
	   outnl = nline
	endif

C	open the output IBIS graphics file
C	----------------------------------
      	status = gropen(fdesc1,ofile,nwd,outnl,1)
	if (status .ne. ESUCC) then
	   status = grclos(fdesc1,ofile)
	   call errmsg('Error opening output file','inscount-open',1)
	   call clean( parblk,'inscount',ofile,1)
	endif

C	create output file
C	------------------
      	ptr = 2
      	do 131 ix = 1,nline
           status = grread(fdesc2,ix,rbuf(ptr))
	   if (status .ne. ESUCC) then
	      status = grclos(fdesc1,ofile)
	      call errmsg('Error reading input file','inscount-read',1)
	      call clean( parblk,'inscount',ofile,1)
	   endif

           status = grwrit(fdesc1,ix,rbuf)
	   if (status .ne. ESUCC) then
	      status = grclos(fdesc1,ofile)
	      call errmsg('Error writing output file','inscount-write',1)
	      call clean( parblk,'inscount',ofile,1)
	   endif
	   rbuf(1) = rbuf(nwd + 1)

 131    continue

C	If a record needs to be added to the output file, 
C	and write out the last coordinate in the last line
C	--------------------------------------------------
	if (outnl .ne. nline) then
	   do 140 i = 2,nwd
	      rbuf(i) = 0.0
140	   continue
           status = grwrit(fdesc1,outnl,rbuf)
	   if (status .ne. ESUCC) then
	      status = grclos(fdesc1,ofile)
	      call errmsg('Error writing output file','inscount-write',1)
	      call clean( parblk,'inscount',ofile,1)
	   endif
	endif

C	close the input and output IBIS files
C	-------------------------------------
      	status = grclos(fdesc1,ofile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','inscount-close',1)
	endif

      	status = grclos(fdesc2,infile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','inscount-close',1)
	endif

C	output completion message
C	-------------------------
	call complt('INSCOUNT',ofile,1)

      	end
