******************************************************************************
C  NAME:	LSQMTB
C
C  PURPOSE:	To perform a least sqaures fit
C
C  PROGRAM HISTORY:
C	Version		Date	Author		Code/Conr	Request
C	-------		----	------		---------	-------
C	   1.0		10/80	A.L.ZOBRIST	JPL/VICAR
C	   2.0		5/87	B.AILTS		   CSB		Conversion to
C								1(newLAS)
C	   5.0		11/88   B. Ailts	EDC/CSB		Conversion to
C								LAS 5.0
C
C  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C		Run under TAE
C
C  PROJECT:	las(LAMS)
C
C  ALGORITHM/DESCRIPTION:
C  After reading the input parameters
C  INFILE are opened for read.  If specified, the group column is read
C  to calculate the number of groups and the rows within each group.
C  The amount of array space needed to calculate the least squares is 
C  calculated and an error message is output if the array space is exceeded.
C  For each group the columns specified by INDCOL and DEPCOL is retrieved
C  and sent to the least squares subroutine.  This subroutine calculates
C  the least squares fit of the data and returns a solution matrix.
C  This matrix is written to the columns specified by COEFCOL and it is
C  used to calculate the residuals between the values of the indepenent
C  columns and the depentent columns.  Row by row the data is placed in the
C  columns specified by COEFCOL and RESCOL of the input file.
C
C  After all the values are written to the correct column, the input file 
C  is closed and the completion message is displayed.
C
C******************************************************************************

	include 'las.fin'

	include 'lams.fin'

	integer*4 MAXSET
	parameter (MAXSET = 450)

	real*4 AUX(MMXCOL * 2)
 	real*4 depbuf(MMXROW)
      	real*4 eps
	real*4 f
        real*4 indbuf(MMXROW)
      	real*4 ROWBUF(MMXCOL)
      	real*4 X(MMXCOL)

        integer*4 clen
	integer*4 coefcnt
        integer*4 coefcol(MMXCOL)
	integer*4 count
	integer*4 depcol
	integer*4 getnam
	integer*4 grpcol
	integer*4 grpcnt
	integer*4 i
	integer*4 ier
	integer*4 indcnt
	integer*4 indcol(MMXCOL)
	integer*4 fdesc1
	integer*4 ipass
	integer*4 ipiv(MMXCOL)
	integer*4 iset
	integer*4 j
	integer*4 k
	integer*4 k0
	integer*4 len
	integer*4 ncol
        integer*4 npoint(MAXSET)
	integer*4 nsets
	integer*4 nstart(MAXSET)
	integer*4 nwrcl

      	integer*4	parb(XPRDIM)

	integer*4 ptr
	integer*4 rescnt
	integer*4 rescol
	integer*4 status
        integer*4 stdout
	integer*4 tbclos
	integer*4 tbopen
	integer*4 tbrcol
	integer*4 tbwrow

      	character*(CMLEN) hosin

      	data eps/1.0E-14/
      	data nsets/1/

C       Initialization of TAE
C       ---------------------
	call inlas(parb,stdout)

C       Get INFILE parameter and open file
C       ----------------------------------
      	status = getnam(parb,'INFILE',1,hosin,'ibis_tab',IREAD)
	if (status .ne. ESUCC) then
	   call errmsg ('Fatal error encountered','lsqmtb-fatal',-1)
	end if

      	status = tbopen(fdesc1,hosin,clen,ncol,3)
	if (status .ne. ESUCC) then
	   call errmsg ('Error opening input file','lsqmtb-open',1)
	   call errmsg ('Fatal error encountered','lsqmtb-fatal',-1)
	end if

C       Get column parameters
C       ---------------------
      	CALL XRINTG(PARB,'INDCOL',MMXCOL,indcol,indcnt,status)
      	CALL XRINTG(PARB,'DEPCOL',1,depcol,count,status)
      	CALL XRINTG(PARB,'COEFCOL',MMXCOL,coefcol,coefcnt,status)
      	CALL XRINTG(PARB,'RESCOL',1,rescol,rescnt,status)
      	CALL XRINTG(PARB,'GRPCOL',1,grpcol,grpcnt,status)

	NWRCL = coefcnt+rescnt
      	coefcol(coefcnt+1) = rescol

C       Calculate the groups based upon the values found in GRPCOL
C       (if specified)
C       ----------------------------------------------------------
      	IF(grpcnt .ne. 0) then
      	   CALL FINDCO(fdesc1,DEPBUF,grpcol,CLEN,NSTART,NPOINT,NSETS,
     .		       status)
	   if (status .ne. ESUCC) then
	      call errmsg ('Fatal error encountered','lsqmtb-fatal',-1)
	   end if
	end if

C       Check that the number of groups will not be exceeded the 
C	array space
C       ---------------------------------------------------------
	if (nsets .gt. MAXSET) then
	   call errmsg('The number of groups has been exceeded',
     .		       'lsqmtb-ngroup',1)
	   call errmsg('Fatal error encountered','lsqmtb-fatal',-1)
	end if

C       Check that the array space will not be exceeded for each group
C       --------------------------------------------------------------
	do 100 i = 1,nsets
	if ((npoint(i) * indcnt) .gt. MMXROW) then
	   call errmsg('No. of rows and columns are too large',
     .		       'lsqtmb-array',1)
	   call errmsg('Fatal error encountered','lsqmtb-fatal',-1)
	end if
100	continue

      	IPASS = 1
      	ptr = 0

C       Do for every group
C       ------------------
	DO 1000 ISET=1,NSETS
    5 	   CONTINUE

C          Retrieve data for multiple sets
C          -------------------------------
      	   IF(grpcnt .ne. 0) then 
              CALL GETSET(fdesc1,indbuf,
     .	                  indcol,indcnt,ISET,npoint,nstart,status)
	      if (status .ne. ESUCC) then
	         call errmsg('Fatal error encountered',
     .		             'lsqmtb-fatal',-1)
   	      end if

      	      LEN = NPOINT(ISET)
      	      CALL GETSET(fdesc1,DEPBUF,depcol,1,ISET,npoint,nstart,
     .			status)
	      if (status .ne. ESUCC) then
	         call errmsg('Fatal error encountered',
     .			     'lsqmtb-fatal',-1)
	      end if

C          Retrieve data for single set
C          ----------------------------
	   else
      	      DO 30 I=1,indcnt
                 status = tbrcol(fdesc1,indcol(I),
     .			         indbuf((I - 1) * clen + 1))
	         if (status .ne. ESUCC) then
	            call errmsg('Fatal error encountered',
     .				'lsqmtb-fatal',-1)
	         end if
   30 	      CONTINUE
      	      LEN = CLEN
      	      status = tbrcol(fdesc1,depcol,DEPBUF)
	      if (status .ne. ESUCC) then
	         call errmsg('Fatal error encountered',
     .			     'lsqmtb-fatal',-1)
	      end if
	   end if

C          Perform least squares fit
C          -------------------------
      	   IF(IPASS.EQ.2) GO TO 45
	   DO 41 k=1,indcnt
 41	      X(k) = 0.
	   IF (LEN.LT.indcnt) GO TO 42

      	   CALL LLSQ(INDBUF,DEPBUF,LEN,indcnt,1,X,IPIV,EPS,IER,AUX)
	   if (ier .ne. 0) then 
	      call errmsg('Fatal error encountered','lsqmtb-fatal',-1)
	   end if

 42   	   IPASS = 2
      	   GO TO 5
 45 	   CONTINUE
      	   IPASS = 1

C          Place the least squares solution coefficients into the 
C          output buffer
C          ------------------------------------------------------
      	   DO 70 K=1,indcnt
              ROWBUF(K) = X(K)
   70 	   CONTINUE

C          Calculate the residuals
C          -----------------------
      	   DO 60 J=1,LEN
              F = DEPBUF(J)
              K0 = 0
              DO 50 K=1,indcnt
                 F = F-INDBUF(K0+J)*X(K)
                 K0 = K0+LEN
   50         CONTINUE

	      IF (LEN.LE.indcnt) F = 0.
              ROWBUF(coefcnt+1) = F
              DEPBUF(J) = F

C             Write the coeficients and/or residuals to the correct 
C	      columns of the tabular file
C             -----------------------------------------------------
              IF((coefcnt .eq. 0) .AND. (rescnt .eq. 0)) GO TO 60
              ptr = ptr+1
              status = tbwrow(fdesc1,coefcol,nwrcl,ptr,ROWBUF)
	      if (status .ne. ESUCC) then
	         call errmsg('Error writing to the output file',
     .		     'lsqmtb-write',1)
	         call errmsg('Fatal error encountered',
     .			     'lsqmtb-fatal',-1)
	      end if
   60 	   CONTINUE
 1000 	CONTINUE

	status = tbclos(fdesc1,hosin)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','lsqmtb-close',1)
	end if

C       Call completion message
C       -----------------------
	call complt('LSQMTB',hosin,1)

      	end
