C     *************************************************************************
C     NAME 			MF       	    		
C
C     PURPOSE	   Performs mathematic operations on one or more
C	 	   columns of an IBIS tabular file.  FORTRAN like
C		   expressions are entered to indicate the operations
C		   to be performed and the input and output column
C		   numbers to use.
C	
C	 	   The expressions are written as a parameter string
C		   and must follow a certain syntax.  The variables
C	 	   representing column numbers are written as key-
C		   words.  C (must be upper case) followed by a 
C		   number indicates a column used as input or output.
C		   The functions available are:  SQRT, ALOG, ALOG10,
C		   AINT, SIN, COS, TAN, ASIN, ACOS, ATAN, ATAN2, ABS,
C		   MIN or AMIN1, MAX or AMAX1, and MOD or AMOD.  The
C		   standard binary operations +, -, *, /, **, and
C		   the logic operations, .AND., .OR., .XOR., and .NOT.
C		   are also included.  All operations listed operate
C		   as in FORTRAN with the exception of MIN and MAX
C		   which are binary only.
C
C		   Column statistics can also be calculated.  This op-
C		   eration is different than the previous operations 
C		   because it operates on a vertical column instead of
C		   horizontally across a row.  These special operations
C		   must be preceded by a # in the expression.  The
C		   allowable column operations are:  #MEAN, #SIGMA,
C		   #MIN, #MAX, #SUM, and #FIRST.  However, these op-
C		   erations cannot be used in an arithmetic expression.
C
C		   The maximum number of columns that can be referenced
C		   in one execution is 19.
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	JPL(A. Zobrist)  		        Original
C	K. Zanter	  March l987		LAMS conversion  
C	B. Ailts	  Nov. 1988		Las 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	MF must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  The input file is
C     opened.  The input expression is parsed to determine the
C     input and output columns numbers and the operations to be
C     performed.  The program used the KNUTH subroutine to compile
C     the expression into machine instructions.  The data from
C     the input columns is retrieved.  The expression is applied
C     to the input data in the XKNUTH subroutine to obtain the
C     result which is written to the indicated output column.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program mf

	include 'las.fin'

	include 'lams.fin'

	real*4		fbuf(250)
	real*4		fcol1(MMXROW)
	real*4		first
	real*4		result
	real*4		rmax
	real*4		rmean
	real*4		rmin
	real*4		rsig
	real*4		rsum
	real*4		tot
	real*4		v

	integer*4	c
	integer*4	clen
	integer*4	cols(19)
	integer*4	fnlen(10)
	integer*4	fnstr1(350)
	integer*4	fnstr2(350)
	integer*4	getnam
	integer*4	i
	integer*4	icntr
	integer*4	fdesc
	integer*4	ipt
	integer*4	ir
	integer*4	ix
	integer*4	j
	integer*4	jdig
	integer*4	jsv
	integer*4	jsvm
	integer*4	jx1
	integer*4	jx2
	integer*4	k
	integer*4	nc
	integer*4	ncol
	integer*4	ndig
	integer*4	nfnbuf
	integer*4	nstrng
	integer*4	num(25)
	integer*4	offset
        integer*4       int4
        integer*2       int2
        integer*1       swpres1(4)
        integer*1       swpres2(4)
        integer*1       tmpi1(4)
        real*4          tmpr1
        real*4          tmpr2

      	integer*4	parblk(XPRDIM)

	integer*4	status
	integer*4	stdout
	integer*4	tbclos
	integer*4	tbopen
	integer*4	tbrcol
	integer*4	tbwcol
	integer*4	tbrrow
	integer*4	wcolop

	logical		blank
	logical		colop

	character*(CMLEN)	infile

	character*35	fnbuf(10)

	data c/32/

      	data num/48,49,50,51,52,53,54,55,56,57,67,36,73,78,32,61,35,
     + 		 65,71,78,88,82,77,85,73/

        equivalence (swpres1, tmpr1)
        equivalence (swpres2, tmpr2)

C	Place the large variables in a common block to work around a
C	recent change in the sgi compiler that causes core dumps when
C	large variables are declared on the stack
	common parblk


C	initialization
C	--------------
	call inlas(parblk,stdout)

C	get the IBIS file name
C	----------------------
	status = getnam(parblk,'INFILE',1,infile,'ibis_tab',0)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','mf-fatal',-1)
	endif

C	open the IBIS file
C	------------------
	status = tbopen(fdesc,infile,clen,ncol,3)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening file','mf-open',1)
	   call errmsg('Fatal error encountered','mf-fatal',-1)
	endif

C	get the expressions
C	-------------------
	call xrstr(parblk,'MATHEXP',10,fnbuf,fnlen,nfnbuf,status)
	do 15 ix = 1,nfnbuf
	   call low2up(fnbuf(ix),fnlen(ix))
15	continue

C  1000 is entry point when processing multiple expressions.
C  COLUMN NUMBERS ARE ASSIGNED TO A STORAGE BUFFER BY POINTER VECTOR COLS.
C  STATISTICAL COLUMN OPERATIONS START AT #20.
C  -----------------------------------------------------------------------
	icntr = 2
	fnstr1(1) = 104
	offset = 1
	do 1 ix = 1,nfnbuf
	   do 2222 i = 1,fnlen(ix)
	      fnstr1(icntr) = ichar(fnbuf(ix)(i:i))
	      icntr = icntr+1
2222	   continue
	   fnstr1(icntr) = num(12)
	   fnstr1(icntr+1) = num(12)
	   fnstr1(icntr+2) = num(12)
	   fnstr1(icntr+3) = num(15)
	   icntr = icntr+4
1	continue

	do 2223 i = icntr,350
	   fnstr1(i) = num(12)
2223	continue

	nstrng = 0

 1000	ix = 0
        blank = .true.
        do 2 i = 1,19
           cols(i) = 0
 2	continue
        jsvm = 0
        colop = .false.

C       THE FOLLOWING SECTION PROCESSES ONE CHARACTER, CHECKS FOR 
C       SPECIAL COLUMN OPERATIONS, AND CHECKS FOR THE COLUMN 
C       CHARACTER 'C'
C       ---------------------------------------------------------
 10     ix = ix + 1
        ipt = ix + offset
	fnstr2(ix) = num(15)
        if (.not. blank) fnstr2(ix) = fnstr1(ipt)
        c = fnstr1(ipt)
        if (c .ne. num(17)) go to 21

        colop = .true.
        c = fnstr1(ipt + 3)
        do 22 i = 1,8
           wcolop = i
           if (c .eq. num(i+17)) go to 10
 22     continue
        call errmsg('Error in input expression','mf-exprssn',1)
        call errmsg('Fatal error encountered','mf-fatal',-1)

 21     if (c .eq. num(12)) go to 11
        if (c .eq. num(16)) blank = .false.
        if (c .ne. num(11)) go to 10

C       THIS SECTION PROCESSES THE COLUMN NUMBER FOLLOWING 'C'
C       ------------------------------------------------------
        ndig = 0
        nc = 0
        do 4 j = 1,2
           c = fnstr1(ipt+j)
           do 4 k = 1,10
              if (c .ne. num(k)) go to 4
              nc = nc*10+k-1
              ndig = ndig + 1
 4      continue
        if (ndig .eq. 0) go to 10
        do 5 j = 1,19
           jsv = j
           if (nc .eq. cols(j)) go to 6
           if (cols(j) .eq. 0) go to 6
 5      continue
	call errmsg('Error in input expression','mf-exprssn',1)
        call errmsg('Fatal error encountered','mf-fatal',-1)

 6      cols(jsv) = nc
        jsvm = MAX0(jsv,jsvm)
        if (blank) go to 10
        jx1 = jsv/10
        jx2 = jsv-jx1*10
        jdig = 1
        if (jsv .ge. 10) jdig = 2
        fnstr2(ix) = num(13)
        fnstr2(ix+1) = num(14)
        fnstr2(ix+2) = num(jx1+1)
        fnstr2(ix+jdig+1) = num(jx2+1)
        ix = ix+jdig+1
        offset = offset+ndig-jdig-1
        go to 10

C       STRING PROCESSING IS COMPLETED.  EITHER GO TO KNUTH OR PERFORM
C       SPECIAL COLUMN OPERATION.
C       --------------------------------------------------------------
 11     offset = offset + ix
	if (jsvm .eq. 0) go to 1000
	nstrng = nstrng + 1
        if (.not. colop) go to 23

        if (cols(2) .eq. 0) cols(2) = cols(1)
C       NOTE:  Changed this so that when doing a column operation
C	       and the input and output column are the same one,
C	       that it actually uses the specified column (instead
C	       of always using column 1) 
C	if (cols(2) .eq. 0) cols(2) = 1
C        rec = nrcol*(cols(2)-1)+nrec

        status = tbrcol(fdesc,cols(2),fcol1)
	if (status .ne. ESUCC) then
	   call errmsg('Error reading file','mf-read',1)
	   call errmsg('Fatal error encountered','mf-fatal',-1)
	endif
        tot = 0.
        rmin = fcol1(1)
        rmax = fcol1(1)
        first = fcol1(1)
        do 24 i = 1,clen
           v = fcol1(i)
           if (v .lt. rmin) rmin = v
           if (v .gt. rmax) rmax = v
 24     tot = tot + v
        rmean = tot/FLOAT(clen)
        rsum = tot
        tot = 0.
        do 25 i = 1,clen
           v = fcol1(i) - rmean
 25     tot = tot+v*v
        rsig = SQRT(tot/FLOAT(MAX0(clen-1,1)))
        do 26 i = 1,clen
           go to (31,32,33,34,35,36,37,38),wcolop
 31        fcol1(i) = rmean
           go to 26
 32        fcol1(i) = rsig
           go to 26
 33        fcol1(i) = rmin
           go to 26
 34        fcol1(i) = rmax
           go to 26
 35        fcol1(i) = first
           go to 26
 36        fcol1(i) = rsum
           go to 26
 37        if (i .eq. 1) go to 26
           fcol1(i) = fcol1(i)+fcol1(i-1)
           go to 26
 38        ir = clen-i+1
           if (ir .eq. 1) go to 26
           fcol1(ir) = fcol1(ir)-fcol1(ir-1)
 26     continue
        go to 71

C	COMPILES THE STRING INTO COMPUTER ARITHMETIC FORMAT
C	---------------------------------------------------
 23     call knuth(fnstr2,fbuf,status)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','mf-fatal',-1)
	endif

        fbuf(20) = 0.
	do 70 i = 1,clen
C	   read a row from the IBIS file
C	   -----------------------------
	   status = tbrrow(fdesc,cols,jsvm,i,fbuf)
	   if (status .ne. ESUCC) then
	      call errmsg('Error reading file','mf-read',1)
	      call errmsg('Fatal error encountered','mf-fatal',-1)
	   endif

	   fbuf(20) = fbuf(20) + 1.

C	   PERFORMS THE DESIRED ARITHMETIC
C	   -------------------------------
	   call xknuth(fbuf,result,status)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','mf-fatal',-1)
	   endif

C	   put the computed result in the output array
C	   But first, completely byte- and word-swap the result
C	   because it gets passed to the ls_replace library
C	   function which is going to do the same thing.  Why?
C	   -------------------------------------------
           tmpr1 = result
           swpres2(1) = swpres1(4)
           swpres2(2) = swpres1(3)
           swpres2(3) = swpres1(2)
           swpres2(4) = swpres1(1)
           result = tmpr2
C          int2 = swpres2(1)
C          swpres2(1) = swpres2(2)
C          swpres2(2) = int2
 	   fcol1(i) = result
C          print *, 'fcol1(i) = ', fcol1(i)
70	continue

C	output the new column to the IBIS file
C	--------------------------------------
 71	status = tbwcol(fdesc,cols(1),fcol1)
C       print *, 'status from tbwcol = ', status

	if (status .ne. ESUCC) then
	   call errmsg('Error writing file','mf-write',1)
	   call errmsg('Fatal error encountered','mf-fatal',-1)
	endif

C	if necessary, loop back to process next function
C	------------------------------------------------
	if (nstrng .lt. nfnbuf) go to 1000

C	close the IBIS file
C	-------------------
	status = tbclos(fdesc,infile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','mf-close',1)
	endif

C	output completion message
C	-------------------------
	call complt('MF',infile,1)

	end

C     ------------------------------------------------------------------
C     IBUF IS TYPE REAL IN CALLING PROGRAM, INTEGER HERE.
C
C     March l987   K. Zanter
C     NOTE:  This subroutine was originally a LAMS library routine.
C 	     Since the only application calling it was "MF", I 
C	     decided to put it in with the MF source file.
C     1.  Changed DATA statement that initializes the charr array
C     2.  Added 'status' to argument list for error trapping
C     3.  Added 'imageio.mfi' for the error status constants
C     ------------------------------------------------------------------
      subroutine KNUTH(FSTRNG,IBUF,status)

      IMPLICIT INTEGER(A-Z)

	include 'worgen.fin'

      integer*4 status,rstat
      LOGICAL SBOP2,BOP3,NBPO,FIRVAR
      COMMON /CKNUTH/FP,SBOP2,CP,NBPO
	DIMENSION FSTRNG(1)
      DIMENSION CHARR(9),IBUF(1),BOP3(40),STACK(50),PRIOR(40),BPOSTK(10)
      DATA PRIOR/4,4,5,5,6,3*7,5*0,1,0,4*7,3*1,7,7,1,3*7,6*3,3*2,7,2*1/
      DATA BOP3/5*.TRUE.,14*.FALSE.,3*.TRUE.,2*.FALSE.,.TRUE.,3*.FALSE.,
     .9*.TRUE.,.FALSE.,2*.TRUE./
      DATA CHARR/40,44,45,43,36,90,32,88,88/
C
      CHARR(8) = CHARR(1)
      DO 26 I=1,1000
         CHARR(9) = FSTRNG(I)
         IF (CHARR(9).EQ.CHARR(5)) GO TO 23
         IF (CHARR(9).EQ.CHARR(7)) GO TO 26
         IF (CHARR(8).NE.CHARR(1).AND.CHARR(8).NE.CHARR(2)) GO TO 22
         IF (CHARR(9).EQ.CHARR(3)) FSTRNG(I) = CHARR(6)
         IF (CHARR(9).EQ.CHARR(4)) FSTRNG(I) = CHARR(7)
 22      CHARR(8) = CHARR(9)
 26   CONTINUE
 23   BP = 0
      OP = 59
      FP = 0
      CP = 20
      SP = 0
      S = 12
      I = 62
      FIRVAR = .TRUE.
      IBUF(63) = 15*65536
      GO TO 4
 2    IF (SBOP2) GO TO 3
 4    SP = SP+1
      STACK(SP) = S
 5    CALL SGET(S,FSTRNG,IBUF,rstat)
      if (rstat .ne. ESUCC) then
	 status = EFAIL
	 return
      endif
      IF (FIRVAR) IBUF(61) = 13*65536+S
      FIRVAR = .FALSE.
      IF (NBPO) GO TO 2
      IF (S.EQ.14) GO TO 21
      BP = BP+1
      BPOSTK(BP) = S
      GO TO 5
 21   S = BPOSTK(BP)
      BP = BP-1
      SBOP2 = .TRUE.
      GO TO 2
 3    S2 = STACK(SP-1)
      IF (PRIOR(S).GT.PRIOR(S2)) GO TO 4
 6    IF (S2.EQ.12) GO TO 24
      IF (S2.EQ.9) GO TO 7
      IF (BOP3(S2)) GO TO 10
      GO TO 9
 7    STACK(SP-1) = STACK(SP)
      SP = SP-1
      GO TO 5
 9    LOC = STACK(SP)
      IBUF(OP+2) = STACK(SP-1)*65536+LOC
      IBUF(OP+4) = 14*65536+I
      OP = OP+4
      SP = SP-1
      GO TO 11
 10   LOC = STACK(SP-2)
      IOP = 14*65536+LOC
      IBUF(OP+2) = IOP-65536
      IF (IBUF(OP).NE.IOP.AND.LOC.GE.61) IBUF(LOC) = 1
      IF (IBUF(OP).EQ.IOP) OP = OP-2
      LOC = STACK(SP)
      IBUF(OP+4) = STACK(SP-1)*65536+LOC
      IBUF(OP+6) = 14*65536+I
      OP = OP+6
      SP = SP-2
 11   STACK(SP) = I
      IF (LOC.GE.61) IBUF(LOC) = 1
      IBUF(I) = 0
      I = I+2
      IBUF(OP+2) = 15*65536
      GO TO 3
 24   PTR = 61
      DO 25 I=61,249,2
         IBUF(PTR) = IBUF(I)
         M = IBUF(I)/65536
         IF (M.EQ.15) go to 99
         N = IBUF(I)-M*65536
         IF (M.NE.14.OR.IBUF(N).NE.0) PTR = PTR+2
 25   CONTINUE

99    continue
      status = ESUCC
      RETURN
      END

C     ------------------------------------------------------------------
C
C     March 1987    K. Zanter
C     NOTE:  This subroutine was originally a LAMS library routine.
C 	     Since the only place is was called was in "knuth", I 
C	     decided to put it in with the MF source file.
C       1.  Changed data statements for C,D,MINUS,AOP, and INT
C	2.  Added 'status' to argument list for error trapping
C	3.  Added 'imageio.mfi' for the error status constants
C     ------------------------------------------------------------------
      subroutine SGET(S,FSTRNG,BUF,status)

      IMPLICIT INTEGER(A-Z)

	include 'worgen.fin'

      REAL RNUM,RFAC,BUF,FLOAT
      integer*4  status
      LOGICAL SBOP2,BOP2,FIRST,NBPO
      COMMON /CKNUTH/FP,SBOP2,CP,NBPO
      DIMENSION FSTRNG(1),BOP2(40),BUF(1),AOP(10),INT(28),COP(20),
     .FCV(44),KCV(44),PRIOR(40),LOPV(10)
	data C/32/
	data D/32/
	data MINUS/45/
      DATA FCV/1661,1662,1663,1664,1665,1666,1667,1668,1669,16610,
     .16611,16612,16613,16614,16615,16616,16617,16618,16619,16620,
     .13686,19357,168481,
     .12677,1234410,12344,20117,1966,2648,1826,134311,134661,13452,1358,
     .1677,2431,2466,2452,128262,12966,13648,12826,24546,24004/
      DATA KCV/1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,3,4,
     .20,8,6,7,16,17,18,19,20,21,22,23,8,20,21,22,25,26,27,28,39,40/
      DATA LOPV/22,31,248,95,63,54,41,5,162,82/
      DATA COP/1,2,3,4,9,10,0,14,24,11,29,30,31,32,33,34,35,36,37,38/
      DATA BOP2/5*.TRUE.,4*.FALSE.,2*.TRUE.,8*.FALSE.,3*.TRUE.,2*.FALSE.
     .,.TRUE.,3*.FALSE.,9*.TRUE.,.FALSE.,2*.TRUE./
      DATA PRIOR/4,4,5,5,6,3*7,5*0,1,0,4*7,3*1,7,7,1,3*7,6*3,3*2,7,2*1/
	data AOP/43,45,42,47,40,41,32,44,90,36/

	data INT/48,49,50,51,52,53,54,55,56,57,46,65,76,79,71,73,78,
     .	         84,83,81,82,88,68,77,66,67,69,80/
     
C
      FIRST = .TRUE.
      NBPO = .TRUE.
      NUM = 0
      RNUM = 0.
      TYPE = 0
 100  C = FSTRNG(FP+1)
      IF (C.EQ.AOP(7)) GO TO 8
      FPX = FP+1
      DO 1 I=1,10
      ISAV = I
      IF (C.EQ.AOP(I)) GO TO 10
 1    CONTINUE
      IF (C.NE.INT(11)) GO TO 39
      LOP = 0
      DO 31 J=2,3
      D = FSTRNG(FP+J)
      DO 31 K=12,28
      IF (D.EQ.INT(K)) LOP = LOP*16+K-12
 31   CONTINUE
      DO 32 I=1,10
      ISAV = I+10
      FPX = FP+4+I/8
      IF (LOP.EQ.LOPV(I)) GO TO 10
 32   CONTINUE
 39   FIRST = .FALSE.
      DO 3 I=1,28
         IF (C.EQ.INT(I)) GO TO 4
 3    CONTINUE
	call errmsg('Error in input expression','mf-exprssn',1)
	status = EFAIL
	return

 4    IF (I-11) 7,6,5
 6    TYPE = 1
      RNUM = FLOAT(NUM)
      RFAC = 1.0
      GO TO 8
 5    TYPE = -1
 7    NUM = NUM*10+I-1
      RFAC = .1*RFAC
      RNUM = RNUM+RFAC*FLOAT(I-1)
 8    FP = FP+1
      GO TO 100
 10   IF (FIRST) GO TO 20
      SBOP2 = .FALSE.
      IF (TYPE) 11,12,13
 11   DO 14 I=1,44
      ISV = I
      IF (NUM.EQ.FCV(I)) GO TO 15
 14   CONTINUE
      call errmsg('Error in input expression','mf-exprssn',1)
      status = EFAIL
      return
 15   S = KCV(ISV)
      IF (ISV.LE.23) goto 99
      SBOP2 = BOP2(S)
      IF (PRIOR(S).EQ.1) NBPO = .FALSE.
      goto 99
 12   CP = CP+1
      BUF(CP) = FLOAT(NUM)
      S = CP
      goto 99
 13   CP = CP+1
      BUF(CP) = RNUM
      S = CP
      goto 99
 20   FP = FPX
      S = COP(ISAV)
      IF (S.EQ.14) NBPO = .FALSE.
      IF (C.EQ.AOP(9)) FSTRNG(FP) = MINUS
      IF (C.NE.AOP(3)) GO TO 21
      C = FSTRNG(FP+1)
      IF (C.NE.AOP(3)) GO TO 21
      S = 5
      FP = FP+1
 21   SBOP2 = BOP2(S)

99    continue
      status = ESUCC
      RETURN
      END

C     ------------------------------------------------------------------
C
C     March l987     K. Zanter
C     NOTE:  This subroutine was originally a LAMS library routine.
C 	     Since the only application calling it was "MF", I 
C	     decided to put it in with the MF source file.
C       1.  Added #IFDEF for system differences
C	2.  Used utility functions to do the bitwise logical operations
C       3.  Added 'status' to argument list for error trapping
C       4.  Added 'imageio.mfi' for the error status constants
C     ------------------------------------------------------------------
      subroutine XKNUTH(BUF,RESULT,status)

      IMPLICIT INTEGER(A-Z)

	include 'worgen.fin'

      REAL*8 DREG,DBLE
      REAL BUF,ROP,REG,RESULT,DIV,T,DSREG(2)
      integer*4 status
      INTEGER*2 OPF(2)
      DIMENSION BUF(1)
      EQUIVALENCE (OPF(1),ROP),(DSREG,DREG)
C
      PTR = 59
 100  PTR = PTR+2
      ROP = BUF(PTR)

C     OP = OPF(1)
C     OPND = OPF(2)
      OP = OPF(2)
      OPND = OPF(1)

      GO TO (1,2,3,4,5,6,7,8,99,99,99,99,13,14,15,16,17,18,19,20,21,22,
     .23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40),OP
 99   status = EFAIL
      return

 1    REG = REG+BUF(OPND)
      GO TO 100

 2    REG = REG-BUF(OPND)
      GO TO 100

 3    REG = REG*BUF(OPND)
      GO TO 100

 4    DIV = BUF(OPND)
      IF (ABS(DIV).GE..000001) GO TO 44
      IF (DIV.GE.0) DIV = DIV+.000001
      IF (DIV.LT.0) DIV = DIV-.000001
 44   REG = REG/DIV
      GO TO 100

 5    REG = AMAX1(REG,.000001)**BUF(OPND)
      GO TO 100

 6    REG = ALOG10(AMAX1(BUF(OPND),.000001))
      GO TO 100

 7    REG = ALOG(AMAX1(BUF(OPND),.000001))
      GO TO 100

 8    REG = AINT(BUF(OPND))
      GO TO 100

 13   REG = BUF(OPND)
      GO TO 100

 14   BUF(OPND) = REG
      GO TO 100

 15   RESULT = REG
      status = ESUCC
      RETURN

 16   REG = SQRT(ABS(BUF(OPND)))
      GO TO 100

 17   REG = SIN(BUF(OPND))
      GO TO 100

 18   REG = COS(BUF(OPND))
      GO TO 100

 19   REG = TAN(BUF(OPND))
      GO TO 100

 20   REG = AMAX1(REG,BUF(OPND))
      GO TO 100

 21   REG = AMIN1(REG,BUF(OPND))
      GO TO 100

 22   DIV = BUF(OPND)
      IF (ABS(DIV).GE..000001) GO TO 222
      IF (DIV.GE.0) DIV = DIV+.000001
      IF (DIV.LT.0) DIV = DIV-.000001
 222  REG = AMOD(REG,DIV)
      GO TO 100

 23   REG = ABS(BUF(OPND))
      GO TO 100

 24   REG = -BUF(OPND)
      GO TO 100

 25   IF((REG.EQ.0.).AND.(BUF(OPND).EQ.0.))GOTO 255
      REG=ATAN2(REG,BUF(OPND))
      GO TO 100
 255  REG = 0.
      GO TO 100

 26   REG = ASIN(BUF(OPND))
      GO TO 100

 27   REG = ACOS(BUF(OPND))
      GO TO 100

 28   REG = ATAN(BUF(OPND))
      GO TO 100

 29   T = 0.
      IF (REG.LT.BUF(OPND)) T = 1.
      REG = T
      GO TO 100

 30   T = 0.
      IF (REG.LE.BUF(OPND)) T = 1.
      REG = T
      GO TO 100

 31   T = 0.
      IF (REG.EQ.BUF(OPND)) T = 1.
      REG = T
      GO TO 100

 32   T = 0.
      IF (REG.NE.BUF(OPND)) T = 1.
      REG = T
      GO TO 100

 33   T = 0.
      IF (REG.GE.BUF(OPND)) T = 1.
      REG = T
      GO TO 100

 34   T = 0.
      IF (REG.GT.BUF(OPND)) T = 1.
      REG = T
      GO TO 100

 35   IBIT = REG
      JBIT = BUF(OPND)
      kbit = bitor(ibit,jbit)
      REG = FLOAT(KBIT)
      GO TO 100

 36   IBIT = REG
      JBIT = BUF(OPND)
      kbit = bitand(ibit,jbit)
      REG = FLOAT(KBIT)
      GO TO 100

 37   IBIT = REG
      JBIT = BUF(OPND)
      kbit = bitxor(ibit,jbit)
      REG = FLOAT(KBIT)
      GO TO 100

 38   REG = 1.-BUF(OPND)
      GO TO 100
 39   DSREG(1) = REG
      DSREG(2) = BUF(OPND)
      GO TO 100
 40   REG = DREG-DBLE(BUF(OPND))
      GO TO 100
      END
