C     *************************************************************************
C     NAME 			MGR2MTB       	    		
C
C     PURPOSE 	   Generates an IBIS tabular file from an IBIS graphics file.
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	JPL(B. Gokhman)  		        Original
C	K. Zanter	  Feb. l987		LAMS conversion  
C	K. Zanter	  March l987	        Increased dimension of the
C					        "buffin" array from 4000 
C						to 1800
C	B. Ailts	  Dec. 1988		LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	MGR2MTB must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  If necessary, the
C     defaulted parameters are set with the appropriate values.  
C     The input and output files are opened, and the output tabular
C     file is generated from the input graphics file according to
C     the given specifications.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
      program mgr2mtb

	include 'las.fin'

	include 'lams.fin'

      real*4		buffin(MMXPT)
      real*4		rowbuf(MMXCOL)

      integer*4		differ
      integer*4		ecount
      integer*4		elemts(MMXPT)
      integer*4		gcount
      integer*4		getnam
	integer*4	grclos
	integer*4	gropen
	integer*4	grread
      integer*4		gsize
      integer*4		i
      integer*4		icolmx
      integer*4		icols(MMXCOL)
      integer*4		icount
      integer  	  	fdesc1
      integer  		fdesc2
      integer*4		il
      integer*4		ik
      integer*4		inels(MMXCOL)
      integer*4		irw
      integer*4		is
      integer*4		jk
      integer*4		k
      integer*4		ncols
      integer*4		ncount
      integer*4		nl
      integer*4		nlocat
      integer*4		nrows
      integer*4		ns
      integer*4		offset

      	integer*4	parblk(XPRDIM)

      integer*4		rcount
      integer*4		status
      integer*4		stdout
	integer*4	tbclos
	integer*4	tbopen
	integer*4	tbwrow

      logical 		otsync

      character*(CMLEN)	infile
      character*(CMLEN)	ofile

      data rowbuf/ MMXCOL*0./ 
      data elemts/MMXPT*1/
C       Place the large variables in a common block to work around a
C       recent change in the sgi compiler that causes core dumps when
C       large variables are declared on the stack
        common buffin

C     initialization
C     --------------
      call inlas(parblk,stdout)

C     get input IBIS graphics file name
C     ---------------------------------
      status = getnam(parblk,'INFILE',1,infile,'ibis_graph',0)
      if (status .ne. ESUCC) then
	 call errmsg('Fatal error encountered','mgr2mtb-fatal',-1)
      endif

C     open the input IBIS graphics file
C     ---------------------------------
      status = gropen(fdesc2,infile,ns,nl,0)
      if (status .ne. ESUCC) then
	 call errmsg('Error opening input file','mgr2mtb-open',1)
         call errmsg('Fatal error encountered','mgr2mtb-fatal',-1)
      endif

      if (ns .gt. MMXPT) then
	 call errmsg('Input buffer is not large enough',
     .		     'mgr2mtb-fatal',-1)
      endif

C     get the NCOLS and OUTCOL parameters
C     -----------------------------------
      call xrintg(parblk,'NCOLS',1,ncols,ncount,status)
      call xrintg(parblk,'OUTCOL',MMXCOL,icols,icount,status)

C     check that both were not defaulted
C     ----------------------------------
      if ((ncount .eq. 0) .and. (icount .eq. 0)) then
	 call errmsg('Either NCOLS or OUTCOL must be specified',
     +		     'mgr2mtb-ncols',1)
         call errmsg('Fatal error encountered','mgr2mtb-fatal',-1)
      endif

C     -------------------------------------------------------
C     |if NCOLS was not specified (OUTCOL was specified)    |
C     |   set "ncols" to the largest column number in OUTCOL|
C     -------------------------------------------------------
      if (ncount .eq. 0) then
         k = icount
         icolmx = 0
	 do 5 i = 1,k
 	    icolmx = MAX(icolmx,icols(i))
5	 continue
	 ncols = icolmx
      endif

C     ------------------------------------------------------
C     |if OUTCOL was not specified (NCOLS was specified)   |
C     |   set output column array ("icols") with 1 to ncols|
C     ------------------------------------------------------
      if (icount .eq. 0) then
	 do 7 i = 1,ncols
	    icols(i) = i
7	 continue
	 k = ncols
      endif

C     ------------------------------------------------------
C     |if both OUTCOL and NCOLS were specified		   |
C     |   make sure "ncols" is set with the largest value  |
C     ------------------------------------------------------
      if ((ncount .ne. 0) .and. (icount .ne. 0)) then
	 k = icount
	 icolmx = 0
	 do 10 i = 1,k
	    icolmx = MAX(icolmx,icols(i))
10	 continue
	 ncols = MAX(ncols,icolmx)
      endif

C     get NELMTS and ELMTNUM parameters
C     ---------------------------------
      call xrintg(parblk,'NELMTS',1,gsize,gcount,status)
      call xrintg(parblk,'ELMNTNUM',MMXCOL,inels,ecount,status)

      if (gcount .ne. 0) k = gsize

      call xrintg(parblk,'NROWS',1,nrows,rcount,status)
      if (rcount .eq. 0)  nrows = nl * ns / k

      if (ecount .ne. 0) then
         do 15 i = 1,k
 	    elemts(i) = 0
15	 continue

 	 do 16 i = 1,ecount
	    elemts(inels(i)) = 1
16	 continue
      endif
      
C     get the output IBIS tabular file name
C     -------------------------------------
      status = getnam(parblk,'OUTFILE',1,ofile,'ibis_tab',1)
      if (status .ne. ESUCC) then
	 call errmsg('Fatal error encountered','mgr2mtb-fatal',-1)
      endif

C     open output IBIS tabular file
C     -----------------------------
      status = tbopen(fdesc1,ofile,nrows,ncols,1)
      if (status .ne. ESUCC) then
	 status = tbclos(fdesc1,ofile)
	 call errmsg('Error opening output file','mgr2mtb-open',1)
	 call clean(parblk,'mgr2mtb',ofile,1)
      endif

C     process input file
C     -------------------
      irw = 0
      otsync = .false.
      do 80 il = 1,nl

         status = grread(fdesc2,il,buffin)
	 if (status .ne. ESUCC) then
	    status = tbclos(fdesc1,ofile)
	    call errmsg('Error reading input file','mgr2mtb-read',1)
	    call clean(parblk,'mgr2mtb',ofile,1)
	 endif

	 if (otsync) then
	     jk = jk - differ
	     do 35 ik = k - differ + 1, k
		if (elemts(ik) .ne. 1) goto 35
		jk = jk + 1
		rowbuf(jk) = buffin(ik-k+differ)
35	     continue

	     otsync = .false.
	     irw = irw + 1
             status = tbwrow(fdesc1,icols,jk,irw,rowbuf)
	     if (status .ne. ESUCC) then
	        status = tbclos(fdesc1,ofile)
	        call errmsg('Error writing output file',
     .			    'mgr2mtb-write',1)
	        call clean(parblk,'mgr2mtb',ofile,1)
	     endif
	    
	     offset = differ + 1
         else
	     offset = 1
	 end if

         do 60 is = offset,ns,k
	    if (irw .ge. nrows) goto 82
            jk = 0
            do 40 ik = 1,k
               if (elemts(ik) .ne. 1) go to 40
               jk = jk + 1
               rowbuf(jk) = buffin(is+ik-1)
   40       continue
	    nlocat = is + k - 1
	    if(nlocat .gt. ns) then
	      otsync = .true.
	      differ = nlocat - ns
	    else
              irw = irw + 1
              status = tbwrow(fdesc1,icols,jk,irw,rowbuf)
 	      if (status .ne. ESUCC) then
	         status = tbclos(fdesc1,ofile)
	         call errmsg('Error writing output file',
     +		             'mgr2mtb-write',1)
	         call clean(parblk,'mgr2mtb',ofile,1)
	      endif
   	    end if
   60    continue
   80 continue

C     column end fill
C     ---------------
   82 irw = irw + 1

      do 84 i = 1,MMXCOL
         rowbuf(i) = 0
   84 continue

      do 88 i = irw,nrows
         status = tbwrow(fdesc1,icols,jk,i,rowbuf)
	 if (status .ne. ESUCC) then
	    status = tbclos(fdesc1,ofile)
 	    call errmsg('Error writing output file','mgr2mtb-write',1)
	    call clean(parblk,'mgr2mtb',ofile,1)
	 endif
   88 continue

C     close the input and output IBIS files
C     -------------------------------------
      status = tbclos(fdesc1,ofile)
      if (status .ne. ESUCC) then
         call errmsg('Error closing output file','mgr2mtb-close',1)
      endif

      status = grclos(fdesc2,infile)
      if (status .ne. ESUCC) then
         call errmsg('Error closing output file','mgr2mtb-close',1)
      endif

C     output completion message
C     -------------------------
      call complt('MGR2MTB',ofile,1)

      end
