/**************************************************************************
NAME:	MGR2TAB

PURPOSE:
To read an IBIS graphics file and write it to a labeled table polygon file. 

PROGRAM HISTORY:
Version		Date	Author		Code/Conr	Request
-------		----	------		--------	-------
  1.0		4/87	B. Ailts 	  CSB		PRF 3427
  2.0		1/89	B. Ailts	EDC/CSB		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        MGR2TAB must be run under TAE.

PROJECT:	LAS		

ALGORITHM DESCRIPTION:
The program begins by initializing TAE.
The user-specified parameters and the LAS global variables 
are then retrieved.  These parameters consist of:
1.  Infile--the input TAE file name of the IBIS graphics file
2.  Outlt--the output TAE file name of the labeled table file

After reading the input parameters, INFILE is opened and the 
polygon vertices are read.  These vertices are then written 
to the file specified in OUTLT in a labeled table polygon  
format.  
After all the polygon vertices are written, all the 
files are closed and the completion message is displayed.

ALGORITHM REFERENCES:	none

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "ltable.h"
#include "lams.h"

main()

{ /* main */
struct	PARBLK	parb;

float	*inbuf;

long	npoints;
long	std;
long	two = 2;

char	taeout[2][CMLEN];
char	hosin[CMLEN];
char	hosout[CMLEN];


/*  Initialize tae 
------------------*/
c_inlas (&parb,&std);
npoints = 1;

init(&parb,taeout,hosin,hosout);

rdgraph(hosin,&inbuf,&npoints);

writetab(hosout,taeout,hosin,inbuf,npoints);

free(inbuf);

c_complt("MGR2TAB",taeout,&two);

} /* main */

/****************************************************************************
*
*	INIT gets the TAE parameters
*
*****************************************************************************/

FUNCTION init(parb,taeout,hosin,hosout)


struct	PARBLK	*parb;
char	taeout[][CMLEN];
char	*hosin;
char	*hosout;


{  /* beginning of init*/
long	access;
long	fatal = -1;
long	status;
long	zero = 0;

/* INIT contructs the descriptors needed to retreive the parameter values
   needed and then retreives the parameter values.
--------------------------------------------------------------------------*/

/*  Retreive the TAE paramters
------------------------------*/
access = IREAD;
status = c_getnam(parb,"INFILE",&zero,hosin,"ibis_graph",&access);
if (status != E_SUCC) 
   c_errmsg("Fatal error encountered","mgr2tab-fatal",&fatal);

access = IWRITE;
status = c_getnam(parb,"OUTLT",&zero,hosout,"lt",&access);
if (status != E_SUCC) 
   c_errmsg("Fatal error encountered","mgr2tab-fatal",&fatal);

/*  TAE name is calculated for the output file in order to be used for the
    completion message
--------------------------------------------------------------------------*/
strcpy(taeout[0], hosout);

return;

}  /*end of init*/

/*****************************************************************************
*
*	WRITETAB writes a labeled table polygon file
*
******************************************************************************/

FUNCTION writetab(out,taeout,in,inbuf,npoints)

char	*out;
char	*in;
char	taeout[][CMLEN];
float	*inbuf;
long	npoints;

{  /*beginning of writetab*/

struct	TAB_DEF tabsub;
struct 	MATRIX  size;
struct	TAB_DEF tab;

long	cnt;
long	one = 1;

char    fdesc[CMLEN];
char 	*make_lt_name();

/*  WRITETAB opens the labeled table file polygon vertices are written 
    to the labeled table file
-----------------------------------------------------------------------*/
tab.ftype = "REGATT";
tabsub.ftype = "REGXY";
sprintf(fdesc,"File generated from file %s",in);
tab.fdesc = fdesc;
tabsub.fdesc = fdesc;
tab.subfile[0] = make_lt_name(out,1);
tab.nsubs = 1;
tabsub.nsubs = 0;
tabsub.subfile[0]="/0";
strcpy(taeout[1], tab.subfile[0]);

size.nrow = 1;
size.ncol = 1;

/*  Open the input file
-----------------------*/
open_tab(&tab,out,IWRITE);

open_tab(&tabsub,tab.subfile[0],IWRITE);

put_vector(&tab,"REGNUM",I4,"unique region number",1,1,FIELD_SEP2);
put_vector(&tab,"LABEL",C,"user defined label description",1,1,RECORD_SEP);
put_vector(&tabsub,"REGNUM",I4,"unique region number",1,1,FIELD_SEP2);
put_vector(&tabsub,"X",I4,"column number",1,1,FIELD_SEP2);
put_vector(&tabsub,"Y",I4,"row number",1,1,RECORD_SEP);

put_matrix(tab.vector,I4,&size,&one);
strcpy(tab.vector->next->ptr.c," ");
put_record(&tab);

/*  Write the x y coordinates to the output file
------------------------------------------------*/
for (cnt = 0;cnt < npoints*2;cnt = cnt + 2)
   {
   put_matrix(tabsub.vector,I4,&size,&one);
   put_matrix(tabsub.vector->next,R4,&size,&inbuf[cnt + 1]);
   put_matrix(tabsub.vector->next->next,R4,&size,&inbuf[cnt]);
   put_record(&tabsub);
   }

/*  Close input file
--------------------*/
close_tab(&tab);
close_tab(&tabsub);

return;

}  /* end of writetab */

/**************************************************************************
*
*	RDGRAPH reads an IBIS graphics 1 file
*
***************************************************************************/

FUNCTION rdgraph(hosin,inbuf,npoints)

char	*hosin;
float	**inbuf;
long	*npoints;

{  /* beginning of rdgraph  */

float	*bufptr;

long	access;
long 	fatal = -1;
long	i;
long	nl;
long	ns;
long	one = 1;
long	rec;
long	status;

int	fdesc1;
/*  RDGRAPH opens and reads the IBIS graphics file and puts the polygon
    coordinates into the output buffer which will be written to the GOF.
------------------------------------------------------------------------*/

/*  initialization of variables  
-------------------------------*/
rec = 1;

/*  Open the output file
------------------------*/
access = 0;
status = c_gropen(&fdesc1,hosin,&ns,&nl,&access);
if (status != E_SUCC)
   {
   c_errmsg("Error opening input file","mgr2tab-open",&one);
   c_errmsg("Fatal error encountered","mgr2tab-fatal",&fatal);
   }

bufptr  = *inbuf = (float *)malloc (ns * nl * 4);
if (*inbuf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","mgr2tab-alloc",&one);
   c_errmsg("Fatal error encountered","mgr2tab-fatal",&fatal);
   }

/*  Read the output records
----------------------------*/
for (i = 0 ; i < nl; i = i + 1,bufptr = bufptr + ns)
   {
   status = c_grread(&fdesc1,&rec,bufptr);
   if (status != E_SUCC)
      {
      c_errmsg("Error reading input file","mgr2tab-read",&one);
      c_errmsg("Fatal error encountered","mgr2tab-fatal",&fatal);
      }
   rec = rec + 1;
   } /* for i */

/*  Calculate the number of points. 
    The IBIS graphics file always contains the same first and last
    points.  A GOF polygon file does not need the first point repeated,
    so the GOf polygon file is written with one less point.
------------------------------------------------------------------------*/
bufptr = *inbuf;
for (i = 2;i < nl * ns ; i = i + 2)
   {
   if ((*(bufptr + i) == *bufptr) && (*(bufptr + i + 1) == *(bufptr + 1))) 
      break;
   else
      *npoints = *npoints + 1;
   }

/*  Close the output file
-------------------------*/
status = c_grclos(&fdesc1,hosin);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","mgr2tab-warn",&one);

return;

}  /*  end of rdgraph */
