C       ************************************************************************
C       NAME 			MTB2MGR       	    		
C
C       PURPOSE     Generates an IBIS graphics file from an IBIS tabular file.
C
C       PROGRAM HISTORY
C	  PROGRAMMER	     DATE		REASON
C	  ----------	     ----		------
C	  JPL (B. Gokhman)		        Original
C	  K. Zanter	     Feb. l987		LAMS conversion
C	  K. Zanter	     March l987		Increased dimension of
C						output buffer ("buf")
C						from 4000 to 7500
C	  B. Ailts	     Dec. 1988		LAS 5.0 conversion
C
C       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	  MTB2MGR must be run under TAE.
C
C       PROJECT		        LAMS		
C
C       ALGORITHM DESCRIPTION
C	   The program begins with the standard initialization and 
C       retrieval of user-specified parameters.  If necessary, the
C       defaulted parameters are set with the appropriate values.
C       The input and output files are opened, and the output graphics
C	file is generated from the input tabular file according to
C       the given specifications.
C
C       ALGORITHM REFERENCES 		none
C       ************************************************************************
	program mtb2mgr

	include 'las.fin'

	include 'lams.fin'

	real*4		buf(MMXPPR + MMXCOL)

	integer*4	clen
	integer*4 	cols(MMXCOL)
	integer*4	getnam
	integer  	fdesc1
	integer  	fdesc2
	integer*4	grclos
	integer*4	gropen
	integer*4	grwrit
	integer*4	i
	integer*4	incol
	integer*4	icount
	integer*4	j
	integer*4	ncol
	integer*4	ncount
	integer*4	nw1
	integer*4 	outnl
	integer*4 	outns

      	integer*4	parblk(XPRDIM)

	integer*4	ptr
	integer*4	rec1
	integer*4	rec2
	integer*4	status
	integer*4	stdout
	integer*4	tbclos
	integer*4	tbopen
	integer*4	tbrrow

	character*(CMLEN)	infile
	character*(CMLEN)	ofile

C	initialization
C	--------------
	call inlas(parblk,stdout)

C	get input IBIS file name
C	------------------------
	status = getnam(parblk,'INFILE',1,infile,'ibis_tab',0)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','mtb2mgr-fatal',-1)
	endif

C	open the input IBIS file
C	------------------------
	status = tbopen(fdesc2,infile,clen,incol,0)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','mtb2mgr-open',1)
	   call errmsg('Fatal error encountered','mtb2mgr-fatal',-1)
	endif

C	get the input column numbers
C	----------------------------
	call xrintg(parblk,'INCOL',MMXCOL,cols,icount,status)

C	get the number of columns
C	-------------------------
	call xrintg(parblk,'NCOLS',1,ncol,ncount,status)

C	validation 
C	-----------
	if ((icount .ne. 0) .and. (ncount .ne. 0)) then
	   call errmsg
     +	   ('NCOLS should not be used if INCOL is supplied',
     +	    'mtb2mgr-ncols',-1)
	endif

	if (ncount .eq. 0) then
	   if (icount .ne. 0) then
	      ncol = icount
	   else
	      ncol = incol
	   endif
	endif

	if (icount .eq. 0) then
	   do 1 i = 1,ncol
	      cols(i) = i
1	   continue
	endif

C	get the output IBIS file name
C	-----------------------------
	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_graph',1)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','mtb2mgr-fatal',-1)
	endif

C	set the size for the output graphics file
C	-----------------------------------------
	if ((ncol * clen) .le. MMXPPR) then
	   outnl = 1
	   outns = ncol * clen
	else
	   outnl = ((ncol * clen) + MMXPPR - 1)/MMXPPR
	   outns = MMXPPR
	endif

C	open the output IBIS graphics file
C	----------------------------------
	status = gropen(fdesc1,ofile,outns,outnl,1)
	if (status .ne. ESUCC) then
	   status = grclos(fdesc1,ofile)
	   call errmsg('Error opening output file','mtb2mgr-open',1)
	   call clean(parblk,'mtb2mgr',ofile,1)
	endif

C	process the file
C	---------------
	rec1 = 1
 	rec2 = 1
	nw1 = outns + 1
	ptr = 1
 10	if (rec2 .gt. clen) go to 99

	status = tbrrow(fdesc2,cols,ncol,rec2,buf(ptr))
	if (status .ne. ESUCC) then
	   status = grclos(fdesc1,ofile)
	   call errmsg('Error reading input file','mtb2mgr-read',1)
	   call clean(parblk,'mtb2mgr',ofile,1)
	endif
	
	rec2 = rec2 + 1
	ptr = ptr + ncol
 20	if (ptr .le. outns) go to 10

	status = grwrit(fdesc1,rec1,buf)
	if (status .ne. ESUCC) then
	   status = grclos(fdesc1,ofile)
	   call errmsg('Error writing output file','mtb2mgr-write',1)
	   call clean(parblk,'mtb2mgr',ofile,1)
	endif

	if (ptr .ne. nw1) then
	j = 1
	do 3 i = nw1,ptr - 1
 	   buf(j) = buf(i)
 3	   j = j + 1
	ptr = j
	else
	ptr = 1
	endif
	rec1 = rec1 + 1
	go to 20
C
 99	if (ptr .le. 1) go to 100
	do 8 i = ptr,outns
 	   buf(i) = 0.
8	continue

	status = grwrit(fdesc1,rec1,buf)
	if (status .ne. ESUCC) then
	   status = grclos(fdesc1,ofile)
	   call errmsg('Error writing output file','mtb2mgr-write',1)
	   call clean(parblk,'mtb2mgr',ofile,1)
	endif

C	close the input and output IBIS files
C	-------------------------------------
 100	continue
	status = grclos(fdesc1,ofile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','mtb2mgr-close',1)
	endif

	status = tbclos(fdesc2,infile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','mtb2mgr-close',1)
	endif

C	output completion message
C	-------------------------
	call complt('MTB2MGR',ofile,1)

	end
