C     ******************************************************************
C     NAME 			MTBGEN       	    		
C
C     PURPOSE	   Generates an IBIS tabular file.  Writes data
C		   values entered by the user or else data values
C		   of zero to an IBIS tabular file.
C	
C     FILE REFERENCES	   Output IBIS tabular file
C		 	   (Default extension is "ibis.tab")
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	JPL(A. Zobrist)  		        Original (called IGENER)
C	K. Zanter	  Apr. l987		LAMS conversion  
C	B. Ailts	  Oct. 1988		LAS 5.0 conversion
C       K. Lowell             4/96              Force blanks in pname for SGi
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	MTBGEN must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION:
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  The output IBIS
C     tabular file is opened and the indicated values are written
C     to it.
C
C     ALGORITHM REFERENCES 		none
C     ******************************************************************
	program mtbgen

	include 'las.fin'

	include 'lams.fin'

	real*4		buf(MMXROW)
	real*4		temp(100)

	integer*4 tbopen
	integer*4 tbwcol
	integer*4 tbclos
	integer*4 acc
	integer*4	count
	integer*4	getnam
	integer*4	i
	integer*4	j
	integer*4	length
	integer*4	maxval
	integer*4	ncols
	integer*4	nrows
	integer*4	ocount
	integer*4	outcol(10)

      	integer*4	parblk(XPRDIM)

	integer*4	ptr
	integer*4	status
	integer*4	stdout

	integer		fdesc

	character*(CMLEN)	ofile
	character*(CMLEN)	subcmd

	character*(ERRLEN)	msg

	character*10	pname
C       Place the large variables in a common block to work around a
C       recent change in the sgi compiler that causes core dumps when
C       large variables are declared on the stack
        common buf

1	FORMAT('Too many values entered in IDVALS',I2)
2       FORMAT('Extra values entered in IDVALS',I2)

C	initialization
C	--------------
	call inlas(parblk,stdout)

	pname(1:10) = 'IDVALS    '

C	get the output IBIS file name
C	-----------------------------
	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_tab',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','mtbgen-fatal',-1)
	endif

C	get the dimensions for the output file
C	--------------------------------------
	call xrintg(parblk,'NROWS',1,nrows,count,status)
	call xrintg(parblk,'NCOLS',1,ncols,count,status)

C	determine the specified subcommand
C	----------------------------------
	call xrstr(parblk,'_SUBCMD',1,subcmd,length,count,status)

C	(Note that for -ZEROS subcommand all that is needed is the 
C        call to "wrfil" since that subroutine will zero out the 
C        output file during the open)

C	open the output IBIS file
C	-------------------------
	acc = IWRITE
	status = tbopen(fdesc,ofile,nrows,ncols,acc)
	if (status .ne. ESUCC) then
	   status = tbclos(fdesc,ofile)
	   call errmsg('Error opening output file','mtbgen-open',1)
	   call clean(parblk,'mtbgen',ofile,1)
	endif

C	if necessary, get the values for the -DATA subcommand
C	-----------------------------------------------------
	if (subcmd .eq. 'DATA') then
	   call xrintg(parblk,'OUTCOL',10,outcol,ocount,status)

	   maxval = 0
	   do 3 i = 1,ocount
	      if (outcol(i) .gt. maxval)  maxval = outcol(i)
3	   continue

	   if (maxval .gt. ncols) then
	      call errmsg('OUTCOL column specification exceeds NCOLS',
     +			  'mtbgen-outcol',1)
	      call errmsg('Fatal error encontered','mtbgen-fatal',-1)
	   endif

C	   (had to add this additional check since users wanted a warning
C 	   message output if extra IDVALS values are ignored)
C	   --------------------------------------------------------------
	   if (ocount .lt. 10) then
	      do 50 i = ocount + 1, 10
                 pname(7:10)='    '
		 if (i .le. 9) then
		    write (pname(7:7),'(I1)') i
	         else
		    write (pname(7:8),'(I2)') i
	         endif

		 call xrreal(parblk,pname,100,temp,count,status)

	         if (count .ne. 0) then
		     write (msg,2) i
		     call errmsg(msg,'mtbgen-extra',1)
		     call errmsg('These values are not processed',
     +				 'mtbgen-extra',1)
		 endif
50	      continue
	   endif

C	   initialize the data buffer pointer
C	   ----------------------------------
	   ptr = 0

C	   zero out the data buffer
C	   ------------------------
	   do 5 i = 1,nrows
	      buf(i) = 0.0
5	   continue

C	   get the data values
C	   -------------------
C	   do 20 i = 1,ocount
	   do 100 i = 1,ocount
	      pname(7:10) = '    '
	      if (i .le. 9) then
		 write (pname(7:7),'(I1)') i
	      else
	  	 write (pname(7:8),'(I2)') i
	      endif

	      call xrreal(parblk,pname,100,temp,count,status)

	      if (count .eq. 0) then
		 call errmsg('Missing IDVALS specification',
     +			     'mtbgen-idvals',1)
		 call errmsg('Fatal error encontered','mtbgen-fatal',-1)
	      endif

	      if (count .gt. nrows) then
	         write (msg,1) i
		 call errmsg(msg,'mtbgen-excess',1)
		 call errmsg('Fatal error encontered','mtbgen-fatal',-1)
	      endif

	      do 10 j = 1,count
		 buf(j) = temp(j)
10	      continue

C20	   continue
C	endif

C	write the entered data values to the output file
C	------------------------------------------------
C	if (subcmd .eq. 'DATA') then
C	    i = 1
C	    do 100 j = 1,ocount
	       status = tbwcol(fdesc,outcol(i),buf)
	       if (status .ne. ESUCC) then
	          status = tbclos(fdesc,ofile)
		  call errmsg('Error writing output file',
     +			      'mtbgen-write',1)
	          call clean(parblk,'mtbgen',ofile,1)
	       endif

C	       i = i + nrows
100	    continue
	endif
	       
C	close the output IBIS file
C	--------------------------
	status = tbclos(fdesc,ofile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','mtbgen-close',1)
	endif

C	output completion message
C	-------------------------
	call complt('MTBGEN',ofile,1)

	end
