C     *************************************************************************
C     NAME 			MTBSORT       	    		
C
C     PURPOSE  	 Sorts all the columns of an IBIS tabular file based
C	  	 upon one or more control key (sort) columns.	   
C	
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	JPL(A. Zobrist)  		        Original
C	K. Zanter	  Feb. l987		LAMS conversion  
C	B. Ailts	  Dec. 1988		LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	MTBSORT must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of the user-specified parameters.  The IBIS tabular
C     file is opened.  Two temporary arrays are used.  The first
C     holds the data to be sorted, and the second is used to define
C     the boundary limits.  Limits are defined using an index number
C     indicating a portion of the file to be sorted.  The boundary
C     index is initializaed to 1 indicating the entire column is to
C     be sorted.  If subsequent sorts need to be made on the file for
C     a multiple column sort, the index number is broken into groupings
C     of the same number indicating the sort limits.  The holding
C     array is sorted each time the sort index changes.  At the com-
C     pletion of the sort process, the reordering is in the form of
C     pointers.  The pointer are used to move all columns of the file
C     one at a time.  The limit array serves as an index.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program mtbsort

	include 'las.fin'

	include 'lams.fin'

	real*4		fcol1(MMXROW)
	real*4		swpdcol(MMXROW)
	real*4		rptr

	integer*4	clen
	integer*4	cnt
	integer*4	count
	integer*4	cs(MMXROW)
	integer*4	cx(MMXROW)
	integer*4	getnam
	integer*4	i
	integer*4	index
	integer*4	iu
	integer*4	ix
	integer		fdesc1
	integer*4	length
	integer*4	lst
	integer*4	ncol
	integer*4	oldcs

      	integer*4	parblk(XPRDIM)

	integer*4	pl
	integer*4	pu
	integer*4	slen
	integer*4	sortcl(MMXCOL)
	integer*4	status
	integer*4	stdout
	integer*4	t
	integer*4	tbclos
	integer*4	tbopen
	integer*4	tbrcol
	integer*4	tbwcol

	logical 	dscnd

	character*(CMLEN)	infile

	character*4	oflag

C	Place the large variables in a common block to work around a
C	recent change in the sgi compiler that causes core dumps when
C	large variables are declared on the stack
	common parblk


C	initialization
C	--------------
	call inlas(parblk,stdout)

C	get IBIS tabular file name
C	--------------------------
	status = getnam(parblk,'INFILE',1,infile,'ibis_tab',0)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','mtbsort-fatal',-1)
	endif

C	open IBIS tabular file
C	----------------------
	status = tbopen(fdesc1,infile,clen,ncol,3)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','mtbsort-open',1)
	   call errmsg('Fatal error encountered','mtbsort-fatal',-1)
	endif

C	get sort column numbers
C	-----------------------
	call xrintg(parblk,'SORTCOL',MMXCOL,sortcl,count,status)
C       print *, 'retrieved sort col number = ', sortcl(1)

C	get index column number
C	----------------------
	call xrintg(parblk,'INDEXCOL',1,index,cnt,status)
	if (cnt .eq. 0) index = 0
C       print *, 'retrieved index col number = ', index

C	get ascending/descending order flag
C	-----------------------------------
	call xrstr(parblk,'LOGORDER',1,oflag,length,cnt,status)

	if (oflag .eq. 'DES') then
	   dscnd = .true.
	else
	   dscnd = .false.
	endif

C  PERFORM SORT.  CS WILL CONTAIN LEXICOGRAPHIC INDEX.  CX WILL CONTAIN
C  POINTERS.  THE FILE IS NOT MOVED HERE.
C  --------------------------------------------------------------------
        do 3 i = 1,clen
           cx(i) = i
           cs(i) = 1
 3      continue

        do 9 ix = 1,count
	   status = tbrcol(fdesc1,sortcl(ix),fcol1)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','mtbsort-fatal',-1)
	   endif

           call swparrbytes(fcol1, swpdcol, clen)
C          call printarr(fcol1, clen)
C          call printarr(swpdcol, clen)
C          print *, 'calling cormov'
C          call cormov(fcol1,cx,clen)
           call cormov(swpdcol,cx,clen)

           pu = 0
 12        pl = pu + 1
           if (pl .gt. clen) go to 20
 14        pu = pu + 1
           if (pu .lt. clen .and. cs(pu+1) .eq. cs(pl)) go to 14
           slen = pu-pl+1
C           call r4sort(fcol1(pl),cx(pl),slen)
           call lclsort(fcol1(pl),cx(pl),slen)
           go to 12
 20        pu = 0
 22        pl = pu + 1
           oldcs = cs(pl)
 24        pu = pu + 1
           cs(pu) = pl
           if (pu .ge. clen) go to 9
           if (fcol1(pu+1) .ne. fcol1(pl)) go to 22
           if (cs(pu+1) .ne. oldcs) go to 22
           go to 24
 9      continue

        if (.not. dscnd) go to 7
        do 4 i = 1,clen/2
           iu = clen+1-i
           t = cs(iu)
           cs(iu) = cs(i)
           cs(i) = t
           t = cx(iu)
           cx(iu) = cx(i)
           cx(i) = t
 4      continue

C     MOVE ENTIRE FILE ACCORDING TO POINTERS CX
C     -----------------------------------------
 7	do 6 ix = 1,ncol
C          print *, 'ix = ', ix
           if (ix .eq. index) go to 30

	   status = tbrcol(fdesc1,ix,fcol1)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','mtbsort-fatal',-1)
	   endif

C          print *, 'calling cormov(2)'
           call cormov(fcol1,cx,clen)
           go to 39

 30        lst = -1
           rptr = 0.
           do 31 i = 1,clen
              if (cs(i) .ne. lst) rptr = rptr+1.
              fcol1(i) = rptr
              lst = cs(i)
C             print *, 'lst = ', lst
 31	   continue

C 39        call printarr(fcol1,clen)
  39       status = tbwcol(fdesc1,ix,fcol1)
           if (status .ne. ESUCC) then
              call errmsg('Fatal error encountered','mtbsort-fatal',-1)	
           endif
 6      continue

C       close IBIS tabular file
C       -----------------------
        status = tbclos(fdesc1,infile)
        if (status .ne. ESUCC) then
           call errmsg('Error closing input file','mtbsort-close',1)	
        endif

C       output completion message
C       -------------------------
        call complt('MTBSORT',infile,1)

        end

C  **************************************************************************
C  *CORMOV PERFORMS AN IN PLACE MOVE OF VECTOR COL ACCORDING TO POINTERS CX.*
C  *FORMS THE DISJOINT CYCLES OF THE PERMUTATION.			    *
C  **************************************************************************
      	subroutine cormov(col,cx,clen)

	include 'las.fin'

	include 'lams.fin'

	integer*4	clen
C	integer*4	col(1)
	integer*4	cx(1)
	integer*4	i
	integer*4	iq
	integer*4	ix
	integer*4	k
C	integer*4	temp
        real*4	temp
 	real*4	col(1)

        do 10 ix = 1,clen
           i = ix
           iq = i
           temp = col(ix)
 1         k = cx(i)
           if (k .lt. 0) go to 10
           cx(i) = -k
           col(i) = col(k)
           iq = i
           i = k
           go to 1
 10        col(iq) = temp
C          print *, 'After do-loop'

        do 20 i = 1,clen
           cx(i) = -cx(i)
C          print *, 'i & cx(i) = ', i, ' & ', cx(i)
 20     continue

        return
        end


      	subroutine printarr(col,clen)
        real*4 col(1)
        integer*4 clen
        integer i

        do 10 i = 1, clen
        print *, 'array(', i, ')= ', col(i)
 10     continue
        print *, '----'

        return
        end

        subroutine lclsort(data,pointr,len)

        real*4 data(len)
        real*4 temp

        integer i
        integer itemp
        integer j
        integer len
        integer pointr(len)

        do 10 i=1,len-1
         do 10 j = 1,len-i
           if(data(j).gt.data(j+1)) then
              temp=data(j)
              data(j)=data(j+1)
              data(j+1)=temp
              itemp=pointr(j)
              pointr(j)=pointr(j+1)
              pointr(j+1)=itemp
           end if
 10     continue
        return
        end


        subroutine swparrbytes(incol, swpdcol, clen)

        real*4 incol(1)
        real*4 swpdcol(1)
        integer*4 clen
        integer*4 i
        integer*1       swpres1(4)
        integer*1       swpres2(4)
        real*4          tmpr1
        real*4          tmpr2
        equivalence (swpres1, tmpr1)
        equivalence (swpres2, tmpr2)

        do 10 i = 1, clen
          tmpr1 = incol(i)
          swpres2(1) = swpres1(4)
          swpres2(2) = swpres1(3)
          swpres2(3) = swpres1(2)
          swpres2(4) = swpres1(1)
          incol(i) = tmpr2
10      continue

        return
        end
