/*****************************************************************************
   NAME:                          CLOSELAS

   FUNCTION:
     Closes all the input files

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      9/99    G.Schmidt          CSB      FORTRAN to C conversion
*****************************************************************************/
#include "las.h"
#include "imageio.h"
#include "pmask.h"

void closelas
(
  struct FDESC *fdesc1[],  /* I: input file descriptors                 */
  long images,             /* I: number of images input from TAE        */
  struct GDESC *gdesc      /* I: group descriptor for OUT               */
)
{
    long fatal = -1;    /* fatal error flag     */
    long i;             /* looping variable     */
    long one = 1;       /* constant one         */
    long status;        /* return status        */

    /* Catalog and close output files
       ------------------------------*/
    if (gdesc)
    {
        status = c_egclse(&gdesc);
        if (status != E_SUCC)
        {
            c_errmsg("Error closing output file", "pmask-close", &one);
            c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
        }
    }
    /* Close all input files
       ---------------------*/
    for (i = 0; i < images; i++)
    {
        status = c_eclose(&fdesc1[i]);
        if (status != E_SUCC)
        {
            c_errmsg("Error closing input file", "pmask-close", &one);
            c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
        }
    }
}
