/*****************************************************************************
   NAME:                          DIRECT

   FUNCTION:
     Determines the direction of the leftmost arc of a given swath, with
     the middle of the swath being Y = YLINE.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      9/99    G.Schmidt          CSB      FORTRAN to C conversion
*****************************************************************************/
#include "las.h"
#include "lams.h"
#include "pmask.h"

void direct
(
  long yline,        /* I: line value for Y = YLINE                   */
  long intarc[],     /* I: arc intersections (needs to be zero based) */
  long nedges,       /* I: number of edges in xvert and yvert         */
  int *clokws,       /* O: flag to specify a clockwise direction      */
  float xvert[],     /* I: x vertices                                 */
  float yvert[],     /* I: y vertices                                 */
  float slope[],     /* I: slope of the lines                         */
  long listin[],     /* I: list of input vertices (needs to be 0-based) */
  long listout[],    /* I: list of output vertices (needs to be 0-based) */
  long npoly,        /* I: number of values in listin/listout         */
  int *flag          /* I/O: boolean flag                             */
)
{
    long cross;        /* cross product between x and y                 */
    long i, j1, j2, k; /* looping and index variables                   */
    long l1, l2;       /* index variables                               */
    long temp;         /* temp storage for the nose result              */
    long vinx;         /* x input vertice                               */
    long viny;         /* y input vertice                               */
    long voutx;        /* x output vertice                              */
    long vouty;        /* y output vertice                              */
    long xinter[100];  /* points of intersection of the line and arcs   */

    if (nedges == 0)
        return;

    /* Find the points of intersection between the line and arcs.
       ----------------------------------------------------------*/
    for (i = 0; i < nedges; i++)
    {
        k = intarc[i];
        if (slope[k] == 99999.0)
        {
            temp = nose(k, listin, listout, npoly);
            if (xvert[k] < xvert[temp])
                xinter[i] = (long) xvert[k];
            else
                xinter[i] = (long) xvert[temp];
        }
        else
            xinter[i] = (long)(xvert[k] + slope[k] *
                               ((float) yline - yvert[k]));
    }

    /* Sort the data array and the corresponding pointer array.
       --------------------------------------------------------*/
    i4sort (xinter, intarc, nedges);

    /* Determine the direction of the leftmost arc.
       --------------------------------------------*/
    if (!(*flag))
    {
        k = intarc[0];
        if (xinter[0] != xinter[1])
        {
            temp = nose(k, listin, listout, npoly);
            if (yvert[k] >= yvert[temp])
                *clokws = TRUE;
            else
                *clokws = FALSE;
        }
        else
        {
            j2 = k;
            j1 = nose(j2, listin, listout, npoly);
            l2 = incarc(k, listin, listout, npoly);
            l1 = nose(l2, listin, listout, npoly);
            vinx = (long) (xvert[l2] - xvert[l1]);
            viny = (long) (yvert[l2] - yvert[l1]);
            voutx = (long) (xvert[j2] - xvert[j1]);
            vouty = (long) (yvert[j2] - yvert[j1]);
            cross = vinx*viny - vouty*voutx;
            if (cross > 0)
                *clokws = FALSE;
            else if (cross < 0)
                *clokws = TRUE;
            else
            { /* cross == 0 */
                if (((vinx > 0) || (viny > 0)) && ((voutx > 0) || (vouty > 0)))
                    *clokws = FALSE;
                else
                    *clokws = TRUE;
            }
        }

        *flag = TRUE;
    }
}
