/*****************************************************************************
   NAME:                          EDGES

   FUNCTION:
     Compiles the list of arcs intersecting the given swath.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      9/99    G.Schmidt          CSB      FORTRAN to C conversion
*****************************************************************************/
#include "las.h"
#include "pmask.h"

void edges
(
  long intarc[],     /* O: arc intersections (needs to be zero based) */
  long iptr[],       /* I: array of indices for listin and listout
                           (zero-based)   */
  long iswath,       /* I: current swath being processed (zero-based) */
  long *nedges,      /* I/O: number of edges                          */
  long listin[],     /* I: list of input vertices (needs to be 0-based) */
  long listout[],    /* I: list of output vertices (needs to be 0-based) */
  long npoly         /* I: number of values in listin/listout         */
)
{
    long i, k;         /* looping and index variables          */
    long isquez;       /* index for the current edge in intarc */
    long ivert;        /* current vertex                       */
    long kedges;       /* keeps track of the added edges       */
    long newarc[2];    /* vertices for the new arc             */
    int skip;          /* flag to specify if the block of code
                          should be skipped                    */

    if (iswath <= 0)
    {

        /* Initialize the intarc list.
           ---------------------------*/
        ivert = iptr[0];
        intarc[0] = incarc (ivert, listin, listout, npoly);
        intarc[1] = ivert;
        *nedges = 2;
        return;
    }
    else
    {
        /* Consider two new arcs adjacent to iswath vertice.
           -------------------------------------------------*/
        ivert = iptr[iswath];
        newarc[0] = incarc (ivert, listin, listout, npoly);
        newarc[1] = ivert;

        /* Add new edges to the list or remvoe old ones.
           ---------------------------------------------*/
        kedges = *nedges;
        for (i = 0; i < 2; i++)
        {
            skip = FALSE;
            for (k = 0; k < *nedges && !skip; k++)
            {
                if (newarc[i] == intarc[k])
                {
                    intarc[k] = -1;
                    skip = TRUE;
                    break;
                }
            }

            if (!skip)
            {
                intarc[kedges] = newarc[i];
                kedges++;
            }
        }

        /* Squeeze the list and calculate the new value for nedges.
           --------------------------------------------------------*/
        isquez = 0;
        for (i = 0; i < kedges; i++)
        {
            if (intarc[i] != -1)
            {
                intarc[isquez] = intarc[i];
                isquez++;
            }
        }

        *nedges = isquez;
    }
}
