/*****************************************************************************
   NAME:                          POLYIN

   FUNCTION:

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      9/99    G.Schmidt          CSB      FORTRAN to C conversion
*****************************************************************************/
#include "las.h"
#include "lams.h"
#include "pmask.h"

void polyin
(
  long npairs,        /* I: number of points available for the polygon
                            (minimum of 4 points needed)                */
  int *newpol,        /* I/O: flag specifying if this is the first call
                              to polyin                                 */
  long *ivert,        /* I/O: current vertex (needs to be 0-based)      */
  long scale,         /* I: scaling factor                              */
  float grbuf[],      /* I: buffer of data from infile or PARAM stmt    */
  float xvert[],      /* I/O: x vertices                                */
  float yvert[],      /* I/O: y vertices                                */
  long listin[],      /* O: list of input vertices (0-based)            */
  long listout[],     /* I/O: list of output vertices (0-based)         */
  long *npoly         /* I/O: number of values in listin/listout        */
)
{
    float x;            /* X value              */
    float y;            /* Y value              */
    long ipair;         /* looping variable     */
    long fatal = -1;    /* fatal error flag     */
    long one = 1;       /* constant one         */

    if (npairs < 4)
    {
        c_errmsg("Minimum of four points is needed", "pmask-points", &one);
        c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
    }

    for (ipair = 0; ipair < npairs; ipair++)
    {
        x = grbuf[2*ipair+1];
        y = grbuf[2*ipair];

        if (scale > 1)
        {
            x /= (float) scale;
            y /= (float) scale;
        }

        /* Add this vertex to the list of vertices in the current polygon
           if it's valid and not the closing vertex of the polygon (identical
           to the initial vertex; "opening" vertex coordinates for each
           polygon are stored in the listout array) */
        if ((x != 0.0 || y != 0.0) && (x != -99.0 || y != -99.0) &&
                (x != xvert[listout[*npoly]] || y != yvert[listout[*npoly]]))
        {
            *newpol = FALSE;
            xvert[*ivert] = x;
            yvert[*ivert] = y;

            (*ivert)++;
            if (*ivert >= MMXPPR)  /* ivert is 0-based */
            {
                c_errmsg ("Maximum number of points exceeded", "pmask-max",
                          &one);
                c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
            }
        }
        else
        {
            if (*newpol)
                continue;
            listin[*npoly] = *ivert - 1;
            listout[*npoly+1] = *ivert;

            /* Save the vertex even though it won't be referenced outside
               this routine (or it may be overwritten) so there is something
               in this index for the previous IF conditional to compare the
               remaining vertices to */
            xvert[*ivert] = x;
            yvert[*ivert] = y;

            (*npoly)++;
            *newpol = TRUE;
        }
    }
}
