C*****************************************************************************
C       NAME: QREP
C
C       FUNCTION:
C       QREP reads an IBIS graphics or tabular file and outputs the 
C       data to the terminal, a file , or a disk file.
C
C       PROGRAM HISTORY:
C       Version         Date    Author          Code/conr       Request
C       -------         ----    ------          ---------       -------
C          1.0                                     JPL          original
C          2.0          7/86    B.AILTS            CSB          LAMS conversion
C          3.0          6/87    B.AILTS            CSB          Remove the GR2
C                                                               option
C          5.0          10/88   B. Ailts           CSB          LAS5.0 update
C          5.1          03/97   D.Etrheim          CSB          Null terminate
C                                                               temp file name
C
C       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C                       Run under TAE 
C
C       PROJECT:        LAMS
C
C       ALGORITHM DESCRIPTION:
C       The tae parmaters are read in.  These parameters consist of:
C
C       The INFILE file is opened  and the data
C       is read printed to the output device that is specified by PRINT. 
C
C       After all the data is printed out, the completion message is 
C       displayed.
C
C       ALGORITHM REFERENCES:   none
C
C******************************************************************************

      program qrep

        include 'las.fin'

        include 'lams.fin'

        integer*4 col(MMXCOL)
        integer*4 count
        integer*4 file
        integer*4 i
        integer*4 len
        integer*4 flen
        integer*4 lp
        integer*4 ncol
        integer*4 nrows

        integer*4       parb(XPRDIM)

        integer*4 setprt
        integer*4 srow
        integer*4 stdout
        integer*4 status
        integer*4 term
	integer*4 getnam

        character*(CMLEN) filename
        character*(CMLEN) prtfile
        character*86 format,form
        character*10 iform,subcmd
	character*(CMLEN) infile

C  Initialize variables
C----------------------
C  The following line is for CR9777, weird but works.
        print *, ""
        call inlas(parb,stdout)

        nrows = MMXROW
        prtfile = 'qrep'        

C  retreive the TAE parameters
C-----------------------------

        call xrstr(parb,'_SUBCMD',1,subcmd,len,count,status)
        
        status = setprt(parb,term,file,lp,prtfile,filename)
        if (status .ne. ESUCC) then
           call errmsg('Fatal error encountered','qrep-fatal',-1)
        end if

        if (subcmd .eq. 'GRAPHICS') then
                call wrgraph(parb,prtfile)
        else
                call xrintg(parb,'COLNUM',MMXCOL,col,ncol,status)

C  if COLNUM equals null then all columns should be printed.
C------------------------------------------------------------
                   if (ncol .le. 0) then
                        do 10 i = 1,MMXCOL
                                col(i) = i
10                      continue
                   ncol = MMXCOL
                   end if
                call xrstr(parb,'OUTFORM',1,form,flen,count,status)
                if (form(1:1) .ne. '(') then
                        format = '(1X,'//form(1:flen)//')'
                        flen = flen + 5
                else
                        format = form(1:1)//'1X,'//form(2:flen)
                        flen = flen + 3
                end if
                call xrintg(parb,'SROW',1,srow,count,status)
                call xrstr(parb,'FILEFORM',1,iform,len,count,status)
                call xrintg(parb,'NROWS',1,nrows,count,status)
                call wrtab(ncol,parb,format,col,nrows,srow,flen,iform,
     +                     prtfile)
        end if
                
        if (file .eq. 1) then
           call complt('QREP',filename,1)
        else
           call complt('QREP',' ',0)
        end if

        end

C******************************************************************************
        subroutine wrgraph(parb,prtfile)

        include 'las.fin'

        include 'lams.fin'

        real*4 rbuf(MEMXPT)

	integer*4 access
	integer*4 getnam
	integer*4 grclos
	integer*4 gropen
	integer*4 grread
	integer   fdesc
	integer*4 i
	integer*4 ip
	integer*4 lsstat
	integer*4 nrec

      	integer*4	parb(XPRDIM)

	integer*4 reclen
	integer*4 samp
	integer*4 status
	integer*4 unit

	character*(CMLEN) hosin
	character*(CMLEN)prtfile

	
       	status = getnam(parb,'INFILE',1,hosin,'ibis_graph',IREAD)
	if (status .ne. ESUCC) then
           call errmsg('Fatal error encoutered','qrep-fatal',-1)
        end if

        unit = 4
        status = lsstat(prtfile,0)
        if (status .eq. ESUCC) then
           call delfil(prtfile)
        end if

        open (unit,file=prtfile,status='new',err=21)

        go to 21
20      continue
           close (unit)
           call errmsg('Error opening output file','qrep-open',1)
           call clean(parb,'qrep',prtfile,1)
21      continue

        access = 0
        status = gropen(fdesc,hosin,reclen,nrec,access)
        if (status .ne. ESUCC) then
           close (unit)   
           call errmsg('Error opening input file','qrep-open',1)
           call clean(parb,'qrep',prtfile,1)
        end if

        DO 201 I=1,nrec
           status = grread(fdesc,i,RBUF)
           if (status .ne. ESUCC) then
              close (unit)   
              call errmsg('Error reading input file','qrep-read',1)
              call clean(parb,'qrep',prtfile,1)
           end if
           if (reclen .gt. 10) then
              do 205 samp = 1,reclen,10
                 if ((rbuf(samp) .eq. 0.0) .and. 
     .               (rbuf(samp + 1) .eq. 0.0)) 
     .              go to 206
                 WRITE(unit,210)(RBUF(IP),IP=samp,samp + 9)
205           continue
           else
                 WRITE(unit,210)(RBUF(IP),IP=1,reclen)
           endif
201     CONTINUE
206     continue
210     FORMAT(10F8.2)

        close (unit)
        status = grclos(fdesc,hosin)
        if (status .ne. ESUCC) then
           call errmsg('Error closing output file','qrep-close',1)
        endif

        return
        end

C******************************************************************************
        subroutine wrtab(ncol,parb,format,col,nrows,srow,flen,
     .                   iform,prtfile)

        include 'las.fin'

        include 'lams.fin'

        real*4 data(MMXCOL)

	integer*4 acc
	integer*4 count
	integer*4 tbclos
	integer*4 tbrrow
	integer*4 tbopen
	integer fdesc
	integer*4 unit
	integer*4 ncol
	integer*4 clen
	integer*4 i
	integer*4 srow
	integer*4 nrows
	integer*4 tcol
	integer*4 nc
	integer*4 nr
	integer*4 col(MMXCOL)
	integer*4 flen,line,min0
	integer*4 getnam
	integer*4 lsstat
	integer*4 ntit(MMXCOL)

      	integer*4	parb(XPRDIM)

	integer*4 status

	character*(CMLEN) hosin
	character*(CMLEN) prtfile

	character*86 format,hform
	character*10 head(MMXCOL),title(MMXCOL)
	character*10 iform

	unit = 4

	if (iform .eq. 'GEO') then
	     call geohead(head,ncol,col)
	else if (iform .eq. 'RAD') then
	     call radhead(head,ncol,col)
	else
       	     call xrstr(parb,'TEXT',MMXCOL,title,ntit,count,status)
	     call tabhead(head,title,ncol,col)
	end if

	call formhd (format,hform,flen)

	status = lsstat(prtfile,0)
	if (status .eq. ESUCC) then
	   call delfil(prtfile)
	end if

	open (unit,file=prtfile,status='new',err=25)

	go to 26
25	continue
	   close (unit)
	   call errmsg('Error opening output file','qrep-open',1)
	   call clean(parb,'qrep',prtfile,1)
26	continue

       	status = getnam(parb,'INFILE',1,hosin,'ibis_tab',IREAD)
	if (status .ne. ESUCC) then
	   close (unit)
	   call clean(parb,'qrep',prtfile,1)
	end if
        acc = IREAD
        status = tbopen(fdesc,hosin,clen,tcol,acc)
        if (status .lt. ESUCC) then
           close (unit)
           call errmsg('Error opening input file','qrep-open',1)
           call clean(parb,'qrep',prtfile,1)
        end if
        nc = min0(ncol,tcol)
        nr = min0(clen - srow + 1,nrows)
        
        write (unit,hform) (head(i),i = 1,nc)
        do 30 line = srow,nr + srow - 1
                status = tbrrow(fdesc,col,nc,line,data)
                if (status .ne. ESUCC) then
                   close (unit)
                   call errmsg('Error reading input file','qrep-read',1)
                   call clean(parb,'qrep',prtfile,1)
                end if
                write (unit,format) (data(i),i = 1,nc)
30      continue

        close (unit)
        status = tbclos(fdesc,hosin)
        if (status .ne. ESUCC) then
           call errmsg('Error closing output file','qrep-close',1)
        endif

        return
        end

C******************************************************************************
        subroutine formhd(format,hform,len)

        integer*4 len,a,b,c,d,e
        character*86 format,hform
        character*1 digit(10)

        data digit/'0','1','2','3','4','5','6','7','8','9'/

        e = 0
        do 10 a = 1,len
        e = e + 1
        if (e .gt. len) go to 11
                if ((format(e:e) .eq. 'F') .or. (format(e:e) .eq. 'I')
     .          .or. (format(e:e) .eq. 'E')) then
                   hform(e:e) = 'A'
                else if (format(e:e) .eq. '.') then
                   hform(e:e) = char(32)
                   do 20 b = e+1,e+3
                     d = 0
                     do 25 c = 1,10
                        if (format(b:b) .eq. digit(c)) then
                           hform(b:b) = char(32)
                           e = e + 1
                        else
                           d = d + 1
                        end if
25                   continue
                     if (d .eq. 9) then
                     go to 10
                     end if
20                 continue
                else
                        hform(e:e) = format(e:e)
                end if
10      continue
11      continue

        do 30 a = 1,len
35              continue
                if (hform(a:a) .eq. char(32)) then
                   do 40 b = a,len
                        hform(b:b) = hform(b+1:b+1)
40                 continue
                   go to 35
                end if
30      continue

        return
        end

C******************************************************************************
        subroutine geohead(head,ncol,col)

        include 'lams.fin'

        integer*4 col(1)
        integer*4 ncol

        character*10 ghead(MMXCOL),head(MMXCOL)

        data ghead/'From Frm','  To Frm',' Point #','Act Flag',
     .  ' In Line',' In Samp',' To Line',' To Samp','Mos Line',
     .  'Mos Samp','Xfr Line','Xfr Samp',' ','Corr Thr','  From Z',
     .  '    To Z',' Delta Z',' ','Flineres','Fsampres','Constant',
     .  'Mlineres','Msampres','  U Pt #',' ',' ',' ','Ave Line',
     .  'Ave Samp',' ',' ',' ',' ',' ',' '/

        do 20 i = 1,ncol
        head(i) = ghead(col(i))
20      continue

        return
        end

C******************************************************************************
        subroutine radhead(head,ncol,col)

        include 'lams.fin'

        integer*4 col(1)
        integer*4 ncol

        character*10 rhead(MMXCOL),head(MMXCOL)

        data rhead/'From Frm','  To Frm',' Point #','Act Flag',
     .  ' In Line',' In Samp','Mos Line','Mos Samp',' To Line',
     .  ' To Samp','F Orig Z','T Orig Z','Odelta Z','Line Off',
     .  'Samp Off',' O Min Z',' O Max Z','O Mean Z','F Mean Z',
     .  'T Mean Z','Mdelta Z','M Min Z','M Max Z','M Mean Z',
     .  ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '/

        do 20 i = 1,ncol
        head(i) = rhead(col(i))
20      continue

        return
        end

C******************************************************************************
        subroutine tabhead(head,title,ncol,col)

        include 'lams.fin'

        integer*4 col(1)
        integer*4 ncol

        character*10 title(MMXCOL),head(MMXCOL)

        do 20 i = 1,ncol
        head(i) = title(i)
20      continue

        return
        end
