/*******************************************************************************
NAME:		RANDPTS	

PURPOSE:    	RANDPTS	Generates random points near a densified cutline
			
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	01/91	New Development
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/

#include <stdio.h>
#include <math.h>

/*****************************************************************************/

/* fortran bridge for unix 
--------------------------*/
#ifdef unix
randpts_(sl,ss,nl,ns,seed,pntinc,yin,xin,npin,buffout,npout)
long 	*sl,*ss,*nl,*ns,*seed,*pntinc,*npin,*npout;
{
randpts(sl,ss,nl,ns,seed,pntinc,yin,xin,npin,buffout,npout);
}
#endif

/*****************************************************************************/

randpts(sl,ss,nl,ns,seed,pntinc,yin,xin,npin,buffout,npout)

long 	*sl,*ss,*nl,*ns,*seed,*pntinc,*npin,*npout;


float	*yin,*xin,*buffout;

{
long	i,j,k,step,offset,ext,r_lin,r_smp,t_sl,t_ss,t_nl,t_ns,npix;
long	nonfatal=1;

char	msg[80];

double 	c_randnm();

/*****************************************************************************/
sprintf (msg,"Number of points in densified cutline: %d",*npin);
c_errmsg (msg," ",&nonfatal);

*npout = 0;

if (*nl > *ns) ext = *nl;
else ext = *ns;

step = 1;

for (i = 1; ext > step * *pntinc; i++)
   {
   step *= 2;

   t_nl = *pntinc * step * 2;  
   t_ns = *pntinc * step * 2; 

   npix = t_nl * t_ns;

   for (j = step; j <= *npin; j += step)
      {
      t_sl = yin[j-1] - *pntinc * step;
      t_ss = xin[j-1] - *pntinc * step; 
      
      for (k = 0; k < 2; k++) 
 	 { 
         offset = npix * c_randnm(seed);

         r_lin = t_sl + (offset / t_ns);
         r_smp = t_ss + (offset % t_ns);

         if ((*sl <= r_lin) && (r_lin < (*sl + *nl))
	    && (*ss <= r_smp) && (r_smp < (*ss + *ns)))
	    {
      	    buffout[(*npout)*2] = r_lin;
      	    buffout[(*npout)*2 + 1] = r_smp; 
     	    (*npout)++;
	    }
         }	
      }
   }
   buffout[(*npout)*2] = *sl - 10;
   buffout[(*npout)*2 + 1] = *ss - 10;
   (*npout)++;
   buffout[(*npout)*2] = *sl - 10;
   buffout[(*npout)*2 + 1] = *ss + *ns + 9;
   (*npout)++;
   buffout[(*npout)*2] = *sl + *nl + 9; 
   buffout[(*npout)*2 + 1] = *ss - 10;
   (*npout)++;
   buffout[(*npout)*2] = *sl + *nl + 9; 
   buffout[(*npout)*2 + 1] = *ss + *ns + 9;
   (*npout)++;

sprintf (msg,"Number of random points added in image: %d",*npout);
c_errmsg (msg," ",&nonfatal);
}
