C     *************************************************************************
C     NAME 			ROWOP       	    		
C
C     PURPOSE	   Deletes rows from an IBIS tabular file.
C	
C		   Values in specified columns are checked against
C		   specified ranges.  If the value in a given column
C		   is in the appropriate range, the row containing
C	 	   the value in the given column is deleted as the
C		   input file is copied to the output file.
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	JPL(H. Wilczynski)  		        Original
C	K. Zanter	  Feb. l987		LAMS conversion  
C	B. Ailts	  Dec. 1988		LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	ROWOP must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  The input file is
C     opened, and rows are scanned to determine if they are to be
C     deleted.  Any deletions are flagged by storing information 
C     in a pointer vector.  The input file is copied column by 
C     column to the output file using the pointer vector to col-
C     lapse the columns.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program rowop

	include 'las.fin'

	include 'lams.fin'

	real*4		buf(MMXCOL)
	real*4		col(MMXROW)
	real*4		lower(MMXCOL)
	real*4		prec
	real*4		upper(MMXCOL)

	integer*4	clen
	integer*4	count
	integer*4	getnam
	integer*4	i
	integer*4	icol
	integer*4	irec
	integer*4	keycol(MMXCOL)
	integer*4	lcount
	integer  	fdesc1
	integer  	fdesc2
	integer*4	ncol
	integer*4	nkey
	integer*4	outlen

      	integer*4	parblk(XPRDIM)

	integer*4	ptr
	integer*4	ucount
	integer*4	status
	integer*4	stdout
	integer*4 	tbclos
	integer*4	tbopen
	integer*4	tbrcol
	integer*4	tbrrow
	integer*4	tbwcol

	logical		keep(MMXROW)

	character*(CMLEN)	infile
	character*(CMLEN)	msg
	character*(CMLEN)	ofile

C       Place the large variables in a common block to work around a
C       recent change in the sgi compiler that causes core dumps when
C       large variables are declared on the stack
        common col, keep

C	initialization
C	--------------
	call inlas(parblk,stdout)

C	get the input IBIS file name
C	----------------------------
	status = getnam(parblk,'INFILE',1,infile,'ibis_tab',0)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','rowop-fatal',-1)
	endif

C	open the input IBIS file
C	------------------------
	status = tbopen(fdesc2,infile,clen,ncol,0)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','rowop-open',1)
	   call errmsg('Fatal error encountered','rowop-fatal',-1)
	endif

C	get lower and upper limits 
C	--------------------------

1	FORMAT('The number of values specified for the LOWVAL, ',
     +	       'HIGHVAL, and COLNUM parameters are not equal')

	call xrreal(parblk,'LOWVAL',MMXCOL,lower,lcount,status)
	call xrreal(parblk,'HIGHVAL',MMXCOL,upper,ucount,status)

	if (lcount .ne. ucount) then
	   write (msg,1)
	   call errmsg(msg,'rowop-unequal',1)
	   call errmsg('Fatal error encountered','rowop-fatal',-1)
	endif

C	get key columns
C	---------------
	call xrintg(parblk,'COLNUM',MMXCOL,keycol,nkey,status)
	if (nkey .ne. lcount) then
	   write (msg,1)
	   call errmsg(msg,'rowop-unequal',1)
	   call errmsg('Fatal error encountered','rowop-fatal',-1)
	endif

C	get precision
C	-------------
	call xrreal(parblk,'OFFSET',1,prec,count,status)

C	initialize output length
C	------------------------
	outlen = 0

	do 10 irec = 1,clen
C	   get a row from the input file
C	   -----------------------------
	   status = tbrrow(fdesc2,keycol,nkey,irec,buf)
	   if (status .ne. ESUCC) then
	      call errmsg('Error reading input file','rowop-read',1)
	      call errmsg('Fatal error encountered','rowop-fatal',-1)
	   endif

C	   determine if the row should be deleted 
C	   --------------------------------------
	   keep(irec) = .true.
	   do 2 i = 1,nkey
	      if ((buf(i) .ge. (lower(i)-prec)) .and.
     +           (buf(i) .le. (upper(i)+prec))) keep(irec) = .false.
 2	   continue

	   if (keep(irec)) outlen = outlen + 1
 10	continue

C	check if all rows would be deleted
C	----------------------------------
	if (outlen .eq. 0) then
	   call errmsg('All rows would be deleted','rowop-delete',1)
	   call errmsg('No output file created','rowop-delete',-1)
	endif

C	get the output IBIS file name
C	-----------------------------
	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_tab',1)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','rowop-fatal',-1)
	endif

C	open the output IBIS file
C	-------------------------
	status = tbopen(fdesc1,ofile,outlen,ncol,1)
	if (status .ne. ESUCC) then
	   status = tbclos(fdesc1,ofile)
	   call errmsg('Error opening output file','rowop-open',1)
	   call clean(parblk,'rowop',ofile,1)
	endif

C	process the file
C	----------------
	do 20 icol = 1,ncol
	   ptr = 1
	   status = tbrcol(fdesc2,icol,col)
	   if (status .ne. ESUCC) then
	      status = tbclos(fdesc1,ofile)
	      call errmsg('Error reading input file','rowop-read',1)
	      call clean(parblk,'rowop',ofile,1)
	   endif

	   do 19 i = 1,clen
	      col(ptr) = col(i)
	      if (keep(i)) ptr = ptr + 1
 19	   continue

	   status = tbwcol(fdesc1,icol,col)
	   if (status .ne. ESUCC) then
	      status = tbclos(fdesc1,ofile)
	      call errmsg('Error writing output file','rowop-write',1)
	      call clean(parblk,'rowop',ofile,1)
	   endif

 20	continue

C	close the input and output IBIS files
C	-------------------------------------
	status = tbclos(fdesc1,ofile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','rowop-close',1)
	endif

	status = tbclos(fdesc2,infile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','rowop-close',1)
	endif

C	output completion message
C	-------------------------
	call complt('ROWOP',ofile,1)

	end
