C     *************************************************************************
C     NAME  	 	      SETGBLS      	    		
C
C     PURPOSE      Sets five global arrays ($OFFL,$OFFS,$XNL,$XNS,$MEANZ)
C	 	   with the values in the specified parameter file.  If the
C		   specified file does not exist, the globals are not changed.
C
C     FILE REFERENCES	  ASCII parameter file  
C			  (Default extension is "txt")
C
C     PROGRAM HISTORY
C	PROGRAMMER	  DATE		REASON
C	----------	  ----		------
C	K. Zanter       Dec. 1986       PRF #3092		 
C	K. Zanter       March l987      A change was made in the parameter
C					file so that the file can be
C					transferred using NCP.
C	B. Ailts	Nov. 1988	LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	SETGBLS must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        After initialization, the name of the parameter file (PARFILE)
C     is retrieved.  The existence of this file is checked.  If it is
C     present on disk, the values are retrieved from the file and used
C     to set the global variables.  The XQ routines are then used to
C     pass out the updated globals.  If the file does not exist, the
C     values of the global variables are not changed. 
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program setgbls

	include 'las.fin'

	include 'lams.fin'

	integer*4	lsstat
	integer*4	meanz(MMXIMG)
	integer*4	mode
	integer*4	offl(MMXIMG)
	integer*4	offs(MMXIMG)

      	integer*4	parblk(XPRDIM)

	integer*4	status
	integer*4	stdout
	integer*4	xnl(MMXIMG)
	integer*4	xns(MMXIMG)

	character*(CMLEN)	pfile

C	initialization
C	--------------
	call inlas(parblk,stdout)

C	get the name of the parameter file
C       (Note:  did not want the error message (File doesn't exist)
C       to be printed, so that's why update mode was used
C	-----------------------------------------------------------
	mode = IUPDAT
	call getnam(parblk,'PARFILE',1,pfile,'txt',mode)

C	determine if the parameter file exists
C	--------------------------------------
	mode = 0
	status = lsstat(pfile,mode)
	
C	---------------------------------------
C	|if (the parameter file exists) then  |
C	|   get the values and set the globals|
C	|   pass out the updated globals      |
C	|endif				      |
C	---------------------------------------
	if (status .eq. ESUCC) then
	   call getpar(pfile,offl,offs,xnl,xns,meanz)
	   call gblout(parblk,offl,offs,xnl,xns,meanz)
	endif

C	terminate application (users do not want a completion message)
C       ---------------------
	call xzexit(1,' ')

	end

C	************************************************************************
C	   GETPAR gets the values from the parameter file and sets
C	   the appropriate globals.
C	************************************************************************
	subroutine getpar(pfile,offl,offs,xnl,xns,meanz)

	include 'las.fin'

	include 'lams.fin'

	integer*4	i
	integer*4	meanz(1)
	integer*4	offl(1)
	integer*4	offs(1)
	integer*4	status
	integer*4	xnl(1)
	integer*4	xns(1)

	character*(CMLEN)	pfile
	character*81	temp

C	open the parameter file
C	-----------------------

	open(unit=3, file=pfile, status='old', access='sequential',
     +	     form='formatted', iostat=status)

	if (status .ne. 0) then
	   call errmsg('Error opening parameter file','setgbls-open',1)
	   call errmsg('Fatal error encountered',
     +	    'setgbls-fatal',-1)
	endif

C	reposition back to the beginning of the file
C	--------------------------------------------
	rewind(unit=3)

C	get the $OFFL values
C	--------------------
	read (unit=3, fmt='(A)', iostat=status) temp
	if (temp .ne. ' $OFFL') then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif

	do 10 i = 1,MMXIMG
	   read (unit=3, fmt=*, iostat=status) offl(i)
	   if (status .ne. 0) then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif
10	continue

C	get the $OFFS values
C	--------------------
	read (unit=3, fmt='(A)', iostat=status) temp
	if (temp .ne. ' $OFFS') then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif

	do 20 i = 1,MMXIMG
	   read (unit=3, fmt=*, iostat=status) offs(i)
	   if (status .ne. 0) then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif
20	continue

C	get the $XNL values
C	-------------------
	read (unit=3, fmt='(A)', iostat=status) temp
	if (temp .ne. ' $XNL') then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif

	do 30 i = 1,MMXIMG
	   read (unit=3, fmt=*, iostat=status) xnl(i)
	   if (status .ne. 0) then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif
30	continue

C	get the $XNS values
C	-------------------
	read (unit=3, fmt='(A)', iostat=status) temp
	if (temp .ne. ' $XNS') then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif

	do 40 i = 1,MMXIMG
	   read (unit=3, fmt=*, iostat=status) xns(i)
	   if (status .ne. 0) then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif
40	continue

C	get the $MEANZ values
C	---------------------
	read (unit=3, fmt='(A)', iostat=status) temp
	if (temp .ne. ' $MEANZ') then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif

	do 50 i = 1,MMXIMG
	   read (unit=3, fmt=*, iostat=status) meanz(i)
	   if (status .ne. 0) then
	   close (unit = 3)
	   call errmsg('Error reading parameter file','setgbls-read',1)
	   call errmsg('Fatal error encountered','setgbls-fatal',-1)
	endif
50	continue

C	close the parameter file
C	------------------------
	close (unit=3)

	return
	end

C	************************************************************************
C	   GBLOUT uses the XQ calls to pass out the updated globals.
C	************************************************************************
	subroutine gblout(vblock,offl,offs,xnl,xns,meanz)

	include 'las.fin'

	include 'lams.fin'

	integer*4	meanz(1)
	integer*4	numel
	integer*4	offl(1)
	integer*4	offs(1)
	integer*4	status
	integer*4	vblock(1)
	integer*4	xnl(1)
	integer*4	xns(1)

	numel = MMXIMG
	call xqini(vblock,XPRDIM,XABORT)
	call xqintg(vblock,'$OFFL',numel,offl,XADD,status)
	call xqintg(vblock,'$OFFS',numel,offs,XADD,status)
	call xqintg(vblock,'$XNL',numel,xnl,XADD,status)
	call xqintg(vblock,'$XNS',numel,xns,XADD,status)
	call xqintg(vblock,'$MEANZ',numel,meanz,XADD,status)
	call xqout(vblock,status)

	return
	end
