/**************************************************************************
NAME:	TAB2MGR

PURPOSE:
To read a labeled table polygon file and write it to a IBIS graphics file.

PROGRAM HISTORY:
Version		Date	Author		Code/Conr	Request
-------		----	------		--------	-------
  1.0		4/87	B. Ailts 	  CSB		PRF 3427
  2.0		1/89	B. Ailts	EDC/CSB		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TAB2MGR must be run under TAE.

PROJECT:	LAS		

ALGORITHM DESCRIPTION:
The program begins by initializing TAE 
The user-specified parameters and the LAS global variables 
are then retrieved.

After reading the input parameters, INLT is opened and  
the polygon vertices are read in.  OUTFILE is then opened
and the polygon vertices are written to OUTFILE.
After all the polygon vertices are written, all the 
files are closed and the completion message is displayed.

ALGORITHM REFERENCES:	none

*******************************************************************************/

#include <stdio.h>

#include "las.h"
#include "util.h"
#include "ltable.h"
#include "lams.h"

main()

{ /* main */
struct	PARBLK	parb;

float	outbuf[MEMXPT];

long	npoints;
long	one = 1;
long	parmreg;
long	std;

char	taeout[CMLEN];
char	hosout[CMLEN];
char	hostin[CMLEN];
char	atrname[3][CMLEN];

/*  Initialize tae 
------------------*/
c_inlas (&parb,&std);

init(&parb,taeout,hostin,hosout,&parmreg,atrname);

readtab(hostin,outbuf,&npoints,&parmreg,atrname);

wrgraph(&parb,hosout,outbuf,&npoints);

c_complt("TAB2MGR",taeout,&one);

} /* main */

/****************************************************************************
*
*	INIT gets the TAE parameters
*
*****************************************************************************/

FUNCTION init(parb,taeout,hostin,hosout,parmreg,atrname)


struct	PARBLK	*parb;

long	*parmreg;

char	*taeout;
char	*hostin;
char	*hosout;
char	atrname[][CMLEN];


{  /* beginning of init*/

struct VARIABLE *val;

long	access;
long 	fatal = -1;
long	i;
long	status;
long 	zero = 0;

/*  INIT initialezes the descriptors and variables needed to retreive the
    input parameters.  These parameters are then retreived.
-------------------------------------------------------------------------*/

/*  Retreive the TAE paramters
------------------------------*/
access = IREAD;
status = c_getnam(parb,"INLT",&zero,hostin,"lt",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tab2mgr-fatal",&fatal);

access = IWRITE;
status = c_getnam(parb,"OUTFILE",&zero,hosout,"ibis_graph",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tab2mgr-fatal",&fatal);

/*  TAE name is calculated for the output file in order to be used for the
    completion message
--------------------------------------------------------------------------*/
strcpy(taeout, hosout);

val = p_find(parb,"POLYNUM");
*parmreg = IVAL(*val,0);

val = p_find(parb,"ATRNAME");
for (i = 0; i < 3; i++)
   strcpy(atrname[i],SVAL(*val,i));

return;

}  /*end of init*/

/*****************************************************************************
*
*	READTAB reads the labeled table polygon file
*
******************************************************************************/

FUNCTION readtab(hostin,outbuf,npoints,parmreg,atrname)

char	*hostin;
float	outbuf[];
long	*npoints;
long	*parmreg;
char	atrname[][CMLEN];

{  /*beginning of readtab*/

struct	TAB_DEF tab;
struct	TAB_DEF tabsub;
struct  VECTOR  *tvector;

long	cnt = 0;
long	i;
long	fatal = -1;
long    one = 1;
long	regind;
long 	region;
long	status;
long	xindex;
long	yindex;

short	regfound;
short	regsub;
short	xfound;
short	xsub;
short	ysub;
short	yfound;

char 	emsg[ERRLEN];

/*  READTAB opens and reads the labeled table for a cutline polygon.  The 
    vertices of that polygon are passed to WRGAPH in order to be written
    to the IBIS graphics file.
____________________________________________________________________________*/

/*  Open the input file
-----------------------*/
open_tab(&tab,hostin,IREAD);
get_vector(&tab);

if (tab.nsubs == 1)
   {
   open_tab(&tabsub,tab.subfile[0],IREAD);
   get_vector(&tabsub);
   }


regfound = FALSE;
yfound = FALSE;
xfound = FALSE;
tvector = tab.vector;
for (i = 0; i < tab.ncol; i++)
   {
   if (strcmp(tvector->label,atrname[0]) == 0)
      {
      regfound = TRUE;
      regind = i;
      regsub = FALSE;
      }
   if (strcmp(tvector->label,atrname[1]) == 0)
      {
      yfound = TRUE;
      yindex = i;
      ysub = FALSE;
      }
   if (strcmp(tvector->label,atrname[2]) == 0)
      {
      xfound = TRUE;
      xindex = i;
      xsub = FALSE;
      }
   tvector = tvector->next;
   }
if (tab.nsubs == 1)
   {
   tvector = tabsub.vector;
   for (i = 0; i < tabsub.ncol; i++)
      {
      if (strcmp(tvector->label,atrname[0]) == 0)
         {
         regfound = TRUE;
         regind = i;
         regsub = TRUE;
         }
      if (strcmp(tvector->label,atrname[1]) == 0)
         {
         yfound = TRUE;
         yindex = i;
         ysub = TRUE;
         }
      if (strcmp(tvector->label,atrname[2]) == 0)
         {
         xfound = TRUE;
         xindex = i;
         xsub = TRUE;
         }
      tvector = tvector->next;
      }
   }
if (!regfound)
   {
   sprintf(emsg,"LT attribute name %s was not found",atrname[0]);
   c_errmsg (emsg,"tab2mgr-attrib",&one);
   c_errmsg ("Fatal error encountered","tab2mgr-fatal",&fatal);
   }
if (!yfound)
   {
   sprintf(emsg,"LT attribute name %s was not found",atrname[1]);
   c_errmsg (emsg,"tab2mgr-attrib",&one);
   c_errmsg ("Fatal error encountered","tab2mgr-fatal",&fatal);
   }
if (!xfound)
   {
   sprintf(emsg,"LT attribute name %s was not found",atrname[2]);
   c_errmsg (emsg,"tab2mgr-attrib",&one);
   c_errmsg ("Fatal error encountered","tab2mgr-fatal",&fatal);
   }

/*  Read the x y coordinates of the input file
-------------------------------------*/
for (;;)
   {
   if ((!regsub) || (!xsub) || (!ysub))
      {
      status = get_record(&tab);
      if (status == E_EOF) break;
      else if (status != E_SUCC)
         {
         c_errmsg("Error reading input file","tab2mgr-read",&one);
         c_errmsg("Fatal error encountered","tab2mgr-fatal",&fatal);
         }
      }
   if ((regsub) || (xsub) || (ysub))
      {
      status = get_record(&tabsub);
      if (status == E_EOF) break;
      else if (status != E_SUCC)
         {
         c_errmsg("Error reading input file","tab2mgr-read",&one);
         c_errmsg("Fatal error encountered","tab2mgr-fatal",&fatal);
         }
      }
   if (!regsub)
      tvector = tab.vector;
   else
      tvector = tabsub.vector;
   for (i = 0; i < regind; i++)
      tvector = tvector->next;
   get_matrix(tvector,I4,&region);

   if (region == *parmreg)
      {
      if (!xsub)
         tvector = tab.vector;
      else
         tvector = tabsub.vector;
      for (i = 0; i < xindex; i++)
         tvector = tvector->next;
      get_matrix(tvector,R4,&outbuf[cnt + 1]);

      if (!ysub)
         tvector = tab.vector;
      else
         tvector = tabsub.vector;
      for (i = 0; i < yindex; i++)
         tvector = tvector->next;
      get_matrix(tvector,R4,&outbuf[cnt]);
      cnt += 2;
      }
   }

/*  Dupilcate the first point in order to close the polygon
-----------------------------------------------------------*/
if ((outbuf[cnt] != outbuf[0]) || (outbuf[cnt + 1] != outbuf[1]))
   {
   outbuf[cnt] = outbuf[0];
   outbuf[cnt + 1] = outbuf[1];
   *npoints = (cnt / 2) + 1;
   }
else
   *npoints = (cnt / 2);

/*  Close input file
--------------------*/
close_tab(&tab);

return;

}  /* end of readtab */

/**************************************************************************
*
*	WRGRAPH writes the IBIS graphics file
*
***************************************************************************/

FUNCTION wrgraph(parb,hosout,outbuf,npoints)

struct  PARBLK *parb;
char 	*hosout;
float	*outbuf;
long	*npoints;

{  /* beginning of wrgraph  */

float	*bufptr;

long	access;
long	i;
long	nl;
long	ns;
long	one = 1;
long	rec;
long	status;

int 	fdesc1;

/*  WRGRAPH opens an IBIS graphics file and writes the cutline vertices
    to it.
-----------------------------------------------------------------------*/

/*  initialization of variables  
-------------------------------*/
rec = 0;
bufptr = outbuf;

/*  calculate the size of the output file
-----------------------------------------*/
if (*npoints * 2 > MMXPPR)
   {
   nl = (*npoints * 2 + (MMXPPR - 1)) / MMXPPR;
   ns = MMXPPR;
   }
else
   {
   nl = 1;
   ns = *npoints * 2;
   }

/*  Open the output file
------------------------*/
access = 1;
status = c_gropen(&fdesc1,hosout,&ns,&nl,&access);
if (status != E_SUCC)
   {
   c_errmsg("Error opening output file","tab2mgr-open",&one);
   status = c_grclos(&fdesc1,hosout);
   c_clean(parb,"tab2mgr",hosout,&one);
   }

/*  Write the output records
----------------------------*/
for (i = 0 ; i < (*npoints * 2); i = i + ns,bufptr = bufptr + ns)
   {
   rec = rec + 1;
   status = c_grwrit(&fdesc1,&rec,bufptr);
   if (status != E_SUCC)
      {
      c_errmsg("Error writing to the output file","tab2mgr-write",&one);
      status = c_grclos(&fdesc1,hosout);
      c_clean(parb,"tab2mgr",hosout,&one);
      }
   } /* for i */

/*  Close the output file
-------------------------*/
status = c_grclos(&fdesc1,hosout);
if (status != E_SUCC)
   {
   c_errmsg("Nonfatal error encountered","tab2mgr-warn",&one);
   }

return;

}  /*  end of wrgraph */
