C     *************************************************************************
C     NAME 			TAB2MTB       	    		
C
C     PURPOSE 	   Generates an IBIS tabular file from a label table file
C		   that contains control point information.
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	B. Ailts  	     4/87	        Original
C	B. Ailts	     2/89		LAS 5.0 conversion
C	D. Etrheim           11/91		ADP #5499  Abort on invalid
C						attribute names. Also up case 
C						the attribute names.
C
C     PROJECT		        newLAS/LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  If necessary, the
C     defaulted parameters are set with the appropriate values.  
C     The input and output files are opened, and the output tabular
C     file is generated from the input label table file according to
C     the given specifications.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
      program tab2mtb

	include 'las.fin'

	include 'ltable.fin'

	include 'lams.fin'

C      	real*4 buffin(MMXCOL)
      	real*4 rowbuf(MMXCOL)

      	integer*4 acount
	integer*4 count
	integer*4 dtype(MMXCOL)
	integer*4 dtypesub(MMXCOL)
	integer*4 fd
	integer*4 fdsub
	integer*4 gblind(MMXCOL)
      	integer*4 getnam
      	integer*4 i
	integer*4 icolmx
      	integer   fdesc1
      	integer*4 il
      	integer*4 ik
	integer*4 j
	integer*4 length(MMXCOL)
      	integer*4 ncols
      	integer*4 ncount
	integer*4 nicol
      	integer*4 nrows
	integer*4 nsubs
	integer*4 nsubsub
	integer*4 ocols(MMXCOL)
      	integer*4 ocount

      	integer*4	parblk(XPRDIM)

	integer*4 rec(MMXCOL)
	integer*4 recsub(MMXCOL)
	integer*4 size(2,MMXCOL)
      	integer*4 status
      	integer*4 stdout
	integer*4 tbcol
	integer*4 tbcolsub
	integer*4 tbclos
	integer*4 tbcnt
	integer*4 tbindex(MMXCOL)
	integer*4 tbopen
	integer*4 tbscnt
	integer*4 tbsindex(MMXCOL)
	integer*4 tbwrow

      	character*(CMLEN) infile
      	character*(CMLEN) ofile
	character*(CMLEN) subfil
	character*(CMLEN) subfilsub
	character*(ERRLEN) errtxt
	character*(FLDLEN) desc(MMXCOL)
	character*(FLDLEN) fdesc
	character*(FLDLEN) ftype
	character*(FLDLEN) label(MMXCOL)
	character*(FLDLEN) labelsub(MMXCOL)
	character*100 atrname(MMXCOL)

	logical found(MMXCOL)

C       Place the large variables in a common block to work around a
C       recent change in the sgi compiler that causes core dumps when
C       large variables are declared on the stack
        common parblk

C     	Initialization of variables
C     	---------------------------
      	call inlas(parblk,stdout)

C     	Get input label table file name
C     	---------------------------------
      	status = getnam(parblk,'INLT',1,infile,'lt',IREAD)
      	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','tab2mtb-fatal',-1)
      	endif

C     	Get the output IBIS tabular file name
C     	-------------------------------------
      	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_tab',IWRITE)
      	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','tab2mtb-fatal',-1)
      	endif

C  Read the control point label table to calculate the number of points in 
C  the label table that are control points
C-------------------------------------------------------------------------
C  	Open the input label tabel file
C  	------------------------------------
      	call foptab(fd,infile,0,nsubs,subfil,ftype,fdesc)
	if (nsubs .gt. 1) then
	   call errmsg('Only one subfile is allowed','tab2mtb-sub',1)
           call errmsg('Fatal error encountered','tab2mtb-fatal',-1)
	endif
	call fgvec(fd,MMXCOL,tbcol,label,dtype,desc,size,rec)

C  	Read in the points
C	--------------------
	count = 0
      	do 70 il = 1,MMXROW
           call frdtab(fd,status)
	   if (status .eq. EEOF) then
	      go to 75
	   endif
	   count =count + 1

70	continue
        call errmsg('MMXROW encountered before EOF in INLT'
     +              ,'tab2mtb-nrows',1)
75	continue

C  Set nrows equal to the number of control points found 
C  -----------------------------------------------------
	call fcltab(fd)
      	nrows = count

C  Get the NCOLS, OUTCOL, and ELMNTNUM parameters
C  ----------------------------------------------
      	call xrstr(parblk,'ATRNAME',35,atrname,length,acount,status)
      	call xrintg(parblk,'OUTCOL',35,ocols,ocount,status)
      	call xrintg(parblk,'NCOLS',1,ncols,ncount,status)

C  If NCOLS was not specified (OUTCOL was specified)    
C  set "ncols" to the largest column number in OUTCOL
C  --------------------------------------------------
      	if (ncount .eq. 0) then
	   nicol = ocount
           ncols = 0
	   do 5 i = 1,ocount
 	      ncols = MAX(ncols,ocols(i))
5	   continue
        else

C  If both OUTCOL and NCOLS were specified	
C  make sure "ncols" is set with the largest value  
C  -----------------------------------------------
	   icolmx = 0
	   nicol = ocount
	   do 100 i = 1,ocount
	      icolmx = MAX(icolmx,ocols(i))
100	   continue
	   ncols = MAX(ncols,icolmx)
        endif

C  Check the no. of elements and the no. of columns
C  ------------------------------------------------
	if (acount .ne. ocount) then
	   call errmsg ('No. of attributes and columns are not equal',
     .			'tab2mtb-equal',1)
	   call errmsg('Fatal error encountered','tab2mtb-fatal',-1)
	end if

C  Open the input label tabel file
C  ------------------------------------
      	call foptab(fd,infile,0,nsubs,subfil,ftype,fdesc)
	call fgvec(fd,MMXCOL,tbcol,label,dtype,desc,size,rec)

	if (nsubs .eq. 1) then
      	   call foptab(fdsub,subfil,0,nsubsub,subfilsub,ftype,fdesc)
	   call fgvec(fdsub,MMXCOL,tbcolsub,labelsub,dtypesub,desc,
     .		   size,recsub)
	endif

C  Find the attribute names that were specified in ATRNAME
C  -------------------------------------------------------
	tbcnt = 0
	tbscnt = 0
	do 20 i = 1,acount
	   found(i) = .FALSE.
	   do 30 j = 1,tbcol
              call low2up(atrname(i),length(i))
	      if (label(j)(1:length(i)) .eq. 
     .		  atrname(i)(1:length(i))) then
	         tbcnt = tbcnt + 1
     	         found(i) = .TRUE.
	         tbindex(tbcnt) = j
	         gblind(i) = tbcnt
	      endif
30	   continue
20	continue

	if (nsubs .eq. 1) then

C          If the label is in both the master file and the subfile, use 
C          the subfile and therefore subtract 1 from tbcnt for every label 
C          that is in both the master and subfile.
C	   ------------------------------------------------------------
	   do 35 i = 1, tbcol
	      do 37 j = 1, tbcolsub
	         if (label(tbindex(i)) .eq. labelsub(j))
     .	            tbcnt = tbcnt - 1
37 	      continue
35	   continue
	   do 40 i = 1,acount
	      do 50 j = 1,tbcolsub
	         if (labelsub(j)(1:length(i)) .eq. 
     .		     atrname(i)(1:length(i))) then
	            tbscnt = tbscnt + 1
     	            found(i) = .TRUE.
	            tbsindex(tbscnt) = j
	            gblind(i) = tbscnt + tbcnt
	         endif
50	      continue
40	   continue
	endif
       do 44 i = 1,acount
	   if (.not. found(i)) then
	      errtxt = 'LT attribute name '//atrname(i)(1:length(i))//
     .			' was not found'
   	      call errmsg (errtxt,'tab2mtb-attrib',1)
              call errmsg('Fatal error encountered','tab2mtb-fatal',-1)
	   endif
44     continue

C  Open output IBIS tabular file
C  -----------------------------
      	status = tbopen(fdesc1,ofile,nrows,ncols,1)
      	if (status .ne. ESUCC) then
	   call errmsg('Error opening output file','tab2mtb-open',1)
      	   status = tbclos(fdesc1,infile)
	   call clean(parblk,'tab2mtb',ofile,1)
      	endif

C  Process input file and write to ibis file row by row
C  ----------------------------------------------------
      	do 80 il = 1,nrows

           call frdtab(fd,status)
	   if (status .ne. ESUCC) then
      	      status = tbclos(fdesc1,infile)
     	      call clean(parblk,'tab2mtb',ofile,1)
	   endif

	   i = 1
	   do 300 ik = 1,tbcnt
	      call retmat(rec,tbindex(ik),dtype,
     .			  rowbuf(gblind(i)),status)
	      if (status .ne. ESUCC) then
      	         status = tbclos(fdesc1,infile)
     	         call clean(parblk,'tab2mtb',ofile,1)
	      endif
	       i = i + 1
300	   continue

	   if (nsubs .eq. 1) then
              call frdtab(fdsub,status)
	      do 310 ik = 1,tbscnt
	         call retmat(recsub,tbsindex(ik),dtypesub,
     .			     rowbuf(gblind(i)),status)
	         if (status .ne. ESUCC) then
      	            status = tbclos(fdesc1,infile)
     	            call clean(parblk,'tab2mtb',ofile,1)
	          endif
	          i = i + 1
310	      continue
	   endif

C	     do 35 ik = 1, nicol
C		rowbuf(ik) = buffin(gblind(ik))
C35	     continue

             status = tbwrow(fdesc1,ocols,nicol,il,rowbuf)
	     if (status .ne. ESUCC) then
	        call errmsg('Error writing output file',
     .			    'tab2mtb-write',1)
      	        status = tbclos(fdesc1,infile)
	        call clean(parblk,'tab2mtb',ofile,1)
	     endif
	    
   80   continue

C     	Close the input and output files
C     	--------------------------------
      	status = tbclos(fdesc1,infile)
	call fcltab(fd)

C     	Output completion message
C     	-------------------------
      	call complt('TAB2MTB',ofile,1)

      	end

C*****************************************************************************
C
C	RETMAT retrieves the data matrix and converts the data type
C	if nessacary
C
C*****************************************************************************
	subroutine retmat(rec,index,dtype,buffin,status)

	include 'las.fin'

	include 'ltable.fin'

	real*8	temp8

	real*4	buffin

	integer*4 dtype(1)
	integer*4 index
	integer*4 output(2)
	integer*4 rec(1)
	integer*4 status
	integer*4 temp4

	integer*2 temp2

	character*1 temp1

	status = ESUCC
	if (dtype(index) .eq. I1) then
	   call fgmat(rec(index),output,temp1)
	   read (temp1(1:1),'(F3.0)') buffin
	else if (dtype(index) .eq. I2) then
	   call fgmat(rec(index),output,temp2)
	   buffin = real(temp2)
	else if (dtype(index) .eq. I4) then
	   call fgmat(rec(index),output,temp4)
	   buffin = real(temp4)
	else if (dtype(index) .eq. R4) then
	   call fgmat(rec(index),output,buffin)
	else if (dtype(index) .eq. R8) then
	   call fgmat(rec(index),output,temp8)
	   buffin = real(temp8)
	else
	   call errmsg('Dtype of label is invalid','tab2mtb-dtype',1)
	   status = EFAIL
	endif

	return
	end
