/******************************************************************************
NAME:   			GETPAR 

ALGORITHM:
   Retrieve the required parameters and globals from the TAE parameter block
   Parse the input image specification by calling GETIN which will validate 
    the window and band specifications  
   Checking to ensure that all the input images have equal window sizes  
   Retreive the output image name
   Retreive the output data type
   Retreive the global parameters
   Return
******************************************************************************/
#include "las.h"
#include "util.h"
#include "exbrows.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block            */
    char *hosin,         	/* host file names of input images 	     */
    char *ptype,			/* product type 		             */
    char *tapeid		/* tapeid				     */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long    access;
long    fatal = -1;		/* fatal error flag			     */
long	status;			/* function return status code               */
long    zero = 0;
char    temp[CMLEN];

char    fext[5];		/* data type buffer			     */

/* Get  product type 
----------------------*/
val = p_find(vblock,"PTYPE");
strcpy(ptype,SVAL(*val,0));
if (strcmp(ptype,"MSP") == 0)
   strcpy(fext,"mpt");
else if (strcmp(ptype,"MSA") == 0)
   strcpy(fext,"mat");
else if (strcmp(ptype,"TMR") == 0)
   strcpy(fext,"trt");
else if (strcmp(ptype,"TMA") == 0)
   strcpy(fext,"tat");
else
   strcpy(fext,"");

/* Get the file name 
--------------------*/
if (strcmp(ptype,"MSX") != 0)
   {
   access = IREAD;
   status = c_getnam (vblock, "INFILE", &zero, hosin, fext, &access);
   if (status != E_SUCC)
      c_errmsg ("Fatal error encountered", "exbrows-fatal", &fatal);
   }
else
   {
   val = p_find(vblock,"INFILE");
   strcpy(temp,SVAL(*val,0));
   c_cmhost(temp,hosin);
   }

/* Get tapeid
-----------*/
val = p_find(vblock,"TAPEID");
strcpy(tapeid,SVAL(*val,0));
return;
}
