/******************************************************************************
NAME:   			PROCES 

PURPOSE:    

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "typlim.h"
#include "exbrows.h"
#include "protos.h"

int proces
(
    char *hosin,			/* host file names of input images      */
    char *ptype,			/* product type                         */
    struct LINE *brw_line,		/* header record for output file     	*/
    char *tapeid			/* tapeid				*/
)
{
struct FDESC *fdesc[2];  /* Image file descriptors used for image I/O */
unsigned char *thebuf;   /* Image I/O buffer  */
long nsamps;             /* Number of samples */
long nlines;             /* Number of lines  */

/*
  Misc allocations
*/
long	k;			/* Looping variable			*/
long    loop;			/* loop counter				*/
long    band;			/* band loop counter			*/
long    len;			/* temporary length variable		*/
long	status;			/* function return status code          */
long	nonfatal = 1;		/* Non-fatal error flag			*/
long	bd;			/* Browse band number			*/
long	use_flag;		/* scene use flag			*/
struct stat fstat;		/* file status				*/


/*
  Ansi header tape information allocations
*/
FILE	*fp;			/* file pointer into the tape directory file */

long    block;			/* image blocking factor		*/
long    bignl;			/* number of lines			*/
long    nbands;			/* number of bands			*/
long    nscenes;		/* number of scenes			*/
long	offset;			/* flag for header type			*/ 

/*
  Database file information allocations
*/
FILE	*fp1;			/* file pointer	into the database file  */
char	infiles[3][CMLEN];	/* input file names			*/
char	outfile[CMLEN];		/* specific output name			*/
char	outname[60][CMLEN];	/* output name				*/
char	scene[61][17];		/* sene id				*/

long    sllist[61];		/* starting line list			*/
long	block_add[61];		/* Block address			*/
long    complete[61];		/* complete or partial scene flag	*/
long	position;		/* position in database file		*/
long	rectec;                 /* Recording technique                  */
long	satnum;                 /* Satellite number                     */

double	center[2];              /* Center latitude/longitude (degrees)  */

/*
  Image I/O data allocations
*/
struct  DDR ddr;		/* DDR structure			*/
struct  BDDR bddr;		/* BDDR structure			*/

long	access;			/* image I/O access type                */
long    opt[3]; 		/* option array for image I/O		*/
long	ss;			/* starting sample			*/
long    one=1;			/* value of one				*/
long    imgsiz;			/* image size 				*/
long    count;			/* histogram count value		*/

char    record[1033];		/* header record			*/
char    wbuf[10];		/* working buffer 			*/
char    syst[12];		/* system				*/
char 	disk[CMLEN];		/* temporary directory name		*/
char	msg[CMLEN];		/* error message			*/

unsigned char *ptr;		/* Place holder for image io address	 */

/*
  Histogram information
*/
long    histo[256];		/* histogram array			*/
long	mean;			/* average value			*/
long	minval;			/* minimum value			*/
long	maxval;			/* maximum value			*/
long	pnt0;			/* Value at  0.05 percent of image data */
long	pnt1;			/* Value at  2.5  percent of image data	*/
long	pnt2;			/* Value at 97.5  percent of image data	*/
long    zero = 0;		/* Pixel fill value of zero		*/
long	nfill = 256;		/* Number of pixels to be filled	*/
long	dtype = ELONG;		/* Data type of the fill data		*/
long    dummy = 0;		/* Dummy variable for pxhist call	*/

double	pc;			/* percentage value			*/
double	cum;			/* cummulative percentage		*/
double	sx;			/* cummulative sum			*/


/* Get the archive directory
--------------------------*/
strcpy(disk,getenv("LBINT"));

/* Open tape directory file
-------------------------*/
if ((fp = fopen(hosin,"r")) == NULL)
     {
     c_errmsg("Error opening tape directory file","exbrows-open",&nonfatal);
     return(E_FAIL);
     }

/* Read the directory header record 
---------------------------------*/
fgets(record,1032,fp);

/* The header record for MSP has two known formats where the HDT ID can
   be either 12 or 13 characters.  The other data types should be correct.
   First set the offset, zero is for old format, one is for new format. 
----------------------------------------------------------------------*/
if (strcmp(ptype,"MSP") == 0)
   offset = 0;
else
   offset = 1;

/* If the data type field does not match what we are expecting, check 
   to see if MSP header record is in the new format, otherwise, we have
   another unknown(?) header format or other serious problems. 
----------------------------------------------------------------------*/
strncpy(wbuf,&record[33+offset],3);
wbuf[3] = '\0';
if (strncmp(wbuf,ptype,3) != 0 )
   {
   if (strcmp(ptype,"MSP") == 0)
      {
      offset = 1;
      strncpy(wbuf,&record[33+offset],3);
      wbuf[3] = '\0';
      if (strncmp(wbuf,ptype,3) != 0 )
         {
         c_errmsg("Error parsing record","exbrows-prase",&nonfatal);
         return(E_FAIL);
         }
      }
   else
      {
      c_errmsg("Error parsing record","exbrows-prase",&nonfatal);
      return(E_FAIL);
      }
   }

/* Extract info for processing 
----------------------------*/
strncpy(wbuf,&record[37+offset],2);
wbuf[2] = '\0';
block = atol(wbuf);

strncpy(wbuf,&record[39+offset],5);
wbuf[5] = '\0';
nsamps = atol(wbuf);
if (strcmp(ptype,"MSA") == 0)
   {
   nsamps = 404;
   ss = 11;
   }
else
   ss = 1;

strncpy(wbuf,&record[44+offset],7);
wbuf[7] = '\0';
bignl = atol(wbuf);

strncpy(wbuf,&record[53+offset],1);
wbuf[1] = '\0';
nbands = atol(wbuf);

strncpy(wbuf,&record[62+offset],2);
wbuf[2] = '\0';
nscenes = atol(wbuf);

if (block != 0)
   {
   c_errmsg("Unable to handle blocking of image records","exbrows-fatal",
	    &nonfatal);
   return(E_FAIL);
   }


/*Create the DDR for the browse set 
---------------------------------*/
c_intddr(&ddr);
c_intbdr(&bddr);

ddr.nl = bignl;
ddr.ns = 512;
ddr.nbands = 1;
ddr.dtype = EBYTE;
if (c_getsys(syst) != E_SUCC)
    c_errmsg("Error returned from getsys", "eopenr-call",&nonfatal);
else
    strcpy(ddr.system,syst);

if (c_putddr(hosin,&ddr) != E_SUCC)
   {
   c_errmsg("Error creating DDR","exbrows-ddr",&nonfatal);
   return(E_FAIL);
   }

bddr.bandno = 1;
if (c_putbdr(hosin,&bddr) != E_SUCC)
   {
   c_errmsg("Error creating DDR","exbrows-ddr",&nonfatal);
   return(E_FAIL);
   }

/*
  Get information about each scene from the tape directory file
*/
getdata(fp,outname,sllist,hosin,infiles,&nscenes,ptype,complete,scene,
	block_add);

/*
  Close the tape directory file
*/
fclose(fp);


sllist[nscenes] = bignl+1;

opt[0] = opt[1] = opt[2] = 0;

fp1 = NULL;
/* Extract each scene 
-------------------*/
for (loop = 0; loop < nscenes; loop++)
    {
    /*
      Scene completeness for TMA actually refers to a partial and a 
      duplicate.  We will skip the scene completly
    */
    /*if ((complete[loop] == 1) && (strncmp(ptype,"TMA",3) == 0))
	{
	c_errmsg("Skipping Partial/Duplicate TMA browse scene",
                 "exbrows-tmr",&nonfatal);
	continue;
	}*/

    /*
      Check to be sure this is the scene to process
    */
    getname(scene[loop],tapeid,sllist[loop],&use_flag,&position,ptype,
	    &fp1,block_add[loop],&rectec,&satnum,center);

    /*
      Found a browse image to extract
    */
    if (use_flag == TRUE)
	{
	/*
	  Reset the third browse band to band six for TM night data
	*/
	len = (long) strlen(infiles[2]);
	if ((strncmp(ptype,"TM",2) == 0) && (rectec == 42))
	    {
	    infiles[0][len-1] = '6';
	    nbands = 1;
	    }
	else if (strncmp(ptype,"TM",2) == 0)
	    {
	    infiles[0][len-1] = '3';
	    nbands = 3;
	    }
	else
	    nbands = 3;

	/*
	  Get the number of lines in the browse image.  In some cases there
          are partial scenes or slivers between the starting lines defined
          on tape.  In this case be conservative!
	*/
        nlines = sllist[loop+1] - sllist[loop];
	if ((strncmp(ptype,"TM",2) == 0) && (nlines > 340))
	    nlines = 336;
	else if ((strncmp(ptype,"MSA",3) == 0) && (nlines > 400))
	    nlines = 400;
	else if ((strncmp(ptype,"MSP",3) == 0) && (nlines > 372))
	    nlines = 372;

 	sprintf(msg,"Number of lines to process: %d",nlines);
 	c_errmsg(msg,"exbrows-lines",&nonfatal);

	/*
	  Allocate space for the entire browse image band
	*/
	imgsiz = nsamps * nlines;
	thebuf = (unsigned char *) malloc(imgsiz);

	/*
	  Save the address of the image io buffer.  This is needed later
          because the jpeg compression function manipulates the image i/o
          buffer to get the job done.
	*/
	ptr = thebuf;

	/*
	  Process each of the bands for this scene
	*/
	for (band = 0; band < nbands; band++)
	    {
	    /*
	      Reset the image size counter for histogramming
	    */
	    count = imgsiz;

	    /*
	      Open the input file
	    */
            access = IREAD;
            status = c_eopenr(&fdesc[0],infiles[band],&one,&sllist[loop],
			      &ss,&nlines,&nsamps,&access,&ddr.dtype,opt,
			      &nlines);
            if (status != E_SUCC)
		return(E_FAIL);

	    /*
	      Append the browse band number to the output file name
	    */
	    len = (long) strlen(infiles[band]);
	    bd = atol(&infiles[band][len-1]);
	    sprintf(outfile,"%s/%s",disk,outname[loop]);
	    sprintf(wbuf,"_%d",bd);
   
	    /*
	      Create the output file name
	    */
	    if ((scene[loop][0] == '1') || (scene[loop][0] == '2') || 
		(scene[loop][0] == '3'))
		{
                if (wbuf[1] == '1')
		    strcat(outfile,"_4");
                else if (wbuf[1] == '2')
                    strcat(outfile,"_5");
                else if (wbuf[1] == '4')
                    strcat(outfile,"_7");
                }
	    else
	        strncat(outfile,wbuf,2);
       
	    /*
	      Initailize the histogram information
	    */
            c_pxfill(histo,&dtype,&nfill,&zero);
   
            /* read entire image into memory
            ------------------------------*/
            if (c_eread(&fdesc[0],&one,&one,thebuf,&nlines) != imgsiz)
		return(E_FAIL);
   
            /* histogram input image
            ----------------------*/
            c_pxhist(&imgsiz,thebuf,&ddr.dtype,histo,&dummy);
   
            count -= histo[0];
            if (strncmp(ptype,"TM",2) == 0)
		{
		count -= histo[255];
                histo[0] = histo[255] = 0;
                }
	    else
                {
                count -= histo[127];
                histo[0] = histo[127] = 0;
                }

            minval = MAXUTINY;
            maxval = MINUTINY;
            cum = 0.0;
            sx = 0.0;
            pnt0 = pnt1 = pnt2 = 0;

            for (k = 0; k <= 255; k++)
		{
                if (histo[k] > 0)
		    {
                    pc = 100.0 * histo[k]/count;
                    cum += pc;

                    if (cum <= 0.05)
                        pnt0 = k;
                    else if (cum <= 2.5)
                        pnt1 = k;
                    else if (cum <= 97.5)
                        pnt2 = k;
                      
                    if (k < minval)
                        minval = k;

                    if (k > maxval)
                        maxval = k;
      
		    sx += (k*histo[k]);
		    }
		}
   
	    /* If this band is zero filled do not process
            -------------------------------------------*/
            if (count != 0)
		{
		mean = (long)sx/count;

		/* Setup header
		-------------*/
		setup_head(brw_line,bd,nlines,nsamps,outname[loop],center,
			   rectec,satnum,minval,maxval,pnt0,pnt1,pnt2,mean);
   
		/* Write JPEG image
		-----------------*/
		write_JPEG_file(thebuf,outfile,80,brw_line,nlines,nsamps,1);

                /* Check file size as a safegaurd.  Some files have
                   been processed with the output file size being zero,
 		   possibly becuase /usr/tmp was full.  
                --------------------------------------------------------*/
                if (stat(outfile,&fstat) != 0)
                   {
                   c_errmsg("File status error","exbrows-stat",&nonfatal);
                   return (E_FAIL);
                   }

                if (fstat.st_size == 0)
                   {
                   c_errmsg("Invalid file size","exbrows-zero",&nonfatal);
                   return (E_FAIL);
                   }
		}
	    else
		{
		sprintf(msg,"Zero data in band %d of %s: Not Processed\n",
			bd,scene[loop]);
		c_errmsg(msg,"exbrows-zero",&nonfatal);
		}

	    /* Close input image
	    ------------------*/
	    c_eclose(&fdesc[0]);

	    /*
	      Reset the image io buffer address.  This gets changed in
	      the call to "write_JPEG_file" and it dosen't hurt if it
	      wasn't called.
	    */
	    thebuf = ptr;
	    }

	/*
	  Update the database record to indicate the record was processed
	*/
	if (updb(fp1,position,scene[loop],tapeid) != E_SUCC)
	    return(E_FAIL);

	/*
	  Free the image I/O buffer space allocated
	*/
	free(thebuf);
   	}
    }

/*
  Close the database file
*/
if (fp1 != NULL)
    fclose(fp1);

return(E_SUCC);
}
