/*******************************************************************************
FUNCTION:	setup_head

PURPOSE:	Set up the browse header record 

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    08/90  D.Hollaren       Modified version of VMSOUT
  5.0    03/92  E.Cihak/         LAS-5.0 version for ADAPS
                T.Rockvam

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "exbrows.h"
#include "fldtypes.h"
#include "protos.h"

void setup_head
(
    struct LINE *brw_line,	/* header for the output browse file 	*/
    long nbnd,			/* number of bands 			*/
    long outlines,		/* number of output lines		*/
    long outsamps,		/* number of output samples		*/
    char *sceneid,		/* sceneid of the LAS image 		*/
    double center[],		/* Center latitude/longitude of scene	*/
    int rectec,
    int sat,
    long minval,
    long maxval,		/* Min and Max values of image data	*/
    long pnt0,                   /* Value at  0.05 percent of image data */
    long pnt1,                   /* Value at  2.5  percent of image data */
    long pnt2,                   /* Value at 97.5  percent of image data */
    long mean			/* Average value of image data		*/
)
{
long	nwlat, nwlon,		/* lat/long of upper left corner 	*/
	nelat, nelon,		/* lat/long of upper right corner 	*/
	swlat, swlon,		/* lat/long of lower left corner 	*/
	selat, selon,		/* lat/long of lower right corner 	*/
	cnlat, cnlon;		/* lat/long of center of pass 		*/

long	clat_l,			/* Center latitude in DDMMSS		*/
	clon_l;			/* Center longitude in DDDMMSS		*/

char	bnd[10];		/* string of band numbers specified 	*/

double	corner_lat[4],		/* Calculated latitude corners		*/
	corner_lon[4];		/* Calculated longitude corners		*/
 
/*
  Calculate the corner coordinates from the center point
*/
clat_l = cnvlong(center[0],LAT_DDMMSS);
clon_l = cnvlong(center[1],LONG_DDDMMSS);
calcorn(clat_l,clon_l,rectec,corner_lat,corner_lon,sat);

/*
  Convert the latitudes and longitudes to total minutes
*/
deg2min(center[0],&cnlat);
deg2min(center[1],&cnlon);

deg2min(corner_lat[0],&nwlat);
deg2min(corner_lon[0],&nwlon);

deg2min(corner_lat[1],&swlat);
deg2min(corner_lon[1],&swlon);

deg2min(corner_lat[2],&selat);
deg2min(corner_lon[2],&selon);

deg2min(corner_lat[3],&nelat);
deg2min(corner_lon[3],&nelon);

/*
  Write out bands specified
*/
sprintf (bnd, "%d", nbnd); 
bnd[1] = '\0';

/*
  Fill in header version
*/
brw_line->version = 0X04;

/*
  Write remaining header fields
*/
sprintf(brw_line->header,
        "%30s%3d%3d%4d%4d%-9s%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d%3d%3d%3d%3d",
        sceneid,minval,maxval,outlines,outsamps,bnd,
        nwlat,nwlon,nelat,nelon,selat,selon,swlat,swlon,cnlat,cnlon,
        pnt0,pnt1,mean,pnt2);

return;
}
