/* System include files
=======================*/
#include <ctype.h>              /* Needed for call to isspace on some systems */
#include <math.h>               /* Needed for call to ceil                    */
#include <stdio.h>              /* Needed for standard i/o                    */
#ifdef vms
#include <file.h>               /* Needed for calls to cmlock                 */
#else
#include <fcntl.h>              /* Needed for calls to cmlock                 */
#ifndef gould
#include <unistd.h>             /* Needed for calls to getcwd                 */
#endif
#endif


/* Special defines for the Gould PowerNode and Data Generals
==============================================================================*/
#ifdef max                      /* To avoid redefine warnings                 */
#undef max
#endif
#ifdef MAX                      /* To avoid redefine warnings                 */
#undef MAX
#endif
#ifdef min                      /* To avoid redefine warnings                 */
#undef min
#endif
#ifdef MIN                      /* To avoid redefine warnings                 */
#undef MIN
#endif
#ifndef SEEK_END                /* Not defined on the Gould PowerNode         */
#define SEEK_END 2
#endif



/* Constants
==============================================================================*/
#define ANSI_LABEL_BYTES   640  /* Total number of ANSI label bytes per image */
#define ANSI_NAME_LENGTH    17  /* Number of characters in an ANSI file name  */
#define ANSI_REC_SIZE       80  /* Size of an ANSI record                     */
#define ANSI_VOL_LENGTH      6  /* Number of characters in an ANSI volume ID  */
#define BLOCK_SIZE       27592  /* Number of bytes in an archive block        */
#define BUFF_SZ             11  /* Number of strings in the tape label buffer */
#define IMAGE_WAIT         120  /* Time to wait between image searches        */
#define IMAGES_PER_TAPE    200  /* Max # of images to stack on a single tape  */
#define MAX_IMAGES          20  /* Max # of images to work with at one time   */
#define MAX_SCENES          60  /* Max # of images for a shipping form page   */
#define NCOLS              3.0  /* # columns for shipping forms (must be real)*/
#define NCOPIES              2  /* # of shipping forms to print               */
#define NTAPE_MARKS          6  /* # of tape marks used for a single image    */
#define NUM_TRIES            5  /* Max # of attempts to mount a tape          */
#define RECORD_SIZE      13796  /* Number of bytes in an archive record       */
#define TAPE_WAIT           60  /* Seconds to wait between tape mounts        */
#define TMARK_3480         152  /* Kbytes used for tape marks on 3480         */
#define TMARK_4MM         2048  /* Kbytes used for tape marks on 4mm          */
#define TMARK_8MM_HD      4096  /* Kbytes used for tape marks on 8mm HD       */
#define TMARK_8MM_LD      2048  /* Kbytes used for tape marks on 8mm LD       */
#define TMARK_9TRACK        25  /* Kbytes used for tape marks on 9-track      */


/* String constants
==============================================================================*/
#define ARCH_HEADER "ahdr"      /* Extension for EDC archive headers          */
#define ARCH_IMAGE  "arch"      /* Extension for EDC archive images           */
#define OPTION_ESA  "ESA"       /* Designation for European Space Agency      */
#define OPTION_SFL  "SFL"       /* Designation for Sioux Falls, SD            */
#define PRINTER     "ops-n"     /* Printer used for shipping forms            */


/* Return status values.  E_SUCC = 0   E_FAIL = -1
==============================================================================*/
#define FULL                -2  /* The current tape is full                   */
#define MOUNT_FAIL          -3  /* Tape was not successfully mounted          */
#define NO_FORM             -4  /* No shipping form for specified option      */
#define NON_ANSI            -5  /* Mounted tape is not ANSI initialized       */
#define RENAME_ERR          -6  /* Error renaming files                       */
#define STOP                -7  /* Manual has been encountered                */


/* System return status
==============================================================================*/
#ifdef vms
#define SYS_SUCC      1         /* VMS  successful exit status                */
#else
#define SYS_SUCC      0         /* UNIX successful exit status                */
#endif
