/*******************************************************************************
FUNCTION:	stoptpacq

PURPOSE:	Stop the AUTOSTACK function at the next convenient time


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    07/92  J.Fenno          Development
  1.2    01/93  J.Fenno          Modified to put the tape drive semaphores in
				 ADAPSTABLES instead of ADAPSARC.  This will
				 provide a common location for systems running
				 AUTOSTACK under ADAPS and Global 1k.


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "stoptpacq.h"
#include "protos.h"


struct PARBLK vblock;        /* TAE parameter block                           */
long std_out;                /* standard output file                          */


int main
(void)
{
char drive[CMLEN];         /* Tape drive being stacked by AUTOSTACK           */
char file_names[1][CMLEN]; /* For call to c_complt                            */
char *ptr;                 /* String pointer                                  */
char string[CMLEN];        /* Temporary string                                */
char tabledir[CMLEN];      /* ADAPSTABLES directory                           */

long fatal    = -1;        /* Fatal error flag                                */
long i;                    /* For call to c_complt                            */
long nonfatal =  1;        /* Nonfatal error flag                             */

struct VARIABLE *p_find(); /* Function to find a TAE variable                 */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Get the environment or logical variable that points to the
   directory containing all of the archive images to be stacked
---------------------------------------------------------------*/
ptr = (char *)getenv("LBTABLES");
if (ptr == NULL)
    {
    c_errmsg("Error retrieving LBTABLES", "stopstack-envron", &nonfatal);
    c_errmsg("Fatal error encountered", "stopstack-fatal", &fatal);
    }
strcpy(tabledir, ptr);
strcat(tabledir, "/");


strcpy(drive,"0");
/* Stop the specified tpacq function
-----------------------------------*/
if (create_semaphore(tabledir, "tpacq", drive, "stop") == E_SUCC)
    {
    sprintf(string, "TPACQ_LB has been signaled to stop");
    c_errmsg(string, " ", &nonfatal);
    }
else
    {
    sprintf(string, "TPACQ_LB is not currently running");
    c_errmsg(string, "stoptpacq-norun", &nonfatal);
    c_errmsg("Fatal error encountered", "stoptpacq-fatal", &fatal);
    }

i = 0;
c_complt("stoptpacq", file_names, &i);

return EXIT_SUCCESS;

}
