/*        jw.h 1/95 CSI        */
#ident "jw.h: $Id: jw.h,v 1.2 2001/02/21 19:58:09 gacke Exp $"

#define JWVERSION "1.0"
#define CSICOPYRIGHT "Copyright, 1996  Cymbolic Sciences International."

#define CONFIG_FILE_NAME "jwconfig.dat"

#define NO_OF_DEFINED_ROWS 10
#define NO_OF_ROW_PARMS 6
#define ROWPAR_ENTRY 0
#define ROWPAR_POSITION 1
#define ROWPAR_HEIGHT 2
#define ROWPAR_TOPGAP 3
#define ROWPAR_BOTGAP 4
#define ROWPAR_JUSTIFICATION 5

#define NO_OF_TOP_ROWS 5
#define ROW_TOPCOORD 4
#define ROW_BOTTOMCOORD 5
#define ROW_SCALE 6
#define ROW_0_NAME "Top Annotation Row 1"
#define ROW_1_NAME "Top Annotation Row 2"
#define ROW_2_NAME "Top Annotation Row 3"
#define ROW_3_NAME "Top Annotation Row 4"
#define ROW_4_NAME "Top Coordinate Row"
#define ROW_5_NAME "Bottom Coordinate Row"
#define ROW_6_NAME "Scale"
#define ROW_7_NAME "Bottom Annotation Row 1"
#define ROW_8_NAME "Bottom Annotation Row 2"
#define ROW_9_NAME "Bottom Annotation Row 3"

#define COORD_TEXT0 "Left Coordinate"
#define COORD_TEXT1 "Right Coordinate"
#define NO_OF_COORDINATES 2

#define ENTRY_ALWAYS 0
#define ENTRY_NEVER 1
#define ENTRY_PROMPT 2

#define JUST_LEFT 0
#define JUST_RIGHT 1
#define JUST_CENTER 2
#define JUST_IMLEFT 3
#define JUST_IMRIGHT 4

/* the next 4 defines are a set; they reflect the characteristics of 
 * the font bitmap file, "fr_font.bmp".  This file has bitmapped cells
 * of 24x14 for each of the 256 characters.
 */
#define ANN_HEIGHT 24
#define ANN_WIDTH 14
#define FONT_HIGHBIT 0x8000
#define FONT_LOWBIT 0x4     /* if not 0, says that there are throwaway bits */

#define ANN_SIZE 256
#define ANN_VALUE 0xFF

/* === LJ5000 change === */
#if (MODEL == 2000)
#  define MAX_ANN_CHARS (MAXPIXPERLINE11 / (ANN_WIDTH * 2))
#elif (MODEL == 5000)
#  define MAX_ANN_CHARS (MAXPIXPERLINE50 / (ANN_WIDTH * 2))
#endif

#define FONT_FILE_NAME "fr_font.bmp"
#define ANN_FILE_NAME "fr_temp "

/* scale parameters */

/* the next set of parameters can interact */
#define SCALE_MINMM_PERTICK5 5.0
#define SCALE_MAXMM_PERTICK5 13.0
#define SCALE_MINMM_PERTICK8 8.0
#define SCALE_MAXMM_PERTICK8 20.0
#define SCALE_MINMM_PERTICK11 11.0
#define SCALE_MAXMM_PERTICK11 28.0

/* === LJ5000 change === */
#define SCALE_MINMM_PERTICK50 8.0
#define SCALE_MAXMM_PERTICK50 20.0

#define SCALE_CHARSIZE5  1	/* size of scale text, in mm */
#define SCALE_CHARSIZE8  2
#define SCALE_CHARSIZE11  3

/* === LJ5000 change === */
#define SCALE_CHARSIZE50  2

#define SCALE_MAXRANGE  2499    /* km per tick */

/* the next set of parameters can interact */
#define SCALE_HEIGHT    10      /* in lines, not including magnification */
#define SCALE_SPACE     10      /* blank lines between scale sections */
#define SCALE_MAXCHARS  200     /* 1 mm chars on the mile scale */
/* total # of lines in (scale) temp file */
#define SCALE_LINES (2*SCALE_HEIGHT + 2*ANN_HEIGHT + SCALE_SPACE)

#define SCALE_EXTRACHARS 8      /* e.g. "miles", "km" (+ spaces ) */
#define SCALE_KMTICKS   5
#define SCALE_MILETICKS 4
#define SCALET_MILES    0
#define SCALET_KM       1
#define KM_PER_MILE     1.609347

#define MAX_RESOLUTION  1000.0  /* in meters */
#define MIN_RESOLUTION  1.0
#define DEF_RESOLUTION  25.0

/* Miscellaneous */
#define NO_OF_COLORS 3
#define MIN_PIX			0	/* pixel values */
#define MAX_PIX			255
#define DEF_PIX_PER_LINE	8000

#define COMMENTCHAR '!'

#define STOPFLAG_READ 0
#define STOPFLAG_SET 1
#define STOPFLAG_CLEAR 2

/* Configuration file */

#define CFNUMBERS 5
#define CF_FILM 0
#define CF_TURRET 1
#define CF_SPEED 2
#define CF_WEDGESIZE 3
#define CF_SKIP_PIX 4
#define FLM_L 0

/* === LJ5000 change === */
#define FLM_H 7            /* new media sizes: 50, 40, 30 wide */

#define TUR_L 0
#define TUR_H 3
#define SPC_L 1
#define SPC_H 6
#define WED_L 0
#define WED_H INT_MAX 
#define SKP_L 0 
#define SKP_H INT_MAX

#define CFSTRINGS 3
#define CF_CHECK_IMFILE 0
#define CF_DEFLUT 1
#define CF_DEVNAME 2

typedef struct {
  int annotation_row[NO_OF_DEFINED_ROWS][NO_OF_ROW_PARMS];
  int intpar[CFNUMBERS];
  char strpar[CFSTRINGS][80];
} image_config_t;


/* structure of image parameters */

#define MAX_IMAGES  20
#define MAX_COPIES  100

struct row_params {
    int class;          /* row characteristics */
    int top_gap;        /* # mm of gap above */
    int bottom_gap;     /* # mm of gap below */
    int height;         /* height in mm */
    int valid;          /* flag indicating valid field to image */
    char text[MAX_ANN_CHARS];
};

struct lj2000_image_param {
    float resolution;   /* meters per pixel */
    float spacing;      /* pixel size in microns */
    int copies;
    int readlen;        /* pixels per line (from the disk) */
	int writelen;		/* pixels per line (to the FIRE) */
	int lines;			/* lines (from the disk) */
    int x_offset;       /* offset to 1st pixel from left of image */
    int numfiles;       /* number of image files (1 or 3) */
	int color;			/* image data is black/white [0] color [1] */
	int image_wedge;	/* whether to expose a wedge */
	int suppress_adv;	/* whether to suppress fllm advance after image */
    struct row_params rowpar[NO_OF_DEFINED_ROWS];  /* annotation info */
    struct lj2000_config_tbl_struct ctable; /* configuration table */
	char lut_file[80];	/* name of file to be used for LUT */
    char image_file[NO_OF_COLORS][80];  /* names of image files */
};

#define IMPAR IP[Inum] /* macro for 'image paramaters for current image' */
#define IMPARP &IP[Inum] 
	/* macro for 'pointer to image paramaters for current image' */
