/* Defines for LJ2000 I/O */
#ident "lj2000.h: $Id: lj2000.h,v 1.2 2001/02/21 19:58:09 gacke Exp $"

#ifdef SGI
typedef unsigned short UInt16;
typedef unsigned char UChar;
#endif

/* Inquiry */

/* === LJ5000 change === */
#if (MODEL == 2000)
#  define LJ_inqstring "LightJet 2000"
#elif (MODEL == 5000)
#  define LJ_inqstring "LightJet 5000"
#endif

#define LJ2000_inqstrloc 16

/* Request Sense */

#define LJ2000_SENSE_BYTES 32
#define LJ2000_RQS_FILM_LOADED 0
#define LJ2000_RQS_FILM_SIZE 1
#define LJ2000_RQS_INTERLOCK_STATUS 2
#define LJ2000_RQS_XPORT_STATUS 3
#define LJ2000_RQS_XSCAN_STATUS 4
#define LJ2000_RQS_XSCAN_HOME 5
#define LJ2000_RQS_SPINNER_STATUS 6
#define LJ2000_RQS_POWERUP_INIT 7
#define LJ2000_RQS_DIAGCMD_STATUS 8
#define LJ2000_RQS_OPTICAL_MOTORS 12
#define LJ2000_RQS_PHOTON_TYPE 13
#define LJ2000_RQS_PHOTON_SYSTEM 14
#define LJ2000_RQS_POWER_SUPPLY 19
#define LJ2000_RQS_PROCESSOR 20
#define LJ2000_RQS_MEMORY 21
#define LJ2000_RQS_CONTROL_CIRCUIT 22
#define LJ2000_RQS_SCSI_INTERFACE 23
#define LJ2000_RQS_INTERPOLATION_CIRCUIT 24
#define LJ2000_RQS_INTERPOLATION_FSTAT 25
#define LJ2000_RQS_IMBUFF_COUNT_MSB 26
#define LJ2000_RQS_IMBUFF_COUNT_LSB 27
#define LJ2000_RQS_COLOR_XFORM_FSTAT 28
#define LJ2000_RQS_SHARPNESS_FSTAT 29

/* Sense Key format */
#define LJ2000_SENSEKEY_HARDWARE_ERR   0x4
#define LJ2000_SENSEKEY_OFFLINE        0x2
#define LJ2000_SENSEKEY_READY      0x0
#define LJ2000_SENSEKEY_INVALID_COMMAND  0x5
#define LJ2000_SENSEKEY_OTHER_CONDITIONS 0x9
      
struct lj2000_sense_struct {
    UChar errorCode;  /* byte 0: Error class and error code */
    UChar t1;         /* byte 1: Value 0 */
    UChar senseKey;       /* byte 2: Sense key */
    UChar t2[4];      /* byte 3-6: Value 0 */
    UChar addtionalLen;   /* byte 7: Additional length (16 bytes) */
    UChar senseData[LJ2000_SENSE_BYTES];   /* byte 8-39: Sense data */
}; 

/* Look Up Table */

#define LUT_ENTRIES 256
#define NO_OF_LUTS 3
#define LUTHEAD_ID 0x0
#define LUTHEAD_ID_S 5
#define LUTHEAD_ID_STR "ICT00"
#define LUTHEAD_NUMCOL 0x6
#define LUTHEAD_NUMCOL_STR '3'
#define LUTHEAD_POINTTOSTART 0x7
#define LUTHEAD_POINTTOSTART_S 3
#define LUTHEAD_POINTTOSTART_STR "600"
#define LUTHEAD_CHECKSUM 0xA
#define LUTHEAD_CHECKSUM_S 4
#define LUTHEAD_BYTES 0x10
#define LUTHEAD_BYTES_S 7
#define LUTHEAD_BYTES_STR "000E526"
#define LUTHEAD_COMMENT 0x17
#define LUTHEAD_COMMENT_S 80
#define LUTHEAD_S 0x600
#define LUT_S 4096
#define PRELUT_S 256
#define THREEDLUT_S 4913
#define LUT3D_CYCLE 17
#define LUT3D_MAX 4095
#define PRE_LUT_MAX3_VAL 4095
#define POSTLUT_PER_ENTRY 16
#define BYTE_MAX 255
#define LUTWIDTH 12
#define MAXLUTWIDTH 12
#define LUT_TIMEOUT 4
#define SCSI_XFR_MAX 65536
#define SCSI_XFR_NEARMAX 60000

struct lj2000_lut_struct {
	UChar header[LUTHEAD_S];
	UInt16 prelut[NO_OF_LUTS][PRELUT_S];
	UInt16 kprelut[NO_OF_LUTS][PRELUT_S];
	UInt16 threedlut[NO_OF_LUTS][THREEDLUT_S];
	UInt16 lut[NO_OF_LUTS][LUT_S];
    };

/* Configuration Table constants */
#define MINREPL 1
#define MAXREPL 16

/* === LJ5000 change === */
#if (MODEL == 2000)
#  define MINSPACING 12.5
#  define MAXSPACING 25.0
#elif (MODEL == 5000)
#  define MINSPACING 62.5    /* res 16 converted to microns */
#  define MAXSPACING 83.3    /* res 12 converted to microns */
#endif

#define MAXPIXPERLINE11 26800
#define MAXLINEPERIMG11 20800
#define MAXPIXPERLINE8 18720
#define MAXLINEPERIMG8 14800
#define MAXPIXPERLINE5 12560
#define MAXLINEPERIMG5 8720

/* === LJ5000 change === */
#define MAXPIXPERLINE50 20720  /* at res 16 */
#define MAXLINEPERIMG50 20720
#define MAXLINEPERIMG40 16640
#define MAXLINEPERIMG30 12640

#define MINEXP -99           /* was -50 to 100 */
#define MAXEXP 2000

#define CT_BILMODE 0x05
/* #define CT_BWMODE 0x85
#define CT_RGBSEQ 0x1B
#define CT_BGRSEQ 0x93 */

/* === LJ5000 change === */
#define CT_PIXEL_INTERLEAVE 0x01 /* LightJet 5000 does not support line interleave */

/* Configuration Table construct */
struct lj2000_config_tbl_struct {
     UChar  colorPalette;
     UChar  imageView;
     UChar  xReplication;
     UChar  yReplication;
     UChar  xSize;
     UChar  ySize;
     UInt16 intx;
     UInt16 fracx;
     UInt16 inty;
     UInt16 fracy;
     UInt16 pixelPerLine;
     UInt16 linePerImage;
     UChar  ICT_control;    
     UChar  RIControl;
/* from here on, table is byte aligned (yikes) */
     UChar  spotsize;
     UChar redExp0;
     UChar redExp1;
     UChar grnExp0;
     UChar grnExp1;
     UChar bluExp0;
     UChar bluExp1;
     UChar colSeq;
     UChar reserved0;
     UChar media_turret;
/* === LJ5000 change === */
     UChar simMode;
     UChar bidirectional;
};

/* Command descriptions for lj2000_action() */
typedef struct {
	const char *desc;				/* descriptor */
	const ulong timeout;			/* timeout length */
	const UChar xfrtype;	/* transfer type code */
} lj2000cmd_t;

/* Some handy variable declarations */
extern lj2000cmd_t LJ2000cmd[];
extern const char *LJ2000action_d[];
extern const char *LJ2000sensekey_d[];
extern const char *LJ2000_rqs_d[][8];

/* Macros for accessing command information */
#define XFRLJ(n) LJ2000cmd[n].xfrtype
#define DSCLJ(n) LJ2000cmd[n].desc
#define TIMLJ(n) LJ2000cmd[n].timeout

/* User Driver Commands */
#define LJ2000CMD_RESET_XSCAN      0
#define LJ2000CMD_LOAD_FILM         1  /* nop in LJ5000 */
#define LJ2000CMD_UNLOAD_FILM       2
#define LJ2000CMD_READ_CONFIG      3
#define LJ2000CMD_WRITE_CONFIG     4
#define LJ2000CMD_READ_STATUS      5
#define LJ2000CMD_WRITE_LUT        6
#define LJ2000CMD_WRITE_ICT        7
#define LJ2000CMD_WRITE_SHARPTAB   8
#define LJ2000CMD_SET_PLOTSPEED    9
#define LJ2000CMD_WRITE_IMAGE_DATA 10
#define LJ2000CMD_TEST_READY       11
#define LJ2000CMD_STOP_IMAGE       12
#define LJ2000CMD_CAL_STRIP	   13
#define LJ2000CMD_INQUIRY          14
#define LJ2000CMD_RETURN_STATUS    15
#define LJ2000CMD_PLOT_LINSTEPS    16
#define LJ2000CMD_ERROR_RESET      17


/* Transfer Types */
#define LJ2000_XFR_WIMG        0x24
#define LJ2000_XFR_STOPIMG     0x81
#define LJ2000_XFR_ERESET      0xE0
#define LJ2000_XFR_LFILM       0xE1
#define LJ2000_XFR_UFILM       0xE2
#define LJ2000_XFR_PLOTSPEED   0xE3
#define LJ2000_XFR_CONF        0xF0
#define LJ2000_XFR_LUT         0xF1
#define LJ2000_XFR_HOME        0xF3
#define LJ2000_XFR_CALSTRIP    0xF4
#define LJ2000_XFR_ICT         0xF5
#define LJ2000_XFR_ST          0xF6
#define LJ2000_XFR_LINSTEPS    0xF7

/* Timeout Values (in milliseconds ) */

#define LJ2000_SCAN_TIMO 120000	/* Scanner to Home */
#define LJ2000_UFILM_TIMO 30000	/* Unload Film */

#define LJ2000_LFILM_TIMO 30000	/* Load Film */
#define LJ2000_CONF_TIMO 10000  /* Read/write Configuration */ 
#define LJ2000_REQS_TIMO 100		/* Request Sense */
#define LJ2000_WLUT_TIMO 10000	/* Write Look Up Table */
#define LJ2000_WICT_TIMO 60000	/* Write ICT */
#define LJ2000_WSHT_TIMO 1000	/* Write Sharpness Table */
#define LJ2000_WIMG_TIMO 60000	/* Write Image Data */
#define LJ2000_PSPEED_TIMO 100	/* Set Plotter Speed ?? */
#define LJ2000_TSTU_TIMO 1000		/* Test Unit Ready */
#define LJ2000_STIM_TIMO 100		/* Stop Image */
#define LJ2000_CALS_TIMO 20000   /* Generate Cal Strip */
#define LJ2000_LINSTEP_TIMO 600000   /* Generate Linsteps */
#define LJ2000_ERESET_TIMO 100   /* Error Reset ?? */
#define LJ2000_INQY_TIMO 100		/* Inquiry */

/* Descriptor Block Lengths */
#define LJ2000_SENSE_LEN 0x28
#define LJ2000_CONFIG_LEN 32
#define LJ2000_LUT_LEN 0xE526
#define LJ2000_INQ_LEN 36

/* Poll values for reset/film */
#define RESET_POLLTIME 2
#define RESET_TIMEOUT 120
#define FILMLOAD_POLLTIME 2
#define FILMLOAD_TIMEOUT 40

/* === LJ5000 change === */
#define MEDIA_ADVANCE_TIMEOUT 120


/* Prototypes for lightjet action calls */

extern int 
hiloget(UChar *),
lightjet_rset_xscan(void),
lightjet_load_film(void),
lightjet_unload_film(void),
lightjet_read_config(struct lj2000_config_tbl_struct *),
lightjet_set_plotspeed(int),
lightjet_write_config(struct lj2000_config_tbl_struct *),
lightjet_write_lut(struct lj2000_lut_struct *, char *),
lightjet_send_data(char *, int),
lightjet_stop_image(void),
lightjet_inquiry(char *),
lightjet_plot_linsteps(void),
lightjet_cal_strip(void),
lightjet_error_reset(void),
lightjet_test_ready(void),
lightjet_get_status(struct lj2000_sense_struct *);

extern void
hiloassign(UChar *, int),
display_config(struct lj2000_config_tbl_struct *),
display_status(struct lj2000_sense_struct);

/* Prototypes for user driver calls */
extern int lj2000_action(int, char *, int);
extern int lj2000_open(char *);
extern void lj2000_close(void);

/* Return codes for lj2000_action().  0 is returned if there was no error */
#define LJ2000AR_OERR 1 /* device not open error */
#define LJ2000AR_CERR 2 /* SCSI command error */
#define LJ2000AR_AERR 3 /* SCSI Check Condition */
#define LJ2000AR_SERR 4 /* Problem flagged by SCSI status phase */
#define LJ2000AR_TERR 5 /* SCSI Timeout error */

/* Return codes for lj2000_open().  0 is returned if there was no error */
#define LJ2000OR_DERR 1 /* Device error */
#define LJ2000OR_SERR 2 /* System error */

#if     !defined(TRUE) || ((TRUE) != 1)
#define TRUE    (1)
#endif
#if     !defined(FALSE) || ((FALSE) != 0)
#define FALSE   (0)
#endif
#if     !defined(OK) || ((OK) != 0)
#define OK      (0)
#endif
#if     !defined(NOTOK) || ((NOTOK) != 0)
#define NOTOK      (-1)
#endif
#define ABORT       -1
#define err_prnt_msg(str)  fprintf(stderr,"ERROR: %s\n",str)
