#ifndef LJSTRUCT_H
#define LJSTRUCT_H

/*
	LightJet 5000 structure definitions
*/

/* #define DEBUG */

#ifdef sun
#ifdef sparc
#define sun4
#endif
#ifdef mc68010
#define sun2
#endif
#ifdef mc68000
#define sun2
#define sun3
#endif
#endif

#define CFHOST     "edcsgw36"
#define CRF_FIL    "lj.req"
#define CRFLCK_FIL "lj.lock"

#define OPGRP "rpops" 

#define NULLVAL -1

#define ABORT_EXIT   1  	/* On interrupt exit program */
#define ABORT_IGNORE 2  	/* Ignore interrupt and continue exectution */

#define	TO_REC	1		/* copy lut from tmp to record structure */
#define	FR_REC	2		/* copy lut from record structure to tmp */

#define PADD	1		/* Program Add Request */
#define	PMOD	2		/* Program Modify Request */
#define	PREP	3		/* Program Report Request */

#define NDAYS	14		/* Number of days before deleting requests */

#define CF_QUEUE	"QUEUE"	/* Request status is in queue */
#define CF_RUN		"RUN"	/* Request status is in process of printing */
#define CF_NET		"NET"	/* Request status is in network transfer */
#define CF_SUCC		"SUCC"	/* Request has been printed */
#define CF_ERROR	"ERROR"	/* Request encountered error while printing */
#define CF_HOLD		"HOLD"	/* Request status is in hold state waiting */
				/*    for modifications */

#define CF_AGFA		0x1

#define CF_BUSY		1	/* CF currently busy */
#define CF_IDLE		2	/* CF currently idle */
#define CF_STOP		3	/* CF flag to stop processing */
#define CF_ERR		4	/* CF flag to indicate printer error */

#define CF_SLEEP	30	/* Number of seconds to sleep */
#define MAXRETRY 	 3	/* Number of retrys if bad printer status */

#define MAX_LJ_PRINTER 100      /* Max number of printers allowed */
#define MAX_LJ_PARAM     8      /* Max number of sub print parameters */

struct LJ_PRINTER
        {
        float msz_line;         /* media size for line direction in mm */
        float msz_sample;       /* media size for sample direction in mm */
	char pname[64];		/* printer name */
        char mtype[64];         /* media type available */
        char owner[MAX_LJ_PARAM][64];
                                /* owner(s) of printer */
        char priv[MAX_LJ_PARAM][64];
                                /* privleged users (able to set priorities) */
        int  exp[2][3];		/* high/low res red, green, blue exposure */
        char emulsion[64];      /* printer emulsion, (up or dn) */
        };


struct IMGOPT
	{
	long imgnam;	/* NEW -TRUE if image name to be annotated */
	long logo;	/* NEW -TRUE if agency logo annot */
	long mapscale;	/* NEW -TRUE if mapscale specified */
	long prntsize;	/* NEW -TRUE if prtsize specified */
	long spot;		/* TRUE if spot annot */
	long wedge;		/* TRUE if wedge annot */
	long stretch;		/* TRUE if stretch annot */
	long title1;		/* TRUE if title annot */
	long title2;		/* TRUE if second line of title annot */
	long annot;		/* TRUE if note annot */
	long annot_ss;		/* Starting pixel for lower annotation */
	long annot_maxchar;	/* Annotation max number of characters */
	long rgb2cmy;		/* TRUE if image needs to be converted */
	long expand;		/* TRUE if image needs to be expanded */
	long reduce;		/* TRUE if image needs to be reduced */
	long font_nmr;		/* index into the anot_tab to select font */
	long font_scale;	/* font scale factor */
	long fontnl;		/* number of lines for annot area */
	long format;		/* LightJet format */
	long margin[2];		/* Line/sample margin around image */
	long maxnl;		/* max number image lines, used for npanels */
	long maxns;		/* max number image samples, used for npanels */
	long outnl;		/* actual number of image lines to be written */
	long outns;		/* actual number of image pixels to be written*/
	long outnb;		/* output number of bands */
	long ss;		/* starting sample within image */
	long sl;		/* starting line within image */
	long nl;		/* number lines within image */
	long ns;		/* number samples within image */
	long time_cpu;		/* start time for cpu */
	long time_elapse;	/* start time for elapse */
	unsigned char *lasbuf;	/* pointer to image buffer */
	char fannot[10][1024];	/* formated annot strings (title1, title2,   */
				/*   spot copyright, stretch, annot, image)  */
	};

#define A_TITLE1      0
#define A_TITLE2      1
#define A_COPYRIGHT   2
#define A_STRETCH     3
#define A_ANNOT       4
#define A_ANNOT2      5
#define A_MAX         6

#define NLUT  3			/* Number of LUT's */
#define MAXPT 4096		/* Max number of points in LUT */

struct lut
   {
   int   in[MAXPT];
   float out[MAXPT];
   long  ne;
   char  desc[88];
   };

struct reclut
   {
   unsigned short in[MAXPT];
   unsigned short out[MAXPT];
   long          ne;
   char          desc[88];
   };

struct recfmt
	{
	long	delflg;		/* delete flag, TRUE => request is deleted */
	long	fptr;		/* points to next record with same priority */
	long	bptr;		/* points to last record with same priority */
	long	lad;		/* last access date -- julian day */
	long	seqnum;	      	/* sequence number to identify requests	*/
	long 	bands[NMRIMG][MAXBND+1]; /* bands specified */
	long	bgrnd;		/* background pixel value */
	long	cal;		/* identification of calibration file */
	long	con;		/* identification of control file */
	long	curcop;		/* current copy being processed */
	long	curpan;		/* current panel to print */
	long	ewr;		/* ewr charge number (NO LONGER IN USE) */
	long	format;       	/* internal characteristics of the color FIRE*/
	long	nb;		/* number of bands */
	long	ncop;		/* number of copies to be produced */
	long	nimg;		/* number of images specified */
	long	nl;		/* number of lines */
	long	npanel;		/* number of panels to print */
	long	nruns;		/* # of times processed by color FIRE */
	long	ns;		/* number of samples */
	long	prio;		/* priority given to the request */
	long	reduce[2];	/* scale factor to reduce image (line & samp) */
	long	sl;		/* starting line of image */
	long	ss;		/* starting sample of image */
	float	expand[2];	/* scale factor to expand image (line & samp) */
	float	margin[2];	/* top/side margin border around image */
	float	mapscale;	/* desired map scale */
	float   pixsize;	/* pixel size in meters */
	float 	prntsize[2];	/* output print size */
	char	aflg[8];	/* yes=>annotate date/time,user,lut,nl/ns,etc)*/
	char	annot[128];	/* user notes to be annotated onto film	*/
	char	chrgcod[16];	/* specifies the accountable project */
	char	color[8];   	/* color sequence (RGB, CMY, CMYK, or BW) */
        char    delfil[8];      /* delete flag (YES or NO) */
	char    dltf[16];	/* dltf entry name */
	char	dorran[3][32];	/* Order, Unit, Bin Dorran parameters */
	char 	errtext[256];	/* error text msg */
	char 	film[32];	/* Media Type */
	char 	fspec[128];	/* LAS Image file specification */
	char    ftype[16];      /* File type (LAS, TIF) */
	char	imgnam[8];	/* Image name annot (YES, or NO) */
	char	infile[128];	/* enhancement or DLT file name */
	char	pname[128];	/* printer name (LJ5000 or LJ2080) */
	char	remote[64];	/* Host name of remote system with image */
        char    special[136];   /* special instruction */
	char 	spot[8];	/* SPOT copyright annot (YES, or NO) */
	char	stretch[4][64];	/* stretch points to be annotated onto film */
	char	stflg[8];	/* Status flag (QUEUE, RUN, SUCC, or ERROR) */
	char	stropt[8];	/* stretch option -- SAME, NONE, VALUE */
	char	submit_date[32];/* submit date for reporting purposes */
	char	tae[NMRIMG][128]; /* TAE image file names */
	char	title[2][128];  /* title of image to be annotated on top line*/
	char	usercode[32];	/* user code -- used for request security */
	char 	wedge[8];	/* NMD color wedge (YES or NO) */
	struct reclut enhlut[NLUT];	/* enh map values */
        };


struct hdrfmt
	{
	int	fptr[10];	/* array of pointers pointing to the first   */
				/* request of each priority.  The zero       */
				/* element points to the first free record.  */
	int	bptr[10];	/* array of pointers pointing to the last    */
				/* request of each priority.		     */
   	};

#define	HDRLEN sizeof(struct hdrfmt)	/* len of header structure for CRF */
#define RECLEN sizeof(struct recfmt)	/* len of record structure for CRF */

struct SOCKET_MSG
	{
	long mode;
	long ncopy;
	long seqnum;
	long panel;
	char frame[32];
	};

struct SOCKET_ACK
	{
	long status;
	long sleep;
	};

#define NL_F0 12			/* define the FONT sizes */
#define NS_F0  7
#define NL_F1 24
#define NS_F1 13
#define NL_F2 40		
#define NS_F2 22
#define NL_F3 48
#define NS_F3 26
#define NL_F4 58
#define NS_F4 32

#define SCALE_F1 1.33333		/* scale ratios for each format */
#define SCALE_F2 1.00
#define SCALE_F3 6.66667
#define SCALE_F4 3.33333

#define NFORMAT 4

			/*     FORMAT TABLE DEFINITION     */
/* The format table defines some of the internal characteristics of the film */
/*   recorder.								     */

struct fmttab
	{
	float	spot;		/* spot size */
	float   ratio;		/* spot size ratios */
	int	calns;		/* cal wedge number of samples (17" strip) */
	int	nl;		/* maximum number of lines (including annot) */
	int	ns;		/* maximum number of samples */
	int	nl_mm;		/* number lines per millimeter */
	char	pname[64];	/* printer name */
	};

struct annot_tab
	{
	int	nl;		/* annotation number of lines per char */
	int	ns;		/* annotation number of samples per char */
	};
#endif
