/******************************************************************************

SUBROUTINE:	abort

PURPOSE:
	Abort a currently running request.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0          8/93        K. Gacke    initial development -- copied from QLP

PROJECT:        Unix Color FIRE

Algorithm:

******************************************************************************/

				
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "lj.h"
#include "ljsupport.h"
#include "local_protos.h"

static long fatal = -1;


int abreq
(
	struct PARBLK *blkptr
)
{
extern struct recfmt rec;
extern struct hdrfmt hdr;
struct VARIABLE *seqnum;
struct LJ_PRINTER printer[MAX_LJ_PRINTER];
long		fd,seq,nrec;
long		crflg;
long		pcnt;
long		rstat;
long 		suflg;
char		msg[80];
char 		user[64];

rstat = E_SUCC;
getuser(user);
seqnum = p_find(blkptr,"RECNUM");  /* retrieve sequence number from V-block  */
seq = IVAL(*seqnum,0);	   	   /* set seq equal to the sequence number   */
(void) crfinit("CF-ABORT");
(void) crfpr(printer,&pcnt,TRUE);  /* retrieve printer info */
(void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec); /* open the CRF */
if (seq > nrec) 		   /* if sequence number > number of requests*/
   {				   /*   write error message & terminate proc */
   sprintf(msg,"Sequence Number %d does not exist",seq);
   rstat = E_FAIL;
   goto xit;
   }
(void) crfrd(fd,TRUE,seq,&rec);	    /* read the request to be modified */
if (strcmp(rec.stflg,CF_RUN) != 0)
   {				   /*   write error message & terminate proc */
   sprintf(msg,"Sequence number %d is not currently being processed by printer",
	seq);
   rstat = E_FAIL;
   goto xit;
   }
if (rec.delflg != -1)	   	   /* if request is not valid then	     */
   {				   /*   write error message & terminate proc */
   sprintf(msg,"Sequence Number %d does not exist",seq);
   rstat = E_FAIL;
   goto xit;
   }
suflg = lj_chkuser(printer,pcnt,&rec);
if (suflg != SU_OWN)
   {
   if (strcmp(rec.usercode,user) != 0)
      {
      sprintf(msg,"ERROR: User <%s> is not owner of custom request %d",
	   user,seq);
      rstat = E_FAIL;
      goto xit;
      }
   }
(void) crfabort(fd,rec.seqnum,FALSE);
xit:
(void) crfcl(fd,TRUE);		    /* close the CRF */
(void) crfdone();
if (rstat != E_SUCC)
   c_errmsg(msg," ",&fatal);
return(E_SUCC);
} 
