/******************************************************************************

SUBROUTINE: dynam

PURPOSE: 
	The purpose of dynam is to build a dynamic parameter block.  The 
	default values for the parameters will be assigned the values contained
	in the global variable of rec.  A dynamic tutor session is initiated, 
	and the new paramter values are received by calling the subroutine of 
	RPARM.  The new values are returned to the calling program through the
	global variable rec.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0          8/93        K. Gacke    initial development -- copied from QLP

PROJECT:	Unix Color FIRE

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "lj.h"
#include "ljsupport.h"
#include "local_protos.h"

int dynam
(
    struct LJ_PRINTER printer[],
    long pcnt
)
{
extern struct recfmt rec;
extern struct hdrfmt hdr;
struct PARBLK	dblock;
double		dblval[2];
int		index;
int		intval[3];
int		rstat;
char		*strval[10];

q_init(&dblock,P_BYTES,P_ABORT);

if (strlen(rec.ftype) == 0)
    q_string(&dblock,"FTYPE",-2,strval,P_ADD);
else
    {
    strval[0] = rec.ftype;
    q_string(&dblock,"FTYPE",1,strval,P_ADD);
    }


if (rec.format == NULLVAL)
   q_intg(&dblock,"MODE",-2,intval,P_ADD);
else
   {
   intval[0] = rec.format;
   q_intg(&dblock,"MODE",1,intval,P_ADD);
   }

if (rec.bgrnd == NULLVAL)
   q_intg(&dblock,"BACKGRND",-2,intval,P_ADD);
else
   {
   intval[0] = rec.bgrnd;
   q_intg(&dblock,"BACKGRND",1,intval,P_ADD);
   }

if (rec.cal == NULLVAL)
   q_intg(&dblock,"CALFILE",-2,intval,P_ADD);
else
   {
   intval[0] = rec.cal;
   q_intg(&dblock,"CALFILE",1,intval,P_ADD);
   }

if (rec.con == NULLVAL)
   q_intg(&dblock,"CONFILE",-2,intval,P_ADD);
else
   {
   intval[0] = rec.con;
   q_intg(&dblock,"CONFILE",1,intval,P_ADD);
   }

if ((rec.expand[0] == NULLVAL) || 
    (rec.prntsize[0] != NULLVAL) || (rec.mapscale != NULLVAL))
   q_real(&dblock,"EXPAND",-2,dblval,P_ADD);
else
   {
   dblval[0] = rec.expand[0];
   dblval[1] = rec.expand[1];
   q_real(&dblock,"EXPAND",2,dblval,P_ADD);
   }

if (rec.mapscale == NULLVAL) 
   q_real(&dblock,"MAPSCALE",-2,dblval,P_ADD);
else
   {
   dblval[0] = rec.mapscale;
   q_real(&dblock,"MAPSCALE",1,dblval,P_ADD);
   }

if (rec.pixsize == NULLVAL) 
   q_real(&dblock,"PIXSIZE",-2,dblval,P_ADD);
else
   {
   dblval[0] = rec.pixsize;
   q_real(&dblock,"PIXSIZE",1,dblval,P_ADD);
   }

if (rec.prntsize[0] == NULLVAL) 
   q_real(&dblock,"PRNTSIZE",-2,dblval,P_ADD);
else
   {
   dblval[0] = rec.prntsize[0];
   dblval[1] = rec.prntsize[1];
   q_real(&dblock,"PRNTSIZE",2,dblval,P_ADD);
   }

if (rec.ncop == NULLVAL)
   q_intg(&dblock,"NCOPIES",-2,intval,P_ADD);
else
   {
   intval[0] = rec.ncop;
   q_intg(&dblock,"NCOPIES",1,intval,P_ADD);
   }

if (rec.prio == NULLVAL)
   q_intg(&dblock,"PRIORITY",-2,intval,P_ADD);
else
   {
   intval[0] = rec.prio;
   q_intg(&dblock,"PRIORITY",1,intval,P_ADD);
   }

if (rec.reduce[0] == NULLVAL)
   q_intg(&dblock,"REDUCE",-2,intval,P_ADD);
else
   {
   intval[0] = rec.reduce[0];
   intval[1] = rec.reduce[1];
   q_intg(&dblock,"REDUCE",1,intval,P_ADD);
   }

if ((rec.curpan == NULLVAL) || (rec.curpan >= rec.npanel))
   q_intg(&dblock,"SPANEL",-2,intval,P_ADD);
else
   {
   intval[0] = rec.curpan;
   q_intg(&dblock,"SPANEL",1,intval,P_ADD);
   }

if (strlen(rec.annot) == 0)
   q_string(&dblock,"ANNOT",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.annot;
   q_string(&dblock,"ANNOT",1,strval,P_ADD);
   }

if (strlen(rec.delfil) == 0)
   q_string(&dblock,"DELFLG",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.delfil;
   q_string(&dblock,"DELFLG",1,strval,P_ADD);
   }

if (strlen(rec.dltf) == 0)
   q_string(&dblock,"INDLTF",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.dltf;
   q_string(&dblock,"INDLTF",1,strval,P_ADD);
   }

if (strlen(rec.fspec) == 0)
   q_string(&dblock,"IN",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.fspec;
   q_string(&dblock,"IN",1,strval,P_ADD);
   }

if (strlen(rec.infile) == 0)
   q_string(&dblock,"INFILE",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.infile;
   q_string(&dblock,"INFILE",1,strval,P_ADD);
   }

if (strlen(rec.pname) == 0)
   q_string(&dblock,"PRINTER",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.pname;
   q_string(&dblock,"PRINTER",1,strval,P_ADD);
   }

if (strlen(rec.film) == 0)
   q_string(&dblock,"MEDIA",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.film;
   q_string(&dblock,"MEDIA",1,strval,P_ADD);
   }

if (strlen(rec.special) == 0)
   q_string(&dblock,"SPECIAL",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.special;
   q_string(&dblock,"SPECIAL",1,strval,P_ADD);
   }

if (strlen(rec.spot) == 0)
   q_string(&dblock,"SPOTCOPY",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.spot;
   q_string(&dblock,"SPOTCOPY",1,strval,P_ADD);
   }

if (strlen(rec.stropt) == 0)
   q_string(&dblock,"STRETCH",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.stropt;
   q_string(&dblock,"STRETCH",1,strval,P_ADD);
   }

dblval[0] = rec.margin[0];
dblval[1] = rec.margin[1];
q_real(&dblock,"MARGIN",2,dblval,P_ADD);

if (strlen(rec.title[0]) == 0)
   q_string(&dblock,"TITLE",-2,strval,P_ADD);
else				
   {				
   index = 1;
   strval[0] = rec.title[0];
   if (strlen(rec.title[1]) > 0)
	{
	++index;
        strval[1] = rec.title[1];
	}
   q_string(&dblock,"TITLE",index,strval,P_ADD);
   }

if (strlen(rec.dorran[0]) == 0)
   q_string(&dblock,"DORRAN",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.dorran[0];
   strval[1] = rec.dorran[1];
   strval[2] = rec.dorran[2];
   q_string(&dblock,"DORRAN",3,strval,P_ADD);
   }

if (strlen(rec.imgnam) == 0)
   q_string(&dblock,"IMGOPT",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.imgnam;
   q_string(&dblock,"IMGOPT",1,strval,P_ADD);
   }

if (strlen(rec.wedge) == 0)
   q_string(&dblock,"WEDGE",-2,strval,P_ADD);
else				
   {				
   strval[0] = rec.wedge;
   q_string(&dblock,"WEDGE",1,strval,P_ADD);
   }

q_dynp(&dblock,"LJDY.PDF",M_FULLPDF);	/* send V-block for dynamic parameter*/
p_inim(&dblock,P_BYTES,P_ABORT);	/* receive V-block from TM	     */

rstat = rparm(&dblock,&rec,printer,pcnt,PMOD);	/* receive parameter values */
return(rstat);
}	/*  END of DYNAM subroutine  */
