/******************************************************************************

PROGRAM:  LJ

PURPOSE: 
	LJ is a routine which allows the user to add, modify, or delete
	requests in the Custom Request File.  The Custom Request File is a file
	containing requests for film production on the LightJet 5000.

	The structure of the file is a header record of 80 bytes followed by
	records containing the requests.  A record is defined by the data 
	structure of recfmt.  

	CF acts as the driver.  It determines what operation is to be
	performed, and then calls the appropiate subroutine to do the file
	operation.

			
PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           2/98       K. Gacke    initial development -- copied from
                                        Unix color FIRE

PROJECT:        LightJet

******************************************************************************/

#include <stdio.h>

#include "las.h"
#include "util.h"
#include "lj.h"
#include "format.h"
#include "ljsupport.h"
#include "local_protos.h"

struct recfmt rec;
struct hdrfmt hdr;

#define LJ_MOD   1
#define LJ_CONT  2

int query_user(void);

int main (void)
{
struct PARBLK	vblock;
struct VARIABLE *sub,*snum,*var;
struct LJ_PRINTER printer[MAX_LJ_PRINTER];
long	fatal = -1;
long	nonfatal = 1;
long	crflg;
long	fd;
long	file;
long	lp;
long	nrec;
long    pcnt;
long 	seqnum[50];
long	std_out;
long	term;
long	zero = 0;
int	i;
char	fname[256];
char	outform[32];
char	outmsg[80];
char	prog[20];
char	prtfile[256];
char	*media_flag[10];
char	*status_flag[10];
char	subcmd[12];

c_inlas(&vblock,&std_out);	  /* initialize the TAE routine packages     */
sub = p_find(&vblock,"_SUBCMD");  /* retrieve the subcommand from V-block    */
strcpy(subcmd,SVAL(*sub,0));  	  /* copy the value of subcommand into subcmd*/
sprintf(prog,"LJ-%s",subcmd);
if (!(strcmp(subcmd,"ADD")))      
   {
   *rec.usercode = '\0';
   (void) crfinit(prog);
   (void) crfpr(printer,&pcnt,TRUE);  /* retrieve printer info */
   if (rparm(&vblock,&rec,printer,pcnt,PADD) != E_SUCC)
	c_errmsg("Error reading input parameters"," ",&fatal);
   if ((rec.npanel == 1) || (chkbat(&vblock)))
      {
      (void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec);
      addreq(fd,crflg,nrec);
      (void) crfcl(fd,TRUE);
      (void) crfdone();
      }

/*
* If the number of panels is greater than one, query the user to see if the
* request should be added to the queue as specified, or allow the user to 
* re-modify the request parameters in an attempt to fit onto a single panel.
*/
   else
      {
      switch (query_user())
	{
	case LJ_CONT:
   	    (void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec);
   	    addreq(fd,crflg,nrec);
   	    (void) crfcl(fd,TRUE);
   	    (void) crfdone();
	    break;
	case LJ_MOD:
	    strcpy(rec.stflg,CF_HOLD);
   	    (void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec);
   	    addreq(fd,crflg,nrec);
   	    (void) crfcl(fd,TRUE);
            for (i=LJ_MOD; i!=LJ_CONT; )
	      {
              if (modreq(rec.seqnum) != E_SUCC)
	          c_errmsg("Error reading input parameters"," ",&fatal);
	      if (rec.npanel == 1)
	          i = LJ_CONT;
	      else
	          i = query_user();
              switch (i)
	          {
	          case LJ_CONT:
		      strcpy(rec.stflg,CF_QUEUE);
		      break;
	          case LJ_MOD:
		      strcpy(rec.stflg,CF_HOLD);
		      break;
	          }
   	      (void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec);
   	      (void) crfwr(fd,TRUE,rec.seqnum,&rec);
   	      (void) crfcl(fd,TRUE);
	      }
   	    (void) crfdone();
	    break;
	}
      }
   sprintf(outmsg,"Created Sequence Number = %d",rec.seqnum);
   c_errmsg(outmsg," ",&nonfatal);
   }
else if (!(strcmp(subcmd,"MODIFY")))
   {
   if (chkbat(&vblock))
      c_errmsg("CF-MODIFY must be executed in INTERACTIVE MODE"," ",&fatal);
   snum = p_find(&vblock,"RECNUM"); /*   retrieve sequence nmr. from V-block */
   seqnum[0] = IVAL(*snum,0);    /*	   assign sequence numbers to seqnum */
   (void) crfinit("CF-MOD");
   for (i=LJ_MOD; i!=LJ_CONT; )
	{
        if (modreq(seqnum[0]) != E_SUCC)
	    c_errmsg("Error reading input parameters"," ",&fatal);
        if (rec.npanel == 1)
	    i = LJ_CONT;
	else
	    i = query_user();
        switch (i)
	    {
	    case LJ_CONT:
		strcpy(rec.stflg,CF_QUEUE);
		break;
	    case LJ_MOD:
		strcpy(rec.stflg,CF_HOLD);
		break;
	    }
   	(void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec);
   	(void) crfwr(fd,TRUE,rec.seqnum,&rec);
   	(void) crfcl(fd,TRUE);
	}
   (void) crfdone();
   }
else if (!(strcmp(subcmd,"DELETE")))
   {				    /* else if subcommand = "DELETE" then    */
   snum = p_find(&vblock,"RECNUM"); /*   retrieve sequence nmr. from V-block */
   for (i=0;i<snum->v_count;i++)    /*   for each sequence number do	     */
      seqnum[i] = IVAL(*snum,i);    /*	   assign sequence numbers to seqnum */
   (void) crfinit(prog);
   (void) crfpr(printer,&pcnt,TRUE);  /* retrieve printer info */
   (void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec);/* open the CRF */
   delreq(printer,pcnt,fd,nrec,seqnum,snum->v_count); /* delete the requests*/
   (void) crfcl(fd,TRUE);		 /* close the CRF */
   (void) crfdone();
   }
else if (!(strcmp(subcmd,"ABORT")))
   {
   if (abreq(&vblock) != E_SUCC)
	c_errmsg("Error aborting request"," ",&fatal);
   }
else if (!(strcmp(subcmd,"LS")))
   {
   snum = p_find(&vblock,"RECNUM");
   for (i=0;i<snum->v_count;i++)
      seqnum[i] = IVAL(*snum,i);
   seqnum[i] = 0;
   var = p_find(&vblock,"STFLAG");
   for (i=0; i<var->v_count; i++)
      status_flag[i] = SVAL(*var,i);
   status_flag[i] = NULL;
   var = p_find(&vblock,"MEDIA");
   for (i=0; i<var->v_count; i++)
      media_flag[i] = SVAL(*var,i);
   media_flag[i] = NULL;
   var = p_find(&vblock,"OUTFORM");
   strcpy(outform,SVAL(*var,0));
   strcpy(prtfile,"cf");
   c_setprt(&vblock,&term,&file,&lp,prtfile,fname);
   (void) crfinit(prog);
   (void) crfpr(printer,&pcnt,TRUE);  /* retrieve printer info */
   print_crf(printer,pcnt,prtfile,seqnum,status_flag,media_flag,outform);
   }
c_complt(prog,NULL,&zero);
return 0;
}

int query_user(void)
{      
long cnt;
long len;
int rstat=LJ_CONT;
char msg[256];
COUNT T_col;                     /* terminal # column */
COUNT T_lin;                     /* terminal # lines */
COUNT t_type;                   /* terminal type */
COUNT t_term;

t_init(&T_lin,&T_col,&t_type);
len = 1;
for (cnt=0; cnt<10; cnt++)
    {
    t_output(T_lin,1,"Multiple Panels: (a=accept, m=modify): ");
    t_read(msg,&t_term);
    (void) c_up2low(msg,&len);
    if (strncmp(msg,"m",len) == 0)
	{
	rstat = LJ_MOD;
	break;
	}
    else if (strncmp(msg,"a",len) == 0)
	{
	rstat = LJ_CONT;
	break;
	}
    }
return(rstat);
}
