#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "lj.h"
#include "ljsupport.h"
#include "local_protos.h"

static long nonfatal = 1;

long lj_parse
(
    long cnt,
    struct LJ_PRINTER printer[],
    char *buf
);

long lj_assign_calcon
(
        struct recfmt *rec,
        long *calnmr,
        long *connmr
)
{
long rstat;

rstat = E_SUCC;
if (rec->cal != NULLVAL)
    *calnmr = rec->cal;
else
    {
    if (strcmp(rec->film,"PAPER40x50") == 0)
        *calnmr = rec->format + 50;
    else if (strcmp(rec->film,"PAPER50x50") == 0)
        *calnmr = rec->format + 50;
    else if ((strcmp(rec->film,"FILM10x13") == 0) || (strcmp(rec->film,"FILM9x9") == 0))
        *calnmr = rec->format + 20;
    else
	{
	*calnmr = 0;
	rstat = E_FAIL;
	}
    }
if (rec->con != NULLVAL)
    *connmr = rec->con;
else
    {
    if (strcmp(rec->film,"PAPER40x50") == 0)
        *connmr = 54;                           /* positive linear, curved */
    else if (strcmp(rec->film,"PAPER50x50") == 0)
        *connmr = 54;                           /* positive linear, curved */
    else if ((strcmp(rec->film,"FILM10x13") == 0) || (strcmp(rec->film,"FILM9x9") == 0))
        *connmr = 24;                           /* positive linear, curved */
    else
	{
	*connmr = 0;
	rstat = E_FAIL;
	}
    }
return(rstat);
}


unsigned short lj_byteswap(unsigned short n)
{
#ifndef DEC_ENDIAN
    return (n >> 8 & 0xFF) | (n << 8 & 0xFF00);
#else
        return (n);
#endif
}

/*
*   Check to see if user is an owner, privilged, or normal privileges
*   of the specified printer within the "rec" structure.  If printer
*   is not specified, search for appropraite printer type (in this case,
*   return the lowest set of privileges common to all printers within
*   the specified printer type).
*/
long lj_chkuser
(
    struct LJ_PRINTER printer[],
    long pcnt,
    struct recfmt *rec
)
{
long cnt;
long cnt1;
long len;
long suflg;
char user[CMLEN];

suflg = SU_NORM;
getuser(user);
len = (long)strlen(user);
c_up2low(user,&len);
if (strcmp(user,"root") == 0) 
    {
    suflg = SU_OWN;
    }
else 
    {
    for (cnt=0; cnt<pcnt; cnt++)
        {
        if (strcmp(rec->film,printer[cnt].mtype) == 0)
            {
            for (cnt1=0; ; cnt1++)
                {
                if (strlen(printer[cnt].priv[cnt1]) == 0)
                    break;
                if (strcmp(printer[cnt].priv[cnt1],user) == 0)
                    suflg = SU_PRIV;
                }
            for (cnt1=0; ; cnt1++)
                {
                if (strlen(printer[cnt].owner[cnt1]) == 0)
                    break;
                if (strcmp(printer[cnt].owner[cnt1],user) == 0)
                    suflg = SU_OWN;
                }
            break;
            }
        }
    }
return(suflg);
}

long lj_format_annot
(
    struct recfmt *rec,
    struct IMGOPT *imgopt
)
{
extern struct fmttab ftable[];
extern struct annot_tab atable[];
struct tm *tptr;                /* time structure */
time_t t;                        /* elapsed time */
long calnum;
long connum;
long cnt;
long len;
long maxchar;
long maxchar_desired;
long rstat;
char temp[256];
char annot_usgs[256];
char annot_spot[256];
char annot_user[256];
char annot_dorran[256];

rstat = E_SUCC;
t = time(NULL);
tptr = localtime(&t);
for (cnt=0; cnt<A_MAX; cnt++)
    imgopt->fannot[cnt][0] = '\0';
if (lj_assign_calcon(rec,&calnum,&connum) != E_SUCC)
    sprintf(rec->errtext,"Error setting calibration/control LUTs");
maxchar = imgopt->outns / (atable[imgopt->font_nmr].ns * imgopt->font_scale * 2);
if (imgopt->title1)
    {
    strcpy(imgopt->fannot[A_TITLE1],rec->title[0]);
    if ((long) strlen(imgopt->fannot[A_TITLE1]) > maxchar)
	{
	imgopt->fannot[A_TITLE1][maxchar] = '\0';
	rstat = E_FAIL;
	}
    }
if (imgopt->title2)
    {
    strcpy(imgopt->fannot[A_TITLE2],rec->title[1]);
    if ((long) strlen(imgopt->fannot[A_TITLE2]) > maxchar)
	{
	imgopt->fannot[A_TITLE1][maxchar] = '\0';
	rstat = E_FAIL;
	}
    }
maxchar = (imgopt->outns - 10) / 
	(atable[imgopt->font_nmr].ns * imgopt->font_scale);
maxchar_desired = 
	(imgopt->outns - (2 * imgopt->margin[1]) - (2 * imgopt->ss)) / 
	(atable[imgopt->font_nmr].ns * imgopt->font_scale);
/*
*  USGS and optionally Spot Copyright Annotation
*/
strcpy(annot_usgs,"USGS EROS Data Center  ");
strcpy(annot_spot,"");
if (imgopt->spot)
    {
    sprintf(annot_spot,"%s%d","SPOT DATA \f CNES ",tptr->tm_year+1900);
    }
sprintf(imgopt->fannot[A_COPYRIGHT],"%s%s",annot_usgs,annot_spot);
/*
*  Mandatory annotation line
*/
sprintf(imgopt->fannot[A_ANNOT],
    "%02d/%02d/%04d %02d:%02d    User: %-8.8s    RecNmr: %03d    ",
        tptr->tm_mon+1,tptr->tm_mday,tptr->tm_year+1900,tptr->tm_hour,
        tptr->tm_min,rec->usercode,rec->seqnum);
sprintf(temp,"Spot: %5.1f    Con: %02d   L/S %dx%d",
        ftable[rec->format].spot,connum,rec->nl,rec->ns);
strcat(imgopt->fannot[A_ANNOT],temp);
if (imgopt->expand)
    {
    if (rec->expand[0] == rec->expand[1])
        sprintf(temp,"    Expand: %5.2f",rec->expand[0]);
    else
        sprintf(temp,"    Expand: %5.2f/%5.2f",rec->expand[0],rec->expand[1]);
    strcat(imgopt->fannot[A_ANNOT],temp);
    }
if (imgopt->reduce)
    {
    sprintf(temp,"    Reduce: %d",rec->reduce[0]);
    strcat(imgopt->fannot[A_ANNOT],temp);
    }
if (rec->npanel > 1)
    {
    sprintf(temp,"    Panel: %d/%d",rec->curpan,rec->npanel);
    strcat(imgopt->fannot[A_ANNOT],temp);
    }
/*
*  Optional user specified annotation and DORRAN specification
*/
if (imgopt->annot)
    {
    sprintf(annot_user,"%s   ",rec->annot);
    strcpy(annot_dorran,"");
    if (strlen(rec->dorran[0]) > 0)
	{
        sprintf(annot_dorran,"Dorran: %s/%s/%s",
                rec->dorran[0],rec->dorran[1],rec->dorran[2]);
	}
    sprintf(imgopt->fannot[A_ANNOT2],"%s%s",annot_user,annot_dorran);
    }
/*
*  Optional stretch point annotation
*/
if (imgopt->stretch)
    {
    if (rec->nb == 1)
	sprintf(imgopt->fannot[A_STRETCH],"%s",rec->stretch[0]);
    else
        sprintf(imgopt->fannot[A_STRETCH],"%s %s %s",
	    rec->stretch[0],rec->stretch[1],rec->stretch[2]);
    }
/*
*  Check to see if the annotation is too long for the image.  Try to left
*  justify with image.  If annotation is too long, try to center annotation
*  below the image.  If annotation is still too long, truncate the annotation.
*/
len = 0;
for (cnt=0; cnt<A_MAX; cnt++)
    {
    if ((long)strlen(imgopt->fannot[cnt]) > len)
	len = (long) strlen(imgopt->fannot[cnt]);
    }
if (len < maxchar_desired)
    {
    sprintf(imgopt->fannot[A_COPYRIGHT],"%s%-*.*s%s",annot_usgs,
	maxchar_desired-len, maxchar_desired-len," ",annot_spot);
    sprintf(imgopt->fannot[A_ANNOT2],"%s%-*.*s%s",annot_user,
	maxchar_desired-len, maxchar_desired-len," ",annot_dorran);
    imgopt->annot_ss = imgopt->margin[1] + imgopt->ss;
    imgopt->annot_maxchar = maxchar_desired;
    }
else if (len < maxchar)
    {
    cnt = len - (long) strlen(annot_usgs) - (long) strlen(annot_spot);
    sprintf(imgopt->fannot[A_COPYRIGHT],"%s%-*.*s%s",annot_usgs,cnt,
        cnt," ",annot_spot);
    cnt = len - (long)strlen(annot_user) - (long)strlen(annot_dorran);
    sprintf(imgopt->fannot[A_ANNOT2],"%s%-*.*s%s",annot_user,cnt,
        cnt," ",annot_dorran);
    imgopt->annot_ss = (imgopt->outns -
         (len * atable[imgopt->font_nmr].ns * imgopt->font_scale)) / 2;
    imgopt->annot_maxchar = len;
    }
else
    {
    for (cnt=0; cnt<A_MAX; cnt++)
	imgopt->fannot[cnt][maxchar] = '\0';
    imgopt->annot_ss = 5;
    imgopt->annot_maxchar = maxchar;
    rstat = E_FAIL;
    }
return(rstat);
}

long lj_getprinter
(
    struct LJ_PRINTER printer[],
    long *pcnt                      /* number of printers defined */
)
{
FILE *fp;
long cnt;
long rstat;
char buf[CMLEN];
char fname[CMLEN];
char msg[CMLEN];
char *ptr;

rstat = E_SUCC;
fp = NULL;
ptr = getenv("QDIR");
if (ptr == NULL)
    {
    c_errmsg("getprinter: Environment variable QDIR not set"," ",&nonfatal);
    goto xit;
    }
sprintf(fname,"%s/%s",ptr,"lj.config");
fp = fopen(fname,"r");
if (fp == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"getprinter: Error opening config file %s",fname);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
for (cnt=0; cnt<MAX_LJ_PRINTER; cnt++)
    strcpy(printer[cnt].mtype,"");
for (cnt=0; cnt<MAX_LJ_PRINTER; )
    {
    if (fgets(buf,256,fp) == NULL)
        break;
    if (*buf == '#')
        continue;
    if (*(buf + strlen(buf) - 1) == '\n')
        *(buf + strlen(buf) - 1) = '\0';
    if (lj_parse(cnt,printer,buf) != E_SUCC)
        continue;
    cnt++;
    }
*pcnt = cnt;
if (cnt >= MAX_LJ_PRINTER)
    {
    rstat = E_FAIL;
    sprintf(msg,"getprinter: Max number of printers is %d\n",MAX_LJ_PRINTER);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
xit:
if (fp != NULL)
    fclose(fp);
return(rstat);
}

long lj_parse
(
    long cnt,
    struct LJ_PRINTER printer[],
    char *buf
)
{
long len;
long loccnt;
long ccnt;
long rstat;
char msg[256];
char line[256];
char owner[256];
char priv[256];
char hires[256];
char lores[256];
char *ptr;

rstat = E_SUCC;
printer[cnt].msz_line = 40;
printer[cnt].msz_sample = 50;
for (loccnt=0; loccnt<MAX_LJ_PARAM; loccnt++)
    {
    strcpy(printer[cnt].owner[loccnt],"");
    strcpy(printer[cnt].priv[loccnt],"");
    for (ccnt=0; ccnt<3; ccnt++)
	{
        printer[cnt].exp[0][ccnt] = 0;
        printer[cnt].exp[1][ccnt] = 0;
	}
    }
strcpy(printer[cnt].pname,"");
strcpy(printer[cnt].mtype,"");
strcpy(printer[cnt].emulsion,"");
strcpy(line,buf);
len = (long)strlen(line);
/* c_up2low(line,&len); */

ptr = strtok(line,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer name configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
len = (long)strlen(ptr);
c_low2up(ptr,&len);
if ((strcmp(ptr,"LJ5000") == 0) ||
    (strcmp(ptr,"LJ2080") == 0))
    {
    strcpy(printer[cnt].pname,ptr);
    }
else
    {
    sprintf(msg,"%s is an invalid printer name",ptr);
    c_errmsg(msg," ",&nonfatal);
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid media type configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
len = (long)strlen(ptr);
/*  c_low2up(ptr,&len); */
if ((strcmp(ptr,"PAPER40x50") == 0) ||
    (strcmp(ptr,"PAPER50x50") == 0) ||
    (strcmp(ptr,"FILM9x9") == 0) ||
    (strcmp(ptr,"FILM10x13") == 0))
    {
    strcpy(printer[cnt].mtype,ptr);
    }
else
    {
    sprintf(msg,"%s is an invalid media type",ptr);
    c_errmsg(msg," ",&nonfatal);
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid media line size configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (sscanf(ptr,"%f",&printer[cnt].msz_line) != 1)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid media line size configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid media sample size configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (sscanf(ptr,"%f",&printer[cnt].msz_sample) != 1)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid media sample size configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer owner configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(owner,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer privilege configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(priv,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer hires configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(hires,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer lores configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(lores,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer status configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (strcmp(ptr," ") == 0)
    strcpy(printer[cnt].emulsion,"");
else
    strcpy(printer[cnt].emulsion,ptr);

/*
* Parse owner string
*/
ptr = strtok(owner,",");
if (ptr == NULL)
    {
    strcpy(printer[cnt].owner[0],owner);
    }
else
    {
    strcpy(printer[cnt].owner[0],ptr);
    for (loccnt=1; loccnt<MAX_LJ_PARAM; loccnt++)
        {
        ptr = strtok(NULL,",");
        if (ptr == NULL)
            break;
        strcpy(printer[cnt].owner[loccnt],ptr);
        }
    }

/*
* Parse privilege string
*/
ptr = strtok(priv,",");
if (ptr == NULL)
    {
    strcpy(printer[cnt].priv[0],priv);
    }
else
    {
    strcpy(printer[cnt].priv[0],ptr);
    for (loccnt=1; loccnt<MAX_LJ_PARAM; loccnt++)
        {
        ptr = strtok(NULL,",");
        if (ptr == NULL)
            break;
        strcpy(printer[cnt].priv[loccnt],ptr);
        }
    }

/*
* Parse hires string
*/
ptr = strtok(hires,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid hires configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
sscanf(ptr,"%d",&printer[cnt].exp[0][0]);
ptr = strtok(NULL,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid hires configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
sscanf(ptr,"%d",&printer[cnt].exp[0][1]);
ptr = strtok(NULL,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid hires configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
sscanf(ptr,"%d",&printer[cnt].exp[0][2]);

/*
* Parse lores string
*/
ptr = strtok(lores,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid lores configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
sscanf(ptr,"%d",&printer[cnt].exp[1][0]);
ptr = strtok(NULL,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid lores configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
sscanf(ptr,"%d",&printer[cnt].exp[1][1]);
ptr = strtok(NULL,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid lores configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
sscanf(ptr,"%d",&printer[cnt].exp[1][2]);

xit:
return(rstat);
}

int lj_setopt
(
    struct LJ_PRINTER printer[],
    struct recfmt *rec,
    struct IMGOPT *imgopt,
    long prtflg
)
{
extern struct fmttab ftable[];
extern struct annot_tab atable[];
long lsamp;			/* subsample required in line direction */
long nannot_line;		/* number of annotation lines */
long maxnl;			/* maxnl for sub paneling */
long maxns;			/* maxns for sub paneling */
long nl;			/* number of lines */
long ns;			/* number of samples */
long norot_reduce;		/* subsample required without rotating */
long norot_npanel;		/* # panels without rotating */
long rstat;			/* return status code */
long ssamp;			/* subsample requried in sample direction */
long temp;			/* temp value */
long tmpnl,tmpns;		/* temp nl, temp ns */
long wind[4];			/* image window size for sub paneling */
char emsg[256];			/* message buffer */

rstat = E_SUCC;
imgopt->spot = (strcmp(rec->spot,"YES") == 0) ? TRUE : FALSE;
imgopt->wedge = (strcmp(rec->wedge,"YES") == 0) ? TRUE : FALSE;
imgopt->imgnam = (strcmp(rec->imgnam,"YES") == 0) ? TRUE : FALSE;
imgopt->stretch = (strcmp(rec->stropt,"YES") == 0) ? TRUE : FALSE;
imgopt->title1 = (strlen(rec->title[0]) > 0) ? TRUE : FALSE;
imgopt->title2 = (strlen(rec->title[1]) > 0) ? TRUE : FALSE;
imgopt->annot = (strlen(rec->annot) || strlen(rec->dorran[0])) ? TRUE : FALSE;
imgopt->expand = (rec->expand[0] != NULLVAL) ? TRUE : FALSE;
imgopt->reduce = (rec->reduce[0] != NULLVAL) ? TRUE : FALSE;
imgopt->mapscale = (rec->mapscale != NULLVAL) ? TRUE : FALSE;
imgopt->prntsize = (rec->prntsize[0] != NULLVAL) ? TRUE : FALSE;
if (strcmp(rec->color,"RGB") == 0)
    {
    imgopt->rgb2cmy = TRUE;
    imgopt->outnb = 3;
    }
else if (strcmp(rec->color,"BW") == 0)
    {
    imgopt->rgb2cmy = TRUE;
    imgopt->outnb = 3;
    }
else if (strcmp(rec->color,"CMY") == 0)
    {
    imgopt->rgb2cmy = FALSE;
    imgopt->outnb = 3;
    }
else if (strcmp(rec->color,"CMYK") == 0)
    {
    imgopt->rgb2cmy = FALSE;
    imgopt->outnb = 4;
    }
/*
* Compute the number of lines required for annotation.
* The gray scale and two lines of bottom annotation is on each frame.
*/
nannot_line = 2;
if (imgopt->title1)
    nannot_line += 3;
if (imgopt->title2)
    nannot_line += 2;
if (imgopt->stretch)
    ++nannot_line;
if ((imgopt->wedge) || (imgopt->imgnam))
    ++nannot_line;
if (imgopt->annot)
    ++nannot_line;
rstat = assign_format(printer,rec,imgopt,nannot_line);
if (rstat != E_SUCC)
    goto xit;
if ((!imgopt->expand) && (!imgopt->reduce))	/* default is to subsample */
    {						/* image to fit on one frame */
/* 
*	Compute subsample factor required 
*/
    lsamp = rec->nl / (ftable[rec->format].nl-imgopt->fontnl-imgopt->margin[0]*2);
    if (rec->nl  % (ftable[rec->format].nl-imgopt->fontnl-imgopt->margin[0]*2) != 0)
        ++lsamp;
    ssamp = rec->ns / (ftable[rec->format].ns-imgopt->margin[1]*2);
    if (rec->ns % (ftable[rec->format].ns-imgopt->margin[1]*2) != 0)
        ++ssamp;
    norot_reduce = (lsamp > ssamp) ? lsamp : ssamp;
    if (norot_reduce > 1)
	{
	rec->reduce[0] = rec->reduce[1] = norot_reduce;
	imgopt->reduce = TRUE;
	if (prtflg)
	    {
	    sprintf(emsg,"NOTE: Automatic image reduction set to %d",
		rec->reduce[0]);
	    c_errmsg(emsg," ",&nonfatal);
	    sprintf(emsg,"  Max image size allowed is: %d lines, %d samples",
		ftable[rec->format].nl-imgopt->fontnl-imgopt->margin[0]*2,
		ftable[rec->format].ns-imgopt->margin[1]*2);
	    c_errmsg(emsg," ",&nonfatal);
	    sprintf(emsg,"  Actual image size is:      %d lines, %d samples",
		    rec->nl,rec->ns);
	    c_errmsg(emsg," ",&nonfatal);
	    temp = rec->nl+(2*atable[imgopt->font_nmr].nl*imgopt->font_scale);
	    if ((lsamp > 1) && (temp < ftable[rec->format].nl))
	        {
	        sprintf(emsg,"  %s %s", 
		  "Image will fit full line resolution by reducing number",
		  "of annotation lines and/or reducing the margin");
	        c_errmsg(emsg," ",&nonfatal);
	        }
	    if ((ssamp > 1) && (rec->ns < ftable[rec->format].ns))
	        {
	        sprintf(emsg,"  %s %s",
		    "Image will fit full sample resolution by",
		    "reducing the margin");
	        c_errmsg(emsg," ",&nonfatal);
	        }
	    sleep(2);
	    }
	}
    }
/*
*  Image sub-sampeling is accomplished within software as the image is
*  transfered from the remote host.  The LightJet cannot reduce the size
*  of an image.  Image reduction must be an integral factor.
*
*  Image expansion is accomplished using the LightJet's interpolator
*  parameters.
*/
if (imgopt->reduce)
    {
    maxnl = (imgopt->maxnl - imgopt->fontnl - (imgopt->margin[0]*2)) *
        rec->reduce[0];
    maxns = (imgopt->maxns - (imgopt->margin[1]*2)) * rec->reduce[1];
    imgopt->nl = rec->nl / rec->reduce[0];
    if (rec->nl % rec->reduce[0] != 0)
	++imgopt->nl;
    imgopt->ns = rec->ns / rec->reduce[1];
    if (rec->ns % rec->reduce[1] != 0)
	++imgopt->ns;
    imgopt->outnl = imgopt->nl + imgopt->fontnl + (imgopt->margin[0] * 2);
    imgopt->outns = imgopt->ns + (imgopt->margin[1] * 2);
    }
else if (imgopt->expand)
    {
    maxnl = (long) (((float) imgopt->maxnl / rec->expand[0]) -
        (float) imgopt->fontnl - (float) (imgopt->margin[0] * 2));
    maxns = (long) (((float) imgopt->maxns / rec->expand[1]) - 
	(float)(imgopt->margin[1] * 2));
    imgopt->nl = (long)((float)rec->nl * rec->expand[0]);
    imgopt->ns = (long)((float)rec->ns * rec->expand[1]);
    imgopt->outnl = rec->nl + imgopt->fontnl + (imgopt->margin[0] * 2);
    imgopt->outns = rec->ns + (imgopt->margin[1] * 2);
    }
else
    {
    maxnl = imgopt->maxnl - imgopt->fontnl - (imgopt->margin[0]*2);
    maxns = imgopt->maxns - (imgopt->margin[1]*2);
    imgopt->nl = rec->nl;
    imgopt->ns = rec->ns;
    imgopt->outnl = rec->nl + imgopt->fontnl + (imgopt->margin[0] * 2);
    imgopt->outns = rec->ns + (imgopt->margin[1] * 2);
    }
wind[SL] = wind[SS] = wind[NL] = wind[NS] = 0;
for (norot_npanel=0; wind[SL]<rec->nl; norot_npanel++)
    {
    if (wind[SS] + maxns < rec->ns)
        {
        wind[SS] += maxns;
        }
    else
        {
        wind[SS] = 0;
        wind[SL] += maxnl;
        }
    }
rec->npanel = norot_npanel;
/*
*  If single panel, Center the image on the LJ2080 and Upper left corner image 
*  on the LJ5000.
*/
if (rec->npanel == 1)
    {
    if ((imgopt->prntsize) || (strcmp(ftable[rec->format].pname,"LJ2080") == 0))
	{
        imgopt->outnl = imgopt->maxnl;
        imgopt->outns = imgopt->maxns;
        if (imgopt->expand)
	    {
	    imgopt->outnl /= rec->expand[0];
	    imgopt->outns /= rec->expand[1];
	    }
	if (imgopt->reduce)
	    {
	    imgopt->sl = 
	     (imgopt->outnl-imgopt->nl-imgopt->fontnl-(imgopt->margin[0]*2)) / 2;
	    imgopt->ss = (imgopt->outns-imgopt->ns-(imgopt->margin[1]*2)) / 2;
	    }
	else
	    {
	    imgopt->sl = 
	     (imgopt->outnl-rec->nl-imgopt->fontnl-(imgopt->margin[0]*2)) / 2;
	    imgopt->ss = (imgopt->outns-rec->ns-(imgopt->margin[1]*2)) / 2;
	    }
	}
    else
	{
        imgopt->sl = 0;
        imgopt->ss = 0;
	}
    }
/*
*  If multiple panel, Upper Left corner image for both the LJ2080 and LJ5000.
*/
else
    {
    if (imgopt->reduce)
	{
	tmpnl = (imgopt->maxnl - imgopt->fontnl - (imgopt->margin[0]*2)) *
		rec->reduce[0];
	tmpns = (imgopt->maxns - (imgopt->margin[1] * 2)) * rec->reduce[1];
        if (imgopt->nl > tmpnl)
            imgopt->nl = tmpnl;
        if (imgopt->ns > tmpns)
            imgopt->ns = tmpns;
        nl = imgopt->nl / rec->reduce[0];
        if (imgopt->nl % rec->reduce[0] != 0)
	    ++nl;
        ns = imgopt->ns / rec->reduce[1];
        if (imgopt->ns % rec->reduce[1] != 0)
	    ++ns;
        imgopt->outnl = nl + imgopt->fontnl + (imgopt->margin[0] * 2);
        imgopt->outns = ns + (imgopt->margin[1] * 2);
	}
    else if (imgopt->expand)
	{
	tmpnl = (long) ((float) imgopt->maxnl / rec->expand[0]) -
		imgopt->fontnl - (imgopt->margin[0]*2);
	tmpns = (long) ((float)imgopt->maxns / rec->expand[1]) - 
		(imgopt->margin[1] * 2);
        if (imgopt->nl > tmpnl)
            imgopt->nl = tmpnl;
        if (imgopt->ns > tmpns)
            imgopt->ns = tmpns;
        imgopt->outnl = imgopt->nl + imgopt->fontnl + (imgopt->margin[0] * 2);
        imgopt->outns = imgopt->ns + (imgopt->margin[1] * 2);
	}
    else
	{
        if (imgopt->nl > imgopt->maxnl-imgopt->fontnl-(imgopt->margin[0]*2))
            imgopt->nl = imgopt->maxnl-imgopt->fontnl-(imgopt->margin[0]*2);
        if (imgopt->ns > imgopt->maxns-(imgopt->margin[1] * 2))
            imgopt->ns = imgopt->maxns-(imgopt->margin[1] * 2);
        imgopt->outnl = imgopt->nl + imgopt->fontnl + (imgopt->margin[0] * 2);
        imgopt->outns = imgopt->ns + (imgopt->margin[1] * 2);
	}
    imgopt->sl = 0;
    imgopt->ss = 0;
    if (prtflg) 
	{
        sprintf(emsg,"NOTE: Automatic image paneling set to %d",rec->npanel);
        c_errmsg(emsg," ",&nonfatal);
        sprintf(emsg,"  Max image size allowed is: %d lines, %d samples",
		maxnl,maxns);
        c_errmsg(emsg," ",&nonfatal);
	sprintf(emsg,"  Actual image size is:      %d lines, %d samples",
		rec->nl,rec->ns);
        c_errmsg(emsg," ",&nonfatal);
        sleep(2);
	}
    }
xit:
return(rstat);
}
