/******************************************************************************

FUNCTION:  LJ_LOGMESS

PURPOSE:
	LJ_LOGMESS is a subroutine which logs messages into a log file.  Along 
	with the message, the current time and the frame id will be written 
	to the log file.

FILE REFERENCES:
  Name		Description
  ----		-----------
  FIRELOG.	Log file

PROGRAM HISTORY:
PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           1/98       K. Gacke    initial development -- copied from VMS
					color FIRE.

PROJECT:	LightJet 5000

LANGUAGE	C

******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <string.h>
#include "las.h"
#include "lj.h"
#include "lj2000.h"


int lj_logmess
(
    long level,         /* verbose level of msg */
    char *frmid,        /* current frame id */
    char *message       /* character string to be written to log file*/
)
{
extern int Vflg;
extern char Firelog[];		/* file name of color FIRE log file */
time_t	sec;			/* variables to obtain elapsed time	     */
long	nonfatal = 1;
char    *cptr;			/* variables to convert time into char string*/
char	msg[2048];		/* message buffer */
char 	*ptr;			/* char pointer */
char	ltime[64];		/* local time buffer */
FILE	*logchan;		/* I/O file pointer for the log file	     */

if (level <= Vflg)
    {
    logchan = fopen(Firelog,"a");  /* open the log file to append 	     */
    if (logchan == NULL)
        {
	sprintf(msg,"Error opening log files <%s>",Firelog);
        c_errmsg(msg,"fire",&nonfatal);
        return(1);
        }
    sec = time(NULL); 		/* receive elapsed time since 1/1/1970       */
    cptr = ctime(&sec);		/* convert time into an ASCII string	     */
    strcpy(ltime,cptr);	
    ptr = strrchr(ltime,'\n');	/* delete the new line character from string */
    if (ptr != NULL)
	*ptr = '\0';
    sprintf(msg,"%8.8s  %s",frmid,message); 
    fprintf(logchan,"%s%s\n",ltime,msg);
    fflush(logchan);		/* flush buffers */
    fclose(logchan);		/* close the log file*/
    c_errmsg(msg," ",&nonfatal);
    }
return(1);			/* return to calling program		     */
}				/* END of LOGMESS subroutine		     */
