/******************************************************************************

	PRCRF prints a request from the custom request to the print file.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           8/93       K. Gacke    initial development -- copied from QLP

PROJECT:        Unix LightJet

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "lj.h"
#include "ljsupport.h"
#include "local_protos.h"

static long fatal = -1;
static long nonfatal = 1;

int prcrf
(
   FILE   *fp,                  /* (I) file descriptor to write data to */
   struct recfmt *rec,          /* (I) request structure to be written */
   long   lutflg                /* (I) if TRUE print enh map pts */
);

int pr_enhlut
(
    FILE *fp,
    struct recfmt *rec
);

int prcrf_short
(
   FILE   *fp,                  /* (I) file descriptor to write data to */
   struct recfmt *rec           /* (I) request structure to be written */
);


int print_crf
(
        struct LJ_PRINTER printer[],
        long pcnt,
	char *prtfile,
	long seqnum[],
	char *status_flag[],
	char *media_flag[],
	char *outform
)
{
struct hdrfmt hdr;
struct recfmt rec;
FILE *fp;
long cnt;
long cnt1;
long crflg;
long fd;
long nrec;
long stflg;
long mdflg;
long priority;
char msg[256];

if ((fp = fopen(prtfile,"w")) == NULL)
   c_errmsg("ERROR creating print file","fire",&fatal);
fprintf(fp,"LightJet Request File\n");
(void) crfop(0,TRUE,ABORT_EXIT,&fd,&crflg,&nrec);
(void) crfrdhdr(fd,TRUE,&hdr);
if (seqnum[0] != 0)
    {
    for (cnt=0; seqnum[cnt]!=0; cnt++)
	{
        (void) crfrd(fd,TRUE,seqnum[cnt],&rec);
        if (rec.delflg != -1)
            {
            sprintf(msg,"Record Number %d does not exist",seqnum[cnt]);
            c_errmsg(msg,"fire",&nonfatal);
            sleep(5);
            continue;
            }
	stflg = FALSE;
	for (cnt1=0; status_flag[cnt1]!=NULL; cnt1++)
	    {
	    if (strcmp(rec.stflg,status_flag[cnt1]) == 0)
		{
		stflg = TRUE;
		break;
		}
	    }
	mdflg = FALSE;
	for (cnt1=0; media_flag[cnt1]!=NULL; cnt1++)
	    {
	    if (strcmp(rec.film,media_flag[cnt1]) == 0)
		{
		mdflg = TRUE;
		break;
		}
	    }
	if ((stflg) && (mdflg))
	    {
	    if ((reqown(printer,pcnt,&rec)) && (strcmp(outform,"LONG") == 0))
                prcrf(fp,&rec,TRUE);       /* print request with enh map pts */
	    if (strcmp(outform,"SHORT") == 0)
                prcrf_short(fp,&rec);
	    }
	}
    }
else
    {
    for (priority=1,cnt=0; priority<=9; priority++)
        {
        seqnum[0] = hdr.fptr[priority];
        while (seqnum[0] != 0)
            {
            (void) crfrd(fd,TRUE,seqnum[0],&rec);
	    stflg = FALSE;
	    for (cnt1=0; status_flag[cnt1]!=NULL; cnt1++)
	        {
	        if (strcmp(rec.stflg,status_flag[cnt1]) == 0)
		    {
		    stflg = TRUE;
		    break;
		    }
	        }
	    mdflg = FALSE;
	    for (cnt1=0; media_flag[cnt1]!=NULL; cnt1++)
	        {
	        if (strcmp(rec.film,media_flag[cnt1]) == 0)
		    {
		    mdflg = TRUE;
		    break;
		    }
	        }
	    if ((stflg) && (mdflg))
		{
	        if ((reqown(printer,pcnt,&rec)) && (strcmp(outform,"LONG")==0))
		    {
                    prcrf(fp,&rec,TRUE);   /* print request with enh map pts */
    		    fprintf(fp,"\f");
		    }
	        if (strcmp(outform,"SHORT") == 0)
                    prcrf_short(fp,&rec);
		}
            seqnum[0] = rec.fptr;
            }
        }
    }
fclose(fp);
(void) crfcl(fd,TRUE);
(void) crfdone();
return(1);
}

int prcrf
(
   FILE   *fp,                  /* (I) file descriptor to write data to */
   struct recfmt *rec,          /* (I) request structure to be written */
   long   lutflg                /* (I) if TRUE print enh map pts */
)
{
long cnt;

fprintf(fp,"Record Number    : %-5d            Last Acess Date  : %d\n",
	rec->seqnum,rec->lad);
fprintf(fp,"User Code        : %-16s Charge Code      : %-8s\n",
	rec->usercode,rec->chrgcod);
fprintf(fp,"Submit Date      : %s\n",rec->submit_date);
fprintf(fp,"Color Order      : %-5s\n",rec->color);
fprintf(fp,"File Type        : %-8s         Remote System    : %-8s\n",
        rec->ftype,rec->remote);
fprintf(fp,"Media Type       : %-12s     Mode             : %-5d\n",
	rec->film,rec->format);
fprintf(fp,"Current Copy     : %-5d            Number of Copies : %-5d\n",
        rec->curcop,rec->ncop);
fprintf(fp,"Current Panel    : %-5d            Number of Panels : %-5d\n",
	rec->curpan,rec->npanel);
fprintf(fp,"Number of Lines  : %-5d            Number of Samples: %-5d\n",
	rec->nl,rec->ns);
fprintf(fp,"Number of Bands  : %-5d            Back Ground      : %-5d\n",
	rec->nb,rec->bgrnd);
fprintf(fp,"Image Name       : %-60.60s\n",rec->fspec);
if (rec->expand[0] == NULLVAL)
    fprintf(fp,"Expand           : --\n");
else
    fprintf(fp,"Expand           : %4.2f, %4.2f\n",
	rec->expand[0],rec->expand[1]);
if (rec->reduce[0] == NULLVAL)
    fprintf(fp,"Reduce           : --\n");
else
    fprintf(fp,"Reduce           : %d, %d\n",rec->reduce[0],rec->reduce[1]);
if (rec->mapscale == NULLVAL)
    fprintf(fp,"Map Scale        : --\n");
else
    fprintf(fp,"Map Scale        : %-7.0f\n",rec->mapscale);
if (rec->pixsize == NULLVAL)
    fprintf(fp,"Pixel Size       : --\n");
else
    fprintf(fp,"Pixel Size       : %-7.0f\n",rec->pixsize);
if (rec->prntsize[0] == NULLVAL)
    fprintf(fp,"Print Size       : --\n");
else
    fprintf(fp,"Print Size       : %5.2fx%5.2f inches\n",
	rec->prntsize[0],rec->prntsize[1]);
fprintf(fp,"Margin           : %5.2f %5.2f\n",rec->margin[0],rec->margin[1]);
fprintf(fp,"Title            : %s\n",rec->title[0]);
fprintf(fp,"Title            : %s\n",rec->title[1]);
fprintf(fp,"Annotation       : %s\n",rec->annot);
fprintf(fp,"Wedge Option     : %s\n",rec->wedge);
fprintf(fp,"Spot Copyright   : %s\n",rec->spot);
fprintf(fp,"Stretch Option   : %s\n",rec->stropt);
if (rec->cal == NULLVAL)
    fprintf(fp,"Calibration  LUT : --\n");
else
    fprintf(fp,"Calibration  LUT : %d\n",rec->cal);
if (rec->con == NULLVAL)
    fprintf(fp,"Control LUT      : --\n");
else
    fprintf(fp,"Control LUT      : %d\n",rec->con);
fprintf(fp,"ENH/DLTF File    : %s\n",rec->infile);
fprintf(fp,"DLTF Entry       : %s\n",rec->dltf);
if (strcmp(rec->stropt,"YES") == 0)
    {
    for (cnt=0; cnt<rec->nb; cnt++)
        fprintf(fp,"Band %d Annot : %s\n",cnt+1,rec->stretch[cnt]);
    }
if (lutflg)
    {
    pr_enhlut(fp,rec);
    }
fprintf(fp,"Priority         : %-5d\n",rec->prio);
if (strlen(rec->dorran[0]))
    {
    fprintf(fp,"Order/Unit/Bin   : %s/%s/%s\n",
	rec->dorran[0],rec->dorran[1],rec->dorran[2]);
    }
fprintf(fp,"Delete File Flag : %-5s\n",rec->delfil);
fprintf(fp,"Number of Runs   : %-5d\n",rec->nruns);
fprintf(fp,"Error Text       : %s\n",rec->errtext);
fprintf(fp,"Status Flag      : %s\n",rec->stflg);
fprintf(fp,"Special Instr    : %s\n",rec->special);
return(1);
}

int prcrf_short
(
   FILE   *fp,                  /* (I) file descriptor to write data to */
   struct recfmt *rec           /* (I) request structure to be written */
)
{
static long First = TRUE;
long npanels;

if (First)
    {
    fprintf(fp,"%s%s\n","                               ",
	"         Media         Total   Submit");
    fprintf(fp,"%s%s\n","Rec   Pri    User     System   ",
	"Status   Type          Panels  Date");
    First = FALSE;
    }
if (rec->curcop == 0)
    npanels=((rec->npanel-rec->curpan+1) * rec->ncop);
else
    npanels=((rec->npanel-rec->curpan) * rec->ncop) + (rec->ncop-rec->curcop);
fprintf(fp,"%3d   %3d    %-8.8s %-8.8s %-8.8s %-11.11s %8d  %s\n",
	rec->seqnum,rec->prio,rec->usercode,rec->remote,rec->stflg,
	rec->film,npanels,rec->submit_date);
return(1);
}

int pr_enhlut
(
    FILE *fp,
    struct recfmt *rec
)
{
long cnt;
long pt;
long start_pt;
char stretch[256];
char temp[10];
char color[NLUT][32];

if (strcmp(rec->color,"RGB") == 0)
    {
    strcpy(color[0],"Red");
    strcpy(color[1],"Green");
    strcpy(color[2],"Blue");
    }
else if (strcmp(rec->color,"CMY") == 0)
    {
    strcpy(color[0],"Cyan");
    strcpy(color[1],"Magenta");
    strcpy(color[2],"Yellow");
    }
else if (strcmp(rec->color,"CMYK") == 0)
    {
    strcpy(color[0],"Cyan");
    strcpy(color[1],"Magenta");
    strcpy(color[2],"Yellow");
    strcpy(color[3],"Black");
    }
else if (strcmp(rec->color,"BW") == 0)
    {
    strcpy(color[0],"Black/White");
    }
for (cnt=0; cnt<rec->nb; cnt++)
    {
    fprintf(fp,"%s Enh Map Points:\n",color[cnt]);
    strcpy(stretch,"  ");
    for (start_pt=0; start_pt<rec->enhlut[cnt].ne; )
	{
	if (start_pt != 0)
	    fprintf(fp,"\n");
        for (pt=start_pt; pt<rec->enhlut[cnt].ne; pt++)
            {
            sprintf(temp,"%3d ",rec->enhlut[cnt].in[pt]);
	    if (strlen(stretch) + strlen(temp) > 80)
	        {
    	        fprintf(fp,"%s\n",stretch);
    	        strcpy(stretch,"  ");
		break;
	        }
	    strcat(stretch,temp);
            }
	if (pt == rec->enhlut[cnt].ne)
            fprintf(fp,"%s\n",stretch);
        strcpy(stretch,"  ");
        for (pt=start_pt; pt<rec->enhlut[cnt].ne; pt++)
            {
            sprintf(temp,"%3d ",(long) rec->enhlut[cnt].out[pt]);
	    if (strlen(stretch) + strlen(temp) > 80)
	        {
    	        fprintf(fp,"%s\n",stretch);
    	        strcpy(stretch,"  ");
		break;
	        }
	    strcat(stretch,temp);
            }
	if (pt == rec->enhlut[cnt].ne)
            fprintf(fp,"%s\n",stretch);
	start_pt = pt;
        }
    }
return(1);
}

int reqown
(
    struct LJ_PRINTER printer[],
    long pcnt,
    struct recfmt *rec
)
{
long suflg;
long len;
long rstat;
char user[64];

rstat = FALSE;
suflg = lj_chkuser(printer,pcnt,rec);
getuser(user);
len = (long) strlen(user);
c_low2up(user,&len);
len = (long) strlen(rec->usercode);
c_low2up(rec->usercode,&len);
if ((suflg > SU_NORM) || (strcmp(rec->usercode,user) == 0))
    rstat = TRUE;
return(rstat);
}
