/******************************************************************************

FUNCTION:  RDDLTF

PURPOSE:
	RDDLTF reads the map points from a specified DLTF file.


PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           8/93       K. Gacke    initial development -- copied from QLP

PROJECT:        Unix Color FIRE

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "lj.h"
#include "dltf.h"
#include "local_protos.h"

static long nonfatal = 1;

int rddltf
(
   char *colorord,		/* color order (RGB, CMY, CMYK, BW) */
   char *fname,			/* file name specification of DLTF file */
   char *dltf,			/* dltf entry name */
   struct lut lut[]		/* the enh map points to be returned */
)
{
struct ASDLTF asdltf;
double offset;
double slope;
long cnt;
long band;
long dtlen;
long endpt;
long fd;
long nlut;
long startpt;
long status;
unsigned char *lutdata;
unsigned char *ptr;
char desc[CMLEN];
char dtype[CMLEN];
char emsg[CMLEN];
char key[16];

if (strcmp(colorord,"RGB"))
    nlut = 3;
else if (strcmp(colorord,"CMY"))
    nlut = 3;
else if (strcmp(colorord,"CMYK"))
    nlut = 4;
else if (strcmp(colorord,"BW"))
    nlut = 1;
strcpy(key,dltf);
status = c_r1dltf(&fd,fname,key,desc,&asdltf,&dtlen,dtype);
if (status != E_SUCC)
    return(status);
if (asdltf.numbnd < nlut)
    {
    sprintf(emsg,"rddltf: DLTF %s consists of less than %d bands",fname,nlut);
    c_errmsg(emsg," ",&nonfatal);
    return(E_FAIL);
    }
lutdata = (unsigned char *) malloc(dtlen);
if (lutdata == NULL)
    {
    c_errmsg("rddltf: Error allocating lut data buffer"," ",&nonfatal);
    return(E_FAIL);
    }
for (band=1; band<=nlut; band++)
    {
    status = c_r2dltf(&fd,key,&band,&startpt,&endpt,&dtlen,lutdata,
	&slope,&offset);
    if (status != E_SUCC)
        return(status);
    ptr = lutdata;
    for (cnt=0; cnt<startpt; cnt++)
	{
        lut[band-1].in[cnt] = cnt;
	lut[band-1].out[cnt] = *ptr;
	}
    for ( ; cnt<endpt; cnt++)
	{
        lut[band-1].in[cnt] = cnt;
	lut[band-1].out[cnt] = *ptr++;
	}
    for (ptr--; cnt<MAXPT; cnt++)
	{
        lut[band-1].in[cnt] = cnt;
	lut[band-1].out[cnt] = *ptr;
	}
    lut[band-1].ne = MAXPT;
    sprintf(lut[band-1].desc,"DLTF data from %s",fname);
    }
free(lutdata);
status = c_cldltf(&fd,fname);
if (status != E_SUCC)
    return(status);
return(E_SUCC);
}
