/******************************************************************************

SUBROUTINE:	rparm

PURPOSE: 
	Subroutine rparm will receive the parameter values from the V-block,
	and return the values in a record format in the global variable "rec".

ARGUMENTS:
  Name		Type	  I/O	Description
  ----		----	  ---	-----------
  *blkptr	PARBLK	   I	pointer to record format for parameter values
  *rec		recfmt	  I/O   pointer to custom request structure to read
				parameter values into
  prog		long	   I	program type, this routine is called by several
				color FIRE modules

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           8/93       K. Gacke    initial development -- copied from QLP
   1.1           5/99       K. Gacke    Y2K and open priorities to all users

PROJECT:        Unix Color FIRE

LANGUAGE	C

PDL:
    BEGIN
	call the function of p_find for each parameter in the v-block that is
		to be read.  The function will return a pointer pointing to 
		the value of the parameter.
	using the c macro of IVAL to access an integer value, and the c
		macro of SVAL to access a string value, assign the parameter
		values to the corresponding fields in the record to be written
		into the custom request file
    END

******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "lj.h"
#include "tiffio.h"
#include "ljsupport.h"
#include "local_protos.h"

#define MAX_NPANEL 5

static long chkimg
(
    struct PARBLK *blkptr,
    struct recfmt *rec
);



int rparm
(
    struct PARBLK *blkptr,
    struct recfmt *rec,
    struct LJ_PRINTER printer[],/* printer definition block */
    long pcnt,                  /* number of printers defined */
    long prog			/* calling program: PADD, PMOD, or REPORT   */
)
{
extern struct annot_tab atable[];
struct VARIABLE *val;
struct IMGOPT imgopt;
struct lut tlut[NLUT];
struct tm *tptr;
time_t  t;
long    access;
long	cnt;
long	cntopt;
long	len;
long	i, j;
long	elem;
long	fatal;
long	nonfatal;
long 	enhflg;	
long	maxchar;
long 	priority;
long	rstat;
long	status;
long    suflg;
int 	ival;
char	emsg[256];
char 	hname[CMLEN];
char 	host[CMLEN];
char	msg[80],temp[10];
char    name[CMLEN];
char	*ptr;

rstat = E_SUCC;
if (prog == PADD)
    {
    getuser(rec->usercode);  	/* set user code */
    rec->nruns = 0;	  	/* set number of runs equal to zero */
    rec->curcop = 0;
    (void) gethostname(rec->remote,CMLEN);
    getchrg(rec->chrgcod);      /* set charge code */
    t = time(NULL);
    tptr = localtime(&t);
    sprintf(rec->submit_date,"%02d/%02d/%04d %02d:%02d",
        tptr->tm_mon+1,tptr->tm_mday,1900 + tptr->tm_year,
	tptr->tm_hour,tptr->tm_min);
    }
if (prog == PMOD)
    {
    strcpy(name,rec->fspec);
    (void) gethostname(host,CMLEN);
    rec->curcop = 0;
    }
nonfatal = 1;
fatal = -1;
/* call the function of p_find for each parameter value that is to be read from
*  the V-block.  The function will return a pointer of type VARIABLE which
*  points to the value of the parameter. */
val = p_find(blkptr,"FTYPE");
if (val->v_count == 0)
    *rec->ftype = '\0';
else
    strcpy(rec->ftype,SVAL(*val,0));
len = (long) strlen(rec->ftype);
c_up2low(rec->ftype,&len);


val = p_find(blkptr,"MODE");	
if (val->v_count == 0)
    rec->format = NULLVAL;
else
    rec->format = IVAL(*val,0);

val = p_find(blkptr,"BACKGRND");	
if (val->v_count == 0)
    rec->bgrnd = NULLVAL;
else
    rec->bgrnd = IVAL(*val,0);

val = p_find(blkptr,"CALFILE");	
if (val->v_count == 0)
    rec->cal = NULLVAL;
else
    rec->cal = IVAL(*val,0);

val = p_find(blkptr,"CONFILE");	
if (val->v_count == 0)
    rec->con = NULLVAL;
else
    rec->con = IVAL(*val,0);

cntopt = 0;
val = p_find(blkptr,"REDUCE");	
if (val->v_count == 0)
    rec->reduce[0] = rec->reduce[1] = NULLVAL;
else if (val->v_count == 1)
    {
    rec->reduce[0] = rec->reduce[1] = IVAL(*val,0);
    ++cntopt;
    }
else
    {
    rec->reduce[0] = IVAL(*val,0);
    rec->reduce[1] = IVAL(*val,1);
    ++cntopt;
    }

val = p_find(blkptr,"EXPAND");	
if (val->v_count == 0)
    rec->expand[0] = rec->expand[1] = NULLVAL;
else if (val->v_count == 1)
    {
    rec->expand[0] = rec->expand[1] = RVAL(*val,0);
    ++cntopt;
    }
else
    {
    rec->expand[0] = RVAL(*val,0);
    rec->expand[1] = RVAL(*val,1);
    ++cntopt;
    }

val = p_find(blkptr,"MAPSCALE");
if (val->v_count == 0)
    rec->mapscale = NULLVAL;
else 
    {
    rec->mapscale = RVAL(*val,0);
    ++cntopt;
    }

val = p_find(blkptr,"PIXSIZE");
if (val->v_count == 0)
    rec->pixsize = NULLVAL;
else 
    rec->pixsize = RVAL(*val,0);

val = p_find(blkptr,"PRNTSIZE");
if (val->v_count == 0)
    rec->prntsize[0] = rec->prntsize[1] = NULLVAL;
else if (val->v_count == 1)
    {
    rec->prntsize[0] = rec->prntsize[1] = RVAL(*val,0);
    ++cntopt;
    }
else
    {
    rec->prntsize[0] = RVAL(*val,0);
    rec->prntsize[1] = RVAL(*val,1);
    ++cntopt;
    }

if (cntopt > 1)
    {
    sprintf(msg,"%s %s","ERROR: Parameters REDUCE, EXPAND, MAPSIZE,",
	"and PRNTSIZE are mutually exclusive");
    c_errmsg(msg," ",&nonfatal);
    rstat = E_FAIL;
    goto xit;
    }

val = p_find(blkptr,"NCOPIES");	
if (val->v_count == 0)
    rec->ncop = NULLVAL;
else
    rec->ncop = IVAL(*val,0);

priority = rec->prio;
val = p_find(blkptr,"PRIORITY");	
if (val->v_count == 0)
    rec->prio = NULLVAL;
else
    rec->prio = IVAL(*val,0);

val = p_find(blkptr,"SPANEL");	
if (val->v_count == 0)
    rec->curpan = 1;
else
    rec->curpan = IVAL(*val,0);

val = p_find(blkptr,"ANNOT");
if (val->v_count == 0)
    *rec->annot = '\0';
else
    strcpy(rec->annot,SVAL(*val,0));

val = p_find(blkptr,"DELFLG");
if (val->v_count == 0)
    *rec->delfil = '\0';
else
    strcpy(rec->delfil,SVAL(*val,0));

val = p_find(blkptr,"IN");
if (val->v_count == 0)
    *rec->fspec = '\0';
else
    strcpy(rec->fspec,SVAL(*val,0));

val = p_find(blkptr,"INDLTF");
if (val->v_count == 0)
    *rec->dltf = '\0';
else
    strcpy(rec->dltf,SVAL(*val,0));

val = p_find(blkptr,"INFILE");
if (val->v_count == 0)
    *rec->infile = '\0';
else
    strcpy(rec->infile,SVAL(*val,0));

val = p_find(blkptr,"PRINTER");
if (val->v_count == 0)
    *rec->pname = '\0';
else
    strcpy(rec->pname,SVAL(*val,0));

val = p_find(blkptr,"MARGIN");
if (val->v_count == 0)
    {
    if (strcmp(rec->pname,"LJ5000") == 0)
	rec->margin[0] = rec->margin[1] = .5;
    else
	rec->margin[0] = rec->margin[1] = .125;
    }
else if (val->v_count == 1)
    rec->margin[0] = rec->margin[1] = RVAL(*val,0);
else
    {
    rec->margin[0] = RVAL(*val,0);
    rec->margin[1] = RVAL(*val,1);
    }

val = p_find(blkptr,"MEDIA");
if (val->v_count == 0)
    *rec->film = '\0';
else
    strcpy(rec->film,SVAL(*val,0));

val = p_find(blkptr,"SPECIAL");
if (val->v_count == 0)
    *rec->special = '\0';
else
    strcpy(rec->special,SVAL(*val,0));

val = p_find(blkptr,"SPOTCOPY");
if (val->v_count == 0)
    *rec->spot = '\0';
else
    strcpy(rec->spot,SVAL(*val,0));

val = p_find(blkptr,"IMGOPT");
if (val->v_count == 0)
    *rec->imgnam = '\0';
else
    strcpy(rec->imgnam,SVAL(*val,0));

val = p_find(blkptr,"TITLE");
*rec->title[0] = *rec->title[1] = '\0';
for (cnt=0; cnt<val->v_count; cnt++)
    strcpy(rec->title[cnt],SVAL(*val,cnt));

val = p_find(blkptr,"DORRAN");
if (val->v_count == 0)
    *rec->dorran[0] = *rec->dorran[1] = *rec->dorran[2] = '\0';
else
    {
    for (cnt=0; cnt<val->v_count; cnt++)
        strcpy(rec->dorran[cnt],SVAL(*val,cnt));
    }

val = p_find(blkptr,"WEDGE");
if (val->v_count == 0)
    *rec->wedge = '\0';
else
    strcpy(rec->wedge,SVAL(*val,0));
val = p_find(blkptr,"STRETCH");
if (val->v_count == 0)
    *rec->stropt = '\0';
else
    strcpy(rec->stropt,SVAL(*val,0));
if ((prog == PMOD) &&
    (strcmp(host,rec->remote) != 0) &&
    (strcmp(name,rec->fspec) != 0))
    {
    sprintf(msg,"ERROR: File %s is on %s.  Cannot modify from remote system",
        rec->fspec,rec->remote);
    c_errmsg(msg," ",&nonfatal);
    rstat = E_FAIL;
    goto xit;
    }
else if ((prog == PADD) || (prog == PMOD))
    {
    if (chkimg(blkptr,rec) != E_SUCC)
	{
	rstat = E_FAIL;
	goto xit;
	}
    rstat = lj_setopt(printer,rec,&imgopt,TRUE);
    if (rstat != E_SUCC)
        goto xit;
    if (lj_format_annot(rec,&imgopt) != E_SUCC)
	{
	sprintf(emsg,"%s %s %s",
		"Image to narrow for annotation.",
		"User can increase image width by increasing parameter MARGIN(1).",
		"Annotation will be truncated as follows:");
	c_errmsg(emsg," ",&nonfatal);
	for (cnt=0; cnt<A_MAX; cnt++)
	    {
	    if (strlen(imgopt.fannot[cnt]) > 0)
	        c_errmsg(imgopt.fannot[cnt]," ",&nonfatal);
	    }
	}
    strcpy(rec->stflg,CF_QUEUE);
    rec->errtext[0] = '\0';
    }
/*
        Check priority value
        DSB has decided all users should have the ability to set priority
        between 1-9 (rather then reserving priorities 1-4 for privlege
        users).  The check is still here; however, the check value was
        changed from 5 to 1. (kpg 5/99).
*/
suflg = lj_chkuser(printer,pcnt,rec);
if (rec->prio != NULLVAL)
    {
    if ((prog == PMOD) && (suflg < SU_PRIV) && 
	(rec->prio < 1) && (priority > rec->prio))
	{
	c_errmsg("User not allowed to set priority less than 1",
		" ",&nonfatal);
	sprintf(msg,"Priority has been reset to original value of %d",priority);
	c_errmsg(msg," ",&nonfatal);
	rec->prio = priority;
	}
    if ((prog == PADD) && (suflg < SU_PRIV) && (rec->prio < 1))
	{
	c_errmsg("User not allowed to set priority less than 1",
		" ",&nonfatal);
	c_errmsg("Priority has been reset to a value of 1"," ",&nonfatal);
	rec->prio = 1;
	}
    }
len = (long) strlen(rec->infile);
c_up2low(rec->infile,&len);
len = (long) strlen(rec->dltf);
c_up2low(rec->dltf,&len);
if (prog != PMOD)
    {
    for (cnt=0; cnt<NLUT; cnt++)
	{
	rec->enhlut[cnt].ne = 0;
	*rec->enhlut[cnt].desc = '\0';
	}
    }
cplut(rec,tlut,FR_REC);
switch (prog)
    {
    case PREP:   enhflg = FALSE;	break;
    case PADD:   enhflg = TRUE;		break;
    case PMOD:   enhflg = FALSE;	break;
    }
if ((prog != PREP) && (strlen(rec->infile) == 0))
    {
    if (prog == PMOD)
	c_errmsg("Enhancement LUT is not interactively updated"," ",&nonfatal);
    else if (chkbat(blkptr))
	{
	sprintf(emsg,"%s  %s","INFILE parameter set to NULL running in batch.",
		"A one-to-one enhancement LUT applied to image.");
	c_errmsg(emsg," ",&nonfatal);
	rec->enhlut[0].ne = rec->enhlut[1].ne = rec->enhlut[2].ne = 2;
	rec->enhlut[0].in[0] = rec->enhlut[1].in[0] = rec->enhlut[2].in[0] = 0;
	rec->enhlut[0].out[0] = rec->enhlut[1].out[0] = 
		rec->enhlut[2].out[0] = 255;
	}
    else 
	setenh(rec->color,NULLVAL,"",tlut,enhflg);
    }
else if ((prog != PREP) && (strlen(rec->infile) > 0))
    {
    if (prog == PMOD)
	c_errmsg("Enhancement LUT is not updated"," ",&nonfatal);
    else if (strlen(rec->dltf) == 0)	/* read LUT data from ASCII file */
        {
        status = c_cmhost(rec->infile,hname);
	if (status != E_SUCC)
	    {
	    c_errmsg("Error translating LUT name into a host file name",
		" ",&fatal);
	    }
        status = rdlut(rec->color,NULLVAL,hname,"ENH",tlut);
        if (status != E_SUCC)
	    {
            sprintf(msg,"Error setting ENH lut values from file %s",
			rec->infile);
	    c_errmsg(msg," ",&fatal);
	    }
        }
    else				/* read LUT data from DLTF file */
        {
	if (strlen(rec->infile) == 0)
	    {
	    ptr = strchr(rec->fspec,'(');	/* strip off image options */
	    if (ptr != NULL)
		*ptr = '\0';
	    sprintf(rec->infile,"%s;dltf",rec->fspec);
	    if (ptr != NULL)
		*ptr = '(';
            status = c_cmhost(rec->infile,hname);
	    if (status != E_SUCC)
	        {
	        c_errmsg("Error converting DLTF TAE name to host name",
		    " ",&fatal);
	        }
	    }
	else
	    {
	    elem = 0;
	    access = IREAD;
	    status = c_getnam(blkptr,"INFILE",&elem,hname,"dltf",&access);
	    if (status != E_SUCC)
	        {
                sprintf(msg,"Error accessing DLTF file %s",rec->infile);
	        c_errmsg(msg," ",&fatal);
	        }
            status = c_cmtae(hname,rec->infile); 
/*            status = c_cmhost(hname,rec->infile);  */
	    if (status != E_SUCC)
	        {
	        c_errmsg("Error converting DLTF host name to TAE name",
		    "fire",&fatal);
	        }
	    }
        status = rddltf(rec->color,hname,rec->dltf,tlut);
        if (status != E_SUCC)
	    {
            sprintf(msg,"Error setting ENH lut values from file %s entry %s",
			rec->infile,rec->dltf);
	    c_errmsg(msg," ",&fatal);
	    }
        }
    }
cplut(rec,tlut,TO_REC);
for (cnt=0; cnt<NLUT; cnt++)		/* Null out stretch annotation */
    *rec->stretch[cnt] = '\0';
if (strcmp(rec->stropt,"YES") == 0)	/* Build stretch annotation string */
    {
    maxchar = (imgopt.maxns / 
		(atable[imgopt.font_nmr].ns * imgopt.font_scale) /
		rec->nb) - 4;
    for (i=0; i<rec->nb; i++)
        {
	strcpy(rec->stretch[i],"");
        for (j=0; j<rec->enhlut[i].ne; j++)
            {
            ival = (int) rec->enhlut[i].out[j];
            if (j == 0)
                sprintf(temp,"%d-%d",rec->enhlut[i].in[j],ival);
            else
                sprintf(temp,",%d-%d",rec->enhlut[i].in[j],ival);
            if (strlen(rec->stretch[i]) + strlen(temp) > maxchar)
                {
                c_errmsg("Stretch point annotation is too long"," ",&nonfatal);
                break;
                }
            strcat(rec->stretch[i],temp);
            }
	strcat(rec->stretch[i],"    ");
        }
    }
if ((prog == PADD) || (prog == PMOD))
    {
    if (rec->ncop * rec->npanel > 1)
        {
        sprintf(msg,"Total frames = %d   (%d panels multiplied by %d copies)",
                rec->ncop * rec->npanel,rec->npanel,rec->ncop);
        c_errmsg(msg," ",&nonfatal);
        }
    }
xit:
return(rstat);
}	/*  END of the RPARM subroutine  */

static long chkimg
(
	struct PARBLK *blkptr,
	struct recfmt *rec
)
{
struct DDR ddr;
long cnt;
long cnt1;
long elem;
long imglimit;
long nbands[NMRIMG];
long nonfatal = 1;
long qflag[NMRIMG][2];
long rstat;
long same_window;
long window[NMRIMG][4];
char emsg[256];
char host[CMLEN];
char hnames[NMRIMG][CMLEN];
char tnames[NMRIMG][CMLEN];
TIFF *tif;
uint32 nl, ns;
uint16 nb;

rstat = E_SUCC;
(void) gethostname(host,CMLEN);
if (strcmp(host,rec->remote) != 0)
    {
    sprintf(emsg,"File is on remote system %s, unable to verify file",
        rec->remote);
    c_errmsg(emsg," ",&nonfatal);
    goto xit;
    }
elem = 0;
imglimit = NMRIMG;
if (strcmp(rec->ftype,"las") == 0)
    {
    rstat = c_getin(blkptr,"IN",&elem,tnames,hnames,&rec->nimg,window,
	rec->bands,nbands,&rec->nb,&imglimit,&same_window,qflag);
    if ((rec->mapscale != NULLVAL) && (rec->pixsize == NULLVAL))
        {
        rstat = c_getddr(hnames[0],&ddr);
        if ((ddr.valid[4] != TRUE) || (ddr.valid[5] != TRUE))
	    {
 	    sprintf(emsg,"%s %s",
		"ERROR: LAS DDR ground units and/or distance not valid.",
		"User should specify value for parameter PIXSIZE.");
	    c_errmsg(emsg," ",&nonfatal);
	    rstat = E_FAIL;
	    }
	else if (strcmp(ddr.proj_units,"meters") != 0)
	    {
 	    sprintf(emsg,"%s %s",
		"ERROR: LAS DDR ground units not specified in meters.",
		"User should specify value for parameter PIXSIZE.");
	    c_errmsg(emsg," ",&nonfatal);
	    rstat = E_FAIL;
	    }
	else if (ddr.pdist_y != ddr.pdist_x)
	    {
 	    sprintf(emsg,"%s %s",
		"ERROR: LAS DDR ground line/sample units are not equal.",
		"User should specify value for parameter PIXSIZE.");
	    c_errmsg(emsg," ",&nonfatal);
	    rstat = E_FAIL;
	    }
	else
	    rec->pixsize = ddr.pdist_y;
	}
    }
else if (strcmp(rec->ftype,"tif") == 0)
    {
    rec->nimg = 0;
    c_cmtae(rec->fspec,rec->tae[0]); 
/*    c_cmhost(rec->fspec,rec->tae[0]); */
    c_cmhost(rec->tae[0],rec->fspec);
    tif = TIFFOpen(rec->fspec,"r");
    if (!tif)
        {
        sprintf(emsg,"Error opening TIFF file %s",rec->fspec);
        c_errmsg(emsg," ",&nonfatal);
        rstat = E_FAIL;
        goto xit;
        }
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &nl);
    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH,  &ns);
    TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &nb);
    window[0][SL] = 1;
    window[0][SS] = 1;
    window[0][NL] = (long) nl;
    window[0][NS] = (long) ns;
    rec->nb = nb;
    }
if (rstat == E_SUCC)
    {
    for (cnt=0; (cnt<rec->nimg-1) && (rstat==E_SUCC); cnt++)
        {
	for (cnt1=0; (cnt1<4) && (rstat==E_SUCC); cnt1++)
	    {
            if (window[cnt][cnt1] != window[cnt+1][cnt1])
	        {
	        c_errmsg("Window sizes of input images must be the same",
		    " ",&nonfatal);
	        rstat = E_FAIL;
	        }
	    }
	}
    rec->sl = window[0][SL];
    rec->ss = window[0][SS];
    rec->nl = window[0][NL];
    rec->ns = window[0][NS];
    if (rec->nb == 3)
	strcpy(rec->color,"RGB");
    else if (rec->nb == 1)
	strcpy(rec->color,"BW");
    else
	{
	sprintf(emsg,"%d number of bands not supported, must be 1 or 3 bands",
		rec->nb);
	c_errmsg(emsg," ",&nonfatal);
	rstat = E_FAIL;
	}
    for (cnt=0; cnt<rec->nimg; cnt++)
	{
	strcpy(rec->tae[cnt],tnames[cnt]); 
/*        c_cmhost(tnames[cnt],rec->tae[cnt]); */
	}
    }
xit:
return(rstat);
}
