/******************************************************************************

	THIS SET OF FUNCTIONS CONTAINS THE ROUTINES TO EDIT A SPECIFIED
	ENHANCEMENT LUT FILE.

******************************************************************************/


/******************************************************************************
	SETENH is the control routine which reads the specified enhancement
	LUT files (RGB, CMY, CMYK, BW).  It also initializes the TAE
	terminal I/O package and displays the file contents onto the 
	screen.  It calls the subroutine modenh to edit the LUT files.

ERROR CONDITIONS:
	1.  ERROR -- Terminal not supported by TAE
		Error occurs if the requests are to be printed to the terminal,
		but the terminal is not supported by TAE.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           8/93       K. Gacke    initial development -- copied from QLP

PROJECT:        Unix Color FIRE

******************************************************************************/

/******************************************************************************
	GLOBAL DECLARAIONS
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "lj.h"

#ifdef vms
#define unlink delete
#endif

#define CONT	 10
#define STOP	 11

#define STOPVAL  -1		/* stop entering points */
#define DELVAL   -2		/* point should be deleted */
#define NOVAL    -3		/* map points not entered */

#define ALL	  4

static long fatal = -1;
static long nonfatal = 1;

static COUNT ncol;
static COUNT nlin;

static long curpg = 1;		/* current page displayed */
static long maxpg = 1;		/* maximum number of pages */
static long page  = 1;		/* new page to be displayed */

static int ccmdlin(void);
static int chpt
(
    long color,
    long pt,
    long inval,
    double outval,
    struct lut *lut
);
static int delpt
(
    long color,
    long pt,
    struct lut *lut
);
static int modenh
(
    long nlut,
    struct lut enhlut[]
);
static int modpt
(
    int pt,
    long nlut,
    struct lut enhlut[],
    long initval,
    long flag
);
static int prntenh
(
    long color,
    int nlut,
    struct lut enhlut[],
    long page
);
static int prntpt
(
    int color,
    int pt,
    struct lut *lut
);
static long savenh
(
    char *colorord,
    int fnmr,
    char *fname,
    long nlut,
    struct lut enhlut[]
);
static int validate_enh
(
    long curlut,
    struct lut *enhlut
);

/*****************************************************************************/

int setenh
(
    char *colorord,
    int fnmr,
    char *fname,
    struct lut enhlut[],
    long enhflg
)
{
CODE	type;
long	cnt;
long 	nlut;

/*
   initialize the TAE terminal I/O package
*/
t_init(&nlin,&ncol,&type);
if (type != T_CRT)
   c_errmsg("ERROR -- Terminal not supported by TAE","fire",&fatal);
/*
   Clear the terminal screen and write the header lines for the enhancement
   LUT to the terminal
*/
t_clear();
if (strcmp(colorord,"RGB") == 0)
    {
    t_output(3,1,"PT #     RED       GREEN      BLUE");
    t_output(4,1,"       IN  OUT    IN  OUT    IN  OUT");
    nlut = 3;
    }
else if (strcmp(colorord,"CMY") == 0)
    {
    t_output(3,1,"PT #     CYAN      MAGENTA    YELLOW");
    t_output(4,1,"       IN  OUT    IN  OUT    IN  OUT");
    nlut = 3;
    }
else if (strcmp(colorord,"CMYK") == 0)
    {
    t_output(3,1,"PT #     CYAN      MAGENTA    YELLOW     BLACK");
    t_output(4,1,"       IN  OUT    IN  OUT    IN  OUT    IN  OUT");
    nlut = 4;
    }
else if (strcmp(colorord,"BW") == 0)
    {
    t_output(3,1,"PT #     BW");
    t_output(4,1,"       IN  OUT");
    nlut = 1;
    }
if (enhflg)
    {
    for (cnt=0; cnt<nlut; cnt++)
	{
	*enhlut[cnt].desc = '\0';
	enhlut[cnt].ne = 2;
	enhlut[cnt].in[0] = 0;
	enhlut[cnt].out[0] = 0;
	enhlut[cnt].in[1] = 255;
	enhlut[cnt].out[1] = 255;
	}
    }
/*
   call the following routines:
      	prntenh to print the contents of the enhancement files
	modenh  to edit the LUT file contents
	savenh  to write the new LUT data back into the enhancement files
*/
curpg = maxpg = page = 1;
prntenh(ALL,nlut,enhlut,page);
modenh(nlut,enhlut);
(void) savenh(colorord,fnmr,fname,nlut,enhlut);
return(1);
}

/* 
   ccmdlin -- clear command lines (terminal lines 22, 23, 24)
*/
static int ccmdlin(void)
{
t_lclear(nlin-3,1);
t_lclear(nlin-2,1);
t_lclear(nlin-1,1);
return(1);
}

static int chpt
(
    long color,
    long pt,
    long inval,
    double outval,
    struct lut *lut
)
{
if (inval != NOVAL) 
    lut->in[pt] = inval;
if (outval != NOVAL) 
    lut->out[pt] = outval;
if ((lut->ne == pt) && (inval != NOVAL) && (outval != NOVAL))
    ++lut->ne;
prntpt(color,pt,lut); 
validate_enh(color,lut);
return(1);
}

static int delpt
(
    long color,
    long pt,
    struct lut *lut
)
{
long i;

if ((lut->ne > 0) && (pt < lut->ne))
    {
    --lut->ne;
    prntpt(color,lut->ne,lut);
    for (i=pt;i<lut->ne;i++)
        {
	lut->in[i] = lut->in[i+1];
        lut->out[i] = lut->out[i+1];
        }
    }
return(1);
}

/******************************************************************************
	MODENH is a control to routine to modify the contents of an 
	enhancement LUT file (red, green, and blue).  It queries the 
	user to edit the entire file, or to modify individual map points.
	In either case the routine modpt is called to modify the specified
	map points of the LUT.
******************************************************************************/

static int modenh
(
    long nlut,
    struct lut enhlut[]
)
{
CODE	term;
int 	cnt;
int	status,i,pt;
char	buf[150],buf1[50],buf2[50],buf3[50];

/*
   Copy the query questions into the various buffers
*/
strcpy(buf1,"Are these parameters correct? (Y or N): ");
strcpy(buf2,"Modify entire file or points? (E, I, or P): ");
strcpy(buf3,"Is the description correct? (Y or N): ");
/*
   Query the user to see if the map points are correct.  While the map points
   are not correct, edit the map points.
*/
ccmdlin();
t_output(nlin-3,1,buf1);
t_read(buf,&term);
while ((buf[0] != 'y') && (buf[0] != 'Y'))
   {
   t_lclear(nlin-3,1);
   t_output(nlin-3,1,buf2);
   t_read(buf,&term);
   t_lclear(nlin-3,1);
/*
	if the entire file is to be edited then enter a for loop to process
	each map point of the file.  For each map point call modpt to modify
	the current map point.  If the status from modpt is STOP, then break
	out of the for loop
*/
   if ((buf[0] == 'e') || (buf[0] == 'E'))
      {
      for (cnt=0; cnt<nlut; cnt++)
	 {
	 *enhlut[cnt].desc = '\0';
	 enhlut[cnt].ne = 0;
	 }
      page = 1;
      prntenh(ALL,nlut,enhlut,page);
      for (i=0;i<MAXPT;i++)
         {
	 status = modpt(i,nlut,enhlut,NOVAL,FALSE);
	 if (status == STOP)
	    break;
         }
      }
   else if ((buf[0] == 'i') || (buf[0] == 'I'))
      { 
      t_output(nlin-3,1,"Point to insert: ");
      t_read(buf,&term);
      if (strlen(buf) > 0)
         sscanf(buf,"%d",&pt);
      if ((pt < STOPVAL) || (pt >= MAXPT))
         c_errmsg("ERROR -- Invalid point to insert (Range -1 - 255)",
			      "fire",&nonfatal);
      if (pt == STOPVAL)
	 break;
      else
	 {
         page = pt / 16 + 1;
         if (page != curpg)
            prntenh(ALL,nlut,enhlut,page);
	 for (cnt=0; cnt<nlut; cnt++)
	    {
	    for (i=enhlut[cnt].ne; i>pt; i--)
	       {
	       enhlut[cnt].in[i] = enhlut[cnt].in[i-1];
	       enhlut[cnt].out[i] = enhlut[cnt].out[i-1];
	       }
	    ++enhlut[cnt].ne;
	    }
  	 status = modpt(pt,nlut,enhlut,DELVAL,TRUE);
	 ccmdlin();
	 }
      }
/*
	if the user specifies individual map points to be modified, enter a
	infinite loop which queries the user as to which map point is to be 
	modified.  If the map point is STOPVAL then break out of the loop, 
	otherwise call modpt to modify the current map point.  If the return 
	status of modpt is STOP then break out of the loop.
*/
   else
      {
      for (;;)
         {
         t_output(nlin-3,1,"Point to modify: ");
         t_read(buf,&term);
         if (strlen(buf) > 0)
            sscanf(buf,"%d",&pt);
         if ((pt < STOPVAL) || (pt > 255))
            c_errmsg("ERROR -- Invalid point to modify (Range -1 - 255)",
			      "fire",&nonfatal);
	 if (pt == STOPVAL)
	    {
	    ccmdlin();
	    break;
	    }
	 else
            {
            page = pt / 16 + 1;
            if (page != curpg)
               prntenh(ALL,nlut,enhlut,page);
  	    status = modpt(pt,nlut,enhlut,NOVAL,FALSE);
            }
	 if (status == STOP)
	    break;
	 }
      }
/*
     Query the user to see if the map points for the LUT file are correct.
*/
   t_lclear(nlin-3,1);
   t_output(nlin-3,1,buf1);
   t_read(buf,&term);
/*******   t_lclear(nlin-2,1);  **********/
   t_lclear(nlin-1,1);
   }
/*
   Query the user to see if the LUT file description is correct.  If not enter
   a while loop which terminates only after the user indicates that the 
   description is correct.  Within the loop the user is asked to input the 
   new file description.
*/
ccmdlin();
t_output(nlin-3,1,buf3);
t_read(buf,&term);
while ((buf[0] == 'n') || (buf[0] == 'N'))
   {
   t_lclear(nlin-3,1);
   t_output(nlin-3,1,"Input the description: ");
   t_read(buf,&term);
   buf[79] = '\0';
   sprintf(enhlut[0].desc,"%-80s",buf);
   t_lclear(3,1);
   t_output(2,14,enhlut[0].desc);
   ccmdlin();
   t_output(nlin-3,1,buf3);
   t_read(buf,&term);
   }
ccmdlin();
return(1);
}

/******************************************************************************
	MODPT is a routine to modify the red, green, and blue enhancement 
	values for a specified map point.
******************************************************************************/

static int modpt
(
    int pt,
    long nlut,
    struct lut enhlut[],
    long initval,
    long flag
)
{
CODE	term;
int	cnt;
int	maxne;
int	rstat,j;
char	buf[150],buf1[20],buf2[20];
int	col,inval;
float	outval;

/*
   Error check to insure the map point is valid
*/
if (pt >= MAXPT)
   {
   c_errmsg("ERROR -- Invalid LUT point to modify","fire",&nonfatal);
   return(STOP);
   }
maxne = 0;
for (cnt=0; cnt<nlut; cnt++)
    maxne = (maxne > enhlut[cnt].ne) ? maxne : enhlut[cnt].ne;
if (pt > maxne)
   {
   c_errmsg("ERROR -- Invalid LUT point to modify","fire",&nonfatal);
   return(STOP);
   }
rstat = CONT;
strcpy(buf1,"In: ");
strcpy(buf2,"Out: ");
ccmdlin();
/*
   For each color read the input and output values.  If input value is STOPVAL 
   then return a status code of STOP to exit the edit routine.  If the input
   value is DELVAL then delete the map point for that color from the LUT by 
   shifting all other values down one element.  If the input and output values
   were specified (if not specified the values remain to be NOVAL) then assign
   the new values to the LUT.
*/
for (j=0,col=2;j<nlut;j++,col+=19)
   {
   outval = inval = initval;
   sprintf(buf,"Point %2d",pt);
   t_output(nlin-3,1,buf);
   t_output(nlin-1,col,buf1);
   t_read(buf,&term);
   if (strlen(buf) > 0)
      {
      sscanf(buf,"%d",&inval);
      if ((inval < DELVAL) || (inval > 255))
         {
	 ccmdlin();
         t_pos(nlin-2,1);
         c_errmsg("ERROR -- Invalid input value (Range -2 - 255)",
		 "fire",&nonfatal);
         return(STOP);
         }
      }
   if (inval == STOPVAL)
      {
      ccmdlin();
      return(STOP);
      }
   else if (inval == DELVAL) 		/* delete map point */
        {
	delpt(j,pt,&enhlut[j]);
        }
   else
	{
   	t_output(nlin-1,col+8,buf2);
   	t_read(buf,&term);
   	if (strlen(buf) > 0)
      	    {
            sscanf(buf,"%f",&outval);
            if ((outval < 0) || (outval > 255))
                {
		ccmdlin();
	 	t_pos(nlin-2,1);
         	c_errmsg("ERROR -- Invalid output value (Range 0 - 255)",
	         	"fire",&nonfatal);
         	return(STOP);
	 	}
      	    }
	chpt(j,pt,inval,outval,&enhlut[j]);
   	}
   if ((flag) || (inval == DELVAL))
      {
      page = pt / 16 + 1;
      if (page != curpg)
         prntenh(ALL,nlut,enhlut,page);
      else
         prntenh(j,nlut,enhlut,page);
      }
   }
return(rstat);
}

/******************************************************************************
	PRNTENH prints all map points (red, green, and blue) of an enhancement 
	LUT file, and the file description.
******************************************************************************/
static int prntenh
(
    long color,
    int nlut,
    struct lut enhlut[],
    long page
)
{
int  cnt;
int  i,lin,start;
int  maxne;
char buf[150];

/*
   Write the new LUT file description
*/
t_lclear(2,1);
sprintf(buf,"DESCRIPTION: %-65.65s",enhlut[0].desc);
t_output(2,1,buf); 
lin = 5;
start = (page -1) * 32;
if (color != ALL)
    {
    for (i=start; i<start+16; i++)
        {
        if (i >= enhlut[color].ne)
           break;
        prntpt(color,i,&enhlut[color]);
        }
    }
else
    {
    for (i=lin;i<=nlin-3;i++)	
        t_lclear(i,1);
    for (i=start; i<start+16; i++)
        {
	for (cnt=0; cnt<nlut; cnt++)
	    {
	    if (i < enhlut[cnt].ne)
         	prntpt(cnt,i,&enhlut[cnt]);
	    }
         }
    }
maxne = 0;
for (cnt=0; cnt<nlut; cnt++)
    maxne = (maxne > enhlut[cnt].ne) ? maxne : enhlut[cnt].ne;
maxpg = (maxne - 1) / 16 + 1;
curpg = page;
sprintf(buf,"page %d of %d",page,maxpg);
t_output(1,ncol-12,buf);
t_pos(nlin-2,1);
return(1);
}


/******************************************************************************
	PRNTPT prints the red, green, and blue enhancement LUT values for
	a specified map point.
******************************************************************************/
static int prntpt
(
    int color,
    int pt,
    struct lut *lut
)
{
char	buf[150];
int	col,lin;
int     locpt;

col = 7 + (11 * color);
locpt = pt % 16;
lin = 5 + locpt;
sprintf(buf,"%2d",pt);
t_output(lin,2,buf);
if (pt < lut->ne)
    {
    sprintf(buf,"%3d  %3.0f",lut->in[pt],lut->out[pt]);
    t_output(lin,col,buf);
    }
else 
    t_output(lin,col,"        ");
return(1);
}

/******************************************************************************
	SAVENH is a routine to write the new enhancement LUT values to 
	their appropriate files.
******************************************************************************/
static long savenh
(
    char *colorord,
    int fnmr,
    char *fname,
    long nlut,
    struct lut enhlut[]
)
{
CODE    term;
long    cnt;
long	len;
long	i,index;
long	hival;
long	loc_stat;
long	savflg;
FILE	*chan;
char    msg[100];
char    enhname[CMLEN];
char    *color[NLUT];
char    buf[100];
char	*ptr, *ptr1;
char    tname[CMLEN];

strcpy(enhname,fname);
loc_stat = 0; 
if (strcmp(colorord,"RGB") == 0)
    {
    color[0] = "RED";
    color[1] = "GREEN";
    color[2] = "BLUE";
    }
else if (strcmp(colorord,"CMY") == 0)
    {
    color[0] = "CYAN";
    color[1] = "MAGENTA";
    color[2] = "YELLOW";
    }
else if (strcmp(colorord,"CMYK") == 0)
    {
    color[0] = "CYAN";
    color[1] = "MAGENTA";
    color[2] = "YELLOW";
/*    color[3] = "KBLACK"; */
    }
else if (strcmp(colorord,"BW") == 0)
    {
    color[0] = "RED";
    color[1] = "GREEN";
    color[2] = "BLUE";
    nlut = 3;
    enhlut[1].ne = enhlut[2].ne = enhlut[0].ne;
    for (cnt=0; cnt<enhlut[0].ne; cnt++)
	{
	enhlut[1].in[cnt] = enhlut[2].in[cnt] = enhlut[0].in[cnt];
	enhlut[1].out[cnt] = enhlut[2].out[cnt] = enhlut[0].out[cnt];
	}
    }
if (fnmr == NULLVAL)
    {
    if (strlen(enhname) == 0)
	{
        t_lclear(nlin-2,1);
        t_output(nlin-2,1,
			"Do you want to save ENH LUT to file? (Y or N) ");
        t_read(buf,&term);
        if ((*buf == 'n') || (*buf == 'N'))
	    savflg = FALSE;
	else
	    {
	    savflg = TRUE;
            t_lclear(nlin-2,1);
            t_output(nlin-2,1,"Enter file name with no extension: ");
            t_read(tname,&term); 
	    (void) c_cmhost(tname,enhname);
	    }
	}
    if (savflg)
	{
        ptr = strrchr(enhname,'.');
	ptr1 = strrchr(enhname,']');
        if (((ptr != NULL) && (ptr1 == NULL)) ||
	    ((ptr != NULL) && (ptr1 != NULL) && (ptr1 < ptr)))
	    {
	    *ptr = '\0';
	    }
        strcat(enhname,".EN");
	}
    }
else
    {
    savflg = TRUE;
    sprintf(enhname,"CFLUT:%2d.EN",fnmr);
    }
ccmdlin();
if (savflg)
    {
    t_output(nlin-2,1,"Saving data please wait.\n");
    }
for (index=0; index<nlut; index++)	/* save LUT data */
    {
    if (savflg)
	{
        sprintf(buf,"%s%1.1c",enhname,*color[index]);
	len = (long)strlen(buf);
	c_up2low(buf,&len);
        (void) unlink(buf);
        if ((chan = fopen(buf,"w")) == NULL)
	    {
	    sprintf(msg,"Error opening enhancement LUT %s",buf);
            c_errmsg(msg,"fire",&fatal);
	    }
        fprintf(chan,"%-80s\n",enhlut[index].desc);
	}
    hival = 0;
    for (i=0;i<enhlut[index].ne;i++)
        {
        if (enhlut[index].in[i] == 255)
            hival = 1;
        if ((i > 0) && (enhlut[index].in[i] < enhlut[index].in[i-1]))
            {
            t_pos(21,1);
	    sprintf(msg,"ERROR: %s %s",color[index],
			"enhance file input values are not ascending");
            c_errmsg(msg,"fire",&nonfatal);
            loc_stat = -1;
            }
        if ((i > 0) && (enhlut[index].in[i] == enhlut[index].in[i-1]))
            {
            t_pos(21,1);
	    sprintf(msg,"ERROR: %s %s",color[index],
              		"enhance file has two equal input brightness values");
            c_errmsg(msg,"fire",&nonfatal);
            loc_stat = -1;
            }
	if (savflg)
	    {
            fprintf(chan," %3d   %3.0f\n",
			enhlut[index].in[i],enhlut[index].out[i]);
	    }
        if ((i+1 == enhlut[index].ne) && (hival == 0))
	    {
	    enhlut[index].in[i+1] = 255;
	    enhlut[index].out[i+1] = 255;
	    ++enhlut[index].ne;
	    }
        }
    if (savflg)
	{
        fflush(chan);
        fclose(chan);
	}
    }
if (loc_stat == -1)
    c_errmsg("Fatal error in enhancement LUT file","fire",&fatal);
return(loc_stat);
}


static int validate_enh
(
    long curlut,
    struct lut *enhlut
)
{
CODE term;
long i;
long status;
char msg[100];

status = 0;
for (i=0;i<enhlut->ne;i++)
    {
    if ((i > 0) && (enhlut->in[i] < enhlut->in[i-1]))
        {
	sprintf(msg,"ERROR: LUT %d %s",curlut+1,
			"enhance file input values are not ascending");
	status = -1;
	break;
        }
    if ((i > 0) && (enhlut->in[i] == enhlut->in[i-1]))
        {
	sprintf(msg,"ERROR: LUT %d %s",curlut+1,
              		"enhance file has two equal input brightness values");
	status = -1;
	break;
        }
    }
if (status == -1)
    {
    ccmdlin();
    t_output(nlin-3,1,msg);
    t_output(nlin-2,1,"Enter <CR> to continue:");
    t_read(msg,&term);
    ccmdlin();
    }
return(1);
}
