/******************************************************************************
FUNCTION:     	GET_SCENEID 

PURPOSE:        Obtain scene ID from leader file and return to TAE 

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      6/96   C. Engebretson   Original development

ALGORITHM REFERENCES:

	"Landsat Technical Working Group (LTWG) Format for Thematic 
	Mapper CCT, Full Scene", EOSAT, January 1, 1990

	"TM CCT Format From TRSA (Thailand)"

*******************************************************************************/

#include <stdio.h>   
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"

int main
(void)
{
struct PARBLK parblk;		/* TAE parameter block			*/
struct VARIABLE *p_find();	/* Function to retrieve TAE variable	*/

FILE *inf;			/* Input file				*/
unsigned char *buffer;		/* Leader file buffer			*/
long recsize = 4320;		/* Size of leader file records		*/
char leader[CMLEN];		/* Name of input leader file		*/
char tempstr[255];		/* General-purpose temporary string 	*/
char *str_ptr[1];		/* Temporary pointer for q_string()	*/
long bytes_read;		/* Error checking on fread()		*/
long zero = 0;			/* Constant of zero 			*/
long fatal = -1;		/* Flag for fatal error			*/ 
long std_out;			/* Standard output			*/
long mode = IREAD;		/* Flag for c_getnam() 			*/


/* Initialize LAS
-----------------*/ 
c_inlas(&parblk, &std_out);


/* Retrieve the input parameters
--------------------------------*/
if (c_getnam(&parblk, "LEADER", &zero, leader, "lead", &mode) != E_SUCC)
	c_errmsg("Fatal error encountered", "ltwg123-fatal", &fatal);


/* Open the input files for reading
-----------------------------------*/
inf = fopen(leader, "r");
if (!inf) c_errmsg("Unable to open leader file for reading",
		   "ltwg123-file", &fatal);


/* Allocate buffer memory and read data in from the leader/VDF file
-------------------------------------------------------------------*/

/* The leader file is composed of 5 records of recsize bytes each, which means
   that we need to allocate 21600 bytes of space to hold it all. */

buffer = (unsigned char *) malloc (5 * recsize);
if (!buffer)
	c_errmsg("Error allocating dynamic memory", "ltwg123-alloc", &fatal);
bytes_read = (long) fread(buffer, sizeof(unsigned char), 5 * recsize, inf);
if (bytes_read != 5 * recsize)
	c_errmsg("Error reading data from leader file", "ltwg123-read", &fatal);
fclose(inf);


/* Determine the scene ID 
-------------------------*/
strncpy(tempstr, (char *) &buffer[recsize + 36], 16);   /* Scene ID           */
tempstr[16] = 0;
str_ptr[0] = tempstr;
if (q_string(&parblk, "SCENEID", 1, str_ptr, P_UPDATE) != SUCCESS)
        c_lasmsg(TRUE, "ltwg123-pass", "Error passing variables to TAE");
if (q_out(&parblk) != SUCCESS)
        c_lasmsg(TRUE, "ltwg123-pass", "Error passing variables to TAE");

/* Clean up
-----------*/
free(buffer);

return EXIT_SUCCESS;

}
