/******************************************************************************
FUNCTION:	MAKEPATH

PURPOSE:	This function creates a fully-qualified path to the CD drive
		the requested CD was mounted on.  For example, if MOUNTCD
		returned "cd1" and the current system was sg2, the path
		would be "/sg2@cd1".

ALGORITHM REFERENCES:	none
******************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include "las.h"

FUNCTION makepath(hpath)

char hpath[CMLEN];

{
long i;				/* Loop counter				*/
long local = TRUE;		/* Flag for a local or remote mount	*/
long length = 100;		/* Length of machine name           	*/
char newpath[CMLEN];		/* Temporary spot for built path	*/
char machine[100];		/* Name of host machine			*/

newpath[0] = '/';		/* Always begin pathname with a "/" 	*/
newpath[1] = '\0';

/* If the value returned by MNTCD does NOT have a "@" sign, assume that the
   cd drive is local to the machine that LTWGCD is running on.
---------------------------------------------------------------------------*/

for (i = 0; i < strlen(hpath); i++)
    if (hpath[i] == '@') 
	local = FALSE;

if (!local)

    /* If a "@" was found in the drive name, assume that it is also the name
       of the directory where the CD is mounted.
    -----------------------------------------------------------------------*/
    strcat(newpath, hpath);
else
    { 

    /* Since the drive returned by MNTCD does not fully specify the correct
       directory, prefix it with the current system name followed by an "@".
       For example, "cd1" would become "/sg2@cd1", where "sg2" is the machine
       name as found by gethostname().
    -----------------------------------------------------------------------*/
    gethostname(machine, length);
    strcat(newpath, machine);
    strcat(newpath, "@");
    strcat(newpath, hpath);
    }

strcat(newpath, "/");		/* End path with another "/" 		   */
strcpy(hpath, newpath);		/* Copy new path to original parameter 	   */
}

