/*****************************************************************************
FUNCTION:	PROCESS_DIR

PURPOSE:	This function scans files in a CD directory, looking for VDF
		and leader files.  If it finds any, it then extracts the
		necessary information from them (path number, row number, and
		acquisition date) to check if this is the image the user
		wants.	The function will call itself recursively for each 
		subdirectory within the current directory until the proper
		image is found.  If no matching image is found, the value
		of foundpath will remain NULL.

ALGORITHM REFERENCES:	none
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"

FUNCTION process_dir(dirname, path, row, date, foundpath)
     
char dirname[CMLEN];	/* Name of the directory to traverse		*/ 
long path;		/* WRS path number				*/
long row;		/* WRS row number	 			*/
char date[9];		/* Acquisition date				*/
char foundpath[CMLEN];	/* Directory containing the correct image	*/

{
char filename[30][CMLEN];/* Array of filenames read from directory	*/
char filespec[CMLEN];	/* File specification to search for		*/
char leafile[CMLEN];	/* Name of leader file				*/
char *ptr;		/* Temporary character pointer			*/
long numfiles;		/* Number of files returned by c_getfiles()	*/
long recsize;		/* Size of leader file records			*/
long i;			/* Loop counter 				*/
long firstcall = TRUE;	/* Flag for first call to c_getfiles		*/

/* If image has already been found, return immediately (unwind stack)
---------------------------------------------------------------------*/
if (strlen(foundpath) != 0)
    return;

/* Get up to 30 filenames from the current directory for each pass     
------------------------------------------------------------------*/
strcpy(filespec, dirname);
strcat(filespec, "*"); /* Current directory + "*" will give all files */

numfiles = 30;
while (numfiles == 30)
    {
    c_getfiles(filespec, filename, &numfiles, 30, firstcall, 0);
    firstcall = FALSE;

    /* Realistically, there should never be more than thirty files in a CD
       subdirectory.  This is not to say that there couldn't theoretically
       be more.  Therefore, this loop will handle as many files as needed,
       thirty at a time. */

    /* Search for known VDF files
    -----------------------------*/ 
    for (i = 0; i < numfiles; i++)
	{

	/* This naming convention is used by South Africa MSS CDs */

	if (strstr(filename[i], "vdf_dat.001") != NULL) 
	    {
	    process_vdf(filename[i], &recsize);
	    strcpy(leafile, dirname);
	    strcat(leafile, "lea_01.001");
	    process_lead(leafile, recsize, path, row, date, foundpath, 0);
	    if (foundpath[0] == 'Y')
		strcpy(foundpath, dirname);
	    }

	/* This naming convention is used by SPOT images */   
    
	if (strstr(filename[i], "vold") != NULL)
	    {
	    ptr = (char *)strstr(filename[i], "vold");
	    process_vdf(filename[i], &recsize);
	    ptr[0] = 'l';
	    ptr[1] = 'e';
	    ptr[2] = 'a';
	    ptr[3] = 'd';
	    strcpy(leafile, filename[i]);
	    process_lead(leafile, recsize, path, row, date, foundpath, 1);
	    if (foundpath[0] == 'Y')
		strcpy(foundpath, dirname);
	    }
	}
    }

/* Read in the subdirectory names from current directory 
--------------------------------------------------------*/
numfiles = 30;
firstcall = TRUE;
while (numfiles == 30)
    {
    c_getdirs(filespec, filename, &numfiles, 30, firstcall);
    firstcall = FALSE;

    /* Again, there should never be more than 30 subdirectories within a
       CD directory.  This while loop ensures that all subdirectories are
       handled, 30 at a time, in case of that rare occurence. */

    /* Recurse through each subdirectory 
    ------------------------------------*/
    for (i = 0; i < numfiles; i++)
	{
	strcat(filename[i], "/");	/* Append a closing slash */
	if (strlen(foundpath) == 0)
	    process_dir(filename[i], path, row, date, foundpath);
	}
    }
}
