/******************************************************************************
FUNCTION:	PROCESS_VDF 

PURPOSE:	Reads an image's VDF file to determine the record size of its
		leader file.  This information is necessary in order to 
		properly extract its WRS path and row numbers.

ALGORITHM REFERENCES:	none
******************************************************************************/

#include <stdio.h>
#include "las.h"

FUNCTION process_vdf(filename, recsize)

char filename[CMLEN];
long *recsize;

{
FILE *fp;	 	/* File pointer for VDF				*/
unsigned char in_buf[720]; /* Read buffer				*/
char tempbuf[9];	/* Temporary buffer				*/
long status;		/* Status of fread() operation			*/
long fatal = -1;	/* Flag for fatal error				*/
long nonfatal = 1;	/* Flag for nonfatal error			*/

/* Open the VDF file and read in some data
------------------------------------------*/
fp = fopen(filename, "r");	/* Open VDF file for reading 		*/
if (fp == NULL)
    c_lasmsg(TRUE, "ltwgcd-diskread", "Error reading file %s", filename); 

/* The size of a record in a VDF file is always 360 bytes.  The leader file
   record size, which is the information that is needed, is found in the
   second record of the VDF file.  Therefore, process_vdf always reads 720
   bytes, which is enough to extract the needed information. */

status = fread(in_buf, 1, 720, fp);
if (status != 720)
    {
    fclose(fp);
    c_lasmsg(TRUE, "ltwgcd-diskread", "Error reading file %s", filename);
    }

/* Extract the size of the leader file records
----------------------------------------------*/
strncpy(tempbuf, &in_buf[468], 8); 
tempbuf[8] = '\0';
*recsize = atol(tempbuf);

fclose(fp);
}
