/*
 *	Front-end program to allow nonprivileged users to
 *	physically mount CDROMs.  This program calls c_tpopen to mount
 *	the cd.  C_tpopen executes the op_mntcd program.  
 *
 * Program History
 *	Development by Ken Gacke.
 *
 *	Command Syntax:
 *		mntcd mntmsg
 *		umntcd ldev
 *			mntmsg - user specified mount message
 *			ldev is the logical device name defined in 
 *				$WORCONFIG/tape.config.  The mount point is
 *				/"ldev" (ie if ldev=cd1 then the mount point
 *				would be /cd1
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/utsname.h>

#include "las.h"
#include "util.h"
#include "tapeio.h"

char	*env[] = { "PATH=/bin:/usr/bin:/usr/ucb:/etc:/usr/etc", 0};
int	Verbose = 0;
char	*Version = "6/21/93";		/* please update */

int cd_mount
(
	int argc,
	char *argv[],
	struct PARBLK *vblock,
	long tae
);

int cd_umount
(
	int argc,
	char *argv[],
	struct PARBLK *vblock,
	long tae
);

int main
(
	int argc,
	char **argv
)
{
struct PARBLK vblock;
struct VARIABLE *var;
char	prog[32];	/* program name */
char	*progname;	/* program name */
long	len;		/* length of char string */
long	tae;		/* tae flag -- TRUE => running from TAE */
long	xit_status;	/* status code */


/*
 *	This is mainly for debugging.
 */
    {
    char *vflag;

    if (vflag = getenv("VERBOSE"))
	Verbose = atoi(vflag);
    if (Verbose)
	fprintf(stderr,"version %s\n", Version);
    }

tae = p_inim(&vblock,sizeof(struct PARBLK),P_CONT) == SUCCESS ? TRUE : FALSE;
/*
 *	The program name determines what we are going to do.
 */
if (tae)
    {
    var = p_find(&vblock,"_PROC");
    strcpy(prog,SVAL(*var,0));
    len = (long)strlen(prog);
    c_up2low(prog,&len);
    progname = prog;
    }
else
    {
    progname = strrchr(argv[0], '/');
    if (progname == (char *)0)
        progname = argv[0];
    else
        progname++;
    }
if (strcmp(progname, "mntcd") == 0)		/* mount a CDROM */
    xit_status = cd_mount(argc,argv,&vblock,tae);
else if (strcmp(progname, "umntcd") == 0)	/* unmount CD */
    xit_status = cd_umount(argc,argv,&vblock,tae);
else
    {
    fprintf(stderr,"Invalid command %s\n",progname);
    xit_status = -1;
    }
/*****
   Modified 5/1/95, call c_complt only if running under tae.  Old way was:
       if (xit_status >= 0)
*****/
if (tae)
    c_complt(progname,progname,&xit_status);
else
    exit(xit_status);

return 0;
}

int cd_mount
(
	int argc,
	char *argv[],
	struct PARBLK *vblock,
	long tae
)
{
struct PARBLK dblock;
struct VARIABLE *var;
char	dens[CMLEN];	/* tape density (not used for cd) */
char	hostnm[CMLEN];	/* host name */
char	mntmsg[CMLEN];	/* mount message */
char	*strval[2];	/* pointer to string to update TAE variable */
char	tapeid[CMLEN];	/* tape/cd id */
char	tmp[CMLEN];	/* temporary string buffer */
long	eofopt;		/* end of file option */
long	mode;		/* error mode */
long	rdwt;		/* read/write access */
long	rewopt;		/* rewind option */
int 	rstat;		/* return status code */
long	skip;		/* skip parameter */
long	status;		/* status code */
long	unit;		/* tape i/o unit */

rstat = E_SUCC;
gethostname(hostnm,CMLEN);
if (tae)
    {
    var = p_find(vblock,"COMMENT");
    strcpy(mntmsg,SVAL(*var,0));
    }
else
    {
    if (argc != 2)
	{
	fprintf(stderr,"Usage:  mntcd mntmsg\n");
	rstat = E_FAIL;
	goto xit;
	}
    strcpy(mntmsg,argv[1]);
    }
skip = 0;
mode = HANDLE;
rewopt = NOREW;
strcpy(dens,"");
rdwt = TREAD;
status = c_tpopen(&unit,&skip,&mode,&rewopt,dens,&rdwt,mntmsg,tapeid);
if (status != TAPE_SUCC)
    {
    rstat = E_FAIL;
    goto xit;
    }
if (tae)				/* if tae, save drive in global var */
    {
    if (strchr(mntmsg,'@') == NULL)
	sprintf(tmp,"%s@%s",hostnm,mntmsg);
    else
        strcpy(tmp,mntmsg);
    strval[0] = tmp;
    strval[1] = NULL;
    q_init(&dblock,P_BYTES,P_CONT);
    q_string(&dblock,"$TAPEDRV",1,strval,P_ADD);
    q_out(&dblock);
    }
eofopt = 0;
rewopt = NOREW;
status = c_tpclos(&unit,&mode,&rewopt,&eofopt);
if (status != TAPE_SUCC)
    {
    rstat = E_FAIL;
    goto xit;
    }
xit:
return(rstat);
}

int cd_umount
(
	int argc,
	char *argv[],
	struct PARBLK *vblock,
	long tae
)
{
struct PARBLK dblock;
struct VARIABLE *var;
char	dens[CMLEN];	/* tape density (not used for cd) */
char	hostnm[CMLEN];	/* host name */
char	mntmsg[CMLEN];	/* mount message */
char	*strval[2];	/* pointer to string to update TAE variable */
long	eofopt;		/* end of file option */
long	mode;		/* error mode */
long	rdwt;		/* read/write access */
long	rewopt;		/* rewind option */
int 	rstat;		/* status code */
long	skip;		/* skip parameter */
long	status;		/* status code */
long	unit;		/* tape i/o unit */

rstat = E_SUCC;
gethostname(hostnm,CMLEN);
if (tae)
    {
    var = p_find(vblock,"LDEV");
    if (var->v_count == 0)
        var = p_find(vblock,"$TAPEDRV");
    strcpy(mntmsg,SVAL(*var,0));
    }
else
    {
    if (argc != 2)
	{
	fprintf(stderr,"Usage:  umntcd ldev\n");
	rstat = E_FAIL;
	goto xit;
	}
    strcpy(mntmsg,argv[1]);
    }
skip = 0;
mode = HANDLE;
rewopt = NOREW;
strcpy(dens,"");
rdwt = TREAD;
status = c_tpreop(&unit, &skip, &mode, &rewopt, dens, &rdwt, mntmsg);
if (status != TAPE_SUCC)
    {
    rstat = E_FAIL;
    goto xit;
    }
eofopt = 0;
rewopt = REWOFFL;
status = c_tpclos(&unit,&mode,&rewopt,&eofopt);
if (status != TAPE_SUCC)
    {
    rstat = E_FAIL;
    goto xit;
    }
if (tae)				/* if tae, clear global var */
    {
    strval[0] = "";
    strval[1] = NULL;
    q_init(&dblock,P_BYTES,P_ABORT);
    q_string(&dblock,"$TAPEDRV",0,strval,P_ADD);
    q_out(&dblock);
    }
xit:
return(rstat);
}
