/******************************
NAME: CREATFILE    
 
PURPOSE:  This program is called by NDFCOPY.PDF to create/write 3 files.
          They are *.SCENE, *.ARCHV and *.DAT. 
 
 
PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     04/96   S.Nath            initial Development
  1.1     07/97   K.Lowell          Added mtype to *.DAT file 
  1.2     10/98   D.Hames           Add c_4digityr for y2k 
  1.3     10/98   J.VanZee          Modified to handle multiple volumes
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
 
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"

main()

{
  FILE *fp1;			/* File pointer .SCENE    */
  FILE *fp2;			/* File pointer .ARCHV    */
  FILE *fp3;			/* File pointer .DAT      */
  FILE *fp4;			/* File pointer .HDR      */
  FILE *fp5;			/* File pointer additional .HDR files */

  char prod_nbr  [80];          /* PDS Product number                  */
  char ordnmbr   [80];          /* PDS Order number                    */
  char rms_error [80];          /* Goes to ==> .SCENE & .DAT files     */
  char project   [80];          /* Goes to ==> .SCENE file             */ 
  char scene_id  [80];          /* Goes to ==> .SCENE & >DAT files     */
  char prod_code [80];          /* Goes to ==> .SCENE file             */ 
  char user_ori  [80];          /* Goes to ==> .SCENE & .DAT files     */
  char rad_enha  [80];          /* Goes to ==> .SCENE & .DAT files     */
  char rad_calib [80];          /* Goes to ==> .SCENE & .DAT files     */
  char dem_line  [80];          /* Goes to ==> .DAT file               */
  char sequence  [10][80];      /* Tape Sequence goes to ==> .DAT&SCENE*/
  long no_seq    = 0;           /* The length of the sequence array    */
  char filename  [100];         /* filename contains disk header file  */
  char spec_band [80];          /* Goes to ==> .SCENE file             */
  char nbr_scenes[10];          /* Total scenes                        */
  char code      [80];          /* Project source                      */
  int  ndf_version = 0;         /* NDF version number                  */
  
  struct PARBLK vblk;           /* Pointer to initial TAE parameter blk*/	
  char buffer   [1000];         /* Buffer for an entire record         */
  long std_out   = 0;		
  char scene_file[80];          /* Name of the SCENE file              */
  char archv_file[80];          /* Name of the ARCHIVE file            */
  char hdr_file[80];            /* Name of the DAT file                */
  char next_hdr[80];            /* Name of the next header file for multiple */
  char ppffile[CMLEN];          /* Name of the ppf file                */

  char orig_source[80]  = "#";  /* Goes to ==> SCENE                   */
  char old_scene[80]    = "#";  /* Goes to ==> SCENE                   */
  char offset[80]       = "#";  /* goes to ==> SCENE                   */
  char use_restr[80]    = "#";  /* goes to ==> SCENE                   */
  char path_nbr[80]     = "#";  /* goes to ==> SCENE                   */
  char row_nbr[80]      = "#";  /* goes to ==> SCENE                   */
  char wrs_f_row[80]    = "#";  /* goes to ==> SCENE                   */
  char wrs_l_row[80]    = "#";  /* goes to ==> SCENE                   */
  char wrs_type[80]     = "#";  /* goes to ==> SCENE                   */
  char acqu_date[80]    = "#";  /* goes to ==> SCENE                   */
  char cloud_cover[80]  = "#";  /* goes to ==> SCENE                   */
  char recd_tech[80]    = "#";  /* goes to ==> SCENE                   */
  char sun_azimuth[80]  = "#";  /* goes to ==> SCENE                   */
  char sun_elevatn[80]  = "#";  /* goes to ==> SCENE                   */
  char center_lat[80]   = "#";  /* goes to ==> SCENE                   */
  char center_lon[80]   = "#";  /* goes to ==> SCENE                   */
  char quality[80]      = "#";  /* goes to ==> SCENE                   */
  char satellite[80]    = "#";  /* goes to ==> SCENE                   */
  char micro_frame[80]  = "#";  /* goes to ==> SCENE                   */
  char condition[80]    = "#";  /* goes to ==> SCENE                   */
  char record_stat[80]  = "#";  /* goes to ==> SCENE                   */
  char resamp_tech[80]  = "#";  /* goes to ==> SCENE                   */
  char user_orient[80]  = "#";  /* goes to ==> SCENE                   */
  char orien_angle[80]  = "#";  /* goes to ==> SCENE                   */
  char horiz_datum[80]  = "#";  /* goes to ==> SCENE                   */
  char edc_process[80]  = "#";  /* goes to ==> SCENE                   */
  char brow_avail[80]   = "#";  /* goes to ==> SCENE                   */
  char prod_media[80]   = "#";  /* goes to ==> SCENE                   */
  char mgmt_code[80]    = "#";  /* goes to ==> SCENE                   */
  char map_projec[80]   = "#";  /* goes to ==> SCENE                   */
  char ims_visibl[80]   = "#";  /* goes to ==> SCENE                   */
  char interlvng[80]    = "#";  /* goes to ==> SCENE                   */
  char prod_size[80]    = "#";  /* goes to ==> SCENE                   */
  char pixel_width[80]  = "#";  /* goes to ==> SCENE                   */
  char pixel_hght[80]   = "#";  /* goes to ==> SCENE                   */
  char dem_avail[80]    = "#";  /* goes to ==> SCENE                   */
  char spect_band[80]   = "#";  /* goes to ==> SCENE                   */
  char rad_c[80]        = "#";  /* goes to ==> SCENE                   */
  char rad_e[80]        = "#";  /* goes to ==> SCENE                   */
  char upp_l_lat[80]    = "#";  /* goes to ==> SCENE                   */
  char upp_l_lon[80]    = "#";  /* goes to ==> SCENE                   */
  char upp_r_lat[80]    = "#";  /* goes to ==> SCENE                   */
  char upp_r_lon[80]    = "#";  /* goes to ==> SCENE                   */
  char low_r_lat[80]    = "#";  /* goes to ==> SCENE                   */
  char low_r_lon[80]    = "#";  /* goes to ==> SCENE                   */
  char low_l_lat[80]    = "#";  /* goes to ==> SCENE                   */
  char low_l_lon[80]    = "#";  /* goes to ==> SCENE                   */
  char map_zone[80]     = "#";  /* goes to ==> SCENE                   */
  char proj_par[15][80];        /* goes to ==> SCENE                   */
  char root_error[80]   = "#";  /* goes to ==> SCENE                   */
  char ellip_maj[80]    = "#";  /* goes to ==> SCENE                   */
  char ellip_min[80]    = "#";  /* goes to ==> SCENE                   */

  long headcnt = 0;	        /* Loop Counter                        */
  long bandcnt = 0;	        /* Loop Counter                        */
  long ii = 0;	                /* Loop Counter                        */
  long jj = 0;	                /* Loop Counter                        */
  long max = 512;               /* Maximum bytes read by fgets         */
  char line [512];              /* Reads a record in line              */
  char wrkline [512];              /* Reads a record in line              */
  char bandline [512];              /* Reads a record in line              */
  char waveline [512];              /* Reads a record in line              */
  char radline [512];              /* Reads a record in line              */
  char *ch_ptr1;                /* Running character pointer           */
  char *ch_ptr2;                /* Running character pointer           */
  char *ch_ptr3;                /* Running character pointer           */
  long index;                   /* Offset of a pattern in a string     */
  long index2;                  /* Offset of a pattern in a string     */
  long fatal = -1;              /* Error counter                       */
  char rms_buff[80];            /* Buffer for Rms_error field          */
  char tm_mss[2];               /* Holds either TM/MSS(1/9)            */
  long rms_exist;               /* Governs the existance of rms_error  */
  long multi_scene;             /* Number scenes in this order         */
  long write_nldc;              /* Nldc metadata flag                  */
  char production[5];           /* Temporary rms_errpr area            */
  char mtype[80];		/* output media type		       */
  double dbl_val[1];            /* generic double variable             */
  long lng_val[1];              /* generic long variable               */
  long year;                    /* year value to convert, added for y2k*/
  long yr;                      /* year                                */
  char clines[512];              /* number of lines/header in char format */
  char cbands[512];              /* number of bands/header in char foramt */
  long lines[5]={0,0,0,0,0};     /* number of lines/header */
  long bands[5]={0,0,0,0,0};     /* number of bands/header */
  long totlines=0;               /* running total of number of lines/header */
  long totbands=0;               /* running total of number of bands/header */
  
  char pdsassm[CMLEN];		/* PDS assemble directory              */

/* Initialize TAE parameter block
-----------------------------------*/
  c_inlas(&vblk,&std_out);

/* Get the variables from the TAE variable block
--------------------------------------------------*/

if ((getpar2(&vblk,prod_nbr,ordnmbr,rad_calib,prod_code,user_ori,rad_enha,
             dem_line,rms_error,project,scene_id,sequence,&no_seq,filename,
             nbr_scenes,spec_band,code,mtype)) != E_SUCC)
    c_errmsg("Fatal error encountered","ndfcopy-fatal",&fatal);

/* upper case the project code */
index = (long)strlen(project);
c_low2up(project,&index);

/* Get the location of the PDS assemble directory
--------------------------------------------------*/
strcpy(pdsassm,getenv("PDSASSM"));

sprintf(scene_file,"%s%s%s%s%s%s",pdsassm,"/",ordnmbr,"/",prod_nbr,".SCENE");
sprintf(archv_file,"%s%s%s%s%s%s",pdsassm,"/",ordnmbr,"/",prod_nbr,".ARCHV");
sprintf(hdr_file,"%s%s%s%s%s%s",pdsassm,"/",ordnmbr,"/",prod_nbr,".DAT");

if ((strncmp(code,"101",3) == 0) || (strncmp(code,"054",3) == 0) )
    write_nldc = 0;
else
    write_nldc = 1; 

if(write_nldc)
 {
   if ((fp1 = fopen(scene_file,"w")) == NULL)  /* Pipe delimited SCENE file*/
        c_errmsg("Error opening .SCENE file","ndfcopy-open",&fatal);

   if ((fp2 = fopen(archv_file,"w")) == NULL)  /* ARCHIVE file     */
        c_errmsg("Error opening .ARCHV file","ndfcopy-open",&fatal);
 }

if ((fp3 = fopen(hdr_file,"w")) == NULL)  /* .DAT Header File */
      c_errmsg("Error opening .DAT file","ndfcopy-open",&fatal);

if ((fp4 = fopen(filename,"r")) == NULL)  /* Header File  */
      c_errmsg("Error opening .HD file","ndfcopy-open",&fatal);

/* used only if there are multiple input tapes and we need to merge
the header files
--------------------------------------------------------------------------*/ 
if (no_seq > 1)
{
printf("Total volumes = %d\n",no_seq);
	for (headcnt = 1; headcnt <= no_seq; headcnt++)
	{
		sprintf(next_hdr,"%s/%s/%s_%d.HD", pdsassm, ordnmbr, ordnmbr,
			headcnt);
		if ((fp5 = fopen(next_hdr,"r")) == NULL)  /* Header File  */
		{	
      		   c_errmsg("Error opening the next .HD file",
			"ndfcopy-open",&fatal);
		}
		while (fgets(line,max,fp5) != NULL) /* Read the Header file */
		{
		
   		if (strncmp(line,"LINES_PER_VOLUME",16) == 0) 
     		{
       			index = (long)strcspn(line,"=") + 1;
       			index2= (long)strcspn(line,";");
       			ch_ptr1 = line + index;
       			strncpy (clines,ch_ptr1,index2-index);
			lines[headcnt] = atoi(clines);
			totlines = totlines + lines[headcnt];
     		} 
   		if (strncmp(line,"NUMBER_OF_BANDS_IN_VOLUME",25) == 0) 
     		{
       			index = (long)strcspn(line,"=") + 1;
       			index2= (long)strcspn(line,";");
       			ch_ptr1 = line + index;
       			strncpy (cbands,ch_ptr1,index2-index);
			bands[headcnt] = atoi(cbands);
			totbands = totbands + bands[headcnt];
     		} 
		}
	fclose(fp5);     /** Close the Old Header file  **/
	}
}

for (ii = 0; ii < 15; ++ii)
     strcpy(proj_par[ii],"#");

ch_ptr1 = scene_id;
index   = (long)strlen(scene_id) - 2;
ch_ptr1 = ch_ptr1 + index;
strncat(tm_mss,ch_ptr1,1);
 
multi_scene = atoi(nbr_scenes);

/* Path_nbr    
----------------*/
ch_ptr1 = scene_id + 1;
strncpy(path_nbr,ch_ptr1,3);

/* Row_nbr    
----------------*/
ch_ptr1 = scene_id + 4;
strncpy(row_nbr,ch_ptr1,3);

/* Determine the RECORDING_TECH        
--------------------------------*/
index = atoi(row_nbr);
if (index >= 120 && index <= 246)
 {
   if (!strncmp(tm_mss,"1",1))   
      strncpy(recd_tech,"42",2);   /** Ascending and TM **/
   else
      strncpy(recd_tech,"39",2);   /** Ascending and MSS **/
 }
else
 {
   if (!strncmp(tm_mss,"1",1))
      strncpy(recd_tech,"41",2);   /** Descending and TM **/
   else
      strncpy(recd_tech,"38",2);   /** Descending and MSS **/
 }

/* Determine the existance of RMS_ERROR
------------------------------------------*/
index = (long)strlen(prod_code);
ch_ptr1 = prod_code + 1;
strcpy(production,ch_ptr1);
rms_exist=atoi(production);
strcpy (rms_buff,"RMS_ERROR = ");
if ((rms_exist >= 300 && rms_exist <= 499) 
    && (strlen(rms_error) > 0) && (strstr(rms_error,"Type") != NULL))
  {
    index = (long)strcspn(rms_error,"Type");
    ch_ptr1= rms_error + index + strlen("Type ");
    index = (long)strcspn(ch_ptr1,"Type");
    if ( index <= 0)
       c_errmsg("Fatal error encountered","ndfcopy-rmserror",&fatal);
    index = (long)strcspn(ch_ptr1, " ");
    strncat(rms_buff,ch_ptr1,index);
    strncpy(root_error,ch_ptr1,index);
    root_error[index] = '\0';
  }
/* Determine WRS_FIRST_ROW and WRS_LAST_ROW
--------------------------------------------*/
if (multi_scene > 1)
 {
   sprintf(ppffile,"%s/%s/%s",pdsassm,ordnmbr,ordnmbr);
   c_getparm(ppffile,"ROW_FIRST",wrs_f_row,"S",dbl_val,lng_val,1);
   c_getparm(ppffile,"ROW_LAST",wrs_l_row,"S",dbl_val,lng_val,1);
 }

/* Determine SATELLITE number               
------------------------------*/
strncpy(satellite,scene_id,1);

/* Determine WRS_TYPE                       
---------------------*/
index = atoi(satellite);
if(index >= 1 && index <= 3)
   strcpy(wrs_type,"1");
else 
   if (index >=4)
      strcpy(wrs_type,"2");

strcpy (record_stat,"G");        /** RECORDING_STATUS  **/
strcpy (brow_avail,"N");         /** BROWSE_AVAIL_FLAG **/
strcpy (ims_visibl,"Y");         /** IMS_VISIBLE_FLAG **/
strcpy (spect_band,spec_band);   /** SPECTRAL_BAND **/


index = (long)strcspn(user_ori, "=") + 2;  /** USER_ORIENTATION  **/
index2 = (long)strcspn(user_ori, ";");
ch_ptr1 = user_ori + index;
strncpy (user_orient,ch_ptr1,index2-index);

if ( !strcmp(project,"HTFPTM") )  /** MANAGEMENT_CODE  **/
   strcpy(mgmt_code,"61");
else if ( !strcmp(project,"HTFPMSS") ) 
       strcpy(mgmt_code,"62");
     else if ( !strcmp(project,"NLDC") )
          strcpy(mgmt_code,"65");

if (!strncmp(dem_line,"Y",1))          /** DEM_AVAIL  **/
    strcpy (dem_avail,"YES");

index = (long)strcspn(rad_calib, "=") + 2;    /** RAD_CALIB  **/
index2 = (long)strcspn(rad_calib, ";");
ch_ptr1 = rad_calib + index;
strncpy (rad_c,ch_ptr1,index2-index);

index = (long)strcspn(rad_enha, "=") + 2;    /** RAD_ENHAN  **/
index2 = (long)strcspn(rad_enha, ";");
ch_ptr1 = rad_enha + index;
strncpy (rad_e,ch_ptr1,index2-index);

while (fgets(line,max,fp4) != NULL)       /* Read the Header file */
  {
   if (strncmp(line,"NDF_REVISION",12) == 0)
     {
       index = (long)strcspn(line,"=") + 1;
       ch_ptr1 = line + index;
       ndf_version = atoi(ch_ptr1);
     }
   else if (strncmp(line,"DATA_SET_TYPE",13) == 0)   /* ORIG_SOURCE    */
     {
       index = (long)strcspn(line,"=") + 1;
       index2= (long)strcspn(line,";");
       ch_ptr1 = line + index;
       strncpy (orig_source,ch_ptr1,index2-index);
     } 
   else if (strncmp(line,"WRS=",4) == 0)             /* WRS_OFFSET     */
     {
       index = (long)strcspn(line,".") + 1;
       if (index < strlen(line))
         {
           index2 = (long)strcspn(line, ";");
           ch_ptr1 = line + index;
           index = index2 - index; 
           strncpy(offset,ch_ptr1,index);
           if (!strncmp(offset,"00000000",index))
              strcpy(offset,"#");
         }
     }

   else if (!strncmp(line,"ACQUISITION_DATE/TIME",21)) /*ACQUISITION DATE*/
     {
       index = (long)strcspn(line, "=") + 1;
       ch_ptr1 = line + index;
      
       /* extract the date using the format for the particular NDF version */
       if (ndf_version < 2)
       {
           /* NDF version 1 format: MMDDYY/hhmmssxx */
           ch_ptr2 = ch_ptr1 + 4;
           sscanf(ch_ptr2,"%d",&year);
           yr = c_4digityr (year);                 /* add c_4digityr for y2k */
           
           sprintf (acqu_date,"%4d",yr);
           strncat (acqu_date,"/",1);        
           strncat (acqu_date,ch_ptr1,2);      /* acqu_date = yyyy/mm  **/
           strncat (acqu_date,"/",1);
           strncat (acqu_date,ch_ptr1+2,2);    /* acqu_date = yyyy/mm/dd **/
       }
       else
       {
           /* NDF version 2 format: YYYY-MM-DDThh:mm:ssZ */
           memset(acqu_date,'\0',sizeof(acqu_date));
           strncpy(acqu_date,ch_ptr1,10);
           for (index = 0; index < 10; index++)
               if (acqu_date[index] == '-')
                   acqu_date[index] = '/';
       }
     }
   else if (!strncmp(line,"SUN_ELEVATION=",14) )    /* SUN_ELEVATION **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(sun_elevatn,ch_ptr1,index2-index);
     } 
   else if (!strncmp(line,"SUN_AZIMUTH=",12) )    /* SUN_AZIMUTH **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(sun_azimuth,ch_ptr1,index2-index);
     } 

   else if (!strncmp(line,"REFERENCE_POSITION=",19) )  
     {
       index = (long)strcspn(line, "=") + 1;            /* CENTER LON **/
       index2 = (long)strcspn(line, ",");
       ch_ptr1 = line + index;
       strncpy(center_lon,ch_ptr1,index2-index);

       ch_ptr1 = line + index2 + 1;
       index2  = (long)strcspn(ch_ptr1, ",");
       strncpy(center_lat,ch_ptr1,index2);
     } 
   else if (!strncmp(line,"RESAMPLING=",11) )    /* RESAMPLING TECH  **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(resamp_tech,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"ORIENTATION=",12) )    /* ORIENTATION ANGLE**/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(orien_angle,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"HORIZONTAL_DATUM=",17) )    /* HORIZONTAL DATUM**/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(horiz_datum,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"PROCESSING_LEVEL=",17) )    /* PROCESSING LEVEL**/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(edc_process,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"MAP_PROJECTION_NAME=",20) )    /* MAP_PROJECTION**/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(map_projec,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"DATA_FILE_INTERLEAVING=",23) ) /* DATA_INTERLEAVNG**/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(interlvng,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"PRODUCT_SIZE=",13) )    /* PRODUCT SIZE  **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(prod_size,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"PIXEL_SPACING=",14) )    
     {
       index = (long)strcspn(line, "=") + 1;         /* PIXEL_WIDTH **/
       index2 = (long)strcspn(line, ",");
       ch_ptr1 = line + index;
       strncpy(pixel_width,ch_ptr1,index2-index);

       index = index2 + 1;                          /** PIXEL_HEIGHT **/
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(pixel_hght,ch_ptr1,index2-index);
     } 
   else if (!strncmp(line,"USGS_MAP_ZONE=",14) ) /* MAP_ZONE **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(map_zone,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"UPPER_LEFT_CORNER=",18) ) /* UPPER LEFT LON/LAT **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ",");
       ch_ptr1 = line + index;
       strncpy(upp_l_lon,ch_ptr1,index2-index);
       ch_ptr1 = line + index2 + 1;
       index2 = (long)strcspn(ch_ptr1, ",");
       strncpy(upp_l_lat,ch_ptr1,index2);
     }
   else if (!strncmp(line,"UPPER_RIGHT_CORNER=",19) ) /* UPPER RIGHT LON/LAT **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ",");
       ch_ptr1 = line + index;
       strncpy(upp_r_lon,ch_ptr1,index2-index);
       ch_ptr1 = line + index2 + 1;
       index2 = (long)strcspn(ch_ptr1, ",");
       strncpy(upp_r_lat,ch_ptr1,index2);
     }
   else if (!strncmp(line,"LOWER_RIGHT_CORNER=",19) ) /* LOWER RIGHT LON/LAT **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ",");
       ch_ptr1 = line + index;
       strncpy(low_r_lon,ch_ptr1,index2-index);
       ch_ptr1 = line + index2 + 1;
       index2 = (long)strcspn(ch_ptr1, ",");
       strncpy(low_r_lat,ch_ptr1,index2);
     }
   else if (!strncmp(line,"LOWER_LEFT_CORNER=",18) ) /* LOWER LEFT LON/LAT **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ",");
       ch_ptr1 = line + index;
       strncpy(low_l_lon,ch_ptr1,index2-index);
       ch_ptr1 = line + index2 + 1;
       index2 = (long)strcspn(ch_ptr1, ",");
       strncpy(low_l_lat,ch_ptr1,index2);
     }
   else if (!strncmp(line,"USGS_PROJECTION_PARAMETERS=",27) )    
     {
       ch_ptr1 = line;                 /* 15 PROJECT PARAMETERS  */
       index = (long)strcspn(line, "=") + 1;   
       ch_ptr1 = ch_ptr1 + index;
       for (ii = 0; ii < 14; ++ii)
         {
           index2 = (long)strcspn(ch_ptr1, ",");
           strncpy(proj_par[ii],ch_ptr1,index2);
           ch_ptr1 = ch_ptr1 + index2 + 1;
         }
       index2 = (long)strcspn(ch_ptr1, ";");
       strncpy(proj_par[ii],ch_ptr1,index2);
       
     } 
   else if (!strncmp(line,"EARTH_ELLIPSOID_SEMI-MAJOR",26) ) /* SEMI_MAJOR **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(ellip_maj,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"EARTH_ELLIPSOID_SEMI-MINOR",26) ) /* SEMI_MINOR **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(ellip_min,ch_ptr1,index2-index);
     }
   else if (!strncmp(line,"EARTH_ELLIPSOID_SEMI-MINOR",26) ) /* SEMI_MINOR **/
     {
       index = (long)strcspn(line, "=") + 1;
       index2 = (long)strcspn(line, ";");
       ch_ptr1 = line + index;
       strncpy(ellip_min,ch_ptr1,index2-index);
     }

   else if (strncmp(line,"LINES_PER_VOLUME",16) == 0 ) /*Modify lines per vol */
	{
		if (no_seq > 1)
		{
			sprintf(line,"LINES_PER_VOLUME=%ld;\n",totlines);
		}
	}

/* Modify bands per volume */
   else if (strncmp(line,"NUMBER_OF_BANDS_IN_VOLUME",25) == 0) 
	{
		if (no_seq > 1)
		{
			sprintf(line,"NUMBER_OF_BANDS_IN_VOLUME=%ld;\n",totbands);
		}
	}


   /** Write the entire HEADER file to .DAT file
    except END_OF_HDR line
    ------------------------------------------------**/
   if (strncmp(line,"END_OF_HDR",10) != 0)   
          fputs(line,fp3);
  }

fclose(fp4);     /** Close the Old Header file  **/

/* append other band information to the .DAT file if there are more
than one header
----------------------------------------------------------------------*/
if (no_seq > 1)
{
	bandcnt = bands[1];
	for (headcnt = 2; headcnt <= no_seq; headcnt++)
	{
		sprintf(next_hdr,"%s/%s/%s_%d.HD", pdsassm, ordnmbr, ordnmbr,
			headcnt);
		if ((fp5 = fopen(next_hdr,"r")) == NULL)  /* Header File  */
		{	
      		   c_errmsg("Error opening the next .HD file",
			"ndfcopy-open",&fatal);
		}
		while (fgets(line,max,fp5) != NULL) /* Read the Header file */
		{
		
   		if (strncmp(line,"BAND",4) == 0) 
     		{
			bandcnt =  bandcnt+1;
			strcpy(wrkline,line);
       			index = (long)strcspn(wrkline, "=") + 1;
       			index2 = (long)strcspn(wrkline, ";");
       			ch_ptr1 = wrkline + index;
       			strncpy(bandline,ch_ptr1,index2-index);
			sprintf(line,"BAND%ld_NAME=%s;\n",
				bandcnt,bandline);
printf("ValLine 1 = %s\n ",bandline);
printf("Line 1 = %s\n",line);
          		fputs(line,fp3);

		        fgets(line,max,fp5);
			strcpy(wrkline,line);
       			index = (long)strcspn(wrkline, "=") + 1;
       			index2 = (long)strcspn(wrkline, ";");
       			ch_ptr2 = wrkline + index;
       			strncpy(waveline,ch_ptr2,index2-index);
			sprintf(line,"BAND%ld_WAVELENGTHS=%s;\n",
				bandcnt,waveline);
printf("ValLine 2 = %s\n",waveline);
printf("Line 2 = %s\n ",line);
          		fputs(line,fp3);

		        fgets(line,max,fp5);
			strcpy(wrkline,line);
       			index = (long)strcspn(wrkline, "=") + 1;
       			index2 = (long)strcspn(wrkline, ";");
       			ch_ptr3 = wrkline + index;
       			strncpy(radline,ch_ptr3,index2-index);
			sprintf(line,"BAND%ld_RADIOMETRIC_GAINS/BIAS=%s;\n",
				bandcnt,radline);
printf("ValLine 3 = %s\n ",radline);
printf("Line 3 = %s\n",line);
          		fputs(line,fp3);
     		} 
     		} 
	fclose(fp5);     /** Close the Old Header file  **/
	}
}

/* Write the New Header file           
-----------------------------------*/
jj  =  sprintf(buffer,"%s%s%s","NEW_SCENE_ID = ",scene_id,";\n");
jj +=  sprintf(buffer+jj,"%s%s",rad_calib,"\n");
jj +=  sprintf(buffer+jj,"%s%s",user_ori,"\n");
jj +=  sprintf(buffer+jj,"%s%s",rad_enha,"\n");
jj +=  sprintf(buffer+jj,"%s%s",dem_line,"\n");
jj +=  sprintf(buffer+jj,"%s%s",rms_buff,";\n");
jj +=  sprintf(buffer+jj,"%s%s%s","PRODUCT_CODE = ",prod_code,";\n");
jj +=  sprintf(buffer+jj,"%s%s%s","NBR_SCENES = ",nbr_scenes,";\n");
jj +=  sprintf(buffer+jj,"%s%s%s","PRODUCT_MEDIA = ",mtype,";\n");

for (ii = 0; ii < no_seq; ++ii)
  {
    jj += sprintf(buffer+jj,"%s%s","Tape id = ",sequence[ii]);
    jj += sprintf(buffer+jj,"%s%d%s%d%s"," is for ",ii+1," of ",no_seq,";\n");
  }
jj +=  sprintf(buffer+jj,"%s","END_OF_HDR;\n");
fputs(buffer,fp3);
fclose(fp3);                     /* Close the New header file  **/

/* Write the pipe delimited Scene_File for data ingestation
------------------------------------------------------------*/
if(write_nldc)
 {
   jj  =  sprintf(buffer,"%s%s",scene_id,"|");
   jj +=  sprintf(buffer + jj,"%s%s",old_scene,"|"); 
   jj +=  sprintf(buffer + jj,"%s%s",orig_source,"|");
   jj +=  sprintf(buffer + jj,"%s%s",project,"|");
   jj +=  sprintf(buffer + jj,"%s%s",offset,"|");
   jj +=  sprintf(buffer + jj,"%s%s",use_restr,"|");
   jj +=  sprintf(buffer + jj,"%s%s",path_nbr,"|");
   jj +=  sprintf(buffer + jj,"%s%s",row_nbr,"|");
   jj +=  sprintf(buffer + jj,"%s%s",wrs_f_row,"|");
   jj +=  sprintf(buffer + jj,"%s%s",wrs_l_row,"|");
   jj +=  sprintf(buffer + jj,"%s%s",wrs_type,"|");
   jj +=  sprintf(buffer + jj,"%s%s",acqu_date,"|");
   jj +=  sprintf(buffer + jj,"%s%s",cloud_cover,"|");
   jj +=  sprintf(buffer + jj,"%s%s",recd_tech,"|");
   jj +=  sprintf(buffer + jj,"%s%s",sun_azimuth,"|");
   jj +=  sprintf(buffer + jj,"%s%s",sun_elevatn,"|");
   jj +=  sprintf(buffer + jj,"%s%s",center_lat,"|");
   jj +=  sprintf(buffer + jj,"%s%s",center_lon,"|");
   jj +=  sprintf(buffer + jj,"%s%s",quality,"|");
   jj +=  sprintf(buffer + jj,"%s%s",satellite,"|");
   jj +=  sprintf(buffer + jj,"%s%s",micro_frame,"|");
   jj +=  sprintf(buffer + jj,"%s%s",condition,"|");
   jj +=  sprintf(buffer + jj,"%s%s",record_stat,"|");
   jj +=  sprintf(buffer + jj,"%s%s",resamp_tech,"|");
   jj +=  sprintf(buffer + jj,"%s%s",user_orient,"|");
   jj +=  sprintf(buffer + jj,"%s%s",orien_angle,"|");
   jj +=  sprintf(buffer + jj,"%s%s",horiz_datum,"|");
   jj +=  sprintf(buffer + jj,"%s%s",edc_process,"|");
   jj +=  sprintf(buffer + jj,"%s%s",brow_avail,"|");
   jj +=  sprintf(buffer + jj,"%s%s",prod_media,"|");
   jj +=  sprintf(buffer + jj,"%s%s",mgmt_code,"|");
   jj +=  sprintf(buffer + jj,"%s%s",map_projec,"|");
   jj +=  sprintf(buffer + jj,"%s%s",ims_visibl,"|");
   jj +=  sprintf(buffer + jj,"%s%s",interlvng,"|");
   jj +=  sprintf(buffer + jj,"%s%s",prod_size,"|");
   jj +=  sprintf(buffer + jj,"%s%s",pixel_width,"|");
   jj +=  sprintf(buffer + jj,"%s%s",pixel_hght,"|");
   jj +=  sprintf(buffer + jj,"%s%s",dem_avail,"|");
   jj +=  sprintf(buffer + jj,"%s%s",spect_band,"|");
   jj +=  sprintf(buffer + jj,"%s%s",rad_e,"|");
   jj +=  sprintf(buffer + jj,"%s%s",rad_c,"|");
   jj +=  sprintf(buffer + jj,"%s%s",upp_l_lat,"|");
   jj +=  sprintf(buffer + jj,"%s%s",upp_l_lon,"|");
   jj +=  sprintf(buffer + jj,"%s%s",upp_r_lat,"|");
   jj +=  sprintf(buffer + jj,"%s%s",upp_r_lon,"|");
   jj +=  sprintf(buffer + jj,"%s%s",low_r_lat,"|");
   jj +=  sprintf(buffer + jj,"%s%s",low_r_lon,"|");
   jj +=  sprintf(buffer + jj,"%s%s",low_l_lat,"|");
   jj +=  sprintf(buffer + jj,"%s%s",low_l_lon,"|");
   jj +=  sprintf(buffer + jj,"%s%s",map_zone,"|");

   for (ii = 0; ii < 15; ++ii)
          jj +=  sprintf(buffer + jj,"%s%s",proj_par[ii],"|");

   jj +=  sprintf(buffer + jj,"%s%s",root_error,"|");
   jj +=  sprintf(buffer + jj,"%s%s",ellip_maj,"|");
   jj +=  sprintf(buffer + jj,"%s%s",ellip_min,"|");

   fputs(buffer,fp1);
   fclose(fp1);

/* Write the pipe delimited Archive file data ingestation
------------------------------------------------------------*/
   for(ii = 0; ii < no_seq; ++ii)
     {
       jj  =  sprintf(buffer,"%s%s",sequence[ii],"|");
       jj +=  sprintf(buffer + jj,"%s","1|");
       jj +=  sprintf(buffer + jj,"%d%s",ii+1,"|");
       jj +=  sprintf(buffer + jj,"%d%s",no_seq,"|");
       jj +=  sprintf(buffer + jj,"%s","NDF|");
       jj +=  sprintf(buffer + jj,"%s|\n",mtype);
       fputs(buffer,fp2);
     }
   fclose(fp2);
 }

return(E_SUCC);
}
