/********************************************************************
NAME: MKNHDR
 
PURPOSE: To create additional NDF header files, as needed for
         volume spanning of output products.
 
PROGRAM HISTORY
Version   Date     Programmer      Code  Reason
-------   ----     --------------  ----- ------
  1.0     06/95    K.Lowell         C    Initial development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE
 
PROJECT: PDS
******************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "string.h"
#include "las.h"
#include "protos.h"

int mknhdr
(
    char ordnum[],          /* PDS Order Number             */
    char ordid[],           /* Order id                     */
    int  *nvol,             /* Number of volumes total      */
    int  bpv[]              /* bands per volume             */
)
{
FILE *in;		/* File pointer			*/
FILE *out;		/* File pointer			*/

long fatal=-1;		/* Fatal error flag		*/
long ii,zz,jj;		/* Incremental counter  	*/
long lpv;		/* Lines per volume             */
long lpf;		/* Lines per file		*/
long tmp;		/* Temp long			*/
long sdf[100];		/* Start data file		*/
long bgnfield;		/* Begin of field to scan	*/
long endfield;		/* End of field to scan		*/
long field_length;	/* Length of field to scan	*/
long tnl;		/* Total number of lines	*/
char nlines[10];	/* number of lines		*/
long totbnd=0;		/* Total number of bands	*/
long bn;		/* Current band number		*/

char *p;                /* header string hold           */
char pdsassm[CMLEN];	/* PDS assemble directory       */
char infile[CMLEN];	/* Input .HD file		*/
char outfile[CMLEN];	/* Output .HX file		*/
char inbuff[CMLEN];	/* input buffer			*/
char outbuff[CMLEN];	/* output buffer		*/
char band[2];		/* Current band			*/


/* Get the location of the PDS assemble directory
---------------------------------------------------*/
strcpy(pdsassm,getenv("PDSASSM"));

/* Calculate total number of bands
-----------------------------------*/
for (ii = 0; ii < 100; ++ii)
      sdf[ii] = 0;
sdf[0]=1;
for (ii=0;ii<*nvol;ii++)
  totbnd=totbnd+bpv[ii];
for (ii=1;ii<*nvol;ii++)
  {
  tmp=0;
  for (jj=0;jj<ii;jj++) tmp=tmp+bpv[jj];
  sdf[ii]=tmp+1;
  }


/* Open input and output files
---------------------------------*/
sprintf(infile,"%s%s%s%s%s%s",pdsassm,"/",ordnum,"/",ordid,".HD");

for (ii=1;ii<=*nvol;ii++)
  { 
  if((in=fopen(infile,"r"))==NULL)
    c_errmsg("Error opening .HD file","mknhdr-file",&fatal);
  sprintf(outfile,"%s%s%s%s%s%s%d",pdsassm,"/",ordnum,"/",ordid,".H",ii);
  if((out=fopen(outfile,"w"))==NULL)
    c_errmsg("Error opening .HX file","mknhdr-file",&fatal);


  for (zz=0;zz<75;zz++)
    { 
    fscanf(in,"%s",inbuff);
    if (strncmp(inbuff,"TAPE_SPANNING_FLAG=",19) == 0)
      {
      sprintf(outbuff,"%s%d%s%d%s\n","TAPE_SPANNING_FLAG=",ii,"/",*nvol,";");
      }
    else if (strncmp(inbuff,"START_DATA_FILE=",16) == 0)
      {
      sprintf(outbuff,"%s%d%s\n","START_DATA_FILE=",sdf[ii-1],";");
      }
    else if (strncmp(inbuff,"LINES_PER_VOLUME=",17) == 0)
      {
      if ((p=strstr(inbuff,"LINES_PER_VOLUME")) != NULL)
        {
        bgnfield = (long)strcspn(p,"=");
        endfield = (long)strcspn(p,";");
        field_length = (endfield - 1) - bgnfield;
        strncpy(nlines,&p[17],field_length);
	nlines[field_length]='\0';
        sscanf(nlines,"%d",&tnl);
        }
      lpf = tnl/totbnd;
      lpv = lpf * bpv[ii-1];
      sprintf(outbuff,"%s%d%s\n","LINES_PER_VOLUME=",lpv,";");
      }
    else if  (strncmp(inbuff,"NUMBER_OF_BANDS_IN_VOLUME=",26) == 0)
      {
      sprintf(outbuff,"%s%d%s\n","NUMBER_OF_BANDS_IN_VOLUME=",bpv[ii-1],";");
      }
    else if (strncmp(inbuff,"BAND",4) == 0)
      {
      if ((p=strstr(inbuff,"BAND")) != NULL)
        {
        bgnfield = (long)strcspn(p,"D");
        endfield = (long)strcspn(p,"_");
        field_length = (endfield - 1) - bgnfield;
        strncpy(band,&p[4],field_length);
        band[field_length]='\0';
        sscanf(band,"%d",&bn);
        }
      if (((bn >= sdf[ii-1]) && (bn < sdf[ii]) && (ii < *nvol)) 
         || ((bn >= sdf[ii-1]) &&
           (ii == *nvol))) 
        sprintf(outbuff,"%s\n",inbuff);
      else
        strcpy(outbuff,""); 
      }
    else
      {
      if (strncmp(inbuff,"END_OF_HDR;",11) == 0) zz=75;
      sprintf(outbuff,"%s\n",inbuff);
      } 
    fprintf(out,"%s",outbuff);  
    } 
  fclose(out);
  fclose(in);
  }

return (E_SUCC);

}
