/********************************************************************
NAME: NLAPCKCOPY
 
PURPOSE: Check for existance of requested files to fill a NLAPS order.
         Check file sizes for tape spanning requirements, and to
         generate additional header files when required to span
         output media volumes.
 
PROGRAM HISTORY
Version   Date     Programmer      Code  Reason
-------   ----     --------------  ----- ------
  1.0     04/96    S.Nath           C    Initial development
  1.1     06/97    K.Lowell         C    Add 3490 output 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE
 
PROJECT: PDS
******************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"


main()

{
FILE *ptr;			/* File pointer                    */
FILE *fp1;			/* File pointer                    */

struct PARBLK vblk;		/* TAE parameter block             */

char *strptr;
long bytesize[25];		/* Size of target file(s)          */
long tpsize[25];		/*temp recsiz		           */
float totsize=0;		/* Size of all order files         */
long fatal = -1;		/* Fatal error flag	           */
long std_out=0;			/* standard out		           */
int  volcount;		        /* Volume counter                  */
long no_files=0;		/* number of files per nlaps order */
int  bpv[25];			/* Bands per volume	           */
long jj,zz,mm,nn,kk;		/* Counter for loops	           */
long ii = 0;			/* Counter for loop                */
long tmpsize = 0;		/* Tmp file size hold              */
long splitflg=0;		/* Split header flag	           */
long temp=0;			/* holding counter	           */
long DDVOL=0;                   /* DEM volume file                 */
long DHVOL=0;                   /* DEM header volume file          */
long DDFILE=0;                  /* DEM detail file                 */
long DHFILE=0;                  /* DEM header file                 */

float ninetysize = 750000000;   /* 3490 storage in bytes  	   */
float dlt4size = 18000000;      /* dlt4000 storage in kbytes       */
float dlt7size = 31500000;      /* dlt4000 storage in kbytes       */
float tsize = 194000000;	/* 3480 storage in bytes           */
float ehsize = 45000;           /* 8mm storage in kbytes            */
float elsize = 23000;           /* 8mm low storage in kbytes        */

char  ordnmbr[9];		/* PDS order tnumber               */
char  mtype[5];		        /* PDS media type                  */
long  nunit;                    /* No of units                     */
long  untnmbr;                  /* unit numbers                    */
int   fpv[25];			/* Files per volume                */
long  density=0;		/* Density of media	           */
char  filenames[25][100];       /* filenames for nlaps order       */
int   recdsz[25];		/* Record size of output files     */
int   newnhdr=0;		/* Number of new nhdr files        */
long  bil_bsq;                  /* Presence of order type          */
long  bands;                    /* Number of bands for this order  */
char  demflg[1];                /* DEM flag (Y/N)                  */
char  rec_length[80];           /* Record length of band file      */
char  new_scene[20];            /* Calculated New scene id         */
char  tstamp[100];              /* Time stamp                      */
long  tmprecs;                  /* Integer value for record length */
char  tempstr [10];             /* Temporary storage variable      */
char  callstr[CMLEN];           /* Dir path structure              */
char  tfiles[25][100];          /* temp file names	           */
char  *tempfile[25];            /* Need this for writing to VBLOCK */
char  *scene_ptr[1];            /* Need this for writing to VBLOCK */ 
char  pdsassm[CMLEN];		/* PDS assemble directory          */
char  ordidt[14];               /* PDS order number                */
char  temps[CMLEN];             /* Temporary storage variable      */  
char  temps1[CMLEN];            /* Temporary storage variable      */ 
char  outdir[CMLEN];            /* The directory for all the files */
char  line1[512];               /* Reads a record in this field    */
long  max = 512;                /* Maximum bytes read for fgets    */ 
char  *ch_ptr1;                 /* Temporary character pointer     */
long  index;                    /* Indexing where a sentence starts*/
long  scene_flag;               /* # of scenes for this order      */
long  brkflg=0;                 /* Flag which breaks read loop     */
long  nbr_scenes;               /* # of scenes for this order      */
char  scene_arr[5][20];         /* All scene_ids go to this array  */
char  scenes[3];                /* How many scenes are there       */
char  tmpfile[20];              /* Temporary file                  */
char  trash[CMLEN];		/* trash recepticle		   */
char  row_first[10];            /* WRS beginning row               */
char  row_last[10];             /* WRS ending row                  */
char  ppffile[CMLEN];           /* PPF filename                    */
long  multiflag = 0;            /* Flag specifying multiple scenes */
double dbl_val[1];              /* double value                    */
long  lng_val[1];               /* long value                      */
int   ndf_version;              /* NDF format version */

/* Initialize TAE parameter block
-----------------------------------*/
c_inlas(&vblk,&std_out);

/* Get the variables from the TAE variable block
--------------------------------------------------*/
if ((getpar(&vblk,ordnmbr,mtype,&nunit,&untnmbr,&density,filenames,&bil_bsq,
            &bands,&no_files,demflg,rec_length, tstamp, &ndf_version)) 
            != E_SUCC)
{
  c_errmsg("Fatal error encountered in getpar","ndfcopy-fatal",&fatal);
}

/* Get the location of the PDS assemble directory
--------------------------------------------------*/
strcpy(pdsassm,getenv("PDSASSM"));

/* Fill the stat structure for the requested file(s)
-----------------------------------------------------*/
for (ii=0; ii <25; ++ii)
   {
     fpv[ii] = 0;
     bpv[ii] = 0;
     recdsz[ii]=0;
     tpsize[ii] = 0;
     strcpy(tfiles[ii],"");
     bytesize[ii]=0;
   }

/* Retrieve the new_sceneid if NDF version 1
-----------------------------------------------------*/
if (ndf_version < 2)
{
    scene_flag = 0;
    for (ii=0;ii<no_files;ii++)
     {
      strptr = filenames[ii] + (strlen(filenames[ii]) - 3);
      if ( !strncmp(strptr,".WO",3) )
        index = ii;
     }  
    if ((fp1 = fopen(filenames[index],"r")) == NULL)
      {
        c_errmsg("Error opening .work file","ndfcopy-open",&fatal);
      }

    nbr_scenes = 0;                       
    for(ii=0; ii < 5; ++ii)
       strcpy(scene_arr[ii], "");

    /* Retrieve new_scene_ids from the Work file
    ---------------------------------------------*/
    while (fgets(line1,max,fp1) != NULL)  /* Read the WORK file  */
        {
          index = (long)strspn(line1, " ");
          ch_ptr1 = line1 + index;

          /* Check to see if all needed data has been
             retrieved from the work file
          --------------------------------------------*/
          if (brkflg >= 2)
             break;

          /* If a scene_id has been found in workfile,
             add it to the scene_arr array
          ---------------------------------------------*/
          if (scene_flag > 0)
             {
               strncat(scene_arr[scene_flag - 1],ch_ptr1,18);
               if(scene_flag++ == nbr_scenes)
                  {
                    scene_flag = 0;
                    brkflg++;
                  }
             }

          /* Retrieve the # of scenes
          ----------------------------*/
          if (strncmp(ch_ptr1,"nbr_scenes",10) == 0)
             {
               index = (long)strcspn(ch_ptr1,"=") + 2;
               ch_ptr1 = ch_ptr1 + index;
               strcpy(scenes,ch_ptr1);    
               nbr_scenes = atoi(scenes);
             }

          /* Look for the new_scene_id
          -----------------------------*/
          if (strncmp(ch_ptr1,"new_scene_id",12) == 0)
              scene_flag = 1;

          /* Look for the product code
          -----------------------------*/
          if (strncmp(ch_ptr1,"product_code",12) == 0)
             {
             index = (long)strcspn(ch_ptr1,"=") + 2;
             ch_ptr1 = ch_ptr1 + index;
             index = (long)strcspn(ch_ptr1," \n");
             ch_ptr1 = ch_ptr1 + index - 1;
             if (ch_ptr1[0] == '1')
                multiflag = 1;
             } 

          /* Look for WRS information in Work file
          -----------------------------------------*/
          if (multiflag == 1)
             if (strncmp(ch_ptr1,"output_prod_spc",15) == 0)
                {
                  brkflg++;
                  ch_ptr1 += 18;
                  sprintf(ppffile,"%s/%s/%s",pdsassm,ordnmbr,ordnmbr);

                  /* Get next 4 lines and process rows
                  -------------------------------------*/
                  for (ii=0; ii < 5; ii++)
                     {
                      zz = 0;
                      fgets(line1,max,fp1);
                      index = (long)strspn(line1, "\t ");
                      ch_ptr1 = line1 + index;
                      if (strncmp(ch_ptr1,"out_prod_uc_row",15) == 0)
                         {
                          ch_ptr1 += 18;
                          strcpy(row_first,ch_ptr1);
                          index = (long)strlen(row_first);
                          if (row_first[index - 1] == '\n')
                             zz++;
                          if (row_first[index - 2] == ' ')
                             zz++;
                          row_first[index - zz] = '\0';
                          c_putparm(ppffile,"ROW_FIRST",row_first,"S",dbl_val,1,lng_val);
                         }
                      if (strncmp(ch_ptr1,"out_prod_lc_row",15) == 0)
                         {
                          ch_ptr1 += 18;
                          strcpy(row_last,ch_ptr1);
                          index = (long)strlen(row_last);
                          if (row_last[index - 1] == '\n')
                             zz++;
                          if (row_last[index - 2] == ' ')
                             zz++;
                          row_last[index - zz] = '\0';
                          c_putparm(ppffile,"ROW_LAST",row_last,"S",dbl_val,1,lng_val);
                         }
                     }
                }
        }  

    fclose(fp1);
    ii = 0;
    if(nbr_scenes == 1)
       {
         ch_ptr1 = scene_arr[ii] + 2;
         strcpy(new_scene,ch_ptr1);
       }
    else
       {
         for(; ii<nbr_scenes; ++ii)
            {
              ch_ptr1 = scene_arr[ii] + 2;
              if(ii == 0)
                strncat(new_scene,ch_ptr1,7);
              if(ii == nbr_scenes -1)
                {
                  ch_ptr1 = ch_ptr1 + 4; 
                  strncat(new_scene,ch_ptr1,3);
                  ch_ptr1 = ch_ptr1 + 5;
                  strcat(new_scene,ch_ptr1);
                }
            }
        }
}  /* end NDF version 1 */


/* Load the recdsz array as needed            
-----------------------------------------------------*/

ii = 0;
recdsz[ii++] = 2600;        /** Hardcoded since it is the Header file  **/

sprintf(outdir,"%s%s%s",pdsassm,"/",ordnmbr);
index   = (long)strcspn(rec_length,";");
strncpy(tempstr,rec_length,index);
strncat(tempstr,"\0",index+1);
tmprecs = atoi(tempstr);

if (bil_bsq)    /**  if ordertype = bsq  **/
  {
    for(; ii <= bands; ++ii)
     {
       recdsz[ii] = tmprecs;
     }      
  }
else
    recdsz[ii++] = tmprecs;

recdsz[ii++] = 8192;        /* work order file size hard coded */
recdsz[ii++] = 8192;        /* History file size, hard coded   */
if (strcmp(demflg,"Y") == 0)
  {
    recdsz[ii++] = 2600;    /* Dem Header file size hard coded */
    recdsz[ii] = tmprecs * 2;     /* Demo file size hard coded */
  }

sprintf(tmpfile,"%s%s","tmpfile.",tstamp); 
 
for (ii=0;ii<no_files;ii++)
 {
  strptr = filenames[ii] + (strlen(filenames[ii]) - 3);
  tmpsize = 0;
  sprintf(callstr,"%s %s > %s","ls -l",filenames[ii],tmpfile);
  system(callstr);
  if ((ptr = fopen(tmpfile,"r")) != NULL)
    {
    fscanf(ptr,"%s %s %s %s %ld",trash,trash,trash,trash,&tmpsize);
      fclose(ptr);
      totsize = totsize + (float)tmpsize;
      bytesize[ii] = bytesize[ii] + tmpsize;
    }
  unlink(tmpfile);
  if ( !strncmp(strptr,".DD",3) )
    {
       if ((float)tmpsize > tsize)
             c_errmsg("Dem file does not fit on one tape","ndfcopy-dem",&fatal);
    }
 }


/* Check file(s) size and compare to specified media
   type for possible volume spanning
------------------------------------------------------*/
if (strcmp(mtype,"8HI") == NULL)
  {
   if (totsize < (ehsize*100000))
    {
      volcount = 1;
      fpv[0] = no_files;
    }
   else
    {
    jj = 0;
    volcount = 1;
    tmpsize = 0;
    for (zz = 0; zz < no_files; zz++)
      {
      tmpsize = tmpsize + bytesize[zz];
      if ((ehsize*100000 - (float)tmpsize) >0)
        {
        fpv[jj]++;
        }
      else
        {
        volcount++;
        jj++;
        fpv[jj]++;
        tmpsize = bytesize[zz];
        }
      }
    }
  }
else if (strcmp(mtype,"8LO") == NULL)
  {
  if (totsize < (elsize*100000))
    {
    volcount = 1;
    fpv[0] = no_files;
    }
  else
    {
    jj = 0;
    volcount = 1;
    tmpsize = 0;
    for (zz = 0; zz < no_files; zz++)
      {
      tmpsize = tmpsize + bytesize[zz];
      if ((elsize*100000 - (float)tmpsize) >0)
        {
        fpv[jj]++;
        }
      else
        {
        volcount++;
        jj++;
        fpv[jj]++;
        tmpsize = bytesize[zz];
        }
      }
    }
  }
else if (strcmp(mtype,"3490") == NULL)
  {
  if (totsize < ninetysize)
    {
    volcount = 1;
    fpv[0] = no_files;
    }
  else
    {
    for (jj = 0; jj < 100; jj++) fpv[jj]=0;
    jj = 0;
    volcount = 1;
    tmpsize = 0;
    for (zz = 0; zz < no_files; zz++)
      {
      tmpsize = tmpsize + bytesize[zz];
      if ((tsize - (float)tmpsize) >0)
        {
        fpv[jj]++;
        }
      else
        {
        volcount++;
        jj++;
        fpv[jj]++;
        tmpsize = bytesize[zz];
        }
      }
    }
  }
else if (strcmp(mtype,"3480") == NULL)
  {
  if (totsize < tsize)
    {
    volcount = 1;
    fpv[0] = no_files;
    }
  else
    {

    jj = 0;
    volcount = 1;
    tmpsize = 0;
    for (zz = 0; zz < no_files; zz++)
      {
        tmpsize = tmpsize + bytesize[zz];
        if ((tsize - (float)tmpsize) >0)
          {
            fpv[jj]++;
          }
        else
          {
            volcount++;
            jj++;
            fpv[jj]++;
            tmpsize = bytesize[zz];
          }
      }
    }
  }
else if (strcmp(mtype,"DLT4000") == NULL)
  {
  if (totsize < tsize)
    {
    volcount = 1;
    fpv[0] = no_files;
    }
  else
    {

    jj = 0;
    volcount = 1;
    tmpsize = 0;
    for (zz = 0; zz < no_files; zz++)
      {
        tmpsize = tmpsize + bytesize[zz];
        if ((dlt4size*1000 - (float)tmpsize) >0)
          {
            fpv[jj]++;
          }
        else
          {
            volcount++;
            jj++;
            fpv[jj]++;
            tmpsize = bytesize[zz];
          }
      }
    }
  }
else if (strcmp(mtype,"DLT7000") == NULL)
  {
  if (totsize < tsize)
    {
    volcount = 1;
    fpv[0] = no_files;
    }
  else
    {

    jj = 0;
    volcount = 1;
    tmpsize = 0;
    for (zz = 0; zz < no_files; zz++)
      {
        tmpsize = tmpsize + bytesize[zz];
        if ((dlt7size*1000 - (float)tmpsize) >0)
          {
            fpv[jj]++;
          }
        else
          {
            volcount++;
            jj++;
            fpv[jj]++;
            tmpsize = bytesize[zz];
          }
      }
    }
  }

/* Call to modify and generate additional headers if
   volume spanning is required
------------------------------------------------------*/
if ((volcount > 1) && (bil_bsq == 1))  /* make more NDF headers */
  {
    mm = 0;
    kk = 0;
    for (ii=0;ii<volcount;ii++)
      {
      mm=mm+fpv[ii];
      for (jj=kk;jj<mm;jj++)
        {
          if ((strptr = strstr(filenames[jj],".I")) != NULL)
            {
              bpv[ii]++;
              splitflg=1;
              if ((strptr = strstr(filenames[jj],".IM")) != NULL)
                splitflg=0;
            }
        }
      kk=kk+fpv[ii];
      if (bpv[ii]>0) newnhdr++;
      }

/* call mknhdr here, unless splitflg is 0, then BIL output file which
   won't be split.  Only option is to have image data on one volume */

   if (splitflg == 1)
     {
     strncpy(ordidt,ordnmbr,8);
     ordidt[9]='\0';
     mknhdr(ordnmbr,ordidt,&newnhdr,bpv);
     for (ii=0;ii<mm;ii++)
       {
       strcpy(tfiles[ii],filenames[ii]);
       tpsize[ii]=recdsz[ii];
       } 
     mm=1;
     for (ii=0;ii<volcount;ii++)
       {
       if (ii == 0)
        {
         sprintf(filenames[0],"%s%s%s%s",outdir,"/",ordidt,".H1");
        }
       else if ((ii>0) && (ii < newnhdr))
         {
         temp=temp+fpv[ii-1];
         sprintf(filenames[temp],"%s%s%s%s%d",outdir,"/",ordidt,".H",ii+1);
         recdsz[temp]=recdsz[0];
         for (jj=temp+1;jj<=no_files;jj++)
           {
           strcpy(filenames[jj],tfiles[jj-mm]);
           recdsz[jj]=tpsize[jj-mm];
           }
         fpv[ii]=fpv[ii]++;
         no_files++;
         mm++;
         }
       } 
     
     /* ensure that .DD and .DH files are on same volume */

     sprintf(temps,"%s%s%s%s",outdir,"/",ordidt,".DH");
     sprintf(temps1,"%s%s%s%s",outdir,"/",ordidt,".DD");
     for (ii=0;ii<no_files;ii++)
       {
       if (strcmp(filenames[ii],temps) == NULL) DHFILE=ii+1;
       if (strcmp(filenames[ii],temps1) == NULL) DDFILE=ii+1; 
       }
     if ((DDFILE>0)&&(DHFILE>0)) /* have dem data */
       {
       mm=0;
       nn=0;
       for (ii=0;ii<volcount;ii++)
         {
         mm=mm+fpv[ii];
         if ((DDFILE<=mm) && (DDFILE>nn)) DDVOL=ii+1;
         if ((DHFILE<=mm) && (DHFILE>nn)) DHVOL=ii+1;
         nn=mm;
         }
       if (DHVOL!=DDVOL)
         {
         fpv[DHVOL-1]--;
         fpv[DDVOL-1]++;
         }
       }
     }
  }
else if  ((volcount > 1) && (bil_bsq == 0))
    {
      c_errmsg("BIL order needs more than 1 tape","ndfcopy-ordtype",&fatal);
    }

/* Write number of volumes and filenames required back to TAE
-----------------------------------------------*/
for (ii = 0; ii < no_files; ii++)
    tempfile[ii] = &filenames[ii][0];
scene_ptr[0] = &new_scene[0];

q_intg(&vblk,"NVOL",1,&volcount,P_UPDATE);
q_intg(&vblk,"FPV",25,fpv,P_UPDATE);
q_intg(&vblk,"RECDSZ",25,recdsz,P_UPDATE);
q_intg(&vblk,"NEWNHDR",1,&newnhdr,P_UPDATE);
q_string(&vblk,"TFILES",no_files,tempfile,P_UPDATE);
if (ndf_version < 2)
    q_string(&vblk,"NEW_SCENE",1,scene_ptr,P_UPDATE);
q_out(&vblk);


return(E_SUCC);
}
