/**************************************************************************
NAME: NLGETPAR  
 
PURPOSE: Gets the user-specified parameters for NLAPCKCOPY
 
PROGRAM HISTORY:
 
Version  Date  Programmer      Code  Reason
-------  ----- ----------      ----  ------
  1.0    04/96 S.Nath            C   Initial development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.
PROJECT:  LAS
***************************************************************************/
 
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblk,        /* pointer to initial TAE parameter block */
    char ordnmbr[9],	    /* PDS product name		              */
    char mtype[5],            /* PDS media type		              */
    long *nunit,		    /* unit Number of this order              */
    long *untnmbr,            /* Number of units                        */
    long *density,            /* Output media density	              */
    char filenames[25][100],  /* Disk filenames                         */
    long *bil_bsq,            /* Value for order type                   */
    long *bands,              /* Number of bands                        */
    long *no_files,           /* # of files are in filenames array      */
    char demflg[2],           /* DEM flag (Y/N)                         */
    char rec_length[80],      /* Record length for bands file           */
    char tstamp[100],         /* Time stamp                             */
    int  *ndf_version         /* NDF format version */
)

{
struct VARIABLE *var;       /* ptr to a variable in the vblk          */

long ii=0;		


/* Get the order number
-------------------------*/
var = p_find(vblk,"ORDNMBR");
strcpy (ordnmbr, SVAL(*var,0));


/* Get the media type
------------------------------*/
var = p_find(vblk,"MTYPE");
strcpy(mtype, SVAL(*var,0));

/* Get the number of units in this order
-----------------------------------------*/
var = p_find(vblk,"NUNIT");
*nunit = IVAL(*var,0);

/* Get the unit number for this tape    
-----------------------------------------*/
var = p_find(vblk,"UNTNMBR");
*untnmbr = IVAL(*var,0);

/* Get the density of the tape          
-----------------------------------------*/
var = p_find(vblk,"DENSITY");
*density = IVAL(*var,0);

/* Get the names of the files       
-----------------------------------------*/
var = p_find(vblk,"FILENAMES");
for (ii=0; ii < var->v_count; ii++)
  {
     strcpy(filenames[ii],SVAL(*var,ii));
  }


/* Get bil or bsq flag                  
-----------------------------------------*/
var = p_find(vblk,"BIL_BSQ");
*bil_bsq = IVAL(*var,0);

/* Get number of Bands                
-----------------------------------------*/
var = p_find(vblk,"BANDS");
*bands = IVAL(*var,0);

/* Get number of files                
-----------------------------------------*/
var = p_find(vblk,"NO_FILES");
*no_files = IVAL(*var,0);

/* Get demo flag                      
-----------------------------------------*/
var = p_find(vblk,"DEMFLG");
strcpy(demflg,SVAL(*var,0));

/* Get rec_length flag                      
-----------------------------------------*/
var = p_find(vblk,"REC_LENGTH");
strcpy(rec_length,SVAL(*var,0));

/* Get timestamp flag                      
-----------------------------------------*/
var = p_find(vblk,"TSTAMP");
strcpy(tstamp,SVAL(*var,0));

/* Get NDF format version
-----------------------------------------*/
var = p_find(vblk,"NDF_VERSION");
*ndf_version = IVAL(*var,0);

return(E_SUCC);
}

