/**************************************************************************
NAME: NLGETPAR2
 
PURPOSE: Gets the user-specified parameters for CREATFILE
 
PROGRAM HISTORY:
 
Version  Date  Programmer      Code  Reason
-------  ----- ----------      ----  ------
  1.0    04/96 S.Nath            C   Initial development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.
PROJECT:  LAS
***************************************************************************/
 
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"
#include "protos.h"

int getpar2 
(
    struct PARBLK *vblk,        /* pointer to initial TAE parameter block   */
    char prod_nbr[80],          /* PDS product name		                */
    char ordnmbr[80],           /* PDS order number                         */
    char rad_calib[80],         /* Rad Calibration field from Work file     */
    char prod_code[80],         /* Product Code from Work file              */
    char user_ori[80],          /* User Orientation from Work file          */
    char rad_enha[80],          /* Rad Enhancement from Work file           */
    char dem_line[80],          /* DEM file existance from Work file        */
    char rms_error[80],         /* Rms Error from Work file                 */
    char project[80],           /* Value for project field                  */
    char scene_id[80],          /* Value for Scene_id                       */
    char sequence[10][80],      /* Tape Location values                     */
    long *no_seq,               /* How many tapes are written               */
    char filename[100],         /* Name of the Header file                  */
    char nbr_scenes[10],        /* Number of Scenes from Work file          */
    char spec_band[80],         /* Spectra Band from Work file              */
    char code[80],              /* Project Source                           */
    char mtype[80]              /* Output media type			*/
)
{

struct VARIABLE *var;       /* ptr to a variable in the vblk             */

long ii=0;		    /* counters				         */


/* Get Production number       
-------------------------*/
var = p_find(vblk,"PROD_NBR");
strcpy (prod_nbr, SVAL(*var,0));

/* Get Order number     
-------------------------*/
var = p_find(vblk,"ORDNMBR");
strcpy (ordnmbr, SVAL(*var,0));

/* Get Rad_Calib       
-------------------------*/
var = p_find(vblk,"RAD_CALIB");
strcpy (rad_calib, SVAL(*var,0));

/* Get Prod_Code        
-------------------------*/
var = p_find(vblk,"PROD_CODE");
strcpy (prod_code, SVAL(*var,0));

/* Get User Orientation
-------------------------*/
var = p_find(vblk,"USER_ORI");
strcpy (user_ori, SVAL(*var,0));

/* Get Rad_Enhancement 
-------------------------*/
var = p_find(vblk,"RAD_ENHA");
strcpy (rad_enha, SVAL(*var,0));

/* Get Dem File Flag   
-------------------------*/
var = p_find(vblk,"DEM_LINE");
strcpy (dem_line, SVAL(*var,0));

/* Get the Line2 which was from .WO file
------------------------------*/
var = p_find(vblk,"RMS_ERROR");
strcpy(rms_error, SVAL(*var,0));

/* Get the Line2 which was from .WO file
------------------------------*/
var = p_find(vblk,"PROJECT");
strcpy(project, SVAL(*var,0));

/* Get the Line3 which was from .WO file
------------------------------*/
var = p_find(vblk,"SCENE_ID");
strcpy(scene_id, SVAL(*var,0));

/* Get the Sequence of Tape nos 
------------------------------*/
var = p_find(vblk,"SEQUENCE");
for (ii=0; ii < var->v_count; ii++)
  {
     strcpy(sequence[ii],SVAL(*var,ii));
  }

/* Get the Array size of sequence
------------------------------*/
var = p_find(vblk,"NO_SEQ");
*no_seq = IVAL(*var,0);

/* Get the File names in the array 
------------------------------*/
var = p_find(vblk,"FILENAME");
     strcpy(filename,SVAL(*var,0));

/* Get Nbr_of_scenes   
-------------------------*/
var = p_find(vblk,"NBR_SCENES");
strcpy (nbr_scenes, SVAL(*var,0));

/* Get Spectral-Bands  
-------------------------*/
var = p_find(vblk,"SPEC_BAND");
strcpy (spec_band, SVAL(*var,0));

/* Get Project_Source  
-------------------------*/
var = p_find(vblk,"CODE");
strcpy (code, SVAL(*var,0));

/* Get Media type
-------------------------*/
var = p_find(vblk,"MTYPE");
strcpy (mtype,SVAL(*var,0));
if (strncmp(mtype,"3490",4) == 0)
  strcpy(mtype,"90");
else if (strncmp(mtype,"3480",4) == 0)
  strcpy(mtype,"34");
else if (strncmp(mtype,"8",1) == 0)
  strcpy(mtype,"8M");
else if (strncmp(mtype,"DLT4000",7) == 0)
  strcpy(mtype,"40");
else if (strncmp(mtype,"DLT7000",7) == 0)
  strcpy(mtype,"70");

return(E_SUCC);
}

