/***********************************************************************

NAME: CEHDR_MERGE 
 
PURPOSE:  This program is called from NLDC321                      
          Creates one CEOS neader from multiple CEOS headers.

RECEIVING: This program receives two input parameters.

           1) DIRECTORY : The name of the directory where all the files 
                          are located.
           2) INFILE    : The base of the input file names.

OUTPUT   : None

NOTES    : This module expects to receive header files named as 
           <infile>hdr1.nhdr, <infile>hdr2.nhdr ...
           Name of the merged header will be hdr1.nhdr and all 
           other header files will be discarded.

ALGORITHM REFERENCES:
           "Landsat Technical Working Group (LTWG) format for
            Thematic Mapper (TM) Computer Compatible Tapes (CCTs)"
            Effective: January 1, 1990

PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     05/98   M.Lundblad          C    Initial Development
 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char directory[120],
    char infile[120]
);

int main
(void)

{
  
  struct PARBLK vblk;

  /* Input Variables */
  char directory[CMLEN];       /* Path for pertinent files                   */ 
  char infile[CMLEN];          /* Input filenames base                       */ 

  char filename[CMLEN];        /* Hold filenames that need to opened         */
  char srcfile[CMLEN];         /* Hold filenames that need to opened         */
  char outfile[1][CMLEN];      /* Hold filenames that need to opened         */
  char cmdline[CMLEN];         /* Hold the name of commands                  */
  unsigned char buffer[8280];  /* Hold info for record of the new header     */
  
  long ii = 0;                 /* For loop index variable                    */
  long std_out = 0;            /* Standard Output                            */
  long status = 0;             /* Completion status variable                 */
  long one = 1;                /* Constant one                               */

  FILE *input;                 /*File pointer for existing header file       */
  FILE *output;                /*File pointer for new header file            */

  /* Get parameters from variable block
  --------------------------------------*/
  c_inlas(&vblk,&std_out);
  if ((getpar(&vblk,directory,infile)) != E_SUCC)
     c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input paramters");


  /* Open volume header file for reading
  ---------------------------------------*/
  sprintf(filename,"%s%shdr1.nhdr",directory,infile);
  input = fopen(filename,"r");
  if (!input)
     c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for ceos format");
  status = (long) fread(buffer,1,8280,input);
  if (!status)
     c_lasmsg(TRUE,"nldc321-readhdr","Error reading from header for ceos format");
  fclose(input);

  /* For Volume Descriptor Record 
     Replace Tape Span fields in buffer
  --------------------------------------*/
  buffer[97] = '1';
  buffer[99] = '1';
  buffer[103] = '1';

  /* For Volume File Pointer Records
     Replace Tape Span fields in buffer
  --------------------------------------*/
  for (ii=1; ii<=21; ii++)
    {
    buffer[(ii * 360) + 141] = '1';
    buffer[(ii * 360) + 143] = '1';
    }

  /* For Volume Text (Supplementary) Text Records
     Replace Tape Span fields in buffer
  ------------------------------------------------*/
  buffer[(22 * 360) + 207] = '1';
  buffer[(22 * 360) + 213] = '1';

  /* Create new Volume Header file
  ---------------------------------*/
  sprintf(outfile[0],"%s%s.HD",directory,infile);
  output = fopen(outfile[0],"w");
  if (!output)
     c_lasmsg(TRUE,"nldc321-newhdr","Error opening new header for ceos format");
  status = (long) fwrite(buffer,1,8280,output);
  if (!status)
     c_lasmsg(TRUE,"nldc321-writehdr","Error writing to new header for ceos format.");
  fclose(output);

  /* Remove old volume header files
  ----------------------------------*/
  sprintf(cmdline,"rm -f %s%shdr*.nhdr",directory,infile);
  system(cmdline);

  /* Create New Volume Supplemental File
      and remove old files
  ---------------------------------------*/
  sprintf(srcfile,"%s%s.sp1",directory,infile);
  sprintf(filename,"%s%s.SP",directory,infile);
  sprintf(cmdline,"mv %s %s",srcfile,filename);
  system(cmdline);
  sprintf(cmdline,"rm -f %s%s.sp*",directory,infile);
  system(cmdline);

  /* Create new null Volume Descriptor File for an
     end of tape marker
  -------------------------------------------------*/
  sprintf(filename,"%s%s.NVD",directory,infile);
  sprintf(cmdline,"dd if=/dev/zero of=%s bs=360 count=1 > /dev/null",filename);
  system(cmdline);
  sprintf(cmdline,"rm -f %s%s.nvd*",directory,infile);
  system(cmdline);

  c_complt("CEHDR_MERGE",outfile,&one);

  return EXIT_SUCCESS;
}
