/***********************************************************************

NAME: FAHDR_MERGE 
 
PURPOSE:  This program is called from NLDC321                      
          Creates one header from multiple headers.

RECEIVING: This program receives three input parameters.

           1) DIRECTORY : The name of the directory where all the files 
		          are located.
           2) TOTAL_HDR : Number of headers that need to be
                          merged into one.
           3) INFILE    : Base of input file names to be merged.

OUTPUT   : None

NOTES    : This module expects to receive header files named as 
           <infile>hdr1.nhdr, <infile>hdr2.nhdr ...
           Name of the merged header will be hdr1.nhdr and all 
           other header files will be discarded.

ALGORITHM REFERENCES:
           "EOSAT FAST Format Document for EOSAT Digital Products
              Version C", Effective April 23, 1996
              (Revised July 29, 1996)

PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     01/97   S.Nath             C    Initial Development
          04/98   M.Lundblad         C    Adapted code to from DACNLDC module
                                            to NLDC321.
                                          Added use of fhdr.h and fast
                                            header functions.
 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include <sys/stat.h> 
#include "string.h" 
#include "las.h" 
#include "header.h"
#include "util.h"
#include "fhdr.h" 

int getpar
(
    struct PARBLK *vblk,
    char directory[CMLEN],
    long *total_hdr,
    char infile[CMLEN]
);

int main
(void)

{
  
  struct PARBLK vblk;

  /* Input Variables */
  char directory[CMLEN];     /* Path for pertinent files                   */ 
  char infile[CMLEN];        /* Base for Input file names                  */
  long total_hdr = 0;        /* No headers that need to be merged in one   */

  char filename[1][CMLEN];   /* Hold filenames that need to opened         */
  char cmdline[CMLEN];       /* Hold the name of commands                  */
  char line[1537];           /* Hold the string that was read from file    */
  char bandstr[9]="\0";      /* Hold information about bands               */
  char *str_ptr1;            /* Pointer to manipulate the data read in line*/
  char buffer[2000];         /* Hold info for record of the new header     */
  char gainsarr[7][17];      /* Gains and Bias Array                       */

  long max  = 1537;          /* Maximum characters will be read from file  */
  long index1 = 0;           /* Indexing string for pointers arithmatic    */
  long ii,kk,jj = 0;         /* Counter variable                           */
  long bandnum = 0;          /* Number of bands in header file             */
  long totband = 0;          /* Number of bands in header file             */
  long std_out = 0;          /* Standard output                            */
  long one = 1;              /* constant of one                            */

  FILE *input;               /*File pointer for existing header file       */
  FILE *output;              /*File pointer for new header file            */

  /* Get the variable block and parameters
  -----------------------------------------*/
  c_inlas(&vblk,&std_out);
  if ((getpar(&vblk,directory,&total_hdr,infile)) != E_SUCC)
      c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input paramters");

  /* Loop through the headers and extract band info
  ---------------------------------------------------*/
  for (ii=0; ii < total_hdr; ii++)
     {
     /* bandstr need to be manufactured from
        existing header files in order to
        merge them into one header file
     -----------------------------------------*/
     sprintf(filename[0],"%s%shdr%d.nhdr",directory,infile,ii+1);
     if ((input = fopen(filename[0],"r")) == NULL) 
        c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for fast format");

     if (fgets(line,max,input) !=NULL)
        {
        /* Read the Bands
        ------------------*/
        memset(buffer,'\0',sizeof(buffer));
        c_fhdrrd(line,95,buffer);
        index1 = (long) strcspn(buffer," ");
        buffer[index1] = '\0';
        strcat(bandstr,buffer);
        bandnum = (long) strlen(buffer);
        totband += bandnum;

        /* Read the Gains/Bias
        -----------------------*/
        for (jj=0; jj<bandnum; jj++)
           {
           kk = 21 + (jj * 2);
           memset(buffer,'\0',sizeof(buffer));
           c_fhdrrd(line,kk,buffer);
           index1 = (long) strlen(buffer);
           buffer[index1] = '\0';
           strcpy(gainsarr[jj+(ii*3)],buffer);
           }
        }
        else
           c_lasmsg(TRUE,"nldc321-readhdr","Error reading header for fast format");
     fclose(input);
     memset(line,'\0',sizeof(line));
     } 

  sprintf(filename[0],"%s%shdr%d.nhdr",directory,infile,1);

  /* Open the header file for reading
  ------------------------------------*/
  if ((input = fopen(filename[0],"r")) == NULL)  /* Header File  */
       c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for fast format");

  /* Get buffer from first header file
  -------------------------------------*/
  if (fgets(line,max,input) == NULL)
     c_lasmsg(TRUE,"nldc321-readhdr","Error reading header for fast format");
  fclose(input);

  /* Replace tape spanning and band fields within
     the first header with values that describe
     all bands.
  ------------------------------------------------*/
  memset(buffer,'\0',sizeof(buffer));
  str_ptr1 = strstr(line, "FLAG=");
  if(str_ptr1 != NULL)  /* EOSAT Fast format  */
     {
     /* Update Gain/Bias info for all bands
     ---------------------------------------*/
     for (ii=0; ii<totband; ii++)
        {
        kk = 21 + (ii * 2);
        strcpy(buffer,gainsarr[ii]);
        c_fhdrup(line,kk,buffer);
        }

     /* Replace tape spanning from
        whatever to 1/1     
     ------------------------------*/
     strcpy(buffer,"1/1");
     c_fhdrup(line,35,buffer);

     /* Replace BANDS PRESENT=<whatever> to 
        BANDS PRESENT=<bandstr>
     ----------------------------------------*/
     c_fhdrup(line,95,bandstr);
     }
  else   /* INPE Fast Format  */
     {
     /* Replace BANDS PRESENT=<whatever>
        to BANDS PRESENT=<bandstr>
     ------------------------------------*/
     c_fhdrup(line,95,bandstr);

     /* Replace tape spanning from
        whatever to 1/1     
     ------------------------------*/
     line[1464] = '1';
     line[1473] = '1';
     }

  /* Open new Header file for this order         
  ---------------------------------------*/
  sprintf(filename[0],"%s%s.HD",directory,infile);
  if ((output = fopen(filename[0],"w")) == NULL) 
     c_lasmsg(TRUE,"nldc321-newhdr","Error opening new header for fast format");
  
  /* Put all-band buffer to the new header file
  ----------------------------------------------*/
  fputs(line, output);
  fclose(output);

  /* Delete existing Header files
     for this file
  --------------------------------*/
  sprintf(cmdline,"rm -f %s%s%s",directory,infile,"hdr*");
  system(cmdline);

  c_complt("FAHDR_MERGE", filename, &one);

  return EXIT_SUCCESS;
}
  
