/***********************************************************************

NAME: NLDC321_METAOUT 
 
PURPOSE:  This program is called from NLDC321                      
          Output metadata to file for database ingest.

RECEIVING: This program receives two input parameters.

           1) INGESTDIR : Remote ingest directory
           2) DATADIR   : Metadata root directory
           3) SCENEID   : The scene identifier
           4) OLDTAPEID : Array of old 3480 Tape ID's
           5) NEWTAPEID : Archive Location (tape id) of 3490 tape.

OUTPUT   : None

NOTES    : This program will recieve parameters and stuffs
           the parameters in a metadata file for ingesting. 


PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     02/98   M.Lundblad          C   Initial Development
 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include <sys/stat.h> 
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char ingestdir[CMLEN],     /* Ingest directory                          */
    char datadir[CMLEN],       /* Data directory                            */
    char sceneid[17],          /* Scene Identifier                          */
    char oldtapeid[10][7],     /* Array of old tape Id's                    */
    char newtapeid[7],         /* New storage Location                      */
    long *tape_count           /* Number of Old Tape Id's                   */
);


int main
(void)

{
  
  struct PARBLK vblk;
  
  /* Input Variables */
  char ingestdir[CMLEN];     /* Ingest directory                          */
  char datadir[CMLEN];       /* Metadata root directory                   */
  char sceneid[17];          /* Scene Identifier                          */
  char oldtapeid[10][7];     /* Array of old tape Id's                    */
  char newtapeid[7];         /* New storage Location                      */
  long tape_count=0;         /* Number of Old Tape Id's                   */

  char olddatadir[CMLEN];    /* Directory for old meta data               */
  char filename[1][CMLEN];   /* Hold filenames that need to opened        */
  char ingsystem[20];        /* Remote system name                        */
  char ingpath[120];         /* Remote path name                          */
  char line[50];             /* Hold the string that was read from file   */
  char buffer[100];          /* Generic buffer                            */
  char cmdline[CMLEN];       /* Command line buffer                       */
  char *str_ptr;             /* Temporary string pointer                  */
  
  long ii = 0;               /* Counter variable                          */
  long index = 0;            /* String index variable                     */
  long status = 0;
  long std_out = 0;
  long one = 1;   

  FILE *input;               /*File pointer for data directory listing     */
  FILE *output;              /*File pointer for metadata file              */

  c_inlas(&vblk,&std_out);

  /* Retrieve input parameters
  -----------------------------*/
  if ((getpar(&vblk,ingestdir,datadir,sceneid,oldtapeid,newtapeid,&tape_count)) != E_SUCC)
      c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input parameters");

  /* Setup old meta directory
  ----------------------------*/
  sprintf(olddatadir,"%sold_data/",datadir);

  /* Extract the system name and path from ingest dir
  ----------------------------------------------------*/
  index = (long) strcspn(ingestdir,":");
  strncpy(ingsystem,ingestdir,index);
  ingsystem[index] = '\0';
  str_ptr = &ingestdir[index + 1];
  strcpy(ingpath,str_ptr);

  /* If metadata already exists, then error out
     else create the metadata file
  -----------------------------------------------*/
  sprintf(filename[0],"%s%s",datadir,newtapeid);
  if (access(filename[0],0) == 0)
     c_lasmsg(TRUE,"nldc321-metaout","New metadata file for %s already exists",newtapeid);
  else
     {
     if ((output = fopen(filename[0],"w")) == NULL) 
        c_lasmsg(TRUE,"nldc321-openmeta","Error opening new metadata file for %s",newtapeid);

     /* For each input tape write a pipe-delim. line
     ------------------------------------------------*/
     for (ii=0; ii<tape_count; ii++)
       {
       sprintf(line,"%s|%s|%s|90|\n",sceneid,oldtapeid[ii],newtapeid);
       status = fputs(line,output);
       if (!status)
         c_lasmsg(TRUE,"nldc321-writmeta","Error writing to new metadata file for %s",newtapeid);
       }

     fclose(output);
     }

  /* Copy the new meta file to the old data directory
     for archival purposes
  ----------------------------------------------------*/
  sprintf(cmdline,"cp %s %s",filename[0],olddatadir);
  system(cmdline);

  /* Copy the new meta data file to a remote
     directory where it can be read for ingest
  ---------------------------------------------*/
  for (ii=0; ii<3; ii++)
     {
     sprintf(cmdline,"rcp %s %s.",filename[0],ingestdir);
     if (system(cmdline) == 0)
        {
        /* Double check the remote copy by listing meta
           file in remote directory
        ------------------------------------------------*/
        sprintf(cmdline,"rsh %s '/usr/bin/ls %s%s'",ingsystem,ingpath,newtapeid);
        if ((input = popen(cmdline, "r")) != NULL)
           if (fgets(buffer,100,input) == NULL)
              c_lasmsg(FALSE,"nldc321-rcpagain","Error remote copying meta data...try again");
           else
              break;
        else
           c_lasmsg(FALSE,"nldc321-rcpagain","Error remote copying meta data...try again");
        }
     else
        c_lasmsg(FALSE,"nldc321-rcpagain","Error remote copying meta data...try again");

     /* If three attempts exceeded, then
          error out with fatal
     ------------------------------------*/
     if (ii == 2)
        {
        /* Remove the meta file
        ------------------------*/
        sprintf(cmdline,"rm -f %s",filename[0]);
        system(cmdline);
        c_lasmsg(TRUE,"nldc321-rcpmeta","Error remote copying %s meta file to directory",newtapeid);
        }
     }

     /* Remove the meta file
     ------------------------*/
     sprintf(cmdline,"rm -f %s",filename[0]);
     system(cmdline);

     c_complt("NLDC321_METAOUT",filename,&one);

     return EXIT_SUCCESS;
}
