/***********************************************************************

NAME: NLDC321_READMETA 
 
PURPOSE:  This program is called from NLDC321                      
          Read meta data from OPS NLDC metadata files.

RECEIVING: This program receives three input parameters.

           1) DIRECTORY  : Working directory.
           2) FORMAT     : Format of scene.
           3) TAPEID     : Input tape location array.

OUTPUT   : This program returns three output parameters.
           1) SCENEID    : Scene identifier.
           1) OLDSCENEID : Scene identifier.
           2) PROJECT    : Project name.
           3) CODE       : Working code.

NOTES    : This program will read metadata file keying on input
           tape locations.  The metadata will verified against 
           the actual data, and the sceneid, project and code will
           be passed back. 


PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     02/98   M.Lundblad          C    Initial Development
 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include <sys/stat.h> 
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char directory[CMLEN],     /* Working directory                      */
    char format[13],           /* Scene Identifier                       */
    char tapeid[10][7],        /* Array of tape ids                      */
    long *tape_count          /* Count of tape ids                      */
);


int main
(void)

{
  
  struct PARBLK vblk;
  
  /* Input Variables */
  char directory[CMLEN]="";    /* Working Directory                         */
  char format[13]="";        /* Scene format                              */
  char tapeid[10][7];        /* Array of old tape Id's                    */
  long tape_count=0;         /* Count of input tapes                      */

  /* Output Variables */
  char sceneid[17]="";       /* Scene Identifier                          */
  char oldsceneid[17]="";    /* Old Scene Identifier                      */
  char project[12]="";       /* Project name                              */
  char code[4]="";           /* Code for the project                      */

  char filename[CMLEN];      /* Hold filenames that need to opened        */
  char line[500];            /* Hold the string that was read from file   */
  char buffer[50];           /* Generic buffer                            */
  char *tempstr[1];          /* Temporary String Pointer                  */
  
  long index;                /* Temporary string index                    */
  long status = -1;          /* Status variable                           */
  long std_out = 0;          /* Standard Output                           */
  long max=500;              /* Maximum number of bytes to read in        */ 

  FILE *input;               /* File pointer for metadata file            */

  /* Inititalize variable block
  ------------------------------*/
  c_inlas(&vblk,&std_out);

  /* Retrieve input parameters
  -----------------------------*/
  if ((getpar(&vblk,directory,format,tapeid,&tape_count)) != E_SUCC)
      c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input parameters");

  /* Open meta file for reading
  ------------------------------*/
  sprintf(filename,"%s%s.meta",directory,format);
  if ((input = fopen(filename,"r")) == NULL) 
     c_lasmsg(TRUE,"nldc321-fatal","Error opening operations work file for %s",format);

  /* Loop until the file tape id matches
     the first element of the tapeid array.
  ------------------------------------------*/
  while (fgets(line,max,input) != NULL)
     {
     strncpy(buffer,&line[3],6);
     buffer[6] = '\0';
     if (strcmp(buffer,tapeid[0]) == 0)
        {
        /* Get New Sceneid
        ---------------*/
        strncpy(sceneid,&line[17],16);
        index = (long) strcspn(sceneid," ");
        if (index > 16)
           index = 16;
        sceneid[index] = '\0';

        /* Get Old Sceneid
        ---------------*/
        strncpy(oldsceneid,&line[35],13);
        index = (long) strcspn(oldsceneid," ");
        if (index > 13)
           index = 13;
        oldsceneid[index] = '\0';

        /* Get Project
        ---------------*/
        strncpy(project,&line[50],11);
        project[11] = '\0';

        /* Get Code
        ---------------*/
        strncpy(code,&line[74],3);
        code[3] = '\0';

        break;
        }
     }

  /* Output Project
  -----------------*/
  tempstr[0] = &project[0];
  status = q_string(&vblk,"PROJECT",1,tempstr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing project to variable block (#%d)",status);

  /* Output New SceneID
  ------------------*/
  tempstr[0] = &sceneid[0];
  status = q_string(&vblk,"SCENEID",1,tempstr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing sceneid to variable block (#%d)",status);

  /* Output Old SceneID
  ------------------*/
  tempstr[0] = &oldsceneid[0];
  status = q_string(&vblk,"OLDSCENEID",1,tempstr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing oldsceneid to variable block (#%d)",status);

  /* Output Code
  ---------------*/
  tempstr[0] = &code[0];
  status = q_string(&vblk,"CODE",1,tempstr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing code to variable block (#%d)",status);

  q_out(&vblk);
  
  exit(E_SUCC);

  return EXIT_SUCCESS;

}
