/***********************************************************************

NAME: NLDC321_UPMETA 
 
PURPOSE:  This program is called from NLDC321                      
          Complete NLDC321 transcription process by updating
          status flag in OPS metadata file to a certain status
          either 'y', 'n', 'e' or 'i'.

RECEIVING: This program receives four input parameters.

           1) DIRECTORY : Meta data directory.
           2) FORMAT    : Format of scene.
           3) TAPEID    : Input tape location array.
           4) TPSTATUS  : Status of the scene.

NOTES    : This program will read metadata file keying on input
           tape locations.  After finding the appropriate metadata
           records, the status flasg will be updated and output
           tapeid will be appended.


PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     03/98   M.Lundblad          C    Initial Development
 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char directory[CMLEN],     /* Working directory                         */
    char format[13],           /* Format of the scene                       */
    char tapeid[10][7],        /* Array of tape ids                         */
    char tpstatus[2],          /* Status of the scene                       */
    long *tape_cnt            /* Count of tape ids                         */
);


int main
(void)

{
  
  struct PARBLK vblk;

  /* Input Variables */
  char directory[CMLEN];       /* Working Directory                         */
  char format[13];           /* Scene format                              */
  char tapeid[10][7];        /* Array of old tape Id's                    */
  char tpstatus[2];          /* Status of the scene                       */
  long tape_cnt = 0;         /* Count of input tapes                      */

  char filename[CMLEN];        /* Hold filenames that need to opened        */
  char lockfile[CMLEN];        /* Hold filenames that need to opened        */
  char cmdline[500];         /* Unix Command Line Operation String        */
  char oldbuf[15];           /* Old buffer                                */
  char newbuf[15];           /* New buffer                                */
  char metastat[2];          /* Tape Id status in working meta file       */
  char buffer[500];          /* Temporary string buffer                   */
  char line[500];            /* Line buffer read from file                */
  
  long ii = 0;               /* Counter variable                          */
  long nvol=0;               /* Number of expected tape volumes           */
  long std_out = 0;
  long tapefound=0;          /* Tape present flag.  1=Found 0=Not Found   */
  long max=500;              /* Maximum number of bytes to read in        */

  FILE *input;               /* File pointer for meta data file           */

  c_inlas(&vblk,&std_out);

  /* Retrieve input parameters
  -----------------------------*/
  if ((getpar(&vblk,directory,format,tapeid,tpstatus,&tape_cnt)) != E_SUCC)
      c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input parameters");

  /* Check to see if meta file is locked
  ---------------------------------------*/
  sprintf(lockfile,"%s%s.lock",directory,format);
  while (access(lockfile,0) == 0)
     sleep(1);

  /* Lock file for rest of this program
  --------------------------------------*/
  sprintf(cmdline,"touch %s",lockfile);
  system(cmdline);

  /* Determine if all tapeids exists in the working meta file
  ------------------------------------------------------------*/
  sprintf(filename,"%s%s.meta",directory,format);
  if ((input = fopen(filename,"r")) == NULL)
     {
     sprintf(cmdline,"rm -f %s",lockfile);
     system(cmdline);
     c_lasmsg(TRUE,"nldc321-openwrk","Error opening operations work file for %s",format);
     }

  /* Loop until the file tape id matches
     the first element of the tapeid array.
  ------------------------------------------*/
  while (fgets(line,max,input) != NULL)
     {
     strncpy(buffer,&line[3],6);
     buffer[6] = '\0';
     if (strcmp(buffer,tapeid[0]) == 0)
        {
        tapefound = 1;

        /* Check to see if # of tape = # exp. tapes
        --------------------------------------------*/
        strncpy(buffer,&line[14],1);
        buffer[1] = '\0';
        nvol = atol(buffer);
        if (nvol != tape_cnt)
           {
           sprintf(cmdline,"rm -f %s",lockfile);
           system(cmdline);
           c_lasmsg(TRUE,"nldc321-tapecnt","Number of input tapes does not match number of expected");
           }

        /* Set status of tape id from meta data file
        ---------------------------------------------*/
        strncpy(metastat,&line[0],1);
        metastat[1] = '\0';

        /* Check for existance of other ids
        ------------------------------------*/
        for (ii=1; ii<tape_cnt; ii++)
           {
           tapefound = 0;
           fgets(line,max,input);
           strncpy(buffer,&line[3],6);
           buffer[6] = '\0';
           if (strcmp(buffer,tapeid[ii]) != 0)
              {
              sprintf(cmdline,"rm -f %s",lockfile);
              system(cmdline);
              c_lasmsg(TRUE,"nldc321-tapelst","Tape series not found in operations working meta file");
              }
           else
              tapefound = 1;
           }
        break;
        }
     }

  fclose(input);

  /* If the tape id was not found, then error out
  ------------------------------------------------*/
  if (tapefound == 0)
     {
     sprintf(cmdline,"rm -f %s",lockfile);
     system(cmdline);
     c_lasmsg(TRUE,"nldc321-tapelst","Tape series not found in operations working meta file");
     }

  /* If the tapes are Inprocesses, do not run
  --------------------------------------------*/
  if (metastat[0] == 'i' && tpstatus[0] == 'i')
     {
     sprintf(cmdline,"rm -f %s",lockfile);
     system(cmdline);
     c_lasmsg(TRUE,"nldc321-tpstatus","Tape ids with a status of inprocess will not be run");
     }

  /* If the tapes have a status of "Y", do not run
  -------------------------------------------------*/
  if (metastat[0] == 'y' && tpstatus[0] != 'n')
     {
     sprintf(cmdline,"rm -f %s",lockfile);
     system(cmdline);
     c_lasmsg(TRUE,"nldc321-tpstatus","Tape ids with a status of complete will not be run");
     }

  /* If the tapes have a status of "e" and
     meta does not have "i",do not run
  ------------------------------------------*/
  if (metastat[0] == 'e' || (tpstatus[0] == 'e' && metastat[0] != 'i'))
     {
     sprintf(cmdline,"rm -f %s",lockfile);
     system(cmdline);
     c_lasmsg(TRUE,"nldc321-tpstatus","Tape ids with a status of error will not be run");
     }

  /* Update the status of the scene in the meta data
     file with the new status using a sed command
  ---------------------------------------------------*/
  ii = 0;
  sprintf(cmdline,"sed");
  for (ii=0; ii<tape_cnt; ii++)
     {
     sprintf(oldbuf,"%s  %s",metastat,tapeid[ii]);
     strcpy(newbuf,oldbuf);
     newbuf[0] = tpstatus[0];
     sprintf(cmdline,"%s -e 's/%s/%s/'",cmdline,oldbuf,newbuf);
     }
  sprintf(cmdline,"%s %s",cmdline,filename);
  sprintf(cmdline,"%s > %s%s.tmp",cmdline,directory,format);
  if (system(cmdline) != 0)
     {
     /* If error
     ------------*/
     sprintf(cmdline,"rm -f %s",lockfile);
     system(cmdline);
     c_lasmsg(TRUE,"nldc321-updwork","Error updating operations work file for %s format",format);
     }
  sprintf(cmdline,"mv %s%s.tmp %s",directory,format,filename);
  system(cmdline);

  /* Unlock meta file by removing .lock
  ---------------------------------------*/
  sprintf(cmdline,"rm -f %s",lockfile);
  system(cmdline);

  exit(E_SUCC);

  return EXIT_SUCCESS;

}
