/***********************************************************************

NAME: NLDCFAST_READ
 
PURPOSE:  This program is called from NLDC321.                     
          1)  Allocates the master header file for the entity
              in the output file array (write_file).
          2)  Calculates the # of bands per tape header and keeps
              the total # of bands for the entity.
          3)  Copies the header files to 'hdr#.nhdr'
          4)  Copy all image files to '<fastfile>.i#'
              of bands, and total # of headers for the entity.

RECEIVING: This program receives four input parameters.

           1) DIRECTORY     : The name of the directory where all the files 
				are located.
           2) TAPE_COUNT    : The number of input tapes read for the entity.
           3) INFILE        : The base file name of all input files.
           4) FILE_PER_TAPE : An array of number of input files per tape.

OUTPUT: This program returns four output parameter.

           1) WRITE_FILE  : Array of sorted file names to be returned to
				the calling program.
           2) REC_SIZE    : Array of Record Lengths of images.
           3) SCENE_ID    : Scene Identifier.
           4) TAPE_DESC   : Archive Tape Additional Description

ALGORITHM REFERENCES:
           "EOSAT FAST Format Document for EOSAT Digital Products
              Version C", Effective April 23, 1996
              (Revised July 29, 1996)

PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     04/98   Lundblad            C   Initial Development
 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include <sys/stat.h> 
#include "string.h" 
#include "las.h" 
#include "header.h"
#include "util.h"
#include "fhdr.h" 

int getpar
(
    struct PARBLK *vblk,
    char directory[CMLEN],
    long *tape_count,
    char infile[CMLEN],
    long file_per_tape[295]
);

int main
(void)

{

  struct PARBLK vblk;        /* Variable block                             */

  /* Input Variables */
  char directory[CMLEN];     /* Path for files that were read from input   */ 
  char infile[CMLEN];        /* Base of file name for input files          */
  long tape_count=0;         /* Number of input tapes read in.             */
  long file_per_tape[295];   /* Array of number of files per input tape    */

  /* Output Variables */
  char write_file[295][CMLEN]; /* Hold file names for writing to tapes.      */
  long rec_size[295];        /* Record Length of the image file            */
  char scene_id[18];         /* Scene Identification Number                */
  char tape_desc[50];        /* Archive Tape Label Additional Description  */

  char ppprrr[10];            /* path/row value                            */
  char momx[5];               /* mode/mux value                            */
  char satno[3];              /* satellite number                          */
  char date[6];               /* julian date                               */
  char month[5],day[5],year[5];/* temp work variables                      */
  char srcfile[200];          /* Source generic file name buffer           */
  char tgtfile[200];          /* Target generic file name buffer           */
  char *tempfile[500];        /* Temporary file buffer                     */
  char *tempstr[1];           /* Temporary string buffer                   */
  char filename[200];         /* Preset "image_" file name                 */
  char cmdline[200];          /* Hold the name of commands                 */
  char header[HEADER_SIZE];   /* Fast Format Header Bufffer                */
  char buffer[512];           /* generic buffer                            */
  char bandstr[9]="\0";       /* Hold info about bands                     */

  long mn,dy,yr,jdate=0;      /* temp work values                          */
  long img_size=0;            /* Temporary Image size                      */
  long max = 2000;            /* Maximum header length                     */
  long std_out = 0;           /* Standard output                           */
  long status = -1;           /* Command status variable                   */
  long ii, jj, kk = 0;        /* Generic increment variables               */
  long img = 0;               /* Image incrementer                         */
  long len;                   /* Temporary length                          */
  long trlflag = 0;           /* Trailer Flag                              */
  long bandfile = 0;          /* Number of bands on each tape              */
  long bandfile1 = 0;         /* Number of Bands for first tape            */
  long index1 = 0;            /* Temporary string index                    */

  FILE *fp;                   /* File pointer for header file              */

  /* Get the variable block
  --------------------------*/
  c_inlas(&vblk,&std_out);
  if ((getpar(&vblk,directory,&tape_count,infile,file_per_tape)) != E_SUCC)
     c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input paramters");

  /* Rename files to working filenames
  -------------------------------------*/
  sprintf(filename,"%s%s%s",directory,infile,".HD");
  strcpy(write_file[kk++],filename);

  for (ii = 1; ii < tape_count +1; ii++)
     {
     /* Get Number of bands on each tape
     ------------------------------------*/
     sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,1,ii);

     /* Open and read header file
     -----------------------------*/
     if ((fp = fopen(srcfile,"r")) == NULL)
        c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for fast format");
     if (fgets(header,max,fp) == NULL)
        c_lasmsg(TRUE,"nldc321-readhdr","Error reading from header for fast format");
     fclose(fp);

     /* Calculate number of band present on tape
     --------------------------------------------*/
     c_fhdrrd(header,95,buffer);
     index1 = (long) strcspn(buffer," ");
     buffer[index1] = '\0';
     strcat(bandstr,buffer);
     bandfile = (long) strlen(buffer);

     /* Save tape 1, bandfile for later use with trailer files
     ---------------------------------------------------------*/
     if (ii == 1)
        bandfile1 = bandfile;

     /* Rename tape header file to hdr#.nhdr
     ----------------------------------------*/
     sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,1,ii);
     sprintf(tgtfile,"%s%shdr%d.nhdr",directory,infile,ii);
     sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
     system(cmdline);

     /* Rename band files on tape to <infile>.i#
     --------------------------------------------*/
     for (jj = 1; jj <= bandfile; jj++)
        {
        sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,jj+1,ii);
        sprintf(tgtfile,"%s%s.i%d",directory,infile,++img);
        sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
        system(cmdline);
        strcpy(write_file[kk++],tgtfile);
        }
     }    

  /* Check to see if trailer files are included on the input
     tapes.  If so, then rename and add to write_file array.
  -----------------------------------------------------------*/
  /* Number of bands on tape1 + header
  -------------------------------------*/
  bandfile1 ++;

  /* If files on tape1 = Bandfile1 + 1, then
     trailer file is present at end of tapes.
  --------------------------------------------*/
  if (file_per_tape[0] == (bandfile1 + 1))
     {
     /* Set trailer flag to true
     ----------------------------*/
     trlflag = 1;

     /* Copy Trailer file1 to output trailer file
     ---------------------------------------------*/
     sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,file_per_tape[0],1);
     sprintf(tgtfile,"%s%s%s",directory,infile,".TR");
     sprintf(cmdline,"cp %s %s",srcfile,tgtfile);
     system(cmdline);
     strcpy(write_file[kk++],tgtfile);

     /* Remove all tape trailer files
        they are no longer needed
     ---------------------------------*/
     for (ii=0; ii < tape_count; ii++)
        {
        sprintf(filename,"%s%s_%d.tape%d",directory,infile,file_per_tape[ii],ii+1);
        sprintf(cmdline,"rm -f %s",filename);
        system(cmdline);
        }
     }
      
  /* Open Tape 1 header and extract information needed for
     tape label and verification.
  ---------------------------------------------------------*/
  sprintf(filename,"%s%shdr1.nhdr",directory,infile);

  /* Open and read header file
  ------------------------------*/
  if ((fp = fopen(filename,"r")) == NULL)
     c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for fast format");
  if (fgets(header,max,fp) == NULL)
     c_lasmsg(TRUE,"nldc321-readhdr","Error reading from header for fast format");
  fclose(fp);

  /* Record Length
  -----------------*/
  strcpy(buffer,"");
  strncat(buffer,&header[1405],5);
  img_size = atol(buffer);
  for (ii=0; ii<kk; ii++)
     rec_size[ii] = img_size;

  /* Record Length of Trailer File is different
  ----------------------------------------------*/
  if (trlflag == 1)
     rec_size[kk - 1] = 80;

  /* Satellite number */
  c_fhdrrd(header,8,satno);
  sprintf(scene_id,"%s",&satno[1]);

  /* path/row */
  c_fhdrrd(header,4,ppprrr);
  len = (long) strlen(ppprrr);
  for (ii=0; ii<len; ii++)
     if (ppprrr[ii] == ' ')
        ppprrr[ii] = '0';
  strncat(scene_id,ppprrr,3);
  strncat(scene_id,&ppprrr[4],3);
  strncat(scene_id,"00",2);

  /* acquistion date */
  c_fhdrrd(header,6,buffer);
  strncat(scene_id,&buffer[2],2);
  strncpy(year,buffer,4);
  strncpy(month,&buffer[4],2);
  strncpy(day,&buffer[6],2);
  yr = atol(year);
  mn = atol(month);
  dy = atol(day);
  c_grejul(&yr,&mn,&dy,&jdate);
  if (jdate < 10)
     sprintf(buffer,"00%d",jdate);
  else if (jdate < 100)
     sprintf(buffer,"0%d",jdate);
  else
     sprintf(buffer,"%d",jdate);
  strncat(scene_id,buffer,3);
  strncpy(date,&scene_id[9],5);

  /* mode/mux */
  c_fhdrrd(header,10,momx);
  if (strcmp(momx,"TM  ") == 0)
     strcpy(momx,"TM10");
  strncat(scene_id,&momx[2],2);

  /* Create additional description for tape label
  ------------------------------------------------*/
  strncpy(buffer,ppprrr,7);
  buffer[3] = ' ';
  buffer[7] = '\0';
  sprintf(tape_desc,"%s %s %02d/%02d/%02d",scene_id,buffer,mn,dy,(yr % 100));


  /* Return the output Parameters
  --------------------------------*/

  /* Ouput filename array
  ------------------------*/
  for (ii=0; ii < kk; ii++)
     tempfile[ii] = &write_file[ii][0];
  status = q_string(&vblk,"WRITE_FILE",kk,tempfile,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing write_file to variable block (#%d)",status);

  /* Output array of record sizes for files
  ------------------------------------------*/
  status = q_intg(&vblk,"REC_SIZE",kk,&rec_size[0],P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing rec_size to variable block (#%d)",status);

  /* Output the scene id
  -----------------------*/
  tempstr[0] = &scene_id[0];
  status = q_string(&vblk,"SCENE_ID",1,tempstr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing scene_id to variable block (#%d)",status);

  /* Output the tape description for the label
  ---------------------------------------------*/
  tempstr[0] = &tape_desc[0];
  status = q_string(&vblk,"TAPE_DESC",1,tempstr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing tape_desc to variable block (#%d)",status);
  
  q_out(&vblk);

  exit(E_SUCC);

  return EXIT_SUCCESS;
}
