/***********************************************************************

NAME: NLDCNDF_MERGE 
 
PURPOSE:  This program is called from NLDC321                      
          Creates one header from multiple headers.

RECEIVING: This program receives five input parameters.

           1) DIRECTORY : The name of the directory where all the files 
		          are located.
           2) INFILE    : Base file name
           3) NHDR      : Number of headers that need to be
                          merged into one.
           4) NBANDS    : Base of input file names to be merged.
           5) FIRSTTAPE : Number of files on first input tape.

OUTPUT   : None

PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
 1.0      05/98   M.Lundblad         C    Initial Development
 
 
NOTES    : This module expects to receive header files named as 
           <infile>hdr1.nhdr, <infile>hdr2.nhdr ...
           Name of the merged header will be hdr.nhdr and all 
           other header files will be discarded.

           This program was based off of "nhdr_merge" written
           by C. Engebretson.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include "math.h"
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char directory[CMLEN],
    char infile[CMLEN],
    long *nhdr,
    long *nbands,
    long *firsttape
);

int main
(void)

{
  struct PARBLK vblk;
  
  /* Input Variables */
  char directory[CMLEN];  /* Path for pertinent files                   */ 
  char infile[CMLEN];     /* Filename base                              */
  long nhdr = 0;          /* Number of header files to merge            */
  long nbands = 0;        /* Number of bands in scene                   */
  long firsttape = 0;     /* Number of files for first tape             */

  char srcfile[CMLEN];    /* Hold filenames that need to opened         */
  char tgtfile[1][CMLEN]; /* Hold filenames that need to opened         */
  char cmdline[CMLEN];    /* Hold the name of commands                  */
  char buf[CMLEN];        /* Hold the string that was read from file    */
  char buffer[CMLEN];     /* Hold the string that was read from file    */
  char *strptr;           /* Pointer to manipulate strings              */

  long index = 0;         /* Indexing string for pointers arithematic   */
  long ii = 0;            /* Counter variable                           */
  long lpv = 0;           /* Lines per volume                           */
  long std_out = 0;       /* Standard output                            */
  long one = 1;           /* constant of one                            */

  FILE *input;            /* File pointer for existing header file      */
  FILE *output;           /* File pointer for new header file           */

  /* Get the variable block and parameters
  -----------------------------------------*/
  c_inlas(&vblk,&std_out);
  if ((getpar(&vblk,directory,infile,&nhdr,&nbands,&firsttape)) != E_SUCC)
      c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input paramters");

  /* Open output file for writing
  --------------------------------*/
  sprintf(tgtfile[0],"%s%shdr.nhdr",directory,infile);
  if ((output = fopen(tgtfile[0],"w")) == NULL) 
     c_lasmsg(TRUE,"nldc321-newhdr","Error opening new header for nldc format");

  /* Loop through the headers and extract band info
  --------------------------------------------------*/
  for (ii=1; ii<=nhdr; ii++)
     {
     /* Open header file to read information
     ----------------------------------------*/
     sprintf(srcfile,"%s%shdr%d.nhdr",directory,infile,ii);
     if ((input = fopen(srcfile,"r")) == NULL) 
        c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for ndf format");

     /* Read each line of the header file, change
        designated fields, and put to output file
     ---------------------------------------------*/
     while (fgets(buf,CMLEN,input) !=NULL)
        {
        /* Update following fields for first file only
        -----------------------------------------------*/
        if (ii == 1)
           {
           /* Put correct tape spanning info to output file
           -------------------------------------------------*/
           if (strncmp(buf,"TAPE_SPANNING_FLAG",18) == 0)
              fputs("TAPE_SPANNING_FLAG=1/1;\n", output);

           /* Put correct number of bands to output file
           ----------------------------------------------*/
           else if (strncmp(buf, "NUMBER_OF_BANDS_IN_VOLUME", 25) == 0)
              {
              sprintf(buf,"NUMBER_OF_BANDS_IN_VOLUME=%d;\n",nbands);
              fputs(buf,output);
              }

           /* Calculate the Lines per Volume based on number
              of bands in scene and output to file
           --------------------------------------------------*/
           else if (strncmp(buf,"LINES_PER_VOLUME",16) == 0)
              {
              strptr = strstr(buf,"=") + 1;
              index = (long) strcspn(strptr,";");
              strncpy(buffer,strptr,index);
              buffer[index] = '\0';
              lpv = (atol(buffer) / firsttape) * nbands;
              sprintf(buf,"LINES_PER_VOLUME=%d;\n",lpv);
              fputs(buf,output);
              }

           /* If buffer is not end of header, then put
              to output file
           --------------------------------------------*/
           else if (strncmp(buf, "END_OF_HDR", 10) != 0)
              fputs(buf, output);
           }
        else
           /* If not first tape and line is Band information,
              then put to output file
           ---------------------------------------------------*/
           if (strncmp(buf, "BAND", 4) == 0)
              fputs(buf, output);
        }

     /* Close input file
     --------------------*/
     fclose(input);
     }

  /* Put End of Header mark on the header
  ----------------------------------------*/
  fputs("END_OF_HDR;\n", output);

  /* Close output file
  ---------------------*/
  fclose(output);

  /* Clean up old header files
  -----------------------------*/
  for (ii=1; ii<=nhdr; ii++)
     {
     sprintf(cmdline,"rm -f %s%shdr%d.nhdr",directory,infile,ii);
     system(cmdline);
     }

  c_complt("NLDCNDF_MERGE", tgtfile, &one);

  return EXIT_SUCCESS;
}
