/***********************************************************************

NAME: NLDCNDF_READ 
 
PURPOSE:  This program is called from NLDC321.                     
          1) Allocates the first element of the output filename array to
             the master header for the entity.
          2) Calculate the number of bands and total # of headers.
          3) If a DEM exists copy DEM image and header to <ordnmbr>.dd and
             <ordnmbr.dh>.
          4) Copy Work Order Request and Entity History files to
             <ordnmbr.wo> and <ordnmbr.hi>.
          5) Copy header files to <hdr#.nhdr> and image files to <ordnmbr.i#>.
          6) Output parameters used for media output and tape label.

RECEIVING: This program receives four input parameters.

          1) DIRECTORY  : The name of the directory where all the files 
				are located.
          2) TAPE_COUNT : The number of tapes read in.
          3) LONE_DEM   : The flag to determine existing DEM files.
          4) INFILE     : The base file name for input files.

OUTPUT: This program returns five output parameter.

          1) WRITE_FILE : Array of file names to be returned to the
				calling program.
          2) TOTAL_BAND : Total # of bands for the entity.
          3) REC_SIZE   : Record Size of each line of image file.
          4) SCENE_ID   : Scene Identifier.
          5) TAPE_DESC  : Archive Tape Additional description.

PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     09/96   Nath/Hopkins        C   Initial Development
          05/98   Lundblad            C   Make changes to the following:
                                            - create rec_size array
                                            - reorganize for clarity
                                            - remove BSQ checking
                                            - extract information from the
                                              header for tape label
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char directory[CMLEN],
    long *tape_count,
    long *lone_dem,
    char infile[CMLEN]
);

int main
(void)

{

  struct PARBLK vblk;

  /* Input Parameters */
  char directory[CMLEN];     /* Path for files read from input media         */ 
  char infile[CMLEN];        /* Base filename for input files                */
  long tape_count = 0;       /* No of units for this order                   */
  long lone_dem = 0;         /* Whether a DEM exists alone on last input tape*/

  /* Output Parameters */
  char write_file[295][CMLEN]; /* Hold file names for writing to tapes       */
  char scene_id[17];         /* Scene Identification                         */
  char tape_desc[50];        /* Additional Description for Tape Label        */
  int  total_band = 0;       /* Total # of bands for the entity.             */
  long recsize[295];         /* Record size of each line of image.           */

  char path[4];              /* Path for Scene used with tape_desc           */
  char row[4];               /* Row for the Scene used with tape_desc        */
  char srcfile[200];         /* Source file name                             */
  char tgtfile[200];         /* Target file name                             */
  char *tempfile[500];       /* Temporary file buffer                        */
  char *tempstr[1];          /* Temporary String buffer                      */
  char filename[200];        /* Preset "image_" file name                    */
  char cmdline[200];         /* Hold the name of commands                    */
  char line[512];            /* Buffer holding each NDF header line          */
  char *str_ptr;             /* generic character pointer                    */
  char buffer[512];          /* generic string buffer use for formatting     */
  char band_file[10];        /* Contains bands found in the scene            */

  long imgsize = 0;          /* Image record size                            */
  long yr = 0,mn = 0,dy = 0; /* Temporary Date pieces                        */
  long max = 512;            /* Maximum Line Size                            */
  long index1 = 0;           /* generic integer index of string formatting   */
  long index2 = 0;           /* generic integer index of string formatting   */
  long tmp = 0;              /* temporary ingteger                           */
  long dem = 0;              /* dem flag                                     */
  long std_out = 0;          /* Standard output                              */
  long status = -1;          /* Status variable                              */
  long ii,jj,kk = 0;         /* Generic incrementers and counters            */
  long img = 0;              /* Count of image files                         */
  long bandnum = 0;          /* Number of bands in the scene                 */

  FILE *fp;                  /* File pointer for header file                 */

  /* Get the variable block
  --------------------------*/
  c_inlas(&vblk,&std_out);
  if ((getpar(&vblk,directory,&tape_count,&lone_dem,infile)) != E_SUCC)
     c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input paramters");

  /* Setup initial variables
  ----------------------------*/
  sprintf(filename,"%s%shdr.nhdr",directory,infile);
  recsize[kk] = 8192;
  strcpy(write_file[kk++],filename);


  /* Loop through all input tapes and perform processing
     to the header file
  -------------------------------------------------------*/
  for (ii = 1; ii <= tape_count; ii++)
     {
     bandnum = 0;
     sprintf(filename,"%s%s_1.tape%d",directory,infile,ii);

     /* Read data from header file if tape is not the last
        tape of the series, and it is not a dem tape
     ------------------------------------------------------*/
     if ((!lone_dem) || (tape_count != ii))
        {
        if ((fp = fopen(filename,"r")) == NULL)  /* Header File  */
           c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for ndf format");

        /* Loop through the header file to extract desired info
        --------------------------------------------------------*/
        jj = 0;
        while (fgets(line,max,fp) != NULL)
           {
           if (strncmp(line,"NUMBER_OF_BANDS_IN_VOLUME",25) == 0)
              {
              index1 = (long) (strcspn(line,"=") + 1);
              index2 = (long) strcspn(line,";");
              str_ptr = line + index1;
              strncpy(buffer,str_ptr,index2 - index1);
              buffer[index2 - index1] = '\0';
              sprintf(band_file,"%s",buffer);
              bandnum = atol(band_file);
              jj++;
              }
           if (strncmp(line,"RECORD_SIZE",11) == 0)
              {
              index1 = (long) (strcspn(line,"=") + 1);
              index2 = (long) strcspn(line,";");
              str_ptr = line + index1;
              strncpy(buffer,str_ptr,index2 - index1);
              buffer[index2 - index1] = '\0';
              imgsize = atol(buffer);
              jj++;
              }
           if (strncmp(line,"ACQUISITION_DATE/TIME",21) == 0)
              {
              index1 = (long) (strcspn(line,"=") + 1);
              str_ptr = line + index1;
              strncpy (buffer,str_ptr,2);
              buffer[2] = '\0';
              mn = atol(buffer);
              str_ptr += 2;
              strncpy (buffer,str_ptr,2);
              buffer[2] = '\0';
              dy = atol(buffer);
              str_ptr += 2;
              strncpy (buffer,str_ptr,2);
              buffer[2] = '\0';
              yr = atol(buffer);
              jj++;
              }
           /* If all data fields have been found,
              then break loop
           ---------------------------------------*/
           if (jj >= 3)
              break;
           }
        fclose(fp);
        }

     total_band += bandnum;

     /* If the tape is not a lone dem tape
     --------------------------------------*/
     if (!lone_dem)
        {
        /* Rename first file on the tape to header file
        ------------------------------------------------*/
        sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,1,ii);
        sprintf(tgtfile,"%s%shdr%d.nhdr",directory,infile,ii);
        sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
        system(cmdline);

        /* Loop for all bands on the tape and rename the
           band image files to .i#
        -------------------------------------------------*/
        for (jj = 1; jj <= bandnum; jj++)
           {
           sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,jj+1,ii);
           sprintf(tgtfile,"%s%s.i%d",directory,infile,++img);
           sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
           system(cmdline);
           recsize[kk] = imgsize;
           strcpy(write_file[kk++],tgtfile);
           }

        /* If it is the last tape, then rename the last
           two files on the tape to the work file (.wo)
           and the history file (.hi)
        ------------------------------------------------*/
        if (tape_count == ii)
           {
           /* Rename last to files on the tape to work file (.wo)
              and history file (.hi)
           -------------------------------------------------------*/
           sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,bandnum+2,ii);
           sprintf(tgtfile,"%s%s.wo",directory,infile);
           sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
           system(cmdline); 
           recsize[kk] = 8192;
           strcpy(write_file[kk++],tgtfile);

           sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,bandnum+3,ii);
           sprintf(tgtfile,"%s%s.hi",directory,infile);
           sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
           system(cmdline); 
           recsize[kk] = 8192;
           strcpy(write_file[kk++],tgtfile);

           /* Open work file and extract the number of dems
              Set dem flag to true if # > than 0
           -------------------------------------------------*/
           sprintf(filename,"%s%s.wo",directory,infile);
           if ((fp = fopen(filename,"r")) == NULL)  
              c_lasmsg(TRUE,"nldc321-openwo","Error opening ndf work file");
 
           /* Loop through work file until nbr_dems is reached
           ----------------------------------------------------*/
           while (fgets(line,max,fp) !=NULL)
              if (strncmp(line,"      nbr_dems",14) == 0)
                 {
                 index1 = (long) (strcspn(line,"=") + 2);
                 str_ptr = line + index1;
                 strncpy(buffer,str_ptr,1);
                 buffer[1] = '\0';
                 tmp = atol(buffer);
                 if(tmp > 0)
                    dem = 1; 
                 else
                    dem = 0;         
                 break;
                 }
 
           fclose(fp);

           /* If dem flag true then rename dem files and add to array
           -----------------------------------------------------------*/
           if (dem)
              {
              sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,bandnum+4,tape_count);
              sprintf(tgtfile,"%s%s.dh",directory,infile);
              sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
              system(cmdline);
              recsize[kk] = imgsize;
              strcpy(write_file[kk++],tgtfile);

              sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,bandnum+5,tape_count);
              sprintf(tgtfile,"%s%s.dd",directory,infile);
              sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
              system(cmdline);
              recsize[kk] = imgsize;
              strcpy(write_file[kk++],tgtfile);
              }
           }
        }
     /* If the tape has two DEM files
     ---------------------------------*/
     else
        {
        /* If it is the last tape, rename the dem
           files and start loop over
        ------------------------------------------*/
        if (tape_count == ii)
           {
           sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,1,ii);
           sprintf(tgtfile,"%s%s.dh",directory,infile);
           sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
           system(cmdline); 
           recsize[kk] = imgsize;
           strcpy(write_file[kk++],tgtfile);

           sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,2,ii);
           sprintf(tgtfile,"%s%s.dd",directory,infile);
           sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
           system(cmdline); 
           recsize[kk] = imgsize;
           strcpy(write_file[kk++],tgtfile);

           continue;
           }

        /* Rename first file on the tape to header file
        ------------------------------------------------*/
        sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,1,ii);
        sprintf(tgtfile,"%s%shdr%d.nhdr",directory,infile,ii);
        sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
        system(cmdline);

        /* Loop for all bands on the tape and rename the
           band image files to .i#
        -------------------------------------------------*/
        for (jj = 1; jj <= bandnum; ++jj)
           {
           sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,jj+1,ii);
           sprintf(tgtfile,"%s%s.i%d",directory,infile,++img);
           sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
           system(cmdline);
           recsize[kk] = imgsize;
           strcpy(write_file[kk++],tgtfile);
           }

        /* If it is the second to last tape, then
           rename the last two files on the tape to
           the work file (.wo) and the history file (.hi)
        --------------------------------------------------*/
        if (tape_count-ii == 1)
           {
           /* WOR file */
           sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,bandnum+2,ii);
           sprintf(tgtfile,"%s%s.wo",directory,infile);
           sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
           system(cmdline); 
           recsize[kk] = 8192;
           strcpy(write_file[kk++],tgtfile);

           /* HI file */
           sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,bandnum+3,ii);
           sprintf(tgtfile,"%s%s.hi",directory,infile);
           sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
           system(cmdline); 
           recsize[kk] = 8192;
           strcpy(write_file[kk++],tgtfile);
           }
        }
     }

   /* Open WOR file and extract sceneid
   -------------------------------------*/
   sprintf(filename,"%s%s.wo",directory,infile);
   if ((fp = fopen(filename,"r")) == NULL)
      c_lasmsg(TRUE,"nldc321-openwo","Error opening ndf work file");

   /* Extract SceneId
   -------------------*/
   while (fgets(line,max,fp) !=NULL)
      if (strncmp(line,"      new_scene_id",18) == 0)
         {
         if (fgets(line,max,fp) == NULL)
            c_lasmsg(TRUE,"nldc321-readwo","Error reading from ndf work file");
         str_ptr = line + 8;
         strncpy(scene_id,str_ptr,16);
         scene_id[16] = '\0';
         break;
         }

   /* Extract Path from SceneId
   -----------------------------*/
   str_ptr = scene_id;
   str_ptr += 1;
   strncpy(path,str_ptr,3);
   path[3] = '\0';

   /* Extract Row from SceneID
   ----------------------------*/
   str_ptr += 3;
   strncpy(row,str_ptr,3);
   row[3] = '\0';

   /* Build Tape Description
   --------------------------*/
   sprintf(tape_desc,"%s %s %s %02d/%02d/%02d",scene_id,path,row,mn,dy,(yr % 100));

   /* Output all Parameters
   -------------------------*/

   /* Output array of file names
   ------------------------------*/
   for(ii=0; ii < kk; ii++)
      tempfile[ii] = &write_file[ii][0];
   status = q_string(&vblk,"WRITE_FILE",kk,tempfile,P_UPDATE);
   if (status != SUCCESS)
      c_lasmsg(TRUE,"nldc321-q_out","Error writing write_file to variable block (#%d)",status);

   /* Output number of bands in scene
   -----------------------------------*/
   status = q_intg(&vblk,"TOTAL_BAND",1,&total_band,P_UPDATE);
   if (status != SUCCESS)
      c_lasmsg(TRUE,"nldc321-q_out","Error writing total_band to variable block (#%d)",status);

   /* Output array of record sizes
   --------------------------------*/
   status = q_intg(&vblk,"REC_SIZE",kk,&recsize[0],P_UPDATE);
   if (status != SUCCESS)
      c_lasmsg(TRUE,"nldc321-q_out","Error writing rec_size to variable block (#%d)",status);

   /* Output the scene id
   -----------------------*/
   tempstr[0] = &scene_id[0];
   status = q_string(&vblk,"SCENE_ID",1,tempstr,P_UPDATE);
   if (status != SUCCESS)
      c_lasmsg(TRUE,"nldc321-q_out","Error writing scene_id to variable block (#%d)",status);

   /* Output tape description for label
   -------------------------------------*/
   tempstr[0] = &tape_desc[0];
   status = q_string(&vblk,"TAPE_DESC",1,tempstr,P_UPDATE);
   if (status != SUCCESS)
      c_lasmsg(TRUE,"nldc321-q_out","Error writing tape_desc to variable block (#%d)",status);

   q_out(&vblk);

   exit(E_SUCC);

   return EXIT_SUCCESS;
}
