/***********************************************************************

NAME: NLDCTRAN_READ 
 
PURPOSE:  This program is called from NLDC321.
          1)  Allocates the master header file for the scene
              in the output file array (write_file).
          2)  Calculates the total # of headers for the scene.
                    total_hdr = total#files / 2
          4)  Copies the header files to '<infile>hdr#.nhdr'
          5)  Copy all image files to '<infile>.i#'
          6)  Returns output parameters for header merge and
              tape label.

RECEIVING: This program receives four input parameters.

           1) DIRECTORY : The name of the directory where all the files 
				are located.
           2) TAPE_COUNT: The number of input tapes read.
           3) TAPE_FILE : Array of number of files per tape.
           4) INFILE    : Base of input file names.

OUTPUT: This program returns four output parameter.

           1) WRITE_FILE: Array of file names to be returned to
				the calling program.
           2) TOTAL_HDR : Total number of headers for the entity.
           3) SCENE_ID  : Scene Identifier.
           4) TAPE_DESC : Tape description used for tape label.
           5) REC_SIZE  : Array of record sizes of the files for scene.

PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     09/96   Nath/Hopkins       C    initial Development
  1.1     02/98   Lundblad           C    Continued development.
                                          Added the record size array, scene_id,
                                          and tape_desc output parameters
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include <sys/stat.h> 
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char directory[CMLEN],
    long *tape_count,
    long tape_file[295],
    char infile[CMLEN]
);

int main
(void)

{
  
  struct PARBLK vblk;
  
  /* Input Variables */
  char directory[CMLEN];     /* Path for files that were read from input    */ 
  char infile[CMLEN];        /* Input file name base.                       */
  long tape_count=0;         /* Total # of input tapes read                 */
  long tape_file[295];       /* Number of files per tape                    */

   /* Output Variables */
  char write_file[295][CMLEN]; /* Hold file names for writing to tapes.       */
  char scene_id[16];         /* Scene Identifier                            */
  char tape_desc[50];        /* Additional Tape Description                 */
  int  total_hdr=0;          /* Hold file names for writing to tapes.       */
  long recsize[295];         /* Image record size                           */

  char path[3],row[3];       /* Path and Row                                */
  char mon[3],day[3],year[3]; /* Temporary Date buffers                     */
  char *outstr[1];            /* Output String pointer buffer               */
  char srcfile[200];         /* Source file name                            */
  char tgtfile[200];         /* Target file name                            */
  char *tempfile[500];       /* Temporary file buffer                       */
  char filename[200];        /* Preset "image_" file name                   */
  char cmdline[200];         /* Hold the name of commands                   */
  char *str_ptr;             /* Generic string pointer                      */
  char tmpstr[20];           /* Temporary String                            */

  long imgsize=0;            /* Image record length                         */
  long total_file = 0;       /* Number of files in scene                    */
  long index=0;              /* integer used for string parsing             */
  long index2=0;             /* integer used for string parsing             */
  long std_out = 0;          /* Standard Output                             */
  long status = -1;          /* Status of commands                          */
  long ii, jj, kk = 0;       /* incrementers and counters                   */
  long img = 0;              /* Number of image files in the scene          */
  long hdr = 0;              /* Number of header files in the scene         */

  char buffer[2000]; /* Binary buffer for Header file              */

  FILE *fptr;                 /* File pointer to header                     */
  
  /* Retrieve variable block
  ---------------------------*/
  c_inlas(&vblk,&std_out);

  /* Get parameters from parameter block
  ---------------------------------------*/
  if ((getpar(&vblk,directory,&tape_count,tape_file,infile)) != E_SUCC)
      c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input paramters");

  /* Allocate first element of write array to master header
  ----------------------------------------------------------*/
  sprintf(filename,"%s%s%s",directory,infile,".HD");
  strcpy(write_file[kk++],filename);

  /* Calculate the total number of files
  ---------------------------------------*/
  for (ii = 0; ii < tape_count; ii++)
       total_file = total_file + tape_file[ii];

  /* Calculate the number of bands and headers for the entity
  ------------------------------------------------------------*/
  total_hdr = total_file/2;

  /* If file is a header copy it to <infile>hdr#.nhdr
     else copy the file to <ordnmbr>.i#
  ----------------------------------------------------*/
  for (ii = 0; ii < tape_count; ii++)
     {
       for (jj=1; jj < tape_file[ii] + 1; ++jj)
          {
            if(jj % 2 )
               {
                  sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,jj,ii+1);
                  sprintf(tgtfile,"%s%shdr%d.nhdr",directory,infile,++hdr);
                  sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
                  system(cmdline);
               }
            else
               { 
                  sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,jj,ii+1);
                  sprintf(tgtfile,"%s%s.i%d",directory,infile,++img);
                  sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
                  system(cmdline);
                  strcpy(write_file[kk++],tgtfile);
               }
          }    
     }

  /* Open the header file for reading
  ------------------------------------*/
  sprintf(filename,"%s%shdr1.nhdr",directory,infile);
  fptr = fopen(filename,"r");
  if (!fptr)
     c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for transfer format");
  status = (long) fread(buffer,1,2000,fptr);
  if (!status)
     c_lasmsg(TRUE,"nldc321-readhdr","Error reading from header for transfer format");

  fclose(fptr);

  /* Extract Scene ID
  --------------------*/
  if (strstr(buffer,"]") != NULL)
     index = (long) strcspn(buffer,"]") + 1;
  else
     index = (long) strcspn(buffer,":") + 1;
  index2 = (long) strcspn(buffer,"!") - index;
  strncpy(scene_id,&buffer[index],index2);
  c_low2up(scene_id,&index2);

  /* Extract Path
  ----------------*/
  index = (long) strcspn(buffer,"!") + 1;
  strncpy(path,&buffer[index],3);

  /* Extract Row
  ----------------*/
  index += 4;
  strncpy(row,&buffer[index],3);

  /* Extract Acquistion Date
  ---------------------------*/
  index += 4;
  strncpy(mon,&buffer[index],2);
  index += 2;
  strncpy(day,&buffer[index],2);
  index += 2;
  strncpy(year,&buffer[index],2);

  /* Build Tape Description
  --------------------------*/
  sprintf(tape_desc,"%s %s %s/%s/%s",path,row,mon,day,year);

  /* Determine the image record length
  -------------------------------------*/
  str_ptr = strstr(buffer,"NS:") + 3;
  strncpy(tmpstr,str_ptr,4);
  tmpstr[4] = '\0';
  imgsize = atol(tmpstr);
  for (ii=0; ii<kk; ii++)
    recsize[ii] = imgsize;


  /* Send Output parameters back to parblock
  -------------------------------------------*/

  /* Output the array of file names
  ----------------------------------*/
  for(ii=0; ii < kk; ii++)
      tempfile[ii] = &write_file[ii][0];
  status = q_string(&vblk,"WRITE_FILE",kk,tempfile,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing write_file to variable block (#%d)",status);

  /* Output the scene id
  -----------------------*/
  outstr[0] = &scene_id[0];
  status = q_string(&vblk,"SCENE_ID",1,outstr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing scene_id to variable block (#%d)",status);

  /* Output the tape description
  -------------------------------*/
  outstr[0] = &tape_desc[0];
  status = q_string(&vblk,"TAPE_DESC",1,outstr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing tape_desc to variable block (#%d)",status);

  /* Output the number of headers
  --------------------------------*/
  status = q_intg(&vblk,"TOTAL_HDR",1,&total_hdr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing total_hdr to variable block (#%d)",status);

  /* Output the array of record sizes
  ------------------------------------*/
  status = q_intg(&vblk,"REC_SIZE",kk,&recsize[0],P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing rec_size to variable block (#%d)",status);

  q_out(&vblk);

 exit(E_SUCC);

 return EXIT_SUCCESS;

}
  
