/***********************************************************************

NAME: TRHDR_MERGE 
 
PURPOSE:  This program is called from NLDC321.
          Creates one transfer header from multiple tranfer-formatted
          headers.

RECEIVING: This program receives three input parameters.

           1) DIRECTORY : The name of the directory where all the files 
		          are located.
           2) TOTAL_HDR : Number of headers that need to be
                          merged into one.
           3) INFILE    : Input file name base.

OUTPUT   : None

NOTES    : This module expects to receive header files named as 
           <infile>hdr1.nhdr, <infile>hdr2.nhdr ...
           Name of the merged header will be <infile>hdr1.nhdr and all 
           other header files will be discarded.


PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     01/97   S.Nath             C     Initial Development
  1.1     02/98   M.Lundblad         C     Continuing Development
                                           Added the check for EOR in the
                                             header buffer.
 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include <sys/stat.h> 
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char directory[CMLEN],
    long *total_hdr,
    char infile[CMLEN]
);

int main
(void)

{
  
  struct PARBLK vblk;
  
  /* Input Variables */
  char directory[CMLEN];     /* Path for pertinent files                   */ 
  char infile[CMLEN];        /* Input file name base                       */ 
  long total_hdr=0;          /* No headers that need to be merged in one   */

  char filename[CMLEN];      /* Hold filenames that need to opened         */
  char outfile[1][CMLEN];    /* Hold output filename                       */
  char cmdline[CMLEN];       /* Hold the name of commands                  */
  char bandnum[2];           /* Hold number of bands                       */
  char ff=12;                /* Form feed character                        */
  char *str_ptr;             /* Generic string pointer                     */
  char *str_ptr2;            /* Generic string pointer                     */
  char *band_ptr;            /* String pointer to band specific info       */
  char buffer[2000]; /* Buffer containing header file             */
  char outbuf[5000]; /* Buffer containing header file             */

  long index=0;              /* temporary array index                      */
  long bufsize=0;            /* Size of a buffer                           */
  long hdrsize=0;            /* Size of a buffer                           */
  long ii=0;                 /* Loop variable counter                      */
  long std_out=0;            /* Standard Output                            */
  long one = 1;              /* Constant of one                            */

  struct stat buf;           /* buffer structure declaration               */

  FILE *input;               /* File pointer for existing header file      */
  FILE *output;              /* File pointer for new header file           */

  /* Retrieve the Variable Block
  -------------------------------*/
  c_inlas(&vblk,&std_out);

  /* Retrieve the input parameters from the variable block
  ---------------------------------------------------------*/
  if ((getpar(&vblk,directory,&total_hdr,infile)) != E_SUCC)
      c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input paramters");

  /* Get size of header file
  ---------------------------*/
  sprintf(filename,"%s%shdr1.nhdr",directory,infile);
  stat(filename,&buf);
  hdrsize = (long) buf.st_size;

  /* Open header file for input
  ------------------------------*/
  if ((input = fopen(filename,"r")) == NULL)  /* Header File  */
     c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for transfer format");

  /* Read the header for the entire size of the header
  -----------------------------------------------------*/
  if (!(fread(buffer,1,hdrsize,input)))
     c_lasmsg(TRUE,"nldc321-readhdr","Error reading from header for transfer format");
  fclose(input);


/* Make changes to buffer to signify no tapespanning
-----------------------------------------------------*/
  /* Change Number Bands (NB) to correct number
  ----------------------------------------------*/
  sprintf(bandnum,"%d",total_hdr);
  str_ptr = strstr(buffer,"NB:") + 3;
  str_ptr[0] = bandnum[0];

  /* Calculate the position of band specific information
  -------------------------------------------------------*/
  index = (long) strlen(buffer);
  if (index < hdrsize)
     {
     index ++;
     band_ptr = &buffer[index];
     }
  else
     {
     band_ptr = strchr(buffer,ff);
     index = (long)band_ptr - (long)buffer;
     }

  /* Copy non-scene-specific info to output buffer
  -------------------------------------------------*/
  strncpy(outbuf,buffer,index);

  /* Calc the Buffer size minus non-scene
  ----------------------------------------*/
  bufsize = hdrsize - index;

  /* Update the minimum and maximum fields
     for TM 7 band and 6 band scenes
  -----------------------------------------*/
  if (total_hdr >= 6)
     {
     /* Update Minimum value in the first band
     ------------------------------------------*/
     str_ptr = strstr(band_ptr,"MINIMUM:") + 8;
     str_ptr2 = strstr(str_ptr,"Valid:") + 6;
     if (strncmp(str_ptr2,"INVALID",7) == 0)
        strncpy(str_ptr2,"VALID  ",7);

     /* Update Maximum value in the first band
     ------------------------------------------*/
     str_ptr = strstr(band_ptr,"MAXIMUM:") + 8;
     str_ptr2 = strstr(str_ptr,"Valid:") + 6;
     if (strncmp(str_ptr2,"INVALID",7) == 0)
        {
        strncpy(str_ptr,"2.55000000000000E+02",20);
        strncpy(str_ptr2,"VALID  ",7);
        }
     }

  /* Make changes to first band's information, then
    append changes for the rest of the files.
  --------------------------------------------------*/
  for (ii = 0; ii < total_hdr; ii++)
    {
     /* Change BAND NO: to the band number
     --------------------------------------*/
     sprintf(bandnum,"%d",ii+1);
     str_ptr2 = strstr(band_ptr,"BAND NO:") + 8;
     str_ptr2[0] = bandnum[0];
 
     /* Copy Band Specific Information to Output Buffer
     ---------------------------------------------------*/
     strncpy(&outbuf[index + (ii * bufsize)],band_ptr,bufsize);
     }

  /* Put buffer changes to new output header
  -------------------------------------------*/
  sprintf(filename,"%s%sndr.nhdr",directory,infile);
  if ((output = fopen(filename,"w")) == NULL) 
     c_lasmsg(TRUE,"nldc321-newhdr","Error opening new header for transfer format");

  /* Write the entire output buffer to the file
  ----------------------------------------------*/
  if (!(fwrite(outbuf,1,(index + (total_hdr * bufsize)),output)))
     c_lasmsg(TRUE,"nldc321-writehdr","Error writing to new header for transfer format");
  fclose(output);

  /* Delete existing Header files for this file
  ----------------------------------------------*/
  sprintf(cmdline,"rm -r %s%shdr*",directory,infile);
  system(cmdline);

  /* Rename header file for appropriate name
  -------------------------------------------*/
  sprintf(filename,"%s%sndr.nhdr",directory,infile);
  sprintf(outfile[0],"%s%s.HD",directory,infile);
  sprintf(cmdline,"mv %s %s",filename,outfile[0]);
  system(cmdline);

  c_complt("TRHDR_MERGE", outfile, &one);

  return EXIT_SUCCESS;
}
  
