/* kernel header file */

/* general kernal types */

typedef K_LONG GSTATUS;
typedef K_LONG SIZE;
typedef K_LONG MILLISECONDS;
typedef K_LONG STATUS;
typedef K_TEXT IDNAME;
typedef K_INT  ID;

/* general kernel definitions */
#define K_NULL 0
#define MAXMSGSZ 8192
#define MBXNAMSIZ 16		/* max. size of a mailbox name */

/*****************************************************************************
 *				Kernel status codes
 ****************************************************************************/
/*
 *	Success codes.
 */
#define	SUCCESS		(GSTATUS) 0	/* successful completion */
/*
 *	Error codes.
 */
#define	FAILURE		(GSTATUS) -101	/* not successful completion */
#define	INVPARM		(GSTATUS) -1	/* invalid parameter */
#define	INSUFMEM	(GSTATUS) -2	/* insufficient memory */
#define	NOPRIV		(GSTATUS) -3	/* no privilege to perform function */
#define	NOTFOUND	(GSTATUS) -4	/* item not found */
#define	DELETED		(GSTATUS) -5	/* structure deleted */
#define	HOSTERROR	(GSTATUS) -6	/* host service error */
#define	DUPLICATE	(GSTATUS) -7	/* duplicate object */
#define	DATALOST	(GSTATUS) -8	/* data lost on data write */
#define	INVMODE		(GSTATUS) -9	/* service not permitted in current mode */
#define	DISCONNECTED	(GSTATUS) -10	/* mailbox disconnected */
#define	NORECEIVE	(GSTATUS) -11	/* no message to receive */
#define	TRUNCATED	(GSTATUS) -12	/* truncation on data read */
#define	NOT_IMPLEMENTED	(GSTATUS) -99	/* service not implemented */
/*
 *	Socket errors.
 */
#define	KC_SOCK_SOCK	(GSTATUS) -401	/* socket error */
#define	KC_SOCK_CON	(GSTATUS) -402	/* connect error */
#define	KC_SOCK_SEND 	(GSTATUS) -403	/* send error */
#define	KC_SOCK_RECV 	(GSTATUS) -404	/* RECV error */
#define	KC_SOCK_SEL 	(GSTATUS) -405	/* select error */
#define	KC_SOCK_IOCTL 	(GSTATUS) -406	/* ioctl error */

/*
 *	Function declarations.
 */
K_FUNCTION GSTATUS BUFCREATE ();
K_FUNCTION GSTATUS BUFDELETE();
K_FUNCTION GSTATUS BUFGSIZE ();
K_FUNCTION GSTATUS BUFREAD ();
K_FUNCTION GSTATUS BUFWRITE ();
K_FUNCTION GSTATUS BUFCOPY ();
K_FUNCTION GSTATUS BUFCONCATENATE ();
K_FUNCTION GSTATUS BUFPREPEND ();
K_FUNCTION GSTATUS BUFAPPEND ();
K_FUNCTION GSTATUS BUFUNAPPEND ();
K_FUNCTION GSTATUS BUFUNPREPEND ();
K_FUNCTION GSTATUS EVASSOCIATE ();
K_FUNCTION GSTATUS EVDEASSOCIATE ();
K_FUNCTION GSTATUS EVAWAIT ();
K_FUNCTION GSTATUS EVAWAITS ();
K_FUNCTION GSTATUS EVDECLARE ();
K_FUNCTION GSTATUS EVCANCEL ();
K_FUNCTION GSTATUS EVCREATE ();
K_FUNCTION GSTATUS EVDELETE ();
K_FUNCTION GSTATUS EVREAD();
K_FUNCTION GSTATUS EVSIGNAL ();
K_FUNCTION GSTATUS ATCHFD();
K_FUNCTION GSTATUS MBXACCEPT ();
K_FUNCTION GSTATUS MBXCONEVENT ();
K_FUNCTION GSTATUS MBXCONNEEVENT ();
K_FUNCTION GSTATUS MBXDISCEVENT ();
K_FUNCTION GSTATUS MBXCONNECT ();
K_FUNCTION GSTATUS MBXGETNAME ();
K_FUNCTION GSTATUS MBXRCVEVENT ();
K_FUNCTION GSTATUS MBXSETNAME ();
K_FUNCTION GSTATUS MBXCREATE ();
K_FUNCTION GSTATUS MBXDELETE ();
K_FUNCTION GSTATUS MBXDISCONNECT ();
K_FUNCTION GSTATUS MBXRECEIVE ();
K_FUNCTION GSTATUS MBXSEND ();
K_FUNCTION GSTATUS TIMCREATE ();
K_FUNCTION GSTATUS TIMDELETE ();
K_FUNCTION GSTATUS TIMEXPEVENT ();
K_FUNCTION GSTATUS TIMSTART ();
K_FUNCTION GSTATUS TIMSTOP ();
