#include "edc_types.h"
#ifdef EGNET
#include "tranlib.h" 
#endif
#include "pserver.h"
#include "ncp.h"

#ifdef EGNET
#define IMG_BUFFSZ BUFFSZ 
#else
#define IMG_BUFFSZ 1048576 
#endif

#define NULLVAL		-1		/* null value */

#define NTOPEN 		 1		/* open network connection */
#define NTCLOSE		 2		/* close network connection */
#define NTCMTAE		 3		/* network convert host to tae name */
#define NTCMHOS		 4		/* network convert tae to host name */
#define NTOPIMG		 5		/* open remote image */
#define NTCLIMG		 6		/* close remote image */
#define NTRDIMG		 7		/* read remote image */
#define NTWRIMG		 8		/* write remote image */
#define NTOPLS		 9		/* open remote label services file */
#define NTCLLS		10		/* close remote label services file */
#define NTRDLS		11		/* read remote label service file */
#define NTWRLS		12		/* write remote label service file */
#define NTMKLS		13		/* make remote LS file name */
#define NTDDR		14		/* update DDR fields on remote host */
#define NTOPFIL		15		/* open flat file */
#define NTRDFIL		16		/* read flat file */
#define NTWRFIL		17		/* write flat file */
#define NTCLFIL		18		/* close flat file */
#define NTDLFIL		19		/* delete flat file */
#define NTPASS		20		/* compute password (UniTree) */
#define NTDLIMG		21		/* delete image */
#define NTSTAGE		22		/* HSM stage file */
#define NTCHGRP		23		/* HSM change group on a file */
#define NTOPTIF		24		/* open remote TIFF image */
#define NTCLTIF		25		/* close remote TIFF image */
#define NTRDTIF		26		/* read remote TIFF image */
#define NTWRTIF		27		/* write remote TIFF image */
#define NTDLTIF		28		/* delete remote TIFF image */
#define NTAPREC		29		/* append record to remote file */
#define NTGETDIR	30		/* HSM get directory listing */
#define NTLIST		31		/* HSM list directory */
#define NTDELFIL	32		/* HSM delete file */
#define NTMKDIR		33		/* HSM make directory */

#define PWLEN		10		/* length of password including null */

struct NTSYS
	{
	char local[100];		/* local computer system name */
	char remote[100];		/* remote computer system name */
	char srcsys[20];		/* source file LAS 5 system name */
	char dstsys[20];		/* destination file LAS 5 system name */
	char cparm[CMLEN];		/* character parameter */
	unsigned char iparm[4];		/* integer parameter */
	unsigned char cmd[4];		/* command option */
	struct SNDACK ack;		/* acknowledge structure (ncp.h) */
	};

#define NMRIMG 3
#define MAX_DSIZE sizeof(float)		/* max # bytes per pixel */

#define SL	0
#define SS	1
#define NL	2
#define NS	3

#define MINMAX_FLG 0
#define HISTRY_FLG 1
#define SCALE_FLG  2
#define CNVBUF_FLG 3

struct NET_IMG
	{
	unsigned char access[4];	/* LAS image I/O access */
	unsigned char idtype[4];	/* input image data type */
	unsigned char nl[4];		/* # lines */
	unsigned char nline[4];		/* # lines per read or write */
	unsigned char nmrimg[4];	/* number of images */
	unsigned char ns[4];		/* # samples */
	unsigned char odtype[4];	/* output image data type */
	unsigned char outnl[4];		/* output image # lines */
	unsigned char outns[4];		/* output image # samples */
	unsigned char totbnd[4];	/* # nbands */
	char flags[4];			/* flags -- minmax, histry, scale */
	char in[NMRIMG][CMLEN];		/* input image host names */
	char inspec[CMLEN];		/* input image specification */
	char out[CMLEN];		/* output image host name */
	char outspec[CMLEN];		/* output image specification */
	struct SNDACK ack;		/* acknowledge structure (ncp.h) */
	};

struct NET_IMG2
	{
	unsigned char sl[4];			/* starting line */
	unsigned char ss[4];			/* starting sample */
	unsigned char nl[4];			/* # lines */
	unsigned char ns[4];			/* # samples */
	unsigned char nbands[4];		/* # bands */
	unsigned char bands[MAXBND+1][4];	/* band specifications */
	struct SNDACK ack;
	};


struct NET_SCALE
	{
	char outmax[100];		/* Output max value for scaling */
	char outmin[100];		/* Output min value for scaling */
	char scale[100];		/* Scale factor to be applied to image*/
	char offset[100];		/* Offset to be applied to image */
	struct SNDACK ack;
	};

struct IMG_INFO
	{
	double maxval[MAXBND];		/* Maximum pixel values */
	double minval[MAXBND];		/* Minimum pixel values */
	double outmax;			/* Output max value for scaling */
	double outmin;			/* Output min value for scaling */
	double scale[MAXBND];		/* Scale factor to be applied to image*/
	double offset[MAXBND];		/* Offset to be applied to image */
	long access;			/* Image access (IREAD or IWRITE) */
	long bands[NMRIMG][MAXBND+1];	/* Input image band specification */
	long fd[NMRIMG];		/* LAS image file descriptor */
	long idtype;			/* Input image data type */
	long insys;			/* Input system for pxsys call */
	long line_inc;			/* Line increment */
	long nbands[NMRIMG];		/* Input image band specification */
	long nline;			/* # lines to read or write per call */
	long nmrimg;			/* Number of input images */
	long odtype;			/* Output image data type */
	long outsys;			/* Output system for pxsys call */
	long samp_inc;			/* sample increment */
	long totbnd;			/* Total number of bands */
	long window[NMRIMG][4];		/* Input image window specification */
	long outwind[4];		/* Output image window size */
	unsigned char *buf;		/* network buffer */
	char *conv_buf;			/* conversion buffer */
	char flags[4];			/* flags -- minmax, histry, scale */
	char in[NMRIMG][CMLEN];		/* Array of input image host names */
	char inspec[CMLEN];		/* Input image specification */
	char out[CMLEN];		/* Output image host name */
	char outspec[CMLEN];		/* Output image specification */
	char tin[NMRIMG][CMLEN];	/* Array of input image TAE names */
	char tout[CMLEN];		/* Output image TAE name */
	};


struct LSINFO			/* Structure for Label Service files */
	{
	long fd;		/* file descriptor to opened LS file */
	long access;		/* file access (read, write, or update */
	long nbytes;		/* total number bytes in network buffer */
	long cnt;		/* number of network reads/writes */
	long insys;		/* input system for pxsys call */
	long outsys;		/* output system for pxsys call */
	long ddrflg;		/* ddr flag, if TRUE then change system name */
	long smflg;		/* too small buffer flag */
	long reccnt;		/* LS record count */
	char fname[CMLEN];	/* LS file name */
	unsigned char *curpos;	/* current position in network buffer */
	unsigned char *ntbuf;	/* LS read/write network buffer */
	};

struct FIL_INFO
	{
	long access;			/* file access (IREAD or IWRITE) */
	long fd;			/* file descriptor */
	long nbyte;			/* number of bytes allocated for buf */
	char *buf;			/* network buffer */
	char fname[CMLEN];		/* Output file host name */
	};
