/*
	Quick Look printer structure definitions
*/

/* #define DEBUG */

#ifdef sun
#ifdef sparc
#define sun4
#endif
#ifdef mc68010
#define sun2
#endif
#ifdef mc68000
#define sun2
#define sun3
#endif
#endif

#define NMRIMG  3                      /* max # images */

#define CFHOST     "edcsgs20"
#define CRF_FIL    "qlp.req"
#define CRFLCK_FIL "qlp.lock"
#define PID_FIL    "qlp.pid"

#define MAX_QLP_PRINTER 100	/* Max number of printers allowed */
#define MAX_QLP_PARAM    8	/* Max number of sub print parameters */

#define OPGRP "rpops" 

#define NULLVAL -1

#define ABORT_EXIT   1  	/* On interrupt exit program */
#define ABORT_IGNORE 2  	/* Ignore interrupt and continue exectution */

#define	TO_REC	1		/* copy lut from tmp to record structure */
#define	FR_REC	2		/* copy lut from record structure to tmp */

#define PADD	1		/* Program Add Request */
#define	PMOD	2		/* Program Modify Request */
#define	PREP	3		/* Program Report Request */
#define PLSP    4		/* Program list available printers */
#define PDEL	5		/* Program delete request */
#define PABORT  6		/* Program abort request */

#define NDAYS	1		/* Number of days before deleting requests */

#define QLP_QUEUE	"QUEUE"	/* Request status is in queue */
#define QLP_RUN		"RUN"	/* Request status is in process of printing */
#define QLP_SUCC	"SUCC"	/* Request has been printed */
#define QLP_ERROR	"ERROR"	/* Request encountered error while printing */

#define FONT_NL		58	/* Number of lines per font character */
#define FONT_NS		32	/* Number of samples per font character */
#define WEDSPOT_ZL	10	/* Number of fill lines between image and */
				/*   color wedge and/or spot copyright */
#define LUT_LENGTH	256	/* QLP LUT Length (max number of map points) */

#define QLP_BUSY	1	/* QLP currently busy */
#define QLP_IDLE	2	/* QLP currently idle */
#define QLP_STOP	3	/* QLP flag to stop processing */
#define QLP_ERR		4	/* QLP flag to indicate printer error */

#define QLP_SLEEP	15	/* Number of seconds to sleep */
#define MAXRETRY 	25	/* Number of retrys if bad printer status */

#define SU_NORM		1	/* no qlp printer privileges */
#define SU_PRIV		2	/* qlp printer privileges to change priorities*/
#define SU_OWN		3	/* qlp printer owner */

struct QLP_PRINTER
	{
	float width;		/* media width in inches */
	float height;		/* media height in inches */
	long dpi;		/* printer dots per inch */
	char desc[64];		/* printer description */
	char depend[MAX_QLP_PARAM][64];
				/* multiple printers defined for multiple */
				/*   media types and/or media sizes */
	char ftype[MAX_QLP_PARAM][64];	
				/* file types accepted */
	char lblpr[64];		/* label printer */
	char lp[64];		/* lp or lpr */
	char owner[MAX_QLP_PARAM][64];	
				/* owner(s) of printer */
	char priv[MAX_QLP_PARAM][64];	
				/* privleged users (able to set priorities) */
	char mtype[64];		/* media type available */
	char lname[64];		/* printer logical name */
	char pname[64];		/* printer physical name */
	char ps[64];		/* postscript binary or ascii */
	char status[64];	/* printer status, (up or dn) */
	char tray[64];          /* file name in $QDIR to set printer tray */
	char type[64];		/* printer type (Phaser480, Phaser300, HP755) */
	};


struct QLPPID
	{
	long qlpd_pid;		/* daemon process ID */
	long status;		/* queue status (busy, idle, etc) */
	char cmd[64];		/* printer config command */
	};

struct IMGOPT
	{
	long curbnd;		/* current band of output image */
	long curlin;		/* current line of output image */
	long annot;		/* TRUE is annotation line is specified */
	long autosz;		/* TRUE if printer automatically resizes image*/
	long spot;		/* TRUE if spot annot */
	long wedge;		/* TRUE if wedge annot */
	long stretch;		/* TRUE if stretch annot */
	long title;		/* TRUE if title annot */
	long rotate;		/* TRUE if image needs to be rotated */
	long rgb2cmy;		/* TRUE if image needs to be converted */
	long expand;		/* TRUE if image needs to be expanded */
	long reduce;		/* TRUE if image needs to be reduced */
	long fontnl;		/* number of lines for annot area */
	long maxnl;		/* max number image lines */
	long maxns;		/* max number image samples */
	long outnb;		/* output number of bands */
	long ss;		/* starting sample within printer image */
	long sl;		/* starting line within printer image */
	long nl;		/* number sample within printer image */
	long ns;		/* number line within printer image */
	long time_cpu;		/* start time for cpu */
	long time_elapse;	/* start time for elapse */
	unsigned char *lasbuf;	/* pointer to image buffer */
	};

#define NLUT  4			/* Number of LUT's */
#define MAXPT 256		/* Max number of points in LUT */

struct lut
   {
   int   in[MAXPT];
   float out[MAXPT];
   long  ne;
   char  desc[88];
   };

struct reclut
   {
   unsigned char in[MAXPT];
   unsigned char out[MAXPT];
   long          ne;
   char          desc[88];
   };

struct recfmt
	{
	long	delflg;		/* delete flag, TRUE => request is deleted */
	long	fptr;		/* points to next record with same priority */
	long	bptr;		/* points to last record with same priority */
	long	lad;		/* last access date -- julian day */
	long	seqnum;	      	/* sequence number to identify requests	*/
	long 	bands[NMRIMG][MAXBND+1]; /* bands specified */
	long	bgrnd;		/* background pixel value */
	long	curcop;		/* current copy being processed */
	long	curpan;		/* current panel to print */
	long	ewr;		/* ewr for accounting purposes */
	long	expand[2];	/* scale factor to expand image (line & samp) */
	long	nb;		/* number of bands */
	long	ncop;		/* number of copies to be produced */
	long	nimg;		/* number of images specified */
	long	nl;		/* number of lines */
	long	npanel;		/* number of panels to print */
	long	nruns;		/* # of times processed by color FIRE */
	long	ns;		/* number of samples */
	long	prio;		/* priority given to the request */
	long	reduce[2];	/* scale factor to reduce image (line & samp) */
	long	sl;		/* starting line of image */
	long	ss;		/* starting sample of image */
	char	aflg[8];	/* annotation option (yes or no) */
	char	annot[128];	/* user notes to be annotated onto film	*/
	char	autosz[8];	/* flag indicating auto resize by printer */
	char	chrgcod[16];	/* specifies the accountable project */
	char	color[8];   	/* color sequence (RGB, CMY, CMYK, or BW) */
	char    delfil[8];      /* delete flag (YES or NO) */
	char    dltf[16];	/* dltf entry name */
	char 	errtext[256];	/* error text msg */
	char 	fspec[128];	/* LAS Image file specification */
	char	ftype[16];	/* File type (LAS, PS, etc) */
	char	infile[128];	/* enhancement or DLT file name */
	char	printer[128];	/* specify printer (phaser480,colordsb,etc) */
	char	ptype[16];	/* printer type */
	char	tae[NMRIMG][128]; /* TAE image file names */
	char	remote[64];	/* Host name of remote system with image */
	char	rotopt[8];	/* Rotate option (YES, NO, or AUTO) */
	char 	spot[8];	/* SPOT copyright annot (YES, or NO) */
	char 	spinst[256];	/* Special instructions */
	char	stretch[4][64];	/* stretch points to be annotated onto film */
	char	stflg[8];	/* Status flag (QUEUE, RUN, SUCC, or ERROR) */
	char	stropt[8];	/* stretch option -- SAME, NONE, VALUE */
	char	submit_date[32];/* submit date for reporting purposes */
	char	title[104];    	/* title of image to be annotated on top line*/
	char	usercode[32];	/* user code -- used for request security */
	char 	wedge[8];	/* NMD color wedge (YES or NO) */
	struct reclut enhlut[NLUT];	/* enh map values */
        };


struct hdrfmt
	{
	int	fptr[10];	/* array of pointers pointing to the first   */
				/* request of each priority.  The zero       */
				/* element points to the first free record.  */
	int	bptr[10];	/* array of pointers pointing to the last    */
				/* request of each priority.		     */
   	};

#define	HDRLEN sizeof(struct hdrfmt)	/* len of header structure for CRF */
#define RECLEN sizeof(struct recfmt)	/* len of record structure for CRF */

