/******************************************************************************

SUBROUTINE:	abort

PURPOSE:
	Abort a currently running request.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   2.0           3/97       KGacke      qlp/qlpmgr unix command line option
   1.0          3/92        K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

Algorithm:

******************************************************************************/

				
#include <stdio.h>
#include "las.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"

static long fatal = -1;

int validpid();

int abreq
(
    int argc,                   /* argument count */
    char *argv[],               /* argument vector */
    long tae,                   /* TAE flag, TRUE=>valid param block */
    struct PARBLK *blkptr	/* TAE parameter block */
)
{
extern struct recfmt rec;
extern struct hdrfmt hdr;
struct VARIABLE *seqnum;
struct QLP_PRINTER printer[MAX_QLP_PRINTER];
int		fd,seq,nrec;
long		crflg;
long		pcnt;
long		rstat;
long 		suflg;
char		msg[256];
char 		user[64];

rstat = E_SUCC;
getuser(user);
if (tae)
    {
    seqnum = p_find(blkptr,"RECNUM");
    seq = IVAL(*seqnum,0);
    }
else
    {
    if (argc <= 1)
        {
        qlp_usage(PABORT);
        return(E_FAIL);
        }
    if (sscanf(argv[1],"%d",&seq) != 1)
        {
        sprintf(msg,"Error retrieving sequence number to abort, arg=%s",
                argv[1]);
        c_errmsg(msg," ",&fatal);
        }
    argc--;
    argv++;
    }
(void) crfinit("QLP-ABORT");
(void) crfpr(printer,&pcnt,TRUE);  /* retrieve printer info */
(void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec); /* open the CRF */
if (seq > nrec) 		   /* if sequence number > number of requests*/
   {				   /*   write error message & terminate proc */
   sprintf(msg,"Sequence Number %d does not exist",seq);
   rstat = E_FAIL;
   goto xit;
   }
(void) crfrd(fd,TRUE,seq,&rec);	    /* read the request to be modified */
if (strcmp(rec.stflg,QLP_RUN) != 0)
   {				   /*   write error message & terminate proc */
   sprintf(msg,"Sequence number %d is not currently being processed by printer",
	seq);
   rstat = E_FAIL;
   goto xit;
   }
if (rec.delflg != -1)	   	   /* if request is not valid then	     */
   {				   /*   write error message & terminate proc */
   sprintf(msg,"Sequence Number %d does not exist",seq);
   rstat = E_FAIL;
   goto xit;
   }
suflg = qlp_chkuser(printer,pcnt,&rec);
if (suflg != SU_OWN)
   {
   if (strcmp(rec.usercode,user) != 0)
      {
      sprintf(msg,"ERROR: User <%s> is not owner of custom request %d",
	   user,seq);
      rstat = E_FAIL;
      goto xit;
      }
   }
(void) crfabort(fd,rec.seqnum,FALSE);
xit:
(void) crfcl(fd,TRUE);		    /* close the CRF */
(void) crfdone();
if (rstat != E_SUCC)
   c_errmsg(msg," ",&fatal);
return(E_SUCC);
} 
