/******************************************************************************

SUBROUTINE: dynam

PURPOSE: 
	The purpose of dynam is to build a dynamic parameter block.  The 
	default values for the parameters will be assigned the values contained
	in the global variable of rec.  A dynamic tutor session is initiated, 
	and the new paramter values are received by calling the subroutine of 
	RPARM.  The new values are returned to the calling program through the
	global variable rec.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   2.0           3/97       KGacke      qlp/qlpmgr unix command line option
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:	Quick Look Printer

LANGUAGE	C
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"

int dynam
(
    int argc,                   /* argument count */
    char *argv[],               /* argument vector */
    long tae,                   /* TAE flag, TRUE=>valid param block */
    struct QLP_PRINTER printer[],
    long pcnt
)
{
extern struct recfmt rec;
extern struct hdrfmt hdr;
struct PARBLK	dblock;
int		intval[3];
int		rstat;
char		*strval[2];

if (tae)
    {
    q_init(&dblock,P_BYTES,P_ABORT);

    if (strlen(rec.fspec) == 0)
       q_string(&dblock,"IN",-2,strval,P_ADD);
    else				
       {				
       strval[0] = rec.fspec;
       q_string(&dblock,"IN",1,strval,P_ADD);
       }
    
    if (rec.ewr == NULLVAL)
       q_intg(&dblock,"EWRNBR",-2,intval,P_ADD);
    else
       {
       intval[0] = rec.ewr;
       q_intg(&dblock,"EWRNBR",1,intval,P_ADD);
       }
    
    if (strlen(rec.ftype) == 0)
       q_string(&dblock,"FTYPE",-2,strval,P_ADD);
    else				
       {				
       strval[0] = rec.ftype;
       q_string(&dblock,"FTYPE",1,strval,P_ADD);
       }
    
    if (strlen(rec.ptype) == 0)
       q_string(&dblock,"PTYPE",-2,strval,P_ADD);
    else				
       {				
       strval[0] = rec.ptype;
       q_string(&dblock,"PTYPE",1,strval,P_ADD);
       }
    
    if (strlen(rec.printer) == 0)
       q_string(&dblock,"PRINTER",-2,strval,P_ADD);
    else				
       {				
       strval[0] = rec.printer;
       q_string(&dblock,"PRINTER",1,strval,P_ADD);
       }
    
    if (rec.ncop == NULLVAL)
       q_intg(&dblock,"NCOPIES",-2,intval,P_ADD);
    else
       {
       intval[0] = rec.ncop;
       q_intg(&dblock,"NCOPIES",1,intval,P_ADD);
       }
    
    if (rec.prio == NULLVAL)
       q_intg(&dblock,"PRIORITY",-2,intval,P_ADD);
    else
       {
       intval[0] = rec.prio;
       q_intg(&dblock,"PRIORITY",1,intval,P_ADD);
       }
    
    if (strlen(rec.delfil) == 0)
       q_string(&dblock,"DELFLG",-2,strval,P_ADD);
    else
       {
       strval[0] = rec.delfil;
       q_string(&dblock,"DELFLG",1,strval,P_ADD);
       }
    
    if ((strcmp(rec.ftype,"las") != 0) && (strcmp(rec.ftype,"tif") != 0))
        {
        q_dynp(&dblock,"QLPDY1.PDF",M_FULLPDF);/* send V-blk for dynamic parm */
        p_inim(&dblock,P_BYTES,P_ABORT);       /* receive V-block from TM */
        }
    else
        {
        if (rec.bgrnd == NULLVAL)
           q_intg(&dblock,"BACKGRND",-2,intval,P_ADD);
        else
           {
           intval[0] = rec.bgrnd;
           q_intg(&dblock,"BACKGRND",1,intval,P_ADD);
           }
        
        if (rec.expand[0] == NULLVAL)
           q_intg(&dblock,"EXPAND",-2,intval,P_ADD);
        else
           {
           intval[0] = rec.expand[0];
           intval[1] = rec.expand[1];
           q_intg(&dblock,"EXPAND",2,intval,P_ADD);
           }
        
        if (rec.reduce[0] == NULLVAL)
           q_intg(&dblock,"REDUCE",-2,intval,P_ADD);
        else
           {
           intval[0] = rec.reduce[0];
           intval[1] = rec.reduce[1];
           q_intg(&dblock,"REDUCE",2,intval,P_ADD);
           }
        
        if ((rec.curpan == NULLVAL) || (rec.curpan >= rec.npanel))
           q_intg(&dblock,"SPANEL",-2,intval,P_ADD);
        else
           {
           intval[0] = rec.curpan;
           q_intg(&dblock,"SPANEL",1,intval,P_ADD);
           }
        
        if (strlen(rec.annot) == 0)
           q_string(&dblock,"ANNOT",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.annot;
           q_string(&dblock,"ANNOT",1,strval,P_ADD);
           }
        
        if (strlen(rec.autosz) == 0)
           q_string(&dblock,"AUTOSZ",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.autosz;
           q_string(&dblock,"AUTOSZ",1,strval,P_ADD);
           }
        
        if (strlen(rec.color) == 0)
           q_string(&dblock,"COLORORD",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.color;
           q_string(&dblock,"COLORORD",1,strval,P_ADD);
           }
        
        if (strlen(rec.dltf) == 0)
           q_string(&dblock,"INDLTF",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.dltf;
           q_string(&dblock,"INDLTF",1,strval,P_ADD);
           }
        
        if (strlen(rec.infile) == 0)
           q_string(&dblock,"INFILE",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.infile;
           q_string(&dblock,"INFILE",1,strval,P_ADD);
           }
        
        if (strlen(rec.rotopt) == 0)
           q_string(&dblock,"ROTOPT",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.rotopt;
           q_string(&dblock,"ROTOPT",1,strval,P_ADD);
           }
        
        if (strlen(rec.spot) == 0)
           q_string(&dblock,"SPOTCOPY",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.spot;
           q_string(&dblock,"SPOTCOPY",1,strval,P_ADD);
           }
        
        if (strlen(rec.stropt) == 0)
           q_string(&dblock,"STRETCH",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.stropt;
           q_string(&dblock,"STRETCH",1,strval,P_ADD);
           }
        
        if (strlen(rec.aflg) == 0)
           q_string(&dblock,"AFLG",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.aflg;
           q_string(&dblock,"AFLG",1,strval,P_ADD);
           }
        
        if (strlen(rec.title) == 0)
           q_string(&dblock,"TITLE",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.title;
           q_string(&dblock,"TITLE",1,strval,P_ADD);
           }
        
        if (strlen(rec.wedge) == 0)
           q_string(&dblock,"WEDGE",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.wedge;
           q_string(&dblock,"WEDGE",1,strval,P_ADD);
           }
        
        if (strlen(rec.spinst) == 0)
           q_string(&dblock,"SPECIAL",-2,strval,P_ADD);
        else				
           {				
           strval[0] = rec.spinst;
           q_string(&dblock,"SPECIAL",1,strval,P_ADD);
           }
        q_dynp(&dblock,"QLPDY.PDF",M_FULLPDF); /* send V-blk for dynamic parm */
        p_inim(&dblock,P_BYTES,P_ABORT);       /* receive V-block from TM */
        }
    }
        
/*
* Retrieve parameter values
*/
rstat = rparm(argc,argv,tae,&dblock,printer,pcnt,&rec,PMOD);
return(rstat);
}	/*  END of DYNAM subroutine  */
