/******************************************************************************

	PRCRF prints a request from the custom request to the print file.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"

static long fatal = -1;
static long nonfatal = 1;
static long first = TRUE;

int prcrf_las
(
   FILE   *fp,              /* (I) file descriptor to write data to */
   struct recfmt *rec,      /* (I) request structure to be written */
   long   lutflg            /* (I) if TRUE print enh map pts */
);

int prcrf_gen
(
   FILE   *fp,              /* (I) file descriptor to write data to */
   struct recfmt *rec       /* (I) request structure to be written */
);

int prcrf_short
(
   FILE   *fp,              /* (I) file descriptor to write data to */
   struct recfmt *rec       /* (I) request structure to be written */
);

int pr_enhlut
(
    FILE *fp,
    struct recfmt *rec
);



int print_crf
(
    struct QLP_PRINTER printer[MAX_QLP_PRINTER],
    long pcnt,
    char *prtfile,
    int  seqnum[],
    char *stflag[],
    char *ls_printer,
    char *outform
)
{
struct hdrfmt hdr;
struct recfmt rec;
FILE *fp;
long cnt;
long cnt1;
long crflg;
long fd;
long nrec;
long priority;
long printer_flg;
char msg[256];
char printer_type[256];

if ((fp = fopen(prtfile,"w")) == NULL)
   c_errmsg("ERROR creating print file","fire",&fatal);
printer_flg = TRUE;
if (ls_printer != NULL)
    {
    for (cnt=0; cnt<MAX_QLP_PRINTER; cnt++)
	{
	if (strcmp(ls_printer,printer[cnt].lname) == 0)
	    {
	    strcpy(printer_type,printer[cnt].type);
	    break;
	    }
	}
    }
(void) crfop(0,TRUE,ABORT_EXIT,&fd,&crflg,&nrec);
(void) crfrdhdr(fd,TRUE,&hdr);
if (seqnum[0] != 0)
    {
    for (cnt=0; seqnum[cnt]!=0; cnt++)
	{
        (void) crfrd(fd,TRUE,seqnum[cnt],&rec);
        if (rec.delflg != -1)
            {
            sprintf(msg,"Record Number %d does not exist",seqnum[cnt]);
            c_errmsg(msg," ",&nonfatal);
            sleep(5);
            continue;
            }
	for (cnt1=0; stflag[cnt1]!=NULL; cnt1++)
	    {
	    if (strcmp(rec.stflg,stflag[cnt1]) == 0)
		break;
	    }
	if (ls_printer != NULL)
	    {
	    printer_flg = FALSE;
	    if (strcmp(ls_printer,rec.printer) == 0)
		printer_flg = TRUE;
	    if ((strlen(rec.printer)==0) && (strcmp(printer_type,rec.ptype)==0))
		printer_flg = TRUE;
	    }
	if ((stflag[cnt1] != NULL) && (printer_flg))
	    {
	    if ((reqown(printer,pcnt,&rec)) && (strcmp(outform,"LONG") == 0))
		{
		if ((strcmp(rec.ftype,"las") == 0) || (strcmp(rec.ftype,"tif") == 0))
                    prcrf_las(fp,&rec,TRUE);
		else
                    prcrf_gen(fp,&rec);
		}
	    if (strcmp(outform,"SHORT") == 0)
                prcrf_short(fp,&rec);
	    }
	}
    }
else
    {
    for (priority=1,cnt=0; priority<=9; priority++)
        {
        seqnum[0] = hdr.fptr[priority];
        while (seqnum[0] != 0)
            {
            (void) crfrd(fd,TRUE,seqnum[0],&rec);
	    for (cnt1=0; stflag[cnt1]!=NULL; cnt1++)
	        {
	        if (strcmp(rec.stflg,stflag[cnt1]) == 0)
		    break;
	        }
	    if (ls_printer != NULL)
	        {
	        printer_flg = FALSE;
	        if (strcmp(ls_printer,rec.printer) == 0)
		    printer_flg = TRUE;
	        if ((strlen(rec.printer)==0)&&(strcmp(printer_type,rec.ptype)==0))
		    printer_flg = TRUE;
	        }
	    if ((stflag[cnt1] != NULL) && (printer_flg))
		{
	        if ((reqown(printer,pcnt,&rec))&&(strcmp(outform,"LONG") == 0))
		    {
		    if (strcmp(rec.ftype,"las") == 0)
                        prcrf_las(fp,&rec,TRUE);
		    else
                        prcrf_gen(fp,&rec);
		    }
	        if (strcmp(outform,"SHORT") == 0)
                    prcrf_short(fp,&rec);
		}
            seqnum[0] = rec.fptr;
            }
        }
    }
fclose(fp);
(void) crfcl(fd,TRUE);
return(1);
}

int prcrf_gen
(
   FILE   *fp,              /* (I) file descriptor to write data to */
   struct recfmt *rec       /* (I) request structure to be written */
)
{
if (first)
    first = FALSE;
else
    fprintf(fp,"\f");
fprintf(fp,"Record Number    : %-5d            Last Acess Date  : %d\n",
	rec->seqnum,rec->lad);
fprintf(fp,"Printer          : %-16s Printer Type     : %s\n",
	rec->printer,rec->ptype);
fprintf(fp,"Charge Code      : %-16s EWR              : %-06d\n",
	rec->chrgcod,rec->ewr);
fprintf(fp,"User Code        : %-16s Submit Date      : %s\n",
	rec->usercode,rec->submit_date);
fprintf(fp,"System           : %-16s Number of Copies : %-5d\n",
	rec->remote,rec->ncop);
fprintf(fp,"File Type        : %-8s\n",rec->ftype);
fprintf(fp,"File Name        : %-60.60s\n",rec->fspec);
fprintf(fp,"Priority         : %-5d\n",rec->prio);
fprintf(fp,"Delete File Flag : %-5s\n",rec->delfil);
fprintf(fp,"Number of Runs   : %-5d\n",rec->nruns);
fprintf(fp,"Error Text       : %s\n",rec->errtext);
fprintf(fp,"Status Flag      : %s\n",rec->stflg);
return(1);
}

int prcrf_las
(
   FILE   *fp,              /* (I) file descriptor to write data to */
   struct recfmt *rec,      /* (I) request structure to be written */
   long   lutflg            /* (I) if TRUE print enh map pts */
)
{
long cnt;

if (first)
    first = FALSE;
else
    fprintf(fp,"\f");
fprintf(fp,"Record Number    : %-5d            Last Acess Date  : %d\n",
	rec->seqnum,rec->lad);
fprintf(fp,"Printer          : %-16s Auto Resize      : %-8s\n",
	rec->printer,rec->autosz);
fprintf(fp,"Printer Type     : %-16s File Type        : %-8s\n",
        rec->ptype,rec->ftype);
fprintf(fp,"Charge Code      : %-16s EWR              : %-06d\n",
	rec->chrgcod,rec->ewr);
fprintf(fp,"User Code        : %-16s Submit Date      : %s\n",
	rec->usercode,rec->submit_date);
fprintf(fp,"Color Order      : %-5s            Rotate Option    : %-5s\n",
	rec->color,rec->rotopt);
fprintf(fp,"Number of Copies : %-5d            Back Ground      : %-5d\n",
        rec->ncop,rec->bgrnd);
fprintf(fp,"Current Panel    : %-5d            Number of Panels : %-5d\n",
	rec->curpan,rec->npanel);
fprintf(fp,"Number of Lines  : %-5d            Number of Samples: %-5d\n",
	rec->nl,rec->ns);
fprintf(fp,"System           : %-16s Number of Bands  : %-5d\n",
	rec->remote,rec->nb);
fprintf(fp,"Image Name       : %-60.60s\n",rec->fspec);
if (rec->expand[0] == NULLVAL)
    fprintf(fp,"Expand           : --\n");
else
    fprintf(fp,"Expand           : %d, %d\n",rec->expand[0],rec->expand[1]);
if (rec->reduce[0] == NULLVAL)
    fprintf(fp,"Reduce           : --\n");
else
    fprintf(fp,"Reduce           : %d, %d\n",rec->reduce[0],rec->reduce[1]);
fprintf(fp,"Title            : %s\n",rec->title);
fprintf(fp,"Annotation       : %s\n",rec->annot);
fprintf(fp,"Wedge Option     : %s\n",rec->wedge);
fprintf(fp,"Spot Copyright   : %s\n",rec->spot);
fprintf(fp,"Stretch Option   : %s\n",rec->stropt);
fprintf(fp,"ENH/DLTF File    : %s\n",rec->infile);
fprintf(fp,"DLTF Entry       : %s\n",rec->dltf);
if (strcmp(rec->stropt,"YES") == 0)
    {
    for (cnt=0; cnt<rec->nb; cnt++)
        fprintf(fp,"Band %d Annot : %s\n",cnt+1,rec->stretch[cnt]);
    }
if (lutflg)
    {
    pr_enhlut(fp,rec);
    }
fprintf(fp,"Priority         : %-5d\n",rec->prio);
fprintf(fp,"Special Instruct : %s\n",rec->spinst);
fprintf(fp,"Delete File Flag : %-5s\n",rec->delfil);
fprintf(fp,"Number of Runs   : %-5d\n",rec->nruns);
fprintf(fp,"Error Text       : %s\n",rec->errtext);
fprintf(fp,"Status Flag      : %s\n",rec->stflg);
return(1);
}

int prcrf_short
(
   FILE   *fp,              /* (I) file descriptor to write data to */
   struct recfmt *rec       /* (I) request structure to be written */
)
{
static long First = TRUE;

if (First)
    {
    fprintf(fp,"%s%s\n","                               ",
	"                            Total Submit");
    fprintf(fp,"%s%s\n","Rec Pri User     System   ",
	"Status Printer        PType      Panel Date");
    First = FALSE;
    }
fprintf(fp,"%3d %3d %-8.8s %-8.8s %-6.6s %-14.14s %-10.10s %5d %s\n",
	rec->seqnum,rec->prio,rec->usercode,rec->remote,rec->stflg,
	rec->printer,rec->ptype,rec->npanel*rec->ncop,rec->submit_date);
return(1);
}

int pr_enhlut
(
    FILE *fp,
    struct recfmt *rec
)
{
long cnt;
long pt;
long start_pt;
char stretch[256];
char temp[10];
char color[NLUT][32];

if (strcmp(rec->color,"RGB") == 0)
    {
    strcpy(color[0],"Red");
    strcpy(color[1],"Green");
    strcpy(color[2],"Blue");
    }
else if (strcmp(rec->color,"CMY") == 0)
    {
    strcpy(color[0],"Cyan");
    strcpy(color[1],"Magenta");
    strcpy(color[2],"Yellow");
    }
else if (strcmp(rec->color,"CMYK") == 0)
    {
    strcpy(color[0],"Cyan");
    strcpy(color[1],"Magenta");
    strcpy(color[2],"Yellow");
    strcpy(color[3],"Black");
    }
else if (strcmp(rec->color,"BW") == 0)
    {
    strcpy(color[0],"Black/White");
    }
for (cnt=0; cnt<rec->nb; cnt++)
    {
    fprintf(fp,"%s Enh Map Points:\n",color[cnt]);
    strcpy(stretch,"  ");
    for (start_pt=0; start_pt<rec->enhlut[cnt].ne; )
	{
	if (start_pt != 0)
	    fprintf(fp,"\n");
        for (pt=start_pt; pt<rec->enhlut[cnt].ne; pt++)
            {
            sprintf(temp,"%3d ",rec->enhlut[cnt].in[pt]);
	    if (strlen(stretch) + strlen(temp) > 80)
	        {
    	        fprintf(fp,"%s\n",stretch);
    	        strcpy(stretch,"  ");
		break;
	        }
	    strcat(stretch,temp);
            }
	if (pt == rec->enhlut[cnt].ne)
            fprintf(fp,"%s\n",stretch);
        strcpy(stretch,"  ");
        for (pt=start_pt; pt<rec->enhlut[cnt].ne; pt++)
            {
            sprintf(temp,"%3d ",(long) rec->enhlut[cnt].out[pt]);
	    if (strlen(stretch) + strlen(temp) > 80)
	        {
    	        fprintf(fp,"%s\n",stretch);
    	        strcpy(stretch,"  ");
		break;
	        }
	    strcat(stretch,temp);
            }
	if (pt == rec->enhlut[cnt].ne)
            fprintf(fp,"%s\n",stretch);
	start_pt = pt;
        }
    }
return(1);
}

int print_listpr
(
    struct QLP_PRINTER printer[MAX_QLP_PRINTER],
    long pcnt,
    char *prtfile,
    char *ls_ptype,
    char *ls_printer
)
{
FILE *fp;
long found;
long cnt;
long cnt1;

if ((fp = fopen(prtfile,"w")) == NULL)
   c_errmsg("ERROR creating print file","qlp",&fatal);
for (cnt=0; cnt<pcnt; cnt++)
    {
    found = FALSE;
    if (ls_ptype != NULL) 
	{
	if (strcmp(ls_ptype,printer[cnt].type) == 0)
	    found = TRUE;
	}
    else if (ls_printer != NULL)
	{
	if (strcmp(ls_printer,printer[cnt].lname) == 0)
	    found = TRUE;
	}
    else
	found = TRUE;
    if (found)
	{
	if (cnt > 0)
	    fprintf(fp,"\n\n\n");
	fprintf(fp,"%s\n",printer[cnt].lname);
	fprintf(fp,"  pname:  %s\n",printer[cnt].pname);
	fprintf(fp,"  desc:   %s\n",printer[cnt].desc);
	fprintf(fp,"  type:   %s\n",printer[cnt].type);
	fprintf(fp,"  media:  %s\n",printer[cnt].mtype);
	fprintf(fp,"  dpi:    %d\n",printer[cnt].dpi);
	fprintf(fp,"  width:  %5.2f\n",printer[cnt].width);
	fprintf(fp,"  height: %5.2f\n",printer[cnt].height);
	fprintf(fp,"  maxnl:  %d\n",
                (int)((float)printer[cnt].dpi*printer[cnt].height));
	fprintf(fp,"  maxns:  %d\n",
                (int)((float)printer[cnt].dpi*printer[cnt].width));
	fprintf(fp,"  ftype:  ");
	for (cnt1=0; cnt1<8; cnt1++)
	    {
	    if (strlen(printer[cnt].ftype[cnt1]) == 0)
		break;
	    fprintf(fp,"%-10.10s",printer[cnt].ftype[cnt1]);
	    }
	fprintf(fp,"\n");
	fprintf(fp,"  owner:  ");
	for (cnt1=0; cnt1<8; cnt1++)
	    {
	    if (strlen(printer[cnt].owner[cnt1]) == 0)
		break;
	    fprintf(fp,"%-10.10s",printer[cnt].owner[cnt1]);
	    }
	fprintf(fp,"\n");
	fprintf(fp,"  priv:   ");
	for (cnt1=0; cnt1<8; cnt1++)
	    {
	    if (strlen(printer[cnt].priv[cnt1]) == 0)
		break;
	    fprintf(fp,"%-10.10s",printer[cnt].priv[cnt1]);
	    }
	fprintf(fp,"\n");
	fprintf(fp,"  depend: ");
	for (cnt1=0; cnt1<8; cnt1++)
	    {
	    if (strlen(printer[cnt].depend[cnt1]) == 0)
		break;
	    fprintf(fp,"%-16.16s",printer[cnt].depend[cnt1]);
	    }
	fprintf(fp,"\n");
	fprintf(fp,"  lblpr : %s\n",printer[cnt].lblpr);
	fprintf(fp,"  status: %s\n",printer[cnt].status);
	}
    }
fclose(fp);
return(1);
}

int reqown
(
    struct QLP_PRINTER printer[MAX_QLP_PRINTER],
    long pcnt,
    struct recfmt *rec
)
{
long len;
long rstat;
long suflg;
char user[64];

rstat = FALSE;
suflg = qlp_chkuser(printer,pcnt,rec);
getuser(user);
len = (long)strlen(user);
c_low2up(user,&len);
len = (long)strlen(rec->usercode);
c_low2up(rec->usercode,&len);
if ((suflg > SU_NORM) || (strcmp(rec->usercode,user) == 0))
    rstat = TRUE;
return(rstat);
}
