/******************************************************************************

PROGRAM:  QLP

PURPOSE: 
	QLP is a routine which allows the user to add, modify, or delete
	requests in the Custom Request File.  The Custom Request File is a file
	containing requests for the Quick Look Printer(s).

	The structure of the file is a header record of 80 bytes followed by
	records containing the requests.  A record is defined by the data 
	structure of recfmt.  

	QLP acts as the driver.  It determines what operation is to be
	performed, and then calls the appropiate subroutine to do the file
	operation.

COMMON BLOCK REFERENCES:  (Located in the include file of "struct.cin")
  Name		Type    Fields   Description
  ----		----	------   -----------
  hdrlen	int		 length of the header record in bytes
  reclen	int		 length of one request record in bytes
  hdr		struct		 definition of the header record
  ftable	struct		 definition of the format table
			
PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   2.1           3/97       KGacke      qlp/qlpmgr unix command line option
   2.0		 9/96	    K. Gacke	allow multiple printers and file types
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

Algorithm:

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "las.h"
#include "util.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"

struct recfmt rec;
struct hdrfmt hdr;

int main
(
    int argc,
    char *argv[]
)
{
struct PARBLK	vblock;
struct VARIABLE *sub,*snum,*var;
struct QLP_PRINTER printer[MAX_QLP_PRINTER];
long	fatal = -1;
long	nonfatal = 1;
long	crflg;
long	fd;
long	file;
long	len;
long    loc_cnt;
long	lp;
long	nrec;
long    pcnt;
long	tae;
long	term;
int 	seqnum[50];
int	i;
long	zero = 0;
char	cmd[256];
char	emsg[256];
char	fname[256];
char	*ls_printer;
char	*ls_ptype;
char	subcmd[64];
char	prog[64];
char	outform[64];
char	outmsg[256];
char	prtfile[256];
char	*stflag[10];
char    pdf_name[256];
char    ps3_name[256];

tae = p_inim(&vblock,sizeof(struct PARBLK),P_CONT) == SUCCESS ? TRUE : FALSE;
/*
*  Retrieve the subcommand to be executed
*/
if (tae)
    {
    sub = p_find(&vblock,"_SUBCMD");  
    strcpy(subcmd,SVAL(*sub,0));  
    }
else
    {
    if (argc <= 1)
	{
	c_errmsg("Error reading input parameters"," ",&fatal);
	}
    strcpy(subcmd,argv[1]);
    len = (long)strlen(subcmd);
    c_low2up(subcmd,&len);
    --argc;
    ++argv;
    }
sprintf(prog,"QLP-%s",subcmd);
if ((strcmp(subcmd,"ADD") == 0) || (strcmp(subcmd,"GENADD") == 0))
    {
    *rec.usercode = '\0';
    
    if (rparm(argc,argv,tae,&vblock,printer,pcnt,&rec,PADD) != E_SUCC)
	c_errmsg("Error reading input parameters"," ",&fatal);
    
    /*addreq(fd,crflg,nrec); */
    /*printf( "Stub routine in qlp add/genadd section...\n");
    printf( "IN=%s PRINTER=%s\n", rec.fspec, rec.printer );
    */
    sprintf( pdf_name, "%s_tmp.pdf", rec.fspec );
    sprintf( cmd, "ps2pdf %s %s", rec.fspec, pdf_name );
    if ( system( cmd ) != 0 )
        {
        sprintf(emsg,"PS to PDF command failed: %s", cmd);
        c_errmsg(emsg," ",&fatal);
	}
    
    sprintf( ps3_name, "%s_tmpps3.ps", rec.fspec );
    sprintf( cmd, "pdf2ps %s %s", pdf_name, ps3_name );
    if ( system( cmd ) != 0 )
        {
        sprintf(emsg,"PDF to PS3 command failed: %s", cmd);
        c_errmsg(emsg," ",&fatal);
	}
    
    remove( pdf_name ); 
    
    sprintf( cmd, "lp -d%s %s", rec.printer, ps3_name );
    /* printf( "cmd= %s\n", cmd ); */
    
    if ( system( cmd ) != 0 )
        {
        sprintf(emsg,"Print command failed: %s", cmd);
        c_errmsg(emsg," ",&fatal);
	}
        
    remove( ps3_name ); 
    
    } 


    

else
    {
    sprintf(emsg,"<%s> is an invalid sub-command",subcmd);
    c_errmsg(emsg," ",&fatal);
    }
c_complt(prog,"",&zero);
return 0;
}	/*  END of QLP program  */
