/******************************************************************************

SUBROUTINE:	rparm

PURPOSE: 
	Subroutine rparm will receive the parameter values from the V-block,
	and return the values in a record format in the global variable "rec".

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   2.1		 3/99	    K. Gacke	allow all users to set priority 1-9
   2.0           3/97       KGacke      qlp/qlpmgr unix command line option
   1.1		 7/93	    K. Gacke	Add submit date field for reports
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

LANGUAGE	C

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "las.h"	
#include "util.h"
#include "tiffio.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"

static int chkimg
(
    long tae,
    struct PARBLK *blkptr,
    struct recfmt *rec
);

int qlp_unix_add
(
    int argc,               /* argument count */
    char *argv[],           /* argument vector */
    struct recfmt *rec,     /* QLP record block */
    long prog               /* calling program: PADD, PMOD, or REPORT   */
);



int rparm
(
    int argc,           /* argument count */
    char *argv[],       /* argument vector */
    long tae,           /* TAE flag, TRUE=>valid param block */
    struct PARBLK *blkptr,  /* TAE param block */
    struct QLP_PRINTER printer[],/* printer definition block */
    long pcnt,              /* number of printers defined */
    struct recfmt *rec,     /* QLP record block */
    long prog               /* calling program: PADD, PMOD, or REPORT   */
)
{
struct VARIABLE *val;
struct IMGOPT imgopt;
struct lut tlut[NLUT];
struct tm *tptr;
time_t  t;
long    suflg;
long    rwflg;
long	cnt;
long	len;
long	i, j;
long	elem;
long	fatal;
long	nonfatal;
long 	enhflg;	
long	maxchar;
long	printer_cnt;
long 	priority;
int 	rstat;
long	status;
int 	ival;
char 	hname[CMLEN];
char 	host[CMLEN];
char	msg[256],temp[10];
char    name[CMLEN];
char	*ptr;
char	wd[256];

rstat = E_SUCC;
nonfatal = 1;
fatal = -1;
if ((prog == PADD) || (prog == PMOD))
    {
    t = time(NULL);
    tptr = localtime(&t);
    sprintf(rec->submit_date,"%02d/%02d/%02d %02d:%02d",
        tptr->tm_mon+1,tptr->tm_mday,(1900 + tptr->tm_year) % 100,
		tptr->tm_hour,tptr->tm_min);
    /* getchrg(rec->chrgcod);       set charge code */
    }
if (prog == PADD)
    {
    /* getuser(rec->usercode);  	 set user code */
    rec->nruns = 0;	  	/* set number of runs equal to zero */
    rec->curcop = 0;
    rec->curpan = 0;
    rec->npanel = 1;
    /* getmachine(rec->remote); XXX */
    strcpy(host,rec->remote);
    strcpy(rec->stflg,QLP_QUEUE);
    rec->errtext[0] = '\0';
    }
if (prog == PMOD)
    {
    strcpy(name,rec->fspec);
    /* getmachine(host); XXX */
    strcpy(rec->stflg,QLP_QUEUE);
    }
/*
*  If not running from tae, retrieve the unix command line arguments
*/
if (!tae)
    {
    printf( "Error: not in TAE.\n" );
    exit(1);
    /*
    rstat = qlp_unix_add(argc,argv,rec,prog);
    if (rstat != E_SUCC)
	goto xit;
    */
    }

/* 
*  If valid tae parameter block, call the function of p_find for each 
*  parameter value that is to be read from the V-block.  The function will 
*  return a pointer of type VARIABLE which points to the value of the 
*  parameter. 
*/
if (tae)
    {
    val = p_find(blkptr,"IN");
    if (val->v_count == 0)
        *rec->fspec = '\0';
    else
        strcpy(rec->fspec,SVAL(*val,0));
    
    val = p_find(blkptr,"EWRNBR");	
    if (val->v_count == 0)
        rec->ewr = NULLVAL;
    else
        rec->ewr = IVAL(*val,0);
    
    val = p_find(blkptr,"FTYPE");
    if (val->v_count == 0)
        *rec->ftype = '\0';
    else
        strcpy(rec->ftype,SVAL(*val,0));
    len = (long)strlen(rec->ftype);
    c_up2low(rec->ftype,&len);

    val = p_find(blkptr,"PTYPE");
    if (val->v_count == 0)
        *rec->ptype = '\0';
    else
        strcpy(rec->ptype,SVAL(*val,0));
    len = (long)strlen(rec->ptype);
    c_up2low(rec->ptype,&len);
    
    val = p_find(blkptr,"PRINTER");
    if (val->v_count == 0)
        *rec->printer = '\0';
    else
        strcpy(rec->printer,SVAL(*val,0));
    len = (long)strlen(rec->printer);
    c_up2low(rec->printer,&len);
        
    val = p_find(blkptr,"NCOPIES");	
    if (val->v_count == 0)
        rec->ncop = NULLVAL;
    else
        rec->ncop = IVAL(*val,0);
        
    priority = rec->prio;
    val = p_find(blkptr,"PRIORITY");	
    if (val->v_count == 0)
        rec->prio = NULLVAL;
    else
        rec->prio = IVAL(*val,0);
        
    val = p_find(blkptr,"DELFLG");
    if (val->v_count == 0)
        *rec->delfil = '\0';
    else
        strcpy(rec->delfil,SVAL(*val,0));
    if ((strcmp(rec->ftype,"las")==0) || (strcmp(rec->ftype,"tif")==0))
        {
        val = p_find(blkptr,"BACKGRND");	
        if (val->v_count == 0)
            rec->bgrnd = NULLVAL;
        else
            rec->bgrnd = IVAL(*val,0);
        
        val = p_find(blkptr,"EXPAND");	
        if (val->v_count == 0)
            rec->expand[0] = rec->expand[1] = NULLVAL;
        else if (val->v_count == 1)
            rec->expand[0] = rec->expand[1] = IVAL(*val,0);
        else
            {
            rec->expand[0] = IVAL(*val,0);
            rec->expand[1] = IVAL(*val,1);
            }
        
        val = p_find(blkptr,"REDUCE");	
        if (val->v_count == 0)
            rec->reduce[0] = rec->reduce[1] = NULLVAL;
        else if (val->v_count == 1)
            rec->reduce[0] = rec->reduce[1] = IVAL(*val,0);
        else
            {
            rec->reduce[0] = IVAL(*val,0);
            rec->reduce[1] = IVAL(*val,1);
            }
        
        val = p_find(blkptr,"SPANEL");	
        if (val->v_count == 0)
            rec->curpan = 1;
        else
            rec->curpan = IVAL(*val,0);
        
        val = p_find(blkptr,"ANNOT");
        if (val->v_count == 0)
            *rec->annot = '\0';
        else
            strcpy(rec->annot,SVAL(*val,0));
        
        val = p_find(blkptr,"AUTOSZ");
        if (val->v_count == 0)
            *rec->autosz = '\0';
        else
            strcpy(rec->autosz,SVAL(*val,0));
        
        val = p_find(blkptr,"COLORORD");
        if (val->v_count == 0)
            *rec->color = '\0';
        else
            strcpy(rec->color,SVAL(*val,0));
        
        val = p_find(blkptr,"INDLTF");
        if (val->v_count == 0)
            *rec->dltf = '\0';
        else
            strcpy(rec->dltf,SVAL(*val,0));
        
        val = p_find(blkptr,"INFILE");
        if (val->v_count == 0)
            *rec->infile = '\0';
        else
            strcpy(rec->infile,SVAL(*val,0));
        
        val = p_find(blkptr,"ROTOPT");
        if (val->v_count == 0)
            *rec->rotopt = '\0';
        else
            strcpy(rec->rotopt,SVAL(*val,0));
        
        val = p_find(blkptr,"SPOTCOPY");
        if (val->v_count == 0)
            *rec->spot = '\0';
        else
            strcpy(rec->spot,SVAL(*val,0));
        
        val = p_find(blkptr,"AFLG");
        if (val->v_count == 0)
            *rec->aflg = '\0';
        else
            strcpy(rec->aflg,SVAL(*val,0));
        
        val = p_find(blkptr,"TITLE");
        if (val->v_count == 0)
            *rec->title = '\0';
        else
            strcpy(rec->title,SVAL(*val,0));
        
        val = p_find(blkptr,"WEDGE");
        if (val->v_count == 0)
            *rec->wedge = '\0';
        else
            strcpy(rec->wedge,SVAL(*val,0));
        
        val = p_find(blkptr,"SPECIAL");
        if (val->v_count == 0)
            *rec->spinst = '\0';
        else
            strcpy(rec->spinst,SVAL(*val,0));
        val = p_find(blkptr,"STRETCH");
        if (val->v_count == 0)
            *rec->stropt = '\0';
        else
            strcpy(rec->stropt,SVAL(*val,0));
        }
    }


return(rstat);
}	/*  END of the RPARM subroutine  */

