/******************************************************************************

PROGRAM:  QLPMGR

PURPOSE: 
	QLPMGRS is the QLP manager program to allow the operator to
	do the following functions:
			
PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   2.1           3/97       KGacke      qlp/qlpmgr unix command line option
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

Algorithm:

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/signal.h>
#include <time.h>

#include "las.h"
#include "util.h"
#include "pixman.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"

static long fatal = -1;
static long nonfatal = 1;

void qlpmgr_usage(void);

int qlp_cal
(
	int argc,               /* unix cmd line argument count */
	char *argv[],           /* unix cmd line argument vector */
	long tae,               /* TAE flag, TRUE=>valid param block */
	struct PARBLK *vblock   /* TAE param block */
);

int qlp_pstatus
(
	int argc,               /* unix cmd line argument count */
	char *argv[],           /* unix cmd line argument vector */
	long tae,               /* TAE flag, TRUE=>valid param block */
	struct PARBLK *vblock   /* TAE param block */
);

int qlp_exec_wedge
(
    struct QLP_PRINTER *printer,
    struct recfmt *rec,
    struct IMGOPT *imgopt
);


int main
(
	int argc,
	char *argv[]
)
{
struct PARBLK	vblock;
struct VARIABLE *sub;
long	zero = 0;
long    len;
long	tae;
char	subcmd[9];
char	prog[20];

tae = p_inim(&vblock,sizeof(struct PARBLK),P_CONT) == SUCCESS ? TRUE : FALSE;
/*
*  Retrieve the subcommand to be executed
*/
if (tae)
    {
    sub = p_find(&vblock,"_SUBCMD");
    strcpy(subcmd,SVAL(*sub,0));
    }
else
    {
    if (argc <= 1)
        {
        qlpmgr_usage();
        c_errmsg("Error reading input parameters"," ",&fatal);
        }
    strcpy(subcmd,argv[1]);
    len = (long)strlen(subcmd);
    c_low2up(subcmd,&len);
    --argc;
    ++argv;
    }
sprintf(prog,"QLPMGR-%s",subcmd);
if (!(strcmp(subcmd,"CAL")))
    {
    qlp_cal(argc,argv,tae,&vblock);
    }
else if (!(strcmp(subcmd,"PSTATUS")))
    {
    qlp_pstatus(argc,argv,tae,&vblock);
    }
c_complt(prog,"",&zero);
return 0;
}

int qlp_pstatus
(
	int argc,               /* unix cmd line argument count */
	char *argv[],           /* unix cmd line argument vector */
	long tae,               /* TAE flag, TRUE=>valid param block */
	struct PARBLK *vblock   /* TAE param block */
)
{
extern int optind;
extern char *optarg;
struct VARIABLE *val;
struct recfmt rec;
struct QLPPID qlppid;
struct QLP_PRINTER printer[MAX_QLP_PRINTER];	/* printer definitions */
int  c;						/* program option */
long len;					/* char string len */
long pcnt;					/* printer count */
long suflg;					/* super user printer flag */
char emsg[256];					/* error message buffer */
char pr[256];					/* printer name */
char pstatus[256];				/* printer status */

if (tae)
    {
    val = p_find(vblock,"PRINTER");
    strcpy(pr,SVAL(*val,0));
    len = (long)strlen(pr);
    c_up2low(pr,&len);
    val = p_find(vblock,"STATUS");
    strcpy(pstatus,SVAL(*val,0));
    len = (long)strlen(pstatus);
    c_up2low(pstatus,&len);
    }
else
    {
    strcpy(pr,"");
    strcpy(pstatus,"");
    while ((c=getopt(argc,argv,"p:s:"))!=EOF)
        {
        switch (c)
            {
            case 'p':
                strcpy(pr,optarg);
                len = (long)strlen(pr);
                c_up2low(pr,&len);
                break;
            case 's':
                strcpy(pstatus,optarg);
                len = (long)strlen(pstatus);
                c_up2low(pstatus,&len);
		if ((strcmp(pstatus,"up") != 0) && (strcmp(pstatus,"dn") != 0))
		    {
		    sprintf(emsg,"%s is invalid for pstatus (dn up)",pstatus);
		    c_errmsg(emsg," ",&nonfatal);
		    qlpmgr_usage();
		    }
                break;
            default:
                sprintf(emsg,"QLPMGR PSTATUS Invalid option...");
                c_errmsg(emsg," ",&fatal);
                break;
            }
        }
    if ((strlen(pr) == 0) || (strlen(pstatus) == 0))
	qlpmgr_usage();
    }
strcpy(rec.printer,pr);
strcpy(rec.ptype,"");
(void) crfinit("QLPMGR-PSTATUS");
(void) crfpr(printer,&pcnt,TRUE);
(void) crfdone();
suflg = qlp_chkuser(printer,pcnt,&rec);
if (suflg != SU_OWN)
    {
    sprintf(emsg,"Not priviliged to change status of printer <%s>",pr);
    c_errmsg(emsg," ",&fatal);
    }
for (;;)
    {
    if (qlp_getpid(&qlppid) != E_SUCC)
        c_errmsg("Error reading QLP media file"," ",&fatal);
    if (strlen(qlppid.cmd) == 0)
	break;
    c_errmsg("QLP Daemon busy, please wait..."," ",&nonfatal);
    sleep(QLP_SLEEP);
    }
sprintf(qlppid.cmd,"%s %s",pr,pstatus);
if (qlp_putpid(&qlppid) != E_SUCC)
    c_errmsg("Error writing QLP media file"," ",&fatal);
for (;;)
    {
    if (qlp_getpid(&qlppid) != E_SUCC)
        c_errmsg("Error reading QLP media file"," ",&fatal);
    if (strlen(qlppid.cmd) == 0)
	break;
    c_errmsg("QLP Daemon busy, please wait..."," ",&nonfatal);
    sleep(QLP_SLEEP);
    }
return(1);
}

int qlp_cal
(
	int argc,               /* unix cmd line argument count */
	char *argv[],           /* unix cmd line argument vector */
	long tae,               /* TAE flag, TRUE=>valid param block */
	struct PARBLK *vblock   /* TAE param block */
)
{
#define WEDSIZ 150
static long wedval[16] = {  0, 17, 34, 51, 68, 85,102,119,
			  136,153,170,187,204,221,238,255};
extern int optind;
extern char *optarg;
struct QLP_PRINTER printer[MAX_QLP_PRINTER];
struct VARIABLE *val;
struct recfmt rec;
struct IMGOPT imgopt;
struct tm *tptr;
time_t t;
long cnt;
long cnt1;
long cnt2;
long dtype;
long len;
long nbytes;
long pcnt;
long printer_cnt;
long rstat;
long sf;
int  c;
unsigned char *bufptr;
unsigned char fillval_byte; 
char *calfile;
char *confile;
char emsg[256];
char *lutdir;
char *pr;
char timestr[64];

t = time(NULL);
tptr = localtime(&t);
sprintf(timestr,"%02d/%02d/%02d %02d:%02d",tptr->tm_mon+1,tptr->tm_mday,
        tptr->tm_year,tptr->tm_hour,tptr->tm_min);
if (tae)
    {
    val = p_find(vblock,"PRINTER");
    pr = SVAL(*val,0);
    }
else
    {
    strcpy(pr,"");
    while ((c=getopt(argc,argv,"p:"))!=EOF)
        {
        switch (c)
            {
            case 'p':
                strcpy(pr,optarg);
                len = (long)strlen(pr);
                c_up2low(pr,&len);
                break;
            default:
                sprintf(emsg,"QLPMGR CAL Invalid option...");
                c_errmsg(emsg," ",&fatal);
                break;
            }
        }
    if (strlen(pr) == 0)
	qlpmgr_usage();
    }
(void) crfinit("CFMGR-CAL");
(void) crfpr(printer,&pcnt,TRUE);
(void) crfdone();
for (printer_cnt=0; printer_cnt<pcnt; printer_cnt++)
    {
    if (strcmp(printer[printer_cnt].lname,pr) == 0)
	break;
    }
if (printer_cnt == pcnt)
    {
    sprintf(emsg,"Printer <%s> does not exist",pr);
    c_errmsg(emsg," ",&fatal);
    }
val = p_find(vblock,"LUTDIR");
lutdir = getenv(SVAL(*val,0));
if (lutdir == NULL)
    {
    sprintf(emsg, "Environment variable %s for LUT directory not set",
	SVAL(*val,0));
    c_errmsg(emsg," ",&fatal);
    }
val = p_find(vblock,"CALFILE");
calfile = SVAL(*val,0);
val = p_find(vblock,"CONFILE");
confile = SVAL(*val,0);
rec.seqnum = 0;
rec.sl = rec.ss = 1;
rec.nl = 150;
rec.ns = 2400;
rec.nb = 1;
rec.curpan = 1;
rec.ncop = 1;
rec.curcop = 1;
rec.reduce[0] = rec.reduce[1] = 1;
rec.expand[0] = rec.expand[1] = NULLVAL;
rec.bgrnd = 0;
strcpy(rec.color,"RGB");
strcpy(rec.rotopt,"NO");
strcpy(rec.spot,"NO");
strcpy(rec.wedge,"NO");
strcpy(rec.stropt,"NONE");
getuser(rec.usercode);
sprintf(rec.annot,"%s  %s  %s",rec.usercode,timestr,printer[printer_cnt].lname);
rec.title[0] = rec.stretch[0][0] = '\0';
for (cnt=0; cnt<NLUT; cnt++)
    {
    rec.enhlut[cnt].in[0] = rec.enhlut[cnt].out[0] = 0;
    rec.enhlut[cnt].in[1] = rec.enhlut[cnt].out[1] = 255;
    rec.enhlut[cnt].ne = 2;
    }
rstat = qlp_setopt(&printer[printer_cnt],&rec,&imgopt);
if (rstat != E_SUCC)
    goto xit;
imgopt.time_elapse = time(NULL);
imgopt.maxnl = WEDSIZ;
nbytes = (imgopt.maxnl + imgopt.fontnl) * imgopt.maxns;
imgopt.lasbuf = (unsigned char *) malloc(nbytes);
if (imgopt.lasbuf == NULL)
    {
    sprintf(rec.errtext,"Error allocating %d bytes memory for image",
                nbytes);
    c_errmsg(rec.errtext," ",&nonfatal);
    rstat = E_FAIL;
    goto xit;
    }
dtype = EBYTE;
fillval_byte = rec.bgrnd;
c_pxfill(imgopt.lasbuf,&dtype,&nbytes,&fillval_byte);
for (cnt=0; cnt<imgopt.maxnl; cnt++)
    {
    bufptr = imgopt.lasbuf + (cnt * imgopt.maxns);
    for (cnt1=0; cnt1<16; cnt1++)
	{
	for (cnt2=0; cnt2<WEDSIZ; cnt2++)
	    {
	    *bufptr++ = wedval[cnt1];
	    }
	}
    }
sf = 1;
for (cnt=0; cnt<imgopt.fontnl; cnt++)
    {
    bufptr = imgopt.lasbuf + ((cnt+imgopt.maxnl) * imgopt.maxns);
    font2(rec.annot,cnt,rec.bgrnd,255-rec.bgrnd,sf,bufptr);
    }
strcpy(rec.printer,printer[printer_cnt].lname);




rstat = qlp_dmpimg_ps(&printer[printer_cnt],&rec,&imgopt,calfile,confile,0);
if (rstat != E_SUCC)
    goto xit;
rstat = qlp_dmpimg_ps(&printer[printer_cnt],&rec,&imgopt,calfile,confile,1);
if (rstat != E_SUCC)
    goto xit;
rstat = qlp_dmpimg_ps(&printer[printer_cnt],&rec,&imgopt,calfile,confile,2);
if (rstat != E_SUCC)
    goto xit;
rstat = qlp_exec_wedge(&printer[printer_cnt],&rec,&imgopt);
if (rstat != E_SUCC)
    goto xit;
xit:
return(1);
}

/*
*  Execute program to convert image into postscript file, and then spool
*  the file to the specified printer.  This function is used to generate
*  the Calibration wedge.
*/
int qlp_exec_wedge
(
    struct QLP_PRINTER *printer,
    struct recfmt *rec,
    struct IMGOPT *imgopt
)
{
long    cnt;			/* automatically retry on error up to 5 times */
long    rstat;                  /* function return status code */
long    status;                 /* status code */
char    cmd[256];		/* system command line */
char    opt[256];		/* sgi2ps options string */
char	sg[256];		/* sgi file name */
char	ps[256];		/* postscript file name */
char	*ptr;

rstat = E_SUCC;
sprintf(sg,"sgi_%d_%d",rec->seqnum,rec->curpan);
sprintf(ps,"ps_%d_%d",rec->seqnum,rec->curpan);
if (strcmp(rec->autosz,"YES") == 0)
    sprintf(opt,"-n %d -z 1",rec->ncop);
else
    sprintf(opt,"-n %d -z 0",rec->ncop);
if (strcmp(printer->ps,"binary") == 0)
    strcat(opt," -B");
if (imgopt->rotate)
    strcat(opt," -r 90");
if (imgopt->outnb == 1)
    sprintf(cmd,"/usr/lib/print/sgi2ps -O %s -C w %s %s",ps,opt,sg);
else if (imgopt->outnb == 3)
    sprintf(cmd,"/usr/lib/print/sgi2ps -O %s -C rgb %s %s",ps,opt,sg);
else if (imgopt->outnb == 4)
    sprintf(cmd,"/usr/lib/print/sgi2ps -O %s -C cmyk %s %s",ps,opt,sg);
printf("qlp_exec: %s\n",cmd);
status = system(cmd);
if (status != 0)
    {
    rstat = E_FAIL;
    strcpy(rec->stflg,QLP_ERROR);
    sprintf(rec->errtext,
	    "Error converting sgi image to postscript, status = %d",status);
    goto xit;
    }
#ifndef DEBUG
    c_delfil(sg);
#endif
/*
*  Check to see if printer is currently busy.  If busy, wait up to a
*  maximum of ten minutes for printer to become idle.  After ten minutes,
*  queue the print job.
*/
for (cnt=0; cnt<600/QLP_SLEEP; cnt++)
    {
    if (qlp_chkprinter(printer))
	break;
    sleep(QLP_SLEEP);
    }
/*
* Submit print job with correct number of copies.  Log each copy individually.
* Multiple copies does not work; therefore, submit for each copy but on the
* last copy specify -r to remove the file.
*/
if (strlen(printer->tray) > 0)
    {
    ptr = getenv("QDIR");
    if (ptr == NULL)
        {
        c_errmsg("qlp_exec: Environment variable QDIR not set"," ",&nonfatal);
	c_errmsg("qlp_exec: Printer tray selection failed"," ",&nonfatal);
        }
    else
	{
	if (strcmp(printer->lp,"lpr") == 0)
	    sprintf(cmd,"lpr -P %s %s/%s",rec->printer,ptr,printer->tray);
	else if (strcmp(printer->lp,"lp") == 0)
	    sprintf(cmd,"lp -d%s %s/%s",rec->printer,ptr,printer->tray);
	status = system(cmd);
	if (status != 0)
            {
            sprintf(rec->errtext,
	            "Error in lp command <%s>, status = %d",cmd,status);
            }
	sleep(5);
	}
    }
if (strcmp(printer->lp,"lpr") == 0)
    sprintf(cmd,"lpr -P %s -s -r %s",rec->printer,ps);
else if (strcmp(printer->lp,"lp") == 0)
    sprintf(cmd,"lp -d%s %s",rec->printer,ps);
#ifdef DEBUG
fprintf(stdout,"lp cmd <%s> --- NOT Executed\n",cmd);
#else
status = system(cmd);
if (status != 0)
    {
    rstat = E_FAIL;
    strcpy(rec->stflg,QLP_ERROR);
    sprintf(rec->errtext,
	        "Error in lp command <%s>, status = %d",cmd,status);
    goto xit;
    }
#endif
for (rec->curcop=1; rec->curcop<=rec->ncop; rec->curcop++)
    {
    qlp_logit(rec,imgopt);
    }
xit:
return(rstat);
}

void qlpmgr_usage(void)
{
fprintf(stderr,"qlpmgr usage:\n\n");
fprintf(stderr,"qlpmgr pstatus {-p printer} {-s status}\n");
fprintf(stderr,"qlpmgr cal {-p printer}\n");
fprintf(stderr,"  -p printer (PRINTER)  %s\n",
	"Printer name to change status or print CAL wedge");
fprintf(stderr,"  -s status  (PSTATUS)  %s\n",
	"Printer status.  Must be \"dn\" or \"up\"");
exit(-1);
}
