/******************************************************************************

SUBROUTINE:	delreq

PURPOSE:
	Subroutine del will delete a request from the Custom Request File.
	The structure of the file is a header record of 80 bytes followed by
	records containing the requests. The header record contains the head 
	of the two linked lists.

	The record field "delflg" forms a singly linked list containing all the
	records that do not have valid requests (i.e. the requests have been
	deleted).  The record fields "bptr" and "fptr" forms a doubly linked
	list among the requests containg the same priority.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

Algorithm:
	Basic algorithm of the del subroutine:  Each request to be deleted is
	identified by the parameter SEQNUM.  For each request to be deleted,
	the following steps are taken.  The priority of the request is deter-
	mined by reading the record.  The request is then removed from its
	priority list.  The record is added to the singly linked list contain-
	ing all records that no longer contain valid requests.  Note that the
	requests are deleted, but that the record containing the request is 
	not deleted from the file.
******************************************************************************/


#include <stdio.h>
#include "las.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"

static long nonfatal = 1;


void delreq
(
    struct QLP_PRINTER printer[MAX_QLP_PRINTER],
    long pcnt,
    long fd,
    long nrec,
    int *seqptr,
    int count
)
{
struct recfmt rec;
struct hdrfmt hdr;
struct recfmt tmprec;
int	     seqnum;
long	     i;
long 	     suflg;
char	     msg[80];
char 	     user[64];

getuser(user);
(void) crfrdhdr(fd,TRUE,&hdr);		/* read hdr record of CRF */
for (i=0;i<count;i++)		    
   {
   seqnum = *seqptr++;
   if (seqnum > nrec)
      {	
      sprintf(msg,"Sequence Number %d does not exist",seqnum);
      c_errmsg(msg," ",&nonfatal);
      continue;
      }
   crfrd(fd,TRUE,seqnum,&rec);	   /* read the CRF request to be deleted */
   if (rec.delflg != -1)	   /* if not a valid request then */
      {				   /*   write error message & go to start */
      sprintf(msg,"Sequence Number %d does not exist",seqnum);
      c_errmsg(msg," ",&nonfatal);
      continue;
      } 
   suflg = qlp_chkuser(printer,pcnt,&rec);
   if ((suflg != SU_OWN) && (strcmp(rec.usercode,user) != 0))
      {
      sprintf(msg,"ERROR: User <%s> is not owner of custom request %d",
	   user,seqnum);
      c_errmsg(msg," ",&nonfatal);
      continue;
      }
   if ((suflg != SU_OWN) && (strcmp(rec.stflg,QLP_RUN) == 0))
      {
      sprintf(msg,"ERROR: Request %d is currently being processed",seqnum);
      c_errmsg(msg," ",&nonfatal);
      continue;
      }
				   /*  UPDATE THE FREE LIST		     */
   rec.delflg = hdr.fptr[0]; 	   /* add record to the free list by setting */
   hdr.fptr[0] = seqnum;	   /* delflg to point to next free record and*/
				   /* set head of list to the record	     */
   (void) crfwr(fd,TRUE,seqnum,&rec); /* write deleted request into the CRF */
				   /* UPDATE THE PRIORITY LIST		     */
   if (rec.bptr == 0)	   	   /* if request is at head of list then     */
      hdr.fptr[rec.prio] = rec.fptr;/*	set next record in list to the head  */
   else				   /* else must update the previous request's*/
      {				   /*   forward pointer   */
      crfrd(fd,TRUE,rec.bptr,&tmprec); 
      tmprec.fptr = rec.fptr;      /* update its front pointer        */
      crfwr(fd,TRUE,rec.bptr,&tmprec);
      }
   if (rec.fptr == 0)	     	     /* if request is at end of list then    */
      hdr.bptr[rec.prio] = rec.bptr; /* set previous record to end of list   */
   else				     /* else must update the next request's  */
      {				     /*	backward pointer */
      crfrd(fd,TRUE,rec.fptr,&tmprec);
      tmprec.bptr = rec.bptr;	     /* update its back pointer */
      crfwr(fd,TRUE,rec.fptr,&tmprec);
      }
   }
crfwrhdr(fd,TRUE,&hdr);
return;
}	/*  END of DEL subroutine  */
