/******************************************************************************

FUNCTION:  GETLUT

PURPOSE:
	GETLUT generates the three look up tables (red, green, and blue).  It 
	constructs the final look up table using a combination of the control, 
	calibration, and enhancement LUT files as specified in the input 
	parameters.  

	NOTE:  The black LUT is currently ignored!!


PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "ntqlp.h"
#include "qlp.h"
#include "local_protos.h"

int getlut
(
    struct QLP_PRINTER *printer,
    struct recfmt *rec,
    char *calfile,
    char *confile,
    char *lutenv,
    unsigned char clut[],
    unsigned char mlut[],
    unsigned char ylut[],
    unsigned char klut[]
)
{
struct lut callut[3];
struct lut conlut[3];
struct lut enhlut[3];
float	enh[256],cal[512],con[256];
long	nonfatal = 1;
long 	i;
long	rstat;
long	status;
short int tlut[3][MAXPT];
char	fname[256];
char	lutdir[256];
char	*lutptr;

rstat = E_SUCC;
for (i=0; i<3; i++)
    {
    enhlut[i].in[0] = enhlut[i].out[0] = 0;
    enhlut[i].in[1] = enhlut[i].out[1] = 255;
    enhlut[i].ne = 2;
    }
lutptr = getenv(lutenv);
if (lutptr == NULL)
    c_errmsg("Environment variable for LUT directory not set"," ",&nonfatal);
sprintf(lutdir,"%s/%s",lutptr,rec->printer);
if (confile == NULL)
    sprintf(fname,"%s/%s",lutdir,printer->mtype);
else
    sprintf(fname,"%s/%s",lutdir,confile);
status = rdlut("RGB",NULLVAL,fname,"CON",conlut); /* read control file */
if (status != E_SUCC)
    {
    sprintf(rec->errtext,"Error reading control LUT %s",fname);
    rstat = E_FAIL;
    goto xit;
    }
if (calfile == NULL)
    sprintf(fname,"%s/%s",lutdir,printer->mtype);
else
    sprintf(fname,"%s/%s",lutdir,calfile);
status = rdlut("RGB",NULLVAL,fname,"CAL",callut); /* read calibration file */
if (status != E_SUCC)
    {
    sprintf(rec->errtext,"Error reading calibration LUT %s",fname);
    rstat = E_FAIL;
    goto xit;
    }
/*
*  Do for each color (red, green, blue).
*  call br_enhance to create the enhance lut
*  call br_calbrt to create the calibration lut
*  call br_contrl to create the control lut
*  call br_punt to combine the enhancement, calibration, and control luts
*/
for (i=0; i<3; i++)
   {
   status = br_enhance(enhlut[i].in,enhlut[i].out,&enhlut[i].ne,enh); 
   if (status == -1)
      {
      sprintf(rec->errtext,"Enhnacement LUT not terminated with 255");
      rstat = E_FAIL;
      goto xit;
      }
   if (status == -2)
      {
      sprintf(rec->errtext,"Enhancement LUT does not begin with 0");
      rstat = E_FAIL;
      goto xit;
      }
   br_calbrt(callut[i].in,callut[i].out,&callut[i].ne,cal);
   br_contrl(conlut[i].in,conlut[i].out,&conlut[i].ne,con);
   br_punt(cal,con,enh,tlut[i]);
   }
for (i=0; i<MAXPT; i++)
    {
    clut[i] = (unsigned char) tlut[0][i];
    mlut[i] = (unsigned char) tlut[1][i];
    ylut[i] = (unsigned char) tlut[2][i];
/*******************
    klut[i] = (clut[i]<mlut[i]) ? clut[i] : mlut[i];
    klut[i] = (klut[i]<ylut[i]) ? klut[i] : ylut[i];
**********************/
    klut[i] = (unsigned char)i;
    }
xit:
return(rstat);
}
