/******************************************************************************

PROGRAM:  QLPDMP

PURPOSE: 
	QLPDMP dumps an image band 

			
PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           2/92       K. Gacke    initial development 

PROJECT:        Quick Look Printer

Algorithm:

******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#ifdef sgi
#include <gl/image.h>
#endif
#include "las.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"

static long fatal = -1;

int qlp_dmpimg_ps
(
    struct QLP_PRINTER *printer,
    struct recfmt *rec,
    struct IMGOPT *imgopt,
    char *calfile,
    char *confile,
    long band
)
{
static IMAGE *img_fp;
static unsigned short *tbuf;
long cnt;
long lin;
long nl;
long ns;
int  rstat;
long status;
unsigned char *bufptr;
char fname[256];
unsigned char lut[4][256];

rstat = E_SUCC;
if ((strcmp(rec->color,"RGB") == 0) || (strcmp(rec->color,"BW") == 0))
    {
    status = getlut(printer,rec,calfile,confile,"QLPLUT",lut[0],
	    lut[1],lut[2],lut[3]);
    if (status != E_SUCC)
        {
        rstat = E_FAIL;
        printf("Error generating LUT\n");
        goto xit;
        }
    }
/*
	CMY and CMYK images use a one to one LUT (ie they are not calibrated)
*/
else if ((strcmp(rec->color,"CMY") == 0) || (strcmp(rec->color,"CMYK") == 0))
    {
    for (cnt=0; cnt<LUT_LENGTH; cnt++)
	{
	lut[0][cnt] = cnt;
	lut[1][cnt] = cnt;
	lut[2][cnt] = cnt;
	lut[3][cnt] = cnt;
	}
    }
sprintf(fname,"%s_%d_%d","sgi",rec->seqnum,rec->curpan);
nl = imgopt->maxnl + imgopt->fontnl;
ns = imgopt->maxns;
if (band == 0)
    {
    unlink(fname);
#ifdef DEBUG
    printf("qlp_dmpimg: ns=%d  nl=%d  outnb=%d\n",ns,nl,imgopt->outnb);
#endif
    img_fp = iopen(fname,"w",RLE(1),2,ns,nl,imgopt->outnb);
    if (img_fp == NULL)
	{
	printf("Error opening sgi tmp file <%s>\n",fname);
	perror("iopen");
	return(E_FAIL);
	}
    isetname(img_fp,"qlp");
    isetcolormap(img_fp,CM_NORMAL);
    tbuf = (unsigned short *) malloc(ns * sizeof(short));
    if (tbuf == NULL)
	c_errmsg("qlp_dmpimg_ps: Error allocating temporary buffer"," ",&fatal);
    }
bufptr = imgopt->lasbuf;
for (lin=0; lin<nl; ++lin)
    {
    for (cnt=0; cnt<ns; cnt++)
	{
	tbuf[cnt] = (unsigned short) (255 - lut[band][*(bufptr+ns-cnt)]);
	}
    putrow(img_fp,tbuf,lin,band);
    bufptr += imgopt->maxns;
    }
xit:
if ((rstat == E_FAIL) || (band + 1 == imgopt->outnb))
    {
    iclose(img_fp);
    free(tbuf);
    }
return(rstat);
}
