/******************************************************************************
FUNCTION:  QLP PID

PURPOSE:
        Keeps track of QLP process IDs.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        QLP

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#ifdef unix
#include <sys/errno.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#else
#include <file.h>
#include <unixio.h>
#include <stat.h>
#endif
#include "las.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"


int qlp_getpid
(
    struct QLPPID *qlppid
)
{
int  fd;
int  prot;
int  rstat;
long status;
char fname[CMLEN];
char *qdir;

rstat = E_SUCC;
qdir = getenv("QDIR");
if (qdir == NULL)
    {
    printf("Environment variable QDIR not defined");
    rstat = E_FAIL;
    goto xit;
    }
sprintf(fname,"%s/%s",qdir,PID_FIL);
if (access(fname,0) == -1)
    {
    qlppid->qlpd_pid = NULLVAL;
    qlppid->status = QLP_IDLE;
    goto xit;
    }
prot = 0777;
fd = open(fname,O_RDONLY,prot);
if (fd == -1)
    {
    perror("qlp_getpid");
    printf("Error opening file %s\n",fname);
    rstat = E_FAIL;
    goto xit;
    }
#ifdef sgi
if (flock(fd,LOCK_SH) == -1)
    {
    perror("qlp_getpid");
    printf("Error setting LOCK_SH on file %s\n",fname);
    rstat = E_FAIL;
    goto xit;
    }
#endif
status = read(fd,qlppid,sizeof(struct QLPPID));
if (status <= 0)
    {
    perror("qlp_getpid");
    printf("Error reading qlppid file, status=%d\n",status);
    rstat = E_FAIL;
    goto xit;
    }
xit:
close(fd);
return(rstat);
}

int qlp_putpid
(
    struct QLPPID *qlppid
)
{
int  fd;
int  prot;
long crflg;
long rstat;
long status;
char fname[CMLEN];
char *qdir;

rstat = E_SUCC;
qdir = getenv("QDIR");
if (qdir == NULL)
    {
    printf("Environment variable QDIR not defined");
    rstat = E_FAIL;
    goto xit;
    }
sprintf(fname,"%s/%s",qdir,PID_FIL);
if (access(fname,0) == 0)
    crflg = FALSE;
else
    crflg = TRUE;
prot = 0777;
fd = open(fname,O_RDWR | O_CREAT,prot);
if (fd == -1)
    {
    perror("qlp_putpid");
    printf("Error opening file %s\n",fname);
    rstat = E_FAIL;
    goto xit;
    }
if (crflg)
    chmod(fname,0777);
#ifdef sgi
if (flock(fd,LOCK_EX) == -1)
    {
    perror("qlp_putpid");
    printf("Error setting LOCK_EX on file %s\n",fname);
    rstat = E_FAIL;
    goto xit;
    }
#endif
status = write(fd,qlppid,sizeof(struct QLPPID));
if (status <= 0)
    {
    perror("qlp_putpid");
    printf("Error writing qlppid, status=%d\n",status);
    rstat = E_FAIL;
    goto xit;
    }
xit:
close(fd);
return(rstat);
}
