/******************************************************************************

SUBROUTINE:	QLP Utilities

PURPOSE: 
	QLP utility functions 

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE
   2		10/96	    K. Gacke	allow multiple printers/printer types

PROJECT:        Quick Look Printer

LANGUAGE	C

******************************************************************************/

/* #define DEBUG 1 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"


static long nonfatal = 1;

int qlp_parse
(
	long cnt,
	struct QLP_PRINTER printer[],
	char *buf
);

/*
*  Change status of specified printer to indicate the printer is currently
*  in use (or down).
*/
int qlp_dnprinter
(
    char *pname,
    struct QLP_PRINTER printer[],
    long pcnt
)
{
long printer_cnt;

for (printer_cnt=0; printer_cnt<pcnt; printer_cnt++)
    {
    if (strcmp(pname,printer[printer_cnt].lname) == 0)
	{
	strcpy(printer[printer_cnt].status,"dn");
	break;
	}
    }
return(1);
}

/*
*  Check status of specified printer.  If not busy return TRUE; otherwise 
*  return FALSE.  Note the configuration file specifies if "lp" or "lpr"
*  is used to access the printer.
*/
int qlp_chkprinter
(
    struct QLP_PRINTER *printer
)
{
FILE *fp;
long cnt;
long len;
long reset;
int rstat;
char cmd[256];
char line[256];

rstat = FALSE;
reset = FALSE;
if (strcmp(printer->lp,"lpr") == 0)
    {
    sprintf(cmd,"lpq -P%s",printer->pname);
    fp = popen(cmd,"r");
    if (fp == NULL)
        {
        fprintf(stderr,"Error in popen call <%s>\n",cmd);
        rstat = E_FAIL;
        goto xit;
        }
    for (cnt=0; ; cnt++)
        {
        if (fgets(line,256,fp) == NULL)
            break;
        len = (long)strlen(line);
        c_up2low(line,&len);
        if (scanbuf(line,"no entries") >= 0) 
            rstat = TRUE;
        if (scanbuf(line,"no jobs queued on this port") >= 0)
            rstat = TRUE;
	if (scanbuf(line,"no daemon present") >= 0)
	    reset = TRUE;
        }
    pclose(fp);
    if (reset)
	{
        sprintf(cmd,"QLP resetting printer %s",printer->pname);
	c_errmsg(cmd," ",&nonfatal);
        sprintf(cmd,"lpc down %s",printer->pname);
        fp = popen(cmd,"r");
        if (fp == NULL)
            {
            fprintf(stderr,"Error in popen call <%s>\n",cmd);
            rstat = E_FAIL;
            goto xit;
            }
        for (cnt=0; ; cnt++)
            {
            if (fgets(line,256,fp) == NULL)
                break;
            }
        pclose(fp);
	sleep(5);
        sprintf(cmd,"lpc up %s",printer->pname);
        fp = popen(cmd,"r");
        if (fp == NULL)
            {
            fprintf(stderr,"Error in popen call <%s>\n",cmd);
            rstat = E_FAIL;
            goto xit;
            }
        for (cnt=0; ; cnt++)
            {
            if (fgets(line,256,fp) == NULL)
                break;
            }
        pclose(fp);
	}
    }
else if (strcmp(printer->lp,"lp") == 0)
    {
    sprintf(cmd,"lpstat -p%s",printer->pname);
    fp = popen(cmd,"r");
    if (fp == NULL)
        {
        fprintf(stderr,"Error in popen call <%s>\n",cmd);
        rstat = E_FAIL;
        goto xit;
        }
    for (cnt=0; ; cnt++)
        {
        if (fgets(line,256,fp) == NULL)
            break;
        len = (long)strlen(line);
        c_up2low(line,&len);
        if (scanbuf(line,"is idle") >= 0) 
            rstat = TRUE;
        }
    pclose(fp);
    }
/*
*  If the lpq returns more than 5 lines, assume the printer is down (even
*  if it found the "no entries" line.  We have seen the Tektronix Phaser480
*  hang, but entries continued to get queued because it still listed 
*  "no entries".
*/
if (cnt > 5)
    rstat = FALSE;
xit:
return(rstat);
}

/*
*   Check to see if user is an owner, privilged, or normal privileges
*   of the specified printer within the "rec" structure.  If printer 
*   is not specified, search for appropraite printer type (in this case, 
*   return the lowest set of privileges common to all printers within
*   the specified printer type).
*/
int qlp_chkuser
(
    struct QLP_PRINTER printer[],
    long pcnt,
    struct recfmt *rec
)
{
long cnt;
long cnt1;
int  suflg;
char user[CMLEN];

suflg = SU_NORM;
getuser(user); 
if ((strcmp(user,"gacke") == 0) || 
    (strcmp(user,"root") == 0))
        {
        suflg = SU_OWN;
        }
else if (strlen(rec->printer) > 0)
    {
    for (cnt=0; cnt<pcnt; cnt++)
        {
	if (strcmp(rec->printer,printer[cnt].lname) == 0)
	    {
	    for (cnt1=0; ; cnt1++)
		{
		if (strlen(printer[cnt].priv[cnt1]) == 0)
		    break;
		if (strcmp(printer[cnt].priv[cnt1],user) == 0)
		    suflg = SU_PRIV;
		}
	    for (cnt1=0; ; cnt1++)
		{
		if (strlen(printer[cnt].owner[cnt1]) == 0)
		    break;
		if (strcmp(printer[cnt].owner[cnt1],user) == 0)
		    suflg = SU_OWN;
		}
	    break;
	    }
	}
    }
else
    {
    for (cnt=0; cnt<pcnt; cnt++)
        {
	if (strcmp(rec->ptype,printer[cnt].type) == 0)
	    {
	    suflg = SU_NORM;
	    for (cnt1=0; ; cnt1++)
		{
		if (strlen(printer[cnt].priv[cnt1]) == 0)
		    break;
		if (strcmp(printer[cnt].priv[cnt1],user) == 0)
		    suflg = SU_PRIV;
		}
	    for (cnt1=0; ; cnt1++)
		{
		if (strlen(printer[cnt].owner[cnt1]) == 0)
		    break;
		if (strcmp(printer[cnt].owner[cnt1],user) == 0)
		    suflg = SU_OWN;
		}
	    if (suflg == SU_NORM)
		break;
	    }
	}
    }
return(suflg);
}

int qlp_getprinter
(
    struct QLP_PRINTER printer[],
    long *pcnt			/* number of printers defined */
)
{
FILE *fp;
long cnt;
int  rstat;
char buf[CMLEN];
char fname[CMLEN];
char msg[CMLEN];
char *ptr;

rstat = E_SUCC;
fp = NULL;
ptr = getenv("QDIR");
if (ptr == NULL)
    {
    c_errmsg("getprinter: Environment variable QDIR not set"," ",&nonfatal);
    goto xit;
    }
sprintf(fname,"%s/%s",ptr,"qlp.config");
fp = fopen(fname,"r");
if (fp == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"getprinter: Error opening config file %s",fname);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
for (cnt=0; cnt<MAX_QLP_PRINTER; cnt++)
    strcpy(printer[cnt].lname,"");
for (cnt=0; cnt<MAX_QLP_PRINTER; )
    {
    if (fgets(buf,256,fp) == NULL)
	break;
    if (*buf == '#')
	continue;
    if (*(buf + strlen(buf) - 1) == '\n')
	*(buf + strlen(buf) - 1) = '\0';
    if (qlp_parse(cnt,printer,buf) != E_SUCC)
	continue;
    cnt++;
    }
*pcnt = cnt;
if (cnt >= MAX_QLP_PRINTER)
    {
    rstat = E_FAIL;
    sprintf(msg,"getprinter: Max number of printers is %d\n",MAX_QLP_PRINTER);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
xit:
if (fp != NULL)
    fclose(fp);
return(rstat);
}

int qlp_putprinter
(
	char *pr,
	char *pstat
)
{
struct QLP_PRINTER printer[MAX_QLP_PRINTER];
struct QLP_PRINTER *pr_ptr;
FILE *fp = NULL;
FILE *fp_out = NULL;
long cnt, cnt1, cnt2;
long pcnt;
long rstat;
char buf[CMLEN];
char fname[CMLEN];
char fname_out[CMLEN];
char msg[CMLEN];
char *ptr;
char desc[256];
char depend[256];
char file_type[256];
char lblpr[256];
char lp[256];
char owner[256];
char priv[256];
char tray[256];

rstat = E_SUCC;
ptr = getenv("QDIR");
if (ptr == NULL)
    {
    c_errmsg("putprinter: Environment variable QDIR not set"," ",&nonfatal);
    goto xit;
    }
sprintf(fname,"%s/%s",ptr,"qlp.config");
fp = fopen(fname,"r");
if (fp == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"putprinter: Error opening config file %s",fname);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
sprintf(fname_out,"%s/%s",ptr,"qlp.config.new");
fp_out = fopen(fname_out,"w");
if (fp == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"putprinter: Error opening config file %s",fname_out);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
for (pcnt=0; pcnt<MAX_QLP_PRINTER; pcnt++)
    strcpy(printer[pcnt].lname,"");
for (pcnt=0; pcnt<MAX_QLP_PRINTER; )
    {
    if (fgets(buf,256,fp) == NULL)
	break;
    if (*buf == '#')
	continue;
    if (*(buf + strlen(buf) - 1) == '\n')
	*(buf + strlen(buf) - 1) = '\0';
    if (qlp_parse(pcnt,printer,buf) != E_SUCC)
	continue;
    pcnt++;
    }
if (pcnt >= MAX_QLP_PRINTER)
    {
    rstat = E_FAIL;
    sprintf(msg,"putprinter: Max number of printers is %d\n",MAX_QLP_PRINTER);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
for (cnt=0; cnt<pcnt; cnt++)
    {
    if (strcmp(printer[cnt].lname,pr) == 0)
	{
	strcpy(printer[cnt].status,pstat);
	pr_ptr = &printer[cnt];
	for (cnt1=0; cnt1<MAX_QLP_PARAM; cnt1++)
	    {
	    for (cnt2=0; cnt2<pcnt; cnt2++)
		{
		if (strcmp(pr_ptr->depend[cnt1],printer[cnt1].lname) == 0)
		    strcpy(printer[cnt1].status,"dn");
		}
	    }
	}
    }
fseek(fp,0,0);
for (cnt=0; cnt<pcnt; )
    {
    if (fgets(buf,256,fp) == NULL)
	break;
    if (*buf == '#')
	{
	fprintf(fp_out,"%s",buf);
	continue;
	}
    strcpy(owner,"");
    strcpy(priv,"");
    strcpy(file_type,"");
    strcpy(depend,"");
    if (strlen(printer[cnt].desc) > 0)
	strcpy(desc,printer[cnt].desc);
    else
	strcpy(desc," ");
    if (strlen(printer[cnt].lp) > 0)
	strcpy(lp,printer[cnt].lp);
    else
	strcpy(lp," ");
    if (strlen(printer[cnt].lblpr) > 0)
	strcpy(lblpr,printer[cnt].lblpr);
    else
	strcpy(lblpr," ");
    if (strlen(printer[cnt].tray) > 0)
	strcpy(tray,printer[cnt].tray);
    else
	strcpy(tray," ");
    for (cnt1=0; cnt1<MAX_QLP_PARAM; cnt1++)
	{
	if (strlen(printer[cnt].owner[cnt1]) == 0)
	    break;
	if (cnt1 > 0)
	    strcat(owner,",");
	strcat(owner,printer[cnt].owner[cnt1]);
	}
    if (strlen(owner) == 0)
	strcpy(owner," ");
    for (cnt1=0; cnt1<MAX_QLP_PARAM; cnt1++)
	{
	if (strlen(printer[cnt].priv[cnt1]) == 0)
	    break;
	if (cnt1 > 0)
	    strcat(priv,",");
	strcat(priv,printer[cnt].priv[cnt1]);
	}
    if (strlen(priv) == 0)
	strcpy(priv," ");
    for (cnt1=0; cnt1<MAX_QLP_PARAM; cnt1++)
	{
	if (strlen(printer[cnt].ftype[cnt1]) == 0)
	    break;
	if (cnt1 > 0)
	    strcat(file_type,",");
	strcat(file_type,printer[cnt].ftype[cnt1]);
	}
    if (strlen(file_type) == 0)
	strcpy(file_type," ");
    for (cnt1=0; cnt1<MAX_QLP_PARAM; cnt1++)
	{
	if (strlen(printer[cnt].depend[cnt1]) == 0)
	    break;
	if (cnt1 > 0)
	    strcat(depend,",");
	strcat(depend,printer[cnt].depend[cnt1]);
	}
    if (strlen(depend) == 0)
	strcpy(depend," ");
    fprintf(fp_out,"%s:%s:%s:%s:%s:%d:%s:%s:%.4fx%.4f:%s:%s:%s:%s:%s:%s:%s\n",
	printer[cnt].lname,printer[cnt].pname,printer[cnt].type,lp,
	printer[cnt].ps, printer[cnt].dpi,owner,priv,printer[cnt].width,
	printer[cnt].height, printer[cnt].mtype,file_type,tray,depend,
	lblpr,desc,printer[cnt].status);
    ++cnt;
    }
xit:
if (fp != NULL)
    fclose(fp);
if (fp_out != NULL)
    fclose(fp_out);
sprintf(fname,"%s/%s",ptr,"qlp.config");
sprintf(fname_out,"%s/%s",ptr,"qlp.config.sav");
rename(fname,fname_out);
sprintf(fname,"%s/%s",ptr,"qlp.config.new");
sprintf(fname_out,"%s/%s",ptr,"qlp.config");
rename(fname,fname_out);
return(rstat);
}

int qlp_parse
(
	long cnt,
	struct QLP_PRINTER printer[],
	char *buf
)
{
long len;
long loccnt;
long rstat;
char depend[256];
char file_type[256];
char msg[256];
char line[256];
char owner[256];
char priv[256];
char *ptr;

rstat = E_SUCC;
printer[cnt].width = 8;
printer[cnt].height = 11;
printer[cnt].dpi = 300;
for (loccnt=0; loccnt<MAX_QLP_PARAM; loccnt++)
    {
    strcpy(printer[cnt].owner[loccnt],"");
    strcpy(printer[cnt].priv[loccnt],"");
    strcpy(printer[cnt].ftype[loccnt],"");
    strcpy(printer[cnt].depend[loccnt],"");
    }
strcpy(printer[cnt].desc,"");
strcpy(printer[cnt].lblpr,"");
strcpy(printer[cnt].lp,"");
strcpy(printer[cnt].mtype,"");
strcpy(printer[cnt].lname,"");
strcpy(printer[cnt].pname,"");
strcpy(printer[cnt].ps,"");
strcpy(printer[cnt].status,"");
strcpy(printer[cnt].tray,"");
strcpy(printer[cnt].type,"");
strcpy(line,buf);
len = (long)strlen(line);
c_up2low(line,&len);
ptr = strtok(line,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid logical printer name configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(printer[cnt].lname,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid physical printer name configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(printer[cnt].pname,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer type configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if ((strcmp(ptr,"claser") == 0) ||
    (strcmp(ptr,"phaser300") == 0) ||
    (strcmp(ptr,"phaser220") == 0) ||
    (strcmp(ptr,"hp755") == 0)     ||
    (strcmp(ptr,"hp2500") == 0)    ||
    (strcmp(ptr,"inkjet") == 0)    ||
    (strcmp(ptr,"photo") == 0)     ||
    (strcmp(ptr,"bw") == 0))
    {
    strcpy(printer[cnt].type,ptr);
    }
else
    {
    sprintf(msg,"%s is an invalid printer type",ptr);
    c_errmsg(msg," ",&nonfatal);
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid lp command configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if ((strcmp(ptr,"lp") == 0)  ||
    (strcmp(ptr,"lpr") == 0) ||
    (strcmp(ptr,"stage") == 0))
    {
    strcpy(printer[cnt].lp,ptr);
    }
else
    {
    sprintf(msg,"%s is an invalid printer command",ptr);
    c_errmsg(msg," ",&nonfatal);
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer data format  configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if ((strcmp(ptr,"binary") == 0) ||
    (strcmp(ptr,"ascii") == 0))
    {
    strcpy(printer[cnt].ps,ptr);
    }
else if (strcmp(ptr," ") == 0)
    strcpy(printer[cnt].ps,"");
else
    {
    sprintf(msg,"%s is an invalid printer data format specification",ptr);
    c_errmsg(msg," ",&nonfatal);
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer dpi configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (sscanf(ptr,"%d",&printer[cnt].dpi) != 1)
    {
    sprintf(msg,"%s is an invalid printer dpi specification",ptr);
    c_errmsg(msg," ",&nonfatal);
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer owner configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(owner,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer privilege configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(priv,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer media size configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (sscanf(ptr,"%fx%f",&printer[cnt].width,&printer[cnt].height) != 2)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer media size configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer media type configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if ((strcmp(ptr,"paper") == 0) ||
    (strcmp(ptr,"mylar") == 0))
    {
    strcpy(printer[cnt].mtype,ptr);
    }
else
    {
    sprintf(msg,"%s is an invalid media type",ptr);
    c_errmsg(msg," ",&nonfatal);
    }

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer file type configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(file_type,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer tray configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (strcmp(ptr," ") == 0)
    strcpy(printer[cnt].tray,"");
else
    strcpy(printer[cnt].tray,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer dependency configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (strcmp(ptr," ") == 0)
    strcpy(depend,"");
else
    strcpy(depend,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid label printer configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (strcmp(ptr," ") == 0)
    strcpy(printer[cnt].lblpr,"");
else
    strcpy(printer[cnt].lblpr,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer description configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (strcmp(ptr," ") == 0)
    strcpy(printer[cnt].desc,"");
else
    strcpy(printer[cnt].desc,ptr);

ptr = strtok(NULL,":");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer status configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
if (strcmp(ptr," ") == 0)
    strcpy(printer[cnt].status,"");
else
    strcpy(printer[cnt].status,ptr);

/*
* Parse owner string 
*/
ptr = strtok(owner,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer owner configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(printer[cnt].owner[0],ptr);
for (loccnt=1; loccnt<MAX_QLP_PARAM; loccnt++)
    {
    ptr = strtok(NULL,",");
    if (ptr == NULL)
	break;
    strcpy(printer[cnt].owner[loccnt],ptr);
    }

/*
* Parse privilege string 
*/
ptr = strtok(priv,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer privilege configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(printer[cnt].priv[0],ptr);
for (loccnt=1; loccnt<MAX_QLP_PARAM; loccnt++)
    {
    ptr = strtok(NULL,",");
    if (ptr == NULL)
	break;
    strcpy(printer[cnt].priv[loccnt],ptr);
    }

/*
* Parse file type string
*/
ptr = strchr(file_type,'\n');
if (ptr != NULL)
    *ptr = '\0';
ptr = strtok(file_type,",");
if (ptr == NULL)
    {
    rstat = E_FAIL;
    sprintf(msg,"Invalid printer file type configuration line <%s>",buf);
    c_errmsg(msg," ",&nonfatal);
    goto xit;
    }
strcpy(printer[cnt].ftype[0],ptr);
for (loccnt=1; loccnt<MAX_QLP_PARAM; loccnt++)
    {
    ptr = strtok(NULL,",");
    if (ptr == NULL)
	break;
    if ((strcmp(ptr,"las") == 0)  ||
	(strcmp(ptr,"ps") == 0)   ||
	(strcmp(ptr,"tif") == 0) ||
	(strcmp(ptr,"hpgl") == 0) ||
	(strcmp(ptr,"rtl") == 0))
	{
        strcpy(printer[cnt].ftype[loccnt],ptr);
	}
    else
	{
	sprintf(msg,"%s is an invalid file type",ptr);
	c_errmsg(msg," ",&nonfatal);
	}
    }

/*
* Parse printer dependency list
*/
if (strlen(depend) == 0)
    strcpy(printer[cnt].depend[0],"");
else
    {
    ptr = strtok(depend,",");
    if (ptr == NULL)
        {
        rstat = E_FAIL;
        sprintf(msg,"Invalid printer dependency configuration line <%s>",buf);
        c_errmsg(msg," ",&nonfatal);
        goto xit;
        }
    strcpy(printer[cnt].depend[0],ptr);
    for (loccnt=1; loccnt<MAX_QLP_PARAM; loccnt++)
        {
        ptr = strtok(NULL,",");
        if (ptr == NULL)
	    break;
        strcpy(printer[cnt].depend[loccnt],ptr);
        }
    strcpy(printer[cnt].depend[loccnt],"");
    }
xit:
return(rstat);
}

int qlp_ping
(
    char *host
)
{
int  rstat;
long status;
char syscmd[256];

rstat = E_SUCC;
#if defined(sgi)
    sprintf(syscmd,
        "/usr/etc/ping -c 1 %s | grep \"1 packets received\" > /dev/null",
        host);
    status = system(syscmd);
    if (status != 0)
        rstat = E_FAIL;
    rstat = E_SUCC;
#endif
#if defined(sun)
    sprintf(syscmd,"/usr/etc/ping %s | grep \"is alive\" > /dev/null",host);
    status = system(syscmd);
    if (status != 0)
        rstat = E_FAIL;
#endif
#if defined(DGUX)
    sprintf(syscmd,"/usr/bin/ping %s | grep \"is alive\" > /dev/null",host);
    status = system(syscmd);
    if (status != 0)
        rstat = E_FAIL;
#endif
#if defined(MULTINET)
    sprintf(syscmd,"multinet ping %s /number_of_packets=1",host);
#endif
#if defined(CMUTCP)
    sprintf(syscmd,"ipncp ping %s",host);
#endif
    return(rstat);
}

int qlp_setopt
(
    struct QLP_PRINTER *printer,
    struct recfmt *rec,
    struct IMGOPT *imgopt
)
{
long lsamp;			/* subsample required in line direction */
long nl;			/* number of lines */
long ns;			/* number of samples */
long norot_line;		/* # lines without rotating */
long norot_reduce;		/* subsample required without rotating */
long norot_samp;		/* # samples without rotating */
long norot_npanel;		/* # panels without rotating */
long rot_line;			/* # lines with rotating */
long rot_reduce;		/* subsample required with rotating */
long rot_samp;			/* # samples with rotating */
long rot_npanel;		/* # panels with rotating */
long ssamp;			/* subsample requried in sample direction */
long val;			/* temp val */

imgopt->annot = (strcmp(rec->aflg,"YES") == 0) ? TRUE : FALSE;
if (strlen(rec->annot) > 0)
    imgopt->annot = TRUE;
imgopt->spot = (strcmp(rec->spot,"YES") == 0) ? TRUE : FALSE;
imgopt->wedge = (strcmp(rec->wedge,"YES") == 0) ? TRUE : FALSE;
imgopt->stretch = (strcmp(rec->stropt,"YES") == 0) ? TRUE : FALSE;
imgopt->title = (strlen(rec->title) > 0) ? TRUE : FALSE;
imgopt->rotate = (strcmp(rec->rotopt,"YES") == 0) ? TRUE : FALSE;
if (strcmp(rec->color,"RGB") == 0)
    {
    imgopt->rgb2cmy = TRUE;
    imgopt->outnb = 3;
    }
else if (strcmp(rec->color,"BW") == 0)
    {
    imgopt->rgb2cmy = TRUE;
    imgopt->outnb = 1;
    }
else if (strcmp(rec->color,"CMY") == 0)
    {
    imgopt->rgb2cmy = FALSE;
    imgopt->outnb = 3;
    }
else if (strcmp(rec->color,"CMYK") == 0)
    {
    imgopt->rgb2cmy = FALSE;
    imgopt->outnb = 4;
    }
imgopt->expand = (rec->expand[0] != NULLVAL) ? TRUE : FALSE;
imgopt->reduce = (rec->reduce[0] != NULLVAL) ? TRUE : FALSE;
imgopt->maxnl = (long)((float)printer->dpi * printer->height + .5F);
imgopt->maxns = (long)((float)printer->dpi * printer->width + .5F);
/* 
*  Less lines available if annotation is to be inserted
*/
imgopt->fontnl = 0;
if (imgopt->annot)
    imgopt->fontnl += FONT_NL;
if (imgopt->stretch)
    imgopt->fontnl += FONT_NL;
if ((imgopt->spot) || (imgopt->wedge))
    imgopt->fontnl += (FONT_NL + WEDSPOT_ZL);
if (imgopt->title)
    imgopt->fontnl += (FONT_NL * 2);
/*
*  If auto size is set to yes, let the printer resize the image.  
*/
if (strcmp(rec->autosz,"YES") == 0)
    {
    imgopt->sl = imgopt->ss = 0;
    if ((rec->nl > imgopt->maxnl) || (rec->ns > imgopt->maxns))
	{
        lsamp = rec->nl / imgopt->maxnl;
        if (rec->nl % imgopt->maxnl != 0)
            ++lsamp;
	--lsamp;
        ssamp = rec->ns / imgopt->maxns;
        if (rec->ns % imgopt->maxns != 0)
            ++ssamp;
	--ssamp;
	imgopt->reduce = TRUE;
	rec->reduce[0] = rec->reduce[1] = (lsamp > ssamp) ? lsamp : ssamp;
        imgopt->nl = imgopt->maxnl = 
		(rec->nl / rec->reduce[0] + 1) + imgopt->fontnl;
        imgopt->ns = imgopt->maxns = (rec->ns / rec->reduce[1] + 1);;
        imgopt->fontnl = 0;
        rec->npanel = 1;
	}
    else
	{
        lsamp = imgopt->maxnl / rec->nl;
        ssamp = imgopt->maxns / rec->ns;
	rec->expand[0] = rec->expand[1] = (lsamp < ssamp) ? lsamp : ssamp;
	if (rec->expand[0] > 1)
	    imgopt->expand = TRUE;
        imgopt->nl = imgopt->maxnl = 
		(rec->nl * rec->expand[0]) + imgopt->fontnl;
        imgopt->ns = imgopt->maxns = (rec->ns * rec->expand[0]);
        imgopt->fontnl = 0;
        rec->npanel = 1;
	}
    }
else
    {
/* 
*  Less lines available if annotation is to be inserted
*/
    imgopt->fontnl = 0;
    if (imgopt->annot)
        imgopt->fontnl += FONT_NL;
    if (imgopt->stretch)
        imgopt->fontnl += FONT_NL;
    if ((imgopt->spot) || (imgopt->wedge))
        imgopt->fontnl += (FONT_NL + WEDSPOT_ZL);
    if (imgopt->title)
        imgopt->fontnl += (FONT_NL * 2);
    if ((!imgopt->expand) && (!imgopt->reduce))	/* default is to subsample */
        {					/* image to fit on one frame */
/* 
*	Compute subsample factor required if image is not rotated.
*/
        lsamp = rec->nl / (imgopt->maxnl - imgopt->fontnl);
        if (rec->nl % (imgopt->maxnl - imgopt->fontnl) != 0)
            ++lsamp;
        ssamp = rec->ns / imgopt->maxns;
        if (rec->ns % imgopt->maxns != 0)
            ++ssamp;
        norot_reduce = (lsamp > ssamp) ? lsamp : ssamp;
/* 
*	Compute subsample factor required if image is rotated.
*/
        lsamp = rec->ns / (imgopt->maxnl - imgopt->fontnl);
        if (rec->ns % (imgopt->maxnl - imgopt->fontnl) != 0)
            ++lsamp;
        ssamp = rec->nl / imgopt->maxns;
        if (rec->nl % imgopt->maxns != 0)
            ++ssamp;
        rot_reduce = (lsamp > ssamp) ? lsamp : ssamp;
/*
*	Set the subsample factor such that the image fits on one frame
*	of film.  If rotate option is set to AUTO, choose the one that
*	results in the least amount of image reduction.
*/
        if (strcmp(rec->rotopt,"AUTO") == 0) 
	    imgopt->rotate = (norot_reduce <= rot_reduce) ? FALSE : TRUE;
        if (!imgopt->rotate)
	    {
	    if (norot_reduce > 1)
	        {
	        rec->reduce[0] = rec->reduce[1] = norot_reduce;
	        imgopt->reduce = TRUE;
	        }
	    }
        if (imgopt->rotate)
	    {
	    if (rot_reduce > 1)
	        {
	        rec->reduce[0] = rec->reduce[1] = rot_reduce;
	        imgopt->reduce = TRUE;
	        }
	    }
        }
    if (imgopt->reduce)
        {
        nl = rec->nl / rec->reduce[0];
        if (rec->nl % rec->reduce[0] != 0)
	    ++nl;
        ns = rec->ns / rec->reduce[1];
        if (rec->ns % rec->reduce[1] != 0)
	    ++ns;
        }
    else if (imgopt->expand)
        {
        nl = rec->nl * rec->expand[0];
        ns = rec->ns * rec->expand[1];
        }
    else
        {
        nl = rec->nl;
        ns = rec->ns;
        }
/*
*  number of panels without rotate 
*/
    norot_line = (nl + imgopt->fontnl) / imgopt->maxnl;
    if ((nl + imgopt->fontnl) % imgopt->maxnl != 0)
        ++norot_line;
    norot_samp = ns / imgopt->maxns;
    if (ns % imgopt->maxns != 0)
        ++norot_samp;
    norot_npanel = norot_line * norot_samp;
/*
*  number of panels with rotate
*/
    rot_line = (ns + imgopt->fontnl) / imgopt->maxnl;
    if ((ns + imgopt->fontnl) % imgopt->maxnl != 0)
        ++rot_line;
    rot_samp = nl / imgopt->maxns;
    if (nl % imgopt->maxns != 0)
        ++rot_samp;
    rot_npanel = rot_line * rot_samp;
    
    if ((strcmp(rec->rotopt,"AUTO") == 0) && (rot_npanel < norot_npanel))
        imgopt->rotate = TRUE;
    rec->npanel = (imgopt->rotate) ? rot_npanel : norot_npanel;
    if (imgopt->rotate)
	{
	val = imgopt->maxnl;
	imgopt->maxnl = imgopt->maxns;
	imgopt->maxns = val;
	}
    imgopt->maxnl -= imgopt->fontnl;
    imgopt->nl = nl;
    imgopt->ns = ns;
/*
*  Center the image if one panel. If printer is not a tektronix phaser 
*  printer, then center to the nearest inch (otherwise printers such as 
*  the HP755 would waste too much media).
*/
    if (rec->npanel == 1)
        {
        if (strncmp(printer->type,"phaser",6) != 0)
	    {
	    val = (imgopt->nl / printer->dpi) + 1;
	    imgopt->maxnl = printer->dpi * val;
	    val = (imgopt->ns / printer->dpi) + 1;
	    imgopt->maxns = printer->dpi * val;
	    }
        imgopt->sl = (imgopt->maxnl - imgopt->nl) / 2;
        imgopt->ss = (imgopt->maxns - imgopt->ns) / 2;
        }
/*
*  Else place image in upper left corner 
*/
    else
        {
        imgopt->sl = imgopt->ss = 0;
        if (imgopt->nl > imgopt->maxnl)
            imgopt->nl = imgopt->maxnl;
        if (imgopt->ns > imgopt->maxns)
            imgopt->ns = imgopt->maxns;
        if (strncmp(printer->type,"phaser",6) != 0)
	    {
	    imgopt->maxnl = imgopt->nl;
	    imgopt->maxns = imgopt->ns;
	    }
	}
    }
return(E_SUCC);
}

int qlp_logit
(
    struct recfmt *rec,
    struct IMGOPT *imgopt
)
{
struct tm *tptr;		/* time structure */
FILE *fp;
long crflg;
long t;
char fname[256];
char *logdir;
char timestr[256];

t = time(NULL);
tptr = localtime(&imgopt->time_elapse);
sprintf(timestr,"%02d/%02d/%02d %02d:%02d",tptr->tm_mon+1,tptr->tm_mday,
	(1900 + tptr->tm_year) % 100,tptr->tm_hour,tptr->tm_min);
logdir = getenv("QLPLOG");
if (logdir == NULL)
    {
    c_errmsg("QLPLOG environment variable not set"," ",&nonfatal);
    return(0);
    }
sprintf(fname,"%s/qlplog.%02d%02d",logdir,(1900 + tptr->tm_year) % 100,
	tptr->tm_mon+1);
crflg = (access(fname,0) == -1) ? TRUE : FALSE;
if (crflg)
    {
    fp = fopen(fname,"w");
    if (fp == NULL)
	return(0);
    fprintf(fp,"%s%s\n",
	"Date     Start Printer        System    User     Charge   EWR    ",
	"Panel  #Copy");
    chmod(fname,0777);
    }
else
    fp = fopen(fname,"a");
if (fp == NULL)
    {
    perror("qlp_logit");
    c_errmsg("Error opening log file"," ",&nonfatal);
    return(0);
    }
fprintf(fp,
   "%s %-14.14s %-9.9s %-8.8s %-8.8s %06d %02d/%02d  %02d/%02d\n",
   timestr,rec->printer,rec->remote,rec->usercode,rec->chrgcod,rec->ewr,
   rec->curpan,rec->npanel,rec->curcop,rec->ncop);
fclose(fp);
imgopt->time_elapse = t;		/* reset starting elapse time */
return(1);
}
