/******************************************************************************

FUNCTION:  RDLUT

PURPOSE:
	RDLUT reads the map points of a specified LUT file.

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "ntqlp.h"
#include "qlp.h"
#include "local_protos.h"

#ifdef vms
#define DIRSPEC "CFLUT:"
#endif

#ifdef unix
#define DIRSPEC ""
#endif

static int valenh
(
    long nlut,
    struct lut enhlut[]
);

static long nonfatal = 1;

int rdlut
(
    char *colorord,	/* (I) color order (RGB, CMY, CMYK, or BW) */
    int fnmr,		/* (I) number of LUT to be opened */
    char *fname,		/* (I) name of LUT to be opened (if fnmr=NULLVAL) */
    char ftype[],	/* (I) LUT file type (ENH, CAL, or CON) */
    struct lut lut[]	/* (O) structure to contain LUT map points */
)
{
long len;
long nlut;
long status,i;
long pt;
long nonfatal = 1;
long rstat;
char name[CMLEN],msg[CMLEN];
char temp[CMLEN];
char *ptr, *ptr1;
char *color_order;
FILE *chan;

rstat = E_SUCC;
color_order = colorord;
if (strcmp(colorord,"RGB") == 0)
    nlut = 3;
else if (strcmp(colorord,"CMY") == 0)
    nlut = 3;
else if (strcmp(colorord,"CMYK") == 0)
    nlut = 4;
else if (strcmp(colorord,"BW") == 0)
    {
    nlut = 3;
    color_order = "RGB";
    }
for (i=0; i<nlut; i++)
    {
    if (fnmr != NULLVAL)
        sprintf(name,"%s%d.%.2s%c",DIRSPEC,fnmr,ftype,*(color_order + i));
    else 
	{
	ptr = strrchr(fname,'.');
	ptr1 = strrchr(fname,']');
        if (((ptr != NULL) && (ptr1 == NULL)) ||
	    ((ptr != NULL) && (ptr1 != NULL) && (ptr1 < ptr)))
	    {
	    *ptr = '\0';
	    }
        sprintf(name,"%s.%.2s%c",fname,ftype,*(color_order + i));
	}
    len = (long)strlen(name);
    c_up2low(name,&len);
    status = access(name,4);
    if (status != 0) 
        {
        sprintf(msg,"LUT File %s does not exist",name);
        c_errmsg(msg," ",&nonfatal);
	if (strcmp(ftype,"ENH") == 0)
	    {
	    sleep(1);
	    rstat = E_FAIL;
	    }
	else if (strcmp(ftype,"CAL") == 0)
	    {
	    lut[i].ne = 2;
	    lut[i].in[0] = 0;
	    lut[i].in[1] = 255;
	    lut[i].out[0] = .001;
	    lut[i].out[1] = 2.0;
	    }
	else if (strcmp(ftype,"CON") == 0)
	    {
	    lut[i].ne = 2;
	    lut[i].in[0] = 0;
	    lut[i].in[1] = 255;
	    lut[i].out[0] = 2.0;
	    lut[i].out[1] = .001;
	    }
	continue;
	}
    chan = fopen(name,"r");
    if (chan == NULL)
        {
        sprintf(msg,"Unable to open LUT File %s",name);
        c_errmsg(msg," ",&nonfatal);
	sleep(1);
	rstat = E_FAIL;
	continue;
	}
    lut[i].ne = 0;
    lut[i].desc[0] = '\0';
    fgets(lut[i].desc,80,chan);
    status = EOF + 1;
    for (pt=0;(status != EOF) && (pt<=MAXPT);pt++)
        status = fscanf(chan,"%d %f",&(lut[i].in[pt]),&(lut[i].out[pt]));
    if (status != EOF)
        {
        sprintf(msg,"ERROR -- LUT File %s is corrupt",name);
        c_errmsg(msg," ",&nonfatal);
	if (pt >= MAXPT)
	    {
            sprintf(msg,"LUT File is limited to %d map points",MAXPT);
            c_errmsg(msg," ",&nonfatal);
	    }
	sleep(1);
	lut[i].ne = pt - 1;
	fclose(chan);
	rstat = E_FAIL;
	continue;
        }
    lut[i].ne = pt - 1;
    fclose(chan);
    }
strcpy(temp,ftype);
len = (long)strlen(temp);
c_up2low(temp,&len);
if ((rstat == E_SUCC) && (strcmp(temp,"enh") == 0))
    {
    rstat = valenh(nlut,lut);
    }
return(rstat);
}

static int valenh
(
    long nlut,
    struct lut enhlut[]
)
{
long cnt;
long i;
char msg[100];

for (cnt=0; cnt<nlut; cnt++)
    {
    for (i=0;i<enhlut[cnt].ne;i++)
        {
        if ((i > 0) && (enhlut[cnt].in[i] < enhlut[cnt].in[i-1]))
            {
	    sprintf(msg,"ERROR: LUT %d %s",cnt+1,
			"enhance file input values are not ascending");
	    c_errmsg(msg," ",&nonfatal);
	    return(E_FAIL);
            }
        if ((i > 0) && (enhlut[cnt].in[i] == enhlut[cnt].in[i-1]))
            {
	    sprintf(msg,"ERROR: LUT %d %s",cnt+1,
              		"enhance file has two equal input brightness values");
	    c_errmsg(msg," ",&nonfatal);
	    return(E_FAIL);
            }
        }
    if (enhlut[cnt].in[enhlut[cnt].ne-1] != 255)
        {
	if (enhlut[cnt].ne + 1 >= MAXPT)
	    {
	    sprintf(msg,"ERROR: LUT %d %s",cnt+1,
              	"enhance file is required to have 255 as an input map point");
	    c_errmsg(msg," ",&nonfatal);
	    return(E_FAIL);
	    }
	enhlut[cnt].in[enhlut[cnt].ne] = 255;
	enhlut[cnt].out[enhlut[cnt].ne] = 255;
	++enhlut[cnt].ne;
	}
    }
return(E_SUCC);
}
