/******************************************************************************

SUBROUTINE:	rparm

PURPOSE: 
	Subroutine rparm will receive the parameter values from the V-block,
	and return the values in a record format in the global variable "rec".

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   2.1		 3/99	    K. Gacke	allow all users to set priority 1-9
   2.0           3/97       KGacke      qlp/qlpmgr unix command line option
   1.1		 7/93	    K. Gacke	Add submit date field for reports
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

LANGUAGE	C

******************************************************************************/


#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include "las.h"	
#include "util.h"
#include "tiffio.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"

static int chkimg
(
    long tae,
    struct PARBLK *blkptr,
    struct recfmt *rec
);

int qlp_unix_add
(
    int argc,               /* argument count */
    char *argv[],           /* argument vector */
    struct recfmt *rec,     /* QLP record block */
    long prog               /* calling program: PADD, PMOD, or REPORT   */
);



int rparm
(
    int argc,           /* argument count */
    char *argv[],       /* argument vector */
    long tae,           /* TAE flag, TRUE=>valid param block */
    struct PARBLK *blkptr,  /* TAE param block */
    struct QLP_PRINTER printer[],/* printer definition block */
    long pcnt,              /* number of printers defined */
    struct recfmt *rec,     /* QLP record block */
    long prog               /* calling program: PADD, PMOD, or REPORT   */
)
{
struct VARIABLE *val;
struct IMGOPT imgopt;
struct lut tlut[NLUT];
struct tm *tptr;
time_t  t;
long    suflg;
long    rwflg;
long	cnt;
long	len;
long	i, j;
long	elem;
long	fatal;
long	nonfatal;
long 	enhflg;	
long	maxchar;
long	printer_cnt;
long 	priority;
int 	rstat;
long	status;
int 	ival;
char 	hname[CMLEN];
char 	host[CMLEN];
char	msg[256],temp[10];
char    name[CMLEN];
char	*ptr;
char	wd[256];

rstat = E_SUCC;
nonfatal = 1;
fatal = -1;
if ((prog == PADD) || (prog == PMOD))
    {
    t = time(NULL);
    tptr = localtime(&t);
    sprintf(rec->submit_date,"%02d/%02d/%02d %02d:%02d",
        tptr->tm_mon+1,tptr->tm_mday,(1900 + tptr->tm_year) % 100,
		tptr->tm_hour,tptr->tm_min);
    getchrg(rec->chrgcod);      /* set charge code */
    }
if (prog == PADD)
    {
    getuser(rec->usercode);  	/* set user code */
    rec->nruns = 0;	  	/* set number of runs equal to zero */
    rec->curcop = 0;
    rec->curpan = 0;
    rec->npanel = 1;
    getmachine(rec->remote);
    strcpy(host,rec->remote);
    strcpy(rec->stflg,QLP_QUEUE);
    rec->errtext[0] = '\0';
    }
if (prog == PMOD)
    {
    strcpy(name,rec->fspec);
    getmachine(host);
    strcpy(rec->stflg,QLP_QUEUE);
    }
/*
*  If not running from tae, retrieve the unix command line arguments
*/
if (!tae)
    {
    rstat = qlp_unix_add(argc,argv,rec,prog);
    if (rstat != E_SUCC)
	goto xit;
    }
/* 
*  If valid tae parameter block, call the function of p_find for each 
*  parameter value that is to be read from the V-block.  The function will 
*  return a pointer of type VARIABLE which points to the value of the 
*  parameter. 
*/
else if (tae)
    {
    val = p_find(blkptr,"IN");
    if (val->v_count == 0)
        *rec->fspec = '\0';
    else
        strcpy(rec->fspec,SVAL(*val,0));
    
    val = p_find(blkptr,"EWRNBR");	
    if (val->v_count == 0)
        rec->ewr = NULLVAL;
    else
        rec->ewr = IVAL(*val,0);
    
    val = p_find(blkptr,"FTYPE");
    if (val->v_count == 0)
        *rec->ftype = '\0';
    else
        strcpy(rec->ftype,SVAL(*val,0));
    len = (long)strlen(rec->ftype);
    c_up2low(rec->ftype,&len);

    val = p_find(blkptr,"PTYPE");
    if (val->v_count == 0)
        *rec->ptype = '\0';
    else
        strcpy(rec->ptype,SVAL(*val,0));
    len = (long)strlen(rec->ptype);
    c_up2low(rec->ptype,&len);
    
    val = p_find(blkptr,"PRINTER");
    if (val->v_count == 0)
        *rec->printer = '\0';
    else
        strcpy(rec->printer,SVAL(*val,0));
    len = (long)strlen(rec->printer);
    c_up2low(rec->printer,&len);
        
    val = p_find(blkptr,"NCOPIES");	
    if (val->v_count == 0)
        rec->ncop = NULLVAL;
    else
        rec->ncop = IVAL(*val,0);
        
    priority = rec->prio;
    val = p_find(blkptr,"PRIORITY");	
    if (val->v_count == 0)
        rec->prio = NULLVAL;
    else
        rec->prio = IVAL(*val,0);
        
    val = p_find(blkptr,"DELFLG");
    if (val->v_count == 0)
        *rec->delfil = '\0';
    else
        strcpy(rec->delfil,SVAL(*val,0));
    if ((strcmp(rec->ftype,"las")==0) || (strcmp(rec->ftype,"tif")==0))
        {
        val = p_find(blkptr,"BACKGRND");	
        if (val->v_count == 0)
            rec->bgrnd = NULLVAL;
        else
            rec->bgrnd = IVAL(*val,0);
        
        val = p_find(blkptr,"EXPAND");	
        if (val->v_count == 0)
            rec->expand[0] = rec->expand[1] = NULLVAL;
        else if (val->v_count == 1)
            rec->expand[0] = rec->expand[1] = IVAL(*val,0);
        else
            {
            rec->expand[0] = IVAL(*val,0);
            rec->expand[1] = IVAL(*val,1);
            }
        
        val = p_find(blkptr,"REDUCE");	
        if (val->v_count == 0)
            rec->reduce[0] = rec->reduce[1] = NULLVAL;
        else if (val->v_count == 1)
            rec->reduce[0] = rec->reduce[1] = IVAL(*val,0);
        else
            {
            rec->reduce[0] = IVAL(*val,0);
            rec->reduce[1] = IVAL(*val,1);
            }
        
        val = p_find(blkptr,"SPANEL");	
        if (val->v_count == 0)
            rec->curpan = 1;
        else
            rec->curpan = IVAL(*val,0);
        
        val = p_find(blkptr,"ANNOT");
        if (val->v_count == 0)
            *rec->annot = '\0';
        else
            strcpy(rec->annot,SVAL(*val,0));
        
        val = p_find(blkptr,"AUTOSZ");
        if (val->v_count == 0)
            *rec->autosz = '\0';
        else
            strcpy(rec->autosz,SVAL(*val,0));
        
        val = p_find(blkptr,"COLORORD");
        if (val->v_count == 0)
            *rec->color = '\0';
        else
            strcpy(rec->color,SVAL(*val,0));
        
        val = p_find(blkptr,"INDLTF");
        if (val->v_count == 0)
            *rec->dltf = '\0';
        else
            strcpy(rec->dltf,SVAL(*val,0));
        
        val = p_find(blkptr,"INFILE");
        if (val->v_count == 0)
            *rec->infile = '\0';
        else
            strcpy(rec->infile,SVAL(*val,0));
        
        val = p_find(blkptr,"ROTOPT");
        if (val->v_count == 0)
            *rec->rotopt = '\0';
        else
            strcpy(rec->rotopt,SVAL(*val,0));
        
        val = p_find(blkptr,"SPOTCOPY");
        if (val->v_count == 0)
            *rec->spot = '\0';
        else
            strcpy(rec->spot,SVAL(*val,0));
        
        val = p_find(blkptr,"AFLG");
        if (val->v_count == 0)
            *rec->aflg = '\0';
        else
            strcpy(rec->aflg,SVAL(*val,0));
        
        val = p_find(blkptr,"TITLE");
        if (val->v_count == 0)
            *rec->title = '\0';
        else
            strcpy(rec->title,SVAL(*val,0));
        
        val = p_find(blkptr,"WEDGE");
        if (val->v_count == 0)
            *rec->wedge = '\0';
        else
            strcpy(rec->wedge,SVAL(*val,0));
        
        val = p_find(blkptr,"SPECIAL");
        if (val->v_count == 0)
            *rec->spinst = '\0';
        else
            strcpy(rec->spinst,SVAL(*val,0));
        val = p_find(blkptr,"STRETCH");
        if (val->v_count == 0)
            *rec->stropt = '\0';
        else
            strcpy(rec->stropt,SVAL(*val,0));
        }
    }
/* 
*	Check priority value
*/
suflg = qlp_chkuser(printer,pcnt,rec);
if (rec->prio != NULLVAL)
    {
    if ((prog == PMOD) && (suflg < SU_PRIV) && 
	(rec->prio < 1) && (priority > rec->prio))
	{
	c_errmsg("User not allowed to set priority less than 1",
		    " ",&nonfatal);
	sprintf(msg,"Priority has been reset to original value of %d",priority);
	c_errmsg(msg," ",&nonfatal);
	rec->prio = priority;
	}
    if ((prog == PADD) && (suflg < SU_PRIV) && (rec->prio < 1))
	{
	c_errmsg("User not allowed to set priority less than 1",
		    " ",&nonfatal);
	c_errmsg("Priority has been reset to a value of 1"," ",&nonfatal);
	rec->prio = 1;
	}
    }
if (strlen(rec->printer) == 0)    	/* printer not specified, search for */
    {					/* appropriate printer type */
    for (printer_cnt=0; printer_cnt<pcnt; printer_cnt++)
	{
	if (strcmp(rec->ptype,printer[printer_cnt].type) == 0)
	    break;
	}
    if (printer_cnt >= pcnt)
	{
	sprintf(msg,"Printer type %s not found in configuration",rec->ptype);
	c_errmsg(msg," ",&fatal);
	}
/*
*	Check file type compatability with printer 
*/
    for (cnt=0 ;; cnt++)
        {
        if (strlen(printer[printer_cnt].ftype[cnt]) == 0)
	    {
	    sprintf(msg,"Printer type %s does not support file type %s",
		        rec->ptype,rec->ftype);
	    c_errmsg(msg," ",&fatal);
	    }
        if (strcmp(rec->ftype,printer[printer_cnt].ftype[cnt]) == 0)
	    break;
	if (((strcmp(rec->ftype,"las")==0) || (strcmp(rec->ftype,"tif")==0)) &&
	    (strcmp("ps",printer[printer_cnt].ftype[cnt]) == 0))
	    break;
        }
    }
else					/* printer specified, search for */
    {					/* printer name in configuration */
    for (printer_cnt=0; printer_cnt<pcnt; printer_cnt++)
	{
	if (strcmp(rec->printer,printer[printer_cnt].lname) == 0)
	    break;
	}
    if (printer_cnt >= pcnt)
	{
	sprintf(msg,"Printer %s not found in configuration",rec->printer);
	c_errmsg(msg," ",&fatal);
	}
    strcpy(rec->ptype,printer[printer_cnt].type);
/*
*	Check file type compatability with printer 
*/
    for (cnt=0 ;; cnt++)
        {
        if (strlen(printer[printer_cnt].ftype[cnt]) == 0)
	    {
	    sprintf(msg,"Printer %s does not support file type %s",
		        rec->printer,rec->ftype);
	    c_errmsg(msg," ",&fatal);
	    }
        if (strcmp(rec->ftype,printer[printer_cnt].ftype[cnt]) == 0)
	    break;
	if (((strcmp(rec->ftype,"las")==0) || (strcmp(rec->ftype,"tif")==0)) &&
	    (strcmp("ps",printer[printer_cnt].ftype[cnt]) == 0))
	    break;
        }
    }

if ((strcmp(rec->ftype,"las") != 0) && (strcmp(rec->ftype,"tif") != 0))
    {
    if (prog == PMOD) 
        {
        if (strcmp(name,rec->fspec) != 0)
	    {
            if (strcmp(host,rec->remote) != 0)
                {
                sprintf(msg,
                    "ERROR: File %s is on %s. Cannot modify from remote system",
                    rec->fspec,rec->remote);
                c_errmsg(msg," ",&nonfatal);
                rstat = E_FAIL;
                goto xit;
                }
	    }
	}
    if (strcmp(host,rec->remote) == 0)
	{
        if (access(rec->fspec,F_OK) != 0)
	    {
	    sprintf(msg,"File %s does not exist",rec->fspec);
	    c_errmsg(msg," ",&fatal);
	    }
        if (access(rec->fspec,R_OK) != 0)
	    {
	    sprintf(msg,"No read permission for file %s",rec->fspec);
	    c_errmsg(msg," ",&fatal);
	    }
        if (rec->fspec[0] != '/')
	    {
	    if (getwd(wd) == NULL)
	        c_errmsg("Error retrieving working directory"," ",&fatal);
	    strcpy(msg,rec->fspec);
	    sprintf(rec->fspec,"%s/%s",wd,msg);
	    }
	}
    }
else
    {
    if (prog == PADD) 
        {
        if (chkimg(tae,blkptr,rec) != E_SUCC)
	    {
	    rstat = E_FAIL;
	    goto xit;
	    }
        rstat = qlp_setopt(&printer[printer_cnt],rec,&imgopt);
        if (rstat != E_SUCC)
            goto xit;
        }
    else if (prog == PMOD) 
        {
        if (strcmp(name,rec->fspec) != 0)
	    {
            if (strcmp(host,rec->remote) != 0)
                {
                sprintf(msg,
                    "ERROR: File %s is on %s. Cannot modify from remote system",
                    rec->fspec,rec->remote);
                c_errmsg(msg," ",&nonfatal);
                rstat = E_FAIL;
                goto xit;
                }
            if (chkimg(tae,blkptr,rec) != E_SUCC)
	        {
	        rstat = E_FAIL;
	        goto xit;
	        }
	    }
        rstat = qlp_setopt(&printer[printer_cnt],rec,&imgopt);
        if (rstat != E_SUCC)
            goto xit;
        }
    len = (long)strlen(rec->infile);
    c_up2low(rec->infile,&len);
    len = (long)strlen(rec->dltf);
    c_up2low(rec->dltf,&len);
    if (prog != PMOD)
        {
        for (cnt=0; cnt<NLUT; cnt++)
	    {
	    rec->enhlut[cnt].ne = 0;
	    *rec->enhlut[cnt].desc = '\0';
	    }
        }
    cplut(rec,tlut,FR_REC);
    switch (prog)
        {
        case PREP:   enhflg = FALSE;	break;
        case PADD:   enhflg = TRUE;	break;
        case PMOD:   enhflg = FALSE;	break;
        }
    if (prog == PMOD)
        {
        c_errmsg("Enh map points will not be modified"," ",&nonfatal);
        }
    else if ((prog != PREP) && (strlen(rec->infile) == 0))
        {
        if (chkbat(tae,blkptr))
	    {
	    c_errmsg("INFILE cannot be set to NULL if running in batch",
		    "fire",&fatal);
	    }
        setenh(rec->color,NULLVAL,"",tlut,enhflg);
        }
    else if ((prog != PREP) && (strlen(rec->infile) > 0))
        {
        if (strlen(rec->dltf) == 0)	/* read LUT data from ASCII file */
            {
            status = c_cmhost(rec->infile,hname);
	    if (status != E_SUCC)
	        {
	        c_errmsg("Error translating LUT name into a host file name",
		    " ",&fatal);
	        }
            status = rdlut(rec->color,NULLVAL,hname,"ENH",tlut);
            if (status != E_SUCC)
	        {
                sprintf(msg,"Error setting ENH lut values from file %s",
			    rec->infile);
	        c_errmsg(msg," ",&fatal);
	        }
            }
        else				/* read LUT data from DLTF file */
            {
	    if (strlen(rec->infile) == 0)
	        {
	        ptr = strchr(rec->fspec,'(');	/* strip off image options */
	        if (ptr != NULL)
		    *ptr = '\0';
	        sprintf(rec->infile,"%s;dltf",rec->fspec);
	        if (ptr != NULL)
		    *ptr = '(';
                status = c_cmhost(rec->infile,hname);
	        if (status != E_SUCC)
	            {
	            c_errmsg("Error converting DLTF TAE name to host name",
		        " ",&fatal);
	            }
	        }
	    else
	        {
	        elem = 0;
	        rwflg = IREAD;
	        status = c_getnam(blkptr,"INFILE",&elem,hname,"dltf",&rwflg);
	        if (status != E_SUCC)
	            {
                    sprintf(msg,"Error accessing DLTF file %s",rec->infile);
	            c_errmsg(msg," ",&fatal);
	            }
                strcpy(rec->infile, hname);
	        }
            status = rddltf(rec->color,hname,rec->dltf,tlut);
            if (status != E_SUCC)
	        {
                sprintf(msg,
		    "Error setting ENH lut values from file %s entry %s",
		    rec->infile,rec->dltf);
	        c_errmsg(msg," ",&fatal);
	        }
            }
        }
    cplut(rec,tlut,TO_REC);
    for (cnt=0; cnt<NLUT; cnt++)	/* Null out stretch annotation */
        *rec->stretch[cnt] = '\0';
    if (strcmp(rec->stropt,"YES") == 0)
        {
	maxchar = (imgopt.maxns / FONT_NS) / rec->nb - 4;
        for (i=0; i<rec->nb; i++)
            {
	    sprintf(rec->stretch[i],"B%d: ",i+1);
            for (j=0; j<rec->enhlut[i].ne; j++)
                {
                ival = (int) rec->enhlut[i].out[j];
                if (j == 0)
                    sprintf(temp,"%d-%d",rec->enhlut[i].in[j],ival);
                else
                    sprintf(temp,",%d-%d",rec->enhlut[i].in[j],ival);
                if (strlen(rec->stretch[i]) + strlen(temp) > maxchar)
                    {
                    c_errmsg("Stretch point annotation is too long",
			    " ",&nonfatal);
                    break;
                    }
                strcat(rec->stretch[i],temp);
                }
	    strcat(rec->stretch[i],"    ");
            }
        }
    if (strcmp(rec->autosz,"YES") == 0)
        {
        sprintf(msg,"%s%s","AUTOSZ set to YES.  Implies parameters EXPAND, ",
	    "REDUCE, and ROTOPT ignored");
        c_errmsg(msg," ",&nonfatal);
        rec->expand[0] = rec->expand[1] = NULLVAL;
        rec->reduce[0] = rec->reduce[1] = NULLVAL;
        strcpy(rec->rotopt,"NO");
/******
        *rec->annot = '\0';
        strcpy(rec->spot,"NO");
        *rec->title = '\0';
        strcpy(rec->wedge,"NO");
        strcpy(rec->stropt,"NO");
        for (cnt=0; cnt<NLUT; cnt++)
            *rec->stretch[cnt] = '\0';
*******/
        }
    }
xit:
return(rstat);
}	/*  END of the RPARM subroutine  */

static int chkimg
(
    long tae,
    struct PARBLK *blkptr,
    struct recfmt *rec
)
{
long cnt;
long cnt1;
long elem;
long imglimit;
long nbands[NMRIMG];
long nonfatal;
long qflag[NMRIMG][2];
int  rstat;
long same_window;
long window[NMRIMG][4];
char emsg[256];
char host[CMLEN];
char hnames[NMRIMG][CMLEN];
char tnames[NMRIMG][CMLEN];
TIFF *tif;
uint32 nl, ns;
uint16 nb;


rstat = E_SUCC;
getmachine(host);
if (strcmp(host,rec->remote) != 0)
    {
    sprintf(emsg,"File is on remote system %s, unable to verify file",
        rec->remote);
    c_errmsg(emsg," ",&nonfatal);
    goto xit;
    }
nonfatal = 1;
elem = 0;
imglimit = NMRIMG;
if (strcmp(rec->ftype,"las") == 0)
    {
    if (tae)
        {
        rstat = c_getin(blkptr,"IN",&elem,tnames,hnames,&rec->nimg,window,
	    rec->bands, nbands,&rec->nb,&imglimit,&same_window,qflag);
        }
    else
        {
        rstat = c_getin2(rec->fspec,tnames,hnames,&rec->nimg,window,
	    rec->bands, nbands,&rec->nb,&imglimit,&same_window,qflag);
        if (rstat != E_SUCC)
	    c_errmsg(emsg," ",&nonfatal);
        }
    }
else if (strcmp(rec->ftype,"tif") == 0)
    {
    rec->nimg = 0;
    strcpy(rec->tae[0], rec->fspec);
    c_cmhost(rec->tae[0],rec->fspec);
    tif = TIFFOpen(rec->fspec,"r");
    if (!tif)
        {
        sprintf(emsg,"Error opening TIFF file %s",rec->fspec);
        c_errmsg(emsg," ",&nonfatal);
	rstat = E_FAIL;
	goto xit;
        }
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &nl);
    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH,  &ns);
    TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &nb);
    window[0][SL] = 1;
    window[0][SS] = 1;
    window[0][NL] = (long)nl;
    window[0][NS] = (long)ns;
    rec->nb = nb;
    }
if (rstat == E_SUCC)
    {
    for (cnt=0; (cnt<rec->nimg-1) && (rstat==E_SUCC); cnt++)
        {
	for (cnt1=0; (cnt1<4) && (rstat==E_SUCC); cnt1++)
	    {
            if (window[cnt][cnt1] != window[cnt+1][cnt1])
	        {
	        c_errmsg("Window sizes of input images must be the same",
		    " ",&nonfatal);
	        rstat = E_FAIL;
	        }
	    }
	}
    rec->sl = window[0][SL];
    rec->ss = window[0][SS];
    rec->nl = window[0][NL];
    rec->ns = window[0][NS];
    if ((strcmp(rec->color,"RGB") == 0) && (rec->nb != 3))
	{
	sprintf(emsg,"Image consists of %d bands, and RGB requires 3 bands",
		rec->nb);
	c_errmsg(emsg," ",&nonfatal);
	rstat = E_FAIL;
	}
    else if ((strcmp(rec->color,"CMY") == 0) && (rec->nb != 3))
	{
	sprintf(emsg,"Image consists of %d bands, and CMY requires 3 bands",
		rec->nb);
	c_errmsg(emsg," ",&nonfatal);
	rstat = E_FAIL;
	}
    else if ((strcmp(rec->color,"CMYK") == 0) && (rec->nb != 4))
	{
	sprintf(emsg,"Image consists of %d bands, and CMYK requires 4 bands",
		rec->nb);
	c_errmsg(emsg," ",&nonfatal);
	rstat = E_FAIL;
	}
    else if ((strcmp(rec->color,"BW") == 0) && (rec->nb != 1))
	{
	sprintf(emsg,"Image consists of %d bands, and BW requires 1 band",
		rec->nb);
	c_errmsg(emsg," ",&nonfatal);
	rstat = E_FAIL;
	}
    for (cnt=0; cnt<rec->nimg; cnt++)
	{
	strcpy(rec->tae[cnt],hnames[cnt]);
	}
    }
xit:
return(rstat);
}

int qlp_usage
(
    int prog
)
{
static int First = TRUE;

if (First)
    {
    First = FALSE;
    fprintf(stderr,"*** QLP parameters (refer to LAS documentation) ***\n\n");
    }
if (prog == PLSP)
    {
    fprintf(stderr,"qlp listpr\n");
    }
if (prog == PREP)
    {
    fprintf(stderr,"qlp ls\n");
    }
if (prog == PDEL)
    {
    fprintf(stderr,"qlp delete {recnmr}\n");
    fprintf(stderr,"   recnmr     %s\n",
	"(RECNMR) QLP record number to be deleted");
    }
if (prog == PDEL)
    {
    fprintf(stderr,"qlp abort {recnmr}\n");
    fprintf(stderr,"   recnmr     %s\n",
	"(RECNMR) QLP record number to be aborted");
    }
if ((prog == PADD) || (prog == PMOD))
    {
    fprintf(stderr,"qlp add -i lasimg [options]\n");
    fprintf(stderr,"qlp genadd -i rawfil -e ewrnbr -f {PS | TIF}\n");
    fprintf(stderr,"qlp modify {recnmr} [options]\n");
    fprintf(stderr,"   recnmr     %s\n",
	"(RECNMR)    QLP record number to be modified");
    fprintf(stderr,"   -A         %s\n",
	"(AUTOSZ)    Automatically resize image to fit page");
    fprintf(stderr,"   -a annot   %s\n",
	"(ANNOT)     Annotation text to be printed on bottom of print");
    fprintf(stderr,"   -b bg      %s\n",
	"(BACKGRND)  Background value");
    fprintf(stderr,"   -C spinst  %s\n",
	"(SPECIAL)   Special instructions");
    fprintf(stderr,"   -c color   %s\n",
	"(COLORORD)  Color order of the LAS image");
    fprintf(stderr,"   -d         %s\n",
	"(DELFLG)    Delete input image/file after network transfer");
    fprintf(stderr,"   -E expand  %s\n",
	"(EXPAND)    Option to expand the image by replicating pixels");
    fprintf(stderr,"   -e ewr     %s\n",
	"(EWRNBR)    EWR number to be charged for the print");
    fprintf(stderr,"   -f ftype   %s\n",
	"(FTYPE)     File type of file to be printed (LAS, PS, or TIF)");
    fprintf(stderr,"   -I enhfil  %s\n",
	"(INFILE)    Input enhancement file");
    fprintf(stderr,"   -i fname   %s\n",
	"(IN)        Input image/file name");
    fprintf(stderr,"   -m stropt  %s\n",
	"(STRETCH)   Annotate stretch points onto the print");
    fprintf(stderr,"   -N         %s\n",
	"(AFLG)      Do not annotate recnmr, uname, and time onto the print");
    fprintf(stderr,"   -n ncop    %s\n",
	"(NCOPIES)   Number of copies (must be value from 1-10)");
    fprintf(stderr,"   -P prio    %s\n",
	"(PRIORITY)  Priority of request (must be value from 1-9)");
    fprintf(stderr,"   -p printer %s\n",
	"(PRINTER)   Output printer name");
    fprintf(stderr,"   -R reduce  %s\n",
	"(REDUCE)    Option to reduce the image by subsample the image");
    fprintf(stderr,"   -r rotopt  %s\n",
	"(ROTOPT)    Rotate option (YES, NO, or AUTO)");
    fprintf(stderr,"   -S span    %s\n",
	"(SPANEL)    Starting panel number");
    fprintf(stderr,"   -s         %s\n",
	"(SPOTCOPY)  Annotate the Spot copyright notice onto the print");
    fprintf(stderr,"   -T ptype   %s\n",
	"(PTYPE)     Printer type (default is claser)");
    fprintf(stderr,"   -t title   %s\n",
	"(TITLE)     Title of print to be annotated at the top of the print");
    fprintf(stderr,"   -w         %s\n",
	"(WEDGE)     Print a color wedge at the bottom of the print");
    }
return(E_SUCC);
}

int qlp_unix_add
(
    int argc,               /* argument count */
    char *argv[],           /* argument vector */
    struct recfmt *rec,     /* QLP record block */
    long prog               /* calling program: PADD, PMOD, or REPORT   */
)
{
extern int optind;
extern char *optarg;
static long nonfatal = 1;
int     c;              	/* program option */
long	len;			/* string length */
long 	rstat;			/* function return status */
char	emsg[256];		/* error message buffer */

rstat = E_SUCC;
if (rec->curpan >= rec->npanel)
    rec->curpan = 0;
if (prog != PMOD)
    {
    strcpy(rec->aflg,"YES");
    *rec->annot = '\0';
    strcpy(rec->autosz,"NO");
    rec->bgrnd = 255;
    strcpy(rec->color,"RGB");
    rec->curpan = 1;
    strcpy(rec->delfil,"NO");
    *rec->dltf = '\0';
    rec->ewr = 0;
    rec->expand[0] = rec->expand[1] = NULLVAL;
    *rec->fspec = '\0';
    strcpy(rec->ftype,"las");
    *rec->infile = '\0';
    rec->ncop = 1;
    *rec->printer = '\0';
    rec->prio = 9;
    rec->reduce[0] = rec->reduce[1] = NULLVAL;
    *rec->rotopt = '\0';
    strcpy(rec->ptype,"claser");
    *rec->spinst = '\0';
    strcpy(rec->spot,"NO");
    strcpy(rec->stropt,"NO");
    *rec->title = '\0';
    strcpy(rec->wedge,"NO");
    }
while ((c=getopt(argc,argv,"Aa:b:C:c:dE:e:f:I:i:mNn:P:p:R:r:S:sT:t:w"))!=EOF)
    {
    switch (c)
        {
        case 'A':
	    strcpy(rec->autosz,"YES");
            break;
        case 'a':
            strcpy(rec->annot,optarg);
	    break;
        case 'b':
            sscanf(optarg,"%d",&rec->bgrnd);
            break;
        case 'C':
            strcpy(rec->spinst,optarg);
	    break;
        case 'c':
            strcpy(rec->color,optarg);
	    if ((strcmp(rec->color,"RGB") != 0) &&
		(strcmp(rec->color,"BW") != 0)  &&
		(strcmp(rec->color,"CMY") != 0)  &&
		(strcmp(rec->color,"CMYK") != 0))
		{
		sprintf(emsg,
			"%s is invalid color order (RGB, BW, CMY, or CMYK)",
			rec->color);
		c_errmsg(emsg," ",&nonfatal);
		rstat = E_FAIL;
		goto xit;
		}
	    break;
        case 'd':
	    strcpy(rec->delfil,"YES");
            break;
        case 'E':
            sscanf(optarg,"%d",&rec->expand[0]);
	    rec->expand[1] = rec->expand[0];
	    if (rec->expand[0] < 1)
		{
		sprintf(emsg,"Expand must be greater than 0");
		c_errmsg(emsg," ",&nonfatal);
		rstat = E_FAIL;
		goto xit;
		}
            break;
        case 'e':
            sscanf(optarg,"%d",&rec->ewr);
            break;
        case 'f':
            strcpy(rec->ftype,optarg);
	    len = (long)strlen(rec->ftype);
	    c_up2low(rec->ftype,&len);
	    if ((strcmp(rec->ftype,"las") != 0) &&
		(strcmp(rec->ftype,"ps") != 0)  &&
		(strcmp(rec->ftype,"rtl") != 0)  &&
		(strcmp(rec->ftype,"hpgl") != 0)  &&
		(strcmp(rec->ftype,"tif") != 0))
		{
		sprintf(emsg,
		    "%s is an invalid file type (LAS, PS, RTL, HPGL, or TIF)",
		    rec->ftype);
		c_errmsg(emsg," ",&nonfatal);
		rstat = E_FAIL;
		goto xit;
		}
            break;
        case 'I':
            strcpy(rec->infile,optarg);
	    break;
        case 'i':
            strcpy(rec->fspec,optarg);
            break;
        case 'm':
	    strcpy(rec->stropt,"YES");
            break;
        case 'N':
	    strcpy(rec->aflg,"NO");
            break;
        case 'n':
            sscanf(optarg,"%d",&rec->ncop);
	    if (rec->ncop < 1)
		{
		sprintf(emsg,"%d is invalid number of copies, setting to 1",
			rec->ncop);
		c_errmsg(emsg," ",&nonfatal);
		rec->ncop = 1;
		}
	    if (rec->ncop > 10)
		{
		sprintf(emsg,"%d is invalid number of copies, setting to 10",
			rec->ncop);
		c_errmsg(emsg," ",&nonfatal);
		rec->ncop = 10;
		}
            break;
        case 'P':
            sscanf(optarg,"%d",&rec->prio);
	    break;
        case 'p':
            strcpy(rec->printer,optarg);
	    len = (long)strlen(rec->printer);
            c_up2low(rec->printer,&len);
            break;
        case 'R':
            sscanf(optarg,"%d",&rec->reduce[0]);
	    rec->reduce[1] = rec->reduce[0];
	    if (rec->reduce[0] < 1)
		{
		sprintf(emsg,"Reduce must be greater than 0");
		c_errmsg(emsg," ",&nonfatal);
		rstat = E_FAIL;
		goto xit;
		}
            break;
        case 'r':
            strcpy(rec->rotopt,optarg);
	    if ((strcmp(rec->rotopt,"YES") != 0) &&
		(strcmp(rec->rotopt,"NO") != 0)  &&
		(strcmp(rec->rotopt,"AUTO") != 0))
		{
		sprintf(emsg,"%s is invalid rotate option (YES, NO, or AUTO)",
			rec->rotopt);
		c_errmsg(emsg," ",&nonfatal);
		rstat = E_FAIL;
		goto xit;
		}
	    break;
        case 'S':
            sscanf(optarg,"%d",&rec->curpan);
	    if (rec->curpan < 1)
		{
		sprintf(emsg,"Starting panel must be greater than 0");
		c_errmsg(emsg," ",&nonfatal);
		rstat = E_FAIL;
		goto xit;
		}
            break;
        case 's':
	    strcpy(rec->spot,"YES");
            break;
        case 'T':
            strcpy(rec->ptype,optarg);
	    len = (long)strlen(rec->ptype);
	    c_up2low(rec->ptype,&len);
	    if ((strcmp(rec->ptype,"claser") != 0) &&
		(strcmp(rec->ptype,"phaser300") != 0) &&
		(strcmp(rec->ptype,"phaser220") != 0) &&
		(strcmp(rec->ptype,"hp755")     != 0) &&
		(strcmp(rec->ptype,"hp2500")    != 0) &&
		(strcmp(rec->ptype,"inkjet")    != 0) &&
		(strcmp(rec->ptype,"photo")     != 0) &&
		(strcmp(rec->ptype,"bw")        != 0))
		{
		sprintf(emsg,"%s is an invalid printer type %s%s",rec->ptype,
		    "(phaser300, phaser220, claser, hp755, hp2500, ",
		    "inkjet, photo, or bw)");
		c_errmsg(emsg," ",&nonfatal);
		rstat = E_FAIL;
		goto xit;
		}
            break;
        case 't':
            strcpy(rec->title,optarg);
	    break;
        case 'w':
	    strcpy(rec->wedge,"YES");
            break;
        default:
            sprintf(emsg,"QLP [ADD | GENADD] Invalid option...");
	    c_errmsg(emsg," ",&nonfatal);
            rstat = E_FAIL;
            goto xit;
	}
    }
if (prog == PADD)
    {
    if (strlen(rec->fspec) == 0)
	{
	sprintf(emsg,"Image/file name must be specified for qlp add/genadd");
	c_errmsg(emsg," ",&nonfatal);
	rstat = E_FAIL;
	}
    if (rec->ewr == 0)
	{
	sprintf(emsg,"EWR number must be specified for qlp add/genadd");
	c_errmsg(emsg," ",&nonfatal);
	rstat = E_FAIL;
	}
    }
xit:
if (rstat != E_SUCC)
    qlp_usage(prog);
return(rstat);
}
