/*******************************************************************************
FUNCTION:	rename_files

PURPOSE:	

Version  Date   Programmer      Modifications
-------  -----  --------------- -----------------------------------------------
  1.0    10/92  J.Fenno         Development
  1.1    07/93  J.Fenno         Stolen from READARCH
  1.1	 11/93	S.Nelson	Copied RENAME_FILES from READARCH.  Adapted to
				convert MSS transfer images to LAS images.


PROJECT:	LAS
*******************************************************************************/
#define BROWSE 2
#define FULL 1

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "diskio.h"
#include "protos.h"


void rename_files
(
    char *images[],	/* IN/OUT - Archive images that have been renamed  */
    long *nimages,       /* IN/OUT - Number of archive image renamed        */
    int  *typeflg	/* flag for type of images browse = 1, full = 2 */
)
{
long fatal    = -1;     /* Fatal error flag                      	*/
long i, j;		/* Loop control variables			*/
long nimag = 0;		/* Number of images that match filename		*/
long tmpnimag = 0;	/* number of images for bands 2-4 and other files */
int  typechk;		/* flag to images used to check for consistancy */

char extension[5];	/* extension of renamed file			*/
char filename[17];      /* IN - Specification for the files to be renamed  */
char files[20][CMLEN];  /* Names of the files returned by c_getfiles	*/
char tmpfiles[20][CMLEN]; /* Temporary files returned by c_getfiles	*/
char newname[CMLEN];	/* New name of file				*/
char tempstr[20];	/* temporary string				*/


/* assign strings for the first band
  ---------------------------------*/
strcpy(filename,"????????????????1");
strcpy(extension,".img1");

/* Get the first bands that matches the given file name
------------------------------------------------------*/
if (c_getfiles(filename, files, &nimag, 15, TRUE, FALSE))
   c_errmsg("Error searching files in directory", "readmss-getfile",&fatal);

/* Rename all files ending in 1 with 17 total characters
-------------------------------------------------------*/
*nimages = 0;
if (nimag == 0)
   c_errmsg("No scenes found","readmss-scenes", &fatal);
/* delete last character and add extension 
  ---------------------------------------*/
typechk = 0;

/* Set TAE variable
   ---------------*/
*nimages = 0;

for (i = 0; i < nimag; i++)
   {
   strcpy(tempstr,files[i]);
   tempstr[16] = '\0';
   strcpy(filename,tempstr);
   strcat(filename,"*");

   /* check number of files per scene id
    -----------------------------------*/
   if (c_getfiles(filename, tmpfiles, &tmpnimag, 20, TRUE, FALSE))
      c_errmsg("Error searching files in directory", "readmss-getfile",&fatal);
   if (tmpnimag == 4)
      typeflg[0] = BROWSE;
   else if (tmpnimag == 7)
      typeflg[0] = FULL;
   else
      c_errmsg("Incorrect number of files per scene","readmss-files",&fatal);
   if ((typechk != 0) && (typechk != typeflg[0]))
      c_errmsg("Incorrect number of files per scene","readmss-files",&fatal);
   typechk = typeflg[0];
      
   /* process band 1
    ---------------*/
   sprintf(newname, "%s%s",tempstr,".img1");
   if (c_dkmove(files[i], newname, NULL))
      c_errmsg("Error returned by c_dkmove", "readmss-support", &fatal);

   /* process band 2
    ---------------*/
   sprintf(newname, "%s%s",tempstr,".img2");
   files[i][16] = '2';
   if (c_dkmove(files[i], newname, NULL))
      c_errmsg("Error returned by c_dkmove", "readmss-support", &fatal);

  /* process band 3
   ---------------*/
   sprintf(newname, "%s%s",tempstr,".img3");
   files[i][16] = '3';
   if (c_dkmove(files[i], newname, NULL))
      c_errmsg("Error returned by c_dkmove", "readmss-support", &fatal);

   /* process band 4
    ---------------*/
   sprintf(newname, "%s%s",tempstr,".img4");
   files[i][16] = '4';
   if (c_dkmove(files[i], newname, NULL))
      c_errmsg("Error returned by c_dkmove", "readmss-support", &fatal);

   /* set string for TAE return variable
     ----------------------------------*/
   images[i] = malloc(strlen(tempstr) + 1);
   if (images[i] == NULL)
      {
      c_errmsg("Error allocating memory", "readmss-alloc", &fatal);
      for (j = 0; j < *nimages; j++)
 	     free(images[j]);
      }
   *nimages += 1;
   strcpy(images[i], tempstr);

   /* rename extra files if full image
    ---------------------------------*/
   if (typeflg[0] == FULL)
       {
       /* process siat file
         -----------------*/
       sprintf(newname, "%s%s",tempstr,".siat");
       files[i][16] = 'o';
       if (c_dkmove(files[i], newname, NULL))
          {
          c_errmsg("Error returned by c_dkmove", "readmss-support", &fatal);
	  for (j = 0; j < *nimages; j++)
 	     free(images[j]);
	  }

       /* process header file
         -------------------*/
       sprintf(newname, "%s%s",tempstr,".ahdr");
       files[i][16] = 'h';
       if (c_dkmove(files[i], newname, NULL))
          {
          c_errmsg("Error returned by c_dkmove", "readmss-support", &fatal);
	  for (j = 0; j < *nimages; j++)
 	     free(images[j]);
	  }

       /* process calibration file
         ------------------------*/
       sprintf(newname, "%s%s",tempstr,".ncal");
       files[i][16] = 'c';
       if (c_dkmove(files[i], newname, NULL))
          {
          c_errmsg("Error returned by c_dkmove", "readmss-support", &fatal);
	  for (j = 0; j < *nimages; j++)
 	     free(images[j]);
	  }
       }
   }

return;
}
