/****************************************************************************  
* Name  RELMAP
*
* Purpose  Create an ASCII table from Oracle
*          and format it for the Renumber function
*
* Program History
*       Programmer         Date           Reason
*       ----------         ----           ------
*       K. Lowell          08/93          Original development
*
* Computer Hardware and/or software limitations
*       Must be run under TAE
*
* PROJECT      LAS
*
* LANGUAGE     C      
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "las.h"  
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK parblk;

FILE  *file_ptr,                /* ptr to input file */
      *fp;                      /* ptr to spool control file */

long  	count = 0,              /* counter variable  */
        fatal = 0,		/* fatal error constant */
        nonfatal = 0,		/* nonfatal status indicator */
        one = 1,                /* constant of 1 */
        std_out,                /* standard output */
        strlong = 2000,         /* maximum string length */
        zero = 0;               /* constant of 0 */

char   from[CMLEN],             /* FROM value of pixel */
       command[CMLEN],          /* system command string */
       orcfldin[CMLEN],         /* Oracle in value column */
       orcfldout[CMLEN],        /* Oracle out value column */
       orcpwd[CMLEN],           /* Oracle user password */
       orctab[CMLEN],           /* Oracle data table */
       orcuid[CMLEN],           /* Oracle user ID */
       spooltxt[1][CMLEN],      /* spool file name */
       to[CMLEN];               /* TO value of pixel */

static char infile[CMLEN],      /* input file */
            instring[CMLEN],    /* input data string sent to TAE */
            outfile[1][CMLEN],  /* output file */
            outstring[CMLEN],   /* output data string sent to TAE */
            line[CMLEN],        /* input file read buffer */
           *strptr1[1],         /* ptr to instring */
           *strptr2[1];         /* ptr to outstring */

/* Initialize TAE parameter block
---------------------------------*/
c_inlas(&parblk,&std_out);

/*  Initialize LINE
-------------------*/
strcpy(line,"");
strcpy(instring,"");
strcpy(outstring,"");
strcpy(command,"");

/* Get user-specified parameters
--------------------------------*/
if (getpar(&parblk,orcuid,orcpwd,orctab,orcfldin,orcfldout)
   != E_SUCC)
   {
   c_errmsg("Fatal error encountered","relmap-fatal",&fatal);
   } 

/* create spool generation com file 
-----------------------------------*/
strcpy(infile,"input.tmp");
strcpy(spooltxt[0],"spool.com");
if ((fp = fopen(spooltxt[0],"w")) == NULL)
  {
  c_clean(&parblk,"relmap",spooltxt,&one);
  c_errmsg("Error opening spool file","relmap-open",&fatal);
  }

fprintf(fp,"set newpage 0\n");
fprintf(fp,"set space 0\n");
fprintf(fp,"set linesize 80\n");
fprintf(fp,"set pagesize 0\n");
fprintf(fp,"set echo off\n");
fprintf(fp,"set feedback off\n");
fprintf(fp,"set heading off\n");
fprintf(fp,"spool %s\n", infile);
fprintf(fp,"select %s, %s from %s;\n", orcfldin, orcfldout, orctab);
fprintf(fp,"spool off\n");
fprintf(fp,"exit");
fclose(fp);

/* generate the input file
---------------------------*/
sprintf(command,"sqlplus %s/%s @%s", orcuid,orcpwd,spooltxt[0]);
if (system(command)
   != E_SUCC)
   {
   c_delfil(spooltxt[0]);
   c_errmsg("Spool process failed","relmap-spool",&fatal);
   }
c_delfil(spooltxt[0]);

/* Open input file
------------------*/
if ((file_ptr = fopen(infile,"r")) == NULL)
  {
  c_errmsg("Error opening input file","relmap-open",&nonfatal);
  c_errmsg("Fatal error encountered","relmap-fatal",&fatal);
  }

/* Read the input file until End of File (EOF) is found
-------------------------------------------------------*/
for (;;)
   {
   if (fgets(line,80,file_ptr) != NULL) 
     {
     if (count != 0)
       {
       strcat(instring, ",");
       strcat(outstring, ",");
       }
     sscanf(line,"%s %s", from,to);
     if((strlen(instring)+strlen(from)) > strlong)
       {
       c_errmsg("String lenghth overrun","relmap-len",&fatal);
       }
     strcat(instring, from);
     if((strlen(instring)+strlen(from)) > strlong)
       {
       c_errmsg("String lenghth overrun","relmap-len",&fatal);
       }
     strcat(outstring, to);
     count++;
     } 
   else 
     {
     fclose(file_ptr);
     strptr1[0] = instring;
     if (q_string(&parblk,"instring",1,strptr1,P_UPDATE)
        != E_SUCC)
        {
        c_errmsg("Error writing to TAE parameter block","relmap-tae",&fatal);
        }
     strptr2[0] = outstring;
     if (q_string(&parblk,"outstring",1,strptr2,P_UPDATE)
        != E_SUCC)
        {
        c_errmsg("Error writing to TAE parameter block","relmap-tae",&fatal);
        }
     q_out(&parblk);
     break;
     } 
   }


c_delfil(infile);

/* Output completion message
-----------------------------*/
c_complt("relmap",outfile,&zero);

return EXIT_SUCCESS;

} 
