/***************************************************************************
*
*  NAME: GETPAR
*  FUNCTION: Get the input parameters
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments
*
******************************************************************************/
#include "las.h"
#include "util.h"
#include "tab2orc.h"
#include "protos.h"
 
int getpar
(
    struct PARBLK *parblk,
    char ltin[][CMLEN],             /* input labeled table */
    char *outfile,                  /* output Oracle table name */
    char *orcuid,                   /* Oracle user ID */
    char *orcpwd,                   /* Oracle user password */
    long *ascfldsz,                 /* optional Oracle table field sizes */
    char *over,                     /* override option */
    long *msgtim                   /* time for processing message */
) 
{
 
 
struct VARIABLE *var;
 
long access = IREAD;            /* file access value */
long i;                         /* loop counter */
long status;                    /* function status */
long fatal = -1;                /* key for fatal error messages */
long nfile;                     /*    */
 
char *getenv();
 
/*  Receive parameter naming the labeled table,
------------------------------------------------*/
status = c_getcnt(parblk,"INLT",&nfile);
for (i = 0; i < nfile; i++)
   {
   status = c_getnam(parblk,"INLT",&i,ltin[i],"lt",&access);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","tab2orc-fatal",&fatal);
   }
 
/*  Receive parameter naming the output Oracle table name
---------------------------------------------------------*/
var = p_find(parblk,"OUTFILE");
strcpy(outfile,SVAL(*var,0));

/*  Receive parameter naming the user's Oracle ID
-------------------------------------------------*/
var = p_find(parblk,"ORCUID");
strcpy(orcuid,SVAL(*var,0));

/*  Receive parameter naming the user's Oracle password
-----------------------------------------------------*/
var = p_find(parblk,"ORCPWD");
strcpy(orcpwd,SVAL(*var,0));

/*  Receive parameter containing the optional table field
    sizes.  These will be entered if the user desires the
    oracle database table fields to be sized differently than
    the defaults listed in the user's guide
-------------------------------------------------------*/
var = p_find(parblk,"ASCFLDSZ");
if ((*var).v_count !=0)
 {
  for (i=0; i < var->v_count; i++)
  ascfldsz[i] = IVAL(*var,i);
 }
/* Receive the parameter containing the override option
-------------------------------------------------------*/
var = p_find(parblk,"OVER");
strcpy(over,SVAL(*var,0));


/* Get message time
------------------*/
var = p_find(parblk,"$MSGTIM");
*msgtim = IVAL(*var,0);
 
return(E_SUCC);
}

