/***************************************************************************
*
*  NAME: TAB2ORC
*
*  FUNCTION: Extract column names and datatypes from a labeled 
*            table, then generates SQL com files for generation
*            of Oracle tables, and SQLloader ctl files for 
*            populating the created database tables.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROGRAM HISTORY:
*   Version        Date      Author       Request
*   -------        ----      ------       -------
*     1.0          8/93      K. Lowell    original development
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments
*
**************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "tab2ras.h"
#include "ltable.h"
#include "typlim.h"
#include "protos.h"

struct PARBLK parblk;

int main
(void)

{
static struct TAB_DEF tab[100];
static struct VECTOR *ptr[100];

long ascfldsz;                   /* optional Oracle table field sizes */
long msgtim;                     /* time for processing message */
long one = 1;                    /* constant of 1 */
long std_out;                    /* standard output */
long tab_ind[100];               /* tab index */
long i;                          /* counters */
long nsvalues;                   /* number of string values */
long fatal = -1;                 /* fatal error constant */
long two = 2;


char ltin[MAX_SUB][CMLEN];       /* labeled table files */
char orcpwd[CMLEN];              /* Oracle user password */
char orcuid[CMLEN];              /* Oracle user ID */
char outfile[1][CMLEN];          /* output Oracle table name */
char orstr[100][CMLEN];          /* Oracle column external type */
char orcmk[100][CMLEN];          /* Oracle column type */
char orcnm[100][CMLEN];          /* Oracle column name */
char over[CMLEN];                /* override option flag */
char *sptr[CMLEN];               /* pointer array for updating vblock */
char mkfile[1][CMLEN];           /* pointer to mktable.com */

short max_col = 0;               /* Max no. of columns of all vectors */

FILE *fp;

/* Initialize TAE parameter block
---------------------------------*/
c_inlas(&parblk,&std_out);

/* Get the parameters and send to VBLOCK
---------------------*/
if (getpar(&parblk,ltin,outfile[0],orcuid,orcpwd,&ascfldsz,over,&msgtim)
   != E_SUCC)
   {
   c_errmsg("Fatal error encountered","tab2orc-fatal",&fatal);
   }

/* Open the input label table files
-----------------------------------*/
open_tab(tab,ltin[0],IREAD);
get_vector(tab);

/* Get the column names
-----------------------*/
nsvalues = 0;
max_col = (short) tab[0].ncol;
  ptr[0] = (struct VECTOR *) tab[0].vector;
  strcpy(orcnm[0], ptr[0]->label);
  for (i = 1; i < max_col; i++)
    {
      ptr[i] = (struct VECTOR *) ptr[i - 1]->next;
      strcpy(orcnm[i], ptr[i]->label);
     }
nsvalues = i;
/* Get the attribute names and data types
-----------------------------------------*/
for (i = 0; i < max_col; i++)
  {
  switch (ptr[i]->dtype)
    {
    case C:  strcpy(orstr[i],"char");
             strcpy(orcmk[i],"char(20)");
             break;
    case I1: strcpy(orstr[i],"integer external");
             strcpy(orcmk[i],"number(4)");
             break;
    case I2: strcpy(orstr[i],"integer external");
             strcpy(orcmk[i],"number(8)"); 
             break;
    case I4: strcpy(orstr[i],"integer external");
             strcpy(orcmk[i],"number(16)");
             break;
    case R4: strcpy(orstr[i],"decimal external");
             strcpy(orcmk[i],"number(20,8)");
             break;
    case R8: strcpy(orstr[i],"decimal external");
             strcpy(orcmk[i],"number(30,8)");
             break;
    case LTDATE: 
             break;
    case LTTIME:
             break;
    }
  }

/* Send the column names and sizes to the parameter block
---------------------------------------------------------*/
for (i = 0; i < max_col; i++)
  {
  sptr[i] = orcnm[i];
  tab_ind[i] = i + 1;
  }
q_string(&parblk,"ASCFLD",nsvalues,sptr,P_UPDATE);
q_intg(&parblk,"OUTCOL",nsvalues,(int *) tab_ind,P_UPDATE);
q_out(&parblk);

/* create table in Oracle database
---------------------------------*/
strcpy(&mkfile[0][0],"mktable.com");
if ((fp = fopen("mktable.com","w")) ==NULL) 
  {
  c_clean(&parblk,"tab2orc",mkfile,&one);
  c_errmsg("Error opening output file","tab2orc-cre",&fatal);
  }
 
for (i = 0; i < max_col; i++)
  {
  if(i == 0) {
    fprintf(fp, "create table %s (\"%s\" %s,\n",
            outfile[0], orcnm[i], orcmk[i]); 
   }
  else if(i == (max_col - 1)) {
    fprintf(fp, "\"%s\" %s);\n", orcnm[i], orcmk[i]);
    fprintf(fp, "EXIT");
   }
  else {
    fprintf(fp, "\"%s\" %s,\n", orcnm[i], orcmk[i]);
   }
 }
fclose(fp);

/*Create SQL*loader control file
---------------------------------*/
strcpy(&mkfile[1][0],"tab.ctl");
if ((fp = fopen("tab.ctl","w")) ==NULL) 
  {
  c_clean(&parblk,"tab2orc",mkfile,&two);
  c_errmsg("Error opening output file","tab2orc-ctl",&fatal);
  }
fprintf(fp, "load data\n");
fprintf(fp, "infile %s.txt\n", outfile[0]);
fprintf(fp, "discardmax 0\n");

if (!strcmp(over,"REP")) fprintf(fp, "REPLACE\n");
if (!strcmp(over,"APPN")) fprintf(fp, "APPEND\n");

fprintf(fp, "into table %s\n", outfile[0]);

for (i = 0; i < max_col; i++)
  {
  if(i == 0) {
    fprintf(fp,"(\"%s\" position(*) %s terminated by \" \",\n",
           orcnm[i], orstr[i]);
   }
  else if(i == (max_col - 1)) {
    fprintf(fp,"\"%s\" position(*) %s terminated by \" \"   )\n",
           orcnm[i], orstr[i]);
   }
  else 
    fprintf(fp,"\"%s\" position(*) %s terminated by \" \",\n",
           orcnm[i], orstr[i]);
  }
fclose(fp);

/* Close the input label table files
------------------------------------*/
close_tab(tab);

/* Output completion message
----------------------------*/
c_complt("TAB2ORC",outfile,&one);

return EXIT_SUCCESS;

}
