/******************************************************************************
FUNCTION:	ALLOC

PURPOSE:	Part of the DITTO subsystem which allocates the input/output
		devices.  If the device is "tape", then the tape is mounted.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      10/85    K. Gacke		initial development
  1.1	    6/86    K. Gacke		LAS standards
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1       5/91    D. Etrheim		Change access for open of output tape
  5.2	    1/92    T. Mittan		Change access for open of output tape.
				        Don't want to write extra tape mark 
					at BOT
  5.3       8/92    K. Gacke		Save tape ID in TAE global variable
					ITAPEID and/or OTAPEID
  5.4       9/93    D. Etrheim		Sent the maximum record and block size
					back in the appropriate globals.
  5.5       6/94    T. Mittan		Modified to accept a DRIVE to open
                                        a tape on	

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	From the input parameters it is determined which input/output DITTO
	device is to be allocated, and whether the device is to be assigned 
	to a disk file or a magnetic tape.  The DITTO device is allocated with 
	one of the following methods.

	  1)  Disk file assigned to input DITTO device:
		Error checking is done to insure the disk file exists and has
		the correct file attributes (i.e. variable or fixed length
		records).  The global variables for the input device are
		assigned appropiately.

	  2)  Disk file assigned to output DITTO device:
		The global variables for the output device are assigned 
		appropiately.

	  3)  Tape assigned to input DITTO device:
		The input tape identified by the tape label is mounted onto
		a tape drive.  The global variables for the input device are
		assigned appropiately.

	  4)  Tape assigned to output DITTO device:
		The output tape identified by the tape label is mounted onto
		a tape drive.  The global variables for the output device are
		assigned appropiately.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "util.h"           /* c_inlas prototype */
#include "ditto.h"			/* ditto include file                */

struct PARBLK   vblock;			/* TAE parameter block               */
struct DITTO_DEV ditto_device;		/* ditto I/O devices descriptors     */

void proc_disk(void);
void proc_tape(void);

int main(void)
{
struct VARIABLE *val;			/* pointer to TAE variable           */

long std_out;				/* Fortran LUN for standard out      */
long zero = 0;				/* value of zero                     */

char out[1][CMLEN];			/* output files for completion msg   */
char *subcmd;				/* subcommand being executed         */

/*
  Initialize TAE
*/
c_inlas(&vblock,&std_out);

/*
  Initialize the ditto device
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Determine which subcommand is being executed.
*/
val = p_find(&vblock,"_SUBCMD");
subcmd = SVAL(*val,0);

/*
  Call the appropriate subroutine to allocate the DITTO input/output device.
*/
if (strcmp(subcmd,"DISK") == 0)
   proc_disk();
else if (strcmp(subcmd,"TAPE") == 0)
   proc_tape();

c_complt("DITTO ALLOC",out,&zero);
return 0;
}
