      SUBROUTINE GETGRD(IFCB,GRID,NDIM,N,LINE,
     *SAMP,CENTR,TRANS,SHIFT,MAGNIF,CHOP,band,status)

C---- VICAR SUBROUTINE "GETGRD".
C     PURPOSE:    TO OBTAIN THE DN-VALUES IN THE POINTS
C   OF THE LINEARLY TRANSFORMED GRID.
C   DN-VALUES ARE OBTAINED BY THE BILINEAR
C   INTERPOLATION BETWEEN FOUR PIXELS
C   ADJACENT TO THE GRID POINT.
C     PARAMETERS: 
C   GRID  - OUTPUT L*1 ARRAY OF INTERPOLATED
C    DN-VALUES.
C   NDIM  - DIMENSION OF THE GRID(NDIM MAX=128), EVEN
C   BUFIN - WORK AREA, TWO-DIMENSIONAL L*1 ARRAY
C    BUFIN(MAXNS,2), LBUFIN>NDIM*MAGN(2)+50
C   N     - DEFINES THE PORTION OF THE GRID
C    TO BE PROCESSED (MUST BE EVEN).
C   LINE  - NUMBER OF LINES IN THE IMAGE.
C   SAMP  - NUMBER OF SAMPLES IN THE IMAGE.
C   CENTR - ARRAY, CENTR(1)=Y OF THE GRID CENTER,
C    CENTR(2)=X OF THE GRID CENTER.
C   TRANS - ARRAY OF THE GRID TRANSFORMATION COEFFICIENTS:
C    YNEW=TRANS(1)*YOLD+TRANS(2)*XOLD+TRANS(3)
C    XNEW=TRANS(4)*YOLD+TRANS(5)*XOLD+TRANS(6)
C   SHIFT - VECTOR TO ADD TO THE TRANSFORMED GRID
C    TO BRING ITS CENTER TO "CENTR".
C   MAGNIF- ARRAY CONTAINING MAGNIFICATION OF
C    X- AND Y- DISTANCES IN THE GRID.
C   CHOP  - NUMBER OF GRID POINTS OUTSIDE THE IMAGE.
C    CORRESPONDING DN-VALUES IN THE "GRID"
C    ARE SET TO ZERO.
C     PROGRAMMER:  BORIS GOKHMAN, OCTOBER 1980.

	include 'las.fin'

	real*4 basey
	real*4 basex
        real*4 bufin(MAXNS,2)
	real*4 CENTR(2)
        real*4 dn1
	real*4 dn12
	real*4 dn2
	real*4 dn3
	real*4 dn34
	real*4 dn4
	real*4 factx
	real*4 gapy
	real*4 gapx
	real*4 grid(1)
	real*4 idn
      	REAL*4 MAGNIF(2)
	real*4 SHIFT(2)
      	real*4 TRANS(6)
      	real*4 t1gapy
	real*4 t2gapx
	real*4 t4gapy
	real*4 t5gapx
      	REAL*4 Y(4)
      	real*4 yinc1(128)
	real*4 yinc4(128)
	real*4 ymax
	real*4 ymin
	real*4 y0
	real*4 X(4)
	real*4 xinc2(128)
	real*4 xinc5(128)
	real*4 xmax
	real*4 xmin
	real*4 x0

	integer*4 band
        integer*4 chop
	integer*4 i
	integer*4 iabbr
	integer*4 icount
	integer*4 ifcb
	integer*4 ihi
	integer*4 ilow
	integer*4 imin
	integer*4 irec
	integer*4 iright
      	integer*4 isgnes(4)
	integer*4 isig
	integer*4 ist
      	integer*4 istart(128)
	integer*4 iy
	integer*4 ix
	integer*4 i0
	integer*4 i1
	integer*4 j
	integer*4 jfin
	integer*4 jsgnes(4)
	integer*4 jsig
	integer*4 jx
	integer*4 jxplus
	integer*4 j0
	integer*4 j1
	integer*4 line
	integer*4 n
	integer*4 noffb
	integer*4 ndim
	integer*4 samp
	integer*4 status

      LOGICAL ENDJ(128)

	common /bufin/bufin

        data dn1/0/,dn2/0/,dn3/0/,dn4/0/
	data idn/0/
        DATA ISGNES / 1, -1, -1, 1 /
        DATA JSGNES / 1, 1, -1, -1 /

C---- PRELIMINARY CALCULATIONS
C-----------------------------
      NOFFB = (NDIM-N)*(NDIM+1)/2
      IHI = 1
      ILOW = 2
      CHOP = 0
      IF(CENTR(1).LT.1..OR.CENTR(1).GT.LINE) then
C	call errmsg ('Grid center not within image bounds',
C     .			'lamsutil-grid',1)
	status = 4
	RETURN 
      end if
      IF(CENTR(2).LT.1..OR.CENTR(2).GT.SAMP) then
C	call errmsg ('Grid center not within image bounds',
C     .			'lamsutil-grid',1)
	status = 4
	RETURN 
      end if
      GAPX = MAGNIF(2)
      GAPY = MAGNIF(1)
      shift(1)= CENTR(1)-(N+1)/2.*(TRANS(1)+TRANS(2))-TRANS(3)
      SHIFT(2) = CENTR(2)-(N+1)/2.*(TRANS(4)+TRANS(5))-TRANS(6)
      XMIN = (N+1)/2.-GAPX*(N-1)/2.
      YMIN = (N+1)/2.-GAPY*(N-1)/2.
      XMAX = (N+1)/2.+GAPX*(N-1)/2.
      YMAX = (N+1)/2.+GAPY*(N-1)/2.

C---- FIND THE UPPERMOST CORNER OF THE TRANSFORMED QUAD
C     AND THE SIGN OF THE ARRAY INDEXES CHANGE.
C------------------------------------------------------
      Y(1) = TRANS(1)*YMIN+TRANS(2)*XMIN+TRANS(3)+SHIFT(1)
      Y(2) = TRANS(1)*YMAX+TRANS(2)*XMIN+TRANS(3)+SHIFT(1)
      Y(3) = TRANS(1)*YMAX+TRANS(2)*XMAX+TRANS(3)+SHIFT(1)
      Y(4) = TRANS(1)*YMIN+TRANS(2)*XMAX+TRANS(3)+SHIFT(1)
      IMIN = 1
      DO 20 I=2,4
          IF(Y(I).LT.Y(IMIN)) IMIN = I
   20 CONTINUE
      ISIG = ISGNES(IMIN)
      JSIG = JSGNES(IMIN)

C---- FIND THE SAMPLE COORDINATES OF LEFTMOST AND
C     RIGHTMOST CORNERS OF THE GRID.
C------------------------------------------------
      X(1) = TRANS(4)*YMIN+TRANS(5)*XMIN+TRANS(6)+SHIFT(2)
      X(2) = TRANS(4)*YMAX+TRANS(5)*XMIN+TRANS(6)+SHIFT(2)
      X(3) = TRANS(4)*YMAX+TRANS(5)*XMAX+TRANS(6)+SHIFT(2)
      X(4) = TRANS(4)*YMIN+TRANS(5)*XMAX+TRANS(6)+SHIFT(2)
      IABBR = X(1)
      IRIGHT = X(1)
      DO 21 I=2,4
          IF(X(I).LT.IABBR) IABBR = X(I)
          IF(X(I).GT.IRIGHT) IRIGHT = X(I)
   21 CONTINUE
      IABBR = MAX0(IABBR-10,0)
      ICOUNT = IRIGHT-IABBR+10

C---- SET UP PARAMETERS FOR THE LOOP.
C------------------------------------
      T1GAPY = TRANS(1)*GAPY
      T2GAPX = TRANS(2)*GAPX
      T4GAPY = TRANS(4)*GAPY
      T5GAPX = TRANS(5)*GAPX
      BASEY = TRANS(1)*(YMIN-GAPY)+TRANS(2)*(XMIN-GAPX)+
     * TRANS(3)+SHIFT(1)
      BASEX = TRANS(4)*(YMIN-GAPY)+TRANS(5)*(XMIN-GAPX)+
     * TRANS(6)+SHIFT(2)
      DO 30 I=1,N
          YINC1(I) = T1GAPY*I
          XINC2(I) = T2GAPX*I
          YINC4(I) = T4GAPY*I
          XINC5(I) = T5GAPX*I
   30 ISTART(I) = 1
      I0 = (N+1)*(1-ISIG)/2
      J0 = (N+1)*(1-JSIG)/2
      JFIN = J0+ISIGN(N,JSIG)
   60 IREC = Y(IMIN)
      	IF(IREC.GT.0.AND.IREC.LE.LINE) then
     	   CALL VREAD(IFCB,IREC,IABBR,ICOUNT,samp,1,band,status)
	      if (status .eq. EFAIL) then
	         return
	      end if
	end if
      IREC = IREC+1
      	IF(IREC.GT.0.AND.IREC.LE.LINE) then
     	   CALL VREAD(IFCB,IREC,IABBR,ICOUNT,samp,2,band,status)
	      if (status .eq. EFAIL) then
		 return
	      end if
	end if

C---- BEGIN THE LOOP
C-------------------
  100 CONTINUE
      DO 300 J1=1,N
      J = J0+ISIGN(J1,JSIG)
      IF(.NOT.ENDJ(J).AND.ISTART(J).EQ.N) GO TO 300
      IST = ISTART(J)
      ENDJ(J) = .FALSE.
      DO 200 I1=IST,N
          I = I0+ISIGN(I1,ISIG)

C  Calculate the y and x location of the kernal grid point
C---------------------------------------------------------
          Y0 = BASEY+YINC1(I)+XINC2(J)
          X0 = BASEX+YINC4(I)+XINC5(J)
          IF(Y0.GE.IREC.AND.I1.EQ.1) GO TO 400
C CHECK IF THE END OF COLUMN #J IN THE STRIP IS REACHED
C------------------------------------------------------
          ISTART(J) = I1
          IF(Y0.LT.IREC) GO TO 240
          ENDJ(J) = .TRUE.
          GO TO 300
  240    CONTINUE
          IF(X0.GE.1.0.AND.X0.LE.SAMP.AND.Y0.GE.1.0.AND.Y0.LE.LINE)
     * GO TO 210
          IDN = 0
          CHOP = CHOP+1
C	IF (CHOP.GT.5000) then
C	   call errmsg('Too many grid points lie outside of image bounds',
C     .	   	       'lamsutil-point',1)
C	   status = EFAIL
C	   return
C	end if
          GO TO 220
  210    CONTINUE
	  ix=x0
	  jx=ix-iabbr
	  jxplus=jx+1
	  iy=y0

C  Interpolation of the four points around the kernal grid point
C---------------------------------------------------------------
	  dn1=bufin(jx,ihi)
	  dn2=bufin(jxplus,ihi)
	  dn3=bufin(jx,ilow)
	  dn4=bufin(jxplus,ilow)
	  factx=(x0-ix)
	  dn12=dn1+(dn2-dn1)*factx
	  dn34=dn3+(dn4-dn3)*factx
C	  idn=dn12+(dn34-dn12)*(y0-iy)+0.5
	  idn=dn12+(dn34-dn12)*(y0-iy)
  220    GRID(NDIM*(I-1)+J+NOFFB) = idn
  200 CONTINUE
  300 CONTINUE

C---- READ THE NEW LINE AND REPEAT THE LOOP OR EXIT
C--------------------------------------------------
  400 CONTINUE
      IF(.NOT.ENDJ(JFIN).AND.ISTART(JFIN).EQ.N) then
		status = ESUCC
		RETURN
	end if

C---- READ THE NEW LINE.
C-----------------------
      I = IHI
      IHI = ILOW
      ILOW = I
      IREC = IREC+1
       	IF(IREC.GT.0.AND.IREC.LE.LINE) then
	   CALL VREAD(IFCB,IREC,IABBR,ICOUNT,samp,ILOW,band,status)
	      if (status .eq. EFAIL) then
		 return
	      end if
	end if
      GO TO 100
      END
