C---------------------------------------------------------------------  
C  SUBROUTINE PSIUD - TO FIND THE UNION,INTERSECTION OR DIFFERENCE      
C                    OF TWO POLYGON SETS.                               
C                                                                       
C            I             SET OPERATION TYPE                           
C                              1  UNION  AUB                            
C                              2  INTER  ANB                            
C                              3  DIFF   A-B                            
C                              4  DIFF   B-A                            
C            AX,AY,IDA     POLYGON SET A.                               
C            BX,BY,IDB     POLYGON SET B.                               
C            RX,RY,IDR     RESULTANT POLYGON SET.                       
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C---------------------------------------------------------------------  
      SUBROUTINE PSIUD(ITYPE,IDA,AX,AY,IDB,BX,BY,IDR,RX,RY)             
      COMMON/PT/X(2500),Y(2500),INT(2500),NPOINT                        
      COMMON/TABLE/LB(2500),LE(2500),NL                                 
      COMMON/D/DEBUG /T/TOL /B/XM1,YM1,XM2,YM2,XM3,YM3,XM4,YM4          
      COMMON/ND/NODE(6,2500),NNODE                                      
      COMMON/TIMES/time1,time2,time3,time4,time5                   
      INTEGER IDR(1),IDA(1),IDB(1)                                      
      LOGICAL DEBUG                                           
      REAL AX(1),AY(1),BX(1),BY(1),RX(1),RY(1)                          
      LOGICAL INT                                                     
      INTEGER*2 NODE                                                    
                                                                        
CCALCULATE "BOX" OF EACH POLYGON-SET.                                  
    1 CALL PSBOX(IDA,AX,AY,XM1,YM1,XM2,YM2)                             
      CALL PSBOX(IDB,BX,BY,XM3,YM3,XM4,YM4)                             
      GO TO 2                                                           
                                                                        
      ENTRY PSIUDB(ITYPE,IDA,AX,AY,X1,Y1,X2,Y2,                         
     2                    IDB,BX,BY,X3,Y3,X4,Y4,                        
     3                    IDR,RX,RY)                                    
      XM1=X1                                                            
      YM1=Y1                                                            
      XM2=X2                                                            
      YM2=Y2                                                            
      XM3=X3                                                            
      YM3=Y3                                                            
      XM4=X4                                                            
      YM4=Y4                                                            
                                                                       
CMAKE SURE BOTH POLYGON-SETS ARE VALID.                                
    2 N=IDA(2)                                                          
      IF(N.LT.0)GO TO 90                                                
      DO 10 I=1,N                                                       
      IF(IDA(I+2).LT.0)GO TO 90                                         
  10  CONTINUE                                                          
      N=IDB(2)                                                          
      IF(N.LT.0)GO TO 90                                                
      DO 15 I=1,N                                                       
      IF(IDB(I+2).LT.0)GO TO 90                                         
  15  CONTINUE                                                          
                                                                        
CDETERMINE BEST TOLERANCE.                                             
      XMAX=AMAX1(ABS(XM1),ABS(XM2),ABS(XM3),ABS(XM4))                   
      DEBUG=.FALSE.                                                     
   4  EXPO=IFIX(ALOG10(XMAX))                                           
      TOL=2.*10**(EXPO-5.)                                              
      NPOINT=0                                                          
      NNODE =0                                                          
C      start = secnds(0.0)
C      time1 = secnds(start)
      IF(.NOT.DEBUG)GO TO 3                                             
      N=IDA(2)+2                                                        
      WRITE(1,100)(IDA(I),I=1,N)                                        
 100  FORMAT(10I4)                                                      
      N=IDA(1)                                                          
      WRITE(1,101)(I,AX(I),AY(I),I=1,N)                                 
 101  FORMAT(I4,'.',2F14.7)                                             
      N=IDB(2)+2                                                        
      WRITE(1,100)(IDB(I),I=1,N)                                        
      N=IDB(1)                                                          
      WRITE(1,101)(I,BX(I),BY(I),I=1,N)                                 
                                                                        
CCOPY SET A TO COMMON AREA.                                            
   3  continue
C	start = secnds(0.0)                                                      
      CALL PSIUD5(IDA,AX,AY)                                            
                                                                        
CCOPY SET B TO COMMON AREA.                                            
      CALL PSIUD5(IDB,BX,BY)                                            
C      time1 = secnds(start)
C      start = secnds(0.0)                                                      
                                                                       
      CALL PSIUD2(IDA,IDB)                                              
C      time2 = secnds(start)
C      start = secnds(0.0)                                                      
      CALL PSIUD6(IDA,IDB)                                              
C      time3 = secnds(start)
C      start = secnds(0.0)                                                      
      GO TO 30                                                          
                                                                       
CENTRY POINT FOR DOING DIFFERENT OPERATION WITH SAME SETS.             
      ENTRY PSIUDR(ITYPE,IDR,RX,RY)                                     
  30  NL=0                                                              
      CALL PSIUD7(ITYPE,IDA,AX,AY,IDB,BX,BY)                            
C      time4 = secnds(start)
C      start = secnds(0.0)                                                      
      IF(NL.GT.2)GO TO 40                                               
  90  IDR(1)=0                                                          
      RETURN                                                            
                                                                       
  40  CALL PSBLD(LB,LE,X,Y,NL,NPOINT,IDR,RX,RY)                         

      RETURN                                                            
      END                                                               
C-----------------------------------------------------------            
C                                                                       
C     SUBROUTINE PSIUD2                                                 
C                                                                       
C     ROUTINE TO SETUP LINKED LIST OF ORIGINAL AND                      
C     INTERSECTION POINTS AND TO GIVE THE POINTS AN                     
C     INITIAL "TYPE" VALUE.                                             
C                                                                       
C     TYPE    CHARACTERISTIC                                            
C      1       INITIAL ENDPOINT OF LINE SEGMENT                         
C              FROM EITHER A OR B                                       
C      2       INTERSECTION POINT NOT COMMON TO ENDPOINTS               
C              OF LINE SEGMENTS FROM EITHER A OR B                      
C      3       INTERSECTION POINT COMMON TO ONE ENDPOINT                
C              OF LINE SEGMENT FROM EITHER A OR B                       
C      4       INTERSECTION POINT COMMON TO AN ENDPOINT                 
C              OF LINE SEGMENTS FROM BOTH A AND B                       
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-----------------------------------------------------------            
      SUBROUTINE PSIUD2(IDA,IDB)                                        
      COMMON/PT/X(2500),Y(2500),INT(2500),NPOINT                        
      COMMON/ND/NODE(6,2500),NNODE                                      
      COMMON/D/DEBUG /T/TOL                                             
     2      /B/XM1,YM1,XM2,YM2,XM3,YM3,XM4,YM4                          
      LOGICAL PARSW,PSBTST,PSPONL,DEBUG,PSEQPT             
      INTEGER IDA(1),IDB(1)                                             
      INTEGER*2 NODE                                                    
      LOGICAL INT                                                     

      NNODE2=NNODE                                                      
      L1=IDA(1)                                                         
      L2=L1+1                                                           
                                                                        
CFIND ALL INTERSECTION POINTS                                          
CI=POLYGON SET A                                                       
CJ=POLYGON SET B                                                       
      DO 30 I=1,L1                                                      
      M=NODE(1,NODE(4,I))                                               
      N=NODE(1,I)                                                       
      X1=X(N)                                                           
      Y1=Y(N)                                                           
      X2=X(M)                                                           
      Y2=Y(M)                                                           
      IF(.NOT.PSBTST(X1,Y1,X2,Y2,XM3,YM3,XM4,YM4,TOL))GO TO 30          
      DO 31 J=L2,NNODE2                                                 
      M=NODE(1,NODE(4,J))                                               
      N=NODE(1,J)                                                       
      X3=X(N)                                                           
      Y3=Y(N)                                                           
      X4=X(M)                                                           
      Y4=Y(M)                                                           
      IF(.NOT.PSBTST(X1,Y1,X2,Y2,X3,Y3,X4,Y4,TOL))GO TO 31              
      IF(DEBUG)WRITE(1,100)I,J,X1,X2,X3,X4,                             
     2                         Y1,Y2,Y3,Y4                              
 100  FORMAT(/'INTERSECTING SEGS:',2I5/4F14.7/4F14.7)                   
      CALL PSLINT(X1,Y1,X2,Y2,X3,Y3,X4,Y4,XI,YI,PARSW)                  
      IF(PARSW)GO TO 31                                                 
      IF(DEBUG)WRITE(1,103)XI,YI                                        
 103  FORMAT(' INTERSECTION POINT:',2F14.7)                             
                                                                        
CFIRST TEST-DOES INTERSECTION POINT FALL ON                            
CLINE SEGMENT FROM POLYGON SET A?                                      
      IF(.NOT.PSPONL(X1,Y1,X2,Y2,XI,YI,TOL))GO TO 31                    
   2  IF(DEBUG)WRITE(1,101)I                                            
 101  FORMAT('   FOUND POINT ON LINE',I4)                               
                                                                       
C SECOND TEST-DOES INTERSECTION POINT FALL ON                           
C LINE SEGMENT FROM POLYGON SET B?                                      
      IF(.NOT.PSPONL(X3,Y3,X4,Y4,XI,YI,TOL))GO TO 31                    
      IF(DEBUG)WRITE(1,101)J                                            
                                                                        
C THE INTERSECTION POINT FALLS ON BOTH LINE                             
C SEGMENTS-NOW DOES IT FALL ON ANY OF THE                               
C END POINTS OF THE LINE SEGMENTS?                                      
  33  ITOT=2                                                            
      K1=0                                                              
      K2=0                                                              
      IF(.NOT.(PSEQPT(XI,YI,X1,Y1,TOL)))GO TO 40                        
      XI=X1                                                             
      YI=Y1                                                             
      ITOT=ITOT+1                                                       
      K1=I                                                              
      IF(DEBUG)WRITE(1,102)NODE(1,K1)                                   
 102  FORMAT('    SAME AS ENDPOINT ',I4)                                
      GO TO 41                                                          
  40  IF(.NOT.(PSEQPT(XI,YI,X2,Y2,TOL)))GO TO 41                        
      XI=X2                                                             
      YI=Y2                                                             
      ITOT=ITOT+1                                                       
      K1=NODE(4,I)                                                      
      IF(DEBUG)WRITE(1,102)NODE(1,K1)                                   
  41  IF(.NOT.(PSEQPT(XI,YI,X3,Y3,TOL)))GO TO 42                        
      XI=X3                                                             
      YI=Y3                                                             
      ITOT=ITOT+1                                                       
      K2=J                                                              
      IF(DEBUG)WRITE(1,102)NODE(1,K2)                                   
      GO TO 43                                                          
  42  IF(.NOT.(PSEQPT(XI,YI,X4,Y4,TOL)))GO TO 43                        
      XI=X4                                                             
      YI=Y4                                                             
      ITOT=ITOT+1                                                       
      K2=NODE(4,J)                                                      
      IF(DEBUG)WRITE(1,102)NODE(1,K2)                                   
  43  IF(ITOT.NE.3 .AND. ITOT.NE.4)GO TO 34                             
                                                                        
CTYPE=3,4. DOES INTERSECTION POINT HAVE MORE THAN ONE NAME?            
C          FIND FIRST OCCURANCE.                                       
      DO 35 L=1,NPOINT                                                  
      IF(PSEQPT(XI,YI,X(L),Y(L),TOL))GO TO 36                           
  35  CONTINUE                                                          
  36  IF(K1.NE.0)NODE(1,K1)=L                                           
      IF(K2.NE.0)NODE(1,K2)=L                                           
      IF((K1.NE.0 .AND. INT(NODE(1,K1))).OR.                            
     1   (K2.NE.0 .AND. INT(NODE(1,K2))))GO TO 31                       
      IF(K1.NE.0)INT(NODE(1,K1))=.TRUE.                                 
      IF(K2.NE.0)INT(NODE(1,K2))=.TRUE.                                 
  34  CALL PSIUD3(XI,YI,I,J,K1,K2,ITOT)                                 
  31  CONTINUE                                                          
  30  CONTINUE                                                          
      RETURN                                                            
      END                                                               
C-----------------------------------------------------------            
C                                                                       
C     SUBROUTINE PSIUD3(XI,YI,I,J,K1,K2,ITYPE)                          
C                                                                       
C     ROUTINE TO ADD THE PROPER NUMBER OF NODES TO THE                  
C     LINKED LISTS OF POLYGON SETS A&B ACCORDING TO THE                 
C     INTERSECTION TYPE AND TO MAKE A LINK BETWEEN THE                  
C     LISTS.                                                            
C                                                                       
C     XI,YI        INTERSECTION POINT                                   
C     I,J          INDEX NUMBER OF LINE SEGMENT FROM A&B                
C     K1,K2        INDEX NUMBER OF LINE SEGMENT FROM A&B                
C                  WITH COMMON ENDPOINT                                 
C     ITYPE        INITIAL INTERSECTION TYPE                            
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-----------------------------------------------------------            
      SUBROUTINE PSIUD3(XI,YI,I,J,K1,K2,ITYPE)                          
      COMMON/PT/X(2500),Y(2500),INT(2500),NPOINT                        
      COMMON/ND/NODE(6,2500),NNODE                                      
      INTEGER TEMP                                                      
      LOGICAL INT                                                     
      INTEGER*2 NODE                                                    
      IF(ITYPE.EQ.3)GO TO 3                                             
      IF(ITYPE.EQ.4)GO TO 4                                             
                                                                        
C TYPE=2  ADD TWO NEW NODES AND ONE NEW POINT                           
   2  NPOINT=NPOINT+1                                                   
      X(NPOINT)=XI                                                      
      Y(NPOINT)=YI                                                      
      INT(NPOINT)=.TRUE.                                                
      NEXT=NNODE+2                                                      
      CALL PSIUD4(I,NPOINT,NEXT,ITYPE)                                  
      NEXT=NNODE                                                        
      CALL PSIUD4(J,NPOINT,NEXT,ITYPE)                                  
      RETURN                                                            
                                                                       
C TYPE=3  ADD ONE NEW NODE TO THE CORRECT LIST                          
   3  IF(K1.NE.0)GO TO 7                                                
      TEMP=NODE(1,K2)                                                   
      CALL PSIUD4(I,TEMP,K2,ITYPE)                                      
      NODE(2,K2)=3                                                      
      RETURN                                                            
   7  IF(K2.NE.0)GO TO 4                                                
      TEMP=NODE(1,K1)                                                   
      CALL PSIUD4(J,TEMP,K1,ITYPE)                                      
      NODE(2,K1)=3                                                      
      RETURN                                                            
                                                                        
C TYPE=4  DON'T ADD ANY NEW NODES                                       
C JUST CONNECT THE EXISTING ONES                                        
   4  NODE(2,K1)=ITYPE                                                  
      NODE(2,K2)=ITYPE                                                  
      NODE(3,K1)=K2                                                     
      NODE(3,K2)=K1                                                     
                                                                        
CMAKE THE NODES POINT TO THE SAME POINT.                               
      IF(K2.LT.K1)NODE(1,K1)=NODE(1,K2)                                 
      IF(K1.LE.K2)NODE(1,K2)=NODE(1,K1)                                 
      RETURN                                                            
      END                                                               
C---------------------------------------------------------              
C                                                                       
C     SUBROUTINE PSIUD4(I,K,L,ITYPE)                                    
C                                                                       
C     ROUTINE TO TEMPORARILY ADD NODES TO A LIST OF                     
C     INTERSECTION POINTS WHICH FALL ON LINE SEGMENT I.                 
C                                                                       
C     I            INDEX NUMBER OF LINE SEGMENT                         
C     K            INDEX NUMBER OF X,Y COORDINATES OF                   
C                  INTERSECTION POINT                                   
C     L            INDEX NUMBER OF NODE ON OTHER LIST                   
C                  FOR LINKING PURPOSES                                 
C     ITYPE        INITIAL INTERSECTION TYPE                            
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C---------------------------------------------------------              
      SUBROUTINE PSIUD4(I,K,L,ITYPE)                                    
      COMMON/ND/NODE(6,2500),NNODE                                      
      INTEGER*2 NODE                                                    
      IT=NODE(6,I)                                                      
      NNODE=NNODE+1                                                     
      NODE(6,I)=NNODE                                                   
      NODE(6,NNODE)=IT                                                  
      NODE(1,NNODE)=K                                                   
      NODE(2,NNODE)=ITYPE                                               
      NODE(3,NNODE)=L                                                   
      NODE(3,L)=NNODE                                                   
      RETURN                                                            
      END                                                               
C-----------------------------------------------------------            
C                                                                       
C     SUBROUTINE PSIUD5(ID,XP,YP)                                       
C                                                                       
C     ROUTINE TO INITIALIZE NODES WITH FORWARD AND                      
C     BACKWARD POINTERS, INTERSECTION TYPE, AND                         
C     X,Y COORDINATE POINTER.                                           
C                                                                       
C     -------------------------------------------------                 
C     ]  X,Y  ] TYPE  ] LINK  ]  FWD  ]  BWD  ] INTER ]                 
C     ]  PTR  ]       ]       ]  PTR  ]  PTR  ] PNTS  ]                 
C     -------------------------------------------------                 
C         1       2       3       4       5       6                     
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-----------------------------------------------------------            
      SUBROUTINE PSIUD5(ID,XP,YP)                                       
      COMMON/ND/NODE(6,2500),NNODE                                      
     2      /PT/X(2500),Y(2500),INT(2500),NPOINT                        
      INTEGER ID(1)                                                     
      REAL XP(1),YP(1)                                                  
      LOGICAL INT                                                     
      INTEGER*2 NODE                                                    
      NPG=ID(2)                                                         
      K=0                                                               
                                                                        
CLOOK AT EACH POLYGON IN SET.                                          
      DO 10 I=1,NPG                                                     
      NPT=ID(I+2)                                                       
                                                                        
CLOOK AT EACH POINT.                                                   
      DO 15 J=1,NPT                                                     
      K=K+1                                                             
      NNODE=NNODE+1                                                     
      NODE(1,NNODE)=NPOINT+1                                            
      NODE(2,NNODE)=1                                                   
      NODE(3,NNODE)=0                                                   
      NODE(4,NNODE)=NNODE+1                                             
      NODE(5,NNODE)=NNODE-1                                             
      NODE(6,NNODE)=0                                                   
      NPOINT=NPOINT+1                                                   
      X(NPOINT)=XP(K)                                                   
      Y(NPOINT)=YP(K)                                                   
      INT(NPOINT)=.FALSE.                                               
  15  CONTINUE                                                          
      NODE(4,NNODE)=NNODE-NPT+1                                         
      NODE(5,NNODE-NPT+1)=NNODE                                         
  10  CONTINUE                                                          
      RETURN                                                            
      END                                                               
C-----------------------------------------------------------            
C                                                                       
C     SUBROUTINE PSIUD6                                                 
C                                                                       
C     ROUTINE TO SORT ALL THE INTERSECTION POINTS COMMON                
C     TO A LINE SEGMENT ACCORDING TO DISTANCE FROM THE                  
C     ENDPOINT, AND TO INSERT THEM IN THE LINKED LIST.                  
C                                                                       
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-----------------------------------------------------------            
      SUBROUTINE PSIUD6(IDA,IDB)                                        
      COMMON/PT/X(2500),Y(2500),INT(2500),NPOINT                        
      COMMON/ND/NODE(6,2500),NNODE                                      
      INTEGER IDA(1),IDB(1)                                             
      LOGICAL INT                 
 
C 1-15-87 B.AILTS increased the arrays indx and dist to 100
C----------------------------------------------------------             
      INTEGER*2 INDX(100),NODE,IT                                        
      REAL DIST(100)                                                     
      L=IDA(1)+IDB(1)                                                   
                                                                       
CTHE ALGORITHM IS DONE LINE SEGMENT BY LINE SEGMENT                    
      DO 17 N=1,L                                                       
      LAST=N                                                            
      IPTR=0                                                            
      M=N                                                               
                                                                       
CTHE INDEX NUMBERS OF THE INTERSECTION POINTS ARE                      
CSTORED IN A TEMPORARY VECTOR INDX(100)                                
   9  M=NODE(6,M)                                                       
      IF(M.EQ.0)GO TO 10                                                
      IPTR=IPTR+1                                                       
      INDX(IPTR)=M                                                      
      GO TO 9                                                           
  10  IF(IPTR.EQ.0)GO TO 17                                             
      IF(IPTR.EQ.1)GO TO 8                                              
      N2=NODE(1,N)                                                      
                                                                        
CTHE DISTANCES ARE STORED IN A TEMPORARY VECTOR DIST(50)               
      DO 11 I=1,IPTR                                                    
  11  DIST(I)=((X(N2)-X(NODE(1,INDX(I))))**2+                           
     1         (Y(N2)-Y(NODE(1,INDX(I))))**2)                           
                                                                        
CTHE DISTANCE VECTOR IS SORTED IN ASCENDING ORDER                      
CWITH THE INDEX VECTOR FOLLOWING                                       
      I2=IPTR-1                                                         
      DO 13 I=1,I2                                                      
      I3=I+1                                                            
      DO 12 J=I3,IPTR                                                   
      IF(DIST(I).LT.DIST(J))GO TO 12                                    
      IT=INDX(I)                                                        
      DT=DIST(I)                                                        
      INDX(I)=INDX(J)                                                   
      DIST(I)=DIST(J)                                                   
      INDX(J)=IT                                                        
      DIST(J)=DT                                                        
  12  CONTINUE                                                          
  13  CONTINUE                                                          
                                                                       
CTHE NODES ARE INSERTED IN THE LINKED LIST                             
   8  DO 16 I=1,IPTR                                                    
      N2=INDX(I)                                                        
      NODE(4,N2)=NODE(4,LAST)                                           
      NODE(5,N2)=LAST                                                   
      NODE(4,LAST)=N2                                                   
      NODE(5,NODE(4,N2))=N2                                             
   7  LAST=N2                                                           
  16  CONTINUE                                                          
  17  CONTINUE                                                          
      RETURN                                                            
      END                                                               
C-----------------------------------------------------------            
C                                                                       
C     SUBROUTINE PSIUD7(ITYPE)                                          
C                                                                       
C     ROUTINE TO CALL SUBROUTINE PSIUD8 FOR EACH CIRCUIT OF             
C     EACH POLYGON SET.                                                 
C                                                                       
C     ITYPE         SET OPERATION TYPE                                  
C                                                                       
C     1             UNION       A U B                                   
C     2             INTER       A N B                                   
C     3             DIFFERENCE  A - B                                   
C     4             DIFFERENCE  B - A                                   
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-----------------------------------------------------------            
      SUBROUTINE PSIUD7(ITYPE,IDA,AX,AY,IDB,BX,BY)                      
      COMMON/B/X1,Y1,X2,Y2,X3,Y3,X4,Y4                                  
      INTEGER IDA(1),IDB(1)                                             
      REAL AX(1),AY(1),BX(1),BY(1)                                      
      NTOP=1                                                            
      L1=IDA(2)                                                         
      DO 10 I=1,L1                                                      
      CALL PSIUD8(1,ITYPE,NTOP,IDB,BX,BY,X3,Y3,X4,Y4)                   
      NTOP=NTOP+IDA(I+2)                                                
  10  CONTINUE                                                          
      L1=IDB(2)                                                         
      DO 20 I=1,L1                                                      
      CALL PSIUD8(2,ITYPE,NTOP,IDA,AX,AY,X1,Y1,X2,Y2)                   
      NTOP=NTOP+IDB(I+2)                                                
  20  CONTINUE                                                          
      RETURN                                                            
      END                                                               
C-----------------------------------------------------------            
C                                                                       
C     SUBROUTINE PSIUD8(ICURVE,ITYPE,NTOP,ID,XP,YP,XMN,YMN,XMX,YMX)     
C                                                                       
C     ROUTINE TO ASSIGN A NEW TYPE TO THE LINES, AND TO                 
C     DECIDE WHICH LINES ARE TO BE USED IN THE RESULTANT                
C     POLYGON SET.                                                      
C                                                                       
C     NTOP         POINTER TO THE STARTING POINT                        
C                  OF A POLYGON IN THE LINKED LIST                      
C     ID,XP,YP     OTHER POLYGON SET                                    
C     ITYPE        SET OPERATION TYPE                                   
C     ICURVE       NUMBER OF CURRENT POLYGON SET                        
C                  1=A   2=B                                            
C                                                                       
C     LINE TYPE    DESCRIPTION                                          
C            1     LINE SEGMENT OUTSIDE OTHER SET                       
C            2     LINE SEGMENT INSIDE OTHER SET                        
C     ONEWAY 3     TWO PARALLEL LINE SEGMENTS RUNNING                   
C                  IN THE SAME DIRECTION                                
C     TWOWAY 3     TWO PARALLEL LINE SEGMENTS RUNNING IN                
C                  OPPOSITE DIRECTIONS                                  
C                                                                       
C     THESE DECISION RULES ARE USED:                                    
C                                                                       
C     OPERATION    ACCEPTABLE LINES                                     
C                                                                       
C     UNION AUB    ALL 1S AND ONEWAY 3S                                 
C     INTER ANB    ALL 2S AND ONEWAY 3S                                 
C     DIFF  A-B    ALL 1S FROM POLYGON SET A                            
C                  ALL 2S FROM POLYGON SET B                            
C                  ALL TWOWAY 3S                                        
C     DIFF  B-A    ALL 1S FROM POLYGON SET B                            
C                  ALL 2S FROM POLYGON SET A                            
C                  ALL TWOWAY 3S                                        
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C----------------------------------------------------------             
                                                                       
      SUBROUTINE PSIUD8(ICURVE,TYPE,NTOP,ID,XP,YP,XMN,YMN,XMX,YMX)      
      COMMON/ND/NODE(6,2500),NNODE /PT/X(2500),Y(2500),INT(2500),NPOINT       
     2      /T/TOL                                                      
      INTEGER ID(1),TYPE                                                
      REAL XP(1),YP(1)                                                  
      LOGICAL PSIO,PSBTST,ADDSW                                         
      INTEGER*2 NODE                                                    
      LOGICAL INT                                                     
      N=NTOP                                                            
      GO TO 9                                                           
   8  IF(INT(NODE(1,N)))GO TO 9                                         
      IF(ADDSW)CALL PSIUD9(LTYPE,N,TYPE)                                
      GO TO 11                                                          
   9  ADDSW=.FALSE.                                                     
      IF(.NOT.(NODE(2,N).EQ.3 .OR. NODE(2,N).EQ.4))GO TO 10             
                                                                        
CTYPE=3   PARALLEL LINE SEGMENTS                                       
      NODE(2,N)=3                                                       
      N1=NODE(1,NODE(4,N))                                              
      N2=NODE(1,NODE(4,NODE(3,N)))                                      
      N3=NODE(1,NODE(5,NODE(3,N)))                                      
      IF(.NOT.((X(N1).EQ.X(N2)).AND.                                    
     1         (Y(N1).EQ.Y(N2))))GO TO 31                               
                                                                        
CTYPE=3   ONEWAY(RUNNING IN THE SAME DIRECTION)                        
      IF(.NOT.((TYPE.EQ.1 .AND. ICURVE.EQ.1).OR.                        
     1   (TYPE.EQ.2 .AND. ICURVE.EQ.1)))GO TO 11                        
      ADDSW=.TRUE.                                                      
      CALL PSIUD9(1,N,TYPE)                                             
      LTYPE=1                                                           
      GO TO 11                                                          
                                                                        
CTYPE=3   TWOWAY(RUNNING IN OPPOSITE DIRECTIONS)                       
  31  IF(.NOT.((X(N1).EQ.X(N3)).AND.                                    
     1         (Y(N1).EQ.Y(N3))))GO TO 10                               
      IF(.NOT.((TYPE.EQ.3 .AND. ICURVE.EQ.1).OR.                        
     1   (TYPE.EQ.4 .AND. ICURVE.EQ.2)))GO TO 11                        
      ADDSW=.TRUE.                                                      
      CALL PSIUD9(3,N,TYPE)                                             
      LTYPE=3                                                           
      GO TO 11                                                          
                                                                        
CTYPE=1   LINE SEGMENT OUTSIDE OTHER POLYGON SET                       
  10  NODE(2,N)=1                                                       
      I1=NODE(1,NODE(4,N))                                              
      I2=NODE(1,N)                                                      
      IF(.NOT.PSBTST(XMN,YMN,XMX,YMX,                                   
     2               X(I1),Y(I1),X(I2),Y(I2),TOL))GO TO 12              
      XHALF=(DBLE(X(I1))+DBLE(X(I2)))/2.                                
      YHALF=(DBLE(Y(I1))+DBLE(Y(I2)))/2.                                
      IF(PSIO(ID,XP,YP,XHALF,YHALF,TOL/2.))GO TO 13                     
  12  IF(.NOT.((TYPE.EQ.1).OR.                                          
     1   (TYPE.EQ.3 .AND. ICURVE.EQ.1).OR.                              
     2   (TYPE.EQ.4 .AND. ICURVE.EQ.2)))GO TO 11                        
      ADDSW=.TRUE.                                                      
      CALL PSIUD9(1,N,TYPE)                                             
      LTYPE=1                                                           
      GO TO 11                                                          
  13  NODE(2,N)=2                                                       
                                                                         
CTYPE=2   LINE SEGMENT INSIDE OTHER POLYGON SET                        
      IF(.NOT.((TYPE.EQ.2).OR.                                          
     1   (TYPE.EQ.3 .AND. ICURVE.EQ.2).OR.                              
     2   (TYPE.EQ.4 .AND. ICURVE.EQ.1)))GO TO 11                        
      ADDSW=.TRUE.                                                      
      CALL PSIUD9(2,N,TYPE)                                             
      LTYPE=2                                                           
  11  N=NODE(4,N)                                                       
      IF(.NOT.(N.EQ.NTOP))GO TO 8                                       
      RETURN                                                            
      END                                                               
C-------------------------------------------------------------          
C                                                                       
C     SUBROUTINE PSIUD9(I,N,TYPE)                                       
C                                                                       
C
C     ROUTINE TO ADD A LINE TO THE TEMPORARY ACCEPTABLE                 
C     LINE VECTORS LB,LE                                                
C                                                                       
C     I            INTERSECTION TYPE                                    
C     N            INDEX NUMBER OF NODE WHOSE LINE SEGMENT              
C                  IS TO BE ADDED                                       
C     TYPE         SET OPERATION TYPE                                   
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-------------------------------------------------------------          
      SUBROUTINE PSIUD9(I,N,TYPE)                                       
      COMMON/TABLE/LB(2500),LE(2500),NL                                 
     2      /ND/NODE(6,2500),NNODE                                      
      INTEGER TYPE                                                      
      INTEGER*2 NODE                                                    
      NL=NL+1                                                           
      IF((TYPE.EQ.3 .OR. TYPE.EQ.4) .AND.                               
     1                     (I.EQ.2))GO TO 12                            
      LB(NL)=NODE(1,N)                                                  
      LE(NL)=NODE(1,NODE(4,N))                                          
      RETURN                                                            
                                                                        
CIF INTERSECTION TYPE=2 AND SET OPERATION TYPE=3 OR 4                  
CTHE LINE SEGMENT IS REVERSED                                          
  12  LB(NL)=NODE(1,NODE(4,N))                                          
      LE(NL)=NODE(1,N)                                                  
      RETURN                                                            
      END                                                               
C-----------------------------------------------------------------------
C SUBROUTINE PSBLD - TO CONVERT THE OUTPUT FROM THE "SKETCH" SUBROUTINE 
C                     (LB,LE,PX,PY,NL,NP) INTO ITS POLYGON SET DEFINITIO
C                     (ID,X,Y).                                         
C PLANE(I) IS .TRUE. WHEN POLYGON "I" OF THE SET IS A COMPLETELY        
C CLOSED CIRCUIT.                                                       
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-----------------------------------------------------------------------
      SUBROUTINE PSBLD(LB,LE,PX,PY,NL,NP,ID,X,Y)                        
      DIMENSION LB(1),LE(1),PX(1),PY(1),ID(1),X(1),Y(1)                 
      INTEGER FRONT,BACK                                                
                                                                        
CINI TIALIZE NUMBER OF POINTS(NPT), NUMBER OF POLYGONS(NCIR).           
      NPT=0                                                             
      NCIR=0                                                            
      ID(1)=0                                                           
      IF(NL.LE.0)RETURN                                                 
                                                                        
CSTA RT THE LINE "CHAIN".                                               
      DO 20 J=1,NL                                                      
      IF(LB(J).LT.0)GO TO 20                                            
      LB(J)=-IABS(LB(J))                                                
      NCIR=NCIR+1                                                       
      ID(NCIR+2)=0                                                      
      BACK=IABS(LB(J))                                                  
      FRONT=LE(J)                                                       
      CALL PSBLD2(PX(BACK),PY(BACK),X,Y,NPT,ID,NCIR)                    
      CALL PSBLD2(PX(FRONT),PY(FRONT),X,Y,NPT,ID,NCIR)                  
      GO TO 5                                                           
                                                                        
CA CONNECTION WAS MADE.                                                
  13  LB(I)=-IABS(LB(I))                                                
                                                                       
CWAS THE CIRCUIT CLOSED?                                               
      IF(FRONT.EQ.BACK)GO TO 20                                         
      CALL PSBLD2(PX(FRONT),PY(FRONT),X,Y,NPT,ID,NCIR)                  
                                                                         
CSEARCH THE REMAINDER OF THE LIST FOR A CONTIGUOUS LINE SEGMENT.       
   5  DO 10 I=J,NL                                                      
      IF(LB(I).LT.0)GO TO 10                                            
      IF(FRONT.NE.LB(I))GO TO 6                                         
      FRONT=LE(I)                                                       
      GO TO 13                                                          
   6  IF(FRONT.NE.LE(I))GO TO 10                                        
      FRONT=LB(I)                                                       
      GO TO 13                                                          
  10  CONTINUE                                                          
                                                                       
CSET OF LINES IS NOT A CLOSED CIRCUIT.                                 
      ID(NCIR+2)=-ID(NCIR+2)                                            
  20  CONTINUE                                                          
                                                                       
CFIX "LB(*)".                                                          
      DO 30 I=1,NL                                                      
      LB(I)=IABS(LB(I))                                                 
  30  CONTINUE                                                          
      ID(1)=NPT                                                         
      ID(2)=NCIR                                                        
                                                                       
CIS POLYGON-SET A LINE-SET?                                            
      DO 40 I=1,NCIR                                                    
      IF(ID(I+2).GT.0)RETURN                                            
  40  CONTINUE                                                          
      DO 45 I=1,NCIR                                                    
      ID(I+2)=IABS(ID(I+2))                                             
  45  CONTINUE                                                          
      ID(2)=-IABS(ID(2))                                                
      RETURN                                                            
      END                                                               
C---------------------------------------------------------------        
C PSBLD2 - USED BY PSBLD TO ADD A POINT TO                              
C          THE POLYGON SET DATA STRUCTURE.                              
C------------------------------------------------------------           
      SUBROUTINE PSBLD2(PX,PY,X,Y,NPT,ID,NCIR)                          
      DIMENSION X(1),Y(1),ID(1)                                         
      NPT=NPT+1                                                         
      X(NPT)=PX                                                         
      Y(NPT)=PY                                                         
      ID(NCIR+2)=ID(NCIR+2)+1                                           
      RETURN                                                            
      END                                                               
C-----------------------------------------------------                  
C PSBOX - ROUTINE TO CALCULATE THE "BOX" PARAMETERS                     
C          OF A POLYGON-SET.                                            
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-----------------------------------------------------                  
      SUBROUTINE PSBOX(ID,X,Y,XMN,YMN,XMX,YMX)                          
      INTEGER ID(1)                                                     
      REAL X(1),Y(1)                                                    
      XMN=X(1)                                                          
      YMN=Y(1)                                                          
      XMX=X(1)                                                          
      YMX=Y(1)                                                          
      NPT=IABS(ID(1))                                                   
      DO 10 I=2,NPT                                                     
      XMN=AMIN1(XMN,X(I))                                               
      YMN=AMIN1(YMN,Y(I))                                               
      XMX=AMAX1(XMX,X(I))                                               
      YMX=AMAX1(YMX,Y(I))                                               
  10  CONTINUE                                                          
      RETURN                                                            
      END                                                               
C---------------------------------------------------------------------  
C PSBTST - FUNCTION TO DECIDE IF TWO BOXES OVERLAP OR NOT.              
C            .TRUE. - OVERLAP                                           
C            .FALSE.- NO-OVERLAP                                        
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C---------------------------------------------------------------------  
      LOGICAL FUNCTION PSBTST(X1,Y1,X2,Y2,X3,Y3,X4,Y4,TOL)              
      PSBTST=.TRUE.                                                     
      XMN1=AMIN1(X1,X2)-TOL                                             
      YMN1=AMIN1(Y1,Y2)-TOL                                             
      XMX1=AMAX1(X1,X2)+TOL                                             
      YMX1=AMAX1(Y1,Y2)+TOL                                             
      XMN2=AMIN1(X3,X4)                                                 
      YMN2=AMIN1(Y3,Y4)                                                 
      XMX2=AMAX1(X3,X4)                                                 
      YMX2=AMAX1(Y3,Y4)                                                 
      IF(XMN1.GT.XMX2 .OR.                                              
     2   XMX1.LT.XMN2 .OR.                                              
     3   YMN1.GT.YMX2 .OR.                                              
     4   YMX1.LT.YMN2)PSBTST=.FALSE.                                    
      RETURN                                                            
      END                                                               
C-------------------------------------------------------------          
C SUBROUTINE PSLINT - ROUTINE TO FIND THE INTERSECTION OF TWO           
C                     LINE-SEGMENTS.                                    
C                                                                       
C     XP,YP        INTERSECTION POINT                                   
C     X1,Y1,X2,Y2  LINE SEGMENT FROM POLYGON SET A                      
C     X3,Y3,X4,Y4  LINE SEGMENT FROM POLYGON SET B                      
C     PARSW        .TRUE. IF LINES WERE PARALELL                        
C                                                                       
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-------------------------------------------------------------          
      SUBROUTINE PSLINT(X1,Y1,X2,Y2,X3,Y3,X4,Y4,XP,YP,PARSW)            
      LOGICAL PARSW,PSEQPT,ERR                                          
      REAL*8 A1,B1,C1,A2,B2,C2                                          
                                                                        
      PARSW=.TRUE.                                                      
      A1=DBLE(Y1)-DBLE(Y2)                                              
      B1=DBLE(X2)-DBLE(X1)                                              
      C1=-(A1*X1+B1*Y1)                                                 
      A2=DBLE(Y3)-DBLE(Y4)                                              
      B2=DBLE(X4)-DBLE(X3)                                              
      C2=-(A2*X3+B2*Y3)                                                 
      CALL PSNRM8(A1,B1,C1,ERR)                                         
      IF(ERR)RETURN                                                     
      CALL PSNRM8(A2,B2,C2,ERR)                                         
      IF(ERR)RETURN                                                     
   6  D=(A1*B2)-(A2*B1)                                                 
                                                                        
CARE LINES PARALLEL?                                                   
      IF(ABS(D) .LT. .005) RETURN                                       
 
                                                                       
CLINES INTERSECT.                                                      
    1 PARSW=.FALSE.                                                     
      BIG=AMAX1(ABS(X1),ABS(Y1),ABS(X2),ABS(Y2))                        
      IEXPO=IFIX(ALOG10(BIG))                                           
      TOL=5.*10**(IEXPO-6)                                              
      IF(.NOT.PSEQPT(X1,Y1,X3,Y3,TOL) .AND.                             
     2   .NOT.PSEQPT(X1,Y1,X4,Y4,TOL))GO TO 2                           
      XP=X1                                                             
      YP=Y1                                                             
      RETURN                                                            
                                                                        
   2  IF(.NOT.PSEQPT(X2,Y2,X3,Y3,TOL) .AND.                             
     2   .NOT.PSEQPT(X2,Y2,X4,Y4,TOL))GO TO 3                           
      XP=X2                                                             
      YP=Y2                                                             
      RETURN                                                            
                                                                        
                                                                       
CPIVOT.                                                                
   3  IF(DABS(A1).GT.DABS(A2))GO TO 4                                   
      YP=(-C1+A1*C2/A2)/(B1-A1*B2/A2)                                   
      XP=-((B2*YP)+C2)/A2                                               
      RETURN                                                            
                                                                        
   4  YP=(-C2+A2*C1/A1)/(B2-A2*B1/A1)                                   
      XP=-((B1*YP)+C1)/A1                                               
      RETURN                                                            
      END                                                               
C------------------------------------------------------------------     
C PSEQPT - LOGICAL FUNCTION TO DECIDE IF TWO COORDINATES ARE            
C          EQUAL (.TRUE.) OR NOT (.FALSE.).                             
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C------------------------------------------------------------------     
      LOGICAL FUNCTION PSEQPT(X1,Y1,X2,Y2,TOL)                          
      LOGICAL PSPONL                                                    
      PSEQPT=.FALSE.                                                    
      IF(PSPONL(X2-TOL,Y2-TOL,X2+TOL,Y2+TOL,X1,Y1,0.))PSEQPT=.TRUE.     
      RETURN                                                            
      END                                                               
C-----------------------------------------------------------------      
C LOGICAL FUNCTION PSIO - TO DETERMINE IF A POINT(X,Y) IS IN            
C                         THE POLYGON SET (ID,X,Y).                     
C A VERTICAL LINE IS ASSUMED THROUGH XP,YP, AND THE SUM                 
C OF ALL THE POLYGON SEGMENTS IT INTERSECTS ABOVE THE POINT             
C IS FOUND. IF THE SUM IS EVEN, THE POINT IS OUTSIDE THE                
C POLYGON SET.                                                          
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C-----------------------------------------------------------------      
      LOGICAL FUNCTION PSIO(ID,X,Y,XP,YP,TOL2)                          
      DIMENSION ID(1),X(1),Y(1)                                         
      REAL*8 XP2                                                        
      INTEGER P1,P2                                                     
                                                                        
CINITIALIZE FUNCTION(PSIO),NUMBER OF INTERSECTIONS ABOVE THE POINT,    
CCOORDINATE INDEX(K),NUMBER OF POLYGONS(NPOLY).                        
      PSIO=.TRUE.                                                       
      TOL=TOL2                                                          
      IF(TOL.EQ.0.)TOL=PSTOL(ID,X,Y)                                    
      ITOT=0                                                            
      K=0                                                               
      NPOLY=IABS(ID(2))                                                 
                                                                        
CANY X COORDINATE IN "X" EQUAL TO "XP"?                                
      XP2=XP                                                            
      NPT=IABS(ID(1))                                                   
      ICOUNT=0                                                          
   1  DO 2 I=1,NPT                                                      
      IF(XP2.NE.X(I))GO TO 2                                            
      XP2=XP2+TOL                                                       
      ICOUNT=ICOUNT+1                                                   
      IF(ICOUNT.LE.NPT)GO TO 1                                          
      PRINT 100                                                         
 100  FORMAT(' PSIO: SOMETHING IS WRONG.')                              
      RETURN                                                            
   2  CONTINUE                                                          
                                                                        
CCHECK EACH POLYGON "I" IN THE SET.                                    
      DO 11 I=1,NPOLY                                                   
      NPTS=ID(I+2)                                                      
      IF(NPTS.LE.0)GO TO 20                                             
                                                                        
CCHECK EACH LINE "J" IN  POLYGON "I".                                  
      DO 10 J=1,NPTS                                                    
      P1=K+J                                                            
      P2=K+J+1                                                          
      IF(J.EQ.NPTS)P2=K+1                                               
      IF(.NOT.((X(P1).LT.XP2 .AND. X(P2).GT.XP2) .OR.                   
     2   (X(P1).GT.XP2 .AND. X(P2).LT.XP2)))GO TO 10                    
      IF(YP.LT.Y(P1)-TOL .AND. YP.LT.Y(P2)-TOL)GO TO 5                  
      DIS=((Y(P2)-Y(P1))/(X(P2)-X(P1)))*(XP2-X(P1))+(Y(P1)-YP)          
      IF(ABS(DIS).LE.TOL)RETURN                                         
      IF(DIS.LT.0.)GO TO 10                                             
   5  ITOT=ITOT+1                                                       
  10  CONTINUE                                                          
  20  K=K+IABS(NPTS)                                                    
  11  CONTINUE                                                          
                                                                        
CIS TOTAL ODD OR EVEN?                                                 
      PSIO=(ITOT/2)*2.NE.ITOT                                           
      RETURN                                                            
      END                                                               
C---------------------------------------------------------------------  
C PSPONL - FUNCTION TO DECIDE IF A POINT IS ON A LINE-SEGMENT OR NOT.   
C                                                                       
C JAMES A. TURNER (313)764-1340                                         
C ARCHITECTURAL RESEARCH LABORATORY                                     
C 2000 BONISTEEL BLVD.                                                  
C ANN ARBOR, MI. 48109                                                  
C---------------------------------------------------------------------  
      LOGICAL FUNCTION PSPONL(X1,Y1,X2,Y2,XP,YP,TOL)                    
      PSPONL=.TRUE.                                                     
      XMIN=AMIN1(X1,X2)-TOL                                             
      YMIN=AMIN1(Y1,Y2)-TOL                                             
      XMAX=AMAX1(X1,X2)+TOL                                             
      YMAX=AMAX1(Y1,Y2)+TOL                                             
      IF(XP.LT.XMIN .OR.                                                
     2   XP.GT.XMAX .OR.                                                
     3   YP.LT.YMIN .OR.                                                
     4   YP.GT.YMAX)PSPONL=.FALSE.                                      
      RETURN                                                            
      END                                                               
C-----------------------------------------------------------            
C ROUTINE:  PSNRM8                                                      
C PURPOSE:  TO NORMALIZE A VECTOR IN 2-SPACE                            
C                                                                       
C DATE:     APRIL '81  J.A.T.                                           
C-----------------------------------------------------------            
      SUBROUTINE PSNRM8(A,B,C,ERR)                                      
      REAL*8 D,A,B,C                                                    
      LOGICAL ERR                                                       
      ERR=.TRUE.                                                        
      D=DSQRT(A*A+B*B)                                                  
      IF(D.EQ.0.)RETURN                                                 
      A=A/D                                                             
      B=B/D                                                             
      C=C/D                                                             
      ERR=.FALSE.                                                       
      RETURN                                                            
      END                                                               
C-------------------------------------------------------------          
C PSTOL - FUNCTION TO DETERMINE THE "TOLERANCE" OF A                    
C         POLYGON-SET.                                                  
C-------------------------------------------------------------          
      FUNCTION PSTOL(ID,X,Y)                                            
      DIMENSION ID(1),X(1),Y(1)                                         
      CALL PSBOX(ID,X,Y,X1,Y1,X2,Y2)                                    
      XMAX=AMAX1(ABS(X1),ABS(Y1),ABS(X2),ABS(Y2))                       
      EXPO=IFIX(ALOG10(XMAX))                                           
      PSTOL=1.*10.**(EXPO-5.)                                           
      RETURN                                                            
      END                                                               
                                                                       
