C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
C	Subroutine: VREAD
C
C	This routine mimics the VICAR read routine by providing
C	similar capability. The calling sequence is NOT the same
C	though. It is assumed that the user has previously
C	opened the file using a TAE OPEN.
C
C	Parameters:
C
C		IFCB	image file control block		i*4,input
C		RECNUM	record number to read, if 0 then	I*4,input
C			read the next record
C		SKPBYT  # of bytes to skip before reading	I*4,input
C		NBYTES	total # of bytes to read		I*4,input
C		BUFFER	image line return buffer		C*1,returned
C		status	return status indicator			I*4,returned
C			0 = success
C			4 = end of file
C			8 = read error
C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C	D. Nichols    2/18/82
C	B.AILTS	      4/3/87   put in newLAS I/O
C  	B.Ailts	      9/28/88  Updated to LAS 5.0
C	B.Ailts	     11/25/88  Updated call to eread
C-------------------------------------------------------------
	SUBROUTINE VREAD (fdesc,RECORD,SKPBYT,NBYTES,ns,ilow,band,status)

	include 'las.fin'

	real*4 BUFFER(MAXNS,2)
	real*4 LOCBUF(MAXNS)

	integer*4 band
	integer*4 bufcnt
	integer*4 eread
	integer*4 fdesc
	integer*4 ilow
	integer*4 ix
	integer*4 nbytes
	integer*4 ns
	integer*4 numbyt
	integer*4 record
	integer*4 skpbyt
	integer*4 status

	LOGICAL	LBUFF

	common /bufin/buffer

	status = 0

C  if sequential reading requested - set record number
C-----------------------------------------------------
	IF(RECORD .LE. 0) record = 1 

C  read a line
C-------------
	IF(SKPBYT.EQ.0 .AND. NBYTES.GE.ns) THEN
	   status = eread(fdesc,band,RECORD,buffer(1,ilow),1)
	      IF(STATUS .lt. ESUCC) THEN
	         call errmsg('Error reading input file','lamsutil-read',1)
	         status = EFAIL
	         return
	      endif
	   LBUFF = .FALSE.
	ELSE
	   status = eread(fdesc,band,RECORD,locbuf,1)
	      IF(STATUS .lt. ESUCC) THEN
	         call errmsg('Error reading input file','lamsutil-read',1)
	         status = EFAIL
	         return
	      endif
	   LBUFF = .TRUE.
	ENDIF

C  Load buffer from local buffer if needed
C-----------------------------------------
	IF (LBUFF) THEN
	   NUMBYT = MIN0(ns,NBYTES)
	   bufcnt=1
	   do 1115 ix=1,numbyt + skpbyt - 1
	     if(ix.gt.skpbyt) then
		buffer(bufcnt,ilow)=locbuf(ix)
		bufcnt=bufcnt+1
             endif
1115	   continue
	ENDIF

	status = ESUCC
	RETURN
	END
