/*
 *	This is equivalent to fgets but modified
 *	to work with socket file descriptors instead
 *	of stream pointers.
 */
#define BUFSZ 4096
#include <errno.h>
char *malloc(long);

struct filebuf
	{
	int	fd;		/* file descriptor */
	char	*buf;		/* buffer data */
	int	offset;		/* current offset in buf */
	int	bufsz;		/* length of buf */
	int	count;		/* amount of valid data in buf */
	};

short getbyte(struct filebuf *);

char *
getline(char *buf, int count, int fd)
	{
	static struct filebuf	*fblist[10];	/* list of filebufs */
	register struct filebuf	*fb;
	static int		fbsz;	/* number of elements in fblist */
	int	i;			/* index */
	short	c;			/* character read */
	char	*p;			/* pointer into buf */

	/*
	 *	Look for the proper filebuf
	 */
	for (i=0; i<fbsz; i++)
		if (fd == fblist[i]->fd)
			break;
	if (i == fbsz)
		{
		/*
		 *	Allocate a new one
		 */
		fblist[i] = fb = (struct filebuf *)malloc(sizeof (struct filebuf));
		fbsz++;
		fb->fd = fd;
		fb->bufsz = BUFSZ;
		fb->buf = malloc(BUFSZ);
		fb->count = 0;
		}
	fb = fblist[i];

	count--;	/* save space for the null terminator */
	p = buf;
	while ((c = getbyte(fb)) != -1)
		{
		*p = c;
		p++;
		count--;
		if ((c == '\n') || (count == 0))
			break;
		}
	if (p == buf)
		return (char *)0;
	*p = '\0';
	return buf;
	}
/*
 *	Functionally equivalent to getchar()
 */
short getbyte(struct filebuf *fb)
	{
	short c;

	if ((fb->count <= 0) || (fb->offset == fb->bufsz))
		{
		/*
		 *	Fill the buffer
		 */
		if (!pollread(fb->fd))
			{
			errno = EWOULDBLOCK;
			return -1;
			}
		errno = 0;
		fb->count = read(fb->fd, fb->buf, fb->bufsz);
		if (fb->count <= 0)
			return -1;
		fb->offset = 0;
		}
	c = fb->buf[fb->offset];
	fb->count--;
	fb->offset++;
	return c;
	}

/*
 *	Poll a file descriptor to see if it is ready to read.
 *	This is to avoid hanging on a socket read.
 */
#include <sys/types.h>
#include <sys/time.h>
int
pollread(int fd)
	{
	fd_set	readfds;
	fd_set	exceptfds;
	struct timeval t;

	FD_ZERO(&readfds);
	FD_ZERO(&exceptfds);
	FD_SET(fd, &readfds);
	FD_SET(fd, &exceptfds);
	/*
	 *	There is a potentialy serious problem here.
	 *	If the message that the other end is sending is larger
	 *	than some magic size (typically 4096 bytes), it may
	 *	be "fragmented", i.e., not all arrive in a single read.
	 *	(BTW, the magic size has nothing to do with the BUFSZ
	 *	define above.)  The second or succeding fragments may
	 *	not arrive before we try to read some more, in which
	 *	case we think we're done, and the rest of the message
	 *	will arrive shortly (and we'll think it's a new one).
	 *	As a band-aid fix, we set the timeout on the select
	 *	to one second, hoping that any fragments will arrive
	 *	before then.
	 */
	t.tv_sec = 1;
	t.tv_usec = 0;
	if (select (fd+1, &readfds, 0, &exceptfds, &t) <= 0)
		return 0;
	if (FD_ISSET(fd, &readfds) || FD_ISSET(fd, &exceptfds))
		return 1;
	return 0;
	}
