
#include <sys/types.h>
#include <sys/times.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>

static long pdsqmgraddr(char *hostname);
static int get_pds_port();

int
pdsconn(char *hostname, int enqreq)
	{
	struct sockaddr_in      sockaddr;       /* socket address */
	int                     options;        /* socket options */
	int			sockt;		/* socket fd to be returned */
	char	*enq = "format_version = 1\nmessage_type = ENQ\nthis is a test\n\n";

	/*
	 *	Create the socket.
	 */
	sockaddr.sin_family = AF_INET;
	if ((sockt = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		return(-1);
	options = 1;
	if (setsockopt(sockt, SOL_SOCKET, SO_REUSEADDR,
			&options, sizeof(options)) < 0)
		{
		close(sockt);
		return(-1);
		}

	/*
	 *	Connect to the given address.  The address should already
	 *	be in network byte order.
	 */
	sockaddr.sin_addr.s_addr = pdsqmgraddr(hostname);
	sockaddr.sin_port = get_pds_port();
	sockaddr.sin_family = AF_INET;
	if (connect(sockt, (caddr_t)&sockaddr, sizeof(sockaddr)) < 0)
		{
		close(sockt);
		return(-1);
		}
	
	/*
	 *	If required, send and ENQ.
	 */
	if (enqreq)
		write(sockt, enq, strlen(enq));
	return sockt;
	}
static long
pdsqmgraddr(char *hostname)
	{
	long		hostaddr;	/* to be returned */
	struct hostent	*hostent;	/* host table entry */
	struct hostent	*gethostbyname();

	if (!hostname)
		hostname = "localhost";
	/*
	 *	First try looking it up by name.
	 */
	if (hostent = gethostbyname(hostname))
		memcpy((char *)&hostaddr, hostent->h_addr_list[0], 4);
	else
		{
		if (isdigit(*hostname))
			{
			if (hostname[1] == 'x')
				{
				/*
				 *	Hexadecimal address
				 */
				sscanf(hostname, "0x%x", &hostaddr);
				}
			else
				{
				/*
				 *	Parse a numeric address of the
				 *	form ddd.ddd.ddd.ddd
				 */
				char tmp[4];	/* temporary for number */
				char *p,*q;	/* for parsing */
				int n;		/* # chars in number */
				char *strchr();

				hostaddr = 0;
				for (p = hostname; p && *p; )
					{
					q = strchr(p, '.');
					if (q)
						{
						n = q-p;
						if (n > 3)
							n = 3;
						strncpy(tmp, p, n);
						p = q + 1;
						}
					else
						{
						n = 3;
						strncpy(tmp, p, n);
						p = q;
						}
					tmp[n] = '\0';
					hostaddr = (hostaddr << 8) | atoi(tmp);
					}
				}
			}
		else
			{
			/*
			 *	None of the above -- just hard code the address.
			 */
			hostaddr = 0x983DC046;
			}
		}
	return htonl(hostaddr);
	}
/*
 *	get_pds_port returns the PDS TCP port number
 */
static int
get_pds_port()
	{
	struct servent *s;
	char *envport;
	char *getenv(char *);

	s = getservbyname("pds", "tcp");
	if (s)
		return(s->s_port);
	else if  (envport = getenv("PDSPORT"))
		return(atoi(envport));
	else
		return(14010);	/* fallback */
	}
