#include <stdio.h>
#include <string.h>
#include "pcs.h"
extern int errno;

/*
 *	Field names
 */
enum field_names {
	FORMAT_VERSION,
	MESSAGE_DATE,
	MESSAGE_TYPE,
	ORDER_NBR,
	UNIT_NBR,
	HOST_NAME,
	PRODUCT_CODE,
	PRIORITY_CODE,
	DATE_DUE,
	INPUT_MEDIA_TYPE,
	INPUT_MEDIA_FORMAT,
	STORAGE_LOC,
	ORDERING_ID,
	PROJECT_ID,
	SPEC_INSTR_TEXT1,
	SPEC_INSTR_TEXT2,
	SPEC_INSTR_TEXT3,
	SPEC_INSTR_TEXT4,
	SPECTRAL_BANDS,
	PRODUCT_FORMAT,
	PRODUCT_MEDIA,
	PRODUCT_DENSITY,
	ANSI_LABEL_F,
	DATA_ORGANIZATION,
	TAPE_BLOCKING,
	BYTE_ORDERING,
	INPUT_DIR,
	CREATE_COL_FIRE_F,
	RETAIN_DEM_F,
	COMPRESS_FILL_F,
	NUMBER_COPIES,
	BIN_NBR,
	RETURN_ADDR,
	E_MAIL_ADDRESS,
	UNKNOWN_KEYWORD};

static char *retaddr(char *fromline);
static enum field_names parse_keyword(char *str);
static struct pcs_info *processline(char *line, struct pcs_info *pcs);
static void RemoveDQuote(char *str);
static char **getstorageloc(char *p);
extern char *malloc();

/*
 *	pdsget reads in an ASCII character version of the pcs_info
 *	structure on the given file descriptor an puts it into a
 *	pcs_info structure.  A pointer to the structure is returned.
 *	If end of file is encountered, NULL is returned.
 */
struct pcs_info *
pdsget(int in)
	{
	struct pcs_info *pcs,*p;	/* to be allocated and returned */
	char	line[1024];		/* line buffer */
	int	ignoremailheader = 0;	/* true while ignoring mail headers */

	/*
	 *	Read the first line (do this before allocating space
	 *	so we can check for end of file).
	 */
	if (!getline(line, 1024, in))
		{
		return(struct pcs_info *) 0;
		}
	pcs = pcs_info_alloc();
	if (!pcs)
		{
		return(struct pcs_info *) 0;
		}
	/*
	 *	Check for a mail header
	 */
	if ((strncmp(line,"From",4) == 0) && (line[4] != ':'))
		ignoremailheader = 1;

	/*
	 *	Read lines of input until EOF or a blank line.
	 */
	p = pcs;
	do
		{
		line[strlen(line)-1] = '\0';	/* clobber the newline */
		/*
		 *	While ignoreing mail headers, the only thing
		 *	we look for is the "From:" line (to save the return
		 *	e-mail address), the "format_version" or blank line
		 *	that indicates the beginning of the message.
		 */
		if (ignoremailheader)
			{
			if (strncmp(line,"From:",5) == 0)
				p->return_addr = retaddr(line);
			if (strncmp(line,"format_version", 14) == 0)
				{
				ignoremailheader = 0;
				p = processline(line, p);
				}
			if (strlen(line) == 0)
				ignoremailheader = 0;
			continue;
			}
		/*
		 *	An empty line signifies end of current record.
		 */
		if (strlen(line) == 0)
			break;
		/*
		 *	We aren't ignoreing mail headers, so process the line.
		 */
		p = processline(line, p);
		}
	while (getline(line, 1024, in));
	return(pcs);
	}

/*
 *	processline handles a single line of input.
 *	Ordinarily, this is simply a matter of looking up the
 *	keyword and copying the value into the structure.  However,
 *	if we find a unit number field, and the unit number is
 *	already filled in, we assume this is another unit for
 *	the same order and we allocate a new structure for the
 *	new unit.  The "current" structure is returned.
 */
static struct pcs_info *
processline(char *line, struct pcs_info *pcs)
	{
	char	*p;
	char	*delimiters = " \t=";

	/*
	 *	Eliminate any leading white space.
	 */
	line += strspn(line, delimiters);

	/*
	 *	Get the first token (the keyword) and look it up.
	 */
	p = line + strcspn(line, delimiters);
	*p = '\0';
	p++;
	p += strspn(p, delimiters);	/* leave it pointing at the next token */
	switch (parse_keyword(line))
		{
		case FORMAT_VERSION:
			pcs->format_version = atoi(p);
			break;
		case MESSAGE_TYPE:
			pcs->message_type = getmsgtype(p);
			break;
		case MESSAGE_DATE:
			strncpy(pcs->message_date, p, 10);
			pcs->message_date[10] = '\0';
			break;
		case ORDER_NBR:
			strncpy(pcs->order_nbr, p, 13);
			pcs->order_nbr[13] = '\0';
			break;
		case UNIT_NBR:
			if (pcs->unit_nbr[0])
				{
				struct pcs_info *newpcs;
				/*
				 *	Allocate a new PCS_INFO structure
				 *	and add it to the list.
				 */
				newpcs = pcs_info_alloc();
				pcs->next = newpcs;
				newpcs->prev = pcs;
				newpcs->format_version = pcs->format_version;
				newpcs->message_type = pcs->message_type;
				strncpy(newpcs->message_date, pcs->message_date, 10);
				strncpy(newpcs->order_nbr, pcs->order_nbr, 13);
				pcs->order_nbr[13] = '\0';
				pcs = newpcs;
				}
			strncpy(pcs->unit_nbr, p, 4);
			pcs->unit_nbr[4] = '\0';
			break;
		case HOST_NAME:
			if (pcs->host_name = malloc(strlen(p) + 1))
				strcpy(pcs->host_name, p);
			break;
		case PRODUCT_CODE:
			strncpy(pcs->product_code, p, 4);
			pcs->product_code[4] = '\0';
			break;
		case PRIORITY_CODE:
			pcs->priority_code = atoi(p);
			break;
		case DATE_DUE:
			strncpy(pcs->date_due, p, 10);
			pcs->date_due[10] = '\0';
			break;
		case INPUT_MEDIA_TYPE:
			strncpy(pcs->input_media_type, p, 2);
			pcs->input_media_type[2] = '\0';
			break;
		case INPUT_MEDIA_FORMAT:
			strncpy(pcs->input_media_format, p, 2);
			pcs->input_media_format[2] = '\0';
			break;
		case STORAGE_LOC:
			pcs->storage_loc  = getstorageloc(p);
			break;
		case ORDERING_ID:
			strncpy(pcs->ordering_id, p, 18);
			pcs->ordering_id[19] = '\0';
			break;
		case PROJECT_ID:
			strncpy(pcs->project_id, p, 15);
			pcs->project_id[15] = '\0';
			break;
		case SPEC_INSTR_TEXT1:
			pcs->spec_instr_text[0] = (char *)malloc(strlen(p)+1);
			strcpy(pcs->spec_instr_text[0], p);
			RemoveDQuote(pcs->spec_instr_text[0]);
			break;
		case SPEC_INSTR_TEXT2:
			pcs->spec_instr_text[1] = (char *)malloc(strlen(p)+1);
			strcpy(pcs->spec_instr_text[1], p);
			RemoveDQuote(pcs->spec_instr_text[1]);
			break;
		case SPEC_INSTR_TEXT3:
			pcs->spec_instr_text[2] = (char *)malloc(strlen(p)+1);
			strcpy(pcs->spec_instr_text[2], p);
			RemoveDQuote(pcs->spec_instr_text[2]);
			break;
		case SPEC_INSTR_TEXT4:
			pcs->spec_instr_text[3] = (char *)malloc(strlen(p)+1);
			strcpy(pcs->spec_instr_text[3], p);
			RemoveDQuote(pcs->spec_instr_text[3]);
			break;
		case SPECTRAL_BANDS:
			strncpy(pcs->spectral_bands, p, 8);
			pcs->spectral_bands[8] = '\0';
			break;
		case PRODUCT_FORMAT:
			strncpy(pcs->product_format, p, 4);
			pcs->product_format[4] = '\0';
			break;
		case PRODUCT_MEDIA:
			strncpy(pcs->product_media, p, 2);
			pcs->product_media[2] = '\0';
			break;
		case PRODUCT_DENSITY:
			strncpy(pcs->product_density, p, 4);
			break;
		case ANSI_LABEL_F:
			pcs->ansi_label_f = p[0];
			break;
		case DATA_ORGANIZATION:
			strncpy(pcs->data_organization, p, 3);
			pcs->data_organization[3] = '\0';
			break;
		case TAPE_BLOCKING:
			strncpy(pcs->tape_blocking, p, 5);
			pcs->tape_blocking[5] = '\0';
			break;
		case BYTE_ORDERING:
			strncpy(pcs->byte_ordering, p, 3);
			pcs->byte_ordering[3] = '\0';
			break;
		case INPUT_DIR:
			if (pcs->input_dir = malloc(strlen(p)+1))
				strcpy(pcs->input_dir, p);
			break;
		case CREATE_COL_FIRE_F:
			pcs->create_col_fire_f = p[0];
			break;
		case RETAIN_DEM_F:
			pcs->retain_dem_f = p[0];
			break;
		case COMPRESS_FILL_F:
			pcs->compress_fill_f = p[0];
			break;
		case NUMBER_COPIES:
			pcs->number_copies = atoi(p);
			break;
		case BIN_NBR:
			strncpy(pcs->bin_nbr, p, 3);
			pcs->bin_nbr[3] = '\0';
			break;
		case E_MAIL_ADDRESS:
			if (pcs->e_mail_address = malloc(strlen(p)+1))
				strcpy(pcs->e_mail_address, p);
			break;
		}
	return pcs;
	}

/*
 *	look up a field name and return the symbolic value.
 */
static enum field_names
parse_keyword(char *str)
	{
	static struct	/* keyword table -- must be sorted for binarysearch */
		{
		char	*keyword;
		int	keyval;
		}
	    table[] = {
		{"ansi_label_f",	ANSI_LABEL_F },
		{"bin_nbr",		BIN_NBR },
		{"byte_ordering",	BYTE_ORDERING },
		{"compress_fill_f",	COMPRESS_FILL_F },
		{"create_col_fire_f",	CREATE_COL_FIRE_F },
		{"data_organization",	DATA_ORGANIZATION },
		{"date_due",		DATE_DUE },
                {"e_mail_address",      E_MAIL_ADDRESS },
		{"format_version",	FORMAT_VERSION },
		{"host_name",		HOST_NAME },
		{"input_directory",	INPUT_DIR },
		{"input_media_format",	INPUT_MEDIA_FORMAT },
		{"input_media_type",	INPUT_MEDIA_TYPE },
		{"message_date",	MESSAGE_DATE },
		{"message_type",	MESSAGE_TYPE },
		{"number_copies",	NUMBER_COPIES },
		{"order_nbr",		ORDER_NBR },
		{"ordering_id",		ORDERING_ID },
		{"priority_code",	PRIORITY_CODE },
		{"product_code",	PRODUCT_CODE },
		{"product_density",	PRODUCT_DENSITY },
		{"product_format",	PRODUCT_FORMAT },
		{"product_media",	PRODUCT_MEDIA },
		{"project_id",		PROJECT_ID },
		{"retain_dem_f",	RETAIN_DEM_F },
		{"spec_instr_text1",	SPEC_INSTR_TEXT1 },
		{"spec_instr_text2",	SPEC_INSTR_TEXT2 },
		{"spec_instr_text3",	SPEC_INSTR_TEXT3 },
		{"spec_instr_text4",	SPEC_INSTR_TEXT4 },
		{"spectral_bands",	SPECTRAL_BANDS },
		{"storage_loc",		STORAGE_LOC },
		{"tape_blocking",	TAPE_BLOCKING },
		{"unit_nbr",		UNIT_NBR },
		};
	int high, low, middle;
	int rslt;

	low = 0;
	high = sizeof(table) / sizeof(table[0]) - 1;
	while (low <= high)
		{
		middle = (low + high) / 2;
		rslt = strcmp(table[middle].keyword, str);
		if (rslt < 0)
			low = middle + 1;
		else if (rslt == 0)
			return table[middle].keyval;
		else
			high = middle - 1;
		}
	return UNKNOWN_KEYWORD;
	}
static void
RemoveDQuote(char *str)
{
    register char *i, *o;
    register n;
    register count;

    if (*str != '\"')
	{
	/*
	 *	There is no leading doublequote, so don't try to remove
	 *	them, but just in case, check for a trailing newline.
	 */
	int len = strlen(str);

	if (str[len-1] == '\n')
		str[len-1] = '\0';
	return;
	}

    for (i=str+1, o=str; *i && *i != '\"'; o++)
    {
	if (*i == '\\')
	{
	    switch (*++i)
	    {
	    case 'n':
		*o = '\n';
		i++;
		break;
	    case 'b':
		*o = '\b';
		i++;
		break;
	    case 'r':
		*o = '\r';
		i++;
		break;
	    case 't':
		*o = '\t';
		i++;
		break;
	    case 'f':
		*o = '\f';
		i++;
		break;
	    case '0':
		if (*++i == 'x')
		    goto hex;
		else
		    --i;
	    case '1': case '2': case '3':
	    case '4': case '5': case '6': case '7':
		n = 0;
		count = 0;
		while (*i >= '0' && *i <= '7' && count < 3)
		{
		    n = (n<<3) + (*i++ - '0');
		    count++;
		}
		*o = n;
		break;
	    hex:
	    case 'x':
		n = 0;
		count = 0;
		while (i++, count++ < 2)
		{
		    if (*i >= '0' && *i <= '9')
			n = (n<<4) + (*i - '0');
		    else if (*i >= 'a' && *i <= 'f')
			n = (n<<4) + (*i - 'a') + 10;
		    else if (*i >= 'A' && *i <= 'F')
			n = (n<<4) + (*i - 'A') + 10;
		    else
			break;
		}
		*o = n;
		break;
	    case '\n':
		i++;	/* punt */
		o--;	/* to account for o++ at end of loop */
		break;
	    case '\"':
	    case '\'':
	    case '\\':
	    default:
		*o = *i++;
		break;
	    }
	}
	else
	    *o = *i++;
    }
    *o = '\0';
}
static char **
getstorageloc(char *p)
	{
	char	**stor;		/* to be returned */
	int	n;		/* number of storage locations found */
	int	maxn;		/* size of allocated space */
	char	*q;		/* scratch pointer */

	if (*p == '{')
		{
		p++;
		p += strspn(p, " \t");
		n = 0;
		maxn = 10;	/* start out with 10, will increase as we go */
		stor = (char **)malloc(maxn * sizeof(char *));
		for (;;)
			{
			if (n >= (maxn-1))
				{
				char **new;

				new = (char **)malloc(2 * maxn * sizeof(char *));
				memcpy(new, stor, maxn * sizeof(char *));
				maxn = 2 * maxn;
				free(stor);
				stor = new;
				}
			q = p + strcspn(p, " \t,}");
			*q = '\0';
			stor[n] = malloc(strlen(p) + 1);
			strcpy(stor[n], p);
			RemoveDQuote(stor[n]);
			n++;
			q++;
			p = q + strspn(q, " \t,}");
			if ((*p == '}') || (*p == '\0'))
				break;
			}
		stor[n] = (char *)0;
		}
	else
		{
		/*
		 *	Easy case: single value.
		 */
		if ((stor = (char **)malloc(2 * sizeof(char *)))
			&& (stor[0] = malloc(strlen(p)+1)))
			{
			strcpy(stor[0], p);
			RemoveDQuote(stor[0]);
			stor[1] = (char *)0;
			}
		}
	return(stor);
	}
/*
 *	retaddr takes the "From: ..." line from an e-mail header
 *	and extracts the e-mail address.  There are four possibilities:
 *
 *		From: address
 *		From: <address>
 *		From: Full Name <address>
 *		From: address (comment)
 *
 *	Space is malloc'ed for the address string and a pointer to it
 *	is returned.
 */
static char *
retaddr(char *fromline)
	{
	char *begin;		/* pointer to the beginning of the address */
	char *end;		/* pointer to the end of the address */
	char *address;		/* to be returned */
	int  addrlen;		/* length of the address */

	fromline += 5; 	/* skip over the "From:" */

	if (begin = strchr(fromline, '<'))
		{
		/*
		 *	Easy case: the address is enclosed in <>
		 */
		end = strchr(begin, '>');
		if  (!end)
			return (char *)0;	/* malformed address  */
		addrlen = end - begin -1;
		begin++; 	/* skip the < */
		}
	else
		{
		/*
		 *	In either of the other two cases, the address is
		 *	the first token.
		 */
		begin = strtok(fromline, " \t\n");
		addrlen = strlen(begin);
		}
	/*
	 *	Copy the address
	 */
	address = (char *)malloc(addrlen+1);
	if (!address)
		return (char *)0;
	strncpy(address, begin, addrlen);
	address[addrlen] = '\0';
	return address;
	}
