#include <stdio.h>
#include "pcs.h"

char *malloc(int);

struct pcs_info *
pcs_info_alloc()
	{
	register struct pcs_info *pcs;

	pcs = (struct pcs_info *)malloc(sizeof(struct pcs_info));
	bzero((char *)pcs, sizeof(struct pcs_info));
	return(pcs);
	}

int
pcs_info_free(struct pcs_info *pcs)
	{
	int	i;	/* index */
	char	**stor;	/* pointer into storage location array */

	/*
	 *	Take care of the list first.
	 */
	if (pcs->next)
		pcs_info_free(pcs->next);

	/*
	 *	If any character strings were allocated, free them.
	 */
	if (pcs->host_name)
		free(pcs->host_name);
	if (pcs->storage_loc)
		{
		for (stor = pcs->storage_loc; *stor; stor++)
			free(*stor);
		free(pcs->storage_loc);
		}
	for (i=0; i<4; i++)
		if (pcs->spec_instr_text[i])
			free(pcs->spec_instr_text[i]);
	if (pcs->input_dir)
		free(pcs->input_dir);
	if (pcs->return_addr)
		free(pcs->return_addr);
	if (pcs->e_mail_address)
		free(pcs->e_mail_address);

	/*
	 *	Free the structure itself.
	 */
	free(pcs);
	}
struct pcs_info *
pcs_info_dup(struct pcs_info *pcs)
	{
	struct pcs_info *newpcs;
	struct pcs_info *p;
	int i;

	if (!(newpcs = pcs_info_alloc()))
		return(newpcs);
	p = newpcs;
	*newpcs = *pcs;
	newpcs->next = (struct pcs_info *)0;

	cp:
	/*
	 *	Separately allocated character strings need to
	 *	be copied.
	 */
	if (pcs->host_name)
		{
		p->host_name = malloc(strlen(pcs->host_name) + 1);
		strcpy(p->host_name, pcs->host_name);
		}
	if (pcs->storage_loc)
		{
		char	**stor;	/* pointer into storage location array */
		char	**newstor;	/* newly allocated storage_loc */

		for (i=0, stor = pcs->storage_loc; *stor; stor++)
			i++;
		newstor = p->storage_loc = (char **)malloc((i + 1) * sizeof(char *));
		for (stor = pcs->storage_loc; *stor; stor++, newstor++)
			{
			*newstor = malloc(strlen(*stor) + 1);
			strcpy(*newstor, *stor);
			}
		*newstor = (char *)0;
		}
	for (i=0; i<4; i++)
		if (pcs->spec_instr_text[i])
			{
			p->spec_instr_text[i] = malloc(strlen(pcs->spec_instr_text[i]) + 1);
			strcpy(p->spec_instr_text[i], pcs->spec_instr_text[i]);
			}
	if (pcs->input_dir)
		{
		p->input_dir = malloc(strlen(pcs->input_dir) + 1);
		strcpy(p->input_dir, pcs->input_dir);
		}
	if (pcs->return_addr)
		{
		p->return_addr = malloc(strlen(pcs->return_addr) + 1);
		strcpy(p->return_addr, pcs->return_addr);
		}
	if (pcs->e_mail_address)
		{
		p->e_mail_address = malloc(strlen(pcs->e_mail_address) + 1);
		strcpy(p->e_mail_address, pcs->e_mail_address);
		}
	if (pcs->next)
		{
		struct pcs_info *last = p;

		if (!(p = pcs_info_alloc()))
			return(p);
		pcs = pcs->next;
		*p = *pcs;
		p->next = (struct pcs_info *)0;
		last->next = p;
		goto cp;
		}

	return(newpcs);
	}
/*
 *	getmsgtype takes a character string and returns a message type.
 *	If it isn't one of the recognized types, INVALID is returned.
 */
enum message_type getmsgtype(char *str)
	{
	if (strcmp(str,"ORD") == 0)
		return(ORD);
	else if (strcmp(str,"VALD") == 0)
		return(VALD);
	else if (strcmp(str,"COMP") == 0)
		return(COMP);
	else if (strcmp(str,"REJD") == 0)
		return(REJD);
	else if (strcmp(str,"ENQ") == 0)
		return(ENQ);
	else if (strcmp(str,"QUIT") == 0)
		return(QUIT);
	else
		return(INVALID);
	}
/*
 *	putmsgtype is the inverse of getmsgtype -- given a message type
 *	code, it returns a pointer to a string.
 */
char *
putmsgtype(enum message_type m)
	{
	switch(m)
		{
		case ORD:
			return("ORD");
		case VALD:
			return("VALD");
		case COMP:
			return("COMP");
		case REJD:
			return("REJD");
		case ENQ:
			return("ENQ");
		case QUIT:
			return("QUIT");
		default:
			return("INVALID");
		}
	}
/*
 *	I've always wanted to write my own printf...
 *
 *	This would be a lot easier with printf, but we don't want
 *	to split writes to the socket, so we buffer the whole
 *	thing up and send it with one write.  This may break
 *	if you try to send incredibly large things with it.
 */
struct buffer
	{
	char	*data;		/* pointer to start of space allocated */
	char	*cur;		/* current offset into data */
	int	bufsz;		/* size of space allocated in data */
	};
/*
 *	addstr adds a string on to the end of a buffer
 */
void
addstr(struct buffer *bufp, char *str)
	{
	int slen;
	char *malloc(int);

	slen = strlen(str);
	if (bufp->cur + slen >= bufp->data + bufp->bufsz)
		{
		/*
		 *	Buffer about to overflow -- allocate
		 *	some more.
		 */
		char *new;
		int newsz;
		int curlen;	/* current length */

		newsz = 2 * bufp->bufsz;
		new = malloc(newsz);
		if (!new)
			{
			/* panic */
			fprintf(stderr,"pdsput: out of memory\n");
			exit(1);
			}
		curlen = bufp->cur - bufp->data;
		memcpy(new, bufp->data, curlen);
		free(bufp->data);
		bufp->data = new;
		bufp->cur = new + curlen;
		bufp->bufsz = newsz;
		}
	strcpy(bufp->cur, str);
	bufp->cur += slen;
	}
/*
 *	addint adds a decimal int on to the end of a buffer
 */
void
addint(struct buffer *bufp, int d)
	{
	char num[10];
	sprintf(num, "%d", d);
	addstr(bufp, num);
	}
/*
 *	addchr adds a single character on to the end of a buffer
 */
void
addchr(struct buffer *bufp, char c)
	{
	char str[2];
	str[0] = c;
	str[1] = '\0';
	addstr(bufp, str);
	}
/*
 *	pcs_out writes out an ASCII character version of the pcs_info
 *	structure on the given file descriptor.
 */
void
pdsput(int outfd, struct pcs_info *pcs)
	{
	char **locs;		/* for stepping through the storage_loc list */
	register i;		/* loop index for spec_instr */
	static struct buffer buffer;	/* output buffer */
	char *malloc();

	if (!buffer.data)
		{
		if (!(buffer.data = malloc(buffer.bufsz = 4096)))
			{
			/* panic */
			fprintf(stderr,"pdsput: out of memory\n");
			exit(1);
			}
		buffer.cur = buffer.data;
		}
	do
		{
		addstr(&buffer, "format_version=");
		addint(&buffer, pcs->format_version);
		if (pcs->message_date[0])
			{
			addstr(&buffer, "\nmessage_date=");
			addstr(&buffer, pcs->message_date);
			}
		addstr(&buffer, "\nmessage_type=");
		addstr(&buffer, putmsgtype(pcs->message_type));
		if (pcs->order_nbr[0])
			{
			addstr(&buffer, "\norder_nbr=");
			addstr(&buffer, pcs->order_nbr);
			}
		if (pcs->unit_nbr[0])
			{
			addstr(&buffer, "\nunit_nbr=");
			addstr(&buffer, pcs->unit_nbr);
			}
		if (pcs->priority_code)
			{
			addstr(&buffer, "\npriority_code=");
			addint(&buffer, pcs->priority_code);
			}
		if (pcs->ordering_id[0])
			{
			addstr(&buffer, "\nordering_id=");
			addstr(&buffer, pcs->ordering_id);
			}
		if (pcs->storage_loc)
			{
			addstr(&buffer, "\nstorage_loc=");
			if (pcs->storage_loc[1])
				{
				addstr(&buffer,"{");
				}
			for (locs = pcs->storage_loc; *locs; locs++)
				{
				addstr(&buffer,"\"");
				addstr(&buffer, *locs);
				addstr(&buffer,"\"");
				if (*(locs+1))
					{
					addstr(&buffer, ", ");
					}
				}
			if (pcs->storage_loc[1])
				{
				addstr(&buffer,"}");
				}
			}
		if (pcs->host_name)
			{
			addstr(&buffer, "\nhost_name=");
			addstr(&buffer, pcs->host_name);
			}
		if (pcs->product_code[0])
			{
			addstr(&buffer, "\nproduct_code=");
			addstr(&buffer, pcs->product_code);
			}
		if (pcs->date_due[0])
			{
			addstr(&buffer, "\ndate_due=");
			addstr(&buffer, pcs->date_due);
			}
		if (pcs->project_id[0])
			{
			addstr(&buffer, "\nproject_id=");
			addstr(&buffer, pcs->project_id);
			}
		if (pcs->input_media_type[0])
			{
			addstr(&buffer, "\ninput_media_type=");
			addstr(&buffer, pcs->input_media_type);
			}
		if (pcs->input_media_format[0])
			{
			addstr(&buffer, "\ninput_media_format=");
			addstr(&buffer, pcs->input_media_format);
			}
		for (i=0; i<4; i++)
			if (pcs->spec_instr_text[i])
				{
				addstr(&buffer, "\nspec_instr_text");
				addint(&buffer, i+1);
				addstr(&buffer, "=\"");
				addstr(&buffer, pcs->spec_instr_text[i]);
				addstr(&buffer, "\"");
				}
		if (pcs->spectral_bands[0])
			{
			addstr(&buffer, "\nspectral_bands=");
			addstr(&buffer, pcs->spectral_bands);
			}
		if (pcs->product_format[0])
			{
			addstr(&buffer, "\nproduct_format=");
			addstr(&buffer, pcs->product_format);
			}
		if (pcs->product_media[0])
			{
			addstr(&buffer, "\nproduct_media=");
			addstr(&buffer, pcs->product_media);
			}
		if (pcs->product_density[0])
			{
			addstr(&buffer, "\nproduct_density=");
			addstr(&buffer, pcs->product_density);
			}
		if (pcs->ansi_label_f)
			{
			addstr(&buffer, "\nansi_label_f=");
			addchr(&buffer, pcs->ansi_label_f);
			}
		if (pcs->data_organization[0])
			{
			addstr(&buffer, "\ndata_organization=");
			addstr(&buffer, pcs->data_organization);
			}
		if (pcs->tape_blocking[0])
			{
			addstr(&buffer, "\ntape_blocking=");
			addstr(&buffer, pcs->tape_blocking);
			}
		if (pcs->byte_ordering[0])
			{
			addstr(&buffer, "\nbyte_ordering=");
			addstr(&buffer, pcs->byte_ordering);
			}
		if (pcs->input_dir)
			{
			addstr(&buffer, "\ninput_directory=");
			addstr(&buffer, pcs->input_dir);
			}
		if (pcs->create_col_fire_f)
			{
			addstr(&buffer, "\ncreate_col_fire_f=");
			addchr(&buffer, pcs->create_col_fire_f);
			}
		if (pcs->retain_dem_f)
			{
			addstr(&buffer, "\nretain_dem_f=");
			addchr(&buffer, pcs->retain_dem_f);
			}
		if (pcs->compress_fill_f)
			{
			addstr(&buffer, "\ncompress_fill_f=");
			addchr(&buffer, pcs->compress_fill_f);
			}
		if (pcs->number_copies)
			{
			addstr(&buffer, "\nnumber_copies=");
			addint(&buffer, pcs->number_copies);
			}
		if (pcs->bin_nbr[0])
			{
			addstr(&buffer, "\nbin_nbr=");
			addstr(&buffer, pcs->bin_nbr);
			}
		if (pcs->return_addr)
			{
			addstr(&buffer, "\nFrom: ");
			addstr(&buffer, pcs->return_addr);
			}
		if (pcs->e_mail_address)
			{
			addstr(&buffer, "\ne_mail_address=");
			addstr(&buffer, pcs->e_mail_address);
			}
		addstr(&buffer, "\n");
		}
	while (pcs = pcs->next);

	addstr(&buffer, "\n");
	write(outfd, buffer.data, buffer.cur - buffer.data);
	buffer.cur = buffer.data;
	}
