#!/bin/csh -f
#
# FUNCTION:	lassetup.csh
#
# PURPOSE:	Create a temporary pdf in the current directory that will
#		initialize TAE for the LAS and ADAPS software packages.
#
#
# Programmer       Date   Modification
# ---------------  -----  ------------------------------------------------------
# J.Fenno          11/95  Development


#--------------------
# Check the arguments
#--------------------
if ($#argv != 1) then
    echo "usage: lassetup.csh <pdf>"
    echo ""
    echo '       This shell script is normally called from $TAEPDF/slogon.pdf'
    echo "       to generate a temporary pdf that will initialize TAE for LAS"
    echo "       and ADAPS.  The function calling this shell script should then"
    echo "       run the temporary pdf that is generated.  The last command in"
    echo "       temporary pdf will remove itself."
    exit 1
endif
set pdfname = $1
set pdffile = $pdfname.pdf


#-----------------
# Set up the shell
#-----------------
/bin/rm -f $pdffile


#-------------------------------------------------
# Determine the system type for $SYSCHAR(4) in TAE
#-------------------------------------------------
switch (`uname -s`)
    case IRIX*:
	set systype = "SGI"
	breaksw
    case SunOS:
	set systype = "SUN"
	breaksw
    case dgux:
	set systype = "DGUX"
	breaksw
    case Linux:
        set systype = "Linux"
        breaksw
endsw


#-------------------------
# Create the temporary pdf
#-------------------------
cat << EOF > $pdffile
procedure $pdfname

refgbl (\$syschar \$prompt \$runtype \$dyntut \$tutor)

body
! Set up the syschar global variable for LAS and ADAPS
let \$syschar(3) = " "
let \$syschar(4) = "$systype"
let \$syschar(5) = " "
let \$syschar(6) = "$SYSDEF"

! Print the LAS welcome message and add all necessary
! directories to the TAE application library
$LASBASE/laslogon
$ADAPSBASE/adapslogon

EOF


#------------------------------------------------
# Check to see if the CAGIS software is installed
#------------------------------------------------
if ($?CAGISBASE) then
    if (-e $CAGISBASE/cagislogon.pdf) then
	cat << EOF >> $pdffile
! Add all necessary CAGIS directories to the TAE application library
$CAGISBASE/cagislogon

EOF
    endif
endif


#-------------------------------------------------
# Add the remaining system independent information
#-------------------------------------------------
cat << EOF >> $pdffile
! Add the LASDITTY directory too
setlib-add \$LASDDP
setlib-add \$LASDITTY

! Set the prompt
let \$PROMPT = "$SYSDEF"

! Set the \$DYNTUT global variable if needed
if (\$runtype = "INTERACTIVE")
    let \$DYNTUT = \$TUTOR
end-if

EOF


#-------------------------------------------
# Check for the existence of a ~/.lasrc file
#-------------------------------------------
if (-e ~/.lasrc) then
    cat << EOF >> $pdffile
! Run the user's ~/.lasrc file to obtain their preferences
~/.lasrc

EOF
endif


#-------------------------
# Finish the temporary pdf
#-------------------------
cat << EOF >> $pdffile
! Now that this is all done, delete yourself
ush /bin/rm -f $pdffile
end-proc
EOF


#----------------------
# Complete successfully
#----------------------
exit 0
