#ifndef INTTAELIB_H
#define INTTAELIB_H

#include <stdarg.h>
#include "taeconf.inp"
#include "comminc.inp"
#include "parblk.inc"
#include "expinc.inc"
#include "eventinc.inp"
#include "syninc.inc"
#include "dirinc.inc"

#define		MAX(A, B)	((A) > (B) ? (A) : (B))
#define	FAIL		0	/* failure function status return	*/

/* include file for tae library prototypes that are only meant to be used 
   internally */

typedef CODE (*FUNCTION_PTR) ();


FUNCTION COUNT s_bcopy
(
    const char *instr,      /* in: input string             */
    TEXT   outstr[],        /* out: output string           */
    FUNINT siz              /* in: max size of outstr       */
);

/* s_equal returns TRUE if strings equal (case insensitive) */
FUNCTION BOOL s_equal
(
    TEXT *s,
    TEXT *t
);

FUNCTION COUNT s_index
(
    TEXT   s[],             /* in: string */
    FUNINT  c               /* in: character to find */
);

/* Convert string to lowercase */
FUNCTION COUNT s_lower 
(
    TEXT *s
);

/* s_lseq returns TRUE if 1st str is left substr of 2nd */
FUNCTION BOOL s_lseq
(
    TEXT *s,
    TEXT *t
);

/* set n characters to a space character */
FUNCTION VOID s_blank
(
    TEXT *s,
    FUNINT n
);

/* shift string n characters left */
FUNCTION COUNT s_shift
(
    TEXT s[],               /* input/output string          */
    FUNINT n                /* input: characters to shift   */
); 

/* strip trailing spaces */
FUNCTION COUNT s_strip
(
    TEXT s[]
);

/* check integrity of parameter block */
FUNCTION CODE chk_parblk 
(
    struct PARBLK *p        /* in/out: PARBLK pointer       */
);

/* build app command */
FUNCTION CODE app_cmd 
(
    TEXT inbuf[],           /* in/out: single input line buffer  */
    TEXT cmdstr[],          /* in/out: accumulated cmd string    */
    BOOL inquote            /* in/out: TRUE if quote in effect   */
);

FUNCTION CODE fl_add
(
    TAEFLOAT in1,           /* in: first number to multiply */
    TAEFLOAT in2,           /* in: second number            */
    TAEFLOAT *out           /* out: result                  */
);

FUNCTION CODE fl_arith
(
    TAEFLOAT in1,           /* in: first number to multiply */
    TAEFLOAT in2,           /* in: second number            */
    TEXT     op,            /* in: operator                 */
    TAEFLOAT *out           /* out: result                  */
);

FUNCTION CODE fl_div
(
    TAEFLOAT in1,           /* in: first number to multiply */
    TAEFLOAT in2,           /* in: second number            */
    TAEFLOAT *out           /* out: result                  */
);

FUNCTION CODE fl_mult
(
    TAEFLOAT in1,           /* in: first number to multiply */
    TAEFLOAT in2,           /* in: second number            */
    TAEFLOAT *out           /* out: result                  */
);

FUNCTION CODE fl_sub
(
    TAEFLOAT in1,           /* in: first number to multiply */
    TAEFLOAT in2,           /* in: second number            */
    TAEFLOAT *out           /* out: result                  */
);

FUNCTION CODE int_add
(
    TAEINT in1,             /* in: first number to multiply */
    TAEINT in2,             /* in: second number            */
    TAEINT *out             /* out: result                  */
);

FUNCTION CODE int_arith
(
    TAEINT in1,             /* in: first number to multiply */
    TAEINT in2,             /* in: second number            */
    TEXT   op,              /* in: operator                 */
    TAEINT *out             /* out: result                  */
);

FUNCTION CODE int_div
(
    TAEINT in1,             /* in: first number to multiply */
    TAEINT in2,             /* in: second number            */
    TAEINT *out             /* out: result                  */
);

FUNCTION CODE int_fl2i
(
    TAEFLOAT fl,            /* in: the floater              */
    TAEINT   *intgr         /* out: the integer             */
);

FUNCTION CODE int_mult
(
    TAEINT in1,             /* in: first number to multiply */
    TAEINT in2,             /* in: second number            */
    TAEINT *out             /* out: result                  */
);

FUNCTION CODE int_sub
(
    TAEINT in1,             /* in: first number to multiply */
    TAEINT in2,             /* in: second number            */
    TAEINT *out             /* out: result                  */
);

void ena_under(void);
void ena_over(void);

/* convert delimited packed string to C string */
FUNCTION VOID s_d2s 
(

    GENPTR d,               /* in:  the delimited packed characters */
    TEXT   *s               /* out: the converted standard C string */
);

/* Convert binary integer to string form  */
FUNCTION CODE s_i2s
(
    TAEINT i,
    TEXT *s
);

/* Convert real to string format  */
FUNCTION CODE s_r2s
(
    TAEFLOAT r,
    TEXT     *s
);

/* Convert decimal string to binary integer */
FUNCTION CODE s_s2i
(
    TEXT   *s,
    TAEINT *i
);

/* convert a simple integer */
FUNCTION CODE s_s2i1
(
    TEXT   *s,
    TAEINT *i
);

/* Convert TAEFLOAT to string */
FUNCTION CODE s_r2ws
(
    TAEFLOAT taefloat,      /* input real                   */
    TEXT     string[]       /* output string                */
);

/* Convert decimal string to TAEFLOAT format */
FUNCTION CODE s_s2r
(
    TEXT     *s,
    TAEFLOAT *r             /* assume TAEFLOAT is DOUBLE  */
);

/* convert a simple real */
FUNCTION CODE s_s2r1
(
    TEXT     *s,
    TAEFLOAT *x 
);

/* get primitive language symbol */
FUNCTION CODE getprim
(
    struct SYNBLK *sb,      /* input:  syntax block         */
    struct VALUE  *value    /* output:  value structure     */
);

/* read a record */
FUNCTION CODE f_read
(
    struct SFILE *f,        /* in/out: SFILE for an opened file  */
    TEXT         string[]   /* out: record with EOS terminator   */
);

/* Format the record with the message. accept a va_list */
FUNCTION VOID m_pfmt_valist
(
    TEXT control[],         /* in: control string           */
    TEXT key[],             /* in: message key              */
    TEXT mputrec[],         /* out: output string           */
    va_list arglist
);

/* Format the record with the message */
FUNCTION VOID  m_pfmt
(
    TEXT control[],         /* in: control string           */
    TEXT key[],             /* in: message key              */
    TEXT mputrec[],         /* out: output string           */
    ...
);

/* decode an integer string of format "low:high" or "low" */
FUNCTION CODE irange
(
    TEXT   string[],        /* integer range string         */
    TAEINT *ilow,           /* out: low value               */
    TAEINT *ihigh           /* out: high value              */
);

/* decode a real string of format "low:high" or "low" */
FUNCTION CODE rrange
(
    TEXT     string[],      /* integer range string         */
    TAEFLOAT *rlow,         /* out: lower value             */
    TAEFLOAT *rhigh         /* out: high value              */
);

/* Hex string to integer conversion */
FUNCTION CODE s_sh2i
(
    TEXT s[],               /* hexadecimal numeric string   */
    FUNINT *i               /* out: the integer             */
);

/* flush the current command string */
FUNCTION VOID flush_cmd
(
    struct SFILE *sptr,     /* IN: sfile of the command file    */
    TEXT         inbuf[],   /* IN/OUT: last line read from file */
    TAEINT       *numrec    /* OUT: number of records read      */
);

/* is this a terminal? */
FUNCTION BOOL f_isterm 
(
    struct SFILE *file
);

/* get attributes from a file spec */
FUNCTION CODE f_attr
(
    TEXT fspec[],                /* in: host file spec           */
    TEXT attr[]                  /* out: attribute field         */
);

/* read binary record */
FUNCTION CODE f_bread
(
    struct SFILE *f,        /* in/out: SFILE for an opened file */
    void         *buffer,   /* out: record read from file       */
    FUNINT       bufsize,   /* in: size of user buffer          */
    COUNT        *recsize   /* out: Number of data bytes read   */
);

/* update a binary record */
FUNCTION CODE f_brewrite
(
    struct SFILE *f,        /* in/out: SFILE for the file   */
    GENPTR       buffer,    /* in: record to be written     */
    COUNT        size       /* in: size of buffer in bytes  */
);

/* write a binary record */
FUNCTION CODE f_bwrite
(
    struct SFILE *f,        /* in/out: SFILE for the file   */
    GENPTR       buffer,    /* in: record to be written     */
    COUNT        size       /* in: size of buffer in bytes  */
);

/* close a file */
FUNCTION CODE f_close
(
    struct SFILE *f,        /* in/out: SFILE for open file    */
    FUNINT       disp       /* in: F_KEEP or F_DELETE disposition */
);

FUNCTION CODE f_crack
(
    TEXT           fspec[],        /* in: host file specification  */
    TEXT           deflibr[],      /* in: default library          */
    TEXT           defname[],      /* in: default name             */
    TEXT           deftype[],      /* in: default type             */
    struct FSBLOCK *fsblock,       /* out: constructed FSBLOCK     */
    TEXT           errstr[]        /* out: 'at or near' string     */
                                   /* (if code is fail)            */
);

/* Move position context */
FUNCTION VOID f_movpos
(
    struct POSCTX *from,    /* in: source of position context       */
    struct POSCTX *to       /* out: destination                     */
);

/* extract library component from file spec */
FUNCTION CODE f_libr
(
    TEXT fspec[],            /* in: host file specification      */
    TEXT library[FLIBRSIZ+1] /* out: library field if any        */
                             /* if this address is NULL, then ok */
);

/* extract file name field from a host file spec */
FUNCTION CODE f_name
(
    TEXT fspec[],           /* in: host file specifiation   */
    TEXT name[FNAMESIZ+1]   /* out: file name field         */
);

/* open a file for block I/O */
FUNCTION CODE f_opnbio
(
    struct SFILE *f,        /* out: file context block      */
    FUNINT       lun,       /* in: lun to use for file      */
    TEXT         fspec[],   /* in: Host file name           */
    FUNINT       access,    /* in: F_READ, F_WRITE,         */
                            /* F_EXTEND, F_OVER, F_CREATE   */
    TEXT         filename[] /* Out: Resultant file name     */
);

/* open file given a FSBLOCK */
FUNCTION CODE f_opnblk
(
    struct SFILE   *sfile,    /* in/out: sfile structure      */
    FUNINT         lun,       /* in: lun to use               */
    struct FSBLOCK *fsblock,  /* in: fsblock                  */
    FUNINT         access     /* in: F_READ, F_WRITE, ...     */
);

/* open sequential file */
FUNCTION CODE f_open
(
    struct SFILE *f,        /* out: file context block      */
    FUNINT       lun,       /* in: lun to use for file      */
    TEXT         library[], /* in: name of library          */
    TEXT         name[],    /* in: name of file             */
    TEXT         type[],    /* in: type of file. no period. */
    FUNINT       access     /* in: F_READ, F_WRITE,         */
                            /* F_EXTEND, F_OVER             */
);

/* open file given host-dependent file spec */
FUNCTION CODE f_ophf
(
    struct SFILE *f,        /* out: file context block      */
    FUNINT       lun,       /* in: lun to use for file      */
    TEXT         fspec[],   /* in: host file name           */
    FUNINT       access     /* in: F_READ, F_WRITE,F_EXTEND */
                            /* F_OVER, or'ed with F_NOLOWER   */
);

/* open file given filespec and defaults */
FUNCTION CODE f_opnspc
(
    struct SFILE *sfile,    /* in/out: SFILE struct         */
    FUNINT       lun,       /* in: lun to use               */
    TEXT         fspec[],   /* in: host file spec           */
    TEXT         deflibr[], /* in: default library          */
    TEXT         defname[], /* in: default file name        */
    TEXT         deftype[], /* in: default file type        */
    FUNINT       access     /* in: F_READ, F_WRITE...       */
);

/* parse file specification */
FUNCTION CODE f_parse
(
    TEXT spec[],                /* in: file specfication        */
    TEXT libr[FLIBRSIZ+1],      /* out: library component       */
    TEXT name[FNAMESIZ+1],      /* out: name component          */
    TEXT type[FTYPESIZ+1],      /* out: type component          */
    TEXT attr[FATTRSIZ+1],      /* out: version string          */
    TEXT errstr[STRINGSIZ+1]    /* out: 'at or near' string if  */
                                /* syntax of spec is bad        */
);

/* read using block io */
FUNCTION CODE f_rdbio
(
    struct SFILE *f,        /* in/out: SFILE for the file   */
    int          sector,    /* in: sector number            */
    GENPTR       buffer,    /* out: buffer with output data */      
    int          size       /* in: size of buffer in bytes  */
);

/* read a string */
FUNCTION CODE f_readstring
(
    struct SFILE *f,        /* in/out: SFILE for an opened file  */
    COUNT        maxsiz,    /* in: max string siz */
    TEXT         string[]   /* out: record with EOS terminator   */
);

/* rewind file */
FUNCTION CODE f_rewind
(
    struct SFILE *f         /* in/out: opened SFILE structure */
);

/* set positioning context for next read */
FUNCTION VOID f_setpos
(
    struct SFILE  *f,       /* in/out: opened SFILE             */
    struct POSCTX *posctx   /* in: postion context              */
);

/* build host file specification */
FUNCTION COUNT f_spec
(
    struct FSBLOCK *fsblock,         /* in: file spec block  */
    TEXT           fspec[FSPECSIZ+1] /* out: VMS file spec   */
);

/* extract file type field from a host file spec */
FUNCTION CODE f_type
(
    TEXT fspec[],           /* in: host file specifiation   */
    TEXT type[FTYPESIZ+1]   /* out: file type field         */
);

/* write text record */
FUNCTION CODE f_write
(
    struct SFILE *f,        /* in/out: SFILE for the file   */
    TEXT         string[]   /* in: record to be written     */
);

/* write using block I/O */
FUNCTION CODE f_wrtbio
(
    struct SFILE *f,        /* in/out: SFILE for the file   */
    int          sector,    /* in: sector number            */
    GENPTR       buffer,    /* in: buffer with data to write */     
    int          size       /* in: size of buffer in bytes  */
);

/* see if file exists */
FUNCTION BOOL f_exists
(
    TEXT fspec[],           /* in: host file spec           */
    TEXT deflibr[],         /* in: default library          */
    TEXT defname[],         /* in: default file name        */
    TEXT deftype[]          /* in: default file type        */
);

/* set event control flag */
FUNCTION VOID e_set
(
    struct ECB *ecb         /* ptr to event control block   */
);

/* get process id */
FUNCTION VOID get_pid
(
    long *pid
);

FUNCTION VOID get_time
(
    TEXT atime[]            /* output: current asci time */
);

/* get login name of the current user */
FUNCTION VOID get_user
(
    TEXT name[USERNAMESIZ+1]      /* out: name of current user  */
);

/* get session id (same as process id) for TM */
FUNCTION VOID getsession
(
    TEXT session[]          /* output: session id   */
);

/* get user library */
FUNCTION VOID getulib
(
    TEXT userlib[]          /* output: user library */
);

/* clear an event control flag */
FUNCTION VOID e_clear
(
    struct ECB *ecb         /* ptr to event control block   */
);

/* return TRUE if the current process is interactive */
FUNCTION BOOL inter_proc(void);

/* sleep for msec millisecond */
FUNCTION VOID wait_hold
(
    FUNINT msec             /* time in milli-seconds        */
);

/* format parameter name */
FUNCTION CODE m_fpname
(
    TEXT  name[],           /* in: variable name    */
    TEXT  line[],           /* IN/OUT: formatted parameter rec */
    COUNT length            /* IN: line length */
);

/* format parameter values */
FUNCTION CODE m_fpval
(
    struct VARIABLE *vv,    /* IN: pointer to a variable */
    COUNT           index,  /* IN:index to value vector  */
    TEXT            line[], /* IN/OUT: formatted parameter rec */
    COUNT           stdlen, /* IN: standard line length  */
    COUNT           longlen /* IN: maximum line length   */
);

/* format reference name for a named parameter */
FUNCTION CODE m_frname
(
    struct VARIABLE *vv,    /* IN: pointer to a variable */
    TEXT            line[], /* IN/OUT: formatted parameter rec */
    COUNT           stdlen, /* IN: standard line length  */
    COUNT           longlen /* IN: maximum line length   */
);

/* add quotes around a string */
FUNCTION VOID addqu
(
    TEXT s[]                /* in/out: string to quote                      */
);

/* check that there are no more non-white tokens */
FUNCTION CODE chkend 
(
    struct SYNBLK *sb       /*  in/out: syntax block                */
);

/* returns TRUE if we are now positioned at "--" followed by white, comsep, 
   or EOS */
FUNCTION BOOL dash_dash 
(
    struct SYNBLK *sb
);

/* report separator token if next in command stream */
FUNCTION CODE fndsep 
(
    struct SYNBLK *sb       /* in/out: syntax block         */
);

/* free command line values from dynamic memory */
FUNCTION VOID free_val 
(
    TEXT   *value[],        /* in/out: array of pointers to strings */
    FUNINT count            /* in/out: actual number of values      */
);

/* get a value field */
FUNCTION CODE getfld 
(
    struct SYNBLK *sb,      /* in/out: syntax block                 */
    TEXT          *field    /* out: field string of length TOKESIZ+1*/
);

/* get a keyword from the command stream */
FUNCTION CODE getkey
(
    struct SYNBLK *sb,            /* in/out: symbol table         */
    TEXT          key[TOKESIZ+1]  /* out: parameter name          */
);

/* get qualifier string */
FUNCTION CODE getqlf 
(
    struct SYNBLK *sb,      /* in/out: syntax block                 */
    TEXT          qualstr[] /* out:  qualifier string               */
);

/* get next token from command stream in syntax block */
FUNCTION CODE gettok 
(
    struct SYNBLK *sb,            /* in/out: syntax block                 */
    TEXT          *token          /* out: token (if the function value
                                     is S_WHITE, token is one blank,
                                     if the function value is S_COMSEP,
                                     token is "," */
);

/* get value string(s) from the command stream */
FUNCTION CODE getval 
(
    struct SYNBLK *sb,      /* in/out: syntax block         */
    TEXT          *value[], /* out: array of pointers to strings    */
    FUNINT        maxval,   /* in:  dimension of value      */
    COUNT         *count    /* out: actual number of values:*/
                            /* -1 = no value; 0 = --;       */
);

/* get verb from a command stream */
FUNCTION CODE getvrb 
(
    struct SYNBLK *sb,      /* in/out: syntax block                 */
    TEXT          *verb     /*out: verb                             */
);

/* initialize syntax block for syntax package */
FUNCTION VOID initok 
(
    struct SYNBLK *sb,      /* out: syntax block to initialize      */
    TEXT          *usrstr   /* in:  command stream                  */
);

/* check if a string is a valid TAE standard name */
FUNCTION CODE name_check
(
    TEXT name[]             /* in: candidate name           */
);

/* strip quote symbols from a quoted string */
FUNCTION VOID strpqu 
(
    TEXT s[]                /* in/out: string to strip              */
);

/* TRUE if string contains a dash for a subcommand */
FUNCTION BOOL subchar
(
    TEXT *s                 /* in:  string                          */
);

/* put a syntax error message in syntax block */
FUNCTION VOID synerr 
(
    struct SYNBLK *sb,      /* in/out: syntax block                 */
    TEXT          *es       /* in:  error message string            */
);

/* Set substitution function and context */
FUNCTION FUNCTION_PTR d_setsub
(
    FUNCTION_PTR new_function     /* in: pointer to substitute function */
);

/* set context to be passed to substitute function */
FUNCTION GENPTR d_setctx 
(
    GENPTR new_context      /* in: new context for substitute */
);

/* initialize directive package */
FUNCTION VOID d_init
(
    struct DIRBLK *db,            /* OUT: pointer to directive block */
    struct SFILE  *sf,            /* IN:  pointer to file's SFILE    */
    TEXT          libr[],         /* IN:  file library name          */
    TEXT          **sys_char,     /* IN:  ptr to system characteristics*/
    FUNINT        char_cnt        /* IN:  count of characteristic strings    */
);

/* read next directive record and return directive type */
FUNCTION CODE d_dirctv
(
    struct DIRBLK *db,      /* IN: pointer to directive block */
    TEXT          dirctv[], /* OUT: directive type (including '.') */ 
    TEXT          field[]   /* OUT: associated text */
);

/* read next directive record and return directive type, allow for 
   continuation */
FUNCTION CODE d_cdirctv
(
    struct DIRBLK *db,      /* IN: pointer to dirctive block     */
    TEXT          dirctv[], /* OUT: directive type (including .) */
    TEXT          field[]   /* OUT: associated text              */
);

/* search for the next directive record of a given name */
FUNCTION CODE d_search
(
    struct DIRBLK *db,      /* IN: directive block               */
    TEXT   dirctv[],        /* IN: directive name  to search for */
    TEXT   field[]          /* OUT: associated text              */
);

/* read the next line of text for a directive */
FUNCTION CODE d_text
(
    struct DIRBLK *db,      /* IN: pointer to directive block */
    TEXT          record[]  /* OUT: text record */
);

/* open the given include file */
FUNCTION CODE d_incopen
(
    struct DIRBLK *db,      /* IN: directive block  */
    struct POSCTX *curinc   /* IN: posctx of .include record */
);

/* close include file */
FUNCTION VOID d_incclose
(
    struct DIRBLK *db       /* directive block      */
);

/* parse the given record into two fields */
FUNCTION VOID parserec
(
    TEXT *recptr,           /* IN: ptr to a DIRECTIVE record */
    TEXT *nameptr,          /* OUT: ptr to name */
    TEXT field[]            /* OUT: associated field */ 
);

/* initializes a block of storage so other r_ calls can use it */
FUNCTION VOID r_init
(
    ALIGN  *storage,        /* output: block of memory      */
    FUNINT bytes            /* input: bytes in storage      */
);

/* allocate restricted storage */
FUNCTION GENPTR r_alloc
(
    ALIGN  *s,              /* input/output: storage block  */
    FUNINT bytes            /* input: bytes to allocate     */
);

/* deallocate within restricted storage */
FUNCTION VOID r_dealloc
(
    ALIGN  *s,              /* storage area                 */
    GENPTR p                /* block to de-allocate         */
);

/* returns GENPTR to the highest+1 byte allocated so far */
FUNCTION GENPTR r_top
(
    ALIGN *s                /* storage area                 */
);

/* makes pointer relative to its storage block */
FUNCTION VOID r_setrel
(
    ALIGN  *s,              /* input: storage area name             */
    GENPTR pp,              /* input: address of value to fix       */
    GENPTR p                /* input: pointer value to make rel     */
);

/* makes a relative pointer an absolute pointer */
FUNCTION GENPTR r_setabs
(
    ALIGN  *s,              /* input: storage block         */
    GENPTR pp               /* input: address of pointer    */
);

/* reset the size of a restricted area when copied to another area */
FUNCTION VOID r_newsiz
(
    ALIGN *s,               /* in: address of pool                  */
    COUNT bytes             /* in: number of bytes in pool          */
);

/* converts all pointers in a symbol table to absolute so the table can be
   easily manipulated */
FUNCTION VOID makeabs
(
    struct SYMTAB *symtab,  /* input/output: symbol table*/
    ALIGN         *s        /* input: pooled storage area*/
);

/* make all pointers in a symbol table relative */
FUNCTION VOID makerel
(
    struct SYMTAB *sym,     /* symbol table to set relative.        */
    ALIGN         *s        /* pointer to storage area              */
);

/* create a path */
FUNCTION CODE c_crepath 
(
    struct PATH *path,      /* Out: Path control block        */
    FUNINT path_size,       /* In: Path size                  */
    TEXT   path_name[],     /* In: Path name                  */
    FUNINT type             /* In: temporary (TMPMBX) or      */
                            /*     permanent (PRMMBX)         */
);

/* delete a path */
FUNCTION CODE c_delpath 
(
    struct PATH *path       /* In: Path Control Block         */
);

/* connect to a path */
FUNCTION CODE c_conpath 
(
    struct PATH *path,      /* Out: Path Control Block        */
    TEXT        path_name[] /* In:  Path name                 */
);

/* disconnect from a path */
FUNCTION CODE c_discpath 
(
    struct PATH *path       /* In: Path Control Block         */
);

/* transmit a message over the specified path */
FUNCTION CODE c_putmsg 
(
    struct PATH *path,      /* In: Path Control Block         */
    GENPTR      buffer,     /* In: Message to be output       */
    FUNINT      size        /* In: Message size               */
);

/* transmit a message over a path created by a process other than the 
   calling process */
FUNCTION CODE c_pmwcd 
(
    TEXT   path_name[],     /* In:  Path name                 */
    GENPTR buffer,          /* In:  Message to be output      */
    FUNINT size,            /* In:  Message size              */
    TEXT   errmsg[]         /* Out: Error message             */
);

/* receive a message from the specified path */
FUNCTION CODE c_getmsg 
(
    struct PATH *path,      /* In:  Path Control Block        */
    GENPTR      buffer,     /* Out: Buffer to store msg in    */
    COUNT       *size       /* In:  Buffer size               */
);

/* read next message, fixed length */
FUNCTION COUNT c_read 
(
    struct PATH *path,      /* in: path structure                   */
    FUNINT  fd,             /* in: file desc from c_accept          */
    GENPTR  buffer,         /* in: buffer to receive next message   */
    FUNINT  size            /* in: buffer size                      */
);

/* deallocate a string in a primitive value */
FUNCTION VOID deal_pval
(
    struct VALUE *value,    /* input: the value             */
    CODE         class      /* input: value class           */
);

/* parse a TCL variable name and extract the simple leaf name */
FUNCTION VOID p_get_leaf 
(
    const char *name,       /* in: full qualified name      */
    TEXT leaf[],            /* out: simple right-hand name  */
    TEXT remainder[]        /* out: left-hand portion       */
); 

/* send a data block to a parent task */
FUNCTION CODE c_sndp
(
    GENPTR block,           /* in: data block to send           */
    FUNINT blksize          /* in: number of characters         */
);

/* receive a data block from a parent task */
FUNCTION CODE c_rcvp
(
    GENPTR block,           /* in: pointer to receive buffer   */
    FUNINT blksize          /* in: number of characters        */
                            /*     dimensioned by 'block'      */
);

/* send a data block to a parent task from an async monitor */
FUNCTION CODE c_snpa
(
    GENPTR block,           /* in: data block to send           */
    FUNINT blksize          /* in: number of characters         */
);

/* process init */
FUNCTION CODE t_pinit 
(
    COUNT *lines,           /* out: number of lines on crt  */
    COUNT *columns,         /* out: number of columns       */
    CODE  *type             /* out: type of terminal:       */
                            /*      T_CRT                   */
                            /*      T_NOTCRT                */
                            /*      T_NOTTERM               */
);

/* enable for next SIGINT signal */
FUNCTION CODE t_attn
(
    struct ECB *ecb         /* in/out: ecb to enable        */
);

/* returns true if an event has occurred */
FUNCTION BOOL e_occur
(
    struct ECB *ecb 
);

/* get next keyboard token and echo */
FUNCTION CODE t_gettok (void);

/* get next keyboard token and echo, assuming the escape of an escape sequence
   has already been read */
FUNCTION CODE t_1gettok (void);

/* enable terminal output logging */
FUNCTION CODE t_entol 
(
    TEXT *fspec,            /* in: file spec for file to log to     */
    TEXT **errmsg           /* out: pointer to errmsg               */
);

/* disable terminal output logging */
FUNCTION CODE t_distol (void);

/* return screen update counter */
FUNCTION LONG t_count(void);

/* get terminal position */
FUNCTION VOID t_getpos 
(
    COUNT *line,            /* out: line number     */
    COUNT *column           /* out: column number   */
);

/* display qualifiers (if any) */
FUNCTION VOID disp_qual 
(
    struct VARIABLE *v,     /* in: variable with (maybe) qualifiers */
    TEXT            name[]  /* in: qualified name of variable       */
);

/* display a variable */
FUNCTION VOID disp_var
(
    struct VARIABLE *v,           /* in: variable being displayed         */
    TEXT            var_name[]    /* in: name to use in var display       */
);

#endif
