#ifndef TAELIB_H
#define TAELIB_H

int m_msg
(
    const char *message,  /* I: message to be logged */
    const char *key       /* I: message key */
);

CODE t_init
(
   COUNT *lines,    /* out: number of lines on crt*/
   COUNT *columns,  /* out: number of columns */
   CODE  *type      /* out: type of terminal: T_CRT, T_NOTCRT, T_NOTTERM */
);

CODE t_pos
(
   FUNINT line,     /* in: line number (start at 1)   */
                    /*  -1 means use current line */
   FUNINT column    /* in: column number (start at 1) */
);

CODE t_output
(
    FUNINT line,    /* in: line number (start at 1)   */
    FUNINT column,  /* in: column number (start at 1) */
    const char *string /* in: string to write  */
);

CODE t_read
(
    TEXT string[],          /* out: string read */
    CODE *term              /* out: terminator used:*/
                            /*      T_CR (carriage return)*/
                            /*      T_ESCAPE (escape)*/
);

/* write line to terminal */
FUNCTION CODE t_write 
(
    const char *string,     /* in: string to write          */
    FUNINT cc               /* in: carriage control:        */
                            /*     T_STDCC (standard)       */
                            /*     T_NULL  (none)           */
                            /*     T_PROMPT                 */
                            /*     T_DOUBLE                 */
);

void t_bell(void);

void t_clear
(
  void
);

int t_highlight
(
    char *string /* I/O: string to highlight.  Make sure there is extra space
                         to attach escape characters. */
);

void q_init
(
    struct PARBLK *p,   /* I/O: PARBLK to initialize */
    int pool_size,      /* I: bytes allocated in the memory pool */
    int mode            /* I: P_ABORT or P_CONT */
);

int q_dynp
(
    struct PARBLK *p,   /* I: PARBLK to send */
    const char *pdfspec,/* I: pdf file spec */
    int  mode           /* I: M_SUBPDF or M_FULLPDF */  
);

int p_inim
(
    struct PARBLK *block,   /* I/O: parm block to be initialized */
    int blksiz,             /* I: size of block in bytes */
    int mode                /* I: P_ABORT or P_CONT */
);

CODE t_input
(
    FUNINT line,            /* in: line number (start at 1)   */
    FUNINT column,          /* in: column number (start at 1) */
    TEXT   string[],        /* out: string typed*/
    CODE   *term            /* out: terminator: */
                            /* T_CR or T_ESCAPE*/
);

int t_lclear
(
    int line,  /* I: line to clear (-1 = current line) */
    int col    /* I: start clearing at this column */
);
/* t_lclear returns SUCCESS when it succeeds (all the time) */

struct VARIABLE *p_find
(
  struct PARBLK *,
  const char *
);

void p_inifor
(
  struct PARBLK *
);


int p_forp
(
   struct PARBLK *,
   char *,
   int
);


int q_intg
(
    struct PARBLK *p,      /* I: V-block                  */
    const char *name,      /* I: variable name            */
    FUNINT count,          /* I: count of variable        */
    TAEINT intg[],         /* I: value vector             */
    FUNINT mode            /* I: P_UPDATE or P_ADD        */
);

int q_real
(
    struct PARBLK *p,      /* I: V-block                  */
    const char *name,      /* I: variable name            */
    FUNINT count,          /* I: count of variable        */
    TAEFLOAT real[],       /* I: reals                    */
    FUNINT mode            /* I: P_UPDATE or P_ADD        */
);

int q_string
(
    struct PARBLK *p,      /* I: V-block                                  */
    const char *name,      /* I: variable name                            */
    FUNINT count,          /* I: count of vector (0 means set count = 0)  */
    const char *vector[],  /* I: vector of string ptrs                    */
    FUNINT mode            /* I: P_UPDATE or P_ADD                        */
);

/* set one string value in PARBLK, rather than a vector of strings as per 
   q_string */
FUNCTION CODE q_one_string
(
    struct PARBLK *p,       /* V-block                      */
    TEXT          name[],   /* in: variable name            */
    COUNT         idx,      /* in: which string value, idx > 0  */
    TEXT          onestr[], /* in: string ptr               */
    FUNINT        mode      /* in: P_UPDATE or P_ADD        */
);

int q_out(struct PARBLK *p);

void z_exit
(
    int sfi,         /* I: return code */
    const char *skey /* I: return key */
);

FUNCTION FILE *z_init 
(
    struct PARBLK *block,   /* out: parameter block         */
    FUNINT        mode      /* in: mode: P_ABORT or P_CONT  */
);

#endif
