/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/



/*
 *	transsym.  Contains functions for transforming symbol tables
 *	to/from relative and absolute pointers.  
 *
 *	CHANGE LOG:
 *
 *	11-jul-83	Purged change log, deleted checkout records,
 *			and audited global definitions...jtm
 *	12-aug-83	Check pointers before using them in makeabs()...dm
 *	09-sep-83	Allow null v_cvp in makeabs...palm
 *	12-oct-83	Fix GENPTR declarations...palm
 *	14-sep-84	PR 639: makerel, makeabs to save special ptrs...peb
 *	07-nov-84	Alphabetize, TCL 67: handle parm qualifiers...peb
 *	13-nov-84	TCL 117: Handle v_nref...peb
 *	20-nov-84	TCL 67: Use v_pv12...peb
 *	29-mar-87	In makeabs, we use LARGE_P_BYTES for the estimated
 *			max pool size so we can take large PARBLKs...palm
 *	14-jul-87	Process qualifiers unconditionally, not just
 *			for PARMs...palm
 *      07-sep-88       Removed upper bound check because ULTRIX
 *                      does not like it...tpl
 *	29-jan-89	change function names for convert utility...palm
 *			delete makerel...palm
 */

#include	"taeconf.inp"
#include	"tminc.inc"
#include	"resinc.inc"		/* inc for restricted allocation*/
#include	"parblk.inc"		/* parameter block structure    */




/*
 *	makeabs. Converts all pointers in a symbol table to absolute so
 *	that the table can be easily manipulated.  We assume that the
 *	symbol table was previously made "all pointers relative".
 *
 *	If a bad link is found, we stop the makesabs process to avoid
 *	crashing.   Such can only happen for blocks from the outside
 *	world (disk or subproces) and, for such blocks, the chk_parblk
 *	function will catch the error.
 *
 *	Handles the special pointers.
 *
 *	TAE versions 1.2 and earlier did use parameter qualifier symbol
 *	tables, nor parameters with type=NAME.  If the v_pv12 bit of 
 *	the variable structure is TRUE, the variable was created by a TAE
 *	version post V 1.2.
 */

    FUNCTION VOID old_makeabs(symtab, s)

	struct SYMTAB	*symtab;		/* input/output: symbol table*/
	ALIGN		*s;			/* input: pooled storage area*/
	
	{
	struct VARIABLE	*v;			/* working VARIABLE pointer  */
	COUNT		i;
    	GENPTR		pool1;
	GENPTR		vgen;
	TEXT 	   	**vc;			/* ptr to string ptr vector  */
	GENPTR		r_setabs();		/* ref'ed by SET_ABS macro   */
	GENPTR		cvp;
	GENPTR		gen;
	BOOL		deref_default;		/* TRUE if deref'd PARM...	*/
						/* DEFAULT -- compiled PDFs only*/
        
	pool1 = (GENPTR)s ;			/* lower limit of block	     */
	SET_ABS((*symtab).link, s, struct VARIABLE *); /* set initial link   */
	for (v=(*symtab).link; v != NULL; v=(*v).v_link)
	    {
	    vgen = (GENPTR) v;			/* generic form of v for...  */
	    if (vgen < pool1)
		break;				/* stop here to avoid crash  */
	    SET_ABS((*v).v_link, s, struct VARIABLE *);
	    deref_default = ((*v).v_class == V_PARM  &&  (*v).v_deref);
	    if ((*v).v_class == V_GLOBAL)	/* fix reqd on defin PDF ptr?*/
		{
		SET_ABS((*v).v_pdf, s, struct DEFPDF *);
		gen = (GENPTR) (*v).v_pdf;
		if (gen != NULL && (gen < pool1 ))
		    break;		
		}
	    if ((*v).v_type == V_NAME)		/* NAME parms only from compiled*/
		{
		if (!(*v).v_pv12)		/* no NAME parms from old TAE vers*/
		    break;
		SET_ABS((*v).v_nref, s, TEXT *);
		gen = (GENPTR) (*v).v_nref;
		if (gen != NULL && (gen < pool1))
		    break;		
		}
	    else				/* not NAME parm		*/
		{
		SET_ABS((*v).v_valid, s, GENPTR);
		gen = (*v).v_valid;
		if (gen != NULL && (gen < pool1 ))
		    break;		
		SET_ABS((*v).v_cvp, s, GENPTR);	/* now the current value     */
		cvp = (*v).v_cvp;
		if (cvp != NULL && (cvp < pool1 ))
		    break;			/* stop here to avoid crash  */
		if ((*v).v_type == V_STRING  &&  !deref_default)
		    {
		    vc = (TEXT **) cvp;		/* ptr to current string ptr */
		    for (i=0; i < (*v).v_count; i++)
			{
			SET_ABS(vc[i], s, TEXT *);
			if ((GENPTR) vc[i] < pool1 )
			    break;		
			}
		    }
		SET_ABS((*v).v_dvp, s, GENPTR);	/* now the default value     */
		cvp = (*v).v_dvp;
		if (cvp != NULL && (cvp < pool1 ))
		    break;			/* stop here to avoid crash  */
		if ((*v).v_type == V_STRING  &&  !deref_default)
		    {
		    vc = (TEXT **) cvp;		/* ptr to current string ptr */
		    for (i=0; i < (*v).v_dcount; i++)
			{
			SET_ABS(vc[i], s, TEXT *);
			if ((GENPTR) vc[i] < pool1 )
			    break;		
			}
		    }
		}
	    if ((*v).v_pv12 && (*v).v_qualst.link != NULL)
		old_makeabs(&(*v).v_qualst, s);	/* recursive - parm qual sym tab*/
	    }
        return;
	}		
