#ifndef LOCAL_PROTOS_H
#define LOCAL_PROTOS_H

#include "forstr.inp"

FUNCTION CODE p_attr 
(
    struct PARBLK *block,         /* in:  parameter block                 */
    TEXT          name[],         /* in:  name of the parameter           */
    CODE          *type,          /* out: parameter  type                 */
    COUNT         *n,             /* out: number of values for the parm   */
    BOOL          *dflt,          /* out: TRUE if parm was defaulted      */
    CODE          *access         /* out: access mode if a file           */
                                  /* P_IN, P_OUT, P_INOUT, P_NONE         */
);

FUNCTION struct VARIABLE *p_fvar 
(
    struct PARBLK *block,   /* in:  parameter block         */
    const TEXT name[]       /* in:  parameter name          */
);

/* check if the process is a child of TM */
FUNCTION BOOL c_child_proc(void);

/* report error and abort if in P_ABORT mode; continue otherwise */
FUNCTION VOID x_error
(
    FUNINT mode,            /* in: P_ABORT or P_CONT */
    TEXT   message[],       /* in: message text (message format) */
    TEXT   key[],           /* in: message key */
    ...                     /* in: variable number of arguments */
);

FUNCTION VOID  parm_err
(
    TEXT message[],         /* in: message */
    TEXT key[]              /* in: message key */
);

/* convert C string to FORTRAN-77 format */
FUNCTION CODE s_c2for
(
    TEXT   *c_string,      /* in: source string            */
    FORSTR *for_string,    /* out: FORTRAN-77 string       */
    FUNINT index           /*in:index in for_string(zero if for_string scalar)*/
);

/* declare a FORTRAN-77 string given a C structure */
FUNCTION VOID s_declfor
(
    FORSTR *for_string,     /* in/out: uninitialized FORTRAN string */
    TEXT   *address,        /* in: address of string        */
    FUNINT length           /* in: fixed-length size        */
);

/* convert FORTRAN-77 string to C string */
FUNCTION CODE s_for2c
(
    FORSTR *for_string,           /* in: FORTRAN string           */
    TEXT   c_string[STRINGSIZ+1], /* out: C string                */
    FUNINT index                  /* in: index in fortran string  */
                                  /* (zero if for_string scalar)  */
);

/* return length of FORTRAN-77 string */
FUNCTION COUNT s_forlen
(
    FORSTR *for_string
);

/* Return start address  of FORTRAN-77 string text */
FUNCTION TEXT *s_fortxt
(
    FORSTR *for_string
);

/* get host spec when TAE spec begins with a percent character */
FUNCTION VOID xzper
(
    FORSTR *tae_spec,       /* in: tae spec FORTRAN string  */
    FORSTR *host_spec,      /* out: host spec FORTRAN string*/
    TAEINT *filemode,       /* in: filemode                 */
    TAEINT *status          /* out: status                  */
);

/* get host spec when tae spec does not begin with percent character */
FUNCTION VOID xzstd
(
    FORSTR *tae_spec,      /* in: tae spec FORTRAN string  */
    FORSTR *host_spec,     /* out: host spec FORTRAN string*/
    TAEINT *filemode,      /* in: filemode                 */
    TAEINT *status         /* out: status                  */
);

/* exit the process with the indicated code (SUCCESS or not) */
FUNCTION VOID procexit
(
    CODE exit_code          /* normal or abnormal   */
);

/* application initialization (empty routine) */
FUNCTION VOID z_call
(
    struct PARBLK *parblk   /* in: initial V-block   */
);

/* convert string to upper case and return length */
FUNCTION COUNT s_upper 
(
    TEXT *s
);

/* return pointer to substring if found in full string */
FUNCTION TEXT *s_substring 
(
    TEXT *substring,
    TEXT *fullstring
);

/* get host spec given TAE spec and file mode */
FUNCTION VOID xzhost
(
    TEXT   tae_spec[],      /* in: tae spec, C string       */
    TEXT   host_spec[FSPECSIZ+1],
                            /* out: host spec, C string     */
    TAEINT *filemode,       /* in: filemode                 */
    TAEINT *status          /* out: status                  */
);

/* get host spec when TAE spec begins with minus character */
FUNCTION VOID xzdash 
(
    FORSTR *tae_spec,       /* in: FORTRAN string           */
    FORSTR *host_spec,      /* out: FORTRAN string          */
    TAEINT *filemode,       /* in: filemode                 */
    TAEINT *status          /* out: status                  */
);

/* terminate application */
FUNCTION VOID BRIDGE2_NAME(xzexit)
(
    TAEINT *sfi,            /* in: success/fail indicator   */
    FORSTR *skey            /* in: error key string         */
);

/* raise a number to exp */
FUNCTION DOUBLE power 
(
    DOUBLE num,             /* input: the number to raise           */
    FUNINT exp              /* input: the exponent                  */
);

/* check the error code and put message in SFILE */
FUNCTION CODE chk_error
(
    struct SFILE *f,        /* in: pointer to SFILE         */
    CODE         err_type   /* in: type of error            */
);

/* perform substitutions in the file spec */
FUNCTION CODE f_subst
(
    TEXT *fspec,            /* in: file spec                        */
    TEXT *subspec           /* out: file spec after substitution    */
);

/* get text corresponding to a host code */
FUNCTION CODE get_host_msg 
(
    FUNINT msg_code,        /* in: host message code            */
    TEXT   message[],       /* out: corresponding message       */
    FUNINT max_length       /* in: max chars caller can handle  */
);

FUNCTION CODE p_mput
(
    TEXT message[],         /* In: message to be logged */
    TEXT key[]              /* In: message key */
); 

/* write message to standard output device, if not terminal */
FUNCTION VOID p_msgout
(
    TEXT msgbuf[],
    TEXT msgkey[]
);

/* write message to standard output device, if not terminal */
FUNCTION VOID m_msgout
(
    TEXT msgbuf[],
    TEXT msgkey[] 
);

/* set string value in PARBLK */
FUNCTION CODE q_shortstring
(
    struct PARBLK *p,       /* V-block                      */
    TEXT          name[],   /* in: variable name            */
    FUNINT        count,    /* in: count of vector          */
                            /* (0 means set count = 0)      */
    FUNINT        strsiz,   /* string size, 0 means set to STRINGSIZ*/
    TEXT          *vector[],/* in: vector of string ptrs    */
    FUNINT        mode      /* in: P_UPDATE or P_ADD        */
);

/* prepare a variable for a q_ operation */
FUNCTION CODE q_prep 
(
    struct PARBLK   *p,     /* in/out: PARBLK               */
    const char      *name,  /* in: paramter name            */
    FUNINT          count,  /* in: proposed parm count      */
    FUNINT          type,   /* in: parm type                */
    FUNINT          mode,   /* in: update or add            */
    struct VARIABLE **vv,   /* out: VARIABLE ptr            */
    FUNINT          strsiz  /* in: string size              */ 
);

/* allocate VARIABLE structure in restricted storage */
FUNCTION struct VARIABLE *q_alloc
(
    struct SYMTAB *symtab,  /* in: symtab to link to        */
    ALIGN         pool[],   /* in: restricted storgae block */
    TEXT          name[],   /* in: name of variable         */
    FUNINT        type,     /* in: variable type            */
    FUNINT        count,    /* in: number of values         */
    FUNINT        strsiz    /* in: string size              */
);

/* copy string to restricted storage */
FUNCTION TEXT *q_save 
(
    ALIGN pool[],           /* restrcted storage            */
    const TEXT string[]     /* string to copy               */
); 

/* set the max count of a variable */
FUNCTION CODE q_max 
(
    struct PARBLK *p,       /* in/out: PARBLK               */
    TEXT          name[],   /* in: paramter name            */
    FUNINT        count     /* in: proposed parm count      */
);

/* set the min count of a variable */
FUNCTION CODE q_min 
(
    struct PARBLK *p,       /* in/out: PARBLK               */
    TEXT          name[],   /* in: paramter name            */
    FUNINT        count     /* in: proposed parm count      */
);

/* set the string length of a variable */
FUNCTION CODE q_stringlength 
(
    struct PARBLK *p,       /* in/out: PARBLK               */
    TEXT          name[],   /* in: paramter name            */
    FUNINT        length    /* in: string length            */
);

/* set the parmpage of a variable */
FUNCTION CODE q_parmpage 
(
    struct PARBLK *p,       /* in/out: PARBLK               */
    TEXT          name[],   /* in: paramter name            */
    BOOL          flag      /* in: TRUE or FALSE flag       */
);

/* allocate VARIABLE structure in dynamic storage */
struct VARIABLE *Vm_Alloc
(
    struct SYMTAB *symtab,  /* in: symtab to link to        */
    TEXT          name[],   /* in: name of variable         */
    FUNINT        type,     /* in: variable type            */
    FUNINT        count,    /* in: number of values         */
    FUNINT        strsiz    /* in: string max size          */
);

/* allocate space for values */
char *Vm_AllocValue
(
    struct VARIABLE *v      /* in/out: variable to have values allocated  */
);

/* build VARIABLE structure */
struct VARIABLE *Vm_AllocVar
(
    struct SYMTAB *head     /* in/out: pointer to symbol table header */
);

/* delete values */
VOID Vm_FreeValue
(
    TEXT   *pv,             /*in: pointer to values */
    FUNINT type,            /*in: type of variable  */
    FUNINT count            /*in: current value cnt */
);

/* determine bytes for one value */
COUNT Vm_ValueSize
(
    FUNINT type             /* V_STRING, V_REAL, V_INTEGER          */
);

/* send PARBLK to parent task */
FUNCTION CODE q_sndp
(
    struct PARBLK *p,       /* PARBLK to send               */
    FUNINT        msgtype   /* P_OUTPUT, P_SUBPDF, P_FULLPDF*/
);

/* initialize parameter block formatting */
FUNCTION VOID p_inifor
(
    struct PARBLK *parblk   /* IN: parameter block */
);

/* format parameters in the parameter block */
FUNCTION CODE p_forp
(
    struct PARBLK *parblk,  /* IN: parameter block */
    TEXT          line[],   /* OUT: formatted parameter rec */
    COUNT         length    /* IN: line length */ 
);

/* get an integer parameter */
FUNCTION CODE p_intg 
(
    struct PARBLK *block,   /* in:  parameter block */
    TEXT          name[],   /* in:  parameter name */
    FUNINT        dimen,    /* in:  dimension of intg */
    TAEINT        intg[],   /* out: receives the values */
    COUNT         *count    /* out: value count */
);

/* get a real parameter */
FUNCTION CODE p_real 
(
    struct PARBLK *block,   /* in:  parameter block                 */
    TEXT          name[],   /* in:  parameter name                  */
    FUNINT        dimen,    /* in:  dimension of intg               */
    TAEFLOAT      real[],   /* out: receives the values             */
    COUNT         *count    /* out: value count                     */
);

/* get a string parameter */
FUNCTION CODE p_string  
(
    struct PARBLK *block,   /* in:  parameter block                 */
    TEXT          name[],   /* in:  parameter name                  */
    TEXT          ***sptr,  /* out: pointer to string vector        */
    COUNT         *count    /* out: number of strings               */
);

/* return host error code */
FUNCTION VOID p_herr
(
    struct PARBLK *block,   /* in: parameter block  */
    CODE          *hcode    /* out: host error code */
);

/* get the lun number assigned to standard output */
VOID FOR_NAME(getlun)
(
    long *lun               /* in: lun number               */      
);

/* set terminal type */
FUNCTION CODE tgetent 
(
    TEXT bp[],              /* out: storage area (not used) */
    TEXT name[]             /* in: name of terminal         */
);

/* move to cursor position */
FUNCTION TEXT *tgoto 
(
    TEXT   cm[],            /* in: "cm" string              */
    FUNINT destcol,         /* in: column                   */
    FUNINT destline         /* in: line                     */
);

/* get capability string */
FUNCTION TEXT *tgetstr 
(
    TEXT id[],              /* in: name of capability       */
    TEXT **area             /* in/out: advancing buffer ptr */
);

/* get integer capability */
FUNCTION int tgetnum 
(
    TEXT id[]               /* in: integer capability name  */      
);

/* get capability existence */
FUNCTION BOOL tgetflag 
(
    TEXT id[]               /* in: capability name          */
);

/* send string with delay information */
FUNCTION VOID tputs 
(
    TEXT   *cp,             /* in: string to write  */
    FUNINT affcnt,          /* in: ignored          */
    VOID   (*outc)()        /* in: output routine   */
);

/* find variable in parameter block */
FUNCTION struct VARIABLE *Vm_FindVar 
(
    GENPTR     h,           /* in:  no-pool parblk handle */
    TEXT name[]             /* in:  parameter name        */
);

#endif
