/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*
 *      m_msg. Send a message (from a 'c' application program )to the 
 *              terminal monitor for logging.
 *
 *      NOTE: THIS MODULE MUST BE MAINTAINED PARALLEL TO MPARM.C
 *
 *      CHANGE LOG:
 *
 *      11-mar-84       Change handshake implementation.
 *      21-aug-84       Fix error loop on handshake failure (PR #790)...dm
 *      14-sep-86       Fix bug in procexit calling seq...dm
 *      29-jan-87       Fix dimension in q_init calling seq...peb
 *      14-may-89       if null key, do not display...palm 
 *
 */

#include <stdio.h>
#include <string.h>
#if defined (sun)
#include <sys/vnode.h>
#endif
#include "taeconf.inp"
#include "parblk.inc"
#include "tminc.inc"
#include "terminc.inc"
#include "fileinc.inp"
#include "taelib.h"
#include "inttaelib.h"
#include "local_protos.h"


GLOBAL TEXT savekey[KEYSIZ+1];    /* last message key     */
GLOBAL FILE *stdo_file = NULL;    /* ptr to FILE for stdout */
GLOBAL BOOL term_std = TRUE;      /* terminal as stdout   */

#define         OUTMSGSIZ   STRINGSIZ           /* size of output message */ 
static    TEXT  stdbuf[OUTMSGSIZ+KEYSIZ+4];     /* standard output buffer */

FUNCTION CODE m_msg
(
    const char *message,    /* In: message to be logged */
    const char *key         /* In: message key */
) 
{
    CODE        code;

/* the following avoids declaring a full PARBLK just to send a message  */
#define ALDIM(bytes)    1+((bytes-1)/sizeof (ALIGN)) 
#define HEAD_SIZ (sizeof(struct PARBLK) - P_BYTES + 8)  /* 8 for align safety*/
#define MBLKDIM (3*sizeof(struct VARIABLE) + 3*OUTMSGSIZ + HEAD_SIZ)
                                                /* block size in bytes  */
    ALIGN       block[ALDIM(MBLKDIM)];          /* parameter block to send */
    struct      PARBLK  *parblk;                /* pointer to parameter block*/

    const TEXT  *vector[1];                     /* pointer to value */
    TEXT        msgbuf[OUTMSGSIZ+1];
    TEXT        msgkey[KEYSIZ+1];

    
    memset(block,0,sizeof(block));
    parblk = (struct PARBLK *) block;
    q_init (parblk, MBLKDIM - HEAD_SIZ, P_ABORT);       /* initialize block */
    memcpy((GENPTR)msgbuf,(GENPTR)message, OUTMSGSIZ);  /* copy locally */
    if (strlen(message) > OUTMSGSIZ)                    /* if too long- */
        msgbuf[OUTMSGSIZ] = EOS;                        /* truncate     */
    vector[0] = msgbuf;
    code = q_string(parblk, "MESSAGE", 1, vector, P_ADD); /* add message */
    if (code == SUCCESS)
    {
        int len = strlen(key);
        if (len > KEYSIZ)
            len = KEYSIZ;                               /* police length */
        strncpy(msgkey,key,len);                        /* copy key locally */
        msgkey[len] = '\0';
        vector[0] = msgkey;                             /* pointer to key   */
        code = q_string(parblk, "KEY", 1, vector, P_ADD); /* add key        */
    }
    if (code == SUCCESS)
    {
        code = q_sndp(parblk, M_HLOGMSG);               /* send message     */
        if (code == SUCCESS)
        {
            code = c_rcvp((GENPTR)parblk, sizeof(block)); /* rcv handsh blk */
            (*parblk).hostcode = code;                  /* save host code   */
            if (code != SUCCESS)
            {
                t_write("[TAE-MSGHANDS] Unable to confirm message reception "
                        "by TM", T_STDCC);
                procexit(code);
            }
        }
    }
    strcpy(savekey,msgkey);                             /* save the key     */
    m_msgout(msgbuf, msgkey);                           /* write to output  */
    return(code);
}

/*
 *      m_msgout. write message to standard output device, if not terminal.
 *
 */

FUNCTION VOID m_msgout
(
    TEXT msgbuf[],
    TEXT msgkey[] 
)
{

    if (!term_std && stdo_file != NULL) /* if stdout is a file */
    {
        stdbuf[0] = EOS;    
        if (msgkey && msgkey[0] != 0)           /* only if key exists   */
        {
            strcpy(stdbuf,"[");         /* add the key          */
            strcat(stdbuf,msgkey);
            strcat(stdbuf,"] ");
        }
        strcat(stdbuf,msgbuf);  /* add the message      */
        strcat(stdbuf,"\n");            /* add new line character */
        fputs(stdbuf, stdo_file);       /* write to stdout      */    
    }
}
