/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*
 * Parameter formatting routines.
 *
 * CHANGE LOG:
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *      22-aug-83       Updated for NULLABLES...dm
 *      25-aug-83       Return code FP_NULL for truncated values...dm
 *      07-sep-83       Updated for formatting function name changes...dm
 *
 ***********************************************************************
 */

#include <string.h>
#if defined (sun)
#include <sys/vnode.h>
#endif
#include "stdh.inp"              /* system standard */
#include "taeconf.inp"           /* TAE configuration */
#include "parblk.inc"            /* parameter block definitions */
#include "resinc.inc"            /* restricted allocation package */
#include "tminc.inc"             /* TM  related definitions */
#include "inttaelib.h"
#include "local_protos.h"

   
struct      FORCTX 
{
    struct  VARIABLE  *nxtvar;      /* variable to work with next */
    COUNT   nxtind;                 /* next index to value vector */
};

    
/*
 *       p_inifor:   Initialise for parameter block formatting
 */
FUNCTION VOID p_inifor
(
    struct PARBLK *parblk   /* IN: parameter block */
)
{
    struct  FORCTX      *forctx;                        /* pointer to pool */
  
    /* allocate storage in pool */
    forctx = (struct FORCTX *) r_alloc((*parblk).pool, sizeof (struct FORCTX));
    if (forctx != NULL)                   /* successful allocation */
    {
        (*forctx).nxtind = -1;            /* set indexes for first call*/
        (*forctx).nxtvar = (*parblk).symtab.link;
    }
    (*parblk).ctxptr = (GENPTR) forctx ;  /* save ptr in parblk */
}

/*
 * m_forp .  format the input variable parameters (in a linked list).
 *
 * TBD: elemination of proc name returned twice.
 */

FUNCTION CODE m_forp
(
    struct FORCTX *forctx,  /* IN: pointer format context block */
    TEXT          line[],   /* OUT: formatted parameter rec */
    COUNT         length    /* IN: line length           */
)
{
    struct VARIABLE *vv;                    /* pointer to a variable     */
    COUNT           index;                  /* index pointer             */
    COUNT           max_index;              /* max value of index ptr    */
    COUNT           nchar;                  /* number of characters in line */
    CODE            code=FAIL;

    if (forctx == NULL)                         /* FORCTX allocated failed   */
        return(FP_EOP); 
    if ((*forctx).nxtvar == NULL)
        return(FP_EOP);                         /* end of parameter list     */

    line[0] = EOS;
    index = (*forctx).nxtind;
    for (vv = (*forctx).nxtvar; vv != NULL; vv = (*vv).v_link)
    {
        if (index == -1)                        /* stuff name into line      */
        {                                   
            code = m_fpname((*vv).v_name, line, length);
            if (code == FAIL) 
                break;                    /* no more room      */
            index++;                            
        }
        max_index = ((*vv).v_count >= 1) ? ((*vv).v_count-1) : 0;
        for ( ; index <= max_index; index++)
        {
            /* format next value */
            code = m_fpval(vv, index, line, length, length);  
            if (code != SUCCESS) break;         /* no more room              */
        }
        if (code != SUCCESS) break;             /* exit outer loop also      */
        index = -1;                             /* reinit for next variable  */
    }
    if (code == TRUNCATE)                       /* couldnt fit in a new line */
        code = FP_NULL;
    else if (code == FAIL)                      /* no more space in line     */
    {
        if (strlen(line) == 0) 
            code = FP_NULL;                     /* line too small            */
        else                                                            
        {
            strcat(line,"+");           /* add continuation          */
            code = SUCCESS;                     /* regard as success         */
        }
    }
    else
    {                                       /* end reached               */
        nchar = strlen(line);
        line[nchar-2] = EOS;                    /* delete hanging ", "       */
    }
    (*forctx).nxtind = index;                   /* save new values           */
    (*forctx).nxtvar = vv;
    return(code);
}

/*
 *      p_forp :  Format parameters in the parameter block.
 */

FUNCTION CODE p_forp
(
    struct PARBLK *parblk,  /* IN: parameter block */
    TEXT          line[],   /* OUT: formatted parameter rec */
    COUNT         length    /* IN: line length */ 
)
{
    CODE                code;

    code = m_forp((struct FORCTX * )(*parblk).ctxptr, line, length);
    return(code);
}
