/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/




/********************************************************************
 *      TAE subroutine package error handling.  
 *
 *      This is in a separate module so that 
 *      a program can override TAE's outboard
 *      error handling, for example, you might
 *      want to use some of TAE's library without
 *      running under TAE and expecting to send
 *      error messages to TM.
 *
 ***************************************************************************
 * 
 * Change log:
 *
 *
 *      01-aug-88   Also handle applications running independent of tm...dm
 *
 */
#include <string.h>
#include <stdarg.h>         /* for variable number of arguments */
#if defined (sun)
#include <sys/vnode.h>
#endif
#include "taeconf.inp"
#include "parblk.inc"
#include "taelib.h"
#include "inttaelib.h"
#include "local_protos.h"


/* 
 *      x_error.   Report error and abort if in P_ABORT mode; continue
 *      otherwise.
 *
 */

FUNCTION VOID x_error
(
    FUNINT mode,            /* in: P_ABORT or P_CONT */
    TEXT   message[],       /* in: message text */
    TEXT   key[],           /* in: message key */
    ...
)
{
    TEXT        msgstring[2*STRINGSIZ+1];   /* formatted message string */
    va_list argumentp;	/* Variable number of arguments for 'format' string   */


    if (mode == P_ABORT || ( (mode & P_MODE_ABORT) == P_MODE_ABORT ) )
    {
        /* get the arguments to this function */
        va_start(argumentp,key);
        va_end(argumentp);

        /* Create the desired output message */
        vsprintf (msgstring, message, argumentp);

        /* allow upto STRINGSIZ, truncating if it is over that length */
        if (strlen(msgstring) > STRINGSIZ)
            strcpy(&msgstring[STRINGSIZ-3],"...");

        if (c_child_proc())                 
        {
            /* running as child of tm */
            parm_err(msgstring, key);
            z_exit(-1, key);            /* set $sfi, $skey              */
        }
        else
        {
            /* task running independent of tm */
            printf(msgstring);
            printf("\n");
            exit(1);
        }
    }
    else
        return;
}
