/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/



/* >>  UNIX  << */
/*      Non-portable Async Processing Routines
 *      --------------------------------------
 *
 * CHANGE LOG:
 *
 *      21-may-84       Initial release...dm
 *      12-jun-85       Update for TAE-V1.3 features...dm
 *      18-jun-85       Do not assume tm from TSYSLIB library...dm
 *      05-jul-85       Add block_opint(), other minor changes...dm
 *      23-jul-85       Fix lint errors...dm
 *      15-aug-85       Use SIGTERM instead of SIGKILL to abort the async
 *                      monitor...dm
 *      16-aug-85       Fix compilation errors for RCJM case...dm
 *      19-aug-85       Add function asy_recs...dm
 *      28-aug-85       Add handshake logic to asy_recs() and sig_io()...dm
 *      08-sep-85       Return SUCCESS on operator interrupt in asy_recs...dm
 *      01-aug-86       Make changes for ASYNC-PROCESS...lia
 *      12-aug-86       Cleanup definition of parent_pid and parent_env...lia
 *      13-aug-86       Fix PARENV define bug...lia
 *      15-aug-86       Check for SYS$OUTPUT set to NULL and reset it...lia
 *      19-aug-86       Add async_stdout() to update _STDOUT for ASYNC-PROCESS
 *                      ...lia
 *      11-dec-86       Fix lint warnings...peb
 *      21-jan-87       Check block size for output via pipe...lia
 ************************** Merge TCP changes...gc, 5 Mar 87 ****************
 *      02-jan-87       Call n_wait() rather than sleep() for TCPIP...dm 
 *      03-jan-87       Ignore operator interrupt before activating child...dm
 *      05-jan-87       Add asy_rcv_init() to initialize async msg recv...dm
 *      07-jan-87       Check for interrupt before n_wait in asy_recs...dm
 *      02-Apr-87       Add parameters to sig_io for test use...gc
 *      24-mar-88       Delete TAE_RCJM conditionals...ljn
 *      25-oct-88       Added #ifdef BSD_UNIX around SIGCONT to match other...ln
 *      08-nov-88       -vfork() defined for SYSV in taeconf.inp
 *                      -#ifdef BSD_UNIX -> #ifndef SYSV...ljn
 *      10-nov-88       If SYSV, #include fcntl.h...ljn
 *      27-jun-89       Temporary fix to enable TAE to build on pure
 *                      SYSV systems: #ifdef out set_signal calls...ljn
 *                      (True fix is to get #ifdef TAE_ASYNC to work again.)
 *      01-jul-89       types.h not included by file.h on SYSV machines...ljn
 *      11-sep-89       Added NOSIGVEC...ljn
 *      23-may-90       Removed RCJM stuff by referring to old TAE_RCJM...ljn
 *                      n_wait() is now sleep(), disable_intp()->signal()...ljn
 *      13-dec-90       TMIMAGE is now taetm...ljn
 *      17-jun-91       PR1099: use $TAEBIN/$TAEPLAT/taetm...ljn
 *      25-may-93       $TAEBIN now is what $TAEBIN/$TAEPLAT used to be...kbs
 *      08-dec-93       Solaris port (backported dag from aug-93)...cob
 *      18-apr-94       Concurrent port: declare sigblock...dgf
 *      24-aug-95       Removed fixed limit on number of environment
 *                      variables passed to child...raw
 *      06-nov-95       Added EROS specific modifications from Brian Davis'
 *                      initial port to solaris.
 *                      Jim Fenno - EROS Data Center
 *
 ****************************************************************************
 */

#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#ifdef macII
#include <sys/types.h>
#else
#ifdef SYSV
#include <fcntl.h>
#include <sys/types.h>
#endif
#endif
#include <sys/file.h>
#include <errno.h>
#include "taeconf.inp"       /* TAE configuration definitions            */
#include "inttaelib.h"
#include "tminc.inc"         /* TM-only host-independent definitions     */
#include "asyncinc.inc"      /* async processing definitions             */
#include "terminc.inc"       /* terminal package definitions             */
#include "tmhost.inp"        
#include "taskinc.inp"       /* task manipulation definitions            */
#include "taelib.h"
#include "local_protos.h"


#ifdef EROS
IMPORT struct SYMTAB glbtab;        /* TCL global symbols               */
#endif

static COUNT  asyact_num = 0; /* number of asyncs active          */
static int    rcv_pipe[2];    /* channel to receive all async msg */
static struct MON_MSG saved_msg; 
static struct TCB *waiting_tcb = NULL;    
                                    /* tcb of child waiting for msg     */
static BOOL msg_read;               /* msg read by asy_recs before exit */
static BOOL msg_exist=FALSE;        /* msg already there               */
static BOOL ignore_int=FALSE;       /* ignore operator interrupt*/
static LONG ackn_msg = ACKN_CODE;   /* max 4 bytes of acknl. to child  */

FUNCTION VOID sig_io
(
    int  sig                /* Signal number */
);

#ifdef masscomp
int sigblock();
#endif

/*
 *      abort_acb. Abort the monitor associated with an acb and
 *      release associated resources
 *              
 *              We also reset the async_request flag if necessary.  Note that
 *              there's a race condition here (a new request comes in after
 *              the call to find_oldest but before async_request is reset)
 *              that we're ignoring.
 */
FUNCTION VOID abort_acb
(
    struct ACB *acb         /* in: the acb              */
)
{
    IMPORT BOOL         async_request;          /* TRUE if requests waiting */
    CODE                code;
    struct ACB          *find_oldest();
    struct SFILE        f;


    /* delete the comm file     */
    if ((*acb).waiting)
    {
        code = f_opnspc(&f, SAVELUN, (*acb).requ_file, "", "",
                    ASY_TYPE, F_READ);
        if (code == SUCCESS) f_close (&f, F_DELETE);
    }
    (*acb).waiting = (*acb).active = FALSE;
    /* reset if no others waiting, otherwise leave it in current state */
    if (find_oldest() == NULL)                  
        async_request = FALSE;                  
    /* all done if a remote job   */
    if ((*acb).remote)                          
        return;                                 
#ifndef SYSV
    /* resume in case suspended   */
    kill ((*(*acb).atcb).pid, SIGCONT);
#endif
    /* force exit */
    kill ((*(*acb).atcb).pid, SIGTERM);
    /* release resources */
    rel_rsrc ((*acb).atcb);
    return;
}

/*
 *  act_mon - activate a copy of the monitor
 *
 *  returns SUCESS or FAIL; failure code in tcb
 *
 */
FUNCTION static CODE act_mon 
(
    struct ATCB *atcb,            /* in/out: address of an atcb   */
                                  /* (*atcb).pid is an output*/
    TEXT        task_name[],      /* in: name of task to activate */
    FUNINT      job_type,         /* in:  async or remote job     */
    FUNINT      parent_pid,       /* in: parent process id        */
    TEXT        *envtbl[],        /* in: environment table        */
    TEXT        job_spec[],       /* in: file spec of .job file   */
    TEXT        log_spec[],       /* in: name of output log file  */
    FUNINT      down_pipe[],      /* in:channel id of send pipe   */
    FUNINT      up_pipe[]         /* in:channel id of receive pipe*/
)
{
    static TEXT runtype[] = "RUNTYPE=ASYNC";
    static TEXT act_mode[] = "TAE_ACTMODE=LOCAL";
    static TEXT infile[FSPECSIZ+20] = "CHILD_CMD_INPUT=";
    static TEXT outfile[FSPECSIZ+20] = "CHILD_JOB_LOG=";
    static TEXT parent_name[40];
    static TEXT snd_pipe0[20], snd_pipe1[20], rcv_pipe0[20], rcv_pipe1[20];
    static TEXT *parent_env[]={runtype, act_mode, parent_name, 
                snd_pipe0, snd_pipe1, rcv_pipe0, rcv_pipe1};
    static TEXT **envlist;              /* final environment list       */
    int         child_pid;              /* async monitor's pid          */
    COUNT       i, k;
    static COUNT envsize = 0;           /* Size of envlist              */
                
#ifdef EROS
    struct VARIABLE *charge;
#endif

#define PARENV  (sizeof(parent_env) / sizeof(TEXT *))
#define         ADDENV 2                /* Extra enviroment variables   */

    if (envsize <= 0)
    {
        for (i = 0; envtbl[i]; i++);
        envsize = i + PARENV + ADDENV;
        envlist = (TEXT **) malloc ( (1 + envsize) * sizeof (TEXT *));
    }

    child_pid = -2;                             /* initialize, not 0 or -1  */
    sprintf(parent_name, "PARENT=%d", parent_pid);
    sprintf(snd_pipe0, "DOWN_PIPE0=%d", down_pipe[0]);
    sprintf(snd_pipe1, "DOWN_PIPE1=%d", down_pipe[1]);
    sprintf(rcv_pipe0, "UP_PIPE0=%d", up_pipe[0]);
    sprintf(rcv_pipe1, "UP_PIPE1=%d", up_pipe[1]);


    /* first copy locals    */
    for (i=0; i<PARENV; i++)
        envlist[i] = parent_env[i];
    /* activating async tm  */
    if (job_type == ASYNC)
    {
        strcpy(&infile[16],job_spec);
        strcpy(&outfile[14],log_spec);
        /* job file name in environ */
        envlist[i++] = infile;
        /* log file name in environ */
        envlist[i++] = outfile;
    }
    /* copy shell  exports  */
    for (k=0; (envlist[i] = envtbl[k]); i++, k++);


    /* use 'vfork' for fastness  */
    child_pid = vfork();
    if (child_pid == 0)
    {
#ifdef EROS
        charge = lookex(&glbtab, "CHARGE");
        if (charge == NULL)
            execle(task_name, task_name, 0, envlist);
        else
        {
            execle("/usr/local/bin/charge", "charge", SVAL(*charge,0), 
                   task_name, 0, envlist);
        }
#else
        execle(task_name, task_name, 0, envlist);
#endif
        /* only if execle failed     */
        _exit(1);
    }
    if (child_pid == -1)
    {
        (*(*atcb).tcb).hostcode = errno;
        return (FAIL);
    }
    (*atcb).pid = child_pid;
    (*(*atcb).tcb).child_pid = child_pid;
    (*(*atcb).tcb).hostcode = 0;
    return (SUCCESS);
}

/*      asy_rcv_init.  Initialize for async msg reception from child.
 *
 *      This routine throws away any old message from the asynchronous
 *      task - used for starting a new job after a control c.
 */
        
FUNCTION VOID asy_rcv_init
(
    struct TCB *tcb         /* in: task control block            */
)
{

    if (msg_exist && (saved_msg.m_childpid == (*tcb).child_pid))
    {
        /* send ackn to waiting child */
        write((*tcb).send_chan, &ackn_msg, ACKN_SIZE);
        /* Throw away old message    */
        msg_exist = FALSE;
    }
}

/*
 *  act_process - activate an ASYNC-PROCESS process
 *
 *  returns SUCCESS or FAIL
 */
FUNCTION static CODE act_process 
(
    struct ATCB *atcb,            /* in/out: address of an atcb   */
                                  /* (*atcb).child_pid is an output*/
    TEXT        task_name[],      /* in: name of task to activate */
    FUNINT      parent_pid,       /* in: parent process id        */
    TEXT        *envtbl[],        /* in: environment table        */
    TEXT        log_spec[],       /* in: name of output log file  */
    FUNINT      down_pipe[],      /* in:channel id of send pipe   */
    FUNINT      up_pipe[]         /* in:channel id of receive pipe*/
)
{
    static TEXT outfile[FSPECSIZ+20] = "PROCESS_JOB_LOG=";
    static TEXT parent_name[40];
    static TEXT snd_pipe0[20], snd_pipe1[20], rcv_pipe0[20], rcv_pipe1[20];
    static TEXT *parent_env[]={parent_name, outfile, 
                                 snd_pipe0, snd_pipe1, rcv_pipe0, rcv_pipe1};
    static TEXT **envlist;           /* final environment list       */
    int         async_proc_pid;         /* async process's pid          */
    COUNT       i, k;
    static COUNT envsize = 0;           /* Size of envlist */
                
#define TASKENV  (sizeof(parent_env) / sizeof(TEXT *))
                                        /* # of env parameters from parent */
    if (envsize <= 0)
    {
        for (i = 0; envtbl[i]; i++);
        envsize = i + TASKENV;
        envlist = (TEXT**) malloc ( (envsize + 1) * sizeof (TEXT*));
    }

    /* initialize, not 0 or -1  */
    async_proc_pid = -2;
    if (log_spec == NULL)
    {
        /* no log file specified  */
        strcpy(&outfile[16],"/dev/null");
    }
    else
    {
        /* log file name in environ */
        strcpy(&outfile[16],log_spec);
    }
    sprintf(parent_name, "PARENT=%d", parent_pid);
    sprintf(snd_pipe0, "DOWN_PIPE0=%d", down_pipe[0]);
    sprintf(snd_pipe1, "DOWN_PIPE1=%d", down_pipe[1]);
    sprintf(rcv_pipe0, "UP_PIPE0=%d", up_pipe[0]);
    sprintf(rcv_pipe1, "UP_PIPE1=%d", up_pipe[1]);


    /* first copy locals    */
    for (i=0; i<TASKENV; i++)
        envlist[i] = parent_env[i];
    /* then shell  exports  */
    for (k=0; (envlist[i] = envtbl[k]); i++, k++);

    /* use 'vfork' for fastness  */
    async_proc_pid = vfork();
    if (async_proc_pid == 0)
    {
        execle(task_name, task_name, 0, envlist);
        /* only if execle failed     */
        _exit(1);
    }
    if (async_proc_pid == -1)
    {
        (*(*atcb).tcb).hostcode = errno;
        return (FAIL);
    }
    (*atcb).pid = async_proc_pid;
    (*(*atcb).tcb).hostcode = 0;
    return (SUCCESS);
}

/*      asy_rcv_unint. Receive the async msg ignoring the operator interrupt.
 *
 *      Note: This routine is usually called instead of  an asy_recv call
 *      to get the message anyway.
 */

FUNCTION CODE asy_rcv_unint 
(
    struct TCB *tcb,        /* in: task control block            */
    GENPTR     block,       /* in: address of receive block      */
    FUNINT     blksize      /* in: number of bytes expected      */
)    
{
    CODE                code;

    /* don't quit on operator interrupt */
    ignore_int = TRUE;
    code = asy_recs(tcb, block, blksize, NULL, NULL);
    ignore_int = FALSE;
    return (code);
}

/*      recv_occur. Check if message reception has occured.
 *                      If TRUE, but not already read by asy_recs, the
 *                      message is retrieved by this routine.
 */

FUNCTION BOOL recv_occur
(
    struct TCB *tcb,        /* in: task control block            */
    GENPTR     block,       /* in: address of receive block      */
    FUNINT     blksize      /* in: number of bytes expected      */
)    
{
    /* asy_recs returned after msg  */
    if (msg_read)
        return (TRUE);
    /* no msg arrived after that    */
    if (!msg_exist)
        return (FALSE);
    if (saved_msg.m_childpid != (*tcb).child_pid)
        return(FALSE);                  /* from some other child        */

    /* transfer the message to user */
    asy_rcv_unint(tcb, block, blksize); 
    return(TRUE);
}

/*
 *      asy_recs.  Receive a message from async task.   
 *
 *      NOTE 1: the action routine and action parameter are ignored.
 *      The signal handler uses its own action routine.
 *
 *      Also, note that we check for child pid of the saved message 
 *      because an unsolicited msg might have sent by another 
 *      terminated or aborted child.
 *
 *      NOTE 2:  Do not clear ecbi here as that is checked afterwards
 *      to determine if operator killed the job.
 */

FUNCTION CODE asy_recs
(
    struct TCB *tcb,        /* in: task control block */
    void       *block,      /* in: address of receive block      */
    FUNINT     blksize,     /* in: number of bytes expected      */
    VOID       (*actadr)(), /* in: address of an action routine  */
    GENPTR     actparm      /* in: action routine parm (ignored) */
)
{
    IMPORT  struct ECB  ecbi;           /* operator interrupt ecb            */
/** int                 sig_mask;       * signal to wait for        * 
  * int                 old_mask;
  */

    if (actadr != NULL)                 /* don't wait for msg arrival  */
        return (SUCCESS);
    else                                /* wait until a message arrives  */
    {
        msg_read = FALSE;
        waiting_tcb = tcb;              /* save child's tcb address     */
        while (FOREVER)
        {
            /* msg has arrived from the child   */
            if (msg_exist && ((*tcb).child_pid == saved_msg.m_childpid))
                break;

/*****  Recheck validity of the following code using sigblock, sigpause ******
 *          sig_mask = 1 << (SIGIO-1);          * right most bit is bit 1    *
 *          old_mask = sigblock(sig_mask);      * block SIGIO temporarily    *
 *          sigpause(0);                        * wait till a signal arrives *
 *******/
            /*  interrupt signal  */
            if ((!ignore_int)  && e_occur(&ecbi))
            {
/*******        sigblock(old_mask);             * restore signal SIGIO ********/
                /* msg will be received later */
                return(SUCCESS);
            }
            /* wait for 1 second          */
            sleep(1);
        }
        waiting_tcb = NULL; 
        memcpy(block,(GENPTR) &saved_msg, blksize); 
        msg_read = TRUE;
        msg_exist = FALSE;
        /* send confirmation to child */
        write((*tcb).send_chan, &ackn_msg, ACKN_SIZE);
    }
    return(SUCCESS);
}

/*
 * asy_snds. Send a message in an asynchronous environment.
 *
 *      Return code :
 *              SUCCESS - Message sent successfully
 *              FAIL - Message could not be sent thru the pipe
 */

FUNCTION CODE asy_snds
(
    struct TCB *tcb,        /* in: task control block       */
    GENPTR     block,       /* in: data block to send       */
    FUNINT     blksize      /* in: number of bytes to send  */
)
{
    int                 count;

    if (blksize >= PIPESIZ)  
    {
        (*tcb).hostcode = EMSGSIZE;
        return (FAIL);
    }
    count = write((*tcb).send_chan, block, blksize);
    if (count != blksize)
    {
        /* error, save error code   */
        (*tcb).hostcode = errno;
        return (FAIL);
    }
    return (SUCCESS);
}

/*
 *      async_stdout - Update _STDOUT local variable for UNIX
 *                     ASYNC-PROCESS.
 *
 *
 *      By default, the _STDOUT is set to the UNIX null device so
 *      we can have fast ASYNC-PROCESS startup.  If the process 
 *      has an explicit STDOUT qualifier, then we set _STDOUT
 *      to the qualifier value; otherwise use "/dev/null" device.
 */

FUNCTION VOID async_stdout 
(
    struct CONTXT *cmdctx,        /* context block for the process */
    TEXT          *stdout_string  /* in: STDOUT value or NULL if   */
                                  /* no STDOUT qualifier present   */
)
{
    struct VARIABLE        *v;
    TEXT                   *svv[1];

    if (stdout_string == NULL)
        stdout_string = "/dev/null" ;
    svv[0] = stdout_string;
    v = lookex (&(*cmdctx).locst, "_STDOUT");
    /* index of 1 will give 1st component */
    set_component (v, svv, 1);
}

/*
 *      block_opint. Block operator interrupt.
 */
 
FUNCTION VOID block_opint(void)
{
    block_sig (SIGINT);    
}

/*      build_name.  Build UNIX process name for ASYNC job.
 *
 *      The name must be unique in the group and we would like
 *      to see the proc name in there also, but we only get
 *      14 characters.  Also, we must allow several jobs
 *      with same proc name to be active at once.
 *
 *      The process name generated here is not the job name
 *      that the TAE user knows about.  The TAE user can't
 *      see ASYNCs from other TMs, so he doesn't want the
 *      TM "pid" field, nor does he want the restriction that
 *      only a part of the proc name is in the job name.
 *
 */

FUNCTION static VOID build_name 
(
    TEXT   pid[],       /* in: process ID of TM         */
    TEXT   proc[],      /* in: proc name (not file spec)*/
                        /* (we use as much of this as possible) */
    FUNINT seq,         /* in: 1-99 sequence number     */
    TEXT   name[]       /* out: process name            */
)
{
    COUNT       i, j;
    TEXT        seqstr[10];     /* TBD: [seqstrSIZ+1]           */

    strcpy ( name,pid);
    strcat(name,proc);
    name [PNAMESIZ-2] = EOS;            /* clip if too long     */
    sprintf (seqstr, "%02x", seq);      /* get sequence as ascii*/
    for (i = 0, j = strlen(seqstr)-1; i < j; i++)
    {
        if (seqstr[0] != '0' ) break;
        s_shift (seqstr, 1);            /* delete leading zeros */
    }
    strcat(name,seqstr);            /* add in sequence nr   */
    return;
}

/*  create_async - Create the async TAE monitor (or a ASYNC-PROCESS process)
 *                 and initialize tcb.
 *
 *  returns SUCCESS or FAIL, and a host-dependent error in tcb
 *
 *      Establish a single pipe to receive message from all
 *      asynchronous processes. In addition, for each activated
 *      async process, create a down pipe to send messages.
 *      No up-pipes are necessary here because all real data
 *      exchanges between the parent tm and the async tm
 *      take place in the form of disc files.
 *
 *      The message sent to tm had the message type and file name
 *      if appropriate. The received message contains the child_pid
 *      to identify the sender.
 *
 */

FUNCTION CODE create_async 
(
    struct ATCB *atcb,      /* in/out: address of an atcb      */
    FUNINT      mb_size,    /* in: size to use for mailbox     */
    TEXT        procname[], /* in: name of proc to be run      */
    FUNINT      job_type,   /* in: ASYNC, ASYNC_PROCESS, REMOTE */
    TEXT        job_spec[], /* in: ASYNC or REMOTE: .job filespec */
                            /*     ASYNC_PROCESS: filespec of executable */
    TEXT        log_spec[]  /* in: ASYNC: output log file name */
                            /*     ASYNC_PROCESS: SYS$OUTPUT file */
)
{
    IMPORT      TEXT    **environ;              /* enviromnent  table         */
    IMPORT  struct ECB  ecbi;                   /* ecb for ^C                 */
    CODE                code;
    TEXT        task_name[FSPECSIZ+1];          /* name of task to create     */
    TEXT        txtname[PNAMESIZ+1];            /* new process name           */
    TEXT        pid_name[PNAMESIZ+1];   
    int         send_pipe[2];
    int         status;
    TAEINT      parent_pid;
    char        *ptr;
    static COUNT        seq_num = 0;            /* for build_name             */

    /*   generate UNIX name for new monitor process   */
    /* process id of tm     */
    parent_pid = getpid();
    /* convert to ascii     */
    sprintf(pid_name, "%d", parent_pid);
    /* build new process name       */
    build_name(pid_name, procname, seq_num, txtname);
    /* is also session_id           */
    strcpy ( (*atcb).session_id,txtname);

    /*  Create a common receive pipe to receive messages from all active
     *  async jobs. Note that this pipe is created only when there are no
     *  async jobs already active and closed when all have terminated.
     */
    if (asyact_num == 0)
    {
        /* none active open the up_pipe  */
        status = pipe(rcv_pipe);
        if (status == -1)       
        {
            /* save error code   */
            (*(*atcb).tcb).hostcode = errno;
            return (FAIL);
        }
        /* The FASYNC flag was deleted from the following call due to 
           conflict with SunView. */
        fcntl(rcv_pipe[0], F_SETFL, FNDELAY);    /* non_block,   */
        set_signal(SIGIO, sig_io);          /* signal on msg arrival */
    }

    /*   Create a down pipe to send data to the process */
    status = pipe(send_pipe);
    if (status == -1)
    {
        /* could not create  */
        goto undo_pipes;
    }
    /* for asy_ protocol */
    (*atcb).send_chan = send_pipe[1];
    /* save in tcb too   */
    (*(*atcb).tcb).send_chan = (*atcb).send_chan;


    /* activate a copy of the monitor/parent agent or an ASYNC-PROCESS process.
       (NOTE: the task is run from the same library in which the parent tae
       monitor resides, except for the async process which should have the 
       library name specified thru input job_spec.
         NOTE: f_libr() call returns the last '/' as part of library name */

    if (job_type == ASYNC_PROCESS)
    {
        strcpy(task_name,job_spec);
        /* activate ASYNC-PROCESS proc */
        code = act_process (atcb, task_name, parent_pid, environ, log_spec, 
                            send_pipe, rcv_pipe);  
    }
    else        /* job_type == ASYNC  ... local async */
    {
        task_name[0] = EOS;
        if ((ptr = getenv("TAEBIN")) != NULL)  
            strcpy(task_name,ptr);
        strcat(task_name,"/taetm");
        /* ignore ^C at child creation  */
        set_signal(SIGINT, SIG_IGN);
        /* activate monitor/agent    */
        code = act_mon(atcb, task_name, job_type, parent_pid, environ, job_spec,
                       log_spec, send_pipe, rcv_pipe);
    }
    if (code != SUCCESS)
        goto undo_pipes;                                

    /* reset ^C handler          */
    t_attn(&ecbi);
#if !defined(NOSIGVEC) && !(defined(sun) && OSMajorVersion >= 5)
    /* dont quit on broken pipe  */
    set_signal (SIGPIPE, SIG_IGN);
#endif
    asyact_num++;
    if (++seq_num == MAXACB) seq_num = 0;

    /* close the unused end      */
    close (send_pipe[0]);
    return(SUCCESS);

undo_pipes:
    if (asyact_num == 0)
    {
        /* close open pipes */
        close(rcv_pipe[0]);
        close(rcv_pipe[1]);     
    }
    (*(*atcb).tcb).hostcode = errno;
    /* close open pipes */
    close(send_pipe[0]);
    close(send_pipe[1]);        
    return(FAIL);
}

/*      host_job.     Write the host part of an ASYNC .job file
 *
 *      Note that for a remote job we run an AGENT rather than TM.
 */
FUNCTION int host_job 
(
    struct SFILE  *job,     /* in/out: SFILE opened to job file   */
    struct CONTXT *cmdctx,  /* in: command context                */
    FUNINT        user_proc /* in: TRUE if job file for user proc */
)
{
    CODE                code;
    struct FSBLOCK      job_block;
    TEXT                file_spec[FSPECSIZ+1];
    COUNT               lun=0;            /* not used in VMS or UNIX      */

    /* File created in def dir   */
    strcpy (job_block.libr,DEF_DIR);

    /* Use proc name as filename */
    strcpy (job_block.name,(*cmdctx).pdf.name);
    /* Extension is "JOB"       */
    strcpy (job_block.type,JOB_TYPE);
    job_block.attr[0] = EOS;
    code = f_opnblk (job, lun, &job_block, F_WRITE);
    if (code != SUCCESS)
    {
        f_spec (&job_block, file_spec);
        tmmsg (PROCFAIL, "Unable to create job file '%s'. %s",
               "TAE-JOBOPN", file_spec, (*job).errmsg);
        return (FAIL);
    }
    return (SUCCESS);
}

/* rel_rsrc - Release system resources
 *
 */
FUNCTION VOID rel_rsrc
(
    struct ATCB *atcb
)
{
    /* close the write pipe */
    close ((*atcb).send_chan);
    /* decrement asy jobs active */
    asyact_num--;
    /* if no more active, also close receive pipe  */
    if (asyact_num == 0)
    {
        close (rcv_pipe[0]);
        close (rcv_pipe[1]);
    }       
}

/*
 *      sig_io. signal catcher for SIGIO .
 */

FUNCTION VOID sig_io
(
    int  sig                /* Signal number */
)
{
    struct  ACB         *acbptr;
    int                 count;
    struct MON_MSG      async_buf;

    /* read the msg      */
    count = read(rcv_pipe[0], (GENPTR) &async_buf, sizeof(struct MON_MSG));
    if (count != sizeof (struct MON_MSG))
    {
        /* none there, internal error */
        tmmsg (PROCFAIL,
                "Error receiving async monitor message. Host code = %d.",
                "TAE-ASYRCV", errno);
        return;                                         
    }
    if (waiting_tcb != NULL)
    {
        /* parent waiting for reply  */
        if ((*waiting_tcb).child_pid == async_buf.m_childpid)
        {
            /* msg came from that child, reset for next msg */
            waiting_tcb = NULL;
            memcpy((GENPTR)&saved_msg,(GENPTR)&async_buf,
                        sizeof (struct MON_MSG));
            msg_exist = TRUE;                           
            return;
        }
    }
    /* get the right acb  */
    acbptr = find_id(async_buf.m_childpid);
    if (acbptr != NULL)
    {
        memcpy((*(*acbptr).atcb).inmsg,(GENPTR)&async_buf,
                        sizeof (struct MON_MSG));
        /* send confirmation to child */
        write((*(*(*acbptr).atcb).tcb).send_chan, &ackn_msg, ACKN_SIZE);
        asy_rcv_done(acbptr);
    }
    else
    {
        memcpy((GENPTR)&saved_msg,(GENPTR)&async_buf,
                        sizeof (struct MON_MSG));
        /* save for next requst       */
        msg_exist = TRUE;
    }
}
