/*******************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ******************************************************************************/

/* <<BSD/UNIX>> 
 *
 *      Batch Processing Routines
 *      -------------------------
 *
 * CHANGE LOG:
 *
 *      28-jan-84       Implement nice...dm
 *      29-jan-84       Make batch job file name unique...dm
 *      10-may-84       Add RUNTYPE=BATCH to environment,
 *                      Move get_runtype() here...dm
 *      14-jun-85       Update for TAE 1.3 implementation...dm
 *      18-jun-85       Do not assume tm running from $TSYSLIB...dm
 *      01-jul-85       Fix UNIX compilation errors...dm
 *      11-dec-86       Fix lint warnings...peb
 *      13-dec-90       removed reference to TMIMAGE...ljn
 *      13-dec-90       use Bourne shell nice...ljn
 *      10-jan-91       Fix possible race condition between closing of
 *                      the job file and submittion of the job...krw
 *      15-mar-91       Make log file name unique (to match job file)
 *                      so we don't overwrite log for subsequent runs...krw
 *      14-jun-91       PR1097:Change taetm to $TAEBIN/$TAEPLAT/taetm...ljn
 *      26-nov-91       PR1256:New file naming scheme for batch .job files...ljn
 *      25-may-93       $TAEBIN now is what $TAEBIN/$TAEPLAT used to be...kbs
 *      22-mar-95       EROS: added gould and sgi batch stuff ... Brian Davis
 *      06-nov-95       Removed all of the obsolete Gould code after checking
 *                      it into CVS for historic purposes.
 *                      Made necessary modifications for the new TAE environment
 *                      variables.
 *                      Jim Fenno - EROS Data Center.
 *
 *******************************************************************************
 */


#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "stdh.inp"      /* standard C definitions                   */
#include "taeconf.inp"   /* TAE configuration definitions            */
#include "inttaelib.h"
#include "tminc.inc"     /* TM-only host-independent definitions     */
#include "fileinc.inp"   /* File handling structures and definitions */
#include "tmhost.inp"    
#include "parblk.inc"
#include "local_protos.h"

#if defined(EROS) && defined(sgi)

#define QNAMESIZE       128             /* Max size of a queue name           */
#define DEFAULTQTYPE    "batch"         /* Default queue type                 */
TEXT taejob[QNAMESIZE];                 /* Tae job name                       */

#else

#define QNAMESIZE       6               /* Max size of a queue name           */
#define DEFAULTQTYPE    "nohup"         /* Default queue type                 */

#endif

static COUNT job_number = 0;    /* number for job file   */

FUNCTION CODE submit 
(
    TEXT que_name[],        /* in: batch queue name (nice/nohup) */
    TEXT job_name[],        /* in: batch input file name    */
    TEXT log_name[]         /* in: batch output file name   */
);

char *currentcharge(void);

/*
 * assign_log: Assigns file names for batch job context
 *
 * NOTE: this name is used for compatibity with the VMS version.
 *
 */

FUNCTION CODE assign_log
(
    TEXT proc_name[],             /* IN: Batch job proc name           */
    struct CONTXT *cmdctx,        /* IN: Context block of Batch command*/
    TEXT que_name[QNAMESIZE],     /* OUT: batch type, nice/nohup       */
    TEXT log_name[]               /* OUT: log file name                */
)
{
    struct FSBLOCK out_file;
    struct VARIABLE *v, *v1;
    TEXT   errstr[STRINGSIZ+1];

    v = lookex(&(*cmdctx).qualst, "STDOUT");    /* Get the STDOUT qualifier */
    v1 = lookex(&(*cmdctx).qualst, "RUNTYPE");  /* and RUNTYPE qualifier    */
    if (v == NULL || v1 == NULL)
    {
        tmierr (1001);                          /* Can't find qualifiers    */
        return (FAIL);
    }

    strcpy(que_name,DEFAULTQTYPE);             /* default queue type        */
    if ((*v1).v_count > 1)                      /* if queue specified        */
    {
        s_bcopy(SVAL(*v1, 1), que_name, (QNAMESIZE - 1)); /* copy queue name */
        s_lower(que_name);


        /*
         * EROS currently supports batch queues on SGI systems only.  The names
         * of the batch queues can change so assume the user has entered a
         * valid name.  If we are not at EROS or we are not on an SGI system,
         * then there is only one of three valid queue names.
         */
#if !defined(EROS) || !defined(sgi)
        if ( !s_equal(que_name, "nice") && !s_equal(que_name, "nohup")
                && !s_equal(que_name, "norun"))
        {
            tmmsg(PROCFAIL, "Invalid batch queue name '%s' specified.",
                "TAE-INVBQNAM", SVAL(*v1, 1));
            return (FAIL);
        }
#endif

    }
    if ((*v).v_count != 0)                      /* User specify log filename? */
    {
        /* Yes, put name in msg  */
        f_crack (SVAL (*v, 0), "","", BLOG_TYPE, &out_file, errstr);   
    }
    else
    {
        TEXT    proc_job[FSPECSIZ+1];               /* 1 up log file name */
        sprintf (proc_job,"%s%d", proc_name, job_number);
        /* No, use job file name */
        f_crack ("", "", proc_job, BLOG_TYPE, &out_file, errstr);
    }
    f_spec (&out_file, log_name);                   /* build log file name   */

    return (SUCCESS);
}

/*    run_batch - Processes a tcl command when the runtype is BATCH.
 *
 *      Function return codes:
 *              
 *              DO_SUCCESS
 *              DO_CHECK
 *
 */
FUNCTION CODE run_batch 
(
    struct CONTXT *procctx, /* IN: Invoking proc context      */
    struct CONTXT *cmdctx   /* IN: proc with batch qualifier  */
)
{
    struct SFILE   bch_file;
    struct FSBLOCK bch_block;
    CODE   code;
    TEXT   que_name[QNAMESIZE];          /* queue name           */
    TEXT   job_name[FSPECSIZ+1];
    TEXT   log_name[FSPECSIZ+1];
    TAEINT lun1 = 2;

    bch_block.libr[0] = EOS;                    /* file created in . dir     */
    job_number++;                               /* increment job number      */
    if (job_number > 999)
        job_number = 1;                         /* roll over                 */

    /*
     * File name = jobnnn.n
     */
#if defined(EROS) && defined(sgi)
    sprintf(bch_block.name, "job%d_%d", getpid(), job_number);
    sprintf(taejob, "%s.%d", (*cmdctx).pdf.name, getpid());
#else
    sprintf(bch_block.name, "job%d", job_number);
#endif

    strcpy(bch_block.type,JOB_TYPE);                        /* type is .job  */
    bch_block.attr[0] = EOS;
    f_spec (&bch_block, job_name);               /* job file spec */
    code = f_opnblk (&bch_file, lun1, &bch_block, F_WRITE); /* Open job file */
    if (code != SUCCESS) goto openerr;

    code = assign_log ((*cmdctx).pdf.name,
           cmdctx, que_name, log_name);/* Generate assignment for TAE */
    if (code != SUCCESS)
        return (DO_CHECK);

    code = job_ins(&bch_file, cmdctx);
    if (code != SUCCESS)
    {
        f_close(&bch_file, F_DELETE);
        return (DO_CHECK);
    }
    code = bld_tcl (cmdctx, &bch_file, BATCH);      /* build TCL cmd      */
    if (code != SUCCESS)
    {
        f_close(&bch_file, F_DELETE);
        return (DO_CHECK);
    }
    f_close (&bch_file, F_KEEP);
    code = submit (que_name, job_name, log_name);   /* Now submit the job    */
    if (code == FAIL)                               /* job failed?           */
   {
       code = f_opnblk (&bch_file, lun1, &bch_block, F_READ);/* Open job file*/
       if (code != SUCCESS) goto openerr;
       f_close (&bch_file, F_DELETE);               /* Delete it             */
   }
    return (SUCCESS);

openerr:
    tmmsg (PROCFAIL, "Unable to create batch job file (%s). %s",
           "TAE-BCHOPN", job_name, bch_file.errmsg);
    return (DO_CHECK);
}

/*    submit -  Submit the batch job using the function system().
 *
 *      Function return codes:
 *              
 *              SUCCESS
 *              FAIL
 */
FUNCTION CODE submit 
(
    TEXT que_name[],        /* in: batch queue name (nice/nohup) */
    TEXT job_name[],        /* in: batch input file name    */
    TEXT log_name[]         /* in: batch output file name   */
)
{
#if defined(EROS) && defined(sgi)
    extern TEXT taejob[];
    FILE        *fp;
    TEXT        message[STRINGSIZ+1];
    TAEINT      status;
    IMPORT struct SYMTAB glbtab;        /* TCL global symbols           */
    struct      VARIABLE        *charge;
    char        cwd[256];
    char        qstr[1024];
    char        *cptr;

    if (s_equal(que_name, "norun"))     /* do not submit the job        */
    {
        sprintf(message, "Batch job file '%s' created successfully.", job_name);
        put_stdout(message);
        return (SUCCESS);
    }
    cptr = getcwd(cwd,256);
    if (cptr == NULL)
    {
        perror("submit");
        printf("error getting current working directory\n");
        return(FAIL);
    }
    fp = fopen(taejob,"w+");
    if (fp == NULL)
    {
        perror("submit");
        printf("error opening batch job file %s\n",taejob);
        return(FAIL);
    }
    fprintf(fp,"setenv RUNTYPE BATCH\n");       /* run type of batch */
    fprintf(fp,"setenv TAE_ACTMODE LOCAL\n");   /* run in local mode */
    fprintf(fp,"setenv CHILD_CMD_INPUT %s/%s\n",cwd,job_name); /* job file */
    fprintf(fp,"setenv CHILD_JOB_LOG %s/%s\n",cwd,log_name);   /* log file */
    fprintf(fp,"setenv PATH /usr/local:$PATH\n");              /* set path */
    fprintf(fp,"cd %s\n",cwd);          /* change to current working dir */
    charge = lookex(&glbtab, "CHARGE");  /* get charge code */
    if (charge != NULL)
        fprintf(fp,"/usr/local/bin/charge %s ",SVAL(*charge,0));
    else
    {
        char *c = currentcharge();
        if (c)
            fprintf(fp,"/usr/local/bin/charge %s ", c);
    }
    if ((cptr = getenv("TAEBIN")) != NULL)
        fprintf(fp, "%s/taetm\n", cptr);        /* get tm name in env list */
    else
        fprintf(fp, "taetm\n");                 /* default tm */
    fclose(fp);
    chmod(taejob,0755);
    sprintf(qstr,"qsub -nr -x -s /bin/csh -q %s -eo -o %s_nqs %s",
        que_name,log_name,taejob);
    status = system(qstr);              /* submit to shell for background run */
    (void) unlink(taejob);
    if (status == 127)
    {
        tmmsg(PROCFAIL,"Could not issue batch run command.", "TAE-BATCHRUN");
        return (FAIL);
    }
    else if (status == 0)
    {
        sprintf (message, "Batch job file '%s' submitted successfully.",
            job_name);
        put_stdout(message);
        return (SUCCESS);
    }
    else
    {
        tmmsg (PROCFAIL, "Batch job submission failure. Status code = %d.",
            "TAE-NOBATCH", status);
        return (FAIL);
    }
#else                                                   /* EROS addition */
    TEXT        string[5*FSPECSIZ+1];
    TEXT        message[STRINGSIZ+1];
    TAEINT      status;

    static  TEXT runtype[]  = "RUNTYPE=BATCH; export RUNTYPE; " ;
    static  TEXT act_mode[] = "TAE_ACTMODE=LOCAL; export TAE_ACTMODE; ";

    if (s_equal(que_name, "norun"))     /* do not submit the job        */
    {
        sprintf (message, "Batch job file '%s' created successfully.",
            job_name);
        put_stdout(message);
        return (SUCCESS);
    }

    /* build the command string for submitting the job */

    strcpy(string,"(");
    strcat(string,runtype);                     /* add runtype to environ     */
    strcat(string,act_mode);                    /* local vs remote activation */
    strcat(string,"CHILD_CMD_INPUT=");  /* add job file name too      */
    strcat(string,job_name);                    
    strcat(string,"; export CHILD_CMD_INPUT; ");
    strcat(string,"CHILD_JOB_LOG=");            /* add log file name too      */
    strcat(string,log_name);                    
    strcat(string,"; export CHILD_JOB_LOG; ");

    if (s_equal(que_name, "nice"))              /* use Bourne Shell nice      */
        strcat(string,"/usr/bin/nice -15");
    else
        strcat(string,que_name);                /* put queue name             */
    strcat(string,"  $TAEBIN/taetm");   /* default tae monitor image  */
    strcat(string," >");                        /* standard output same as    */
    strcat(string,log_name);                    /* log file (Append to it)    */
    strcat(string," & )");                      /* place command in background*/
    status = system(string);                    /* submit command to shell    */
    if (status == 127)
    {
        tmmsg(PROCFAIL,"Could not issue batch run command.", "TAE-BATCHRUN");
        return (FAIL);
    }
    else if (status == 0)
    {
        sprintf (message, "Batch job file '%s' submitted successfully.",
            job_name);
        put_stdout(message);
        return (SUCCESS);
    }
    else
    {
        tmmsg (PROCFAIL, "Batch job submission failure. Status code = %d.",
            "TAE-NOBATCH", status);
        return (FAIL);
    }
#endif
}


#if defined(EROS) && defined(sgi)
#include <grp.h>
char *currentcharge(void)
{
    struct group *g;

    if (g = getgrgid(getgid()))
        return(g->gr_name);
    return(0);
}
#endif
