/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*
 * This file contains:
 *
 *      - The do function for the COMPILE intrinsic command,
 *        along with supporting functions.  The COMPILE command
 *        compiles a PDF into a faster executing binary file.
 *        This compilation is an optimization step that is not
 *        required for valid execution of a PDF.
 *
 *      - The function "prc_compiled" which is called by pdftab,
 *        after a proc is determined to be compiled,
 *        in order to build the context block symbol tables.
 *
 *      - The function "prep_c_help" which does preparation
 *        for displaying help on a compiled PDF.
 *
 * CHANGE LOG:
 *
 *      20-oct-84       Add prep_c_help function...peb
 *      23-oct-84       Add logic for subcommand compilation...peb
 *      24-oct-84       Add logic for input of compiled w/ subcs...peb
 *      01-nov-84       compilation to set "curproc" C global...peb
 *      12-nov-84       Handle NAME parms & gbl deref for PARM DEFAULT...peb
 *      25-jan-85       PR on lost open file quota for certain compiles...peb
 *      01-feb-85       Fix bug on too many parms...peb
 *      31-may-85       Fix crash on deref of gbl in PARM DEFAULT...peb
 *      16-july-85      Conditional compilation of f_bread before f_brewrite
 *                      for VAX_VMS...dm
 *      25-jul-85       Fix UNIX lint compilation errors...dm
 *      03-oct-85       PR 946: Changed 'in_comp_pbk' to do run-time validation 
 *                      on variables which are of type "file"...dab
 *      29-oct-85       PR 933: Allow compilation of parameter qualifiers by
 *                      changing 'compile_do' and 'compilation' to properly open
 *                      and position PDF file for qualifier data..dab
 *      08-dec-86       PR 1021: Reopened to fix PARM DEFAULT dereferencing
 *                      a global in a compiled process...lia
 *      29-mar-87       Use LARGE_PARBLK to read PARBLKs from compiled PDFs
 *                      so CPDs created outboard do not have to chunk into
 *                      small records like COMPILE does...palm
 *      22-jul-87       Add get_compcmd() in effort to force TM into
 *                      libraries...ljn
 *      30-jul-87       Make GLOBAL proc execution pick up current global
 *                      values like it works for non-compiled PDFs.  That is,
 *                      here we must do the work of declare:gbl_defaults...palm
 *      07-aug-87       PR1223: zero structs so spare fields get zeros...palm
 *      09-aug-87       Make cmd table GLOBAL; see explanation in
 *                      intrinsic.c...palm
 *      05-dec-87       Executable PAR files...palm
 *      12-feb-88       Call pblk_out with LARGE_PARBLK so very large,
 *                      qualified variables may be saved...palm
 * 01-feb-89    Portable VAX C doesn't like x = y = NULL...ljn
 * 08-feb-89    Use new parhdrRead...palm
 * 28-jun-90    Removed get_compcmd()...ljn
 * 19-apr-91    Because of VMS def of NULL, some rec fields changed to '0'...ljn
 * 22-oct-91    PR1220: Allow passthru to work from compiled PDF...ljn/bdean
 * 22-oct-92    Prototype of tae_alloc uncessary and Ultrix 4.3 does not like
 *              it...rt
 */

#include <string.h>
#include "taeconf.inp"           /* TAE configuration definitions        */
#include "inttaelib.h"
#include "fileinc.inp"           /* file & file position context structs */
#include "symtab.inc"            /* symbol table                         */
#include "tminc.inc"             /* TM-only host-independent definitions */
#include "tmhost.inp"            /* TM-only host dependent definitions   */
#include "pgminc.inc"            /* for M_ codes                         */
#include "parblk.inc"            /* PARBLK structs and defs              */
#include "compiled.inc"          /* for compiled PDFs                    */
#include "dirinc.inc"            /* directive block struct and defs      */
#include "local_protos.h"


FUNCTION CODE compile_do 
(
    struct CONTXT *cpctx,   /* current proc context           */
    struct CONTXT *npctx    /* 'compile' command line context */
);
FUNCTION static CODE get_help 
(
    TEXT help_spec[FSPECSIZ+1],   /* in: help file spec   */
    struct SFILE *fileout,        /* in/out: compilation output file*/
    struct CP_HEADER *header,     /* in/out: compil. output header record*/
    TEXT name_in[],               /* in:  PDF file name (for help search) */
    TEXT libr_in[]                /* in:  PDF file library (for help search)*/
);
FUNCTION static VOID init_header 
(
    struct CONTXT    *ctx,        /* in:  compilation proc context        */
    struct CP_HEADER *header,     /* out: skel compiled PDF header        */
    FUNINT           body_line    /* in:  line # of BODY in uncompiled PDF*/
);
FUNCTION static VOID make_intro 
(
    struct CONTXT *ctx,     /* in:  compilation proc context        */
    TEXT          string[]  /* out: intro record string             */
);
FUNCTION static CODE move_body 
(
    struct SFILE     *filein,     /* in/out: compil. source file  */
    struct SFILE     *fileout,    /* in/out: compil. output file  */
    FUNINT           local_help,  /* in:  TRUE if help text is in source*/
    struct CP_HEADER *header      /* in/out: compil. output header record*/
);
FUNCTION static CODE out_comp_pbk 
(
    struct SFILE  *file,          /* in/out: compiled PDF file            */
    FUNINT        msgtyp,         /* in:  PARBLK message type             */
    struct SYMTAB *st,            /* in:  symbol table to build from      */
    struct POSCTX *first_pos      /* out: file posit of 1st record in grp */
);
FUNCTION static CODE out_comp_sub 
(
    struct SFILE  *file,    /* in/out: compilation output file      */
    struct SUBCMD *s1,      /* in:  first SUBCMD in chain           */
    FUNINT        mode      /* in:  CP_WRITE or CP_REWRITE          */
);
FUNCTION static VOID pos_directive 
(
    struct SFILE     *file,       /* in:  file context (has position)     */
    TEXT             string[],    /* in:  last record that was read       */
    struct CP_HEADER *header      /* in/out: comp'd proc header record    */
);
FUNCTION static CODE var_add 
(
    struct CONTXT   *ctx,   /* in/out: context to add to            */
    struct VARIABLE *var    /* in:  variable to add                 */
);
static VOID         init_header();
static VOID         make_intro();
static VOID         pos_directive();
static CODE         get_help();
static CODE         move_body();
static CODE         out_comp_pbk();
static CODE         out_comp_sub();
static CODE         var_add();

#define CP_WRITE        0
#define CP_REWRITE      1
#define PAR_FILE        0               /* std PAR files have .msgtyp=0 */

/* Memory-resident PDF data for the COMPILE command.
 * The parameters for the command are defined
 * by a vector of RESIDVAR structures.
 * The RESIDVAR structure is explained in the TMINC include file.
 */


static struct RESIDVAR ptcomp[] =
{
    /* name    type      k  m maxc    size     dc val      dvp*/
    {"INPROC",  V_STRING, 0, 1, 1,    FSPECSIZ,   -1, NULL,    NULL},
    {"OUTPROC", V_STRING, 0, 0, 1,    FSPECSIZ,   0,  NULL,    NULL}
};


GLOBAL struct ITRCMD compcmd[] =            /* compile commands  */
{
    {5, "COMPILE",  "", Y_GENERAL,  I_NPM(ptcomp), ptcomp,  compile_do },
    {0, ""} /* TERMINATOR ENTRY: REQUIRED AT END */
};


/*
 *      bld_n_var - build a name parameter given a proc context built so far
 *      and a variable read from a compiled proc.
 */

FUNCTION static CODE bld_n_var 
(
    struct VARIABLE *vin,   /* in:  name parm read from compiled    */
    struct VARIABLE *vout,  /* out: built var w/ ptrs to ref'd var  */
    struct CONTXT   *ctx    /* in:  proc context built so far       */
)
{
    strcpy((*vout).v_name,(*vin).v_name);
    (*vout).v_type  = V_NAME;
    (*vout).v_class = V_PARM;
    if ((*vin).v_nref == NULL)
    {
        /* if no variable named yet             */
        (*vout).v_ref = (*vout).v_dref = NULL;
    }
    else
    {
        /* otherwise point to ref'd var         */
        (*vout).v_ref = (*vout).v_dref = search((*vin).v_nref, ctx);
    }
    return(SUCCESS);
}

/*
 *      chk_directive - returns TRUE if this is a help directive.
 */

FUNCTION static BOOL chk_directive 
(
    TEXT string[]           /* in:  string to check for directive   */
)
{
    if ((s_lseq(".TITLE", string))  ||
        (s_lseq(".HELP", string))   ||
        (s_lseq(".LEVEL1", string)) ||
        (s_lseq(".LEVEL2", string)))
    {
        return(TRUE);
    }

    return(FALSE);
}

/*
 *      compilation - do the guts of PDF compilation.
 *
 *      Assumes the input and output files have been opened.
 */

FUNCTION static VOID compilation 
(
    struct SFILE *filein,        /* in/out: file to compile      */
    struct SFILE *fileout,       /* in/out: compiled output file */
    TEXT         name_in[],      /* in:  PDF file name (for help search
                                         and internal proc search) */
    TEXT         libr_in[],      /* in:  PDF file library (for help search
                                         and internal proc search) */
    TEXT         type_in[],      /* in:  PDF file type (for internal proc
                                         search) */
    TEXT         attr_in[]       /* in:  PDF file attribute (for internal
                                         proc search) */
)
{
    IMPORT struct CONTXT *curproc;      /* current proc context         */

    struct CONTXT       *saved_curproc;
    CODE                code;
    TEXT                string[STRINGSIZ+1];
    COUNT               body_line;
    struct CONTXT       ctx_compile;    /* context during compilation*/
    struct COMP_HOUSEKEEP housekeep;    /* local copy of housekeeping info*/
    struct CP_HEADER    header;         /* local copy of file header    */
#ifdef  VAX_VMS
    COUNT               recsize;
    struct CP_HEADER    tmp_header;
#endif
    struct SUBCMD       *s;
    struct POSCTX       pos;
    BOOL                local_help;     /* TRUE if help text is in PDF  */

    /* initialize the compilation context & local tables    */
    saved_curproc = curproc;
    curproc = &ctx_compile;
    /* arbitrary -- so inictx...    */
    ctx_compile.prclevel = 2;
    /* won't add locals             */
    code = inictx(&ctx_compile);
    if (code != SUCCESS)
        return;
    strcpy(ctx_compile.pdf.name,name_in);
    strcpy(ctx_compile.pdf.libr,libr_in);
    strcpy(ctx_compile.pdf.type,type_in);
    strcpy(ctx_compile.pdf.attr,attr_in);
    ctx_compile.intrinsic = FALSE;
    ctx_compile.special   = COMPILING;
    ctx_compile.comp      = &housekeep;
    housekeep.header      = &header;
    housekeep.before.link = NULL;
    housekeep.after.link = NULL;
    housekeep.cursub = NULL;

    /* compile the pre-body part of the input file  */

    /* build symbol tabs */
    code = pdftab(&ctx_compile, filein);        
    if (code != SUCCESS)
        goto pdf_err;
    local_help = s_equal(ctx_compile.help_spec, "*");
    /* BODY line # in uncompiled PDF*/
    body_line = ctx_compile.pdf_line;

    /* build intro record string    */
    make_intro(&ctx_compile, string);
    /* intro record to compiled file*/
    code = f_write(fileout, string);
    if (code != SUCCESS)
        goto write_err;

    /* initialize header rec*/
    init_header(&ctx_compile, &header, body_line);
    /* skeleton header rec to compiled*/
    code = f_bwrite(fileout, (GENPTR)&header, sizeof(struct CP_HEADER));
    if (code != SUCCESS)
        goto write_err;
    /* save header pos*/
    MOVE_STRUCT((*fileout).posctx, housekeep.pos_header);

    /* write skeleton CP_SUBCMDs    */
    code = out_comp_sub(fileout, ctx_compile.subptr, CP_WRITE);
    if (code != SUCCESS)
        goto write_err;

    /* build & write "before" PARBLK(s) */
    code = out_comp_pbk(fileout, C_BEFORE, &housekeep.before, &pos);
    if (code != SUCCESS)
        goto write_err;
    /* save "before" pos */
    MOVE_STRUCT(pos, header.before);

    /* for each subc*/
    for (s = ctx_compile.subptr; s != NULL; s = (*s).link)
    {
        /* build & write this subc's PARBLK(s)*/
        code = out_comp_pbk(fileout, C_SUBPAR, &(*s).symtab, &pos);
        if (code != SUCCESS)
            goto write_err;
        /* save PARBLK pos      */
        MOVE_STRUCT(pos, (*s).position);
    }

    /* build & write "after" PARBLK(s) */
    code = out_comp_pbk(fileout, C_AFTER, &housekeep.after, &pos);
    if (code != SUCCESS)
        goto write_err;
    /* save "after" pos     */
    MOVE_STRUCT(pos, header.after);

    /* now the body and help info   */

    /* BODY record to compiled file*/
    code = f_write(fileout, "BODY");
    if (code != SUCCESS)
        goto write_err;
    /* save BODY pos                */
    MOVE_STRUCT((*fileout).posctx, header.body);
    /* move rest of proc (& help text if in PDF) */
    code = move_body(filein, fileout, local_help, &header);
    if (code != SUCCESS)
        goto move_body_err;
    if (!local_help)
    {
        /* if help is in external file, get help text */
        code = get_help(ctx_compile.help_spec, fileout, &header, name_in, 
                        libr_in);
        if (code != SUCCESS)
            goto get_help_err;
    }

    /* Now re-write the header with the pointers filled in.
     * Note that for UNIX systems we do not read the record before 
     * writing as read and write to the same stream does not work properly.
     * So we position ourselves to the right record and rewrite the record
     * (i.e. under UNIX f_brewrite is the same as f_bwrite).
     */
    f_setpos(fileout, &housekeep.pos_header);
#ifdef  VAX_VMS
    /* dummy read to lock position  */
    code = f_bread(fileout, (GENPTR)&tmp_header, sizeof(struct CP_HEADER),
                   &recsize);              
    if (code != SUCCESS)
        goto reread_err;
    if (recsize != sizeof(struct CP_HEADER))
    {
        /* another wrong rec possibility*/
        code = F_BTOOSMALL;
        goto reread_err;
    }
#endif
    /* CP_HEADER record to compiled file*/ 
    code = f_brewrite(fileout,  (GENPTR)&header, sizeof(struct CP_HEADER));
    if (code != SUCCESS)
        goto write_err;

    /* Finally, re-write the CP_SUBCMD records with the pointers filled in. */
    /* write filled in CP_SUBCMDs   */
    code = out_comp_sub(fileout, ctx_compile.subptr, CP_REWRITE);
    if (code != SUCCESS)
        goto write_err;

    clsctx(&ctx_compile);
    curproc = saved_curproc;
    return;

/*reread_err:
    if (code == F_EOF)
    {
        tmmsg(PROCFAIL, "Early EOF in re-reading compilation output file.",
                "TAE-RDEOF");
    }
    else if (code == F_BTOOSMALL)
    {
        tmmsg(PROCFAIL, "Wrong record in re-read of compilation output file.",
                "TAE-WRGREC");
    }
    else
    {
        tmmsg(PROCFAIL, "Unable to re-read compilation output file. %s.",
                "TAE-RDERR", (*fileout).errmsg);
    }
    goto close_err;
*/
write_err:
    tmmsg(PROCFAIL, "Unable to write compilation output file. %s.",
            "TAE-WRTERR", (*fileout).errmsg);
    goto close_err;

pdf_err:
move_body_err:
get_help_err:
close_err:
    clsctx(&ctx_compile);
    curproc = saved_curproc;
}

/*
 *      compile_do - Compile the specified proc.
 *
 *      The default input file type is "PDF".
 *      The default output file type is "CPD".
 */

FUNCTION CODE compile_do 
(
    struct CONTXT *cpctx,   /* current proc context           */
    struct CONTXT *npctx    /* 'compile' command line context */
)
{
    struct VARIABLE     *vin, *vout;
    struct SFILE        fileout;
    IMPORT struct SFILE prcfil;
    CODE                code;
    TEXT                fspecout[FSPECSIZ+1];
    TEXT                name_in[FNAMESIZ+1];
    TEXT                libr_in[FLIBRSIZ+1];
    TEXT                type_in[FTYPESIZ+1];
    TEXT                attr_in[FATTRSIZ+1];

    /* if not called from secondary level close calling proc and save pos. */
    if ((*npctx).prclevel > 1)
        clssav (cpctx);

    vin  = lookex(&(*npctx).parmst, "INPROC");
    vout = lookex(&(*npctx).parmst, "OUTPROC");
    /* open PDF for read    */
    code = f_opnspc(&prcfil, PDFLUN, SVAL(*vin, 0), "", "", PDF_TYPE, F_READ);
    if (code != SUCCESS)
    {
        /* report error         */
        goto openin_err;
    }
    /* extract input PDF file name  */
    f_name(prcfil.full_spec, name_in);
    /* extract input PDF file library*/
    f_libr(prcfil.full_spec, libr_in);
    /* extract input PDF file type */
    f_type(prcfil.full_spec, type_in);
    /* extract input PDF file attribute*/
    f_attr(prcfil.full_spec, attr_in);
    /* if output file not specd*/
    if ((*vout).v_count == 0)
    {
        /* use input file name  */
        strcpy(fspecout,name_in);
    }
    else
    {
        /* use specified output file    */
        strcpy(fspecout,SVAL(*vout, 0));
    }
    /* open compile output  */
    code = f_opnspc(&fileout, CPDLUN, fspecout, "", "", CPD_TYPE, F_WRITE);
    if (code != SUCCESS)
    {
        /* report error         */
        goto openout_err;
    }

    /* do the compilation   */
    compilation(&prcfil, &fileout, name_in, libr_in, type_in, attr_in);

    f_close(&prcfil, F_KEEP);
    f_close(&fileout, F_KEEP);
    /* if not called from secondary level reopen and reposition proc */
    if ((*npctx).prclevel > 1)
       opnsav (cpctx);
    return(DO_CHECK);

openin_err:
    tmmsg(PROCFAIL, "Unable to open file to be compiled. %s.",
            "TAE-OPNRD", prcfil.errmsg);
    /* if not called from secondary level reopen and reposition proc */
    if ((*npctx).prclevel > 1)          
       opnsav (cpctx);
    return (DO_CHECK);

openout_err:
    tmmsg(PROCFAIL, "Unable to open compilation output file. %s.",
            "TAE-OPNWRT", fileout.errmsg);
    f_close(&prcfil, F_KEEP);
    /* if not called from secondary level reopen and reposition proc */
    if ((*npctx).prclevel > 1)          
       opnsav (cpctx);
    return (DO_CHECK);
}

/*
 *      get_help - open external help file & move text to compilation
 *      output file.
 *      Since help text is not required, function only fails on a read error,
 *      or a write error, not on a help file open failure.
 */

FUNCTION static CODE get_help 
(
    TEXT help_spec[FSPECSIZ+1],   /* in: help file spec   */
    struct SFILE *fileout,        /* in/out: compilation output file*/
    struct CP_HEADER *header,     /* in/out: compil. output header record*/
    TEXT name_in[],               /* in:  PDF file name (for help search) */
    TEXT libr_in[]                /* in:  PDF file library (for help search)*/
)
{
    CODE                code;
    struct SFILE        help_file;
    TEXT                string[STRINGSIZ+1];

    if (NULLSTR(help_spec))
    {
        /* open help w/ PDF name, library*/
        code = f_opnspc(&help_file, HELPLUN, "", libr_in, name_in,
                        HLP_TYPE, F_READ);
    }
    else
    {
        /* open specified help file     */
        code = f_opnspc(&help_file, HELPLUN, help_spec, "", "",
                        HLP_TYPE, F_READ);
    }
    if (code != SUCCESS)
        return(SUCCESS);
    while ((code = f_read(&help_file, string)) != F_EOF)
    {
        if (code != SUCCESS)
            goto read_err;
        /* pass the input record thru*/
        code = f_write(fileout, string);
        if (code != SUCCESS)
            goto write_err;
        /* save position in header, if this is help directive    */
        pos_directive(fileout, string, header); 
    }
    f_close (&help_file, F_KEEP);
    return(SUCCESS);

read_err:
    tmmsg(PROCFAIL, "Unable to read external help file. %s.",
            "TAE-RDERR", help_file.errmsg);
    goto error_ret;

write_err:
    tmmsg(PROCFAIL,"Unable to write help text to compilation output file. %s.",
            "TAE-WRTERR", (*fileout).errmsg);
    goto error_ret;

error_ret:
    f_close (&help_file, F_KEEP);
    return(FAIL);
}

/*
 *      init_header - build skeleton compiled PDF header record.
 */

FUNCTION static VOID init_header 
(
    struct CONTXT    *ctx,        /* in:  compilation proc context        */
    struct CP_HEADER *header,     /* out: skel compiled PDF header        */
    FUNINT           body_line    /* in:  line # of BODY in uncompiled PDF*/
)
{
    IMPORT TEXT         vrsion[];
    TEXT                time_buf[STRINGSIZ+1];

    memset (header, 0, sizeof (*header));
    (*header).type       = C_HEADER;
    /* TM executable version*/
    s_bcopy(vrsion, (*header).version, CP_TIMESIZ);
    get_time(time_buf);
    /* save time of compil  */
    s_bcopy(time_buf, (*header).timedate, CP_TIMESIZ);
    (*header).body_line = body_line;
    /* EXECUTE= file spec   */
    strcpy((*header).execute,(*ctx).exe_spec);
    (*header).interrupt = (*ctx).interrupt;
    /* no file posits known */
    (*header).body.possav       =                       
        (*header).title.possav  =
        (*header).help.possav   =
        (*header).level1.possav =
        (*header).level2.possav =
        (*header).before.possav =
        (*header).after.possav  = FALSE;
}

/*
 *      in_comp_pbk - input variables from PARBLK records in a compiled PDF.
 *
 *      Assumes the compiled PDF is open and positioned to the first PARBLK
 *      record to be read.
 *      The variables encountered are added to the context block symbol tables.
 *      (In the case of global variables, reference pointers are added to the
 *      context, pointing to the actual globals in TM's global symbol table;
 *      this is analogous to REFGBL processing in an uncompiled PDF.
 *
 *      This function assumes that all variables in the PARBLK are to be
 *      added to the context (rather than updating the context).
 */

FUNCTION static CODE in_comp_pbk 
(
    struct SFILE  *file,    /* in/out: compiled PDF                 */
    struct CONTXT *ctx,     /* in/out: context to receive vars      */
    FUNINT        req_type  /* in: C_BEFORE or C_AFTER              */
)
{
    struct LARGE_PARBLK p;        /* PARBLK buffer for file reads         */
    struct VARIABLE     *v; 
    CODE                code;
    COUNT               recsize;

    while (FOREVER)
    {
        code = f_bread(file, (GENPTR)&p, sizeof(p), &recsize);
        if (code == F_EOF)
        {
            /* many PAR files don't have p.last */
            if (req_type == PAR_FILE)
                break;
            else
                goto eof_err;
        }
        if (code != SUCCESS)
            goto read_err;
        if (p.msgtyp != req_type)
            goto type_err;
        /* make pointers absolute               */
        makeabs(&p.symtab, p.pool);
        /* check internal integrity             */
        if (chk_parblk((struct PARBLK *)&p) != SUCCESS)
            goto format_err;
        for (v = p.symtab.link; v != NULL; v = (*v).v_link)
        {
            /* must verify file vars at run-time */
            if ((*v).v_file)
            {
                code = chk_vector(v,(*v).v_type,(*v).v_cvp,(*v).v_count,FALSE);
                if (code != SUCCESS)
                    goto access_err;
            }
            /* add variable to context              */
            code = var_add(ctx, v);
            if (code != SUCCESS)
                goto close_err;
        }
        /* if this is the last PARBLK in group  */
        if (p.last)
            break;
    }
    return(SUCCESS);

eof_err:
    tmmsg(PROCFAIL, "Early EOF in compiled PDF.", "TAE-EOFERR");
    return(FAIL);

read_err:
    tmmsg(PROCFAIL, "Unable to read compiled PDF.  %s.",
            "TAE-RDERR", (*file).errmsg);
    return(FAIL);

type_err:
format_err:
    tmmsg(PROCFAIL, "Compiled PDF is malformed.", "TAE-CPDFMALF");
    return(FAIL);

access_err:
    tmmsg(PROCFAIL, "Invalid DEFAULT or INITIAL specification for '%s'.",
                    "TAE-DFTERR", (*v).v_name);
    return(FAIL);

close_err:
    return(FAIL);

}

/*
 *      in_comp_sub - find chosen subcommand in compiled PDF.
 *      Builds a SUBCMD struct for the chosen subcommand.
 *      Returns empty string in sub.name if FAIL or if no subcommand was
 *      specified and none exists in the compiled PDF.
 *
 *      NOTE:  Assumes file is open and positioned such that the next
 *              sequential read will be of the first C_SUBCMD record.
 */

FUNCTION static CODE in_comp_sub 
(
    struct SFILE  *file,    /* in/out: compiled PDF                 */
    struct CONTXT *ctx,     /* in/out: proc context                 */
    struct SUBCMD *sub      /* out:  SUBCMD for chosen subc         */
)
{
    struct SUBCMD       *s;       /* current entry */
    struct SUBCMD       *found_s = NULL;
    struct CP_SUBCMD    sub_rec;
    COUNT               recsize;
    CODE                code;
    BOOL                found;
    struct VARIABLE     *v;

    /* until we find chosen subcommand      */
    (*sub).name[0] = EOS;

    /* First build the subcommand chain     */
    /* build SUBCMD chain                   */
    while (FOREVER)
    {
        /* read next CP_SUBCMD        */
        code = f_bread(file,(GENPTR)&sub_rec,sizeof(struct CP_SUBCMD),&recsize);
        if (code != SUCCESS)
            goto read_err;
        if (sub_rec.type != C_SUBCMD)
            goto type_err;
        /* if end of CP_SUBCMDs sentinal        */
        if (NULLSTR(sub_rec.subcmd))
            break;
        /* new SUBCMD struct into table         */
        s = allsub(&(*ctx).subptr);
        strcpy((*s).name,sub_rec.subcmd);
        (*s).deflt = sub_rec.defalt;
        MOVE_STRUCT(sub_rec.pos_parblk, (*s).position);
    }

    /* Now find the chosen subcommand       */

    if ((*ctx).subptr == NULL  &&  NULLSTR((*ctx).subcmd))
        return(SUCCESS);
    found = FALSE;
    /* look thru chain      */
    for (s = (*ctx).subptr; s != NULL; s = (*s).link)
    {
        /* the proc has subcommands     */
        (*ctx).subs = TRUE;
        /* if no subc specified...      */
        if (NULLSTR((*ctx).subcmd))
        {
            /* use the default              */
            if ((*s).deflt)
            {
                found = TRUE;
                found_s = s;
                break;
            }
        }
        /* if cmd line has substring match*/
        else if (s_lseq((*ctx).subcmd, (*s).name)) 
        {
            /* if 2nd match                 */
            if (found)
                goto ambig_err;
            found = TRUE;
            found_s = s;
        }
    }
    /* if chosen subc not found, let pdftab figure it out     */
    if (!found)
        return(SUCCESS);

    /* move SUBCMD struct to caller */
    MOVE_STRUCT(*found_s, *sub);
    (*ctx).subfnd = TRUE;
    /* move full name to context    */
    strcpy((*ctx).subcmd,(*found_s).name);
    v = lookex(&(*ctx).locst, "_SUBCMD");
    set_string(v, (*found_s).name);
    return(SUCCESS);

read_err:
    tmmsg(PROCFAIL, "Unable to read compiled PDF.  %s.",
            "TAE-RDERR", (*file).errmsg);
    return(FAIL);

type_err:
    tmmsg(PROCFAIL, "Compiled PDF is malformed.", "TAE-CPDFMALF");
    return(FAIL);

ambig_err:
    tmmsg(SUCCESS, "Subcommand '%s' specified twice or '%s' ambiguous.",
        "TAE-SUBTWC", (*s).name, (*ctx).subcmd);
    /* flag to pdftab -- emulates subcmd_do */
    (*ctx).special = SUB_SEARCH;
    return(SUCCESS);
}

/*
 *      make_intro - Construct compiled PDF intro record.
 */

FUNCTION static VOID make_intro 
(
    struct CONTXT *ctx,     /* in:  compilation proc context        */
    TEXT          string[]  /* out: intro record string             */
)
{
    if ((*ctx).proctype == Y_PROCESS)
        strcpy(string,"PROCESS");
    else if ((*ctx).proctype == Y_PROCEDURE)
        strcpy(string,"PROCEDURE");
    else if ((*ctx).proctype == Y_PARMSET)
        strcpy(string,"PARMSET");
    else if ((*ctx).proctype == Y_GLOBAL)
        strcpy(string,"GLOBAL");
    strcat(string,"-COMPILED");
}

/*
 *      move_body - move body of proc (& help text if in PDF) to compilation
 *                      output file.
 *      Assumes the files are positioned correctly such that the
 *      text move can proceed sequentially.
 */

FUNCTION static CODE move_body 
(
    struct SFILE     *filein,     /* in/out: compil. source file  */
    struct SFILE     *fileout,    /* in/out: compil. output file  */
    FUNINT           local_help,  /* in:  TRUE if help text is in source*/
    struct CP_HEADER *header      /* in/out: compil. output header record*/
)
{
    CODE                code;
    TEXT                string[STRINGSIZ+1];

    while ((code = f_read(filein, string)) != F_EOF)
    {
        if (code != SUCCESS)
            goto read_err;
        /* if help text not expected    */
        if (!local_help)
        {
            /* and if this is a help directive*/
            if (chk_directive(string))
                goto unexpect_dir_err;
        }
        /* pass the input record thru*/
        code = f_write(fileout, string);
        if (code != SUCCESS)
            goto write_err;
        /* if help text expected in source PDF*/
        if (local_help)
        {
            /* save position in header if this is help directive    */
            pos_directive(fileout, string, header); 
        }
    }
    return(SUCCESS);

read_err:
    tmmsg(PROCFAIL, "Unable to read file to be compiled. %s.",
            "TAE-RDERR", (*filein).errmsg);
    return(FAIL);

write_err:
    tmmsg(PROCFAIL, "Unable to write compilation output file. %s.",
            "TAE-WRTERR", (*fileout).errmsg);
    return(FAIL);

unexpect_dir_err:
    tmmsg(PROCFAIL, "Unexpected help text encountered in compilation source.",
            "TAE-UNEXPECTH");
    return(FAIL);
}

/*
 *      out_comp_pbk - build PARBLK record group from a symbol table
 *      and output to the compiled PDF file.
 */

FUNCTION static CODE out_comp_pbk 
(
    struct SFILE  *file,          /* in/out: compiled PDF file            */
    FUNINT        msgtyp,         /* in:  PARBLK message type             */
    struct SYMTAB *st,            /* in:  symbol table to build from      */
    struct POSCTX *first_pos      /* out: file posit of 1st record in grp */
)
{
    struct LARGE_PARBLK p;
    CODE                code;
    struct VARIABLE     *v;
    COUNT               num_var;        /* sent so far */
    struct VARIABLE     *first_p_var;   /* first var in "p" */

    (*first_pos).possav = FALSE;
    /* get all spare fields zero */
    memset (&p, 0, sizeof (p)); 
    p.msgtyp = msgtyp;
    /* init PARBLK storage area */
    r_init(p.pool, sizeof p.pool);
    /* in case no variables */
    p.symtab.link = NULL;
    p.last        = FALSE;
    num_var       = 0;

    /* Pack all the variables in each 'symtab' into as many parblks as 
       necessary.  */
    for (v = (*st).link; v != NULL; v = (*v).v_link)
    {
        num_var++;
        /* add variable */
        code = pblk_out(file, (struct PARBLK *)&p, sizeof(p.pool), v);
        if (code != SUCCESS)
            return(code);
        first_p_var = p.symtab.link;
        /* if we just wrote to disk for the first time */
        if (num_var > 1  && (*first_p_var).v_link == NULL  &&   
            !(*first_pos).possav)
        {
            /* save pos of 1st rec */
            MOVE_STRUCT((*file).posctx, *first_pos);
        }
    }
    /* write last block     */
    if (p.symtab.link != NULL)
    {
        /* make ptrs relative   */
        makerel(&p.symtab, p.pool);
        p.last = TRUE;
        p.blksiz = r_top(p.pool) - (GENPTR) &p;
        code = f_bwrite(file, (GENPTR) &p, p.blksiz);
        if (code != SUCCESS)
            return (code);
        /* if was 1st rec save its position */
        if (!(*first_pos).possav)
            MOVE_STRUCT((*file).posctx, *first_pos);
    }
    return(SUCCESS);
}

/*
 *      out_comp_sub - Build a CP_SUBCMD record for each SUBCMD struct
 *      in the SUBCMD chain.  Write each CP_SUBCMD record to the compilation
 *      output file.
 *      If the mode is CP_REWRITE the CP_SUBCMD records are written over
 *      existing CP_SUBCMD records (i.e., the compilation second pass).
 *      The file is assumed to be positioned such that the next sequential
 *      read/write operation is to the first CP_SUBCMD record position.
 */

FUNCTION static CODE out_comp_sub 
(
    struct SFILE  *file,    /* in/out: compilation output file      */
    struct SUBCMD *s1,      /* in:  first SUBCMD in chain           */
    FUNINT        mode      /* in:  CP_WRITE or CP_REWRITE          */
)
{
    struct CP_SUBCMD    out_rec;
    struct SUBCMD       *s;
    CODE                code;
#ifdef VAX_VMS
    struct CP_SUBCMD    tmp_rec;  /* for read before rewrite              */
    COUNT               recsize;
#endif

    /* for each SUBCMD in chain     */
    for (s = s1; s != NULL; s = (*s).link)
    {
        memset (&out_rec, 0, sizeof (out_rec));
        out_rec.type = C_SUBCMD;
        /* subcommand name              */
        strcpy(out_rec.subcmd,(*s).name);
        /* default subcommand?          */
        out_rec.defalt = (*s).deflt;
        /* PARBLK file position */
        MOVE_STRUCT((*s).position, out_rec.pos_parblk); 
        /* if this is the first write   */
        if (mode == CP_WRITE)
        {
            /* CP_SUBCMD record to compiled file*/
            code = f_bwrite(file, (GENPTR)&out_rec, sizeof(struct CP_SUBCMD));
            if (code != SUCCESS)
                goto write_err;
        }
        else
        {
            /* CP_REWRITE                   */
#ifdef VAX_VMS
            /* dummy read to lock position  */
            code = f_bread(file, (GENPTR)&tmp_rec, sizeof(struct CP_SUBCMD), 
                           &recsize);
            if (code != SUCCESS)
                goto reread_err;
            if (recsize != sizeof(struct CP_SUBCMD))
            {
                /* another wrong rec possibility*/
                code = F_BTOOSMALL;
                goto reread_err;
            }
#endif
            /* CP_HEADER record to compiled file*/ 
            code = f_brewrite(file, (GENPTR)&out_rec, sizeof(struct CP_SUBCMD));
            if (code != SUCCESS)
                goto write_err; 
        } 
    } 
    /* if this is the first write           */
    if (mode == CP_WRITE)
    {
        memset (&out_rec, 0, sizeof (out_rec));
        out_rec.type = C_SUBCMD;
        /* build & write termination...         */
        out_rec.subcmd[0] = EOS;
        /* CP_SUBCMD record to compiled file    */
        code = f_bwrite(file, (GENPTR)&out_rec, sizeof(struct CP_SUBCMD));
        if (code != SUCCESS)
            goto write_err;
    }
    return(SUCCESS);

/*reread_err:
    if (code == F_EOF)
    {
        tmmsg(PROCFAIL, "Early EOF in re-reading compilation output file.",
                "TAE-RDEOF");
    }
    else if (code == F_BTOOSMALL)
    {
        tmmsg(PROCFAIL, "Wrong record in re-read of compilation output file.",
                "TAE-WRGREC");
    }
    else
    {
        tmmsg(PROCFAIL, "Unable to re-read compilation output file. %s.",
                "TAE-RDERR", (*file).errmsg);
    }
    return(FAIL);
*/
write_err:
    tmmsg(PROCFAIL, "Unable to write compilation output file. %s.",
            "TAE-WRTERR", (*file).errmsg);
    return(FAIL);
}

/*
 *      pos_directive - save file position in compiled PDF header
 *      record if the last record read was a help directive.
 */

FUNCTION static VOID pos_directive 
(
    struct SFILE     *file,       /* in:  file context (has position)     */
    TEXT             string[],    /* in:  last record that was read       */
    struct CP_HEADER *header      /* in/out: comp'd proc header record    */
)
{
    if (s_lseq(".TITLE", string))
        MOVE_STRUCT((*file).posctx, (*header).title);
    else if (s_lseq(".HELP", string))
        MOVE_STRUCT((*file).posctx, (*header).help);
    else if (s_lseq(".LEVEL1", string))
        MOVE_STRUCT((*file).posctx, (*header).level1);
    else if (s_lseq(".LEVEL2", string))
        MOVE_STRUCT((*file).posctx, (*header).level2);
}

/*
 *      prc_compiled.  Build symbol tables from a compiled PDF.
 *
 *      Assumes that the compiled PDF is open and positioned such that
 *      the next sequential read will be of the C_HEADER record.
 */

FUNCTION CODE prc_compiled 
(
    struct SFILE  *file,    /* in/out: compiled PDF control block   */
    struct CONTXT *ctx      /* in/out: context for compiled PDF     */
)
{
    struct CP_HEADER    header;   /* buffer for compiled files header rec */
    CODE                code;
    COUNT               recsize;
    struct SUBCMD       sub;      /* the chosen subcommand                */

    /* First read & process the header record */
    code = f_bread(file, (GENPTR)&header, sizeof(struct CP_HEADER), &recsize);
    if (code != SUCCESS)
        goto read_err;
    if (header.type != C_HEADER)
        goto hdr_type_err;
    s_bcopy(header.execute, (*ctx).exe_spec, FSPECSIZ);
    (*ctx).interrupt = header.interrupt;
    /* use line #...*/
    (*ctx).body_line = header.body_line;
    /* from uncompiled PDF  */
    (*ctx).pdf_line  = header.body_line - 1;

    /* Next read CP_SUBCMD records, pick the chosen subcmd, & build SUBCMD 
       struct*/
    /* find correct subcmd          */
    code = in_comp_sub(file, ctx, &sub);
    if (code != SUCCESS)
        goto in_sub_err;

    /* Now the PARBLKs      */

    /* if "before" PARBLK exists    */
    if (header.before.possav)
    {
        /* pos to 1st "before" PARBLK   */
        f_setpos(file, &header.before);
        /* read the before PARBLKs      */
        code = in_comp_pbk(file, ctx, C_BEFORE);
        if (code != SUCCESS)
            goto in_pbk_err;
    }

    /* if there's a subcommand & associated PARBLK */
    if (!NULLSTR(sub.name)  &&  sub.position.possav)
    {
        /* pos to 1st subc PARBLK       */
        f_setpos(file, &sub.position);          
        /* read the subc PARBLKs        */
        code = in_comp_pbk(file, ctx, C_SUBPAR);
        if (code != SUCCESS)
            goto in_pbk_err;
    }

    /* if "after" PARBLK exists     */
    if (header.after.possav)
    {
        /* pos to 1st "after" PARBLK    */
        f_setpos(file, &header.after);
        /* read the after PARBLKs       */
        code = in_comp_pbk(file, ctx, C_AFTER);
        if (code != SUCCESS)
            goto in_pbk_err;
    }

    /* Now position for normal PDF procedure processing.    */
    /* I.e., the next sequential read will be the BODY statement    */

    /* position for proced body (or help)*/
    f_setpos(file, &header.body);
    MOVE_STRUCT(header.body, (*ctx).prcpos);
    /* so passthru will work */
    if ((*ctx).proctype == Y_PROCESS)
        (*ctx).inbody = TRUE;
    return(SUCCESS);


read_err:
    tmmsg(PROCFAIL, "Unable to read compiled PDF. %s.",
            "TAE-RDERR", (*file).errmsg);
    return(FAIL);

hdr_type_err:
    tmmsg(PROCFAIL, "Compiled PDF has bad header record.", "TAE-CPDFBHDR");
    return(FAIL);

in_sub_err:
in_pbk_err:
    return(FAIL);
}

/*      prc_par_file.   Build symbol tables from a simple PAR file.
 *
 *      This is used when a PAR file is the "proc".
 */

FUNCTION CODE prc_par_file 
(
    struct SFILE  *file,    /* opened PAR file      */
    struct CONTXT *ctx      /* contxt to build      */
)
{
    struct PARHDR parhdr;               /* PAR file header block*/
    CODE        code;
    TEXT        errmsg[STRINGSIZ+1], errkey[STRINGSIZ+1];
    
    /* make it look just like PARMSET */
    (*ctx).proctype = Y_PARMSET;
    code = parhdrRead (file, &parhdr, errmsg, errkey);
    if (code != SUCCESS)
    {
        tmmsg (PROCFAIL, errmsg, errkey);
        return (FAIL);
    }
    code = in_comp_pbk (file, ctx, PAR_FILE);
    return (code);
}

/*
 *      prep_c_help - prepare to display help on a compiled PDF.
 *
 *      Assumes file is positioned so next sequential read is of the
 *      header record.
 */

FUNCTION CODE prep_c_help 
(
    struct DIRBLK  *dirblk, /* in/out: directive block              */
    struct TXTSTOR *title,  /* out: title text                      */
    TEXT           msg[],   /* error message (if any)               */
    TEXT           key[]    /* error key (if any)                   */
)
{
    struct SFILE        *file;
    struct CP_HEADER    header;     /* compiled PDF header record           */
    COUNT               recsize;
    TEXT                string[STRINGSIZ+1];
    CODE                code;

    file = (*dirblk).sfileptr;
    code = f_bread(file, (GENPTR)&header, sizeof(struct CP_HEADER), &recsize);
    if (code != SUCCESS)
        goto read_err;
    if (header.type != C_HEADER)
        goto hdr_type_err;
    /* if title text exists */
    if (header.title.possav)
    {
        /* position to .TITLE */
        f_setpos(file, &header.title);
        /* read the .TITLE record */
        code = f_read(file, string);
        if (code != SUCCESS)
            goto read_err;
        code = gettitle(dirblk, title, msg, key);
        if (code != SUCCESS)
            return(FAIL);
    }
    else
    {
        /* no title */
        (*title).numline = 0;
    }
    /* position for rest of help processing */
    f_setpos(file, &header.body);
    return(SUCCESS);

read_err:
    strcpy(msg,"Error reading the compiled PDF.");
    strcpy(key,"TAE-RDERR");
    return(FAIL);

hdr_type_err:
    strcpy(msg,"Compiled PDF has bad header record.");
    strcpy(key,"TAE-CPDFBHDR");
    return(FAIL);
}

/*
 *      var_add - Add a variable (from a compiled PARBLK) to a proc context.
 *
 *      If the variable is of type V_GLOBAL, only the reference pointer
 *      is added.
 *
 *      Note that for GLOBAL procs, the PARMs remain parameters until the
 *      proc is executed.
 */

FUNCTION static CODE var_add 
(
    struct CONTXT   *ctx,   /* in/out: context to add to            */
    struct VARIABLE *var    /* in:  variable to add                 */
)
{
    IMPORT struct SYMTAB glbtab;

    struct VARIABLE     *targvar;
    struct VARIABLE     *gv;
    struct VARIABLE     *save_link;
    struct VARIABLE     *defd_var;
    struct VARIABLE     *refd_var;
    CODE                type, class;
    CODE                code;
    TEXT                *save_deref;

    type  = (*var).v_type;
    class = (*var).v_class;
    /* analogous to REFGBL sttmt for 1 var  */
    if (class == V_GLOBAL)
    {
        defd_var = search((*var).v_name, ctx);
        if (defd_var != NULL)
        {
            if ((*defd_var).v_class == V_GLOBAL  &&  (*defd_var).v_implicit)
            {
                /* if implicit, skip with no EM         */
                return(SUCCESS);        
            }
            else
                goto dblref_err;
        }
        if (lookex(&glbtab, (*var).v_name) == NULL)
            goto undef_err;
        addref(ctx, (*var).v_name);
    }
    else
    {
        if (class == V_PARM)
        {
            /* alloc new var in ctx parm st */
            targvar = allvar(&(*ctx).parmst);
        }
        else
        {
            /* alloc new var in ctx loc st  */
            targvar = allvar(&(*ctx).locst);
        }
        save_deref = NULL;
        /* if NAME parm, point to ref'd var*/
        if (type == V_NAME)
        {
            /* build name parameter        */
            code = bld_n_var(var, targvar, ctx);
            if (code != SUCCESS)
                goto badref_err;
        }
        else
        {
            if (class == V_PARM  &&  (*var).v_deref)
            {                               
                /* if deref'd default value save deref pointer */
                save_deref = (TEXT *)(*var).v_cvp;
                /* so specvcopy isn't fooled    */
                (*var).v_cvp = NULL;
            }
            save_link = (*targvar).v_link;
            /* copy var to ctxt symb tab var*/
            specvcopy(var, targvar);
            /* if PARM DEFAULT deref'd      */
            if (save_deref != NULL)
            {
                (*targvar).v_deref = 0;
                /* search for referenced var        */
                refd_var = search(save_deref, ctx); 
                if (refd_var == NULL)
                    goto pd_undef_err;
                /* alloc space for val vector*/
                (*targvar).v_cvp = allval (targvar);
                code = set_value(targvar,(*refd_var).v_cvp,(*refd_var).v_count);
            }
            /* specvcopy nulled the link    */
            (*targvar).v_link = save_link;      
        }
        if ((*ctx).proctype == Y_GLOBAL && class == V_PARM)
        {
            /* global exists?     */
            gv = lookex (&glbtab, (*var).v_name);       
            if (gv != NULL)
            {
                code=deep_value_copy (gv, targvar, (*gv).v_name, (*gv).v_name);
                if (code != SUCCESS)
                    goto global_mismatch;                       
            }
        }
    }
    return(SUCCESS);

dblref_err:
    tmmsg(PROCFAIL, "Variable '%s' previously defined or referenced.",
        "TAE-REFEX", (*var).v_name);
    return(FAIL);

badref_err:
    tmmsg(PROCFAIL, "Reference to undefined variable '%s'.",
        "TAE-UNDEFVAR", (*var).v_name);
    return(FAIL);

undef_err:
    tmmsg(PROCFAIL, "Reference to undefined global variable '%s'.",
        "TAE-UNDEFGBL", (*var).v_name);
    return(FAIL);

pd_undef_err:
    tmmsg(PROCFAIL,
        "PARM DEFAULT references undefined global variable '%s'.",
        "TAE-UNDEFGBL", (*var).v_name);
    return(FAIL);

global_mismatch:
    tmmsg (PROCFAIL, "'%s' is incorrect re-definition of existing global.", 
           "TAE-GBLREDEF", (*var).v_name);
    return (FAIL);
}
