/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*
 *      Variable declaration intrinsic commands.
 *
 *      CHANGE LOG:
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *      19-aug-83       Nullables; new rule for DEFAULT names; 
 *                      PR 429 fix...palm
 *      29-aug-83       Keyword parameters...palm
 *      02-sep-83       PR 429: check PARM versus existing gbl in
 *                      gbl_defaults; PR 488...palm
 *      06-sep-83       Make DEFGBL have INITIAL rather than DEFAULT...palm
 *      08-sep-83       Set v_file bit when type=file...palm
 *                      Valid table for ACCESS parameter...palm
 *                      V_NOTFILE changed to V_NOCHECK...palm
 *      14-sep-83       Remove allocation of v_dvp; only tutor needs this
 *                      so we let it create v_dvp...palm
 *      21-sep-83       Order global symbol table alphabetically...dm
 *      21-sep-83       Fix RESIDVAR init for UNIX compatibility...palm
 *      21-sep-83       Fix v_count error in set_defaults...palm
 *      25-sep-83       Fix s_s2i for v_size as output...palm
 *      19-oct-83       PR 539 fix (COUNT mishandled)...palm
 *      20-jan-84       New PARM-PAGE feature...palm
 *      07-feb-84       Replace Y_BATCH with Y_ABI...nhe
 *      13-mar-84       Allow REFGBL in body...palm
 *      25-mar-84       Fix message for double refgbl (PR 555)...palm
 *      04-may-84       VALUE_x to xVAL ... ces
 *      06-may-84       Conform to no .defalt in RESIDVAR...peb
 *      07-may-84       Clean-up the usage of IMPORT...lim
 *      06-jun-84       New logic in set_default to catch situation where
 *                      a DEFAULT or INITIAL specification is de-referenced.
 *                      (TCL-102)...palm
 *      8-jun-84        TCL-99: check level 0 locals before permitting
 *                      a global to be defined...palm
 *      25-jun-84       PR 782: fix error msg in refgbl_do...palm
 *      17-oct-84       PR 847: refgbl_do to accept refs of implicits...peb
 *      23-oct-84       TCL 117: Compilation (also alphabetize fcns)...peb
 *      30-oct-84       TCL 117: Enforce PDF restrictions for compilation...peb
 *      04-nov-84       TCL 67: Build parm qualifier symbol table...peb
 *      09-nov-84       PR 892: fix scoping rule; check new REFGBLs 
 *                      against MAXREF...palm
 *      12-nov-84       TCL 117: Handle NAME parms & PARM DEFAULT derefs...peb
 *****************************************************************************
 * CHANGES MADE IN THE RCJM TREE:
 *
 *      02-jan-85       Add -REDEFINE...nhe
 *      01-mar-85       Make -REDEFINE portable...nhe
 *      23-may-85       Delete -REDEFINE subcommand, merge with TAE-V1.3...dm   
 ****************************************************************************
 * MERGE BACK WITH TAE-V1.3...dm (23-may-85)
 *
 *      01-jul-85       Fix UNIX compilation errors...dm
 *      03-oct-85       PR 946: 'set_defaults' passes "compiling" flag to
 *                      'chk_vector' so no file access check if compiling...dab
 *      13-jul-87       Locals with qualifiers...palm
 *      19-jul-87       Fix gbl_defaults so that all parm qualifiers pick
 *                      up default values from the global qualifiers.  In
 *                      addition to gbl_defaults change, the QUALs are 
 *                      now processed BEFORE defaults...palm
 *      22-jul-87       Add get_declearcmd () as part of effort to force TM 
 *                      into libraries...ljn
 *      02-aug-87       Add TYPE=VOID and ACCESS=EVENT for TaePlus...palm
 *                      (VOID types not fully implemented for now)
 *      09-aug-87       Make cmd table GLOBAL; see explanation in
 *                      intrinsic.c...palm
 *      11-dec-87       New VIEW statement...palm
 *      20-jan-88       New REFVIEW command for declaring a view file...tpl
 *      21-feb-88       Fix DEFGBL crash by picking the right name of
 *                      the initial value (INITIAL rather than DEFAULT)...palm
 *      26-jan-89       MAXVAL -> INT_MAXVAL in RESIDVAR; new valid list...palm
 *                      Note that some RESIDVAR parms must be MAXVAL, for
 *                      example, DEFAULT needs MAXVAL in order to set the
 *                      value of a large parm...palm
 *      06-jan-90       Removed VIEW statement. Conditionally compiled out
 *                      since we are so close to v4.1 delivery..krw     
 *                      To compile with the code, define ADD_VIEW
 *      28-jun-90       Removed get_declarecmd()...ljn
 *      01-aug-91       Braces for static string initializers...ljn
 */

#include <string.h>
#include "taeconf.inp"           /* TAE configuration definitions        */
#include "inttaelib.h"
#include "fileinc.inp"           /* file & file position context structs */
#include "syninc.inc"            /* syntax package structure and defs    */
#include "tmhost.inp"            /* host-dependent defs                  */
#include "symtab.inc"            /* symbol table                         */
#include "tminc.inc"             /* TM-only host-independent definitions */
#include "local_protos.h"
    

FUNCTION CODE refview_do 
(
    struct CONTXT *procctx, /* in:  context of containing proc      */
    struct CONTXT *cmdctx   /* in:  context of PARM sttmt           */
);
FUNCTION CODE defgbl_do 
(
    struct CONTXT *procctx, /* in:  context of containing proc      */
    struct CONTXT *cmdctx   /* in:  context of PARM sttmt           */
);
FUNCTION static CODE gbl_defaults 
(
    struct VARIABLE *tv,    /* in/out: variable being defined       */
    struct VARIABLE *gl     /* in: existing global                  */
);
FUNCTION CODE local_do 
(
    struct CONTXT *procctx, /* in/out:  containing proc context     */
    struct CONTXT *cmdctx   /* in:  ctxt from LOCAL command         */
);
FUNCTION CODE parm_do 
(
    struct CONTXT *procctx, /* in:  context of containing proc      */
    struct CONTXT *cmdctx   /* in:  context of PARM sttmt           */
);
FUNCTION CODE redef_var 
(
    struct CONTXT *procctx, /* in:  context of containing proc   */
    struct CONTXT *cmdctx,  /* in:  context of delcaration sttmt */
    FUNINT        class     /* in:  V_GLOBAL or V_LOCAL only     */
);
FUNCTION CODE refgbl_do 
(
    struct CONTXT *procctx, /* in/out: context of containing proc   */
    struct CONTXT *cmdctx   /* in:  context from REFGBL cmd line    */
);
FUNCTION CODE reserved 
(
    TEXT name[]             /* in: candidate name           */
);
FUNCTION static CODE set_defaults
(
    struct VARIABLE *tv,          /* in/out: variable being defined       */
    struct VARIABLE *dv,          /* in: DEFAULT= variable                */
    FUNINT          compiling     /* in: TRUE if we're compiling a PDF    */
);
FUNCTION static CODE set_p_quals 
(
    struct CONTXT *procctx,       /* in: proc context -- rcvs parm quals  */
    TEXT          qual_proc[],    /* in: specifier for the qualifier proc */
    CODE          class           /* in: V_PARM, V_LOCAL, etc. */
);

static TEXT *one[]= {"1"};
static TEXT *null_string[] = {""};
static TEXT *nocheck[] = {"NOCHECK"};       
static TEXT *defstring[] = {"STRING"};/* default type is string     */

BEGIN_VALIDS(access, 5)
    {{"NOCHECK"}, {"IN"}, {"INOUT"}, {"OUT"}, {"EVENT"}}
END_VALIDS

static struct RESIDVAR ptlocl[] =   /* LOCAL and DEFGBL PDF         */
{
    /* name    type      k  m maxc      size     dc val      dvp    */
    {"NAME",    V_STRING, 0, 1, INT_MAXVAL, NAMESIZ, -1, NULL,  NULL},
    {"TYPE",    V_STRING, 0, 1, 2,      NAMESIZ,  1, NULL,  (GENPTR)defstring},
    {"COUNT",   V_STRING, 0, 1, 2,      NAMESIZ,  1, NULL,  (GENPTR)one},
    {"INITIAL", V_STRING, 0, 0, MAXVAL, TOKESIZ,  0, NULL,  NULL},
    {"VALID",   V_STRING, 0, 1, MAXVAL, VALIDSIZ, 1, NULL, (GENPTR)null_string},
    {"ACCESS",  V_STRING, 1, 1, 1,      TOKESIZ,  1, (GENPTR)&access,
                                                            (GENPTR)nocheck},
    {"QUALS",   V_STRING, 0, 0, 1,      TOKESIZ,  0, NULL,   NULL}
};


static struct RESIDVAR ptparm[] =   /* PARM PDF                     */
{
    /* name    type      k  m maxc      size     dc val      dvp    */
    {"NAME",    V_STRING, 0, 1, INT_MAXVAL, NAMESIZ, -1, NULL,   NULL},
    {"TYPE",    V_STRING, 0, 1, 2,      NAMESIZ,  1, NULL,   (GENPTR)defstring},
    {"COUNT",   V_STRING, 0, 1, 2,      NAMESIZ,  1, NULL,   (GENPTR)one},
    {"DEFAULT", V_STRING, 0, 0, MAXVAL, TOKESIZ,  0, NULL,   NULL},
    {"VALID",   V_STRING, 0, 1, MAXVAL, VALIDSIZ, 1, NULL, (GENPTR)null_string},
    {"ACCESS",  V_STRING, 1, 1, 1,      TOKESIZ,  1, (GENPTR)&access,
                                                              (GENPTR)nocheck},
    {"QUALS",   V_STRING, 0, 0, 1,      TOKESIZ,  0, NULL,   NULL}
};

#ifdef ADD_VIEW
static struct RESIDVAR ptview[] =   /* VIEW PDF                     */
{
    /* name    type      k  m maxc      size     dc val      dvp    */
    {"NAME",    V_STRING, 0, 1, INT_MAXVAL, NAMESIZ, -1, NULL,   NULL},
    {"TYPE",    V_STRING, 0, 1, 2,      NAMESIZ,  1, NULL,   (GENPTR)defstring},
    {"COUNT",   V_STRING, 0, 1, 2,      NAMESIZ,  1, NULL,   (GENPTR)one},
    {"TITLE",   V_STRING, 0, 0, MAXVAL, TOKESIZ,  0, NULL,   NULL},
    {"VALID",   V_STRING, 0, 1, MAXVAL, VALIDSIZ, 1, NULL, (GENPTR)null_string},
    {"ACCESS",  V_STRING, 1, 1, 1,      TOKESIZ,  1, (GENPTR)&access,
                                                              (GENPTR)nocheck},
    {"PRESENT", V_STRING, 0, 0, 1,      TOKESIZ,  0, NULL,   NULL}
};
#endif

static struct RESIDVAR ptrefg[] =   /* REFGBL PDF                   */
{
    /* name    type      k  m maxc      size     dc val      dvp    */
    {"NAME",    V_STRING, 0, 1, INT_MAXVAL, NAMESIZ,  -1, NULL,    NULL}
};

static struct RESIDVAR ptrefview[] =        /* REFVIEW PDF          */
{
    /* name    type      k  m maxc      size          dc val      dvp       */
    {"NAME",    V_STRING, 0, 1,    1, MAXSTRSIZ,    -1, NULL,   NULL}
};


#ifdef ADD_VIEW
FUNCTION CODE view_do 
(
    struct CONTXT *procctx, /* in:  context of containing proc      */
    struct CONTXT *cmdctx   /* in:  context of PARM sttmt           */
);
#endif

/* note: we allow LOCAL pre-body and body                               */
#define Y_LOCAL         Y_ABI | Y_PROCSYN | Y_PREBODY | Y_BODY |Y_CMD


GLOBAL struct ITRCMD declarecmd[] =
{
    {0, "LOCAL",    "",  Y_LOCAL,   I_NPM(ptlocl), ptlocl,  local_do},
#ifdef ADD_REDEFINE
    {0, "LOCAL","REDEFINE",Y_LOCAL, I_NPM(ptlocl), ptlocl,  local_do},
#endif
    {4, "REFVIEW",  "",  Y_DECLARE, I_NPM(ptrefview), ptrefview, refview_do},
    /*{4, "REFVIEW",     "SUBCMD",  Y_DECLARE,   I_NPM(ptrefview), ptrefview,
                                                                 refview_do},
    {4, "REFVIEW",  "QUALIF",  Y_DECLARE,   I_NPM(ptrefview), ptrefview,
                                                                 refview_do},
    */
#ifdef ADD_VIEW
    {0, "VIEW",     "",  Y_DECLARE,  I_NPM(ptview), ptview,  view_do},
#endif
    {0, "PARM",     "",  Y_DECLARE,  I_NPM(ptparm), ptparm,  parm_do},
    {0, "PARM", "PAGE",  Y_DECLARE,  I_NPM(ptparm), ptparm,  parm_do},
    {0, "REFGBL",   "",  Y_LOCAL,    I_NPM(ptrefg), ptrefg,  refgbl_do},
    {0, "DEFGBL",   "",  Y_LOCAL,    I_NPM(ptlocl), ptlocl,  defgbl_do},
#ifdef ADD_REDEFINE
    {0, "DEFGBL","REDEFINE",Y_LOCAL, I_NPM(ptlocl), ptlocl,  defgbl_do},
#endif
    {0, ""} /* TERMINATOR ENTRY: REQUIRED AT END */
};

#ifdef ADD_VIEW
#define V_VIEW 0xF
#if  V_VIEW == V_PARM  || V_VIEW == V_LOCAL || V_VIEW == V_GLOBAL
        compilation error: we picked a bad constant for V_VIEW
#endif
#endif

/*      addvar - add a parameter to a symbol table.
 *      Adds a variable to a symbol table given a temporary symbol table
 *      built from a DEFGBL, PARM, or LOCAL command.
 *
 *      Return: VARIABLE ptr if success; NULL otherwise.
 */
FUNCTION static struct VARIABLE *addvar 
(
    struct CONTXT *procctx, /* in/out: context of enclosing proc*/
    struct CONTXT *cmdctx,  /* in: command context              */
    FUNINT        class     /* in: variable class               */
)
{
    IMPORT struct SYMTAB        glbtab;
    IMPORT struct CONTXT        primctx;        /* level 0 context      */

    struct VARIABLE     *tv=NULL;       /* target (new) variable        */
    struct VARIABLE     *pv;            /* variable from statement      */
    struct VARIABLE     *nv;            /* NAME variable in cmdctx      */
    TEXT   vname[NAMESIZ+1];            /* name of variable being added */
    TAEINT              size, rmin, rmax; 
    COUNT               i;
    COUNT               j;              /* index in list of names       */
    struct SYMTAB       *destst;        /* table to hold new variable   */
    struct VARIABLE     *cv, *fv, *vv, *qv;     /* pointers in cmdctx.parmst*/
    struct VARIABLE     *gl;            /* referenced global            */
    CODE                code;
    struct SUBCMD       *s;
    BOOL                compiling;
    BOOL                void_type;      /* TRUE if type=VOID            */
    BOOL                viewStatement = FALSE;
    TEXT                *parmName;

#ifdef ADD_VIEW
    /* a VIEW is pretty much a PARM */
    if (class == V_VIEW)
    {
        class = V_PARM;
        /* and this used to catch differences */
        viewStatement = TRUE;
    }
#endif

    /* addvar: NAME processing      */
    compiling = ((*procctx).special == COMPILING);
    /* if we're just compiling      */
    if (compiling)
    {
        /* if in a SUBCMD block */
        if ((*procctx).subblk)
        {
            /* current SUBCMD table entry*/
            s = (*(*procctx).comp).cursub;
            /* sym tab for this SUBCMD blk*/
            destst = &(*s).symtab;
        }
        else
        {
            /* if any SUBCMDs found yet     */
            if ((*procctx).subs)
            {
                /* after subs symbol table      */
                destst = &(*(*procctx).comp).after;
            }
            else
            {
                /* before subs symbol table    */
                destst = &(*(*procctx).comp).before;
            }
        }
    }
    else
    {
        /* not compiling a PDF  */
        if (class == V_LOCAL)
        {
            /* use local table      */
            destst = &(*procctx).locst;
        }
        else if (class == V_PARM)
        {
            /* use parameter table  */
            destst = &(*procctx).parmst;
        }
        else
        {
            /* use global table     */
            destst = &glbtab;
        }
    }
    nv = lookex(&(*cmdctx).parmst, "NAME");

    /* for each name (i.e., "PARM (a,b,c)") */
    for (j=0; j < (*nv).v_count; j++)
    {
        if (name_check(SVAL(*nv,j)) != SUCCESS)     
        {
            /* invalid name */
            goto dp_badname;
        }
        /* get the name */
        strcpy(vname,SVAL(*nv, j));
        if (reserved (vname))
        {
            /* Reserved word */
            goto dp_reserved;
        }
        if (class == V_GLOBAL)                      
        {                                       
            /* already exists: ignore */
            if (lookex (&glbtab, vname) != NULL)
                continue;
            /* globals must be unique agaist level 0 locals  */
            if (lookex (&primctx.locst, vname) != NULL)
                goto dp_dupname;
        }                                
        else if (int_search(vname, procctx) != NULL)
        {
            /* name already exists  */
            goto dp_dupname;
        }
        if (class == V_GLOBAL)
        {
            /* for globals alloc and link alphabetically */     
            tv = alpha_var(vname, destst);
        }
        else
        {
            /* alloc new vrbl & link into tab*/
            tv = allvar(destst);
        }
        (*tv).v_class = class;
        strcpy((*tv).v_name,vname);

        /*   addvar: TYPE processing                */

        /* TYPE parm from sttmt  */
        pv  = lookex(&(*cmdctx).parmst, "TYPE");
        void_type = s_lseq(SVAL(*pv,0), "VOID");
        if (s_lseq(SVAL(*pv, 0), "INTEGER")  ||  void_type)
        {
            /* if string size specified     */
            if ((*pv).v_count > 1)
                goto dp_cnerr;
            /* for now, let VOID=INTEGER    */
            (*tv).v_type = V_INTEGER;
        }
        else if (s_lseq(SVAL(*pv, 0), "REAL"))
        {
            /* if string size specified     */
            if ((*pv).v_count > 1)
                goto dp_cnerr;
            (*tv).v_type = V_REAL;
        }
        else if (s_lseq(SVAL(*pv, 0), "STRING"))
        {
            (*tv).v_type = V_STRING;
            /* if string size specified     */
            if ((*pv).v_count == 2)
            {
                /* get & convrt size*/
                if (s_s2i(SVAL(*pv, 1), &size) != SUCCESS)
                    goto dp_szerr;
                /* if str size specified too large*/
                if (size <= 0 || size > MAXSTRSIZ)
                    goto dp_mszer;
                (*tv).v_size = size;
            }
            else
            {
                /* string size default  */ 
                (*tv).v_size = MAXSTRSIZ;
            }
        }
        else if (s_lseq(SVAL(*pv, 0), "KEYWORD"))
        {
            /* actually a string */
            (*tv).v_type = V_STRING;
            if ((*tv).v_class == V_PARM)
            {
                /* make a keyword parm  */
                (*tv).v_keyword = TRUE;
            }
            (*tv).v_size = VALIDSIZ;
        }
        else if (s_lseq(SVAL(*pv, 0), "FILE"))
        {
            /* if string size specified     */
            if ((*pv).v_count > 1)
                goto dp_cnerr;
            (*tv).v_file = TRUE;
            /* a FILE is a special string   */
            (*tv).v_type = V_STRING;
            /* OK if MAXSTRSIZ>filespec size        */ 
            (*tv).v_size = MAXSTRSIZ;
        }
        /* checks for TYPE=NAME:*/
        else if (s_lseq(SVAL(*pv,0), "NAME"))
        {
            if (class != V_PARM)
                goto dp_namerr;
            (*tv).v_type = V_NAME;
            (*tv).v_ref = (*tv).v_dref = NULL;
            (*tv).v_nref = NULL;
            if ((*procctx).proctype == Y_GLOBAL)  
                goto dp_namerr;
        }
        else
        {
            /* otherwise, illegal parm type */
            goto dp_tperr;                          
        }

        if (viewStatement && (*tv).v_type != V_STRING)
            goto dp_tperr;

        /* addvar: COUNT, VALID, and ACCESS */
        cv  = lookex(&(*cmdctx).parmst,  "COUNT");  
        vv  = lookex(&(*cmdctx).parmst,  "VALID");  
        fv  = lookex(&(*cmdctx).parmst,  "ACCESS");
        if ((*tv).v_type == V_NAME)
        {
            if (!(*cv).v_default || !(*vv).v_default || !(*fv).v_default)
            {
                /* invalid combination  */
                goto dp_combo;
            }
        }
        else
        {
            /* flag: no maxc yet    */
            (*tv).v_maxc = 0;
            /* assume not nullable  */
            (*tv).v_nullable = FALSE;
            /* for each 'range'     */
            for (i=0; i < (*cv).v_count; i++)
            {
                if (irange(SVAL(*cv, i), &rmin, &rmax) != SUCCESS)
                    goto dp_nierr;
                if (rmin == 0)
                {
                    /* keep v_maxc > 0      */
                    rmin = 1;
                    (*tv).v_nullable = TRUE;
                }
                if (rmax != 0)
                {
                    /* a 'real' range only one 'real' range allowed */
                    if ((*tv).v_maxc != 0)
                        goto dp_nierr;
                    if (rmax > MAXVAL)
                        goto dp_nverr1;
                    if (rmin < 1)
                        goto dp_nverr2;
                    (*tv).v_minc =  rmin;
                    (*tv).v_maxc =  rmax;
                }
            }
            if ((*tv).v_maxc == 0) 
            {
                /* must have some substance*/
                goto dp_nverr2;
            }
            /* assume no value for now */
            (*tv).v_count   = (*tv).v_dcount = -1;
            (*tv).v_cvp     = (*tv).v_dvp = NULL;
            if ((*tv).v_class == V_PARM  &&  (*tv).v_keyword  
                &&  (*vv).v_default)
            {
                goto valid_required;
            }
            if (!(*vv).v_default)
            {
                if (bld_valid(destst,tv,(*vv).v_cvp, (*vv).v_count) != SUCCESS)
                    goto dp_badvalid;
            }
            /* "EVENT" okay for all types */
            if (s_equal(SVAL(*fv,0), "EVENT"))
                (*tv).v_event = TRUE;
            else if (!(*fv).v_default  &&  !(*tv).v_file)
            {
                /* some ACCESS only good for file */
                goto dp_fmerr1;
            }
            else if (s_equal(SVAL(*fv,0), "NOCHECK"))
                (*tv).v_filemode = V_NOCHECK;
            else if (s_equal(SVAL(*fv,0), "IN"))
                (*tv).v_filemode = V_IN;
            else if (s_equal(SVAL(*fv,0), "INOUT"))
                (*tv).v_filemode = V_INOUT;
            else if (s_equal(SVAL(*fv,0), "OUT"))
                (*tv).v_filemode = V_OUT;
            else 
                goto dp_fmerr;
        }

        /* addvar:  QUALS (parameter qualifiers)        */

        /*  Class GLOBAL should be allowed but is not because of the
         *  way that TM:save_p_quals determines the target variable:
         *  it looks at the last in symbol table and uses that one.
         *  For globals, the last in table isn't the right one.  If
         *  you want qualified globals, go the formal route (use a GLOBAL PDF)
         *  and everything will work.
         */

        qv = lookex(&(*cmdctx).parmst, (viewStatement) ? "PRESENT" : "QUALS");
        /* if a qualifier set was declared*/
        if ((*qv).v_count > 0)
        {
            if (class == V_GLOBAL)
            {
                tmmsg (PROCFAIL, "Qualfiers not allowed for DEFGBL.",
                            "TAE-NOQUAL");
                goto dp_badpqual;
            }
            code = set_p_quals(procctx, SVAL(*qv,0), class);  
            if (code != SUCCESS)
                goto dp_badpqual;
        }

        /* addvar: TITLE or INITIAL or DEFAULT  */
        if ((*tv).v_type != V_NAME)
        {
            /* allocate value vector    */
            (*tv).v_cvp = allval(tv);
        }
        if (viewStatement)
            parmName = "TITLE";
        else if (class == V_GLOBAL || class == V_LOCAL)
            parmName = "INITIAL";
        else 
            parmName = "DEFAULT";
        /* get initial value */ 
        pv  = lookex(&(*cmdctx).parmst,  parmName);

        code = SUCCESS;                             
        /* get defaults from corresponding global */
        if ((*procctx).proctype == Y_GLOBAL  && (*tv).v_class == V_PARM  &&
            (gl = lookex(&glbtab, (*tv).v_name)) != NULL)
        {
            code = gbl_defaults(tv, gl);
        }
        else if (!(*pv).v_default  &&  (*tv).v_type != V_NAME)
            code = set_defaults(tv, pv, compiling);
        else if (!(*pv).v_default  &&  (*tv).v_type == V_NAME)
        {
            if (compiling)
            {
                if ((*pv).v_type == V_NAME)
                    (*tv).v_nref = strdup((*(*pv).v_ref).v_name);
                else
                    (*tv).v_nref = strdup(SVAL(*pv, 0));
            }
            else                                    
            {
                /* not compiling                */
                if ((*pv).v_type == V_NAME)
                {
                    /* DEFAULT=@x   */
                    (*tv).v_dref = (*tv).v_ref = (*pv).v_ref;
                }
                else
                {
                    /* DEFAULT=x    */
                    (*tv).v_dref = (*tv).v_ref = search(SVAL(*pv,0), procctx);
                }
                if ((*tv).v_ref == NULL)
                    goto dp_badref;
                (*tv).v_default = TRUE;
            }
        }
        if (code != SUCCESS)
            goto dp_baddef;
        if (void_type)
        {
            /* void ignores DEFAULT, COUNT, etc     */
            /* allow null value     */
            (*tv).v_nullable = TRUE;
            (*tv).v_minc = 0;
            (*tv).v_maxc = 1;                       
            /* set null value       */
            (*tv).v_count = 0;
        }
    }
    return (tv);


dp_badname:
    tmmsg(PROCFAIL, "Invalid variable name '%s'.", "TAE-INVNAME",
        SVAL(*nv,j));
    return(NULL);

dp_reserved:
    tmmsg(PROCFAIL, "'%s' is a reserved TCL word.", "TAE-RESERVED",
        SVAL(*nv,j));
    return(NULL);

dp_namerr:
    tmmsg(PROCFAIL, "TYPE=NAME not allowed for '%s'.", "TAE-TYPERRN",
          (*tv).v_name);
    delvar(destst, tv);                 /* delete the new variable      */
    return (NULL);

dp_dupname:
    tmmsg(PROCFAIL, "'%s' name conflicts with existing variable.",
                 "TAE-VARCNFLCT", SVAL(*nv,j));
    return (NULL);

dp_cnerr:
dp_tperr:
    tmmsg(PROCFAIL, "Invalid TYPE specification for '%s'.",
                 "TAE-TYPERR", (*tv).v_name);
    delvar(destst, tv);                 /* delete the new variable      */
    return(NULL);

dp_szerr:
dp_mszer:
    tmmsg(PROCFAIL, "Invalid string size specification for '%s'.", 
                "TAE-STRSIZ", (*tv).v_name);
    delvar(destst, tv);                 /* delete the new variable      */
    return(NULL);

dp_combo:
    tmmsg(PROCFAIL, "VALID, ACCESS, and COUNT not allowed with TYPE = NAME.",
         "TAE-NAMERR");
    delvar(destst, tv);                 /* delete the new variable      */
    return (NULL);

dp_nverr1:
dp_nverr2:
dp_nierr:
    tmmsg(PROCFAIL, "Invalid COUNT specification for '%s'.", 
        "TAE-CNTERR", (*tv).v_name);
    delvar(destst, tv);                 /* delete the new variable      */
    return(NULL);

dp_fmerr:
    tmmsg(PROCFAIL, "Invalid ACCESS specification for '%s'.",
        "TAE-ACCERR",  (*tv).v_name);
    delvar(destst, tv);                 /* delete the new variable      */
    return(NULL);

dp_fmerr1:
    tmmsg(PROCFAIL, "ACCESS specification not allowed for non-FILE '%s'.",
          "TAE-ACCNOTALL", (*tv).v_name);
    delvar(destst, tv);                 /* delete the new variable      */
    return (NULL);

valid_required:
    tmmsg(PROCFAIL, "VALID specification required for KEYWORD parameter '%s'.", 
          "TAE-VALIDREQ", (*tv).v_name);
    delvar(destst, tv);
    return (NULL);

dp_badvalid:
dp_baddef:
dp_badpqual:
    delvar(destst, tv);
    return(NULL);

dp_badref:
    tmmsg(PROCFAIL, "Undefined DEFAULT variable reference '%s'.",
          "TAE-BADREF", SVAL(*pv,0));
    return (NULL);                      /* parm only so no delete       */
}

/*      refview_do - define a view by adding the intrinsic local _VIEW
 *      to the symbol table with the specified name as value.
 *      Build from a REFVIEW command.
 *
 *      Return: SUCCESS or FAIL
 */

FUNCTION CODE refview_do 
(
    struct CONTXT *procctx, /* in:  context of containing proc      */
    struct CONTXT *cmdctx   /* in:  context of PARM sttmt           */
)
{
    TEXT                viewfile[FSPECSIZ+1];  /* Full view file name*/

    struct VARIABLE     *tv;            /* target (new) variable        */
    struct VARIABLE     *nv;            /* NAME variable in cmdctx      */
    struct SYMTAB       *destst;        /* table to hold new variable   */
    CODE                code;
    struct SUBCMD       *s;
    BOOL                compiling;
    TEXT                viewname[10];

    compiling = ((*procctx).special == COMPILING);
    /* if we're just compiling      */
    if (compiling)
    {
        /* if in a SUBCMD block */
        if ((*procctx).subblk)
        {
            /* current SUBCMD table entry*/
            s = (*(*procctx).comp).cursub;
            /* sym tab for this SUBCMD blk*/
            destst = &(*s).symtab;
        }
        else
        {
            /* if any SUBCMDs found yet     */
            if ((*procctx).subs)
            {
                /* after subs symbol table      */
                destst = &(*(*procctx).comp).after;
            }
            else
            {
                /* before subs symbol table    */
                destst = &(*(*procctx).comp).before;
            }
        }
    }
    else
    {
        /* not compiling a PDF  */
        /* use local table      */
        destst = &(*procctx).locst;
    }

    strcpy(viewname , "_VIEWPDF");
    strcpy(viewfile , "tutorview");

    if ( (*cmdctx).subcmd != NULL )
    {
        if (s_equal((*cmdctx).subcmd, "SUBCMD"))
        {
            /* if subcmd            */
            strcpy(viewname , "_SUBCPDF");
            strcpy(viewfile , "subcview");
        }
        else if (s_equal((*cmdctx).subcmd, "QUALIF"))
        {
            /* if qualifier view   */
            strcpy(viewname , "_QUALPDF");
            strcpy(viewfile , "qualview");
        }
    }

    nv = lookex(&(*cmdctx).parmst, "NAME");

    if ( nv != NULL )
    {
        /* get the name */
        strcpy(viewfile,SVAL(*nv, 0));
    }



    tv = int_search(viewname, procctx);
    if (tv == NULL)
    {
        /* alloc new vrbl & link into tab*/
        tv = allvar(destst);
        (*tv).v_class = V_LOCAL;
        strcpy((*tv).v_name ,viewname);
        (*tv).v_file = TRUE;
        (*tv).v_type = V_STRING;
        (*tv).v_size = MAXSTRSIZ;               /* string size default  */ 
        (*tv).v_maxc = 1;                       /* flag: no maxc yet    */
        (*tv).v_minc = 1;                       /* flag: no maxc yet    */
        (*tv).v_nullable = TRUE;                /* assume not nullable  */
        (*tv).v_count   = (*tv).v_dcount = -1;  /* assume no value for now */
        (*tv).v_cvp     = (*tv).v_dvp = NULL;
        (*tv).v_cvp = allval(tv);               /* allocate value vector    */
    }

    code = set_string  (tv, viewfile );
    if ( code == SUCCESS )
       return (DO_SUCCESS);
    else
       return (DO_RETURN);
}

/*      defgbl_do - perform DEFGBL command.
 *      
 *      Creates a user defined global.
 *
 *!     If the user wants a "REDEFINE" we first save the old global in case
 *!     the new definition is no good.
 */

FUNCTION CODE defgbl_do 
(
    struct CONTXT *procctx, /* in:  context of containing proc      */
    struct CONTXT *cmdctx   /* in:  context of PARM sttmt           */
)
{
    struct VARIABLE     *v;
#ifdef ADD_REDEFINE
    IMPORT struct SYMTAB glbtab;

    struct SYMTAB       tempst;                 /* a temporary symbol table  */
    CODE                code;
    BOOL                redef = FALSE;

    tempst.link = NULL;
#endif

    /* if we're compiling a PDF...  */
    if ((*procctx).special == COMPILING)
    {
        /* DEFGBL not allowed           */
        tmmsg(PROCFAIL, "DEFGBL statement not allowed in compiled proc.",
            "TAE-COMPDEFG");
        return (DO_RETURN);
    }
#ifdef ADD_REDEFINE
    /* redefine? */
    if (s_equal ((*cmdctx).subcmd, "REDEFINE"))
    {
        code = redef_var (procctx, cmdctx, V_GLOBAL);
        return ((code == SUCCESS)? DO_SUCCESS : DO_RETURN);
    } 
#endif
    v = addvar(procctx, cmdctx, V_GLOBAL);
    return ((v != NULL)? DO_SUCCESS : DO_RETURN);  /* SUCCESS if not NULL    */
}

/*      gbl_defaults.   Grab parm defaults from an existing global.
 *      This is called when (a) we are in a GLOBAL PDF, (b) we are
 *      defining a PARM, and (c) the PARM name is same as an existing
 *      global.  Note that any DEFAULT on the PARM statement is ignored.
 */

FUNCTION static CODE gbl_defaults 
(
    struct VARIABLE *tv,    /* in/out: variable being defined       */
    struct VARIABLE *gl     /* in: existing global                  */
)
{
    struct VARIABLE *tvq;       /* target variable qualifier            */
    struct VARIABLE *gvq;       /* global variable qualifier            */
    CODE                code;

    if ((*tv).v_type != (*gl).v_type ||
        (*tv).v_size >  (*gl).v_size ||
        (*tv).v_minc <  (*gl).v_minc ||
        (*tv).v_maxc >  (*gl).v_maxc ||
        (*tv).v_nullable != (*gl).v_nullable)
    {
        goto bad_ref;
    }
    (*tv).v_default = TRUE;
    code = chk_vector (tv, (*gl).v_type, (*gl).v_cvp, (*gl).v_count, FALSE);
    if (code != SUCCESS)
        goto bad_ref;
    code = set_value (tv, (*gl).v_cvp, (*gl).v_count);
    if (code != SUCCESS)
        return (code);

    /* qualifiers: make sure every global qualifier has a destination  */
    for (gvq = (*gl).v_qualst.link; gvq != NULL; gvq = (*gvq).v_link)
    {
        tvq = lookex (&(*tv).v_qualst,  (*gvq).v_name);
        if (tvq == NULL)
            goto bad_qref;
        code = gbl_defaults (tvq, gvq);
        if (code != SUCCESS)
            return (code);
    }
    return (SUCCESS);


bad_ref:
    tmmsg(PROCFAIL, "Inconsistent re-definition of existing global '%s'.",
          "TAE-REDEF", (*gl).v_name);
    return (FAIL);

bad_qref:
    tmmsg(PROCFAIL, "Re-definition of existing global omits qualifier '%s'.",
          "TAE-REDEF", (*gvq).v_name);

    return (FAIL);
}

/*
 *      local_do - perform LOCAL command.
 */

FUNCTION CODE local_do 
(
    struct CONTXT *procctx, /* in/out:  containing proc context     */
    struct CONTXT *cmdctx   /* in:  ctxt from LOCAL command         */
)
{
    struct VARIABLE     *v;
#ifdef ADD_REDEFINE
    CODE                code;
    /* redefine?        */
    if (s_equal ((*cmdctx).subcmd, "REDEFINE"))
    {
        code = redef_var (procctx, cmdctx, V_LOCAL);
        return ((code==SUCCESS)? DO_SUCCESS : DO_RETURN);
    }
#endif
    v = addvar(procctx, cmdctx, V_LOCAL);
    return ((v!=NULL)? DO_SUCCESS : DO_RETURN);
}

/*      parm_do - perform PARM command.
 *      Adds a variable to the dynamic symbol table of the proc being defined
 *      by this PDF.
 *
 *      Note that for GLOBAL procs, the parameters remain parameters until
 *      the proc is "executed".
 */

FUNCTION CODE parm_do 
(
    struct CONTXT *procctx, /* in:  context of containing proc      */
    struct CONTXT *cmdctx   /* in:  context of PARM sttmt           */
)
{
    struct VARIABLE     *v;

    v = addvar(procctx, cmdctx, V_PARM);
    if (v != NULL)
    {
        (*v).v_page = s_equal ((*cmdctx).subcmd, "PAGE");
        return (DO_SUCCESS);
    }
    else
        return (DO_RETURN);
}

/*      view_do.
 *
 *      A VIEW statement is just like a PARM but some
 *      of the parameter names are changed to make
 *      VIEW statements look good.
 */

#ifdef ADD_VIEW
FUNCTION CODE view_do 
(
    struct CONTXT *procctx, /* in:  context of containing proc      */
    struct CONTXT *cmdctx   /* in:  context of PARM sttmt           */
)
{
    struct VARIABLE     *v;

    v = addvar(procctx, cmdctx, V_VIEW);
    if (v != NULL)
        return (DO_SUCCESS);
    else
        return (DO_RETURN);
}
#endif

#ifdef ADD_REDEFINE
/*      redef_var.      Execute REDEFINE subcommand for declarations.
 *
 *      "Redefine" means to replace if the variable exists and just
 *      define as usual if it does not.  Note that we don't allow 
 *      replacement of "protected" globals; we then just use the new value.
 *
 *      Returns SUCCESS/FAIL
 */
FUNCTION CODE redef_var 
(
    struct CONTXT *procctx, /* in:  context of containing proc   */
    struct CONTXT *cmdctx,  /* in:  context of delcaration sttmt */
    FUNINT        class     /* in:  V_GLOBAL or V_LOCAL only     */
)
{
    IMPORT struct SYMTAB        glbtab; /* globals symbol table              */

    struct SYMTAB       *stpt;
    struct SYMTAB       tempst;         /* a temporary symbol table  */
    struct VARIABLE     *v, *oldv, *iv, *genv;
    struct VARIABLE     *savev = NULL;
    CODE                code;
    BOOL                redef = FALSE;

    tempst.link = NULL;
    genv = lookex (&(*cmdctx).parmst, "NAME");
    if ((*genv).v_count > 1 )
    {
        tmmsg (PROCFAIL, "Only one variable at a time can be redefined.",
                         "TAE-MAXREDEF");
        return (FAIL);
    }
    stpt = ((class == V_LOCAL)? &(*procctx).locst : &glbtab);
    oldv = lookex (stpt, SVAL(*genv,0));        
    /* see if existing          */
    if (oldv != NULL)
    {
        if (class == V_GLOBAL && (*oldv).v_protect)
        {
            /* exists & protected, just update current   */
            /* declared initial value            */
            iv = lookex(&(*cmdctx).parmst,  "INITIAL");
            code = set_defaults(oldv, iv, FALSE);
            return ((code==SUCCESS)? SUCCESS : FAIL);
        }
        /* we'll save oldv */
        savev = allvar(&tempst);
        /* save the old one */
        specvcopy (oldv, savev);
        if (class == V_GLOBAL)
        {
            /* and delete it */
            del_gbl (&glbtab, oldv);
        }
        else
        {
            /* delete local */
            del_loc (procctx, oldv);
        }
    }
    v = addvar(procctx, cmdctx, class);
    if (v != NULL)                              
    {
        /* success */
        if (savev != NULL)
        {
            /* successful redefinition */
            tempst.link = savev;
            /* get rid of saved var */
            delvar (&tempst, savev);
        }
        return (SUCCESS);
    }
    /* failed while working on a redefinition */
    else if (savev != NULL && class == V_GLOBAL)
    {                                      
        /* if 'redefine' put back old in correct alphabetic slot */
        genv = find_slot ((*savev).v_name, &glbtab);
        if (genv == NULL)
        {
            (*savev).v_link = glbtab.link;
            glbtab.link = savev;
        }
        else
        {
            (*savev).v_link = (*genv).v_link;
            (*genv).v_link = savev;
        }
    }
    /* failed redefining a local */
    else if (savev != NULL && class == V_LOCAL) 
    {
        /* find end of chain         */
        for (genv = (struct VARIABLE *)&(*procctx).locst; 
             (*genv).v_link != NULL; genv = (*genv).v_link)
            /* NULL statement */;
        (*genv).v_link = savev;
    }
    return (FAIL);
}
#endif

/*      refgbl_do - perform REFGBL command.
 *      Create a reference to a global for this proc context.
 *      Assumes the global is not already ref'd by this proc.
 *      Ignore (after checking for presence of global) if from primary level.
 *
 *      If we're compiling a PDF, we build a skeleton variable in the 
 *      appropriate compilation symbol table, rather than creating an actual
 *      reference.
 */

FUNCTION CODE refgbl_do 
(
    struct CONTXT *procctx, /* in/out: context of containing proc   */
    struct CONTXT *cmdctx   /* in:  context from REFGBL cmd line    */
)
{
    IMPORT struct SYMTAB glbtab;

    struct SYMTAB       *destst;  /* destination sym tab (for comp'd PDFs*/
    struct SUBCMD       *s;
    struct VARIABLE     *tv;      /* target var (for comp'd PDFs) */
    struct VARIABLE     *nv;      /* NAME variable                */
    COUNT               i;
    struct VARIABLE     *defd_var, *rv;

    nv   = lookex(&((*cmdctx).parmst), "NAME"); 
    /* for each global */
    for (i=0; i < (*nv).v_count; i++)
    {
        if ((*procctx).special == COMPILING)
        {
            /* if in a SUBCMD block */
            if ((*procctx).subblk)
            {
                /* current SUBCMD table entry*/
                s = (*(*procctx).comp).cursub;
                /* sym tab for this SUBCMD blk*/
                destst = &(*s).symtab;
            }
            else
            {
                /* if any SUBCMDs found yet*/
                if ((*procctx).subs)
                {
                    /* after subs symbol table*/
                    destst = &(*(*procctx).comp).after;
                }
                else
                {
                    /* before subs symbol table*/
                    destst = &(*(*procctx).comp).before;
                }
            }
            tv = allvar(destst);
            strcpy((*tv).v_name,SVAL(*nv,i));
            (*tv).v_class = V_GLOBAL;
            /* placeholder */
            (*tv).v_type  = V_INTEGER;
            (*tv).v_count = 0;
            (*tv).v_cvp = NULL;
            (*tv).v_dvp = NULL;
        }
        else
        {
            defd_var = search(SVAL(*nv,i), procctx);
            if (defd_var != NULL)
            {
                if ((*defd_var).v_class == V_GLOBAL  &&  (*defd_var).v_implicit)
                {
                    /* if implicit, skip with no EM         */
                    continue;
                }
                else
                {
                    tmmsg(PROCFAIL, "Variable '%s' previously defined or "
                          "referenced.", "TAE-REFEX", SVAL(*nv,i));
                    return(DO_RETURN);
                }
            }
            rv = lookex (&glbtab, SVAL(*nv,i));
            if (rv == NULL)
            {
                tmmsg(PROCFAIL, "Reference to undefined global variable '%s'.",
                    "TAE-UNDEFGBL", SVAL(*nv, i));
                return(DO_RETURN);
            }
            if ((*procctx).numrefs >= MAXREF)
            {
                tmmsg (PROCFAIL, "Too many global references.", "TAE-REFS");
                return (DO_RETURN);
            }
            
            (*procctx).refs[(*procctx).numrefs] = rv;
            (*procctx).numrefs++;
            /* bump the ref count for this global */
            (*rv).v_refcnt++;
        }
    }
    return(DO_SUCCESS);
}

/*      reserved - Check the name of a PARM GLOBAL or LOCAL to make 
 *      sure it's not reserved.
 *
 *      Return codes:
 *      
 *      TRUE - Name is one of the reserved words.
 *      FALSE - Name is not reserved.
 */

FUNCTION CODE reserved 
(
    TEXT name[]             /* in: candidate name           */
)
{
    static TEXT *res_words [] = {"NOT",
                                "AND",
                                "OR",
                                NULL};
    CODE        i;

    for (i = 0; res_words[i] != NULL; i++)
    {
        if (s_equal (name,res_words[i]))
        {
            /* A match was found */
            return (TRUE);
        }
    }
    /* Not reserved */
    return (FAIL);
}

/*
 *      set_defaults.   Set non-NAME defaults (or INITIAL)
 *      into current value.
 */
FUNCTION static CODE set_defaults
(
    struct VARIABLE *tv,          /* in/out: variable being defined       */
    struct VARIABLE *dv,          /* in: DEFAULT= variable                */
    FUNINT          compiling     /* in: TRUE if we're compiling a PDF    */
)
{
    COUNT  i;
    struct VARIABLE     *ref;
    CODE                code;

    (*tv).v_default = TRUE;
    /* was the value de-ref'd? (See notes in repval) */
    if ((*dv).v_type == V_NAME)
    {
        /* use the reference */
        ref = (*dv).v_ref;
        if (compiling  &&  (*ref).v_class == V_GLOBAL)
        {
            (*tv).v_deref = TRUE;
            (*tv).v_cvp = strdup((*ref).v_name);
        }
        else
        {
            code = chk_vector (tv, (*ref).v_type, (*ref).v_cvp,
                               (*ref).v_count, compiling);
            if (code != SUCCESS)
                goto dp_dferr;
            code = set_value (tv, (*ref).v_cvp, (*ref).v_count);
        }
    }
    else
    {
        /* DEFAULT is directly valued   */
        /* DEFAULT is null value                */
        if ((*dv).v_count == 0)
        {
            if (!(*tv).v_nullable)
                goto dp_mcerr;
        }
        else                    
        {
            /* DEFAULT is substantial               */
            if ((*dv).v_count < (*tv).v_minc  ||  (*dv).v_count > (*tv).v_maxc)
                goto dp_mcerr;
        }
        (*tv).v_count = (*dv).v_count;
        if ((*tv).v_type == V_INTEGER)
        {
            for (i = 0; i < (*dv).v_count; i++)
            {
                if (s_s2i(SVAL(*dv, i), &IVAL(*tv, i)) != SUCCESS)
                    goto dp_dferr;
            }
        }
        else if ((*tv).v_type == V_REAL)
        {
            for (i=0; i < (*dv).v_count; i++)
            {
                if (s_s2r(SVAL(*dv, i), &RVAL(*tv, i)) != SUCCESS)
                    goto dp_dferr;
            }
        }
        else if ((*tv).v_type == V_STRING)
        {
            for (i = 0; i < (*dv).v_count; i++)
                SVAL(*tv, i)  = strdup(SVAL(*dv, i)) ;
        }
        if (chk_vector(tv, (*tv).v_type, (*tv).v_cvp, (*tv).v_count,
                       compiling) != SUCCESS) 
            goto dp_dferr;
        /* translate to valid form      */
        if (trans_valid(tv) != SUCCESS)
            goto dp_dferr;
    }
    return(SUCCESS);

dp_mcerr:
dp_dferr:
    tmmsg(PROCFAIL, "Invalid DEFAULT or INITIAL specification for '%s'.",
        "TAE-DFTERR", (*tv).v_name);
    return(FAIL);
}

/*
 *      set_p_quals - build parameter qualifier symbol table.
 *
 *      Uses prccmd to open proc containing parameter qualifiers and build
 *      the qualifier symbol table.  The GET_PARM_QUAL value for the
 *      special flag causes prccmd to:
 *
 *      - stop processing the proc at the BODY statement if the proc is a
 *        procedure.
 *      - at the close of processing of qual_proc, move qual_proc's parameter
 *        symbol table to the qualst member of the last variable in
 *        "procctx"'s parmst chain.
 *
 */

FUNCTION static CODE set_p_quals 
(
    struct CONTXT *procctx,       /* in: proc context -- rcvs parm quals  */
    TEXT          qual_proc[],    /* in: specifier for the qualifier proc */
    CODE          class           /* in: V_PARM, V_LOCAL, etc. */
)
{
    CODE                saved_special;
    BOOL                saved_parmqual;
    CODE                code;

    if (class == V_GLOBAL)
        return (FAIL); 
    saved_special = (*procctx).special;
    /* flag to prccmd -- we're getting qualifiers; don't execute */
    (*procctx).special = GET_PARM_QUAL;
    saved_parmqual = (*procctx).parmqual;
    (*procctx).parmqual = (class == V_PARM);
    /* otherwise, process like any command  */
    code = prccmd(qual_proc, procctx);
    (*procctx).special = saved_special;
    (*procctx).parmqual = saved_parmqual;
    if (code == DO_SUCCESS)
        code = SUCCESS;
    else
        code = FAIL;
    return(code);
}

