/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*
                Dynamic parameter handling
*/

/*      Change log:
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *
 *      05-aug-83       Fix parmst deallocation bugin dynget()...dm
 *      23-aug-83       Add installation exit call to dyncom()...peb
 *      02-sep-83       PR 360 fix; PR 464 fix...palm
 *      08-sep-83       Fix length of prompt string...palm
 *      08-sep-83       PR 495: dynamic parm mode in BATCH...palm
 *      12-sep-83       Re-structure for new tutor interface;
 *                      fix PR 464 (easy, fast request for current proc)...palm
 *      22-sep-83       Fix bug in dynget call to dp_ins()...peb
 *      26-sep-83       New dp_ins call for dynamic subcommands...palm
 *      05-oct-83       Change getint() to cmd_noscreen()...dm
 *      11-oct-83       R for RUN; unix compilation errors...palm
 *      30-oct-83       Dyn tutor activation change & PR fixes...peb
 *      25-jan-84       New cmd_noscreen call; archive commands...palm
 *      05-feb-84       Fix bug: stdout from user goes to proc's stdout...nhe
 *      07-feb-84       Add Y_ASYNC for Y_RUN...nhe
 *      14-feb-84       Runtyp made CODE run_type...palm
 *      19-feb-84       New dynamic tutor rules...palm
 *      09-mar-84       Fix double close...palm
 *      11-mar-84       Pass 'dynamic' flag to opn_tutor as argument...dm
 *      15-mar-84       Print parcmp error message here...dm
 *      04-may-84       VALUE_x to xVAL ... ces
 *      07-may-84       Clean-up the IMPORT declarations...lim
 *      24-aug-84       Change dynamic tutor request to NOFORCE...lia
 *      15-oct-84       Created bld_pdf_ctx and moved logic from dynget...jtm
 *      16-oct-84       New call to opnpdf...nhe
 *      25-oct-84       Handle DO_HOLD from in dynget from tutor...nhe
 *
 *************************************************************************
 * CHANGES MADE IN THE RCJM TREE
 *
 *      18-apr-85       Support dynamic parameter req from remote jobs...dm
 *      19-apr-85       Update comments in bld_pdf_ctx...dm
 *      25-apr-85       Errmsg if bld_pdf_ctx fails for remote jobs...dm
 *      27-apr-85       Update get_remote_file calling seq...dm
 *
 *********************************************************************
 *      
 *      24-jul-85       Fix UNIX lint compilation errors...dm
 *      26-AUG-85       PR 806: Return M_DYNEXIT when exit from dyn tut...dab
 *      09-sep-85       PR 961: Argument to 'dynget' tells if tutor for sync or
 *                      async job, passed to 'tutor' to reject HOLD if sync...dab
 *      23-apr-87       Tell parmrg to merge VALIDs...palm
 *      22-jul-87       Add get_dyncmd () as part of effort to force TM 
 *                      into libraries...ljn
 *      09-aug-87       Make cmd table GLOBAL; see explanation in
 *                      intrinsic.c...palm      
 *      08-sep-87       Add pool size in calls to pack_parm and dynget
 *                      calling sequence...tpl
 *      24-feb-88       PR 1504: Change label dimension to LABELSIZ+1...ljn
 *      24-mar-88       Added braces to structure initializer...ljn
 *      24-mar-88       Delete TAE_RCJM conditionals...ljn
 *      01-may-89       new dyncmd for dynamic command from process...palm 
 *      23-may-90       No more RCJM...ljn
 *      28-jun-90       Removed get_dyncmd()...ljn
 */

#include <string.h>
#include "taeconf.inp"   /* TAE configuration (REQUIRED)         */
#include "inttaelib.h"
#include "symtab.inc"    /* TM symbol table                      */
#include "syninc.inc"    /* syntax package                       */
#include "terminc.inc"   /* terminal package                     */
#include "tminc.inc"     /* TM definitions                       */
#include "parblk.inc"    /* For definition of M_CONTINUE         */
#include "local_protos.h"

/*
 *      Currently, there are no commands here;  the table is referenced
 *      from INTRINSIC.C and must have at least the terminator entry.
 */

GLOBAL struct ITRCMD dyncmd[] =
{
    {0, ""} /* TERMINATOR ENTRY: REQUIRED AT END */
};

FUNCTION CODE get_dynpname
(
    struct CONTXT *ctx,     /* in: context of executing proc     */
    struct PARBLK *parblk,  /* in/out: parameter block           */
    TEXT          dynspec[],/* out: dynamic tutor pdf spec       */
    TEXT          subcmd[]  /* out: subcommand for dynamic tutor */
);

/*      bld_pdf_ctx - Build a context block for dynamic parameter mode
 *
 *      This function is called prior to entering dynamic parameter mode to
 *      obtain parameters for:
 *
 *      - a process, executed from an interactive TM, who has issued an
 *        xqdynp request
 *      
 *      - an asynchronous process who has issued an xqdynp request
 *      
 *      - an asynchronous procedure who has issued a GETPAR request
 *
 *      It's purpose is to initialize a context block for eventual use by
 *      tutor. This function does not build the symbol tables.
 *
 *** NOTE **
 *      In case of local asynchronous jobs the original proc
 *      context was no longer retained. Therefore, the input context
 *      block was a madeup context block created from primctx, but
 *      with executing proc's pdf spec etc.
 *
 *      RETURN CODES:
 *      
 *      FAIL -          Could not build context block, or user is trying to
 *                      do dynamic tutor from BATCH. Error message has been
 *                      issued, PDF file closed, and context block cleaned up.
 *
 *      SUCCESS -       The context block was successfully initialized, and
 *                      the appropriate PDF was opened.
 */
        
FUNCTION CODE bld_pdf_ctx 
(
    struct CONTXT *ctx,           /* in: context of executing process  */
    struct PARBLK *parblk,        /* in/out: parameter block           */
    struct CONTXT *dctx,          /* out: context block for tutor      */
    struct SFILE  *dynpdf,        /* out: address of dynamic pdf SFILE */
    BOOL          *dash_present   /* out: TRUE if subcommand indicated */
)
{
    IMPORT struct CONTXT *curproc;      /* current context              */
    IMPORT CODE         run_type;       /* INTER, BATCH, ASYCH          */

    CODE                code;
    TEXT                fspec[FSPECSIZ+1];


    /* must be interactive  */
    if (run_type != INTER)
        goto not_inter;                         
    /* for NAME resolution  */
    (*dctx).backlink = curproc;
    (*dctx).prclevel = (*curproc).prclevel + 1;
    if (s_equal((*ctx).pdf.libr, "/LOCAL/"))
        goto intern_try;
    /* init context         */
    code = inictx(dctx);
    if (code != SUCCESS)
        goto over_err;
    /* get dynamic pdf name */
    code = get_dynpname(ctx, parblk, fspec, (*dctx).subcmd);
    if (code != SUCCESS)
        goto dynpdf_err;
    *dash_present = !NULLSTR((*dctx).subcmd) || (*dctx).subcmd[1] == '-';
    (*dctx).subsiz = strlen((*dctx).subcmd);
    /* only PDFs dynamically  */
    (*dctx).intrinsic = FALSE;
    /* open the PDF           */
    code = opnpdf(ctx, dctx, fspec, dynpdf);
    if (code != SUCCESS)                        
        goto preopn_err;        
    /* Return ins. exit status */
    return (SUCCESS);

not_inter:
    tmmsg(PROCFAIL, "Attempt to enter dynamic tutor from BATCH.",
          "TAE-DYNBATCH");
    return (FAIL);

intern_try:
    tmmsg(PROCFAIL, "Attempt to enter dynamic tutor for an internal proc.",
          "TAE-DYNINTERN");
    return (FAIL);

over_err:
    /* report memory overflow       */
    overr();
    goto clsctx_kill;

dynpdf_err:
    tmmsg(PROCFAIL, "Could not get user pdf '%s' for dynamic tutoring.",
        "TAE-NODYNPDF", fspec);
    goto clsctx_kill;

preopn_err:
clsctx_kill:
    clsctx(dctx);
    return (FAIL);
}


/*      get_dynpname. Get name of pdf for dynamic tutoring.
 */

FUNCTION CODE get_dynpname
(
    struct CONTXT *ctx,     /* in: context of executing proc     */
    struct PARBLK *parblk,  /* in/out: parameter block           */
    TEXT          dynspec[],/* out: dynamic tutor pdf spec       */
    TEXT          subcmd[]  /* out: subcommand for dynamic tutor */
)
{

    struct SYNBLK       sb;             /* syntax block                 */
    TEXT        label[LABELSIZ+1], cmdstr[CMDLINSIZ+1];

    CODE                code;
    struct VARIABLE     *v;

    /* find proc name       */
    v = lookex(&(*parblk).symtab, "_PROC");
    if (v == NULL)
        cmdstr[0] = EOS;
    else
        strcpy(cmdstr,SVAL(*v, 0));
    code = cmd_parse(&sb, cmdstr, label, dynspec, subcmd);
    if (code == FAIL)
        return (FAIL);
    /* if _PROC null, use curr*/
    if (NULLSTR(dynspec))
        f_spec (&(*ctx).pdf, dynspec);

    return (SUCCESS);
}

/*      dynget - initiate dynamic tutor or dynamic parameter mode
 *
 *      This is called when a process wants dynamic parameters.
 *      The output is an updated parblk with relative pointers or,
 *      if the process is to be killed, a parblk with no parameters
 *      and the message type set to M_KILLED.  If the user decided not
 *      to do anything after all, the message type is set to M_HOLD.
 *      If the user entered exit from tutor, return a parblk with
 *      M_DYNEXIT to indicate this, with no parameters; differs from
 *      M_KILLED in that process does not get aborted by parent TM.
 */

FUNCTION VOID dynget 
(
    struct CONTXT *ctx,           /* in: context of executing process */
    struct PARBLK *parblk,        /* in/out: parameter block          */
    FUNINT        pool_size,      /* in: size of parblk.pool      */
    BOOL          asytut_req      /* in: async or sync tutoring ? */
)
{
    struct CONTXT       dctx;           /* dynamic context              */
    CODE                code, mode;
    static TEXT         notused_key[] = "TAE-NOTUSED";
    static struct SYMTAB nullst = {NULL}; /* null symbol table          */
    struct VARIABLE     *v;
    struct SFILE        dynpdf;         
    TEXT                held_msg[STRINGSIZ+1];
    BOOL                dash_present;

    /* Build tutor CONTXT */
    code = bld_pdf_ctx (ctx, parblk, &dctx, &dynpdf, &dash_present);
    if (code == FAIL)
        goto kill;
    /* Installation exit */
    code = dp_ins (parblk, &dynpdf, &dctx, dash_present);

    if (code == DP_NOTDONE)
    {
        /* Work not done by Inst. exit */
        /* Make standard locals */
        code = plcini (&dctx);
        if (code != SUCCESS)
            goto postopn_err;
        /* Build symbol tables */
        code = pdftab (&dctx, &dynpdf);
        if (code != SUCCESS)
            goto postopn_err;
        held_msg[0] = EOS;
        code = parcmp(&(*parblk).symtab, &dctx.parmst, held_msg);
        if (code != SUCCESS)
        {
            /* print err msg */
            tmmsg(PROCFAIL, held_msg, notused_key);
        }
        mode = ((*parblk).msgtyp == M_FULLPDF) ? FULLSET : SUBSET;
        code = parmrg(&(*parblk).symtab, &dctx.parmst, mode|VM_VALID);
        if (code != SUCCESS)
            goto postopn_err;
        v = lookex(&(*parblk).symtab, "_PREFACE");
        code = opn_tutor (&dynpdf, &dctx, NOFORCE, v, TRUE);
        if (code != SUCCESS)
            goto postopn_err;
        /* tutor for async or sync ? */
        dctx.asydyntut = asytut_req;
        code = tutor (&dynpdf, &dctx);
        /* close tutor             */
        cls_tutor (&dynpdf, &dctx);
    }
    /* initialize parblk tab   */
    (*parblk).symtab.link = NULL;
    if (code == DO_RUN)
    {
        /* continue the process    */
        if (pack_parm(&dctx.parmst, parblk, pool_size) != SUCCESS)
            goto pack_err;
        (*parblk).msgtyp = M_CONTINUE;
    }
    else if (code == DO_HOLD)
    {
        /* tell caller to hold on  */
        (*parblk).msgtyp = M_HOLD;
    }
    else
    {
        /* tell process of exit */
        if (pack_parm(&nullst, parblk, pool_size) != SUCCESS)
            goto pack_err;
        (*parblk).msgtyp = M_DYNEXIT;
    }
    clsctx(&dctx);
    f_close (&dynpdf, F_KEEP);
    return;

pack_err:
    tmmsg(PROCFAIL, "Parameter values overflow process message capacity.",
        "TAE-PSETOVER");
    goto postopn_err;

postopn_err:
    f_close(&dynpdf, F_KEEP);
    goto clsctx_kill;

clsctx_kill:
    clsctx(&dctx);
    goto kill;

kill:
    pack_parm(&nullst, parblk, pool_size);
    (*parblk).msgtyp = M_KILLED;
    return;
}

/*      dyncommand - execute a TAE (i.e, TCL) command from process. 
 *
 *      The command is assumed to be the string named "_COMMAND"
 *      in the parblk.  The executing process can get hold of
 *      output values from the command by examining the returned
 *      parblk.  See notes below.
 */
FUNCTION VOID dyncommand 
(
    struct CONTXT *ctx,     /* in/out: parameter context    */
    struct PARBLK *parblk,  /* in/out: absolute parblk */
    int           pool_size 
)
{
    IMPORT struct VARIABLE *sfi_gbl, *skey_gbl;
    IMPORT struct CONTXT *curproc;              /* pointer to cur proc  */
    IMPORT CODE          cmd_mode;              /* for prccmd loop      */
    struct VARIABLE     *cmd;
    struct CONTXT       *saved_cur;
    CODE                saved_mode;
    TEXT                cmdstr[CMDLINSIZ+1];
    CODE                code;

    /* save old command mode */
    saved_mode = cmd_mode;
    /* save the 'current' */
    saved_cur  = curproc;
    /* dynamic cmd mode */      
    cmd_mode   = DYNMODE;
    curproc    = ctx;                           
    /* so commands work okay */
    (*ctx).inbody = TRUE;
    cmd = lookex (&(*parblk).symtab, "_COMMAND");
    if (!cmd || (*cmd).v_type != V_STRING || (*cmd).v_count < 1)
    {
        IVAL(*sfi_gbl, 0) = -1; 
        set_string (skey_gbl, "TAE-DYN_COMMAND_BAD");
        code = FAIL;
    }
    else
    {
        /* get it wide enough for  */
        strcpy(cmdstr,SVAL(*cmd,0));
        /* process the command  */
        code = prccmd(cmdstr, ctx);
    }
    /* restore old mode */
    cmd_mode = saved_mode;
    /* restore the cur proc */
    curproc = saved_cur; 

    /*
     *  Here, we package the context of the process PDF for return to
     *  the executing process.   Since the dynamic command runs
     *  in the context of the process PDF, the  dynamic command
     *  can set output variables and the executing process can
     *  get hold of the values.   $SFI/$SKEY may be of most interest.
     */

    /* re-build parblk      */
    code = package (ctx, parblk, pool_size, 0);
    if (code != SUCCESS)
    {
        /* make a good null block */
        r_init ((*parblk).pool, pool_size);
        (*parblk).symtab.link = NULL;
    }
    (*parblk).msgtyp = M_CONTINUE;
    return;
}
