/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*
 *      Installation Exits.  Note that the installation exit for file
 *      verification has been moved to the fileins.c source module.
 *
 */

#include "stdh.inp"      /* system standard  (REQUIRED)          */
#include "taeconf.inp"   /* TAE configuration (REQUIRED)         */
#include "inttaelib.h"
#include "fileinc.inp"   /* file package                         */
#include "parblk.inc"    /* parameter block definitions          */
#include "symtab.inc"    /* TM symbol table                      */
#include "tmhost.inp"    /* TM host-dependent definitions        */
#include "tminc.inc"     /* TM definitions                       */
#include "local_protos.h"

/*
 *  Change log
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *      23-aug-83       Add dp_ins()...peb
 *      26-aug-83       Change to comment in dp_ins()...peb
 *      07-sep-83       Added batchgbl_ins()...dm
 *      24-sep-83       Update dp_ins calling sequence...peb
 *      24-sep-83       Added ds_ins()...peb
 *      26-sep-83       Removed ds_ins; new dp_ins arguments...palm
 *      26-sep-83       New gp_ins...palm
 *      02-feb-84       Add job_ins()...peb
 *      22-jul-87       Add get_commands_ins() as part of effort to force TM 
 *                      into libraries...ljn
 *      09-aug-87       Make cmd table GLOBAL; see explanation in
 *                      intrinsic.c...palm
 *      15-feb-89       New job_ins1...palm
 *      01-may-89       new set_process_name stub....palm
 *      28-jun-90       Removed get_commands_ins()...ljn
 */


/*

At strategic places in TM, function calls are made to allow for 
installation-specific logic in TM.EXE.  These functions are called 
"installation exits".

The source module, EXITS.C, defines the standard TAE processing for
the installation exits.  EXITS.C will be delivered with the standard
TAE distribution; all include files will also have to be delivered so
that EXITS.C may be compiled.  (Compiling EXITS.C is not a requirement
for installing TAE because we do not require existence of a C
compiler.) 

An installation may re-LINK TM, replacing EXITS.OBJ with 
installation-specific code.

In the function descriptions below, the TM function that performs the
call is shown in brackets. 

In addition to writing installation-specific code for the exits,
new TCL commands may be defined.   Also, to assist in writing 
installation-specific code, a new field, ins_context, has been
reserved in the CONTXT block.

*/

/*
 *      init_ins.  TM initialization exit.

This call is made after the greeting message, after the initialization 
of *primctx, and after definition of implicit global variables.
The call is made before the SLOGON proc is executed.

If a fatal error is detected in init_ins, the function should diplay 
an error message and terminate TM.

 */

FUNCTION VOID init_ins 
(
    struct CONTXT *primctx  /* in/out: primary level context  */
)
{
    return;                     /* stub                           */
}

/*
 *      pa_ins.  Proc activation user exit.

This call is made before execution of a proc (process, procedure, 
global PDF, or PARMSET PDF). The call is made for every level of 
procedure nesting.  pa_ins is only called when the PDF has been 
successfully found and the parameters have been verified.

 */

FUNCTION VOID pa_ins 
(
    struct CONTXT *pctx     /* in/out: context of proc      */
)
{
    return;                     /* stub                         */
}

/*
 *      pt_ins.  Proc termination exit.

pt_ins is called upon proc termination and before clsctx is executed 
against the CONTXT block. 

 */

FUNCTION VOID pt_ins 
(
    struct CONTXT *pctx     /* in/out: context of proc      */
)
{
    return;
}

/*      dynamic_ins.  Dynamic message processing.
 *
dynamic_ins is called when an unrecognized PARBLK is received by TM 
from an executing process.  An unrecognized PARBLK is one with 
PARBLK.msgtyp set to a value other than the defined "M_" codes in the
PARBLK include file.
 *
 */
FUNCTION VOID dynamic_ins 
(
    struct CONTXT *pctx,    /* in/out: process CONTXT */
    struct PARBLK *parblk   /* in: PARBLK from process*/
)
{
    tmmsg(PROCFAIL, "Unrecognized message (%d) from application process.",
          "TAE-BADMSG", (*parblk).msgtyp);
}

/*
 *      dp_ins.  Called by dynget before processing a dynamic parameter
 *      request (either from an application process or a GETPAR statement
 *      in a procedure).  The context block passed in here is the block
 *      to apply the user's parameters against.  
 *
 *      The dummy function does no processing, but installation specific
 *      code could get the requested parameters from the user in a way not
 *      normally supported by TM.  This function should return DP_NOTDONE
 *      unless it has completed the dynamic parameter request.  DP_NOTDONE will
 *      cause dynget to complete the parameter request itself. If dp_ins has
 *      completed the request it should return DO_EXIT or DO_RUN
 *      (similar to dyncom).
 *
 *      Note that if command processing is started by dp_ins and
 *      the user EXITs, DO_EXIT should be returned as this counts as completing
 *      the parameter request.
 */

FUNCTION CODE dp_ins 
(
    struct PARBLK *parblk,        /* in: parblk from process */
    struct SFILE  *pdf,           /* in: opened PDF          */
    struct CONTXT *ctx,           /* in/out: dynamic context */
    FUNINT        dash_present    /* in: TRUE if '-' present */
)
{
    return(DP_NOTDONE);
}

/*      gp_ins.    Called for getpar command.
 */
FUNCTION CODE gp_ins 
(
    struct SFILE  *pdf,     /* in: SFILE for opened pdf*/
    struct CONTXT *ctx      /* in/out: dynamic context */
)
{
    return (DP_NOTDONE);
}

/*
 *      term_ins.   Called by exit_do and logoff_do after the
 *      proc execution and just before the image exit
 */
    
FUNCTION VOID term_ins(void)
{
    return;
}

/*
 *      refs_ins.   Called when implicit references are being added
 *      to a CONTXT block.  This function allows installation globals
 *      to be implicitly referenced (just like $SFI is).
 *
 *      To add a reference:
 *              addref(pctx, "global-name");
 */

FUNCTION VOID refs_ins
(
    struct CONTXT *pctx     /* in/out: context block        */
)
{
    return;
}

/*      job_ins.  Called from run_batch just before call to bld_cmd.
 *      This installation exit is useful to insert additional TAE commands
 *      into the job file before the invocation of the requested proc.
 *      E.g., JPL uses this to insert a LISTPROC command
 *
 *              LISTPROC procname
 *
 *      where "procname" is the text string in (*cmdctx).pdf.name.
 *
 *      This function should return SUCCESS unless an error is encountered.
 *      This function must do its own error reporting.
 */

FUNCTION CODE job_ins 
(
    struct SFILE  *jobfile, /* in/out: already open job file*/
    struct CONTXT *cmdctx   /* in:  command context         */
)
{
    return(SUCCESS);
}

/*      Another installation exit for job (called from bldjob.c)
 *      See notes above for job_ins.
 *
 */

FUNCTION CODE job_ins1 
(
    struct SFILE  *jobfile, /* in/out: already open job file*/
    struct CONTXT *cmdctx   /* in:  command context         */
)
{
    return (SUCCESS);
}

/*      set process name for ASYNC and BATCH
 *
 */

FUNCTION VOID set_process_name 
(
    CODE runtype,           /* BATCH or ASYNC */
    TEXT name[]             /* job name     */
)
{
    return;
}

/*
 *      Installation-Specific Commands.

A global ITRCMD table named commands_ins becomes an extension to the 
TM intrinsic command table.  Installation-specific commands may be 
defined in this table.
 
 *
 */

GLOBAL struct ITRCMD commands_ins[] =
{
    {0, ""}             /* terminator entry     */
};
