/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*      help_do
 *
 * This file contains the functions to support command mode help processing.
 *
 * A table of associated intrinsic command structure entries is at the
 * end of the file and associated resident PDFs is near the beginning.
 *
 * CHANGE LOG:
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *      18-jul-83       Don't allow help cmd from a proc...peb
 *      16-aug-83       Move message help function to helpsubs, 
 *                      implement helpblk for interface control...dm
 *      18-aug-83       Implement "?" for help on last message...dm
 *      21-sep-83       Fix RESIVARs for UNIX compatibility...palm
 *      11-oct-83       Fix unix compilation errors...palm
 *      11-oct-83       Fix RESIDVAR for help-parm...palm
 *      31-oct-83       Fix Unix bug for msg_do crash...dm
 *      20-jan-84       Implement "HELP-GLOBAL"...lim 
 *      03-feb-84       Check for help on interrupt mode...nhe
 *      28-feb-84       Fix HELP-GLOBAL errors...lim
 *      04-may-84       VALUE_x to xVAL ...ces
 *      06-may-84       Conform to no .defalt in RESIDVAR...peb
 *      07-may-84       Move IMPORT declarations...lim
 *                      No more dyncmd mode...nhe
 *      04-jun-84       Implement default intrinsic subcommand...lim
 *      07-jun-84       Allow help from proc (pr #626)...dm
 *      24-aug-84       Allow help to go to stdout (807)...nhe
 *      20-oct-84       TCL 117:  Init helpblk for .compiled...peb
 *      25-oct-84       Add logic to handle subcommand help...lia
 *      07-nov-84       Add logic to handle saved proc and subcommand,
 *                      especially for HELP-PARM...lia
 *      08-dec-84       PR 901: set up helpblk.helpobj correctly...lia
 *      16-dec-84       TCL 67: Audit NAMESIZ: replace by F_Q_NAMESIZ?...peb
 *
 ****************************************************************************
 * CHANGES MADE IN THE RCJM TREE:
 *
 *      05-feb-85       Update for getting help on remote proc...nhe
 *      13-mar-85       Renamed get_remote_help() to get_remote_phelp...dm
 *      21-apr-85       Rename get_remote_phelp() to get_rem_inihelp()...dm
 *      22-apr-85       update for remote .include file access...dm
 *      25-apr-85       Fix PR on help-global bug...dm
 *
 *****************************************************************************
 *
 *      11-sep-86       PR 965: Deliver error msg on HELP-COMMAND,
 *                      HELP-PROC with no parameter.  Done by creating
 *                      a new default subcommand, "", for HELP and causing
 *                      HELP-COMMAND, HELP-PROC to each require a parm...peb
 *      19-SEP-86       PR 965: change "" subcommand to GENERAL...peb
 *      08-may-87       PR 1181: change "INTMODE" and "COMMODE" to lower...ljn
 *      22-jul-87       Add get_helpcmd() as part of effort to force TM 
 *                      into libraries...ljn
 *      09-aug-87       Make cmd table GLOBAL; see explanation in
 *                      intrinsic.c...palm
 *      24-mar-88       Delete TAE_RCJM conditionals...ljn
 *      03-may-89       new hardcopy help...palm 
 *      23-may-90       Pull out RCJM...ljn
 *      28-jun-90       Removed get_helpcmd()...ljn
 *      01-aug-91       Braces for static string initializors...ljn
 */

#include <string.h>
#include "stdh.inp"              /* standard C definitions               */
#include "taeconf.inp"           /* TAE configuration definitions        */
#include "inttaelib.h"
#include "tmhost.inp"            /* host-dependent defs                  */
#include "symtab.inc"            /* symbol table                         */
#include "tminc.inc"             /* TM-only host-independent definitions */
#include "dirinc.inc"            /* directive package                    */
#include "syninc.inc"            /* syntax package                       */
#include "helpinc.inc"           /* help output control block            */
#include "terminc.inc"           /* terminal package                     */
#include "local_protos.h"


/*      Memory-resident PDF data for commands performed in this file.
 *
 */

/* HELP with no subcommand */

TEXT  *null_str[] = {""};
static struct RESIDVAR pthelp0[] =
{
    /* name    type      k  m maxc      size     dc val      dvp*/
    {"PROC",    V_STRING, 0, 1, 1,      FSPECSIZ, 1, NULL,   (GENPTR)null_str},
    {"NODE",    V_STRING, 0, 0, 2,   NODENAMESIZ, 0, NULL,   NULL}
};

/* HELP-PROC */

static struct RESIDVAR pthelp1[] =
{
    /* name    type      k  m maxc      size     dc val      dvp*/
    {"PROC",    V_STRING, 0, 1, 1,      FSPECSIZ, 0, NULL,   NULL},
    {"NODE",    V_STRING, 0, 0, 2,   NODENAMESIZ, 0, NULL,   NULL}
};


/* HELP-COMMAND */
static struct RESIDVAR pthelp2[] =
{
    /* name    type      k  m maxc      size     dc val      dvp*/
    {"COMMAND", V_STRING, 0, 1, 1,      FSPECSIZ, 0, NULL,   NULL},
    {"NODE",    V_STRING, 0, 0, 2,   NODENAMESIZ, 0, NULL,   NULL}
};


/* HELP-MESSAGE */
static struct RESIDVAR pthelp3[] =
{
    /* name    type      k  m maxc      size     dc val      dvp*/
    {"KEY",     V_STRING, 0, 1, 1,      KEYSIZ,   1, NULL,   (GENPTR)null_str}
};


/* HELP-PARM */
static struct RESIDVAR pthelp4[] =          /* normal command mode */
{
    /* name    type      k  m maxc      size     dc val      dvp*/
    {"PARM",    V_STRING, 0, 1, 1,   F_Q_NAMESIZ,  -1, NULL,   NULL},
    {"PROC",    V_STRING, 0, 1, 1,      FSPECSIZ,  1, NULL,   (GENPTR)null_str},
    {"NODE",    V_STRING, 0, 0, 2,   NODENAMESIZ, 0, NULL,   NULL}
};


/* HELP-GLOBAL */
static struct RESIDVAR pthelp5[] =
{
    /* name     type      k  m maxc      size     dc val      dvp*/
    {"VARIABLE", V_STRING, 0, 1, 1,      NAMESIZ,  -1, NULL,   NULL}
};

/* HELP-HARDCOPY */
static struct RESIDVAR pthelp6[] =
{
    /* name     type      k  m maxc      size     dc val      dvp*/
    {"PROC",    V_STRING, 0, 1, 1,      FSPECSIZ, -1, NULL,    NULL}, 
    {"OUTPUT",  V_STRING, 0, 1, 1,      FSPECSIZ,  1, NULL,   (GENPTR)null_str}
};

/*
 *      help_do intrinsic command table 
 *      Note that the help command is not allowed from a proc.
 */
FUNCTION CODE help_do 
(
    struct CONTXT *cpctx,   /* in:  containing proc context    */
    struct CONTXT *npctx    /* in:  proc ctx for HELP command  */
);
FUNCTION CODE helpmsg_do 
(
    struct CONTXT *cpctx,   /* in:  containing proc context    */
    struct CONTXT *npctx    /* in:  proc ctx for HELP command  */
);
FUNCTION static CODE msg_do
(
    TEXT msgkey[]           /* message key to help on  */
); 

#define Y_HELP Y_GENERAL

GLOBAL struct ITRCMD helpcmd[] =    /* help commands  */
{
           /* subcommand */     
    {1, "HELP",     "GENERAL",   Y_HELP|Y_DEFSUB,
                                            I_NPM(pthelp0), pthelp0, help_do },
    {1, "HELP",     "PROC",      Y_HELP,    I_NPM(pthelp1), pthelp1, help_do },
    {1, "HELP",     "COMMAND",   Y_HELP,    I_NPM(pthelp2), pthelp2, help_do },
    {1, "HELP",     "MESSAGE",   Y_HELP,    I_NPM(pthelp3), pthelp3, help_do },
    {1, "HELP",     "PARM",      Y_HELP,    I_NPM(pthelp4), pthelp4, help_do },
    {1, "HELP",     "GLOBAL",    Y_HELP,    I_NPM(pthelp5), pthelp5, help_do },
    {1, "HELP",     "HARDCOPY",  Y_HELP,    I_NPM(pthelp6), pthelp6, help_do },
    {1, "?",        "",          Y_HELP,    0,              NULL,   helpmsg_do},
    {0, ""}        /* TERMINATOR ENTRY: REQUIRED AT END */
};


/*      help_do - perform HELP command if entered from command (non-interrupt)
 *       mode.
 *
 *      Assumes that the command was entered from command mode.
 *
 */

FUNCTION CODE help_do 
(
    struct CONTXT *cpctx,   /* in:  containing proc context    */
    struct CONTXT *npctx    /* in:  proc ctx for HELP command  */
)
{
    IMPORT TEXT lastkey[];              /* most recent message key      */
    IMPORT TEXT prevproc[];             /* most recent cmd or proc help */
    IMPORT TEXT prevsub[];              /* most recent subcommand help  */
    IMPORT TEXT next_cmd[NEXTSIZ+1];
    IMPORT CODE cmd_mode;
    IMPORT CODE help_proc;              /* NOPROC or SAVEPROC           */
    IMPORT struct SYMTAB glbtab;
    IMPORT BOOL help_to_term;   /* TRUE if help output must be terminal */
    IMPORT BOOL full_scr_help;
    IMPORT struct SFILE  *pstdo_fil;    /* pointer to std out file context */
    IMPORT CODE termtype;               /* T_CRT or not                 */

    struct  VARIABLE    *v;                /* variable in symbol table */
    struct  SFILE       fctx;                   /* help file context        */
    CODE                code;
    struct DEFPDF       *defpdf;              /* global pdf block         */
    TEXT                name[STRINGSIZ+1];
    TEXT                procname[FSPECSIZ+1];
    TEXT                subname[SUBCMDSIZ+1];
    TEXT                parmname[STRINGSIZ+1];
    TEXT                libname[FLIBRSIZ+1],curlibname[FLIBRSIZ+1];
    TEXT                type[STRINGSIZ+1];
    TEXT                msgkey[KEYSIZ+1];
    TEXT                subcmd[SUBCMDSIZ+1];
    BOOL                intrin = FALSE;                 /* true if intrinsic */
    struct  HELPBLK     helpblk;                /* help output control block */

    help_to_term = FALSE;                 /* doesn't have to be terminal  */
    /* for full-screen      */
    full_scr_help = ((termtype == T_CRT) && (pstdo_fil == NULL)); 
    if (s_equal((*npctx).subcmd, "MESSAGE"))            /* help-message */
    {
        help_proc = NOPROC;
        v = lookex(&(*npctx).parmst, "KEY");
        if (v != NULL)
            strcpy(msgkey,SVAL(*v, 0));         /* get from ctx block */
        if (NULLSTR(msgkey))                    /* if a null key  */   
            strcpy(msgkey,lastkey);             /* get most recent key */
        code = msg_do(msgkey); 
        return(DO_CHECK);
    }                       

    /*      Comes here for other types of help              */

    helpblk.compiled   = FALSE;
    helpblk.nextcmd[0] = EOS;                           /* initialize   */
    helpblk.helpobj[0] = EOS;
    parmname[0] = EOS;

    strcpy(subcmd,(*npctx).subcmd);
    if (s_equal(subcmd, "PARM") || s_equal(subcmd, "GLOBAL"))
    {
        if (s_equal(subcmd, "PARM"))
        {
            v  = lookex(&(*npctx).parmst, "PARM");
            if (v != NULL)
                strcpy(parmname,SVAL(*v, 0));   /* copy parm name */
            v  = lookex(&(*npctx).parmst, "PROC");     /* in case "proc" */
            strcpy(name,SVAL(*v, 0));           /* copy proc name */
            if (help_proc == SAVEPROC)          /* fill in missing proc & sub */
            {
                strcpy(procname,name);
                if (NULLSTR(name) || s_lseq("-", name))
                    strcpy(procname,prevproc);
                if (NULLSTR(name) && !NULLSTR(prevsub))
                {
                    strcat(procname,"-");
                    strcat(procname,prevsub);
                }
                else if (s_lseq("-", name))
                    strcat(procname,name);
                strcpy(name,procname);
            }
            if (NULLSTR(name))
            {
                tmmsg(PROCFAIL, "Missing parameter:  PROC.", "TAE-MISPAR");
                return (DO_CHECK);
            }
            if (s_lseq("-", name)) goto dh_parmerr;
            if (!subchar(name))
                strcat(name,"-");
            strcpy(type,"proc");
        }
        else
        {
            help_proc = NOPROC;
            v = lookex(&(*npctx).parmst, "VARIABLE");
            if (v != NULL)
                strcpy(parmname,SVAL(*v, 0));
            v = lookex(&glbtab, parmname);
            if (v != NULL  &&  (*v).v_class == V_GLOBAL)
            {
                defpdf = (*v).v_pdf;
                if (defpdf != NULL) 
                    f_spec(&(*defpdf).pdf, name);     /* fetch pdf file spec */
                else
                {
                    tmmsg(PROCFAIL, "No help available on '%s'.", "TAE-NOHELP",
                          parmname);
                    return (DO_CHECK);
                } 
            }
            else
            {
                tmmsg(PROCFAIL, "Global variable '%s' does not exist.",
                      "TAE-NOGLB", parmname);
                return (DO_CHECK);
            }
            strcpy(type,"global");
        }
        code = opnhlp(&fctx, name, procname, subname, libname,     
                &intrin, &helpblk);                   /* open help file       */
        if (code != SUCCESS) goto dh_hlperr;
        if (NULLSTR(parmname))
            strcpy(helpblk.helpobj,name);               /* give proc name */
        else
            strcpy(helpblk.helpobj,parmname);           /* give parm name */
        code = proc_help(&fctx, libname, "", procname, subname, parmname, 
                type, &helpblk);                        /* process help file */
    }
    else                                /* mode help or help on cmd, proc*/
    {
        if ((v = lookex(&(*npctx).parmst, "PROC")) != NULL)     /* help  proc */
            strcpy(name,SVAL(*v, 0));           /* get explicit name */
        else if ((v = lookex(&(*npctx).parmst, "COMMAND")) != NULL)     
            strcpy(name,SVAL(*v, 0));           /* help  command */

        if (NULLSTR(name))                              /* mode  help */
        {
            help_proc = NOPROC;
            strcpy(type,"mode");
            if (cmd_mode == INTMODE)
            {
                strcpy(name,"intmode");
                strcpy(helpblk.helpobj,"interrupt mode");
            }
            else
            {
                strcpy(name,"commode");
                strcpy(helpblk.helpobj,"command mode");
            }
            code = f_open(&fctx, HELPLUN, HELPLIB, name, HLP_TYPE, F_READ);
            if (code != SUCCESS) goto dh_operr;
            code = proc_help(&fctx, HELPLIB, "", "", "", "",
                    "", &helpblk);              /* process help file    */
        }
        else
        {
            if (s_lseq("-", name))              /* missing command name */
            {
                if (help_proc != SAVEPROC) goto dh_parmerr;
                strcpy(procname,prevproc);
                strcat(procname,name);
                strcpy(name,procname);
            }
            code = opnhlp(&fctx, name, procname, subname, libname, 
                    &intrin, &helpblk);         /* open help file       */
            if (code != SUCCESS) goto dh_hlperr;
            strcpy(curlibname,libname); /* libe for opened file */
            (intrin) ? strcpy(type,"command") : strcpy(type,"proc");
            if (NULLSTR(subname))
                strcpy(helpblk.helpobj,procname);
            else
                strcpy(helpblk.helpobj,subname);
            if (s_equal (subcmd, "HARDCOPY"))
            {
                v = lookex (&(*npctx).parmst, "OUTPUT");
                strcpy(name,SVAL(*v,0));
                code = hard_help (&fctx, libname, procname, name, &helpblk);
            }
            else
                code = proc_help(&fctx, curlibname, libname, procname, 
                    subname, "", type, &helpblk);       /* process help file */
        }
    }
    f_close(&fctx, F_KEEP);
    if (code != SUCCESS) goto dh_hlperr;
    if (s_equal(type, "proc") || s_equal(type, "command"))
    {
        help_proc = SAVEPROC;
        strcpy(prevproc,procname);      /* save command or proc name    */
        strcpy(prevsub,subname);        /* save subcommand name         */
    }
    strcpy(next_cmd,helpblk.nextcmd);   /* prev mode command from help  */
    goto help_exit;
        
dh_operr:

    tmmsg(PROCFAIL, "No help available on '%s'.", "TAE-NOHELP", 
          helpblk.helpobj);
    goto help_exit;

dh_hlperr:

    tmmsg(PROCFAIL, helpblk.errmsg, helpblk.errkey); /* display error message */
    goto help_exit;

dh_parmerr:

    tmmsg(PROCFAIL, "Missing proc name for initial help request.",
          "TAE-MISPROC");
    goto help_exit;

help_exit:
    return (DO_CHECK);
}

/*
 * helpmsg_do.  Help on last displayed message.
 */

FUNCTION CODE helpmsg_do 
(
    struct CONTXT *cpctx,   /* in:  containing proc context    */
    struct CONTXT *npctx    /* in:  proc ctx for HELP command  */
)
{
    IMPORT TEXT lastkey[];      /* key of last displayed message   */
    IMPORT BOOL help_to_term;   /* TRUE if help output must be terminal */
    IMPORT BOOL full_scr_help;  /* TRUE if we want help with paging */
    IMPORT struct SFILE *pstdo_fil;   /* pointer to std out file context      */
    IMPORT CODE termtype;             /* T_CRT or not                 */

    help_to_term = FALSE;             /* not necessarily to terminal     */
    full_scr_help = ((termtype == T_CRT) && (pstdo_fil == NULL));
    msg_do(lastkey);
    return(DO_CHECK);
}

/* 
 * msg_do.  Get help on error message with given key    .
 */

FUNCTION static CODE msg_do
(
    TEXT msgkey[]           /* message key to help on  */
) 
{
    IMPORT  TEXT        next_cmd[NEXTSIZ+1];

    CODE                code;
    struct  SFILE       fctx;                   /* sfile for help use   */
    struct  HELPBLK     helpblk;                /* help control block   */

    if (NULLSTR(msgkey))                        /* no key value         */
    {
        tmmsg(PROCFAIL, "No message key specified.", "TAE-NOMSG");
        return(FAIL);
    }
    helpblk.compiled   = FALSE;
    helpblk.nextcmd[0] = EOS;                           /* initialize   */
    code = msg_help(&fctx, msgkey, &helpblk);   /* display help on key */
    if (code == SUCCESS)
        strcpy(next_cmd,helpblk.nextcmd);       /* previous mode help cmd   */
    else
        tmmsg(PROCFAIL, helpblk.errmsg, helpblk.errkey);    /* report error */
    return(code);
}
