/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/



/*<<BSD/UNIX>>*/
/*
 *      hostcmd.   Host-dependent TCL intrinsic commands.
 *
 *      CHANGE LOG:
 *
 *      21-dec-83       Set $SFI, $SKEY after shell termination..dm
 *      10-may-84       set menu_screen (for TAE V1.2)...dm
 *      16-may-84       get cmd interprter name as a defined symbol...dm
 *      01-jul-85       Change VALUE_x to xVAL, Y_BATCH to Y_ABI...dm
 *      15-aug-85       Add exit handler to abort children when tm dies...dm
 *      12-sep-85       Add set_exit_sig for exit time cleanup...dm
 *      03-dec-86       V1.4: Honor host_enabled flag in shell_do...peb
 *      15-dec-86       Fix call seq in abort_remote call...peb
 *      04-jan-87       Add new HI009 intrinsic for FDs...palm
 *      19-jan-87       Make interactive and non-interactive shells to
 *                      be consistent (PR #1104)...dm
 *      12-aug-87       Added get_hostcmd()...tpl
 *      18-feb-88       Bring HI002 from VMS to unix...dm
 *      24-mar-88       Delete TAE_RCJM conditionals...ljn
 *      10-nov-88       SYSV needs to #include fcntl.h...ljn
 *      31-jan-89       from hi002, call delete_ivp_pool to clean
 *                      things up for dynamic memory monitoring...palm
 *      01-may-89       alter check on cmd_mode...palm 
 *      01-jul-89       types.h not included by file.h on SYSV machines...ljn
 *      23-may-90       Remove RCJM stuff by referring to old TAE_RCJM...ljn
 *      28-jun-90       Removed get_hostcmd()...ljn
 *      06-feb-91       dm_bytes stuff ifdef'd out for FAST_ALLOC...ljn
 *      22-mar-95       EROS: added cd command to change current directory.
 *                      Also added code to set the charge code from the
 *                      "CHARGE" global.
 *                      Brian Davis - EROS Data Center
 */

#include <stdlib.h>  /* for system call */
#include <ctype.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include "taeconf.inp"   /* TAE configuration definitions        */
#include "inttaelib.h"
#include "syninc.inc"    /* syntax block                         */
#include "tminc.inc"     /* TM-only host-independent definitions */
#ifdef TAE_ASYNC
#include "asyncinc.inc"
#endif
#ifdef macII
#include <compat.h>
#include <sys/types.h>
#else
#ifdef SYSV
#include <fcntl.h>
#include <sys/types.h>
#endif
#endif
#include <sys/file.h>    /* for open/close HI009 use             */
#include "local_protos.h"

FUNCTION VOID term_sig(int signum);

/*
 * Include the TAE global variables.  This is used to access the correct
 * chargecode for all host commands.
 */
#ifdef EROS
IMPORT struct SYMTAB glbtab;
#endif EROS


/* Table of intrinsic commands in this source module:   */

FUNCTION CODE shell_do
(
    struct CONTXT *cpctx,   /* in: current proc context     */
    struct CONTXT *npctx    /* in: context of DCL command   */
);

FUNCTION CODE fd_do(void);

#ifdef EROS
FUNCTION CODE chdir_do
(
    struct CONTXT *cpctx,   /* in: current proc context     */
    struct CONTXT *npctx    /* in: context of DCL command   */
);
#endif EROS

GLOBAL struct ITRCMD hostcmd[] =        
{

/*  name        sub     flags                     pcount    pdf    do-fun   */
#ifdef EROS
{0, "CHDIR",    "",  Y_ABI|Y_BODY|Y_CLEAR|Y_CMD|Y_DEFSUB,
                                                   0,     NULL,   chdir_do },
#endif EROS

{0, "USH",      "",  Y_ABI|Y_BODY|Y_CLEAR|Y_CMD|Y_DEFSUB,
                                                   0,     NULL,   shell_do }, 

{0, "HI009",    "",  Y_ABI|Y_BODY|Y_CLEAR|Y_CMD|Y_DEFSUB,
                                                   0,     NULL,    fd_do},
{0, ""}   /* TERMINATOR ENTRY: required */ 
};



#ifdef EROS

FUNCTION CODE chdir_do
(
    struct CONTXT *cpctx,   /* in: current proc context     */
    struct CONTXT *npctx    /* in: context of DCL command   */
)
{
    IMPORT struct VARIABLE *skey_gbl;   /* prt to $skey         */
    IMPORT struct VARIABLE *sfi_gbl;    /* ptr to $sfi          */
    IMPORT CODE   menu_screen;
    IMPORT CODE   cmd_mode;             /* intmode or normmode  */

    struct SYNBLK *sb;                  /* ptr to syntax block  */
    TEXT *i;
    CODE status;
    TEXT *sh_string;
    TEXT skey_val[KEYSIZ+1];
    TEXT *value[1];                     /* scratch value vector */

    /* If not first time force a menu prompt */
    if (menu_screen != UNDEFINE)
        menu_screen = PROMPT_PAINT;

    if (cmd_mode == INTMODE)
    {
        tmmsg(PROCFAIL, "'CHDIR' only available in normal command mode.",
            "TAE-BADCMD");
        return (DO_CHECK);
    }
    
    /* Ensure the last character of the path is not a '\' character.  Ignore
       all trailing blanks when examining the last character. */
    sb = (*npctx).sb;
    for (i=(*sb).curchr; *i != EOS; i++);
    for(--i; *i == ' ' || *i == '\t'; i--);
    if (*i == '\\')
    {
        tmmsg(PROCFAIL, "'\' invalid as last character.", "TAE-INVSHL");
        return (DO_CHECK);
    }

    /* Set a pointer to the rest of the directory path */
    sh_string = (*sb).curchr;

    /* Make sure there is a directory path */
    if (chkend(sb) == SUCCESS)
    {
        tmmsg(PROCFAIL, "'CHDIR' NO parameters.", "TAE-BADCMD");
        return (DO_CHECK);
    }

    /* Ignore all leading white space */
    while (isspace(*sh_string))
        sh_string++;

    /* Change to the specified directory */
    status =chdir(sh_string);
    if (status == -1)
        tmmsg(PROCFAIL, "Can't change to that dir.", "TAE-BADCMD");
    IVAL(*sfi_gbl, 0) = (status == 0) ? 1 : -1;     /* set $skey         */
    sprintf(skey_val, "%o", status);                /* convert to string */
    value[0] = &skey_val[0];                        /* skey value poiner */
    set_value(skey_gbl, value, 1);                  /* set $skey         */
    return(DO_CHECK);
}
#endif


/*
 *      shell_do.   Shell command.  Everything past the command field is sent
 *      to the shell as the command to be executed.
 *
 *      While the shell command is executing, to TM it is like a
 *      process is running. TM waits until shell execution is complete.
 *      However no message communication exists between the shell and
 *      TM and proc interrupt mode is not available.
 *      Note that if no parameter follows the shell command when invoked
 *      by the user, then shell is run interactively.
 */

FUNCTION CODE shell_do
(
    struct CONTXT *cpctx,   /* in: current proc context     */
    struct CONTXT *npctx    /* in: context of DCL command   */
)
{
    IMPORT struct VARIABLE *skey_gbl;   /* prt to $skey         */
    IMPORT struct VARIABLE *sfi_gbl;    /* ptr to $sfi          */      
    IMPORT CODE menu_screen;
    IMPORT CODE cmd_mode;               /* intmode or normmode  */
    IMPORT BOOL host_enabled;           /* TRUE if USH command OK*/

    struct SYNBLK *sb;                  /* ptr to syntax block  */
    TEXT *i;
    CODE status;
    TEXT *sh_string;
    TEXT skey_val[KEYSIZ+1];
    TEXT *value[1];                     /* scratch value vector */      
    TEXT shell_string[CMDLINSIZ+1];
    TEXT *ptr;
    COUNT len;                          /* length of string     */

#ifdef EROS
    struct  VARIABLE        *charge;
#endif EROS

    /* if not first time force menu prompt */
    if (menu_screen != UNDEFINE)
        menu_screen = PROMPT_PAINT;
    if (!host_enabled)
    {
        tmmsg (PROCFAIL, "'USH' command has been disabled.", "TAE-DISHOST");
        return (DO_CHECK);
    }
    if (cmd_mode != NORMCMD)    /* only from normal cmd mode! */
    {
        tmmsg(PROCFAIL, "'USH' only available in normal command mode.", 
            "TAE-BADCMD");
        return (DO_CHECK);
    }
    sb = (*npctx).sb;
    for (i=(*sb).curchr; *i != EOS; i++);       /* go to end of string  */
    for(--i; *i == ' ' || *i == '\t'; i--);     /* ignore trail white space  */
    if (*i == '\\')                             /* last character is '\'     */
    {
        tmmsg(PROCFAIL, "'\' invalid as last character.", "TAE-INVSHL");
        return (DO_CHECK);
    }
    ptr = getenv("SHELL");

#ifdef EROS
    /* Use the correct charge code when at the EROS Data Center.  Simply add
       the charge command and the desired chargecode to the front of the shell
       command. */
    charge = lookex(&glbtab, "CHARGE");
    if (charge == NULL)
    {
        /* No chargecode */
        strcpy(shell_string,"");
    }
    else
    {
        /* Found a chargecode */
        strcpy(shell_string,"charge "); 
        strcat(shell_string,SVAL(*charge, 0));
        strcat(shell_string," ");
    }
#endif


    if (ptr == NULL)
    {
        /* shell not specified, so default to using sh */
        strcpy(shell_string,"sh");
    }
    else
        s_bcopy(ptr, shell_string, FSPECSIZ);

    /* ptr to rest of TCL cmd */
    sh_string = (*sb).curchr;           
    if (chkend(sb) == SUCCESS)
    {          
        /* command is shell only make shell interactive */
        strcat(shell_string," -i");
    }
    else
    {
        /* add flag to shell that command follows  */
        strcat(shell_string," -c \"");
        len = strlen(shell_string);

        /* strip leading white space */
        while (*sh_string == ' ' || *sh_string == '\t')
            sh_string++;                        

        /* add user command */ 
        s_bcopy(sh_string, &shell_string[len], CMDLINSIZ-len-2);
        strcat(shell_string,"\"");      /* end with quotes */
    }
    /* issue the shell command */
    status = system(shell_string);
    if (status == 0177)
    {
        /* could not run shell */
        tmmsg(PROCFAIL, "Could not invoke shell.", "TAE-RUNSHELL");
    }
    else
    {
        if (status != 0)
        {
            tmmsg(PROCFAIL, "Abnormal shell termination.", "TAE-ABNSHELL");
        }
    }
    IVAL(*sfi_gbl, 0) = (status == 0) ? 1 : -1; /* set $skey         */ 
    sprintf(skey_val, "%o", status);            /* convert to string */
    value[0] = &skey_val[0];                    /* skey value poiner */
    set_value(skey_gbl, value, 1);              /* set $skey         */
    return(DO_CHECK);
}

/*      host_init.   Host-dependent initialization.
 *
 *      For UNIX, we specify exit handlers (signal catchers for SIGTERM
 *      and SIGQUIT) for killing all active children when the parent
 *      monitor dies. Note that we don't catch  other signals such as 
 *      bus error because they are not  part of normal scenario.
 *
 */

FUNCTION VOID host_init (void)
{

    /* set up an exit handler */
#ifdef macII
    setcompat (COMPAT_BSDSIGNALS | COMPAT_EXEC | COMPAT_BSDTTY);
#endif
    set_signal(SIGTERM, term_sig);
    set_signal(SIGQUIT, term_sig);
}

/*  The exit handler */
FUNCTION VOID term_sig(int signum)
{

    IMPORT TAEINT sync_child;       /* pid of running child */

    /* if child running, kill it */
    if (sync_child > 0)
        kill (sync_child, SIGKILL);
 
#ifdef TAE_ASYNC
    /* kill all async jobs  */
    term_async();
#endif
    /* died due to signal   */
    exit(-1);
}

#ifdef TAE_ASYNC

FUNCTION VOID term_async(void)
{
    IMPORT struct ACB   *acb_head;              /* acb list-head        */
    struct ACB          *acb;


    /* kill async processes */
    for (acb = acb_head; acb != NULL; acb = (*acb).link)
    {
        if ((*acb).active)
        {
            /* abort the local process  */
            abort_acb (acb);
            tmmsg (SUCCESS, "Job '%s' aborted.", "TAE-ABORTOK",(*acb).name);
        }
    }
}
#endif

/* determine how many file descriptors are available */
FUNCTION CODE fd_do(void)
{
    IMPORT struct VARIABLE *sfi_gbl;
    long max;

    max = sysconf(_SC_OPEN_MAX);
    tmmsg (SUCCESS, "%d", "TAE-FDSAVAIL", max);
    IVAL (*sfi_gbl, 0) = max;                   /* return max as $SFI */
    return (SUCCESS);
}
