#ifndef LOCAL_PROTOS_H
#define LOCAL_PROTOS_H

#include <stdarg.h>
#include "asyncinc.inc"
#include "helpinc.inc"
#include "dirinc.inc"
#include "expinc.inc"

FUNCTION GENPTR allleg
(
    struct VARIABLE *v,     /* in/out: ptr to var being validized   */
    FUNINT lcount           /* in:  number of valid ranges       */
);

/* Allocate space for values */
FUNCTION TEXT *allval
(
    struct VARIABLE *v      /* in/out: variable to have values allocated  */
);

/* Allocates structure and places it at end of symbol table chain */
FUNCTION struct VARIABLE *allvar
(
    struct SYMTAB *head     /* in/out: pointer to symbol table header */
);

/* Add a variable in alphbetic order to symbol table */
FUNCTION struct VARIABLE *alpha_var
(
    TEXT          name[],   /* in: name of variable to be added       */
    struct SYMTAB *head     /* in/out: pointer to symbol table header */
);

/* delete symbol table */
FUNCTION VOID  deltab
(
    struct SYMTAB *head     /* in/out: symbol table header */
);

/* delte values from symbol table */
FUNCTION VOID delval
(
    TEXT   *pv,             /*in: pointer to values */
    FUNINT type,            /*in: type of variable  */
    FUNINT count            /*in: current value cnt */
);

/* Return size of valid structure */
FUNCTION COUNT valid_size
(
    FUNINT type,            /* in: V_INTEGER, V_REAL, V_STRING */
    FUNINT count            /* in: number of ranges */
);

/* delete variable from symbol table */
FUNCTION VOID delvar
(
    struct SYMTAB *head,    /* in/out: symbol table */
    struct VARIABLE *v      /* in:  entry to delete */
);

/* find a slot in the symbol table */
FUNCTION struct VARIABLE *find_slot
(
    TEXT          name[],   /* in: variable name to match    */
    struct SYMTAB *symtab   /* in: symbol table to look thru */
);

/* get a variable block for an intrinsic parameter and link it to the 
   symbol table */
FUNCTION struct VARIABLE *get_ivp
(
    struct SYMTAB *head     /* in/out: symbol table header  */
);

/* initialize a symbol table */
FUNCTION VOID initab
(
    struct SYMTAB *head     /* header to init */
);

/* find a symbol in a symbol table using an exact match */
FUNCTION struct VARIABLE *lookex
(
    struct SYMTAB *head,    /* in: symbol tab to search     */
    TEXT name[]             /* in: name to find             */
);

/* find a symbol in a symbol table allowing abbreviations */
FUNCTION CODE lookab 
(
    struct SYMTAB   *head,  /* in:  header to symbol table          */
    TEXT            name[], /* in:  name of variable (parm) to find */
    struct VARIABLE **v     /* out: variable found (NULL unless SUCCESS)*/
);

/* delete tutor extensions from a variable in a symbol table */
FUNCTION VOID deltut 
(
    struct VARIABLE *v      /* in/out: variable to deallocate from  */
);

/* determine bytes for one value */
FUNCTION COUNT valsize
(
    FUNINT type             /* V_STRING, V_REAL, V_INTEGER */
);

/* issue TAE logon greeting */
FUNCTION VOID greet 
(
    TEXT vrsion[]           /* in:  TM version string */
);

/* Copy VARIABLE structure to new allocated storage */
FUNCTION CODE vcopy
(
    struct VARIABLE *vin,  /* in: VARIABLE struct to move */
    struct VARIABLE *vout  /* in: VARIABLE struct to move to */
);

/* Copy a value vector */
FUNCTION CODE cpy_val
(
    FUNINT type,            /* in: variable type */
    void   *in,             /* in value(s) */
    GENPTR out,             /* out value(s) */
    FUNINT count            /* in: number of in values */
);

/* Copy a variable's valid structure */
FUNCTION VOID cpy_vld
(
    FUNINT type,            /* in: variable type */
    GENPTR in,              /* in: valid structure */
    GENPTR out              /* out: valid structure */
);

/* Build x_VALID structure for a VARIABLE */
FUNCTION CODE bld_valid
(
    struct SYMTAB   *st,        /* in: symbol table of var (for keyword 
                                   validation)     */
    struct VARIABLE *v,         /* in/out: VARIABLE structure   */
    TEXT            *value[],   /* in: ranges in string format  */
    FUNINT          count       /* in: count of value vector    */
);

/* Verify that a proposed value is compatible with VALID */
FUNCTION int chk_valid
(
    struct VARIABLE *v,     /* VARIABLE to check            */
    GENPTR          value,  /* genptr to value vector       */
    FUNINT          count   /* count of value               */
);

/* Transform string values to VALID form */
FUNCTION CODE trans_valid
(
    struct VARIABLE *v      /* VARIABLE to transform        */
);

/* Translate a keyword into the associated variable structure */
FUNCTION CODE key_trans
(
    struct SYMTAB   *parmst,/* in: parameter symbol table   */
    TEXT            key[],  /* in: keyword value            */
    struct VARIABLE **v     /* out: VARIABLE pointer        */
);

/* Verify that a value vector is compatible with a VARIABLE structure */
FUNCTION CODE chk_vector
(
    struct VARIABLE *v,     /* in: VARIABLE to check against        */
    FUNINT          type,   /* in: type of value                    */
    GENPTR          value,  /* in: pointer to value vector to check */
    FUNINT          count,  /* in: count of value vector            */
    BOOL            compiling   /* in: compiling a proc ? */
);

/* Verify that a index value is compatible with a VARIABLE structure */
FUNCTION CODE chk_component
(
    struct VARIABLE *v,     /* in: VARIABLE to check against        */
    FUNINT          type,   /* in: type of value                    */
    GENPTR          value   /* in: pointer to value vector to check */
);

/* fill a variable with count null values */
FUNCTION CODE fill_value
(
    struct VARIABLE *v,     /* in/out: VARIABLE structure   */
    FUNINT          count   /* in: new value count          */
);

/* move a value to a variable */
FUNCTION CODE set_component
(
    struct VARIABLE *v,     /* structure to be updated      */
    void            *value, /* in: pointer to value vector  */
    FUNINT          index   /* in: TCL index (value of 1 to  count) */
);

/* set a string value of a variable */
FUNCTION CODE set_string 
(
    struct VARIABLE *v,      /* in/out:  variable to set */
    TEXT            string[] /* in:  value to set variable to */
);

/* move a value to a variable */
FUNCTION CODE set_value
(
    struct VARIABLE *v,     /* structure to be updated      */
    void            *value, /* in: pointer to value vector  */
    FUNINT          count   /* in: vector count             */
);

/* transfer input string value vector to current value vector */
FUNCTION CODE trs_value
(
    struct VARIABLE *v,     /* structure to be updated      */
    GENPTR          value,  /* in: pointer to value vector  */
    FUNINT          count   /* in: vector count             */
);

/* Pop an element from from a stack of TINY */
FUNCTION CODE poptin
(
    TINY stack[]            /* IN/OUT: Stack to access */
);

/* push element onto a stack of TINY */
FUNCTION CODE pushti
(
    TINY   stack[],         /* IN/OUT: Stack to access */
    FUNINT value            /* IN: Value to be pushed onto stack */
);

/* initialize a stack of TINY elements */
FUNCTION VOID stk_in
(
    TINY stack[],           /* IN/OUT: Stack to access */
    TINY s_length,          /* IN: Allowable number of entries on stack */
    TINY inival             /* IN: Initial value to pushed on the stack */
);

/* get value for the top of a stack of TINY elements */
FUNCTION CODE toptin
(
    TINY stack[]            /* IN: Stack to access */
);

/* Add an integer variable to a symbol table */
FUNCTION CODE addint 
(
    struct SYMTAB *st,            /* in/out: symbol table                 */
    TEXT          name[],         /* in:  name of variable to add         */
    FUNINT        maxc,           /* in:  maximum count                   */
    FUNINT        count,          /* in:  count                           */
    FUNINT        valvec[],       /* in:  values of integer               */
    FUNINT        class           /* in: class (V_LOCAL, etc.)            */
);

/* add a reference in the current proc context to an existing global variable */
FUNCTION VOID addref 
(
    struct CONTXT *pctx,          /* in/out: proc ocntext */
    TEXT          name[]          /* in:  name of global variable */
);

/* add a string variable to a symbol table */
FUNCTION CODE addstr 
(
    struct SYMTAB *st,            /* in/out: symbol table         */
    TEXT          name[],         /* in:  name of variable to add */
    FUNINT        maxc,           /* in: max value count          */
    FUNINT        count,          /* in: current value count      */
    TEXT          *val[],         /* in:  string value vector     */
    FUNINT        class           /* in: class (for v_class)      */
);

/* add proc info (name and line number) to message if in a PDF */
FUNCTION VOID addproc
(
    struct CONTXT *pctx,          /* IN: proc context */
    TEXT          record[]        /* IN/OUT: formatted line */
);

/* display header on formatted screen */
FUNCTION VOID dsphdr 
(

    TEXT   dspname[],              /* in:  display name            */
    TEXT   left_str[],             /* in:  left string for header  */
    FUNINT pagenum,                /* in:  page number             */
                                   /* (.le. 0  means no page nrs)  */
    FUNINT lastflag                /* in:  TRUE if last page       */
);

/* read in .TITLE text and return it */
FUNCTION CODE gettitle 
(

    struct DIRBLK  *db,            /* in/out: directive block for help file*/
    struct TXTSTOR *title,         /* out: title text in dynamic storage   */
    TEXT           errmsg[],       /* out: error message                   */
    TEXT           key[]           /* out: error key                       */
);

/* build a left justified field with highlighting */
FUNCTION VOID high_lfield
(
    TEXT   input[],        /* in: input string             */
    FUNINT width,          /* in: width of field           */
    TEXT   output[]        /* out: string with high sequence */
);

/* returns true if file looks like a binary PAR file */
FUNCTION BOOL IsParFile 
(
        struct SFILE *sfile
);

/* build the left header with file/library name for display */
FUNCTION VOID left_fbld
(
    TEXT libname[],              /* IN: library name             */
    TEXT fname[],                /* IN: proc/mdf  name           */
    TEXT subc[],                 /* IN: subcmd name (ignor if null)*/
    TEXT type[],                 /* IN: mdf/proc/command */
    TEXT lefthead[]              /* OUT: string to be displayed  */
);

/* build left side of header for global help */
FUNCTION VOID left_gbld
(
    TEXT pdfname[],      /* in: the name of the pdf file         */
    TEXT varname[],      /* in: the name of the global variable          */
    TEXT type[],         /* in: global                           */
    TEXT lefthead[]      /* out: left side of the header         */
);

/* build left side of header for parameter help */
FUNCTION VOID left_pbld
(
    TEXT procname[],     /* in: the name of the proc             */
    TEXT subcname[],     /* in: the name of the subcommand       */
    TEXT parmname[],     /* in: the name of the parm             */
    TEXT type[],         /* in: proc/command                     */
    TEXT lefthead[]      /* out: left side of the header         */
);

/* build left side of header for subcommand help */
FUNCTION VOID left_sbld
(
    TEXT procname[],     /* in: the name of the proc             */
    TEXT subcname[],     /* in: the name of the subcommand       */
    TEXT type[],         /* in: proc/command                     */
    TEXT lefthead[]      /* out: left side of the header         */
);

/* exact search of local, parameters, and references */
FUNCTION struct VARIABLE *search 
(
    TEXT          name[],         /* in:  variable name           */
    struct CONTXT *pctx           /* in:  proc context            */
);

/* same as search, but variable pointer is not resolved */
FUNCTION struct VARIABLE *usearch 
(
    TEXT          name[],         /* in:  variable name           */
    struct CONTXT *pctx           /* in:  proc context            */
);

/* exact search of local, parameters, and references internal to current proc */
FUNCTION struct VARIABLE *int_search 
(
    TEXT          name[],         /* in:  variable name           */
    struct CONTXT *pctx           /* in:  proc context            */
);

/* add text to a TXTSTOR block */
FUNCTION CODE addtxt 
(
    struct TXTSTOR *block,         /* in/out: bloct to add a line of text to*/
    TEXT           str[]           /* in:  string to add                   */
);

/* break text up for display field width */
FUNCTION CODE brktxt 
(
    struct TXTSTOR *block,         /* in/out: text storage block           */
    FUNINT         width           /* in:  field width                     */
);

/* chop a TXTSTOR structure off at a max number of strings */
FUNCTION VOID choptxt 
(
    struct TXTSTOR *block,         /* in/out: block to chop off            */
    FUNINT         maxstr          /* in:  max # strings allowed in block  */
);

/* display right-justified, dynamically stored multi-line text starting at
   specified line */
FUNCTION VOID disprtxt 
(
    FUNINT line,           /* in:  display line # to start         */
    FUNINT rtcol,          /* in:  column number to right-justify against*/
    struct TXTSTOR *block  /* in:  storage block containing dynam stored text*/
);

/* display dynamically stored multi-line text starting at line and column */
FUNCTION VOID disptxt 
(
    FUNINT line,           /* in:  display line # to start         */
    FUNINT col,            /* in:  column number to start each line*/
    struct TXTSTOR *block  /* in:  storage block containing dynam stored text*/
);

/* free stored text from dynamic memory */
FUNCTION VOID fretxt 
(
    struct TXTSTOR *block
);

/* display right justified text */
FUNCTION VOID hdisprtxt 
(
    FUNINT         line,    /* in:  display line # to start         */
    FUNINT         rtcol,   /* in:  column number to right-justify against*/
    struct TXTSTOR *block,  /* in:  storage block containing dynam stored text*/
    FUNINT         width,   /* in: width of field                   */
    BOOL           highlight  /* in: TRUE to highlight; FALSE to      */
                              /*     turn highlight off               */
);

/* display left justified text */
FUNCTION VOID hdisptxt 
(
    FUNINT line,           /* in:  display line # to start         */
    FUNINT col,            /* in:  column number to start each line*/
    struct TXTSTOR *block, /* in:  storage block containing dynam stored text*/
    FUNINT width,          /* in: width of field                   */
    BOOL   highlight       /* in: TRUE to highlight; FALSE         */
                           /*     to turn highlight off            */
);

/* initialize a TXTSTOR block */
FUNCTION VOID initxt 
(
    struct TXTSTOR *block          /* in/out:  block to initialize            */
);

/* read and check a record of a par file */
FUNCTION CODE parhdrRead 
(
    struct SFILE  *sfile,         /* in: opened SFILE     */
    struct PARHDR *parhdr,        /* out: buffer to receive hdr */
    TEXT          errmsg[STRINGSIZ+1],
    TEXT          errkey[STRINGSIZ+1] 
);

/* process endif statement */
FUNCTION CODE endif_do 
(
    struct CONTXT *proctx, /* IN/OUT: Stack to access */
    struct CONTXT *cmdctx  /* IF command line context (w/ syntax block) */
);

/* true if the current proc statement is on the true side of if/then */
FUNCTION BOOL true_part 
(
    struct CONTXT *pctx  /* in: proc context                     */
);

/* display help on a mdf entry */
FUNCTION  CODE  entr_help
(
    struct SFILE   *fctx,          /* IN: SFILE of file with entries   */
    TEXT           libname[],      /* IN: library name for file        */
    TEXT           filename[],     /* IN: name of file with entries    */
    struct HELPBLK *helpblk        /* IN/OUT: help control block       */
);

/* position to and display general (i.e. .HELP) data */
FUNCTION CODE genhelp
(
    struct SFILE   *fctx,          /* In: file ctx block for opened file */
    TEXT           libname[FLIBRSIZ+1],    /* library where file exists  */
    TEXT           type[8],        /* In: menu/gen                       */
    TEXT           lefthead[],     /* In: left side string of help disp  */
    struct HELPBLK *helpblk        /* In/out: help output control block  */
);

/* process help file text and handle help displays */
FUNCTION CODE helper
(
    struct DIRBLK  *dirblk,    /* in: directive control block pointing */
                               /*     to start of help text            */
    TEXT           type[],     /* in: menu/proc/command/parm/msg/gen   */
    TEXT           lefthead[], /* in: string to follow "HELP DISPLAY: " */
                               /*     in the help display header       */
    struct TXTSTOR *titlept,   /* in: pointer to title string. */
                               /*     null if no title or null title   */
    struct HELPBLK *helpblk    /* out: help control block              */
);

/* read a line to see if it's EOT */
FUNCTION CODE lookahead
(
    struct DIRBLK *dirblk   /* in: directive control block  */
);

/* help on a message key */
FUNCTION CODE msg_help
(
    struct SFILE   *fctx,       /* In: file context block for opened file */
    TEXT           msgkey[],    /* In: message key              */
    struct HELPBLK *helpblk     /* Out: help output control blk */
);

/* open a help file */
FUNCTION CODE opnhlp 
(
    struct SFILE    *fctx,                  /* out: opened help file context */
    TEXT            procstr[],              /* in/out: proc(-sub) name      */
    TEXT            proc[],                 /* out: proc name to help on    */
    TEXT            sub[],                  /* out: sub name to help on     */
    TEXT            library[],              /* out: library name            */
    FUNINT          *itrinsic,              /* out: TRUE if intrinsic cmd   */
    struct  HELPBLK *helpblk                /* out: help output control blk */
);

/* detailed help on a parameter */
FUNCTION CODE parm_help
(
    struct VARIABLE *v,           /* in: VARIABLE struct for the parameter */
    FUNINT          infile,       /* in: TRUE if parm help text is in file */
    struct DIRBLK   *dirblk,      /* in: directive control block pointing */
                                  /*     to start of help text            */
    TEXT            lefthead[],   /* in: string to follow "HELP DISPLAY: " */
                                  /*     in the help display header       */
    struct TXTSTOR  *titlept,     /* in: pointer to title string.         */
                                  /*     null if no title or null title   */
    struct HELPBLK  *helpblk      /* out: help control block              */
);

/* process and display an opened help file */
FUNCTION CODE proc_help
(
    struct SFILE   *fctx,         /* In: file context block for opened file */
    TEXT           curlib[],      /* In: library name of opened file      */
    TEXT           libname[],     /* In: library name for LHS of header    */
    TEXT           procname[],    /* In: proc name for LHS of help display */
    TEXT           subname[],     /* In: name of subcommand, or NULL if none */
    TEXT           parmname[],    /* In/out: name of parm, or NULL if none */
    TEXT           type[],        /* In: proc/menu/command/global */
    struct HELPBLK *helpblk       /* out: help output control block    */
);

/* intrinsic initialization function */
FUNCTION CODE intrin_init (void);

/* find command in intrinsic tables */
FUNCTION struct ITRCMD *intrin
(
    TEXT *cmd               /* in: command */
);

/* locates ITRCMD structure given a subcommand and the pointer to the first
   ITRCMD sturcture for a command */
FUNCTION struct ITRCMD *itrsub
(
    TEXT          subcmd[], /* in: subcommand       */
    struct ITRCMD *p        /* in: first ITRCMD     */
);

/* lookup intrinsic command-subcommand */
FUNCTION CODE itr_lookup 
(
    TEXT          cmd[],    /* in: command          */
    TEXT          subcmd[], /* in: subcommand       */
    struct ITRCMD **itrcmd  /* out: entry pointer   */
);

/* process a label found in a PDF */
FUNCTION VOID chk_label
(
    struct CONTXT *proctx   /* IN/OUT: Current context block */
);

/* deallocate memory associated with all iteration control blocks for proc */
FUNCTION VOID cleanlp
(
    GENPTR ptr              /*IN:OUT: Pointer to ICB link list*/
);

/* deallocate memory with an iteration control block */
FUNCTION VOID delicb
(
    struct ICBLOCK *icb     /*IN:OUT: Pointer to ICB */
);

/* process END-FOR command */
FUNCTION CODE endfor_do
(
    struct CONTXT *proctx,  /* IN/OUT: Current context block */
    struct CONTXT *cmdctx   /* IN/OUT: New context block */
);

/* process END-LOOP command */
FUNCTION CODE endloop_do
(
    struct CONTXT *proctx, /* IN/OUT: Current context block */
    struct CONTXT *cmdctx  /* I/O: New context block (not used by endloop_do) */
);

/* process FOR command */
FUNCTION CODE for_do
(
    struct CONTXT *proctx,  /* IN/OUT: Current context block */
    struct CONTXT *cmdctx   /* IN/OUT: New context block */
);

/* process LOOP command */
FUNCTION CODE loop_do 
(
    struct CONTXT *proctx, /* IN/OUT: Current context block */
    struct CONTXT *cmdctx  /* IN/OUT: New context block (not used by loop_do) */
);

/* process END-GLOBAL statement */
FUNCTION CODE endgbl_do 
(
    struct CONTXT *cpctx,         /* in:  context of containing proc      */
    struct CONTXT *npctx          /* in:  proc ctx for END-GLOBAL command */
);

/* process END-PROC statement */
FUNCTION CODE endproc_do
(
    struct CONTXT *cpctx,   /* in/out: context of containing proc   */
    struct CONTXT *npctx    /* in:  ctx from END-PROC cmd (null)    */
);

/* process END-SUB command */
FUNCTION CODE endsub_do 
(
    struct CONTXT *cpctx,   /* in/out: context of containing proc   */
    struct CONTXT *npctx    /* in:  ctx from END-SUB cmd (null)     */
);

FUNCTION VOID put_stdout
(
    TEXT record[]           /* in: record to put            */
);

/* change the standard output for a new process or procedure */
FUNCTION CODE chg_stdout
(
    struct CONTXT *procctx  /* in: current proc context     */
);

/* list missing parameter names */
FUNCTION CODE list_missing
(
    struct SYMTAB *symtab   /* in: pointer to symbol table  */
);

/* compare variables in two symtabs */
FUNCTION CODE parcmp 
(
    struct SYMTAB *st1,           /* in:  1st symbol table                */
    struct SYMTAB *st2,           /* in:  2st symbol table                */
    TEXT          errmsg[STRINGSIZ+1] /* out: error msg string        */
);

/* merge parameters from st1 into st2 */
FUNCTION CODE parmrg 
(
    struct SYMTAB *st1,     /* in:  1st symbol table                */
    struct SYMTAB *st2,     /* in/out: sym tab to merge into        */
    FUNINT        mode      /* in:  FULLSET or SUBSET VM_VALID may be 
                               or'd here to request VALID merging */
);

/* position for scroll mode */
FUNCTION VOID pos_scroll(void);

/* reset stdout */
FUNCTION CODE re_stdout
(
    struct CONTXT *cmdctx   /* in: just completed 'new' context     */
);

/* perform parameter block restore */
FUNCTION CODE rest_parm
(
    TEXT          restspec[],   /* in:  restore file spec or null       */
    struct CONTXT *pctx,        /* in/out:  proc context                */
    TEXT          errmsg[]      /* out: returned error message if fail  */
);

/* save parameter block to disk */
FUNCTION CODE save_parm 
(
    TEXT          savespec[],   /* in:  save file spec          */
    struct CONTXT *pctx,        /* in:  proc context            */
    CODE          access,       /* in:  F_WRITE or F_OVER       */
                                /* (F_OVER used for SAVE LAST)  */
    TEXT          errmsg[]      /* out: error message if FAIL   */
);

/* return TRUE if string value vector contains certain string */
FUNCTION BOOL search_vector 
(
    TEXT   *string_vv[],    /* in: string value vector */
    FUNINT count,           /* in: number of strings   */
    TEXT   search_string[]  /* in: string to search for */
);

/* update parameter qualifiers from command line */
FUNCTION CODE upd_p_qlf 
(
    struct SYNBLK   *sb,    /* in/out: command line syntax block    */
    struct VARIABLE *v      /* in/out: parm which the quals apply to*/
);

/* set the _STDOUT local variable value */
FUNCTION CODE set_stdout 
(
    struct CONTXT *procctx, /* in: outgoing proc context            */
    struct CONTXT *cmdctx   /* in/out: new proc context             */
);

/* perform hierarchy search on specified file */
FUNCTION CODE hierarchy
(
    struct SFILE *sfile,        /* out: SFILE to use for open           */
    TEXT verbstring[],          /* in: command string: may be filespec  */
    TEXT type[],                /* in: default file type                */
    FUNINT lun,                 /* in: lun to use for file open         */
    struct FSBLOCK *fsblock,    /* out: FSBLOCK for PDF where found     */
    TEXT errstr[STRINGSIZ+1]    /* out: 'at or near' string             */
);

/* report memory overflow error */
FUNCTION VOID overr (void);

/* put a message on the output device */
FUNCTION CODE put_outmsg
(
    TEXT string[]           /* in: text string  to output    */
);

/* report TM internal error */
FUNCTION VOID tmierr 
(
    FUNINT number           /* in:  error number            */
);

/* writes message to output devices. accepts va_list argument */
FUNCTION VOID m_put_valist 
(
    TEXT control[],         /* in: control string           */
    TEXT key[],             /* in: message key              */
    va_list arglist
);

/* writes message to output devices */
FUNCTION VOID m_put 
(
    TEXT control[],         /* in: control string           */
    TEXT key[],             /* in: message key              */
    ...
);

/* write message to standard CRT output.  accepts va_list argument */
FUNCTION VOID m_cput_valist
(
    FUNINT line,            /* in: line number to write message  */
    TEXT   control[],       /* in: control string           */
    TEXT   key[],           /* in: message key              */
    va_list arglist
);

/* Writes message to standard CRT output */
FUNCTION VOID m_cput 
(
    FUNINT line,            /* in: line number to write message  */
    TEXT   control[],       /* in: control string           */
    TEXT   key[],           /* in: message key              */
    ...
);

/* Standard TAE monitor message function. Accepts valist argument  */
FUNCTION VOID tmmsg_valist 
(
    FUNINT scode,          /* in:  status code to set              */
    TEXT   msg[],          /* in:  message control text            */
    TEXT   key[],          /* in:  message key                     */
    va_list arglist
);

/* Standard TAE monitor message function */
FUNCTION VOID tmmsg 
(
    FUNINT scode,          /* in:  status code to set              */
    TEXT   msg[],          /* in:  message control text            */
    TEXT   key[],          /* in:  message key                     */
    ...
);

/* host dependent initialization */
FUNCTION VOID host_init (void);

/* set the signal handler */
FUNCTION CODE set_signal
(
    FUNINT signum,        /* signal number         */
    void (*sig_hand)(int) /* signal handler function */
);

/* receive a message from async task */
FUNCTION CODE asy_recs
(
    struct TCB *tcb,        /* in: task control block */
    void       *block,      /* in: address of receive block      */
    FUNINT     blksize,     /* in: number of bytes expected      */
    VOID       (*actadr)(), /* in: address of an action routine  */
    GENPTR     actparm      /* in: action routine parm (ignored) */
);

/* abort the monitor associated with an acb and release resources */
FUNCTION VOID abort_acb
(
    struct ACB *acb         /* in: the acb              */
);

/* close a proc and save the position */
FUNCTION VOID clssav 
(
    struct CONTXT *pctx     /* in/out: proc context */
);

/* get parameter values for a parblk from parent monitor */
FUNCTION CODE get_pblk_remote 
(
    struct CONTXT   *proc_ctx,    /* in: Context of executing proc */
    struct PARBLK   *parblk,      /* in/out: parameter block */
    struct CODE_ERR *(*errmsg)    /* out: pointer to err msg */
);

/* get one PDF statement */
FUNCTION CODE getpst 
(
    struct SFILE  *pdffil,        /* in/out: PDF file context */
    TEXT          sttmt[],        /* out: PDF statement */
    COUNT         *records,       /* out: number of records read */
    COUNT         *num_prec,      /* out: # comments, null lines */
                                  /*      preceding 1st real sttmt rec*/
    struct POSCTX *first_rec      /* out: position of 1st record */
);

/* initialize $SFI and $SKEY */
FUNCTION VOID ini_status (void);

/* handle TAE interrupt mode processing */
FUNCTION CODE interrupt 
(
    struct CONTXT *ctx      /* in: context of executing proc */
);

/* deep copy a value from source to destination */
FUNCTION CODE deep_value_copy 
(
    struct VARIABLE *sv,        /* in: source variable */
    struct VARIABLE *dv,        /* in: dest variable */
    TEXT            sname[],    /* in: source name */
    TEXT            dname[]     /* in: dest name */
);

/* set output values from a process */
FUNCTION CODE outputs
(
    struct CONTXT *procctx, /* in: proc context */
    struct PARBLK *parblk   /* in: PARBLK from process */
);

/* parse statement and build symbol table for this statement */
FUNCTION CODE prsstt 
(
    TEXT          sttmt[CMDLINSIZ+1], /* in/out:  PDF statement*/
    struct ITRCMD **itrcmd,           /* out: ITRCMD for cmd */
    struct SYMTAB *pdfstb             /* out: cmd symbol table */
);

/* find a PDF and open it */
FUNCTION CODE opnpdf 
(
    struct CONTXT *parctx,     /* in: parent proc context */
    struct CONTXT *pctx,       /* in/out: proc context */
    TEXT          cmdstring[], /* in: command string:may be a file spec */
    struct SFILE  *file        /* out: the file control block */
);

/* open (or reopen) a proc and position to the previously saved position */
FUNCTION CODE opnsav 
(
    struct CONTXT *pctx     /* in/out: proc context */
);

/* build symbol table from PDF */
FUNCTION CODE pdftab 
(
    struct CONTXT *pctx,    /* in:  proc context */
    struct SFILE  *file     /* in/out: PDF file control block */
);

/* run a proc */
FUNCTION CODE run_proc 
(
    struct CONTXT *pctx     /* in:  proc context */
);

/* work toward building a full TAE command string */
FUNCTION CODE bldcmd 
(
    TEXT inbuf[],           /* in/out: single input line buffer     */
    TEXT cmdstr[]           /* in/out: accumulated cmd string       */
);

/* check the environment of an intrinsic command against bits in the ITRCMD */
FUNCTION CODE chk_do
(
    struct CONTXT *ctx,     /* in: current proc context     */
    struct ITRCMD *itrcmd   /* in: ptr to ITRCMD entry      */
);

/* clear latching */
FUNCTION VOID clr_latch 
(
    struct CONTXT *pctx     /* in:  proc context */
);

/* close a proc context */
FUNCTION VOID clsctx 
(
    struct CONTXT *pctx     /* in:  proc context            */
);

/* get command line */
FUNCTION CODE getcln 
(
    TEXT prompt[],          /* in: prompt (without the '>') */
    TEXT cmdstr[]           /* out: command string          */
);

/* save command string in $LASCMD global */
FUNCTION VOID history_save
(
    TEXT cmdstr[]           /* in: string at least MAXSTRSIZ */
);

/* initialize proc context block */
FUNCTION CODE inictx 
(
    struct CONTXT *pctx     /* in/out: proc context block           */
);

/* build a symbol table from a table of resident variables */
FUNCTION CODE memtab 
(
    struct RESIDVAR *resvar,        /* table of resident variables          */
    FUNINT          numres,         /* number of resident variables         */
    struct SYMTAB   *s              /* out: symbol table header             */
);

/* return list missing parameters */
FUNCTION COUNT misprm 
(
    struct SYMTAB *symtab,            /* in:  symbol table            */
    TEXT          mislst[STRINGSIZ+1] /* out: list of missing */
);

/* proc local symbol table initialization */
FUNCTION CODE plcini 
(
    struct CONTXT *pctx     /* in/out:  proc context        */
);

/* process TAE command mode command string */
FUNCTION CODE prccmd 
(
    TEXT   cmdstr[CMDLINSIZ+1],   /* in/out:  command string */
    struct CONTXT  *procctx       /* in:  current proc context */
);

/* replace current value for a variable */
FUNCTION CODE repval 
(
    struct VARIABLE *v,     /* in/out: symbol table entry           */
    TEXT   *value[],        /* in:  val str ptrs (vals to repl with)*/
    FUNINT count,           /* in:  count of values for this parm   */
    FUNINT by_name          /* in: TRUE if value[0] is the name of  */
                            /* a referenced variable                */
);

/* perform restore qualifier operation */
FUNCTION CODE restore 
(
    struct CONTXT *ctx      /* in/out: command context      */
);

/* classify the runtype qualifier */
FUNCTION CODE runtype 
(
    struct SYMTAB *qualst   /* in: qualifier symtab */
);

/* perform save qualifier operation */
FUNCTION CODE save 
(
    struct CONTXT *ctx      /* in/out: command context      */
);

/* update a symbol table from a command mode command line */
FUNCTION CODE updtab 
(
    struct SYMTAB *symtab,  /* in/out: symbol table */
    struct SYNBLK *sb       /* in/out:  syntax block */
);

/* obtain label, command, and subcommand for a TCL command line */
FUNCTION CODE cmd_parse 
(
    struct SYNBLK *sb,                    /* out: syntax block    */
    TEXT          cmdstr[CMDLINSIZ+1],    /* in/out: command str  */
    TEXT          *label,                 /* out: command label   */
    TEXT          cmd[FSPECSIZ+1],        /* out: command         */
    TEXT          subcmd[SUBCMDSIZ+1]     /* out: subcommand      */
);

/* do command line substitution in place */
FUNCTION CODE substitute 
(
    TEXT          cmdstr[],  /* [siz] */  /* in/out: command string*/
    FUNINT        siz,                    /* in:  dim-1 of cmdstr  */
    struct CONTXT *pctx,                  /* in:  proc context     */
    BOOL          report                  /* in:  true if err msg report  */
                                          /*      is desired              */
);

/* check if the input command matches with the defined command in the 
   given command string */
FUNCTION CODE cmdmatch
(
    TEXT        incmd[],    /* in: input command                */
    struct VARIABLE **defv, /* out: $DEFCMD VARIABLE of match   */
    COUNT       *index      /* out: index to matching string    */
);

/* build SUBCMD structure */
FUNCTION struct SUBCMD *allsub 
(
    struct SUBCMD **subptr  /* in/out: ptr to 1st SUBCMD struct */
);

/* delete SUBCMD chain */
FUNCTION VOID delsub 
(
    struct SUBCMD **subptr  /* in/out: ptr to 1st SUBCMD struct     */
);

/* find name in a SUBCMD chain using exact match */
FUNCTION struct SUBCMD *subex 
(
    struct SUBCMD *subptr,  /* in:  1st struct in chain to search   */
    TEXT          name[]    /* in:  name to find                    */
);

/* lookup name in SUBCMD chain allowing abbreviations */
FUNCTION CODE subab 
(
    struct SUBCMD *subptr,  /* in:  1st struct in SUBCMD chain      */
    TEXT          name[],   /* in:  name of subcommand to find      */
    struct SUBCMD **subc    /* out: SUBCMD found (NULL unless SUCCESS)*/
);

/* delete a global variable */
FUNCTION CODE del_gbl
(
    struct SYMTAB   *st,    /* in/out: gbl symbol table   */
    struct VARIABLE *gv     /* in: ptr to variable in tbl */
);

/* delete a local variable */
FUNCTION CODE del_loc
(
    struct CONTXT   *cpc,   /* in/out: containing proc context */
    struct VARIABLE *vptr   /* in: ptr to variable in tbl */
);

/* get parameters from parent monitor */
FUNCTION CODE get_parms_remote 
(
    struct CONTXT *proc_ctx,/* in: Context of executing proc    */
    TEXT          **preface,/* in: preface for tutoring         */
    struct CONTXT *parctx,  /* in/out: parameter context        */
    FUNINT        vcount    /* in: number of preface lines      */
);

/* called after CONTXT has been set with default parameters */
FUNCTION CODE action_setup 
(
    struct CONTXT *ctx      /* in/out: current proc context */
);

/* called when user runs a proc */
FUNCTION CODE action_run 
(
    struct CONTXT *ctx      /* in/out: current proc context */
);

/* send a message in an asynchronous environment */
FUNCTION CODE asy_snds
(
    struct TCB *tcb,        /* in: task control block       */
    GENPTR     block,       /* in: data block to send       */
    FUNINT     blksize      /* in: number of bytes to send  */
);

/* update _STDOUT local variable for UNIX ASYNC-PROCESS */
FUNCTION VOID async_stdout 
(
    struct CONTXT *cmdctx,        /* context block for the process */
    TEXT          *stdout_string  /* in: STDOUT value or NULL if   */
                                  /* no STDOUT qualifier present   */
);

/* block operator interrupt */
FUNCTION VOID block_opint(void);

/* create async TAE monitor and initialize tcb */
FUNCTION CODE create_async 
(
    struct ATCB *atcb,      /* in/out: address of an atcb      */
    FUNINT      mb_size,    /* in: size to use for mailbox     */
    TEXT        procname[], /* in: name of proc to be run      */
    FUNINT      job_type,   /* in: ASYNC, ASYNC_PROCESS, REMOTE */
    TEXT        job_spec[], /* in: ASYNC or REMOTE: .job filespec */
                            /*     ASYNC_PROCESS: filespec of executable */
    TEXT        log_spec[]  /* in: ASYNC: output log file name */
                            /*     ASYNC_PROCESS: SYS$OUTPUT file */
);

/* write the host part of an ASYNC .job file */
FUNCTION int host_job 
(
    struct SFILE  *job,     /* in/out: SFILE opened to job file   */
    struct CONTXT *cmdctx,  /* in: command context                */
    FUNINT        user_proc /* in: TRUE if job file for user proc */
);

/* release system resources */
FUNCTION VOID rel_rsrc
(
    struct ATCB *atcb
);

/* asynch routine triggered when receive is complete */
FUNCTION VOID asy_rcv_done 
(
    struct ACB *acb         /* in: pointer to acb               */
);

/* execute ABORT-JOB */
FUNCTION CODE async_abo 
(
    struct CONTXT *procctx, /* in: proc context     */
    struct CONTXT *cmdctx   /* in: command context  */
);

/* exit processing for async jobs */
FUNCTION VOID async_exit (void);

/* sends the status code to the parent TM */
FUNCTION VOID emit_stat(void);

/* get parameters from local async parent */
FUNCTION CODE get_parm_locpar
(
    TEXT   send_file[],     /* in: name of parfile to send to tm  */
    TEXT   recv_file[],     /* out: name of pafile recved from tm */
    FUNINT req_process,     /* in: TRUE if request from a process */
    CODE   *send_stat,      /* out: host_code if error in sending */
    CODE   *recv_stat       /* out: host_code if error in receive */
);

/* return acb with the specified process id, or NULL */
FUNCTION struct ACB *find_id 
(
    FUNINT pid              /* in: job id */
);

/* return acb with the specified job name, or NULL */
FUNCTION struct ACB *find_job 
(
    TEXT name[]             /* in: job name */
);

/* returns a pointer to the 'oldest' waiting ACB, or NULL */
FUNCTION struct ACB *find_oldest(void);

/* get the rest of the monitor context for an async job */
FUNCTION CODE get_asyctx (void);

/* insert a string at a specified position on the provided record */
FUNCTION VOID insert 
(
    TEXT   *s,              /* in: string to insert         */
    TEXT   *rec,            /* in/out: record               */
    FUNINT width            /* in: max field width          */
);

/* restores variables after a dynamic encounter with the parent */
FUNCTION CODE get_syms 
(
    TEXT          save_file[],    /* in: name of file to restore */
    struct CONTXT *ctx            /* in/out: context with symbols */
);

/* initiate an asynchronous proc */
FUNCTION CODE run_async 
(
    struct CONTXT *procctx, /* in: context of invoking proc         */
    struct CONTXT *cmdctx   /* in: context of cmd line for new proc */
);

/* show SHOW-ASYNC and SHOW-WAITING */
FUNCTION CODE showasy 
(
    struct CONTXT *procctx, /* current context      */
    struct CONTXT *cmdctx   /* context of SHOW      */
);

/* log the execution of all PDFs run in TAE in a file in $APPLOG */
FUNCTION VOID pdflog 
(
    struct FSBLOCK *fsblock /* in: fsblock for the pdf            */
);

/* build TCL poriton of .job file for batch and async jobs */
FUNCTION CODE bld_tcl 
(
    struct CONTXT *cmdctx,  /* in: command context          */
    struct SFILE  *job,     /* in/out: SFILE for job        */
    FUNINT        run_type  /* in: type of job:             */
                            /*     ASYNC or BATCH           */
);

/* TM initialization exit */
FUNCTION VOID init_ins 
(
    struct CONTXT *primctx  /* in/out: primary level context  */
);

/* proc activation user exit */
FUNCTION VOID pa_ins 
(
    struct CONTXT *pctx     /* in/out: context of proc      */
);

/* proc termination exit */
FUNCTION VOID pt_ins 
(
    struct CONTXT *pctx     /* in/out: context of proc      */
);

/* dynamic message processing */
FUNCTION VOID dynamic_ins 
(
    struct CONTXT *pctx,    /* in/out: process CONTXT */
    struct PARBLK *parblk   /* in: PARBLK from process*/
);

/* future portability function */
FUNCTION CODE dp_ins 
(
    struct PARBLK *parblk,        /* in: parblk from process */
    struct SFILE  *pdf,           /* in: opened PDF          */
    struct CONTXT *ctx,           /* in/out: dynamic context */
    FUNINT        dash_present    /* in: TRUE if '-' present */
);

/* called for getpar command */
FUNCTION CODE gp_ins 
(
    struct SFILE  *pdf,     /* in: SFILE for opened pdf*/
    struct CONTXT *ctx      /* in/out: dynamic context */
);

/* apparently unused */
FUNCTION VOID term_ins(void);

/* called when implicit references are being added to a CONTXT block */
FUNCTION VOID refs_ins
(
    struct CONTXT *pctx     /* in/out: context block        */
);

/* installation exit useful to insert additional TAE commands into the job
   file before the invocation of the requested proc */
FUNCTION CODE job_ins 
(
    struct SFILE  *jobfile, /* in/out: already open job file*/
    struct CONTXT *cmdctx   /* in:  command context         */
);

/* another installation exit for job */
FUNCTION CODE job_ins1 
(
    struct SFILE  *jobfile, /* in/out: already open job file*/
    struct CONTXT *cmdctx   /* in:  command context         */
);

/* set process name for ASYNC and BATCH */
FUNCTION VOID set_process_name 
(
    CODE runtype,           /* BATCH or ASYNC */
    TEXT name[]             /* job name     */
);

/* restore local/global variables from saved file */
FUNCTION CODE restore_do
(
    struct CONTXT *cpctx,   /* current proc context           */
    struct CONTXT *npctx    /* 'restore' command line context */
);

/* check if a TCL variable is excluded from restore */
FUNCTION BOOL exclude
(
    struct VARIABLE *v      /* in: pointer to global variable */
);

/* save the specified variables into a standard save files */
FUNCTION CODE save_pfile 
(
    struct SFILE    *sf,          /* in: pointer to opened file context*/
    FUNINT          msgtype,      /* in: message type for p.msgtyp     */
    struct VARIABLE *vector[],    /* in: array of variables to save    */
    FUNINT          vcnt,         /* in: number of vars in vector      */
    FUNINT          scnt,         /* in: number of sym tabs following  */
    struct SYMTAB   **symtab      /* in: ptr to array of symtab ptrs   */
);

/* accumulate variables in a parblk, and write it when full */
FUNCTION CODE pblk_out 
(
    struct SFILE    *sf,            /* in: file context             */
    struct PARBLK   *par,           /* in/out: parblk               */
    FUNINT          poolsize,       /* bytes in parblk.pool         */
    struct VARIABLE *var            /* in: the variable to add      */
);

/* builds a PARBLK from context block */
FUNCTION CODE package
(
    struct CONTXT *ctx,     /* input: current context block */
    struct PARBLK *par,     /* output: parameter block      */
    FUNINT        pool_size,/* input: size of PARBLK.pool   */
    FUNINT        scope     /* input: VM_VALID or zero      */
);

/* build PARBLK for one symbol table only */
FUNCTION CODE pack_parm
(
    struct SYMTAB *symtab,  /* in: symbol table to use      */
    struct PARBLK *p,       /* out: PARBLK                  */
    FUNINT        pool_size /* in: pool space size          */
);

/* move VARIABLE structure to restricted storage and return its pointer */
FUNCTION struct VARIABLE *vmove
(
    struct VARIABLE *p,         /* input: VARIABLE struct to move       */
    ALIGN           *s,         /* input: restricted storage block      */
    FUNINT          specials    /* input: bits indicating special...    */
                                /*        parts of VARIABLE to move     */
);

/* write an error message, accepting a valist argument */
FUNCTION VOID wrterr_valist
(
    TEXT msg[],                  /* IN: error message to output */
    TEXT key[],                  /* IN: message key */
    va_list arglist
);

/* write an error message */
FUNCTION VOID wrterr
(
    TEXT msg[],                  /* IN: error message to output */
    TEXT key[],                  /* IN: message key */
    ...
);

/* display text on terminal */
FUNCTION VOID wrttxt
(
    FUNINT line,            /* IN: line number to write to */
    FUNINT col,             /* IN: column number to write to */
    TEXT   text[],          /* IN: message text */
    FUNINT blank            /* IN: True if rest to be blanked */
);

/* create predefined globals */
FUNCTION CODE ini_globals(void);

/* adjust the value panel top and bottom component numbers */
FUNCTION VOID adjval 
(
    struct VARIABLE *v,     /* in/out: var containing panel to be adjusted*/
    FUNINT          compnum /* in:  component # of new panel top    */
);

/* display a specific tutor page */
FUNCTION CODE dispbld 
(
    struct CONTXT  *pctx,   /* in/out: proc context                 */
    struct TXTSTOR *title,  /* in:  .TITLE text                     */
    struct SFILE   *hf      /* in/out: help file control block      */
);

/* get level 1 display page number of a proc parameter */
FUNCTION CODE getpgnum 
(
    struct CONTXT *pctx,          /* in/out: proc context to find parm in */
    TEXT          parmname[],     /* in:  proc parameter name             */
    COUNT         *pagnum,        /* out: display page number             */
    struct SFILE  *hf             /* in/out: help file control block      */
);

/* get level 1 display page number of a proc subcommand */
FUNCTION CODE getspnum 
(
    struct CONTXT *pctx,    /* in/out: proc context to find subc in */
    TEXT          subname[],/* in:  subcmd name                     */
    COUNT         *pagnum,  /* out: display page number             */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* highlight or unhighlight a parameter name and one component's value */
FUNCTION VOID high_value 
(
    struct VARIABLE *v,           /* in: parameter to highlight   */
    FUNINT          index,        /* in: index to highlight       */
    BOOL            highlight     /* in: TRUE to highlight;       */
                                  /*     FALSE to turn off        */
);

/* display level 1 help for a parm */
FUNCTION VOID h1dsp_parm 
(
    struct CONTXT   *ctx,   /* in:  proc context                    */
    struct VARIABLE *v,     /* variable to display help on          */
    struct SFILE    *hf     /* in:  help file                       */
);

/* display level 1 help for a subcommand */
FUNCTION COUNT h1dsp_subc 
(
    struct CONTXT *ctx,     /* in:  proc context             */
    struct SUBCMD *s,       /* subcommand to display help on */
    struct SFILE  *hf       /* in:  help file                */
);

/* get the first parameter of a given page */
FUNCTION struct VARIABLE *get_parm1 
(
    struct CONTXT *pctx,    /* in: proc context             */
    FUNINT        page,     /* in: page number              */
    struct SFILE  *hf       /* in: help file                */
);

/* initialize tutor value panel position array */
FUNCTION CODE inipan 
(
    struct VARIABLE *v      /* in/out: variable containing the panel*/
);

/* list a parm and its value */
FUNCTION VOID list_parm 
(
    struct VARIABLE *v      /* in:  variable to list                */
);

/* display caller-supplied interface */
FUNCTION VOID list_preface 
(
    struct VARIABLE *p      /* in: strings to display       */
);

/* list a subcommand and whether or not it's the default */
FUNCTION VOID list_subc 
(
    struct SUBCMD *s        /* in:  subcommand to list              */
);

/* list parameter names for NOSCREEN tutor */
FUNCTION VOID listpnam 
(
    struct SYMTAB *symtab   /* in:  symbol table containing parms   */
);

/* list names of qualifiers desired for a particular parameter */
FUNCTION VOID listqnam 
(
    struct SYMTAB *symtab,        /* in:  symbol table containing quals   */
    TEXT          parm_name[]     /* in:  name of parm being qualified    */
);

/* list subcommand names for NOSCREEN tutor */
FUNCTION VOID listsnam 
(
    struct SUBCMD *subptr   /* in:  subcommand chain                */
);

/* return the max number of lines required for the components of one parm
   value in a tutor display */
FUNCTION COUNT nlinval 
(
    struct VARIABLE *v      /* in:  variable to fin # lines for     */
);

/* display a value component */
FUNCTION VOID dispcomp 
(
    struct VARIABLE *v,     /* in: the variable to display         */
    FUNINT          compnum,/* in: subscr (starting with 1) of the component*/
    FUNINT          fillcomp/*in: lowest #d comp that was filled (0 = no fill)*/
);

/* display value components for this parameter on tutor display */
FUNCTION VOID dispval 
(
    struct VARIABLE *v      /* in:  variable to display value of    */
);

/* display an error message to the proper display */
FUNCTION VOID tutmsg 
(
    TEXT msg[],
    TEXT key[],
    ...
);

FUNCTION CODE tutaccept 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform NOSCREEN tutor DISPLAY command */
FUNCTION CODE tutdisplay 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor mode EXIT command */
FUNCTION CODE tutexit 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor help mode */
FUNCTION CODE tuthlp 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor mode HOLD command */
FUNCTION CODE tuthold 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform NOSCREEN tutor LIST command */
FUNCTION CODE tutlist 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor mode HELP on last message key */
FUNCTION CODE tutmsghlp 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* NOSCREEN command */
FUNCTION CODE tutnoscreen 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor mode PAGE command */
FUNCTION CODE tutpage 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor mode parm= command */
FUNCTION CODE tutprmeq 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perfrom tutor QUALIFY command */
FUNCTION CODE tutqual 
(
    TEXT          cmdstr[],       /* in:  command string                  */
    TEXT          cmdprm[],       /* in/out:  the command parameter if any*/
    struct CONTXT *pctx,          /* in:  proc context                    */
    struct SFILE  *hf             /* in/out: help file control block      */
);

/* perform tutor mode RESTORE command */
FUNCTION CODE tutrstr 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor mode RUN command */
FUNCTION CODE tutrun 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor mode SAVE command */
FUNCTION CODE tutsav 
(
    TEXT          cmdstr[], /* in:  command string          */
    TEXT          cmdprm[], /* in:  the command parameter   */
    struct CONTXT *pctx     /* in:  proc context            */
);

/* SCREEN command */
FUNCTION CODE tutscreen 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor mode command that handles <CR> */
FUNCTION CODE tutscrol 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform the SET command */
FUNCTION CODE tutset 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor show command */
FUNCTION CODE tutshow 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in/out:  the command parameter if any*/
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* perform tutor subcommand mode "-subcommand" command */
FUNCTION CODE tutsubeq 
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* deactivate statics so that tutor can be used again */
FUNCTION VOID cls_tutor 
(
    struct SFILE  *pdf,     /* in/out: sfile of pdf */
    struct CONTXT *context  /* in/out: proc context */
);

/* initialize tutor module so sub_tutor and tutor may be called */
FUNCTION CODE opn_tutor 
(
    struct SFILE    *pdf,           /* in/out: sfile opened to the pdf  */
    struct CONTXT   *context,       /* in: context as set by opnpdf     */
    FUNINT          screen_select,  /* in: FORCE_SCREEN, FORCE_NOSCREEN,*/
                                    /* or NOFORCE                       */
    struct VARIABLE *vpreface,      /* in: preface strings (or NULL)    */
    FUNINT          dynamic         /* TRUE if dynamic tutor            */
);

/* perform parameter tutoring */
FUNCTION CODE tutor 
(
    struct SFILE  *pdf,     /* in/out: opened pdf           */
    struct CONTXT *context  /* in/out: context block        */
);

/* get parm name and subscript from a tutor mode parm= command */
FUNCTION CODE gettpnam
(
    struct SYNBLK *sb,            /* in/out: syntax block                 */
    TEXT          name[NAMESIZ+1],/* out: parm name                       */
    COUNT         *subscr         /* out: subscript on parm               */
);

/* get subcommand name from a tutor mode "-subcommand" command */
FUNCTION CODE gettsnam 
(
    struct SYNBLK *sb,              /* in/out: syntax block         */
    TEXT          name[SUBCMDSIZ+1] /* out: subcommand name         */
);

/* get variable name list for NOSCREEN tutor LIST or DISPLAY command */
FUNCTION CODE gettvnam 
(
    TEXT  cmdstr[],         /* in:  command string incl verb        */
    TEXT  *varname[MAXVAL], /* out: variable name list      */
    COUNT *numvar           /* out: number of variable names in list*/
);

/* save message and key in held_msg, held_key */
FUNCTION VOID hold_msg 
(
    TEXT msg[],          /* in: control string           */
    TEXT key[],          /* in: message key              */
    ...
);

/* get and execute a parameter qualifier tutor display command */
FUNCTION CODE q_get_exec 
(
    struct CONTXT *ctx      /* in/out: child proc context   */
);

/* add a value component's text to a TXTSTOR structure */
FUNCTION CODE addcomp 
(
    struct TXTSTOR  *txt,   /* in/out: struct to add text to        */
    struct VARIABLE *v,     /* in:  variable containing the value   */
    FUNINT          compnum /* in:  component number                */
);

/* clear multi-lines on display */
FUNCTION VOID clrmlin 
(
    FUNINT line,            /* in:  display line # at which to start clear*/
    FUNINT col,             /* in:  display column # at which to start clear*/
    FUNINT numlin           /* in:  number of lines to clear        */
);

/* display tutor field labels */
FUNCTION VOID dispflab 
(
    FUNINT line,        /* in:  screen line number              */
    FUNINT subtut,      /* in:  TRUE if tutoring on subc FALSE if on parms*/
    FUNINT qualtut      /* in:  TRUE if tutoring on parm quals FALSE otherwise*/
);

/* enclose a set of values in parenthesis */
FUNCTION CODE parenval 
(
    struct SYNBLK *sb       /* in/out: syntax block                 */
);

/* reset initial values for one symbol table */
FUNCTION CODE rsetini 
(
    struct SYMTAB *st       /* in/out: symbol table.*/
);

/* text read for tutor */
FUNCTION CODE txtread 
(
    struct DIRBLK *db,      /* in/out: directive block (for d_ pkg) */
    TEXT          str[],    /* out: text string read                */
    struct POSCTX *posctx   /* out: the saved position context      */
);

/* read in help level 1 data for a subcommand and for the remaining subcommands
   on this tutor page */
FUNCTION CODE bldshlp 
(
    struct CONTXT  *pctx,   /* in/out: all the parameters           */
    struct SUBCMD  *s,      /* in/out: the 1st subcommand of this page*/
    FUNINT         lsubpg,  /* in:  page # of last page built       */
    struct TXTSTOR *title,  /* in:  title text (just to know # lines)*/
    struct SFILE   *hf      /* in/out: help file control block      */
);

/* read in help level 1 data for a parm and for the remaining parms in this
   tutor page */
FUNCTION CODE bldvhlp 
(
    struct CONTXT   *pctx,  /* in/out: all the parameters           */
    struct VARIABLE *v,     /* in/out: the 1st variable of this page*/
    FUNINT          lvarpg, /* in:  page # of last page built       */
    struct TXTSTOR  *title, /* in:  title text (just to know # lines)*/
    struct SFILE    *hf     /* in/out: help file control block      */
);

/* input level 2 help text for a parm or subcommand */
FUNCTION CODE inlevel2 
(
    struct CONTXT *pctx,    /* in/out: proc context w/ tab to update.*/
    struct SFILE  *hf,      /* in/out: help file control block      */
    TEXT          type[],   /* in: subcmd/parm                      */
    TEXT          subcmd[]  /* in: subcommand qualifier to search   */
);

/* open .HELP file for tutor and return .TITLE text. on return positioned
   at first line of .LEVEL1 text. */
FUNCTION CODE tutohlp 
(
    struct SFILE   *pdf,    /* in:  PDF                             */
    struct CONTXT  *pctx,   /* in:  proc context                    */
    struct TXTSTOR *title,  /* out: block of .TITLE text (in dyn mem)*/
    struct SFILE   **ohf    /* out: help file control block */
);

/* initial function called for tutor mode */
FUNCTION CODE tutselect
(
    TEXT          cmdstr[], /* in:  command string                  */
    TEXT          cmdprm[], /* in:  the command parameter if any    */
    struct CONTXT *pctx,    /* in:  proc context                    */
    struct SFILE  *hf       /* in/out: help file control block      */
);

/* set the qualifiers */
FUNCTION CODE setqlf 
(
    FUNINT        intrin,   /* true if intrinsic            */
    struct SYNBLK *sb,      /* in/out: syntax block (command stream)*/
    struct SYMTAB *st       /* out:  qualifier symbol table         */
);

/* create input mailbox so the RECVAR command can be used to receive PARBLKS */
FUNCTION CODE enable_recvar 
(
    struct CONTXT *proc_ctx,/* in/out: proc context */
    struct CONTXT *cmd_ctx  /* in: SENDVAR context  */
);

/* move a symbol table */
FUNCTION CODE movest 
(
    struct SYMTAB *st1,     /* in:  symbol table to move from */
    struct SYMTAB *st2      /* out: symbol table to move to */
);

/* disable RECVAR command by deleting TM's mailbox */
FUNCTION CODE disable_recvar (void);

/* send a PARBLK to the task identified by the JOB parameter */
FUNCTION CODE sendvar_do 
(
    struct CONTXT *proc_ctx,/* in/out: proc context */
    struct CONTXT *cmd_ctx  /* in: SENDVAR context  */
);

/* receive TCL variables from another task */
FUNCTION CODE recvar_do 
(
    struct CONTXT *proc_ctx,/* in/out: proc context */
    struct CONTXT *cmd_ctx  /* in: SENDVAR context  */
);

/* session log proc activation information */
FUNCTION VOID slproc
(
    struct CONTXT *context          /* in: context block for the proc */
);

/* log current time to session log file */
FUNCTION VOID sltime(void);

/* parameter formatting function */
FUNCTION VOID slparm
(
    TEXT          prefix[], /* record prefix           */
    struct SYMTAB *symtab   /* symbol table address    */
);

/* log proc termination */
FUNCTION VOID slterm(void);

/* write data to session log file */
FUNCTION VOID slwrite
(
    TEXT prefix[],          /* IN: prefix to line   */
    TEXT line[]             /* IN: line to log      */
);

/* log message from subprocess */
FUNCTION VOID logmsg
(
    struct CONTXT *ctx,     /* in: context of executing process */
    struct PARBLK *parblk   /* in: parblk from subprocess   */
);

/* log parameters for dynamic command processing */
FUNCTION VOID logdyn
(
    struct PARBLK *parblk   /* in: parblk from subprocess   */
);

/* copy VARIABLE structure to new allocated storage and return its pointer */
FUNCTION CODE specvcopy
(
    struct VARIABLE *vin,   /* input: VARIABLE struct to move       */
    struct VARIABLE *vout   /* input: VARIABLE struct to move to    */
);

/* get interactive command string in no-screen mode */
FUNCTION VOID cmd_noscreen 
(
    CODE archive,           /* in: A_MENU, A_TUTOR, A_LASTCMD*/
    TEXT pmtbuf[],          /* in:  prompt buffer           */
    TEXT cmdstr[]           /* out: command string          */
);

/* get an interactive command string in screen mode */
FUNCTION CODE cmd_screen 
(
    CODE archive,           /* in: A_MENU, A_TUTOR, A_LASTCMD */
    TEXT cmdstr[],          /* out: command string            */
    CODE *terminat          /* command line terminator        */
);

/* get the next line from script file */
FUNCTION CODE getscr
(
    TEXT   line[],          /* OUT: record from script file*/
    CODE   *term,           /* OUT: terminator character */
    FUNINT screen           /* IN: true if screen mode   */
);

/* get command parameter from a command string */
FUNCTION CODE intprm 
(
    FUNINT maxparm,         /* in: max number of parameters (0/1) */
    TEXT   cmdnm[],         /* in: command name */
    TEXT   cmdstr[],        /* in:  the command string      */
    TEXT   cmdprm[],        /* out: the command parameter   */
    TEXT   msg[],           /* out: error message           */
    TEXT   key[]            /* out: error key               */
);

/* get the next interactive line in no-screen mode */
FUNCTION CODE line_noscreen 
(
    CODE archive,           /* in: A_MENU, A_TUTOR, A_LASTCMD    */
    TEXT prompt[],          /* IN: prompt string                 */
    TEXT line[],            /* OUT: line from interactive source */
    CODE *term              /* OUT: line terminator              */
);

/* get the next interactive line in screen mode */
FUNCTION CODE line_screen 
(
    CODE  archive,          /* in: A_MENU, A_TUTOR, A_LASTCMD    */
    COUNT line_num,         /* IN: input line number             */
    TEXT  prompt[],         /* IN: prompt string                 */
    TEXT  line[],           /* OUT: line from interactive source */
    CODE  *term             /* OUT: line terminator              */
);

/* command mode micro-editor */
FUNCTION CODE cmd_editor 
(
    CODE   archive,         /* in: A_NONE or A_LASTCMD              */
    TEXT   line[],          /* out: user-typed line                 */
    TEXT   prompt[],        /* in: prompt string                    */
    COUNT  line_num,        /* in: line number on screen            */
                            /*     (-1 if line number not known)    */
    FUNINT screen           /* TRUE if in screen mode               */
);

/* tutor mode micro-editor */
FUNCTION CODE tut_editor 
(
    TEXT  line[],           /* out: user-typed line                 */
    TEXT  prompt[],         /* in: characters in prompt string      */
    COUNT line_num          /* in: line number                      */
);

/* menu mode micro-editor */
FUNCTION CODE menu_editor 
(
    TEXT  line[],           /* out: user-typed line                 */
    TEXT  prompt[],         /* in: characters in prompt string      */
    COUNT line_num          /* in: line number                      */
);

/* (re)enter menu mode */
FUNCTION VOID menmod
(
    TEXT tmcmd[]            /* OUT: command string returned to TM */
);

/* process a tcl command when the runtype is BATCH */
FUNCTION CODE run_batch 
(
    struct CONTXT *procctx, /* IN: Invoking proc context      */
    struct CONTXT *cmdctx   /* IN: proc with batch qualifier  */
);

/* generate a help file */
FUNCTION CODE hard_help
(
    struct SFILE *fctx,         /* In: file context block for opened file */
    TEXT           curlib[],    /* In: library name of opened file        */
    TEXT           procname[],  /* In: proc name for LHS of help display  */
    TEXT           outname[],   /* In: name of output file                */
    struct HELPBLK *helpblk     /* out: help output control block         */
);

/* verify a file variable before assignment */
FUNCTION CODE file_ins 
(
    struct VARIABLE *v,           /* in: VARIABLE definition    */
    GENPTR          vector,       /* in: value vector           */
    FUNINT          count,        /* in: value count            */
    TEXT            msgkey[],     /* out: message key (if error)*/
    TEXT            msg[]         /* out: message (if error)    */
);

/* builds a context block for dynamic parameter mode */
FUNCTION CODE bld_pdf_ctx 
(
    struct CONTXT *ctx,           /* in: context of executing process  */
    struct PARBLK *parblk,        /* in/out: parameter block           */
    struct CONTXT *dctx,          /* out: context block for tutor      */
    struct SFILE  *dynpdf,        /* out: address of dynamic pdf SFILE */
    BOOL          *dash_present   /* out: TRUE if subcommand indicated */
);

/* initiate dynamic tutor or dynamic parameter mode */
FUNCTION VOID dynget 
(
    struct CONTXT *ctx,           /* in: context of executing process */
    struct PARBLK *parblk,        /* in/out: parameter block          */
    FUNINT        pool_size,      /* in: size of parblk.pool      */
    BOOL          asytut_req      /* in: async or sync tutoring ? */
);

/* execute a TAE command from a process */
FUNCTION VOID dyncommand 
(
    struct CONTXT *ctx,     /* in/out: parameter context    */
    struct PARBLK *parblk,  /* in/out: absolute parblk */
    int           pool_size 
);

/* block a signal from being delivered */
FUNCTION CODE block_sig(int signum);

FUNCTION CODE process 
(
    struct CONTXT       *pctx,      /* in:  proc context */
    struct ECB          *ecbi,      /* in/out: event blk for interrupts */
    struct CODE_ERR     *(*errmsg)  /* out: errmsg if code FAIL, else "" */
);

/* parse the line starting at the current syntax block pointer until EOS */
FUNCTION CODE parser
(
    struct SYNBLK *sb,            /* in: syntax block             */
    struct PSTATE *statelist[],   /* in: list of state pointers   */
    FUNINT        state0,         /* in: index of pointer to start state*/
                                  /* (W.R.T. the statelist)       */
    struct ERRMSG *(*errmsg)      /* out: ptr to error msg struct */
);

/* initialize a collection of state tables */
FUNCTION void parser_init 
(
    struct PSTATE *statelist[],   /* in: vector of state ptrs     */
    FUNINT         n              /* in: entries in vector        */
);

/* pop an element off the top of the value stack */
FUNCTION CODE popvstack
(
    struct VALUE *value     /* out: the value popped        */
);

/* push a value on the value stack */
FUNCTION CODE pushvstack
(
    struct VALUE *value     /* in: the structure to push    */
);

/* build symbol tables from a compiled PDF */
FUNCTION CODE prc_compiled 
(
    struct SFILE  *file,    /* in/out: compiled PDF control block   */
    struct CONTXT *ctx      /* in/out: context for compiled PDF     */
);

/* build symbol tables from a simple PAR file */
FUNCTION CODE prc_par_file 
(
    struct SFILE  *file,    /* opened PAR file      */
    struct CONTXT *ctx      /* contxt to build      */
);

/* prepare to display help on a compiled PDF */
FUNCTION CODE prep_c_help 
(
    struct DIRBLK  *dirblk, /* in/out: directive block              */
    struct TXTSTOR *title,  /* out: title text                      */
    TEXT           msg[],   /* error message (if any)               */
    TEXT           key[]    /* error key (if any)                   */
);

/* evaluate a TCL expression */
FUNCTION CODE evalexp
(

    struct SYNBLK *synblk,  /* in: syntax block             */
    struct CONTXT *contxt,  /* in: proc context block       */
    CODE          type,     /* in: integer, string, or real */
    GENPTR        *value,   /* out: pointer to the result; NULL if fail     */
    COUNT         *count,   /* out: number of outputs       */
    CODE          *term     /* out: terminator: EOS or S_COMSEP */
);

/* find the descriptor for the specified TCL function */
FUNCTION struct TCLFUNC *fn_search
(
    TEXT name[]             /* in: name of the function to find     */
);

/* set some default value elements */
FUNCTION VOID inival
(
    struct VALUE *value     /* in/out: the value    */
);

/* return 1 if panel is displayed, 0 otherwise */
FUNCTION CODE panel_func
(
    struct VALUE  *nameval,       /* in: value containing the name*/
    struct CONTXT *context,       /* in: TM context               */
    struct VALUE  *numval,        /* out: value containing the number*/
    struct ERRMSG *(*errmsg)      /* out: error message           */
);

#ifdef TAE_ASYNC
/* terminate asynchronous task */
FUNCTION VOID term_async(void);
#endif

/* free the dynamic storage used by an acb */
FUNCTION VOID free_acb 
(
    struct ACB *acb         /* in: the acb                  */
);

/* perform logoff command */
FUNCTION CODE logoff_do 
(
    struct CONTXT *cpctx,   /* in:  containing proc context         */
    struct CONTXT *npctx    /* in:  proc ctx for LOGOFF command     */
);

#endif
