/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/
/*
 *      Session logging.
 *
 * 
 *      
 ********************************************************************************
 *
 * CHANGE LOG:
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *      22-aug-83       Updated for NULLABLES...dm
 *      02-sep-83       Logged reference name for NAME parameters...dm
 *      07-sep-83       Updated for forp... function name changes...dm
 *      10-oct-83       Fixed unix lint errors...palm
 *      31-oct-83       PR 566: change m_put call to put_stdout...dm
 *      01-nov-83       Fix recursion bug,add operator atten to logmsg...dm
 *      23-nov-83       Fix 'no echo to terminal' bug  (pr #593)...dm
 *      12-mar-84       Use s_bcopy to set lastkey...palm
 *      19-mar-84       Remove double check of msgtype for logging...dm
 *      04-may-84       VALUE_x to xVAL ... ces
 *      07-may-84       Clean-up the IMPORT declarations...lim
 *                      'toprefs'-->'numrefs'...nhe
 *      15-aug-84       Modify log message process, added a parameter to logmsg
 *                      and created a new function add_trace (pr #628)...lia
 *      08-dec-84       Delete use of $MESSAGE(2) and eliminated display
 *                      of trace information...lia
 *      06-may-85       Fix dimension with SIZ to SIZ+1...lia
 *      01-jul-85       Fix UNIX compilation error...dm
 *      11-may-89       Don't set lastkey when null key...palm 
 *      22-oct-89       if null key, put_stdout rather than put_msgout...palm 
 *
 ******************************************************************************
 */

#include <string.h>
#include "taeconf.inp"           /* TAE configuration */
#include "inttaelib.h"
#include "parblk.inc"            /* parameter block definitions */
#include "terminc.inc"           /* terminal package */
#include "tminc.inc"             /* TM  related definitions */
#include "sessinc.inc"           /* session log struct */
#include "local_protos.h"

#ifdef SESSION_LOG

#define         PARLOGSIZ   STRINGSIZ+6         /* allow for ""), +  */
#define         MSGLOGSIZ   STRINGSIZ+KEYSIZ+3  /* allow for key string */


FUNCTION static VOID slprocnm
(
    struct CONTXT *ctx      /* in: context block for proc */
);
FUNCTION static VOID slforprm
(
    struct CONTXT *ctx      /* IN: proc context block */
);
FUNCTION static VOID slforgbl
(
    struct CONTXT *ctx      /* IN: proc context block  */
);
FUNCTION static VOID outname
(
    TEXT  prefix[],         /* IN: output record prefix */
    TEXT  name[],           /* IN: name of variable */
    TEXT  line[],           /* IN/OUT: record to add name */
    COUNT length            /* IN: max record length */
);
FUNCTION static VOID outvalue
(
    TEXT            prefix[],     /* IN: output record prefix */
    struct VARIABLE *vv,          /* IN: pointer to variable */
    COUNT           index,        /* IN: index to value vector */
    TEXT            line[],       /* IN/OUT: record to add name */
    COUNT           stdlen,       /* IN: standard line size */
    COUNT           longlen,      /* IN: max record length */
    FUNINT          lastflag      /* IN: true if last value */
);
FUNCTION static VOID add_trace
(
    struct CONTXT *ctx,     /* IN: context of executing process */
    TEXT          record[]  /* OUT: formatted message       */
);

/*
 * slproc - session log proc activation information
 *
 */

FUNCTION VOID slproc
(
    struct CONTXT *context          /* in: context block for the proc */
)
{
    IMPORT  struct  SESSLOG     sesslog;        /* session log struct */

    if (!sesslog.enable)
        return;                         /* by-pass overhead if disabled */
    sltime();                           /* log current time */
    slprocnm(context);                  /* log proc name */
    slforprm(context);                  /* log proc parameters */
    slforgbl(context);                  /* log proc referrenced globals */
}

/*
 * sltime - log current time to session log file 
 */

FUNCTION VOID sltime(void)
{
    TEXT time[STRINGSIZ+1];  /* current time(dd-mmm-yyyy hh:mm:ss)*/

    time[0] = EOS;                              /* safety catch         */
    get_time(time);                             /* get current time     */
    slwrite("TS: ", time);                      /* log time to file only*/
}

/*
 *
 *      slprocnm - log proc name to session log file
 */

FUNCTION static VOID slprocnm
(
    struct CONTXT *ctx      /* in: context block for proc */
)
{
    TEXT record[STRINGSIZ+1];            /* output buffer */

    strcpy(record,"\"");
    f_spec(&(*ctx).pdf, &record[1]);            /* build host file spec */
    strcat(record,"\"");                        /* add end quote        */
    if (!NULLSTR((*ctx).subcmd))                        
    {                                       /* if subcmd present    */
        strcat(record,"-");
        strcat(record,(*ctx).subcmd);   /* append subcommand name */
    }
    if ((*ctx).parmst.link != NULL)             /* variables to follow ? */
        strcat(record,"+");                     /* append trailing +    */
    slwrite("PA: ", record);                    /* log to file only     */
}

/*
 * slforprm .  format the input parameter variables  (in a linked list) 
 */

FUNCTION static VOID slforprm
(
    struct CONTXT *ctx      /* IN: proc context block */
)
{
    slparm("PA: ", &(*ctx).parmst);             /* format for proc activation */
}

/*
 * slparm . parameter formatting function
 */

FUNCTION VOID slparm
(
    TEXT          prefix[], /* record prefix           */
    struct SYMTAB *symtab   /* symbol table address    */
)
{
    IMPORT COUNT   termcols;       /* number of columns on terminal */

    TEXT           line[PARLOGSIZ+1];      /* OUT: formatted parameter rec */
    struct  VARIABLE    *vv;                    /* pointer to a variable   */
    COUNT               index;                  /* index pointer           */
    COUNT               max_index;              /* max value of index ptr  */
    BOOL                lastflag;               /* true if last variable   */
    COUNT               stdsiz;                 /* standard length of line */

    stdsiz = termcols-PREFIXSIZ;
    line[0] = EOS;
    for (vv = (*symtab).link; vv != NULL; vv = (*vv).v_link)
    {
        outname(prefix, (*vv).v_name, line, stdsiz);     /* output name */
        if ((*vv).v_class == V_PARM && (*vv).v_type == V_NAME)/* if NAME var */
            max_index = 0;
        else
            max_index = ((*vv).v_count >= 1) ? ((*vv).v_count-1) : 0;
        for (index = 0; index <= max_index; index++)
        {
            lastflag = ((*vv).v_link == NULL  &&   index == max_index) ;
            outvalue(prefix, vv, index, line, stdsiz, PARLOGSIZ, lastflag); 
        }
    }
}

/*
 * slforgbl .  format the input global variables  (in a refed list) 
 */

FUNCTION static VOID slforgbl
(
    struct CONTXT *ctx      /* IN: proc context block  */
)
{
    IMPORT COUNT     termcols;       /* number of columns on terminal */

    TEXT             line[PARLOGSIZ+1];      /* OUT: formatted parameter rec */
    struct  VARIABLE    *vv;                    /* pointer to a variable   */
    COUNT               index;                  /* index pointer to value  */
    COUNT               max_index;              /* max value of index ptr  */
    COUNT               i;                      /* index to refs array     */
    BOOL                lastflag;               /* true if last value      */
    COUNT               stdsiz;                 /* standard length of line */

    stdsiz = termcols-PREFIXSIZ;
    line[0] = EOS;
    for (i = 0; i < (*ctx).numrefs; i++)                /* for each global */
    {
        vv = (*ctx).refs[i];                   /* point to VARIABLE struct */
        outname("PR: ", (*vv).v_name, line, stdsiz);    /* output name     */
        max_index = ((*vv).v_count >= 1) ? ((*vv).v_count-1) : 0;
        for (index = 0; index <= max_index; index++)
        {
            lastflag =  (i == ((*ctx).numrefs)-1  &&   index == max_index);
            outvalue("PR: ", vv, index, line, stdsiz, PARLOGSIZ, lastflag); 
        }
    }
}

/*
 *      outname. Add variable name to output buffer.
 *      If buffer is full, write to session log file and start with
 *      a fresh buffer.
 */

FUNCTION static VOID outname
(
    TEXT  prefix[],         /* IN: output record prefix */
    TEXT  name[],           /* IN: name of variable */
    TEXT  line[],           /* IN/OUT: record to add name */
    COUNT length            /* IN: max record length */
)
{
    CODE                code;
    
    code = m_fpname(name, line, length);        /* get formatted name */
    if (code == FAIL)                           /* no more room in buffer */
    {
        strcat(line,"+");                       /* put continuation char */
        slwrite(prefix, line);                  /* write to session log file */
        line[0] = EOS;                          /* start with a fresh buffer */
        m_fpname(name, line, length);           /* add name to new line */
    }
}

/*
 *      outvalue. Add variable value(s) to output buffer.
 *      If buffer is full, write to session log file and start with
 *      a fresh  buffer.
 *      NOTE: If a NAME variable, add reference name rather than value.
 */

FUNCTION static VOID outvalue
(
    TEXT            prefix[],     /* IN: output record prefix */
    struct VARIABLE *vv,          /* IN: pointer to variable */
    COUNT           index,        /* IN: index to value vector */
    TEXT            line[],       /* IN/OUT: record to add name */
    COUNT           stdlen,       /* IN: standard line size */
    COUNT           longlen,      /* IN: max record length */
    FUNINT          lastflag      /* IN: true if last value */
)
{
    CODE                code;
    COUNT               nchar;

    if ((*vv).v_type == V_NAME)                 /* if name parameter         */
        code = m_frname(vv, line, stdlen, longlen);         /* get ref name  */
    else
        code = m_fpval(vv, index, line, stdlen, longlen);   /* get value     */
 
    if (code == FAIL)                           /* no more room in buffer    */
    {
        strcat(line,"+");                       /* put continuation char     */
        slwrite(prefix, line);                  /* write to session log file */
        line[0] = EOS;                          /* start with a fresh buffer */
        outvalue(prefix, vv, index, line, stdlen, longlen, FALSE);
    }
    if (lastflag)                            /* last value of last variable */
    {
        nchar = strlen(line);
        line[nchar-2] = EOS;                    /* del hanging separator ", " */
        slwrite(prefix, line);                  /* write the last line */
    }
}

/*
 *      slterm. Log proc termination.
 */
  
FUNCTION VOID slterm(void)
{
    IMPORT  COUNT               termcols;       /* number of columns on terminal */
    IMPORT  struct  SESSLOG     sesslog;        /* session log struct */
    IMPORT  struct  VARIABLE  *sfi_gbl;         /* pointer to $sfi      */
    IMPORT  struct  VARIABLE  *skey_gbl;        /* pointer to $skey     */

    TEXT                line[PARLOGSIZ+1];   /* OUT: formatted parameter rec */
    struct  VARIABLE    *vv;                    /* pointer to a variable */
    COUNT               stdsiz;                 /* standard length of line */


    if (!sesslog.enable)
        return;                             /* by-pass overhead if disabled */
    sltime();                                   /* time stamp   */
    stdsiz = termcols-PREFIXSIZ;
    line[0] = EOS;
    vv = sfi_gbl;                                            /* log $sfi     */
    outname("PT: ", (*vv).v_name, line, stdsiz);             /* output name  */
    outvalue("PT: ", vv, 0, line, stdsiz, PARLOGSIZ, FALSE); /* output value */

    vv = skey_gbl;                                           /* log $skey    */
    outname("PT: ", (*vv).v_name, line, stdsiz);             /* output name  */
    outvalue("PT: ", vv, 0, line, stdsiz, PARLOGSIZ, TRUE);  /* output value */
}

/*
 *      slwrite. Write data to session log file.
 *      
 */

FUNCTION VOID slwrite
(
    TEXT prefix[],          /* IN: prefix to line   */
    TEXT line[]             /* IN: line to log      */
)
{
    IMPORT  struct  SESSLOG     sesslog;        /* session log struct */

    TEXT                record[PREFIXSIZ+PARLOGSIZ+1];  /* local buffer  */
    COUNT               nchar;                          /* record length */

    if (sesslog.enable)
    {
        strcpy(record,prefix);                  /* copy prefix */
        strcat(record,line);                    /* add text to be logged */
        nchar = strlen(record);
        if (nchar > STRINGSIZ)                  /* if record too long   */
            strcpy(&record[STRINGSIZ-3],"..."); /* truncate record      */
        f_write(&sesslog.sfile, record);        /* write to log file    */
    }
}
#endif

/*
 *      logmsg .  Log message from subprocess.
 */

FUNCTION VOID logmsg
(
    struct CONTXT *ctx,     /* in: context of executing process */
    struct PARBLK *parblk   /* in: parblk from subprocess   */
)
{
    IMPORT TEXT         lastkey[];      /* most recent message key      */

    struct VARIABLE     *v1, *v2;
    TEXT                msgtext[MSGLOGSIZ+1];   /* local buffer for msg + key */
    TEXT                tracetxt[STRINGSIZ+1];  /* local buf for trace info */
    COUNT               nchar;                  /* len of output string */
    BOOL                nullKey;


    v1 = lookex(&(*parblk).symtab, "MESSAGE");          /* find message test */
    v2 = lookex(&(*parblk).symtab, "KEY");              /* find message key  */
    if (v1 == NULL || v2 == NULL)                       /* garbage from proc?*/ 
        return;                                         
    nullKey = NULLSTR (SVAL(*v2,0)) || s_equal (SVAL(*v2,0), " ");
    if (nullKey)
    {
        s_bcopy (SVAL(*v1, 0), msgtext, sizeof (msgtext) - 1);
        put_stdout (msgtext);
    }
    else
    {
        s_bcopy(SVAL(*v2, 0), lastkey, KEYSIZ);         /* save for help-msg */
        msgtext[0] = EOS;
        m_pfmt(SVAL(*v1, 0), SVAL(*v2, 0), msgtext);    /* format output  */
        put_outmsg (msgtext);                   /* write to output devices   */
    }

#ifdef SESSION_LOG
    tracetxt[0] = EOS;
    add_trace (ctx, tracetxt);
    if (tracetxt[0] == EOS)
        slwrite ("MS: ", msgtext);
    else
    {
        nchar = strlen(msgtext);
        if (nchar > STRINGSIZ-7)                /* if record too long   */
            strcpy(&msgtext[STRINGSIZ-10],"...; +");       /* truncate */
        else
            strcat(msgtext,"; +");
        slwrite ("MS: ", msgtext);              /* log to file  */
        slwrite ("MS: ", tracetxt);
    }
#endif
}

/*
 *      add_trace .  Add proc information (name and line number) to
 *      message for locating where message is sent.  If location is
 *      in the body portion of the PDF only the proc name will be added.
 */

FUNCTION static VOID add_trace
(
    struct CONTXT *ctx,     /* IN: context of executing process */
    TEXT          record[]  /* OUT: formatted message       */
)
{
    TEXT                linenum[12];

    strcpy (record," proc '");          /* start new line       */
    strcat(record,(*ctx).pdf.name);     /* copy proc name       */
    if ((*ctx).proctype == Y_PROCESS && (*ctx).inbody)
        strcat(record,"'");
    else
    {
        strcat(record,"', line ");
        s_i2s ((*ctx).pdf_line + 1, linenum); /* convert line to int    */
        strcat(record,linenum);         /* copy line number     */
    }
}


#ifdef SESSION_LOG
/*
 *      logdyn .  Log parameters for dynamic command processing .
 */

FUNCTION VOID logdyn
(
    struct PARBLK *parblk   /* in: parblk from subprocess   */
)
{
    if ((*parblk).msgtyp == M_CONTINUE)             /* dynamic cmd param */
    {
        makeabs(&(*parblk).symtab, (*parblk).pool); /* make pointers absolute */
        slparm("DP: ", &(*parblk).symtab);
        makerel(&(*parblk).symtab, (*parblk).pool); /* make relative again */
    }
}
#endif
