/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/


/* Process PDF for terminal monitor.
 *
 * CHANGE LOG:
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *      12-jul-83       Rewrote process()...dm
 *      14-jul-83       Display host error code in subprocess related
 *                      messages...dm
 *      29-jul-83       Clear "receive" event flag in process()...dm
 *      15-aug-83       Fix comment to align with delgbl_do fix...peb
 *      02-sep-83       Fix error msg in opnpdf (PR 432)...palm
 *                      Also, new dynget calling sequence...palm
 *      08-sep-83       Set $status into $skey if DCL command...palm
 *      12-sep-83       Build global symbol table in alphabetic order...dm
 *      12-sep-83       Condititional compilation on function 'process'
 *                      preparing for UNIX (where it will be cnp)...palm
 *      25-sep-83       Accept output variables from exit handlers of
 *                      an aborted task...palm
 *      10-oct-83       Fix unix compilation errors...palm
 *      21-oct-83       Fix stdout reset bug...dm
 *      03-feb-84       Update for full-blown proc interrupt mode...ne
 *      22-feb-84       In pdftab(): save curproc explicitly...peb
 *      27-feb-84       In process():  Add message handshake...peb
 *      03-mar-84       In process(): Handle param request while in async...nhe
 *      11-mar-84       Make parblk relative before sending back to process
 *                      for handsake...dm
 *      12-mar-84       Make outputs and interrupt EXTERN for UNIX...palm
 *      13-mar-84       Update to accomodate bldcmd changes...dm
 *      15-mar-84       Move logging of dyn pars to elsewhere...nhe
 *      04-may-84       VALUE_x to xVAL ... ces
 *      07-may-84       Clean-up the IMPORT declarations...lim
 *                      'toprefs'-->'numrefs'...nhe
 *      17-mar-84       New hierarchy and f_crack calling sequence...palm
 *      8-jun-84        TCL-99: avoid global conflicts with
 *                      level 0 locals...palm
 *      12-jun-84       TCL-21: CONTROL/C handling in procedure...palm
 *      21-jun-84       pr706: get errmsgs from process into stdout for async...nhe
 *      09-jul-84       pr664: move call to pt_ins(), in run_proc...peb
 *      13-jul-84       errmsgs-->process(),errmsg-->CODE_ERR in tminc...nhe
 *      01-aug-84       file_name-->file_spec in struct MON_MSG...nhe
 *      03-aug-84       pr396: give proc name on set_outval failure...peb
 *      16-aug-84       pr396: TAE-COMOPN --> TAE-COMOPEN...peb
 *      22-aug-84       Pass proc context to logmsg...lia
 *      08-oct-84       Update opnpdf to handle remote case...nhe
 *      11-oct-84       Add conditional compilation for c_snpa/c_sndp call
 *                      as implemented for UNIX 1.2...dm
 *      12-oct-84       Conditional compilation for get_pblk_remote()...dm
 *      14-oct-84       Changes for internal procs...nhe
 *      17-oct-84       TCL 117: pdftab logic for compiled PDFs...peb
 *      18-oct-84       Enhancements to internal procs...nhe
 *      23-oct-84       TCL 117: compilation of SUBCMDs...peb
 *      07-dec-84       PR 908: incomplete reinit of VARIABLE struct in
 *                      converting the parm to a global in make_gbl...lia
 *      11-dec-84       TCL 97: maintain flag for executing onfail cmd...lia
 *      18-dec-84       Fix infinite loop on bad onfail cmd...lia
 *
 ***************************************************************************
 * CHANGES MADE IN THE RCJM TREE:
 *
 *      06-feb-85       Check for link-abort event for remote jobs...dm
 *      09-feb-85       At CTL/C  abort_remote if processing an rpd...dm
 *      13-mar-85       Update for new member names of MON_MSG...dm
 *      06-apr-85       Update get_pblk_remote() for remote jobs; delete
 *                      reference to ASYNCINC...dm
 *      17-apr-85       Add _DYNPDF for remote process's dynpar block...dm
 *
 ***********************************************************************
 *
 *      25-jul-85       Fix UNIX lint compilation errors...dm
 *      01-aug-85       Delete ecb_link reference due to link-abort 
 *                      redesign...dm
 *      26-aug-85       PR 806: recognize user exit from dynamic tutor after
 *                      call to get_parm_locpar...dab
 *      09-sep-85       PR 961: Call to 'dynget' by 'process' indicates that 
 *                      will be tutoring for sync proc (to prevent HOLD)...dab
 *      03-oct-85       PR 946: Added "not compiling" (FALSE) parameter to
 *                      calling sequence of 'chk_vector'...dab
 *      17-feb-86       Honor OPTIONS=INTERRUPT for procedures; Honor
 *                      int_enabled flag (as set by DISABLE-INTERRUPT and
 *                      ENABLE-INTERRUPT).
 *      24-mar-86       Pass current contxt to interrupt from procedure and
 *                      process...palm
 *      28-may-86       Fix bug in make_gbl whereby adding a variable to
 *                      BEGINNING of global table caused crash (we forgot
 *                      to unlink the VARIABLE from the parmst before linking
 *                      into glbtab)...palm
 *      16-jul-86       Honor ACTION= on parmset execution...palm
 *      05-sep-86       PR 1129 fix.  PDF line number now correct in
 *                      error messages when comments, null lines precede
 *                      erroroneous statement...peb
 *      05-sep-86       Remove anachronous null str
 *                      check after getpst calls...peb
 *      11-sep-86       PR 1100 fix.  Modified process() to check for entry via
 *                      menu.  If so, request for prompt before repaint...lia
 *      28-mar-87       New calling sequence for package in process. Also
 *                      we use LARGE_PARBLK for sending to subprocess...palm
 *      19-jul-87       Fix make_gbl to pick up qualifiers...palm
 *      02-aug-87       Fix missing key in deep_value_copy tmmmsg...palm
 *      19-aug-87       Maintain consistency with old executables wrt
 *                      the M_KILLED flag:  The V1.3 outboard library checks for
 *                      M_KILLED PARBLK.msgtyp to determine that the tutor
 *                      user typed "EXIT".  The V1.4 library and beyond check
 *                      for either M_KILLED or M_DYNEXIT, and dynget sets
 *                      msgtyp to M_DYNEXIT.   Here, we convert from
 *                      M_DYNEXIT to M_KILLED before sending a parblk to an
 *                      application process so that old executables work.
 *                      This mess started with PR806...palm
 *      26-aug-87       Fix qualptr casting and definition in opnpdf...palm
 *      08-sep-87       Add size in call to dynget...tpl
 *      17-dec-87       Executable PAR files...palm
 *      18-dec-87       Add CheckXEvent call to run_proc...palm
 *      24-feb-88       PR 1504: Change label dimension to LABELSIZ+1...ljn
 *      26-feb-88       Reset EventProcCtx when coming out of a procedure 
 *                      that has an event loop (using WPT-EVENT)...palm
 *      17-mar-88       PR 1499: Add call to isTerm()...ljn
 *      24-mar-88       Delete TAE_RCJM conditionals...ljn
 *      02-may-88       Remove restriction that only files with type .PAR
 *                      can be executed as PAR files.  See call to IsParFile.
 *                      ...palm
 *      05-may-88       isTerm() -> f_isterm()...ljn
 *      01-dec-88       package valids for SELFTUTOR...palm
 *      13-feb-89       Zero parblk header before sending to process...palm
 *      01-may-89       disallow process execution while process already
 *                      running...palm 
 *      12-sep-89       Port PM's 1-may-89 fix on UNIX...ljn
 *      20-dec-89       Fixed handling of M_COMMAND for VMS...tpl
 *      23-may-90       Remove RCJM stuff referring to old TAE_RCJM...ljn
 *      27-jun-90       Remove Facelift code...ljn
 *      11-may-92       Used P_BIGSENTINEL instead of P_SENTINEL...tpl
 *      11-sep-92 PR875 In getpst, changed inbuf size to CMDLINSIZ instead of
 *                      STRINGSIZ and use f_readstring instead of f_read...tpl
 *      22-oct-92       Prototype of tae_alloc unnecessary and Ultrix 4.3 does
 *                       not like it...rt
 */

#include <string.h>
#include "taeconf.inp"   /* TAE configuration definitions */
#include "inttaelib.h"
#include "terminc.inc"   /* terminal package definitions */
#include "syninc.inc"    /* syntax package structure and defs */
#include "eventinc.inp"  /* e_ package defs & structs */
#include "taskinc.inp"   /* c_ package defs & structs */
#include "symtab.inc"
#include "tmhost.inp"
#include "parblk.inc"
#include "tminc.inc"     /* TM-only host-independent definitions */
#include "taelib.h"
#include "local_protos.h"

struct VARIABLE *srchrf();  /* search for variab among list of ref'd glbls*/

FUNCTION static CODE make_gbl 
(
    struct CONTXT *pctx     /* in/out: global proc context */
);
FUNCTION static CODE set_outval
(
    struct VARIABLE *v,     /* in/out: VARIABLE to set */
    struct PARBLK   *parblk /* in: PARBLK from process */
);
FUNCTION static VOID msg_subs 
(
    struct CONTXT *ctx      /* in:  proc context */
);
FUNCTION static CODE procedure 
(
    struct CONTXT *pctx,    /* in:  context for this procedure */
    struct ECB    *ecbi     /* in:  event blk for operator interr. */
);

#define BC_PARMST       1
#define BC_LOCST        2
#define BC_GLOBREFS     3


/* build_prompt - Build a new prompt string, "<old prompt>-INTERRUPT>"
 *
 */
FUNCTION static VOID build_prompt
(
    TEXT prompt[]           /* out: the prompt built */
)
{
    IMPORT  struct  VARIABLE    *prompt_gbl;    /* pointer to $prompt */

    /* get current prompt */
    strcpy(prompt,SVAL(*prompt_gbl, 0));
    strcat(prompt,"-INTERRUPT>");
}

/*
 *      chk_onfail - check a command to make sure it is an
 *      allowed _ONFAIL command.
 *
 */

FUNCTION static CODE chk_onfail 
(
    TEXT cmdstr[]           /* in:  command string */
)
{
    struct SYNBLK sb;             /* syntax block */
    TEXT          verb[TOKESIZ+1];
    struct ITRCMD *itrcmd;        /* pointer to intrinsic table */
    struct ITRCMD *intrin();      /* get ITRCMD entry for command */

    /* init syntax block for syntax pkg */
    initok(&sb, cmdstr);
    /* get verb from cmd str */
    if (getvrb(&sb, verb) == S_SYNERR) 
    {
        tmmsg(PROCFAIL, "Incorrect command format in _ONFAIL command: %s.",
              "TAE-ONFAIL", sb.errmsg);
        /* DO_RETURN avoids _ONFAIL loop */
        return(FAIL);
    }
    /* get pointer to ITRCMD entry */
    itrcmd = intrin(verb);
    if (itrcmd != NULL)
    {
        if (((*itrcmd).flags & Y_ONFAIL)  != 0)
            return (SUCCESS);
    }
    tmmsg(PROCFAIL, "'%s' not allowed as _ONFAIL command.", "TAE-ONFAIL", verb);
    return (FAIL);
}

/*
 *      clssav - close a proc and save the position.
 */

FUNCTION VOID clssav 
(
    struct CONTXT *pctx     /* in/out: proc context */
)
{
    IMPORT struct SFILE prcfil;   /* proc file context */

    f_movpos(&(prcfil.posctx), &((*pctx).prcpos));
    f_close(&prcfil, F_KEEP);   
}

#ifdef TAE_ASYNC
/*  get_pblk_remote - Get parameter values for a parblk from parent monitor
 *
 *  Input is an 'absolute' parblk and output is a 'relative' parblk (imitates
 *  dynget in this).
 *
 *  Returns SUCCESS/FAIL
 */
FUNCTION CODE get_pblk_remote 
(
    struct CONTXT   *proc_ctx,    /* in: Context of executing proc */
    struct PARBLK   *parblk,      /* in/out: parameter block */
    struct CODE_ERR *(*errmsg)    /* out: pointer to err msg */
)
{
    IMPORT TEXT         comm_file[];    /* name of mon-to-mon comm'n file */
    IMPORT CODE         parent_runtype; /* run_type of parent monitor */
    TEXT                recv_file[FSPECSIZ+1];  /* file received from parent */
    struct SFILE        sf;
    struct PARHDR       ph;
    struct SYMTAB       *st;
    CODE                code;
    CODE                send_stat, recv_stat;
    COUNT               recsize;

    static struct CODE_ERR er_asyparm=
        {"Async jobs may request parms only if invoked interactively.",
         "TAE-ASYPARM", 0};

    static struct CODE_ERR er_comopn=
        {"Error opening async communication file.",
         "TAE-COMOPEN", 0};

    static struct CODE_ERR er_comwrt=
        {"Error writing to async communication file.",
         "TAE-COMWRT", 0};

    static struct CODE_ERR er_comread=
        {"Error reading async communication file.",
         "TAE-COMREAD", 0};

    static struct CODE_ERR er_fmtcomm=
        {"Async communication file not properly formatted.",
         "TAE-FMTCOMM", 0};

    static struct CODE_ERR er_asysnd=
        {"Error sending message to parent monitor.  Host error code = %d.",
         "TAE-ASYSND", 0};

    static struct CODE_ERR er_asyrcv=
        {"Error receiving message from parent monitor.  Host error code = %d.",
         "TAE-ASYRCV", 0};

    if (parent_runtype == ASYNC || parent_runtype == BATCH)
    {
        *errmsg = &er_asyparm;
        return (FAIL);
    }

    /* Save the parblk in the communication file */

    code = f_opnspc (&sf, SAVELUN, comm_file, "", "", ASY_TYPE, F_WRITE);
    if (code != SUCCESS) goto open_err;
    st = &(*parblk).symtab;
    /* write parmst to file */
    code = save_pfile (&sf, (*parblk).msgtyp, NULL, 0, 1, &st); 
    if (code != SUCCESS) goto write_err;
    f_close (&sf, F_KEEP);

    /* Send the message to the parent monitor and receive return reply */
    /* get parm from local parent */
    code = get_parm_locpar(comm_file, recv_file, TRUE, &send_stat, &recv_stat);
    if (code == KILLED)                         /* user wants to exit the job */
    {
        (*parblk).msgtyp = M_KILLED;
        return (SUCCESS);
    }
    else if (code == DO_EXIT)                   /* user exited from dyn tutor */
    {
        (*parblk).msgtyp = M_DYNEXIT;
        return (SUCCESS);
    }
    if (code != SUCCESS) 
        goto comm_err;         /* communication error */

    /* open for read */
    code = f_opnspc(&sf, SAVELUN, recv_file, "", "", ASY_TYPE, F_READ);
    if (code != SUCCESS)  
        goto open_err;        /* report error */
    code = f_bread(&sf, (GENPTR) &ph, sizeof(struct PARHDR), &recsize); 
    if (code != SUCCESS)  goto read_err;
    if (!s_equal(ph.sentinel, P_BIGSENTINEL)) 
        goto format_err;  /* not proper format */
    code = f_bread(&sf, (GENPTR) parblk, sizeof(struct PARBLK), &recsize);
    if (code != SUCCESS) goto read_err;
    f_close (&sf, F_DELETE);
    return(SUCCESS);


/*  on errors we point to an error msg in front of this module */
open_err:
    *errmsg = &er_comopn;
    return(FAIL);

write_err:
    *errmsg = &er_comwrt;
    f_close(&sf, F_DELETE);
    return(FAIL);

read_err:
    *errmsg = &er_comread;
    f_close(&sf, F_KEEP);
    return(FAIL);

format_err:
    *errmsg = &er_fmtcomm;
    f_close(&sf, F_KEEP);
    return (FAIL);

comm_err:
    if (send_stat != SUCCESS)           /* msg could not be sent */
    {
        *errmsg = &er_asysnd;
        er_asysnd.code = send_stat;
    }
    else                                /* error in receiving reply */
    {
        *errmsg = &er_asyrcv;
        er_asyrcv.code = recv_stat;
    }
    return(FAIL);
}
#endif

/*      getpst - get one PDF statement.
 *      Skips comments & null lines.
 *      The statement may result from multiple single PDF lines, all
 *      but the last ending in "+".  As these multiple lines are concatenated,
 *      each continuation "+" is replaced with a blank.
 */

FUNCTION CODE getpst 
(
    struct SFILE  *pdffil,        /* in/out: PDF file context */
    TEXT          sttmt[],        /* out: PDF statement */
    COUNT         *records,       /* out: number of records read */
    COUNT         *num_prec,      /* out: # comments, null lines */
                                  /*      preceding 1st real sttmt rec*/
    struct POSCTX *first_rec      /* out: position of 1st record */
)
{
    CODE                code;
    CODE                cmdcode;        /* SUCCESS, CONTINUE, or FAIL */
    COUNT               nrec;           /* number of records flushed */
    BOOL                cont;
    BOOL                in_sttmt;       /* TRUE if actual statement started*/
    TEXT                inbuf[CMDLINSIZ+1];

    *records  = 0;
    *num_prec = 0;
    in_sttmt  = FALSE;
    sttmt[0]  = EOS;
    cont = FALSE;
    do
    {
        /* get single PDF line */
        if ((code = f_readstring (pdffil,CMDLINSIZ, inbuf)) != SUCCESS)
            goto gp_rderr;
        (*records)++;
        /* assume comment or null line */
        if (!in_sttmt)
            (*num_prec)++;
        /* save pos'n of 1st rec */
        if (!cont)
            f_movpos (&(*pdffil).posctx, first_rec);
        if (NULLSTR(inbuf))
        {
            cont = TRUE;
            continue;
        }
        /* append to any previous line */
        cmdcode = bldcmd(inbuf, sttmt);
        if (!in_sttmt  &&  !NULLSTR(sttmt))
        {
            /* no more preceding comments */
            in_sttmt = TRUE;
            /* this line wasn't comment */
            (*num_prec)--;
        }
        /* no more space */
        if (cmdcode == FAIL)
        {
            /* flush the rest of statement */
            flush_cmd(pdffil, inbuf, &nrec);
            *records = *records+nrec;
            cont = FALSE;
        }
        else if (cmdcode == CONTINUE)
            cont = TRUE;
        else
        {
            /* continue if comment recs */
            cont = NULLSTR(sttmt);
        }
    } while (cont);

    return (SUCCESS);

gp_rderr:
    if (code == F_EOF)
        tmmsg(PROCFAIL, "Premature end of proc definition file.", "TAE-PREEOF");
    else
    {
        tmmsg(PROCFAIL, "Error reading proc definition file. %s",
                "TAE-RDERR", (*pdffil).errmsg);
    }
    sttmt[0] = EOS;
    return(FAIL);
}

/*      
 *      ini_status- initialize $SFI and $SKEY
 */

FUNCTION VOID ini_status (void)
{
    IMPORT struct VARIABLE *sfi_gbl;    /* pointer to global $SFI */
    IMPORT struct VARIABLE *skey_gbl;   /* pointer to global $SKEY */

    static TAEINT       one   = 1;      /* value for $SFI */

    IVAL(*sfi_gbl,0) = one;
    set_string (skey_gbl, "");
}

/*      interrupt - handle TAE interrupt mode processing.
 *
 */

FUNCTION CODE interrupt 
(
    struct CONTXT *ctx      /* in: context of executing proc */
)
{
    IMPORT CODE         cmd_mode;   /* dynamic vs interrupt vs norm */
    IMPORT CODE         usermode;   /* for prccmd loop; menu or cmd */
    IMPORT struct SFILE prcfil;     /* context of proc that requested parms */
    IMPORT struct CONTXT *curproc;              /* pointer to cur proc */
    IMPORT TEXT         next_cmd[];
    IMPORT struct CONTXT primctx;               /* primary context */
    IMPORT struct SFILE *pstdo_fil, *saved_pstd;/* ptr to stdout SFILE */
    IMPORT struct VARIABLE  *skey_gbl, *sfi_gbl;
    IMPORT        BOOL  int_enabled;            /* TRUE if ENABLE-INT */

    struct SFILE        saved_fil;
    CODE                saved_mode;
    CODE                saved_user;
    TEXT                cmdstr[CMDLINSIZ+1];
    CODE                code=FAIL;
    TEXT                new_prompt[STRINGSIZ+1];
    TEXT                skey_save[STRINGSIZ+1];
    TAEINT              sfi_save;
    struct CONTXT       *saved_cur;

    /* proc inhibits intrpt mode? */
    if (!(*ctx).interrupt)
        return (DO_CONTINUE);
    /* intrpt mode not allowed? */
    if (!int_enabled)
    {
        /* if not, do an abort */
        return (DO_ABORT);
    }
    /* save status globals */
    strcpy(skey_save,SVAL(*skey_gbl,0));
    sfi_save = IVAL (*sfi_gbl,0);
    /* save old command mode */
    saved_mode = cmd_mode;
    /* proc interr mode */
    cmd_mode   = INTMODE;
    /* saved old user mode */
    saved_user = usermode;
    /* stay here until 'MENU' */
    usermode   = CMDMODE;
    /* save ptr to 'current' ctx */
    saved_cur  = curproc;
    /* for err msgs */
    curproc    = &primctx;
    /* save the calling file */
    MOVE_STRUCT(prcfil, saved_fil);
    saved_pstd = pstdo_fil;
    /* implies term'l, by convention */
    pstdo_fil = NULL;
    /* build interrupt prompt */
    build_prompt(new_prompt);
    /* null help cmd to start */
    next_cmd[0] = EOS;
    do
    {
        /* no saved HELP cmd? */
        if (NULLSTR(next_cmd))
        {
            if (usermode == CMDMODE)
                cmd_noscreen (A_LASTCMD, new_prompt, cmdstr);
            else
            {
                /* assume menu mode, display menu */
                menmod(cmdstr);
            }
        }
        else
        {
            /* HELP supplied the lin */
            strcpy(cmdstr,next_cmd);
            next_cmd[0] = EOS;
        }
        /* if null input, get command again */
        if (NULLSTR(cmdstr))
            continue;
        history_save (cmdstr);
        /* process the command */
        code = prccmd(cmdstr, &primctx);
    }
    while ( (code != DO_ABORT) && (code != DO_CONTINUE));
    /* restore prcfil */
    MOVE_STRUCT(saved_fil, prcfil);
    /* restore old mode */
    cmd_mode = saved_mode;
    usermode = saved_user;
    /* restore the cur proc */
    curproc = saved_cur;
    /* go back to old stdout */
    pstdo_fil = saved_pstd;
    /* restore status */
    set_string (skey_gbl, skey_save); 
    IVAL(*sfi_gbl,0) = sfi_save;
    return(code);
}

/*      int_find - Find an internal proc the specified name
 *
 *      returns the address of its ipblock or NULL
 */
FUNCTION static struct IPBLOCK *int_find
(
    struct CONTXT *ctx,     /* in: proc context */
    TEXT          name[]    /* in: name of proc to find */
)
{
    struct IPBLOCK      *ipbpt;
    struct CONTXT       *tmpctx;

    /* keep looking in outer nested contexts until we're not in an internal 
       procs */
    tmpctx=ctx;
    for (tmpctx=ctx;(s_equal((*tmpctx).pdf.libr, "/LOCAL/")); 
         tmpctx=(*tmpctx).backlink)
    {
        if ((*tmpctx).int_procs == NULL)
            continue;
        for (ipbpt = (struct IPBLOCK *) (*tmpctx).int_procs; ipbpt != NULL; 
             ipbpt = (*ipbpt).flink)
        {
            if (s_equal ((*ipbpt).procname, name))
                return(ipbpt);
        }
    }

    /* check the first non-local */
    if ((*tmpctx).int_procs == NULL)
        return(NULL);
    for (ipbpt = (struct IPBLOCK *) (*tmpctx).int_procs; ipbpt != NULL; 
         ipbpt = (*ipbpt).flink)
    {
        if (s_equal ((*ipbpt).procname, name))
            return(ipbpt);
    }
    return (NULL);
}

/*      make_gbl. Move values of all PARMs to corresponding globals.
 *                This is "execution" of a GLOBAL PDF.
 *
 *      NOTE:   New variables are inserted to the global symbol table
 *              in alphabetically ascending order.
 *
 *      We don't have to chk_vector here: the declaration of
 *      a PARM in a GLOBAL PDF makes sure that the PARM and
 *      the corresponding global are compatible.  (See gbl_defaults
 *      in DECLARE.C.)
 */
FUNCTION static CODE make_gbl 
(
    struct CONTXT *pctx     /* in/out: global proc context */
)
{
    IMPORT struct SYMTAB glbtab;        /* global symbol table */
    IMPORT struct CONTXT primctx;       /* level 0 context */

    struct  VARIABLE    *pv;            /* current PARM */
    struct  VARIABLE    *ppv;           /* previous PARM in parmst */
    struct  VARIABLE    *lgv;           /* last global in glbtab */
    struct  VARIABLE    *gv;            /* current global */
    struct  DEFPDF      *defpdf;

    defpdf = NULL;
    /* previous pointer */
    ppv = (struct VARIABLE *) &(*pctx).parmst.link;
    for (pv=(*pctx).parmst.link; pv != NULL; )
    {
        gv = lookex (&glbtab, (*pv).v_name);
        /* define new global */
        if (gv == NULL)
        {
            if (lookex (&primctx.locst, (*pv).v_name) != NULL)
            {
                tmmsg (PROCFAIL,
                "Global variable name '%s' conflicts with level 0 variable.",
                "TAE-LOCCONFLICT", (*pv).v_name);
                /* previous PARM */
                ppv = pv;
                /* current PARM */
                pv = (*pv).v_link;
                continue;
            }
            /* unlink from parmst */
            (*ppv).v_link = (*pv).v_link;
            /* find slot in tbl */
            lgv = find_slot((*pv).v_name, &glbtab);
            if (lgv == NULL)
            {
                (*pv).v_link = glbtab.link;
                /* link at beginning */
                glbtab.link = pv;
            }
            else
            {
                (*pv).v_link = (*lgv).v_link;   
                /* link into glbtab */
                (*lgv).v_link = pv;
            }
            /* delete tutor extent */
            deltut(pv);
            (*pv).v_class = V_GLOBAL;           
            (*pv).v_protect = FALSE;
            (*pv).v_intrinsic = FALSE;
            (*pv).v_implicit = FALSE;
            (*pv).v_refcnt = 0;                 
            /* build DEFPDF */
            if (defpdf == NULL)
            {
                defpdf =
                    (struct DEFPDF *) tae_alloc (1, sizeof(struct DEFPDF));
                (*defpdf).refcount = 0;
                MOVE_STRUCT ((*pctx).pdf, (*defpdf).pdf);
            }               
            (*defpdf).refcount ++;
            (*pv).v_pdf = defpdf;                       
            /* next parm; ppv same */
            pv = (*ppv).v_link;
        }
        else
        {
            deep_value_copy (pv, gv, (*pv).v_name, (*gv).v_name);
            /* previous PARM */
            ppv = pv;
            /* current PARM */
            pv = (*pv).v_link;
        }
    }
    return (DO_SUCCESS);
}           

/*      deep_value_copy
 *
 *      Does a value copy from source variable to 
 *      destination variable, including qualifiers
 *      to any level.
 *
 *      The return code here is a chk_vector return code
 *      or SUCCESS.
 *      
 */
FUNCTION CODE deep_value_copy 
(
    struct VARIABLE *sv,        /* in: source variable */
    struct VARIABLE *dv,        /* in: dest variable */
    TEXT            sname[],    /* in: source name */
    TEXT            dname[]     /* in: dest name */
)
{
    CODE                code, worst_code;
    struct VARIABLE     *dvq;                   /* dest qualifier */
    struct VARIABLE     *svq;                   /* source qualifier */
    TEXT                qsname[STRINGSIZ+1];
    TEXT                qdname[STRINGSIZ+1];

    dv = RESOLVE(dv);
    code = chk_vector (dv, (*sv).v_type, (*sv).v_cvp, (*sv).v_count, FALSE);
    if (code == SUCCESS)
        set_value (dv, (*sv).v_cvp, (*sv).v_count);
    else
    {
        tmmsg (PROCFAIL, "Variable '%s' not compatible with variable '%s'.",
                "TAE-MISMATCH", dname, sname);
        return (code);
    }

    /*  Now do qualifiers (the "deep" part) */

    worst_code = SUCCESS;
    for (dvq = (*dv).v_qualst.link; dvq != NULL;  dvq = (*dvq).v_link)
    {
        svq = lookex (&(*sv).v_qualst, (*dvq).v_name);
        if (svq == NULL)
        {
            /* exact structure match not required */
            continue;
        }

        /* build variable names for recursive call to deep_value_copy */
        strcpy (qsname,sname);
        strcat(qsname,".");
        strcat(qsname,(*svq).v_name);
        strcpy (qdname,dname);
        strcat(qdname,".");
        strcat(qdname,(*dvq).v_name);

        code = deep_value_copy  (svq, dvq, qsname, qdname); 
        if (code != SUCCESS)
            worst_code = code;  
    }
    return (worst_code);
}

/*
 *      outputs.   Set output values from a process.
 *
 *      A process has sent a parblk containing output values.
 *
 */

FUNCTION CODE outputs
(
    struct CONTXT *procctx, /* in: proc context */
    struct PARBLK *parblk   /* in: PARBLK from process */
)
{
    COUNT               i;
    struct VARIABLE     *v;
    CODE                code;

    /* Strategy: the 'candidates' for output are global refs and parameters
     * of TYPE=NAME.  For each candidate, check to see if the parblk
     * contains a corresponding value.
     */

    for (i=0; i < (*procctx).numrefs; i++)
    {
        v = (*procctx).refs[i];
        code = set_outval(v, parblk);
        if (code != SUCCESS)
            goto outval_error;
    }
    for (v=(*procctx).parmst.link; v != NULL; v=(*v).v_link)
    {
        if ((*v).v_type == V_NAME)
        {
            code = set_outval(v, parblk);
            if (code != SUCCESS)
                goto outval_error;
        }
    }
    return (SUCCESS);

outval_error:
    tmmsg(PROCFAIL, "Invalid output value for '%s' in proc '%s'.",
        "TAE-INVOUT", (*v).v_name, (*procctx).pdf.name);
    return(FAIL);
}

/*
 *       prsstt - parse statement and build symbol table for this statement.
 *
 *      For context reasons, we don't do substitution here; the impact
 *      is that introductory commands cannot have substitution if
 *      HELP is to work.
 *
 */

FUNCTION CODE prsstt 
(
    TEXT          sttmt[CMDLINSIZ+1], /* in/out:  PDF statement*/
    struct ITRCMD **itrcmd,           /* out: ITRCMD for cmd */
    struct SYMTAB *pdfstb             /* out: cmd symbol table */
)
{
    struct SYNBLK       sb;                     /* syntax block */
    TEXT                cmd[FSPECSIZ+1];
    TEXT                subcmd[SUBCMDSIZ+1];
    TEXT                label[LABELSIZ+1];
    CODE                code;

    code = cmd_parse(&sb, sttmt, label, cmd, subcmd);
    if (code != SUCCESS)
        goto pd_synerr;
    /* find command in tables */
    *itrcmd = intrin(cmd);
    if (*itrcmd == NULL)        
        goto pd_niter;
    /* find subcommand */
    *itrcmd = itrsub(subcmd, *itrcmd);
    if (*itrcmd == NULL)
        goto pd_niter;
    if (memtab((**itrcmd).partab, (**itrcmd).numprm, pdfstb) != SUCCESS)
        return(FAIL);
    /* update from cmd string */
    if (updtab(pdfstb, &sb) != SUCCESS)
        return(FAIL);   
    return(SUCCESS);

pd_synerr:
    tmmsg(PROCFAIL, sb.errmsg, "TAE-FMTERR");
    return(FAIL);

pd_niter:
    tmmsg(PROCFAIL, "Unrecognized command '%s'.",
        "TAE-UNRECCMD", cmd);
    return(FAIL);
}

/*
 *      set_outval.    Set output variable if a corresponding
 *      value exists in the PARBLK.
 */

FUNCTION static CODE set_outval
(
    struct VARIABLE *v,     /* in/out: VARIABLE to set */
    struct PARBLK   *parblk /* in: PARBLK from process */
)
{
    struct VARIABLE     *pv, *rv;
    CODE                code;

    pv = lookex(&(*parblk).symtab, (*v).v_name);
    if (pv == NULL)
        return (SUCCESS);       
    rv = RESOLVE(v);    
    code = chk_vector(rv, (*pv).v_type, (*pv).v_cvp, (*pv).v_count, FALSE);
    if (code != SUCCESS)
        return (FAIL);
    code = set_value(rv, (*pv).v_cvp, (*pv).v_count);
    if (code != SUCCESS)
        return (FAIL);
    return (SUCCESS);
}

/*
 *       opnpdf - find PDF and open it.
 *
 *      Return code:
 *
 *      SUCCESS -- pdf opened successfully.
 *      FAIL --  pdf cannot be opened
 *
 *      Upon successful return, (*pctx).pdf is the FSBLOCK of the PDF.
 */

FUNCTION CODE opnpdf 
(
    struct CONTXT *parctx,     /* in: parent proc context */
    struct CONTXT *pctx,       /* in/out: proc context */
    TEXT          cmdstring[], /* in: command string:may be a file spec */
    struct SFILE  *file        /* out: the file control block */
)
{
    CODE                code;           
    struct CONTXT       *ctx;
    TEXT                errstr[STRINGSIZ+1];
    struct IPBLOCK      *ipbpt;
    TEXT                lc_cmdstring[CMDLINSIZ+1]; /* lower case command line */
    char *temp_ptr;

    /* make sure the cmdstring is lower case since all .pdf files are named
       with lower case.  Note: This was previously done at a lower level, but
       it would lower case the entire pathname, causing any paths with upper 
       case to break. Make sure that only the pdf name is lower-cased and not
       any path information. */
    strcpy(lc_cmdstring,cmdstring);
    temp_ptr = strrchr(lc_cmdstring,'/');
    if (temp_ptr == NULL)
        temp_ptr = lc_cmdstring;
    s_lower(temp_ptr);
    
    /* before hierarchy search, see if the proc is internal */
    if ((ipbpt = int_find (parctx, lc_cmdstring)) != NULL)
    {
        /* first find the file spec ("/LOCAL/" --> internal ) */
        for (ctx = parctx; (s_equal((*ctx).pdf.libr,"/LOCAL/")); 
             ctx = (*ctx).backlink);
        /* open it */
        code = f_opnblk (file, PDFLUN, &(*ctx).pdf, F_READ);
        if (code != SUCCESS)
        {
            tmmsg (PROCFAIL, "Error finding internal proc. Code '%s'.",
                        "TAE-INTPDF", (*file).errmsg);
            return (FAIL);
        }
        /* position to int'l proc */
        f_setpos (file, &(*ipbpt).position);
        (*file).posctx.possav = TRUE;
        /* retain file name */
        strcpy ((*pctx).pdf.name,(*ipbpt).procname);
        /* flag for local */
        strcpy ((*pctx).pdf.libr,"/LOCAL/");
        /* line # relative to int'l proc */
        (*pctx).pdf_line = 0;
        return (SUCCESS);
    }
    code = hierarchy(file, lc_cmdstring,PDF_TYPE, PDFLUN, &(*pctx).pdf, errstr);
    if (code == SUCCESS)
    {
        /* if terminal */
        if (f_isterm (file))
        {
            /* request TAE-PROCSYN err */
            code = FAIL;
        }
        /* TBD: fix this kludge */
        if (code == SUCCESS)
        {
            (*pctx).parfile = IsParFile (file);
            return(SUCCESS);
        }
    }
    /* syntax error in file spec: */
    if (code == FAIL)
    {
        tmmsg (PROCFAIL, "Error in proc name at or near '%s'.",
               "TAE-PROCSYN", errstr);
        return (FAIL);
    }
    else if (code == F_FILERR)
    {
        tmmsg(PROCFAIL, "Proc '%s' in '%s':  %s.",
             "TAE-PDFRD", (*pctx).pdf.name, (*pctx).pdf.libr,
             (*file).errmsg);
        return (FAIL);
    }
    else
    {
        tmmsg(PROCFAIL, "Unable to locate proc '%s'.", "TAE-NOPROC", 
              lc_cmdstring);
        return (FAIL);
    }
}

/*
 *      opnsav - open (actually, reopen) a proc and position to the previously
 *      saved position.
 *
 *      If pdf.libr is "\LOCAL\", the proc is internal, and we backlink
 */

FUNCTION CODE opnsav 
(
    struct CONTXT *pctx     /* in/out: proc context */
)
{
    IMPORT struct SFILE prcfil;   /* proc file context */

    CODE                code;
    TEXT                buf[STRINGSIZ+1];
    struct CONTXT       *ctx;


    /* first find the pdf spec */
    for (ctx = pctx; (s_equal((*ctx).pdf.libr,"/LOCAL/")); 
         ctx = (*ctx).backlink);
    code = f_opnblk(&prcfil, PDFLUN, &(*ctx).pdf, F_READ);
    /* reopen the proc */
    if (code != SUCCESS)
        return(code);
    /* now use the file position from the original, possibly internal, proc */
    /* posit to where we left off */
    f_setpos(&prcfil, &((*pctx).prcpos));
    /* mark file position known */
    prcfil.posctx.possav = TRUE;
    /* posit no longer saved */
    (*pctx).prcpos.possav = FALSE;
    /* & then to next record */
    code = f_read(&prcfil, buf);
    return(code);
}

/*
 *      pdftab - build symbol table from PDF.
 *      If a GLOBAL PDF, the parm_do function will define the globals along
 *      the way.
 */

FUNCTION CODE pdftab 
(
    struct CONTXT *pctx,    /* in:  proc context */
    struct SFILE  *file     /* in/out: PDF file control block */
)
{
    IMPORT struct CONTXT *curproc;
    IMPORT struct VARIABLE *sfi_gbl;

    TEXT                sttmt[CMDLINSIZ+1];  /* PDF statement */
    CODE                cmdcod;      /* command code */
    COUNT               records;     /* number records read for statement */
    COUNT               num_prec;    /* # comment, null lines preceding stmt*/
    struct CONTXT       *save_curproc;
    CODE                startmode;/*init search mode:SUB_SEARCH or NOT_SPECIAL*/
    CODE                code;

    startmode = (*pctx).special;
    code      = SUCCESS;
    save_curproc = curproc;
    /* so errors reported correctly */
    curproc   = pctx;
    while (FOREVER)
    {
        /* executing PAR file? */
        if ((*pctx).parfile)
        {
            code = prc_par_file (file, pctx);
            if (code != SUCCESS) 
                goto fail;
            break;
        }
        /* get statement */
        if (getpst(file, sttmt, &records, &num_prec, &(*pctx).prcpos) 
                    != SUCCESS)
            goto fail;
        /* adj for comment,null lines */
        (*pctx).pdf_line += num_prec;
        /* process the statement*/
        cmdcod = prccmd(sttmt, pctx);
        /* if PROCESS-COMPILED, etc. */
        if ((*pctx).compiled)
        {
            /* PDF is compiled -- process it */
            code = prc_compiled(file, pctx);
            if (code != SUCCESS)
                goto fail;
            break;
        }
        (*pctx).pdf_line += records - num_prec;
        if (IVAL(*sfi_gbl,0) < 0)
            goto fail;
        if ((*pctx).inbody || cmdcod == DO_RETURN)
        {
            /* end of declarations */
            break;
        }
        if (cmdcod != DO_SUCCESS)
            goto fail;
    }
    (*pctx).body_line = (*pctx).pdf_line;
    /* back to prev context */
    curproc = save_curproc;
    if (startmode != SUB_SEARCH  &&  startmode != TUTOR_ATTEMPT  &&
        startmode != COMPILING)
    {
        /* subcommand present? */
        if (NULLSTR((*pctx).subcmd)  &&  (*pctx).special != SUB_SEARCH)
        {
            if (!(*pctx).subfnd && (*pctx).subs)
            {
                tmmsg(PROCFAIL, "Subcommand required.", "TAE-SUBREQ");
                code = FAIL;    
            }
        }
        /* if mode changed by subcmd_do */
        else if ((*pctx).special == SUB_SEARCH)
        {
            (*pctx).special = NOT_SPECIAL;
            code = FAIL;
        }
        /* and not found... */
        else if (!(*pctx).subfnd)
        {
            tmmsg(PROCFAIL, "Unrecognized subcommand '%s'.", "TAE-UNRECSUB",
                (*pctx).subcmd);
            code = FAIL;
        }
        if (code == FAIL)
        {
            /* disp avail subcmds */
            msg_subs(pctx);
        }
    }
    else if (startmode == TUTOR_ATTEMPT)
    {
        /* subcommand present? */
        if (NULLSTR((*pctx).subcmd)  &&  (*pctx).special != SUB_SEARCH)
        {
            if (!(*pctx).subfnd && (*pctx).subs)
            {
                /* force tutor subc display*/
                (*pctx).special = SUB_SEARCH;
            }
        }
    }
    return(code);

fail:
    /* back to prev context */
    curproc = save_curproc;
    return (FAIL);
}

/* msg_subs - display a list of available subcommands.
 */

FUNCTION static VOID msg_subs 
(
    struct CONTXT *ctx      /* in:  proc context */
)
{
    struct SUBCMD       *s;
    TEXT                buf[STRINGSIZ+1];
    BOOL                still_adding;
    COUNT               len;

    static TEXT         inibuf[] = "Valid subcommands:  ";

    still_adding = TRUE;
    strcpy(buf,inibuf);
    for (s = (*ctx).subptr; s != NULL; s = (*s).link)
    {
        if (still_adding)
        {
            if (strlen((*s).name) + strlen(buf) + 5 > STRINGSIZ)
            {
                strcat(buf,"...");
                still_adding = FALSE;
            }
            else
            {
                strcat(buf,(*s).name);
                strcat(buf,", ");
            }
        }
    }
    if (still_adding  &&  (*ctx).subptr != NULL)
    {
        /* strip trailing ", " */
        len = strlen(buf);
        buf[len-2] = EOS;
    }
    if (!s_equal(buf, inibuf))
        tmmsg(PROCFAIL, buf, "TAE-SUBAVL");
}

/*
 *      procedure - interpret procedure.
 */

FUNCTION static CODE procedure 
(
    struct CONTXT *pctx,    /* in:  context for this procedure */
    struct ECB    *ecbi     /* in:  event blk for operator interr. */
)
{
    IMPORT struct SFILE prcfil;   /* proc file context */
    IMPORT struct VARIABLE *sfi_gbl, *skey_gbl;
    TEXT                cmdstr[CMDLINSIZ+1]; /* command string */
    struct VARIABLE     *on;                 /* onfail VARIABLE */
    CODE                cmdcod;              /* command return code */
    COUNT               records;
    COUNT               num_prec;  /* # comment, null lines preced sttmt */
    COUNT               onfail;              /* _ONFAIL index to execute */

    /* _ONFAIL VARIABLE */
    on = lookex(&((*pctx).locst), "_ONFAIL");
    cmdcod = DO_SUCCESS;
    while (FOREVER)
    {
        if ((e_occur(ecbi) && interrupt (pctx) == DO_ABORT)
                || cmdcod == DO_ABORT)
        {
            e_clear (ecbi);
            /* this proc handles ABORT? */
            if ((*on).v_count > 1)
                onfail = 1;                     
            else
            {
                /* set abort indicator */
                IVAL (*sfi_gbl, 0) = -10;
                /* set abort status */
                set_string (skey_gbl, "TAE-ABORT");
                /* pass up */
                return (DO_ABORT);
            }
        }
        else if (cmdcod != DO_SUCCESS &&  IVAL(*sfi_gbl,0) < 0) 
        {
            /* proc failed */
            onfail = 0;
        }
        else
        {
            /* command executed ok */
            onfail = -1;
        }
        /* execute _ONFAIL? */
        if (onfail >= 0)
        {
            strcpy (cmdstr,SVAL(*on, onfail));  
            /* check command */
            if (chk_onfail(cmdstr) != SUCCESS)
            {
                /* quit if bad command */
                return (DO_CHECK);
            }
            /* set executing onfail */
            (*pctx).onfailcmd = TRUE;
            /* process the command */
            cmdcod = prccmd(cmdstr, pctx);
            if (cmdcod == DO_CHECK && IVAL(*sfi_gbl,0) < 0)
                return (DO_CHECK);
        }
        else
        {
            /* execute next command */
            /* clear onfailcmd indicator */
            (*pctx).onfailcmd = FALSE;
            /* get cmd line */
            if (getpst(&prcfil, cmdstr, &records, &num_prec, &(*pctx).prcpos) 
                        != SUCCESS)
            {
                /* terminate if error */
                return (DO_CHECK);
            }
            /* adj for comment, null lines */
            (*pctx).pdf_line += num_prec;
            /* process the command */
            cmdcod = prccmd(cmdstr, pctx);
            (*pctx).pdf_line += records - num_prec;
        }
        if (cmdcod == DO_STOP)
            return (DO_STOP);
        /* convert for invoker */
        else if (cmdcod == DO_RETURN)
            return (DO_CHECK);

        /* NOTE: all other return codes are handled as
           DO_CHECK on next interation */
    }
}

#if 0 /*#ifndef PROCESS_CNP*/
/*
 *      process - run a TAE process.
 *
 *      This function sends a message received handshake to the subprocess
 *      if the message received was of type M_HLOGMSG.  This allows application
 *      software linked before TAE V1.2 to run without the handshake, since
 *      the older software uses the message code M_LOGMSG.
 *
 *      (The compilation of this function may be suppressed by
 *      defining PROCESS_CNP, in which case a host-dependent version
 *      may be written in a process.cnp module, for example, UNIX.)
 */

    FUNCTION CODE process (pctx, ecbi, errmsg)

    struct CONTXT       *pctx;          /* in:  proc context */
    struct ECB          *ecbi;          /* in/out: event blk for interrupts */
    struct CODE_ERR     *(*errmsg);     /* out: error msg if CODE is FAIL, else "" */

    {
    IMPORT CODE         run_type;       /* ASYNC or BATCH or INTER */
    IMPORT CODE         menu_screen;    /* menu screen status indicator */
    IMPORT CODE         usermode;       /* user mode: CMDMODE or MENUMODE */
    IMPORT struct VARIABLE *skey_gbl, *sfi_gbl;
#ifdef VAX_VMS
    IMPORT TEXT         *dcl_string;    /* ptr to dcl string to run */
    TEXT                *value[1];      /* scrach value vector */
#endif
    struct LARGE_PARBLK parblk;         /* message block to send to subprocess */
    struct TCB          tcb;            /* task control block (for c_ package) */
    CODE                comcod;         /* task wait completion code */
    CODE                status;
    TEXT                exespec[FSPECSIZ+1];
    struct FSBLOCK      fsblock;                
    CODE                code, code2, ret_code;
    TEXT                errstr[STRINGSIZ+1];
    TEXT                deflibe[FLIBRSIZ+1];
    struct CONTXT       *ctx;
    BOOL                asytut_req;     /* tutor for async job ? (false here) */
    struct VARIABLE     *_tutor;
    CODE                packageFlag;    /* used to package valid lists for */
                                        /* SELFTUTOR */        

/* error messages from function process().  We have them here because
   process cannot output them, having no standard output to output them to
 */

    static struct CODE_ERR er_over=
        {"Parameter values overflow capacity of message to block.",
        "TAE-PSETOVER", 0};

    static struct CODE_ERR er_ini=
        {"Unable to initiate process; host code = %d.",
        "TAE-PRCSINI", 0};

    static struct CODE_ERR er_notfnd =
        {"Process cannot be found or activated; host code = %d.",
        "TAE-PRCSACT", 0};

    static struct CODE_ERR er_term=
        {"Abnormal process termination; host code = %d.",
        "TAE-PRCSTRM", 0};

    static struct CODE_ERR er_imsg=
        {"Error sending initiation message to process; host code = %d.",
        "TAE-IMSGERR", 0};

    static struct CODE_ERR er_rcv=
        {"Error receiving message from process; host code = %d.",
         "TAE-RCVERR", 0};

    static struct CODE_ERR er_par=
        {"Message from process incorrectly formatted.",
         "TAE-PARBLK", 0};

    static struct CODE_ERR er_snd=
        {"Error sending message to process, host code = %d.",
         "TAE-SNDERR", 0};


/* first make sure we get a good library name by finding the parent pdf */
    for (ctx = pctx; (s_equal((*ctx).pdf.libr,"/LOCAL/")); ctx = (*ctx).backlink);
    strcpy (deflibe,(*ctx).pdf.libr);
    f_crack((*pctx).exe_spec, deflibe, (*pctx).pdf.name,
             EXE_TYPE, &fsblock, errstr);       /* apply defaults... */
    f_spec(&fsblock, exespec);                  /* build file spec to run */
    memset(&parblk,0,sizeof(parblk));
    parblk.last = FALSE;
    parblk.msgtyp = M_INIPAR;                   /* initial parameter block */
    _tutor = lookex (&(*pctx).locst, "_TUTOR");
    packageFlag = (_tutor && IVAL(*_tutor,0)) ? VM_VALID : 0; 
    if (package(pctx, &parblk, sizeof parblk.pool, packageFlag) != SUCCESS)
        goto rp_overr;                          
    code = c_act(&tcb, exespec);                /* activate task */
    if (code !=SUCCESS) goto rp_tiner;          /* error in queing for activation */
    code = c_snds(&tcb, (GENPTR) &parblk, parblk.blksiz);  /* send init msg blk*/
    if (code != SUCCESS) goto rp_msger;
    e_clear(&tcb.ecbrcv);                       /* clear receive efn */



/* Task started.  Now wait on completion, event, or message.
 * NOTE: We queue to receive the next message from subtask only
 * after a send is complete, or after a message is received from it.
 * So a subtask can not send a message to tm until it has read the
 * block sent to it from tm. (An attempt to do so will hang the subtask).
 */

    *errmsg = NULL;                             /* no err msg yet */
    ret_code = DO_CHECK;
    while (FOREVER)
        {
        e_wait(4, &tcb.ecbsnd, &tcb.ecbrcv, &tcb.ecbtsk, ecbi);
                        /* wait on send, receive, completion, or event */

        if (e_occur(&(tcb.ecbtsk)))             /* if task completion, no more loop */
            {
            if (usermode == MENUMODE)           /* if process activate by menu */
                menu_screen = PROMPT_PAINT;     /* prompt before repaint menu */
#ifdef VAX_VMS
            if (dcl_string != NULL)             
                {                               /* DCL command finished */
                value[0] = &(tcb.statmsg[2]);   /* (ignore the %x on front) */
                set_value(skey_gbl, (GENPTR)value, 1);  /* move $STATUS to $SKEY */
                IVAL(*sfi_gbl,0) = (tcb.compcd == C_NORMAL) ? 1 : -1;
                break;
                }
#endif
            if (ret_code == DO_ABORT || tcb.compcd == C_NORMAL)
                break;                  /* normal or expected abnormal */
            goto rp_wterr;              /* unexpected abnormal termination */
            }
        else if (e_occur(ecbi))                 /* operator attention */
            {
            e_clear (ecbi);                     /* clear the event */
            if ((*pctx).interrupt)              /* interrupt allowed? */
                {
                c_suspend(&tcb);
                code = interrupt (pctx);        /* do interrupt commands */
                if (code == DO_ABORT)
                    {
                    /* here we initiate an abort but continue the while loop
                       so that messages from exit handlers in the process
                       will be received (especially, output variables). */
                    if (c_iabort(&tcb) != SUCCESS)              /* initiate abort */
                        m_put("Unable to abort process; host code = %d.",
                              "TAE-PRCSABT", tcb.hostcode);
                    e_clear(ecbi);      /* so nested levels won't hit proc int */
                    IVAL (*sfi_gbl, 0) = -10;
                    set_string (skey_gbl, "TAE-ABORT");
                    ret_code = DO_ABORT;                        /* return code */
                    }
                else
                    c_resume(&tcb);
                }
            }
        else if (e_occur(&tcb.ecbsnd))                  /* if a send */
            {
            e_clear(&tcb.ecbsnd);                       /* clear  ECB */
            code = c_recs(&tcb, (GENPTR)&parblk, sizeof parblk);
            if (code != SUCCESS) goto rp_rcver;
            }
        else if (e_occur(&tcb.ecbrcv))                  /* a message received */
            {
            e_clear(&tcb.ecbrcv);
            makeabs(&parblk.symtab, parblk.pool);       /* make pointers absolute*/
            if (chk_parblk (&parblk) != SUCCESS)
                goto rp_badparblk;
            if (parblk.msgtyp == M_FULLPDF  ||  parblk.msgtyp == M_SUBPDF)
                {
                if (run_type == ASYNC)
                    {
#ifdef TAE_ASYNC
                    code = get_pblk_remote (pctx, &parblk, errmsg);
                                                /* get parms from parent mon */
                    if (code != SUCCESS) goto rp_rmerr;
#endif
                    }
                else
                    {
                    asytut_req = FALSE;                 /* tutor for sync proc */
                    dynget(pctx, &parblk, sizeof ( parblk.pool ), 
                                    asytut_req);        /* dynamic parameter */
                    }
                if (parblk.msgtyp == M_DYNEXIT)         /* see change log */
                    parblk.msgtyp = M_KILLED;           /* for comments */
                code = c_snds(&tcb, (GENPTR)&parblk, parblk.blksiz);
                if (code != SUCCESS) goto rp_dmser;
                }
            else if (parblk.msgtyp == M_COMMAND)        /* execute a command line */
                {
                dyncommand (pctx, &parblk, sizeof parblk.pool);
                code = c_snds(&tcb, (GENPTR)&parblk, parblk.blksiz);
                if (code != SUCCESS) goto rp_msger;
                }
            else if (parblk.msgtyp == M_LOGMSG  ||  parblk.msgtyp == M_HLOGMSG)
                {
                logmsg(pctx, &parblk);          /* log the message */
                if (parblk.msgtyp == M_HLOGMSG)
                    {                           /* handshake */
                    parblk.msgtyp = M_CONTINUE;
                    makerel(&parblk.symtab, parblk.pool);   /* make relative */
                    code = c_snds(&tcb, (GENPTR)&parblk, parblk.blksiz);
                    if (code != SUCCESS) goto rp_msger;
                    }
                else                            /* for old applic compatibility */
                    {
                    code = c_recs(&tcb, (GENPTR)&parblk, sizeof parblk);
                    if (code != SUCCESS) goto rp_rcver;
                    }
                }
            else if (parblk.msgtyp == M_OUTPUT)
                {
                outputs(pctx, &parblk);         /* apply output values */
                code = c_recs(&tcb, (GENPTR)&parblk, sizeof parblk);
                if (code != SUCCESS) goto rp_rcver;
                }
            else
                {
                dynamic_ins (pctx, &parblk);    /* installation exit */
                code = c_recs(&tcb, (GENPTR)&parblk, sizeof parblk);
                if (code != SUCCESS) goto rp_rcver;
                }
            }
        }
    return (ret_code);  

rp_overr:
    *errmsg = &er_over;
    return (FAIL);

rp_tiner:
    *errmsg = &er_ini;
    er_ini.code = tcb.hostcode;
    return(FAIL);

rp_wterr:
    if (tcb.compcd == C_ACTERR)
        {
        *errmsg = &er_notfnd;
        er_notfnd.code = tcb.hostcode;
        }
    else
        {
        *errmsg = &er_term;
        er_term.code = tcb.hostcode;
        }
    return(FAIL);

rp_msger:
    *errmsg = &er_imsg;
    er_imsg.code = tcb.hostcode;
    c_abort(&tcb);
    return(FAIL);

rp_rcver:
    *errmsg = &er_rcv;
    er_rcv.code = tcb.hostcode;
    c_abort(&tcb);
    return(FAIL);

rp_badparblk:
    *errmsg = &er_par;
    c_abort(&tcb);
    return(FAIL);

rp_dmser:
    *errmsg = &er_snd;
    er_snd.code = tcb.hostcode;
    c_abort(&tcb);
    return(FAIL);


rp_rmerr:               /* error on getting parms from remote monitor */
    c_abort(&tcb);
    return(FAIL);
    }
#endif

/*      run_proc - run a proc.
 *
 *      Note that if tutor runs from deep level, run_proc will not
 *      produce a session log of the parameters supplied -- this is
 *      because we only log primary level.
 */

FUNCTION CODE run_proc 
(
    struct CONTXT *pctx     /* in:  proc context */
)
{
    IMPORT struct CONTXT  *curproc; /* current proc context (for errs) */
    IMPORT struct ECB   ecbi;       /* event block for operator interrupts */
    struct CODE_ERR     *errmsg;    /* ptr to an errmsg defined statically */
    CODE                code;
    CODE                code1, code2;
    static int processRunning = FALSE;

    /* Log the running of this PDF */
    pdflog(&pctx->pdf);

#ifdef SESSION_LOG
    /* if activating from level 0 */
    if ((*pctx).prclevel <= 1)
    {
        /* log the activation */
        slproc(pctx);
    }
#endif
    /* installation exit */
    pa_ins (pctx);
    code = DO_CHECK;
    /* set new stdout */
    code1 = chg_stdout(pctx);
    if (code1 != SUCCESS)
    {
        /* error exit */
        goto reset;
    }
    /* if a procedure */
    if ((*pctx).proctype == Y_PROCEDURE)
    {
        /* for error messages */
        curproc = pctx;
        /* interpret procedure */
        code = procedure(pctx, &ecbi);
        /* nest up */
        curproc = (*pctx).backlink;
    }
    else if ((*pctx).proctype == Y_PROCESS)
    {
        /* proc is a process */
        if (processRunning)
        {
            tmmsg (PROCFAIL,"Attempt to nest process execution.", "TAE-NOSYNC");
            return (DO_CHECK);  
        }
        processRunning = TRUE;
        /* run the process */
        code = process(pctx, &ecbi, &errmsg);
        processRunning = FALSE;
    }
    else if ((*pctx).proctype == Y_GLOBAL)
    {
        /* proc is GLOBAL, make parms globals */
        code = make_gbl (pctx);
    }
    else if ((*pctx).proctype == Y_PARMSET)
    {
        /* honor ACTION= */
        code = action_run (pctx);
    }

reset:
    /* reset stdout */
    code2 = re_stdout(pctx);
    /* installation exit */
    pt_ins (pctx);
#ifdef SESSION_LOG
    /* if activated from level 0 */
    if ((*pctx).prclevel <= 1)
    {
        /* log the termination */
        slterm();
    }
#endif
    /* stdout related error */
    if (code1 != SUCCESS || code2 != SUCCESS)
        return(DO_CHECK);       
    /* if process was run, saved errmsg */
    if ((*pctx).proctype == Y_PROCESS && code ==FAIL)
        tmmsg(PROCFAIL, (*errmsg).text, (*errmsg).key, (*errmsg).code);

    /*  
     *  A proc only returns one of the following codes:
     *
     *     DO_STOP means "return to primary level".
     *     DO_CHECK means invoker must check the $SFI/$SKEY.
     *     DO_ABORT means pi mode ABORT has been done.
     *
     *  The following code is for safety to enforce this rule.
     *
     *  Note that a proc does not have the capability to return DO_RETURN
     *  or DO_SUCCESS -- these are special codes reserved for intrinsics.
     */

    if (code == DO_STOP || code == DO_ABORT)
        return (code);
    return (DO_CHECK);
}
