/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

 /*<<<<<<<<<<<<    PROCESS ACTIVATION UNDER UNIX    >>>>>>>>>>>>>
 *
 *      Open issues:
 *
 *      Does the interrupt program change any statics, like
 *      cmd_mode, etc.?  If so, the longjump could screw things
 *      up.
 *
 *
 *      When pipe_sig gets an EXIT return from dyncmd, it must
 *              flag primary level (via longjmp?).
 *
 *      Need more work here so that when the Abort/continue question
 *              is active, packet traffic discontinues.
 *
 *
 *      
 *      HOW THE TASK PACKAGE WORKS UNDER UNIX:
 *
 *      The coding here is very delicate w.r.t. missing signals, race
 *      conditions, etc: be careful when making changes.
 *      
 *      Two pipes are established: one to send to the subprocess and
 *      one to receive from the subprocess.  The parent starts the
 *      ball rolling by sending an initial message to the subprocess.
 *      When the process wants to send a message to parent, it hits
 *      signal 16 for the parent.  The parent then reads the "from"
 *      pipe.
 *
 *      Messages from the subprocess are received and processed in the
 *      signal catcher for the pipe signal (16).  We do it in the
 *      catcher to avoid race problems with re-signalling on the primary
 *      level.
 *
 *      When an EXIT is recieved from a message processor (like dyncmd),
 *      the signal catcher forces back to primary level with a longjmp
 *      so a proper return to run_proc can be made.
 *
 */

/*
 *      process - run a TAE process.
 *
 *      (The compilation of this function may be suppressed by
 *      defining PROCESS_CNP, in which case a host-dependent version
 *      may be written in a process.cnp module, for example, UNIX.)
 */


/*
 *      CHANGE LOG:
 *
 *      10-nov-83       Make arguments to execl strings...dm
 *      12-dec-83       Return host code for TAE-PRCSCOM...dm
 *                      Also close open pipes before return...dm
 *      13-dec-83       Ignore signal for broken pipe...dm
 *      23-apr-84       Put pipe numbers in environment list,
 *                      implement debugging...dm
 *      29-apr-84       Change for 4.2 implementation...dm
 *      18-jun-84       Move BSD_4P2 symbol defition to TAECONF...dm
 *      03-jul-84       Make the default debugger "dbx" for BSD 4.2...dm
 *      01-aug-84       Make get_pblk_remote call conditional...dm
 *      16-jun-85       Allow M_HLOGMSG  msg type under non-BSD_4.2 version,
 *                      (bug fix), update for TAE V1.3 implementation...dm
 *      01-jul-85       Fix UNIX compilation errors...dm
 *      10-jul-85       Update calling seq for logmsg() and get_pblk_remote()
 *                      functions...dm
 *      25-jul-85       Fix UNIX lint errors...dm
 *      15-aug-85       Save child_pid so that child can be aborted when
 *                      parent dies (via a SIGTERM or SIGQUIT signal)...dm
 *      20-sep-85       Return FAIL if error return in process()...dm
 *      04-dec-86       PR 961: Indicate to 'dynget' that we're tutoring for
 *                      sync proc (to prevent HOLD)...peb
 *      04-dec-86       Pass current context to interrupt from process...peb
 *      04-dec-86       PR 1100: Before returning, check for entry via menu.  
 *                      If so, request prompt before repaint...peb
 *      12-dec-86       Check return code from outputs()...peb
 *      21-jan-87       Check block size for write to pipe...lia
 *      24-nov-87       Add size in call to dynget...tpl
 *      25-nov-87       Add size in call to package...tpl
 *      03-mar-88       Fix race condition where first SIGREAD comes
 *                      in right after the initial write of the initial
 *                      PARBLK before setjmp is called.  The signal processor
 *                      uses the context block left over from the
 *                      previous execution of process.  The fix has been
 *                      implemented for the BSD 4.2 code only...palm
 *
 *                      The fix consists of writing the first message AFTER 
 *                      setjmp has been done.    Then if the message comes
 *                      in immediately, we are ready.
 *      08-may-88       Removed #ifndef PROCESS_CNP since it does nothing..tpl
 *      26-jun-88       Added remark to act_task() on socket creation...ljn
 *      07-oct-88       Use sizeof(PARBLK) rather than P_BYTES on read...nhe
 *      08-nov-88       -vfork() defined for SYSV in taeconf.inp
 *                      -#ifdef BSD_UNIX -> #ifndef SYSV...ljn
 *      09-nov-88       The application debugging problems appear to occur
 *                      only on the Sun. Ergo, #ifdef sun...ljn
 *      10-nov-88       -Cut out BSD 4.1 version; removed all BSD_4P2 defs
 *                      -#ifdef SV_BSDSIG and sigvector() for hpux...ljn
 *      01-dec-88       new package calling sequence; package valids for
 *                      SELFTUTOR...palm
 *      09-feb-89       Zero parblk header before sending to process...palm
 *      01-may-89       new M_COMMAND msg...palm
 *      27-jun-89       -Restored BSD 4.1 code, but use #ifndef NOSIGVEC.
 *                      -Applied PAM's timing bug fix to NOSIGVEC version...ljn
 *      24-aug-89       #include vfork.h to cure sparc problems debugging
 *                      applications...ljn
 *      17-jan-90       Made a FUNINT a void to quell compiler warning...ljn
 *      27-sep-91       Removed special handling of hpux signals...ljn
 *      04-sep-92 PR 871,444 Added multiple read/write for large par block...tpl
 *      06-oct-92       SGI port.  Conditionally defined the simple macro
 *                      _BSD_SIGNALS before the <signal.h> header file is
 *                      referenced, so that sigvec is defined...bth
 *      08-dec-93       Solaris port (backported dag from aug-93)...cob
 *      07-nov-95       Combine modifications from Richard White at Penn State
 *                      University and Brian Davis from EROS Data Center.
 *                      Jim Fenno - EROS Data Center.
 */


#include "stdh.inp"
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>
#include <setjmp.h>
#include <string.h>
#include <stdio.h>

#if defined(sparc) && !(defined(sun) && OSMajorVersion >= 5)
#include <vfork.h>
#endif 

#ifdef SYSV
#include <unistd.h>
#endif

#if defined(sun) || defined(DGUX) || defined(sgi) || defined(linux)
#include        <sys/types.h>
#include        <sys/socket.h>
#endif

#include "taeconf.inp"
#include "inttaelib.h"
#include "taskinc.inp"
#include "eventinc.inp"
#include "local_protos.h"
#include "tminc.inc"
#include "parblk.inc"

#define WRITECHUNK min(PIPESIZ, parblk.blksiz-already_wrote)

/* error messages from function process(). We have them here because
   process can not output them, having no standard output to output them to.
 */

static struct CODE_ERR er_over=
    {"Parameter values overflow capacity of message to block.",
    "TAE-PSETOVER", 0};

/*static struct CODE_ERR er_size=
    {"Initiation message to process too large for pipe.", "TAE-SIZERR", 0};
*/
static struct CODE_ERR er_ini=
    {"Unable to initiate  the process, host code = %d.",
        "TAE-PRCSINI", 0};

static struct CODE_ERR er_comini=
    {"Cannot establish communication with the process, host code = %d.",
        "TAE-PRCSCOM", 0};

static struct CODE_ERR er_term=
    {"Abnormal process termination; process status code = %d.",
    "TAE-PRCSTRM", 0};

static struct CODE_ERR er_imsg=
    {"Error sending initiation message to process; host code = %d.",
    "TAE-IMSGERR", 0};

static struct CODE_ERR er_out=
    {"Error in process output values.", "TAE-INVOUT", 0};

static struct CODE_ERR er_rcv=
    {"Error receiving message from process; host code = %d.",
     "TAE-RCVERR", 0};

static struct CODE_ERR er_par=
    {"Message from process incorrectly formatted.",
      "TAE-PARBLK", 0};

/*static struct CODE_ERR er_toolg=
    {"Message to process too large for pipe.", "TAE-SIZERR", 0};
*/
static struct CODE_ERR er_snd=
    {"Error sending message to process, host code = %d.",
     "TAE-SNDERR", 0};

static struct CODE_ERR er_acknsnd=
    {"Error sending handshake message to process; host code =  %d.",
    "TAE-SNDERR", 0};

GLOBAL    TAEINT  sync_child = 0;   /* pid of child running synchronously, 
                                     * zero implies no such child active */


#ifdef EROS
IMPORT struct SYMTAB glbtab;        /* TCL global symbols           */
#endif

#ifdef DEBUG_COMM
#define TRACE_COMM(x) printf(x)
#else
#define TRACE_COMM(x)
#endif

static    BOOL  opwait;         /* true if operator int waiting     */
static    int   chldint;        /* true if chld int received */
static int sigwinch_blocked=0;  /* flag indicates SIGWINCH signal is blocked */

VOID    int_sig(int);           /* handle interrupt signals         */
VOID    pipe_sig(int);          /* handle pope io signals           */
FUNCTION CODE unblock_sig(int signum);
static VOID chld_sig(int);
static VOID c_suspend(int);
static VOID c_resume(int);
FUNCTION CODE act_task
(
    TEXT        task_name[], /* in:  task name */
    FUNINT      parent_pid,  /* in:  parent pid */
    TEXT        *envtbl[],   /* in: environment table */
    FUNINT      *read_pipe,  /* out: read end of up-pipe */
    FUNINT      *write_pipe, /* out: write end of down-pipe */
    FUNINT      *child_pid   /* out: process id of activated task */
);


FUNCTION CODE process 
(
    struct CONTXT       *pctx,      /* in:  proc context */
    struct ECB          *ecbi,      /* in/out: event blk for interrupts */
    struct CODE_ERR     *(*errmsg)  /* out: errmsg if code FAIL, else "" */
)
{
    IMPORT  TEXT        **environ;      /* shell environment list            */
    IMPORT  CODE        menu_screen;    /* menu screen status indicator */
    IMPORT  CODE        usermode;       /* user mode: CMDMODE or MENUMODE*/
    IMPORT  CODE        run_type;       /* INTER, ASYNC or BATCH             */
    struct LARGE_PARBLK parblk;         /* message block to send to task     */
    CODE                status;
    TEXT                exespec[FSPECSIZ+1];
    struct FSBLOCK      fsblock;
    TAEINT              pid, child_pid; 
    TAEINT              read_chan, write_chan;  /* read/write channels      */
    CODE                code, ret_code;
    COUNT               count;
    BOOL                asytut_req;
    LONG                ackn_msg;               /* max 4 bytes of ackn msg */
    CODE                pr_stat;                /* process status          */
    CODE                code1;
    TEXT                errstr[STRINGSIZ+1];
    struct CONTXT       *ctx;
    struct VARIABLE     *_tutor;
    CODE                packageFlag;            /* used to request that valids*/
                                                /* be packaged for SELFTUTOR  */
    int aborting;  /* flag to indicate the process was aborted by the user */

    COUNT        already_read;
    COUNT        already_wrote;
    GENPTR       p;
    char *temp_ptr;

    /* clear the aborting flag */
    aborting = 0;
    chldint = 0;
    
    /* First make sure we get a good library name by finding the parent pdf */
    for (ctx = pctx; (s_equal((*ctx).pdf.libr,"/LOCAL/")); 
        ctx = (*ctx).backlink);
    f_crack((*pctx).exe_spec, (*ctx).pdf.libr, (*pctx).pdf.name,
             EXE_TYPE, &fsblock, errstr);       /* apply defaults...       */
    f_spec(&fsblock, exespec);                  /* build file spec to run  */

    /* force the process name to lowercase while leaving the rest of the path
       name in the specified case.  This is to allow directory names with 
       uppercase letters to be used.  However, pdf and executable names still 
       need to be lower case. */
    temp_ptr = strrchr(exespec,'/');
    if (temp_ptr == NULL)
        temp_ptr = exespec;
    s_lower(temp_ptr);

    memset(&parblk,0,sizeof(parblk));
    parblk.last = FALSE;
    parblk.msgtyp = M_INIPAR;                   /* initial parameter block */
    _tutor = lookex (&(*pctx).locst, "_TUTOR");
    packageFlag = (_tutor && IVAL(*_tutor,0)) ? VM_VALID : 0;

    /* build the parblk */
    if (package(pctx, (struct PARBLK*)&parblk, sizeof parblk.pool, packageFlag)
        != SUCCESS)
        goto rp_overr;  

    read_chan = -1;                             /* initialize to -ve value */
    write_chan = -1;                    
    pid = getpid();                             /* process id of tm        */
    code = act_task(exespec, pid, environ,&read_chan,&write_chan,&child_pid);
    if (code != SUCCESS)
        goto rp_tiner;                          /* task initiation error   */
    sync_child = child_pid;                     /* save child's pid        */

    p = (GENPTR)&parblk;
    set_signal (SIGPIPE, SIG_IGN);              /* don't quit on broken pipe */

    *errmsg = NULL;                     /* no error msg yet             */
    ret_code = DO_CHECK;                /* error return                 */
    ackn_msg = ACKN_CODE;               /* acknowledgement code for subtask*/
    set_signal (SIGINT, int_sig);       /* enable operator interrupt    */
    
    /* ignore the old signal that used to be used by child processes in 
       case there are old children that have not been recompiled to with the
       current tae libraries.  This should be deleted eventually */
    set_signal (SIGREAD, SIG_IGN);

    /*
     *  code1 = 1 for normal pipe i/o
     *  code1 = 2 for interrupt
     *  Be very careful when making changes in this loop.
     *  Make sure functions in $TAESRSTAE/lib/tae/taskpro.np.c are changed
     *  accordingly if you made any changes here.
     */
    opwait = FALSE;

    /* write first chunk of parameter block to child process */
    already_wrote = 0;
    already_read = 0;
    count = write(write_chan, &p[already_wrote], WRITECHUNK );
    if (count <= 0 ) 
            goto rp_msger;
    already_wrote += count;
    if ( already_wrote == parblk.blksiz )   /* all done */ 
        already_wrote = 0;
    else if (already_wrote > parblk.blksiz ) 
        goto rp_msger;

    goto WAITLOOP;                          /* continue to  wait for msg */

process_message:
    if (code1 == 1)                        /* message arrived           */
    {
        if ( already_wrote > 0 )        /* write some more of the parblk */
        {
            count = write(write_chan, &p[already_wrote], WRITECHUNK );
            if ( count <= 0 )
                goto rp_msger;
            already_wrote += count;

            if ( already_wrote == parblk.blksiz )       /* all done? */
                already_wrote = 0;
            else if ( already_wrote > parblk.blksiz )
                goto rp_msger;
            
            goto WAITLOOP;                      /* continue to  wait for msg */
        }
        else
        {
            /* read msg from proc, possibly a continuation */
            count = read(read_chan, &p[already_read], 
                sizeof(parblk)-already_read ); 
            already_read += count;
            if (count < 0) 
                goto rp_rcver;      
            else if (count == 0)
            {
                /* if no bytes are read from the pipe, the child process
                   has exited */
                TRACE_COMM("tae: read zero bytes from pipe\n");
                chldint = 1;
                goto WAITLOOP;
            }
            if ( already_read < parblk.blksiz )  
            {
                goto WAITLOOP;                  /* continue to  wait for msg */
            }
            if ( already_read > parblk.blksiz ) 
                goto rp_rcver;
            if ( already_read ==  parblk.blksiz ) /* got it all, process it */
                already_read = 0;
        }

        /*******   now process the message    ************/

        /* make pointers absolute    */
        makeabs(&parblk.symtab, parblk.pool);
        if (chk_parblk ((struct PARBLK *)&parblk) != SUCCESS)
            goto rp_badparblk;

        if (parblk.msgtyp == M_FULLPDF  ||  parblk.msgtyp == M_SUBPDF)
        {
            if (run_type == ASYNC)
            {
#ifdef TAE_ASYNC
                code = get_pblk_remote(pctx, (struct PARBLK *)&parblk, errmsg);
                                                /* get parms from parent mon */
                if (code != SUCCESS) goto rp_rmerr;
#endif
            }
            else
            {
                asytut_req = FALSE;             /* tutor for sync proc  */
                                                /* dynamic parameter    */
                dynget(pctx, (struct PARBLK *)&parblk, sizeof ( parblk.pool), 
                        asytut_req);
                /* log dynamic parameters */
                logdyn((struct PARBLK *)&parblk); 
            }

            /* send ackn */
            count = write(write_chan, &ackn_msg, ACKN_SIZE);    
            if ( count <= 0 )
                goto rp_dmser;

            /* now send parameter block  */
            already_wrote = 0;
            count = write(write_chan, &p[already_wrote], WRITECHUNK );
            if ( count <= 0 )
                goto rp_dmser;
            already_wrote += count;
            if ( already_wrote == parblk.blksiz )
                already_wrote = 0;
        }
        else if (parblk.msgtyp == M_COMMAND)
        {
            /* execute a command line */
            dyncommand (pctx, (struct PARBLK *)&parblk, sizeof parblk.pool);

            /* send ackn */
            count = write(write_chan, &ackn_msg, ACKN_SIZE);    

            /* now send parameter block  */
            already_wrote = 0;
            count = write(write_chan, &p[already_wrote], WRITECHUNK );
            if ( count <= 0 )
                goto rp_dmser;
            already_wrote += count;
            if ( already_wrote == parblk.blksiz )
                already_wrote = 0;
        }
        else if (parblk.msgtyp == M_LOGMSG || parblk.msgtyp == M_HLOGMSG)
        {
            /* log the message  */
            logmsg(pctx, (struct PARBLK *)&parblk);

            /* send ackn */
            count = write(write_chan, &ackn_msg, ACKN_SIZE);

            if (parblk.msgtyp == M_HLOGMSG)
            {
                /* handshake */
                parblk.msgtyp = M_CONTINUE;
                makerel(&parblk.symtab, parblk.pool);
                already_wrote = 0;
                count = write(write_chan, &p[already_wrote], WRITECHUNK );
                if ( count <= 0 )
                    goto rp_mgser;
                already_wrote += count;
                if ( already_wrote == parblk.blksiz )
                    already_wrote = 0;
            }
        }
        else if (parblk.msgtyp == M_OUTPUT)
        {
            /* apply output values       */
            code = outputs(pctx, (struct PARBLK *)&parblk);
            if (code != SUCCESS) 
                goto rp_outerr;

            /* send ackn */
            count = write(write_chan, &ackn_msg, ACKN_SIZE);    
        }
        else
        {
            /* installation exit */
            dynamic_ins (pctx, (struct PARBLK *)&parblk);

            /* send ackn */
            count = write(write_chan, &ackn_msg, ACKN_SIZE);
        }
    }

    if (code1 == 2 || opwait)          /* if ^C occured                 */
    {
        opwait = FALSE;                 /*  no more waiting              */

        /* interrupt allowed? */
        if ((*pctx).interrupt)
        {
            /* suspend the task */
            c_suspend(child_pid);

            /* do interrupt processing */
            code = interrupt(pctx);
            if (code == DO_ABORT)       /* this dialog will be aborted   */
                                        /* if signal comes in the        */
                                        /* the middle                    */
            {
                /* when aborting, attempt a SIGTERM first to allow the 
                   application to catch it and clean up after itself.
                   If the user aborts a second time, use SIGKILL. */
                if (aborting != 0)
                {
                    /* already tried SIGTERM on the process, so KILL it 
                       this time in case it is really locked up */
                    kill(child_pid, SIGKILL);
                }
                else
                {
                    aborting = 1;
                    /* resume suspended task */
                    c_resume(child_pid);

                    /* abort the task by sending it a SIGTERM to let it clean
                       up after itself */
                    kill(child_pid, SIGTERM);
                }
            }
            else
                c_resume(child_pid);    /* resume suspended task        */
        }

        /* enable operator intrpt since it was disabled when ^C was received */
        set_signal (SIGINT, int_sig);
    }

    /*  now wait for the process to terminate       */
WAITLOOP:
    while (FOREVER)     
    {
        /* variables for calling select */
        int max_fd = 0;
        fd_set read_set;
        fd_set write_set;

        /* zero out the descriptor sets for the select */
        FD_ZERO(&read_set);
        FD_ZERO(&write_set);

        /* add the read channel to the read file descriptor set and update the
           count of max_fd for the select call */
        FD_SET(read_chan,&read_set);
        max_fd = read_chan + 1;

        /* if partially through sending a block to the child process, add 
           the write channel to the write set of file descriptors */
        if (already_wrote > 0)
        {
            FD_SET(write_chan,&write_set);
            if (write_chan >= max_fd)
                max_fd = write_chan + 1;
        }

        /* wait for the pipe to be ready to write or read, with not 
           exception descriptors and no time out */
        code = select(max_fd,&read_set,&write_set,NULL,NULL);
        if ((code < 0) || chldint)
        {
            /* error on select.  Check errno to see what action to take */
            if ((errno == EINTR) || chldint)
            {
                /* signal received, so check which one */
                if (chldint)
                {
                    chldint = 0;

                    /* child signal received, so reap the child */
                    code = wait(&status);

                    /* if the pid that was reaped is the child process,
                       deal with it */
                    if (code == child_pid)
                    {
                        if (aborting)
                        {
                            /* aborting, so set abort return code and exit */
                            ret_code = DO_ABORT;
                            goto close_ret;
                        }
                        /* mark that no child active */
                        sync_child = 0;

                        /*low byte is process status */
                        pr_stat = (status & 0177);

                        /* if process stopped (interrupted) ignore the signal */
                        if (pr_stat == 0177)
                            continue;

                        /* if process activated by menu prompt before repaint 
                           menu*/
                        if (usermode == MENUMODE)
                            menu_screen = PROMPT_PAINT;

                        /* if not normal termination */
                        if (pr_stat != 0)
                            goto rp_wterr;

                        /* child terminated so break out of the loop */
                        break;
                    }
                }
                else if (opwait)
                {
                    code1 = 2;
                    goto process_message;
                }
                else 
                {
                    /* why would we be here? */
                    TRACE_COMM("weird interrupt\n");
                }
            }
            else
            {
                /* unexpected errno set by select.  Issue a warning message 
                   and exit */
                printf("ERROR: TAE received errno %d from select\n",
                       errno);
                exit (1);
            }
        }

        code1 = 1;
        goto process_message;
    }

    /* unblock the window resizing signal if it was blocked for debug mode */
    if (sigwinch_blocked)
    {
        unblock_sig(SIGWINCH);
        sigwinch_blocked = 0;
    }

    close (read_chan);                          /* close the pipes      */
    close (write_chan);
    t_attn(ecbi);                               /* reset oper int handler */
    return (DO_CHECK);

    rp_overr:
        *errmsg = &er_over;
        return (FAIL);

/*    rp_sizerr:
        *errmsg = &er_size;
        return (FAIL);
*/
    rp_tiner:
        if (child_pid == -1)
        {
            *errmsg = &er_ini;
            er_ini.code = errno;
        }
        else
        {
            *errmsg = &er_comini;
            er_comini.code = errno;
        }
        ret_code = FAIL;
        goto close_ret;

    rp_wterr:
        *errmsg = &er_term;
        er_term.code = (status&0177);
        ret_code = FAIL;
        goto close_ret;

    rp_msger:
        *errmsg = &er_imsg;
        er_imsg.code = errno;
        kill(child_pid, SIGKILL);
        ret_code = FAIL;
        goto close_ret;

    rp_outerr:
        *errmsg = &er_out;
        kill (child_pid, SIGKILL);
        ret_code = FAIL;
        goto close_ret;

    rp_rcver:
        *errmsg = &er_rcv;
        er_rcv.code = errno;
        kill(child_pid, SIGKILL);
        ret_code = FAIL;
        goto close_ret;

    rp_badparblk:
        *errmsg = &er_par;
        kill(child_pid, SIGKILL);
        ret_code = FAIL;
        goto close_ret;

/*    rp_toolarge:
        *errmsg = &er_toolg;
        kill(child_pid, SIGKILL);
        ret_code =FAIL;
        goto close_ret;
*/
    rp_dmser:
        *errmsg = &er_snd;
        er_snd.code = errno;
        kill(child_pid, SIGKILL);
        ret_code = FAIL;
        goto close_ret;

    rp_mgser:
        *errmsg = &er_acknsnd;
        er_acknsnd.code = errno;
        kill(child_pid, SIGKILL);
        ret_code = FAIL;
        goto close_ret;

    rp_rmerr:
        kill(child_pid, SIGKILL);
        ret_code = FAIL;
        goto close_ret;

    close_ret:
        /* unblock the window resizing signal if it was blocked for debug 
           mode */
        if (sigwinch_blocked)
        {
            unblock_sig(SIGWINCH);
            sigwinch_blocked = 0;
        }
        
        sync_child = 0;                 /* no child active      */
        if (read_chan != -1)            /* channel open         */
            close(read_chan);           /* close it             */
        if (write_chan != -1)
            close(write_chan);          /* close write channel  */
        t_attn(ecbi);                   /* reset ^C handler     */
        return (ret_code);
}

/*
 * SIGNAL CATCHERS.
 */

FUNCTION VOID int_sig(int dum)  /* here for operator attn         */
{
    set_signal(SIGINT, SIG_IGN);        /* disable operator intrpt        */
    /* set the flag and return */
    opwait = TRUE;
}

/*
 *      set_signal. Set the signal handler (under BSD 4.2)
 */

FUNCTION CODE set_signal
(
    FUNINT signum,        /* signal number         */
    void (*sig_hand)(int) /* signal handler function */
)
{

    CODE code;
    struct sigaction act;
    struct sigaction oact;

    act.sa_handler = sig_hand;  /* set the signal handling function */
    sigemptyset(&act.sa_mask);  /* don't mask any extra signals */
    act.sa_flags = 0;           /* no special flags */
    code = sigaction(signum, &act, &oact);     /* assign handler to signum */
    return (code);
}

/*
 *    block_sig.  Temporarily block a signal from being delivered.
 */

FUNCTION CODE block_sig(int signum)
{
    sigset_t mask;

    sigemptyset(&mask);
    sigaddset(&mask,signum);
    return (sigprocmask(SIG_BLOCK,&mask,NULL));
}

/*
 *    unblock_sig.  unblock a signal to allow it to be delivered.
 */

FUNCTION CODE unblock_sig(int signum)
{
    sigset_t mask;

    sigemptyset(&mask);
    sigaddset(&mask,signum);
    return (sigprocmask(SIG_UNBLOCK,&mask,NULL));
}

/*
*  FUNCTION:    act_task
*
*  DESCRIPTION:  This function activates a child process and start up
*  the specified image file.  It can also start the debugger and ask it
*  to start up the image file.  It also creates a socket for use in
*  passing V-blocks to/from the child task.
*
*  CONTROL:
*       Decide if debugger is to be used and what its name is.
*       Create socket for communications with child:
*           Create socket.
*           Bind name to socket.
*           Listen for connection requests.
*       Add parameters to environment to tell child pid and socket name.
*       Fork new process.
*       Exec requested task or debugger with requested task.
*       Accept connection to socket.
*
*/
FUNCTION CODE act_task
(
    TEXT        task_name[], /* in:  task name */
    FUNINT      parent_pid,  /* in:  parent pid */
    TEXT        *envtbl[],   /* in: environment table */
    FUNINT      *read_pipe,  /* out: read end of up-pipe */
    FUNINT      *write_pipe, /* out: write end of down-pipe */
    FUNINT      *child_pid   /* out: process id of activated task */
)
{

    IMPORT struct VARIABLE *switch_gbl; /* TAE switch bits              */
    IMPORT CODE run_type;               /* INTER, ASYNC, or BATCH       */
    int         ns = -1, nfds;
    fd_set      readfds;
    int         down_pipe[2], up_pipe[2];
    static TEXT snd_pipe0[20], snd_pipe1[20], rcv_pipe0[20], rcv_pipe1[20];
    static TEXT parent_name[40];
    static TEXT snd_skname[20];
    static TEXT dbxflagname[20];
    static TEXT *parent_env[]={parent_name, snd_pipe0, snd_pipe1,
                        rcv_pipe0, rcv_pipe1};

#if defined(sun) || defined(DGUX) || defined(sgi) || defined(linux)
    static BOOL sockflag = 0;
    static int  sd = 0;
    static struct sockaddr sockname;
    struct sockaddr acname;
    int socknamelen, acnamelen;
#endif

    CODE        status;
    BOOL        debug;                  /* TRUE if debug switch set     */
    TEXT        dbg_name[FSPECSIZ+1];   /* debugger name                */
    COUNT       i, k;
    TEXT        *dbgptr;                /* pointer to debugger symbol   */
    TEXT        **envlist = NULL;       /* Augmented environment list   */

#ifdef EROS
    struct VARIABLE *charge;
#endif

    /*----------------------------------------------------------------------*/
    /*  EXECUTABLE STATEMENTS.                                          */
    /*                                                                  */


    /*
     * Allocate space for the augmented environment list
     */
    for (i=0; envtbl[i]; i++);
    envlist = (TEXT **) malloc((i + MAXADDENV + 1) * sizeof(TEXT *));


    *child_pid = -2;                            /* initialize           */

    /*  Decide if debugger is to be used and what its name is.          */
    /*                                                                  */
    /*  Debugger is used iff SW_DEBUG bit is set in $SWITCH TAE global  */
    /*  (i.e. $SWITCH = 1) and RUNTYPE is INTERACTIVE. The standard     */
    /*  4.2BSD debugger, dbx, will be used unless another name is       */
    /*  passed in via the environment variable DEBUUGGER.               */

    debug = (run_type == INTER) && (IVAL(*switch_gbl, 0) & SW_DEBUG);

    /* Pipes work just fine for Apollo for debugging purposes. Sockets
     * of family AF_UNIX in fact do not. (Apollo has only AF_INET.)
     */
#if defined(sun) || defined(DGUX) || defined(sgi) || defined(linux)
    if (debug)
    {
        if (debug)   /* Redundant. For debug of debug. */
        {
            dbgptr = getenv( "DEBUGGER" );
            if ( dbgptr == NULL)
                strcpy(dbg_name,"dbx");
            else
                s_bcopy(dbgptr, dbg_name, FSPECSIZ);
        }

        /*  Create socket for communications with child.                    */
        /*  Create the socket, bind a name to it, listen for connections.   */
        /*  This is done only once.                                         */

        /*** This section of code ought to be done at a higher level. ***/
        if(!sockflag)
        {
            sockflag = 1;
            sd = socket( AF_UNIX, SOCK_STREAM, 0 );
            sockname.sa_family = AF_UNIX;
            sprintf( sockname.sa_data, "%d.TM", parent_pid );
            socknamelen = sizeof(sockname.sa_family) + strlen(sockname.sa_data);
            status = bind(sd, &sockname, socknamelen);
            status = listen(sd, 1);
        }
        /*** That way, the socket could be properly UNLINKed when TM exits. ***/

        /* Add parameters to environment to tell child pid and socket name. */
        sprintf(parent_name, "PARENT=%d",   parent_pid);
        sprintf(snd_skname,  "SOCKNAME=%s", sockname.sa_data);
        sprintf(dbxflagname, "DBXFLAG=%d",  1);

        i = 0;  k = 0;
        envlist[i++] = parent_name;
        envlist[i++] = snd_skname;
        envlist[i++] = dbxflagname;
        while ((envlist[i++] = envtbl[k++]));
        envlist[i] = 0;

        /*  Fork new process. */
        *child_pid = vfork();

        /*  Exec requested task or debugger with requested task. */
        if (*child_pid == -1)
            return (FAIL);
        else if (*child_pid == 0)
        {
#ifdef EROS
            charge = lookex(&glbtab, "CHARGE");      /* get charge code */
            if (charge == NULL)
            {
                if (debug)
                    execle(dbg_name, task_name, task_name, 0, envlist);
                else
                    execle(task_name, task_name, 0, envlist);
            }
            else
            {
                if (debug)
                    execle("/usr/local/bin/charge", "charge", SVAL(*charge, 0),
                        dbg_name, task_name, 0, envlist);
                else
                    execle("/usr/local/bin/charge", "charge", SVAL(*charge, 0),
                        task_name, 0, envlist);
            }
#else
            if (debug)
                execle(dbg_name, task_name, task_name, 0, envlist);
            else
                execle(task_name, task_name, 0, envlist);
#endif
            _exit(1);
        }


        /*  Accept connection to socket.                                    */
        set_signal( SIGCHLD, chld_sig );
        chldint = 0;

        FD_ZERO(&readfds);
        FD_SET(sd, &readfds);

        /* block SIGWINCH for debug mode.  When it is enabled, it kills a 
           debugging session if the window is resized on most platforms.  It
           also causes debugging to not work at all on sun platforms when using
           the gdb debugger. */
        block_sig(SIGWINCH);
        sigwinch_blocked = 1;

        nfds = select( sd + 1, &readfds, 0, 0, 0 );

        set_signal( SIGCHLD, SIG_DFL );

        if( nfds > 0 )
        {
            acnamelen = sizeof( acname );
            ns = accept( sd, &acname, &acnamelen );
        }

        *read_pipe = ns;
        *write_pipe = ns;
    }
    else
#endif
    {
        status = pipe(down_pipe);                   /* create pipe  #1      */
        if (status == -1) return (FAIL);
        status = pipe(up_pipe);                     /* create pipe #2       */
        if (status == -1)
        {
            close(down_pipe[0]);                    /* close the open pipes */
            close(down_pipe[1]);
            return (FAIL);
        }

        sprintf(parent_name, "PARENT=%d",     parent_pid);
        sprintf(snd_pipe0,   "DOWN_PIPE0=%d", down_pipe[0]);
        sprintf(snd_pipe1,   "DOWN_PIPE1=%d", down_pipe[1]);
        sprintf(rcv_pipe0,   "UP_PIPE0=%d",   up_pipe[0]);
        sprintf(rcv_pipe1,   "UP_PIPE1=%d",   up_pipe[1]);

        debug = FALSE;
        if (run_type == INTER)                /* debug only if interactive */
            debug = (IVAL(*switch_gbl, 0) & SW_DEBUG);
        if (debug)
        {
            dbgptr = getenv("DEBUGGER");            /* get debugger name    */
            if (dbgptr == NULL)                     /* not defined          */
                strcpy(dbg_name,"dbx");
            else
                s_bcopy(dbgptr, dbg_name, FSPECSIZ);  /* copy debugger name */
        }

        for (i=0; i<5; i++)
            envlist[i] = parent_env[i];                /* first copy locals  */
        for (k=0; (envlist[i] = envtbl[k]); i++, k++); /* copy shell exports */
        envlist[i] = 0;

        *child_pid = vfork();                   /* use 'vfork' for fastness  */

        if (*child_pid == 0)
        {
#ifdef EROS
            charge = lookex(&glbtab, "CHARGE");      /* get charge code */
            if (charge == NULL)
            {
                if (debug)
                    execle(dbg_name, task_name, task_name, 0, envlist);
                else
                    execle(task_name, task_name, 0, envlist);
            }
            else
            {
                if (debug)
                    execle("/usr/local/bin/charge", "charge", SVAL(*charge, 0),
                        dbg_name, task_name, 0, envlist);
                else
                    execle("/usr/local/bin/charge", "charge", SVAL(*charge, 0),
                        task_name, 0, envlist);
            }
#else

            if (debug)
                execle(dbg_name, task_name, task_name, 0, envlist);
            else
                execle(task_name, task_name, 0, envlist);
#endif
            _exit(1);
        }

        free (envlist);
        if (*child_pid == -1) 
            return (FAIL);
        *read_pipe = up_pipe[0];            /* end to read from             */
        *write_pipe = down_pipe[1];         /* end to write to              */
        close (down_pipe[0]);               /* close unused ends            */
        close (up_pipe[1]);
    }
    return (SUCCESS);
}

/* Catch SIGCHLD and do nothing. */
FUNCTION static VOID chld_sig(int unused)
{
    chldint=1;
}

/*
 *      c_suspend.  Suspend the executing task  .
 */

FUNCTION static VOID c_suspend
(
   int child_pid        /* process id of task to suspend */
)
{
    kill(child_pid, SIGSTOP);           /* send signal to stop           */
}

/*
 *      c_resume.  Resume the suspended task.
 */

FUNCTION static VOID c_resume
(
    int child_pid       /* process id of suspended task */
)
{
    kill(child_pid, SIGCONT);           /* send signal to continue      */
}

