/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/



/* TAE monitor (TM) source file.
 * After functions tm and local_tm, the functions in this
 * source file are in alphabetical order.
 *
 * CHANGE LOG:
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *      15-jul-83       Updated for host-code return from c_init()...dm
 *      18-jul-83       Fix latch bug on substitute error...peb
 *      29-jul-83       Updated to use intrinsic parameter pool(ipp) for
 *                      intrinsic  parameter processing...dm
 *      05-aug-83       Cosmetic changes...dm
 *      11-aug-83       Deallocate value for NAME variables in repval()...dm
 *      16-aug-83       Add comment relating prccmd to delgbl_do...peb
 *      20-aug-83       Nullables;
 *                      Primary CONTXT made global for substitution...palm
 *      25-aug-83       New returns from updtab.
 *      29-aug-83       Keyword parameters...palm
 *      08-sep-83       PR 481, maximum pdf nesting enforced...palm
 *      12-sep-83       Added $LASTCMD to hold most recent command...palm
 *      12-sep-83       Clean-up clr_latch, clsctx, latching...palm
 *      16-sep-83       Fix clsctx so double call for same ctx okay...palm
 *      23-sep-83       Fix PR 444 on TAE-BADCMD...palm
 *      26-sep-83       Implement de-referencing...palm
 *      03-oct-83       Fix lint-identified errors (f_close)...palm
 *      05-oct-83       Change getint() function name to cmd_noscreen...dm
 *      20-oct-83       Fix PR 560, TAE-BADKEY to TAE-KEYWORD...palm
 *      25-oct-83       Add host_init() call...palm
 *      30-oct-83       Set prim_act in tm primary loop...peb
 *      20-jan-84       Fix [TAE-TOOMANY] bad message when "@x" passed
 *                      to proc and x is null...palm
 *      23-jan-84       History_save made external...palm
 *      25-jan-84       Save $LASCMD before prccmd (for nesting problem);
 *                      New cmd_noscreen call...palm
 *      03-feb-84       Changes to prccmd() for proc interrupt mode...nhe
 *                      changes to chk_do to allow all cmds in int mode
 *      09-feb-84       Move bldcmd processing logic to app_cmd() in the
 *                      library, and let bldcmd invoke it...dm
 *      14-feb-84       Don't continue if c_init error...palm
 *      14-feb-84       Change runtyp to run_type and CODE...palm
 *      22-feb-84       Remove "cmdstr" arg from call seq to run_batch...peb
 *      22-feb-84       Async changes...nhe
 *      01-mar-84       Fix memtab to set dvp NULL...palm
 *      05-mar-84       Args to main for UNIX...dm
 *      06-mar-84       Update bldcmd, call flush_cmd in case of overflow...dm
 *      11-mar-84       Fix repval crash for V_NAME parameters...dm
 *      14-mar-84       Further async changes...nhe
 *      15-mar-84       Proper check for BATCH in chk_do
 *      19-mar-84       Update REPLY request message...nhe
 *      27-apr-84       Make value vectors a union in repval...lim
 *      04-may-84       VALUE_x to xVAL, set_string added, addref removed due
 *                      to use of run_gbl and sess_gbl...ces
 *      07-may-84       Clean-up the IMPORT declarations...lim
 *                      No more dyncmd mode; allow async in proc int mode...nhe
 *                      'toprefs'-->'numrefs'...nhe
 *      06-jun-84       Fix repval to allow de-referencing on DEFAULT and
 *                      INITIAL...palm
 *      08-jun-84       Update updtab() errmsgs to cover qualifier case
 *      11-jun-84       Make _ONFAIL doubly valued for TCL-21...palm
 *      11-jun-84       New echo scheme for 1.3...palm
 *      14-jun-84       Fix double echo for ASYNC; new RUNTYPE values...palm
 *      14-jun-84       Y_PDF ITRCMD entry support...palm
 *      09-jul-84       Fix stdout name bug for batch (as in UNIX)...dm
 *      06-aug-84       New BATCH/ASYNC strategy...palm
 *      24-aug-84       Alter setting of $TUTOR for 1st entry only...lia
 *      13-sep-84       runtype() changed to default to ASYNC if NODE set...nhe
 *      17-sep-84       Add process for error code V_BADAMBIG...lia
 *      08-oct-84       Handle acbptr in CONTXT (for RCJM)...nhe
 *      10-oct-84       New test for interactiveness...nhe
 *      12-oct-84       Conditional compilation of async features as done for
 *                      UNIX 1.2...dm
 *      14-oct-84       Changes for internal procs...nhe
 *      17-oct-84       TCL 117: inictx to init ctx.compiled...peb
 *      17-oct-84       PR 847: inictx to ref implicits via v_implicit...peb
 *      24-oct-84       TCL 117: call_do update for compiling PDFs...peb
 *      31-oct-84       TCL 117: handle de-referencing in compilation...peb
 *      02-nov-84       Add 'remote' flag for RCJM remote job processing...dm
 *      05-nov-84       TCL 67: prccmd -- (*ctx).special=GET_PARM_QUAL...peb
 *      09-nov-84       PR 879: remove bogus prccmd call;
 *                      PR 885: echo=TRACE...palm
 *      11-nov-84       runtype(): check for null RUNTYPE cmd qual...nhe
 *      21-nov-84       Add processing for NORUN RUNTYPE cmd qual...nhe
 *      25-nov-84       TCL 67: updtab to set parameter qualifiers...peb
 *      29-nov-84       TUTOR 15: clear saved proc & sub name for HELP...lia
 *      04-dec-84       Implement loopback of remote tm for next job...dm
 *      08-dec-84       PR 898: set $TUTOR back to single element...lia
 *      11-dec-84       TCL 97: initialize new flag for executing onfail
 *                      cmd in CONTXT block...lia
 *      18-dec-84       TCL 67: in misprm, enforce missing parm qualifs...peb
 *      27-dec-84       Updated to loop for next job in 'remote' mode...dm
 *      03-jan-85       Do not abort RCJM agent on pdf processing error...dm
 *
 **************************************************************************
 * CHANGES MADE IN THE RCJM TREE:
 *
 *      17-jan-85       Update to getcln() for 'include' command files...dm
 *      02-feb-85       Combine remote tm and remote agent for remote job
 *                      processing...dm
 *      16-feb-85       Call c_maxsub() to allow only one subprocess...dm
 *      18-feb-85       Fix bug due to input/output flipping for async...dm
 *      19-feb-85       Break up main tm to smaller modules for readability...dm
 *      22-feb-85       Update remote_tm to recognize JOB_ERR...dm
 *      13-mar-85       Update for new remote job execution protocol...dm
 *      18-mar-85       Do not exit Remote tm if invalid proc/stdout name...dm
 *
 ***************************************************************************
 * MERGE THE FOLLOWING TAE-V1.3 CHANGES...dm (21-may-85)
 *
 *      25-jan-85       Change TAE-BADDCL to BADDECL to avoid conflict...nhe
 *
 ****************************************************************************
 *      
 *      14-jun-85       Delete tm_argc, tm_argv args to main routine...dm
 *      01-jul-85       Fix UNIX compilation errors...dm
 *      25-jul-85       Fix UNIX lint errors...dm
 *      27-jul-85       Determine input from run_type rather than termtype
 *                      in getcln()...dm
 *      01-aug-85       Fix next_cmd misspelling in local_tm()...dm
 *      01-aug-85       Add some time tracing for remote tm...nhe
 *      13-aug-85       Factor tm sub creation out of loop for remote tm...nhe
 *      21-aug-85       Terminate async job for inaccessible stdout file...joh
 *      12-sep-85       PR 995: Add chk_asy_done to check for async job
 *                      completion msgs; called by prc_nextcmd...dab
 *      03-oct-85       PR 946: Added "not compiling" (FALSE) parameter to
 *                      calling sequence of 'chk_vector'...dab
 *      29-oct-85       PR 933: Allow param quals when compiling. 'save_p_qual'
 *                      now saves quals in proc context compile data area when
 *                      compiling a proc...dab
 *      25-feb-86       "Captive" change: init CONTXT.interrupt to FALSE
 *                      not TRUE in INICTX...palm
 *      16-jul-86       Honor RUNTYPE=ASYNC-PROCESS...palm
 *      16-jul-86       Honor ACTION= setup for PARMSETs...palm
 *      12-nov-86       Added TAE_FACELIFT code...krw
 ************* RCJM Merge into 1.4 -  3Nov86...nhe **********************
 *      12-feb-86       Perform RSLOGON and save context before remote tm loop,
 *                      restore context for remote proc execution (implemented 
 *                      for RCJM optimization)...dm
 ************************************************************************
 *      28-mar-87       Allow qualifiers on intrinsics if Y_QUALIF set.
 *                      This for the window creation intrinsics...palm
 *      08-may-87       PR1181: changed "SLOGON" and "RSLOGON" to lower...ljn
 *      13-jul-87       Locals with qualifiers.  Here, save_p_qual updated
 *                      to get proper symbol table...palm
 *      13-jul-87       Disallow NAMEd parameters as qualifiers.  See note
 *                      in save_p_qual below explaining why such can't 
 *                      be allowed...palm
 *      15-jul-87       Initialize the d_ package so that MDF/HLP 
 *                      MDF/HLP substitution is enabled...palm
 *      23-jul-87       Now call intrin_init() which calls each of the
 *                      modules with intrinsic command tables to get them
 *                      to return a ptr to their respective table...ljn
 *      12-aug-87       Allow QUALS= to have a real TCL command line
 *                      so that defaults can be set for the qualifiers.
 *                      Example:  PARM A QUALS="Q |RESTORE=X| Y=2"...palm
 *      20-nov-87       Call CheckXEvent() in prccmd() to repaint exposed 
 *                      windows...dm
 *      08-feb-88       Unknown terminal type generates error msg...ljn
 *      24-mar-88       Delete TAE_RCJM conditionals...ljn
 *      02-may-88       Initialize new CONTXT.parfile bit...palm
 *      11-may-88       Fixed error message for de-referenced variable...hsk
 *      11-may-88       New "utility execution mode" where placing a TCL
 *                      command on the shell command line causes execution 
 *                      in interactive mode, without greeting and slogon...palm
 *      13-may-88       Avoid t_init call unless truly interactive, otherwise
 *                      you get screen init for an ASYNC job...palm
 *      28-nov-88       Add _TUTOR as new implicit local...palm
 *      05-dec-88       Init CONTXT.selftutor bit to FALSE in inictx...palm
 *      09-dec-88       Init CONTXT.compress bit to FALSE in inictx...palm
 *      27-jan-89       memtab: use IVP pool if v_maxc < INT_MAXVAL, else
 *                      use normal allocation.  This to allow for very large
 *                      MAXVAL and to keep the IVP pool at reasonable size.
 *                      Also, Reals in intrinsics now allowed...palm
 *      01-feb-89       VAX C requires IMPORT BOOLS of termlines, termcols...ljn
 *      14-feb-89       fix keywords for intrinsics...palm
 *                      make repval extern...palm
 *      01-may-89       remove check on executing process from INTMODE--it's
 *                      now in pdfprc:run_proc...palm 
 *      01-may-89       hook to set process name for BATCH and ASYNc...palm
 *      12-jun-89       Removed TAE_FACELIFT...ljn
 *      13-dec-89       Exit from utility command with proper status...ljn
 *      23-may-90       Remove RCJM stuff...ljn
 *      27-jun-90       Remove Facelift code...ljn
 *      26-dec-90       Call t_pinit if TM is in "utility mode"...ljn
 *      01-aug-91       Braces for static string initializers...ljn
 *      27-jan-94       v53 SGI port:free internal proc blocks differently...dag
 *      16-feb-94       PR2621: we were calling delvar then referencing the
 *                      deleted VARIABLE to get its link. This is illegal..krw
 *      10-aug-94       PR2830: added maxc, count and valvec to addint...krw
 *      28-aug-95       Changed S_NONE to S_NOKW, to avoid conflict with
 *                      /usr/include/sys/signal.h...raw
 *      
 */

#include <stdlib.h>
#include <string.h>
#include "taeconf.inp"   /* TAE configuration definitions        */
#include "inttaelib.h"
#include "fileinc.inp"   /* file & file position context structs */
#include "terminc.inc"   /* terminal package definitions         */
#include "syninc.inc"    /* syntax package structure and defs    */
#include "eventinc.inp"  /* e_ package defs & structs            */
#include "tmhost.inp"
#include "symtab.inc"
#include "tminc.inc"     /* TM-only host-independent definitions */
#ifdef  TAE_ASYNC        /* Needed for remote and async cases only */
#include "asyncinc.inc"
#endif
#include "taelib.h"
#include "local_protos.h"


GLOBAL struct CONTXT primctx;       /* proc context primary level */
GLOBAL TEXT next_cmd[NEXTSIZ+1]=""; /* previous mode help command */

static TEXT tutor_msg[] =
    "Re-enter the command line or type TUTOR to recover specified values.";

FUNCTION static VOID local_tm
(
    FUNINT argc,
    TEXT   *argv[] 
);
FUNCTION static BOOL call_do
(
    struct CONTXT *ctx,     /* in: proc context             */
    struct ITRCMD *itrcmd   /* in: ITRCMD entry for cmd     */
);
FUNCTION static VOID echo 
(
    struct CONTXT *ctx,     /* input: enclosing proc context        */
    TEXT          cmdstr[]  /* input: command string                */
);
FUNCTION static CODE init_localjob
(
    FUNINT *run_type        /* out: runtype; ASYNC or BATCH   */
);
FUNCTION static CODE prc_nextcmd
(
    TEXT        next_cmd[], /* in/out: next TCL  command to execute */
    FUNINT      usermode,   /* in: user mode, COMMAND vs MENU     */
    struct  CONTXT *primctx /* in: address of primary context blk */
);
FUNCTION static VOID save_p_qual 
(
    struct CONTXT *par_ctx, /* in/out: parent context -- gets quals */
    struct CONTXT *cmd_ctx  /* in/out: command context -- quals in parmst*/
);


/*      tm - TAE monitor main program.
 */

FUNCTION int main (FUNINT argc, TEXT *argv[])
{
    IMPORT COUNT         termcols;
    IMPORT COUNT         termlines;     /* # lines on terminal screen      */
    IMPORT CODE          termtype;      /* terminal type                   */

    /* host-dependent initialization */
    host_init();                        
    /* enable MDF/HLP substitution */
    d_setsub (substitute);
    /* enable MDF/HLP substitution */
    d_setctx (&primctx);
    termlines = 0;  
    termcols = 0;  
    termtype = T_NOTTERM;
    /* build intrinsics table */
    intrin_init();
    /* do processing as local tm */
    local_tm(argc, argv);
    return 0;
}       

/*      local_tm. Process user commands as the local TAE monitor.
 *      
 *      This module handles:
 *              - user commands as the interactive monitor
 *              - a single batch job as the batch monitor
 *              - a single async job as the async monitor
 *
 *      NOTE: For simplicity, we refer to a 'job' in interactive mode as
 *      equivalent to a TAE session, i.e. it starts with the first user
 *      command and terminates at the executuion of a "LOGOFF" or "EXIT"
 *      command.
 */

FUNCTION static VOID local_tm
(
    FUNINT argc,
    TEXT   *argv[] 
)
{
    IMPORT CODE          cmd_mode;      /* NORMCMD or INTMODE */
    IMPORT CODE          usermode;      /* MENUMODE or CMDMODE */
    IMPORT CODE          prim_act;      /* TUTOR_ACTIVATION or CMD_ACTIVATION */
    IMPORT TEXT          vrsion[];      /* program version */
    IMPORT struct CONTXT *curproc;
    IMPORT CODE          run_type;      /* INTER, ASYNC, or BATCH       */
    IMPORT struct VARIABLE *run_gbl;    /* pointer to $RUNTYPE          */
    IMPORT struct VARIABLE *tutor_gbl;  /* pointer to $TUTOR variable   */
    IMPORT struct VARIABLE *sfi_gbl;    /* $SFI                         */
    IMPORT CODE          termtype;      /* terminal type                */
    IMPORT struct CONTXT primctx;
    IMPORT TEXT          next_cmd[];
    IMPORT COUNT         termcols;
    IMPORT COUNT         termlines;     /* # lines on terminal screen      */
    static TEXT          *screen_mode[2] = {"SCREEN", "NOSCREEN"};
    CODE                 code;
    TEXT                *utilityCommand;

    /* take first cmd line arg as cmd */ 
    utilityCommand = argv[1];           
    curproc = &primctx;
    /* mark this primary level proc */
    primctx.prclevel = 0;
    /* this is top level */
    primctx.backlink = NULL;
    /* always start in command mode */
    usermode  = CMDMODE;
    next_cmd[0] = EOS;
    /* i.e., not inter'pt */
    cmd_mode = NORMCMD;
    /* initialize globals */
    ini_globals();
    /* init primary context */
    inictx(&primctx);

    if (inter_proc())
    {
        /* interactive TAE monitor */
        if (utilityCommand)
            t_pinit (&termlines, &termcols, &termtype);
        else
            t_init (&termlines, &termcols, &termtype);
        run_type = INTER;
        if (utilityCommand)
        {
            /* prime first command */
            strcpy (next_cmd,utilityCommand);
        }
        else
        {
            /* prime first command */
            strcpy(next_cmd,"slogon");
        }
        set_string (run_gbl, "INTERACTIVE");
        /* enable formatted screen tutor */
        if (termtype == T_CRT)
            set_value (tutor_gbl, (GENPTR)screen_mode, 1);
        else if (termtype == T_NOTCRT)
        {
            tmmsg (SUCCESS, "Unknown terminal type.", "TAE-UNRECTTY");
            t_bell ();
        }
    }
    else
    {
        /* force this */
        termtype = T_NOTTERM;
        /* init for a local job */
        code = init_localjob(&run_type);
        if (code == FAIL)
            exit (TM_EXIT);
    }
    if (!utilityCommand)
        greet (vrsion);

    /* installation exit */
    init_ins (&primctx);

    /* primary cmd level loop   */
    while (FOREVER)
    {
        /* assume procs activated by cmd method */
        prim_act = CMD_ACTIVATION;
        prc_nextcmd(next_cmd, usermode, &primctx);
        if (utilityCommand)
        {
            /* only one shot if utility */
#ifdef VMS
            exit (IVAL(*sfi_gbl,0) <= 0 ? 0 : 1);
#else
            exit (IVAL(*sfi_gbl,0) <= 0 ? 1 : 0);
#endif
        }
    } /* exit from logoff_do/exit_do */
}

/* bldcmd - work toward building a full TAE command string, given the
 * the accumulated string so far and the next single input line.
 * Strips comments and trailing blanks.
 * "+" as last non-comment character on input line indicates continuation.
 * Function returns :
 *              CONTINUE if another continuation line is expected,
 *              SUCCESS if this is the last line of the command,
 *              FAIL    if command line is longer than the maximum size.
 *
 * NOTE: This function takes the single input line as well as the accumulated
 *       command string.
 *
 * NOTE:  This function assumes that a quoted string must be closed on one line
 *        if the line has a comment at the end.
 */

FUNCTION CODE bldcmd 
(
    TEXT inbuf[],           /* in/out: single input line buffer     */
    TEXT cmdstr[]           /* in/out: accumulated cmd string       */
)
{
    BOOL        inquote;    /* TRUE if current character is quoted  */
    CODE        code;

    inquote = FALSE;
    /* append inbuf to cmdstr */
    code = app_cmd(inbuf, cmdstr, inquote);
    if (code == FAIL)
    {
        /* no more room in cmdstr */
        tmmsg(PROCFAIL, "Command too long.  Command ignored.",
                "TAE-LONGCMDLINE");
        cmdstr[0] = EOS;
    }
    return(code);
}

/*
 *      call_do.   Returns true if the intrinsic command should be
 *      executed.
 */

FUNCTION static BOOL call_do
(
    struct CONTXT *ctx,     /* in: proc context             */
    struct ITRCMD *itrcmd   /* in: ITRCMD entry for cmd     */
)
{
    LONG        flags;
    struct SEARCH *srchpt;

    flags = (*itrcmd).flags;
    /* in a subcmd block? */
    if ((*ctx).subblk)
    {
        if ((*ctx).subact  ||  (*ctx).special == COMPILING)
        {
            /* always call if active subcmd */
            return (TRUE);
        }
        else
        {
            /* call if END-SUB */
            return (flags & Y_ENDSUB);
        }
    }
    /* if search mode...        */
    if ((srchpt = (struct SEARCH *)(*ctx).srchblk) != NULL)
    {
        if ((*srchpt).type != SRCH_PROC)
        {
            /* call only if requested       */
            return (flags & Y_SEARCH);
        }
        else
            return (flags & Y_PROCSEARCH);
    }
    else if (!(*ctx).inbody)
    {
        /* if pre-body and no search always call */
        return (TRUE);
    }
    else if (true_part(ctx))
    {
        /* if true part of if always call */
        return (TRUE);
    }
    else
    {
        /* false part: call if request  */
        return (flags & Y_FALSE);
    }
}

/*   check_asy_req. Check if a request has arrived from an async job.
 */

FUNCTION static VOID check_asy_req(void)
{
#ifdef TAE_ASYNC
    IMPORT BOOL         async_request;  /* TRUE if async request is pending  */
                        
    if (async_request)                  
    {
        async_request = FALSE;
        tmmsg (SUCCESS, "Async job requesting parameters.  Use REPLY command.",
                "TAE-ASYNCREQ");
    }
#endif
}

/*   chk_asy_done. Check if any async jobs have completed and require
     that job completion messages be generated.
 */

FUNCTION static VOID chk_asy_done(void)
{
#ifdef TAE_ASYNC
    IMPORT struct ACB   *acb_head;      /* Points to ACB list */

    struct ACB          *acb;
    TEXT                job_done_msg[STRINGSIZ+1];

    for (acb = acb_head; acb != NULL; acb = (*acb).link)
    {
        if ( (*acb).waiting  &&  !(*acb).active )
        {
            (*acb).waiting = FALSE;
            strcpy (job_done_msg,"Job \"");
            strcat(job_done_msg,(*acb).name);
            strcat(job_done_msg,"\" complete.");
            tmmsg (SUCCESS, job_done_msg, "TAE-ASYNCDONE");
        }
    }
#endif
}

/*      chk_do.  Check the environment of an intrinsic command against
 *      bits in the ITRCMD entry.
 */

FUNCTION CODE chk_do
(
    struct CONTXT *ctx,     /* in: current proc context     */
    struct ITRCMD *itrcmd   /* in: ptr to ITRCMD entry      */
)
{
    IMPORT CODE         cmd_mode;       /* NORMCMD or INTMODE           */
    IMPORT CODE         run_type;       /* INTER, ASYNC, or BATCH       */

    LONG                flags;
    TEXT                *mode_name;

    flags = (*itrcmd).flags;            
    /* first check against current command processing mode */
    if (cmd_mode == NORMCMD)
    {
        if (!(flags & Y_CMD)) 
            goto mode_err;
    }

    if (run_type == BATCH && !(flags & Y_BATCH))
    {
        tmmsg(PROCFAIL, "'%s' not available in batch.",
              "TAE-NOTBATCHCMD", (*itrcmd).cmd);
        return (FAIL);
    }
    if (run_type == ASYNC && !(flags & Y_ASYNC))
    {
        tmmsg(PROCFAIL, "'%s' not available in an asynchronous proc.",
              "TAE-NOTASYNCMD", (*itrcmd).cmd);
        return (FAIL);
    }
    /* if both zero, set both       */
    if ((flags & (Y_PRIM | Y_PROC)) == 0)
        flags |=  (Y_PRIM | Y_PROC);
    /* primary/proc level check     */
    if ((*ctx).prclevel == 0)
    {
        if (!(flags & Y_PRIM))
        {
            tmmsg(PROCFAIL, "'%s' available only in procedures.",
                  "TAE-PROCONLY", (*itrcmd).cmd);
            return (FAIL);
        }
    }
    else
    {
        if(!(flags & Y_PROC))
        {
            tmmsg(PROCFAIL, "'%s' not allowed in proc definition file.",
                  "TAE-NOTINPDF", (*itrcmd).cmd);
            return (FAIL);
        }
    }
    /* check state of procedure...  */
    if ((*ctx).prclevel == 0)
        return (SUCCESS);
    else if ((*ctx).proctype == Y_UNKNOWN)      
    {
        if (!(flags & Y_INTRO))
        {
            tmmsg(PROCFAIL, "Introductory command (PROCEDURE, PROCESS, ...) "
                  "missing.", "TAE-MISINTRO");
            return (FAIL);
        }
    }
    else if ((*ctx).inbody)
    {
        if (!(flags & Y_BODY))
        {
            tmmsg(PROCFAIL, "'%s' not allowed in procedure body.",
                  "TAE-NOTINBODY", (*itrcmd).cmd);
            return (FAIL);
        }
    }
    else
    {
        if (!(flags & Y_PREBODY))
        {
            tmmsg(PROCFAIL, "'%s' not allowed before procedure body.",
                  "TAE-NOTINPRE", (*itrcmd).cmd);
            return (FAIL);
        }
    }
    return (SUCCESS);

mode_err:
    mode_name = "not-normal command";
    if (cmd_mode == NORMCMD) mode_name = "normal command";
    tmmsg(PROCFAIL, "'%s' not available in %s mode.",
                "TAE-BADCMD", (*itrcmd).cmd, mode_name);
    return (FAIL);
}   

/*
 *      clr_latch - clear latching.
 */

FUNCTION VOID clr_latch 
(
    struct CONTXT *pctx     /* in:  proc context */
)
{
    IMPORT struct SFILE  prcfil;        /* proc file context (for f_ functions) */
    IMPORT CODE          parmlatch;     /* parm latch flag              */
    IMPORT struct CONTXT latchctx;      /* latched parameter proc contxt*/

    if (parmlatch == RUNLATCH)
    {
        parmlatch = NOLATCH;
        /* close if not intrinsic */
        if (!latchctx.intrinsic)
            f_close(&prcfil, F_KEEP);
        clsctx(&latchctx);
    }
}

/*      clsctx - close proc context.
 *      Free any allocated structures.
 *      Decrement reference counts for referenced global variables.
 */

FUNCTION VOID clsctx 
(
    struct CONTXT *pctx     /* in:  proc context            */
)
{
    COUNT               i;
    struct VARIABLE     *v;
    struct IPBLOCK      *ipbpt;

    /* delete local symbol table */
    deltab(&((*pctx).locst));
    /* delete parm symbol table */
    deltab(&((*pctx).parmst));

    /* for each referenced global   */
    for (i = 0; i < (*pctx).numrefs; i++)
    {
        /* point to the variable */
        v = (*pctx).refs[i];
        if (v == NULL)
            continue;
        /* decrement the reference count */
        (*v).v_refcnt--;
        (*pctx).refs[i] = NULL;
    }
    /* delete qualifier symbol table */
    deltab(&(*pctx).qualst);
    /* delete SUBCMD chain */
    delsub(&(*pctx).subptr);

    /* if there's an ACB, release it */
    if ((*pctx).acbptr != NULL)
        free_acb((*pctx).acbptr);

    /* internal proc blocks */
    ipbpt = (struct IPBLOCK *)(*pctx).int_procs;
    if (ipbpt != NULL)
    {
        struct IPBLOCK  *nextBlk;
        for (nextBlk = (*ipbpt).flink; ipbpt != NULL; )
        {
            tae_free (ipbpt);
            ipbpt = nextBlk;
            if (nextBlk != NULL) nextBlk = (*ipbpt).flink;
        }
    }

    /* The iteration control blocks must be deleted because we may have
       a RETURN command in the middle of constructs...  */
    /* delete iteration blocks */
    cleanlp((*pctx).icbptr);    
}

/*
 *      comp_search - search for variable in context of a compiling proc.
 *
 *      Assumes the variable to find is in one of:
 *
 *      - the "before" symbol table (i.e., the declarations before the
 *        first SUBCMD statement).
 *      - if we are currently in a SUBCMD block, the declarations that
 *        have already occurred in this block.
 *      - the "after" symbol table (i.e., the declarations after the
 *        last SUBCMD block).
 */

FUNCTION static struct VARIABLE *comp_search 
(
    TEXT          name[],   /* in: name of variable to find */
    struct CONTXT *pctx     /* in: compilation proc context */
)
{
    struct COMP_HOUSEKEEP *comp;
    struct VARIABLE     *v;
    struct SUBCMD       *cursub;

    /* point to compilation housekeeping */
    comp = (*pctx).comp;

    /* each var in sym tab*/
    for (v = (*comp).before.link; v != NULL; v = (*v).v_link)
    {
        /* if found, return variable */
        if (s_equal(name, (*v).v_name))
            return(v);
    }

    /* if we're in a SUBCMD statement block */
    if ((*pctx).subblk)
    {
        /* point to current SUBCMD struct */
        cursub = (*comp).cursub;
        /* each var in sym tab */
        for (v = (*cursub).symtab.link; v != NULL; v = (*v).v_link)
        {
            /* if found, return variable */
            if (s_equal(name, (*v).v_name))
                return(v);
        }
    }

    /* each var in sym tab*/
    for (v = (*comp).after.link; v != NULL; v = (*v).v_link)
    {
        /* if found, return variable */
        if (s_equal(name, (*v).v_name))
            return(v);
    }

    /* not found */
    return(NULL);
}

/*      echo.   Echo the command string if appropriate.
 */

FUNCTION static VOID echo 
(
    struct CONTXT *ctx,     /* input: enclosing proc context        */
    TEXT          cmdstr[]  /* input: command string                */
)
{
    IMPORT struct VARIABLE      *echo_gbl, *becho_gbl, *aecho_gbl;
    IMPORT CODE                 run_type;
    struct VARIABLE             *echo_var;
    TEXT                        flag;
    COUNT                       index;

    if ((*ctx).prclevel == 0)
    {
        /* level 0: */
        /* always echo unless interactive   */
        if (run_type != INTER)          
            put_stdout (cmdstr);
    }
    else
    {
        /* get controlling global */
        if (run_type == INTER)
            echo_var = echo_gbl;
        else if (run_type == BATCH)
            echo_var = becho_gbl;
        else
            echo_var = aecho_gbl;
        /* if echo short, use last */
        if ((*ctx).prclevel > (*echo_var).v_count) 
            index = (*echo_var).v_count - 1;
        else
        {
            /* use corresponding first character  */
            index = (*ctx).prclevel  - 1;
        }
        flag = *(SVAL(*echo_var, index));
        if (flag == 'F')
            put_stdout (cmdstr);                                
        else if ((flag == 'Y' || flag == 'B')  && (*ctx).inbody)
            put_stdout (cmdstr);
    }
}

/*      tr_echo.   Generate proc trace if appropriate.
 *
 */

FUNCTION static VOID tr_echo 
(
    struct CONTXT *ctx,     /* input: enclosing proc context        */
    TEXT          proc[]    /* input: name of proc                  */
)
{
    IMPORT struct VARIABLE      *echo_gbl, *becho_gbl, *aecho_gbl;
    IMPORT CODE                 run_type;
    struct VARIABLE             *echo_var;
    TEXT                        flag;
    COUNT                       index;

    /* get proper global */
    if (run_type == INTER)
        echo_var = echo_gbl;
    else if (run_type == BATCH)
        echo_var = becho_gbl;
    else
        echo_var = aecho_gbl;

    /* if echo short, use last */
    if ((*ctx).prclevel > (*echo_var).v_count)
        index = (*echo_var).v_count - 1;
    else if ((*ctx).prclevel == 0)
        index = 0;
    else
    {
        /* use corresponding first character */
        index = (*ctx).prclevel  - 1;
    }
    flag = *(SVAL(*echo_var, index));

    /* trace proc name  */
    if (flag == 'T')
        put_stdout (proc);     

}

/* getcln - get command line.
 *
 * Gets full TCL command line, including continuation lines.
 * On return, all continuation lines have been appended to the first
 * line, with the "+" replaced by a blank separator.
 * Comments and trailing blanks are stripped.
 *
 *      Return codes:
 *
 *              SUCCESS - line read from input successfully
 *              F_EOF - End of fie was encountered on batch input file
 *              F_FILERR - Error reading batch input file of script file
 */

FUNCTION CODE getcln 
(
    TEXT prompt[],          /* in: prompt (without the '>') */
    TEXT cmdstr[]           /* out: command string          */
)
{
    IMPORT CODE          run_type;      /* terminal monitor run type    */
    IMPORT struct SFILE  job_file;      /* batch/async primary input    */

    CODE        code;
    COUNT       nrec;                   /* record counter, ignored      */
    TEXT        pmtbuf[STRINGSIZ+1];    /* prompt buffer                */
    TEXT        inbuf[CMDLINSIZ+1];     /* single line input buffer     */

    cmdstr[0] = EOS;
    /* If non-interactive, read from job file */
    if (run_type != INTER)
    {
        do
        {
            code = f_read (&job_file, inbuf);
            if (code == F_EOF) return (code);
            if (code != SUCCESS)
            {
                m_put ("Error reading job file. %s",
                        "TAE-BCHRD", job_file.errmsg);
                return (F_EOF);
            }
            /* append cmd to full command string */
            code = bldcmd(inbuf, cmdstr);
        }
        while (code == CONTINUE); /* loop if continuation line    */

        if (code == FAIL)
        {
            /* get past this cmd */
            flush_cmd(&job_file, inbuf, &nrec); 
        }
        /* SUCCESS or FAIL */
        return (code);
    }
    else
    {
        /* get prompt string */
        strcpy(pmtbuf,prompt);
        strcat(pmtbuf,">");
        /* read command string */
        cmd_noscreen (A_LASTCMD, pmtbuf, cmdstr);
        return (SUCCESS);
    }
}

/*      history_save.   Save command string in $LASTCMD global.
 *      $LASTCMD(1) has the most recent command, $LASTCMD(2) the next oldest,
 *      and etc.
 */

FUNCTION VOID history_save
(
    TEXT cmdstr[]           /* in: string at least MAXSTRSIZ */
)
{
    IMPORT struct VARIABLE *last_gbl;

    COUNT       i;
    TEXT        save_char;
    TEXT        *saved;

    /* push down each current value, add new value at first component   */
    /* if null, no history saving (for allocation studies) */
    if ((*last_gbl).v_count == 0)
        return;
    /* max size allowed */
    i = (*last_gbl).v_size;             
    save_char = cmdstr[i];
    /* clip to largest allowed */
    cmdstr[i] = EOS;
    /* save in dynamic memory */
    saved = strdup(cmdstr);
    /* restore character */
    cmdstr[i] = save_char;
    /* if no memory, do nothing */
    if (saved == NULL)
        return;         
    if ((*last_gbl).v_count == (*last_gbl).v_maxc)
        free (SVAL(*last_gbl, (*last_gbl).v_maxc - 1));
    else
        (*last_gbl).v_count++;
    for (i=(*last_gbl).v_count - 1;   i > 0;   i--)
        SVAL(*last_gbl, i) = SVAL(*last_gbl, i-1);
    SVAL(*last_gbl, 0) = saved;
}

/*      inictx - initialize proc context block.
 *      Returns FAIL if a memory overflow occurs.
 *      Must be coordinated with reini_ctx (in tutor.c).
 *
 *      NOTE:  The list of implicitly ref'd globals has been moved
 *              to tminit.c.  inictx now only checks v_implicit
 *              to see if the global is implicit.
 */

FUNCTION CODE inictx 
(
    struct CONTXT *pctx     /* in/out: proc context block           */
)
{
    IMPORT struct SYMTAB glbtab;         /* global symbol table  */
    COUNT           i;
    struct VARIABLE *p;

    static TEXT *standout[] = {TERMINAL, ""};

    memset(pctx->refs,0,MAXREF * sizeof (GENPTR));
    /* init to no global refs */
    (*pctx).numrefs    = 0;
    (*pctx).parmst.link = NULL;
    (*pctx).locst.link = NULL;
    (*pctx).qualst.link = NULL;
    /* if not on primary level */
    if ((*pctx).prclevel > 0)
    {
        for (p=glbtab.link, i=0; p != NULL; p=(*p).v_link)
        {
            if ((*p).v_implicit)
            {
                /* ref the implicits */
                (*pctx).refs[i] = p;
                i++;
            }
        }
        (*pctx).numrefs = i;
        /* installation exit for other refs */
        refs_ins(pctx);
        /* not in procedure body */
        (*pctx).inbody     = FALSE;
        (*pctx).pdf.name[0] = EOS;
        /* no internal procs yet */
        (*pctx).int_procs = NULL;
    }
    else
    {
        /* fake for environment checking */
        (*pctx).inbody = TRUE;
        strcpy((*pctx).pdf.name,"TERMINAL");
        /* add stdout name */
        if (addstr(&((*pctx).locst), "_STDOUT", 2, 2, standout, V_LOCAL)
            != SUCCESS)
        {
            /* report memory overflow */
            overr();
            return(FAIL);
        }
    }
    /* proc position not yet saved */
    (*pctx).prcpos.possav = FALSE;
    /* init to proc type unknown */
    (*pctx).proctype   = Y_UNKNOWN;
    (*pctx).pdf.libr[0] = EOS;
    (*pctx).pdf.type[0] = EOS;
    (*pctx).pdf.attr[0] = EOS;
    (*pctx).subcmd[0]  = EOS;
    (*pctx).subsiz = 0;
    (*pctx).exe_spec[0] = EOS;
    (*pctx).help_spec[0] = EOS;
    /* not yet in any SUBCMD block */
    (*pctx).subblk     = FALSE;
    /* not yet in wrong SUBCMD block */
    (*pctx).subact     = TRUE;     
    /* correct SUBCMD block not yet found */
    (*pctx).subfnd     = FALSE;      
    /* no subcmds seen yet */
    (*pctx).subs       = FALSE;
    /* interrupts not allowed */
    (*pctx).interrupt  = FALSE;
    /* not a compiled PDF */
    (*pctx).compiled   = FALSE;
    /* assume not a PAR file */
    (*pctx).parfile    = FALSE;
    /* assume not selftutor */
    (*pctx).selftutor  = FALSE;
    /* assume not compressed tutor */
    (*pctx).compress = FALSE;
    /* not executing onfail cmd */
    (*pctx).onfailcmd  = FALSE;
    /* init local if stack */
    stk_in((*pctx).locifs, MAXIF, TRUE);
    /* init global if stack */
    stk_in((*pctx).glbifs, MAXIF, TRUE);
    /* init type stack */
    stk_in((*pctx).typstk, MAXNEST, TYPE_NONE);
    /* current line number */
    (*pctx).pdf_line = 0;
    /* init chain of iteration blocks */
    (*pctx).icbptr = NULL;
    /* no search in progress */
    (*pctx).srchblk = NULL;
    /* no subcmd chain */
    (*pctx).subptr = NULL;
    /* installation courtesy */
    (*pctx).ctx_ins = NULL;
    /* no special processing yet */
    (*pctx).special = NOT_SPECIAL;
    /* no compilation data yet */
    (*pctx).comp = NULL;
    /* pointer to ACB being constructed */
    (*pctx).acbptr = NULL;
    return(SUCCESS);
}

/*      init_localjob. Perform initialization for local job processing
 */

FUNCTION static CODE init_localjob
(
    FUNINT *run_type        /* out: runtype; ASYNC or BATCH   */
)
{
    IMPORT struct SFILE  stdo_fil;      /* Actual stdout SFILE          */
    IMPORT struct SFILE  *pstdo_fil;    /* Pointer to stdout SFILE      */
    IMPORT struct SFILE  job_file;      /* primary input file           */
    IMPORT struct VARIABLE *run_gbl;    /* ptr to runtype global var    */
    IMPORT struct VARIABLE *sfi_gbl;    /* $SFI                         */
    IMPORT struct VARIABLE *skey_gbl;   /* $SKEY                        */
    IMPORT struct CONTXT   primctx;     /* primary proc context         */

    struct VARIABLE     *stdoptr;       /* Pointer to _stdout local symbol */
    static TEXT         *standout[2] = {NULL, "APPEND"};
    TEXT                record[STRINGSIZ+1];
    CODE                code;

    pstdo_fil = &stdo_fil;              
    code = f_ophf (pstdo_fil, STDOUTLUN, PRIMARY_STDOUT, F_WRITE);
    if (code != SUCCESS)
        pstdo_fil = NULL;
    code = f_ophf (&job_file, STDINLUN, PRIMARY_INPUT, F_READ);
    if (code != SUCCESS)
    {
        put_stdout ("Unable to access job file.");
        return (FAIL);
    }
    if (pstdo_fil != NULL)
    {
        stdoptr = lookex (&primctx.locst, "_STDOUT");   
        /* value vector */
        standout[0] = (*pstdo_fil).full_spec;
        /* set _STDOUT */
        set_value (stdoptr, (GENPTR) standout, 2);
    }
    /* read first data record which tells us BATCH vs ASYNC..*/
    f_read (&job_file, record);
    if (s_lseq ("BATCH", record))
    {
        /* read new process name */
        f_read (&job_file, record);
        set_process_name (BATCH, record);
        *run_type = BATCH;
        set_string (run_gbl, "BATCH");
    }
#ifdef TAE_ASYNC
    else if (s_lseq ("ASYNC", record))
    {
        /* read new process name */
        f_read (&job_file, record);
        set_process_name (ASYNC, record);
        /* local job execution  */
        *run_type = ASYNC;
        set_string (run_gbl, "ASYNC");
        /* read init message */
        get_asyctx();
        /* if no stdout file set error status */
        if (pstdo_fil == NULL)
        {
            set_string (skey_gbl, "TAE-STDOUTOPEN");
            IVAL (*sfi_gbl, 0) = -1;
            /* send status code to parent TM */
            emit_stat();
            /* terminate job */
            async_exit();
            return (FAIL);
        }
    }
#endif
    else
    {
        put_stdout ("Job file incorrectly formatted.");
#ifdef TAE_ASYNC
        if (*run_type == ASYNC)
        {
            set_string (skey_gbl, "TAE-BADJOBFILE");
            IVAL (*sfi_gbl, 0) = -1;
            async_exit();
        }
#endif
        return (FAIL);
    }
    return (SUCCESS);
}

/*      memtab - build a symbol table from a table of resident variables.
 *
 * NOTE: For speed, memtab gets VARIABLE blocks from the intrinsic parameter
 * pool (ipp) rather than allocating them in dynamic memory. This block
 * already has the v_cvp set so that the value vector is not allocated any
 * more. memtab points the v_valid field to the RESIDVAR, which is
 * typically static.
 */

FUNCTION CODE memtab 
(
    struct RESIDVAR *resvar,        /* table of resident variables          */
    FUNINT          numres,         /* number of resident variables         */
    struct SYMTAB   *s              /* out: symbol table header             */
)
{
    struct RESIDVAR *p;         /* table of resident variables          */
    struct VARIABLE *v;        /* dynamic in-memory variable           */
    COUNT           i;
    COUNT           j;
    TEXT            **bidsv;    /* blt-in dflt val ptr to vec of ptrs   */
    COUNT           bytes;

    /* initialize symbol table */
    initab(s);
    p = resvar;
    /* step thru resident parm table */
    for (j = 0; j < numres; j++, p++)
    {
        /*   If possible, we allocate from the special IVP VARIABLE pool
        because intrinsic processing is so important to TCL performance.
        In the pool, the largest v_maxc is INT_MAXVAL and REALs are not 
        allowed--this covers most intrinsic parms and keeps the pool at
        a reasonable size.  Note that DEFAULT, VALID, and INITIAL in 
        the PARM and LOCAL statements demand the full MAXVAL and these 
        cannot use the fast IVP pool.
        */
        if ((*p).maxc <= INT_MAXVAL && (*p).type != V_REAL)     
        {
            /* get an ivp at end of chain */
            v = get_ivp(s);
            (*v).v_valid = (*p).valid;
        }
        else
        {
            /* use slow (but large) pool */
            /* for simplicity: no valids here */
            if ((*p).valid)             
                tmierr (503);   
            /* allocate normally (but slow) */
            v = allvar (s);
            (*v).v_valid = NULL;
            /* must set these fields for allval */
            (*v).v_maxc = (*p).maxc;
            (*v).v_type = (*p).type;
            /* allocate value vector */
            (*v).v_cvp = allval (v);
        }
        /* move parm name */
        strcpy((*v).v_name,(*p).name);
        (*v).v_class = V_PARM;
        /* TBD: allow NAME variables in memtab; following assumes 'valued' */
        (*v).v_type     = (*p).type;
        (*v).v_keyword  = (*p).kiv;
        /* assume no value for now */
        (*v).v_count    = -1;
        (*v).v_nullable = ((*p).minc == 0);
        (*v).v_minc     = ((*p).minc == 0) ? 1 : (*p).minc;
        (*v).v_maxc     = (*p).maxc;
        (*v).v_dcount = (*p).dcount;

        /* If the RESIDVAR specifies a .dcount of zero, yet the .minc does not
           allow nullables, we assume that there is no default. */
        (*v).v_default = (*p).dcount > 0 ||
                         (*v).v_nullable && (*p).dcount == 0 ;
        (*v).v_size = (*p).size;
        (*v).v_dvp = NULL;
        /* default value present? */
        if ((*v).v_default)
        {
            (*v).v_count = (*p).dcount;
            /* copy values if integer */
            if ((*v).v_type == V_INTEGER)
            {
                bytes = (*v).v_count * valsize((*v).v_type);
                /* set current */
                memcpy((*v).v_cvp,(*p).dvp,bytes);
            }
            else if ((*v).v_type == V_STRING)
            {
                /* intrin deflt */
                bidsv = (TEXT **) (*p).dvp;
                for (i = 0; i < (*v).v_dcount; i++)
                {
                    if (( SVAL(*v, i)=strdup(bidsv[i])) == NULL)
                        goto mt_overr;
                }
            }
        }
    }
    return(SUCCESS);

mt_overr:
    /* report memory overflow */
    overr();
    /* delete corrupted symbol table */
    deltab(s);
    return(FAIL);
}

/*      misprm - return list of missing parameters, i.e., parameters
 *      which do not have a current value specified.
 *      Parms in returned list are separated by commas.
 *      Parameter qualifiers are also checked and, if missing, added to the
 *      list as "parm.qual".
 *
 *      Returns number of missing parms (that fit in mislst).
 */

FUNCTION COUNT misprm 
(
    struct SYMTAB *symtab,            /* in:  symbol table            */
    TEXT          mislst[STRINGSIZ+1] /* out: list of missing */
)
{
    struct VARIABLE *v;            /* variable in symbol table     */
    struct VARIABLE *vq;
    COUNT           listlen;        /* length of missing parms      */
    BOOL            missing;        
    COUNT           count;

    count = 0;
    /* clear the list */
    mislst[0] = EOS;
    listlen = 0;
    for (v = (*symtab).link; v != NULL; v = (*v).v_link)
    {
        missing = ((*v).v_type == V_NAME && (*v).v_ref == NULL) ||
                  ((*v).v_type != V_NAME && (*v).v_count == -1);
        if (missing)
        {
            count++;
            if ((strlen((*v).v_name) + listlen + 2) > STRINGSIZ)
            {
                /* term string leaving comma */
                mislst[listlen] = EOS;
                return(count);
            }
            /* append name to list */
            strcat(mislst,(*v).v_name);
            strcat(mislst,",");
            listlen = strlen(mislst);
        }
        /* quals possible? */
        if ((*v).v_pv12)           
        {
            /* for each qualif of this parm */
            for (vq = (*v).v_qualst.link; vq != NULL; vq = (*vq).v_link)
            {
                missing = ((*vq).v_type == V_NAME && (*vq).v_ref == NULL) ||
                          ((*vq).v_type != V_NAME && (*vq).v_count == -1);
                if (missing)
                {
                    count++;
                    if ((strlen((*v).v_name) +
                         strlen((*vq).v_name) + listlen + 3) > STRINGSIZ)
                    {
                        /* term string leaving comma */
                        mislst[listlen] = EOS;
                        return(count);
                    }
                    strcat(mislst,(*v).v_name);
                    strcat(mislst,".");
                    /* appends parm.qual to list */
                    strcat(mislst,(*vq).v_name);
                    strcat(mislst,", ");
                    listlen = strlen(mislst);
                }
            }
        }
    }
    if (listlen > 1)
    {
        /* delete last comma */
        mislst[listlen-2] = EOS;                
    }
    return(count);
}

/*
 *      plcini - proc local symbol table initialization.
 *
 *      NOTE: When plcini is called (for procs only) the pdf must be
 *      open so that CONTXT.pdf and CONTXT.subcmd are set up.
 */

FUNCTION CODE plcini 
(
    struct CONTXT *pctx     /* in/out:  proc context        */
)
{
    TEXT        *vptr[1];               /* string value vector          */
    TAEINT      intval[1];              /* _TUTOR value vector          */

    static TEXT *standout[] = {TERMINAL, ""};
    static TEXT *retern[] = {"RETURN"};

    /* add stdout name */
    if (addstr(&((*pctx).locst), "_STDOUT", 2, 2, standout, V_LOCAL) != SUCCESS)
        goto pl_overr;
    if (addstr(&((*pctx).locst), "_ONFAIL", 2, 1, retern, V_LOCAL) != SUCCESS)
        goto pl_overr;
    vptr[0] = (*pctx).pdf.name;
    if (addstr(&((*pctx).locst), "_PROC", 1, 1, vptr, V_LOCAL) != SUCCESS)
        goto pl_overr;
    vptr[0] = (*pctx).subcmd;
    if (addstr(&((*pctx).locst), "_SUBCMD", 1, 1, vptr, V_LOCAL) != SUCCESS)
        goto pl_overr;
    intval[0] = 0;
    if (addint(&((*pctx).locst), "_TUTOR", 1, 1, intval, V_LOCAL) != SUCCESS)
        goto pl_overr;
    return(SUCCESS);

pl_overr:
    /* report memory overflow */
    overr();
    return(FAIL);
}

/*
 *      prc_nextcmd. Process the next TCL command.
 *
 *      The next command is received from the argument next_cmd.
 *      if next_cmd is empty, then the command is received from the
 *      operator (for interactive case) or from the job file (for batch
 *      and async case).
 */

FUNCTION static CODE prc_nextcmd
(
    TEXT        next_cmd[], /* in/out: next TCL  command to execute */
    FUNINT      usermode,   /* in: user mode, COMMAND vs MENU     */
    struct  CONTXT *primctx /* in: address of primary context blk */
)
{
    IMPORT struct VARIABLE *prompt_gbl; /* prompt string */
    TEXT        cmdstr[CMDLINSIZ+1];
    CODE        code;

    if (usermode == CMDMODE)
    {
        /* HELP user gave next cmd  */
        if (!NULLSTR(next_cmd))
        {                   
            strcpy (cmdstr,next_cmd);
            /* delete it for next run */
            next_cmd[0] = EOS;
        }
        else
        {
            /* check for an async request */
            check_asy_req();
            /* check for async jobs done and waiting for completion msg */
            chk_asy_done();
            /* get cmd from operator/job file   */
            code = getcln (SVAL(*prompt_gbl,0), cmdstr);
            if (code == F_EOF)
            {
                /* end of job file */
                /* End of batch/async err */
                logoff_do(primctx, primctx);
                return (F_EOF);
            }
        }
    }
    else
    {
        /* assume menu mode */
        /* display menu */
        menmod(cmdstr);
    }
    /* if null input, get command again */
    if (NULLSTR(cmdstr))
        return (SUCCESS);

    history_save(cmdstr);
    /* process command */
    prccmd(cmdstr, primctx);
    return (SUCCESS);
}

/*      prccmd - process TAE command mode command string.
 *      Executes proc activation and intrinsic commands.
 *      WARNING:  delgbl_do imitates this function.  When making any changes
 *                here, analyze delgbl_do to see if analogous changes are
 *                required there.
 *
 *      Return codes:
 *              DO_*** codes as for intrinsic commands.
 */

FUNCTION CODE prccmd 
(
    TEXT   cmdstr[CMDLINSIZ+1],   /* in/out:  command string */
    struct CONTXT  *procctx       /* in:  current proc context */
)
{
    IMPORT struct ECB    ecbi;      /* event block for operator interrupts  */
    IMPORT struct SFILE  prcfil;    /* proc file context (for f_ functions) */
    IMPORT CODE          parmlatch; /* parm latch flag                      */
    IMPORT struct CONTXT latchctx;  /* latched context                      */
    IMPORT CODE          run_type;  /* BATCH, INTER, or ASYNC               */
    IMPORT CODE          help_proc; /* SAVEPROC or NOPROC                   */
#ifdef TAE_ASYNC
    IMPORT struct SYMTAB asy_parmst;/* params from parent monitor, async    */
#endif
    IMPORT CODE          usermode;
    IMPORT struct VARIABLE *skey_gbl; /* pointer to $SKEY variable          */

    CODE                runt;           /* run type requested */
    struct SYNBLK       sb;             /* syntax block (for syntax package) */
    TEXT                cmd[FSPECSIZ+1];/* command line command */
    struct ITRCMD       *itrcmd;        /* intrinsic command */
    struct CONTXT       cmdctx;         /* command context */
    CODE                cmdcod;         /* command return code */
    CODE                qual_code;      /* qualifier return code */
    CODE                code;
    BOOL                active;         /* true if we are executing commands */
    BOOL                we_care;        /* do we care about subsitute errors */
    BOOL                get_parm_qual;  /* quick flag for... */

    get_parm_qual =  (*procctx).special == GET_PARM_QUAL;
    /* mark new proc as next level */
    cmdctx.prclevel = (*procctx).prclevel + 1;
    if (cmdctx.prclevel > MAX_PROC_NEST)
        {
        tmmsg(PROCFAIL, "Proc nesting exceeds %d levels.",
              "TAE-MAXPNEST", MAX_PROC_NEST);
        return (DO_RETURN);
        }
    if (inictx(&cmdctx) != SUCCESS)
    {
        /* init context of next proc */
        return (DO_RETURN);
    }
    /* link to previous */
    cmdctx.backlink = procctx;
    /* let point to syntax block */
    cmdctx.sb = &sb;
    /* if interactive level enable operator attention */
    if ((*procctx).prclevel == 0)
        t_attn(&ecbi);


    active = (*procctx).srchblk == NULL &&  true_part(procctx);
    we_care = active;
    /* perform substitution */
    code = substitute (cmdstr, CMDLINSIZ, procctx, we_care);
    /* if substitute error */
    if (code != SUCCESS  &&  we_care)
        goto subst_err;

    code = cmd_parse (&sb, cmdstr, (*procctx).label, cmd, cmdctx.subcmd);
    if (code != SUCCESS)
        goto syntax_err;
    cmdctx.subsiz = strlen (cmdctx.subcmd);
    /* resolve GOTO search */
    chk_label(procctx);
    /* re-calc for found label */
    active = (*procctx).srchblk == NULL &&  true_part(procctx);
    /* in case !active */
    cmdcod = DO_SUCCESS;
    /* ignore null lines */
    if (NULLSTR(cmd))
        cmdcod = DO_SUCCESS;
    else if ((code = itr_lookup(cmd, cmdctx.subcmd, &itrcmd)) == SUCCESS)
    {
        /* intrinsic: */
        /* no intrin parms as qualifiers */
        if (get_parm_qual)
            goto itr_p_qual_err;
        /* if latched, tutor will know */
        cmdctx.intrinsic = TRUE;
        /* save cmd for tutor, etc */
        strcpy(cmdctx.pdf.name,(*itrcmd).cmd);
        /* get full subcmd name */
        strcpy(cmdctx.subcmd,(*itrcmd).subcmd);
        /* clr latching if not tutor */
        if (!s_equal("TUTOR", (*itrcmd).cmd))
            clr_latch (procctx);
        /* clr previous help proc name */
        if (!s_equal("HELP", (*itrcmd).cmd))
            help_proc = NOPROC;
        /* if "do" should be called */
        if (call_do(procctx, itrcmd))
        {
            /* echo if appropriate */
            echo (procctx, cmdstr);
            /* check command environment */
            if (chk_do(procctx, itrcmd) != SUCCESS)
                goto close_return;
            qual_code = setqlf(cmdctx.intrinsic, &sb, &cmdctx.qualst) ;

            /* an intrinsic may have qualifiers (if Y_QUALIF) but must 
               do its own processing of the qualifiers */
            if (qual_code != S_NOKW   && !((*itrcmd).flags & Y_QUALIF))
            {
                /* not allowed for intrinsics   */
                goto qual_err;
            }
            /* proc invocation syntax */
            if ((*itrcmd).flags & Y_PROCSYN)
            {
                /* so do syntax scan            */
                if (memtab((*itrcmd).partab, (*itrcmd).numprm, 
                           &(cmdctx.parmst)) != SUCCESS)
                    goto close_return;
                if (updtab(&cmdctx.parmst, &sb) != SUCCESS)
                    goto intrinsic_cmderr;
                if (list_missing (&cmdctx.parmst) != SUCCESS)
                    goto intrinsic_cmderr;
            }
            /* init $SFI/$SKEY? */
            if ((*itrcmd).flags & Y_CLEAR)
                ini_status();

            /* call "do" function */
            cmdcod = (*(*itrcmd).cproc)(procctx, &cmdctx);
        }
    }
    else if (code == ITR_BADSUB)
    {
        /* bad intrinsic subcmd */
        goto subcmd_err;
    }

    /* Here, the command is a proc (or an intrinsic with Y_PDF set)  */
    else if (!(*procctx).inbody && active && !get_parm_qual)
    {
        /* pre-body proc activation: */
        tmmsg(PROCFAIL, "Unrecognized declaration command '%s'.",
              "TAE-BADDECL", cmd);
        /* no _ONFAIL pre-body */
        cmdcod = DO_RETURN;
    }
    /* activate proc if 'active'    */
    else if (active)
    {
        /* get full spellings: */
        if (code == ITR_PDF)
        {
            strcpy(cmd,(*itrcmd).cmd);
            strcpy(cmdctx.subcmd,(*itrcmd).subcmd);     
        }
        /* clear latching if necessary */
        clr_latch(procctx);
        /* not an intrinsic */
        cmdctx.intrinsic = FALSE;

        /* set qualifers */
        if (setqlf(cmdctx.intrinsic, &sb, &cmdctx.qualst) == FAIL)
            goto close_return;
        if (get_parm_qual)
        {
            /* force NORUN   */
            runt = NORUN;
        }
        else
        {
            /* get |RUNTYPE| */
            runt = runtype (&cmdctx.qualst);
        }
        /* echo if appropriate */
        echo (procctx, cmdstr);
        /* if not from primary level, close calling proc & save pos */
        if ((*procctx).prclevel > 0)
            clssav(procctx);
        /* initialize $SFI and $SKEY */
        ini_status();

        if (opnpdf(procctx, &cmdctx, cmd, &prcfil) != SUCCESS)
            goto re_open;
        /* trace if selected */
        tr_echo (procctx, prcfil.full_spec);
        /* init implicit locals         */
        if (plcini(&cmdctx) != SUCCESS)
            goto pdf_error;
        if (pdftab(&cmdctx, &prcfil) != SUCCESS)        
            goto pdf_error;
        /* save posit for latching */
        f_movpos(&prcfil.posctx, &cmdctx.prcpos);
        /* do RESTORE qualifier */
        if (restore (&cmdctx) != SUCCESS)
            goto pdf_error;
        if (action_setup (&cmdctx) != SUCCESS)
            goto pdf_error;
        if (!get_parm_qual && procctx == &primctx && run_type == ASYNC)   
        {
            /* use params from parent monitor */
#ifdef TAE_ASYNC
            code = parmrg (&asy_parmst.link, &(cmdctx.parmst), FULLSET);
            /* NOTE: we assume here that NAME */
            deltab (&asy_parmst);
#endif
        }
        /* are not in asy_parmst        */
        else
        {
            if (updtab(&cmdctx.parmst, &sb) != SUCCESS)
                goto pdf_cmderr;        
            if (!get_parm_qual && list_missing (&cmdctx.parmst) != SUCCESS)
                goto pdf_cmderr;
        }
        /* do SAVE qualifier */
        if (save (&cmdctx) != SUCCESS)
            goto pdf_error;
        /* but ok for ASYNC-PROCESS */
        if (runt == BATCH || runt == ASYNC)
        {
            if (s_equal(cmdctx.pdf.libr, "/LOCAL/"))
                goto runt_error;
        }
        if (runt == BATCH)
            cmdcod = run_batch (procctx, &cmdctx);
#ifdef TAE_ASYNC
        else if (runt == ASYNC  ||  runt == ASYNC_PROCESS)
            cmdcod = run_async (procctx, &cmdctx);
#endif
        else if (runt == INTER)
        {
            /* set new STDOUT */
            if (set_stdout(procctx, &cmdctx) != SUCCESS) 
                goto pdf_error;
            /* delete qualst to avoid nesting */
            deltab(&cmdctx.qualst);
            /* re-delete by clsctx won't hurt */
            cmdcod = run_proc(&cmdctx);
        }
        else if (get_parm_qual)
        {
            /* attach param quals    */
            save_p_qual(procctx, &cmdctx);
        }
        else if (runt == NORUN) 
        {
            /* $SKEY = PDF file spec */
            set_string (skey_gbl, prcfil.full_spec);
        }
        else
        {
            tmmsg (PROCFAIL, "Unrecognized RUNTYPE qualifier.", "TAE-RUNTYPE");
            cmdcod = DO_CHECK;
        }
        /* close PDF */
        f_close(&prcfil, F_KEEP);
        /* if not from primary level */
        if ((*procctx).prclevel > 0)
        {
            if (opnsav(procctx) != SUCCESS)
                return(DO_RETURN);
        }
    }
    /* close proc context */
    clsctx(&cmdctx);
    return(cmdcod);

subst_err:
    clr_latch(procctx);
    goto close_return;

syntax_err:
    /* report error */
    tmmsg(PROCFAIL, sb.errmsg, "TAE-FMTERR");
    goto close_return;

subcmd_err:
    if (NULLSTR(cmdctx.subcmd))
        tmmsg(PROCFAIL, "Subcommand is required for '%s'.", "TAE-SUBREQ", cmd);
    else
    {
        tmmsg(PROCFAIL, "Undefined or ambiguous subcommand '%s'.",
              "TAE-UNDEFSUB", cmdctx.subcmd);
    }
    goto close_return;

qual_err:
    tmmsg(PROCFAIL, "Command qualifiers not allowed for '%s'.",
          "TAE-NOQUAL", cmd);
    goto close_return;

itr_p_qual_err:
    tmmsg(PROCFAIL, "Intrinsic command not allowed as parameter qualifier "
          "proc.", "TAE-ITRPQUAL");
    goto close_return;

close_return:
    clsctx(&cmdctx);
    return(DO_RETURN);

intrinsic_cmderr:
    if ((*procctx).prclevel == 0 && usermode != MENUMODE)
    {
        parmlatch = RUNLATCH;
        /* save context for tutor */
        MOVE_STRUCT(cmdctx, latchctx);
        put_stdout(tutor_msg);
    }
    else
        clsctx(&cmdctx);
    return (DO_CHECK);

pdf_error:
    f_close(&prcfil, F_KEEP);
    goto re_open;

re_open:
    clsctx(&cmdctx);
    if ((*procctx).prclevel > 0)
    {
        /* reopen calling proc & reposition     */
        opnsav(procctx);
    }
    return(DO_CHECK);

pdf_cmderr:
    if ((*procctx).prclevel == 0  &&  usermode != MENUMODE)
    {
        parmlatch = RUNLATCH;
        /* save context for tutor */
        MOVE_STRUCT(cmdctx, latchctx);
        put_stdout(tutor_msg);
    }
    else
    {
        clsctx(&cmdctx);
        f_close (&prcfil, F_KEEP);
    }
    if ((*procctx).prclevel > 0)
        opnsav(procctx);
    return (DO_CHECK);

runt_error:
    tmmsg (PROCFAIL, "Attempt to run an internal proc in async or batch.",
                        "TAE-RUNINT");
    f_close (&prcfil, F_KEEP);
    goto re_open;
}

/*      repval - replace current value for a variable.
 *      Assumes that current value array has already been allocated (but not
 *      necessarily any strings).
 *
 *      NOTE: This function also deallocates the input value strings if they
 *      are no longer needed.
 */

FUNCTION CODE repval 
(
    struct VARIABLE *v,     /* in/out: symbol table entry           */
    TEXT   *value[],        /* in:  val str ptrs (vals to repl with)*/
    FUNINT count,           /* in:  count of values for this parm   */
    FUNINT by_name          /* in: TRUE if value[0] is the name of  */
                            /* a referenced variable                */
)
{
    IMPORT struct CONTXT *curproc;      /* current proc context         */

    union                               /* union of value vectors       */
    {
        TAEINT          ival[MAXVAL];   /* temp integer array           */
        TAEFLOAT        rval[MAXVAL];   /* temp real array              */
    } vect;
    GENPTR              valvec=NULL;    /* pointer to value vector      */
    COUNT               i;
    COUNT               val_count;      /* value count (of ref var)     */
    struct VARIABLE     *ref=NULL;      /* reference                    */
    CODE                code, val_type;
    BOOL                compiling;      /* TRUE if we're compiling a proc*/


    compiling = ((*curproc).special == COMPILING);
    if ((*v).v_type == V_NAME  ||  by_name)
    {
        if (count != 1)
            goto count_error;
        if (compiling)
            ref = comp_search(value[0], curproc);
        else
            ref = search (value[0], curproc);
        if (ref == NULL)
            {
            tmmsg(PROCFAIL, "Reference to undefined variable '%s'.",
                "TAE-UNDEFVAR", value[0]);
            goto err_exit;
            }
        /* no de-ref globals, except parm dflts */
        if (compiling && (*ref).v_class == V_GLOBAL &&
            !((*v).v_iparm  && s_equal("DEFAULT",(*v).v_name)))
        {
            goto deref_error;
        }
    }

    /*****

    The following "if" is a kludge to handle de-referencing for DEFAULT
    or INITIAL.   Since we don't know the type of the variable being
    defined, we can't properly check the reference.  The approach here
    is to change the type of the DEFAULT or INITIAL variable to be
    V_NAME so the reference can be remembered for parm_do and local_do.

    Normally, when one changes the v_type of a variable from
    "directly valued" to "named", there is a value vector to
    de-allocate.  However, for v_iparm VARIABLE structures, the value
    vector is self-contained and need not be explicitly de-allocated.

    BEWARE: this code makes DEFAULT and INITIAL special parameter
    names for intrinsic commands.

    ******/

    if (by_name && (*v).v_iparm &&
        (s_equal ("DEFAULT", (*v).v_name) || s_equal ("INITIAL", (*v).v_name)))
    {
        (*v).v_type = V_NAME;
    }

    if ((*v).v_type == V_NAME)
    {
        (*v).v_ref = ref;
        (*v).v_default = FALSE;
        /* free string with variable name */
        free (value[0]);
        return (SUCCESS);
    }

    /* for de-referenced variable */
    if (by_name)
    {
        valvec = (*ref).v_cvp;
        val_count = (*ref).v_count;
        val_type = (*ref).v_type;
    }
    else
    {
        val_count = count;
        val_type = (*v).v_type;
        if (val_type == V_INTEGER)
        {
            for (i = 0; i < val_count; i++)
            {
                if (NULLSTR(value[i]))
                    vect.ival[i] = 0;
                else
                    if (s_s2i(value[i], &(vect.ival[i])) != SUCCESS)    
                        goto int_error;
            }
            valvec = (GENPTR) vect.ival;
        }
        else if (val_type == V_REAL)
        {
            for (i=0; i < val_count; i++)
            {
                if (NULLSTR(value[i]))
                    vect.rval[i] = 0.0;
                else
                {
                    /* convert to binary */
                    if (s_s2r(value[i], &(vect.rval[i])) != SUCCESS)
                        goto real_error;
                }
            }
            valvec = (GENPTR) vect.rval;
        }
        else if (val_type == V_STRING)
            valvec = (GENPTR) value;
    }
    code = chk_vector (v, val_type, valvec, val_count, FALSE);
    if (code != SUCCESS)
        goto value_error;
    /* do not free input value strings */
    if (val_type == V_STRING  &&  !by_name)
    {
        /* transfer input string pointers */
        trs_value (v, valvec, val_count);
    }
    else
    {
        code = set_value(v, valvec, val_count); 
        if (code != SUCCESS)
            goto no_memory;
        /* free values' space from dyn memory */
        free_val(value, count);
    }
    /* value no longer defaulted */
    (*v).v_default = FALSE;
    return(SUCCESS);

value_error:
    if (code == V_BADCOUNT)
        goto val_count_error;
    if (code == V_BADSIZE)
        goto size_error;
    if (code == V_BADTYPE)
        goto type_error;
    if (code == V_BADVALID)
        goto valid_error;
    if (code == V_BADAMBIG)
        goto ambig_error;
    /* assume file error otherwise */
    goto file_error;

deref_error:
    tmmsg(PROCFAIL, "De-referencing of globals not allowed in compiled procs.",
        "TAE-COMPDRFG");
    goto err_exit;

val_count_error:
    if (val_count == 0)
    {
        tmmsg (PROCFAIL, "'%s' does not accept a null value.",
               "TAE-NOTNULL", (*v).v_name, (*v).v_minc);
    }
    else if (val_count < (*v).v_minc)
    {
        tmmsg(PROCFAIL, "Too few values. '%s' must have at least %d values.",
              "TAE-TOOFEW", (*v).v_name, (*v).v_minc);
    }
    else
    {
        tmmsg(PROCFAIL, "Too many values. '%s' must have at most %d values.",
              "TAE-TOOMANY", (*v).v_name, (*v).v_maxc);
    }
    goto err_exit;

count_error:
    if (count == 0)
    {
        tmmsg (PROCFAIL, "'%s' does not accept a null value.",
               "TAE-NOTNULL", (*v).v_name, (*v).v_minc);
    }
    else if (count < (*v).v_minc)
    {
        tmmsg(PROCFAIL, "Too few values. '%s' must have at least %d values.",
              "TAE-TOOFEW", (*v).v_name, (*v).v_minc);
    }
    else
    {
        tmmsg(PROCFAIL, "Too many values. '%s' must have at most %d values.",
              "TAE-TOOMANY", (*v).v_name, (*v).v_maxc);
    }
    goto err_exit;

int_error:
    tmmsg(PROCFAIL, "Invalid integer value for '%s'.",
          "TAE-INVINT", (*v).v_name);
    goto err_exit;

real_error:
    tmmsg(PROCFAIL, "Invalid real value for '%s'.",
          "TAE-INVREAL", (*v).v_name);
    goto err_exit;

size_error:
    tmmsg(PROCFAIL, "String too long.  Max string size for '%s' is %d.",
          "TAE-LONGSTR", (*v).v_name, (*v).v_size);
    goto err_exit;

type_error:
    tmmsg(PROCFAIL, "Type mismatch assigning '%s' to '%s'.",
          "TAE-BADTYPE", (*ref).v_name, (*v).v_name);
    goto err_exit;

file_error:
    tmmsg(PROCFAIL, "File not found or incorrect file specification for '%s'.",
          "TAE-BADFILE", (*v).v_name);
    goto err_exit;

no_memory:
    /* report memory overflow */
    overr();
    goto err_exit;

valid_error:
    if ((*v).v_type == V_STRING)
    {
        tmmsg(PROCFAIL, "Value not one of the acceptable strings defined "
              "for '%s'.", "TAE-INVSTR", (*v).v_name);
    }
    else
    {
        tmmsg(PROCFAIL, "Value out of the defined range for '%s'.", "TAE-RANGE",
              (*v).v_name);
    }
    goto err_exit;

ambig_error:
    tmmsg(PROCFAIL, "Value ambiguous for '%s'.", "TAE-AMBIGVAL", (*v).v_name);
    goto err_exit;

err_exit:
    /* free values' space from dyn memory */
    free_val(value, count);
    return (FAIL);
}

/*
 *      restore.  Perform RESTORE qualifier operation;
 */

FUNCTION CODE restore 
(
    struct CONTXT *ctx      /* in/out: command context      */
)
{
    struct VARIABLE             *v;
    TEXT                        errmsg[STRINGSIZ+1];
    CODE                        code;

    v = lookex(&(*ctx).qualst, "RESTORE");
    code = SUCCESS;
    if (!(*v).v_default)
    {
        code = rest_parm (SVAL(*v,0), ctx, errmsg);
        if (code == SOME_REJECTED)
        {
            /* information only */
            tmmsg(SUCCESS, errmsg, "TAE-NOTUSED");
            code = SUCCESS;
        }   
        else if (code != SUCCESS)
            tmmsg (PROCFAIL, errmsg, "TAE-BADPFILE");
    }
    return (code);
}

/*
 *      runtype.  Classify the runtype qualifier.
 *
 *      Returns BATCH, INTER, ASYNC, NORUN, or FAIL
 */

FUNCTION CODE runtype 
(
    struct SYMTAB *qualst   /* in: qualifier symtab */
)
{
    struct VARIABLE *v;
    TEXT            *value;

    v = lookex (qualst, "RUNTYPE");
    if (v == NULL)
        return (FAIL);
    if ((*v).v_default)
    {
        /* no, default to INTER */
        return (INTER);
    }
    value = SVAL(*v,0);
    if (s_lseq (value, "INTERACTIVE") || s_lseq (value, "SYNCHRONOUS"))
        return (INTER);
    if (s_lseq(value, "BATCH"))
        return (BATCH);
#ifdef TAE_ASYNC
    if (s_lseq(value, "ASYNCHRONOUS"))
        return (ASYNC);
    else if (s_lseq (value, "ASYNC-PROCESS"))
        return (ASYNC_PROCESS);
#endif
    if (s_lseq(value, "NORUN"))
        return (NORUN);
    return (FAIL);
}

/*
 *      save.   Perform SAVE qualifier operation.
 */

FUNCTION CODE save 
(
    struct CONTXT *ctx      /* in/out: command context      */
)
{
    struct VARIABLE             *v;
    TEXT                        errmsg[STRINGSIZ+1];
    CODE                        code;

    v = lookex(&(*ctx).qualst, "SAVE");
    code = SUCCESS;
    if (!(*v).v_default)
    {
        code = save_parm (SVAL(*v,0), ctx, F_WRITE, errmsg);
        if (code != SUCCESS)
            tmmsg(PROCFAIL, errmsg, "TAE-SAVERR");
    }
    return (code);
}

/*
 *      save_p_qual - Save parameter qualifiers into parent proc's
 *      latest variable's qualifier symbol table.
 *      Assumes that the qualifiers have been built as a parameter
 *      symbol table in the current command's context.
 *
 *      This function is useful because we assume the last parameter
 *      in the parent's symbol table is the one we're currently getting
 *      qualifiers for.
 */

FUNCTION static VOID save_p_qual 
(
    struct CONTXT *par_ctx, /* in/out: parent context -- gets quals */
    struct CONTXT *cmd_ctx  /* in/out: command context -- quals in parmst*/
)
{
    struct VARIABLE     *v;
    struct SYMTAB       *destst;        /* where to save qualifiers */
    struct SYMTAB       *qualtab;       /* qualifier symbol table   */
    struct SUBCMD       *s;             /* subcmd compilation block */

    /* are we not compiling? */
    if ( ((*par_ctx).comp) == NULL)
        destst= ((*par_ctx).parmqual) ? &(*par_ctx).parmst : &(*par_ctx).locst; 
    else
    {
        /* compilaton mode: */
        /* if in a SUBCMD block */
        if ((*par_ctx).subblk)
        {
            /* current SUBCMD table entry */
            s = (*(*par_ctx).comp).cursub;
            /* sym tab for this SUBCMD blk */
            destst = &(*s).symtab;
        }
        else
        {
            /* if any SUBCMDs found yet */
            if ((*par_ctx).subs)
            {
                /* after subs symbol table */
                destst = &(*(*par_ctx).comp).after;
            }
            else
            {
                /* before subs symbol table */
                destst = &(*(*par_ctx).comp).before;
            }
        }
    }

    /* find last parm in parent's sym tab */
    for (v = (*destst).link; v != NULL; v = (*v).v_link)
    {
        /* if we've found last parm */
        if ((*v).v_link == NULL)
            break;
    }
    /* in case no parms -- shouldn't happen */
    if (v != NULL)
    {
        qualtab = &(*v).v_qualst;
        /* move parm sym tab to this parm's qualifier sym tab 
           i.e., the parms become parameter qualifiers */
        (*qualtab).link = (*cmd_ctx).parmst.link;
        /* prevent early de-allocation by prccmd */
        (*cmd_ctx).parmst.link = NULL;

        /********
            A variable is not allowed to have a NAMEd qualifier because
            there are too many problems, e.g., the default value of the NAME
            might be a local--local to the qualifier PDF--and not exist any
            more.   Another example: it might point to a GLOBAL, resulting
            in the proc with the qualified variable having an undocumented 
            reference to a GLOBAL.

            Here, we delete NAME parameters from the qualifier list 
            (before they cause any more chaos).

        *********/

        for (v = (*qualtab).link;  v != NULL;)
        {
            struct VARIABLE     *v_link = (*v).v_link;  /* PR2621 */
            if ((*v).v_type == V_NAME)
                delvar (qualtab, v);
            v = v_link;
        }
    }
}

/*      updtab - update a symbol table from a command mode command line.
 *      Assumes proc invocation syntax.
 *      Note: The syntax block (sb) is modified by this routine.
 *
 *      Returns:
 *              SUCCESS -- ok
 *              FAIL -- invalid variable specification found
 *              SOME_REJECTED -- some mandatory parm missing
 */

FUNCTION CODE updtab 
(
    struct SYMTAB *symtab,  /* in/out: symbol table */
    struct SYNBLK *sb       /* in/out:  syntax block */
)
{
    GENPTR      pos;
    CODE        code;
    TEXT        *msg;
    TEXT        key[TOKESIZ+1];         /* keyword */
    TEXT        *value[MAXVAL];         /* value pointers */
    COUNT       count;                  /* number of values for single parm */
    COUNT       i;
    struct VARIABLE *v;

    /* Start out assuming values by position        */
    for (v=(*symtab).link; FOREVER; v=(*v).v_link)
    {
        /* save position in cmd stream */
        pos = (*sb).curchr;
        /* try to get a "parm=" */
        code = getkey(sb, key);
        /* if key present, no more positionals */
        if (code == SUCCESS  ||  code == S_KEYWORD)
            break;
        if (code == EOS)
            goto ud_cdone;
        /* no key; start over for positional */
        (*sb).curchr = pos;
        /* get value */
        code = getval(sb, value, MAXVAL, &count);
        if (code == S_SYNERR)
            goto ud_gverr;
        /* done if EOS */
        if (code == EOS)
            goto ud_cdone;
        /* too many positionals */
        if (v == NULL)
            goto ud_toomany;
        /* if value is present */
        if (count >= 0)
        {
            if (repval(v, value, count, code==S_DEREF) != SUCCESS)
                return (FAIL);
            /* update parm quals from cmd line */
            code = upd_p_qlf(sb, v);
            if (code != SUCCESS)
                return (FAIL);
        }
    }


    /* Remaining values must be by parameter name or flagged keyword */
    /* restore posit to get 1st keyword */
    (*sb).curchr = pos;
    while (FOREVER)
    {
        /* get next keyword */
        if ((code = getkey(sb, key)) == EOS)
        {
            /* done if EOS */
            goto ud_cdone;
        }
        /* if parm specified positionally */
        if (code == S_NOKW)
            goto ud_poserr;
        if (code != SUCCESS  &&  code != S_KEYWORD)
            goto ud_gkerr;
        if (code == S_KEYWORD)
        {
            /* get keywords */
            code = getval(sb, value, MAXVAL, &count);
            if (code == S_SYNERR)
                goto ud_gverr;
            if (code == EOS)
                goto key_syntax;
            if (count < 1)
                goto key_error;

            /* get associated parm */
            code = key_trans (symtab, value[0], &v);
            if (code != SUCCESS)
                goto bad_keyword;
        }
        else
        {
            /* find variable name */
            code  = lookab(symtab, key, &v);
            if (code != SUCCESS)
                goto ud_noparm;
            /* get the value */
            code = getval(sb, value, MAXVAL, &count);
            if (code == S_SYNERR)
                goto ud_gverr;
            /* "key=" at end of line */
            if (code == EOS)
                goto ud_cdone;
        }
        /* if value present */
        if (count >= 0)
        {
            if (repval(v, value, count, code==S_DEREF) != SUCCESS)
                return (FAIL);
            /* update parm quals from cmd line */
            code = upd_p_qlf(sb, v);
            if (code != SUCCESS)
                return (FAIL);
        }
    }

ud_cdone:
    return(SUCCESS);

ud_toomany:
    tmmsg(PROCFAIL, "Too many parameter values for this command or too many "
          "qualifiers.", "TAE-PARS");
    for (i=0; i < count; i++)
        free(value[i]);
    return (FAIL);

ud_gverr:
    tmmsg(PROCFAIL, (*sb).errmsg, "TAE-INVPVAL");
    return (FAIL);

key_syntax:
key_error:
    synerr(sb, "Invalid keyword syntax");
    tmmsg(PROCFAIL, (*sb).errmsg, "TAE-KEYSYNTAX");
    return (FAIL);

bad_keyword:
    if (code == AMBIG)
        msg = "Ambiguous keyword '%s'.";
    else
        msg = "Undefined keyword '%s'.";
    tmmsg (PROCFAIL, msg, "TAE-KEYWORD", value[0]);
    return (FAIL);

ud_gkerr:
    tmmsg(PROCFAIL, (*sb).errmsg, "TAE-INVPNAME");
    return (FAIL);

ud_poserr:
    tmmsg(PROCFAIL, "Positional values may not follow values specified by "
          "name.", "TAE-POSERR");
    return (FAIL);

ud_noparm:
    if (code == AMBIG)
        tmmsg(PROCFAIL, "Ambiguous abbreviation '%s'.", "TAE-AMBIGPAR", key);
    else
    {
        tmmsg(PROCFAIL, "'%s' is an undefined parameter or unknown qualifier.",
              "TAE-BADPAR", key, key);
    }
    return (FAIL);
}

