/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/* WRTMSG  SOURCE FILE.
 *
 * Write messages to standard output device ( CRT or hardcopy terminal )
 *
 * CHANGE LOG:
 *
 *      11-jul-83       Purged change log, deleted checkout records,
 *                      and audited global definitions...jtm
 *      14-jul-83       Deleted include of PRIMINC...dm
 *      10-otc-83       Fixed unix lint errors...palm
 *
 */

#include <stdarg.h>
#include "stdh.inp"
#include "taeconf.inp"
#include "inttaelib.h"
#include "tmhost.inp"
#include "terminc.inc"
#include "tminc.inc"
#include "vmcoproto.h"
#include "taelib.h"
#include "local_protos.h"

/*
 * wrterr_valist  - write an error message, accepting a valist argument
 */

FUNCTION VOID wrterr_valist
(
    TEXT msg[],                  /* IN: error message to output */
    TEXT key[],                  /* IN: message key */
    va_list arglist
)
{
    IMPORT  COUNT  termlines;           /* number of lines on crt screen */
    IMPORT  CODE   termtype;            /* terminal type */

    if (termtype == T_CRT)
    {
        /* write to crt */
        m_cput_valist((termlines - ERRLIN), msg, key, arglist);
    }
    else
    {
        /* write to terminal */
        m_put_valist(msg, key, arglist);
    }
}
/*
 * wrterr  - write an error message.
 */

FUNCTION VOID wrterr
(
    TEXT msg[],                  /* IN: error message to output */
    TEXT key[],                  /* IN: message key */
    ...
)
{
    va_list arglist;

    /* get the arguments to this function */
    va_start(arglist,key);
    va_end(arglist);

    wrterr_valist(msg,key,arglist);
}

/*
 * wrtpmt - write a prompt message to terminal
 */

FUNCTION VOID wrtpmt
(
    TEXT msg[]              /* IN: prompt message */
) 
{
    IMPORT  COUNT  termlines;           /* number of lines on crt screen */
    IMPORT  CODE   termtype;            /* terminal type */

 
    if (termtype == T_CRT)                      /* terminal is CRT */
    {
        /* clear prompt line */
        t_lclear(termlines - PMTLIN, 1);
        /* write prompt  */
        t_output((termlines - PMTLIN), 1, msg);
    }
    else
        t_write(msg, T_PROMPT);                 /* write to terminal */ 
}

/*
 * wrttxt - display text on terminal 
 */

FUNCTION VOID wrttxt
(
    FUNINT line,            /* IN: line number to write to */
    FUNINT col,             /* IN: column number to write to */
    TEXT   text[],          /* IN: message text */
    FUNINT blank            /* IN: True if rest to be blanked */
)
{
    IMPORT  CODE   termtype;            /* terminal type */

    if (termtype == T_CRT)
    {
        if (blank) t_lclear(line, 1);           /* write a blank line */
        t_output(line, col, text);              /* output with no cleanup */
    }
    else 
        t_write(text, T_STDCC);                 /* write with carriage return */
}
