/******************************************************************************
NAME:   put_string()
	
PURPOSE: Puts annotation text into an image.

ALGORITHM DESCRIPTION:
   for number of annotation strings
      for number of input images
         for number of bands in input image
            get the line position for the string
            read input image line into a line buffer
            if background grey level is specified use it
            call GENCHR to generate the annotation
            write the line buffer to output


The complete set of printable characters is shown below. Any character not
in this list will appear as a question mark.
	SP  !  "  #  $  %  &  '  (  )  *  +  ,  -  .  /
   	0  1  2  3  4  5  6  7  8  9  :  ;  <  =  >  ?
   	@  A  B  C  D  E  F  G  H  I  J  K  L  M  N  O
   	P  Q  R  S  T  U  V  W  X  Y  Z  [  \  ]  ^  _
   	`  a  b  c  d  e  f  g  h  i  j  k  l  m  n  o
   	p  q  r  s  t  u  v  w  x  y  z  {  |  }  ~  

	    
Here's an example of "font3" for the letter "A":
	                  	 1 1 1 1 1 1 1 1 1 1 2 2 2
	       1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2
       	     1 . . . . . . . X X X X . . . . . . . . . . .
             2 . . . . . . . X X X X . . . . . . . . . . .
       	     3 . . . . . . X X X X X X . . . . . . . . . .
      	     4 . . . . . . X X X X X X . . . . . . . . . .
             5 . . . . . . X X X X X X . . . . . . . . . .
 	     6 . . . . . X X X X X X X X . . . . . . . . .
     	     7 . . . . . X X X . . X X X . . . . . . . . .
             8 . . . . . X X X . . X X X . . . . . . . . .
      	     9 . . . . X X X X . . X X X X . . . . . . . .
            10 . . . . X X X . . . . X X X . . . . . . . .
            11 . . . . X X X . . . . X X X . . . . . . . .
            12 . . . . X X X . . . . X X X . . . . . . . .
     	    13 . . . X X X X . . . . X X X X . . . . . . .
            14 . . . X X X . . . . . . X X X . . . . . . .
     	    15 . . . X X X . . . . . . X X X . . . . . . .
            16 . . . X X X . . . . . . X X X . . . . . . .
     	    17 . . X X X X . . . . . . X X X X . . . . . .
            18 . . X X X X X X X X X X X X X X . . . . . .
            19 . . X X X X X X X X X X X X X X . . . . . .
     	    20 . . X X X X X X X X X X X X X X . . . . . .
            21 . X X X X . . . . . . . . X X X X . . . . .
            22 . X X X . . . . . . . . . . X X X . . . . .
            23 . X X X . . . . . . . . . . X X X . . . . .
            24 . X X X . . . . . . . . . . X X X . . . . .
            25 X X X X . . . . . . . . . . X X X X . . . .
            26 X X X . . . . . . . . . . . . X X X . . . .
            27 X X X . . . . . . . . . . . . X X X . . . .
            28 . . . . . . . . . . . . . . . . . . . . . .
            29 . . . . . . . . . . . . . . . . . . . . . .
            30 . . . . . . . . . . . . . . . . . . . . . .
            31 . . . . . . . . . . . . . . . . . . . . . .
            32 . . . . . . . . . . . . . . . . . . . . . .
            33 . . . . . . . . . . . . . . . . . . . . . .
            34 . . . . . . . . . . . . . . . . . . . . . .
            35 . . . . . . . . . . . . . . . . . . . . . .
            36 . . . . . . . . . . . . . . . . . . . . . .
            37 . . . . . . . . . . . . . . . . . . . . . .
            38 . . . . . . . . . . . . . . . . . . . . . .
            39 . . . . . . . . . . . . . . . . . . . . . .
            40 . . . . . . . . . . . . . . . . . . . . . .


Fonts 2-5 use this format

-----------------------------------------------------------------------
This block data initializes dot table - each character consists of 7x9 dots
to form that character shape..  but 8x12 buffer area used here and 8x10
(cellwide x cellhigh) dots will be used in actual processing..

ex) character "a"

  h-axis dot no.-->  1 2 3 4 5 6 7 8
     1               . . . x . . . .
     2               . . x . x . . .
     3               . x . . . x . .
     4               x . . . . . x .
     5               x x x x x x x .
     6               x . . . . . x .
     7               x . . . . . x .
     8               x . . . . . x .
     9               x . . . . . x .
    10               . . . . . . . .
    11               . . . . . . . .
    12               . . . . . . . .

Font 1 uses this format

******************************************************************************/
#include "las.h"    /* las header file */
#include "iovars.h" /* declare variables for image IO */
#include "util.h"
#include "pixman.h"
#include "annotate.h"

/*****************************************************************************

  Initialize the font table.  Characters are defined in ASCII order, and each 
  character is separated by { }.

******************************************************************************/
unsigned long font1[96][3] =
{0x00000000,0x00000000,0x00000000,           /* "sp" */
0x10101010,0x10100000,0x10000000,            /* "!"  */
0xeeeeee44,0x88000000,0x00000000,            /* "quotes" */
0x002424ff,0x2424ff24,0x24000000,            /* "#" */
0x287ca8a8,0x7c2a2a7c,0x28000000,            /* "$" */
0xe0a2e408,0x10204e8a,0x0e000000,            /* "%" */
0x38444438,0x60908a84,0x7a000000,            /* "&" */
0x38383810,0x20000000,0x00000000,            /* "'" */
0x04081020,0x20201008,0x04000000,            /* "(" */
0x40201008,0x08081020,0x40000000,            /* ")" */
0x00925438,0xfe385492,0x00000000,            /* "*" */
0x00101010,0xfe101010,0x00000000,            /* "+" */
0x00000038,0x38380810,0x20000000,            /* "," */
0x00000000,0xfe000000,0x00000000,            /* "-" */
0x00000000,0x1c1c1c00,0x00000000,            /* "." */
0x00020408,0x10204080,0x00000000,            /* "/" */

0x3846868a,0x92a2c2c4,0x38000000,            /* "0" */
0x10305010,0x10101010,0x7c000000,            /* "1" */
0x3844820c,0x10204080,0xfe000000,            /* "2" */
0xfe040810,0x08048244,0x38000000,            /* "3" */
0x08182848,0xfe080808,0x08000000,            /* "4" */
0xfe8080f8,0x04020204,0xf8000000,            /* "5" */
0x3e408098,0xa4c28244,0x38000000,            /* "6" */
0xfe040810,0x10101010,0x10000000,            /* "7" */
0x7c828282,0x7c828282,0x7c000000,            /* "8" */
0x7c828282,0x7e020204,0x78000000,            /* "9" */
0x00383838,0x00383838,0x00000000,            /* ":" */
0x38383800,0x38383810,0x20000000,            /* ";" */
0x04081020,0x40201008,0x04000000,            /* "<" */
0x000000fe,0x00fe0000,0x00000000,            /* "=" */
0x40201008,0x04081020,0x40000000,            /* ">" */
0x7c820204,0x08100000,0x10000000,            /* "?" */

0x7c28289e,0x92929e80,0x7e000000,            /* "@" */
0x10284482,0x82fe8282,0x82000000,            /* "a" */
0xfc828282,0xfc828282,0xfc000000,            /* "b" */
0x7c828080,0x80808082,0x7c000000,            /* "c" */
0xfc424242,0x42424242,0xfc000000,            /* "d" */
0xfe808080,0xf8808080,0xfe000000,            /* "e" */
0xfe808080,0xf8808080,0x80000000,            /* "f" */
0x38448280,0x808e8246,0x3a000000,            /* "g" */
0x82828282,0xfe828282,0x82000000,            /* "h" */
0x7c101010,0x10101010,0x7c000000,            /* "i" */
0x1c080808,0x08088850,0x20000000,            /* "j" */
0x82848890,0xa0d08884,0x82000000,            /* "k" */
0x80808080,0x80808080,0xfe000000,            /* "l" */
0x8282c6aa,0x92828282,0x82000000,            /* "m" */
0x8282c2a2,0x928a8682,0x82000000,            /* "n" */
0x38448282,0x82828244,0x38000000,            /* "o" */
0xfc828282,0xfc808080,0x80000000,            /* "p" */
0x38448282,0x82928a44,0x3a000000,            /* "q" */
0xfc828282,0xfc908884,0x82000000,            /* "r" */
0x38448240,0x38048244,0x38000000,            /* "s" */
0xfe101010,0x10101010,0x10000000,            /* "t" */
0x82828282,0x82828282,0x7c000000,            /* "u" */
0x82828244,0x44282810,0x10000000,            /* "v" */
0x82828282,0x92aac682,0x82000000,            /* "w" */
0x82824428,0x10284482,0x82000000,            /* "x" */
0x82824428,0x10101010,0x10000000,            /* "y" */
0xfe020408,0x10204080,0xfe000000,            /* "z" */
0x3c202020,0x20202020,0x3c000000,            /* "[" */
0x80402010,0x08040201,0x00000000,            /* "\" */
0x3c040404,0x04040404,0x3c000000,            /* "]" */
0x00000010,0x28440000,0x00000000,            /* "^" */
0x00000000,0x00000000,0xfe000000,            /* "_" */

0x00000000,0x00000000,0x00000000,            /* "not used" */
0x00007884,0x047c8484,0x7a000000,            /* "a" */
0x00008080,0x80fc8282,0xfc000000,            /* "b" */
0x00003c42,0x80808042,0x3c000000,            /* "c" */
0x00000202,0x027e8282,0x7e000000,            /* "d" */
0x00003c42,0x427e4042,0x3c000000,            /* "e" */
0x00000c10,0x10fe1050,0x30000000,            /* "f" */
0x00007a84,0x84780484,0x78000000,            /* "g" */
0x00004040,0x5c624242,0x42000000,            /* "h" */
0x00000800,0x18080808,0x1c000000,            /* "i" */
0x00000800,0x18080848,0x30000000,            /* "j" */
0x00004042,0x44487844,0x42000000,            /* "k" */
0x00001808,0x08080808,0x1c000000,            /* "l" */
0x000054aa,0xaa2a2a2a,0x29000000,            /* "m" */
0x00006c92,0x92121212,0x11000000,            /* "n" */
0x00003c42,0x42424242,0x3c000000,            /* "o"*/

0x0000bc42,0x42625c40,0x40000000,            /* "p" */
0x00007a84,0x848c7404,0x04000000,            /* "q" */
0x00002c12,0x10101010,0x38000000,            /* "r" */
0x00003c42,0x403c0242,0x3c000000,            /* "s" */
0x0000107c,0x10101014,0x08000000,            /* "t" */
0x00008484,0x84848484,0x7a000000,            /* "u" */
0x00008282,0x82824428,0x10000000,            /* "v" */
0x00008282,0x829292aa,0x44000000,            /* "w" */
0x00008244,0x28102844,0x82000000,            /* "x" */
0x00008244,0x28102040,0x80000000,            /* "y" */
0x0000fe04,0x08102040,0xfe000000,            /* "z" */
0x08101010,0x20101010,0x08000000,            /* "{" */
0x10101010,0x10101010,0x10000000,            /* "|" */
0x10080808,0x04080808,0x10000000,            /* "}" */
0x00000022,0x54880000,0x00000000,            /* "~" */
0x00000000,0x00000000,0x00000000};           /* "not used" */


unsigned int font2[96][24] = {
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       192,       192,       192,       192,       192,
                        192,       192,       192,       192,       192,
                        192,       192,         0,         0,       192,
                        192,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       816,       816,       816,       816,       816,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,      1584,      1584,      1584,      8188,
                       8188,      1584,      1584,      8188,      8188,
                       1584,      1584,      1584,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       192,      1016,      4088,      7360,      6336,
                       7360,      4080,      2040,       220,       204,
                        204,       220,      4088,      4080,       192,
                        192,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         4,      1804,      3480,      6360,      3504,
                       1888,        96,       192,       384,       768,
                        880,      1752,      3468,      3288,      6256,
                       4096,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       960,      1632,      3120,      3120,      1632,
                       1728,       960,      1920,      3456,      7360,
                       6244,      7276,      3128,      1592,      1900,
                        964,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       192,       192,       192,       192,       192,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {        12,        24,        48,        48,        96,
                         96,       192,       192,       192,       192,
                         96,        96,        48,        48,        24,
                         12,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6144,      3072,      1536,      1536,       768,
                        768,       384,       384,       384,       384,
                        768,       768,      1536,      1536,      3072,
                       6144,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,      3096,      3096,
                       1584,       864,      8188,      8188,       864,
                       1584,      3096,      3096,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,       384,       384,
                        384,       384,      8184,      8184,       384,
                        384,       384,       384,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,       384,
                        384,       128,       256,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,      8184,      8184,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,       384,
                        384,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         4,        12,        24,        24,        48,
                         96,        96,       192,       384,       768,
                        768,      1536,      3072,      3072,      6144,
                       4096,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,      2032,      3640,      3128,      3192,
                       7292,      6348,      6348,      6540,      6540,
                       7964,      3864,      3608,      3640,      2032,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       192,       448,       960,       704,       192,
                        192,       192,       192,       192,       192,
                        192,       192,       192,       192,      1008,
                       1008,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      1984,      4080,      7224,      6172,        28,
                         28,        56,       112,       224,       448,
                        896,      1792,      3584,      7168,      8188,
                       8188,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8188,      8188,        12,        24,        48,
                         96,       192,       384,       224,        56,
                         12,      6156,      6172,      3128,      2032,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {        48,       112,       240,       432,       816,
                       1584,      3120,      6192,      6192,      8188,
                       8188,        48,        48,        48,        48,
                         48,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8184,      8184,      6144,      6144,      6144,
                       6144,      6624,      8176,      7736,        28,
                         12,        28,      6168,      7224,      4080,
                        960,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       192,       384,       768,      1536,      1536,
                       3072,      3520,      8176,      7736,      7196,
                       6156,      6156,      7196,      3640,      2032,
                        448,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8188,      8188,        28,        56,       112,
                        112,       224,       448,       448,       896,
                       1792,      1792,      3584,      7168,      7168,
                       6144,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       448,      2032,      3640,      3096,      3096,
                       3640,      2032,      2032,      3640,      7196,
                       6156,      6156,      7196,      3640,      2032,
                        448,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       448,      2032,      3640,      7196,      6156,
                       6156,      6156,      7196,      3644,      2044,
                        472,        24,        48,        48,        96,
                        192,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,       192,       192,         0,         0,
                          0,         0,         0,         0,       192,
                        192,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,       192,       192,         0,         0,
                          0,         0,         0,         0,       192,
                        192,        64,       128,         0,         0,
                          0,         0,         0,         0 },
                {        28,        56,       112,       224,       448,
                        896,      1792,      3584,      3584,      1792,
                        896,       448,       224,       112,        56,
                         28,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                       8188,      8188,         0,         0,      8188,
                       8188,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      7168,      3584,      1792,       896,       448,
                        224,       112,        56,        56,       112,
                        224,       448,       896,      1792,      3584,
                       7168,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       448,      2032,      3640,      7196,      7180,
                       6156,      6172,        56,       112,       224,
                        448,       384,         0,         0,       384,
                        384,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,      2032,      3640,      7196,      6604,
                       7148,      7020,      7020,      6648,      6384,
                       6144,      7168,      7168,      3584,      2040,
                       1016,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       128,       448,       448,       992,       864,
                        864,      1904,      1584,      1584,      3640,
                       4088,      4088,      7196,      6156,      6156,
                       6156,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8160,      8176,      3128,      3100,      3084,
                       3100,      3128,      4080,      4080,      3128,
                       3100,      3084,      3100,      3128,      8176,
                       8160,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,      2032,      3640,      3096,      7196,
                       6156,      6144,      6144,      6144,      6144,
                       6156,      7196,      3096,      3640,      2032,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8128,      8160,      3184,      3120,      3128,
                       3096,      3100,      3084,      3084,      3100,
                       3096,      3128,      3120,      3184,      8160,
                       8128,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8188,      8188,      6144,      6144,      6144,
                       6144,      6144,      8128,      8128,      6144,
                       6144,      6144,      6144,      6144,      8188,
                       8188,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8188,      8188,      6144,      6144,      6144,
                       6144,      6144,      8128,      8128,      6144,
                       6144,      6144,      6144,      6144,      6144,
                       6144,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,      2032,      3640,      3096,      7196,
                       6156,      6144,      6144,      6144,      6144,
                       6396,      7420,      3100,      3640,      2032,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6156,      6156,      6156,      6156,      6156,
                       6156,      6156,      8188,      8188,      6156,
                       6156,      6156,      6156,      6156,      6156,
                       6156,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      1008,      1008,       192,       192,       192,
                        192,       192,       192,       192,       192,
                        192,       192,       192,       192,      1008,
                       1008,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       252,       252,        48,        48,        48,
                         48,        48,        48,        48,        48,
                         48,      6192,      7280,      3808,      1984,
                        896,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6172,      6200,      6256,      6368,      6592,
                       7040,      7936,      7680,      7680,      7936,
                       7040,      6592,      6368,      6256,      6200,
                       6172,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6144,      6144,      6144,      6144,      6144,
                       6144,      6144,      6144,      6144,      6144,
                       6144,      6144,      6144,      6144,      8188,
                       8188,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      7196,      7740,      7740,      8060,      7020,
                       7020,      7148,      6604,      6604,      6284,
                       6156,      6156,      6156,      6156,      6156,
                       6156,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      7692,      7692,      7948,      6924,      6924,
                       7052,      6540,      6540,      6604,      6348,
                       6348,      6380,      6252,      6252,      6268,
                       6204,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,      2032,      3640,      3096,      3096,
                       7196,      6156,      6156,      6156,      6156,
                       7196,      3096,      3096,      3640,      2032,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8160,      8176,      6200,      6172,      6156,
                       6172,      6200,      8176,      8160,      6144,
                       6144,      6144,      6144,      6144,      6144,
                       6144,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,      2032,      3640,      3096,      3096,
                       7196,      6156,      6156,      6156,      6284,
                       7388,      3192,      3128,      3640,      2044,
                        996,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8160,      8176,      6200,      6172,      6156,
                       6172,      6200,      8176,      8160,      7936,
                       7040,      6592,      6368,      6256,      6200,
                       6172,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,      2032,      3640,      7196,      6156,
                       6144,      7168,      4064,      1016,        28,
                         12,      6156,      7196,      3640,      2032,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8184,      8184,       384,       384,       384,
                        384,       384,       384,       384,       384,
                        384,       384,       384,       384,       384,
                        384,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6156,      6156,      6156,      6156,      6156,
                       6156,      6156,      6156,      6156,      6156,
                       6156,      6156,      7196,      3640,      2032,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6156,      6156,      7196,      3096,      3096,
                       3640,      1584,      1584,      1904,       864,
                        864,       992,       992,       448,       448,
                        128,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6156,      6156,      6156,      6156,      6156,
                       6156,      6284,      6604,      6604,      7148,
                       7020,      7020,      8060,      7740,      7740,
                       7196,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6156,      3096,      3096,      1584,      1584,
                        864,       864,       448,       448,       864,
                        864,      1584,      1584,      3096,      3096,
                       6156,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6156,      7196,      7196,      3640,      3640,
                       1904,      1904,       992,       992,       448,
                        448,       448,       448,       448,       448,
                        448,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      8188,      8188,        28,        56,       112,
                        112,       224,       448,       896,       896,
                       1792,      3584,      7168,      7168,      8188,
                       8188,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,       992,       768,       768,       768,
                        768,       768,       768,       768,       768,
                        768,       768,       768,       768,       992,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      4096,      6144,      3072,      3072,      1536,
                        768,       768,       384,       192,        96,
                         96,        48,        24,        24,        12,
                          4,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       992,       992,        96,        96,        96,
                         96,        96,        96,        96,        96,
                         96,        96,        96,        96,       992,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       448,       992,      1904,      3640,      7196,
                       6156,      4100,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,      8188,
                       8188,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       192,       192,        64,       128,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                       2032,      4088,      2060,      1004,      2044,
                       3100,      6156,      6156,      3100,      2044,
                       1004,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      6144,      6144,      6144,      6144,      6144,
                       6144,      7136,      8176,      7736,      7192,
                       6156,      6156,      7192,      7736,      8176,
                       7136,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      2032,      4088,      7196,      6144,
                       6144,      6144,      6144,      7196,      4088,
                       2032,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {        12,        12,        12,        12,        12,
                         12,      1004,      2044,      3644,      3100,
                       6156,      6156,      3100,      3644,      2044,
                       1004,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      2032,      4088,      7196,      6156,
                       8188,      6144,      6144,      7196,      4088,
                       2032,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {        64,       224,       432,       400,       384,
                        384,       384,      2016,      2016,       384,
                        384,       384,       384,       384,       384,
                        384,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      1004,      2044,      3100,      6156,
                       6156,      3100,      2044,      1004,        12,
                         12,      6156,      3096,      2032,       992,
                          0,         0,         0,         0 },
                {      6144,      6144,      6144,      6144,      6144,
                       6144,      7136,      8176,      7192,      6156,
                       6156,      6156,      6156,      6156,      6156,
                       6156,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,       192,       192,         0,
                          0,       448,       448,       192,       192,
                        192,       192,       192,       192,       480,
                        480,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,        12,        12,         0,
                          0,        12,        12,        12,        12,
                         12,        12,        12,        12,        12,
                         12,      6156,      3096,      2032,       992,
                          0,         0,         0,         0 },
                {      3072,      3072,      3072,      3072,      3072,
                       3072,      3072,      3096,      3120,      3168,
                       3264,      3456,      4032,      3680,      3120,
                       3096,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       448,       448,       192,       192,       192,
                        192,       192,       192,       192,       192,
                        192,       192,       192,       192,       480,
                        480,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      1904,      3416,      6284,      6284,
                       6284,      6284,      6284,      6284,      6284,
                       6284,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      7136,      8176,      7192,      6156,
                       6156,      6156,      6156,      6156,      6156,
                       6156,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,       992,      2032,      3640,      3096,
                       6156,      6156,      3096,      3640,      2032,
                        992,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      7136,      8176,      7192,      7196,
                       7196,      7192,      8176,      7136,      6144,
                       6144,      6144,      6144,      6144,      6144,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      1004,      2044,      3100,      6156,
                       6156,      3100,      2044,      1004,        12,
                         12,        12,        12,        12,        12,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      3552,      4080,      3608,      3072,
                       3072,      3072,      3072,      3072,      3072,
                       3072,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      2032,      4088,      7196,      7168,
                       4080,      2040,        28,      7196,      4088,
                       2032,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,       192,       192,       192,
                       2040,      2040,       192,       192,       192,
                        192,       192,       192,       192,       232,
                        112,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      6156,      6156,      6156,      6156,
                       6156,      6156,      3100,      3644,      2044,
                       1004,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      6156,      3096,      3096,      1584,
                       1584,       864,       864,       448,       448,
                        128,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      6284,      6284,      6284,      6284,
                       6284,      6284,      6284,      6284,      3416,
                       1584,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      6156,      3096,      1584,       864,
                        448,       448,       864,      1584,      3096,
                       6156,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      6156,      6156,      6156,      6156,
                       6156,      3100,      2044,      1004,        12,
                         12,      6156,      3096,      2032,       992,
                          0,         0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,      8188,      8188,        56,       112,
                       4088,      4088,      1792,      3584,      8188,
                       8188,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {        16,        48,        96,        96,        96,
                         96,       192,       896,       896,       192,
                         96,        96,        96,        96,        48,
                         16,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       192,       192,       192,       192,       192,
                        192,       192,       192,       192,       192,
                        192,       192,       192,       192,       192,
                        192,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {      1024,      1536,       768,       768,       768,
                        768,       384,       224,       224,       384,
                        768,       768,       768,       768,      1536,
                       1024,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {       512,      1792,      3456,      6540,       216,
                        216,       112,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                {         0,      1016,      2044,      1548,      3598,
                       7399,      6643,      6915,      6915,      6643,
                       7399,      3598,      3612,      1016,       496,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0 },
                            };
unsigned int font3[96][40] = {
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,
			  0,	     0,		0,	   0,         0 },
		{     28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,         0,         0,         0,      8192, 
		      28672,      8192,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    232960,    232960,    232960,    232960,    232960, 
		     232960,    232960,    232960,    232960,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,    460544,    460544,    460544, 
		     460544,    460544,    460544,   4194272,   4194272, 
		    4194272,    460544,    460544,    460544,    460544, 
		     460544,   4194272,   4194272,   4194272,    460544, 
		     460544,    460544,    460544,    460544,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,   
			  0,	     0,		0,	   0,	      0 },
		{     14336,     14336,     14336,    262080,   1048512, 
		    2097088,   1980416,   3946496,   3684352,   3684352, 
		    3946496,   1980416,   2096896,   1048512,    262112, 
		      14816,     14576,     14448,     14448,     14576, 
		      14816,   1048544,   1048512,   1048320,     14336, 
		      14336,     14336,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{        16,        48,    393264,    983136,   1671392, 
		    1671360,   3195328,   3195776,   1671936,   1672960, 
		     986624,    396288,      6144,     14336,     28672, 
		      24960,     50112,    116320,    231008,    199728, 
		     396336,    919136,   1836640,   1573824,   3146112, 
		    3145728,   2097152,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     65024,    130816,    116480,    230272,    230272, 
		     196992,    196992,    230272,    230272,    116480, 
		      60928,     64512,     63488,    129024,    243712, 
		     203776,    465920,    396800,    921088,    788224, 
		    1836848,   1573744,   1836000,    917952,    459744, 
		     262000,    130608,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{       112,       240,       480,       960,       896, 
		       1920,      1792,      3840,      3584,      3584, 
		       7168,      7168,      7168,     14336,     14336, 
		       7168,      7168,      7168,      3584,      3584, 
		       1792,      1792,       896,       960,       480, 
		        240,       112,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670016,   3932160,   1966080,    983040,    458752, 
		     229376,    229376,    114688,    114688,     57344, 
		      57344,     57344,     28672,     28672,     57344, 
		      57344,     57344,    114688,    114688,    245760, 
		     229376,    491520,    458752,    983040,   1966080, 
		    3932160,   3670016,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		     459648,    459648,    231168,    231168,    118272, 
		     118272,     64512,   4194288,   4194288,   4194288, 
		      64512,    118272,    118272,    231168,    231168, 
		     459648,    459648,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,   4194288,   4194288,   4194288, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     28672, 
		      28672,     28672,     12288,     24576,     16384, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,   4194272,   4194272,   4194272, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     28672, 
		      28672,     28672,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{        48,       112,       112,       224,       480, 
		        448,       896,      1920,      3840,      3584, 
		       7680,      7168,     14336,     30720,     28672, 
		      57344,    122880,    114688,    229376,    491520, 
		     458752,    917504,   1966080,   1835008,   3670016, 
		    3670016,   3145728,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     30720,    130560,    261888,    511872,    459648, 
		     917952,    918464,    919488,   1837024,   1838816, 
		    1842912,   3677296,   3684464,   3700848,   3698800, 
		    3727472,   1958112,   1949920,   2064608,   1016256, 
		     983488,    917952,    459648,    511872,    261888, 
		     130560,     30720,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     14336,     30720,     63488,    129024,    129024, 
		     112640,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,    130816, 
		     130816,    130816,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    129024,    523776,   1048320,   1968000,   1835968, 
		    3670496,   3145968,       240,       112,       240, 
		        496,       992,      1920,      3840,      7680, 
		      15360,     30720,     61440,    122880,    245760, 
		     491520,    983040,   1966080,   3932160,   4194288, 
		    4194288,   4194288,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4194288,   4194288,   4194288,       112,       224, 
		        448,       896,      1792,      3584,      7168, 
		      14336,     28672,     28672,     15360,      7936, 
		       1984,       480,       224,       240,   3670128, 
		    3932400,   1966304,   2032096,   1034176,    524160, 
		     130560,     30720,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{       896,      1920,      3968,      8064,     15232, 
		      29568,     58240,    115584,    230272,    459648, 
		     918400,   1835904,   3670912,   3670912,   4194288, 
		    4194288,   4194288,       896,       896,       896, 
		        896,       896,       896,       896,       896, 
		        896,       896,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4194240,   4194240,   4194240,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3734528,   3800832,   4194176,   4162496,   4063680, 
		        480,       224,       240,       112,       112, 
		    3670112,   3932384,   1966528,   1017792,    524160, 
		     261888,     64512,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     28672,     57344,    114688,    114688,    229376, 
		     229376,    458752,    458752,    393216,    917504, 
		     982016,   1048320,   2097024,   2066368,   1966528, 
		    1835232,   3932384,   3670128,   3670128,   3670128, 
		    3932384,   1835232,    917952,   1017792,    524160, 
		     261888,     64512,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4194288,   4194288,   4194288,       240,       480, 
		        960,      1920,      1920,      3840,      7680, 
		       7680,     15360,     30720,     30720,     61440, 
		     122880,    122880,    245760,    491520,    491520, 
		     983040,   1966080,   1966080,   1835008,   3932160, 
		    3670016,   3670016,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     30720,    130560,    524160,   1036224,   2032608, 
		    1835232,   3670128,   3670128,   1835232,   2032608, 
		    1036224,    524160,    524160,   1017792,    917952, 
		    1835232,   3932384,   3670128,   3670128,   3670128, 
		    3932384,   1835232,    917952,   1017792,    524160, 
		     261888,     64512,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     64512,    261888,    524160,   1017792,    917952, 
		    1835232,   1835248,   3670128,   3670128,   3670128, 
		    1835248,   1835232,    917984,   1017824,    524256, 
		     262080,     64960,       448,       384,       896, 
		        896,      1792,      1792,      3584,      3584, 
		       7168,     14336,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     28672, 
		      28672,     28672,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     28672, 
		      28672,     28672,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     28672, 
		      28672,     28672,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     28672, 
		      28672,     28672,     12288,     24576,     16384, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{        16,        48,       112,       480,       960, 
		       3968,      7680,     31744,     61440,    253952, 
		     491520,   2031616,   3932160,   3670016,   3932160, 
		    2031616,    491520,    253952,     61440,     31744, 
		       7680,      3968,       960,       480,       112, 
		         48,        16,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   4194272, 
		    4194272,   4194272,         0,         0,         0, 
		    4194272,   4194272,   4194272,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   2097152,   3145728,   3670016,   1966080,    983040, 
		     507904,    122880,     63488,     15360,      7936, 
		       1920,       992,       240,       112,       240, 
		        992,      1920,      7936,     15360,     63488, 
		     122880,    507904,    983040,   1966080,   3670016, 
		    3145728,   2097152,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     30720,    130560,    261888,    493440,    984000, 
		     917952,   1966304,   1835232,   1835120,   3670128, 
		    3670240,   3670496,      1984,      7936,     15360, 
		      30720,     28672,     61440,     57344,     57344, 
		      57344,         0,         0,         0,     57344, 
		      57344,     57344,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    130560,    524160,   1048512,   1017792,   1966560, 
		    1966560,   1966560,   3932400,   3932400,   3944688, 
		    3700848,   3734640,   3722352,   3722352,   3722464, 
		    3735520,   3702720,   3948288,   3932160,   3932160, 
		    1966080,   1966080,   1966080,   1015808,   1048320, 
		     524032,    130816,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     30720,     30720,     64512,     64512,     64512, 
		     130560,    118272,    118272,    249600,    231168, 
		     231168,    231168,    493440,    459648,    459648, 
		     459648,    984000,   1048512,   1048512,   1048512, 
		    1966560,   1835232,   1835232,   1835232,   3932400, 
		    3670128,   3670128,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4192256,   4193792,   4194176,   1836992,   1835488, 
		    1835232,   1835120,   1835120,   1835120,   1835232, 
		    1835488,   1836992,   2097024,   2096640,   2097024, 
		    1836992,   1835488,   1835232,   1835120,   1835120, 
		    1835120,   1835232,   1835488,   1836992,   4194176, 
		    4193792,   4192256,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    130560,    524160,   1048512,   1017792,   1966560, 
		    1966560,   1966560,   3932400,   3932400,   3932400, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3932400,   3932400,   3932400, 
		    1966560,   1966560,   1966560,   1017792,   1048512, 
		     524160,    130560,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4190208,   4193280,   4194048,   1851264,   1838976, 
		    1835968,   1835456,   1835456,   1835232,   1835232, 
		    1835232,   1835120,   1835120,   1835120,   1835120, 
		    1835120,   1835232,   1835232,   1835232,   1835456, 
		    1835456,   1835968,   1838976,   1851264,   4194048, 
		    4193280,   4190208,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4194288,   4194288,   4194288,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   4193280,   4193280,   4193280, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   4194288, 
		    4194288,   4194288,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4194288,   4194288,   4194288,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   4193280,   4193280,   4193280, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    130560,    524160,   1048512,   1017792,   1966560, 
		    1966560,   1966560,   3932400,   3932400,   3932400, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3940336,   3940336,   3940336, 
		    1966560,   1966560,   1966560,   1017792,   1048512, 
		     524160,    130560,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   4194288,   4194288,   4194288, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    261632,    261632,    261632,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,    261632, 
		     261632,    261632,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{      8176,      8176,      8176,       896,       896, 
		        896,       896,       896,       896,       896, 
		        896,       896,       896,       896,       896, 
		        896,       896,       896,       896,   3671936, 
		    3671936,   3671808,   3936000,   4071168,   2096640, 
		    1047552,    258048,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670512,   3671008,   3672000,   3673984,   3677952, 
		    3685888,   3701760,   3733504,   3796992,   3923968, 
		    4177920,   4161536,   4128768,   4063232,   4128768, 
		    4161536,   4177920,   3923968,   3796992,   3733504, 
		    3701760,   3685888,   3677952,   3673984,   3672000, 
		    3671008,   3670512,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   4194288, 
		    4194288,   4194288,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4063728,   4129776,   4129776,   3901296,   3901296, 
		    3788400,   3788400,   3734640,   3734640,   3700848, 
		    3700848,   3682416,   3682416,   3670128,   3670128, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3932272,   4063344,   4063344,   4128880,   4128880, 
		    4161648,   3899504,   3915888,   3784816,   3793008, 
		    3727472,   3731568,   3698800,   3700848,   3684464, 
		    3685488,   3677296,   3677808,   3673712,   3673968, 
		    3671920,   3672048,   3671024,   3671024,   3670512, 
		    3670512,   3670256,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    130560,    524160,   1048512,   1017792,   1966560, 
		    1966560,   1966560,   3932400,   3932400,   3932400, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3932400,   3932400,   3932400, 
		    1966560,   1966560,   1966560,   1017792,   1048512, 
		     524160,    130560,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4192256,   4193792,   4194176,   3672000,   3670496, 
		    3670240,   3670128,   3670128,   3670128,   3670240, 
		    3670496,   3672000,   4194176,   4193792,   4192256, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,   3670016,   3670016,   3670016, 
		    3670016,   3670016,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    130560,    524160,   1048512,   1017792,   1966560, 
		    1966560,   1966560,   3932400,   3932400,   3932400, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3676272,   3939568,   3935984,   3934192, 
		    1967072,   1966560,   1966560,   1017792,   1048544, 
		     524272,    130608,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4192256,   4193792,   4194176,   3672000,   3670496, 
		    3670240,   3670128,   3670128,   3670128,   3670240, 
		    3670496,   3672000,   4194176,   4193792,   4192256, 
		    3700736,   3684352,   3685376,   3677696,   3673600, 
		    3673856,   3671936,   3670912,   3670976,   3670496, 
		    3670240,   3670256,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     30720,    130560,    524160,   1036224,   2032608, 
		    1835504,   3932400,   3670128,   3932160,   3932160, 
		    2031616,   2080768,   1048320,    262080,     65504, 
		        992,       240,       240,       112,   3670128, 
		    3932400,   3932400,   2032608,   1036224,    524160, 
		     130560,     30720,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4194288,   4194288,   4194288,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3932400,   3932400,   3932400, 
		    1966560,   1966560,   1966560,   1016768,   1048512, 
		     524160,    130560,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670128,   3932400,   3932400,   1835232,   1966560, 
		    1966560,   1966560,    917952,    984000,    984000, 
		     984000,    459648,    493440,    493440,    493440, 
		     231168,    249600,    249600,    249600,    118272, 
		     130560,    130560,    130560,     64512,     64512, 
		      64512,     30720,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3670128,   3670128,   3670128, 
		    3670128,   3670128,   3670128,   3670128,   3682416, 
		    3682416,   3700848,   3700848,   3734640,   3734640, 
		    3788400,   3788400,   3901296,   3901296,   4129776, 
		    4129776,   4063728,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670128,   1835232,   1835232,    917952,    917952, 
		     459648,    459648,    231168,    231168,    118272, 
		     118272,     64512,     64512,     30720,     64512, 
		      64512,    118272,    118272,    231168,    231168, 
		     459648,    459648,    917952,    917952,   1835232, 
		    1835232,   3670128,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3670128,   1835232,   1835232,    917952,    917952, 
		     459648,    459648,    231168,    231168,    118272, 
		     118272,     64512,     64512,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4194288,   4194288,   4194288,       112,       224, 
		        448,       896,       896,      1792,      3584, 
		       3584,      7168,      7168,     14336,     28672, 
		      57344,     57344,    114688,    229376,    229376, 
		     458752,    917504,   1835008,   3670016,   4194288, 
		    4194288,   4194288,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{      8176,      8176,      8176,      7168,      7168, 
		       7168,      7168,      7168,      7168,      7168, 
		       7168,      7168,      7168,      7168,      7168, 
		       7168,      7168,      7168,      7168,      7168, 
		       7168,      7168,      7168,      7168,      8176, 
		       8176,      8176,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3145728,   3670016,   3670016,   1835008,    917504, 
		     917504,    458752,    229376,    229376,    114688, 
		      57344,     57344,     28672,     14336,     14336, 
		       7168,      3584,      3584,      1792,       896, 
		        896,       448,       224,       224,       112, 
		         48,        48,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   4186112,   4186112,   4186112,     57344,     57344, 
		      57344,     57344,     57344,     57344,     57344, 
		      57344,     57344,     57344,     57344,     57344, 
		      57344,     57344,     57344,     57344,     57344, 
		      57344,     57344,     57344,     57344,   4186112, 
		    4186112,   4186112,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     30720,     64512,    130560,    249600,    493440, 
		     984000,   1966560,   3932400,   3670128,   3145776, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   4194272, 
		    4194272,   4194272,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     28672,     28672,     28672,     12288,     24576, 
		      16384,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,    522240, 
		    1048064,    794368,      1920,     61824,    261568, 
		    1023936,   1968064,   1573312,   3670464,   3145920, 
		    3145920,   3670464,   1573312,   1968096,   1023840, 
		     261232,     61488,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3145728,   3145728,   3145728,   3145728,   3145728, 
		    3145728,   3145728,   3145728,   3145728,   3210240, 
		    3407616,   3639168,   4063680,   3932384,   3670112, 
		    3670128,   3145776,   3145776,   3145776,   3145776, 
		    3670128,   3670112,   3932384,   4063680,   3639168, 
		    3407616,   3210240,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,    261888, 
		    1048512,   1966560,   1572960,   3670128,   3670128, 
		    3145728,   3145728,   3145728,   3145728,   3145728, 
		    3145728,   3670128,   3670128,   1572976,   1966560, 
		    1048512,    261888,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{        48,        48,        48,        48,        48, 
		         48,        48,        48,        48,     64560, 
		     261936,    493488,    918000,   1835248,   1572976, 
		    3670128,   3145776,   3145776,   3145776,   3145776, 
		    3670128,   1572976,   1835248,    918000,    493488, 
		     261936,     64560,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     64512, 
		     261888,    493440,    917952,   1835232,   1572960, 
		    3670128,   3145776,   4194288,   4194288,   3145728, 
		    3670016,   1572864,   1835008,    917504,    493312, 
		     261888,     64512,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{      3072,      7680,     14080,     13056,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,    130560,    130560, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     30768, 
		     130608,    511920,    984048,   1835248,   1572976, 
		    3670128,   3145776,   3670128,   1572976,   1835248, 
		     984048,    511920,    130608,     30768,        48, 
		         48,        48,   1572912,   1835120,    917728, 
		     459200,    262016,     65024,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   3145728,   3145728,   3145728,   3145728,   3145728, 
		    3145728,   3145728,   3145728,   3145728,   3210240, 
		    3407616,   3639168,   4063680,   3932384,   3670112, 
		    3670128,   3145776,   3145776,   3145776,   3145776, 
		    3145776,   3145776,   3145776,   3145776,   3145776, 
		    3145776,   3145776,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,     24576, 
		      24576,         0,         0,         0,     61440, 
		      61440,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      64512,     64512,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,     24576, 
		      24576,         0,         0,         0,     61440, 
		      61440,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,   3158016,   3698688,   1597440, 
		    1892352,   1032192,    196608,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   1572864,   1572864,   1572864,   1572864,   1572864, 
		    1572864,   1572864,   1572864,   1572864,   1573056, 
		    1573312,   1573760,   1574656,   1576448,   1580032, 
		    1587200,   1601536,   1630208,   1687552,   1826816, 
		    2060288,   1980416,   1842176,   1576448,   1574656, 
		    1573760,   1573312,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     61440,     61440,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      12288,     12288,     12288,     12288,     12288, 
		      64512,     64512,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   3343104, 
		    3639168,   3639232,   3984608,   3984608,   3688560, 
		    3700848,   3700848,   3158064,   3158064,   3158064, 
		    3158064,   3158064,   3158064,   3158064,   3158064, 
		    3158064,   3158064,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   3210240, 
		    3407616,   3639168,   4063680,   3932384,   3670112, 
		    3670128,   3145776,   3145776,   3145776,   3145776, 
		    3145776,   3145776,   3145776,   3145776,   3145776, 
		    3145776,   3145776,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     64512, 
		     261888,    493440,    917952,   1835232,   1572960, 
		    3670128,   3145776,   3145776,   3145776,   3145776, 
		    3670128,   1572960,   1835232,    917952,    493440, 
		     261888,     64512,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   3176448, 
		    3276288,   3657600,   4129728,   3932384,   3670112, 
		    3670128,   3145776,   3670128,   3670112,   3932384, 
		    4129728,   3669888,   3276288,   3176448,   3145728, 
		    3145728,   3145728,   3145728,   3145728,   3145728, 
		    3145728,   3145728,   3145728,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     30768, 
		     130608,    511920,   1016816,   1835248,   1572976, 
		    3670128,   3145776,   3670128,   1572976,   1835248, 
		     984048,    511920,    130608,     30768,        48, 
		         48,        48,        48,        48,        48, 
		         48,        48,        48,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,    818688, 
		     917376,   1016256,    983232,    917504,    786432, 
		     786432,    786432,    786432,    786432,    786432, 
		     786432,    786432,    786432,    786432,    786432, 
		     786432,    786432,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,    261888, 
		    1048512,   1966560,   1572960,   3670128,   3145840, 
		    3670016,   2096128,   2096896,      1920,       480, 
		        112,   3670064,   3670064,   1572960,   1966560, 
		    1048512,    261888,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,     24576, 
		      24576,     24576,     24576,     24576,   1048320, 
		    1048320,     24576,     24576,     24576,     24576, 
		      24576,     24576,     24576,     24576,     24576, 
		      24576,     24576,     24576,     24960,     29568, 
		      16128,      3072,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   3145776, 
		    3145776,   3145776,   3145776,   3145776,   3145776, 
		    3145776,   3145776,   3145776,   3145776,   3145776, 
		    3670128,   1572976,   1835248,    918000,    493488, 
		     261936,     64560,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   3145776, 
		    3145776,   1572960,   1572960,    786624,    786624, 
		     393600,    393600,    197376,    197376,     99840, 
		      99840,     52224,     52224,     52224,     30720, 
		      30720,     12288,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   3145776, 
		    3145776,   3145776,   3158064,   3158064,   3158064, 
		    3158064,   3158064,   3158064,   3158064,   3700848, 
		    1603680,   1591392,    838848,    838848,    493440, 
		     493440,    197376,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   3145776, 
		    3670128,   1835232,    917952,    459648,    231168, 
		     118272,     64512,     30720,     30720,     64512, 
		     118272,    231168,    459648,    917952,   1835232, 
		    3670128,   3145776,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   3145776, 
		    3145776,   3145776,   3145776,   3145776,   3145776, 
		    3145776,   3145776,   3670128,   1572976,   1835248, 
		     984048,    511920,    130608,     30768,        48, 
		         48,        48,   1572912,   1835120,    917728, 
		     459200,    262016,     65024,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,   4194288, 
		    4194288,       224,       448,       896,      1792, 
		       3584,      7168,    261888,    261888,     57344, 
		     114688,    229376,    458752,    917504,   1835008, 
		    4194288,   4194288,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{        16,       112,       240,       224,       192, 
		        448,       448,       384,       384,       384, 
		        896,      3840,      7680,      7168,      7680, 
		       3840,       896,       384,       384,       384, 
		        448,       448,       192,       224,       240, 
		        112,        16,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{     28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{   2097152,   3670016,   3932160,   1835008,    786432, 
		     917504,    917504,    393216,    393216,    393216, 
		     458752,    245760,    122880,     57344,    122880, 
		     245760,    458752,    393216,    393216,    393216, 
		     917504,    917504,    786432,   1835008,   3932160, 
		    3670016,   2097152,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{    131072,   1015808,   2080768,   1957888,   3727360, 
		    3176496,   3176496,      7280,      3936,      2016, 
		       1984,       256,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
		{         0,         0,         0,         0,     64512, 
		     261888,    524160,    984000,   1966560,   1835232, 
		    3701872,   3735152,   3262256,   3244080,   3244080, 
		    3262256,   3735152,   3701872,   1835232,   1966560, 
		     984000,    524160,    261888,     64512,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0,    
			  0,	     0,		0,	   0,	      0 },
			    };

unsigned int font4[96][48] = {
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,         0,
                          0,         0,         0,     49152,    122880,
                     122880,     49152,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1973760,   1973760,   1973760,   1973760,   1973760,
                    1973760,   1973760,   1973760,   1973760,   1973760,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,   3936000,   3936000,   3936000,
                    3936000,   3936000,   3936000,  67108848,  67108848,
                   67108848,  67108848,   3936000,   3936000,   3936000,
                    3936000,  67108848,  67108848,  67108848,  67108848,
                    3936000,   3936000,   3936000,   3936000,   3936000,
                    3936000,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    122880,    122880,    122880,   1048512,   4194240,
                   16777152,  33554368,  66183168,  65134592,  63037440,
                   63037440,  65134592,  32628736,  16776704,   8388480,
                    4194240,   1048544,    123872,    123376,    123120,
                     123120,    123120,    123376,    123872,    124896,
                   16777152,  16777088,  16776960,  16776192,    122880,
                     122880,    122880,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {        48,   3932272,  16711920,  33521888,  63160800,
                   58835904,  58835840,  63162240,  33525504,  16715264,
                    3939840,     15360,     14336,     30720,     61440,
                      57344,    122880,    245760,    229376,    491520,
                     983040,    921344,   1982400,   3964896,   3731696,
                    7921776,  15786096,  14741744,  31490016,  62930880,
                   58724096,  50331648,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   2096640,   3936000,   7866240,
                   15729600,  15729600,   7866304,   3673984,   3940096,
                    1998336,    982016,   1046528,   1040384,   2080768,
                    4177920,   8249344,  15785984,  31518720,  29390848,
                   62928896,  58735632,  62922288,  29363824,  31461360,
                   14682080,  15729600,   7866304,   3936224,   2096864,
                    1047792,    258160,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     61440,     61440,     61440,     61440,     61440,
                      61440,     61440,     61440,     61440,     61440,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {       112,       240,       480,       960,      1920,
                       3840,      3840,      7680,     15360,     30720,
                      30720,     61440,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                      61440,     30720,     30720,     15360,      7680,
                       3840,      3840,      1920,       960,       480,
                        240,       112,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  58720256,  62914560,  31457280,  15728640,   7864320,
                    3932160,   3932160,   1966080,    983040,    491520,
                     491520,    245760,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     245760,    491520,    491520,    983040,   1966080,
                    3932160,   3932160,   7864320,  15728640,  31457280,
                   62914560,  58720256,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                   14680512,  15729600,   7340928,   7866240,   3671808,
                    3936000,   1838592,   1973760,    998400,  67108848,
                   67108848,  67108848,  67108848,    998400,   1973760,
                    1838592,   3936000,   3671808,   7866240,   7340928,
                   15729600,  14680512,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,  67108848,
                   67108848,  67108848,  67108848,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,    122880,    122880,
                     122880,    122880,     57344,    114688,    229376,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,  67108848,
                   67108848,  67108848,  67108848,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,    122880,    122880,
                     122880,    122880,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {        48,       112,       240,       224,       480,
                        960,       896,      1920,      3840,      3584,
                       7680,     15360,     14336,     30720,     61440,
                      57344,    122880,    245760,    229376,    491520,
                     983040,    917504,   1966080,   3932160,   3670016,
                    7864320,  15728640,  14680064,  31457280,  62914560,
                   58720256,  50331648,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   4194048,   8130432,  16256960,
                   15732672,  32514016,  31465440,  31473120,  65027568,
                   62945520,  62945520,  62976240,  62976240,  63037680,
                   63037680,  63160560,  63160560,  63406320,  63406320,
                   63897840,  63897840,  64881136,  33423840,  33292768,
                   33293280,  16253888,  16254912,   8130432,   4194048,
                    1047552,    258048,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    122880,    253952,    516096,   1040384,   1957888,
                    3792896,   7462912,   6414336,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,   2096640,   2096640,
                    2096640,   2096640,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1044480,   4193280,   8388096,  16776960,  32509824,
                   65013696,  62915552,  58720752,       496,       496,
                       1008,       992,      1984,      3968,      7936,
                      15872,     31744,     63488,    126976,    253952,
                     507904,   1015808,   2031616,   4063232,   8126464,
                   16252928,  32505856,  66060288,  67108848,  67108848,
                   67108848,  67108848,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108848,  67108848,  67108848,  67108848,       240,
                        480,       960,      1920,      3840,      7680,
                      15360,     30720,     61440,    122880,    245760,
                     253952,    129024,     64512,     16128,      4032,
                        992,       496,  58720496,  62914800,  62914800,
                   65012192,  32506848,  15730624,   8388480,   4194048,
                    1047552,    258048,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {      1920,      3968,      8064,     16256,     32640,
                      63360,    124800,    247680,    493440,    984960,
                    1968000,   3934080,   7866240,  15730560,  31459200,
                   62916480,  62916480,  62916480,  67108848,  67108848,
                   67108848,  67108848,      1920,      1920,      1920,
                       1920,      1920,      1920,      1920,      1920,
                       1920,      1920,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108800,  67108800,  67108800,  67108800,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  63175680,  63962880,  65011584,
                   66588608,  66061280,  65012192,       496,       240,
                        240,       240,       496,       480,  62915552,
                   65012672,  31459264,  32509824,  16260864,   8388096,
                    4193280,   1044480,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     61440,    122880,    245760,    491520,    983040,
                    1966080,   3932160,   3932160,   7864320,   7864320,
                   15728640,  15728640,  31715328,  32504832,  67108608,
                   66850688,  66586560,  66061248,  66061280,  65012192,
                   65012208,  62914800,  62914800,  65012208,  31457760,
                   32506848,  15729600,  16254912,   8130432,   4194048,
                    1047552,    258048,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108848,  67108848,  67108848,  67108848,       496,
                        992,      1984,      3968,      3840,      7936,
                      15872,     15360,     31744,     63488,    126976,
                     122880,    253952,    507904,   1015808,    983040,
                    2031616,   4063232,   8126464,   7864320,  16252928,
                   32505856,  32505856,  65011712,  65011712,  62914560,
                   62914560,  62914560,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   4194048,   8130432,  16254912,
                   32506848,  31457760,  31457760,  32506848,  16254912,
                    8130432,   4194048,   2096640,   2096640,   4194048,
                    8130432,  16254912,  15729600,  32506848,  31457760,
                   65012208,  62914800,  62914800,  65012208,  31457760,
                   32506848,  15729600,  16254912,   8130432,   4194048,
                    1047552,    258048,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   4194048,   8130432,  16254912,
                   15729600,  32506848,  31457760,  65012208,  62914800,
                   62914800,  65012208,  31457776,  32506864,  15729648,
                   16254960,   8130544,   4193520,   1047008,    258528,
                        960,      1920,      1920,      3840,      7680,
                       7680,     15360,     30720,     61440,     61440,
                     122880,    245760,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    122880,    122880,    122880,
                     122880,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,    122880,    122880,
                     122880,    122880,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    122880,    122880,    122880,
                     122880,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,    122880,    122880,
                     122880,    122880,     57344,    114688,    229376,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {      1008,      2016,      4032,      8064,     16128,
                      32256,     64512,    129024,    258048,    516096,
                    1032192,   2064384,   4128768,   8257536,  16515072,
                   33030144,  33030144,  16515072,   8257536,   4128768,
                    2064384,   1032192,    516096,    258048,    129024,
                      64512,     32256,     16128,      8064,      4032,
                       2016,      1008,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                   67108848,  67108848,  67108848,  67108848,         0,
                          0,         0,         0,  67108848,  67108848,
                   67108848,  67108848,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  66060288,  33030144,  16515072,   8257536,   4128768,
                    2064384,   1032192,    516096,    258048,    129024,
                      64512,     32256,     16128,      8064,      4032,
                       2016,      2016,      4032,      8064,     16128,
                      32256,     64512,    129024,    258048,    516096,
                    1032192,   2064384,   4128768,   8257536,  16515072,
                   33030144,  66060288,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   4194048,   8388480,  16254912,
                   32506848,  31457760,  65012208,  65011952,  62914800,
                   62914800,  62914800,       496,       992,      1984,
                       3968,      7936,     15872,     31744,     63488,
                     126976,    253952,    245760,    245760,         0,
                          0,         0,         0,    245760,    245760,
                     245760,    245760,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   4194048,   8388480,  16254912,
                   15729600,  32506848,  31457760,  65135088,  63172848,
                   63437040,  63387888,  63387888,  63387888,  63387888,
                   63387888,  63388128,  63388608,  63176576,  63045376,
                   62946304,  62914560,  65011712,  31457280,  32505856,
                   15728640,  16252928,   8126464,   4194272,   2097120,
                     524256,    131040,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     49152,    122880,    258048,    522240,    998400,
                     998400,   2047488,   1973760,   1973760,   4071168,
                    3936000,   3936000,   3936000,   8130432,   7866240,
                    7866240,   7866240,  16254912,  15729600,  15729600,
                   16777152,  33554400,  33554400,  33554400,  31457760,
                   31457760,  31457760,  65012208,  62914800,  62914800,
                   62914800,  62914800,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67104768,  67107840,  67108608,  67108736,  31459264,
                   31458272,  31457776,  31457520,  31457520,  31457520,
                   31457520,  31457776,  31458272,  31459264,  33554304,
                   33553920,  33553920,  33554304,  31459264,  31458272,
                   31457776,  31457520,  31457520,  31457520,  31457520,
                   31457776,  31458272,  31459264,  67108736,  67108608,
                   67107840,  67104768,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   2096640,   4194048,   8130432,
                   16254912,  15729600,  32506848,  31457760,  31457776,
                   65011952,  62914800,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914800,  65011952,  31457776,  31457760,  32506848,
                   15729600,  16254912,   8130432,   4194048,   2096640,
                    1047552,    258048,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67104768,  67107840,  67108352,  67108608,  31461248,
                   31459264,  31458240,  31458240,  31458272,  31457760,
                   31457760,  31457776,  31457520,  31457520,  31457520,
                   31457520,  31457520,  31457520,  31457520,  31457520,
                   31457776,  31457760,  31457760,  31458272,  31458240,
                   31458240,  31459264,  31461248,  67108608,  67108352,
                   67107840,  67104768,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108848,  67108848,  67108848,  67108848,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  67104768,
                   67104768,  67104768,  67104768,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  67108848,  67108848,
                   67108848,  67108848,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108848,  67108848,  67108848,  67108848,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  67104768,
                   67104768,  67104768,  67104768,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   2096640,   4194048,   8130432,
                   16254912,  15729600,  32506848,  31457760,  31457776,
                   65011952,  62914800,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   63176688,  65273840,  31719408,  31719392,  32506848,
                   15729600,  16254912,   8130432,   4194048,   2096640,
                    1047552,    258048,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  67108848,
                   67108848,  67108848,  67108848,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   2096640,   2096640,   2096640,   2096640,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,   2096640,   2096640,
                    2096640,   2096640,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    131040,    131040,    131040,    131040,      7680,
                       7680,      7680,      7680,      7680,      7680,
                       7680,      7680,      7680,      7680,      7680,
                       7680,      7680,      7680,      7680,      7680,
                       7680,      7680,      7680,  31464960,  31464960,
                   32521728,  15744000,  16284672,   7895040,   4190208,
                    2088960,   1032192,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62915056,  62915552,  62916544,  62918528,  62922496,
                   62930432,  62946304,  62978048,  63041536,  63168512,
                   63422464,  63930368,  64946176,  66977792,  66846720,
                   66846720,  66846720,  66846720,  66977792,  64946176,
                   63930368,  63422464,  63168512,  63041536,  62978048,
                   62946304,  62930432,  62922496,  62918528,  62916544,
                   62915552,  62915056,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  67108848,  67108848,
                   67108848,  67108848,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  66061296,  66586608,  66586608,  66850800,  64753392,
                   64888560,  63839472,  63913200,  63387888,  63437040,
                   63172848,  63172848,  63037680,  63037680,  62963952,
                   62963952,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  65011952,  66060528,  66060528,  66584816,  66584816,
                   66846960,  64749808,  64880880,  63832304,  63897840,
                   63373552,  63406320,  63144176,  63160560,  63029488,
                   63037680,  62972144,  62976240,  62943472,  62945520,
                   62929136,  62930160,  62921968,  62922480,  62918384,
                   62918640,  62916592,  62916592,  62915568,  62915568,
                   62915056,  62915056,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   2096640,   4194048,   8130432,
                   16254912,  15729600,  32506848,  31457760,  31457760,
                   65012208,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  65012208,  31457760,  31457760,  32506848,
                   15729600,  16254912,   8130432,   4194048,   2096640,
                    1047552,    258048,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67104768,  67107840,  67108352,  67108608,  62922624,
                   62916544,  62915552,  62915056,  62915056,  62915056,
                   62915056,  62915552,  62916544,  62922624,  67108608,
                   67108352,  67107840,  67104768,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    258048,   1047552,   2096640,   4194048,   8130432,
                   16254912,  15729600,  32506848,  31457760,  31457760,
                   65012208,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62943472,
                   62945520,  65027568,  31464928,  31465440,  32509920,
                   15730624,  16254912,   8130496,   4194272,   2096864,
                    1047792,    258160,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67104768,  67107840,  67108352,  67108608,  62922624,
                   62916544,  62915552,  62915056,  62915056,  62915056,
                   62915056,  62915552,  62916544,  62922624,  67108608,
                   67108352,  67107840,  67106816,  64880640,  63897600,
                   63406080,  63160320,  63037440,  62976000,  62945280,
                   62929920,  62922240,  62918400,  62916480,  62915520,
                   62915040,  62914800,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1047552,   4194048,   8388480,  16777152,  33032160,
                   31457760,  65012208,  62914800,  62914800,  62914800,
                   65011712,  32505856,  33292288,  16711680,   8372224,
                    2095104,    261632,     32640,      8128,      2016,
                        992,       496,  62914800,  62914800,  62914800,
                   65012208,  31457760,  32506848,  16777152,   8388480,
                    4194048,   1047552,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108848,  67108848,  67108848,  67108848,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  65012208,  31457760,
                   32506848,  15729600,  16519104,   8388480,   4194048,
                    2096640,    522240,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  58720368,  62914800,  62914800,  29360352,  31457760,
                   31457760,  14680512,  15729600,  15729600,   7340928,
                    7866240,   7866240,   3671808,   3936000,   3936000,
                    1838592,   1973760,   1973760,    924672,    998400,
                     998400,    473088,    522240,    522240,    258048,
                     258048,    258048,    122880,    122880,    122880,
                      49152,     49152,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62914800,  62914800,  62914800,  62914800,
                   62914800,  62963952,  62963952,  63037680,  63037680,
                   63172848,  63172848,  63437040,  63387888,  63913200,
                   63839472,  64888560,  64753392,  66850800,  66586608,
                   66586608,  66061296,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914800,  31457760,  15729600,  15729600,   7866240,
                    7866240,   3936000,   3936000,   1973760,   1973760,
                     998400,    998400,    522240,    522240,    258048,
                     258048,    258048,    258048,    522240,    522240,
                     998400,    998400,   1973760,   1973760,   3936000,
                    3936000,   7866240,   7866240,  15729600,  15729600,
                   31457760,  62914800,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914800,  31457760,  15729600,  15729600,   7866240,
                    7866240,   3936000,   3936000,   1973760,   1973760,
                     998400,    998400,    522240,    522240,    258048,
                     258048,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108848,  67108848,  67108848,  67108848,       496,
                        992,      1984,      1920,      3968,      7936,
                       7680,     15872,     31744,     30720,     63488,
                     126976,    122880,    253952,    507904,    491520,
                    1015808,   2031616,   1966080,   4063232,   8126464,
                   16252928,  32505856,  65011712,  67108848,  67108848,
                   67108848,  67108848,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1047552,   1047552,   1047552,   1047552,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,   1047552,   1047552,
                    1047552,   1047552,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  50331648,  58720256,  62914560,  29360128,  31457280,
                   15728640,   7340032,   7864320,   3932160,   1835008,
                    1966080,    983040,    458752,    491520,    245760,
                     114688,    122880,     61440,     28672,     30720,
                      15360,      7168,      7680,      3840,      1792,
                       1920,       960,       448,       480,       240,
                        112,        48,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1047552,   1047552,   1047552,   1047552,     15360,
                      15360,     15360,     15360,     15360,     15360,
                      15360,     15360,     15360,     15360,     15360,
                      15360,     15360,     15360,     15360,     15360,
                      15360,     15360,     15360,     15360,     15360,
                      15360,     15360,     15360,   1047552,   1047552,
                    1047552,   1047552,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    122880,    258048,    522240,   1047552,   2047488,
                    4071168,   8130432,  16254912,  32506848,  65012208,
                   62914800,  58720368,  50331696,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,  67108848,  67108848,
                   67108848,  67108848,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     61440,     61440,     61440,     61440,     28672,
                      57344,    114688,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,   4194048,   4194176,   4194240,       480,
                        224,    258288,   1047664,   4194160,   7866352,
                   15729648,  31457776,  62914800,  62914800,  62914800,
                   62914800,  31457776,  15729648,   7866352,   4194160,
                    1047664,    258096,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  59242496,  60816896,  62914304,
                   66850688,  66061248,  65012192,  62914784,  62914800,
                   58720368,  58720368,  58720368,  58720368,  62914800,
                   62914784,  65012192,  66061248,  66850688,  62914304,
                   60816896,  59242496,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,   2096640,   8388480,  16777152,
                   32506848,  62914800,  58720368,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720368,  62914800,  32506848,  16777152,
                    8388480,   2096640,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {       112,       112,       112,       112,       112,
                        112,       112,       112,       112,       112,
                        112,       112,    522352,   2096752,   4194160,
                    8130544,  15729648,  31457776,  29360368,  62914800,
                   58720368,  58720368,  58720368,  58720368,  62914800,
                   29360368,  31457776,  15729648,   8130544,   4194160,
                    2096752,    522352,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,   1047552,   4194048,   8388480,
                   16254912,  31457760,  29360352,  58720368,  58720368,
                   67108848,  67108848,  67108848,  58720256,  58720256,
                   62914560,  29360128,  31457760,  16254912,   8388480,
                    4194048,   1047552,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     28672,     63488,    121856,    118272,    118272,
                     116480,    115456,    114688,    114688,    114688,
                     114688,    114688,    114688,    114688,   4193792,
                    4193792,   4193792,   4193792,    114688,    114688,
                     114688,    114688,    114688,    114688,    114688,
                     114688,    114688,    114688,    114688,    114688,
                     114688,    114688,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    522352,   2096752,   4194160,
                    7866352,  15729648,  31457776,  62914800,  62914800,
                   62914800,  62914800,  31457776,  15729648,   7866352,
                    4194160,   2096240,    522352,       112,       112,
                        112,       112,  58720368,  58720368,  62914800,
                   31457760,  15729600,   8388480,   4194048,   1047552,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  59242496,  60816896,  62914304,
                   66850688,  66061248,  65012192,  62914784,  62914672,
                   58720368,  58720368,  58720368,  58720368,  58720368,
                   58720368,  58720368,  58720368,  58720368,  58720368,
                   58720368,  58720368,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                      57344,     57344,     57344,         0,         0,
                          0,         0,    516096,    516096,    516096,
                      57344,     57344,     57344,     57344,     57344,
                      57344,     57344,     57344,     57344,     57344,
                      57344,     57344,     57344,     57344,    523264,
                     523264,    523264,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                        896,       896,       896,         0,         0,
                          0,         0,      3968,      3968,      3968,
                        896,       896,       896,       896,       896,
                        896,       896,       896,       896,       896,
                        896,       896,       896,       896,       896,
                        896,       896,       896,   7340928,   7340928,
                    7866240,   3936000,   1973760,   1047552,    258048,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  14680064,  14680064,  14680064,  14680064,  14680064,
                   14680064,  14680064,  14680064,  14680064,  14680064,
                   14680064,  14680064,  14680512,  14681024,  14681984,
                   14683904,  14687744,  14695424,  14710784,  14741504,
                   14802944,  14925824,  15187968,  15720448,  16707584,
                   16545792,  16268288,  15736320,  14683904,  14681984,
                   14681024,  14680512,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    516096,    516096,    516096,     57344,     57344,
                      57344,     57344,     57344,     57344,     57344,
                      57344,     57344,     57344,     57344,     57344,
                      57344,     57344,     57344,     57344,     57344,
                      57344,     57344,     57344,     57344,     57344,
                      57344,     57344,     57344,    523264,    523264,
                     523264,    523264,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  60558848,  67059584,  66319296,
                   65135072,  62963952,  58769520,  58769520,  58769520,
                   58769520,  58769520,  58769520,  58769520,  58769520,
                   58769520,  58769520,  58769520,  58769520,  58769520,
                   58769520,  58769520,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  59242496,  60816896,  62914304,
                   66850688,  66061248,  65012192,  62914784,  62914672,
                   58720368,  58720368,  58720368,  58720368,  58720368,
                   58720368,  58720368,  58720368,  58720368,  58720368,
                   58720368,  58720368,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    522240,   2096640,   4194048,
                    8130432,  15729600,  31457760,  29360352,  62914800,
                   58720368,  58720368,  58720368,  58720368,  62914800,
                   29360352,  31457760,  15729600,   8130432,   4194048,
                    2096640,    522240,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  59242496,  60816896,  62914304,
                   66586496,  66061248,  65012192,  62914800,  62914800,
                   62914800,  62914800,  65012192,  66061248,  66586496,
                   62914304,  60816896,  59242496,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    522352,   2096752,   4194160,
                    7866352,  15729648,  31457776,  62914800,  62914800,
                   62914800,  62914800,  31457776,  15729648,   7866352,
                    4194160,   2096240,    522352,       112,       112,
                        112,       112,       112,       112,       112,
                        112,       112,       112,       112,       112,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  15204096,  15728512,  16777152,
                   16515552,  16252928,  15728640,  14680064,  14680064,
                   14680064,  14680064,  14680064,  14680064,  14680064,
                   14680064,  14680064,  14680064,  14680064,  14680064,
                   14680064,  14680064,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,   1047552,   4194048,   8388480,
                   15729600,  31457760,  62914800,  62914560,  65011712,
                   33521664,   8387584,   2097024,     32736,       496,
                        240,  62914800,  31457760,  15729600,   8388480,
                    4194048,   1047552,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,     57344,
                      57344,     57344,     57344,     57344,     57344,
                    8388544,   8388544,   8388544,     57344,     57344,
                      57344,     57344,     57344,     57344,     57344,
                      57344,     57344,     57344,     57344,     57344,
                      57344,     57344,     57344,     57536,     61888,
                      32640,     15872,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58720368,  58720368,  58720368,
                   58720368,  58720368,  58720368,  58720368,  58720368,
                   58720368,  58720368,  58720368,  58720368,  62914800,
                   29360368,  31457776,  15729648,   8130544,   4194160,
                    2096752,    522352,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58720368,  29360352,  29360352,
                   14680512,  14680512,   7340928,   7340928,   3671808,
                    3671808,   1838592,   1838592,    924672,    924672,
                     473088,    473088,    258048,    258048,    122880,
                     122880,     49152,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58720368,  58720368,  58720368,
                   58769520,  58769520,  58769520,  58769520,  58769520,
                   58769520,  58769520,  58769520,  58769520,  58769520,
                   58769520,  62963952,  29409504,  31580640,  14938560,
                    8339328,   1838592,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  62914800,  31457760,  15729600,
                    7866240,   3936000,   1973760,    998400,    522240,
                     258048,    122880,    122880,    258048,    522240,
                     998400,   1973760,   3936000,   7866240,  15729600,
                   31457760,  62914800,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58720368,  58720368,  58720368,
                   58720368,  58720368,  58720368,  58720368,  58720368,
                   58720368,  62914800,  31457776,  15729648,   7866352,
                    4194160,   2096240,    522352,       112,       112,
                        112,       112,  58720368,  58720368,  62914800,
                   31457760,  15729600,   8388480,   4194048,   1047552,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  67108848,  67108848,  67108848,
                        480,       960,      3968,      7680,     15360,
                      30720,   8388480,   8388480,    491520,    983040,
                    1966080,   8126464,  15728640,  31457280,  67108848,
                   67108848,  67108848,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {       768,      1792,      3840,      7680,     15360,
                      15360,     15360,     15360,     15360,     15360,
                      15360,     15360,     30720,     61440,    245760,
                     983040,    983040,    245760,     61440,     30720,
                      15360,     15360,     15360,     15360,     15360,
                      15360,     15360,     15360,      7680,      3840,
                       1792,       768,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,    122880,    122880,    122880,
                     122880,    122880,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   3145728,   3670016,   3932160,   1966080,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    491520,    245760,     61440,
                      15360,     15360,     61440,    245760,    491520,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,   1966080,   3932160,
                    3670016,   3145728,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    917504,   4161536,   8110080,  15843328,  14794752,
                   31580160,  62972016,  58777840,     61920,     29120,
                      29632,     31616,     16128,      7680,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,    258048,   1047552,   2096640,
                    4071168,   7866240,  15729600,  14680512,  31457760,
                   29483232,  29618400,  63387888,  59645040,  60558448,
                   60293232,  60293232,  60558448,  59645040,  63387888,
                   29618400,  29483232,  31457760,  14680512,  15729600,
                    7866240,   4071168,   2096640,   1047552,    258048,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                            };
unsigned int font5[96][58] = {
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                          0,         0,         0,         0,     12288,
                      30720,     30720,     12288,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    493440,    493440,    493440,    493440,    493440,
                     493440,    493440,    493440,    493440,    493440,
                     493440,    493440,    493440,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,   1966560,   1966560,
                    1966560,   1966560,   1966560,   1966560,   1966560,
                    1966560,  67108863,  67108863,  67108863,  67108863,
                    1966560,   1966560,   1966560,   1966560,   1966560,
                    1966560,   1966560,  67108863,  67108863,  67108863,
                   67108863,   1966560,   1966560,   1966560,   1966560,
                    1966560,   1966560,   1966560,   1966560,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     30720,     30720,     30720,     30720,   1048568,
                    4194296,  16777208,  33554424,  32536576,  65042432,
                   62945280,  62945280,  65042432,  65042432,  66091008,
                   33060864,  33454080,  16777088,   8388576,   2097144,
                     524284,     31230,     30846,     30783,     30751,
                      30751,     30735,     30735,     30751,     30782,
                    8388606,   8388604,   8388592,   8388544,     30720,
                      30720,     30720,     30720,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         3,         7,   1966095,   8355854,  16760862,
                   31580220,  62976056,  58749048,  58749168,  62976224,
                   31580640,  16761792,   8356736,   1968000,      3840,
                       3584,      7680,     15360,     14336,     30720,
                      61440,     57344,    122880,    245760,    229856,
                     493560,    987132,    925214,   1981455,   3946503,
                    3684359,   7879695,  15736350,  14684156,  31459320,
                   62915040,  58720256,  50331648,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    130560,    524160,   2097120,   4194288,   8258040,
                    8126712,  16253052,  16253052,   7864444,   8126588,
                    3932408,   3932656,   1967072,   1048448,   1048320,
                     523264,   2095104,   4159488,   8141824,  16268288,
                   15736320,  32513536,  31461120,  65015552,  62916480,
                   62916480,  62915523,  62915527,  65012207,  31457775,
                   32506110,  15728894,  16253052,   8126716,   4194302,
                    2097150,    524175,    130575,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {        15,        63,       126,       252,       504,
                       1008,      2016,      1984,      4032,      8064,
                      16128,     15872,     32256,     31744,     64512,
                      63488,     63488,     63488,     63488,     63488,
                      63488,     63488,     63488,     64512,     31744,
                      32256,     15872,     16128,      8064,      4032,
                       1984,      2016,      1008,       504,       252,
                        126,        63,        15,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914560,  66060288,  33030144,  16515072,   8257536,
                    4128768,   2064384,   1015808,   1032192,    516096,
                     258048,    126976,    129024,     63488,     64512,
                      31744,     31744,     31744,     31744,     31744,
                      31744,     31744,     31744,     64512,     63488,
                     129024,    126976,    258048,    516096,   1032192,
                    1015808,   2064384,   4128768,   8257536,  16515072,
                   33030144,  66060288,  62914560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,   7340088,   7864440,   3932400,
                    1966560,    984000,    984000,    493440,    249600,
                     249600,    130560,  67108863,  67108863,  67108863,
                   67108863,    130560,    249600,    249600,    493440,
                     984000,   1966560,   1966560,   3932400,   7864440,
                    7340088,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,  67108863,  67108863,  67108863,
                   67108863,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,     30720,
                      30720,     30720,     30720,     14336,     28672,
                      57344,    114688,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  67108863,  67108863,  67108863,
                   67108863,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,     30720,
                      30720,     30720,     30720,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         3,         7,        15,        14,        30,
                         60,        56,       120,       240,       224,
                        480,       960,       896,      1920,      3840,
                       3584,      7680,     15360,     14336,     30720,
                      61440,     57344,    122880,    245760,    229376,
                     491520,    983040,    917504,   1966080,   3932160,
                    3670016,   7864320,  15728640,  14680064,  31457280,
                   62914560,  58720256,  50331648,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     64512,    261888,   1048512,   2097120,   4129776,
                    8258040,   8126712,  16253052,  16253052,  15728700,
                   32505918,  32505982,  32506110,  31457790,  65012703,
                   65013663,  65015583,  62922255,  62929935,  62945295,
                   62976015,  65134623,  65257503,  65503263,  32440350,
                   33423422,  33292350,  33030206,  15728700,  16253052,
                   16253052,   8126712,   8258040,   4129776,   2097120,
                    1048512,    261888,     64512,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     30720,     63488,    129024,    260096,    489472,
                     948224,   1865728,   1865728,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,   1048512,
                    1048512,   1048512,   1048512,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    523776,   2097024,   8388576,  16516080,  32506104,
                   31457404,  62914622,  62914622,        31,        31,
                         31,        31,        63,        62,       126,
                        252,       504,      1008,      2016,      4032,
                       8064,     16128,     32256,     64512,    129024,
                     258048,    516096,   1032192,   2064384,   4128768,
                    8257536,  16515072,  33030144,  66060288,  67108863,
                   67108863,  67108863,  67108863,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108863,  67108863,  67108863,  67108863,        31,
                         62,       124,       248,       496,       992,
                       1984,      3968,      7936,     15872,     31744,
                      63488,    126976,    130048,     32512,      8128,
                       2016,      1008,       248,       124,        62,
                         62,        31,  62914591,  62914622,  65011774,
                   32505980,  16253176,  16253424,   8259552,   4194240,
                    2097024,    523776,    129024,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {       240,       496,      1008,      2032,      4080,
                       7920,     15600,     30960,     61680,    123120,
                     246000,    491760,    983280,   1966320,   3932400,
                    7864560,  15728880,  31457520,  67108863,  67108863,
                   67108863,  67108863,       240,       240,       240,
                        240,       240,       240,       240,       240,
                        240,       240,       240,       240,       240,
                        240,       240,       240,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108856,  67108856,  67108856,  67108856,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  63045120,
                   63438720,  65011680,  67108848,  66584824,  66060412,
                   65011772,        62,        30,        31,        15,
                         15,        15,        15,        31,  62914590,
                   65011774,  31457340,  32505980,  16253176,   8388592,
                    2097120,    524160,    130560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     63488,    126976,    253952,    507904,   1015808,
                    2031616,   1966080,   4063232,   3932160,   8126464,
                    7864320,  16252928,  15728640,  32505856,  32636416,
                   31981440,  66060256,  67108848,  66978296,  66584700,
                   66060348,  66060350,  65011742,  65011743,  65011743,
                   65011743,  65011743,  65011743,  65011743,  31457310,
                   32505918,  15728700,  16253052,   8258040,   4194288,
                    2097120,    524160,    130560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108863,  67108863,  67108863,  67108863,        63,
                         62,        62,       126,       124,       252,
                        504,       496,      1008,      2016,      1984,
                       4032,      8064,      7936,     16128,     32256,
                      31744,     64512,    129024,    126976,    258048,
                     516096,    507904,   1032192,   2064384,   2031616,
                    4128768,   8257536,   8126464,  16515072,  33030144,
                   32505856,  66060288,  65011712,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    130560,    524160,   2097120,   4194288,   8258040,
                    8126712,  16253052,  16253052,  16253052,  16253052,
                    8126712,   4063728,   2032608,   1048512,    524160,
                     524160,   2097120,   4129776,   8126712,  16253052,
                   15728700,  32505918,  31457310,  65011743,  62914575,
                   62914575,  62914575,  62914575,  65011743,  31457310,
                   32505918,  15728700,  16253052,   8126712,   4194288,
                    2097120,    524160,    130560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    130560,    524160,   2097120,   4194288,   8258040,
                   16253052,  15728700,  32505918,  31457310,  65011743,
                   65011743,  65011743,  65011743,  65011743,  65011743,
                   31457311,  32505919,  15728703,  16253055,   8258047,
                    4194303,   2097135,    524175,    130590,        60,
                         60,       124,       120,       248,       240,
                        496,       480,       992,      1984,      3968,
                       7936,     15872,     31744,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,     30720,     30720,
                      30720,     30720,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,     30720,
                      30720,     30720,     30720,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,     30720,     30720,
                      30720,     30720,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,     30720,
                      30720,     30720,     30720,     14336,     28672,
                      57344,    114688,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         7,        31,        62,       124,       248,
                        496,      2016,      3968,      7936,     15872,
                      64512,    126976,    253952,   1032192,   2031616,
                    4063232,  16515072,  32505856,  65011712,  65011712,
                   32505856,  16515072,   4063232,   2031616,   1032192,
                     253952,    126976,     64512,     15872,      7936,
                       3968,      2016,       496,       248,       124,
                         62,        31,         7,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,  67108863,  67108863,
                   67108863,  67108863,         0,         0,         0,
                          0,  67108863,  67108863,  67108863,  67108863,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  58720256,  65011712,  32505856,  16252928,   8126464,
                    4063232,   2064384,    507904,    253952,    126976,
                      64512,     15872,      7936,      4032,       992,
                        496,       252,        62,        31,        31,
                         62,       252,       496,       992,      4032,
                       7936,     15872,     64512,    126976,    253952,
                     507904,   2064384,   4063232,   8126464,  16252928,
                   32505856,  65011712,  58720256,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     64512,    261888,   1048512,   2097120,   4129776,
                    8258040,   8126712,  16253052,  15728700,  32505918,
                   31457310,  65011743,  62914591,  62914591,        30,
                         62,       252,       496,       992,      4032,
                       7936,     15872,     31744,     63488,    126976,
                     122880,    122880,    122880,    122880,    122880,
                          0,         0,         0,         0,    122880,
                     122880,    122880,    122880,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    524032,   2097088,   4194272,   8388592,  16515576,
                   16253176,  33030396,  32505980,  66060414,  65011774,
                   65011775,  65011743,  62979103,  63045135,  63176463,
                   63164175,  63145743,  63145743,  63145743,  63145743,
                   63145758,  63145758,  63176700,  63045624,  62980080,
                   62930880,  62914560,  65011712,  65011712,  32505856,
                   32505856,  33030144,  16515072,  16646144,   8388544,
                    4194240,   2097088,    524224,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     30720,     64512,    130560,    261888,    249600,
                     493440,   1017792,    984000,   2032608,   2032608,
                    1966560,   1966560,   4063728,   4063728,   3932400,
                    3932400,   8126712,   8126712,   7864440,   7864440,
                   16253052,  16253052,  15728700,  16777212,  33554430,
                   33554430,  33554430,  31457310,  31457310,  31457310,
                   65011743,  65011743,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67107840,  67108608,  67108800,  67108832,  15730672,
                   15729144,  15728892,  15728764,  15728702,  15728702,
                   15728671,  15728671,  15728671,  15728702,  15728702,
                   15728764,  15728888,  16777184,  16777088,  16777088,
                   16777184,  15728888,  15728764,  15728702,  15728702,
                   15728671,  15728671,  15728655,  15728671,  15728671,
                   15728670,  15728702,  15728764,  15728888,  16777200,
                   16777184,  16777088,  16776704,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    130560,    524160,   1048512,   2097120,   4163568,
                    8324088,   8258040,  16515324,  16253052,  32505918,
                   32505918,  65011743,  65011743,  65011743,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  65011743,
                   65011743,  65011743,  32505918,  32505918,  16253052,
                   16515324,   8258040,   8324088,   4163568,   2097120,
                    1048512,    524160,    130560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67106816,  67108352,  67108736,  67108800,  15732704,
                   15730672,  15729648,  15729144,  15728888,  15728892,
                   15728764,  15728702,  15728702,  15728670,  15728671,
                   15728671,  15728655,  15728655,  15728655,  15728655,
                   15728655,  15728655,  15728671,  15728671,  15728670,
                   15728702,  15728702,  15728764,  15728892,  15728888,
                   15729144,  15729648,  15730672,  15732704,  16777152,
                   16777088,  16776704,  16775168,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108863,  67108863,  67108863,  67108863,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  67108352,  67108352,  67108352,
                   67108352,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  67108863,
                   67108863,  67108863,  67108863,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108863,  67108863,  67108863,  67108863,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  67108352,  67108352,  67108352,
                   67108352,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    130560,    524160,   1048512,   2097120,   4163568,
                    8324088,   8258040,  16515324,  16253052,  32505918,
                   32505918,  65011743,  65011712,  65011712,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62947327,  65044479,
                   65044479,  65044479,  32505918,  32505918,  16253052,
                   16515324,   8258040,   8324088,   4163568,   2097120,
                    1048512,    524160,    130560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  67108863,  67108863,  67108863,
                   67108863,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1048512,   1048512,   1048512,   1048512,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,   1048512,
                    1048512,   1048512,   1048512,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     16383,     16383,     16383,     16383,       480,
                        480,       480,       480,       480,       480,
                        480,       480,       480,       480,       480,
                        480,       480,       480,       480,       480,
                        480,       480,       480,       480,       480,
                        480,  62915040,  62915040,  65012192,  65012192,
                   66061280,  32506848,  33032128,  33034176,  16777088,
                    8388352,   4193792,   1046528,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914623,  62914686,  62914812,  62915064,  62915568,
                   62916576,  62918592,  62922624,  62930688,  62946816,
                   62979072,  63043584,  63172608,  63430656,  63946752,
                   64978944,  67043328,  66977792,  66846720,  66846720,
                   66977792,  67043328,  64978944,  63946752,  63430656,
                   63172608,  63043584,  62979072,  62946816,  62930688,
                   62922624,  62918592,  62916576,  62915568,  62915064,
                   62914812,  62914686,  62914623,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  67108863,
                   67108863,  67108863,  67108863,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  66584703,  66846975,  66846975,  64881135,  63832527,
                   63898575,  63374223,  63408015,  63145743,  63164175,
                   63032847,  63045135,  62979087,  62979087,  62945295,
                   62945295,  62926863,  62926863,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  65011727,  66060303,  66584591,  66584591,  66846735,
                   66846735,  66977807,  64880655,  64946191,  63897615,
                   63930383,  63406095,  63422479,  63160335,  63168527,
                   63037455,  63041551,  62976015,  62978063,  62946319,
                   62929935,  62930447,  62922255,  62922511,  62918415,
                   62918543,  62916495,  62916559,  62915535,  62915567,
                   62915055,  62915071,  62914815,  62914815,  62914687,
                   62914687,  62914623,  62914591,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    130560,    524160,   2097120,   4194288,   4129776,
                    8258040,   8126712,  16253052,  16253052,  15728700,
                   32505918,  32505918,  32505918,  31457310,  65011743,
                   65011743,  65011743,  62914575,  62914575,  62914575,
                   62914575,  65011743,  65011743,  65011743,  31457310,
                   32505918,  32505918,  32505918,  15728700,  16253052,
                   16253052,   8126712,   8258040,   4129776,   4194288,
                    2097120,    524160,    130560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67107840,  67108608,  67108800,  67108832,  62915576,
                   62914812,  62914684,  62914622,  62914590,  62914591,
                   62914591,  62914591,  62914590,  62914622,  62914684,
                   62914812,  62915576,  67108832,  67108800,  67108608,
                   67107840,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,  62914560,  62914560,
                   62914560,  62914560,  62914560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    130560,    524160,   2097120,   4194288,   4129776,
                    8258040,   8126712,  16253052,  16253052,  15728700,
                   32505918,  32505918,  32505918,  31457310,  65011743,
                   65011743,  65011743,  62914575,  62914575,  62914575,
                   62914575,  65011743,  65011743,  65026079,  31464478,
                   32513598,  32509758,  32507710,  15730620,  16253948,
                   16253436,   8126712,   8258044,   4129788,   4194302,
                    2097134,    524175,    130567,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67107840,  67108608,  67108800,  67108832,  62915576,
                   62914812,  62914684,  62914622,  62914590,  62914591,
                   62914591,  62914591,  62914590,  62914622,  62914684,
                   62914812,  62915576,  67108832,  67108800,  67108608,
                   67107840,  64946176,  63930368,  63422464,  63168512,
                   63041536,  62978048,  62946304,  62930432,  62922496,
                   62918528,  62916544,  62915552,  62915056,  62914808,
                   62914684,  62914622,  62914591,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    524160,   2097120,   4194288,   8388600,  16646652,
                   16253052,  32505918,  32505886,  31457311,  65011727,
                   65011727,  65011712,  31457280,  32505856,  16515072,
                    8323072,   4177920,   1044480,    261632,     65408,
                       8160,      2040,       508,       124,        62,
                         30,        31,  62914591,  62914591,  65011742,
                   31457342,  32505918,  16253052,  16646652,   8388600,
                    4194288,   2097120,    524160,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108863,  67108863,  67108863,  67108863,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  65011743,  65011743,  31457310,  32505918,
                   16253052,  16515324,   8258040,   8324088,   4194288,
                    2097120,   1048512,    261888,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914575,  65011743,  65011743,  31457310,  32505918,
                   32505918,  15728700,  16253052,  16253052,   7864440,
                    8126712,   8126712,   3932400,   4063728,   4063728,
                    1966560,   2032608,   2032608,    984000,   1017792,
                    1017792,    493440,    511872,    511872,    249600,
                     261888,    261888,    130560,    130560,    130560,
                      64512,     64512,     64512,     30720,     30720,
                      30720,     12288,     12288,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62914575,  62914575,
                   62914575,  62914575,  62914575,  62926863,  62926863,
                   62945295,  62945295,  62979087,  62979087,  63045135,
                   63032847,  63164175,  63145743,  63408015,  63374223,
                   63898575,  63832527,  64881135,  64749807,  66846975,
                   66584703,  66584703,  66060351,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914575,  31457310,  15728700,  15728700,   7864440,
                    7864440,   3932400,   3932400,   1966560,   1966560,
                     984000,    984000,    493440,    493440,    249600,
                     249600,    130560,    130560,     64512,     64512,
                     130560,    130560,    249600,    249600,    493440,
                     493440,    984000,    984000,   1966560,   1966560,
                    3932400,   3932400,   7864440,   7864440,  15728700,
                   15728700,  31457310,  62914575,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  62914575,  31457310,  15728700,  15728700,   7864440,
                    7864440,   3932400,   3932400,   1966560,   1966560,
                     984000,    984000,    493440,    493440,    249600,
                     249600,    130560,     64512,     64512,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  67108863,  67108863,  67108863,  67108863,        15,
                         30,        30,        60,       120,       240,
                        480,       480,       960,      1920,      3840,
                       3840,      7680,     15360,     30720,     30720,
                      61440,    122880,    245760,    245760,    491520,
                     983040,   1966080,   1966080,   3932160,   7864320,
                   15728640,  31457280,  31457280,  62914560,  67108863,
                   67108863,  67108863,  67108863,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1048512,   1048512,   1048512,   1048512,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,    983040,    983040,
                     983040,    983040,    983040,    983040,   1048512,
                    1048512,   1048512,   1048512,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  50331648,  58720256,  62914560,  29360128,  31457280,
                   15728640,   7340032,   7864320,   3932160,   1835008,
                    1966080,    983040,    458752,    491520,    245760,
                     114688,    122880,     61440,     28672,     30720,
                      15360,      7168,      7680,      3840,      1792,
                       1920,       960,       448,       480,       240,
                        112,       120,        60,        28,        30,
                         15,         7,         3,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1048512,   1048512,   1048512,   1048512,       960,
                        960,       960,       960,       960,       960,
                        960,       960,       960,       960,       960,
                        960,       960,       960,       960,       960,
                        960,       960,       960,       960,       960,
                        960,       960,       960,       960,       960,
                        960,       960,       960,       960,   1048512,
                    1048512,   1048512,   1048512,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     30720,     64512,    130560,    261888,    511872,
                    1017792,   2032608,   4063728,   8126712,  16253052,
                   32505918,  65011743,  62914575,  58720263,  50331651,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,  67108863,
                   67108863,  67108863,  67108863,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     30720,     30720,     30720,     30720,     14336,
                      28672,     57344,    114688,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,   4192256,  16776704,  33554304,
                   62915520,  58720736,       224,    258288,   1047792,
                    4194288,   8130544,  15729648,  31457776,  29360368,
                   62914800,  58720368,  58720368,  58720368,  58720368,
                   62914800,  29360368,  31457776,  15729656,   8130492,
                    4194078,   1047567,    258055,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58850816,  59244416,  60817376,
                   62850032,  66846968,  66584700,  66060348,  65011742,
                   65011742,  62914574,  62914575,  58720263,  58720263,
                   58720263,  58720263,  62914575,  62914574,  65011742,
                   65011742,  66060348,  66584700,  66846968,  62850032,
                   60817376,  59244416,  58850816,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,   1048512,   4194288,   8388600,
                   16515324,  32505918,  31457310,  65011743,  62914575,
                   62914575,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  62914575,
                   62914575,  65011743,  31457310,  32505918,  16515324,
                    8388600,   2097136,    524224,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         7,         7,         7,         7,         7,
                          7,         7,         7,         7,         7,
                          7,         7,    130567,    524167,   2097127,
                    4129783,   8126719,  16253055,  15728703,  31457311,
                   31457311,  29360143,  62914575,  58720263,  58720263,
                   58720263,  58720263,  62914575,  29360143,  31457311,
                   31457311,  15728703,  16253055,   8126719,   4129783,
                    2097127,    524167,    130567,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    130560,    524160,   2097120,
                    4129776,   8126712,  16253052,  15728700,  31457310,
                   31457310,  29360142,  62914575,  58720263,  67108863,
                   67108863,  67108863,  62914560,  29360128,  31457280,
                   31457280,  15728640,  16252984,   8126584,   4129008,
                    2097120,    524160,    130560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {      7168,     15872,     16128,     31616,     29120,
                      28896,     28896,     28672,     28672,     28672,
                      28672,     28672,     28672,     28672,     28672,
                      28672,     28672,     28672,    524032,    524032,
                     524032,     28672,     28672,     28672,     28672,
                      28672,     28672,     28672,     28672,     28672,
                      28672,     28672,     28672,     28672,     28672,
                      28672,     28672,     28672,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    130567,    524167,   2097127,
                    4129783,   8126719,  16253055,  15728703,  31457311,
                   29360143,  62914575,  62914575,  62914575,  62914575,
                   29360143,  31457311,  15728703,  16253055,   8126719,
                    4129783,   2097127,    524167,    130567,         7,
                          7,         7,         7,  58720263,  58720263,
                   62914575,  31457310,  15728700,   7864440,   4194288,
                    1048512,    261888,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58850816,  59244416,  60817376,
                   62850032,  66846968,  66584700,  66060348,  65011742,
                   65011742,  62914574,  62914575,  58720263,  58720263,
                   58720263,  58720263,  58720263,  58720263,  58720263,
                   58720263,  58720263,  58720263,  58720263,  58720263,
                   58720263,  58720263,  58720263,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                      14336,     14336,     14336,         0,         0,
                          0,         0,    129024,    129024,    129024,
                      14336,     14336,     14336,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                     130816,    130816,    130816,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                        224,       224,       224,         0,         0,
                          0,         0,      2016,      2016,      2016,
                        224,       224,       224,       224,       224,
                        224,       224,       224,       224,       224,
                        224,       224,       224,       224,       224,
                        224,       224,       224,       224,       224,
                        224,       224,       224,   1835232,   1835232,
                    1966560,    917952,    984000,    493440,    261888,
                     130560,     30720,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {  14680064,  14680064,  14680064,  14680064,  14680064,
                   14680064,  14680064,  14680064,  14680064,  14680064,
                   14680064,  14680064,  14680092,  14680124,  14680184,
                   14680304,  14680544,  14681024,  14681984,  14683904,
                   14687744,  14695424,  14710784,  14741504,  14802944,
                   14925824,  15187968,  15720448,  16707584,  16545792,
                   16268288,  15736320,  14683904,  14681984,  14681024,
                   14680544,  14680304,  14680184,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    129024,    129024,    129024,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                      14336,     14336,     14336,     14336,     14336,
                     130816,    130816,    130816,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  59507072,  60687296,  60687296,
                   62851040,  62851040,  66293360,  66293360,  65133624,
                   65133624,  62978076,  62978076,  58748942,  58748942,
                   58748942,  58748942,  58748942,  58748942,  58748942,
                   58748942,  58748942,  58748942,  58748942,  58748942,
                   58748942,  58748942,  58748942,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58850816,  59244416,  60817376,
                   62850032,  66846968,  66584700,  66060348,  65011742,
                   65011742,  62914574,  62914575,  58720263,  58720263,
                   58720263,  58720263,  58720263,  58720263,  58720263,
                   58720263,  58720263,  58720263,  58720263,  58720263,
                   58720263,  58720263,  58720263,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    130560,    524160,   2097120,
                    4129776,   8126712,  16253052,  15728700,  31457310,
                   31457310,  29360142,  62914575,  58720263,  58720263,
                   58720263,  58720263,  62914575,  29360142,  31457310,
                   31457310,  15728700,  16253052,   8126712,   4129776,
                    2097120,    524160,    130560,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58850816,  59244416,  60817376,
                   62850032,  66846968,  66584700,  66060348,  65011742,
                   62914574,  62914575,  62914575,  62914575,  62914575,
                   62914574,  65011742,  66060348,  66584700,  66846968,
                   62850032,  60817376,  59244416,  58850816,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,  58720256,  58720256,  58720256,
                   58720256,  58720256,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,    130567,    524167,   2097127,
                    4129783,   8126719,  16253055,  15728703,  31457311,
                   29360143,  62914575,  62914575,  62914575,  62914575,
                   29360143,  31457311,  15728703,  16253055,   8126719,
                    4129783,   2097127,    524167,    130567,         7,
                          7,         7,         7,         7,         7,
                          7,         7,         7,         7,         7,
                          7,         7,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,   7405312,   7602112,   7864288,
                    8323312,   8257776,   8126576,   7864320,   7340032,
                    7340032,   7340032,   7340032,   7340032,   7340032,
                    7340032,   7340032,   7340032,   7340032,   7340032,
                    7340032,   7340032,   7340032,   7340032,   7340032,
                    7340032,   7340032,   7340032,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,   1048512,   4194288,   8388600,
                   16515324,  15728700,  31457310,  29360143,  62914567,
                   58720256,  62914560,  33554304,  16777184,   4194296,
                        124,        30,        15,        15,         7,
                   58720271,  62914574,  31457310,  15728700,  16515324,
                    8388600,   4194288,   1048512,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                      28672,     28672,     28672,     28672,     28672,
                      28672,     28672,     28672,   4194272,   4194272,
                    4194272,     28672,     28672,     28672,     28672,
                      28672,     28672,     28672,     28672,     28672,
                      28672,     28672,     28672,     28672,     28672,
                      28672,     28672,     28784,     28784,     30944,
                      16320,      8064,      3840,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58720263,  58720263,  58720263,
                   58720263,  58720263,  58720263,  58720263,  58720263,
                   58720263,  58720263,  58720263,  58720263,  58720263,
                   58720263,  58720263,  62914575,  29360143,  31457311,
                   31457311,  15728703,  16253055,   8126719,   4129783,
                    2097127,    524167,    130567,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  50331651,  58720263,  58720263,
                   29360142,  29360142,  14680092,  14680092,   7340088,
                    7340088,   3670128,   3670128,   1835232,   1835232,
                     917952,    917952,    459648,    459648,    231168,
                     231168,    118272,    118272,     64512,     64512,
                      30720,     30720,     12288,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58720270,  58720270,  58720270,
                   58720270,  58748942,  58748942,  58748942,  58748942,
                   58748942,  58748942,  58748942,  58748942,  58748942,
                   58748942,  58748942,  29423644,  29423644,  14801976,
                   14801976,   7573104,   7573104,   4130784,   4130784,
                    1967040,   1967040,    786816,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58720263,  62914575,  31457310,
                   15728700,   7864440,   3932400,   1966560,    984000,
                     493440,    249600,    130560,     64512,     30720,
                      30720,     64512,    130560,    249600,    493440,
                     984000,   1966560,   3932400,   7864440,  15728700,
                   31457310,  62914575,  58720263,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  58720263,  58720263,  58720263,
                   58720263,  58720263,  58720263,  58720263,  58720263,
                   58720263,  58720263,  58720263,  58720263,  62914575,
                   29360143,  31457311,  15728703,  16253055,   8126719,
                    4129783,   2097127,    524167,    130567,         7,
                          7,         7,         7,  58720263,  58720263,
                   62914575,  31457310,  15728700,   7864440,   4194288,
                    1048512,    261888,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },

                {         0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,  67108863,  67108863,  67108863,
                         30,        60,       120,       240,       480,
                        960,      1920,      3840,      7680,   4194272,
                    4194272,   4194272,    245760,    491520,    983040,
                    1966080,   3932160,   7864320,  15728640,  31457280,
                   67108863,  67108863,  67108863,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {      8128,     16320,     32704,     64512,     63488,
                      61440,     61440,     61440,     61440,     61440,
                      61440,     61440,     61440,     61440,     61440,
                     122880,    245760,    983040,   1966080,   1966080,
                     983040,    245760,    122880,     61440,     61440,
                      61440,     61440,     61440,     61440,     61440,
                      61440,     61440,     61440,     63488,     64512,
                      32704,     16320,      8128,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {     30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,     30720,     30720,
                      30720,     30720,     30720,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {   1040384,   1044480,   1046528,     64512,     31744,
                      15360,     15360,     15360,     15360,     15360,
                      15360,     15360,     15360,     15360,     15360,
                       7680,      3840,       960,       480,       480,
                        960,      3840,      7680,     15360,     15360,
                      15360,     15360,     15360,     15360,     15360,
                      15360,     15360,     15360,     31744,     64512,
                    1046528,   1044480,   1040384,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {    786432,   4128768,   8355840,  15974400,  14802944,
                   31514627,  29421575,  62945295,  58735630,  50338846,
                       7708,      3900,      2040,      1008,       192,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 },
                {         0,         0,    130560,    524160,   2097120,
                    4163568,   4063728,   8126712,   7864440,  15728700,
                   15728700,  14680092,  31489054,  31522334,  31584030,
                   29590414,  63373775,  63307983,  63832079,  63832079,
                   63307983,  63373775,  29590414,  31584030,  31522334,
                   31489054,  14680092,  15728700,  15728700,   7864440,
                    8126712,   4063728,   4163568,   2097120,    524160,
                     130560,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0,         0,         0,
                          0,         0,         0 }
                                   };

/*****************************************************************************/
void put_string
(
    long sp[],                  /* I: pixel positions for strings       */
    long sl[],                  /* I: line positions for strings        */
    long textlen[],             /* I: lengths of annotation strings     */
    long scale[],               /* I: scale factors for strings         */
    long use_bgrey,             /* I: flag, use bgrey? = T or F         */
    double bgrey,               /* I: background grey level             */
    float fgrey[],              /* I: character grey level              */
    char direction[][5],        /* I: annotation direction              */
    char string[][264],         /* I: annotation strings                */
    long *num_strings,          /* I: no. annotation strings            */
    long num_inimages,          /* I: no. input images                  */
    long num_image_bands[],     /* I: no. bands in images               */
    long image_band_num[][MAXBND+1], /* I: image band numbers           */
    long *nsout,                /* I: no. samples/line                  */
    long *image_type,           /* I: image data type                   */
    char font[],                /* I: size of font specified by user    */
    long cellwide,              /* I: width of each font character      */
    long cellhigh               /* I: heigth of each font character     */
)

{

/* declare variables */
   extern long curcat;    /* message utility */
   extern long curent;    /* parameters for processing */
   float        *iobuf;   /* image i/o buffer */
   float        *tmpbuf;  /* temporary buffer */
   long band;
   long ftype = EREAL;    /* image data type as processed by put_string() */
   long image, ipos;
   long msg_flag = GEN2;
   long i, inum, k;
   struct CCELL ccell; /* info on the character cell */
   float bgreyf;       /* a copy of bgrey for c_pxfill() */
   bgreyf = bgrey;     /* bgrey is actually a double, c_pxfill() needs float */


/* allocate dynamic memory */
   if ((iobuf  = (float *) malloc (*nsout * sizeof (float))) == NULL ||
       (tmpbuf = (float *) malloc (*nsout * sizeof (float))) == NULL)
      c_lasmsg (TRUE, "annotate-alloc", "Error allocating dynamic memory");

/* initialize processsing message */
   c_inittm ("ANNOTATE", "Inserted", "string", " ", " ", num_strings, &zero,
             &msgtim, &msg_flag);
   curent = 1;
   c_startm ();

/* loop for all annotation strings */
   for (inum = 0; inum < *num_strings; inum++, curent++)
      {
      if (direction[inum][0] == 'H')  /* horizontal */
         {
         ccell.horizontal = TRUE;
         ccell.nsamps = scale[inum] * cellwide * textlen[inum];
         ccell.nlines = cellhigh * scale[inum];
         }
      else             /* vertical */
         {
         ccell.horizontal = FALSE;
         ccell.nsamps = cellwide * scale[inum];
         ccell.nlines = scale[inum] * cellhigh * textlen[inum];
         } 
      ccell.table0 = 1;
      ccell.sample = sp[inum];
      ccell.background = bgrey;
      ccell.foreground = fgrey[inum];
      ccell.nchars = textlen[inum];
      ccell.table7 = 1;
      ccell.table8 = 0;
      ccell.table9 = 2;
      ccell.use_back = use_bgrey;

      for (image = 0; image < num_inimages; image++)
         {
         for (k = 0; k < num_image_bands[image]; k++)
            {
            ccell.table8 = 0;
            band = image_band_num[image][k];
            for (i = 1; i <= ccell.nlines; i++)
               {
               ipos = sl[inum] + i - 1;

   /***************************************************************************
   Image I/O has a problem, when the images are opened for update access, and
   the data are converted to real on read and back to original on write (they
   are not being converted back). Hence, explicit conversion is done here.
   -K. A. Narayanan, July 19, 1989
   ***************************************************************************/
               if (c_eread ((struct FDESC **)&fdesc[image], &band, &ipos,
                            (unsigned char *)iobuf, &one) == E_FAIL)
                  c_lasmsg (TRUE, "annotate-fatal", "Fatal error encountered");
               if (use_bgrey)
                  c_pxfill ((unsigned char *)&tmpbuf[ccell.sample-1], &ftype,
                            &ccell.nsamps, (unsigned char *)&bgreyf);
               c_pxconv (image_type, &ftype,(unsigned char *)iobuf,
                        (unsigned char *)tmpbuf, nsout);
               genchr (tmpbuf, &i, &ccell, string[inum],font,cellwide,
                       cellhigh);
               c_pxconv (&ftype, image_type,(unsigned char *)tmpbuf,
                        (unsigned char *)iobuf, nsout);
               if (c_ewrite((struct FDESC **)&fdesc[image], &band, &ipos,
                             (unsigned char *)iobuf, &one) == E_FAIL)
                  c_lasmsg (TRUE, "annotate-fatal", "Fatal error encountered");
               } 
            } 
         }   
      }
   c_stoptm ();

/* free dynamic memory */
   free (iobuf);
   free (tmpbuf);

/* close images */
   for (image = 0; image < num_inimages; image++)
      if (c_eclose ((struct FDESC **)&fdesc[image]) != E_SUCC)
         c_lasmsg (FALSE, "annotate-nonfatal", "Nonfatal error encountered");
}

